@@ -15,4 +15,4 @@
 # ==============================================================================
 source "${BASH_SOURCE%/*}/utilities/setup.sh"
 
-tfrun bats ./ci/official/utilities/code_check_full.bats --timing --output build
+tfrun bats ./ci/official/utilities/code_check_full.bats --timing --output "$TFCI_OUTPUT_DIR"
\ No newline at end of file
