@@ -17,6 +17,7 @@ limitations under the License.
 
 #include <algorithm>
 #include <cstddef>
+#include <cstdint>
 #include <memory>
 #include <string>
 #include <utility>
@@ -37,6 +38,7 @@ limitations under the License.
 #include "tensorflow/c/tf_buffer_internal.h"
 #include "tensorflow/c/tf_status.h"
 #include "tensorflow/c/tf_tensor_internal.h"
+#include "xla/tsl/c/tsl_status_internal.h"
 #include "tensorflow/core/common_runtime/copy_tensor.h"
 #include "tensorflow/core/common_runtime/device.h"
 #include "tensorflow/core/common_runtime/device_factory.h"
@@ -166,17 +168,31 @@ TF_CAPI_EXPORT extern void TFE_ContextSetServerDef(TFE_Context* ctx,
                                                    const void* proto,
                                                    size_t proto_len,
                                                    TF_Status* status) {
-  TFE_ContextSetServerDefWithTimeout(ctx, keep_alive_secs, proto, proto_len,
-                                     /*init_timeout_in_ms=*/0, status);
+  TFE_ContextSetServerDefWithTimeoutAndRetries(
+      ctx, keep_alive_secs, proto, proto_len, /*init_timeout_in_ms=*/0,
+      /*retries=*/0, status, /*clear_existing_contexts=*/false);
+}
+
+// Set server def with timeout.
+TF_CAPI_EXPORT extern void TFE_ContextSetServerDefWithTimeout(
+    TFE_Context* ctx, int keep_alive_secs, const void* proto, size_t proto_len,
+    int64_t init_timeout_in_ms, TF_Status* status,
+    bool clear_existing_contexts) {
+  TFE_ContextSetServerDefWithTimeoutAndRetries(
+      ctx, keep_alive_secs, proto, proto_len, init_timeout_in_ms,
+      /*retries=*/0, status, clear_existing_contexts);
 }
 
 // Set server_def on the context, possibly updating it.
-// TODO(b/291142876) Simplify TFE_ContextSetServerDefWithTimeout and
+// TODO(b/291142876) Simplify TFE_ContextSetServerDefWithTimeoutAndRetries and
 // TFE_ContextUpdateServerDefWithTimeout to be simple wrappers around the same
 // C++ function.
-TF_CAPI_EXPORT extern void TFE_ContextSetServerDefWithTimeout(
+// Retries are used for CreateContext calls, which is used in
+// ParameterServerStrategy initialization to be robust to worker preemption.
+TF_CAPI_EXPORT extern void TFE_ContextSetServerDefWithTimeoutAndRetries(
     TFE_Context* ctx, int keep_alive_secs, const void* proto, size_t proto_len,
-    int64_t init_timeout_in_ms, TF_Status* status) {
+    int64_t init_timeout_in_ms, int retries, TF_Status* status,
+    bool clear_existing_contexts) {
 #if defined(IS_MOBILE_PLATFORM)
   status->status = tensorflow::errors::Unimplemented(
       "TFE_ContextSetServerDef not supported on mobile");
@@ -190,7 +206,7 @@ TF_CAPI_EXPORT extern void TFE_ContextSetServerDefWithTimeout(
   status->status =
       tensorflow::unwrap(ctx)->GetDistributedManager()->SetOrUpdateServerDef(
           server_def, /*reset_context=*/true, keep_alive_secs,
-          init_timeout_in_ms);
+          init_timeout_in_ms, retries, clear_existing_contexts);
 #endif  // !IS_MOBILE_PLATFORM
 }
 
@@ -225,7 +241,7 @@ TF_CAPI_EXPORT extern void TFE_ContextUpdateServerDefWithTimeout(
   status->status =
       tensorflow::unwrap(ctx)->GetDistributedManager()->SetOrUpdateServerDef(
           server_def, /*reset_context=*/false, keep_alive_secs,
-          init_timeout_in_ms);
+          init_timeout_in_ms, /*retries=*/0);
 #endif  // !IS_MOBILE_PLATFORM
 }
 
@@ -280,8 +296,8 @@ TFE_TensorHandle* TFE_NewTensorHandle(const TF_Tensor* t, TF_Status* status) {
 void TFE_DeleteTensorHandle(TFE_TensorHandle* h) {
   if (h == nullptr) return;
 
-  tensorflow::profiler::TraceMe activity(
-      "TFE_DeleteTensorHandle", tensorflow::profiler::TraceMeLevel::kInfo);
+  tsl::profiler::TraceMe activity("TFE_DeleteTensorHandle",
+                                  tsl::profiler::TraceMeLevel::kInfo);
   if (h) {
     tensorflow::unwrap(h)->Unref();
   }
@@ -423,7 +439,7 @@ class CustomDeviceAPI : public tensorflow::CustomDevice {
 
   const string& name() override { return name_; }
 
-  tensorflow::Status CopyTensorToDevice(
+  absl::Status CopyTensorToDevice(
       ImmediateExecutionTensorHandle* handle,
       ImmediateExecutionTensorHandle** result) override {
     handle->Ref();
@@ -438,7 +454,7 @@ class CustomDeviceAPI : public tensorflow::CustomDevice {
     return status.status;
   }
 
-  tensorflow::Status CopyTensorFromDevice(
+  absl::Status CopyTensorFromDevice(
       ImmediateExecutionTensorHandle* handle,
       const tensorflow::string& target_device_name,
       ImmediateExecutionTensorHandle** result) override {
@@ -455,9 +471,9 @@ class CustomDeviceAPI : public tensorflow::CustomDevice {
     return status.status;
   }
 
-  tensorflow::Status Execute(const ImmediateExecutionOperation* op,
-                             ImmediateExecutionTensorHandle** retvals,
-                             int* num_retvals) override {
+  absl::Status Execute(const ImmediateExecutionOperation* op,
+                       ImmediateExecutionTensorHandle** retvals,
+                       int* num_retvals) override {
     std::vector<TFE_TensorHandle*> outputs(*num_retvals);
     TF_Status status;
     device_.execute(tensorflow::wrap(op), num_retvals, outputs.data(), &status,
@@ -472,8 +488,8 @@ class CustomDeviceAPI : public tensorflow::CustomDevice {
     return status.status;
   }
 
-  tensorflow::Status Pack(absl::Span<ImmediateExecutionTensorHandle*> handles,
-                          ImmediateExecutionTensorHandle** result) override {
+  absl::Status Pack(absl::Span<ImmediateExecutionTensorHandle*> handles,
+                    ImmediateExecutionTensorHandle** result) override {
     TF_Status status;
     *result = tensorflow::unwrap(device_.pack(context_,
                                               tensorflow::wrap(handles.data()),
@@ -481,7 +497,7 @@ class CustomDeviceAPI : public tensorflow::CustomDevice {
     return status.status;
   }
 
-  tensorflow::StatusOr<bool> ShallPinToThisDevice(
+  absl::StatusOr<bool> ShallPinToThisDevice(
       const ImmediateExecutionOperation* op) override {
     TF_Status status;
     // Let this custom device choose the device to pin this op on if it
@@ -514,12 +530,12 @@ class CAPICustomDeviceTensorHandle
 
   ~CAPICustomDeviceTensorHandle() override { methods_.deallocator(data_); }
   void* DevicePointer() const override { return data_; }
-  Status NumDims(int* num_dims) const override {
+  absl::Status NumDims(int* num_dims) const override {
     TF_Status s;
     *num_dims = methods_.num_dims(data_, &s);
     return s.status;
   }
-  Status Dim(int dim_index, int64_t* dim) const override {
+  absl::Status Dim(int dim_index, int64_t* dim) const override {
     TF_Status s;
     *dim = methods_.dim(data_, dim_index, &s);
     return s.status;
@@ -529,7 +545,7 @@ class CAPICustomDeviceTensorHandle
     return methods_.summarize != nullptr;
   }
 
-  Status SummarizeValue(std::string& summary) const override {
+  absl::Status SummarizeValue(std::string& summary) const override {
     if (methods_.summarize == nullptr) {
       return tensorflow::CustomDeviceTensorHandle::SummarizeValue(summary);
     }
@@ -541,7 +557,7 @@ class CAPICustomDeviceTensorHandle
     }
     summary = std::string(reinterpret_cast<const char*>(summary_buffer->data),
                           summary_buffer->length);
-    return OkStatus();
+    return absl::OkStatus();
   }
 
  private:
