@@ -17,6 +17,7 @@ limitations under the License.
 
 #include <string.h>
 
+#include <memory>
 #include <string>
 
 // clang-format off
@@ -31,11 +32,15 @@ limitations under the License.
 #include "tensorflow/c/eager/c_api_test_util.h"
 #include "tensorflow/c/eager/tfe_op_internal.h"
 #include "tensorflow/c/eager/tfe_tensorhandle_internal.h"
+#include "tensorflow/c/tf_datatype.h"
+#include "tensorflow/c/tf_status.h"
+#include "tensorflow/c/tf_tensor.h"
 #include "tensorflow/core/common_runtime/eager/eager_operation.h"
 #include "tensorflow/core/common_runtime/eager/tensor_handle.h"
 #include "tensorflow/core/distributed_runtime/rpc/grpc_server_lib.h"
 #include "tensorflow/core/framework/function.pb.h"
 #include "tensorflow/core/platform/casts.h"
+#include "tensorflow/core/platform/env.h"
 #include "tensorflow/core/platform/logging.h"
 #include "tensorflow/core/platform/macros.h"
 #include "tensorflow/core/platform/protobuf.h"
@@ -982,7 +987,7 @@ REGISTER_KERNEL_BUILDER(
     Name("TestCommUnavailable").Device(tensorflow::DEVICE_DEFAULT),
     TestUnavailableErrorOp);
 
-string FunctionWithErrorOp(const tensorflow::StringPiece op_name) {
+string FunctionWithErrorOp(const absl::string_view op_name) {
   const std::string& func_str =
       "    signature {"
       "      name: 'FunctionWith__OP_NAME__'"
@@ -2227,7 +2232,16 @@ tensorflow::ServerDef GetClusterServerDef(const string& worker_job_name,
   return server_def;
 }
 
-TEST(CAPI, SingleHostServerDefWorks) {
+// This test verifies the following:
+// 1. Start the GRPC server for worker 1 using single host server def A with
+//    only worker 1.
+// 2. Create a context B using A.
+// 3. Create the variable in B.
+// 4. Create another single host server def C with only worker 0.
+// 5. Start the GRPC server for worker 0 using C.
+// 6. Create a context D using the full cluster server def E.
+// 7. Read the variable in D.
+TEST(CAPI, SingleHostServerDefV1Works) {
   // Create a server def that represents a 2-process cluster and a client.
   // Example:
   //
@@ -2353,4 +2367,108 @@ TEST(CAPI, SingleHostServerDefWorks) {
   worker_server0.release();
 }
 
+// This test verifies the following:
+// 1. Start the GRPC servers for both worker 0 and 1 using the full cluster
+//    server def A.
+// 2. Create a context B using the full cluster server def A.
+// 3. Create the variable in B.
+// 4. Create a context C using a single host server def D with only worker 1.
+// 5. Read the variable in C.
+TEST(CAPI, SingleHostServerDefV2Works) {
+  tensorflow::ServerDef cluster_server_def = GetClusterServerDef("worker", 2);
+
+  cluster_server_def.set_task_index(0);
+  cluster_server_def.set_job_name("worker");
+  std::unique_ptr<tensorflow::GrpcServer> worker_server0;
+  ASSERT_TRUE(tensorflow::GrpcServer::Create(cluster_server_def,
+                                             tensorflow::Env::Default(),
+                                             &worker_server0)
+                  .ok());
+  ASSERT_TRUE(worker_server0->Start().ok());
+
+  cluster_server_def.set_task_index(1);
+  cluster_server_def.set_job_name("worker");
+  std::unique_ptr<tensorflow::GrpcServer> worker_server1;
+  ASSERT_TRUE(tensorflow::GrpcServer::Create(cluster_server_def,
+                                             tensorflow::Env::Default(),
+                                             &worker_server1)
+                  .ok());
+  ASSERT_TRUE(worker_server1->Start().ok());
+
+  // Create a context for the whole cluster using cluster server def.
+  // This is initiated from the client.
+  cluster_server_def.set_task_index(0);
+  cluster_server_def.set_job_name("client");
+  TFE_Context* ctx_with_cluster_server_def =
+      CreateContext(cluster_server_def.SerializeAsString(),
+                    /*isolate_session_state=*/false,
+                    /*init_timeout_in_ms=*/0);
+
+  const char worker_1_device[] = "/job:worker/replica:0/task:1/device:CPU:0";
+
+  // Create a variable `var` using `ctx_with_cluster_server_def`.
+  TFE_TensorHandle* handle_0 =
+      CreateVariable(ctx_with_cluster_server_def, 1.2, worker_1_device,
+                     /*variable_name=*/"var");
+  TF_Status* status = TF_NewStatus();
+  TFE_ContextAsyncWait(ctx_with_cluster_server_def, status);
+  EXPECT_EQ(TF_OK, TF_GetCode(status)) << TF_Message(status);
+  TF_DeleteStatus(status);
+  TFE_DeleteTensorHandle(handle_0);
+
+  tensorflow::ServerDef worker_1_server_def =
+      CreateSingleHostServerDef(cluster_server_def, 1);
+
+  // Create a new context for worker 1 using single host server def.
+  // This is initiated from the client.
+  worker_1_server_def.set_task_index(0);
+  worker_1_server_def.set_job_name("client");
+  TFE_Context* ctx_with_worker_1_server_def =
+      CreateContext(worker_1_server_def.SerializeAsString(),
+                    /*isolate_session_state=*/false,
+                    /*init_timeout_in_ms=*/0);
+
+  // Read the variable `var` using `ctx_with_worker_1_server_def`.
+  {
+    TFE_TensorHandle* var_handle = CreateVarHandle(
+        ctx_with_worker_1_server_def, worker_1_device, /*variable_name=*/"var");
+
+    TFE_TensorHandle* handle_1 = nullptr;
+    int num_retvals = 1;
+    TF_Status* status = TF_NewStatus();
+    TFE_Op* op =
+        TFE_NewOp(ctx_with_worker_1_server_def, "ReadVariableOp", status);
+    ASSERT_EQ(TF_OK, TF_GetCode(status)) << TF_Message(status);
+    TFE_OpSetAttrType(op, "dtype", TF_FLOAT);
+    TFE_OpAddInput(op, var_handle, status);
+    ASSERT_EQ(TF_OK, TF_GetCode(status)) << TF_Message(status);
+    TFE_Execute(op, &handle_1, &num_retvals, status);
+    ASSERT_EQ(TF_OK, TF_GetCode(status)) << TF_Message(status);
+    TFE_DeleteOp(op);
+
+    ASSERT_EQ(1, num_retvals);
+    EXPECT_EQ(TF_FLOAT, TFE_TensorHandleDataType(handle_1));
+    EXPECT_EQ(0, TFE_TensorHandleNumDims(handle_1, status));
+    ASSERT_EQ(TF_OK, TF_GetCode(status)) << TF_Message(status);
+
+    // Read the value of tensor handle `handle_1`.
+    float value = 0.0f;
+    TF_Tensor* t = TFE_TensorHandleResolve(handle_1, status);
+    ASSERT_EQ(TF_OK, TF_GetCode(status)) << TF_Message(status);
+    ASSERT_EQ(sizeof(float), TF_TensorByteSize(t));
+    memcpy(&value, TF_TensorData(t), sizeof(float));
+    TF_DeleteTensor(t);
+    EXPECT_EQ(1.2f, value);
+    TFE_DeleteTensorHandle(handle_1);
+    TF_DeleteStatus(status);
+    TFE_DeleteTensorHandle(var_handle);
+  }
+
+  TFE_DeleteContext(ctx_with_worker_1_server_def);
+  TFE_DeleteContext(ctx_with_cluster_server_def);
+
+  worker_server1.release();
+  worker_server0.release();
+}
+
 }  // namespace
