@@ -15,17 +15,21 @@ limitations under the License.
 
 #include "tensorflow/c/eager/parallel_device/parallel_device.h"
 
-#include <cstring>
+#include <cstdint>
 #include <memory>
+#include <string>
+#include <utility>
+#include <vector>
 
+#include "absl/status/status.h"
 #include "absl/strings/str_cat.h"
 #include "absl/types/optional.h"
 #include "absl/types/variant.h"
-#include "tensorflow/c/c_api.h"
 #include "tensorflow/c/eager/c_api.h"
 #include "tensorflow/c/eager/c_api_experimental.h"
 #include "tensorflow/c/eager/parallel_device/parallel_device_lib.h"
 #include "tensorflow/c/eager/tfe_tensorhandle_internal.h"
+#include "tensorflow/c/tf_buffer.h"
 #include "tensorflow/c/tf_status.h"
 #include "tensorflow/c/tf_status_helper.h"
 
@@ -209,7 +213,7 @@ void ParallelTensorDeallocator(void* data) {
 // number of dimensions of a parallel tensor.
 int ParallelTensorNumDims(void* data, TF_Status* status) {
   const std::vector<int64_t>* shape;
-  Status s = reinterpret_cast<ParallelTensor*>(data)->Shape(&shape);
+  absl::Status s = reinterpret_cast<ParallelTensor*>(data)->Shape(&shape);
   if (!s.ok()) {
     tsl::Set_TF_Status_from_Status(status, s);
     return -1;
@@ -221,7 +225,7 @@ int ParallelTensorNumDims(void* data, TF_Status* status) {
 // dimension of a parallel tensor.
 int64_t ParallelTensorDim(void* data, int dim_index, TF_Status* status) {
   const std::vector<int64_t>* shape;
-  Status s = reinterpret_cast<ParallelTensor*>(data)->Shape(&shape);
+  absl::Status s = reinterpret_cast<ParallelTensor*>(data)->Shape(&shape);
   if (!s.ok()) {
     tsl::Set_TF_Status_from_Status(status, s);
     return -1;
@@ -232,7 +236,7 @@ int64_t ParallelTensorDim(void* data, int dim_index, TF_Status* status) {
 TF_Buffer* ParallelTensorSummarize(void* data, TF_Status* status) {
   ParallelTensor* parallel_tensor = reinterpret_cast<ParallelTensor*>(data);
   std::string summary;
-  Status cpp_status = parallel_tensor->SummarizeValue(summary);
+  absl::Status cpp_status = parallel_tensor->SummarizeValue(summary);
   if (!cpp_status.ok()) {
     tsl::Set_TF_Status_from_Status(status, cpp_status);
     return nullptr;
