@@ -17,9 +17,24 @@ limitations under the License.
 #include <stdlib.h>
 #include <string.h>
 
+#include <algorithm>
+#include <cstdint>
+#include <cstdlib>
+#include <functional>
+#include <ios>
+#include <memory>
+#include <string>
+#include <utility>
+#include <variant>
+#include <vector>
+
+#include "absl/base/thread_annotations.h"
 #include "absl/strings/numbers.h"
 #include "absl/strings/str_cat.h"
-#include "absl/types/variant.h"
+#include "absl/synchronization/mutex.h"
+#include "third_party/cloud_cpp/google/cloud/common_options.h"
+#include "third_party/cloud_cpp/google/cloud/options.h"
+#include "third_party/cloud_cpp/google/cloud/status_or.h"
 #include "google/cloud/storage/client.h"
 #include "tensorflow/c/env.h"
 #include "tensorflow/c/experimental/filesystem/plugins/gcs/gcs_helper.h"
@@ -60,6 +75,9 @@ constexpr char kAppendMode[] = "GCS_APPEND_MODE";
 // objects.
 constexpr char kComposeAppend[] = "compose";
 
+// User-agent to be passed to GCS requests.
+constexpr char kUserAgent[] = "TensorFlow-C-API";
+
 // We can cast `google::cloud::StatusCode` to `TF_Code` because they have the
 // same integer values. See
 // https://github.com/googleapis/google-cloud-cpp/blob/6c09cbfa0160bc046e5509b4dd2ab4b872648b4a/google/cloud/status.h#L32-L52
@@ -425,7 +443,7 @@ namespace tf_gcs_filesystem {
 // TODO(vnvo2409): We could do some cleanups like `return TF_SetStatus`.
 // TODO(vnvo2409): Refactor the filesystem implementation when
 // https://github.com/googleapis/google-cloud-cpp/issues/4482 is done.
-GCSFile::GCSFile(google::cloud::storage::Client&& gcs_client)
+GCSFile::GCSFile(gcs::Client&& gcs_client)
     : gcs_client(gcs_client), block_cache_lock() {
   const char* append_mode = std::getenv(kAppendMode);
   compose = (append_mode != nullptr) && (!strcmp(kAppendMode, append_mode));
@@ -494,12 +512,19 @@ GCSFile::GCSFile(google::cloud::storage::Client&& gcs_client, bool compose,
       stat_cache_max_age, stat_cache_max_entries);
 }
 
+// Set the UserAgent to identify the client for storage.
+static google::cloud::StatusOr<gcs::Client> GetStorageClient() {
+  google::cloud::Options options =
+      google::cloud::Options{}.set<google::cloud::UserAgentProductsOption>(
+          {kUserAgent});
+  return gcs::Client(options);
+}
+
 void InitTest(TF_Filesystem* filesystem, bool compose, uint64_t block_size,
               size_t max_bytes, uint64_t max_staleness,
               uint64_t stat_cache_max_age, size_t stat_cache_max_entries,
               TF_Status* status) {
-  google::cloud::StatusOr<gcs::Client> client =
-      gcs::Client::CreateDefaultClient();
+  google::cloud::StatusOr<gcs::Client> client = GetStorageClient();
   if (!client) {
     TF_SetStatusFromGCSStatus(client.status(), status);
     return;
@@ -512,8 +537,7 @@ void InitTest(TF_Filesystem* filesystem, bool compose, uint64_t block_size,
 }
 
 void Init(TF_Filesystem* filesystem, TF_Status* status) {
-  google::cloud::StatusOr<gcs::Client> client =
-      gcs::Client::CreateDefaultClient();
+  google::cloud::StatusOr<gcs::Client> client = GetStorageClient();
   if (!client) {
     TF_SetStatusFromGCSStatus(client.status(), status);
     return;
@@ -771,9 +795,9 @@ static std::vector<std::string> GetChildrenBounded(
       return result;
     }
     auto value = *std::move(item);
-    std::string children = absl::holds_alternative<std::string>(value)
-                               ? absl::get<std::string>(value)
-                               : absl::get<gcs::ObjectMetadata>(value).name();
+    std::string children = std::holds_alternative<std::string>(value)
+                               ? std::get<std::string>(value)
+                               : std::get<gcs::ObjectMetadata>(value).name();
     auto pos = children.find(prefix);
     if (pos != 0) {
       TF_SetStatus(status, TF_INTERNAL,
