@@ -13,9 +13,11 @@ See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
 
+#include "absl/log/log.h"
 #include "tensorflow/c/experimental/filesystem/filesystem_interface.h"
 #include "tensorflow/c/experimental/filesystem/modular_filesystem_registration.h"
 #include "tensorflow/c/experimental/filesystem/plugins/posix/posix_filesystem.h"
+#include "tensorflow/core/platform/status.h"
 
 namespace tensorflow {
 
@@ -24,7 +26,8 @@ namespace tensorflow {
 bool StaticallyRegisterLocalFilesystems() {
   TF_FilesystemPluginInfo info;
   TF_InitPlugin(&info);
-  Status status = filesystem_registration::RegisterFilesystemPluginImpl(&info);
+  absl::Status status =
+      filesystem_registration::RegisterFilesystemPluginImpl(&info);
   if (!status.ok()) {
     VLOG(0) << "Static POSIX filesystem could not be registered: " << status;
     return false;
