@@ -51,15 +51,16 @@ using StructuredValueDictEntry =
 // Maps from a Nodedef's name to its corresponding AttrValues, for a given
 // Graphdef
 using NodeAttrMap =
-    gtl::FlatMap<StringPiece, const AttrValueMap*, StringPieceHasher>;
+    gtl::FlatMap<absl::string_view, const AttrValueMap*, StringPieceHasher>;
 
 // Maps from a FunctionDef's name to FunctionDef, for a given FunctionDefLibrary
-using FunctionDefMap = gtl::FlatMap<StringPiece, const tensorflow::FunctionDef*,
-                                    StringPieceHasher>;
+using FunctionDefMap =
+    gtl::FlatMap<absl::string_view, const tensorflow::FunctionDef*,
+                 StringPieceHasher>;
 
 // Looks up a SavedConstant's associated tensorproto from the NodeAttrMap and
 // returns a tensorflow::Constant.
-Status ConstantFromSavedConstant(
+absl::Status ConstantFromSavedConstant(
     ImmediateExecutionContext* ctx,
     const tensorflow::SavedConstant& saved_constant,
     const NodeAttrMap& node_attr_map, std::unique_ptr<Constant>* output) {
@@ -83,7 +84,7 @@ Status ConstantFromSavedConstant(
 // Perform some basic sanity checks on SavedConcreteFunction's input and
 // output signatures with respect to the corresponding FunctionDef's input
 // and output args.
-Status ValidateSavedFunctionCompatibleWithFunctionDef(
+absl::Status ValidateSavedFunctionCompatibleWithFunctionDef(
     const SavedConcreteFunction& saved_concrete_function,
     const FunctionDef* function_def) {
   // tf.functions go through many transformations before becoming FunctionDefs
@@ -135,13 +136,13 @@ Status ValidateSavedFunctionCompatibleWithFunctionDef(
         " flattened outputs.");
   }
 
-  return Status();
+  return absl::Status();
 }
 
 }  // namespace
 
-Status GetSignaturesMap(const SavedObjectGraph& saved_objects,
-                        gtl::FlatMap<std::string, int>* signatures_map) {
+absl::Status GetSignaturesMap(const SavedObjectGraph& saved_objects,
+                              gtl::FlatMap<std::string, int>* signatures_map) {
   if (saved_objects.nodes().empty()) {
     return errors::FailedPrecondition("Saved Object Graph was empty.");
   }
@@ -179,10 +180,11 @@ Status GetSignaturesMap(const SavedObjectGraph& saved_objects,
   for (const auto& child : signatures->children()) {
     (*signatures_map)[child.local_name()] = child.node_id();
   }
-  return Status();
+  return absl::Status();
 }
 
-Status ValidateSingleConcreteFunction(const SavedFunction& saved_function) {
+absl::Status ValidateSingleConcreteFunction(
+    const SavedFunction& saved_function) {
   // We only allow loading functions that have an annotated input signature,
   // which means there is 1:1 correspondence between tf.function
   // <=> SavedFunction <=> SavedConcreteFunction <=> FunctionDef. This is
@@ -194,13 +196,14 @@ Status ValidateSingleConcreteFunction(const SavedFunction& saved_function) {
         "by SavedModelAPI. This means that there should only be a single "
         "ConcreteFunction per tf.function");
   }
-  return Status();
+  return absl::Status();
 }
 
-Status LoadSavedAsset(ImmediateExecutionContext* ctx, const SavedAsset& asset,
-                      const std::string& saved_model_dir,
-                      absl::Span<const AssetFileDef> assets,
-                      std::unique_ptr<Asset>* output) {
+absl::Status LoadSavedAsset(ImmediateExecutionContext* ctx,
+                            const SavedAsset& asset,
+                            const std::string& saved_model_dir,
+                            absl::Span<const AssetFileDef> assets,
+                            std::unique_ptr<Asset>* output) {
   int asset_index = asset.asset_file_def_index();
   if (asset_index >= assets.size()) {
     return errors::FailedPrecondition(
@@ -211,9 +214,9 @@ Status LoadSavedAsset(ImmediateExecutionContext* ctx, const SavedAsset& asset,
   return Asset::Create(ctx, saved_model_dir, asset_filename, output);
 }
 
-Status TensorProtoToConstant(ImmediateExecutionContext* ctx,
-                             const TensorProto& proto,
-                             std::unique_ptr<Constant>* output) {
+absl::Status TensorProtoToConstant(ImmediateExecutionContext* ctx,
+                                   const TensorProto& proto,
+                                   std::unique_ptr<Constant>* output) {
   tensorflow::Tensor tensor;
   bool parse_result = tensor.FromProto(proto);
   if (!parse_result) {
@@ -226,9 +229,9 @@ Status TensorProtoToConstant(ImmediateExecutionContext* ctx,
 
 // This follows the python variable restoration logic:
 // https://github.com/tensorflow/tensorflow/blob/516608035f85cec8b126712b0ff8407220206b22/tensorflow/python/saved_model/load.py#L407
-Status LoadSavedVariable(ImmediateExecutionContext* ctx,
-                         const SavedVariable& variable,
-                         std::unique_ptr<Variable>* output) {
+absl::Status LoadSavedVariable(ImmediateExecutionContext* ctx,
+                               const SavedVariable& variable,
+                               std::unique_ptr<Variable>* output) {
   const std::string& name = variable.name();
   tensorflow::TensorShape shape(variable.shape());
   tensorflow::DataType dtype = variable.dtype();
@@ -243,10 +246,10 @@ Status LoadSavedVariable(ImmediateExecutionContext* ctx,
       ctx, dtype, shape, name,
       variable.device().empty() ? nullptr : variable.device().c_str(),
       component_devices, output));
-  return Status();
+  return absl::Status();
 }
 
-Status LoadTFConcreteFunction(
+absl::Status LoadTFConcreteFunction(
     const SavedConcreteFunction& saved_concrete_function,
     const FunctionDef* function_def,
     const std::unordered_map<int, std::unique_ptr<TensorHandleConvertible>>&
@@ -272,8 +275,9 @@ Status LoadTFConcreteFunction(
                                     out);
 }
 
-Status FlattenSignature(const StructuredValue& signature,
-                        std::vector<const TensorSpecProto*>* flattened_specs) {
+absl::Status FlattenSignature(
+    const StructuredValue& signature,
+    std::vector<const TensorSpecProto*>* flattened_specs) {
   // This follows the logic from
   // https://github.com/tensorflow/tensorflow/blob/1c064ab76064c58e54261b805027474885a1534d/tensorflow/compiler/mlir/tensorflow/translate/import_model.cc#L2775
   switch (signature.kind_case()) {
@@ -295,31 +299,31 @@ Status FlattenSignature(const StructuredValue& signature,
       for (const auto& entry : entries) {
         TF_RETURN_IF_ERROR(FlattenSignature(entry->second, flattened_specs));
       }
-      return Status();
+      return absl::Status();
     }
     case StructuredValue::kTupleValue: {
       const TupleValue& tuple = signature.tuple_value();
       for (const StructuredValue& value : tuple.values()) {
         TF_RETURN_IF_ERROR(FlattenSignature(value, flattened_specs));
       }
-      return Status();
+      return absl::Status();
     }
     case StructuredValue::kListValue: {
       const ListValue& list = signature.list_value();
       for (const StructuredValue& value : list.values()) {
         TF_RETURN_IF_ERROR(FlattenSignature(value, flattened_specs));
       }
-      return Status();
+      return absl::Status();
     }
     case StructuredValue::kTensorSpecValue: {
       flattened_specs->push_back(&signature.tensor_spec_value());
-      return Status();
+      return absl::Status();
     }
     case StructuredValue::kNoneValue: {
       // Base case: do nothing.
       // This arises, for example, as the top-level object of an output
       // signature when there are no return values.
-      return Status();
+      return absl::Status();
     }
     default: {
       return errors::Internal("Unhandled structured value kind ",
@@ -328,7 +332,7 @@ Status FlattenSignature(const StructuredValue& signature,
   }
 }
 
-absl::optional<int> FindNodeAtPath(StringPiece path,
+absl::optional<int> FindNodeAtPath(absl::string_view path,
                                    const SavedObjectGraph& object_graph) {
   const auto& nodes = object_graph.nodes();
   if (nodes.empty()) {
@@ -358,18 +362,21 @@ absl::optional<int> FindNodeAtPath(StringPiece path,
   return node_id;
 }
 
-gtl::FlatMap<StringPiece, const AttrValueMap*, StringPieceHasher> NodeToAttrMap(
-    const tensorflow::GraphDef& graphdef) {
-  gtl::FlatMap<StringPiece, const AttrValueMap*, StringPieceHasher> result;
+gtl::FlatMap<absl::string_view, const AttrValueMap*, StringPieceHasher>
+NodeToAttrMap(const tensorflow::GraphDef& graphdef) {
+  gtl::FlatMap<absl::string_view, const AttrValueMap*, StringPieceHasher>
+      result;
   for (const tensorflow::NodeDef& node : graphdef.node()) {
     result[node.name()] = &node.attr();
   }
   return result;
 }
 
-gtl::FlatMap<StringPiece, const tensorflow::FunctionDef*, StringPieceHasher>
+gtl::FlatMap<absl::string_view, const tensorflow::FunctionDef*,
+             StringPieceHasher>
 FunctionNameToFunctionDefMap(const FunctionDefLibrary& library) {
-  gtl::FlatMap<StringPiece, const tensorflow::FunctionDef*, StringPieceHasher>
+  gtl::FlatMap<absl::string_view, const tensorflow::FunctionDef*,
+               StringPieceHasher>
       result;
   for (const FunctionDef& function_def : library.function()) {
     result[function_def.signature().name()] = &function_def;
@@ -377,10 +384,9 @@ FunctionNameToFunctionDefMap(const FunctionDefLibrary& library) {
   return result;
 }
 
-Status PartiallyReviveSavedModelObjects(const MetaGraphDef& metagraph,
-                                        ImmediateExecutionContext* context,
-                                        const std::string& directory,
-                                        PartiallyRevivedObjects* objects) {
+absl::Status PartiallyReviveSavedModelObjects(
+    const MetaGraphDef& metagraph, ImmediateExecutionContext* context,
+    const std::string& directory, PartiallyRevivedObjects* objects) {
   // This is needed to restore "Constant" nodes by looking up their
   // "Value" attribute.
   NodeAttrMap node_attr_map = NodeToAttrMap(metagraph.graph_def());
@@ -527,7 +533,7 @@ Status PartiallyReviveSavedModelObjects(const MetaGraphDef& metagraph,
 
   objects->signatures_map = std::move(signatures_map);
 
-  return Status();
+  return absl::Status();
 }
 
 }  // namespace internal
