@@ -15,10 +15,13 @@ limitations under the License.
 
 #include "tensorflow/c/experimental/saved_model/core/tensor_spec.h"
 
+#include <cstdint>
 #include <initializer_list>
+#include <utility>
 
 #include "tensorflow/core/framework/tensor_shape.h"
 #include "tensorflow/core/framework/types.pb.h"
+#include "tensorflow/core/protobuf/struct.pb.h"
 
 namespace tensorflow {
 
