@@ -15,9 +15,11 @@ limitations under the License.
 
 #include "tensorflow/c/experimental/saved_model/public/saved_model_api.h"
 
+#include <cstdint>
 #include <string>
 #include <vector>
 
+#include "absl/status/status.h"
 #include "tensorflow/c/eager/c_api.h"
 #include "tensorflow/c/eager/c_api_experimental.h"
 #include "tensorflow/c/eager/c_api_test_util.h"
@@ -33,6 +35,7 @@ limitations under the License.
 #include "tensorflow/c/tf_shape.h"
 #include "tensorflow/c/tf_status.h"
 #include "tensorflow/c/tf_tensor.h"
+#include "tensorflow/core/framework/types.pb.h"
 #include "tensorflow/core/lib/io/path.h"
 #include "tensorflow/core/platform/status.h"
 #include "tensorflow/core/platform/stringpiece.h"
@@ -46,7 +49,7 @@ using tensorflow::tstring;
 constexpr char kTestData[] = "cc/saved_model/testdata";
 const char* kServeTag[] = {"serve"};
 
-std::string SavedModelPath(tensorflow::StringPiece saved_model_dir) {
+std::string SavedModelPath(absl::string_view saved_model_dir) {
   return tensorflow::io::JoinPath(tensorflow::testing::TensorFlowSrcRoot(),
                                   kTestData, saved_model_dir);
 }
@@ -506,12 +509,11 @@ TEST_P(CSavedModelAPITest, LoadSavedModelWithUninitializedVariable) {
       tensorflow::down_cast<tensorflow::TFSavedModelAPI*>(
           tensorflow::unwrap(saved_model));
   tensorflow::Variable* uninitialized_variable;
-  ASSERT_EQ(::tensorflow::OkStatus(),
-            model_api->GetVariable("uninitialized_variable",
-                                   &uninitialized_variable));
+  ASSERT_EQ(absl::OkStatus(), model_api->GetVariable("uninitialized_variable",
+                                                     &uninitialized_variable));
   ASSERT_EQ(tensorflow::DT_FLOAT, uninitialized_variable->dtype());
 
-  ASSERT_EQ(::tensorflow::OkStatus(),
+  ASSERT_EQ(absl::OkStatus(),
             model_api->GetVariable("sub_module.uninitialized_variable",
                                    &uninitialized_variable));
   ASSERT_EQ(tensorflow::DT_INT64, uninitialized_variable->dtype());
