@@ -21,26 +21,43 @@ limitations under the License.
 // device.
 #include "tensorflow/c/experimental/stream_executor/stream_executor.h"
 
+#include <cstdint>
+#include <memory>
+#include <optional>
 #include <string>
 #include <utility>
+#include <variant>
 
 #include "absl/functional/any_invocable.h"
+#include "absl/log/check.h"
+#include "absl/log/log.h"
+#include "absl/status/status.h"
+#include "absl/status/statusor.h"
+#include "absl/strings/str_format.h"
+#include "absl/types/optional.h"
 #include "tensorflow/c/c_api_macros.h"
 #include "tensorflow/c/c_api_macros_internal.h"
 #include "tensorflow/c/experimental/stream_executor/stream_executor_internal.h"
+#include "tensorflow/c/tf_status.h"
 #include "tensorflow/c/tf_status_helper.h"
-#include "tensorflow/compiler/xla/stream_executor/executor_cache.h"
-#include "tensorflow/compiler/xla/stream_executor/multi_platform_manager.h"
-#include "tensorflow/compiler/xla/stream_executor/platform.h"
-#include "tensorflow/compiler/xla/stream_executor/stream.h"
-#include "tensorflow/compiler/xla/stream_executor/stream_executor_internal.h"
-#include "tensorflow/compiler/xla/stream_executor/stream_executor_pimpl.h"
+#include "xla/stream_executor/allocator_stats.h"
+#include "xla/stream_executor/device_description.h"
+#include "xla/stream_executor/device_memory.h"
+#include "xla/stream_executor/event.h"
+#include "xla/stream_executor/executor_cache.h"
+#include "xla/stream_executor/generic_memory_allocation.h"
+#include "xla/stream_executor/generic_memory_allocator.h"
+#include "xla/stream_executor/memory_allocation.h"
+#include "xla/stream_executor/memory_allocator.h"
+#include "xla/stream_executor/platform.h"
+#include "xla/stream_executor/platform_manager.h"
+#include "xla/stream_executor/stream.h"
+#include "xla/stream_executor/stream_executor.h"
+#include "xla/stream_executor/stream_executor_common.h"
+#include "xla/tsl/platform/errors.h"
+#include "xla/tsl/platform/status.h"
 #include "tensorflow/core/common_runtime/device/device_utils.h"
 #include "tensorflow/core/platform/env.h"
-#include "tensorflow/core/platform/errors.h"
-#include "tensorflow/core/platform/logging.h"
-#include "tensorflow/core/platform/status.h"
-#include "tensorflow/core/platform/strcat.h"
 #include "tensorflow/core/platform/stringpiece.h"
 
 using tensorflow::StatusFromTF_Status;
@@ -52,7 +69,7 @@ using tensorflow::StringPiece;
 using OwnedTFStatus = tensorflow::TF_StatusPtr;
 
 namespace {
-tsl::Status ValidateSPPlatform(const SP_Platform& platform) {
+absl::Status ValidateSPPlatform(const SP_Platform& platform) {
   TF_VALIDATE_STRUCT_SIZE(SP_Platform, platform, SP_PLATFORM_STRUCT_SIZE);
   TF_VALIDATE_NOT_NULL(SP_Platform, platform, name);
   TF_VALIDATE_NOT_NULL(SP_Platform, platform, type);
@@ -61,10 +78,10 @@ tsl::Status ValidateSPPlatform(const SP_Platform& platform) {
   TF_RETURN_IF_ERROR(
       tensorflow::device_utils::ValidateDeviceType(platform.type));
   // `visible_device_count` could be 0 at initialization time.
-  return ::tensorflow::OkStatus();
+  return absl::OkStatus();
 }
 
-tsl::Status ValidateSPPlatformFns(const SP_PlatformFns& platform_fns) {
+absl::Status ValidateSPPlatformFns(const SP_PlatformFns& platform_fns) {
   TF_VALIDATE_STRUCT_SIZE(SP_PlatformFns, platform_fns,
                           SP_PLATFORM_FNS_STRUCT_SIZE);
   TF_VALIDATE_NOT_NULL(SP_PlatformFns, platform_fns, create_device);
@@ -73,37 +90,37 @@ tsl::Status ValidateSPPlatformFns(const SP_PlatformFns& platform_fns) {
   TF_VALIDATE_NOT_NULL(SP_PlatformFns, platform_fns, destroy_stream_executor);
   TF_VALIDATE_NOT_NULL(SP_PlatformFns, platform_fns, create_device_fns);
   TF_VALIDATE_NOT_NULL(SP_PlatformFns, platform_fns, destroy_device_fns);
-  return ::tensorflow::OkStatus();
+  return absl::OkStatus();
 }
 
-tsl::Status ValidateSPAllocatorStats(const SP_AllocatorStats& stats) {
+absl::Status ValidateSPAllocatorStats(const SP_AllocatorStats& stats) {
   TF_VALIDATE_STRUCT_SIZE(SP_AllocatorStats, stats,
                           SP_ALLOCATORSTATS_STRUCT_SIZE);
   // All other fields could theoretically be zero/null.
-  return ::tensorflow::OkStatus();
+  return absl::OkStatus();
 }
 
-tsl::Status ValidateSPDeviceMemoryBase(const SP_DeviceMemoryBase& mem) {
+absl::Status ValidateSPDeviceMemoryBase(const SP_DeviceMemoryBase& mem) {
   TF_VALIDATE_STRUCT_SIZE(SP_DeviceMemoryBase, mem,
                           SP_DEVICE_MEMORY_BASE_STRUCT_SIZE);
   // All other fields could theoretically be zero/null.
-  return ::tensorflow::OkStatus();
+  return absl::OkStatus();
 }
 
-tsl::Status ValidateSPDevice(const SP_Device& device) {
+absl::Status ValidateSPDevice(const SP_Device& device) {
   TF_VALIDATE_STRUCT_SIZE(SP_Device, device, SP_DEVICE_STRUCT_SIZE);
   // All other fields could theoretically be zero/null.
-  return ::tensorflow::OkStatus();
+  return absl::OkStatus();
 }
 
-tsl::Status ValidateSPDeviceFns(const SP_DeviceFns& device_fns) {
+absl::Status ValidateSPDeviceFns(const SP_DeviceFns& device_fns) {
   TF_VALIDATE_STRUCT_SIZE(SP_DeviceFns, device_fns, SP_DEVICE_FNS_STRUCT_SIZE);
   // All other fields could theoretically be zero/null.
-  return ::tensorflow::OkStatus();
+  return absl::OkStatus();
 }
 
-tsl::Status ValidateSPStreamExecutor(const SP_StreamExecutor& se,
-                                     const SP_Platform& platform) {
+absl::Status ValidateSPStreamExecutor(const SP_StreamExecutor& se,
+                                      const SP_Platform& platform) {
   TF_VALIDATE_STRUCT_SIZE(SP_StreamExecutor, se,
                           SP_STREAM_EXECUTOR_STRUCT_SIZE);
   TF_VALIDATE_NOT_NULL(SP_StreamExecutor, se, allocate);
@@ -135,45 +152,20 @@ tsl::Status ValidateSPStreamExecutor(const SP_StreamExecutor& se,
   TF_VALIDATE_NOT_NULL(SP_StreamExecutor, se, mem_zero);
   TF_VALIDATE_NOT_NULL(SP_StreamExecutor, se, memset);
   TF_VALIDATE_NOT_NULL(SP_StreamExecutor, se, memset32);
-  return ::tensorflow::OkStatus();
+  return absl::OkStatus();
 }
 
-tsl::Status ValidateSEPlatformRegistrationParams(
+absl::Status ValidateSEPlatformRegistrationParams(
     const SE_PlatformRegistrationParams& params) {
   TF_VALIDATE_STRUCT_SIZE(SE_PlatformRegistrationParams, params,
                           SE_PLATFORM_REGISTRATION_PARAMS_STRUCT_SIZE);
   TF_VALIDATE_NOT_NULL(SE_PlatformRegistrationParams, params, destroy_platform);
   TF_VALIDATE_NOT_NULL(SE_PlatformRegistrationParams, params,
                        destroy_platform_fns);
-  return ::tensorflow::OkStatus();
+  return absl::OkStatus();
 }
 #undef TF_VALIDATE_NOT_NULL
 
-// Converts SE_EventStatus to Event::Status.
-Event::Status SEEventStatusToEventStatus(SE_EventStatus s) {
-  switch (s) {
-    case SE_EVENT_ERROR:
-      return Event::Status::kError;
-    case SE_EVENT_PENDING:
-      return Event::Status::kPending;
-    case SE_EVENT_COMPLETE:
-      return Event::Status::kComplete;
-    default:
-      return Event::Status::kUnknown;
-  }
-}
-
-// Converts DeviceMemoryBase to a C struct.
-SP_DeviceMemoryBase DeviceMemoryBaseToC(const DeviceMemoryBase* mem) {
-  SP_DeviceMemoryBase device_memory_base{SP_DEVICE_MEMORY_BASE_STRUCT_SIZE};
-  // `opaque` field inside SP_DeviceMemoryBase is not const.
-  // Therefore, we need to cast away the constness before setting it.
-  device_memory_base.opaque = const_cast<void*>(mem->opaque());
-  device_memory_base.size = mem->size();
-  device_memory_base.payload = mem->payload();
-  return device_memory_base;
-}
-
 DeviceMemoryBase DeviceMemoryBaseFromC(const SP_DeviceMemoryBase& mem) {
   DeviceMemoryBase base(mem.opaque, mem.size);
   base.SetPayload(mem.payload);
@@ -182,7 +174,7 @@ DeviceMemoryBase DeviceMemoryBaseFromC(const SP_DeviceMemoryBase& mem) {
 
 // Wrapper that allows passing std::function across C API.
 struct HostCallbackContext {
-  absl::AnyInvocable<tsl::Status() &&> callback;
+  absl::AnyInvocable<absl::Status() &&> callback;
 };
 
 // This wrapper allows calling `HostCallbackContext::callback` across C API.
@@ -190,76 +182,69 @@ struct HostCallbackContext {
 // `callback_fn` to `host_callback` in `SP_StreamExecutor`.
 void HostCallbackTrampoline(void* ctx, TF_Status* status) {
   HostCallbackContext* host_ctx = static_cast<HostCallbackContext*>(ctx);
-  tsl::Status s = std::move(host_ctx->callback)();
+  absl::Status s = std::move(host_ctx->callback)();
   tsl::Set_TF_Status_from_Status(status, s);
   delete host_ctx;
 }
 
-class CStreamExecutor : public internal::StreamExecutorInterface {
+namespace {
+
+// Creates a MemoryAllocation that wraps a heap-allocated buffer.
+absl::StatusOr<std::unique_ptr<MemoryAllocation>> AllocateHostMemory(
+    SP_StreamExecutor* stream_executor, SP_Device* device, uint64_t size) {
+  void* ptr = stream_executor->host_memory_allocate(device, size);
+  if (ptr == nullptr && size > 0) {
+    return absl::InternalError("Failed to allocate host memory");
+  }
+  return std::make_unique<GenericMemoryAllocation>(
+      ptr, size, [stream_executor, device](void* ptr, uint64_t size) {
+        stream_executor->host_memory_deallocate(device, ptr);
+      });
+}
+}  // namespace
+
+class CStreamExecutor : public StreamExecutorCommon {
  public:
-  explicit CStreamExecutor(SP_Device device, SP_DeviceFns* device_fns,
+  explicit CStreamExecutor(Platform* se_platform, SP_Device device,
+                           SP_DeviceFns* device_fns,
                            SP_StreamExecutor* stream_executor,
                            SP_Platform* platform, SP_PlatformFns* platform_fns,
-                           SP_TimerFns* timer_fns, const std::string& name,
-                           int visible_device_count)
-      : device_(std::move(device)),
+                           const std::string& name)
+      : StreamExecutorCommon(se_platform),
+        device_(std::move(device)),
         device_fns_(device_fns),
         stream_executor_(stream_executor),
         platform_(platform),
         platform_fns_(platform_fns),
-        timer_fns_(timer_fns),
-        platform_name_(name),
-        visible_device_count_(visible_device_count) {}
+        platform_name_(name) {}
 
   ~CStreamExecutor() override {
     platform_fns_->destroy_device(platform_, &device_);
   }
 
-  tsl::Status Init(int device_ordinal, DeviceOptions device_options) override {
-    return ::tensorflow::OkStatus();
-  }
+  absl::Status Init() override { return absl::OkStatus(); }
 
-  DeviceMemoryBase Allocate(uint64 size, int64_t memory_space) override {
+  DeviceMemoryBase Allocate(uint64_t size, int64_t memory_space) override {
     SP_DeviceMemoryBase mem = {SP_DEVICE_MEMORY_BASE_STRUCT_SIZE};
     stream_executor_->allocate(&device_, size, memory_space, &mem);
-    tsl::Status status = ValidateSPDeviceMemoryBase(mem);
+    absl::Status status = ValidateSPDeviceMemoryBase(mem);
     if (!status.ok()) {
       LOG(ERROR) << status.message();
     }
     return DeviceMemoryBaseFromC(mem);
   }
-  DeviceMemoryBase Allocate(uint64 size) {
+  DeviceMemoryBase Allocate(uint64_t size) {
     return Allocate(size, /*memory_space=*/0);
   }
-  void* GetSubBuffer(DeviceMemoryBase* parent, uint64 offset,
-                     uint64 size) override {
-    LOG(FATAL) << "GetSubBuffer is not supported by pluggable device.";
-  }
 
   void Deallocate(DeviceMemoryBase* mem) override {
     SP_DeviceMemoryBase device_memory_base = DeviceMemoryBaseToC(mem);
     stream_executor_->deallocate(&device_, &device_memory_base);
   }
 
-  void* HostMemoryAllocate(uint64 size) override {
-    return stream_executor_->host_memory_allocate(&device_, size);
-  }
-
-  void HostMemoryDeallocate(void* mem) override {
-    stream_executor_->host_memory_deallocate(&device_, mem);
-  }
-
-  bool HostMemoryRegister(void* mem, uint64 size) override { return false; }
-  bool HostMemoryUnregister(void* mem) override { return false; }
-
-  void* UnifiedMemoryAllocate(uint64 size) override {
-    CHECK(stream_executor_->unified_memory_allocate);
-    return stream_executor_->unified_memory_allocate(&device_, size);
-  }
-
-  void UnifiedMemoryDeallocate(void* mem) override {
-    CHECK(stream_executor_->unified_memory_deallocate);
-    stream_executor_->unified_memory_deallocate(&device_, mem);
+  absl::StatusOr<std::unique_ptr<MemoryAllocation>> HostMemoryAllocate(
+      uint64_t size) override {
+    return AllocateHostMemory(stream_executor_, &device_, size);
   }
 
   absl::optional<AllocatorStats> GetAllocatorStats() override {
@@ -269,7 +254,7 @@ class CStreamExecutor : public internal::StreamExecutorInterface {
     if (!has_stats) {
       return absl::nullopt;
     }
-    tsl::Status status = ValidateSPAllocatorStats(c_stats);
+    absl::Status status = ValidateSPAllocatorStats(c_stats);
     if (!status.ok()) {
       LOG(ERROR) << status.message();
       return absl::nullopt;
@@ -299,235 +284,46 @@ class CStreamExecutor : public internal::StreamExecutorInterface {
     }
     return true;
   }
-  tsl::Status SynchronousMemZero(DeviceMemoryBase* location,
-                                 uint64 size) override {
+  absl::Status SynchronousMemZero(DeviceMemoryBase* location,
+                                  uint64_t size) override {
     // TODO(annarev): figure out if we should support memzero/memset
     // functionality by allocating on host and then copying to device.
     return tsl::errors::Unimplemented(
         "SynchronousMemZero is not supported by pluggable device.");
   }
-  tsl::Status SynchronousMemSet(DeviceMemoryBase* location, int value,
-                                uint64 size) override {
-    return tsl::errors::Unimplemented(
-        "SynchronousMemSet is not supported by pluggable device.");
-  }
-  tsl::Status SynchronousMemcpy(DeviceMemoryBase* gpu_dst, const void* host_src,
-                                uint64 size) override {
+  absl::Status SynchronousMemcpy(DeviceMemoryBase* gpu_dst,
+                                 const void* host_src, uint64_t size) override {
     OwnedTFStatus c_status(TF_NewStatus());
     SP_DeviceMemoryBase device_memory_base = DeviceMemoryBaseToC(gpu_dst);
     stream_executor_->sync_memcpy_htod(&device_, &device_memory_base, host_src,
                                        size, c_status.get());
     return StatusFromTF_Status(c_status.get());
   }
-  tsl::Status SynchronousMemcpy(void* host_dst, const DeviceMemoryBase& gpu_src,
-                                uint64 size) override {
+  absl::Status SynchronousMemcpy(void* host_dst,
+                                 const DeviceMemoryBase& gpu_src,
+                                 uint64_t size) override {
     OwnedTFStatus c_status(TF_NewStatus());
     SP_DeviceMemoryBase device_memory_base = DeviceMemoryBaseToC(&gpu_src);
     stream_executor_->sync_memcpy_dtoh(&device_, host_dst, &device_memory_base,
                                        size, c_status.get());
     return StatusFromTF_Status(c_status.get());
   }
-  tsl::Status SynchronousMemcpyDeviceToDevice(DeviceMemoryBase* gpu_dst,
-                                              const DeviceMemoryBase& gpu_src,
-                                              uint64 size) override {
-    OwnedTFStatus c_status(TF_NewStatus());
-    SP_DeviceMemoryBase device_mem_dst = DeviceMemoryBaseToC(gpu_dst);
-    SP_DeviceMemoryBase device_mem_src = DeviceMemoryBaseToC(&gpu_src);
-    stream_executor_->sync_memcpy_dtod(&device_, &device_mem_dst,
-                                       &device_mem_src, size, c_status.get());
-    return StatusFromTF_Status(c_status.get());
-  }
-  tsl::Status MemZero(Stream* stream, DeviceMemoryBase* location,
-                      uint64 size) override {
-    OwnedTFStatus c_status(TF_NewStatus());
-    SP_Stream stream_handle =
-        static_cast<CStream*>(stream->implementation())->Handle();
-    SP_DeviceMemoryBase device_mem = DeviceMemoryBaseToC(location);
-    stream_executor_->mem_zero(&device_, stream_handle, &device_mem, size,
-                               c_status.get());
-    return StatusFromTF_Status(c_status.get());
-  }
-  tsl::Status Memset(Stream* stream, DeviceMemoryBase* location, uint8 pattern,
-                     uint64 size) override {
-    OwnedTFStatus c_status(TF_NewStatus());
-    SP_Stream stream_handle =
-        static_cast<CStream*>(stream->implementation())->Handle();
-    SP_DeviceMemoryBase device_mem = DeviceMemoryBaseToC(location);
-    stream_executor_->memset(&device_, stream_handle, &device_mem, pattern,
-                             size, c_status.get());
-    return StatusFromTF_Status(c_status.get());
-  }
-  tsl::Status Memset32(Stream* stream, DeviceMemoryBase* location,
-                       uint32 pattern, uint64 size) override {
-    OwnedTFStatus c_status(TF_NewStatus());
-    SP_Stream stream_handle =
-        static_cast<CStream*>(stream->implementation())->Handle();
-    SP_DeviceMemoryBase device_mem = DeviceMemoryBaseToC(location);
-    stream_executor_->memset32(&device_, stream_handle, &device_mem, pattern,
-                               size, c_status.get());
-    return StatusFromTF_Status(c_status.get());
-  }
-  bool Memcpy(Stream* stream, void* host_dst, const DeviceMemoryBase& gpu_src,
-              uint64 size) override {
-    OwnedTFStatus c_status(TF_NewStatus());
-    SP_Stream stream_handle =
-        static_cast<CStream*>(stream->implementation())->Handle();
-    SP_DeviceMemoryBase device_mem_src = DeviceMemoryBaseToC(&gpu_src);
-    stream_executor_->memcpy_dtoh(&device_, stream_handle, host_dst,
-                                  &device_mem_src, size, c_status.get());
-    if (TF_GetCode(c_status.get()) != TF_OK) {
-      LOG(ERROR) << TF_Message(c_status.get());
-      return false;
-    }
-    return true;
-  }
-  bool Memcpy(Stream* stream, DeviceMemoryBase* gpu_dst, const void* host_src,
-              uint64 size) override {
-    OwnedTFStatus c_status(TF_NewStatus());
-    SP_Stream stream_handle =
-        static_cast<CStream*>(stream->implementation())->Handle();
-    SP_DeviceMemoryBase device_mem_dst = DeviceMemoryBaseToC(gpu_dst);
-    stream_executor_->memcpy_htod(&device_, stream_handle, &device_mem_dst,
-                                  host_src, size, c_status.get());
-    if (TF_GetCode(c_status.get()) != TF_OK) {
-      LOG(ERROR) << TF_Message(c_status.get());
-      return false;
-    }
-    return true;
-  }
-  bool MemcpyDeviceToDevice(Stream* stream, DeviceMemoryBase* gpu_dst,
-                            const DeviceMemoryBase& gpu_src,
-                            uint64 size) override {
-    OwnedTFStatus c_status(TF_NewStatus());
-    SP_Stream stream_handle =
-        static_cast<CStream*>(stream->implementation())->Handle();
-    SP_DeviceMemoryBase device_mem_dst = DeviceMemoryBaseToC(gpu_dst);
-    SP_DeviceMemoryBase device_mem_src = DeviceMemoryBaseToC(&gpu_src);
-    stream_executor_->memcpy_dtod(&device_, stream_handle, &device_mem_dst,
-                                  &device_mem_src, size, c_status.get());
-    if (TF_GetCode(c_status.get()) != TF_OK) {
-      LOG(ERROR) << TF_Message(c_status.get());
-      return false;
-    }
-    return true;
-  }
-  bool HostCallback(Stream* stream,
-                    absl::AnyInvocable<tsl::Status() &&> callback) override {
-    SP_Stream stream_handle =
-        static_cast<CStream*>(stream->implementation())->Handle();
-    HostCallbackContext* ctx = new HostCallbackContext{std::move(callback)};
-    return stream_executor_->host_callback(&device_, stream_handle,
-                                           &HostCallbackTrampoline, ctx);
-  }
-  tsl::Status AllocateEvent(Event* event) override {
-    DCHECK(event != nullptr);
-    return static_cast<CEvent*>(event->implementation())->Create();
-  }
-  tsl::Status DeallocateEvent(Event* event) override {
-    static_cast<CEvent*>(event->implementation())->Destroy();
-    return ::tensorflow::OkStatus();
-  }
-  tsl::Status RecordEvent(Stream* stream, Event* event) override {
-    SP_Stream stream_handle =
-        static_cast<CStream*>(stream->implementation())->Handle();
-    return static_cast<CEvent*>(event->implementation())->Record(stream_handle);
-  }
-  tsl::Status WaitForEvent(Stream* stream, Event* event) override {
-    SP_Stream stream_handle =
-        static_cast<CStream*>(stream->implementation())->Handle();
-    SP_Event event_handle =
-        static_cast<CEvent*>(event->implementation())->Handle();
-    OwnedTFStatus c_status(TF_NewStatus());
-    stream_executor_->wait_for_event(&device_, stream_handle, event_handle,
-                                     c_status.get());
-    tsl::Status s = StatusFromTF_Status(c_status.get());
-    return s;
-  }
-  Event::Status PollForEventStatus(Event* event) override {
-    SP_Event event_handle =
-        static_cast<CEvent*>(event->implementation())->Handle();
-    SE_EventStatus event_status =
-        stream_executor_->get_event_status(&device_, event_handle);
-    return SEEventStatusToEventStatus(event_status);
-  }
-  bool AllocateStream(Stream* stream) override {
-    DCHECK(stream != nullptr);
-    tsl::Status status =
-        static_cast<CStream*>(stream->implementation())->Create();
-    // TODO(annarev): update AllocateStream to return status instead
-    // (similar to AllocateEvent).
-    return status.ok();
-  }
   void DeallocateStream(Stream* stream) override {
-    static_cast<CStream*>(stream->implementation())->Destroy();
+    static_cast<CStream*>(stream)->Destroy();
   }
-  bool CreateStreamDependency(Stream* dependent, Stream* other) override {
+  absl::Status BlockHostForEvent(Stream* stream, Event* event) {
     OwnedTFStatus c_status(TF_NewStatus());
-    SP_Stream dependent_handle =
-        static_cast<CStream*>(dependent->implementation())->Handle();
-    SP_Stream other_handle =
-        static_cast<CStream*>(other->implementation())->Handle();
-    stream_executor_->create_stream_dependency(&device_, dependent_handle,
-                                               other_handle, c_status.get());
-    if (TF_GetCode(c_status.get()) != TF_OK) {
-      LOG(ERROR) << TF_Message(c_status.get());
-      return false;
-    }
-    return true;
-  }
-  tsl::Status BlockHostForEvent(Stream* stream, Event* event) {
-    OwnedTFStatus c_status(TF_NewStatus());
-    SP_Event event_handle =
-        static_cast<CEvent*>(event->implementation())->Handle();
+    SP_Event event_handle = static_cast<CEvent*>(event)->Handle();
     stream_executor_->block_host_for_event(&device_, event_handle,
                                            c_status.get());
     return StatusFromTF_Status(c_status.get());
   }
 
-  tsl::Status BlockHostUntilDone(Stream* stream) override {
-    OwnedTFStatus c_status(TF_NewStatus());
-    SP_Stream stream_handle =
-        static_cast<CStream*>(stream->implementation())->Handle();
-
-    // If `block_host_until_done` is set, use it.
-    if (stream_executor_->block_host_until_done != nullptr) {
-      stream_executor_->block_host_until_done(&device_, stream_handle,
-                                              c_status.get());
-      return StatusFromTF_Status(c_status.get());
-    }
-    // Create and record an event and then wait for it.
-    SP_Event event_handle;
-    stream_executor_->create_event(&device_, &event_handle, c_status.get());
-    TF_RETURN_IF_ERROR(StatusFromTF_Status(c_status.get()));
-    stream_executor_->record_event(&device_, stream_handle, event_handle,
-                                   c_status.get());
-    tsl::Status s = StatusFromTF_Status(c_status.get());
-    if (!s.ok()) {
-      stream_executor_->destroy_event(&device_, event_handle);
-      return s;
-    }
-    stream_executor_->block_host_for_event(&device_, event_handle,
-                                           c_status.get());
-    stream_executor_->destroy_event(&device_, event_handle);
-    return StatusFromTF_Status(c_status.get());
-  }
-
-  tsl::Status GetStatus(Stream* stream) override {
-    OwnedTFStatus c_status(TF_NewStatus());
-    SP_Stream stream_handle =
-        static_cast<CStream*>(stream->implementation())->Handle();
-    stream_executor_->get_stream_status(&device_, stream_handle,
-                                        c_status.get());
-    return StatusFromTF_Status(c_status.get());
-  }
-  int PlatformDeviceCount() override { return visible_device_count_; }
-  tsl::Status EnablePeerAccessTo(StreamExecutorInterface* other) override {
+  absl::Status EnablePeerAccessTo(StreamExecutor* other) override {
     return tsl::errors::Unimplemented(
         "EnablePeerAccessTo is not supported by pluggable device.");
   }
-  bool CanEnablePeerAccessTo(StreamExecutorInterface* other) override {
-    return false;
-  }
+  bool CanEnablePeerAccessTo(StreamExecutor* other) override { return false; }
 
   bool DeviceMemoryUsage(int64_t* free, int64_t* total) const override {
     return stream_executor_->device_memory_usage(
@@ -537,56 +333,78 @@ class CStreamExecutor : public internal::StreamExecutorInterface {
 
   // Creates a new DeviceDescription object.
   // Ownership is transferred to the caller.
-  tsl::StatusOr<std::unique_ptr<DeviceDescription>> CreateDeviceDescription()
+  absl::StatusOr<std::unique_ptr<DeviceDescription>> CreateDeviceDescription()
       const override {
     OwnedTFStatus c_status(TF_NewStatus());
 
-    internal::DeviceDescriptionBuilder builder;
+    DeviceDescription desc;
     if (device_.hardware_name != nullptr) {
-      builder.set_name(device_.hardware_name);
+      desc.set_name(device_.hardware_name);
     }
     if (device_.device_vendor != nullptr) {
-      builder.set_device_vendor(device_.device_vendor);
+      desc.set_device_vendor(device_.device_vendor);
     }
     if (device_.pci_bus_id != nullptr) {
-      builder.set_pci_bus_id(device_.pci_bus_id);
+      desc.set_pci_bus_id(device_.pci_bus_id);
     }
 
     if (device_fns_->get_numa_node != nullptr) {
       int32_t numa_node = device_fns_->get_numa_node(&device_);
       if (numa_node >= 0) {
-        builder.set_numa_node(numa_node);
+        desc.set_numa_node(numa_node);
       }
     }
 
     if (device_fns_->get_memory_bandwidth != nullptr) {
       int64_t memory_bandwidth = device_fns_->get_memory_bandwidth(&device_);
       if (memory_bandwidth >= 0) {
-        builder.set_memory_bandwidth(memory_bandwidth);
+        desc.set_memory_bandwidth(memory_bandwidth);
       }
     }
     // TODO(annarev): Add gflops field in DeviceDescription and set it here.
     // TODO(annarev): Perhaps add `supports_unified_memory` in
     // DeviceDescription.
-    return builder.Build();
+    return std::make_unique<DeviceDescription>(std::move(desc));
   }
 
-  // Each call creates a new instance of the platform-specific implementation of
-  // the corresponding interface type.
-  std::unique_ptr<internal::EventInterface> CreateEventImplementation()
-      override {
-    return std::unique_ptr<internal::EventInterface>(
-        new CEvent(&device_, stream_executor_));
+  absl::StatusOr<std::unique_ptr<Event>> CreateEvent() override {
+    auto c_event = std::make_unique<CEvent>(&device_, stream_executor_);
+    TF_RETURN_IF_ERROR(c_event->Create());
+    return std::move(c_event);
   }
-  std::unique_ptr<internal::KernelInterface> CreateKernelImplementation()
-      override {
-    LOG(FATAL)
-        << "CreateKernelImplementation is not supported by pluggable device.";
+
+  absl::StatusOr<std::unique_ptr<Stream>> CreateStream(
+      std::optional<std::variant<StreamPriority, int>> priority) override {
+    auto stream = std::make_unique<CStream>(&device_, stream_executor_, this);
+    TF_RETURN_IF_ERROR(stream->Create());
+    return std::move(stream);
   }
-  std::unique_ptr<internal::StreamInterface> GetStreamImplementation()
-      override {
-    return std::unique_ptr<internal::StreamInterface>(
-        new CStream(&device_, stream_executor_));
+
+  absl::StatusOr<std::unique_ptr<MemoryAllocator>> CreateMemoryAllocator(
+      MemoryType type) override {
+    if (type == MemoryType::kUnified) {
+      return std::make_unique<GenericMemoryAllocator>(
+          [this](uint64_t size)
+              -> absl::StatusOr<std::unique_ptr<MemoryAllocation>> {
+            void* ptr =
+                stream_executor_->unified_memory_allocate(&device_, size);
+            if (ptr == nullptr) {
+              return absl::InternalError("Failed to allocate unified memory");
+            }
+            return std::make_unique<GenericMemoryAllocation>(
+                ptr, size, [this](void* ptr, uint64_t size) {
+                  stream_executor_->unified_memory_deallocate(&device_, ptr);
+                });
+          });
+    } else if (type == MemoryType::kHost) {
+      return std::make_unique<GenericMemoryAllocator>(
+          [this](uint64_t size)
+              -> absl::StatusOr<std::unique_ptr<MemoryAllocation>> {
+            return AllocateHostMemory(stream_executor_, &device_, size);
+          });
+    }
+    return absl::UnimplementedError(
+        absl::StrFormat("Unsupported memory type %d", type));
   }
 
  private:
@@ -595,9 +413,7 @@ class CStreamExecutor : public internal::StreamExecutorInterface {
   SP_StreamExecutor* stream_executor_;
   SP_Platform* platform_;
   SP_PlatformFns* platform_fns_;
-  SP_TimerFns* timer_fns_;
   std::string platform_name_;
-  int visible_device_count_;
 };
 }  // namespace
 
@@ -617,7 +433,6 @@ CPlatform::CPlatform(SP_Platform platform,
       name_(platform.name) {}
 
 CPlatform::~CPlatform() {
-  executor_cache_.DestroyAllExecutors();
   platform_fns_.destroy_device_fns(&platform_, &device_fns_);
   platform_fns_.destroy_stream_executor(&platform_, &stream_executor_);
   platform_fns_.destroy_timer_fns(&platform_, &timer_fns_);
@@ -625,40 +440,30 @@ CPlatform::~CPlatform() {
   destroy_platform_fns_(&platform_fns_);
 }
 
-tsl::StatusOr<std::unique_ptr<DeviceDescription>>
+absl::StatusOr<std::unique_ptr<DeviceDescription>>
 CPlatform::DescriptionForDevice(int ordinal) const {
   // TODO(annarev): see if we can get StreamExecutor instance
   // and call GetDeviceDescription. executor_cache_.Get would need
   // to be made const for it to work.
-  internal::DeviceDescriptionBuilder builder;
-  builder.set_name(name_);
-  return builder.Build();
+  DeviceDescription desc;
+  desc.set_name(name_);
+  return std::make_unique<DeviceDescription>(std::move(desc));
 }
-tsl::StatusOr<StreamExecutor*> CPlatform::ExecutorForDevice(int ordinal) {
-  stream_executor::StreamExecutorConfig config;
-  config.ordinal = ordinal;
-  return GetExecutor(config);
+absl::StatusOr<StreamExecutor*> CPlatform::FindExisting(int ordinal) {
+  return executor_cache_.Get(ordinal);
 }
-tsl::StatusOr<StreamExecutor*> CPlatform::ExecutorForDeviceWithPluginConfig(
-    int ordinal, const PluginConfig& plugin_config) {
-  StreamExecutorConfig config;
-  config.ordinal = ordinal;
-  config.plugin_config = plugin_config;
-  return GetExecutor(config);
-}
-tsl::StatusOr<StreamExecutor*> CPlatform::GetExecutor(
-    const StreamExecutorConfig& config) {
+absl::StatusOr<StreamExecutor*> CPlatform::ExecutorForDevice(int ordinal) {
   return executor_cache_.GetOrCreate(
-      config, [&]() { return GetUncachedExecutor(config); });
+      ordinal, [this, ordinal]() { return GetUncachedExecutor(ordinal); });
 }
-tsl::StatusOr<std::unique_ptr<StreamExecutor>> CPlatform::GetUncachedExecutor(
-    const StreamExecutorConfig& config) {
+absl::StatusOr<std::unique_ptr<StreamExecutor>> CPlatform::GetUncachedExecutor(
+    int ordinal) {
   // Fill device creation params
   SE_CreateDeviceParams device_params{SE_CREATE_DEVICE_PARAMS_STRUCT_SIZE};
   SP_Device device{SP_DEVICE_STRUCT_SIZE};
   device_params.device = &device;
   device_params.ext = nullptr;
-  device_params.ordinal = config.ordinal;
+  device_params.ordinal = ordinal;
   OwnedTFStatus c_status(TF_NewStatus());
 
   // Create Device
@@ -666,22 +471,14 @@ tsl::StatusOr<std::unique_ptr<StreamExecutor>> CPlatform::GetUncachedExecutor(
   TF_RETURN_IF_ERROR(StatusFromTF_Status(c_status.get()));
   TF_RETURN_IF_ERROR(ValidateSPDevice(device));
 
-  // Get Device Count
-  int visible_device_count = 0;
-  platform_fns_.get_device_count(&platform_, &visible_device_count,
-                                 c_status.get());
-  TF_RETURN_IF_ERROR(StatusFromTF_Status(c_status.get()));
-
-  auto executor = absl::make_unique<CStreamExecutor>(
-      std::move(device), &device_fns_, &stream_executor_, &platform_,
-      &platform_fns_, &timer_fns_, name_, visible_device_count);
-  auto result = absl::make_unique<StreamExecutor>(this, std::move(executor),
-                                                  config.ordinal);
-  return result;
+  return std::make_unique<CStreamExecutor>(this, std::move(device),
+                                           &device_fns_, &stream_executor_,
+                                           &platform_, &platform_fns_, name_);
 }
 
-tsl::Status InitStreamExecutorPlugin(void* dso_handle, std::string* device_type,
-                                     std::string* platform_name) {
+absl::Status InitStreamExecutorPlugin(void* dso_handle,
+                                      std::string* device_type,
+                                      std::string* platform_name) {
   tensorflow::Env* env = tensorflow::Env::Default();
 
   // Step 1: Load symbol for `TF_InitPlugin`
@@ -694,9 +491,9 @@ tsl::Status InitStreamExecutorPlugin(void* dso_handle, std::string* device_type,
   return InitStreamExecutorPlugin(init_fn, device_type, platform_name);
 }
 
-tsl::Status InitStreamExecutorPlugin(SEInitPluginFn init_fn,
-                                     std::string* device_type,
-                                     std::string* platform_name) {
+absl::Status InitStreamExecutorPlugin(SEInitPluginFn init_fn,
+                                      std::string* device_type,
+                                      std::string* platform_name) {
   SE_PlatformRegistrationParams params{
       SE_PLATFORM_REGISTRATION_PARAMS_STRUCT_SIZE};
   SP_Platform platform{SP_PLATFORM_STRUCT_SIZE};
@@ -747,10 +544,10 @@ tsl::Status InitStreamExecutorPlugin(SEInitPluginFn init_fn,
           std::move(platform), params.destroy_platform, std::move(platform_fns),
           params.destroy_platform_fns, std::move(device_fns), std::move(se),
           std::move(timer_fns)));
-  TF_CHECK_OK(stream_executor::MultiPlatformManager::RegisterPlatform(
-      std::move(cplatform)));
+  TF_CHECK_OK(
+      stream_executor::PlatformManager::RegisterPlatform(std::move(cplatform)));
   // TODO(annarev): Return `use_bfc_allocator` value in some way so that it is
   // available in `PluggableDeviceProcessState` once the latter is checked in.
-  return ::tensorflow::OkStatus();
+  return absl::OkStatus();
 }
 }  // namespace stream_executor
