@@ -14,15 +14,28 @@ limitations under the License.
 ==============================================================================*/
 #include "tensorflow/c/experimental/stream_executor/stream_executor.h"
 
+#include <cstddef>
+#include <cstdint>
+#include <cstdlib>
+#include <cstring>
 #include <functional>
+#include <memory>
+#include <string>
 #include <utility>
 
+#include <gmock/gmock.h>
+#include "absl/memory/memory.h"
+#include "absl/status/status.h"
+#include "absl/status/statusor.h"
+#include "absl/types/optional.h"
 #include "tensorflow/c/experimental/stream_executor/stream_executor_internal.h"
 #include "tensorflow/c/experimental/stream_executor/stream_executor_test_util.h"
-#include "tensorflow/compiler/xla/stream_executor/event.h"
-#include "tensorflow/compiler/xla/stream_executor/multi_platform_manager.h"
-#include "tensorflow/compiler/xla/stream_executor/stream.h"
-#include "tensorflow/compiler/xla/stream_executor/stream_executor_pimpl.h"
+#include "xla/stream_executor/event.h"
+#include "xla/stream_executor/platform_manager.h"
+#include "xla/stream_executor/stream.h"
+#include "xla/stream_executor/stream_executor.h"
+#include "xla/tsl/platform/statusor.h"
+#include "xla/tsl/protobuf/error_codes.pb.h"
 #include "tensorflow/core/lib/core/status_test_util.h"
 #include "tensorflow/core/platform/test.h"
 #include "tensorflow/core/protobuf/error_codes.pb.h"
@@ -38,17 +51,17 @@ TEST(StreamExecutor, SuccessfulRegistration) {
     test_util::PopulateDefaultPlatformRegistrationParams(params);
   };
   std::string device_type, platform_name;
-  tsl::Status status =
+  absl::Status status =
       InitStreamExecutorPlugin(plugin_init, &device_type, &platform_name);
   TF_ASSERT_OK(status);
-  tsl::StatusOr<Platform*> maybe_platform =
-      MultiPlatformManager::PlatformWithName("MY_DEVICE");
+  absl::StatusOr<Platform*> maybe_platform =
+      PlatformManager::PlatformWithName("MY_DEVICE");
   TF_ASSERT_OK(maybe_platform.status());
   Platform* platform = std::move(maybe_platform).value();
   ASSERT_EQ(platform->Name(), test_util::kDeviceName);
   ASSERT_EQ(platform->VisibleDeviceCount(), test_util::kDeviceCount);
 
-  tsl::StatusOr<StreamExecutor*> maybe_executor =
+  absl::StatusOr<StreamExecutor*> maybe_executor =
       platform->ExecutorForDevice(0);
   TF_ASSERT_OK(maybe_executor.status());
 }
@@ -62,7 +75,7 @@ TEST(StreamExecutor, NameNotSet) {
   };
 
   std::string device_type, platform_name;
-  tsl::Status status =
+  absl::Status status =
       InitStreamExecutorPlugin(plugin_init, &device_type, &platform_name);
   ASSERT_EQ(status.code(), tensorflow::error::FAILED_PRECONDITION);
   ASSERT_EQ(status.message(), "'name' field in SP_Platform must be set.");
@@ -77,7 +90,7 @@ TEST(StreamExecutor, InvalidNameWithSemicolon) {
   };
 
   std::string device_type, platform_name;
-  tsl::Status status =
+  absl::Status status =
       InitStreamExecutorPlugin(plugin_init, &device_type, &platform_name);
   ASSERT_EQ(status.code(), tensorflow::error::FAILED_PRECONDITION);
   EXPECT_THAT(
@@ -94,7 +107,7 @@ TEST(StreamExecutor, InvalidNameWithSlash) {
   };
 
   std::string device_type, platform_name;
-  tsl::Status status =
+  absl::Status status =
       InitStreamExecutorPlugin(plugin_init, &device_type, &platform_name);
   ASSERT_EQ(status.code(), tensorflow::error::FAILED_PRECONDITION);
   EXPECT_THAT(status.message(),
@@ -110,7 +123,7 @@ TEST(StreamExecutor, CreateDeviceNotSet) {
   };
 
   std::string device_type, platform_name;
-  tsl::Status status =
+  absl::Status status =
       InitStreamExecutorPlugin(plugin_init, &device_type, &platform_name);
   ASSERT_EQ(status.code(), tensorflow::error::FAILED_PRECONDITION);
   ASSERT_EQ(status.message(),
@@ -126,7 +139,7 @@ TEST(StreamExecutor, UnifiedMemoryAllocateNotSet) {
   };
 
   std::string device_type, platform_name;
-  tsl::Status status =
+  absl::Status status =
       InitStreamExecutorPlugin(plugin_init, &device_type, &platform_name);
   ASSERT_EQ(status.code(), tensorflow::error::FAILED_PRECONDITION);
   ASSERT_EQ(
@@ -152,7 +165,7 @@ class StreamExecutorTest : public ::testing::Test {
           platform_, test_util::DestroyPlatform, platform_fns_,
           test_util::DestroyPlatformFns, device_fns_, se_, timer_fns_);
     }
-    tsl::StatusOr<StreamExecutor*> maybe_executor =
+    absl::StatusOr<StreamExecutor*> maybe_executor =
         cplatform_->ExecutorForDevice(ordinal);
     TF_CHECK_OK(maybe_executor.status());
     return std::move(maybe_executor).value();
@@ -184,7 +197,6 @@ TEST_F(StreamExecutorTest, Allocate) {
   ASSERT_NE(mem.opaque(), nullptr);
   ASSERT_EQ(mem.size(), 2 * sizeof(int));
   executor->Deallocate(&mem);
-  ASSERT_EQ(mem.opaque(), nullptr);
 }
 
 TEST_F(StreamExecutorTest, HostMemoryAllocate) {
@@ -200,11 +212,34 @@ TEST_F(StreamExecutorTest, HostMemoryAllocate) {
   };
   StreamExecutor* executor = GetExecutor(0);
   ASSERT_FALSE(allocate_called);
-  void* mem = executor->HostMemoryAllocate(8);
-  ASSERT_NE(mem, nullptr);
+  TF_ASSERT_OK_AND_ASSIGN(auto mem, executor->HostMemoryAllocate(8));
+  ASSERT_NE(mem->opaque(), nullptr);
   ASSERT_TRUE(allocate_called);
   ASSERT_FALSE(deallocate_called);
-  executor->HostMemoryDeallocate(mem);
+  mem.reset();
+  ASSERT_TRUE(deallocate_called);
+}
+
+TEST_F(StreamExecutorTest, HostMemoryAllocator) {
+  static bool allocate_called = false;
+  static bool deallocate_called = false;
+  se_.host_memory_allocate = [](const SP_Device* const device, uint64_t size) {
+    allocate_called = true;
+    return malloc(size);
+  };
+  se_.host_memory_deallocate = [](const SP_Device* const device, void* mem) {
+    free(mem);
+    deallocate_called = true;
+  };
+  StreamExecutor* executor = GetExecutor(0);
+  ASSERT_FALSE(allocate_called);
+  TF_ASSERT_OK_AND_ASSIGN(auto allocator,
+                          executor->CreateMemoryAllocator(MemoryType::kHost));
+  TF_ASSERT_OK_AND_ASSIGN(auto mem, allocator->Allocate(8));
+  ASSERT_NE(mem->opaque(), nullptr);
+  ASSERT_TRUE(allocate_called);
+  ASSERT_FALSE(deallocate_called);
+  mem.reset();
   ASSERT_TRUE(deallocate_called);
 }
 
@@ -222,11 +257,13 @@ TEST_F(StreamExecutorTest, UnifiedMemoryAllocate) {
   };
   StreamExecutor* executor = GetExecutor(0);
   ASSERT_FALSE(allocate_called);
-  void* mem = executor->UnifiedMemoryAllocate(8);
-  ASSERT_NE(mem, nullptr);
+  TF_ASSERT_OK_AND_ASSIGN(
+      auto allocator, executor->CreateMemoryAllocator(MemoryType::kUnified));
+  TF_ASSERT_OK_AND_ASSIGN(auto mem, allocator->Allocate(8));
+  ASSERT_NE(mem->opaque(), nullptr);
   ASSERT_TRUE(allocate_called);
   ASSERT_FALSE(deallocate_called);
-  executor->UnifiedMemoryDeallocate(mem);
+  mem.reset();
   ASSERT_TRUE(deallocate_called);
 }
 
@@ -280,12 +317,10 @@ TEST_F(StreamExecutorTest, CreateStream) {
 
   StreamExecutor* executor = GetExecutor(0);
   ASSERT_FALSE(stream_created);
-  Stream* stream = new Stream(executor);
-  stream->Init();
-  ASSERT_TRUE(stream->ok());
+  TF_ASSERT_OK_AND_ASSIGN(auto stream, executor->CreateStream());
   ASSERT_TRUE(stream_created);
   ASSERT_FALSE(stream_deleted);
-  delete stream;
+  stream.reset();
   ASSERT_TRUE(stream_deleted);
 }
 
@@ -299,12 +334,10 @@ TEST_F(StreamExecutorTest, CreateStreamDependency) {
   };
 
   StreamExecutor* executor = GetExecutor(0);
-  Stream dependent(executor);
-  dependent.Init();
-  Stream other(executor);
-  other.Init();
+  TF_ASSERT_OK_AND_ASSIGN(auto dependent, executor->CreateStream());
+  TF_ASSERT_OK_AND_ASSIGN(auto other, executor->CreateStream());
   ASSERT_FALSE(create_stream_dependency_called);
-  dependent.ThenWaitFor(&other);
+  TF_ASSERT_OK(dependent->WaitFor(other.get()));
   ASSERT_TRUE(create_stream_dependency_called);
 }
 
@@ -320,13 +353,11 @@ TEST_F(StreamExecutorTest, StreamStatus) {
   };
 
   StreamExecutor* executor = GetExecutor(0);
-  Stream stream(executor);
-  stream.Init();
-  ASSERT_TRUE(stream.ok());
-  TF_ASSERT_OK(stream.RefreshStatus());
+  TF_ASSERT_OK_AND_ASSIGN(auto stream, executor->CreateStream());
+  TF_ASSERT_OK(stream->RefreshStatus());
   status_ok = false;
-  auto updated_status = stream.RefreshStatus();
-  ASSERT_FALSE(stream.ok());
+  auto updated_status = stream->RefreshStatus();
+  ASSERT_FALSE(stream->ok());
   ASSERT_EQ(updated_status.message(), "Test error");
 }
 
@@ -348,11 +379,10 @@ TEST_F(StreamExecutorTest, CreateEvent) {
 
   StreamExecutor* executor = GetExecutor(0);
   ASSERT_FALSE(event_created);
-  Event* event = new Event(executor);
-  event->Init();
+  TF_ASSERT_OK_AND_ASSIGN(auto event, executor->CreateEvent());
   ASSERT_TRUE(event_created);
   ASSERT_FALSE(event_deleted);
-  delete event;
+  event.reset();
   ASSERT_TRUE(event_deleted);
 }
 
@@ -371,11 +401,10 @@ TEST_F(StreamExecutorTest, PollForEventStatus) {
   };
 
   StreamExecutor* executor = GetExecutor(0);
-  Event event(executor);
-  event.Init();
-  ASSERT_EQ(event.PollForStatus(), Event::Status::kComplete);
+  TF_ASSERT_OK_AND_ASSIGN(auto event, executor->CreateEvent());
+  ASSERT_EQ(event->PollForStatus(), Event::Status::kComplete);
   event_status = SE_EVENT_ERROR;
-  ASSERT_EQ(event.PollForStatus(), Event::Status::kError);
+  ASSERT_EQ(event->PollForStatus(), Event::Status::kError);
 }
 
 TEST_F(StreamExecutorTest, RecordAndWaitForEvent) {
@@ -409,15 +438,13 @@ TEST_F(StreamExecutorTest, RecordAndWaitForEvent) {
   };
 
   StreamExecutor* executor = GetExecutor(0);
-  Event event(executor);
-  event.Init();
-  Stream stream(executor);
-  stream.Init();
+  TF_ASSERT_OK_AND_ASSIGN(auto event, executor->CreateEvent());
+  TF_ASSERT_OK_AND_ASSIGN(auto stream, executor->CreateStream());
   ASSERT_FALSE(record_called);
-  stream.ThenRecordEvent(&event);
+  TF_ASSERT_OK(stream->RecordEvent(event.get()));
   ASSERT_TRUE(record_called);
   ASSERT_FALSE(wait_called);
-  stream.ThenWaitFor(&event);
+  TF_ASSERT_OK(stream->WaitFor(event.get()));
   ASSERT_TRUE(wait_called);
 }
 
@@ -439,15 +466,13 @@ TEST_F(StreamExecutorTest, MemcpyToHost) {
   };
 
   StreamExecutor* executor = GetExecutor(0);
-  Stream stream(executor);
-  stream.Init();
+  TF_ASSERT_OK_AND_ASSIGN(auto stream, executor->CreateStream());
   size_t size = sizeof(int);
   int src_data = 34;
   int dst_data = 2;
   DeviceMemoryBase device_src(&src_data, size);
-  Stream& stream_ref = stream.ThenMemcpy(&dst_data, device_src, size);
+  TF_ASSERT_OK(stream->Memcpy(&dst_data, device_src, size));
   ASSERT_EQ(dst_data, 34);
-  ASSERT_EQ(stream_ref.implementation(), stream.implementation());
 }
 
 TEST_F(StreamExecutorTest, MemcpyFromHost) {
@@ -460,13 +485,12 @@ TEST_F(StreamExecutorTest, MemcpyFromHost) {
   };
 
   StreamExecutor* executor = GetExecutor(0);
-  Stream stream(executor);
-  stream.Init();
+  TF_ASSERT_OK_AND_ASSIGN(auto stream, executor->CreateStream());
   size_t size = sizeof(int);
   int src_data = 18;
   int dst_data = 0;
   DeviceMemoryBase device_dst(&dst_data, size);
-  stream.ThenMemcpy(&device_dst, &src_data, size);
+  TF_ASSERT_OK(stream->Memcpy(&device_dst, &src_data, size));
   ASSERT_EQ(dst_data, 18);
 }
 
@@ -480,14 +504,13 @@ TEST_F(StreamExecutorTest, MemcpyDeviceToDevice) {
   };
 
   StreamExecutor* executor = GetExecutor(0);
-  Stream stream(executor);
-  stream.Init();
+  TF_ASSERT_OK_AND_ASSIGN(auto stream, executor->CreateStream());
   size_t size = sizeof(int);
   int src_data = 18;
   int dst_data = 0;
   DeviceMemoryBase device_dst(&dst_data, size);
   DeviceMemoryBase device_src(&src_data, size);
-  stream.ThenMemcpy(&device_dst, device_src, size);
+  TF_ASSERT_OK(stream->Memcpy(&device_dst, device_src, size));
   ASSERT_EQ(dst_data, 18);
 }
 
@@ -525,25 +548,6 @@ TEST_F(StreamExecutorTest, SyncMemcpyFromHost) {
   ASSERT_EQ(dst_data, 18);
 }
 
-TEST_F(StreamExecutorTest, SyncMemcpyDeviceToDevice) {
-  se_.sync_memcpy_dtod = [](const SP_Device* const device,
-                            SP_DeviceMemoryBase* const device_dst,
-                            const SP_DeviceMemoryBase* const device_src,
-                            uint64_t size, TF_Status* const status) {
-    TF_SetStatus(status, TF_OK, "");
-    std::memcpy(device_dst->opaque, device_src->opaque, size);
-  };
-
-  StreamExecutor* executor = GetExecutor(0);
-  size_t size = sizeof(int);
-  int src_data = 18;
-  int dst_data = 0;
-  DeviceMemoryBase device_dst(&dst_data, size);
-  DeviceMemoryBase device_src(&src_data, size);
-  ASSERT_TRUE(executor->SynchronousMemcpy(&device_dst, device_src, size));
-  ASSERT_EQ(dst_data, 18);
-}
-
 TEST_F(StreamExecutorTest, BlockHostForEvent) {
   static bool block_host_for_event_called = false;
   se_.create_event = [](const SP_Device* const device, SP_Event* event,
@@ -561,10 +565,9 @@ TEST_F(StreamExecutorTest, BlockHostForEvent) {
   };
 
   StreamExecutor* executor = GetExecutor(0);
-  Stream stream(executor);
-  stream.Init();
+  TF_ASSERT_OK_AND_ASSIGN(auto stream, executor->CreateStream());
   ASSERT_FALSE(block_host_for_event_called);
-  TF_ASSERT_OK(stream.BlockHostUntilDone());
+  TF_ASSERT_OK(stream->BlockHostUntilDone());
   ASSERT_TRUE(block_host_for_event_called);
 }
 
@@ -586,10 +589,9 @@ TEST_F(StreamExecutorTest, BlockHostUntilDone) {
   };
 
   StreamExecutor* executor = GetExecutor(0);
-  Stream stream(executor);
-  stream.Init();
+  TF_ASSERT_OK_AND_ASSIGN(auto stream, executor->CreateStream());
   ASSERT_FALSE(block_host_until_done_called);
-  TF_ASSERT_OK(stream.BlockHostUntilDone());
+  TF_ASSERT_OK(stream->BlockHostUntilDone());
   ASSERT_TRUE(block_host_until_done_called);
 }
 
@@ -618,13 +620,11 @@ TEST_F(StreamExecutorTest, HostCallbackOk) {
     return ok;
   };
   StreamExecutor* executor = GetExecutor(0);
-  Stream stream(executor);
-  stream.Init();
+  TF_ASSERT_OK_AND_ASSIGN(auto stream, executor->CreateStream());
   std::function<absl::Status()> callback = []() -> absl::Status {
     return absl::OkStatus();
   };
-  stream.ThenDoHostCallbackWithStatus(callback);
-  ASSERT_TRUE(stream.ok());
+  TF_ASSERT_OK(stream->DoHostCallbackWithStatus(callback));
 }
 
 TEST_F(StreamExecutorTest, HostCallbackError) {
@@ -638,13 +638,11 @@ TEST_F(StreamExecutorTest, HostCallbackError) {
     return ok;
   };
   StreamExecutor* executor = GetExecutor(0);
-  Stream stream(executor);
-  stream.Init();
-  std::function<tsl::Status()> callback = []() -> tsl::Status {
+  TF_ASSERT_OK_AND_ASSIGN(auto stream, executor->CreateStream());
+  std::function<absl::Status()> callback = []() -> absl::Status {
     return tsl::errors::Unimplemented("Unimplemented");
   };
-  stream.ThenDoHostCallbackWithStatus(callback);
-  ASSERT_FALSE(stream.ok());
+  ASSERT_FALSE(stream->DoHostCallbackWithStatus(callback).ok());
 }
 
 TEST_F(StreamExecutorTest, DeviceDescription) {
@@ -717,14 +715,12 @@ TEST_F(StreamExecutorTest, MemZero) {
   };
 
   StreamExecutor* executor = GetExecutor(0);
-  Stream stream(executor);
-  stream.Init();
+  TF_ASSERT_OK_AND_ASSIGN(auto stream, executor->CreateStream());
   size_t size = sizeof(int);
   int data = 2;
   DeviceMemoryBase device_data(&data, size);
-  Stream& stream_ref = stream.ThenMemZero(&device_data, size);
+  TF_ASSERT_OK(stream->MemZero(&device_data, size));
   ASSERT_EQ(data, 0);
-  ASSERT_EQ(stream_ref.implementation(), stream.implementation());
 }
 
 TEST_F(StreamExecutorTest, Memset32) {
@@ -748,14 +744,12 @@ TEST_F(StreamExecutorTest, Memset32) {
   };
 
   StreamExecutor* executor = GetExecutor(0);
-  Stream stream(executor);
-  stream.Init();
+  TF_ASSERT_OK_AND_ASSIGN(auto stream, executor->CreateStream());
   size_t size = sizeof(int);
   int data = 2;
   DeviceMemoryBase device_data(&data, size);
-  Stream& stream_ref = stream.ThenMemset32(&device_data, 18, size);
+  TF_ASSERT_OK(stream->Memset32(&device_data, 18, size));
   ASSERT_EQ(data, 18);
-  ASSERT_EQ(stream_ref.implementation(), stream.implementation());
 }
 
 }  // namespace
