@@ -15,33 +15,57 @@ limitations under the License.
 
 #include "tensorflow/c/kernels.h"
 
+#include <algorithm>
+#include <cstdint>
+#include <cstring>
+#include <functional>
 #include <memory>
+#include <string>
 #include <vector>
 
+#include "absl/status/status.h"
+#include "absl/strings/str_cat.h"
+#include "tensorflow/c/c_api.h"
 #include "tensorflow/c/c_api_internal.h"
 #include "tensorflow/c/c_api_macros.h"
+#include "tensorflow/c/experimental/stream_executor/stream_executor.h"
+#include "tensorflow/c/tf_buffer.h"
 #include "tensorflow/c/tf_buffer_internal.h"
+#include "tensorflow/c/tf_datatype.h"
+#include "tensorflow/c/tf_status.h"
 #include "tensorflow/c/tf_status_helper.h"
+#include "tensorflow/c/tf_tensor.h"
 #include "tensorflow/c/tf_tensor_internal.h"
+#include "xla/tsl/c/tsl_status_internal.h"  // IWYU pragma: keep
+#include "tensorflow/core/framework/allocator.h"
 #include "tensorflow/core/framework/attr_value.pb.h"
+#include "tensorflow/core/framework/attr_value_util.h"
+#include "tensorflow/core/framework/device_base.h"
 #include "tensorflow/core/framework/kernel_def_builder.h"
+#include "tensorflow/core/framework/node_def_util.h"
 #include "tensorflow/core/framework/op_kernel.h"
 #include "tensorflow/core/framework/register_types.h"
+#include "tensorflow/core/framework/resource_handle.h"
 #include "tensorflow/core/framework/resource_handle.pb.h"
 #include "tensorflow/core/framework/resource_mgr.h"
+#include "tensorflow/core/framework/tensor_shape.h"
 #include "tensorflow/core/framework/types.h"
+#include "tensorflow/core/platform/notification.h"
+#include "tensorflow/core/protobuf/config.pb.h"
+
 // Required for IS_MOBILE_PLATFORM definition
-#include "tensorflow/core/platform/platform.h"
-#include "tensorflow/core/platform/types.h"
+#include "tsl/platform/platform.h"  // IWYU pragma: keep
+
 #if !defined(IS_MOBILE_PLATFORM) && !defined(IS_SLIM_BUILD)
 #include "tensorflow/c/experimental/stream_executor/stream_executor_internal.h"
-#include "tensorflow/compiler/xla/stream_executor/stream.h"
+#include "xla/stream_executor/stream.h"
+#include "xla/tsl/framework/device_id_utils.h"
+#include "xla/tsl/platform/statusor.h"
+#include "tensorflow/core/common_runtime/next_pluggable_device/c/tf_rendezvous_c_api.h"
+#include "tensorflow/core/common_runtime/next_pluggable_device/c/tf_rendezvous_c_api_internal.h"
 #include "tensorflow/core/framework/device.h"
-#include "tensorflow/tsl/framework/device_id_utils.h"
-#include "tensorflow/tsl/platform/statusor.h"
 #endif  // !defined(IS_MOBILE_PLATFORM) && !defined(IS_SLIM_BUILD)
 
-using tensorflow::errors::InvalidArgument;
 // This file forms the basis of a stable ABI for third-party kernel
 // implementations. It is crucial that changes to this file are made cautiously
 // and with a focus on maintaining both source and binary compatibility.
@@ -118,7 +142,8 @@ void AddTypeConstraint(TF_KernelBuilder* kernel_builder, const char* attr_name,
     TF_CALL_quint16(CASE);
     TF_CALL_qint16(CASE);
     default:
-      status->status = errors::Unimplemented("Unexpected type ", dtype);
+      status->status =
+          absl::UnimplementedError(absl::StrCat("Unexpected type ", dtype));
       return;
   }
   TF_SetStatus(status, TF_OK, "");
@@ -136,8 +161,9 @@ const tensorflow::AttrValue* GetAttrValue(TF_OpKernelConstruction* ctx,
   const tensorflow::AttrValue* attr =
       ::tensorflow::AttrSlice(cc_ctx->def()).Find(attr_name);
   if (attr == nullptr) {
-    status->status = InvalidArgument("Operation '", cc_ctx->def().name(),
-                                     "' has no attr named '", attr_name, "'.");
+    status->status = absl::InvalidArgumentError(
+        absl::StrCat("Operation '", cc_ctx->def().name(),
+                     "' has no attr named '", attr_name, "'."));
   }
   return attr;
 }
@@ -225,7 +251,8 @@ class CAsyncOpKernel : public AsyncOpKernel {
     n.WaitForNotification();
   }
 
-  void ComputeAsync(OpKernelContext* ctx, AsyncOpKernelDoneCallback done) {
+  void ComputeAsync(OpKernelContext* ctx,
+                    AsyncOpKernelDoneCallback done) override {
     (*compute_async_func_)(
         c_kernel_, reinterpret_cast<TF_OpKernelContext*>(ctx),
         reinterpret_cast<TF_AsyncOpKernelDoneCallback*>(&done));
@@ -315,7 +342,7 @@ void TF_RegisterKernelBuilderWithKernelDef(const char* serialized_kernel_def,
 // This function is experimental and subject to change.
 SP_Stream TF_GetStream(TF_OpKernelContext* ctx, TF_Status* status) {
 #if defined(IS_MOBILE_PLATFORM) || defined(IS_SLIM_BUILD)
-  status->status = tensorflow::errors::Unimplemented(
+  status->status = absl::UnimplementedError(
       "Accessing device stream is not supported on mobile. File a bug at "
       "https://github.com/tensorflow/tensorflow/issues if this feature is "
       "important to you");
@@ -323,17 +350,17 @@ SP_Stream TF_GetStream(TF_OpKernelContext* ctx, TF_Status* status) {
 #else
   auto* cc_ctx = reinterpret_cast<::tensorflow::OpKernelContext*>(ctx);
   if (cc_ctx->op_device_context() == nullptr) {  // CPU Device
-    status->status = tensorflow::errors::FailedPrecondition(
+    status->status = absl::FailedPreconditionError(
         "Accessing device stream is not supported for a CPU device.");
     return nullptr;
   } else if (!cc_ctx->op_device_context()->IsPluggableDevice()) {
-    status->status = tensorflow::errors::FailedPrecondition(
+    status->status = absl::FailedPreconditionError(
         "Accessing device stream is only supported for pluggable devices.");
     return nullptr;
   } else {  // Is a PluggableDevice
     TF_SetStatus(status, TF_OK, "");
     auto c_stream = static_cast<stream_executor::CStream*>(
-        cc_ctx->op_device_context()->stream()->implementation());
+        cc_ctx->op_device_context()->stream());
     return c_stream->Handle();
   }
 #endif  // defined(IS_MOBILE_PLATFORM) || defined(IS_SLIM_BUILD)
@@ -393,8 +420,7 @@ void TF_SetOutput(TF_OpKernelContext* ctx, int i, const TF_Tensor* tensor,
     return;
   }
   ::tensorflow::Tensor cc_tensor;
-  ::tensorflow::Status s = ::tensorflow::TF_TensorToTensor(tensor, &cc_tensor);
-  TF_SetStatus(status, TF_OK, "");
+  absl::Status s = ::tensorflow::TF_TensorToTensor(tensor, &cc_tensor);
   ::tensorflow::Set_TF_Status_from_Status(status, s);
   if (s.ok()) {
     cc_ctx->set_output(i, cc_tensor);
@@ -459,13 +485,13 @@ void TF_GetSerializedResourceHandleProto(
 void TF_OpKernelConstruction_Failure(TF_OpKernelConstruction* ctx,
                                      TF_Status* status) {
   auto* cc_ctx = reinterpret_cast<::tensorflow::OpKernelConstruction*>(ctx);
-  ::tensorflow::Status s(::tensorflow::StatusFromTF_Status(status));
+  absl::Status s(tsl::StatusFromTF_Status(status));
   cc_ctx->CtxFailure(s);
 }
 
 void TF_OpKernelContext_Failure(TF_OpKernelContext* ctx, TF_Status* status) {
   auto* cc_ctx = reinterpret_cast<::tensorflow::OpKernelContext*>(ctx);
-  ::tensorflow::Status s(::tensorflow::StatusFromTF_Status(status));
+  absl::Status s(tsl::StatusFromTF_Status(status));
   cc_ctx->CtxFailure(s);
 }
 
@@ -537,8 +563,8 @@ void TF_OpKernelConstruction_GetAttrSize(TF_OpKernelConstruction* ctx,
       break;
 
     case tensorflow::AttrValue::VALUE_NOT_SET:
-      status->status =
-          InvalidArgument("Attribute '", attr_name, "' has no value set");
+      status->status = absl::InvalidArgumentError(
+          absl::StrCat("Attribute '", attr_name, "' has no value set"));
       break;
   }
 }
@@ -550,7 +576,7 @@ void TF_OpKernelConstruction_GetAttrSize(TF_OpKernelConstruction* ctx,
     TF_SetStatus(status, TF_OK, "");                                           \
     cc_type v;                                                                 \
     auto* cc_ctx = reinterpret_cast<::tensorflow::OpKernelConstruction*>(ctx); \
-    ::tensorflow::Status s = cc_ctx->GetAttr(attr_name, &v);                   \
+    absl::Status s = cc_ctx->GetAttr(attr_name, &v);                           \
     ::tensorflow::Set_TF_Status_from_Status(status, s);                        \
     if (s.ok()) {                                                              \
       *val = static_cast<c_type>(v);                                           \
@@ -563,8 +589,8 @@ void TF_OpKernelConstruction_GetAttrSize(TF_OpKernelConstruction* ctx,
     const tensorflow::AttrValue* attr = GetAttrValue(ctx, attr_name, status);  \
     if (!status->status.ok()) return;                                          \
     if (attr->value_case() != tensorflow::AttrValue::kList) {                  \
-      status->status =                                                         \
-          InvalidArgument("Value for '", attr_name, "' is not a list.");       \
+      status->status = absl::InvalidArgumentError(                             \
+          absl::StrCat("Attribute '", attr_name, "' is not a list."));         \
       return;                                                                  \
     }                                                                          \
     status->status =                                                           \
@@ -588,7 +614,7 @@ void TF_OpKernelConstruction_GetAttrString(TF_OpKernelConstruction* ctx,
                                            TF_Status* status) {
   std::string v;
   auto* cc_ctx = reinterpret_cast<::tensorflow::OpKernelConstruction*>(ctx);
-  ::tensorflow::Status s = cc_ctx->GetAttr(attr_name, &v);
+  absl::Status s = cc_ctx->GetAttr(attr_name, &v);
   ::tensorflow::Set_TF_Status_from_Status(status, s);
 
   if (!status->status.ok()) return;
@@ -607,7 +633,7 @@ void TF_OpKernelConstruction_GetAttrStringList(TF_OpKernelConstruction* ctx,
                                                TF_Status* status) {
   std::vector<std::string> v;
   auto* cc_ctx = reinterpret_cast<::tensorflow::OpKernelConstruction*>(ctx);
-  ::tensorflow::Status s = cc_ctx->GetAttr(attr_name, &v);
+  absl::Status s = cc_ctx->GetAttr(attr_name, &v);
   ::tensorflow::Set_TF_Status_from_Status(status, s);
 
   if (!status->status.ok()) return;
@@ -619,11 +645,11 @@ void TF_OpKernelConstruction_GetAttrStringList(TF_OpKernelConstruction* ctx,
     values[i] = p;
     lengths[i] = s.size();
     if ((p + s.size()) > (static_cast<char*>(storage) + storage_size)) {
-      status->status = InvalidArgument(
+      status->status = absl::InvalidArgumentError(
           "Not enough storage to hold the requested list of strings");
       return;
     }
-    memcpy(values[i], s.data(), s.size());
+    std::memcpy(values[i], s.data(), s.size());
     p += s.size();
   }
 }
@@ -634,7 +660,7 @@ void TF_OpKernelConstruction_GetAttrTensor(TF_OpKernelConstruction* ctx,
   *val = nullptr;
   ::tensorflow::Tensor t;
   auto* cc_ctx = reinterpret_cast<::tensorflow::OpKernelConstruction*>(ctx);
-  ::tensorflow::Status s = cc_ctx->GetAttr(attr_name, &t);
+  absl::Status s = cc_ctx->GetAttr(attr_name, &t);
   ::tensorflow::Set_TF_Status_from_Status(status, s);
 
   if (!status->status.ok()) return;
@@ -648,7 +674,7 @@ void TF_OpKernelConstruction_GetAttrTensorList(TF_OpKernelConstruction* ctx,
                                                TF_Status* status) {
   std::vector<::tensorflow::Tensor> v;
   auto* cc_ctx = reinterpret_cast<::tensorflow::OpKernelConstruction*>(ctx);
-  ::tensorflow::Status s = cc_ctx->GetAttr(attr_name, &v);
+  absl::Status s = cc_ctx->GetAttr(attr_name, &v);
   ::tensorflow::Set_TF_Status_from_Status(status, s);
 
   if (!status->status.ok()) return;
@@ -768,13 +794,30 @@ int TF_GetDeviceId(TF_OpKernelContext* ctx) {
 #else
   const auto* device = reinterpret_cast<const tensorflow::Device*>(
       device_base->UnderlyingDevice());
-  const tsl::StatusOr<int> id = tsl::GetDeviceIdFromDeviceParsedName(
-      device->parsed_name(), tensorflow::DeviceType(device->device_type()));
-  if (!id.ok()) return -1;
-  return *id;
+  return tsl::GetDeviceIdFromDeviceParsedName(device->parsed_name());
 #endif  // defined(IS_MOBILE_PLATFORM) || defined(IS_SLIM_BUILD)
 }
 
+TF_StringView TF_GetDeviceName(TF_OpKernelContext* ctx) {
+  const auto& device_name =
+      reinterpret_cast<tensorflow::OpKernelContext*>(ctx)->device()->name();
+  TF_StringView device_name_sv;
+  device_name_sv.data = device_name.data();
+  device_name_sv.len = device_name.length();
+  return device_name_sv;
+}
+
+#if !defined(IS_MOBILE_PLATFORM) && !defined(IS_SLIM_BUILD)
+TF_RendezvousThunk TF_GetRendezvous(TF_OpKernelContext* ctx) {
+  TF_RendezvousThunk* thunk =
+      ToC(reinterpret_cast<tensorflow::OpKernelContext*>(ctx)->rendezvous());
+  // Makes a copy of the thunk to simplify lifetime management.
+  TF_RendezvousThunk res = *thunk;
+  delete thunk;
+  return res;
+}
+#endif
+
 TF_StringView TF_GetOpKernelName(TF_OpKernelContext* ctx) {
   auto cc_ctx = reinterpret_cast<::tensorflow::OpKernelContext*>(ctx);
   TF_StringView opkernel_name_sv;
@@ -810,7 +853,7 @@ TF_Tensor* TF_AllocateOutput(TF_OpKernelContext* context, int index,
   tensorflow::gtl::ArraySlice<const int64_t> dimarray(
       reinterpret_cast<const int64_t*>(dims), num_dims);
   tensorflow::Tensor* tensor;
-  tensorflow::Status s = cc_ctx->allocate_output(
+  absl::Status s = cc_ctx->allocate_output(
       index, tensorflow::TensorShape(dimarray), &tensor);
   if (!s.ok()) {
     ::tensorflow::Set_TF_Status_from_Status(status, s);
@@ -832,12 +875,12 @@ TF_Tensor* TF_ForwardInputOrAllocateOutput(
   TF_SetStatus(status, TF_OK, "");
   auto* cc_ctx = reinterpret_cast<::tensorflow::OpKernelContext*>(context);
 
-  tensorflow::gtl::ArraySlice<int> input_indices_array(
-      candidate_input_indices, num_candidate_input_indices);
+  absl::Span<const int> input_indices_array(candidate_input_indices,
+                                            num_candidate_input_indices);
   tensorflow::gtl::ArraySlice<const int64_t> output_dimarray(
       reinterpret_cast<const int64_t*>(output_dims), output_num_dims);
   tensorflow::Tensor* output_tensor_pointer;
-  tensorflow::Status s = cc_ctx->forward_input_or_allocate_output(
+  absl::Status s = cc_ctx->forward_input_or_allocate_output(
       input_indices_array, output_index,
       tensorflow::TensorShape(output_dimarray), &output_tensor_pointer,
       forwarded_input);
@@ -855,13 +898,13 @@ TF_Tensor* TF_ForwardInputOrAllocateOutput(
 
 TF_Tensor* TF_AllocateTemp(TF_OpKernelContext* context, TF_DataType dtype,
                            const int64_t* dims, int num_dims,
-                           TF_AllocatorAttributes* attributes,
+                           TF_AllocatorAttributes* alloc_attrs,
                            TF_Status* status) {
   auto* cc_ctx = reinterpret_cast<::tensorflow::OpKernelContext*>(context);
   TF_SetStatus(status, TF_OK, "");
   tensorflow::gtl::ArraySlice<const int64_t> dimarray(
       reinterpret_cast<const int64_t*>(dims), num_dims);
-  if (attributes && !attributes->struct_size) {
+  if (alloc_attrs && !alloc_attrs->struct_size) {
     TF_SetStatus(
         status, TF_INVALID_ARGUMENT,
         "TF_AllocatorAttributes struct "
@@ -869,10 +912,10 @@ TF_Tensor* TF_AllocateTemp(TF_OpKernelContext* context, TF_DataType dtype,
     return nullptr;
   }
   tensorflow::AllocatorAttributes allocator_attr;
-  if (attributes && attributes->on_host) {
+  if (alloc_attrs && alloc_attrs->on_host) {
     allocator_attr.set_on_host(true);
   }
-  tensorflow::Status s;
+  absl::Status s;
   tensorflow::Tensor tensor;
   s = cc_ctx->allocate_temp(static_cast<tensorflow::DataType>(dtype),
                             tensorflow::TensorShape(dimarray), &tensor,
@@ -889,3 +932,15 @@ TF_Tensor* TF_AllocateTemp(TF_OpKernelContext* context, TF_DataType dtype,
   }
   return tf_tensor;
 }
+
+void TF_IncNumDeferredOps(TF_OpKernelContext* context) {
+  tensorflow::OpKernelContext* cc_ctx =
+      reinterpret_cast<::tensorflow::OpKernelContext*>(context);
+  cc_ctx->inc_num_deferred_ops_function()();
+}
+
+void TF_DecNumDeferredOps(TF_OpKernelContext* context) {
+  tensorflow::OpKernelContext* cc_ctx =
+      reinterpret_cast<::tensorflow::OpKernelContext*>(context);
+  cc_ctx->dec_num_deferred_ops_function()();
+}
