@@ -25,6 +25,7 @@ limitations under the License.
 #include "tensorflow/core/framework/allocation_description.pb.h"
 #include "tensorflow/core/framework/log_memory.h"
 #include "tensorflow/core/framework/tensor.h"
+#include "tensorflow/core/framework/tensor_shape.h"
 #include "tensorflow/core/framework/tensor_shape.pb.h"
 #include "tensorflow/core/framework/types.pb.h"
 #include "tensorflow/core/lib/core/coding.h"
@@ -36,6 +37,8 @@ using tensorflow::TensorBuffer;
 using tensorflow::errors::FailedPrecondition;
 using tensorflow::errors::InvalidArgument;
 
+#ifndef LIBTPU_EXCLUDE_C_API_IMPL
+
 namespace tensorflow {
 void* allocate_tensor(const char* operation, size_t len, Allocator* allocator) {
   void* data = allocator->AllocateRaw(EIGEN_MAX_ALIGN_BYTES, len);
@@ -75,16 +78,14 @@ TF_Tensor* CreateTensor(TF_ManagedBuffer* buf, TF_DataType dtype,
     dimvec[i] = static_cast<int64_t>(dims[i]);
   }
 
-  // TODO(gjn): Make the choice of interface a compile-time configuration.
-  tensorflow::TensorInterface ret(
-      Tensor(static_cast<tensorflow::DataType>(dtype),
-             tensorflow::TensorShape(dimvec), buf));
+  Tensor ret(static_cast<tensorflow::DataType>(dtype),
+             tensorflow::TensorShape(dimvec), buf);
   buf->Unref();
   size_t elem_size = TF_DataTypeSize(dtype);
   if (elem_size > 0 && len < (elem_size * ret.NumElements())) {
     return nullptr;
   }
-  return new TF_Tensor{new tensorflow::TensorInterface(ret)};
+  return new TF_Tensor{new tensorflow::TensorInterface(std::move(ret))};
 }
 }  // namespace
 
@@ -129,6 +130,8 @@ TF_Tensor* TF_NewTensor(TF_DataType dtype, const int64_t* dims, int num_dims,
   return CreateTensor(buf, dtype, dims, num_dims, len);
 }
 
+size_t TF_TensorDefaultAlignment() { return EIGEN_MAX_ALIGN_BYTES; }
+
 TF_Tensor* TF_TensorMaybeMove(TF_Tensor* t) {
   return t->tensor->CanMove() ? t : nullptr;
 }
@@ -186,6 +189,8 @@ void TF_TensorBitcastFrom(const TF_Tensor* from, TF_DataType type,
   tsl::Set_TF_Status_from_Status(status, cc_status);
 }
 
+#endif  // LIBTPU_EXCLUDE_C_API_IMPL
+
 namespace tensorflow {
 
 void TensorInterface::Release() {
@@ -244,8 +249,10 @@ void TensorInterface::SetShape(const int64_t* dims, int num_dims) {
   tensor_.set_shape(s);
 }
 
-Status TensorInterface::BitcastFrom(const TensorInterface& from, DataType type,
-                                    const int64_t* new_dims, int num_new_dims) {
+absl::Status TensorInterface::BitcastFrom(const TensorInterface& from,
+                                          DataType type,
+                                          const int64_t* new_dims,
+                                          int num_new_dims) {
   tensorflow::TensorShape s;
   for (int i = 0; i < num_new_dims; ++i) {
     TF_RETURN_IF_ERROR(s.AddDimWithStatus(new_dims[i]));
@@ -253,9 +260,9 @@ Status TensorInterface::BitcastFrom(const TensorInterface& from, DataType type,
   return tensor_.BitcastFrom(from.tensor_, type, s);
 }
 
-Status TensorInterface::FromProto(const tensorflow::TensorProto& from) {
+absl::Status TensorInterface::FromProto(const tensorflow::TensorProto& from) {
   bool success = tensor_.FromProto(from);
-  if (success) return OkStatus();
+  if (success) return absl::OkStatus();
   return errors::InvalidArgument("Unparseable tensor proto");
 }
 
@@ -290,8 +297,8 @@ static TF_Tensor* EmptyTensor(TF_DataType dtype,
 namespace tensorflow {
 
 AbstractTensorInterface* TensorInterfaceFromTensor(const Tensor& src,
-                                                   Status* status) {
-  *status = OkStatus();
+                                                   absl::Status* status) {
+  *status = absl::OkStatus();
   if (!src.IsInitialized()) {
     *status = FailedPrecondition(
         "attempt to use a tensor with an uninitialized value");
@@ -313,13 +320,14 @@ AbstractTensorInterface* TensorInterfaceFromTensor(const Tensor& src,
 }
 
 // Non-static for testing.
-TF_Tensor* TF_TensorFromTensor(const tensorflow::Tensor& src, Status* status) {
+TF_Tensor* TF_TensorFromTensor(const tensorflow::Tensor& src,
+                               absl::Status* status) {
   return new TF_Tensor{TensorInterfaceFromTensor(src, status)};
 }
 
 TF_Tensor* TF_TensorFromTensorShallow(const tensorflow::Tensor& src,
-                                      Status* status) {
-  *status = OkStatus();
+                                      absl::Status* status) {
+  *status = absl::OkStatus();
   if (!src.IsInitialized()) {
     *status = FailedPrecondition(
         "attempt to use a tensor with an uninitialized value");
@@ -331,14 +339,14 @@ TF_Tensor* TF_TensorFromTensorShallow(const tensorflow::Tensor& src,
   return new TF_Tensor{new tensorflow::TensorInterface(src)};
 }
 
-Status TF_TensorToTensor(const TF_Tensor* src, Tensor* dst) {
+absl::Status TF_TensorToTensor(const TF_Tensor* src, Tensor* dst) {
   return tensorflow::down_cast<const tensorflow::TensorInterface*>(src->tensor)
       ->ToTensor(dst);
 }
 
-Status TensorInterface::ToTensor(tensorflow::Tensor* dst) const {
+absl::Status TensorInterface::ToTensor(tensorflow::Tensor* dst) const {
   *dst = tensor_;
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 bool TensorInterface::IsAligned() const { return tensor_.IsAligned(); }
