@@ -58,31 +58,31 @@ class SymbolicGradientBuilder {
                           const std::vector<Output>& grad_inputs,
                           std::vector<Output>* grad_outputs);
 
-  Status AddGradients();
+  absl::Status AddGradients();
 
   static Output NoGradient() { return Output(nullptr, -1); }
 
  private:
-  Status Initialize();
+  absl::Status Initialize();
 
   // For each forward edge from `src` to `dst` in the initial/forward graph:
   // propagates gradients `dst_grad` backwards along the edge from `src`
   // to `dst` in the graph. This will add `dst_grad` to the list of pending
   // gradients for the node associated with `src`.
-  Status BackpropAlongEdge(const Output& dst_grad, const Output& src);
+  absl::Status BackpropAlongEdge(const Output& dst_grad, const Output& src);
 
   // Adds a node to the graph (returned in `grad`) that sums the in-bound
   // gradients to `src` (if there are more than one).
-  Status SumGradients(const Output& src, Output* grad);
+  absl::Status SumGradients(const Output& src, Output* grad);
 
   // Returns true if `opname` is registered in `registry_` with no gradient
   // function, false otherwise.
   bool IsPrimitiveOpWithNoGrad(const string& opname);
 
   // Call the gradient function for `op`, storing the result in `grad_outputs`.
-  Status CallGradFunction(const Operation& op,
-                          const std::vector<Output>& grad_inputs,
-                          std::vector<Output>* grad_outputs);
+  absl::Status CallGradFunction(const Operation& op,
+                                const std::vector<Output>& grad_inputs,
+                                std::vector<Output>* grad_outputs);
 
   // Returns a list mapping whether each node in the graph is reachable
   // from outputs_. Keyed by node id.
@@ -93,7 +93,7 @@ class SymbolicGradientBuilder {
   // nodes (which are the first nodes of a loop encountered in the backwards
   // pass) are passed to this function rather than processed normally.
   // `summed_grads` is the sum of `exit_node`s gradients.
-  Status ProcessWhileLoop(Node* exit_node, const Output& summed_grads);
+  absl::Status ProcessWhileLoop(Node* exit_node, const Output& summed_grads);
 
   // Gets the set of node ids at which to stop backprop. These are all elements
   // of `outputs_` that do not get transitively consumed by other `outputs_`.
@@ -138,7 +138,8 @@ class SymbolicGradientBuilder {
   // multiple incoming gradients, but we only store the combined Output here).
   std::map<WhileContext*, std::map<Node*, Output>> while_backprops_;
 
-  TF_DISALLOW_COPY_AND_ASSIGN(SymbolicGradientBuilder);
+  SymbolicGradientBuilder(const SymbolicGradientBuilder&) = delete;
+  void operator=(const SymbolicGradientBuilder&) = delete;
 };
 
 SymbolicGradientBuilder::SymbolicGradientBuilder(
@@ -152,8 +153,8 @@ SymbolicGradientBuilder::SymbolicGradientBuilder(
       grad_inputs_(grad_inputs),
       grad_outputs_(grad_outputs) {}
 
-Status SymbolicGradientBuilder::BackpropAlongEdge(const Output& dst_grad,
-                                                  const Output& src) {
+absl::Status SymbolicGradientBuilder::BackpropAlongEdge(const Output& dst_grad,
+                                                        const Output& src) {
   if (src.node() == nullptr) {
     return errors::Internal("Attempted to backprop along an invalid edge.");
   }
@@ -165,7 +166,7 @@ Status SymbolicGradientBuilder::BackpropAlongEdge(const Output& dst_grad,
       ready_.push_back(src.node());
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 std::vector<bool> SymbolicGradientBuilder::GetReachableNodes() {
@@ -250,7 +251,7 @@ std::unordered_set<int> SymbolicGradientBuilder::GetStopBackpropNodes(
   return stop_backprop_nodes;
 }
 
-Status SymbolicGradientBuilder::Initialize() {
+absl::Status SymbolicGradientBuilder::Initialize() {
   if (outputs_.size() != grad_inputs_.size()) {
     return errors::InvalidArgument(
         "Must specify a gradient input for each output.");
@@ -340,10 +341,11 @@ Status SymbolicGradientBuilder::Initialize() {
       TF_RETURN_IF_ERROR(BackpropAlongEdge(grad_inputs_[i], outputs_[i]));
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status SymbolicGradientBuilder::SumGradients(const Output& src, Output* grad) {
+absl::Status SymbolicGradientBuilder::SumGradients(const Output& src,
+                                                   Output* grad) {
   auto iter = backprops_.find(src);
   if (iter == backprops_.end()) {
     return errors::Internal("Unable to find backprop list for node.id ",
@@ -371,27 +373,27 @@ Status SymbolicGradientBuilder::SumGradients(const Output& src, Output* grad) {
     *grad = ops::AddN(scope_, grads_to_keep);
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 bool SymbolicGradientBuilder::IsPrimitiveOpWithNoGrad(const string& opname) {
   ops::GradFunc grad_fn;
-  Status s = registry_->Lookup(opname, &grad_fn);
+  absl::Status s = registry_->Lookup(opname, &grad_fn);
   return s.ok() && (grad_fn == nullptr);
 }
 
-Status SymbolicGradientBuilder::CallGradFunction(
+absl::Status SymbolicGradientBuilder::CallGradFunction(
     const Operation& op, const std::vector<Output>& grad_inputs,
     std::vector<Output>* grad_outputs) {
   ops::GradFunc grad_fn;
   TF_RETURN_IF_ERROR(registry_->Lookup(op.node()->type_string(), &grad_fn));
   TF_RETURN_IF_ERROR(grad_fn(scope_, op, grad_inputs, grad_outputs));
   TF_RETURN_IF_ERROR(scope_.status());
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status SymbolicGradientBuilder::ProcessWhileLoop(Node* exit_node,
-                                                 const Output& summed_grads) {
+absl::Status SymbolicGradientBuilder::ProcessWhileLoop(
+    Node* exit_node, const Output& summed_grads) {
   // TODO(skyewm): detect second-order gradient and return bad status
   // TODO(skyewm): handle (or at least detect) nested while loops
 
@@ -413,7 +415,8 @@ Status SymbolicGradientBuilder::ProcessWhileLoop(Node* exit_node,
   // Wait until we have all exit nodes' backprops collected before processing
   // the while loop.
   // TODO(skyewm): what if not all the exit nodes are reachable?
-  if (backprops.size() < while_ctx->exit_nodes().size()) return OkStatus();
+  if (backprops.size() < while_ctx->exit_nodes().size())
+    return absl::OkStatus();
 
   // We've seen all the exit nodes for this loop and have collected all the
   // backprops. Create the gradient graph for the while loop.
@@ -434,10 +437,10 @@ Status SymbolicGradientBuilder::ProcessWhileLoop(Node* exit_node,
       TF_RETURN_IF_ERROR(BackpropAlongEdge(dx[i], {e->src(), e->src_output()}));
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status SymbolicGradientBuilder::AddGradients() {
+absl::Status SymbolicGradientBuilder::AddGradients() {
   // Initialize backprops.
   TF_RETURN_IF_ERROR(Initialize());
 
@@ -552,25 +555,25 @@ Status SymbolicGradientBuilder::AddGradients() {
     int num_requested_inputs = p.first->num_outputs() - pending_[p.first->id()];
     CHECK_EQ(num_requested_inputs, p.second);
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace
 
-Status AddSymbolicGradients(const Scope& scope,
-                            const std::vector<Output>& outputs,
-                            const std::vector<Output>& inputs,
-                            const std::vector<Output>& grad_inputs,
-                            std::vector<Output>* grad_outputs) {
+absl::Status AddSymbolicGradients(const Scope& scope,
+                                  const std::vector<Output>& outputs,
+                                  const std::vector<Output>& inputs,
+                                  const std::vector<Output>& grad_inputs,
+                                  std::vector<Output>* grad_outputs) {
   SymbolicGradientBuilder builder(scope, ops::GradOpRegistry::Global(), outputs,
                                   inputs, grad_inputs, grad_outputs);
   return builder.AddGradients();
 }
 
-Status AddSymbolicGradients(const Scope& scope,
-                            const std::vector<Output>& outputs,
-                            const std::vector<Output>& inputs,
-                            std::vector<Output>* grad_outputs) {
+absl::Status AddSymbolicGradients(const Scope& scope,
+                                  const std::vector<Output>& outputs,
+                                  const std::vector<Output>& inputs,
+                                  std::vector<Output>* grad_outputs) {
   std::vector<Output> grad_inputs;
   grad_inputs.reserve(outputs.size());
   for (const Output& output : outputs) {
