@@ -41,7 +41,7 @@ const char kScopeSeparator[] = "/";
 const char kSuffixSeparator[] = "_";
 }  // namespace
 
-Scope::Impl::Impl(Graph* graph, Status* status, NameMap* name_map,
+Scope::Impl::Impl(Graph* graph, absl::Status* status, NameMap* name_map,
                   ShapeRefiner* refiner, bool disable_shape_inference)
     : graph_(graph),
       status_(status),
@@ -52,7 +52,7 @@ Scope::Impl::Impl(Graph* graph, Status* status, NameMap* name_map,
       disable_shape_inference_(disable_shape_inference) {}
 
 Scope::Impl::Impl(const std::shared_ptr<Graph>& graph,
-                  const std::shared_ptr<Status>& status,
+                  const std::shared_ptr<absl::Status>& status,
                   const std::shared_ptr<NameMap>& name_map,
                   const std::shared_ptr<ShapeRefiner>& refiner)
     : graph_(graph),
@@ -67,7 +67,7 @@ Scope Scope::NewRootScope() {
   Graph* graph = new Graph(OpRegistry::Global());
   ShapeRefiner* refiner =
       new ShapeRefiner(graph->versions(), graph->op_registry());
-  return Scope(new Impl(graph, new Status, new Impl::NameMap, refiner,
+  return Scope(new Impl(graph, new absl::Status, new Impl::NameMap, refiner,
                         /* disable_shape_inference */ false));
 }
 
@@ -75,7 +75,7 @@ Scope Scope::DisabledShapeInferenceScope() {
   Graph* graph = new Graph(OpRegistry::Global());
   ShapeRefiner* refiner =
       new ShapeRefiner(graph->versions(), graph->op_registry());
-  return Scope(new Impl(graph, new Status, new Impl::NameMap, refiner,
+  return Scope(new Impl(graph, new absl::Status, new Impl::NameMap, refiner,
                         /* disable_shape_inference */ true));
 }
 
@@ -274,7 +274,7 @@ std::unordered_set<string> Scope::Impl::GetColocationConstraints(
   std::vector<string> node_constraints;
   if (TryGetNodeAttr(attrs, kColocationAttrName, &node_constraints)) {
     for (const string& entry : node_constraints) {
-      StringPiece s(entry);
+      absl::string_view s(entry);
       if (absl::ConsumePrefix(&s, kColocationGroupPrefix)) {
         current_constraints.emplace(s);
       }
@@ -293,28 +293,28 @@ std::shared_ptr<Graph> Scope::graph_as_shared_ptr() const {
   return impl()->graph_;
 }
 
-Status Scope::status() const { return *impl()->status_; }
+absl::Status Scope::status() const { return *impl()->status_; }
 
 const std::vector<Operation>& Scope::control_deps() const {
   return impl()->control_deps_;
 }
 
-void Scope::UpdateStatus(const Status& s) const {
+void Scope::UpdateStatus(const absl::Status& s) const {
   impl()->status_->Update(s);
   if (impl()->exit_on_error_ && !ok()) {
     LOG(FATAL) << *impl()->status_;
   }
 }
 
-Status Scope::ToGraphDef(GraphDef* gdef, bool include_debug_info) const {
+absl::Status Scope::ToGraphDef(GraphDef* gdef, bool include_debug_info) const {
   if (!ok()) {
     return *impl()->status_;
   }
   graph()->ToGraphDef(gdef, /*include_flib_def=*/true, include_debug_info);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status Scope::ToGraph(Graph* g, GraphConstructorOptions opts) const {
+absl::Status Scope::ToGraph(Graph* g, GraphConstructorOptions opts) const {
   if (ok()) {
     GraphDef graph_def;
     graph()->ToGraphDef(&graph_def);
@@ -427,7 +427,7 @@ Scope Scope::WithOpNameImpl(const string& op_name) const {
 }
 
 Scope Scope::WithControlDependencies(
-    const gtl::ArraySlice<Operation> control_deps) const {
+    const absl::Span<const Operation> control_deps) const {
   return Scope(
       new Impl(*this, Impl::Tags::ControlDeps(),
                std::vector<Operation>(control_deps.begin(), control_deps.end()),
@@ -498,15 +498,16 @@ CompositeOpScopes Scope::GetCompositeOpScopes(
   }
 }
 
-Status Scope::DoShapeInference(Node* node) const {
-  if (impl_->disable_shape_inference_) return OkStatus();
+absl::Status Scope::DoShapeInference(Node* node) const {
+  if (impl_->disable_shape_inference_) return absl::OkStatus();
   return impl_->refiner_->AddNode(node);
 }
 
 class InternalScope {
  public:
   // NewScope doesn't take ownership of the inputs.
-  static Scope NewScope(Graph* graph, Status* status, ShapeRefiner* refiner) {
+  static Scope NewScope(Graph* graph, absl::Status* status,
+                        ShapeRefiner* refiner) {
     Scope::Impl::NameMap* name_map = new Scope::Impl::NameMap;
     for (const Node* node : graph->nodes()) {
       const string& name = node->name();
@@ -521,19 +522,20 @@ class InternalScope {
     // since the caller owns them and doesn't want the scope to destroy them.
     return Scope(new Scope::Impl(
         std::shared_ptr<Graph>(graph, [](Graph*) {}),
-        std::shared_ptr<Status>(status, [](Status*) {}),
+        std::shared_ptr<absl::Status>(status, [](absl::Status*) {}),
         std::shared_ptr<Scope::Impl::NameMap>(name_map),
         std::shared_ptr<ShapeRefiner>(refiner, [](ShapeRefiner*) {})));
   }
 };
 
-Scope NewInternalScope(Graph* graph, Status* status, ShapeRefiner* refiner) {
+Scope NewInternalScope(Graph* graph, absl::Status* status,
+                       ShapeRefiner* refiner) {
   return InternalScope::NewScope(graph, status, refiner);
 }
 
-Status CreateOutputWithScope(string op_name,
-                             absl::Span<const ::tensorflow::Input> inputs,
-                             const Scope& scope, Output* output) {
+absl::Status CreateOutputWithScope(string op_name,
+                                   absl::Span<const ::tensorflow::Input> inputs,
+                                   const Scope& scope, Output* output) {
   TF_RETURN_IF_ERROR(scope.status());
   const auto unique_name = scope.GetUniqueNameForOp(op_name);
   auto builder = ::tensorflow::NodeBuilder(unique_name, op_name);
@@ -547,7 +549,7 @@ Status CreateOutputWithScope(string op_name,
   scope.UpdateStatus(builder.Finalize(scope.graph(), &ret));
   TF_RETURN_IF_ERROR(scope.status());
   *output = Output(ret, 0);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace tensorflow
