@@ -18,6 +18,7 @@ limitations under the License.
 
 #include <vector>
 
+#include "absl/status/status.h"
 #include "tensorflow/cc/framework/ops.h"
 #include "tensorflow/cc/framework/scope.h"
 
@@ -27,9 +28,9 @@ namespace test {
 /// Calls the gradient function registered for 'op', adding gradient operations
 /// to the graph associated with 'scope'. Gradient outputs for each 'op' input
 /// are returned in 'grad_outputs'.
-Status CallGradFunction(const Scope& scope, const Operation& op,
-                        const std::vector<Output>& grad_inputs,
-                        std::vector<Output>* grad_outputs);
+absl::Status CallGradFunction(const Scope& scope, const Operation& op,
+                              const std::vector<Output>& grad_inputs,
+                              std::vector<Output>* grad_outputs);
 
 }  // namespace test
 }  // namespace tensorflow
