@@ -14,7 +14,10 @@ limitations under the License.
 ==============================================================================*/
 
 #include <functional>
+#include <tuple>
+#include <vector>
 
+#include <gtest/gtest.h>
 #include "tensorflow/cc/client/client_session.h"
 #include "tensorflow/cc/framework/grad_op_registry.h"
 #include "tensorflow/cc/framework/gradient_checker.h"
@@ -24,6 +27,7 @@ limitations under the License.
 #include "tensorflow/cc/ops/math_ops.h"
 #include "tensorflow/cc/ops/standard_ops.h"
 #include "tensorflow/core/framework/tensor_testutil.h"
+#include "tensorflow/core/framework/types.pb.h"
 #include "tensorflow/core/lib/core/status_test_util.h"
 #include "tensorflow/core/lib/random/random.h"
 
@@ -38,6 +42,7 @@ using ops::Atan2;
 using ops::BatchMatMul;
 using ops::BatchMatMulV3;
 using ops::Cast;
+using ops::ClipByValue;
 using ops::Const;
 using ops::Cumsum;
 using ops::Div;
@@ -1164,5 +1169,15 @@ TEST_F(NaryGradTest, UnsortedSegmentSumGrad) {
   RunTest({x}, {shape}, {y}, {y_shape});
 }
 
+TEST_F(NaryGradTest, ClipByValueGrad) {
+  TensorShape shape({3, 1, 5});
+  auto x = Placeholder(scope_, DT_FLOAT, Placeholder::Shape(shape));
+  auto y = ClipByValue(scope_, x, 0.0f, 5.0f);
+  Tensor x_init_value =
+      test::AsTensor<float>(kUnsortedSegmentMinMaxTestValue, shape);
+  TensorShape y_shape({3, 1, 5});
+  RunTest(x, x_init_value, y, y_shape);
+}
+
 }  // namespace
 }  // namespace tensorflow
