@@ -15,6 +15,7 @@ limitations under the License.
 
 #include <vector>
 
+#include "absl/status/status.h"
 #include "tensorflow/cc/framework/grad_op_registry.h"
 #include "tensorflow/cc/framework/gradients.h"
 #include "tensorflow/cc/ops/array_ops.h"
@@ -23,9 +24,9 @@ namespace tensorflow {
 namespace ops {
 namespace {
 
-Status ReadVariableOpGrad(const Scope& scope, const Operation& op,
-                          const std::vector<Output>& grad_inputs,
-                          std::vector<Output>* grad_outputs) {
+absl::Status ReadVariableOpGrad(const Scope& scope, const Operation& op,
+                                const std::vector<Output>& grad_inputs,
+                                std::vector<Output>* grad_outputs) {
   grad_outputs->push_back(Identity(scope, grad_inputs[0]));
   return scope.status();
 }
