@@ -19,10 +19,11 @@ limitations under the License.
 #include <string>
 #include <unordered_set>
 
+#include "tensorflow/c/tf_status.h"
 #include "tensorflow/cc/experimental/base/public/runtime.h"
 #include "tensorflow/cc/experimental/base/public/runtime_builder.h"
 #include "tensorflow/cc/experimental/base/public/status.h"
-#include "tensorflow/core/lib/io/path.h"
+#include "tensorflow/core/platform/path.h"
 #include "tensorflow/core/platform/stringpiece.h"
 #include "tensorflow/core/platform/test.h"
 
@@ -36,7 +37,7 @@ using tensorflow::experimental::cc::Status;
 
 constexpr char kTestData[] = "cc/saved_model/testdata";
 
-std::string SavedModelPath(tensorflow::StringPiece saved_model_dir) {
+std::string SavedModelPath(absl::string_view saved_model_dir) {
   return tensorflow::io::JoinPath(tensorflow::testing::TensorFlowSrcRoot(),
                                   kTestData, saved_model_dir);
 }
