@@ -374,8 +374,8 @@ TEST(XlaCompilationTest, CallXlaDeviceFuncWithResourceOp) {
   EXPECT_NE(clusters["A"], "");
 }
 
-static Status GradForUnaryCwise(FunctionDef* g,
-                                std::vector<FunctionDefHelper::Node> nodes) {
+static absl::Status GradForUnaryCwise(
+    FunctionDef* g, std::vector<FunctionDefHelper::Node> nodes) {
   for (auto& n : nodes) {
     if (n.attr.empty()) {
       n.attr = {{"T", DT_FLOAT}};
@@ -390,11 +390,11 @@ static Status GradForUnaryCwise(FunctionDef* g,
       {},
       // Nodes
       nodes);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // A gradient containing only supported operators
-Status SupportedGrad(const AttrSlice& attrs, FunctionDef* g) {
+absl::Status SupportedGrad(const AttrSlice& attrs, FunctionDef* g) {
   // clang-format off
   return GradForUnaryCwise(g, {
       {{"y"}, "Tanh", {"x"}},
@@ -408,7 +408,7 @@ Status SupportedGrad(const AttrSlice& attrs, FunctionDef* g) {
 REGISTER_OP_GRADIENT("Supported", SupportedGrad);
 
 // A gradient containing an unsupported operator.
-Status UnsupportedGrad(const AttrSlice& attrs, FunctionDef* g) {
+absl::Status UnsupportedGrad(const AttrSlice& attrs, FunctionDef* g) {
   // clang-format off
   return GradForUnaryCwise(g, {
       {{"y"}, "Tanh", {"x"}},
@@ -799,7 +799,7 @@ TEST(XlaCompilationTest, IllegalCycle_UsefulErrorMessage) {
       NodeDef def;
       TF_CHECK_OK(builder.Finalize(&def));
 
-      Status status;
+      absl::Status status;
       Node* node = graph->AddNode(def, &status);
       TF_CHECK_OK(status);
       return node;
@@ -815,7 +815,8 @@ TEST(XlaCompilationTest, IllegalCycle_UsefulErrorMessage) {
 
   TF_EXPECT_OK(root.ToGraph(graph.get()));
 
-  Status status = MarkForCompilationPassTestHelper::MarkForCompilation(&graph);
+  absl::Status status =
+      MarkForCompilationPassTestHelper::MarkForCompilation(&graph);
   EXPECT_FALSE(status.ok());
   EXPECT_TRUE(absl::StrContains(status.ToString(),
                                 "Edge from c to a would create a cycle.\n"
@@ -1009,7 +1010,7 @@ TEST(XlaCompilationTest, RandomShapeWithFunc) {
   NodeDef call_node;
   call_node.set_name("fn_call");
   call_node.set_op("Stateful_func");
-  Status status;
+  absl::Status status;
   Node* call = root.graph()->AddNode(call_node, &status);
   TF_ASSERT_OK(status);
 
@@ -1816,7 +1817,7 @@ TEST(XlaCompilationTest, DeterministicClusterNames) {
           " rhs: ", rhs_cluster_name);
     }
 
-    return OkStatus();
+    return absl::OkStatus();
   };
 
   testing::ResetClusterSequenceNumber();
@@ -1903,7 +1904,7 @@ Node* MakeStageNode(GraphDefBuilder& builder, string name,
 }  // namespace
 
 TEST(XlaCompilationTest, StagePipelinePreservedByClusterScopingPass) {
-  auto build_staged_graph = [](std::unique_ptr<Graph>* graph) -> Status {
+  auto build_staged_graph = [](std::unique_ptr<Graph>* graph) -> absl::Status {
     // Construct a graph as below with two pipeline stages and test that nodes
     // in different stages will not be merged if ClusterScopingPass is on.
     //
