@@ -28,9 +28,13 @@ limitations under the License.
 #include "tensorflow/compiler/jit/variable_info.h"
 #include "tensorflow/compiler/jit/variable_info_util.h"
 #include "tensorflow/compiler/tf2xla/xla_op_registry.h"
-#include "tensorflow/compiler/xla/pjrt/pjrt_client.h"
-#include "tensorflow/compiler/xla/pjrt/tfrt_cpu_pjrt_client.h"
-#include "tensorflow/compiler/xla/tests/literal_test_util.h"
+#include "xla/pjrt/pjrt_client.h"
+#include "xla/pjrt/pjrt_common.h"
+#include "xla/pjrt/plugin/xla_cpu/cpu_client_options.h"
+#include "xla/pjrt/plugin/xla_cpu/xla_cpu_pjrt_client.h"
+#include "xla/tests/literal_test_util.h"
+#include "xla/tsl/framework/device_id_utils.h"
+#include "xla/tsl/lib/core/status_test_util.h"
 #include "tensorflow/core/framework/allocator.h"
 #include "tensorflow/core/framework/device.h"
 #include "tensorflow/core/framework/fake_input.h"
@@ -42,10 +46,8 @@ limitations under the License.
 #include "tensorflow/core/platform/refcount.h"
 #include "tensorflow/core/tfrt/common/create_pjrt_client_util.h"
 #include "tensorflow/core/tfrt/common/pjrt_util.h"
-#include "tensorflow/tsl/framework/allocator.h"
-#include "tensorflow/tsl/lib/core/status_test_util.h"
-#include "tensorflow/tsl/platform/status.h"
-#include "tensorflow/tsl/platform/statusor.h"
+#include "tsl/platform/status.h"
+#include "tsl/platform/statusor.h"
 
 namespace tensorflow {
 namespace {
@@ -88,10 +90,11 @@ class PjRtExecutionUtilTest : public OpsTestBase {
     SetDevice(device_type, std::move(device));
 
     // Create PjRtClient for XLA_CPU.
+    xla::CpuClientOptions options;
+    options.asynchronous = true;
+    options.cpu_device_count = 1;
     TF_CHECK_OK(SetPjRtClientInTFGlobalResourceManager(
-        device_type,
-        xla::GetTfrtCpuClient(/*asynchronous=*/true, /*cpu_device_count=*/1)
-            .value()));
+        device_type, xla::GetXlaPjrtCpuClient(options).value()));
 
     // device_context_ should be a PjRtDeviceContext.
     TF_CHECK_OK(device_->TryGetDeviceContext(&device_context_));
@@ -185,13 +188,14 @@ class PjRtExecutionUtilTest : public OpsTestBase {
 
   // Runs a PjRtLoadedExecutable with the given inputs, variables. Requires the
   // XlaCompiler::CompilationResult that was used to build the executable.
-  StatusOr<std::vector<std::unique_ptr<xla::PjRtBuffer>>> RunExecutable(
+  absl::StatusOr<std::vector<std::unique_ptr<xla::PjRtBuffer>>> RunExecutable(
       const std::vector<const Tensor*>& inputs,
       const std::vector<VariableInfo>& variables,
       const XlaCompiler::CompilationResult* result,
       xla::PjRtLoadedExecutable* executable) {
-    TF_ASSIGN_OR_RETURN(auto pjrt_device, pjrt_client_->LookupAddressableDevice(
-                                              device_->parsed_name().id));
+    TF_ASSIGN_OR_RETURN(auto pjrt_device,
+                        pjrt_client_->LookupAddressableDevice(
+                            xla::PjRtLocalDeviceId(device_->parsed_name().id)));
 
     std::vector<xla::PjRtBuffer*> executable_args;
     executable_args.reserve(result->input_mapping.size());
@@ -630,5 +634,74 @@ TEST_F(PjRtExecutionUtilTest,
   test::ExpectTensorEqual<int32>(*expected, *GetOutput(0));
 }
 
+TEST_F(PjRtExecutionUtilTest, RunPjRtExecutableWithoutCtx) {
+  XlaOpRegistry::RegisterCompilationKernels();
+  TF_ASSERT_OK(NodeDefBuilder("AddV2", "AddV2")
+                   .Input(FakeInput(DT_INT32))
+                   .Input(FakeInput(DT_INT32))
+                   .Attr("T", DT_INT32)
+                   .Device("/job:localhost/replica:0/task:0/device:XLA_CPU:0")
+                   .Finalize(node_def()));
+  TF_ASSERT_OK(InitOp());
+
+  AddVariableInput<int32>("var1", TensorShape({1, 2}), {1, 2});
+  AddVariableInput<int32>("var2", TensorShape({1, 2}), {3, 4});
+
+  CreateContext();
+
+  std::vector<XlaCompiler::Argument> args(2);
+  args[0].kind = XlaCompiler::Argument::kParameter;
+  args[0].initialized = true;
+  args[0].type = DT_INT32;
+  args[0].shape = TensorShape({1, 2});
+  args[1].kind = XlaCompiler::Argument::kParameter;
+  args[1].initialized = true;
+  args[1].type = DT_INT32;
+  args[1].shape = TensorShape({1, 2});
+
+  const XlaCompiler::CompilationResult* result;
+  xla::PjRtLoadedExecutable* executable;
+  CompileToExecutable(args, &result, &executable);
+
+  std::vector<const Tensor*> inputs = InputsFromContext(context_.get());
+  std::vector<int> variables_indices =
+      GetResourceVariableIndicesFromContext(context_.get());
+  std::vector<VariableInfo> variables;
+  variables.reserve(variables_indices.size());
+  TF_ASSERT_OK(GetVariableInfosFromInputs(context_->resource_manager(),
+                                          context_->device(), inputs,
+                                          variables_indices, &variables));
+  const bool use_pjrt_tensor_buffer = context_->device()
+                                          ->tensorflow_accelerator_device_info()
+                                          ->use_pjrt_tensor_buffer;
+  const DeviceType& device_type = GetDeviceType(context_.get());
+  const int pjrt_device_id =
+      tsl::GetDeviceIdFromDeviceParsedName(context_->device()->parsed_name());
+  TF_ASSERT_OK_AND_ASSIGN(xla::PjRtDevice * pjrt_device,
+                          pjrt_client_->LookupAddressableDevice(
+                              xla::PjRtLocalDeviceId(pjrt_device_id)));
+
+  absl::flat_hash_map<int, const Tensor*> variable_snapshots;
+  for (int i = 0; i < variables.size(); i++) {
+    variable_snapshots[variables[i].index()] = variables[i].var()->tensor();
+  }
+
+  TF_ASSERT_OK_AND_ASSIGN(
+      std::vector<std::unique_ptr<xla::PjRtBuffer>> execute_outputs,
+      RunPjRtExecutable(/*num_missing_prefix_ctx_inputs=*/0, inputs,
+                        variable_snapshots, variables, device_type,
+                        use_pjrt_tensor_buffer, *result, pjrt_device,
+                        pjrt_client_, executable));
+
+  for (const auto& output : execute_outputs) {
+    TF_ASSERT_OK(output->GetReadyFuture().Await());
+  }
+
+  ASSERT_EQ(execute_outputs.size(), 1);
+  std::shared_ptr<xla::Literal> literal = *execute_outputs[0]->ToLiteralSync();
+  EXPECT_TRUE(xla::LiteralTestUtil::Equal(
+      *literal, xla::LiteralUtil::CreateR2<int32_t>({{4, 6}})));
+}
+
 }  // namespace
 }  // namespace tensorflow
