@@ -15,17 +15,21 @@ limitations under the License.
 
 #include "tensorflow/compiler/mlir/lite/quantization/device_target.h"
 
-#include <algorithm>
+#include <cassert>
+#include <cmath>
 #include <functional>
 #include <optional>
 
-#include "absl/types/optional.h"
 #include "llvm/ADT/SmallVector.h"
 #include "llvm/ADT/StringRef.h"
 #include "llvm/Support/Casting.h"
-#include "llvm/Support/raw_ostream.h"
-#include "mlir/Dialect/Quant/QuantTypes.h"  // from @llvm-project
+#include "mlir/Dialect/Quant/IR/QuantTypes.h"  // from @llvm-project
 #include "mlir/IR/Attributes.h"  // from @llvm-project
+#include "mlir/IR/BuiltinAttributes.h"  // from @llvm-project
+#include "mlir/IR/BuiltinTypes.h"  // from @llvm-project
+#include "mlir/IR/MLIRContext.h"  // from @llvm-project
+#include "mlir/IR/Operation.h"  // from @llvm-project
+#include "mlir/IR/Types.h"  // from @llvm-project
 #include "mlir/Support/LogicalResult.h"  // from @llvm-project
 #include "tensorflow/compiler/mlir/lite/quantization/ir/QuantOps.h"
 #include "tensorflow/compiler/mlir/lite/quantization/numerical_utils.h"
@@ -38,7 +42,7 @@ constexpr int k32Bits = 32;
 constexpr unsigned kSigned = QuantizationFlags::Signed;
 
 DeviceTarget::DeviceTarget(MLIRContext* ctx) : ctx_(ctx) {
-  f32_ = FloatType::getF32(ctx_);
+  f32_ = Float32Type::get(ctx_);
   i8_ = IntegerType::get(ctx_, k8Bits);
   i8_min_ = QuantizedType::getDefaultMinimumForInteger(kSigned, k8Bits);
   i8_max_ = QuantizedType::getDefaultMaximumForInteger(kSigned, k8Bits);
@@ -68,11 +72,11 @@ ScaleDecomposeFn DeviceTarget::GetDecomposeFn(
 
 void DeviceTarget::AppendToSignature(Type spec,
                                      KernelSpecs::Signature* signature) {
-  if (auto quant = spec.dyn_cast_or_null<UniformQuantizedType>()) {
+  if (auto quant = llvm::dyn_cast_or_null<UniformQuantizedType>(spec)) {
     signature->push_back(AnyQuantizedType::get(
         quant.getFlags(), quant.getStorageType(), quant.getExpressedType(),
         quant.getStorageTypeMin(), quant.getStorageTypeMax()));
-  } else if (auto any = spec.dyn_cast_or_null<AnyQuantizedType>()) {
+  } else if (auto any = llvm::dyn_cast_or_null<AnyQuantizedType>(spec)) {
     signature->push_back(any);
   } else {  // float
     signature->push_back(AnyQuantizedType());
@@ -109,17 +113,17 @@ LogicalResult DeviceTarget::DecomposeMultiplyAccumulateScale(
 
   llvm::SmallVector<Type, 4> input_specs, out_specs;
   for (auto spec : rop.getInputSpecs()) {
-    input_specs.push_back(spec.cast<TypeAttr>().getValue());
+    input_specs.push_back(llvm::cast<TypeAttr>(spec).getValue());
   }
   for (auto spec : rop.getOutputSpecs()) {
-    out_specs.push_back(spec.cast<TypeAttr>().getValue());
+    out_specs.push_back(llvm::cast<TypeAttr>(spec).getValue());
   }
 
-  auto in_spec = input_specs[0].dyn_cast<UniformQuantizedType>();
+  auto in_spec = llvm::dyn_cast<UniformQuantizedType>(input_specs[0]);
   // TODO(fengliuai): handles the PerAxis QuantizedType.
-  auto w_spec = input_specs[1].dyn_cast<UniformQuantizedType>();
-  auto b_spec = input_specs[2].dyn_cast<UniformQuantizedType>();
-  auto o_spec = out_specs[0].dyn_cast<UniformQuantizedType>();
+  auto w_spec = llvm::dyn_cast<UniformQuantizedType>(input_specs[1]);
+  auto b_spec = llvm::dyn_cast<UniformQuantizedType>(input_specs[2]);
+  auto o_spec = llvm::dyn_cast<UniformQuantizedType>(out_specs[0]);
   if (!in_spec || !w_spec || !b_spec || !o_spec) return failure();
 
   double scale_product = in_spec.getScale() * w_spec.getScale();
@@ -160,10 +164,8 @@ LogicalResult DeviceTarget::DecomposeSameScale(
     output_multipliers->push_back(kUnitQuantizedMultiplier);
   }
 
-  auto o_spec = rop.getOutputSpecs()[0]
-                    .cast<TypeAttr>()
-                    .getValue()
-                    .dyn_cast<UniformQuantizedType>();
+  auto o_spec = llvm::dyn_cast<UniformQuantizedType>(
+      llvm::cast<TypeAttr>(rop.getOutputSpecs()[0]).getValue());
   if (!o_spec) return failure();
 
   // output ranges
