@@ -15,14 +15,15 @@ limitations under the License.
 #ifndef TENSORFLOW_COMPILER_MLIR_LITE_QUANTIZATION_LITE_QUANTIZE_MODEL_H_
 #define TENSORFLOW_COMPILER_MLIR_LITE_QUANTIZATION_LITE_QUANTIZE_MODEL_H_
 
+#include <optional>
 #include <string>
 #include <unordered_set>
 
 #include "absl/container/flat_hash_set.h"
+#include "absl/status/status.h"
 #include "absl/strings/string_view.h"
-#include "tensorflow/lite/c/c_api_types.h"
-#include "tensorflow/lite/core/api/error_reporter.h"
-#include "tensorflow/lite/schema/schema_generated.h"
+#include "tensorflow/compiler/mlir/lite/debug/debug_options.pb.h"
+#include "tensorflow/compiler/mlir/lite/schema/schema_generated.h"
 
 namespace mlir {
 namespace lite {
@@ -44,17 +45,20 @@ namespace lite {
 // When `legacy_float_scale` is true, the quantizer will use float scale instead
 // of double, and call TOCO's quantization routines to maintain bit-exactness of
 // the values with the TOCO quantizer.
-TfLiteStatus QuantizeModel(
-    absl::string_view model_buffer, const tflite::TensorType& input_type,
-    const tflite::TensorType& output_type,
-    const tflite::TensorType& inference_type,
-    const std::unordered_set<std::string>& operator_names,
-    bool disable_per_channel, bool fully_quantize, std::string& output_buffer,
-    tflite::ErrorReporter* error_reporter, bool verify_numeric = false,
-    bool whole_model_verify = false, bool legacy_float_scale = true,
-    const absl::flat_hash_set<std::string>& denylisted_ops = {},
-    const absl::flat_hash_set<std::string>& denylisted_nodes = {},
-    bool enable_variable_quantization = false);
+absl::Status QuantizeModel(
+    absl::string_view model_buffer, const tflite::TensorType &input_type,
+    const tflite::TensorType &output_type,
+    const tflite::TensorType &inference_type,
+    const std::unordered_set<std::string> &operator_names,
+    bool disable_per_channel, bool fully_quantize, std::string &output_buffer,
+    bool verify_numeric = false, bool whole_model_verify = false,
+    bool legacy_float_scale = true,
+    const absl::flat_hash_set<std::string> &denylisted_ops = {},
+    const absl::flat_hash_set<std::string> &denylisted_nodes = {},
+    bool enable_variable_quantization = false,
+    bool disable_per_channel_for_dense_layers = false,
+    const std::optional<const tensorflow::converter::DebugOptions>
+        &debug_options = std::nullopt);
 
 }  // namespace lite
 }  // namespace mlir
