@@ -18,6 +18,7 @@ limitations under the License.
 
 #include <memory>
 #include <string>
+#include <vector>
 
 #include "mlir/Pass/Pass.h"  // from @llvm-project
 
@@ -26,8 +27,8 @@ namespace odml {
 
 // Creates a pass which prints out a detailed report of conversion stats with:
 // success or not, % of Ops non-converted, list of non-converted Ops, etc.
-// It get the stats based on the list in `TFL::mhlo::GetAcceptedDialects()`.
-std::unique_ptr<Pass> createPrintOpStatsPass();
+std::unique_ptr<Pass> createPrintOpStatsPass(
+    std::vector<std::string> accepted_dialects);
 
 }  // namespace odml
 }  // namespace mlir
