@@ -25,30 +25,33 @@ limitations under the License.
 #include <climits>
 #include <complex>
 #include <cstdint>
+#include <memory>
 #include <utility>
 
+#include "absl/status/statusor.h"
 #include "llvm/ADT/APInt.h"
 #include "llvm/ADT/ArrayRef.h"
 #include "llvm/ADT/Hashing.h"
 #include "llvm/ADT/StringSwitch.h"
 #include "llvm/Support/Threading.h"
 #include "llvm/Support/raw_ostream.h"
-#include "mlir/Dialect/Quant/QuantOps.h"  // from @llvm-project
+#include "mlir/Dialect/Arith/IR/Arith.h"  // from @llvm-project
+#include "mlir/Dialect/Quant/IR/Quant.h"  // from @llvm-project
 #include "mlir/IR/Attributes.h"  // from @llvm-project
+#include "mlir/IR/BuiltinAttributes.h"  // from @llvm-project
 #include "mlir/IR/BuiltinTypes.h"  // from @llvm-project
 #include "mlir/IR/Diagnostics.h"  // from @llvm-project
 #include "mlir/IR/MLIRContext.h"  // from @llvm-project
+#include "mlir/IR/Matchers.h"  // from @llvm-project
 #include "mlir/IR/Operation.h"  // from @llvm-project
 #include "mlir/IR/PatternMatch.h"  // from @llvm-project
+#include "mlir/IR/Types.h"  // from @llvm-project
 #include "mlir/Pass/Pass.h"  // from @llvm-project
 #include "mlir/Support/LLVM.h"  // from @llvm-project
 #include "mlir/Support/LogicalResult.h"  // from @llvm-project
 #include "mlir/Transforms/DialectConversion.h"  // from @llvm-project
 #include "tensorflow/compiler/mlir/lite/ir/tfl_ops.h"
-#include "tensorflow/compiler/mlir/lite/quantization/ir/FakeQuantSupport.h"
 #include "tensorflow/compiler/mlir/lite/quantization/ir/QuantOps.h"
-#include "tensorflow/compiler/mlir/lite/quantization/ir/UniformSupport.h"
-#include "tensorflow/compiler/mlir/lite/quantization/quantization_utils.h"
 #include "tensorflow/compiler/mlir/lite/transforms/passes.h"
 #include "tensorflow/compiler/mlir/lite/utils/attribute_utils.h"
 #include "tensorflow/compiler/mlir/lite/utils/constant_utils.h"
@@ -57,8 +60,6 @@ limitations under the License.
 #include "tensorflow/compiler/mlir/tensorflow/ir/tf_ops_a_m.h"
 #include "tensorflow/compiler/mlir/tensorflow/transforms/lower_tf.h"
 #include "tensorflow/compiler/mlir/tensorflow/utils/mangling_util.h"
-#include "tensorflow/compiler/xla/status.h"
-#include "tensorflow/compiler/xla/statusor.h"
 #include "tensorflow/core/framework/tensor.pb.h"
 #include "tensorflow/core/framework/tensor_shape.pb.h"
 #include "tensorflow/core/framework/types.pb.h"
@@ -88,6 +89,11 @@ class LegalizeTFPass : public impl::LegalizeTFPassBase<LegalizeTFPass> {
     this->preserve_assert_op_ = preserve_assert_op;
   }
 
+  explicit LegalizeTFPass(const LegalizeTFPassOptions& options) {
+    this->run_tfl_runtime_verification_ = options.run_tfl_runtime_verification_;
+    this->preserve_assert_op_ = options.preserve_assert_op_;
+  }
+
   /// Performs the lowering to TFLite dialect.
   void runOnOperation() override;
 };
@@ -95,7 +101,7 @@ class LegalizeTFPass : public impl::LegalizeTFPassBase<LegalizeTFPass> {
 // Util that casts 'val' to Int32 by adding a cast Op.
 Value CreateCastToInt32(Value val, Location loc, PatternRewriter& rewriter) {
   IntegerType new_ele_type = rewriter.getIntegerType(32);
-  if (auto shaped_type = val.getType().dyn_cast<RankedTensorType>()) {
+  if (auto shaped_type = mlir::dyn_cast<RankedTensorType>(val.getType())) {
     ShapedType new_type =
         RankedTensorType::get(shaped_type.getShape(), new_ele_type);
     return rewriter.createOrFold<TF::CastOp>(loc, new_type, val,
@@ -106,9 +112,37 @@ Value CreateCastToInt32(Value val, Location loc, PatternRewriter& rewriter) {
       rewriter.getBoolAttr(false));
 }
 
+// Utility function to-
+// 1. Create a tfl.const op with an int32_t values, from an MLIR Value, if the
+// `Value` can be matched to a Constant DenseIntElementsAttr.
+// This will make sure the dynamic dimensions are asigned to be `-1`
+// 2. In the default case, cast the `Value` to an int32_t.
+Value CreateInt32ConstOrCast(Value val, Location loc,
+                             PatternRewriter& rewriter) {
+  if (mlir::cast<ShapedType>(val.getType()).hasStaticShape()) {
+    DenseElementsAttr shape_value_attr;
+    if (matchPattern(val, m_Constant(&shape_value_attr))) {
+      SmallVector<int32_t, 4> new_shape_array_i32;
+      auto shape_value_array = shape_value_attr.getValues<llvm::APInt>();
+      for (int32_t idx = 0; idx < shape_value_array.size(); ++idx) {
+        auto size = shape_value_array[idx].getSExtValue();
+        new_shape_array_i32.push_back(
+            ShapedType::isDynamic(size) ? -1 : static_cast<int32_t>(size));
+      }
+      return rewriter.create<arith::ConstantOp>(
+          loc, DenseIntElementsAttr::get(
+                   RankedTensorType::get(new_shape_array_i32.size(),
+                                         rewriter.getIntegerType(32)),
+                   new_shape_array_i32));
+    }
+  }
+
+  return CreateCastToInt32(val, loc, rewriter);
+}
+
 // Get shape of an operand or result, support both dynamic and static shape.
 Value GetShape(Value input, Location loc, PatternRewriter& rewriter) {
-  auto shaped_type = input.getType().cast<ShapedType>();
+  auto shaped_type = mlir::cast<ShapedType>(input.getType());
   if (shaped_type.hasStaticShape()) {
     auto static_shape = shaped_type.getShape();
     auto static_shape_type =
@@ -242,7 +276,7 @@ bool ConvertTFBatchMatMulOp2TFLFullyConnectedOp(Operation* bmm_op,
 
   // Create a tfl.transpose op that performs ZX transpose on `input`.
   auto create_z_x_transpose_op = [&](Value input) -> Value {
-    RankedTensorType input_type = input.getType().cast<RankedTensorType>();
+    RankedTensorType input_type = mlir::cast<RankedTensorType>(input.getType());
     const int input_rank = input_type.getRank();
 
     // Create a 1D I32 tensor for representing the dimension permutation.
@@ -335,7 +369,7 @@ LogicalResult ConvertTFMatMulOp::matchAndRewrite(
   auto rhs = op->getOperand(1);
   auto transpose = [&](Value input) -> std::pair<LogicalResult, Value> {
     RankedTensorType type =
-        input.getType().dyn_cast_or_null<RankedTensorType>();
+        mlir::dyn_cast_or_null<RankedTensorType>(input.getType());
     if (!type || type.getRank() != 2) return {failure(), nullptr};
 
     auto permute_attr = DenseIntElementsAttr::get(
@@ -554,15 +588,15 @@ bool ConvertTFMatrixDiagV2orV3(Operation* op, PatternRewriter* rewriter) {
   // Verify padding_value is a tensor with all 0s.
   mlir::Value padding_value = tf_matrix_diag_v2_or_v3_op.getPaddingValue();
   mlir::Type element_type =
-      padding_value.getType().cast<ShapedType>().getElementType();
-  if (element_type.isa<FloatType>()) {
+      mlir::cast<ShapedType>(padding_value.getType()).getElementType();
+  if (mlir::isa<FloatType>(element_type)) {
     DenseFPElementsAttr padding_attr;
     if (!matchPattern(padding_value, m_Constant(&padding_attr)) ||
         !padding_attr.isSplat() ||
         !padding_attr.getSplatValue<APFloat>().isZero()) {
       return false;
     }
-  } else if (element_type.isa<IntegerType>()) {
+  } else if (mlir::isa<IntegerType>(element_type)) {
     DenseIntElementsAttr padding_attr;
     if (!matchPattern(padding_value, m_Constant(&padding_attr)) ||
         !padding_attr.isSplat() ||
@@ -613,7 +647,7 @@ struct LegalizeUnidirectionalSequenceLstm : public RewritePattern {
 
     SmallVector<int64_t, 20> tflite_indices;
     for (auto index_attr : tflite_indices_attr.getValue()) {
-      IntegerAttr index = index_attr.cast<IntegerAttr>();
+      IntegerAttr index = mlir::cast<IntegerAttr>(index_attr);
       tflite_indices.push_back(index.getInt());
     }
 
@@ -744,13 +778,13 @@ class ApplyExplicitBroadcasting : public OpRewritePattern<SourceOp> {
     SmallVector<int64_t, 4> symbolic_broadcast_shape;
     // Matches fail when lhs or rhs is unranked tensor.
     // TODO(b/176202543): Support unranked tensor.
-    if (!lhs.getType().cast<ShapedType>().hasRank() ||
-        !rhs.getType().cast<ShapedType>().hasRank()) {
+    if (!mlir::cast<ShapedType>(lhs.getType()).hasRank() ||
+        !mlir::cast<ShapedType>(rhs.getType()).hasRank()) {
       return failure();
     }
     if (!OpTrait::util::getBroadcastedShape(
-            lhs.getType().cast<ShapedType>().getShape(),
-            rhs.getType().cast<ShapedType>().getShape(),
+            mlir::cast<ShapedType>(lhs.getType()).getShape(),
+            mlir::cast<ShapedType>(rhs.getType()).getShape(),
             symbolic_broadcast_shape)) {
       return failure();
     }
@@ -795,13 +829,13 @@ class ApplyExplicitBroadcasting : public OpRewritePattern<SourceOp> {
     auto lhs = op->getOperand(0);
     auto rhs = op->getOperand(1);
 
-    if (!lhs.getType().cast<ShapedType>().hasStaticShape() ||
-        !rhs.getType().cast<ShapedType>().hasStaticShape()) {
+    if (!mlir::cast<ShapedType>(lhs.getType()).hasStaticShape() ||
+        !mlir::cast<ShapedType>(rhs.getType()).hasStaticShape()) {
       return rewriteOpWithDynamicInput(op, rewriter);
     }
 
-    auto lhs_shape = lhs.getType().cast<ShapedType>().getShape();
-    auto rhs_shape = rhs.getType().cast<ShapedType>().getShape();
+    auto lhs_shape = mlir::cast<ShapedType>(lhs.getType()).getShape();
+    auto rhs_shape = mlir::cast<ShapedType>(rhs.getType()).getShape();
 
     if (lhs_shape == rhs_shape) {
       return failure();
@@ -863,23 +897,23 @@ class ApplyExplicitBroadcasting<TF::SelectV2Op>
 
     // Matches fail when lhs|rhs|cond is unranked tensor.
     // TODO(b/176202543): Support unranked tensor.
-    if (!lhs.getType().cast<ShapedType>().hasRank() ||
-        !rhs.getType().cast<ShapedType>().hasRank() ||
-        !cond.getType().cast<ShapedType>().hasRank()) {
+    if (!mlir::cast<ShapedType>(lhs.getType()).hasRank() ||
+        !mlir::cast<ShapedType>(rhs.getType()).hasRank() ||
+        !mlir::cast<ShapedType>(cond.getType()).hasRank()) {
       return failure();
     }
 
     // Calculates symbolic broadcast shape that is only used in types.
     SmallVector<int64_t, 4> symbolic_broadcast_lhs_rhs_shape;
     if (!OpTrait::util::getBroadcastedShape(
-            lhs.getType().cast<ShapedType>().getShape(),
-            rhs.getType().cast<ShapedType>().getShape(),
+            mlir::cast<ShapedType>(lhs.getType()).getShape(),
+            mlir::cast<ShapedType>(rhs.getType()).getShape(),
             symbolic_broadcast_lhs_rhs_shape)) {
       return failure();
     }
     SmallVector<int64_t, 4> symbolic_broadcast_shape;
     if (!OpTrait::util::getBroadcastedShape(
-            cond.getType().cast<ShapedType>().getShape(),
+            mlir::cast<ShapedType>(cond.getType()).getShape(),
             symbolic_broadcast_lhs_rhs_shape, symbolic_broadcast_shape)) {
       return failure();
     }
@@ -935,15 +969,15 @@ class ApplyExplicitBroadcasting<TF::SelectV2Op>
     auto rhs = op->getOperand(2);
 
     // Should have static shapes to calculate the broadcasted shape.
-    if (!lhs.getType().cast<ShapedType>().hasStaticShape() ||
-        !rhs.getType().cast<ShapedType>().hasStaticShape() ||
-        !cond.getType().cast<ShapedType>().hasStaticShape()) {
+    if (!mlir::cast<ShapedType>(lhs.getType()).hasStaticShape() ||
+        !mlir::cast<ShapedType>(rhs.getType()).hasStaticShape() ||
+        !mlir::cast<ShapedType>(cond.getType()).hasStaticShape()) {
       return rewriteOpWithDynamicInput(op, rewriter);
     }
 
-    auto lhs_shape = lhs.getType().cast<ShapedType>().getShape();
-    auto rhs_shape = rhs.getType().cast<ShapedType>().getShape();
-    auto cond_shape = cond.getType().cast<ShapedType>().getShape();
+    auto lhs_shape = mlir::cast<ShapedType>(lhs.getType()).getShape();
+    auto rhs_shape = mlir::cast<ShapedType>(rhs.getType()).getShape();
+    auto cond_shape = mlir::cast<ShapedType>(cond.getType()).getShape();
 
     if (lhs_shape == rhs_shape && cond_shape == lhs_shape) {
       return failure();
@@ -1066,10 +1100,10 @@ void LegalizeTFPass::runOnOperation() {
   addPatterns(context, stage1Patterns, this->preserve_assert_op_);
 
   FrozenRewritePatternSet stage1FrozenPatterns(std::move(stage1Patterns));
-  if (!applyPatterns(func, target, stage1FrozenPatterns))
+  if (!applyPatterns(func, target, stage1FrozenPatterns)) {
     return signalPassFailure();
-
-  // Explict BroadcastTo addition for left-over broadcast-able ops.
+  }
+  // Explicit BroadcastTo addition for left-over broadcast-able ops.
   // The following pattern matchings should be done after the other legalization
   // rules in order not to add unnecessary BroadcastTo ops.
   RewritePatternSet stage2Patterns(&getContext());
@@ -1097,8 +1131,9 @@ void LegalizeTFPass::runOnOperation() {
                      ApplyExplicitBroadcasting<TF::SelectV2Op>>(context);
 
   FrozenRewritePatternSet stage2FrozenPatterns(std::move(stage2Patterns));
-  if (!applyPatterns(func, target, stage2FrozenPatterns))
+  if (!applyPatterns(func, target, stage2FrozenPatterns)) {
     return signalPassFailure();
+  }
 }
 
 }  // namespace
@@ -1110,6 +1145,12 @@ std::unique_ptr<OperationPass<func::FuncOp>> CreateLegalizeTFPass(
                                           preserve_assert_op);
 }
 
+// Creates an instance of the TensorFlow Lite dialect LegalizeTF pass.
+std::unique_ptr<OperationPass<func::FuncOp>> CreateLegalizeTFPass(
+    const LegalizeTFPassOptions& options) {
+  return std::make_unique<LegalizeTFPass>(options);
+}
+
 std::unique_ptr<OperationPass<func::FuncOp>> CreateLegalizeTFPass() {
   return std::make_unique<LegalizeTFPass>();
 }
