@@ -16,21 +16,19 @@ limitations under the License.
 #include "tensorflow/compiler/mlir/lite/utils/lstm_utils.h"
 
 #include <algorithm>
-#include <optional>
+#include <cstdint>
 #include <vector>
 
 #include "llvm/ADT/ArrayRef.h"
+#include "llvm/ADT/STLExtras.h"
 #include "llvm/ADT/SmallVector.h"
 #include "llvm/ADT/StringRef.h"
-#include "llvm/Support/Casting.h"
-#include "llvm/Support/raw_ostream.h"
 #include "mlir/Dialect/Arith/IR/Arith.h"  // from @llvm-project
 #include "mlir/Dialect/Func/IR/FuncOps.h"  // from @llvm-project
 #include "mlir/Dialect/Tensor/IR/Tensor.h"  // from @llvm-project
 #include "mlir/IR/Attributes.h"  // from @llvm-project
 #include "mlir/IR/Builders.h"  // from @llvm-project
 #include "mlir/IR/BuiltinAttributes.h"  // from @llvm-project
-#include "mlir/IR/BuiltinOps.h"  // from @llvm-project
 #include "mlir/IR/BuiltinTypes.h"  // from @llvm-project
 #include "mlir/IR/Location.h"  // from @llvm-project
 #include "mlir/IR/MLIRContext.h"  // from @llvm-project
@@ -41,6 +39,7 @@ limitations under the License.
 #include "mlir/Support/LLVM.h"  // from @llvm-project
 #include "mlir/Support/LogicalResult.h"  // from @llvm-project
 #include "tensorflow/compiler/mlir/lite/ir/tfl_ops.h"
+#include "tensorflow/compiler/mlir/lite/utils/utils.h"
 #include "tensorflow/compiler/mlir/tensorflow/ir/tf_ops.h"
 #include "tensorflow/compiler/mlir/tensorflow/utils/dynamic_shape_utils.h"
 
@@ -127,7 +126,7 @@ Value Reverse(OpBuilder* builder, Value value_to_reverse, int axis,
 }
 
 ArrayRef<int64_t> GetRankedTensorShape(Value value) {
-  return value.getType().cast<RankedTensorType>().getShape();
+  return mlir::cast<RankedTensorType>(value.getType()).getShape();
 }
 
 Value SliceRankedTensor(OpBuilder* builder, Value input,
@@ -159,7 +158,7 @@ Value SliceRankedTensor(OpBuilder* builder, Value input,
       location,
       RankedTensorType::get(
           size_values,
-          input.getType().cast<RankedTensorType>().getElementType()),
+          mlir::cast<RankedTensorType>(input.getType()).getElementType()),
       input, slice_i2c_begin, slice_i2c_size);
 }
 
@@ -170,7 +169,8 @@ Value CreateStridedSliceOp(mlir::Location loc, ArrayRef<int64_t> output_shape,
                            int64_t ellipsis_mask, int64_t new_axis_mask,
                            int64_t shrink_axis_mask, OpBuilder* builder) {
   auto output_type = RankedTensorType::get(
-      output_shape, input.getType().cast<RankedTensorType>().getElementType());
+      output_shape,
+      mlir::cast<RankedTensorType>(input.getType()).getElementType());
   auto begin_tensor = CreateI32DenseConst(builder, begin, loc);
   auto end_tensor = CreateI32DenseConst(builder, end, loc);
   auto strides_tensor = CreateI32DenseConst(builder, strides, loc);
@@ -387,7 +387,8 @@ void ConvertLSTMCellSimpleToFusedLSTM::UpdateFuncSignature() {
   SmallVector<int64_t, 2> output_shape{1, tensorflow::kTFDynamicSize};
   auto input_types = fused_func_op_.getFunctionType().getInputs();
   auto output_type = tensorflow::GetTypeFromTFTensorShape(
-      output_shape, input_.getType().cast<RankedTensorType>().getElementType());
+      output_shape,
+      mlir::cast<RankedTensorType>(input_.getType()).getElementType());
   fused_func_op_.setType(mlir::FunctionType::get(fused_func_op_.getContext(),
                                                  input_types, output_type));
 }
@@ -410,7 +411,8 @@ LogicalResult ConvertLSTMCellSimpleToFusedLSTM::RewriteFunc() {
   // Create the fused LSTM op.
   SmallVector<int64_t, 2> output_shape = {1, n_output_};
   auto result_type = mlir::RankedTensorType::get(
-      output_shape, input_.getType().cast<RankedTensorType>().getElementType());
+      output_shape,
+      mlir::cast<RankedTensorType>(input_.getType()).getElementType());
   lstm_ = builder_.create<mlir::TFL::LSTMOp>(
       fused_func_op_.getLoc(), result_type, input_, input2input_, input2forget_,
       input2cell_, input2output_, rec2input_, rec2forget_, rec2cell_,
@@ -436,7 +438,7 @@ LogicalResult ConvertLSTMCellSimpleToFusedLSTM::RewriteFunc() {
   SmallVector<int64_t, 2> func_output_shape = {1, tensorflow::kTFDynamicSize};
   auto func_result_type = tensorflow::GetTypeFromTFTensorShape(
       func_output_shape,
-      input_.getType().cast<RankedTensorType>().getElementType());
+      mlir::cast<RankedTensorType>(input_.getType()).getElementType());
 
   auto tensor_cast = builder_.create<mlir::tensor::CastOp>(
       fused_func_op_.getLoc(), func_result_type, lstm_.getResult());
@@ -491,7 +493,7 @@ LogicalResult ConvertLSTMCellSimpleToFusedLSTM::Initialize() {
   bias_ = fused_func_op_.getArgument(2);
 
   weight_ = fused_func_op_.getArgument(1);
-  weight_type_ = weight_.getType().cast<RankedTensorType>();
+  weight_type_ = mlir::cast<RankedTensorType>(weight_.getType());
 
   if (weight_type_.getRank() != 2) {
     return fused_func_op_.emitError() << "The weight tensor was not of rank 2";
@@ -505,7 +507,7 @@ LogicalResult ConvertLSTMCellSimpleToFusedLSTM::Initialize() {
   n_cell_ = weight_type_.getDimSize(1) / num_gates_;
 
   projection_ = fused_func_op_.getArgument(3);
-  projection_type_ = projection_.getType().cast<RankedTensorType>();
+  projection_type_ = mlir::cast<RankedTensorType>(projection_.getType());
   if (projection_type_.getRank() != 2) {
     n_output_ = n_cell_;
   } else {
@@ -532,7 +534,8 @@ LogicalResult ConvertLayerNormalizedLSTMCellSimpleToFusedLSTM::Initialize() {
   }
 
   layer_norm_scale_ = fused_func_op_.getArgument(4);
-  layer_norm_scale_type_ = layer_norm_scale_.getType().cast<RankedTensorType>();
+  layer_norm_scale_type_ =
+      mlir::cast<RankedTensorType>(layer_norm_scale_.getType());
   if (layer_norm_scale_type_.getRank() != 1) {
     return fused_func_op_.emitError()
            << "The layer_norm_scale tensor was not of rank 1";
@@ -607,7 +610,7 @@ TF::ReshapeOp CreateFlattenOP(const Value& input, Location loc,
 LogicalResult CreateEqualSizeSplitVOp(Value input, int axis, int splits,
                                       Location loc, OpBuilder* builder,
                                       Operation** result) {
-  auto input_type = input.getType().cast<RankedTensorType>();
+  auto input_type = mlir::cast<RankedTensorType>(input.getType());
   SmallVector<int64_t, 4> output_shape;
   int size_of_splits;
   if (input_type.getRank() < axis || axis < 0) return failure();
@@ -666,7 +669,7 @@ LogicalResult ConvertKerasLSTMLayer(mlir::func::FuncOp func_op,
   if (time_major_attr == nullptr) return failure();
 
   bool time_majored = time_major_attr.getValue();
-  auto input_type = input.getType().dyn_cast_or_null<RankedTensorType>();
+  auto input_type = mlir::dyn_cast_or_null<RankedTensorType>(input.getType());
   if (!input_type) {
     func_op.emitError() << "Input type is not a ranked tensor type";
     return failure();
@@ -692,7 +695,7 @@ LogicalResult ConvertKerasLSTMLayer(mlir::func::FuncOp func_op,
 
   // Setup correct weights.
   RankedTensorType weight_type =
-      weight_kernel.getType().cast<RankedTensorType>();
+      mlir::cast<RankedTensorType>(weight_kernel.getType());
   if (weight_type.getRank() != 2)
     return func_op.emitError() << "The weight should be rank of 2";
 
@@ -700,7 +703,7 @@ LogicalResult ConvertKerasLSTMLayer(mlir::func::FuncOp func_op,
       Transpose2D(builder, weight_kernel, weight_type, func_op.getLoc());
 
   RankedTensorType recurrent_kernel_type =
-      recurrent_kernel.getType().cast<RankedTensorType>();
+      mlir::cast<RankedTensorType>(recurrent_kernel.getType());
   const int64_t n_output = recurrent_kernel_type.getDimSize(0);
 
   Value transpose_recurrent_kernel = Transpose2D(
@@ -726,28 +729,28 @@ LogicalResult ConvertKerasLSTMLayer(mlir::func::FuncOp func_op,
   // IndyLSTMs are a LSTM variant with diagonal recurrent weight
   // matrices. For optimization purposes these are provided as vectors.
   Value recurrent_to_input_weights =
-      indy ? CreateFlattenOP(recurrent_weights_array->getResult(0),
-                             func_op.getLoc(), builder)
-                 .getResult()
-                 .cast<Value>()
+      indy ? mlir::cast<Value>(
+                 CreateFlattenOP(recurrent_weights_array->getResult(0),
+                                 func_op.getLoc(), builder)
+                     .getResult())
            : recurrent_weights_array->getResult(0);
   Value recurrent_to_forget_weights =
-      indy ? CreateFlattenOP(recurrent_weights_array->getResult(1),
-                             func_op.getLoc(), builder)
-                 .getResult()
-                 .cast<Value>()
+      indy ? mlir::cast<Value>(
+                 CreateFlattenOP(recurrent_weights_array->getResult(1),
+                                 func_op.getLoc(), builder)
+                     .getResult())
            : recurrent_weights_array->getResult(1);
   Value recurrent_to_cell_weights =
-      indy ? CreateFlattenOP(recurrent_weights_array->getResult(2),
-                             func_op.getLoc(), builder)
-                 .getResult()
-                 .cast<Value>()
+      indy ? mlir::cast<Value>(
+                 CreateFlattenOP(recurrent_weights_array->getResult(2),
+                                 func_op.getLoc(), builder)
+                     .getResult())
            : recurrent_weights_array->getResult(2);
   Value recurrent_to_output_weights =
-      indy ? CreateFlattenOP(recurrent_weights_array->getResult(3),
-                             func_op.getLoc(), builder)
-                 .getResult()
-                 .cast<Value>()
+      indy ? mlir::cast<Value>(
+                 CreateFlattenOP(recurrent_weights_array->getResult(3),
+                                 func_op.getLoc(), builder)
+                     .getResult())
            : recurrent_weights_array->getResult(3);
 
   // Splits the bias into 4:
@@ -765,7 +768,7 @@ LogicalResult ConvertKerasLSTMLayer(mlir::func::FuncOp func_op,
   }
   auto result_type = mlir::RankedTensorType::get(
       output_shape,
-      final_inputs.getType().cast<RankedTensorType>().getElementType());
+      mlir::cast<RankedTensorType>(final_inputs.getType()).getElementType());
 
   Value none = CreateNoneValue(builder, func_op.getLoc());
   auto lstm = builder->create<mlir::TFL::UnidirectionalSequenceLSTMOp>(
@@ -867,7 +870,7 @@ LogicalResult ConvertKerasLSTMLayer(mlir::func::FuncOp func_op,
   // All the rest: states, device.
   for (int i = 2; i < 5; ++i) {
     auto result_type =
-        func_op.getCallableResults()[i].dyn_cast<RankedTensorType>();
+        mlir::dyn_cast<RankedTensorType>(func_op.getResultTypes()[i]);
     outputs.push_back(CreatTfF32ConstOp(builder, result_type.getShape(), 0.0f,
                                         func_op.getLoc()));
     output_types.push_back(result_type);
