@@ -14,16 +14,18 @@ limitations under the License.
 ==============================================================================*/
 
 #include <algorithm>
+#include <cstdint>
 #include <memory>
 #include <string>
 #include <utility>
 
-#include "absl/strings/string_view.h"
 #include "llvm/ADT/SmallVector.h"
 #include "llvm/ADT/StringRef.h"
+#include "llvm/Support/Casting.h"
 #include "mlir/Dialect/Func/Extensions/AllExtensions.h"  // from @llvm-project
 #include "mlir/Dialect/Func/IR/FuncOps.h"  // from @llvm-project
-#include "mlir/Dialect/Quant/QuantOps.h"  // from @llvm-project
+#include "mlir/Dialect/Quant/IR/Quant.h"  // from @llvm-project
+#include "mlir/Dialect/Quant/IR/QuantTypes.h"  // from @llvm-project
 #include "mlir/Dialect/Shape/IR/Shape.h"  // from @llvm-project
 #include "mlir/IR/Attributes.h"  // from @llvm-project
 #include "mlir/IR/BuiltinAttributeInterfaces.h"  // from @llvm-project
@@ -31,61 +33,41 @@ limitations under the License.
 #include "mlir/IR/BuiltinTypeInterfaces.h"  // from @llvm-project
 #include "mlir/IR/BuiltinTypes.h"  // from @llvm-project
 #include "mlir/IR/MLIRContext.h"  // from @llvm-project
+#include "mlir/IR/Matchers.h"  // from @llvm-project
 #include "mlir/IR/PatternMatch.h"  // from @llvm-project
 #include "mlir/IR/TypeUtilities.h"  // from @llvm-project
+#include "mlir/IR/Value.h"  // from @llvm-project
 #include "mlir/Pass/Pass.h"  // from @llvm-project
 #include "mlir/Support/LLVM.h"  // from @llvm-project
 #include "mlir/Support/LogicalResult.h"  // from @llvm-project
 #include "mlir/Transforms/DialectConversion.h"  // from @llvm-project
 #include "stablehlo/dialect/ChloOps.h"  // from @stablehlo
 #include "tensorflow/compiler/mlir/quantization/stablehlo/passes/bridge/passes.h"
+#include "tensorflow/compiler/mlir/quantization/stablehlo/utils/tf_type_utils.h"
 #include "tensorflow/compiler/mlir/tensorflow/ir/tf_dialect.h"
 #include "tensorflow/compiler/mlir/tensorflow/ir/tf_ops.h"
 #include "tensorflow/compiler/mlir/tensorflow/ir/tf_types.h"
 #include "tensorflow/compiler/mlir/tensorflow/utils/mangling_util.h"
 #include "tensorflow/compiler/mlir/tf2xla/transforms/utils.h"
-#include "tensorflow/compiler/xla/mlir_hlo/mhlo/IR/hlo_ops.h"
-#include "tensorflow/compiler/xla/translate/hlo_to_mhlo/attribute_importer.h"
-#include "tensorflow/compiler/xla/xla_data.pb.h"
+#include "xla/hlo/translate/hlo_to_mhlo/attribute_importer.h"
+#include "xla/mlir_hlo/mhlo/IR/hlo_ops.h"
+#include "xla/xla_data.pb.h"
 #include "tensorflow/core/framework/numeric_types.h"
 #include "tensorflow/core/framework/tensor.h"
 #include "tensorflow/core/framework/tensor.pb.h"
 #include "tensorflow/core/util/quantization/uniform_quant_ops_attr.pb.h"
 #include "tensorflow/core/util/quantization/uniform_quant_ops_params.h"
 
-// TODO: b/289560952 - Move to mlir::quant::stablehlo namespace.
-namespace mlir {
-namespace stablehlo {
+namespace mlir::quant::stablehlo {
 namespace {
 
+using quant::tensorflow::GetDenseAttrFromTensorProtoAttr;
+using quant::tensorflow::GetIntTypeFromTFQint;
+using quant::tensorflow::IsTFQintType;
+
 #define GEN_PASS_DEF_CONVERTTFQUANTOPSTOMHLO
 #include "tensorflow/compiler/mlir/quantization/stablehlo/passes/bridge/passes.h.inc"
 
-FailureOr<IntegerType> GetStorageType(Operation *op,
-                                      Type original_output_element_type,
-                                      PatternRewriter &rewriter) {
-  if (original_output_element_type.isa<TF::Qint8Type>()) {
-    return rewriter.getIntegerType(8);
-  } else if (original_output_element_type.isa<TF::Qint32Type>()) {
-    return rewriter.getIntegerType(32);
-  } else {
-    return rewriter.notifyMatchFailure(
-        op, "Quantized type must be qint8 or qint32.");
-  }
-}
-
-TensorType GetSameShapeTensorType(TensorType tensor_type, Type element_type) {
-  if (auto ranked_tensor_ty =
-          tensor_type.dyn_cast_or_null<RankedTensorType>()) {
-    return RankedTensorType::get(ranked_tensor_ty.getShape(), element_type);
-  }
-  if (auto unranked_tensor_ty =
-          tensor_type.dyn_cast_or_null<UnrankedTensorType>()) {
-    return UnrankedTensorType::get(element_type);
-  }
-  llvm_unreachable("unhandled type");
-}
-
 template <typename UniformQuantizedOp>
 FailureOr<TensorType> GetUniformQuantizedType(
     UniformQuantizedOp op, Type original_type,
@@ -105,17 +87,21 @@ FailureOr<TensorType> GetUniformQuantizedType(
     return rewriter.notifyMatchFailure(op, "zero_points must be constant");
   }
 
-  auto storage_type_or =
-      GetStorageType(op, getElementTypeOrSelf(original_type), rewriter);
-  if (failed(storage_type_or)) {
-    return failure();
+  auto original_element_type = getElementTypeOrSelf(original_type);
+  if (!mlir::isa<TF::Qint8Type, TF::Quint8Type, TF::Qint32Type>(
+          original_element_type)) {
+    return rewriter.notifyMatchFailure(
+        op, "Quantized type must be qint8, quint8 or qint32.");
   }
+  auto storage_type = GetIntTypeFromTFQint(original_element_type);
 
-  const unsigned flags = quant::QuantizationFlags::Signed;
+  const unsigned flags = mlir::isa<TF::Quint8Type>(original_element_type)
+                             ? 0
+                             : quant::QuantizationFlags::Signed;
   Type elem_ty;
   if (quantized_dimension == -1) {
     elem_ty = quant::UniformQuantizedType::get(
-        flags, *storage_type_or, expressed_type, scales.getValues<float>()[0],
+        flags, storage_type, expressed_type, scales.getValues<float>()[0],
         zero_points.getValues<int32_t>()[0], storage_type_min,
         storage_type_max);
   } else {
@@ -125,52 +111,35 @@ FailureOr<TensorType> GetUniformQuantizedType(
     for (auto elem : zero_points.getValues<int32_t>())
       zero_points_vec.push_back(elem);
     elem_ty = quant::UniformQuantizedPerAxisType::get(
-        flags, *storage_type_or, expressed_type, scales_vec, zero_points_vec,
+        flags, storage_type, expressed_type, scales_vec, zero_points_vec,
         quantized_dimension, storage_type_min, storage_type_max);
   }
 
-  return GetSameShapeTensorType(original_type.cast<TensorType>(), elem_ty);
+  return mlir::cast<TensorType>(original_type).clone(elem_ty);
 }
 
-template <typename TFQuantizedType, typename UniformQuantizedOp>
-FailureOr<mhlo::ConstantOp> CreateConstantOp(UniformQuantizedOp op,
-                                             Value original_operand,
-                                             TensorType new_operand_type,
-                                             PatternRewriter &rewriter) {
+// If operand is TF const op, create MHLO constant op from the contents.
+// Otherwise convert the operand to the desired type.
+FailureOr<Value> CreateConstantOrConvertOp(Operation *op, Value operand,
+                                           TensorType new_operand_type,
+                                           PatternRewriter &rewriter) {
   // Check whether the rhs operand has constant op.
   TF::TensorProtoAttr tensor_proto_attr;
-  if (!matchPattern(original_operand, m_Constant(&tensor_proto_attr))) {
-    return rewriter.notifyMatchFailure(op, "operand must be constant.");
+  if (!matchPattern(operand, m_Constant(&tensor_proto_attr))) {
+    return Value(rewriter.create<mhlo::BitcastConvertOp>(
+        op->getLoc(), new_operand_type, operand));
   }
 
-  llvm::StringRef mangled_tensor = tensor_proto_attr.getValue();
-  absl::string_view tensor_view(mangled_tensor.data(), mangled_tensor.size());
-  // TODO(hinsu): Instead of getting the weight from TensorProto, use MLIR
-  // constant attribute to avoid depending on the Tensor proto.
-  tensorflow::TensorProto tensor_proto;
-  tensorflow::Status status =
-      tensorflow::mangling_util::DemangleTensor(tensor_view, &tensor_proto);
-  if (!status.ok()) {
-    return rewriter.notifyMatchFailure(op, status.message());
-  }
+  auto dense_attr_or = GetDenseAttrFromTensorProtoAttr(
+      tensor_proto_attr.getValue(), new_operand_type);
+  if (failed(dense_attr_or)) return failure();
 
-  tensorflow::Tensor t;
-  if (!t.FromProto(tensor_proto)) {
-    return op.emitError("Failed to convert tensor proto to Tensor.");
-  }
-
-  auto arr = t.flat<TFQuantizedType>();
-  auto dense_attr = mlir::DenseElementsAttr::get(
-      GetSameShapeTensorType(
-          new_operand_type,
-          rewriter.getIntegerType(8 * sizeof(TFQuantizedType))),
-      llvm::ArrayRef(arr.data(), arr.size()));
-  return rewriter.create<mhlo::ConstantOp>(op.getLoc(), new_operand_type,
-                                           dense_attr);
+  return Value(rewriter.create<mhlo::ConstantOp>(op->getLoc(), new_operand_type,
+                                                 *dense_attr_or));
 }
 
 xla::ConvolutionDimensionNumbers ConvertConvolutionDimensionNumbers(
-    const tensorflow::UniformQuantizedConvolutionDimensionNumbersAttr
+    const ::tensorflow::UniformQuantizedConvolutionDimensionNumbersAttr
         &dnums_input) {
   xla::ConvolutionDimensionNumbers dnums;
   dnums.set_input_batch_dimension(dnums_input.input_batch_dimension());
@@ -212,8 +181,8 @@ FailureOr<ElementsAttr> ConvertPaddingAttr(
     const xla::ConvolutionDimensionNumbers &dnums, PatternRewriter &rewriter) {
   StringAttr conv_padding = op.getPaddingAttr();
   SmallVector<int64_t> padding_nums;
-  ShapedType lhs_shape = op.getLhs().getType().template cast<ShapedType>();
-  ShapedType rhs_shape = op.getRhs().getType().template cast<ShapedType>();
+  ShapedType lhs_shape = mlir::cast<ShapedType>(op.getLhs().getType());
+  ShapedType rhs_shape = mlir::cast<ShapedType>(op.getRhs().getType());
 
   // Handle only static shape cases.
   // TODO(b/260284866): Handle dynamic shape cases.
@@ -226,33 +195,33 @@ FailureOr<ElementsAttr> ConvertPaddingAttr(
 
   const int64_t padding_nums_size = 2 * (rhs_shape.getRank() - 2);
   padding_nums.reserve(padding_nums_size);
-  if (conv_padding.strref().equals("EXPLICIT")) {
+  if (conv_padding.strref() == "EXPLICIT") {
     for (auto padding_elem :
          op.getExplicitPaddingAttr().template getAsRange<IntegerAttr>()) {
       padding_nums.push_back(padding_elem.getInt());
     }
-  } else if (conv_padding.strref().equals("VALID")) {
+  } else if (conv_padding.strref() == "VALID") {
     padding_nums.resize(padding_nums_size, 0);
   } else {
     padding_nums.resize(padding_nums_size);
     for (int i = 0; i < dnums.input_spatial_dimensions_size(); ++i) {
       const int64_t stride =
-          op.getWindowStridesAttr()[i].template cast<IntegerAttr>().getInt();
+          mlir::cast<IntegerAttr>(op.getWindowStridesAttr()[i]).getInt();
       const int64_t lhs_size_dilated =
-          tensorflow::UniformQuantizedConvolutionParams::DilatedSize(
+          ::tensorflow::UniformQuantizedConvolutionParams::DilatedSize(
               lhs_shape.getDimSize(dnums.input_spatial_dimensions(i)),
-              op.getLhsDilationAttr()[i].template cast<IntegerAttr>().getInt());
+              mlir::cast<IntegerAttr>(op.getLhsDilationAttr()[i]).getInt());
       const int64_t rhs_size_dilated =
-          tensorflow::UniformQuantizedConvolutionParams::DilatedSize(
+          ::tensorflow::UniformQuantizedConvolutionParams::DilatedSize(
               rhs_shape.getDimSize(dnums.kernel_spatial_dimensions(i)),
-              op.getRhsDilationAttr()[i].template cast<IntegerAttr>().getInt());
+              mlir::cast<IntegerAttr>(op.getRhsDilationAttr()[i]).getInt());
 
       const int64_t output_size = (lhs_size_dilated + stride - 1) / stride;
       const int64_t total_padding = std::max(
           (output_size - 1) * stride + rhs_size_dilated - lhs_size_dilated,
           static_cast<int64_t>(0));
-      const int64_t padding_end = total_padding / 2;
-      const int64_t padding_begin = total_padding - padding_end;
+      const int64_t padding_begin = total_padding / 2;
+      const int64_t padding_end = total_padding - padding_begin;
       padding_nums[2 * i] = padding_begin;
       padding_nums[2 * i + 1] = padding_end;
     }
@@ -270,7 +239,7 @@ FailureOr<SmallVector<NamedAttribute>> ConvertToMhloConvolutionOpAttrs(
     UniformQuantizedConvolutionOp op, PatternRewriter &rewriter) {
   // TODO(b/261005147): Update the lowering logic after migration to mhlo
   // ConvolutionDimensionNumbers.
-  tensorflow::UniformQuantizedConvolutionDimensionNumbersAttr dnums_input;
+  ::tensorflow::UniformQuantizedConvolutionDimensionNumbersAttr dnums_input;
   if (!dnums_input.ParseFromString(std::string(op.getDimensionNumbers()))) {
     return op->emitError("Parse dimension_numbers failed.");
   }
@@ -296,7 +265,7 @@ FailureOr<SmallVector<NamedAttribute>> ConvertToMhloConvolutionOpAttrs(
                attr.getName() == op.getLhsDilationAttrName() ||
                attr.getName() == op.getRhsDilationAttrName()) {
       attr.setValue(ConvertToDenseElementsAttr(
-          attr.getValue().template cast<ArrayAttr>(), rewriter));
+          mlir::cast<ArrayAttr>(attr.getValue()), rewriter));
       converted_attrs.push_back(attr);
     }
   }
@@ -306,12 +275,14 @@ FailureOr<SmallVector<NamedAttribute>> ConvertToMhloConvolutionOpAttrs(
 // TODO(hinsu): Move this pattern to legalize_tf after resolving the dependency
 // on the tensor proto.
 class ConvertUniformQuantizedDotHybridOp
-    : public OpRewritePattern<TF::UniformQuantizedDotHybridOp> {
+    : public OpConversionPattern<TF::UniformQuantizedDotHybridOp> {
  public:
-  using OpRewritePattern::OpRewritePattern;
+  using OpConversionPattern::OpConversionPattern;
 
-  LogicalResult matchAndRewrite(TF::UniformQuantizedDotHybridOp op,
-                                PatternRewriter &rewriter) const override {
+  LogicalResult matchAndRewrite(
+      TF::UniformQuantizedDotHybridOp op,
+      TF::UniformQuantizedDotHybridOpAdaptor adaptor,
+      ConversionPatternRewriter &rewriter) const override {
     // Uniform Quantized type for the rhs.
     int64_t rhs_quantized_dimension = op.getRhsQuantizationAxis();
     // Currently for dot, PTQ supports per-tensor quantization.
@@ -327,26 +298,28 @@ class ConvertUniformQuantizedDotHybridOp
       return failure();
     }
 
-    auto rhs = CreateConstantOp<tensorflow::qint8>(op, op.getRhs(), *rhs_type,
-                                                   rewriter);
-    if (failed(rhs)) {
+    auto rhs_or =
+        CreateConstantOrConvertOp(op, adaptor.getRhs(), *rhs_type, rewriter);
+    if (failed(rhs_or)) {
       return failure();
     }
 
     rewriter.replaceOpWithNewOp<mhlo::DotOp>(op, op.getType(), op.getLhs(),
-                                             *rhs,
+                                             *rhs_or,
                                              /*precision_config=*/nullptr);
     return success();
   }
 };
 
 class ConvertUniformQuantizedConvolutionHybridOp
-    : public OpRewritePattern<TF::UniformQuantizedConvolutionHybridOp> {
+    : public OpConversionPattern<TF::UniformQuantizedConvolutionHybridOp> {
  public:
-  using OpRewritePattern::OpRewritePattern;
+  using OpConversionPattern::OpConversionPattern;
 
-  LogicalResult matchAndRewrite(TF::UniformQuantizedConvolutionHybridOp op,
-                                PatternRewriter &rewriter) const override {
+  LogicalResult matchAndRewrite(
+      TF::UniformQuantizedConvolutionHybridOp op,
+      TF::UniformQuantizedConvolutionHybridOpAdaptor adaptor,
+      ConversionPatternRewriter &rewriter) const override {
     // Uniform Quantized type for the rhs.
     auto rhs_type = GetUniformQuantizedType(
         op, op.getRhs().getType(), op.getRhsScales(), op.getRhsZeroPoints(),
@@ -356,9 +329,9 @@ class ConvertUniformQuantizedConvolutionHybridOp
       return failure();
     }
 
-    auto rhs = CreateConstantOp<tensorflow::qint8>(op, op.getRhs(), *rhs_type,
-                                                   rewriter);
-    if (failed(rhs)) {
+    auto rhs_or =
+        CreateConstantOrConvertOp(op, adaptor.getRhs(), *rhs_type, rewriter);
+    if (failed(rhs_or)) {
       return failure();
     }
 
@@ -366,7 +339,7 @@ class ConvertUniformQuantizedConvolutionHybridOp
     if (failed(converted_attrs_or)) {
       return failure();
     }
-    SmallVector<Value, 2> operands{op.getLhs(), *rhs};
+    SmallVector<Value, 2> operands{op.getLhs(), *rhs_or};
     rewriter.replaceOpWithNewOp<mhlo::ConvolutionOp>(op, op.getType(), operands,
                                                      *converted_attrs_or);
     return success();
@@ -388,8 +361,15 @@ class ConvertUniformQuantizeOp
       return failure();
     }
 
-    rewriter.replaceOpWithNewOp<mhlo::UniformQuantizeOp>(op, *output_type,
-                                                         op.getInput());
+    auto result = rewriter.create<mhlo::UniformQuantizeOp>(
+        op->getLoc(), *output_type, op.getInput());
+    rewriter.replaceOpWithNewOp<mhlo::BitcastConvertOp>(
+        op,
+        output_type->clone(
+            mlir::dyn_cast<quant::QuantizedType>(output_type->getElementType())
+                .getStorageType()),
+        result);
+
     return success();
   }
 };
@@ -409,6 +389,16 @@ class ConvertUniformDequantizeOp
       ConversionPatternRewriter &rewriter) const override {
     Value input = adaptor.getInput();
 
+    auto input_quant_type = GetUniformQuantizedType(
+        op, op.getInput().getType(), op.getScales(), op.getZeroPoints(),
+        /*expressed_type=*/rewriter.getF32Type(), op.getQuantizationMinVal(),
+        op.getQuantizationMaxVal(), op.getQuantizationAxis(), rewriter);
+    if (failed(input_quant_type)) {
+      return failure();
+    }
+    input = rewriter.create<mhlo::BitcastConvertOp>(op->getLoc(),
+                                                    *input_quant_type, input);
+
     rewriter.replaceOpWithNewOp<mhlo::UniformDequantizeOp>(
         op, op.getOutput().getType(), input);
     return success();
@@ -425,6 +415,16 @@ class ConvertUniformRequantizeOp
       ConversionPatternRewriter &rewriter) const override {
     Value input = adaptor.getInput();
 
+    auto input_quant_type = GetUniformQuantizedType(
+        op, op.getInput().getType(), op.getInputScales(),
+        op.getInputZeroPoints(),
+        /*expressed_type=*/rewriter.getF32Type(),
+        op.getInputQuantizationMinVal(), op.getInputQuantizationMaxVal(),
+        op.getInputQuantizationAxis(), rewriter);
+    if (failed(input_quant_type)) {
+      return failure();
+    }
+
     auto output_type = GetUniformQuantizedType(
         op, op.getOutput().getType(), op.getOutputScales(),
         op.getOutputZeroPoints(),
@@ -435,8 +435,16 @@ class ConvertUniformRequantizeOp
       return failure();
     }
 
-    rewriter.replaceOpWithNewOp<mhlo::UniformQuantizeOp>(op, *output_type,
-                                                         input);
+    auto input_quant = rewriter.create<mhlo::BitcastConvertOp>(
+        op->getLoc(), *input_quant_type, input);
+    auto result = rewriter.create<mhlo::UniformQuantizeOp>(
+        op->getLoc(), *output_type, input_quant);
+    rewriter.replaceOpWithNewOp<mhlo::BitcastConvertOp>(
+        op,
+        output_type->clone(
+            mlir::dyn_cast<quant::QuantizedType>(output_type->getElementType())
+                .getStorageType()),
+        result);
     return success();
   }
 };
@@ -451,6 +459,16 @@ class ConvertUniformQuantizedDotOp
       ConversionPatternRewriter &rewriter) const override {
     Value lhs = adaptor.getLhs();
 
+    auto lhs_quant_type = GetUniformQuantizedType(
+        op, op.getLhs().getType(), op.getLhsScales(), op.getLhsZeroPoints(),
+        /*expressed_type=*/rewriter.getF32Type(), op.getLhsQuantizationMinVal(),
+        op.getLhsQuantizationMaxVal(), op.getLhsQuantizationAxis(), rewriter);
+    if (failed(lhs_quant_type)) {
+      return failure();
+    }
+    lhs = rewriter.create<mhlo::BitcastConvertOp>(op->getLoc(), *lhs_quant_type,
+                                                  adaptor.getLhs());
+
     // Uniform Quantized type for the rhs.
     int64_t rhs_quantized_dimension = op.getRhsQuantizationAxis();
     // Currently for dot, PTQ supports per-tensor quantization.
@@ -459,16 +477,15 @@ class ConvertUniformQuantizedDotOp
           op, "Legalization supports only rhs_quantization_axis -1.");
     }
     auto rhs_type = GetUniformQuantizedType(
-        op, adaptor.getRhs().getType(), op.getRhsScales(),
-        op.getRhsZeroPoints(),
+        op, op.getRhs().getType(), op.getRhsScales(), op.getRhsZeroPoints(),
         /*expressed_type=*/rewriter.getF32Type(), op.getRhsQuantizationMinVal(),
         op.getRhsQuantizationMaxVal(), rhs_quantized_dimension, rewriter);
     if (failed(rhs_type)) {
       return failure();
     }
 
-    auto rhs_or = CreateConstantOp<tensorflow::qint8>(op, op.getRhs(),
-                                                      *rhs_type, rewriter);
+    auto rhs_or =
+        CreateConstantOrConvertOp(op, adaptor.getRhs(), *rhs_type, rewriter);
     if (failed(rhs_or)) {
       return failure();
     }
@@ -483,8 +500,15 @@ class ConvertUniformQuantizedDotOp
       return failure();
     }
 
-    rewriter.replaceOpWithNewOp<mhlo::DotOp>(op, *output_type, lhs, *rhs_or,
-                                             /*precision_config=*/nullptr);
+    auto result =
+        rewriter.create<mhlo::DotOp>(op->getLoc(), *output_type, lhs, *rhs_or,
+                                     /*precision_config=*/nullptr);
+    rewriter.replaceOpWithNewOp<mhlo::BitcastConvertOp>(
+        op,
+        output_type->clone(
+            mlir::dyn_cast<quant::QuantizedType>(output_type->getElementType())
+                .getStorageType()),
+        result);
     return success();
   }
 };
@@ -500,17 +524,26 @@ class ConvertUniformQuantizedConvolutionOp
       ConversionPatternRewriter &rewriter) const override {
     Value lhs = adaptor.getLhs();
 
+    auto lhs_quant_type = GetUniformQuantizedType(
+        op, op.getLhs().getType(), op.getLhsScales(), op.getLhsZeroPoints(),
+        /*expressed_type=*/rewriter.getF32Type(), op.getLhsQuantizationMinVal(),
+        op.getLhsQuantizationMaxVal(), op.getLhsQuantizationAxis(), rewriter);
+    if (failed(lhs_quant_type)) {
+      return failure();
+    }
+    lhs = rewriter.create<mhlo::BitcastConvertOp>(op->getLoc(), *lhs_quant_type,
+                                                  adaptor.getLhs());
+
     auto rhs_type = GetUniformQuantizedType(
-        op, adaptor.getRhs().getType(), op.getRhsScales(),
-        op.getRhsZeroPoints(),
+        op, op.getRhs().getType(), op.getRhsScales(), op.getRhsZeroPoints(),
         /*expressed_type=*/rewriter.getF32Type(), op.getRhsQuantizationMinVal(),
         op.getRhsQuantizationMaxVal(), op.getRhsQuantizationAxis(), rewriter);
     if (failed(rhs_type)) {
       return failure();
     }
 
-    auto rhs_or = CreateConstantOp<tensorflow::qint8>(op, op.getRhs(),
-                                                      *rhs_type, rewriter);
+    auto rhs_or =
+        CreateConstantOrConvertOp(op, adaptor.getRhs(), *rhs_type, rewriter);
     if (failed(rhs_or)) {
       return failure();
     }
@@ -530,8 +563,14 @@ class ConvertUniformQuantizedConvolutionOp
       return failure();
     }
     SmallVector<Value, 2> operands{lhs, *rhs_or};
-    rewriter.replaceOpWithNewOp<mhlo::ConvolutionOp>(op, *output_type, operands,
-                                                     *converted_attrs_or);
+    auto result = rewriter.create<mhlo::ConvolutionOp>(
+        op->getLoc(), *output_type, operands, *converted_attrs_or);
+    rewriter.replaceOpWithNewOp<mhlo::BitcastConvertOp>(
+        op,
+        output_type->clone(
+            mlir::dyn_cast<quant::QuantizedType>(output_type->getElementType())
+                .getStorageType()),
+        result);
     return success();
   }
 };
@@ -546,26 +585,36 @@ class ConvertUniformQuantizedAddOp
       ConversionPatternRewriter &rewriter) const override {
     Value lhs = adaptor.getLhs();
 
-    auto lhs_type = lhs.getType().cast<ShapedType>();
+    auto lhs_type = mlir::cast<ShapedType>(lhs.getType());
     if (!lhs_type.hasRank()) {
       return rewriter.notifyMatchFailure(
           op, "Legalization supports cases where only lhs rank known.");
     }
+
+    auto lhs_quant_type = GetUniformQuantizedType(
+        op, op.getLhs().getType(), op.getLhsScales(), op.getLhsZeroPoints(),
+        /*expressed_type=*/rewriter.getF32Type(), op.getLhsQuantizationMinVal(),
+        op.getLhsQuantizationMaxVal(), op.getLhsQuantizationAxis(), rewriter);
+    if (failed(lhs_quant_type)) {
+      return failure();
+    }
+    lhs = rewriter.create<mhlo::BitcastConvertOp>(op->getLoc(), *lhs_quant_type,
+                                                  adaptor.getLhs());
+
     // rhs (bias) is always 1D that broadcasts to the last dim of lhs.
     auto broadcast_dims =
-        mhlo::GetI64ElementsAttr({lhs_type.getRank() - 1}, &rewriter);
+        rewriter.getDenseI64ArrayAttr({lhs_type.getRank() - 1});
 
     auto rhs_type = GetUniformQuantizedType(
-        op, adaptor.getRhs().getType(), op.getRhsScales(),
-        op.getRhsZeroPoints(),
+        op, op.getRhs().getType(), op.getRhsScales(), op.getRhsZeroPoints(),
         /*expressed_type=*/rewriter.getF32Type(), op.getRhsQuantizationMinVal(),
         op.getRhsQuantizationMaxVal(), op.getRhsQuantizationAxis(), rewriter);
     if (failed(rhs_type)) {
       return failure();
     }
 
-    auto rhs_or = CreateConstantOp<tensorflow::qint32>(op, op.getRhs(),
-                                                       *rhs_type, rewriter);
+    auto rhs_or =
+        CreateConstantOrConvertOp(op, adaptor.getRhs(), *rhs_type, rewriter);
     if (failed(rhs_or)) {
       return failure();
     }
@@ -582,8 +631,14 @@ class ConvertUniformQuantizedAddOp
 
     // lhs, rhs, output scales and zero_points are guaranteed (by the TF
     // quantizer) to be identical, respectively.
-    rewriter.replaceOpWithNewOp<chlo::BroadcastAddOp>(op, *output_type, lhs,
-                                                      *rhs_or, broadcast_dims);
+    auto result = rewriter.create<chlo::BroadcastAddOp>(
+        op->getLoc(), *output_type, lhs, *rhs_or, broadcast_dims);
+    rewriter.replaceOpWithNewOp<mhlo::BitcastConvertOp>(
+        op,
+        output_type->clone(
+            mlir::dyn_cast<quant::QuantizedType>(output_type->getElementType())
+                .getStorageType()),
+        result);
     return success();
   }
 };
@@ -604,23 +659,22 @@ class ConvertUniformQuantizedClipByValueOp
     if (quantization_axis >= 0) {
       broadcast_dims_values.push_back(quantization_axis);
     }
-    auto broadcast_dims =
-        mhlo::GetI64ElementsAttr(broadcast_dims_values, &rewriter);
+    auto broadcast_dims = rewriter.getDenseI64ArrayAttr(broadcast_dims_values);
 
     auto min_max_type = GetUniformQuantizedType(
-        op, adaptor.getMin().getType(), op.getScales(), op.getZeroPoints(),
+        op, op.getMin().getType(), op.getScales(), op.getZeroPoints(),
         /*expressed_type=*/rewriter.getF32Type(), op.getQuantizationMinVal(),
         op.getQuantizationMaxVal(), op.getQuantizationAxis(), rewriter);
     if (failed(min_max_type)) {
       return failure();
     }
-    auto min_or = CreateConstantOp<tensorflow::qint32>(op, op.getMin(),
-                                                       *min_max_type, rewriter);
+    auto min_or = CreateConstantOrConvertOp(op, adaptor.getMin(), *min_max_type,
+                                            rewriter);
     if (failed(min_or)) {
       return failure();
     }
-    auto max_or = CreateConstantOp<tensorflow::qint32>(op, op.getMax(),
-                                                       *min_max_type, rewriter);
+    auto max_or = CreateConstantOrConvertOp(op, adaptor.getMax(), *min_max_type,
+                                            rewriter);
     if (failed(max_or)) {
       return failure();
     }
@@ -632,11 +686,41 @@ class ConvertUniformQuantizedClipByValueOp
     if (failed(output_type)) {
       return failure();
     }
+    operand = rewriter.create<mhlo::BitcastConvertOp>(op->getLoc(),
+                                                      *output_type, operand);
 
     Value res_min_clipped = rewriter.create<chlo::BroadcastMaxOp>(
         op->getLoc(), *output_type, operand, *min_or, broadcast_dims);
-    rewriter.replaceOpWithNewOp<chlo::BroadcastMinOp>(
-        op, *output_type, res_min_clipped, *max_or, broadcast_dims);
+    Value res_max_clipped = rewriter.create<chlo::BroadcastMinOp>(
+        op->getLoc(), *output_type, res_min_clipped, *max_or, broadcast_dims);
+    rewriter.replaceOpWithNewOp<mhlo::BitcastConvertOp>(
+        op,
+        output_type->clone(
+            mlir::dyn_cast<quant::QuantizedType>(output_type->getElementType())
+                .getStorageType()),
+        res_max_clipped);
+    return success();
+  }
+};
+
+// This pattern converts qint <-> int CastOp to int -> int ConvertOps.
+// The former are introduced in ConvertTFQuantTypes pass. The resulting int <->
+// int ConvertOps are no-ops and can be removed later in a Canonicalizer pass.
+class ConvertTfCastOp : public OpConversionPattern<TF::CastOp> {
+ public:
+  using OpConversionPattern::OpConversionPattern;
+
+  LogicalResult matchAndRewrite(
+      TF::CastOp op, TF::CastOpAdaptor adaptor,
+      ConversionPatternRewriter &rewriter) const override {
+    Type output_type = op.getDstT();
+    if (!IsTFQintType(output_type) && !IsTFQintType(op.getSrcT())) {
+      // skip CastOps with no qint types.
+      return failure();
+    }
+    Value input = adaptor.getX();
+    rewriter.replaceOpWithNewOp<mhlo::ConvertOp>(
+        op, input, GetIntTypeFromTFQint(output_type));
     return success();
   }
 };
@@ -663,6 +747,10 @@ void ConvertTFQuantOpsToMHLO::runOnOperation() {
       TF::UniformQuantizedConvolutionOp,
       TF::UniformQuantizedConvolutionHybridOp, TF::UniformQuantizedAddOp,
       TF::UniformQuantizedClipByValueOp>();
+  target.addDynamicallyLegalOp<TF::CastOp>([](Operation *op) {
+    auto cast_op = llvm::dyn_cast<TF::CastOp>(op);
+    return !IsTFQintType(cast_op.getSrcT()) && !IsTFQintType(cast_op.getDstT());
+  });
 
   RewritePatternSet patterns(ctx);
   PopulateLegalizeTfQuantizationPatterns(ctx, &patterns);
@@ -681,7 +769,7 @@ void PopulateLegalizeTfQuantizationPatterns(MLIRContext *context,
             ConvertUniformQuantizeOp, ConvertUniformRequantizeOp,
             ConvertUniformDequantizeOp, ConvertUniformQuantizedDotOp,
             ConvertUniformQuantizedConvolutionOp, ConvertUniformQuantizedAddOp,
-            ConvertUniformQuantizedClipByValueOp>(context);
+            ConvertUniformQuantizedClipByValueOp, ConvertTfCastOp>(context);
 }
 
 std::unique_ptr<OperationPass<func::FuncOp>>
@@ -689,5 +777,4 @@ CreateConvertTFQuantOpsToMHLOPass() {
   return std::make_unique<ConvertTFQuantOpsToMHLO>();
 }
 
-}  // namespace stablehlo
-}  // namespace mlir
+}  // namespace mlir::quant::stablehlo
