@@ -16,17 +16,17 @@ See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
 
+#include <cstdint>
+
 #include "mlir/Support/LogicalResult.h"  // from @llvm-project
 
-namespace mlir {
-namespace stablehlo {
+namespace mlir::quant::stablehlo {
 
 // Decomposes a given floating point value num into a normalized and quantized
 // fraction and an integral power of two.
 LogicalResult QuantizeMultiplier(double double_multiplier,
                                  int32_t& quantized_fraction, int32_t& shift);
 
-}  // namespace stablehlo
-}  // namespace mlir
+}  // namespace mlir::quant::stablehlo
 
 #endif  // TENSORFLOW_COMPILER_MLIR_QUANTIZATION_STABLEHLO_UTILS_MATH_UTILS_H_
