@@ -13,30 +13,41 @@ See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
 #include <algorithm>
+#include <cstdint>
 #include <iterator>
 #include <memory>
 #include <string>
-#include <tuple>
-#include <utility>
 #include <vector>
 
 #include "absl/algorithm/container.h"
 #include "absl/strings/str_cat.h"
 #include "absl/strings/str_join.h"
+#include "absl/strings/str_split.h"
 #include "absl/strings/string_view.h"
 #include "llvm/ADT/STLExtras.h"
+#include "llvm/ADT/StringMap.h"
 #include "llvm/ADT/StringRef.h"
-#include "llvm/ADT/StringSet.h"
+#include "llvm/ADT/StringSet.h"  // IWYU pragma: keep
 #include "mlir/Dialect/Func/IR/FuncOps.h"  // from @llvm-project
+#include "mlir/IR/Attributes.h"  // from @llvm-project
 #include "mlir/IR/Builders.h"  // from @llvm-project
+#include "mlir/IR/BuiltinOps.h"  // from @llvm-project
+#include "mlir/IR/Location.h"  // from @llvm-project
+#include "mlir/IR/MLIRContext.h"  // from @llvm-project
 #include "mlir/IR/OperationSupport.h"  // from @llvm-project
+#include "mlir/IR/SymbolTable.h"  // from @llvm-project
+#include "mlir/IR/TypeRange.h"  // from @llvm-project
+#include "mlir/IR/Value.h"  // from @llvm-project
+#include "mlir/Interfaces/FunctionInterfaces.h"  // from @llvm-project
 #include "mlir/Pass/Pass.h"  // from @llvm-project
+#include "mlir/Pass/PassRegistry.h"  // from @llvm-project
 #include "mlir/Support/LLVM.h"  // from @llvm-project
+#include "mlir/Support/TypeID.h"  // from @llvm-project
+#include "tensorflow/compiler/mlir/quantization/common/attrs_and_constraints.h"
 #include "tensorflow/compiler/mlir/quantization/tensorflow/passes/passes.h"
 #include "tensorflow/compiler/mlir/tensorflow/ir/tf_ops.h"
 #include "tensorflow/compiler/mlir/tensorflow/ir/tf_saved_model.h"
 #include "tensorflow/compiler/mlir/tensorflow/translate/import_model.h"
-#include "tensorflow/core/platform/macros.h"
 
 namespace mlir {
 namespace quant {
@@ -99,7 +110,7 @@ void SetFunctionPrivate(func::FuncOp func) {
   // The `tf_saved_model` attributes can only be applied to public functions.
   for (auto& attr : func->getAttrs()) {
     StringRef attr_name = attr.getName().getValue();
-    if (attr_name.startswith("tf_saved_model.")) {
+    if (attr_name.starts_with("tf_saved_model.")) {
       func->removeAttr(attr_name);
     }
   }
@@ -108,7 +119,7 @@ void SetFunctionPrivate(func::FuncOp func) {
   for (int i = 0; i < func.getNumArguments(); ++i) {
     for (auto& attr : iface.getArgAttrs(i)) {
       const StringAttr& attr_name = attr.getName();
-      if (attr_name.getValue().startswith("tf_saved_model.")) {
+      if (attr_name.getValue().starts_with("tf_saved_model.")) {
         func.removeArgAttr(i, attr_name);
       }
     }
@@ -116,7 +127,7 @@ void SetFunctionPrivate(func::FuncOp func) {
   for (int i = 0; i < func.getNumResults(); ++i) {
     for (auto& attr : iface.getResultAttrs(i)) {
       const StringAttr& attr_name = attr.getName();
-      if (attr_name.getValue().startswith("tf_saved_model.")) {
+      if (attr_name.getValue().starts_with("tf_saved_model.")) {
         func.removeResultAttr(i, attr_name);
       }
     }
@@ -153,7 +164,7 @@ void GetUniqueInputOutputNodeNames(ModuleOp module_op,
 
       if (auto inputs_attr = tf_attrs.get("inputs")) {
         const std::string inputs_attr_str =
-            inputs_attr.cast<StringAttr>().getValue().str();
+            mlir::cast<StringAttr>(inputs_attr).getValue().str();
         std::vector<std::string> fn_input_names =
             absl::StrSplit(inputs_attr_str, ',', absl::SkipEmpty());
 
@@ -173,7 +184,7 @@ void GetUniqueInputOutputNodeNames(ModuleOp module_op,
 
       if (auto outputs_attr = tf_attrs.get("outputs")) {
         const std::string outputs_attr_str =
-            outputs_attr.cast<StringAttr>().getValue().str();
+            mlir::cast<StringAttr>(outputs_attr).getValue().str();
         std::vector<std::string> fn_output_names =
             absl::StrSplit(outputs_attr_str, ',', absl::SkipEmpty());
 
@@ -294,7 +305,8 @@ bool CreateMainFunction(ModuleOp module_op) {
     result_idx += func_op.getNumResults();
 
     auto call_op = builder.create<TF::PartitionedCallOp>(
-        module_op.getLoc(), new_types, new_args,
+        module_op.getLoc(), new_types, new_args, /*args_attrs=*/nullptr,
+        /*res_attrs=*/nullptr,
         SymbolRefAttr::get(context, func_op.getSymName()),
         /*config=*/builder.getStringAttr(""),
         /*config_proto=*/builder.getStringAttr(""),
