@@ -15,35 +15,35 @@ limitations under the License.
 // Copied and modified from
 // //third_party/tensorflow/compiler/mlir/lite/transforms/prepare_quantize.cc
 // This transformation pass applies quantization propagation on TF dialect.
-#include <initializer_list>
 #include <iterator>
 #include <memory>
 #include <optional>
-#include <string>
 #include <utility>
 
+#include "llvm/Support/Casting.h"
+#include "llvm/Support/CommandLine.h"
 #include "mlir/Dialect/Func/IR/FuncOps.h"  // from @llvm-project
-#include "mlir/Dialect/Quant/QuantOps.h"  // from @llvm-project
-#include "mlir/Dialect/Quant/QuantTypes.h"  // from @llvm-project
+#include "mlir/Dialect/Quant/IR/Quant.h"  // from @llvm-project
+#include "mlir/IR/Builders.h"  // from @llvm-project
 #include "mlir/IR/BuiltinAttributes.h"  // from @llvm-project
-#include "mlir/IR/BuiltinOps.h"  // from @llvm-project
-#include "mlir/IR/BuiltinTypes.h"  // from @llvm-project
+#include "mlir/IR/BuiltinTypeInterfaces.h"  // from @llvm-project
 #include "mlir/IR/MLIRContext.h"  // from @llvm-project
 #include "mlir/IR/Operation.h"  // from @llvm-project
 #include "mlir/IR/PatternMatch.h"  // from @llvm-project
+#include "mlir/IR/Types.h"  // from @llvm-project
 #include "mlir/IR/Value.h"  // from @llvm-project
 #include "mlir/Pass/Pass.h"  // from @llvm-project
+#include "mlir/Pass/PassRegistry.h"  // from @llvm-project
+#include "mlir/Support/LLVM.h"  // from @llvm-project
+#include "mlir/Support/TypeID.h"  // from @llvm-project
 #include "mlir/Transforms/GreedyPatternRewriteDriver.h"  // from @llvm-project
-#include "tensorflow/compiler/mlir/lite/quantization/ir/FakeQuantSupport.h"
-#include "tensorflow/compiler/mlir/lite/quantization/ir/QuantOps.h"
-#include "tensorflow/compiler/mlir/lite/quantization/quantization_config.h"
-#include "tensorflow/compiler/mlir/lite/quantization/quantization_traits.h"
-#include "tensorflow/compiler/mlir/lite/quantization/quantization_utils.h"
-#include "tensorflow/compiler/mlir/lite/transforms/passes.h"
+#include "tensorflow/compiler/mlir/quantization/common/ir/QuantOps.h"
+#include "tensorflow/compiler/mlir/quantization/common/quantization_lib/quantization_config.h"
+#include "tensorflow/compiler/mlir/quantization/common/quantization_lib/quantization_driver.h"
+#include "tensorflow/compiler/mlir/quantization/common/quantization_lib/quantization_utils.h"
 #include "tensorflow/compiler/mlir/quantization/tensorflow/ops/tf_op_quant_spec.h"
-#include "tensorflow/compiler/mlir/quantization/tensorflow/passes/utils.h"
 #include "tensorflow/compiler/mlir/tensorflow/ir/tf_dialect.h"
-#include "tensorflow/compiler/mlir/tensorflow/ir/tf_ops.h"
+#include "tensorflow/compiler/mlir/tensorflow/ir/tf_ops.h"  // IWYU pragma: keep
 
 //===----------------------------------------------------------------------===//
 // The prepare-quantize Pass.
@@ -53,8 +53,10 @@ namespace quant {
 
 namespace {
 
-using QuantMethod =
-    tensorflow::quantization::QuantizationMethod::ExperimentalMethod;
+using ::mlir::quant::ir::DequantizeCastOp;
+using ::mlir::quant::ir::QuantizeCastOp;
+
+using QuantMethod = tensorflow::quantization::QuantizationMethod::PresetMethod;
 
 // Applies prepare quantization on the model in TF dialect. This pass runs
 // before the quantization pass and propagate the quantization parameters
@@ -64,8 +66,8 @@ using QuantMethod =
 class PrepareQuantizePass
     : public PassWrapper<PrepareQuantizePass, OperationPass<func::FuncOp>> {
   void getDependentDialects(DialectRegistry& registry) const override {
-    registry.insert<TF::TensorFlowDialect, ::mlir::quant::QuantizationDialect,
-                    ::mlir::quantfork::QuantizationForkDialect>();
+    registry.insert<TF::TensorFlowDialect, ::mlir::quant::QuantDialect,
+                    ::mlir::quant::ir::TFQuantDialect>();
   }
 
  public:
@@ -84,8 +86,8 @@ class PrepareQuantizePass
     quant_specs_.inference_type = tensorflow::DT_QINT8;
     enable_per_channel_quantization_ = !quant_specs_.disable_per_channel;
     enable_post_training_quantize_ =
-        (quantization_method ==
-         tensorflow::quantization::QuantizationMethod::STATIC_RANGE);
+        (quantization_method == tensorflow::quantization::QuantizationMethod::
+                                    METHOD_STATIC_RANGE_INT8);
   }
 
   PrepareQuantizePass(const PrepareQuantizePass& other) {
@@ -169,14 +171,13 @@ class PrepareQuantizePass
 bool PrepareQuantizePass::SetInputNodesQuantizationParams(func::FuncOp func) {
   StringRef func_name = func.getName();
   auto has_quantize_op = [&](const Value arg) {
-    return (arg.hasOneUse() &&
-            llvm::isa<quantfork::QuantizeCastOp>(*arg.user_begin()));
+    return (arg.hasOneUse() && llvm::isa<QuantizeCastOp>(*arg.user_begin()));
   };
 
   bool need_to_set_input_nodes_quantization_params = false;
   for (const BlockArgument arg : func.getArguments()) {
-    auto shaped = arg.getType().dyn_cast<ShapedType>();
-    if (shaped && shaped.getElementType().isa<FloatType>() &&
+    auto shaped = mlir::dyn_cast<ShapedType>(arg.getType());
+    if (shaped && mlir::isa<FloatType>(shaped.getElementType()) &&
         !has_quantize_op(arg)) {
       need_to_set_input_nodes_quantization_params = true;
       break;
@@ -201,8 +202,8 @@ bool PrepareQuantizePass::SetInputNodesQuantizationParams(func::FuncOp func) {
   auto add_quantize_op = [&](Location loc, Type input_type, Block* block,
                              Block::iterator insertion_point, Value arg,
                              int i) {
-    if (auto shaped = input_type.dyn_cast<ShapedType>()) {
-      if (shaped.getElementType().isa<FloatType>()) {
+    if (auto shaped = mlir::dyn_cast<ShapedType>(input_type)) {
+      if (mlir::isa<FloatType>(shaped.getElementType())) {
         // If there are existing quantize ops, they are from training and we
         // should respect them.
         if (has_quantize_op(arg)) {
@@ -218,10 +219,9 @@ bool PrepareQuantizePass::SetInputNodesQuantizationParams(func::FuncOp func) {
             builder.getF64FloatAttr(min_max.second.value()),
             /*quant_dim=*/-1, num_bits, narrow_range, is_signed);
         builder.setInsertionPoint(block, insertion_point);
-        auto q_op = builder.create<quantfork::QuantizeCastOp>(
-            loc, params.getValue(), arg);
-        auto dq_op = builder.create<quantfork::DequantizeCastOp>(
-            loc, input_type, q_op.getResult());
+        auto q_op = builder.create<QuantizeCastOp>(loc, params.getValue(), arg);
+        auto dq_op =
+            builder.create<DequantizeCastOp>(loc, input_type, q_op.getResult());
         arg.replaceAllUsesWith(dq_op.getResult());
         q_op.setOperand(arg);
       }
@@ -243,8 +243,8 @@ bool PrepareQuantizePass::RemoveRedundantStats(func::FuncOp func) {
 }
 
 static Value Quantized(Operation* user) {
-  if (auto q = llvm::dyn_cast_or_null<quantfork::QuantizeCastOp>(user)) {
-    if (auto dq = llvm::dyn_cast_or_null<quantfork::DequantizeCastOp>(
+  if (auto q = llvm::dyn_cast_or_null<QuantizeCastOp>(user)) {
+    if (auto dq = llvm::dyn_cast_or_null<DequantizeCastOp>(
             *q.getResult().user_begin())) {
       return dq.getResult();
     }
@@ -286,10 +286,10 @@ void PrepareQuantizePass::SanityCheckAndAdjustment(func::FuncOp func) {
   // dialect (presumably enable re-use in xla etc) quantfork::*QuantizeCastOp
   // we're matching here.
   //
-  func.walk([&](quantfork::QuantizeCastOp q_op) {
+  func.walk([&](QuantizeCastOp q_op) {
     // If up with end up with
-    auto dq_op = dyn_cast_or_null<quantfork::DequantizeCastOp>(
-        q_op.getOperand().getDefiningOp());
+    auto dq_op =
+        dyn_cast_or_null<DequantizeCastOp>(q_op.getOperand().getDefiningOp());
     if (!dq_op) {
       return;
     }
@@ -309,7 +309,7 @@ void PrepareQuantizePass::SanityCheckAndAdjustment(func::FuncOp func) {
     //
     // as otherwise qdq pair would have been optimized away.
     auto qd_arg_def_q_op =
-        dyn_cast_or_null<quantfork::QuantizeCastOp>(dq_arg.getDefiningOp());
+        dyn_cast_or_null<QuantizeCastOp>(dq_arg.getDefiningOp());
     if (!qd_arg_def_q_op) {
       return;
     }
@@ -334,19 +334,18 @@ void PrepareQuantizePass::SanityCheckAndAdjustment(func::FuncOp func) {
 // lost for %2. This pattern avoids that by converting from f32 -> qtype2
 // directly.
 class MergeConsecutiveQuantizeCast
-    : public mlir::OpRewritePattern<quantfork::QuantizeCastOp> {
+    : public mlir::OpRewritePattern<QuantizeCastOp> {
  public:
   explicit MergeConsecutiveQuantizeCast(MLIRContext* context)
-      : OpRewritePattern<quantfork::QuantizeCastOp>(context) {}
+      : OpRewritePattern<QuantizeCastOp>(context) {}
 
  private:
-  LogicalResult matchAndRewrite(quantfork::QuantizeCastOp q_op,
+  LogicalResult matchAndRewrite(QuantizeCastOp q_op,
                                 PatternRewriter& rewriter) const override {
-    auto preceding_qcast =
-        q_op.getArg().getDefiningOp<quantfork::QuantizeCastOp>();
+    auto preceding_qcast = q_op.getArg().getDefiningOp<QuantizeCastOp>();
     if (!preceding_qcast) return failure();
 
-    auto new_qcast = rewriter.create<quantfork::QuantizeCastOp>(
+    auto new_qcast = rewriter.create<QuantizeCastOp>(
         q_op.getLoc(), q_op.getType(), preceding_qcast.getArg());
     new_qcast->setAttr(kVolatileOpAttrName, rewriter.getUnitAttr());
     q_op->replaceAllUsesWith(new_qcast);
@@ -356,14 +355,13 @@ class MergeConsecutiveQuantizeCast
 
 bool PrepareQuantizePass::ContainsQuantizeOps(func::FuncOp func) {
   for (const auto& op : func.getOps()) {
-    if (llvm::isa<quantfork::DequantizeCastOp>(op)) return true;
+    if (llvm::isa<DequantizeCastOp>(op)) return true;
   }
   return false;
 }
 
 using PrepareQuantStats =
-    quant::ConvertStatsToQDQs<quantfork::QuantizeCastOp,
-                              quantfork::DequantizeCastOp>;
+    quant::ConvertStatsToQDQs<QuantizeCastOp, DequantizeCastOp>;
 
 #include "tensorflow/compiler/mlir/quantization/tensorflow/passes/prepare_quantize.inc"
 
@@ -400,12 +398,12 @@ void PrepareQuantizePass::runOnOperation() {
   // convert all of them to signed.
   RewritePatternSet patterns(ctx);
   populateWithGenerated(patterns);
-  patterns.add<quant::ConvertUnsignedToSigned<quantfork::QuantizeCastOp>>(ctx);
+  patterns.add<ConvertUnsignedToSigned<QuantizeCastOp>>(ctx);
   // Convert quant stats to int8 quantization parameters.
   // Currently, only activation stats are imported, so narrow_range = false.
   patterns.add<PrepareQuantStats>(bit_width, false, true,
                                   /*legacy_float_scale=*/false, ctx);
-  if (failed(applyPatternsAndFoldGreedily(func, std::move(patterns)))) {
+  if (failed(applyPatternsGreedily(func, std::move(patterns)))) {
     signalPassFailure();
   }
 
@@ -416,11 +414,11 @@ void PrepareQuantizePass::runOnOperation() {
   ApplyQuantizationParamsPropagation(
       func, is_signed, /*bit_width=*/8, !enable_per_channel_quantization_,
       GetTFOpQuantSpec, GetTfQuantScaleSpec, infer_tensor_range,
-      quant_specs_.legacy_float_scale);
+      quant_specs_.legacy_float_scale, /*is_qdq_conversion=*/false);
 
   RewritePatternSet patterns2(ctx);
   patterns2.add<MergeConsecutiveQuantizeCast>(ctx);
-  if (failed(applyPatternsAndFoldGreedily(func, std::move(patterns2)))) {
+  if (failed(applyPatternsGreedily(func, std::move(patterns2)))) {
     signalPassFailure();
   }
 }
