@@ -12,9 +12,6 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
-// Copied and modified from
-// //third_party/tensorflow/compiler/mlir/lite/transforms/quantize.cc
-// This transformation pass applies quantization on TF dialect.
 #include <memory>
 #include <string>
 #include <utility>
@@ -25,7 +22,7 @@ limitations under the License.
 #include "llvm/Support/Casting.h"
 #include "llvm/Support/CommandLine.h"
 #include "mlir/Dialect/Func/IR/FuncOps.h"  // from @llvm-project
-#include "mlir/Dialect/Quant/QuantTypes.h"  // from @llvm-project
+#include "mlir/Dialect/Quant/IR/QuantTypes.h"  // from @llvm-project
 #include "mlir/IR/Attributes.h"  // from @llvm-project
 #include "mlir/IR/Builders.h"  // from @llvm-project
 #include "mlir/IR/BuiltinAttributes.h"  // from @llvm-project
@@ -42,12 +39,10 @@ limitations under the License.
 #include "mlir/Support/LogicalResult.h"  // from @llvm-project
 #include "mlir/Support/TypeID.h"  // from @llvm-project
 #include "mlir/Transforms/GreedyPatternRewriteDriver.h"  // from @llvm-project
-#include "tensorflow/compiler/mlir/lite/quantization/ir/QuantOps.h"
-#include "tensorflow/compiler/mlir/lite/quantization/quantization_config.h"
-#include "tensorflow/compiler/mlir/lite/quantization/quantization_utils.h"
-#include "tensorflow/compiler/mlir/lite/transforms/passes.h"
+#include "tensorflow/compiler/mlir/quantization/common/ir/QuantOps.h"
+#include "tensorflow/compiler/mlir/quantization/common/quantization_lib/quantization_config.h"
+#include "tensorflow/compiler/mlir/quantization/common/quantization_lib/quantization_utils.h"
 #include "tensorflow/compiler/mlir/quantization/tensorflow/ops/tf_op_quant_spec.h"
-#include "tensorflow/compiler/mlir/quantization/tensorflow/passes/utils.h"
 #include "tensorflow/compiler/mlir/quantization/tensorflow/quantization_options.pb.h"
 #include "tensorflow/compiler/mlir/tensorflow/ir/tf_ops.h"
 #include "tensorflow/core/framework/types.pb.h"
@@ -60,19 +55,22 @@ namespace quant {
 //===----------------------------------------------------------------------===//
 namespace {
 
+using ::mlir::quant::ir::DequantizeCastOp;
+using ::mlir::quant::ir::QuantizeCastOp;
+using ::mlir::quant::ir::StorageCastOp;
+using ::tensorflow::quantization::OpSet;
+
 enum QuantizationTrait { kFullQuantization, kDynamicRangeQuantization };
 
 // Base struct for quantization.
 template <QuantizationTrait quantization_trait, typename ConcreteT,
-          typename RootOpT = quantfork::DequantizeCastOp>
+          typename RootOpT = DequantizeCastOp>
 struct TFQuantizationBase
-    : public QuantizationPattern<ConcreteT, quantfork::QuantizeCastOp,
-                                 quantfork::DequantizeCastOp,
+    : public QuantizationPattern<ConcreteT, QuantizeCastOp, DequantizeCastOp,
                                  /*VerifierT=*/void, RootOpT> {
   explicit TFQuantizationBase(MLIRContext* ctx,
                               const QuantPassSpec& quant_params)
-      : QuantizationPattern<ConcreteT, quantfork::QuantizeCastOp,
-                            quantfork::DequantizeCastOp,
+      : QuantizationPattern<ConcreteT, QuantizeCastOp, DequantizeCastOp,
                             /*VerifierT=*/void, RootOpT>(ctx, quant_params) {}
 
   // Custom op quantization is not supported.
@@ -87,7 +85,7 @@ struct TFQuantizationBase
       Operation* quantized_op, const CustomMap& custom_op_map) {
     auto call_op = cast<TF::PartitionedCallOp>(quantized_op);
     StringRef function_name =
-        call_op.getFAttr().cast<FlatSymbolRefAttr>().getValue();
+        llvm::cast<FlatSymbolRefAttr>(call_op.getFAttr()).getValue();
     // The below can be generalized as there are more read-only ops added such
     // as slice.
     const bool is_gather = function_name.contains("gather");
@@ -100,7 +98,7 @@ struct TFQuantizationBase
                                                const CustomMap& custom_op_map) {
     auto call_op = cast<TF::PartitionedCallOp>(quantized_op);
     StringRef function_name =
-        call_op.getFAttr().cast<FlatSymbolRefAttr>().getValue();
+        llvm::cast<FlatSymbolRefAttr>(call_op.getFAttr()).getValue();
     // The below can be generalized as there are more read-only ops added such
     // as slice.
     bool is_gather = false;
@@ -133,11 +131,11 @@ struct TFFullQuantization
 // the quantizable ops without floating-point operands.
 struct TFFullQuantizationReverse
     : public TFQuantizationBase<kFullQuantization, TFFullQuantizationReverse,
-                                quantfork::QuantizeCastOp> {
+                                QuantizeCastOp> {
   explicit TFFullQuantizationReverse(MLIRContext* ctx,
                                      const QuantPassSpec& quant_params)
       : TFQuantizationBase<kFullQuantization, TFFullQuantizationReverse,
-                           quantfork::QuantizeCastOp>(ctx, quant_params) {}
+                           QuantizeCastOp>(ctx, quant_params) {}
 };
 
 // Dynamic range quantization rewrite pattern using DQ as the root op.
@@ -153,14 +151,13 @@ struct TFDynamicRangeQuantization
 // Removes quantize-dequantize pairs that are not used in the quantization.
 // The benefit of this pattern is set to lower value than other patterns, so
 // that the other patterns can work on quantize/dequantize ops first.
-class RemoveUnusedQdqPattern
-    : public OpRewritePattern<quantfork::DequantizeCastOp> {
+class RemoveUnusedQdqPattern : public OpRewritePattern<DequantizeCastOp> {
  public:
   explicit RemoveUnusedQdqPattern(MLIRContext* context)
-      : OpRewritePattern<quantfork::DequantizeCastOp>(context) {}
-  LogicalResult matchAndRewrite(quantfork::DequantizeCastOp dq_op,
+      : OpRewritePattern<DequantizeCastOp>(context) {}
+  LogicalResult matchAndRewrite(DequantizeCastOp dq_op,
                                 PatternRewriter& rewriter) const override {
-    auto q_op = dq_op.getArg().getDefiningOp<quantfork::QuantizeCastOp>();
+    auto q_op = dq_op.getArg().getDefiningOp<QuantizeCastOp>();
     if (!q_op) return failure();
 
     dq_op.replaceAllUsesWith(q_op.getArg());
@@ -168,19 +165,18 @@ class RemoveUnusedQdqPattern
   }
 };
 
-class QuantizeSameScaleOpsPattern
-    : public OpRewritePattern<quantfork::DequantizeCastOp> {
+class QuantizeSameScaleOpsPattern : public OpRewritePattern<DequantizeCastOp> {
  public:
   explicit QuantizeSameScaleOpsPattern(
       MLIRContext* context, OpQuantScaleSpecGetter op_quant_scale_spec_getter,
       OpSet target_opset)
       // Set the score to a large number so it is always preferred, after
       // quantization patterns.
-      : OpRewritePattern<quantfork::DequantizeCastOp>(context, /*benefit=*/200),
+      : OpRewritePattern<DequantizeCastOp>(context, /*benefit=*/200),
         op_quant_scale_spec_getter_(op_quant_scale_spec_getter),
         target_opset_(target_opset) {}
 
-  LogicalResult matchAndRewrite(quantfork::DequantizeCastOp op,
+  LogicalResult matchAndRewrite(DequantizeCastOp op,
                                 PatternRewriter& rewriter) const override {
     SmallVector<Operation*, 4> quantizing_ops;
     auto users = op.getResult().getUsers();
@@ -191,8 +187,7 @@ class QuantizeSameScaleOpsPattern
     // preceding dequantize ops and succeding quantize ops.
     for (Operation* quantizing_op : quantizing_ops) {
       // If it is requantize op, we shouldn't rewrite this op.
-      if (llvm::isa<quantfork::QuantizeCastOp, quantfork::DequantizeCastOp>(
-              quantizing_op)) {
+      if (llvm::isa<QuantizeCastOp, DequantizeCastOp>(quantizing_op)) {
         return failure();
       }
 
@@ -223,17 +218,17 @@ class QuantizeSameScaleOpsPattern
       inputs.reserve(quantizing_op->getNumOperands());
       for (const auto& operand : quantizing_op->getOperands()) {
         Type operand_type = operand.getType();
-        if (operand_type.isa<NoneType>()) {
+        if (isa<NoneType>(operand_type)) {
           inputs.push_back(operand);
           continue;
         }
 
-        Type elem_type = operand_type.cast<TensorType>().getElementType();
-        if (auto dq_op = dyn_cast_or_null<quantfork::DequantizeCastOp>(
-                operand.getDefiningOp())) {
-          auto dq_arg_type = dq_op.getArg().getType().cast<TensorType>();
-          auto qtype = dq_arg_type.getElementType().cast<QuantizedType>();
-          auto scast_op = rewriter.create<quantfork::StorageCastOp>(
+        Type elem_type = llvm::cast<TensorType>(operand_type).getElementType();
+        if (auto dq_op =
+                dyn_cast_or_null<DequantizeCastOp>(operand.getDefiningOp())) {
+          auto dq_arg_type = llvm::cast<TensorType>(dq_op.getArg().getType());
+          auto qtype = llvm::cast<QuantizedType>(dq_arg_type.getElementType());
+          auto scast_op = rewriter.create<StorageCastOp>(
               dq_op->getLoc(), dq_arg_type.clone(qtype.getStorageType()),
               dq_op.getArg());
           inputs.push_back(scast_op.getResult());
@@ -255,23 +250,20 @@ class QuantizeSameScaleOpsPattern
            llvm::enumerate(quantizing_op->getResults())) {
         Value result = enumerated_result.value();
         Type result_type = result.getType();
-        if (result_type.isa<NoneType>()) {
+        if (isa<NoneType>(result_type)) {
           outputs_replaced.insert({result, enumerated_result.index()});
           output_types.push_back(result_type);
           continue;
         }
-        auto result_tensor_type = result_type.cast<TensorType>();
+        auto result_tensor_type = llvm::cast<TensorType>(result_type);
         // If the user is the Quantize op, it must be the only user.
         if (result.hasOneUse() &&
-            llvm::isa<quantfork::QuantizeCastOp>(*result.user_begin())) {
-          auto user =
-              llvm::cast<quantfork::QuantizeCastOp>(*result.user_begin());
+            llvm::isa<QuantizeCastOp>(*result.user_begin())) {
+          auto user = llvm::cast<QuantizeCastOp>(*result.user_begin());
           outputs_replaced.insert(
               {user.getResult(), enumerated_result.index()});
-          auto qtype = user.getType()
-                           .cast<TensorType>()
-                           .getElementType()
-                           .cast<QuantizedType>();
+          auto qtype = llvm::cast<QuantizedType>(
+              llvm::cast<TensorType>(user.getType()).getElementType());
           output_types.push_back(
               result_tensor_type.clone(qtype.getStorageType()));
         } else if (!result_tensor_type.getElementType().isF32()) {
@@ -304,7 +296,7 @@ class QuantizeSameScaleOpsPattern
       for (const auto& output_index_pair : outputs_replaced) {
         Value output = output_index_pair.getFirst();
         int output_index = output_index_pair.getSecond();
-        auto scast_op = rewriter.create<quantfork::StorageCastOp>(
+        auto scast_op = rewriter.create<StorageCastOp>(
             output.getLoc(), output.getType(),
             quantized_op->getResult(output_index));
         output.replaceAllUsesWith(scast_op);
@@ -315,7 +307,7 @@ class QuantizeSameScaleOpsPattern
   }
 
  private:
-  // Checks whether the operation is connnected with a composite function.
+  // Checks whether the operation is connected with a composite function.
   // If not, the same-scale op will not be quantized. This decision is based
   // on the current assumption that the performance gain of the same-scale
   // op itself could not beat the overhead of the quantize and dequantize
@@ -323,8 +315,7 @@ class QuantizeSameScaleOpsPattern
   // this policy might change as well.
   bool IsConnectedWithCompsiteFunction(Operation* same_scale_op) const {
     for (const auto& operand : same_scale_op->getOperands()) {
-      auto dq_op = dyn_cast_or_null<quantfork::DequantizeCastOp>(
-          operand.getDefiningOp());
+      auto dq_op = dyn_cast_or_null<DequantizeCastOp>(operand.getDefiningOp());
       if (!dq_op) continue;
 
       Operation* preceding_op = dq_op.getArg().getDefiningOp();
@@ -338,9 +329,9 @@ class QuantizeSameScaleOpsPattern
       }
 
       // Check if the preceding op is a quantized same-scale op.
-      if (llvm::isa<quantfork::StorageCastOp>(preceding_op)) {
-        auto sc_op = llvm::cast<quantfork::StorageCastOp>(preceding_op);
-        auto sc_arg_type = sc_op.getArg().getType().dyn_cast<TensorType>();
+      if (llvm::isa<StorageCastOp>(preceding_op)) {
+        auto sc_op = llvm::cast<StorageCastOp>(preceding_op);
+        auto sc_arg_type = llvm::dyn_cast<TensorType>(sc_op.getArg().getType());
         if (sc_arg_type.getElementType().isInteger(8)) {
           return true;
         }
@@ -350,11 +341,11 @@ class QuantizeSameScaleOpsPattern
     for (const auto& result : same_scale_op->getResults()) {
       // If the user is the Quantize op, it must be the only user.
       if (!result.hasOneUse() ||
-          !llvm::isa<quantfork::QuantizeCastOp>(*result.user_begin())) {
+          !llvm::isa<QuantizeCastOp>(*result.user_begin())) {
         continue;
       }
 
-      auto q_op = llvm::cast<quantfork::QuantizeCastOp>(*result.user_begin());
+      auto q_op = llvm::cast<QuantizeCastOp>(*result.user_begin());
       for (auto following_op : q_op->getUsers()) {
         // Check whether the preceding op is a quantized composite function.
         if (llvm::isa<TF::PartitionedCallOp>(following_op)) {
@@ -364,9 +355,10 @@ class QuantizeSameScaleOpsPattern
         }
 
         // Check if the preceding op is a quantized same-scale op.
-        if (llvm::isa<quantfork::StorageCastOp>(following_op)) {
-          auto sc_op = llvm::cast<quantfork::StorageCastOp>(following_op);
-          auto sc_arg_type = sc_op.getResult().getType().dyn_cast<TensorType>();
+        if (llvm::isa<StorageCastOp>(following_op)) {
+          auto sc_op = llvm::cast<StorageCastOp>(following_op);
+          auto sc_arg_type =
+              llvm::dyn_cast<TensorType>(sc_op.getResult().getType());
           if (sc_arg_type.getElementType().isInteger(8)) {
             return true;
           }
@@ -383,28 +375,28 @@ class QuantizeSameScaleOpsPattern
       return false;
     }
 
-    const auto f_attr = call_op.getFAttr().dyn_cast<FlatSymbolRefAttr>();
-    if (!f_attr || !f_attr.getValue().startswith("composite_")) {
+    const auto f_attr = llvm::dyn_cast<FlatSymbolRefAttr>(call_op.getFAttr());
+    if (!f_attr || !f_attr.getValue().starts_with("composite_")) {
       return false;
     }
 
     bool has_quantized_types = false;
     for (Value input : call_op.getArgs()) {
-      if (auto type = input.getType().dyn_cast<TensorType>()) {
-        if (type.getElementType().isa<FloatType>()) {
+      if (auto type = llvm::dyn_cast<TensorType>(input.getType())) {
+        if (isa<FloatType>(type.getElementType())) {
           return false;
         }
-        if (type.getElementType().isa<QuantizedType>()) {
+        if (isa<QuantizedType>(type.getElementType())) {
           has_quantized_types = true;
         }
       }
     }
     for (Value output : call_op.getOutput()) {
-      if (auto type = output.getType().dyn_cast<TensorType>()) {
-        if (type.getElementType().isa<FloatType>()) {
+      if (auto type = llvm::dyn_cast<TensorType>(output.getType())) {
+        if (isa<FloatType>(type.getElementType())) {
           return false;
         }
-        if (type.getElementType().isa<QuantizedType>()) {
+        if (isa<QuantizedType>(type.getElementType())) {
           has_quantized_types = true;
         }
       }
@@ -420,24 +412,24 @@ class QuantizeSameScaleOpsPattern
 // we cast its input to float and its output to int8 as a workaround.
 // TODO(b/229183248): Remove this workaround after int8 kernels have been
 // added to TF and XLA.
-struct QuantizeAvgPoolOpPattern
-    : public OpRewritePattern<quantfork::StorageCastOp> {
+struct QuantizeAvgPoolOpPattern : public OpRewritePattern<StorageCastOp> {
   explicit QuantizeAvgPoolOpPattern(MLIRContext* context)
-      : OpRewritePattern<quantfork::StorageCastOp>(context, /*benefit=*/100) {}
+      : OpRewritePattern<StorageCastOp>(context, /*benefit=*/100) {}
 
-  LogicalResult matchAndRewrite(quantfork::StorageCastOp sc_op,
+  LogicalResult matchAndRewrite(StorageCastOp sc_op,
                                 PatternRewriter& rewriter) const override {
     auto avg_pool_op = sc_op.getArg().getDefiningOp<TF::AvgPoolOp>();
     if (!avg_pool_op) return failure();
-    auto preceding_sc_op = dyn_cast_or_null<quantfork::StorageCastOp>(
-        avg_pool_op.getValue().getDefiningOp());
+    auto preceding_sc_op =
+        dyn_cast_or_null<StorageCastOp>(avg_pool_op.getValue().getDefiningOp());
     if (!preceding_sc_op) return failure();
 
     // Check if the same-scale requirement is met.
-    auto dq_arg_type = preceding_sc_op.getArg().getType().cast<TensorType>();
-    auto qtype = dq_arg_type.getElementType().cast<QuantizedType>();
-    auto q_result_type = sc_op.getType().cast<TensorType>();
-    auto out_qtype = q_result_type.getElementType().cast<QuantizedType>();
+    auto dq_arg_type =
+        llvm::cast<TensorType>(preceding_sc_op.getArg().getType());
+    auto qtype = llvm::cast<QuantizedType>(dq_arg_type.getElementType());
+    auto q_result_type = llvm::cast<TensorType>(sc_op.getType());
+    auto out_qtype = llvm::cast<QuantizedType>(q_result_type.getElementType());
     if (qtype != out_qtype) {
       avg_pool_op.emitError(
           "The preceding StorageCastOp and the following "
@@ -552,14 +544,14 @@ void QuantizePass::runOnOperation() {
                                               target_opset_);
     patterns.add<QuantizeAvgPoolOpPattern>(ctx);
   }
-  if (failed(applyPatternsAndFoldGreedily(func, std::move(patterns)))) {
+  if (failed(applyPatternsGreedily(func, std::move(patterns)))) {
     func.emitWarning("Failed to converge pattern at QuantizePass.");
   }
 
   if (!shouldKeepUnusedQdqPattern()) {
     RewritePatternSet patterns_2(&getContext());
     patterns_2.add<RemoveUnusedQdqPattern>(ctx);
-    if (failed(applyPatternsAndFoldGreedily(func, std::move(patterns_2)))) {
+    if (failed(applyPatternsGreedily(func, std::move(patterns_2)))) {
       signalPassFailure();
     }
   }
