@@ -16,31 +16,42 @@ limitations under the License.
 #include <algorithm>
 #include <cstdint>
 #include <memory>
-#include <numeric>
 #include <optional>
 #include <string>
 #include <utility>
 #include <vector>
 
 #include "absl/algorithm/container.h"
+#include "absl/container/flat_hash_set.h"
 #include "absl/strings/match.h"
 #include "absl/strings/str_cat.h"
 #include "absl/strings/str_format.h"
 #include "absl/strings/string_view.h"
+#include "llvm/ADT/Sequence.h"
 #include "llvm/ADT/SmallVector.h"
+#include "mlir/Dialect/Func/IR/FuncOps.h"  // from @llvm-project
+#include "mlir/Dialect/Traits.h"  // from @llvm-project
+#include "mlir/IR/Builders.h"  // from @llvm-project
+#include "mlir/IR/BuiltinAttributes.h"  // from @llvm-project
+#include "mlir/IR/BuiltinTypeInterfaces.h"  // from @llvm-project
 #include "mlir/IR/BuiltinTypes.h"  // from @llvm-project
+#include "mlir/IR/Diagnostics.h"  // from @llvm-project
 #include "mlir/IR/MLIRContext.h"  // from @llvm-project
+#include "mlir/IR/PatternMatch.h"  // from @llvm-project
 #include "mlir/IR/Value.h"  // from @llvm-project
 #include "mlir/IR/ValueRange.h"  // from @llvm-project
 #include "mlir/Pass/Pass.h"  // from @llvm-project
+#include "mlir/Pass/PassRegistry.h"  // from @llvm-project
+#include "mlir/Support/LLVM.h"  // from @llvm-project
 #include "mlir/Support/LogicalResult.h"  // from @llvm-project
+#include "mlir/Support/TypeID.h"  // from @llvm-project
 #include "mlir/Transforms/GreedyPatternRewriteDriver.h"  // from @llvm-project
-#include "tensorflow/compiler/mlir/lite/transforms/passes.h"
+#include "tensorflow/compiler/mlir/quantization/common/attrs_and_constraints.h"
 #include "tensorflow/compiler/mlir/quantization/tensorflow/cc/constant_fold.h"
-#include "tensorflow/compiler/mlir/quantization/tensorflow/passes/utils.h"
+#include "tensorflow/compiler/mlir/quantization/tensorflow/passes/passes.h"
 #include "tensorflow/compiler/mlir/quantization/tensorflow/utils/tf_to_xla_attribute_utils.h"
 #include "tensorflow/compiler/mlir/tensorflow/ir/tf_ops.h"
-#include "tensorflow/compiler/xla/xla_data.pb.h"
+#include "xla/xla_data.pb.h"
 
 namespace mlir::quant {
 namespace {
@@ -77,8 +88,9 @@ void PrepareXlaConvParams(OpBuilder &builder, Location loc, ArrayAttr strides,
   SmallVector<int32_t> lhs_dilation_values(num_dims - 2, 1);
   SmallVector<int32_t> stride_values, rhs_dilation_values;
   for (int64_t i : llvm::seq<int64_t>(1, num_dims - 1)) {
-    stride_values.push_back(strides[i].cast<IntegerAttr>().getInt());
-    rhs_dilation_values.push_back(dilations[i].cast<IntegerAttr>().getInt());
+    stride_values.push_back(mlir::cast<IntegerAttr>(strides[i]).getInt());
+    rhs_dilation_values.push_back(
+        mlir::cast<IntegerAttr>(dilations[i]).getInt());
   }
   window_strides = Create1DConstValue<int32_t>(builder, loc, stride_values);
   lhs_dilation = Create1DConstValue<int32_t>(builder, loc, lhs_dilation_values);
@@ -96,7 +108,7 @@ Value CreateZeroPointPartialOffset(OpBuilder &builder, Location loc,
     return CreateScalarConstValue<int32_t>(builder, loc, 0);
   }
 
-  auto shape = tensor.getType().template cast<ShapedType>();
+  auto shape = mlir::cast<ShapedType>(tensor.getType());
   SmallVector<int64_t> non_output_indices;
   for (int64_t i : llvm::seq<int64_t>(0, shape.getRank())) {
     if (absl::c_count(output_dims, i) == 0) {
@@ -108,7 +120,7 @@ Value CreateZeroPointPartialOffset(OpBuilder &builder, Location loc,
       Create1DConstValue<int64_t>(builder, loc, non_output_indices);
   auto zp = CreateScalarConstValue<int32_t>(builder, loc, other_tensor_zp);
 
-  TensorType tensor_type = tensor.getType().dyn_cast<TensorType>();
+  TensorType tensor_type = mlir::dyn_cast<TensorType>(tensor.getType());
   Value tensor_i32 = builder.create<TF::CastOp>(
       loc, tensor_type.clone(builder.getIntegerType(32)), tensor);
   auto reduced =
@@ -136,7 +148,7 @@ Value MergeZeroPointOffset(OpBuilder &builder, Location loc, Value weight,
                            int8_t input_zp, int8_t weight_zp,
                            Value zp_input_contribution,
                            Value zp_weight_contribution) {
-  auto weight_shape = weight.getType().template cast<ShapedType>();
+  auto weight_shape = mlir::cast<ShapedType>(weight.getType());
   SmallVector<int64_t> weight_non_output_indices;
   for (auto i : llvm::seq<int64_t>(0, weight_shape.getRank())) {
     if (absl::c_count(weight_output_dims, i) == 0) {
@@ -498,7 +510,7 @@ Value CreateZeroPointPartialOffsetXlaDotV2(
     return CreateScalarConstValue<int32_t>(builder, loc, 0);
   }
 
-  auto shape = tensor.getType().template cast<ShapedType>();
+  auto shape = mlir::cast<ShapedType>(tensor.getType());
   SmallVector<int64_t> tensor_shape;
   for (auto v : shape.getShape()) {
     tensor_shape.push_back(v);
@@ -506,7 +518,7 @@ Value CreateZeroPointPartialOffsetXlaDotV2(
 
   auto zp = CreateScalarConstValue<int32_t>(builder, loc, other_tensor_zp);
 
-  TensorType tensor_type = tensor.getType().dyn_cast<TensorType>();
+  TensorType tensor_type = mlir::dyn_cast<TensorType>(tensor.getType());
   Value tensor_i32 = builder.create<TF::CastOp>(
       loc, tensor_type.clone(builder.getIntegerType(32)), tensor);
 
@@ -596,7 +608,7 @@ Value CalculateZeroPointOffsetXLADotV2(OpBuilder &builder, Location loc,
   Value zp_weight_contribution = CreateZeroPointPartialOffsetXlaDotV2(
       builder, loc, weight, input_zp, dnums, /*is_lhs=*/false, output_rank);
 
-  auto weight_shape = weight.getType().template cast<ShapedType>();
+  auto weight_shape = mlir::cast<ShapedType>(weight.getType());
 
   absl::flat_hash_set<int64_t> rhs_contracting_dims;
   for (auto dim : dnums.rhs_contracting_dimensions()) {
@@ -626,8 +638,7 @@ Value CreateXlaConvOp(OpBuilder &builder, Location loc, Value input,
                       Value filter, Value input_zp, Value conv_output,
                       ArrayAttr strides, ArrayAttr dilations,
                       StringAttr conv_padding, ArrayAttr explicit_paddings,
-                      int feature_group_cnt, bool four_bit = false,
-                      int num_dims = 4) {
+                      int feature_group_cnt, int num_dims = 4) {
   int32_t input_zp_value;
   if (!GetSplatValue(input_zp, input_zp_value)) {
     emitError(loc,
@@ -673,14 +684,6 @@ Value CreateXlaConvOp(OpBuilder &builder, Location loc, Value input,
       conv_padding, explicit_paddings, padding, num_dims);
 
   std::string precision_config_str;
-  if (four_bit) {
-    input = PackOperand(builder, loc, input, /*pack_dim=*/num_dims - 1);
-    filter = PackOperand(builder, loc, filter, /*pack_dim=*/num_dims - 2);
-    xla::PrecisionConfig precision_config;
-    precision_config.add_operand_precision(xla::PrecisionConfig::PACKED_NIBBLE);
-    precision_config.add_operand_precision(xla::PrecisionConfig::PACKED_NIBBLE);
-    precision_config_str = precision_config.SerializeAsString();
-  }
   Value xla_conv_output =
       builder
           .create<TF::XlaConvV2Op>(
@@ -711,8 +714,8 @@ Value CreateXlaConvOpFromTfConv2dOp(OpBuilder &builder, Location loc,
                                     ArrayAttr dilations,
                                     StringAttr conv_padding,
                                     ArrayAttr explicit_paddings) {
-  auto input_shape = input.getType().template cast<ShapedType>();
-  auto filter_shape = filter.getType().template cast<ShapedType>();
+  auto input_shape = mlir::cast<ShapedType>(input.getType());
+  auto filter_shape = mlir::cast<ShapedType>(filter.getType());
   if (!input_shape.hasRank() || input_shape.getRank() != 4 ||
       !filter_shape.hasRank() || filter_shape.getRank() != 4) {
     emitError(loc, "input and filter are expected to be 4D tensors");
@@ -731,8 +734,8 @@ Value CreateXlaConvOpFromTfDepthwiseConv2dOp(
     OpBuilder &builder, Location loc, Value input, Value filter, Value input_zp,
     Value conv_output, ArrayAttr strides, ArrayAttr dilations,
     StringAttr conv_padding, ArrayAttr explicit_paddings) {
-  auto input_shape = input.getType().template cast<ShapedType>();
-  auto filter_shape = filter.getType().template cast<ShapedType>();
+  auto input_shape = mlir::cast<ShapedType>(input.getType());
+  auto filter_shape = mlir::cast<ShapedType>(filter.getType());
   if (!input_shape.hasRank() || input_shape.getRank() != 4 ||
       !filter_shape.hasRank() || filter_shape.getRank() != 4) {
     emitError(loc, "input and filter are expected to be 4D tensors");
@@ -759,8 +762,8 @@ Value CreateXlaConvOpFromTfConv3dOp(OpBuilder &builder, Location loc,
                                     Value conv_output, ArrayAttr strides,
                                     ArrayAttr dilations,
                                     StringAttr conv_padding) {
-  auto input_shape = input.getType().template cast<ShapedType>();
-  auto filter_shape = filter.getType().template cast<ShapedType>();
+  auto input_shape = mlir::cast<ShapedType>(input.getType());
+  auto filter_shape = mlir::cast<ShapedType>(filter.getType());
   if (!input_shape.hasRank() || input_shape.getRank() != 5 ||
       !filter_shape.hasRank() || filter_shape.getRank() != 5) {
     emitError(loc, "input and filter are expected to be 5D tensors");
@@ -772,14 +775,13 @@ Value CreateXlaConvOpFromTfConv3dOp(OpBuilder &builder, Location loc,
   return CreateXlaConvOp(builder, loc, input, filter, input_zp, conv_output,
                          strides, dilations, conv_padding,
                          /*explicit_paddings=*/nullptr, feature_group_cnt,
-                         /*four_bit=*/false, /*num_dims=*/5);
+                         /*num_dims=*/5);
 }
 
 // Helper function to create an XlaDotV2Op.
 Value CreateXlaDotV2Op(OpBuilder &builder, Location loc, Value input,
                        Value weight, Value input_zp, Value weight_zp,
-                       Value output, const xla::DotDimensionNumbers &dnums,
-                       bool four_bit = false) {
+                       Value output, const xla::DotDimensionNumbers &dnums) {
   int32_t input_zp_value = 0;
   int32_t weight_zp_value = 0;
   if (input_zp != nullptr && !GetSplatValue(input_zp, input_zp_value)) {
@@ -795,14 +797,6 @@ Value CreateXlaDotV2Op(OpBuilder &builder, Location loc, Value input,
   }
 
   std::string precision_config_str;
-  if (four_bit) {
-    input = PackOperand(builder, loc, input, /*pack_dim=*/1);
-    weight = PackOperand(builder, loc, weight, /*pack_dim=*/0);
-    xla::PrecisionConfig precision_config;
-    precision_config.add_operand_precision(xla::PrecisionConfig::PACKED_NIBBLE);
-    precision_config.add_operand_precision(xla::PrecisionConfig::PACKED_NIBBLE);
-    precision_config_str = precision_config.SerializeAsString();
-  }
 
   Value dot_result =
       builder
@@ -819,7 +813,7 @@ Value CreateXlaDotV2Op(OpBuilder &builder, Location loc, Value input,
 
   Value zp_offset = CalculateZeroPointOffsetXLADotV2(
       builder, loc, input, weight, input_zp_value, weight_zp_value, dnums,
-      output.getType().template cast<ShapedType>().getRank());
+      mlir::cast<ShapedType>(output.getType()).getRank());
 
   return builder.create<TF::SubOp>(loc, dot_result, zp_offset);
 }
@@ -891,8 +885,8 @@ GetBroadcastShapesForBatchMatmul(ShapedType input_type,
 // function, except BroadcastTo, are expected to be folded.
 void BroadcastBatchDimensionsForBatchMatMul(OpBuilder &builder, Location loc,
                                             Value &input, Value &weight) {
-  ShapedType input_type = input.getType().template cast<ShapedType>();
-  ShapedType weight_type = weight.getType().template cast<ShapedType>();
+  ShapedType input_type = mlir::cast<ShapedType>(input.getType());
+  ShapedType weight_type = mlir::cast<ShapedType>(weight.getType());
   const int32_t input_rank = input_type.getRank();
   const int32_t weight_rank = weight_type.getRank();
   const int32_t broadcasted_rank = std::max(input_rank, weight_rank);
@@ -984,7 +978,7 @@ Value CreateXlaDotV2OpFromTfBatchMatMulOp(OpBuilder &builder, Location loc,
   BroadcastBatchDimensionsForBatchMatMul(builder, loc, input, weight);
 
   // Both input and weight have the same rank after broadcasting.
-  ShapedType weight_shape = weight.getType().template cast<ShapedType>();
+  ShapedType weight_shape = mlir::cast<ShapedType>(weight.getType());
   int num_batch_dim = weight_shape.getRank() - 2;
 
   // Transpose and constant-fold the weight if needed.
@@ -1016,7 +1010,7 @@ Value CreateXlaDotV2OpFromTfBatchMatMulOp(OpBuilder &builder, Location loc,
 
 // Check if the given value is a ranked type with specified integer width.
 bool IsRankedInt(Value value, const int integer_width) {
-  ShapedType value_type = value.getType().template cast<ShapedType>();
+  ShapedType value_type = mlir::cast<ShapedType>(value.getType());
   if (!value_type.hasRank()) return false;
   if (!value_type.getElementType().isInteger(integer_width)) return false;
 
@@ -1163,7 +1157,7 @@ void ReplaceCastHacksWithTFXLAOpsPass::runOnOperation() {
   MLIRContext *ctx = &getContext();
   RewritePatternSet patterns(ctx);
   populateWithGenerated(patterns);
-  if (failed(applyPatternsAndFoldGreedily(func, std::move(patterns)))) {
+  if (failed(applyPatternsGreedily(func, std::move(patterns)))) {
     func.emitError() << "quant-replace-cast-hacks-with-tf-xla-ops failed.";
     signalPassFailure();
   }
