@@ -33,7 +33,7 @@ config.name = 'MLIR ' + os.path.basename(config.mlir_test_dir)
 config.test_format = lit.formats.ShTest(not llvm_config.use_lit_shell)
 
 # suffixes: A list of file extensions to treat as test files.
-config.suffixes = ['.cc', '.hlo', '.hlotxt', '.json', '.mlir', '.pbtxt', '.py']
+config.suffixes = ['.cc', '.hlo', '.json', '.mlir', '.pbtxt', '.py']
 
 # test_source_root: The root path where tests are located.
 config.test_source_root = config.mlir_test_dir
@@ -70,19 +70,17 @@ tool_names = [
     'dtensor-opt',
     'flatbuffer_to_string',
     'flatbuffer_translate',
-    'hlo_to_llvm_ir',
-    'ifrt-opt',
+    'hlo_to_kernel',
     'json_to_flatbuffer',
     'kernel-gen-opt',
-    'lhlo-tfrt-opt',
-    'mlir-bisect',
+    'litert-opt',
     'mlir-hlo-opt',
-    'mlir-interpreter-runner',
     'mlir-opt',
     'mlir-tflite-runner',
     'mlir-translate',
     'odml-to-stablehlo-opt',
     'odml_to_stablehlo',
+    'odml-converter',
     'stable-quant-opt',
     'tac-opt-all-backends',
     'tac-translate',
@@ -93,20 +91,11 @@ tool_names = [
     'tf-tfrt-opt',
     'tf_tfjs_translate',
     'tf_tfl_translate',
-    'tf_to_gpu_binary',
-    'tf_to_kernel',
     'tfcompile',
     'tfg-opt-no-passes',
     'tfg-transforms-opt',
     'tfg-translate',
     'tfjs-opt',
-    'xla-cpu-opt',
-    'xla-gpu-opt',
-    'xla-mlir-gpu-opt',
-    'xla-runtime-opt',
-    'xla-translate',
-    'xla-translate-gpu-opt',
-    'xla-translate-opt',
 ]
 tools = [ToolSubst(s, unresolved='ignore') for s in tool_names]
 llvm_config.add_tool_substitutions(tools, tool_dirs)
