@@ -15,34 +15,42 @@ limitations under the License.
 
 #include "tensorflow/compiler/mlir/tensorflow/analysis/resource_alias_analysis.h"
 
+#include <cassert>
 #include <cstdint>
 #include <initializer_list>
 #include <optional>
 #include <utility>
 
+#include "absl/log/log.h"
 #include "llvm/ADT/ArrayRef.h"
 #include "llvm/ADT/DenseMap.h"
 #include "llvm/ADT/SCCIterator.h"
 #include "llvm/ADT/STLExtras.h"
+#include "llvm/ADT/SetVector.h"
+#include "llvm/ADT/SmallSet.h"
 #include "llvm/ADT/SmallVector.h"
 #include "llvm/ADT/iterator_range.h"
 #include "llvm/Support/Casting.h"
 #include "mlir/Analysis/CallGraph.h"  // from @llvm-project
 #include "mlir/Dialect/Func/IR/FuncOps.h"  // from @llvm-project
-#include "mlir/IR/Attributes.h"  // from @llvm-project
 #include "mlir/IR/Block.h"  // from @llvm-project
+#include "mlir/IR/BuiltinAttributes.h"  // from @llvm-project
 #include "mlir/IR/BuiltinOps.h"  // from @llvm-project
-#include "mlir/IR/BuiltinTypes.h"  // from @llvm-project
+#include "mlir/IR/OpDefinition.h"  // from @llvm-project
 #include "mlir/IR/Operation.h"  // from @llvm-project
 #include "mlir/IR/Value.h"  // from @llvm-project
+#include "mlir/IR/ValueRange.h"  // from @llvm-project
 #include "mlir/IR/Visitors.h"  // from @llvm-project
 #include "mlir/Interfaces/CallInterfaces.h"  // from @llvm-project
+#include "mlir/Interfaces/SideEffectInterfaces.h"  // from @llvm-project
 #include "mlir/Support/LLVM.h"  // from @llvm-project
-#include "mlir/Support/LogicalResult.h"  // from @llvm-project
+#include "mlir/Support/TypeID.h"  // from @llvm-project
 #include "tensorflow/compiler/mlir/tensorflow/ir/tf_device.h"
 #include "tensorflow/compiler/mlir/tensorflow/ir/tf_executor.h"
 #include "tensorflow/compiler/mlir/tensorflow/ir/tf_op_interfaces.h"
 #include "tensorflow/compiler/mlir/tensorflow/ir/tf_ops.h"
+#include "tensorflow/compiler/mlir/tensorflow/ir/tf_traits.h"
+#include "tensorflow/compiler/mlir/tensorflow/ir/tf_types.h"
 
 namespace mlir {
 namespace TF {
@@ -74,7 +82,7 @@ class BacktrackAnalysisInfo {
   // the result cannot be backtracked to a region argument, returns
   // std::nullopt.
   std::optional<int> GetArg(int result_index) const {
-    if (auto arg = GetValue(result_index).dyn_cast<BlockArgument>())
+    if (auto arg = mlir::dyn_cast<BlockArgument>(GetValue(result_index)))
       if (arg.getParentBlock() == &region_->front()) return arg.getArgNumber();
     return std::nullopt;
   }
@@ -191,7 +199,7 @@ BacktrackAnalysis::BacktrackAnalysis(
 // possible.
 Value BacktrackAnalysis::BacktrackValue(Value value) {
   while (Operation* op = value.getDefiningOp()) {
-    int res_index = value.cast<OpResult>().getResultNumber();
+    int res_index = mlir::cast<OpResult>(value).getResultNumber();
     if (auto graph = dyn_cast<tf_executor::GraphOp>(op)) {
       value = graph.GetFetch().getOperand(res_index);
     } else if (auto island = dyn_cast<tf_executor::IslandOp>(op)) {
@@ -203,7 +211,7 @@ Value BacktrackAnalysis::BacktrackValue(Value value) {
       value = op->getOperand(res_index);
     } else if (auto call = dyn_cast<CallOpInterface>(op)) {
       func::FuncOp func = dyn_cast<func::FuncOp>(
-          call.resolveCallable(&symbol_table_collection_));
+          call.resolveCallableInTable(&symbol_table_collection_));
       if (!func) break;
       // Check if the function being called has been analyzed. if not,
       // we cannot backtrack the value further.
@@ -399,7 +407,7 @@ ResourceAliasAnalysisInfo::ResourceAliasAnalysisInfo(
       AnalyzeRegionCaseOrIfOp(op, backtrack_analysis);
     } else if (auto call = dyn_cast<CallOpInterface>(op)) {
       func::FuncOp func = dyn_cast_or_null<func::FuncOp>(
-          call.resolveCallable(&symbol_table_collection));
+          call.resolveCallableInTable(&symbol_table_collection));
       if (!func) {
         assign_unknown_id_to_all(op->getResults());
         return WalkResult::advance();
