@@ -13,10 +13,10 @@ See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
 
-#include <algorithm>
+#include <cassert>
+#include <iterator>
 #include <memory>
 #include <queue>
-#include <string>
 #include <utility>
 #include <vector>
 
