@@ -15,8 +15,10 @@ limitations under the License.
 
 // This pass lifts resource variable operations outside of device computation.
 
-#include <cstddef>
+#include <cassert>
 #include <cstdint>
+#include <memory>
+#include <utility>
 
 #include "llvm/ADT/BitVector.h"
 #include "llvm/ADT/DenseMap.h"
@@ -30,11 +32,11 @@ limitations under the License.
 #include "mlir/Dialect/Func/IR/FuncOps.h"  // from @llvm-project
 #include "mlir/IR/Attributes.h"  // from @llvm-project
 #include "mlir/IR/Block.h"  // from @llvm-project
-#include "mlir/IR/IRMapping.h"  // from @llvm-project
 #include "mlir/IR/Builders.h"  // from @llvm-project
 #include "mlir/IR/BuiltinOps.h"  // from @llvm-project
 #include "mlir/IR/BuiltinTypes.h"  // from @llvm-project
 #include "mlir/IR/Diagnostics.h"  // from @llvm-project
+#include "mlir/IR/IRMapping.h"  // from @llvm-project
 #include "mlir/IR/Operation.h"  // from @llvm-project
 #include "mlir/IR/Region.h"  // from @llvm-project
 #include "mlir/IR/SymbolTable.h"  // from @llvm-project
@@ -74,14 +76,14 @@ struct ResourceOpLiftingPass
 };
 
 bool IsResource(Value value) {
-  return getElementTypeOrSelf(value.getType()).isa<TF::ResourceType>();
+  return mlir::isa<TF::ResourceType>(getElementTypeOrSelf(value.getType()));
 }
 
 // Get the type of the data contained in a resource. Returns null if there is
 // no single type in the resource.
 Type GetResourceSubtype(Value value) {
   auto resource_type =
-      getElementTypeOrSelf(value.getType()).dyn_cast<TF::ResourceType>();
+      mlir::dyn_cast<TF::ResourceType>(getElementTypeOrSelf(value.getType()));
   auto subtypes = resource_type.getSubtypes();
   if (subtypes.size() == 1) return subtypes[0];
   return nullptr;
@@ -679,7 +681,7 @@ llvm::SmallDenseMap<int64_t, ResourceArgUseInfo> MergeArgResourceUseInfo(
 // removed). If remaining_resource_data_types is provided, it will store the
 // data types of the remaining resource arguments, where the indices are after
 // removing unused ones.
-void RemoveUnusedResourceArgumentsAndForwardedRetvals(
+LogicalResult RemoveUnusedResourceArgumentsAndForwardedRetvals(
     const llvm::SmallDenseMap<int64_t, ResourceArgUseInfo>& infos,
     func::FuncOp func_op,
     llvm::SmallVector<int64_t, 4>* old_to_new_arg_indices = nullptr,
@@ -691,7 +693,7 @@ void RemoveUnusedResourceArgumentsAndForwardedRetvals(
   int64_t skipped_retvals = 0;
   for (auto entry : llvm::enumerate(old_return_vals)) {
     auto return_val = entry.value();
-    if (auto arg = return_val.dyn_cast<BlockArgument>()) {
+    if (auto arg = mlir::dyn_cast<BlockArgument>(return_val)) {
       auto it = infos.find(arg.getArgNumber());
       if (it != infos.end() && !it->getSecond().used) {
         return_op->eraseOperand(entry.index() - skipped_retvals++);
@@ -720,10 +722,13 @@ void RemoveUnusedResourceArgumentsAndForwardedRetvals(
       }
     }
   }
-  func_op.eraseArguments(indices_to_erase);
+  if (failed(func_op.eraseArguments(indices_to_erase))) {
+    return failure();
+  }
   func_op.setType(
       FunctionType::get(func_op.getContext(), new_types,
                         llvm::to_vector<4>(return_op->getOperandTypes())));
+  return success();
 }
 
 // Lifts reads/writes of resource arguments from func_op and changes its
@@ -747,7 +752,7 @@ LogicalResult LiftArgRetResourcesForFunction(
   // with type replaced.
   llvm::SmallVector<Value, 4> skipped_args;
   for (auto& it : hoister.GetResources()) {
-    BlockArgument arg = it.first.dyn_cast<BlockArgument>();
+    BlockArgument arg = mlir::dyn_cast<BlockArgument>(it.first);
     assert(arg && "Expect resources for FuncOp to be its arguments");
     auto type_iter = resource_data_types.find(arg.getArgNumber());
     if (type_iter == resource_data_types.end()) {
@@ -772,7 +777,7 @@ LogicalResult LiftArgRetResourcesForFunction(
     Value resource = assign_variable_op.getResource();
     if (!hoister.Contains(resource)) continue;
 
-    auto arg = resource.dyn_cast<BlockArgument>();
+    auto arg = mlir::dyn_cast<BlockArgument>(resource);
     handle_updated_arg_value(arg.getArgNumber(), assign_variable_op.getValue());
     assign_variable_op.erase();
   }
@@ -846,10 +851,15 @@ LogicalResult HandleWhileLoop(TF::WhileOp while_op, func::FuncOp body,
   // Remove unused resources in functions.
   llvm::SmallVector<int64_t, 4> old_to_new_indices;
   llvm::SmallDenseMap<int64_t, Type> remaining_resource_data_types;
-  RemoveUnusedResourceArgumentsAndForwardedRetvals(
-      resource_arg_uses, body, &old_to_new_indices,
-      &remaining_resource_data_types);
-  RemoveUnusedResourceArgumentsAndForwardedRetvals(resource_arg_uses, cond);
+  if (failed(RemoveUnusedResourceArgumentsAndForwardedRetvals(
+          resource_arg_uses, body, &old_to_new_indices,
+          &remaining_resource_data_types))) {
+    return failure();
+  }
+  if (failed(RemoveUnusedResourceArgumentsAndForwardedRetvals(resource_arg_uses,
+                                                              cond))) {
+    return failure();
+  }
   (void)LiftArgRetResourcesForFunction(
       body, remaining_resource_data_types,
       [&](int64_t index, Value value) { return_op->setOperand(index, value); });
@@ -914,11 +924,18 @@ LogicalResult HandleCaseOrIfOp(CaseOrIfOp op, ArrayRef<func::FuncOp> branches) {
   if (resource_arg_uses.empty()) return success();
   // Remove unused resources in functions.
   llvm::SmallDenseMap<int64_t, Type> remaining_resource_data_types;
-  RemoveUnusedResourceArgumentsAndForwardedRetvals(
-      resource_arg_uses, branches.front(), /*old_to_new_arg_indices=*/nullptr,
-      &remaining_resource_data_types);
-  for (auto func : branches.drop_front())
-    RemoveUnusedResourceArgumentsAndForwardedRetvals(resource_arg_uses, func);
+  if (failed(RemoveUnusedResourceArgumentsAndForwardedRetvals(
+          resource_arg_uses, branches.front(),
+          /*old_to_new_arg_indices=*/nullptr,
+          &remaining_resource_data_types))) {
+    return failure();
+  }
+  for (auto func : branches.drop_front()) {
+    if (failed(RemoveUnusedResourceArgumentsAndForwardedRetvals(
+            resource_arg_uses, func))) {
+      return failure();
+    }
+  }
 
   // Forward resource inputs updated in any branch to the outputs of both
   // branches. First prepare the mapping from arg to new update output.
@@ -1018,11 +1035,11 @@ LogicalResult HandlePartitionedCallOpCallee(
   for (auto entry :
        llvm::enumerate(callee.front().getTerminator()->getOperands())) {
     auto retval = entry.value();
-    if (!getElementTypeOrSelf(retval.getType()).isa<TF::ResourceType>()) {
+    if (!mlir::isa<TF::ResourceType>(getElementTypeOrSelf(retval.getType()))) {
       result->old_to_new_output_indices.push_back(non_resource_results++);
       continue;
     }
-    auto aliasing_arg = retval.dyn_cast<BlockArgument>();
+    auto aliasing_arg = mlir::dyn_cast<BlockArgument>(retval);
     if (!aliasing_arg) {
       return callee.emitOpError("unsupported function call: ")
              << "resource return value does not alias an input.";
@@ -1053,9 +1070,11 @@ LogicalResult HandlePartitionedCallOpCallee(
 
   // Remove unused resources in functions.
   llvm::SmallDenseMap<int64_t, Type> remaining_resource_data_types;
-  RemoveUnusedResourceArgumentsAndForwardedRetvals(
-      result->use_info, callee, /*old_to_new_arg_indices=*/nullptr,
-      &remaining_resource_data_types);
+  if (failed(RemoveUnusedResourceArgumentsAndForwardedRetvals(
+          result->use_info, callee, /*old_to_new_arg_indices=*/nullptr,
+          &remaining_resource_data_types))) {
+    return failure();
+  }
   for (const auto& entry : remaining_resource_data_types) {
     result->arg_data_type_and_updated_output_index[entry.getFirst()] = {
         entry.getSecond(), -1};
@@ -1063,7 +1082,7 @@ LogicalResult HandlePartitionedCallOpCallee(
   llvm::SmallVector<int64_t, 4> retval_indices_to_preserve;
   for (auto& val : callee.front().getTerminator()->getOpOperands()) {
     // Store indices of results that are not resources.
-    if (!getElementTypeOrSelf(val.get().getType()).isa<TF::ResourceType>())
+    if (!mlir::isa<TF::ResourceType>(getElementTypeOrSelf(val.get().getType())))
       retval_indices_to_preserve.push_back(val.getOperandNumber());
   }
   int64_t num_retvals = retval_indices_to_preserve.size();
