@@ -19,11 +19,13 @@ limitations under the License.
 #include <variant>
 
 #include "llvm/ADT/BitVector.h"
+#include "llvm/Support/LogicalResult.h"
 #include "mlir/Dialect/Func/IR/FuncOps.h"  // from @llvm-project
 #include "mlir/IR/Attributes.h"  // from @llvm-project
 #include "mlir/IR/BuiltinOps.h"  // from @llvm-project
 #include "mlir/IR/Value.h"  // from @llvm-project
 #include "mlir/IR/Visitors.h"  // from @llvm-project
+#include "mlir/Support/LLVM.h"  // from @llvm-project
 #include "tensorflow/compiler/mlir/tensorflow/ir/tf_device.h"
 #include "tensorflow/compiler/mlir/tensorflow/ir/tf_ops.h"
 #include "tensorflow/compiler/mlir/tensorflow/ir/tf_types.h"
@@ -32,7 +34,7 @@ namespace mlir {
 namespace {
 
 bool IsResource(Value value) {
-  return getElementTypeOrSelf(value.getType()).isa<TF::ResourceType>();
+  return mlir::isa<TF::ResourceType>(getElementTypeOrSelf(value.getType()));
 }
 
 // Checks if a cast op is casting a resource -> resource.
@@ -171,8 +173,8 @@ func::FuncOp CloneFunctionIfNeeded(func::FuncOp func) {
 // branch functions to (a) drop the ununsed return values, and (b) as a result
 // if some argument becomes unused in all branches, drop that argument and the
 // corresponding if/case input operand.
-void EliminateUnusedResultsForIfCase(Operation *op,
-                                     ArrayRef<func::FuncOp> branches) {
+LogicalResult EliminateUnusedResultsForIfCase(Operation *op,
+                                              ArrayRef<func::FuncOp> branches) {
   // Clone branch functions if needed since we will be mutating them.
   SmallVector<func::FuncOp, 2> cloned_branches;
   cloned_branches.reserve(branches.size());
@@ -182,7 +184,7 @@ void EliminateUnusedResultsForIfCase(Operation *op,
     if (cloned == func) continue;
     // Patch up the op attribute to point to the new function.
     for (NamedAttribute attr : op->getAttrs()) {
-      auto symref = attr.getValue().dyn_cast<FlatSymbolRefAttr>();
+      auto symref = mlir::dyn_cast<FlatSymbolRefAttr>(attr.getValue());
       if (!symref) continue;
       if (symref.getValue() != func.getName()) continue;
       op->setAttr(attr.getName(),
@@ -215,7 +217,11 @@ void EliminateUnusedResultsForIfCase(Operation *op,
     // Traverse arguments backward so that indices to be deleted stay unchanged.
     for (int idx = num_args - 1; idx >= 0; --idx) {
       if (used_args.test(idx)) continue;
-      for (func::FuncOp func : cloned_branches) func.eraseArgument(idx);
+      for (func::FuncOp func : cloned_branches) {
+        if (failed(func.eraseArgument(idx))) {
+          return failure();
+        }
+      }
       // For if/case, arg #i of attached function corresponds to operand #i+1
       op->eraseOperand(idx + 1);
     }
@@ -230,10 +236,11 @@ void EliminateUnusedResultsForIfCase(Operation *op,
   }
 
   EliminateUnusedResults(op);
+  return success();
 }
 
 // Eliminated unused results from a functional while.
-void EliminateUnusedResultsForWhile(TF::WhileOp op) {
+LogicalResult EliminateUnusedResultsForWhile(TF::WhileOp op) {
   func::FuncOp cond = op.cond_function();
   func::FuncOp body = op.body_function();
 
@@ -253,7 +260,7 @@ void EliminateUnusedResultsForWhile(TF::WhileOp op) {
     }
   }
 
-  if (can_eliminate.empty()) return;
+  if (can_eliminate.empty()) return success();
 
   func::FuncOp cloned_cond = CloneFunctionIfNeeded(cond);
   func::FuncOp cloned_body = CloneFunctionIfNeeded(body);
@@ -267,9 +274,13 @@ void EliminateUnusedResultsForWhile(TF::WhileOp op) {
   // deleted stay unchanged.
   for (int idx = op.getNumResults() - 1; idx >= 0; --idx) {
     if (!can_eliminate.test(idx)) continue;
-    cloned_cond.eraseArgument(idx);
+    if (failed(cloned_cond.eraseArgument(idx))) {
+      return failure();
+    }
     cloned_body.front().getTerminator()->eraseOperand(idx);
-    cloned_body.eraseArgument(idx);
+    if (failed(cloned_body.eraseArgument(idx))) {
+      return failure();
+    }
   }
 
   // Patch up branch function types.
@@ -279,6 +290,7 @@ void EliminateUnusedResultsForWhile(TF::WhileOp op) {
                           func.front().getTerminator()->getOperandTypes()));
   }
   EliminateUnusedResults(op, &can_eliminate);
+  return success();
 }
 
 // For resource results, replace all uses with the resource input to which the
@@ -301,7 +313,8 @@ LogicalResult ForwardCommonArgToOutput(Operation *op,
     std::optional<int> common_arg_index;
     for (func::FuncOp func : branches) {
       auto ret = func.front().getTerminator();
-      auto block_arg = ret->getOperand(result_idx).dyn_cast<BlockArgument>();
+      auto block_arg =
+          mlir::dyn_cast<BlockArgument>(ret->getOperand(result_idx));
       if (!block_arg) {
         return op->emitOpError("result #")
                << result_idx << " not tied to function argument for branch @"
@@ -346,7 +359,9 @@ LogicalResult CanonicalizeFunctionalIfCase(Operation *op,
   if (!has_resource_result) return success();
 
   // Drop unused results.
-  EliminateUnusedResultsForIfCase(op, branches);
+  if (failed(EliminateUnusedResultsForIfCase(op, branches))) {
+    return failure();
+  }
   return success();
 }
 
@@ -366,7 +381,9 @@ LogicalResult CanonicalizeFunctionalWhile(TF::WhileOp op) {
   if (!has_resource_result) return success();
 
   // Drop unused results.
-  EliminateUnusedResultsForWhile(op);
+  if (failed(EliminateUnusedResultsForWhile(op))) {
+    return failure();
+  }
   return success();
 }
 
