@@ -13,7 +13,7 @@ See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
 
-#include <cstddef>
+#include <memory>
 #include <utility>
 
 #include "llvm/ADT/DenseMap.h"
