@@ -13,17 +13,22 @@ See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
 
-#include "absl/strings/match.h"
+#include <cassert>
+#include <memory>
+#include <string>
+#include <utility>
+
 #include "llvm/ADT/DenseSet.h"
 #include "llvm/ADT/STLExtras.h"
 #include "llvm/Support/FormatVariadic.h"
 #include "mlir/IR/Attributes.h"  // from @llvm-project
-#include "mlir/IR/IRMapping.h"  // from @llvm-project
 #include "mlir/IR/Builders.h"  // from @llvm-project
 #include "mlir/IR/BuiltinAttributes.h"  // from @llvm-project
+#include "mlir/IR/IRMapping.h"  // from @llvm-project
 #include "mlir/IR/PatternMatch.h"  // from @llvm-project
 #include "mlir/IR/SymbolTable.h"  // from @llvm-project
 #include "mlir/IR/Visitors.h"  // from @llvm-project
+#include "mlir/Support/LLVM.h"  // from @llvm-project
 #include "mlir/Transforms/DialectConversion.h"  // from @llvm-project
 #include "tensorflow/compiler/mlir/tensorflow/dialect_registration.h"
 #include "tensorflow/compiler/mlir/tensorflow/ir/tf_executor.h"
@@ -58,13 +63,13 @@ static mlir::LogicalResult FilterTfgSpecificArgResultAttributes(
     llvm::SmallVector<mlir::DictionaryAttr> &output_attrs) {
   for (auto it : llvm::zip(
            types, array_attr.template getAsRange<mlir::DictionaryAttr>())) {
-    if (std::get<0>(it).isa<tfg::ControlType>()) continue;
+    if (mlir::isa<tfg::ControlType>(std::get<0>(it))) continue;
     output_types.push_back(std::get<0>(it));
 
     mlir::NamedAttrList list;
     for (mlir::NamedAttribute attr : std::get<1>(it).getValue()) {
       // Skip if the attribute has "tfg" prefix.
-      if (attr.getName().getValue().startswith("tfg")) continue;
+      if (attr.getName().getValue().starts_with("tfg")) continue;
       list.append(attr);
     }
     output_attrs.push_back(list.getDictionary(context));
@@ -80,7 +85,7 @@ static mlir::LogicalResult ReformatOpAttributes(
             mlir::tfg::TFGraphDialect::getDeviceAttrKey())) {
       tensorflow::DeviceNameUtils::ParsedName parsed_name;
       if (!tensorflow::DeviceNameUtils::ParseFullName(
-              attr.getValue().cast<mlir::StringAttr>().getValue().str(),
+              mlir::cast<mlir::StringAttr>(attr.getValue()).getValue().str(),
               &parsed_name))
         return mlir::failure();
       if (!parsed_name.has_type) {
@@ -106,7 +111,7 @@ static mlir::LogicalResult ReformatOpAttributes(
 static void FilterOutBlockArgControlDep(
     ValueRange operands, llvm::SmallVectorImpl<Value> &filtered) {
   for (Value value : operands)
-    if (!value.isa<mlir::BlockArgument>()) filtered.push_back(value);
+    if (!mlir::isa<mlir::BlockArgument>(value)) filtered.push_back(value);
 }
 
 // Split the tfg.NextIteration into tf_executor::NextIterationSourceOp and
@@ -114,7 +119,7 @@ static void FilterOutBlockArgControlDep(
 static void SplitNextIteration(Block &block) {
   // TODO(b/207144333): Supports callback for unregistered ops
   block.walk([&](Operation *op) {
-    if (!op->getName().getStringRef().equals("tfg.NextIteration")) return;
+    if (op->getName().getStringRef() != "tfg.NextIteration") return;
     mlir::OpBuilder builder(op);
 
     llvm::SmallVector<Value, 2> new_operands;
@@ -218,7 +223,7 @@ class ConvertGraphFuncOp : public OpConversionPattern<tfg::GraphFuncOp> {
     Block &block = graph_func.getBody().front();
     for (auto iter = block.args_begin(), end_iter = block.args_end();
          iter != end_iter; ++iter) {
-      if (!iter->getType().isa<tfg::ControlType>())
+      if (!mlir::isa<tfg::ControlType>(iter->getType()))
         iter->replaceAllUsesWith(func.getBody().getArgument(idx++));
     }
 
@@ -412,9 +417,9 @@ class ConvertGeneralOp : public ConversionPattern {
     for (Value value : operands) {
       // Because of the property of graph region, the control operands may
       // not have been converted to tf_executor::ControlType.
-      if (value.getType().isa<tfg::ControlType>() ||
-          value.getType().isa<tf_executor::ControlType>()) {
-        if (!value.isa<BlockArgument>())
+      if (mlir::isa<tfg::ControlType>(value.getType()) ||
+          mlir::isa<tf_executor::ControlType>(value.getType())) {
+        if (!mlir::isa<BlockArgument>(value))
           island_control_operands.push_back(value);
       } else {
         inner_op_operands.push_back(value);
@@ -458,9 +463,10 @@ class ConvertGeneralOp : public ConversionPattern {
             op->getAttrOfType<BoolAttr>("_disable_call_shape_inference")
                 .getValue();
       }
-      inner_op =
-          rewriter.create<LegacyCallOp>(loc, new_types, inner_op_operands,
-                                        op_name, disable_call_shape_inference);
+      inner_op = rewriter.create<LegacyCallOp>(
+          loc, new_types, inner_op_operands,
+          /*args_attrs=*/nullptr,
+          /*res_attrs=*/nullptr, op_name, disable_call_shape_inference);
     }
 
     rewriter.create<tf_executor::YieldOp>(loc, inner_op->getResults());
