@@ -13,6 +13,9 @@ See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
 
+#include <cstdint>
+#include <memory>
+
 #include "llvm/ADT/ArrayRef.h"
 #include "llvm/ADT/DenseMap.h"
 #include "llvm/ADT/STLExtras.h"
@@ -86,7 +89,7 @@ bool IsSupportedInputOp(
       resource_alias_analysis.GetResourceAliases(resource_iterator);
 
   auto is_generator = [](Value val) {
-    if (val.isa<BlockArgument>()) return true;
+    if (mlir::isa<BlockArgument>(val)) return true;
     Operation* definition = val.getDefiningOp();
     return definition->getNumOperands() == 0 &&
            definition->getNumResults() == 1;
@@ -99,7 +102,7 @@ bool IsSupportedInputOp(
     if (!is_generator(alias)) return true;
 
     StringAttr device;
-    if (auto arg = alias.dyn_cast<BlockArgument>()) {
+    if (auto arg = mlir::dyn_cast<BlockArgument>(alias)) {
       device = func.getArgAttrOfType<mlir::StringAttr>(arg.getArgNumber(),
                                                        kFuncDeviceAttr);
     } else {
@@ -186,10 +189,8 @@ bool HandleReplicatedInputs(
         BuildCopyWithLayout(execute_launch, compile_launch, get_layout,
                             entry.value().get(), &builder);
 
-    auto device_list = replicate.getDevices()
-                           .value()
-                           .get(execute_launch.getDevice())
-                           .cast<ArrayAttr>();
+    auto device_list = mlir::cast<ArrayAttr>(
+        replicate.getDevices().value().get(execute_launch.getDevice()));
     copy_with_layout->setAttr(kDeviceAttr,
                               device_list.getValue()[entry.index()]);
 
@@ -225,7 +226,7 @@ void HandleCompileAndExecutes(
     for (const auto& input_and_idx : llvm::enumerate(execute.getArgs())) {
       Value input = input_and_idx.value();
       const int64_t execute_arg_index = input_and_idx.index();
-      if (auto block_arg = input.dyn_cast<BlockArgument>()) {
+      if (auto block_arg = mlir::dyn_cast<BlockArgument>(input)) {
         // For a block argument, consider transforms only when it is a
         // replicated input (defining ops will be outside the replicate node).
         if (maybe_replicate != block_arg.getParentRegion()->getParentOp() ||
