@@ -16,7 +16,9 @@ limitations under the License.
 #include "tensorflow/compiler/mlir/tfr/ir/tfr_ops.h"
 
 #include <algorithm>
-#include <iterator>
+#include <cassert>
+#include <cstddef>
+#include <cstdint>
 #include <string>
 
 #include "llvm/ADT/ArrayRef.h"
@@ -24,33 +26,36 @@ limitations under the License.
 #include "llvm/ADT/STLExtras.h"
 #include "llvm/ADT/SmallVector.h"
 #include "llvm/ADT/StringExtras.h"
-#include "llvm/ADT/StringSet.h"
 #include "llvm/ADT/Twine.h"
 #include "llvm/Support/Casting.h"
-#include "llvm/Support/raw_ostream.h"
+#include "llvm/Support/ErrorHandling.h"
 #include "mlir/Dialect/Arith/IR/Arith.h"  // from @llvm-project
 #include "mlir/Dialect/Func/IR/FuncOps.h"  // from @llvm-project
-#include "mlir/Dialect/Quant/QuantTypes.h"  // from @llvm-project
+#include "mlir/Dialect/Quant/IR/QuantTypes.h"  // from @llvm-project
 #include "mlir/Dialect/Shape/IR/Shape.h"  // from @llvm-project
 #include "mlir/IR/Attributes.h"  // from @llvm-project
 #include "mlir/IR/Builders.h"  // from @llvm-project
 #include "mlir/IR/BuiltinAttributeInterfaces.h"  // from @llvm-project
 #include "mlir/IR/BuiltinAttributes.h"  // from @llvm-project
-#include "mlir/IR/BuiltinOps.h"  // from @llvm-project
+#include "mlir/IR/BuiltinTypeInterfaces.h"  // from @llvm-project
 #include "mlir/IR/BuiltinTypes.h"  // from @llvm-project
 #include "mlir/IR/DialectImplementation.h"  // from @llvm-project
-#include "mlir/IR/FunctionImplementation.h"  // from @llvm-project
 #include "mlir/IR/MLIRContext.h"  // from @llvm-project
 #include "mlir/IR/Matchers.h"  // from @llvm-project
 #include "mlir/IR/OpDefinition.h"  // from @llvm-project
 #include "mlir/IR/OpImplementation.h"  // from @llvm-project
+#include "mlir/IR/OperationSupport.h"  // from @llvm-project
 #include "mlir/IR/PatternMatch.h"  // from @llvm-project
 #include "mlir/IR/Types.h"  // from @llvm-project
+#include "mlir/Interfaces/CallInterfaces.h"  // from @llvm-project
+#include "mlir/Interfaces/FunctionImplementation.h"  // from @llvm-project
+#include "mlir/Support/LLVM.h"  // from @llvm-project
 #include "mlir/Support/LogicalResult.h"  // from @llvm-project
+#include "mlir/Support/TypeID.h"  // from @llvm-project
 #include "mlir/Transforms/InliningUtils.h"  // from @llvm-project
 #include "tensorflow/compiler/mlir/tensorflow/ir/tf_attributes.h"
+#include "tensorflow/compiler/mlir/tensorflow/ir/tf_dialect.h"
 #include "tensorflow/compiler/mlir/tensorflow/ir/tf_ops.h"
-#include "tensorflow/compiler/mlir/tensorflow/ir/tf_types.h"
 #include "tensorflow/compiler/mlir/tfr/ir/tfr_types.h"
 
 namespace mlir {
@@ -89,8 +94,7 @@ class TFRInlinerInterface : public DialectInlinerInterface {
 
   // Handle the given inlined terminator by replacing it with a new operation
   // as necessary. Required when the region has only one block.
-  void handleTerminator(Operation *op,
-                        ArrayRef<Value> valuesToRepl) const final {
+  void handleTerminator(Operation *op, ValueRange valuesToRepl) const final {
     auto retValOp = dyn_cast<TFRReturnOp>(op);
     if (!retValOp) return;
 
@@ -107,12 +111,11 @@ class TFRInlinerInterface : public DialectInlinerInterface {
   Operation *materializeCallConversion(OpBuilder &builder, Value input,
                                        Type result_type,
                                        Location conversion_loc) const final {
-    if (!input.getType().isa<IntegerType>() ||
-        !result_type.isa<IntegerType>()) {
+    if (!isa<IntegerType>(input.getType()) || !isa<IntegerType>(result_type)) {
       return nullptr;
     }
-    auto input_itype = input.getType().cast<IntegerType>();
-    auto result_itype = result_type.cast<IntegerType>();
+    auto input_itype = llvm::cast<IntegerType>(input.getType());
+    auto result_itype = llvm::cast<IntegerType>(result_type);
     if (input_itype.getWidth() == result_itype.getWidth()) return nullptr;
     if (input_itype.getWidth() > result_itype.getWidth()) {
       return builder.create<arith::TruncIOp>(conversion_loc, result_type,
@@ -146,10 +149,10 @@ Operation *TFRDialect::materializeConstant(OpBuilder &builder, Attribute value,
                                            Type type, Location loc) {
   if (arith::ConstantOp::isBuildableWith(value, type))
     return builder.create<arith::ConstantOp>(loc, type,
-                                             value.cast<TypedAttr>());
+                                             llvm::cast<TypedAttr>(value));
   if (func::ConstantOp::isBuildableWith(value, type))
-    return builder.create<func::ConstantOp>(loc, type,
-                                            value.cast<FlatSymbolRefAttr>());
+    return builder.create<func::ConstantOp>(
+        loc, type, llvm::cast<FlatSymbolRefAttr>(value));
   return nullptr;
 }
 
@@ -176,11 +179,11 @@ LogicalResult ConstantTensorOp::verify() {
   auto input_type = op.getArg().getType();
   auto output_type = op.getOut().getType();
 
-  if (auto output_tensor_type = output_type.dyn_cast<TFRTensorType>()) {
+  if (auto output_tensor_type = llvm::dyn_cast<TFRTensorType>(output_type)) {
     return success();
   }
 
-  auto output_tensor_type = output_type.dyn_cast<RankedTensorType>();
+  auto output_tensor_type = llvm::dyn_cast<RankedTensorType>(output_type);
   if (!output_tensor_type || !output_tensor_type.hasStaticShape()) {
     op.emitError("output type should be static and ranked.");
     return failure();
@@ -194,7 +197,7 @@ LogicalResult ConstantTensorOp::verify() {
     return success(same_scalar);
   }
 
-  if (auto input_vector_type = input_type.dyn_cast<VectorType>()) {
+  if (auto input_vector_type = llvm::dyn_cast<VectorType>(input_type)) {
     bool same_element_type = output_tensor_type.getElementType() ==
                              input_vector_type.getElementType();
     bool same_shape =
@@ -226,7 +229,7 @@ LogicalResult TFRFuncOp::verify() {
   for (auto arg : llvm::enumerate(func.getFunctionType().getInputs())) {
     Type arg_type = arg.value();
 
-    if (auto tensor = arg_type.dyn_cast<TFRTensorType>()) {
+    if (auto tensor = llvm::dyn_cast<TFRTensorType>(arg_type)) {
       if (first_tensor == -1) {
         first_tensor = arg.index();
       }
@@ -236,7 +239,7 @@ LogicalResult TFRFuncOp::verify() {
       continue;
     }
 
-    if (auto tensor_list = arg_type.dyn_cast<TFRTensorListType>()) {
+    if (auto tensor_list = llvm::dyn_cast<TFRTensorListType>(arg_type)) {
       if (first_tensor_list == -1) {
         first_tensor_list = arg.index();
       }
@@ -246,7 +249,7 @@ LogicalResult TFRFuncOp::verify() {
       continue;
     }
 
-    if (!arg_type.isa<TensorType>()) {
+    if (!isa<TensorType>(arg_type)) {
       if (first_attr == -1) {
         first_attr = arg.index();
       }
@@ -303,7 +306,7 @@ LogicalResult TFRFuncOp::verify() {
   bool seen_tensor_list = false, has_tensor_list_order_error = false,
        has_multiple_tensor_lists_error = false;
   for (auto result_type : func.getFunctionType().getResults()) {
-    if (auto tensor = result_type.dyn_cast<TFRTensorType>()) {
+    if (auto tensor = llvm::dyn_cast<TFRTensorType>(result_type)) {
       if (seen_tensor_list) {
         has_tensor_list_order_error = true;
       } else {
@@ -313,7 +316,7 @@ LogicalResult TFRFuncOp::verify() {
       continue;
     }
 
-    if (auto tensor_list = result_type.dyn_cast<TFRTensorListType>()) {
+    if (auto tensor_list = llvm::dyn_cast<TFRTensorListType>(result_type)) {
       if (seen_tensor_list) {
         has_multiple_tensor_lists_error = true;
       } else {
@@ -409,7 +412,7 @@ class ConvertConstToTensorConst : public OpRewritePattern<ConstantTensorOp> {
     if (matchPattern(cst_tensor_op.getArg(), m_Constant(&array))) {
       llvm::DenseSet<Type> all_types;
       for (auto it : array) {
-        TypedAttr typed_attr = it.dyn_cast<TypedAttr>();
+        TypedAttr typed_attr = llvm::dyn_cast<TypedAttr>(it);
         if (!typed_attr) return failure();
         all_types.insert(typed_attr.getType());
       }
@@ -419,7 +422,7 @@ class ConvertConstToTensorConst : public OpRewritePattern<ConstantTensorOp> {
       DenseElementsAttr attr =
           DenseElementsAttr::get(new_out_type, array.getValue());
       new_cst = rewriter.create<TF::ConstOp>(loc, new_out_type, attr);
-      if (out_type.isa<TFRTensorType>()) {
+      if (isa<TFRTensorType>(out_type)) {
         new_cst = rewriter.create<CastOp>(loc, out_type, new_cst->getResult(0));
       }
       rewriter.replaceOp(cst_tensor_op, new_cst->getResult(0));
@@ -430,7 +433,7 @@ class ConvertConstToTensorConst : public OpRewritePattern<ConstantTensorOp> {
     if (matchPattern(cst_tensor_op.getArg(), m_Constant(&scalar))) {
       Type new_out_type = RankedTensorType::get({}, scalar.getType());
       new_cst = rewriter.create<TF::ConstOp>(loc, new_out_type, scalar);
-      if (out_type.isa<TFRTensorType>()) {
+      if (isa<TFRTensorType>(out_type)) {
         new_cst = rewriter.create<CastOp>(loc, out_type, new_cst->getResult(0));
       }
       rewriter.replaceOp(cst_tensor_op, new_cst->getResult(0));
@@ -441,9 +444,9 @@ class ConvertConstToTensorConst : public OpRewritePattern<ConstantTensorOp> {
 };
 
 inline bool isQuantizedType(Type type) {
-  auto tensor_type = type.dyn_cast<TensorType>();
+  auto tensor_type = llvm::dyn_cast<TensorType>(type);
   return (tensor_type &&
-          tensor_type.getElementType().isa<quant::QuantizedType>());
+          isa<quant::QuantizedType>(tensor_type.getElementType()));
 }
 
 class RemoveRedundantCast : public OpRewritePattern<CastOp> {
@@ -467,8 +470,8 @@ class RemoveRedundantCast : public OpRewritePattern<CastOp> {
       return failure();
     }
 
-    auto input_tensor_type = input_type.dyn_cast<TensorType>();
-    auto output_tensor_type = output_type.dyn_cast<TensorType>();
+    auto input_tensor_type = llvm::dyn_cast<TensorType>(input_type);
+    auto output_tensor_type = llvm::dyn_cast<TensorType>(output_type);
     if (!input_tensor_type || !output_tensor_type) {
       return failure();
     }
@@ -489,7 +492,7 @@ class RemoveRedundantCast : public OpRewritePattern<CastOp> {
 
     // If the two types are the same, the back-to-back tfr.cast ops can be
     // removed.
-    if (input_type == output_type || output_type.isa<UnrankedTensorType>()) {
+    if (input_type == output_type || isa<UnrankedTensorType>(output_type)) {
       rewriter.replaceOp(cast_op, {input});
       return success();
     }
@@ -497,8 +500,8 @@ class RemoveRedundantCast : public OpRewritePattern<CastOp> {
     // If the rank of the input tensor isn't ranked, we replace the pair
     // with tf.EnsureShape op so it can be removed after shape inference or
     // confirmed at runtime.
-    if (input_type.isa<UnrankedTensorType>()) {
-      auto shape = output_type.cast<ShapedType>().getShape();
+    if (isa<UnrankedTensorType>(input_type)) {
+      auto shape = llvm::cast<ShapedType>(output_type).getShape();
       auto shape_attr = TF::ShapeAttr::get(rewriter.getContext(), shape);
       rewriter.replaceOpWithNewOp<TF::EnsureShapeOp>(cast_op, output_type,
                                                      input, shape_attr);
@@ -544,7 +547,7 @@ class RemoveRedundantGetElement : public OpRewritePattern<GetElementOp> {
     Value input = preceding_build_list.getOperand(index.getInt());
     Type output_type = ge_op.getType();
     if (input.getType() != output_type &&
-        !output_type.isa<UnrankedTensorType>()) {
+        !isa<UnrankedTensorType>(output_type)) {
       return failure();
     }
     rewriter.replaceOp(ge_op, {input});
@@ -595,10 +598,8 @@ quant::QuantizedType getQuantizedElementType(CastOp cast_op) {
   if (!cast_op || !cast_op.getInputElementType()) {
     return {};
   }
-  return cast_op.getInputElementType()
-      .cast<TypeAttr>()
-      .getValue()
-      .dyn_cast<quant::QuantizedType>();
+  return llvm::dyn_cast<quant::QuantizedType>(
+      llvm::cast<TypeAttr>(cast_op.getInputElementType()).getValue());
 }
 
 class RemoveRawDataOp : public OpRewritePattern<TFRQuantRawDataOp> {
@@ -677,15 +678,15 @@ class RemoveQParamsOp : public OpRewritePattern<TFRQuantQParamsOp> {
     // them to constants.
     rewriter.setInsertionPoint(qparams_op);
     Location loc = qparams_op->getLoc();
-    if (auto qtype = cast_qtype.dyn_cast<quant::UniformQuantizedType>()) {
+    if (auto qtype = llvm::dyn_cast<quant::UniformQuantizedType>(cast_qtype)) {
       scale_op = rewriter.create<TF::ConstOp>(
           loc, RankedTensorType::get({}, rewriter.getF32Type()),
           rewriter.getF32FloatAttr(qtype.getScale()));
       zp_op = rewriter.create<TF::ConstOp>(
           loc, RankedTensorType::get({}, rewriter.getI32Type()),
           rewriter.getI32IntegerAttr(qtype.getZeroPoint()));
-    } else if (auto qtype =
-                   cast_qtype.dyn_cast<quant::UniformQuantizedPerAxisType>()) {
+    } else if (auto qtype = llvm::dyn_cast<quant::UniformQuantizedPerAxisType>(
+                   cast_qtype)) {
       SmallVector<float> scales(qtype.getScales().begin(),
                                 qtype.getScales().end());
       SmallVector<int32_t> zps(qtype.getZeroPoints().begin(),
@@ -741,7 +742,7 @@ class RemoveScaleFactorOp : public OpRewritePattern<TFRQuantScaleFactorOp> {
       return failure();
     }
     const double out_scale =
-        out_scale_op.getValue().cast<FloatAttr>().getValueAsDouble();
+        llvm::cast<FloatAttr>(out_scale_op.getValue()).getValueAsDouble();
 
     auto in_scales_op =
         scale_factor_op.getInScales().getDefiningOp<BuildListOp>();
@@ -774,7 +775,8 @@ class RemoveScaleFactorOp : public OpRewritePattern<TFRQuantScaleFactorOp> {
 
     // The shape of scale_type is {} (rank 0) for per-tensor quantized tensor,
     // and {num_channels} (rank 1) for per-channel quantized one.
-    auto scale_type = filter_scale_attr.getType().dyn_cast<RankedTensorType>();
+    auto scale_type =
+        llvm::dyn_cast<RankedTensorType>(filter_scale_attr.getType());
     if (scale_type.getRank() != 0 && scale_type.getRank() != 1) {
       return failure();
     }
@@ -831,26 +833,32 @@ class RemoveRescaleOp : public OpRewritePattern<TFRQuantRescaleOp> {
     auto cast_input_to_float_op = rewriter.create<CallOp>(
         loc, result_types,
         SymbolRefAttr::get(rewriter.getContext(), "tf__cast"),
-        ArrayRef<Value>{input, constant_f32_op, c_false});
+        ArrayRef<Value>{input, constant_f32_op, c_false},
+        /*args_attrs=*/nullptr, /*res_attrs=*/nullptr);
     auto input_x_scale_op = rewriter.create<CallOp>(
         loc, result_types, SymbolRefAttr::get(rewriter.getContext(), "tf__mul"),
-        ArrayRef<Value>{cast_input_to_float_op.getResult(0), scale});
+        ArrayRef<Value>{cast_input_to_float_op.getResult(0), scale},
+        /*args_attrs=*/nullptr, /*res_attrs=*/nullptr);
     auto round_rescaled_op = rewriter.create<CallOp>(
         loc, result_types,
         SymbolRefAttr::get(rewriter.getContext(), "tf__round"),
-        ArrayRef<Value>{input_x_scale_op->getResult(0)});
+        ArrayRef<Value>{input_x_scale_op->getResult(0)},
+        /*args_attrs=*/nullptr, /*res_attrs=*/nullptr);
     auto cast_zp_to_float_op = rewriter.create<CallOp>(
         loc, result_types,
         SymbolRefAttr::get(rewriter.getContext(), "tf__cast"),
-        ArrayRef<Value>{zp_cast, constant_f32_op, c_false});
+        ArrayRef<Value>{zp_cast, constant_f32_op, c_false},
+        /*args_attrs=*/nullptr, /*res_attrs=*/nullptr);
     auto recentered_op = rewriter.create<CallOp>(
         loc, result_types, SymbolRefAttr::get(rewriter.getContext(), "tf__add"),
         ArrayRef<Value>{round_rescaled_op->getResult(0),
-                        cast_zp_to_float_op->getResult(0)});
+                        cast_zp_to_float_op->getResult(0)},
+        /*args_attrs=*/nullptr, /*res_attrs=*/nullptr);
     auto cast_output_to_i32 = rewriter.create<CallOp>(
         loc, result_types,
         SymbolRefAttr::get(rewriter.getContext(), "tf__cast"),
-        ArrayRef<Value>{recentered_op->getResult(0), constant_i32_op, c_false});
+        ArrayRef<Value>{recentered_op->getResult(0), constant_i32_op, c_false},
+        /*args_attrs=*/nullptr, /*res_attrs=*/nullptr);
     rescale_op.getOutput().replaceAllUsesWith(cast_output_to_i32.getResult(0));
     return success();
   }
@@ -930,21 +938,6 @@ Region *TFRFuncOp::getCallableRegion() {
   return isExternal() ? nullptr : &getBody().front();
 }
 
-// CallableOpInterface
-ArrayRef<Type> TFRFuncOp::getCallableResults() {
-  return getFunctionType().getResults();
-}
-
-// CallableOpInterface
-::mlir::ArrayAttr TFRFuncOp::getCallableArgAttrs() {
-  return getArgAttrs().value_or(nullptr);
-}
-
-// CallableOpInterface
-::mlir::ArrayAttr TFRFuncOp::getCallableResAttrs() {
-  return getResAttrs().value_or(nullptr);
-}
-
 //===----------------------------------------------------------------------===//
 // Dialect type definitions
 //===----------------------------------------------------------------------===//
@@ -1000,14 +993,14 @@ Type TFRDialect::parseType(DialectAsmParser &parser) const {
 void TFRDialect::printType(Type type, DialectAsmPrinter &os) const {
   llvm::ArrayRef<StringAttr> attrs;
 
-  if (type.isa<TFRAttrType>()) {
+  if (isa<TFRAttrType>(type)) {
     os << "attr";
     return;
   }
-  if (auto tensor_ty = type.dyn_cast<TFRTensorType>()) {
+  if (auto tensor_ty = llvm::dyn_cast<TFRTensorType>(type)) {
     attrs = tensor_ty.getAttrKeys();
     os << "tensor";
-  } else if (auto tensor_list_ty = type.dyn_cast<TFRTensorListType>()) {
+  } else if (auto tensor_list_ty = llvm::dyn_cast<TFRTensorListType>(type)) {
     attrs = tensor_list_ty.getAttrKeys();
     os << "tensor_list";
   } else {
