@@ -25,6 +25,7 @@ limitations under the License.
 #include "mlir/IR/BuiltinOps.h"  // from @llvm-project
 #include "mlir/IR/BuiltinTypes.h"  // from @llvm-project
 #include "mlir/IR/Operation.h"  // from @llvm-project
+#include "mlir/Support/LLVM.h"  // from @llvm-project
 #include "mlir/Transforms/DialectConversion.h"  // from @llvm-project
 #include "tensorflow/compiler/mlir/tools/kernel_gen/ir/tf_framework_ops.h"
 #include "tensorflow/compiler/mlir/tools/kernel_gen/transforms/rewriters.h"
@@ -71,7 +72,7 @@ class ConvertToLLVMCallOpPattern : public ConvertOpToLLVMPattern<OpTy> {
     if (!attr.has_value() || attr.value().empty()) {
       Value zero = rewriter->create<LLVM::ConstantOp>(
           loc, size_ty, rewriter->getIntegerAttr(size_ty, 0));
-      Value null_ptr = rewriter->create<LLVM::NullOp>(loc, ptr_ty);
+      Value null_ptr = rewriter->create<LLVM::ZeroOp>(loc, ptr_ty);
       return std::make_pair(zero, null_ptr);
     }
 
@@ -96,14 +97,14 @@ class ConvertToLLVMCallOpPattern : public ConvertOpToLLVMPattern<OpTy> {
       Location loc, Type size_ty, Type element_ty,
       std::optional<ArrayAttr> attr,
       ConversionPatternRewriter *rewriter) const {
-    assert(size_ty.isa<IntegerType>() && "expect integer size type");
-    assert(element_ty.isa<IntegerType>() && "expect integer element type");
+    assert(mlir::isa<IntegerType>(size_ty) && "expect integer size type");
+    assert(mlir::isa<IntegerType>(element_ty) && "expect integer element type");
     return ConvertArrayAttrToStackAllocatedArray(
         loc, size_ty, element_ty, attr, rewriter, [&](Attribute attr) {
           return rewriter->create<LLVM::ConstantOp>(
               loc, element_ty,
               rewriter->getIntegerAttr(element_ty,
-                                       attr.cast<IntegerAttr>().getInt()));
+                                       mlir::cast<IntegerAttr>(attr).getInt()));
         });
   }
 };
@@ -190,7 +191,7 @@ class TFAllocOpConverter : public ConvertToLLVMCallOpPattern<TFAllocOp> {
                                           MemRefType memref_type,
                                           Value allocated_byte_ptr,
                                           ArrayRef<Value> sizes) const {
-    auto memref_desc = MemRefDescriptor::undef(
+    auto memref_desc = MemRefDescriptor::poison(
         rewriter, loc, typeConverter->convertType(memref_type));
 
     // TF AllocateRaw returns aligned pointer => AllocatedPtr == AlignedPtr.
@@ -203,7 +204,8 @@ class TFAllocOpConverter : public ConvertToLLVMCallOpPattern<TFAllocOp> {
     }
 
     // Compute strides and populate descriptor `size` and `stride` fields.
-    Value stride_carried = createIndexConstant(rewriter, loc, 1);
+    Value stride_carried =
+        createIndexAttrConstant(rewriter, loc, getIndexType(), 1);
     for (int pos = sizes.size() - 1; pos >= 0; --pos) {
       Value size = sizes[pos];
       memref_desc.setSize(rewriter, loc, pos, size);
@@ -226,7 +228,7 @@ class TFDeallocOpConverter : public ConvertToLLVMCallOpPattern<TFDeallocOp> {
       TFDeallocOp op, OpAdaptor adaptor,
       ConversionPatternRewriter &rewriter) const override {
     // TODO(herhut) Support unranked memrefs.
-    if (!op.getMemref().getType().isa<MemRefType>()) return failure();
+    if (!mlir::isa<MemRefType>(op.getMemref().getType())) return failure();
     MemRefDescriptor memref(adaptor.getMemref());
 
     Value allocated_bytes_ptr = memref.allocatedPtr(rewriter, op.getLoc());
@@ -270,8 +272,6 @@ class JITCompileFromStrOpConverter
         ConvertIntegerArrayAttrToStackAllocatedArray(
             loc, rewriter.getI64Type(), rewriter.getI64Type(),
             op.getUnrollFactors(), &rewriter);
-    Value max_supported_rank = rewriter.create<LLVM::ConstantOp>(
-        loc, rewriter.getI64Type(), op.getMaxSupportedRankAttr());
     Value enable_ftz = rewriter.create<LLVM::ConstantOp>(
         loc, rewriter.getI1Type(), op.getEnableFtzAttr());
     Value index_64bit = rewriter.create<LLVM::ConstantOp>(
@@ -284,8 +284,8 @@ class JITCompileFromStrOpConverter
         op, getVoidPtrType(), tf_func_ref,
         llvm::ArrayRef({adaptor.getCtx(), jit_module_code, tile_sizes.first,
                         tile_sizes.second, unroll_factors.first,
-                        unroll_factors.second, max_supported_rank, enable_ftz,
-                        index_64bit, cpu_codegen}));
+                        unroll_factors.second, enable_ftz, index_64bit,
+                        cpu_codegen}));
     return success();
   }
 
@@ -303,7 +303,6 @@ class JITCompileFromStrOpConverter
                            /*int64_t* tile_sizes_ptr*/ ptr_ty,
                            /*int64_t num_unroll_factors*/ i64_ty,
                            /*int64_t* unroll_factors_ptr*/ ptr_ty,
-                           /*int64_t max_supported_rank*/ i64_ty,
                            /*bool enable_ftz*/ i1_ty,
                            /*bool index_64bit*/ i1_ty,
                            /*bool cpu_codegen*/ i1_ty});
@@ -431,7 +430,7 @@ class ReportErrorOpConverter
     std::string err_str;
     llvm::raw_string_ostream err_stream(err_str);
     err_stream << message;
-    if (!loc.isa<UnknownLoc>()) {
+    if (!mlir::isa<UnknownLoc>(loc)) {
       err_stream << " at ";
       loc.print(err_stream);
     }
@@ -450,7 +449,7 @@ class NullContextOpConverter : public ConvertOpToLLVMPattern<NullContextOp> {
   LogicalResult matchAndRewrite(
       NullContextOp op, OpAdaptor /*adaptor*/,
       ConversionPatternRewriter &rewriter) const override {
-    rewriter.replaceOpWithNewOp<LLVM::NullOp>(op, getVoidPtrType());
+    rewriter.replaceOpWithNewOp<LLVM::ZeroOp>(op, getVoidPtrType());
     return success();
   }
 };
@@ -463,23 +462,26 @@ class NullMemRefOpConverter : public ConvertOpToLLVMPattern<NullMemRefOp> {
       NullMemRefOp null_memref_op, OpAdaptor /*adaptor*/,
       ConversionPatternRewriter &rewriter) const override {
     Location loc = null_memref_op->getLoc();
-    LLVMTypeConverter type_converter = *getTypeConverter();
+    const LLVMTypeConverter &type_converter = *getTypeConverter();
     MLIRContext *ctx = null_memref_op.getContext();
     mlir::Operation *op = null_memref_op.getOperation();
 
-    auto shaped_result_type = null_memref_op.getType().cast<BaseMemRefType>();
-    auto mem_space =
-        shaped_result_type.getMemorySpace().dyn_cast_or_null<IntegerAttr>();
+    auto shaped_result_type =
+        mlir::cast<BaseMemRefType>(null_memref_op.getType());
+    auto mem_space = mlir::dyn_cast_or_null<IntegerAttr>(
+        shaped_result_type.getMemorySpace());
     unsigned address_space =
         static_cast<unsigned>(mem_space ? mem_space.getInt() : 0);
     LLVM::LLVMPointerType llvm_ptr_type =
         LLVM::LLVMPointerType::get(ctx, address_space);
 
-    Value zero = createIndexConstant(rewriter, loc, 0);
-    if (auto result_type = null_memref_op.getType().dyn_cast<MemRefType>()) {
+    Value zero = createIndexAttrConstant(rewriter, loc, getIndexType(), 0);
+    if (auto result_type =
+            mlir::dyn_cast<MemRefType>(null_memref_op.getType())) {
       // Set all dynamic sizes to 1 and compute fake strides.
-      SmallVector<Value, 4> dyn_sizes(result_type.getNumDynamicDims(),
-                                      createIndexConstant(rewriter, loc, 1));
+      SmallVector<Value, 4> dyn_sizes(
+          result_type.getNumDynamicDims(),
+          createIndexAttrConstant(rewriter, loc, getIndexType(), 1));
       SmallVector<Value, 4> sizes, strides;
       Value sizeBytes;
       getMemRefDescriptorSizes(loc, result_type, dyn_sizes, rewriter, sizes,
@@ -487,7 +489,7 @@ class NullMemRefOpConverter : public ConvertOpToLLVMPattern<NullMemRefOp> {
 
       // Prepare packed args [allocatedPtr, alignedPtr, offset, sizes, strides]
       // to create a memref descriptor.
-      Value null = rewriter.create<LLVM::NullOp>(loc, llvm_ptr_type);
+      Value null = rewriter.create<LLVM::ZeroOp>(loc, llvm_ptr_type);
       SmallVector<Value, 12> packed_values{null, null, zero};
       packed_values.append(sizes);
       packed_values.append(strides);
@@ -498,16 +500,16 @@ class NullMemRefOpConverter : public ConvertOpToLLVMPattern<NullMemRefOp> {
       return success();
     }
 
-    auto result_type = null_memref_op.getType().cast<UnrankedMemRefType>();
+    auto result_type = mlir::cast<UnrankedMemRefType>(null_memref_op.getType());
     Type llvm_result_type = type_converter.convertType(result_type);
 
     auto desc =
-        UnrankedMemRefDescriptor::undef(rewriter, loc, llvm_result_type);
+        UnrankedMemRefDescriptor::poison(rewriter, loc, llvm_result_type);
     desc.setRank(rewriter, loc, zero);
 
     // Extract address space and element type.
     auto targetType =
-        null_memref_op.getResult().getType().cast<UnrankedMemRefType>();
+        mlir::cast<UnrankedMemRefType>(null_memref_op.getResult().getType());
     unsigned addressSpace =
         *getTypeConverter()->getMemRefAddressSpace(targetType);
 
@@ -522,7 +524,7 @@ class NullMemRefOpConverter : public ConvertOpToLLVMPattern<NullMemRefOp> {
         sizes.front());
 
     // Populate underlying ranked descriptor.
-    Value null = rewriter.create<LLVM::NullOp>(loc, llvm_ptr_type);
+    Value null = rewriter.create<LLVM::ZeroOp>(loc, llvm_ptr_type);
     UnrankedMemRefDescriptor::setAllocatedPtr(
         rewriter, loc, underlying_desc_ptr, llvm_ptr_type, null);
     UnrankedMemRefDescriptor::setAlignedPtr(rewriter, loc, *getTypeConverter(),
@@ -550,10 +552,10 @@ class IsValidMemRefOpConverter
     MemRefDescriptor desc(adaptor.getArg());
 
     // Compare every size in the descriptor to 0 to check num_elements == 0.
-    int64_t rank = op.getArg().getType().cast<MemRefType>().getRank();
+    int64_t rank = mlir::cast<MemRefType>(op.getArg().getType()).getRank();
     Value is_empty_shape = rewriter.create<LLVM::ConstantOp>(
         loc, rewriter.getI1Type(), rewriter.getBoolAttr(false));
-    Value zero = createIndexConstant(rewriter, loc, 0);
+    Value zero = createIndexAttrConstant(rewriter, loc, getIndexType(), 0);
     for (int i = 0; i < rank; ++i) {
       Value size = desc.size(rewriter, loc, i);
       Value is_zero_size = rewriter.create<LLVM::ICmpOp>(
@@ -563,7 +565,7 @@ class IsValidMemRefOpConverter
     }
 
     Value ptr = desc.allocatedPtr(rewriter, loc);
-    Value null = rewriter.create<LLVM::NullOp>(loc, getVoidPtrType());
+    Value null = rewriter.create<LLVM::ZeroOp>(loc, getVoidPtrType());
     Value is_not_nullptr = rewriter.create<LLVM::ICmpOp>(
         loc, rewriter.getI1Type(), LLVM::ICmpPredicate::ne, ptr, null);
 
