@@ -84,12 +84,12 @@ class UnaryOpsTest(xla_test.XLATestCase):
       self.assertAllClose(result[i], expected[i], rtol, atol)
 
   def AssertCloseAndSorted(self, result, expected, rtol, atol):
-    """Tests that result and expeted are both close and sorted."""
+    """Tests that result and expected are both close and sorted."""
     self.assertAllClose(result, expected, rtol, atol)
     self.assertAllEqual(np.sort(result), result)
 
   def AssertAllEqual(self, result, expected, rtol, atol):
-    """Tests that result and expeted are exactly equal."""
+    """Tests that result and expected are exactly equal."""
     self.assertAllEqual(result, expected)
 
   def testAllTypeOps(self):
@@ -260,7 +260,7 @@ class UnaryOpsTest(xla_test.XLATestCase):
 
       self._assertOpOutputMatchesExpected(
           math_ops.is_finite,
-          np.array([[np.NINF, -2, -1, 0, 0.5, 1, 2, np.inf, np.nan]],
+          np.array([[-np.inf, -2, -1, 0, 0.5, 1, 2, np.inf, np.nan]],
                    dtype=dtype),
           expected=np.array([[0, 1, 1, 1, 1, 1, 1, 0, 0]], dtype=np.bool_))
 
@@ -651,20 +651,32 @@ class UnaryOpsTest(xla_test.XLATestCase):
 
       self._assertOpOutputMatchesExpected(
           math_ops.expm1,
-          np.array([[1e-11 + 1j, -1e-11 - 1j, 1. + 1e-11j,
-                     -1. - 1e-11j, 1e-13j + 1e-13j]], dtype=dtype),
-          # TODO(srvasude): Use numpy as the source of truth after we depend on
-          # latest numpy with this pull request:
-          # https://github.com/numpy/numpy/pull/15110.
-          # The numbers below were generated by scipy.special.expm1.
-          expected=np.array([[
-              -4.59697694e-01+8.41470985e-01j,
-              -4.59697694e-01-8.41470985e-01j,
-              1.71828183e+00+2.71828183e-11j,
-              -6.32120559e-01-3.67879441e-12j,
-              -2.00000000e-26+2.00000000e-13j]], dtype=dtype),
+          np.array(
+              [[
+                  1e-11 + 1j,
+                  -1e-11 - 1j,
+                  1.0 + 1e-11j,
+                  -1.0 - 1e-11j,
+                  1e-13j + 1e-13j,
+              ]],
+              dtype=dtype,
+          ),
+          expected=np.expm1(
+              np.array(
+                  [[
+                      1e-11 + 1j,
+                      -1e-11 - 1j,
+                      1.0 + 1e-11j,
+                      -1.0 - 1e-11j,
+                      1e-13j + 1e-13j,
+                  ]],
+                  dtype=dtype,
+              ),
+              dtype=dtype,
+          ),
           rtol=1e-09,
-          atol=1e-20)
+          atol=1e-20,
+      )
 
       self._assertOpOutputMatchesExpected(
           math_ops.reciprocal,
@@ -761,7 +773,7 @@ class UnaryOpsTest(xla_test.XLATestCase):
           expected=np.array([1, -4, 2.7, 0], dtype=ctypes[dtype]))
 
   def testIntOps(self):
-    for dtype in self.int_types:
+    for dtype in self.int_types - self.unsigned_int_types:
       self._assertOpOutputMatchesExpected(
           bitwise_ops.invert,
           np.array([0, -1, 1, 16, 42], dtype=dtype),
@@ -834,12 +846,12 @@ class UnaryOpsTest(xla_test.XLATestCase):
     for dtype in self.float_types:
       self._assertOpOutputMatchesExpected(
           math_ops.is_inf,
-          np.array([[np.NINF, -2, -1, 0, 0.5, 1, 2, np.inf, np.nan]],
+          np.array([[-np.inf, -2, -1, 0, 0.5, 1, 2, np.inf, np.nan]],
                    dtype=dtype),
           expected=np.array([[1, 0, 0, 0, 0, 0, 0, 1, 0]], dtype=np.bool_))
       self._assertOpOutputMatchesExpected(
           math_ops.is_nan,
-          np.array([[np.NINF, -2, -1, 0, 0.5, 1, 2, np.inf, np.nan]],
+          np.array([[-np.inf, -2, -1, 0, 0.5, 1, 2, np.inf, np.nan]],
                    dtype=dtype),
           expected=np.array([[0, 0, 0, 0, 0, 0, 0, 0, 1]], dtype=np.bool_))
       self._assertOpOutputMatchesExpected(
@@ -879,7 +891,13 @@ class UnaryOpsTest(xla_test.XLATestCase):
       # TODO(b/271327511): Fix issue where casts to FP8 very rarely result in
       # NaN on Mac
       self.skipTest("Casts to FP8 sometimes result in NaN on Mac")
-    fp8_types = {dtypes.float8_e5m2, dtypes.float8_e4m3fn}
+    fp8_types = {
+        dtypes.float8_e5m2,
+        dtypes.float8_e4m3fn,
+        dtypes.float8_e4m3fnuz,
+        dtypes.float8_e4m3b11fnuz,
+        dtypes.float8_e5m2fnuz,
+    }
     other_types = {
         dtypes.bool, dtypes.float32, dtypes.float64, dtypes.complex64,
         dtypes.int32, dtypes.int64, dtypes.uint32, dtypes.uint64
@@ -911,7 +929,10 @@ class UnaryOpsTest(xla_test.XLATestCase):
       if src_type.is_integer:
         imin = np.iinfo(src_np_dtype).min
         imax = np.iinfo(src_np_dtype).max
-        src = np.array([imin, imax, 0, 1, -1], dtype=src_np_dtype)
+        if src_type.is_unsigned:
+          src = np.array([imin, imax, 0, 1], dtype=src_np_dtype)
+        else:
+          src = np.array([imin, imax, 0, 1, -1], dtype=src_np_dtype)
       elif src_type in self.float_tf_types:
         if dst_type.is_integer:
           imin = np.iinfo(dst_np_dtype).min
@@ -924,63 +945,75 @@ class UnaryOpsTest(xla_test.XLATestCase):
           eps = np.finfo(dst_np_dtype).eps
           src = np.array(
               [fmin, fmax, np.nan, eps, -eps, tiny, -tiny, np.inf, -np.inf],
-              dtype=src_np_dtype)
+              dtype=src_np_dtype,
+          )
       dst = src.astype(dst_np_dtype)
       self._assertOpOutputMatchesExpected(
           lambda x, dst_type=dst_type: math_ops.cast(x, dst_type),
           src,
-          expected=dst)
+          expected=dst,
+      )
 
   def testBitcast(self):
     self._assertOpOutputMatchesExpected(
         lambda x: array_ops.bitcast(x, dtypes.int32),
-        np.array([1, 0x3f800000], np.int32),
-        expected=np.array([1, 0x3f800000], np.int32))
+        np.array([1, 0x3F800000], np.int32),
+        expected=np.array([1, 0x3F800000], np.int32),
+    )
     self._assertOpOutputMatchesExpected(
         lambda x: array_ops.bitcast(x, dtypes.float32),
-        np.array([1, 0x3f800000], np.int32),
-        expected=np.array([1e-45, 1.0], np.float32))
+        np.array([1, 0x3F800000], np.int32),
+        expected=np.array([1e-45, 1.0], np.float32),
+    )
     self._assertOpOutputMatchesExpected(
         lambda x: array_ops.bitcast(x, dtypes.int32),
         np.array([1e-45, 1.0], np.float32),
-        expected=np.array([1, 0x3f800000], np.int32))
+        expected=np.array([1, 0x3F800000], np.int32),
+    )
     if np.int64 in self.numeric_types:
       self._assertOpOutputMatchesExpected(
           lambda x: array_ops.bitcast(x, dtypes.int64),
-          np.array([1, 0x100000003f800000], np.uint64),
-          expected=np.array([1, 0x100000003f800000], np.int64))
+          np.array([1, 0x100000003F800000], np.uint64),
+          expected=np.array([1, 0x100000003F800000], np.int64),
+      )
       self._assertOpOutputMatchesExpected(
           lambda x: array_ops.bitcast(x, dtypes.uint64),
-          np.array([1, 0x100000003f800000], np.int64),
-          expected=np.array([1, 0x100000003f800000], np.uint64))
+          np.array([1, 0x100000003F800000], np.int64),
+          expected=np.array([1, 0x100000003F800000], np.uint64),
+      )
       self._assertOpOutputMatchesExpected(
           lambda x: array_ops.bitcast(x, dtypes.float64),
           np.array(
-              [0, 0x3FF0000000000000, 0xc3af161421c8e000, 0x4032000000000007],
+              [0, 0x3FF0000000000000, 0xC3AF161421C8E000, 0x4032000000000007],
               np.uint64,
           ),
           expected=np.array(
               [0, 1.0, -1.12e+18, 18.000000000000024869], np.float64
           ),
-          atol=0
+          atol=0,
       )
 
   def testBitcastInt8ToFloat(self):
     self._assertOpOutputMatchesExpected(
         lambda x: array_ops.bitcast(x, dtypes.float32),
-        np.array([[1, 0, 0, 0], [0xd0, 0x0f, 0x49, 0x40]], np.int8),
-        expected=np.array([1e-45, 3.14159], np.float32))
+        np.array([[1, 0, 0, 0], [0xD0, 0x0F, 0x49, 0x40]]).astype(np.int8),
+        expected=np.array([1e-45, 3.14159], np.float32),
+    )
     self._assertOpOutputMatchesExpected(
         lambda x: array_ops.bitcast(x, dtypes.np.int8),
         np.array([1e-45, 3.14159], np.float32),
-        expected=np.array([[1, 0, 0, 0], [0xd0, 0x0f, 0x49, 0x40]], np.int8))
+        expected=np.array([[1, 0, 0, 0], [0xD0, 0x0F, 0x49, 0x40]]).astype(
+            np.int8
+        ),
+    )
 
   def testInvertPermutation(self):
     for np_dtype in [np.int32, np.int64]:
       self._assertOpOutputMatchesExpected(
           array_ops.invert_permutation,
           np.array([1, 2, 0], np_dtype),
-          expected=np.array([2, 0, 1], dtype=np_dtype))
+          expected=np.array([2, 0, 1], dtype=np_dtype),
+      )
 
   def testInvertPermutationTwiceIsNoop(self):
 
@@ -1001,12 +1034,12 @@ class UnaryOpsTest(xla_test.XLATestCase):
       self._assertOpOutputMatchesExpected(
           rank_op, np.array([[], []], dtype=dtype), expected=np.int32(2))
       self._assertOpOutputMatchesExpected(
-          rank_op, np.array([-1, 1], dtype=dtype), expected=np.int32(1))
+          rank_op, np.array([0, 1], dtype=dtype), expected=np.int32(1))
       self._assertOpOutputMatchesExpected(
-          rank_op, np.array([[-1, 1]], dtype=dtype), expected=np.int32(2))
+          rank_op, np.array([[0, 1]], dtype=dtype), expected=np.int32(2))
       self._assertOpOutputMatchesExpected(
           rank_op,
-          np.array([[-1], [1], [4]], dtype=dtype),
+          np.array([[0], [1], [4]], dtype=dtype),
           expected=np.int32(2))
 
   def testShape(self):
@@ -1020,15 +1053,15 @@ class UnaryOpsTest(xla_test.XLATestCase):
           expected=np.array([2, 0], dtype=np.int32))
       self._assertOpOutputMatchesExpected(
           shape_op,
-          np.array([-1, 1], dtype=dtype),
+          np.array([0, 1], dtype=dtype),
           expected=np.array([2], dtype=np.int32))
       self._assertOpOutputMatchesExpected(
           shape_op,
-          np.array([[-1, 1]], dtype=dtype),
+          np.array([[0, 1]], dtype=dtype),
           expected=np.array([1, 2], dtype=np.int32))
       self._assertOpOutputMatchesExpected(
           shape_op,
-          np.array([[-1], [1], [4]], dtype=dtype),
+          np.array([[0], [1], [4]], dtype=dtype),
           expected=np.array([3, 1], dtype=np.int32))
 
   def testSize(self):
@@ -1039,12 +1072,12 @@ class UnaryOpsTest(xla_test.XLATestCase):
       self._assertOpOutputMatchesExpected(
           size_op, np.array([[], []], dtype=dtype), expected=np.int32(0))
       self._assertOpOutputMatchesExpected(
-          size_op, np.array([-1, 1], dtype=dtype), expected=np.int32(2))
+          size_op, np.array([0, 1], dtype=dtype), expected=np.int32(2))
       self._assertOpOutputMatchesExpected(
-          size_op, np.array([[-1, 1]], dtype=dtype), expected=np.int32(2))
+          size_op, np.array([[0, 1]], dtype=dtype), expected=np.int32(2))
       self._assertOpOutputMatchesExpected(
           size_op,
-          np.array([[-1], [1], [4]], dtype=dtype),
+          np.array([[0], [1], [4]], dtype=dtype),
           expected=np.int32(3))
 
   def testSizeWithInt64OutType(self):
@@ -1055,7 +1088,7 @@ class UnaryOpsTest(xla_test.XLATestCase):
     for dtype in self.numeric_types:
       self._assertOpOutputMatchesExpected(
           size_op,
-          np.array([[-1], [1], [4]], dtype=dtype),
+          np.array([[0], [1], [4]], dtype=dtype),
           expected=np.int64(3))
 
   def testUnpack(self):
