@@ -22,7 +22,7 @@ import numpy as np
 from tensorflow.compiler.tests import xla_test
 from tensorflow.compiler.tf2xla.ops import gen_xla_ops
 from tensorflow.compiler.tf2xla.python import xla
-from tensorflow.compiler.xla import xla_data_pb2
+from local_xla.xla import xla_data_pb2
 from tensorflow.python.eager import def_function
 from tensorflow.python.framework import dtypes
 from tensorflow.python.framework import errors
@@ -1270,13 +1270,13 @@ class XlaOpsShapeInferenceTest(xla_test.XLATestCase, parameterized.TestCase):
     ):
       reduce_with_shapes((None, 4, 5), (3, None, 5), (13, 4, 5))
 
-  @parameterized.parameters(
-      random_ops_util.Algorithm.THREEFRY,
-      random_ops_util.Algorithm.PHILOX,
-      random_ops_util.Algorithm.AUTO_SELECT,
+  @parameterized.product(
+      algorithm=[random_ops_util.Algorithm.THREEFRY,
+                 random_ops_util.Algorithm.PHILOX,
+                 random_ops_util.Algorithm.AUTO_SELECT],
+      dtype=[np.uint8, np.uint64],
   )
-  def testRngBitGenerator(self, algorithm):
-    dtype = np.uint64
+  def testRngBitGenerator(self, algorithm, dtype):
     initial_state = array_ops.placeholder(np.uint64, shape=(2,))
     shape = (2, 3)
     res = xla.rng_bit_generator(algorithm, initial_state, shape, dtype=dtype)
@@ -1306,7 +1306,53 @@ class XlaOpsShapeInferenceTest(xla_test.XLATestCase, parameterized.TestCase):
     with self.assertRaisesRegex(
         TypeError, 'Failed to convert elements .* to Tensor'
     ):
-      res = xla.rng_bit_generator(algorithm, initial_state, shape, dtype=dtype)
+      _ = xla.rng_bit_generator(algorithm, initial_state, shape, dtype=dtype)
+
+  def testGatherShapeInference(self):
+    operand = np.arange(10, dtype=np.int32).reshape([2, 5])
+    start_indices = np.array([2], np.int32)
+    slice_sizes = np.array([1, 3], np.int32)
+    dimension_numbers = xla_data_pb2.GatherDimensionNumbers(
+        offset_dims=[1],
+        collapsed_slice_dims=[0],
+        start_index_map=[0],
+        index_vector_dim=1,
+    )
+
+    res = xla.gather(operand, start_indices, dimension_numbers, slice_sizes)
+    self.assertEqual(res.shape, tensor_shape.TensorShape([1, 3]))
+
+  def testGatherShapeInferenceDynamicSlice(self):
+    operand = np.arange(12, dtype=np.int32).reshape([3, 2, 2])
+    start_indices = array_ops.placeholder(np.int32, shape=(3, None, 2))
+    slice_sizes = np.array([1, 2, 2], np.int32)
+    dimension_numbers = xla_data_pb2.GatherDimensionNumbers(
+        offset_dims=[2, 3],
+        collapsed_slice_dims=[0],
+        start_index_map=[0, 1],
+        index_vector_dim=2,
+    )
+
+    res = xla.gather(operand, start_indices, dimension_numbers, slice_sizes)
+    self.assertEqual(res.shape, tensor_shape.TensorShape([3, None, 2, 2]))
+
+  def testGatherShapeInferenceDynamicInput(self):
+    operand = array_ops.placeholder(np.int32, shape=(None, 5))
+    start_indices = np.array([2], np.int32)
+    slice_sizes = np.array([1, 3], np.int32)
+    dimension_numbers = xla_data_pb2.GatherDimensionNumbers()
+
+    res = xla.gather(operand, start_indices, dimension_numbers, slice_sizes)
+    self.assertEqual(res.shape, tensor_shape.unknown_shape())
+
+  def testGatherShapeInferenceUnknownSliceSizes(self):
+    operand = np.arange(10, dtype=np.int32).reshape([2, 5])
+    start_indices = np.array([2], np.int32)
+    slice_sizes = array_ops.placeholder(np.int32, shape=(2,))
+    dimension_numbers = xla_data_pb2.GatherDimensionNumbers()
+
+    res = xla.gather(operand, start_indices, dimension_numbers, slice_sizes)
+    self.assertEqual(res.shape, tensor_shape.unknown_shape())
 
 
 if __name__ == '__main__':
