@@ -21,7 +21,7 @@ limitations under the License.
 #include "absl/algorithm/container.h"
 #include "tensorflow/compiler/tf2xla/tf2xla_util.h"
 #include "tensorflow/compiler/tf2xla/xla_op_registry.h"
-#include "tensorflow/compiler/xla/status_macros.h"
+#include "xla/status_macros.h"
 #include "tensorflow/core/common_runtime/function.h"
 #include "tensorflow/core/framework/attr_value.pb.h"
 #include "tensorflow/core/framework/function.h"
@@ -33,21 +33,23 @@ namespace tensorflow {
 
 namespace {
 
-Status GetFunctionBody(FunctionLibraryRuntime* flib_runtime,
-                       const NodeDef& node, StringPiece func_attr_name,
-                       const FunctionBody** fbody) {
+absl::Status GetFunctionBody(FunctionLibraryRuntime* flib_runtime,
+                             const NodeDef& node,
+                             absl::string_view func_attr_name,
+                             const FunctionBody** fbody) {
   NameAttrList name_attr_list;
   TF_RETURN_IF_ERROR(GetNodeAttr(node, func_attr_name, &name_attr_list));
   FunctionLibraryRuntime::Handle func_handle;
   TF_RETURN_IF_ERROR(flib_runtime->Instantiate(
       name_attr_list.name(), AttrSlice(&name_attr_list.attr()), &func_handle));
   *fbody = flib_runtime->GetFunctionBody(func_handle);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status GetFunctionBodies(FunctionLibraryRuntime* flib_runtime,
-                         const NodeDef& node, StringPiece func_list_attr_name,
-                         std::vector<const FunctionBody*>* fbodies) {
+absl::Status GetFunctionBodies(FunctionLibraryRuntime* flib_runtime,
+                               const NodeDef& node,
+                               absl::string_view func_list_attr_name,
+                               std::vector<const FunctionBody*>* fbodies) {
   std::vector<NameAttrList> name_attr_lists;
   TF_RETURN_IF_ERROR(GetNodeAttr(node, func_list_attr_name, &name_attr_lists));
   for (const NameAttrList& name_attr_list : name_attr_lists) {
@@ -57,15 +59,16 @@ Status GetFunctionBodies(FunctionLibraryRuntime* flib_runtime,
         &func_handle));
     fbodies->push_back(flib_runtime->GetFunctionBody(func_handle));
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status CondConstInputIndices(
+absl::Status CondConstInputIndices(
     absl::Span<const FunctionBody* const> branch_bodies,
     std::vector<int>* const_input_idxs, FunctionLibraryRuntime* flib_runtime) {
   TF_RET_CHECK(!branch_bodies.empty());
   TF_RET_CHECK(branch_bodies[0] != nullptr);
-  int num_inputs = branch_bodies[0]->fdef.signature().input_arg_size();
+  int num_inputs =
+      branch_bodies[0]->record->fdef().signature().input_arg_size();
   // Stores indices of the "branch function" inputs that are expected to be
   // compile time constants.
   std::vector<bool> compile_time_const_arg_indices(num_inputs);
@@ -83,13 +86,14 @@ Status CondConstInputIndices(
       const_input_idxs->push_back(i + 1);
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status GetCompileTimeConstInputs(const NodeDef& node, const OpKernel* op_kernel,
-                                 const OpDef* op_def,
-                                 std::vector<int>* const_input_idxs,
-                                 FunctionLibraryRuntime* flib_runtime) {
+absl::Status GetCompileTimeConstInputs(const NodeDef& node,
+                                       const OpKernel* op_kernel,
+                                       const OpDef* op_def,
+                                       std::vector<int>* const_input_idxs,
+                                       FunctionLibraryRuntime* flib_runtime) {
   DCHECK(op_def != nullptr || op_kernel != nullptr);
   if (node.op() == "While" || node.op() == "StatelessWhile") {
     // For While nodes, recurse into the body and cond graphs.
@@ -99,7 +103,7 @@ Status GetCompileTimeConstInputs(const NodeDef& node, const OpKernel* op_kernel,
     TF_RETURN_IF_ERROR(GetFunctionBody(flib_runtime, node, "body", &fbody));
     TF_RET_CHECK(fcond);
     TF_RET_CHECK(fbody);
-    int num_inputs = fbody->fdef.signature().input_arg_size();
+    int num_inputs = fbody->record->fdef().signature().input_arg_size();
 
     // Stores which of the loop inputs are expected to be compile time
     // constants.
@@ -132,7 +136,7 @@ Status GetCompileTimeConstInputs(const NodeDef& node, const OpKernel* op_kernel,
         }
       }
     }
-    return OkStatus();
+    return absl::OkStatus();
   } else if (node.op() == "If" || node.op() == "StatelessIf") {
     const FunctionBody* fthen = nullptr;
     const FunctionBody* felse = nullptr;
@@ -151,7 +155,7 @@ Status GetCompileTimeConstInputs(const NodeDef& node, const OpKernel* op_kernel,
              node.op() == "StatefulPartitionedCall") {
     const FunctionBody* fbody;
     TF_RETURN_IF_ERROR(GetFunctionBody(flib_runtime, node, "f", &fbody));
-    int num_inputs = fbody->fdef.signature().input_arg_size();
+    int num_inputs = fbody->record->fdef().signature().input_arg_size();
     std::vector<bool> compile_time_const_arg_indices(num_inputs);
     TF_RETURN_IF_ERROR(BackwardsConstAnalysis(
         *(fbody->graph), &compile_time_const_arg_indices,
@@ -161,7 +165,7 @@ Status GetCompileTimeConstInputs(const NodeDef& node, const OpKernel* op_kernel,
         const_input_idxs->push_back(i);
       }
     }
-    return OkStatus();
+    return absl::OkStatus();
   } else if (op_def != nullptr) {
     return XlaOpRegistry::CompileTimeConstantInputs(node, *op_def,
                                                     const_input_idxs);
@@ -171,9 +175,9 @@ Status GetCompileTimeConstInputs(const NodeDef& node, const OpKernel* op_kernel,
   }
 }
 
-Status GetCompileTimeConstInputs(const Node* node,
-                                 std::vector<int>* const_input_idxs,
-                                 FunctionLibraryRuntime* flib_runtime) {
+absl::Status GetCompileTimeConstInputs(const Node* node,
+                                       std::vector<int>* const_input_idxs,
+                                       FunctionLibraryRuntime* flib_runtime) {
   return GetCompileTimeConstInputs(node->def(), /*op_kernel=*/nullptr,
                                    &node->op_def(), const_input_idxs,
                                    flib_runtime);
@@ -183,7 +187,7 @@ Status GetCompileTimeConstInputs(const Node* node,
 
 // Backwards dataflow analysis that finds arguments to a graph that must be
 // compile-time constants.
-Status BackwardsConstAnalysis(
+absl::Status BackwardsConstAnalysis(
     const Graph& g, std::vector<bool>* compile_time_const_arg_indices,
     std::vector<bool>* compile_time_const_nodes,
     FunctionLibraryRuntime* flib_runtime,
@@ -192,7 +196,7 @@ Status BackwardsConstAnalysis(
       !edge_filter_input) {
     VLOG(5) << "Using cached argument indices on graph " << &g;
     *compile_time_const_arg_indices = g.GetConstArgIndicesCache().value();
-    return OkStatus();
+    return absl::OkStatus();
   }
   auto edge_filter = [&](const Edge& e) {
     return edge_filter_input ? edge_filter_input(e) : true;
@@ -206,7 +210,7 @@ Status BackwardsConstAnalysis(
     compile_time_const_nodes = &compile_time_const_nodes_impl;
   }
 
-  Status status;
+  absl::Status status;
   auto visit = [&](Node* node) {
     if (!status.ok()) return;
 
@@ -293,9 +297,9 @@ Status BackwardsConstAnalysis(
   return status;
 }
 
-Status GetCompileTimeConstInputs(const OpKernel* op_kernel,
-                                 std::vector<int>* const_input_idxs,
-                                 FunctionLibraryRuntime* flib_runtime) {
+absl::Status GetCompileTimeConstInputs(const OpKernel* op_kernel,
+                                       std::vector<int>* const_input_idxs,
+                                       FunctionLibraryRuntime* flib_runtime) {
   return GetCompileTimeConstInputs(op_kernel->def(), op_kernel,
                                    /*op_def=*/nullptr, const_input_idxs,
                                    flib_runtime);
