@@ -14,31 +14,32 @@ limitations under the License.
 ==============================================================================*/
 
 #include <array>
+#include <cstdint>
 #include <numeric>
 #include <string>
 #include <vector>
 
+#include "absl/status/statusor.h"
 #include "absl/types/span.h"
 #include "tensorflow/compiler/tf2xla/kernels/gather_op_helpers.h"
-#include "tensorflow/compiler/tf2xla/lib/util.h"
-#include "tensorflow/compiler/tf2xla/mlir_xla_op_kernel.h"
 #include "tensorflow/compiler/tf2xla/xla_helpers.h"
 #include "tensorflow/compiler/tf2xla/xla_op_kernel.h"
 #include "tensorflow/compiler/tf2xla/xla_op_registry.h"
-#include "tensorflow/compiler/xla/client/lib/arithmetic.h"
-#include "tensorflow/compiler/xla/client/lib/comparators.h"
-#include "tensorflow/compiler/xla/client/lib/constants.h"
-#include "tensorflow/compiler/xla/client/lib/dynamic_shaped_ops.h"
-#include "tensorflow/compiler/xla/client/lib/loops.h"
-#include "tensorflow/compiler/xla/client/lib/sorting.h"
-#include "tensorflow/compiler/xla/client/xla_builder.h"
-#include "tensorflow/compiler/xla/shape_util.h"
-#include "tensorflow/compiler/xla/util.h"
-#include "tensorflow/compiler/xla/xla_data.pb.h"
+#include "xla/hlo/builder/lib/arithmetic.h"
+#include "xla/hlo/builder/lib/comparators.h"
+#include "xla/hlo/builder/lib/constants.h"
+#include "xla/hlo/builder/lib/dynamic_shaped_ops.h"
+#include "xla/hlo/builder/lib/loops.h"
+#include "xla/hlo/builder/lib/sorting.h"
+#include "xla/hlo/builder/xla_builder.h"
+#include "xla/tsl/platform/statusor.h"
+#include "xla/xla_data.pb.h"
+#include "tensorflow/core/framework/op_kernel.h"
+#include "tensorflow/core/framework/op_requires.h"
 #include "tensorflow/core/framework/tensor_shape.h"
 #include "tensorflow/core/framework/types.pb.h"
 #include "tensorflow/core/platform/errors.h"
-#include "tensorflow/core/platform/status.h"
+#include "tensorflow/core/platform/types.h"
 
 namespace tensorflow {
 namespace {
@@ -347,8 +348,8 @@ struct WhileCondFn {
   explicit WhileCondFn(int64_t num_boxes, int64_t output_size)
       : num_boxes(num_boxes), output_size(output_size) {}
 
-  StatusOr<xla::XlaOp> operator()(absl::Span<const xla::XlaOp> values,
-                                  xla::XlaBuilder* cond_builder) const {
+  absl::StatusOr<xla::XlaOp> operator()(absl::Span<const xla::XlaOp> values,
+                                        xla::XlaBuilder* cond_builder) const {
     xla::XlaOp row_idx = values[0];
     xla::XlaOp row_in_bounds =
         xla::Lt(row_idx, xla::ConstantR0<int32>(cond_builder, num_boxes));
@@ -368,7 +369,7 @@ struct SuppressBodyFn {
 
   explicit SuppressBodyFn(int64_t num_boxes) : num_boxes(num_boxes) {}
 
-  StatusOr<std::vector<xla::XlaOp>> operator()(
+  absl::StatusOr<std::vector<xla::XlaOp>> operator()(
       absl::Span<const xla::XlaOp> values, xla::XlaBuilder* builder) const {
     auto row_idx = values[0];
     auto num_outputs_so_far = values[1];
@@ -619,8 +620,9 @@ class NonMaxSuppressionOp : public XlaOpKernel {
                   /*dtype=*/gather_type, DT_INT32, builder, &selected_indices));
 
     if (!pad_to_max_output_size) {
-      StatusOr<xla::XlaOp> rebounded_result = xla::SetDimensionSizeWithRebound(
-          &context->value_inference(), selected_indices, num_valid, 0);
+      absl::StatusOr<xla::XlaOp> rebounded_result =
+          xla::SetDimensionSizeWithRebound(&context->value_inference(),
+                                           selected_indices, num_valid, 0);
       if (rebounded_result.ok()) {
         selected_indices = *rebounded_result;
       } else {
