@@ -15,40 +15,74 @@ limitations under the License.
 
 #include "tensorflow/compiler/tf2xla/kernels/light_outside_compilation.h"
 
-#include <algorithm>
+#include <cstddef>
+#include <cstdint>
 #include <deque>
 #include <memory>
-#include <numeric>
 #include <string>
 #include <utility>
 #include <vector>
 
+#include "absl/algorithm/container.h"
+#include "absl/base/const_init.h"  // NOLINT incorrectly flagged as unused
+#include "absl/base/thread_annotations.h"
+#include "absl/cleanup/cleanup.h"
+#include "absl/container/flat_hash_map.h"
+#include "absl/container/inlined_vector.h"
+#include "absl/log/check.h"
+#include "absl/log/log.h"
+#include "absl/memory/memory.h"
+#include "absl/status/status.h"
+#include "absl/strings/escaping.h"
+#include "absl/strings/match.h"
+#include "absl/strings/str_cat.h"
+#include "absl/strings/str_format.h"
+#include "absl/strings/str_join.h"
+#include "absl/strings/string_view.h"
+#include "absl/synchronization/mutex.h"
 #include "absl/types/span.h"
 #include "tensorflow/compiler/tf2xla/kernels/callback.pb.h"
-#include "tensorflow/compiler/tf2xla/lib/util.h"
 #include "tensorflow/compiler/tf2xla/shape_util.h"
-#include "tensorflow/compiler/tf2xla/type_util.h"
-#include "tensorflow/compiler/tf2xla/xla_helpers.h"
 #include "tensorflow/compiler/tf2xla/xla_op_kernel.h"
 #include "tensorflow/compiler/tf2xla/xla_op_registry.h"
-#include "tensorflow/compiler/xla/service/custom_call_status.h"
-#include "tensorflow/compiler/xla/service/custom_call_target_registry.h"
-#include "tensorflow/compiler/xla/shape_util.h"
-#include "tensorflow/compiler/xla/xla_data.pb.h"
-#include "tensorflow/core/common_runtime/gpu/gpu_cudamalloc_allocator.h"
-
-#if GOOGLE_CUDA || TENSORFLOW_USE_ROCM
-#include "tensorflow/compiler/xla/stream_executor/gpu/gpu_executor.h"
-#include "tensorflow/compiler/xla/stream_executor/gpu/gpu_stream.h"
-#include "tensorflow/compiler/xla/stream_executor/gpu/gpu_types.h"
-#include "tensorflow/core/common_runtime/gpu/gpu_device.h"
-#endif
-
+#include "xla/hlo/builder/xla_builder.h"
+#include "xla/layout_util.h"
+#include "xla/service/custom_call_status.h"
+#include "xla/service/custom_call_target_registry.h"
+#include "xla/service/hlo.pb.h"
+#include "xla/shape.h"
+#include "xla/shape_util.h"
+#include "xla/status_macros.h"
+#include "xla/stream_executor/device_memory.h"
+#include "xla/stream_executor/platform.h"
+#include "xla/stream_executor/platform_manager.h"
+#include "xla/stream_executor/stream.h"
+#include "xla/stream_executor/stream_finder.h"
+#include "xla/tsl/lib/strings/proto_serialization.h"
+#include "xla/tsl/platform/errors.h"
+#include "xla/tsl/platform/statusor.h"
+#include "xla/util.h"
 #include "tensorflow/core/common_runtime/gpu/gpu_process_state.h"
+#include "tensorflow/core/common_runtime/process_state.h"
+#include "tensorflow/core/framework/allocator.h"
+#include "tensorflow/core/framework/device_base.h"
+#include "tensorflow/core/framework/op.h"
+#include "tensorflow/core/framework/op_def_builder.h"
+#include "tensorflow/core/framework/op_kernel.h"
+#include "tensorflow/core/framework/op_requires.h"
 #include "tensorflow/core/framework/shape_inference.h"
 #include "tensorflow/core/framework/tensor_shape.h"
+#include "tensorflow/core/framework/types.h"
 #include "tensorflow/core/framework/types.pb.h"
-#include "tensorflow/tsl/framework/device_id.h"
+#include "tensorflow/core/platform/env.h"
+#include "tensorflow/core/platform/refcount.h"
+#include "tensorflow/core/platform/status.h"
+#include "tensorflow/core/platform/types.h"
+#include "tsl/platform/fingerprint.h"
+
+#if GOOGLE_CUDA || TENSORFLOW_USE_ROCM
+#include "tensorflow/core/common_runtime/gpu/gpu_device.h"
+#endif
 
 namespace tensorflow {
 
@@ -56,9 +90,136 @@ namespace {
 
 const char* const kTfCallbackCustomCall = "GenericTfCallbackGPU";
 
+// Each unique TfCallbackData in a call to Compile() generates a new
+// KernelInstantiation.  We cache these in a global hashtable, because creating
+// OpKernels is expensive.
+//
+// The KernelInstantiation contains a list of OpKernel+DeviceBase objects.
+// These are all equivalent to each other.  When we want to run the kernel, we
+// "check out" an element from the list, removing it.  When we're done, we add
+// it back.  If there are no available elements in the list, we create one.
+struct KernelInstantiation {
+  static absl::StatusOr<std::unique_ptr<KernelInstantiation>> Create(
+      TfCallbackData callback_data) {
+    auto instantiation = std::make_unique<KernelInstantiation>();
+    instantiation->input_shapes.reserve(callback_data.inputs_size());
+    for (const auto& input : callback_data.inputs()) {
+      TensorShape shape;
+      TF_RETURN_IF_ERROR(TensorShape::BuildTensorShape(
+          input.buffer_description().shape(), &shape));
+      instantiation->input_shapes.push_back(std::move(shape));
+    }
+    instantiation->callback_data = std::move(callback_data);
+    return instantiation;
+  }
+
+  // Cache of
+  //   TensorShape::BuildTensorShape(
+  //     callback_data.inputs(i).buffer_description().shape()),
+  // because calling this for every invocation of the op is expensive.
+  std::vector<TensorShape> input_shapes;
+
+  TfCallbackData callback_data;
+
+  // In order to run an Instantiation, we need a TF OpKernel object.  Creating
+  // one is expensive, so we cache them.  When we run an op, we "check out" an
+  // element from this vector, and then we return it when we're done.
+  absl::Mutex mu;
+  std::vector<std::pair<std::unique_ptr<DeviceBase>, std::unique_ptr<OpKernel>>>
+      devices_and_kernels ABSL_GUARDED_BY(mu);
+};
+
+absl::StatusOr<std::string> MakeOpaque(TfCallbackData callback_data) {
+  // Clear the `name` field in the callback_data, because this contains the full
+  // TF op name scope.  We want ops with different names to map to the same
+  // Instantiation (so that if the same op with the same shape appears 10 times
+  // in a model, they all have the same logical TfCallbackData.)
+  callback_data.mutable_op()->clear_name();
+
+  std::string serialized_data;
+  if (!tsl::SerializeToStringDeterministic(callback_data, &serialized_data)) {
+    return absl::InternalError(
+        "Failed in serializing TfCallbackData to string");
+  }
+  auto fingerprint = tsl::Fingerprint128(serialized_data);
+  return absl::StrFormat(
+      "fingerprint128=%s serialized=%s (%s)",
+      absl::Base64Escape(absl::string_view(
+          reinterpret_cast<char*>(&fingerprint), sizeof(fingerprint))),
+      absl::Base64Escape(serialized_data), callback_data.op().op());
 }
 
-static StatusOr<Tensor> TensorFromProto(const TensorProto& proto) {
+absl::StatusOr<KernelInstantiation*> GetInstantiation(
+    absl::string_view opaque) {
+  constexpr absl::string_view kFingerprintPrefix = "fingerprint128=";
+  if (!absl::StartsWith(opaque, kFingerprintPrefix)) {
+    return xla::Internal("Invalid opaque; must start with '%s', but was '%s'",
+                         kFingerprintPrefix, opaque);
+  }
+  opaque.remove_prefix(kFingerprintPrefix.length());
+
+  // We use a 128-bit fingerprint, encoded as base64.  It's 24 chars long
+  // including padding.
+  constexpr int kFingerprintLen = 24;
+  absl::string_view fingerprint_str = opaque.substr(0, kFingerprintLen);
+  opaque.remove_prefix(kFingerprintLen);
+  if (fingerprint_str.length() != kFingerprintLen) {
+    return xla::Internal("Invalid opaque; fingerprint is wrong length: '%s'",
+                         fingerprint_str);
+  }
+
+  static absl::Mutex mu{absl::kConstInit};
+  static auto& instantiations ABSL_GUARDED_BY(mu) =
+      *new absl::flat_hash_map<std::string /*base64-encoded fingerprint*/,
+                               std::unique_ptr<KernelInstantiation>>();
+
+  absl::MutexLock lock(&mu);
+  std::unique_ptr<KernelInstantiation>& instantiation =
+      instantiations[fingerprint_str];
+  if (instantiation == nullptr) {
+    // Cache miss; create the instantiation.  To do this, we need to parse out
+    // the serialized TfCallbackData from the opaque.
+    constexpr absl::string_view kSerializedPrefix = " serialized=";
+    if (!absl::StartsWith(opaque, kSerializedPrefix)) {
+      return xla::Internal("Invalid opaque; must start with '%s', but was '%s'",
+                           kSerializedPrefix, opaque);
+    }
+    opaque.remove_prefix(kSerializedPrefix.length());
+
+    // Find the end of the base64-encoded serialized proto.
+    opaque = opaque.substr(0, opaque.find_first_of(' '));
+
+    // Unescape the base64 string, then parse the proto.
+    std::string unescaped_opaque;
+    if (!absl::Base64Unescape(opaque, &unescaped_opaque)) {
+      return xla::Internal("Failed to base64 decode opaque %s", opaque);
+    }
+    TfCallbackData callback_data;
+    if (!callback_data.ParseFromString(unescaped_opaque)) {
+      return xla::Internal("Failed to parse TfCallbackData from opaque %s",
+                           opaque);
+    }
+    TF_ASSIGN_OR_RETURN(instantiation,
+                        KernelInstantiation::Create(std::move(callback_data)));
+
+    // Log a warning every power of 2 if `instantiations` gets large.
+    if (size_t n = instantiations.size();
+        n >= (1 << 12) && (n & (n - 1)) == 0) {
+      LOG(WARNING)  //
+          << "Light outside compilation has compiled " << n
+          << " unique op+shape combinations.  Each of these permanently "
+             "leaks CPU memory.  Exactly how much depends on the op, but "
+             "expect ~1kb per op.  This is probably happening because you're "
+             "recompiling an XLA model many times with different shapes.  "
+             "This message will be logged again at the next power of 2.";
+    }
+  }
+  return instantiation.get();
+}
+
+}  // namespace
+
+static absl::StatusOr<Tensor> TensorFromProto(const TensorProto& proto) {
   Tensor out;
   if (!out.FromProto(proto)) {
     return tsl::errors::Internal("Failed deserializing a TensorProto");
@@ -66,21 +227,8 @@ static StatusOr<Tensor> TensorFromProto(const TensorProto& proto) {
   return out;
 }
 
-static StatusOr<TfCallbackData> CallbackDataFromProto(const char* opaque,
-                                                      size_t opaque_len) {
-  TfCallbackData callback_data;
-  absl::string_view data{opaque, opaque_len};
-  callback_data.ParseFromString(std::string{data});  // NOLINT: OSS req
-  return callback_data;
-}
-
-static StatusOr<std::string> SerializeCallbackData(const TfCallbackData& data) {
-  return data.SerializeAsString();
-}
-
-Status LightOutsideCompilationOp::CompileToCustomCallCallingTfKernel(
-    int graph_def_version, const NodeDef& node_def,
-    XlaOpKernelContext* ctx) const {
+absl::Status LightOutsideCompilationOp::CompileToCustomCallCallingTfKernel(
+    int graph_def_version, const NodeDef& node_def, XlaOpKernelContext* ctx) {
   const OpRegistrationData* data = OpRegistry::Global()->LookUp(node_def.op());
   int num_inputs = ctx->num_inputs();
   int num_outputs = ctx->num_outputs();
@@ -207,21 +355,18 @@ Status LightOutsideCompilationOp::CompileToCustomCallCallingTfKernel(
     output_xla_shapes.push_back(output_shape);
   }
 
-  xla::Shape output_shape =
-      xla::ShapeUtil::MakeMaybeTupleShape(output_xla_shapes);
-
+  TF_ASSIGN_OR_RETURN(
+      auto output_shape,
+      xla::ShapeUtil::MakeValidatedMaybeTupleShape(output_xla_shapes));
   VLOG(1) << "Created output shape: " << output_shape.ToString();
 
-  TF_ASSIGN_OR_RETURN(std::string callback_data_serialized,
-                      SerializeCallbackData(callback_data));
-
+  TF_ASSIGN_OR_RETURN(std::string opaque, MakeOpaque(std::move(callback_data)));
   xla::XlaOp out = xla::CustomCallWithLayout(
       ctx->builder(), kTfCallbackCustomCall, operands, output_shape,
-      /*operand_shapes_with_layout=*/operand_shapes_with_layout,
-      /*opaque=*/callback_data_serialized,
+      operand_shapes_with_layout, opaque,
       /*has_side_effect=*/false,
-      /*output_operand_aliasing=*/{},
-      /*literal=*/nullptr, xla::CustomCallSchedule::SCHEDULE_NONE,
+      /*output_operand_aliasing=*/{}, /*literal=*/nullptr,
+      xla::CustomCallSchedule::SCHEDULE_NONE,
       xla::CustomCallApiVersion::API_VERSION_STATUS_RETURNING);
 
   for (int i = 0; i < num_outputs; ++i) {
@@ -229,7 +374,7 @@ Status LightOutsideCompilationOp::CompileToCustomCallCallingTfKernel(
                    output_shape.IsTuple() ? xla::GetTupleElement(out, i) : out);
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 namespace {
@@ -280,25 +425,32 @@ int GetOutputBufferId(int output_num, const TfCallbackData& callback_data) {
 }
 
 int64_t BufferSize(const TfCallbackData::BufferDescription& descr) {
-  TensorShape shape;
-  TF_CHECK_OK(TensorShape::BuildTensorShape(descr.shape(), &shape));  // Crash OK
-  return shape.num_elements() * DataTypeSize(descr.type());
+  // Do this without calling TensorShape::BuildTensorShape, because that's
+  // nontrivially expensive.
+  int64_t num_elems = 1;
+  for (const auto& d : descr.shape().dim()) {
+    num_elems *= d.size();
+  }
+  return num_elems * DataTypeSize(descr.type());
 }
 
 class TfCallbackDevice : public DeviceBase {
  public:
-  explicit TfCallbackDevice(se::Stream* stream, void** buffers,
-                            const TfCallbackData& callback_data)
+  TfCallbackDevice()
       : DeviceBase(Env::Default()),
-        stream_(stream),
-#if GOOGLE_CUDA || TENSORFLOW_USE_ROCM
-        gpu_allocator_(GPUProcessState::singleton()->GetGPUAllocator(
-            *BaseGPUDevice::FindTfDeviceId(stream))),
-#else
-        gpu_allocator_(nullptr),
-#endif
         cpu_allocator_(
-            ProcessState::singleton()->GetCPUAllocator(/*numa_node=*/0)) {
+            ProcessState::singleton()->GetCPUAllocator(/*numa_node=*/0)) {}
+
+  void SetUpForCall(se::Stream* stream, void** buffers,
+                    const TfCallbackData& callback_data) {
+    stream_ = stream;
+
+#if GOOGLE_CUDA || TENSORFLOW_USE_ROCM
+    gpu_allocator_ = GPUProcessState::singleton()->GetGPUAllocator(
+        *BaseGPUDevice::FindTfDeviceId(stream));
+#endif
+
+    allocators_.clear();
     for (int i = 0; i < callback_data.outputs_size(); ++i) {
       int buffer_num = GetOutputBufferId(i, callback_data);
       VLOG(1) << "Binding output " << i << " to buffer " << buffers[buffer_num];
@@ -322,15 +474,13 @@ class TfCallbackDevice : public DeviceBase {
 #endif
   }
 
-  Status ReinitializeGpuDevice(OpKernelContext* context, PerOpGpuDevice* device,
-                               DeviceContext* dc,
-                               Allocator* allocator) override {
+  absl::Status ReinitializeGpuDevice(OpKernelContext* context,
+                                     PerOpGpuDevice* device, DeviceContext* dc,
+                                     Allocator* allocator) override {
 #if GOOGLE_CUDA || TENSORFLOW_USE_ROCM
     auto concrete_device = static_cast<ConcretePerOpGpuDevice*>(device);
-    const void* gpu_stream = reinterpret_cast<const void*>(
-        stream_->implementation()->GpuStreamMemberHack());
     concrete_device->Reinitialize(
-        context, gpu_stream,
+        context, stream_->platform_specific_handle().stream,
         /*platform_device_id=*/
         tsl::PlatformDeviceId(stream_->parent()->device_ordinal()), allocator,
         // TODO(cheshire): Pass meaningful scratch buffer.
@@ -366,9 +516,9 @@ class TfCallbackDevice : public DeviceBase {
 
  private:
   std::vector<WriteIntoXlaBufferAllocator> allocators_;
-  se::Stream* stream_;  // NOLINT (used under GOOGLE_CUDA)
-  Allocator* gpu_allocator_;
-  Allocator* cpu_allocator_;
+  se::Stream* stream_ = nullptr;  // NOLINT (used under GOOGLE_CUDA)
+  Allocator* gpu_allocator_ = nullptr;
+  Allocator* cpu_allocator_ = nullptr;
   AcceleratorDeviceInfo accelerator_device_info_;
   std::string name_ = "tf_callback_device";
 };
@@ -376,9 +526,10 @@ class TfCallbackDevice : public DeviceBase {
 // Populate the output with actual dimensions of the allocated shapes.
 //
 // Populates the vector on the host and then copies it over to the GPU.
-Status PopulateMetadataBufferIfNeeded(OpKernelContext& ctx,
-                                      const TfCallbackData& callback_data,
-                                      void** buffers, se::Stream* stream) {
+absl::Status PopulateMetadataBufferIfNeeded(OpKernelContext& ctx,
+                                            const TfCallbackData& callback_data,
+                                            void** buffers,
+                                            se::Stream* stream) {
   for (int i = 0; i < ctx.num_outputs(); i++) {
     if (callback_data.outputs(i).is_dynamically_padded()) {
       Tensor* allocated = ctx.mutable_output(i);
@@ -398,11 +549,11 @@ Status PopulateMetadataBufferIfNeeded(OpKernelContext& ctx,
       void* location = static_cast<char*>(allocated->data()) +
                        xla::ShapeUtil::ByteSizeOf(xla_shape);
       se::DeviceMemoryBase m{location, num_dimensions * sizeof(int32_t)};
-      stream->ThenMemcpy(&m, shape_info.data(),
-                         num_dimensions * sizeof(int32_t));
+      TF_RETURN_IF_ERROR(stream->Memcpy(&m, shape_info.data(),
+                                        num_dimensions * sizeof(int32_t)));
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 class FakeDeviceContext : public DeviceContext {
@@ -414,21 +565,74 @@ class FakeDeviceContext : public DeviceContext {
   se::Stream* stream_;
 };
 
-Status CallTfKernel(void* stream_handle, void** buffers, const char* opaque,
-                    int opaque_len) {
-  TF_ASSIGN_OR_RETURN(se::Platform * platform,
-                      se::MultiPlatformManager::PlatformWithName("CUDA"));
-  se::StreamExecutorConfig config;
-  config.gpu_stream = stream_handle;
-  TF_ASSIGN_OR_RETURN(se::StreamExecutor * executor,
-                      platform->GetExecutor(config));
-  se::Stream* stream = executor->FindAllocatedStream(stream_handle);
+absl::Status CallTfKernel(void* stream_handle, void** buffers,
+                          const char* opaque, int opaque_len) {
+  // Look up the platform only once, for a small performance gain.
+  static absl::Status* platform_status = nullptr;
+  static se::Platform* platform = [&]() -> se::Platform* {
+    absl::StatusOr<se::Platform*> p =
+        se::PlatformManager::PlatformWithName("CUDA");
+    if (!p.ok()) {
+      platform_status = new absl::Status(p.status());
+      return nullptr;
+    }
+    return *p;
+  }();
+  if (platform_status != nullptr) return *platform_status;
+
+  TF_ASSIGN_OR_RETURN(se::Stream * stream,
+                      stream_executor::FindStream(platform, stream_handle));
   if (!stream) {
-    return xla::InternalError("Stream not found for %p", stream_handle);
+    return xla::Internal("Stream not found for %p", stream_handle);
   }
-  TF_ASSIGN_OR_RETURN(TfCallbackData callback_data,
-                      CallbackDataFromProto(opaque, opaque_len));
-  TfCallbackDevice device(stream, buffers, callback_data);
+
+  TF_ASSIGN_OR_RETURN(KernelInstantiation * instantiation,
+                      GetInstantiation(absl::string_view(opaque, opaque_len)));
+  const TfCallbackData& callback_data = instantiation->callback_data;
+
+  // Get an existing TfCallbackDevice + OpKernel pair from the instantiation, or
+  // create a new set.
+  std::unique_ptr<TfCallbackDevice> device;
+  std::unique_ptr<OpKernel> kernel;
+  {
+    absl::MutexLock lock(&instantiation->mu);
+
+    if (instantiation->devices_and_kernels.empty()) {
+      auto device = std::make_unique<TfCallbackDevice>();
+
+      absl::Status nested_status;
+      auto kernel =
+          CreateOpKernel(DeviceType(DEVICE_GPU),
+                         /*device=*/device.get(),
+                         // NB: real allocator is passed with device, the one
+                         // here is only called during the kernel construction.
+                         // TODO(cheshire): Pass scratch allocator.
+                         /*allocator=*/nullptr, callback_data.op(),
+                         /*graph_def_version=*/1, &nested_status);
+      TF_RETURN_IF_ERROR(nested_status);
+
+      instantiation->devices_and_kernels.push_back(
+          {std::move(device), std::move(kernel)});
+    }
+
+    // Grab the last element of devices_and_kernels.
+    auto& dk = instantiation->devices_and_kernels.back();
+    device =
+        absl::WrapUnique(static_cast<TfCallbackDevice*>(dk.first.release()));
+    kernel = std::move(dk.second);
+    instantiation->devices_and_kernels.pop_back();
+  }
+
+  // Put callback_device and kernel back in `devices_and_kernels` when we're
+  // done with them.
+  auto cleanup = absl::MakeCleanup([&] {
+    absl::MutexLock lock(&instantiation->mu);
+    instantiation->devices_and_kernels.push_back(
+        std::make_pair(std::move(device), std::move(kernel)));
+  });
+
+  // Point this fake device at our stream and buffers.
+  device->SetUpForCall(stream, buffers, callback_data);
 
   std::vector<AllocatorAttributes> allocator_attributes;
   for (int output_idx = 0; output_idx < callback_data.outputs_size();
@@ -440,29 +644,17 @@ Status CallTfKernel(void* stream_handle, void** buffers, const char* opaque,
     allocator_attributes.push_back(attr);
   }
 
-  Status nested_status;
-  std::unique_ptr<OpKernel> kernel =
-      CreateOpKernel(DeviceType(DEVICE_GPU),
-                     /*device=*/&device,
-
-                     // NB: real allocator is passed with device, the one here
-                     // is only called during the kernel construction.
-                     // TODO(cheshire): Pass scratch allocator.
-                     /*allocator=*/nullptr, callback_data.op(),
-                     /*graph_def_version=*/1, &nested_status);
-  TF_RETURN_IF_ERROR(nested_status);
-
   auto device_context =
       core::RefCountPtr<FakeDeviceContext>(new FakeDeviceContext(stream));
 
   OpKernelContext::Params params;
   params.output_attr_array = allocator_attributes.data();
   params.op_kernel = kernel.get();
-  params.device = &device;
+  params.device = device.get();
   params.ensure_eigen_gpu_device();
   params.op_device_context = device_context.get();
 
-  absl::InlinedVector<TensorValue, 4> inputs;
+  absl::InlinedVector<TensorValue, 16> inputs;
 
   // Deque usage is important to avoid moving objects.
   std::deque<WriteIntoXlaBufferAllocator> input_allocators;
@@ -470,12 +662,12 @@ Status CallTfKernel(void* stream_handle, void** buffers, const char* opaque,
 
   int constant_offset = 0;
 
+  TF_RET_CHECK(callback_data.inputs_size() ==
+               instantiation->input_shapes.size());
+
   for (int i = 0; i < callback_data.inputs_size(); ++i) {
     DataType dt = callback_data.inputs(i).buffer_description().type();
-
-    TensorShape shape;
-    TF_RETURN_IF_ERROR(TensorShape::BuildTensorShape(
-        callback_data.inputs(i).buffer_description().shape(), &shape));
+    const TensorShape& shape = instantiation->input_shapes[i];
 
     VLOG(2) << "Input shape: " << shape.DebugString();
     int64_t input_size = shape.num_elements() * DataTypeSize(dt);
@@ -484,10 +676,10 @@ Status CallTfKernel(void* stream_handle, void** buffers, const char* opaque,
       // Value provided at compile time: reconstruct the tensor.
       TF_ASSIGN_OR_RETURN(Tensor input,
                           TensorFromProto(callback_data.inputs(i).value()));
-      input_tensors.push_back(input);
 
-      constant_offset++;
       VLOG(1) << "Input " << i << " is a tensor: " << input.DebugString();
+      input_tensors.push_back(std::move(input));
+      constant_offset++;
     } else {
       VLOG(1) << "Reading into the buffer for the input " << i;
 
@@ -500,7 +692,7 @@ Status CallTfKernel(void* stream_handle, void** buffers, const char* opaque,
     inputs.emplace_back(&input_tensors.back());
   }
 
-  params.inputs = inputs;
+  params.inputs = absl::MakeSpan(inputs);
   OpKernelContext ctx(&params, callback_data.outputs_size());
   kernel->Compute(&ctx);
 
@@ -514,12 +706,12 @@ Status CallTfKernel(void* stream_handle, void** buffers, const char* opaque,
   }
 
   TF_RETURN_IF_ERROR(ctx.status());
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 void GenericTfCallback(void* stream_handle, void** buffers, const char* opaque,
                        int opaque_len, XlaCustomCallStatus* status) {
-  Status s = CallTfKernel(stream_handle, buffers, opaque, opaque_len);
+  absl::Status s = CallTfKernel(stream_handle, buffers, opaque, opaque_len);
   if (!s.ok()) {
     auto msg = s.message();
     XlaCustomCallStatusSetFailure(status, msg.data(), msg.size());
