@@ -15,39 +15,48 @@ limitations under the License.
 
 // XLA specific pooling ops.
 
+#include <cstddef>
+#include <cstdint>
 #include <optional>
 #include <string>
 #include <vector>
 
+#include "absl/container/inlined_vector.h"
+#include "absl/status/status.h"
+#include "absl/status/statusor.h"
 #include "tensorflow/compiler/tf2xla/mlir_xla_op_kernel.h"
 #include "tensorflow/compiler/tf2xla/shape_util.h"
 #include "tensorflow/compiler/tf2xla/type_util.h"
 #include "tensorflow/compiler/tf2xla/xla_helpers.h"
 #include "tensorflow/compiler/tf2xla/xla_op_kernel.h"
 #include "tensorflow/compiler/tf2xla/xla_op_registry.h"
-#include "tensorflow/compiler/xla/client/lib/arithmetic.h"
-#include "tensorflow/compiler/xla/client/lib/constants.h"
-#include "tensorflow/compiler/xla/client/lib/pooling.h"
-#include "tensorflow/compiler/xla/client/value_inference.h"
-#include "tensorflow/compiler/xla/client/xla_builder.h"
-#include "tensorflow/compiler/xla/client/xla_computation.h"
-#include "tensorflow/compiler/xla/literal.h"
-#include "tensorflow/compiler/xla/util.h"
-#include "tensorflow/core/framework/bounds_check.h"
+#include "xla/hlo/builder/lib/arithmetic.h"
+#include "xla/hlo/builder/lib/constants.h"
+#include "xla/hlo/builder/lib/pooling.h"
+#include "xla/hlo/builder/padding.h"
+#include "xla/hlo/builder/value_inference.h"
+#include "xla/hlo/builder/xla_builder.h"
+#include "xla/hlo/builder/xla_computation.h"
+#include "xla/shape.h"
+#include "xla/shape_util.h"
+#include "xla/tsl/platform/errors.h"
+#include "xla/xla_data.pb.h"
 #include "tensorflow/core/framework/op_kernel.h"
 #include "tensorflow/core/framework/op_requires.h"
-#include "tensorflow/core/framework/register_types.h"
-#include "tensorflow/core/framework/tensor.h"
+#include "tensorflow/core/framework/tensor_shape.h"
+#include "tensorflow/core/framework/types.pb.h"
 #include "tensorflow/core/platform/errors.h"
+#include "tensorflow/core/platform/status.h"
+#include "tensorflow/core/platform/types.h"
 #include "tensorflow/core/util/determinism.h"
+#include "tensorflow/core/util/padding.h"
 #include "tensorflow/core/util/tensor_format.h"
-#include "tensorflow/tsl/platform/errors.h"
 
 namespace tensorflow {
 namespace {
 
 template <typename T>
-static Status ValidateKernelSizes(const T& ksizes) {
+static absl::Status ValidateKernelSizes(const T& ksizes) {
   for (size_t i = 0; i < ksizes.size(); ++i) {
     if (ksizes[i] <= 0) {
       return errors::InvalidArgument(
@@ -55,11 +64,11 @@ static Status ValidateKernelSizes(const T& ksizes) {
           " must be positive but is ", ksizes[i]);
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 template <typename T>
-static Status ValidateStrides(const T& strides) {
+static absl::Status ValidateStrides(const T& strides) {
   for (size_t i = 0; i < strides.size(); ++i) {
     if (strides[i] <= 0) {
       return errors::InvalidArgument(
@@ -67,7 +76,7 @@ static Status ValidateStrides(const T& strides) {
           " must be positive but is ", strides[i]);
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Superclass of pooling ops.
@@ -110,7 +119,7 @@ class PoolingOp : public XlaOpKernel {
   int num_dims() const { return num_spatial_dims_ + 2; }
 
  protected:
-  StatusOr<std::vector<int64_t>> GetKernelSize(XlaOpKernelContext* ctx) {
+  absl::StatusOr<std::vector<int64_t>> GetKernelSize(XlaOpKernelContext* ctx) {
     std::vector<int64_t> ksize;
     if (ctx->num_inputs() == 1) {
       ksize = ksize_;
@@ -136,7 +145,7 @@ class PoolingOp : public XlaOpKernel {
     return ksize;
   }
 
-  StatusOr<std::vector<int64_t>> GetStride(XlaOpKernelContext* ctx) {
+  absl::StatusOr<std::vector<int64_t>> GetStride(XlaOpKernelContext* ctx) {
     std::vector<int64_t> stride;
     if (ctx->num_inputs() == 1) {
       stride = stride_;
@@ -216,7 +225,7 @@ class MaxPoolOp : public PoolingOp {
 
     xla::XlaOp input = ctx->Input(0);
 
-    StatusOr<xla::Shape> input_shape = ctx->builder()->GetShape(input);
+    absl::StatusOr<xla::Shape> input_shape = ctx->builder()->GetShape(input);
     OP_REQUIRES_OK(ctx, input_shape.status());
 
     // For VECT_C max-pool ops, transpose to plain NCHW, do the max-pool, and
@@ -231,7 +240,7 @@ class MaxPoolOp : public PoolingOp {
       OP_REQUIRES_OK(ctx, input_shape.status());
     }
 
-    OP_REQUIRES(ctx, input_shape->dimensions_size() == num_dims(),
+    OP_REQUIRES(ctx, input_shape->dimensions().size() == num_dims(),
                 errors::InvalidArgument("Input to ", type_string(),
                                         " operator must have ", num_dims(),
                                         " dimensions"));
@@ -239,10 +248,11 @@ class MaxPoolOp : public PoolingOp {
         input, ksize, stride, padding_,
         XlaTensorFormat(
             data_format_ == FORMAT_NCHW_VECT_C ? FORMAT_NCHW : data_format_,
-            input_shape->dimensions_size() - 2));
+            input_shape->dimensions().size() - 2));
 
     if (data_format_ == FORMAT_NCHW_VECT_C) {
-      StatusOr<xla::Shape> result_shape = ctx->builder()->GetShape(pooling);
+      absl::StatusOr<xla::Shape> result_shape =
+          ctx->builder()->GetShape(pooling);
       OP_REQUIRES_OK(ctx, result_shape.status());
 
       int64 num_channels = result_shape->dimensions(1);
