@@ -15,26 +15,31 @@ limitations under the License.
 
 // XLA TensorArray operators.
 
-#include <limits>
+#include <cstdint>
 #include <vector>
 
+#include "absl/log/check.h"
+#include "absl/status/status.h"
+#include "absl/strings/str_cat.h"
+#include "absl/types/span.h"
 #include "tensorflow/compiler/tf2xla/kernels/gather_op_helpers.h"
 #include "tensorflow/compiler/tf2xla/shape_util.h"
-#include "tensorflow/compiler/tf2xla/type_util.h"
 #include "tensorflow/compiler/tf2xla/xla_helpers.h"
 #include "tensorflow/compiler/tf2xla/xla_op_kernel.h"
 #include "tensorflow/compiler/tf2xla/xla_op_registry.h"
 #include "tensorflow/compiler/tf2xla/xla_resource.h"
-#include "tensorflow/compiler/xla/client/xla_builder.h"
-#include "tensorflow/compiler/xla/literal.h"
-#include "tensorflow/core/framework/bounds_check.h"
+#include "xla/hlo/builder/xla_builder.h"
+#include "xla/status_macros.h"
+#include "xla/tsl/platform/errors.h"
 #include "tensorflow/core/framework/op_kernel.h"
-#include "tensorflow/core/framework/partial_tensor_shape.h"
-#include "tensorflow/core/framework/register_types.h"
+#include "tensorflow/core/framework/op_requires.h"
 #include "tensorflow/core/framework/tensor.h"
+#include "tensorflow/core/framework/tensor_shape.h"
 #include "tensorflow/core/framework/tensor_types.h"
 #include "tensorflow/core/framework/types.h"
+#include "tensorflow/core/framework/types.pb.h"
 #include "tensorflow/core/lib/core/status.h"
+#include "tensorflow/core/platform/errors.h"
 #include "tensorflow/core/platform/types.h"
 
 namespace tensorflow {
@@ -47,9 +52,9 @@ namespace {
 // the TensorArray with elements of `elem_shape`. For both initialized and
 // uninitialized TensorArrays, checks that the tensor has a type compatible with
 // 'dtype' and shape compatible with 'elem_shape'.
-Status MaybeInitializeTensorArray(xla::XlaBuilder* builder,
-                                  XlaResource* resource, DataType dtype,
-                                  const TensorShape& elem_shape) {
+absl::Status MaybeInitializeTensorArray(xla::XlaBuilder* builder,
+                                        XlaResource* resource, DataType dtype,
+                                        const TensorShape& elem_shape) {
   if (resource->kind() != XlaResource::kTensorArray) {
     return errors::InvalidArgument("Unexpected non-TensorArray resource");
   }
@@ -84,14 +89,14 @@ Status MaybeInitializeTensorArray(xla::XlaBuilder* builder,
           shape.DebugString());
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Checks that the TensorArray 'resource' has been initialized, and has type
 // 'dtype'. Sets 'shape' to the shape
-Status CheckTensorArrayIsInitialized(const string& op_name,
-                                     const XlaResource* resource,
-                                     DataType dtype) {
+absl::Status CheckTensorArrayIsInitialized(const string& op_name,
+                                           const XlaResource* resource,
+                                           DataType dtype) {
   if (resource->kind() != XlaResource::kTensorArray) {
     return errors::InvalidArgument(
         "Unexpected non-TensorArray resource passed to ", op_name);
@@ -106,14 +111,14 @@ Status CheckTensorArrayIsInitialized(const string& op_name,
         " but op has dtype ", DataTypeString(dtype), ".");
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status GetTensorArrayShape(const XlaResource* resource,
-                           xla::XlaBuilder* builder, TensorShape* shape) {
+absl::Status GetTensorArrayShape(const XlaResource* resource,
+                                 xla::XlaBuilder* builder, TensorShape* shape) {
   *shape = resource->shape();
   shape->InsertDim(0, resource->max_array_size());
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Like XlaBuilder::DynamicUpdateSlice, but adds 'update' to the
@@ -181,7 +186,8 @@ class TensorArrayOp : public XlaOpKernel {
   DataType dtype_;
   string tensor_array_name_;
 
-  TF_DISALLOW_COPY_AND_ASSIGN(TensorArrayOp);
+  TensorArrayOp(const TensorArrayOp&) = delete;
+  void operator=(const TensorArrayOp&) = delete;
 };
 
 REGISTER_XLA_OP(Name("TensorArrayV3").CompileTimeConstantInput("size"),
@@ -236,7 +242,8 @@ class TensorArrayWriteOp : public XlaOpKernel {
  private:
   DataType dtype_;
 
-  TF_DISALLOW_COPY_AND_ASSIGN(TensorArrayWriteOp);
+  TensorArrayWriteOp(const TensorArrayWriteOp&) = delete;
+  void operator=(const TensorArrayWriteOp&) = delete;
 };
 
 REGISTER_XLA_OP(Name("TensorArrayWriteV3"), TensorArrayWriteOp);
@@ -280,7 +287,8 @@ class TensorArrayReadOp : public XlaOpKernel {
  private:
   DataType dtype_;
 
-  TF_DISALLOW_COPY_AND_ASSIGN(TensorArrayReadOp);
+  TensorArrayReadOp(const TensorArrayReadOp&) = delete;
+  void operator=(const TensorArrayReadOp&) = delete;
 };
 
 REGISTER_XLA_OP(Name("TensorArrayReadV3"), TensorArrayReadOp);
@@ -313,7 +321,7 @@ class TensorArrayGatherOp : public XlaOpKernel {
     // Look for the case where the gather takes a simple slice from the
     // tensor array (0, 1, 2, 3, 4, ..., N)
     std::vector<int64_t> const_indices;
-    Status status = ctx->ConstantInputAsIntVector(1, &const_indices);
+    absl::Status status = ctx->ConstantInputAsIntVector(1, &const_indices);
     if (status.ok()) {
       bool gather_is_dense_slice = true;
       for (auto i = 0; i < const_indices.size(); i++) {
@@ -347,7 +355,8 @@ class TensorArrayGatherOp : public XlaOpKernel {
  private:
   DataType dtype_;
 
-  TF_DISALLOW_COPY_AND_ASSIGN(TensorArrayGatherOp);
+  TensorArrayGatherOp(const TensorArrayGatherOp&) = delete;
+  void operator=(const TensorArrayGatherOp&) = delete;
 };
 
 REGISTER_XLA_OP(Name("TensorArrayGatherV3"), TensorArrayGatherOp);
@@ -384,7 +393,7 @@ class TensorArrayScatterOp : public XlaOpKernel {
     // tensor array implementation allows for this to be a straight addition.
     bool scatter_all_elements_in_order = false;
     std::vector<int64_t> const_indices;
-    Status status = ctx->ConstantInputAsIntVector(1, &const_indices);
+    absl::Status status = ctx->ConstantInputAsIntVector(1, &const_indices);
     if (status.ok() && num_indices == value_shape.dim_size(0)) {
       scatter_all_elements_in_order = true;
       for (auto i = 0; i < num_indices; i++) {
@@ -434,7 +443,8 @@ class TensorArrayScatterOp : public XlaOpKernel {
  private:
   DataType dtype_;
 
-  TF_DISALLOW_COPY_AND_ASSIGN(TensorArrayScatterOp);
+  TensorArrayScatterOp(const TensorArrayScatterOp&) = delete;
+  void operator=(const TensorArrayScatterOp&) = delete;
 };
 
 REGISTER_XLA_OP(Name("TensorArrayScatterV3"), TensorArrayScatterOp);
@@ -474,7 +484,8 @@ class TensorArrayConcatOp : public XlaOpKernel {
  private:
   DataType dtype_;
 
-  TF_DISALLOW_COPY_AND_ASSIGN(TensorArrayConcatOp);
+  TensorArrayConcatOp(const TensorArrayConcatOp&) = delete;
+  void operator=(const TensorArrayConcatOp&) = delete;
 };
 
 REGISTER_XLA_OP(Name("TensorArrayConcatV3"), TensorArrayConcatOp);
@@ -544,7 +555,8 @@ class TensorArraySplitOp : public XlaOpKernel {
  private:
   DataType dtype_;
 
-  TF_DISALLOW_COPY_AND_ASSIGN(TensorArraySplitOp);
+  TensorArraySplitOp(const TensorArraySplitOp&) = delete;
+  void operator=(const TensorArraySplitOp&) = delete;
 };
 
 REGISTER_XLA_OP(Name("TensorArraySplitV3").CompileTimeConstantInput("lengths"),
@@ -563,7 +575,8 @@ class TensorArraySizeOp : public XlaOpKernel {
   }
 
  private:
-  TF_DISALLOW_COPY_AND_ASSIGN(TensorArraySizeOp);
+  TensorArraySizeOp(const TensorArraySizeOp&) = delete;
+  void operator=(const TensorArraySizeOp&) = delete;
 };
 
 REGISTER_XLA_OP(Name("TensorArraySizeV3"), TensorArraySizeOp);
@@ -598,7 +611,8 @@ class TensorArrayGradOp : public XlaOpKernel {
  private:
   string source_;
 
-  TF_DISALLOW_COPY_AND_ASSIGN(TensorArrayGradOp);
+  TensorArrayGradOp(const TensorArrayGradOp&) = delete;
+  void operator=(const TensorArrayGradOp&) = delete;
 };
 
 REGISTER_XLA_OP(Name("TensorArrayGradV3"), TensorArrayGradOp);
@@ -612,7 +626,8 @@ class TensorArrayCloseOp : public XlaOpKernel {
   }
 
  private:
-  TF_DISALLOW_COPY_AND_ASSIGN(TensorArrayCloseOp);
+  TensorArrayCloseOp(const TensorArrayCloseOp&) = delete;
+  void operator=(const TensorArrayCloseOp&) = delete;
 };
 
 REGISTER_XLA_OP(Name("TensorArrayCloseV3"), TensorArrayCloseOp);
