@@ -15,19 +15,25 @@ limitations under the License.
 
 #include "tensorflow/compiler/tf2xla/kernels/tensor_list_utils.h"
 
+#include <cstdint>
 #include <vector>
 
-#include "tensorflow/compiler/tf2xla/shape_util.h"
-#include "tensorflow/compiler/xla/client/xla_builder.h"
-#include "tensorflow/compiler/xla/literal_util.h"
-#include "tensorflow/compiler/xla/shape.h"
-#include "tensorflow/compiler/xla/shape_util.h"
-#include "tensorflow/compiler/xla/status_macros.h"
-#include "tensorflow/compiler/xla/util.h"
-#include "tensorflow/compiler/xla/xla_data.pb.h"
-#include "tensorflow/core/framework/tensor_shape.h"
+#include "absl/log/log.h"
+#include "absl/status/status.h"
+#include "tensorflow/compiler/tf2xla/xla_expression.h"
+#include "tensorflow/compiler/tf2xla/xla_op_kernel.h"
+#include "xla/hlo/builder/xla_builder.h"
+#include "xla/literal_util.h"
+#include "xla/shape.h"
+#include "xla/shape_util.h"
+#include "xla/status_macros.h"
+#include "xla/tsl/platform/errors.h"
+#include "xla/tsl/platform/statusor.h"
+#include "xla/util.h"
+#include "xla/xla_data.pb.h"
 #include "tensorflow/core/lib/core/errors.h"
-#include "tensorflow/core/platform/statusor.h"
+#include "tensorflow/core/platform/status.h"
+#include "tensorflow/core/platform/types.h"
 
 // TensorList is represented by a tuple.
 // - The first part of the tuple is a buffer containing all the tensors,
@@ -114,13 +120,13 @@ bool IsTensorListInput(XlaOpKernelContext* ctx, int index) {
   return ctx->InputExpression(index).kind() == XlaExpression::Kind::kTensorList;
 }
 
-Status IsTensorListInitialized(xla::XlaOp list, bool* is_initialized) {
+absl::Status IsTensorListInitialized(xla::XlaOp list, bool* is_initialized) {
   TF_ASSIGN_OR_RETURN(xla::Shape list_shape, list.builder()->GetShape(list));
   *is_initialized = list_shape.IsTuple();
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status IsNestedTensorList(xla::XlaOp list, bool* is_nested_list) {
+absl::Status IsNestedTensorList(xla::XlaOp list, bool* is_nested_list) {
   bool is_initialized;
   TF_RETURN_IF_ERROR(IsTensorListInitialized(list, &is_initialized));
   if (!is_initialized) {
@@ -128,17 +134,18 @@ Status IsNestedTensorList(xla::XlaOp list, bool* is_nested_list) {
   }
   TF_ASSIGN_OR_RETURN(xla::Shape list_shape, list.builder()->GetShape(list));
   *is_nested_list = (xla::ShapeUtil::TupleElementCount(list_shape) > 2);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status BuildNonNestedTensorList(xla::XlaOp buffer, xla::XlaOp push_index,
-                                xla::XlaOp* output_list) {
+absl::Status BuildNonNestedTensorList(xla::XlaOp buffer, xla::XlaOp push_index,
+                                      xla::XlaOp* output_list) {
   TF_RET_CHECK(buffer.builder());
   *output_list = xla::Tuple(buffer.builder(), {buffer, push_index});
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status GetTensorListBufferShape(xla::XlaOp list, xla::Shape* buffer_shape) {
+absl::Status GetTensorListBufferShape(xla::XlaOp list,
+                                      xla::Shape* buffer_shape) {
   bool is_initialized;
   TF_RETURN_IF_ERROR(IsTensorListInitialized(list, &is_initialized));
   if (!is_initialized) {
@@ -146,20 +153,20 @@ Status GetTensorListBufferShape(xla::XlaOp list, xla::Shape* buffer_shape) {
   }
   TF_ASSIGN_OR_RETURN(xla::Shape list_shape, list.builder()->GetShape(list));
   *buffer_shape = xla::ShapeUtil::GetTupleElementShape(list_shape, 0);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status GetTensorListBuffer(xla::XlaOp list, xla::XlaOp* buffer) {
+absl::Status GetTensorListBuffer(xla::XlaOp list, xla::XlaOp* buffer) {
   bool is_initialized;
   TF_RETURN_IF_ERROR(IsTensorListInitialized(list, &is_initialized));
   if (!is_initialized) {
     return errors::InvalidArgument("TensorList is not initialized");
   }
   *buffer = xla::GetTupleElement(list, 0);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status GetTensorListPushIndex(xla::XlaOp list, xla::XlaOp* push_index) {
+absl::Status GetTensorListPushIndex(xla::XlaOp list, xla::XlaOp* push_index) {
   bool is_initialized;
   TF_RETURN_IF_ERROR(IsTensorListInitialized(list, &is_initialized));
   if (!is_initialized) {
@@ -168,11 +175,11 @@ Status GetTensorListPushIndex(xla::XlaOp list, xla::XlaOp* push_index) {
   TF_ASSIGN_OR_RETURN(xla::Shape list_shape, list.builder()->GetShape(list));
   int tuple_size = xla::ShapeUtil::TupleElementCount(list_shape);
   *push_index = xla::GetTupleElement(list, tuple_size - 1);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status SetTensorListPushIndex(xla::XlaOp list, xla::XlaOp push_index,
-                              xla::XlaOp* result) {
+absl::Status SetTensorListPushIndex(xla::XlaOp list, xla::XlaOp push_index,
+                                    xla::XlaOp* result) {
   bool is_initialized;
   TF_RETURN_IF_ERROR(IsTensorListInitialized(list, &is_initialized));
   if (!is_initialized) {
@@ -187,7 +194,7 @@ Status SetTensorListPushIndex(xla::XlaOp list, xla::XlaOp push_index,
   }
   result_parts.push_back(push_index);
   *result = xla::Tuple(list.builder(), result_parts);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 xla::XlaOp BuildUninitializedTensorList(xla::XlaBuilder* b,
@@ -205,9 +212,9 @@ xla::XlaOp BuildUninitializedTensorList(xla::XlaBuilder* b,
   }
 }
 
-Status GetLeadingDimForTensorList(xla::XlaOp list, int64_t* leading_dim,
-                                  bool* leading_dim_is_dynamic,
-                                  xla::XlaOp* leading_dim_dynamic_size) {
+absl::Status GetLeadingDimForTensorList(xla::XlaOp list, int64_t* leading_dim,
+                                        bool* leading_dim_is_dynamic,
+                                        xla::XlaOp* leading_dim_dynamic_size) {
   bool is_initialized;
   TF_RETURN_IF_ERROR(IsTensorListInitialized(list, &is_initialized));
   TF_ASSIGN_OR_RETURN(xla::Shape list_shape, list.builder()->GetShape(list));
@@ -222,10 +229,10 @@ Status GetLeadingDimForTensorList(xla::XlaOp list, int64_t* leading_dim,
     *leading_dim = list_shape.dimensions(0);
     *leading_dim_dynamic_size = xla::GetDimensionSize(list, 0);
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status GetTensorListShapeFromElementTensorListShape(
+absl::Status GetTensorListShapeFromElementTensorListShape(
     const xla::Shape& element_tensor_list_shape, int64_t leading_dim,
     bool leading_dim_is_dynamic, xla::Shape* tensor_list_shape) {
   std::vector<xla::Shape> shapes;
@@ -244,18 +251,18 @@ Status GetTensorListShapeFromElementTensorListShape(
   shapes.push_back(xla::ShapeUtil::MakeShape(xla::PrimitiveType::S32,
                                              std::vector<int64_t>{}));
   *tensor_list_shape = xla::ShapeUtil::MakeTupleShape(shapes);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status GetTensorListShapeFromElementShape(const xla::Shape& element_shape,
-                                          int64_t leading_dim,
-                                          bool leading_dim_is_dynamic,
-                                          xla::Shape* tensor_list_shape) {
+absl::Status GetTensorListShapeFromElementShape(const xla::Shape& element_shape,
+                                                int64_t leading_dim,
+                                                bool leading_dim_is_dynamic,
+                                                xla::Shape* tensor_list_shape) {
   if (!element_shape.IsArray()) {
     return errors::InvalidArgument(
         "GetTensorListShapeFromElementShape() only supports normal tensor "
         "shape. But element shape is ",
-        element_shape.DebugString());
+        element_shape.ToString());
   }
   std::vector<xla::Shape> shapes;
   std::vector<int64_t> dimensions =
@@ -267,10 +274,10 @@ Status GetTensorListShapeFromElementShape(const xla::Shape& element_shape,
   shapes.push_back(xla::ShapeUtil::MakeShape(xla::PrimitiveType::S32,
                                              std::vector<int64_t>{}));
   *tensor_list_shape = xla::ShapeUtil::MakeTupleShape(shapes);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status CreateZerosTensorListWithShape(
+absl::Status CreateZerosTensorListWithShape(
     xla::XlaBuilder* b, const xla::Shape& list_shape,
     const std::vector<std::vector<xla::XlaOp>>& dynamic_dims,
     xla::XlaOp* list) {
@@ -283,9 +290,11 @@ Status CreateZerosTensorListWithShape(
     xla::XlaOp zero =
         xla::ConstantLiteral(b, xla::LiteralUtil::Zero(shape.element_type()));
     xla::XlaOp zeros = xla::Broadcast(zero, shape.dimensions());
-    TF_RET_CHECK(dynamic_dims[i].size() == shape.dimensions_size());
-    for (int64_t dim = 0; dim < shape.dimensions_size(); ++dim) {
-      zeros = xla::SetDimensionSize(zeros, dynamic_dims[i][dim], dim);
+    TF_RET_CHECK(dynamic_dims[i].size() == shape.dimensions().size());
+    for (int64_t dim = 0; dim < shape.dimensions().size(); ++dim) {
+      if (shape.is_dynamic_dimension(dim)) {
+        zeros = xla::SetDimensionSize(zeros, dynamic_dims[i][dim], dim);
+      }
     }
     elements.push_back(zeros);
   }
@@ -294,12 +303,13 @@ Status CreateZerosTensorListWithShape(
                    .element_type() == xla::S32);
   elements.push_back(xla::ConstantLiteral(b, xla::LiteralUtil::Zero(xla::S32)));
   *list = xla::Tuple(b, elements);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status GetInitializedTensorListForElement(xla::XlaOp list, xla::XlaOp element,
-                                          bool element_is_tensor_list,
-                                          xla::XlaOp* initialized_list) {
+absl::Status GetInitializedTensorListForElement(xla::XlaOp list,
+                                                xla::XlaOp element,
+                                                bool element_is_tensor_list,
+                                                xla::XlaOp* initialized_list) {
   int64_t leading_dim;
   xla::XlaOp leading_dim_dynamic_size;
   bool leading_dim_is_dynamic;
@@ -324,16 +334,16 @@ Status GetInitializedTensorListForElement(xla::XlaOp list, xla::XlaOp element,
     TF_ASSIGN_OR_RETURN(xla::Shape original_list_shape, b->GetShape(list));
     if (!xla::ShapeUtil::Compatible(original_list_shape, list_shape)) {
       return errors::Internal(
-          "Invalid TensorList shape: ", original_list_shape.DebugString(),
-          ", expected: ", list_shape.DebugString());
+          "Invalid TensorList shape: ", original_list_shape.ToString(),
+          ", expected: ", list_shape.ToString());
     }
     *initialized_list = list;
-    return OkStatus();
+    return absl::OkStatus();
   } else {
     // Prepare dynamic dimension dimensions for zero tensor list. The dynamic
     // sizes are created by reading the dynamic dimension size of sub-elements.
     std::vector<std::vector<xla::XlaOp>> list_dynamic_dims;
-    for (int i = 0; i < list_shape.tuple_shapes_size() - 1; ++i) {
+    for (int i = 0; i < list_shape.tuple_shapes().size() - 1; ++i) {
       std::vector<xla::XlaOp> dynamic_dims;
       const xla::Shape& shape = list_shape.tuple_shapes(i);
       dynamic_dims.push_back(leading_dim_dynamic_size);
@@ -343,7 +353,7 @@ Status GetInitializedTensorListForElement(xla::XlaOp list, xla::XlaOp element,
       } else {
         sub_element = element;
       }
-      for (int64_t dim = 0; dim < shape.dimensions_size() - 1; ++dim) {
+      for (int64_t dim = 0; dim < shape.dimensions().size() - 1; ++dim) {
         dynamic_dims.push_back(xla::GetDimensionSize(sub_element, dim));
       }
       list_dynamic_dims.push_back(dynamic_dims);
@@ -353,9 +363,9 @@ Status GetInitializedTensorListForElement(xla::XlaOp list, xla::XlaOp element,
   }
 }
 
-Status ExecuteTensorListPushBack(xla::XlaOp list, xla::XlaOp element,
-                                 bool element_is_tensor_list,
-                                 xla::XlaOp* result) {
+absl::Status ExecuteTensorListPushBack(xla::XlaOp list, xla::XlaOp element,
+                                       bool element_is_tensor_list,
+                                       xla::XlaOp* result) {
   bool is_initialized;
   TF_RETURN_IF_ERROR(IsTensorListInitialized(list, &is_initialized));
   if (!is_initialized) {
@@ -382,7 +392,7 @@ Status ExecuteTensorListPushBack(xla::XlaOp list, xla::XlaOp element,
       element_part = xla::Reshape(element_part, element_part_dims);
 
       std::vector<xla::XlaOp> start_indices(
-          element_part_shape.dimensions_size() + 1,
+          element_part_shape.dimensions().size() + 1,
           xla::ConstantR0<int32>(b, 0));
       start_indices[0] = push_index;
 
@@ -398,7 +408,7 @@ Status ExecuteTensorListPushBack(xla::XlaOp list, xla::XlaOp element,
     element_dims.insert(element_dims.begin(), 1);
     xla::XlaOp update = xla::Reshape(element, element_dims);
 
-    std::vector<xla::XlaOp> start_indices(element_shape.dimensions_size() + 1,
+    std::vector<xla::XlaOp> start_indices(element_shape.dimensions().size() + 1,
                                           xla::ConstantR0<int32>(b, 0));
     start_indices[0] = push_index;
 
@@ -412,12 +422,12 @@ Status ExecuteTensorListPushBack(xla::XlaOp list, xla::XlaOp element,
   result_parts.push_back(updated_push_index);
 
   *result = xla::Tuple(b, result_parts);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status ExecuteTensorListPopBack(xla::XlaOp list, xla::XlaOp* list_result,
-                                xla::XlaOp* element_result,
-                                bool* element_is_tensor_list) {
+absl::Status ExecuteTensorListPopBack(xla::XlaOp list, xla::XlaOp* list_result,
+                                      xla::XlaOp* element_result,
+                                      bool* element_is_tensor_list) {
   bool is_initialized;
   TF_RETURN_IF_ERROR(IsTensorListInitialized(list, &is_initialized));
   if (!is_initialized) {
@@ -437,7 +447,7 @@ Status ExecuteTensorListPopBack(xla::XlaOp list, xla::XlaOp* list_result,
   for (int i = 0; i < list_tuple_size - 1; i++) {
     const xla::Shape& list_part_shape =
         xla::ShapeUtil::GetTupleElementShape(list_shape, i);
-    std::vector<xla::XlaOp> start_indices(list_part_shape.dimensions_size(),
+    std::vector<xla::XlaOp> start_indices(list_part_shape.dimensions().size(),
                                           xla::ConstantR0<int32>(b, 0));
     start_indices[0] = push_index;
 
@@ -461,11 +471,11 @@ Status ExecuteTensorListPopBack(xla::XlaOp list, xla::XlaOp* list_result,
     *element_result = element_result_parts[0];
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status ExecuteTensorListSetItem(xla::XlaOp list, xla::XlaOp index,
-                                xla::XlaOp element, xla::XlaOp* result) {
+absl::Status ExecuteTensorListSetItem(xla::XlaOp list, xla::XlaOp index,
+                                      xla::XlaOp element, xla::XlaOp* result) {
   bool is_initialized;
   TF_RETURN_IF_ERROR(IsTensorListInitialized(list, &is_initialized));
   if (!is_initialized) {
@@ -485,11 +495,32 @@ Status ExecuteTensorListSetItem(xla::XlaOp list, xla::XlaOp index,
   element_dims.insert(element_dims.begin(), 1);
   xla::XlaOp update = xla::Reshape(element, element_dims);
 
-  std::vector<xla::XlaOp> start_indices(element_shape.dimensions_size() + 1,
+  std::vector<xla::XlaOp> start_indices(element_shape.dimensions().size() + 1,
                                         xla::ConstantR0<int32>(b, 0));
   start_indices[0] = index;
 
   xla::XlaOp list_part = xla::GetTupleElement(list, 0);
+  {
+    TF_ASSIGN_OR_RETURN(const xla::Shape* list_part_shape,
+                        b->GetShapePtr(list_part));
+    TF_ASSIGN_OR_RETURN(const xla::Shape* update_shape, b->GetShapePtr(update));
+    for (int i = 0; i < list_part_shape->dimensions().size(); ++i) {
+      auto list_part_dim_size = list_part_shape->dimensions(i);
+      auto update_dim_size = update_shape->dimensions(i);
+      // If the update is larger than the list part, the DynamicUpdateSlice will
+      // fail so just ignore this operation and return list as is.
+      if (update_dim_size > list_part_dim_size) {
+        LOG_FIRST_N(WARNING, 1)
+            << "Warning: TensorListSetItem: ignoring set item because the "
+               "update dim ["
+            << update_dim_size << "] is larger than the list dim ["
+            << list_part_dim_size << "] at dimension " << i << ".";
+
+        *result = list;
+        return absl::OkStatus();
+      }
+    }
+  }
   xla::XlaOp updated_list_part =
       xla::DynamicUpdateSlice(list_part, update, start_indices);
 
@@ -497,11 +528,11 @@ Status ExecuteTensorListSetItem(xla::XlaOp list, xla::XlaOp index,
   result_parts.push_back(updated_list_part);
   result_parts.push_back(xla::GetTupleElement(list, 1));
   *result = xla::Tuple(b, result_parts);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status ExecuteTensorListGetItem(xla::XlaOp list, xla::XlaOp index,
-                                xla::XlaOp* result) {
+absl::Status ExecuteTensorListGetItem(xla::XlaOp list, xla::XlaOp index,
+                                      xla::XlaOp* result) {
   bool is_initialized;
   TF_RETURN_IF_ERROR(IsTensorListInitialized(list, &is_initialized));
   if (!is_initialized) {
@@ -518,7 +549,7 @@ Status ExecuteTensorListGetItem(xla::XlaOp list, xla::XlaOp index,
   TF_ASSIGN_OR_RETURN(xla::Shape list_shape, b->GetShape(list));
   const xla::Shape& buffer_shape =
       xla::ShapeUtil::GetTupleElementShape(list_shape, 0);
-  std::vector<xla::XlaOp> start_indices(buffer_shape.dimensions_size(),
+  std::vector<xla::XlaOp> start_indices(buffer_shape.dimensions().size(),
                                         xla::ConstantR0<int32>(b, 0));
   start_indices[0] = index;
 
@@ -530,7 +561,7 @@ Status ExecuteTensorListGetItem(xla::XlaOp list, xla::XlaOp index,
   xla::XlaOp read = xla::DynamicSlice(list_part, start_indices, slice_shape);
   // Propagate dynamic dimensions from buffer to the sliced buffer, except for
   // leading dimension (which is always static 1).
-  for (int64_t i = 1; i < buffer_shape.dimensions_size(); ++i) {
+  for (int64_t i = 1; i < buffer_shape.dimensions().size(); ++i) {
     if (buffer_shape.is_dynamic_dimension(i)) {
       auto buffer = xla::GetTupleElement(list, 0);
       auto gds = xla::GetDimensionSize(buffer, i);
@@ -539,24 +570,24 @@ Status ExecuteTensorListGetItem(xla::XlaOp list, xla::XlaOp index,
   }
   slice_shape.erase(slice_shape.begin());
   *result = xla::Reshape(read, slice_shape);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status ExecuteTensorListFromTensor(int push_index, xla::XlaOp tensor,
-                                   xla::XlaOp* result) {
+absl::Status ExecuteTensorListFromTensor(int push_index, xla::XlaOp tensor,
+                                         xla::XlaOp* result) {
   xla::XlaBuilder* b = tensor.builder();
   TF_ASSIGN_OR_RETURN(xla::Shape shape, b->GetShape(tensor));
   if (!shape.IsArray()) {
     return errors::InvalidArgument(
         "ExecuteTensorListFromTensor() only supports normal tensor. But input "
         "shape is ",
-        shape.DebugString());
+        shape.ToString());
   }
 
   std::vector<xla::XlaOp> result_parts{tensor,
                                        xla::ConstantR0<int32>(b, push_index)};
   *result = xla::Tuple(b, result_parts);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace tensorflow
