@@ -15,37 +15,52 @@ limitations under the License.
 
 #include "tensorflow/compiler/tf2xla/kernels/while_op.h"
 
+#include <cstdint>
 #include <memory>
 #include <utility>
 #include <vector>
 
-#include "absl/strings/str_split.h"
-#include "tensorflow/compiler/tf2xla/const_analysis.h"
+#include "absl/container/inlined_vector.h"
+#include "absl/log/log.h"
+#include "absl/status/status.h"
+#include "absl/strings/str_join.h"
 #include "tensorflow/compiler/tf2xla/kernels/if_while_utils.h"
 #include "tensorflow/compiler/tf2xla/kernels/tensor_list_utils.h"
-#include "tensorflow/compiler/tf2xla/shape_util.h"
 #include "tensorflow/compiler/tf2xla/side_effect_util.h"
 #include "tensorflow/compiler/tf2xla/tf2xla_util.h"
-#include "tensorflow/compiler/tf2xla/type_util.h"
 #include "tensorflow/compiler/tf2xla/xla_compiler.h"
 #include "tensorflow/compiler/tf2xla/xla_helpers.h"
 #include "tensorflow/compiler/tf2xla/xla_op_kernel.h"
 #include "tensorflow/compiler/tf2xla/xla_op_registry.h"
-#include "tensorflow/compiler/xla/client/xla_builder.h"
-#include "tensorflow/compiler/xla/client/xla_computation.h"
-#include "tensorflow/compiler/xla/literal.h"
-#include "tensorflow/compiler/xla/status_macros.h"
+#include "tensorflow/compiler/tf2xla/xla_resource.h"
+#include "xla/client/client.h"
+#include "xla/hlo/builder/lib/tuple.h"
+#include "xla/hlo/builder/xla_builder.h"
+#include "xla/hlo/builder/xla_computation.h"
+#include "xla/shape.h"
+#include "xla/shape_tree.h"
+#include "xla/shape_util.h"
+#include "xla/status_macros.h"
+#include "xla/tsl/platform/errors.h"
+#include "xla/tsl/platform/statusor.h"
+#include "xla/xla_data.pb.h"
 #include "tensorflow/core/framework/attr_value.pb.h"
 #include "tensorflow/core/framework/function.h"
 #include "tensorflow/core/framework/op_kernel.h"
+#include "tensorflow/core/framework/op_requires.h"
+#include "tensorflow/core/framework/types.h"
+#include "tensorflow/core/framework/types.pb.h"
+#include "tensorflow/core/platform/errors.h"
+#include "tensorflow/core/platform/status.h"
+#include "tensorflow/core/platform/types.h"
 
 namespace tensorflow {
 
 namespace {
 
 // Verify that input resources are grouped in the end.
-Status VerifyResourceArgsGroupedAtEnd(XlaOpKernelContext* ctx,
-                                      const NameAttrList& body_name_attr) {
+absl::Status VerifyResourceArgsGroupedAtEnd(
+    XlaOpKernelContext* ctx, const NameAttrList& body_name_attr) {
   const FunctionBody* body;
   TF_RETURN_IF_ERROR(ctx->compiler()->FindFunctionBody(body_name_attr, &body));
   bool has_seen_resource = false;
@@ -64,11 +79,11 @@ Status VerifyResourceArgsGroupedAtEnd(XlaOpKernelContext* ctx,
       }
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Builds XlaCompiler argument descriptions `args` from `ctx`.
-Status MakeXlaCompilerArgumentsFromInputs(
+absl::Status MakeXlaCompilerArgumentsFromInputs(
     XlaOpKernelContext* ctx, std::vector<XlaCompiler::Argument>* args,
     bool* has_uninitialized_vars, bool* has_tensor_arrays,
     bool* has_uninitialized_tensor_lists) {
@@ -115,7 +130,7 @@ Status MakeXlaCompilerArgumentsFromInputs(
       }
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Populates loop invariant indices to true in `loop_invariants`.
@@ -127,7 +142,7 @@ void GetLoopInvariants(XlaOpKernelContext* ctx,
   const tensorflow::FunctionLibraryDefinition* fld =
       ctx->compiler()->flib_runtime()->GetFunctionLibraryDefinition();
   for (int i = 0; i < body->ret_nodes.size(); i++) {
-    StatusOr<bool> is_loop_invariant = IsLoopInvariant(body, i, fld);
+    absl::StatusOr<bool> is_loop_invariant = IsLoopInvariant(body, i, fld);
     OP_REQUIRES_OK(ctx, is_loop_invariant.status());
     (*loop_invariants)[i] = *is_loop_invariant;
     VLOG(2) << "Arg " << i << " of " << body_name_attr.name() << " is "
@@ -138,7 +153,7 @@ void GetLoopInvariants(XlaOpKernelContext* ctx,
 // Converts entries in `args` which are loop invariants and have compile time
 // constant inputs and need to be constants in order to be compilable to
 // constants so that they can be propagated in the loop body.
-Status ConvertLoopInvariantsToConst(
+absl::Status ConvertLoopInvariantsToConst(
     XlaOpKernelContext* ctx, const NameAttrList& body_name_attr,
     const NameAttrList& cond_name_attr,
     std::vector<XlaCompiler::Argument>* args,
@@ -173,10 +188,10 @@ Status ConvertLoopInvariantsToConst(
     compile_time_const_arg_indices->at(arg_idx) = true;
     (*num_compile_time_const_args)++;
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status VerifyBodyInputAndOutputShapeMatch(
+absl::Status VerifyBodyInputAndOutputShapeMatch(
     XlaOpKernelContext* ctx,
     const std::vector<bool>& compile_time_const_arg_indices,
     const XlaCompiler::CompilationResult& body, bool has_token_input_output) {
@@ -200,10 +215,10 @@ Status VerifyBodyInputAndOutputShapeMatch(
         xla::ShapeUtil::HumanString(body_input_shape), " vs. ",
         xla::ShapeUtil::HumanString(body_output_shape));
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-StatusOr<xla::XlaComputation> BuildWrappedCond(
+absl::StatusOr<xla::XlaComputation> BuildWrappedCond(
     XlaOpKernelContext* ctx, const XlaCompiler::CompilationResult& cond) {
   xla::Shape cond_input_shape = cond.xla_input_shapes[0];
   std::unique_ptr<xla::XlaBuilder> cb =
@@ -214,11 +229,12 @@ StatusOr<xla::XlaComputation> BuildWrappedCond(
   return cb->Build();
 }
 
-StatusOr<xla::XlaComputation> BuildWrappedBody(
+absl::StatusOr<xla::XlaComputation> BuildWrappedBody(
     XlaOpKernelContext* ctx, const XlaCompiler::CompilationResult& body,
     const std::vector<bool>& compile_time_const_arg_indices,
     int num_compile_time_const_args, bool has_token_input_output) {
-  if (num_compile_time_const_args <= 0) {
+  if (num_compile_time_const_args <= 0 &&
+      body.xla_input_shapes[0] == body.xla_output_shape) {
     return xla::XlaComputation(body.computation->proto());
   }
   xla::XlaComputation body_wrapper;
@@ -231,10 +247,41 @@ StatusOr<xla::XlaComputation> BuildWrappedBody(
   // the inputs and outputs of its body function to match.
   auto outputs = xla::Call(cb.get(), *body.computation, {inputs});
   std::vector<xla::XlaOp> non_compile_time_const_outputs;
+  int input_num = 0;
   for (int i = 0; i < compile_time_const_arg_indices.size(); i++) {
     if (!compile_time_const_arg_indices[i]) {
-      non_compile_time_const_outputs.push_back(
-          xla::GetTupleElement(outputs, i));
+      xla::XlaOp output = xla::GetTupleElement(outputs, i);
+      const xla::Shape& input_shape = body_input_shape.tuple_shapes(input_num);
+      const xla::Shape& output_shape = body.xla_output_shape.tuple_shapes(i);
+      TF_RET_CHECK(xla::ShapeUtil::Compatible(input_shape, output_shape));
+      if (input_shape != output_shape) {
+        TF_ASSIGN_OR_RETURN(xla::ShapeTree<xla::XlaOp> disassembled_tuple,
+                            xla::DisassembleTuple(output));
+        disassembled_tuple.ForEachMutableElement(
+            [&](const xla::ShapeIndex& index, xla::XlaOp* element) {
+              const xla::Shape& output_subshape =
+                  xla::ShapeUtil::GetSubshape(output_shape, index);
+              if (output_subshape.IsArray()) {
+                const xla::Shape& input_subshape =
+                    xla::ShapeUtil::GetSubshape(input_shape, index);
+                for (int d = 0; d < output_subshape.dimensions().size(); ++d) {
+                  if (input_subshape.is_dynamic_dimension(d) &&
+                      !output_subshape.is_dynamic_dimension(d)) {
+                    *element = xla::SetDimensionSize(
+                        *element,
+                        xla::ConstantR0(
+                            cb.get(),
+                            static_cast<int32_t>(output_shape.dimensions()[d])),
+                        d);
+                  }
+                }
+              }
+            });
+        output =
+            xla::AssembleTuple(output.builder(), std::move(disassembled_tuple));
+      }
+      non_compile_time_const_outputs.push_back(output);
+      ++input_num;
     }
   }
   // If `body` has a token output, append it to
@@ -529,7 +576,7 @@ void XlaWhileOp::Compile(XlaOpKernelContext* ctx) {
       if (input_shape != list_shape) {
         // Prepare dynamic dimensions for element shapes.
         std::vector<std::vector<xla::XlaOp>> list_dynamic_dims;
-        for (int i = 0; i < list_shape.tuple_shapes_size() - 1; ++i) {
+        for (int i = 0; i < list_shape.tuple_shapes().size() - 1; ++i) {
           std::vector<xla::XlaOp> dynamic_dims;
 
           const xla::Shape& shape = list_shape.tuple_shapes(i);
@@ -549,7 +596,7 @@ void XlaWhileOp::Compile(XlaOpKernelContext* ctx) {
           // Set dynamic dimension size to 0 for element value. Inside the while
           // loop, TensorlistSetItem will properly set the element shape's
           // dynamic dimension.
-          for (int64_t dim = 1; dim < shape.dimensions_size(); ++dim) {
+          for (int64_t dim = 1; dim < shape.dimensions().size(); ++dim) {
             int32_t dim_size = shape.dimensions(dim);
             if (shape.is_dynamic_dimension(dim)) {
               dim_size = 0;
@@ -575,12 +622,12 @@ void XlaWhileOp::Compile(XlaOpKernelContext* ctx) {
   VLOG(1) << "Building while loop";
 
   // Wraps the condition in a computation that unpacks the output tuple.
-  StatusOr<xla::XlaComputation> cond_result = BuildWrappedCond(ctx, cond);
+  absl::StatusOr<xla::XlaComputation> cond_result = BuildWrappedCond(ctx, cond);
   OP_REQUIRES_OK(ctx, cond_result.status());
   xla::XlaComputation wrapped_cond = std::move(cond_result.value());
 
   // Remove compile time const args from the list of body outputs.
-  StatusOr<xla::XlaComputation> body_result =
+  absl::StatusOr<xla::XlaComputation> body_result =
       BuildWrappedBody(ctx, *body.get(), compile_time_const_arg_indices,
                        num_compile_time_const_args, has_token_input_output_);
   OP_REQUIRES_OK(ctx, body_result.status());
