@@ -19,11 +19,17 @@ limitations under the License.
 #include <utility>
 #include <vector>
 
+#include "absl/log/log.h"
 #include "absl/status/status.h"
 #include "absl/strings/str_cat.h"
 #include "absl/strings/str_join.h"
+#include "absl/strings/string_view.h"
 #include "llvm/ADT/ArrayRef.h"
+#include "llvm/ADT/DenseSet.h"
+#include "llvm/ADT/STLExtras.h"
 #include "llvm/ADT/SmallVector.h"
+#include "llvm/Support/Casting.h"
+#include "llvm/Support/LogicalResult.h"
 #include "mlir/Dialect/Func/Extensions/AllExtensions.h"  // from @llvm-project
 #include "mlir/Dialect/Func/IR/FuncOps.h"  // from @llvm-project
 #include "mlir/IR/Attributes.h"  // from @llvm-project
@@ -36,8 +42,12 @@ limitations under the License.
 #include "mlir/IR/SymbolTable.h"  // from @llvm-project
 #include "mlir/IR/TypeRange.h"  // from @llvm-project
 #include "mlir/IR/TypeUtilities.h"  // from @llvm-project
+#include "mlir/IR/ValueRange.h"  // from @llvm-project
+#include "mlir/IR/Visitors.h"  // from @llvm-project
 #include "mlir/Pass/PassManager.h"  // from @llvm-project
 #include "mlir/Support/DebugStringHelper.h"  // from @llvm-project
+#include "mlir/Support/LLVM.h"  // from @llvm-project
+#include "stablehlo/dialect/StablehloOps.h"  // from @stablehlo
 #include "tensorflow/compiler/mlir/tensorflow/utils/convert_type.h"
 #include "tensorflow/compiler/mlir/tensorflow/utils/dump_mlir_util.h"
 #include "tensorflow/compiler/tf2xla/kernels/xla_call_module_loader.h"
@@ -45,20 +55,24 @@ limitations under the License.
 #include "tensorflow/compiler/tf2xla/xla_compiler.h"
 #include "tensorflow/compiler/tf2xla/xla_op_kernel.h"
 #include "tensorflow/compiler/tf2xla/xla_op_registry.h"
-#include "tensorflow/compiler/xla/client/xla_builder.h"
-#include "tensorflow/compiler/xla/client/xla_computation.h"
-#include "tensorflow/compiler/xla/mlir_hlo/mhlo/IR/hlo_ops.h"
-#include "tensorflow/compiler/xla/mlir_hlo/mhlo/transforms/passes.h"
-#include "tensorflow/compiler/xla/shape_util.h"
-#include "tensorflow/compiler/xla/translate/hlo_to_mhlo/hlo_to_mlir_hlo.h"
-#include "tensorflow/compiler/xla/translate/mhlo_to_hlo/type_to_shape.h"
-#include "tensorflow/compiler/xla/util.h"
+#include "xla/debug_options_flags.h"
+#include "xla/hlo/builder/xla_builder.h"
+#include "xla/hlo/builder/xla_computation.h"
+#include "xla/hlo/ir/hlo_instruction.h"
+#include "xla/hlo/translate/mhlo_to_hlo/type_to_shape.h"
+#include "xla/hlo/translate/stablehlo.h"
+#include "xla/service/hlo_module_config.h"
+#include "xla/shape.h"
+#include "xla/shape_util.h"
+#include "xla/tsl/platform/errors.h"
+#include "xla/tsl/platform/statusor.h"
+#include "xla/util.h"
 #include "tensorflow/core/framework/attr_value.pb.h"
 #include "tensorflow/core/framework/op_kernel.h"
 #include "tensorflow/core/framework/op_requires.h"
+#include "tensorflow/core/framework/tensor_shape.h"
+#include "tensorflow/core/platform/types.h"
 #include "tensorflow/core/tpu/tpu_defs.h"
-#include "tensorflow/tsl/platform/errors.h"
-#include "tensorflow/tsl/platform/statusor.h"
 
 namespace tensorflow {
 namespace {
@@ -73,13 +87,9 @@ absl::StatusOr<mlir::func::FuncOp> ImportXlaComputation(
   mlir::SymbolTable &symbol_table =
       symbol_table_collection.getSymbolTable(module);
 
-  mlir::OwningOpRef<mlir::ModuleOp> imported =
-      mlir::ModuleOp::create(mlir::UnknownLoc::get(context));
-  context->loadDialect<mlir::func::FuncDialect>();
-  context->loadDialect<mlir::mhlo::MhloDialect>();
-  TF_RETURN_IF_ERROR(
-      xla::ConvertHloToMlirHlo(*imported, &computation.proto(),
-                               /*import_all_computations=*/true));
+  TF_ASSIGN_OR_RETURN(
+      mlir::OwningOpRef<mlir::ModuleOp> imported,
+      xla::ConvertHloToStablehlo(*context, &computation.proto()));
   if (VLOG_IS_ON(5)) {
     DumpMlirOpToFile("xla_call_module.imported_tf_func", *imported);
   }
@@ -137,6 +147,9 @@ class XlaCallModuleOp : public XlaOpKernel {
     OP_REQUIRES_OK(ctx, ctx->GetAttr("Tout", &expected_output_dtypes));
     std::vector<string> dim_args_spec;
     OP_REQUIRES_OK(ctx, ctx->GetAttr("dim_args_spec", &dim_args_spec));
+    OP_REQUIRES(ctx, dim_args_spec.empty(),
+                absl::UnimplementedError(
+                    "dim_args_spec attribute is no longer supported"));
     OP_REQUIRES(ctx,
                 expected_output_shapes.size() == expected_output_dtypes.size(),
                 absl::InvalidArgumentError(absl::StrCat(
@@ -147,52 +160,76 @@ class XlaCallModuleOp : public XlaOpKernel {
     OP_REQUIRES_OK(ctx, ctx->GetAttr("disabled_checks", &disabled_checks));
     std::vector<string> platforms;
     OP_REQUIRES_OK(ctx, ctx->GetAttr("platforms", &platforms));
-
-    string loading_device_type = ctx->device_type().type_string();
-    string loading_platform = "";
-    if (loading_device_type == DEVICE_CPU_XLA_JIT) {
-      loading_platform = "CPU";
-    } else if (loading_device_type == DEVICE_GPU_XLA_JIT) {
-#if GOOGLE_CUDA
-      loading_platform = "CUDA";
-#elif TENSORFLOW_USE_ROCM
-      loading_platform = "ROCM";
-#else
-      OP_REQUIRES(ctx, false,
-                  absl::UnimplementedError("CUDA or ROCM build required"));
-#endif
-    } else if (loading_device_type == DEVICE_TPU_XLA_JIT) {
-      loading_platform = "TPU";
-    } else {
-      OP_REQUIRES(ctx, false,
-                  absl::UnimplementedError(absl::StrCat(
-                      "Unexpected device type ", loading_device_type)));
-    }
-    VLOG(3) << "Initialized XlaCallModuleOp on " << loading_platform;
-    if (!ctx->GetAttr("has_token_input_output", &module_has_token_input_output_)
+    // TODO(necula): change this to OP_REQUIRES_OK when 6 months have passed
+    // since we added the function_list and has_token_input_output
+    // attributes (May 25, 2023).
+    bool main_has_token_input_output = false;
+    if (!ctx->GetAttr("has_token_input_output", &main_has_token_input_output)
              .ok()) {
-      module_has_token_input_output_ = false;
+      // Whether the StableHLO module's main function has token input/output as
+      // the first argument and the first result.
+      // This is used only prior to version 9; afterwards, we just look for
+      // tokens among the types of the arguments and results, and we support
+      // multiple tokens, not necessarily at the start.
+      main_has_token_input_output = false;
     }
-    {
-      auto loader = XlaCallModuleLoader::Create(
-          &context_, version, std::move(module_str), std::move(dim_args_spec),
-          std::move(disabled_checks), std::move(platforms), loading_platform,
-          /*num_invocation_args=*/ctx->num_inputs(),
-          module_has_token_input_output_);
-      OP_REQUIRES_OK(ctx, loader.status());
-      loader_ = *std::move(loader);
-    }
-    OP_REQUIRES_OK(ctx, loader_->ValidateDialect());
-
     if (!ctx->GetAttr("function_list", &function_list_).ok()) {
       function_list_.clear();
     }
 
-    if (!ctx->GetAttr(kXlaTokenInputNodesAttrName, &token_input_nodes_).ok()) {
-      token_input_nodes_.clear();
-      op_has_token_input_output_ = false;
+    bool use_shardy_partitioner = false;
+    if (!ctx->GetAttr("use_shardy_partitioner", &use_shardy_partitioner).ok()) {
+      use_shardy_partitioner = false;
+    }
+    if (VLOG_IS_ON(3)) {
+      VLOG(3) << "Initializing XlaCallModuleOp (version = " << version
+              << ", platforms = [" << absl::StrJoin(platforms, ", ")
+              << "], has_token_input_output = " << main_has_token_input_output
+              << ", use_shardy_partitioner = " << use_shardy_partitioner
+              << ", disabled_checks = [" << absl::StrJoin(disabled_checks, ", ")
+              << "], "
+              << "function_list = ["
+              << absl::StrJoin(function_list_, ",",
+                               [](std::string *out, NameAttrList x) {
+                                 absl::StrAppend(out, x.name());
+                               })
+              << "])";
+    }
+    string compilation_device_type = ctx->device_type().type_string();
+    compilation_platform_ = "";
+    if (compilation_device_type == DEVICE_CPU_XLA_JIT) {
+      compilation_platform_ = "CPU";
+    } else if (compilation_device_type == DEVICE_GPU_XLA_JIT) {
+#if GOOGLE_CUDA
+      compilation_platform_ = "CUDA";
+#elif TENSORFLOW_USE_ROCM
+      compilation_platform_ = "ROCM";
+#else
+      OP_REQUIRES(ctx, false,
+                  absl::UnimplementedError("CUDA or ROCM build required"));
+#endif
+    } else if (compilation_device_type == DEVICE_TPU_XLA_JIT) {
+      compilation_platform_ = "TPU";
     } else {
-      op_has_token_input_output_ = !token_input_nodes_.empty();
+      OP_REQUIRES(ctx, false,
+                  absl::UnimplementedError(absl::StrCat(
+                      "Unexpected device type ", compilation_device_type)));
+    }
+    VLOG(3) << "Initializing XlaCallModuleOp on " << compilation_platform_;
+    {
+      auto loader = XlaCallModuleLoader::Create(
+          &context_, version, module_str, std::move(disabled_checks),
+          std::move(platforms),
+          /*num_invocation_args=*/ctx->num_inputs(),
+          main_has_token_input_output, use_shardy_partitioner);
+      OP_REQUIRES_OK(ctx, loader.status());
+      loader_ = *std::move(loader);
+    }
+    OP_REQUIRES_OK(ctx, loader_->ValidateXlaCallModuleInvariants());
+
+    if (!ctx->GetAttr(kXlaTokenInputNodesAttrName, &op_token_input_nodes_)
+             .ok()) {
+      op_token_input_nodes_.clear();
     }
     if (!ctx->GetAttr(kXlaOriginalOutsideCompilationNodeName,
                       &original_node_name_)
@@ -210,42 +247,49 @@ class XlaCallModuleOp : public XlaOpKernel {
     xla::XlaBuilder *const b = ctx->builder();
 
     std::vector<xla::Shape> input_shapes;
-    if (module_has_token_input_output_) {
-      input_shapes.push_back(xla::ShapeUtil::MakeTokenShape());
-    }
     for (int i = 0; i < ctx->num_inputs(); ++i) {
       auto shape = ctx->InputXlaShape(i);
       OP_REQUIRES_OK(ctx, shape.status());
       input_shapes.push_back(*std::move(shape));
     }
+    OP_REQUIRES_OK(ctx, loader_->SetPlatformIndex(compilation_platform_));
     OP_REQUIRES_OK(ctx, loader_->RefineDynamicShapes(input_shapes));
     OP_REQUIRES_OK(ctx, loader_->ValidateStaticShapes());
-    OP_REQUIRES_OK(ctx, loader_->LowerModuleToMhlo());
+    OP_REQUIRES_OK(ctx, loader_->PrepareStablehloForLowering());
     if (!function_list_.empty()) {
       OP_REQUIRES_OK(ctx, LowerTfFunctionCalls(ctx));
     }
 
-    std::vector<xla::XlaOp> inputs;
-    if (module_has_token_input_output_) {
-      // The main function expects a token input at the start.
-      if (!token_input_nodes_.empty()) {
-        std::vector<xla::XlaOp> token_inputs;
-        for (const string &node_name : token_input_nodes_) {
-          auto token = compiler->GetNodeToken(node_name);
-          OP_REQUIRES_OK(ctx, token.status());
-          token_inputs.push_back(token.value());
-        }
-        inputs.push_back(xla::AfterAll(b, token_inputs));
-      } else {
-        // Generate a dummy token if the main function expects a token but the
-        // XlaCallModule doesn't take one.
-        inputs.push_back(xla::CreateToken(b));
+    xla::XlaOp token_input;
+    if (!op_token_input_nodes_.empty()) {
+      std::vector<xla::XlaOp> token_inputs;
+      for (const string &node_name : op_token_input_nodes_) {
+        auto token = compiler->GetNodeToken(node_name);
+        OP_REQUIRES_OK(ctx, token.status());
+        token_inputs.push_back(token.value());
       }
-    }
-    for (int i = 0, end = ctx->num_inputs(); i < end; ++i) {
-      inputs.push_back(ctx->Input(i));
+      token_input = xla::AfterAll(b, token_inputs);
     }
 
+    // Set input and output types after removing platform args, but before
+    // lowering the XLA computation since that will mutate the module in place
+    // to use MHLO types.
+    auto input_types = loader_->InputTypes();
+    auto output_types = loader_->OutputTypes();
+
+    std::vector<xla::XlaOp> inputs;
+    int next_actual_input = 0;
+    for (mlir::Type inputType : input_types) {
+      if (IsTokenType(inputType)) {
+        if (token_input.IsUninitialized()) {
+          // Generate a dummy token if the XlaCallModule doesn't take one.
+          token_input = xla::CreateToken(b);
+        }
+        inputs.push_back(token_input);
+      } else {
+        inputs.push_back(ctx->Input(next_actual_input++));
+      }
+    }
     auto xla_computation = loader_->ToXlaComputation();
     OP_REQUIRES_OK(ctx, xla_computation.status());
 
@@ -263,53 +307,66 @@ class XlaCallModuleOp : public XlaOpKernel {
                                      hlo_module->ToString(options)));
     }
 
-    xla::XlaOp output = xla::Call(b, *xla_computation, inputs);
+    xla::XlaOp computation_output = xla::Call(b, *xla_computation, inputs);
 
     // Check that the resulting computation returns the expected shape
-    OP_REQUIRES_VALUE(xla::Shape found_output_shape, ctx, b->GetShape(output));
+    OP_REQUIRES_VALUE(xla::Shape found_output_shape, ctx,
+                      b->GetShape(computation_output));
     VLOG(3) << "XlaCallModule compiled output shape : "
             << xla::ShapeUtil::HumanString(found_output_shape);
-
-    std::vector<xla::XlaOp> outputs;
-    if (loader_->nr_outputs() == 1) {
-      outputs.push_back(output);
+    std::vector<xla::XlaOp> computation_outputs;
+    if (loader_->NrOutputs() == 1) {
+      computation_outputs.push_back(computation_output);
     } else {
-      for (int i = 0; i < loader_->nr_outputs(); ++i) {
-        outputs.push_back(xla::GetTupleElement(output, i));
+      for (int i = 0; i < loader_->NrOutputs(); ++i) {
+        computation_outputs.push_back(
+            xla::GetTupleElement(computation_output, i));
       }
     }
 
-    xla::XlaOp token_output;
-    if (module_has_token_input_output_) {
-      // The main function returns a token as the first output.
-      token_output = outputs.front();
-      outputs.erase(outputs.begin());
-      auto shape = b->GetShape(token_output);
+    // Collect the token outputs and set the non-token outputs
+    std::vector<xla::XlaOp> token_outputs;
+    int next_actual_output = 0;
+    for (auto it : llvm::enumerate(output_types)) {
+      int i = it.index();
+      mlir::Type output_type = it.value();
+      auto shape = b->GetShape(computation_outputs[i]);
       OP_REQUIRES_OK(ctx, shape.status());
-      OP_REQUIRES(ctx, shape->IsToken(),
-                  absl::FailedPreconditionError(
-                      absl::StrCat("Token output is not token type: ",
-                                   xla::ShapeUtil::HumanString(*shape))));
+      if (IsTokenType(output_type)) {
+        OP_REQUIRES(ctx, shape->IsToken(),
+                    absl::FailedPreconditionError(absl::StrCat(
+                        "Token output at index ", i, " (",
+                        mlir::debugString(output_type), ") is not token type: ",
+                        xla::ShapeUtil::HumanString(*shape))));
+        token_outputs.push_back(computation_outputs[i]);
+      } else {
+        OP_REQUIRES(ctx, !shape->IsToken(),
+                    absl::FailedPreconditionError(absl::StrCat(
+                        "Non-token output at index ", i, " (",
+                        mlir::debugString(output_type), ") is not token type: ",
+                        xla::ShapeUtil::HumanString(*shape))));
+        ctx->SetOutput(next_actual_output++, computation_outputs[i]);
+      }
     }
-    if (op_has_token_input_output_) {
-      if (token_output.IsUninitialized()) {
-        // The main function does not return any token, but the XlaCallModule is
-        // expected to return one. Create a dummy token.
-        token_output = xla::CreateToken(b);
+
+    if (!op_token_input_nodes_.empty()) {
+      xla::XlaOp token_output = token_input;
+      if (!token_outputs.empty()) {
+        token_output = xla::AfterAll(b, token_outputs);
+      } else {
+        if (token_output.IsUninitialized()) {
+          token_output = xla::CreateToken(b);
+        }
       }
       OP_REQUIRES_OK(ctx,
                      compiler->SetNodeToken(original_node_name_, token_output));
     }
-
-    for (int i = 0; i < outputs.size(); ++i) {
-      ctx->SetOutput(i, outputs[i]);
-    }
   }
 
  private:
-  // Lowers `mhlo.CustomCall` ops representing TF function calls into nested XLA
-  // computation. The called TF functions are lowered into MHLO and inserted as
-  // function calls in the main module.
+  // Lowers `stablehlo.CustomCall` ops representing TF function calls into
+  // nested XLA computation. The called TF functions are lowered into StableHLO
+  // and inserted as function calls in the main module.
   //
   // This is implemented here instead of in xla_call_module_loader.cc in order
   // to prevent cyclic dependency with TF MLIR passes.
@@ -319,7 +376,8 @@ class XlaCallModuleOp : public XlaOpKernel {
 
     llvm::SmallDenseSet<mlir::func::FuncOp> updated_funcs;
 
-    auto lower = [&](mlir::mhlo::CustomCallOp custom_call) -> absl::Status {
+    auto lower =
+        [&](mlir::stablehlo::CustomCallOp custom_call) -> absl::Status {
       if (custom_call.getCallTargetName() != "tf.call_tf_function") {
         return absl::OkStatus();
       }
@@ -361,13 +419,11 @@ class XlaCallModuleOp : public XlaOpKernel {
       }
 
       // Lower the called TF function into an HLO module.
-
       std::vector<XlaCompiler::Argument> arguments;
       {
         mlir::TypeRange input_types(custom_call->getOperandTypes());
         if (custom_call_has_token_input_output) {
-          if (input_types.empty() ||
-              !input_types.front().isa<mlir::mhlo::TokenType>()) {
+          if (input_types.empty() || !IsTokenType(input_types.front())) {
             return absl::InvalidArgumentError(absl::StrCat(
                 "stablehlo.custom_call with has_token_input_output = true is "
                 "expected to take !stablehlo.token as the first argument, but "
@@ -385,8 +441,7 @@ class XlaCallModuleOp : public XlaOpKernel {
 
         mlir::TypeRange result_types(custom_call->getResultTypes());
         if (custom_call_has_token_input_output) {
-          if (result_types.empty() ||
-              !result_types.front().isa<mlir::mhlo::TokenType>()) {
+          if (result_types.empty() || !IsTokenType(result_types.front())) {
             return absl::InvalidArgumentError(absl::StrCat(
                 "stablehlo.custom_call with has_token_input_output = true is "
                 "expected to return !stablehlo.token as the first result, but "
@@ -396,19 +451,22 @@ class XlaCallModuleOp : public XlaOpKernel {
         }
       }
 
+      // There are times when TPU prefers tuple arguments, i.e. when there are
+      // 2k+ input arguments, but for the most part we want to avoid tuples.
       XlaCompiler::CompileOptions options;
-      options.use_tuple_arg = true;
-      options.always_return_tuple = true;
+      options.use_tuple_arg = false;
+      options.always_return_tuple = false;
       options.is_entry_computation = false;
       // Propagate tokens from XlaCallModule to inner computation.
-      options.add_token_input_output = op_has_token_input_output_;
+      options.add_token_input_output = !op_token_input_nodes_.empty();
 
       XlaCompiler::CompilationResult result;
       TF_RETURN_IF_ERROR(
           ctx->compiler()->CompileFunction(options, f, arguments, &result));
 
-      // Import the lowered HLO module into StableHLO functions in `module`. The
-      // main function accepts tupled arguments and returns tupled results.
+      // Import the lowered HLO module into StableHLO functions in `module`.
+      // The main function accepts variadic arguments and returns variadic
+      // results.
       TF_ASSIGN_OR_RETURN(mlir::func::FuncOp main_func,
                           ImportXlaComputation(symbol_table_collection, module,
                                                *result.computation));
@@ -417,9 +475,8 @@ class XlaCallModuleOp : public XlaOpKernel {
       mlir::OpBuilder builder(custom_call);
       auto loc = custom_call.getLoc();
 
-      // Pack all arguments into a tuple (`options.use_tuple_arg` is true). If
-      // `has_tuple_input_output` is true, the first argument is a token type.
-      mlir::Value arg_tuple;
+      // If `has_tuple_input_output` is true, the first argument is a token type
+      llvm::SmallVector<mlir::Value> input_args;
       {
         llvm::SmallVector<mlir::Value> args(custom_call->getOperands());
         if (custom_call_has_token_input_output) {
@@ -430,21 +487,18 @@ class XlaCallModuleOp : public XlaOpKernel {
         } else if (options.add_token_input_output) {
           // Add a dummy token if the inner computation takes a token but the
           // custom call doesn't have a token argument.
-          args.push_back(builder.create<mlir::mhlo::CreateTokenOp>(loc));
+          args.push_back(builder.create<mlir::stablehlo::CreateTokenOp>(loc));
         }
 
-        llvm::SmallVector<mlir::Value> elements;
-        elements.reserve(result.input_mapping.size());
+        input_args.reserve(result.input_mapping.size());
         for (int index : result.input_mapping) {
-          elements.push_back(args[index]);
+          input_args.push_back(args[index]);
         }
-        arg_tuple =
-            builder.create<mlir::mhlo::TupleOp>(loc, elements).getResult();
       }
 
       // Call the lowered function.
-      auto call = builder.create<mlir::func::CallOp>(
-          loc, main_func, mlir::ValueRange(arg_tuple));
+      auto call =
+          builder.create<mlir::func::CallOp>(loc, main_func, input_args);
 
       // Unpack the result tuple (`options.always_return_tuple` is true). If
       // `has_tuple_input_output` is true, the first result is a token type.
@@ -462,7 +516,7 @@ class XlaCallModuleOp : public XlaOpKernel {
             mlir::Value token = results.back();
             if (!token.use_empty()) {
               token.replaceAllUsesWith(
-                  builder.create<mlir::mhlo::CreateTokenOp>(loc));
+                  builder.create<mlir::stablehlo::CreateTokenOp>(loc));
             }
             results.pop_back();
           }
@@ -470,10 +524,7 @@ class XlaCallModuleOp : public XlaOpKernel {
 
         for (const auto &it : llvm::enumerate(results)) {
           if (!it.value().use_empty()) {
-            auto get_tuple_element =
-                builder.create<mlir::mhlo::GetTupleElementOp>(
-                    loc, call.getResults().front(), it.index());
-            it.value().replaceAllUsesWith(get_tuple_element.getResult());
+            it.value().replaceAllUsesWith(call.getResult(it.index()));
           }
         }
       }
@@ -485,13 +536,14 @@ class XlaCallModuleOp : public XlaOpKernel {
     };
 
     absl::Status status;
-    mlir::WalkResult result = module->walk([&](mlir::mhlo::CustomCallOp op) {
-      status.Update(lower(op));
-      if (!status.ok()) {
-        return mlir::WalkResult::interrupt();
-      }
-      return mlir::WalkResult::advance();
-    });
+    mlir::WalkResult result =
+        module->walk([&](mlir::stablehlo::CustomCallOp op) {
+          status.Update(lower(op));
+          if (!status.ok()) {
+            return mlir::WalkResult::interrupt();
+          }
+          return mlir::WalkResult::advance();
+        });
     if (result.wasInterrupted()) {
       return status;
     }
@@ -514,12 +566,10 @@ class XlaCallModuleOp : public XlaOpKernel {
   mlir::MLIRContext context_{mlir::MLIRContext::Threading::DISABLED};
   std::unique_ptr<XlaCallModuleLoader> loader_;
   std::vector<NameAttrList> function_list_;
+  std::string compilation_platform_;
 
-  // Whether the StableHLO module's main function has token input/output.
-  bool module_has_token_input_output_;
   // Whether the XlaCallModule op has token input/output.
-  bool op_has_token_input_output_;
-  std::vector<std::string> token_input_nodes_;
+  std::vector<std::string> op_token_input_nodes_;
   std::string original_node_name_;
 };
 
