@@ -15,41 +15,55 @@ limitations under the License.
 
 #include "tensorflow/compiler/tf2xla/literal_util.h"
 
+#include "absl/status/status.h"
+#include "absl/status/statusor.h"
+#include "absl/types/span.h"
 #include "tensorflow/compiler/tf2xla/shape_util.h"
 #include "tensorflow/compiler/tf2xla/type_util.h"
-#include "tensorflow/compiler/xla/literal.h"
+#include "xla/literal.h"
+#include "xla/shape.h"
+#include "xla/shape_util.h"
+#include "xla/status_macros.h"
+#include "xla/xla_data.pb.h"
 #include "tensorflow/core/common_runtime/dma_helper.h"
+#include "tensorflow/core/framework/tensor.h"
+#include "tensorflow/core/framework/tensor_shape.h"
+#include "tensorflow/core/framework/types.h"
+#include "tensorflow/core/framework/types.pb.h"
+#include "tensorflow/core/platform/errors.h"
+#include "tensorflow/core/platform/status.h"
+#include "tsl/platform/errors.h"
 
 namespace tensorflow {
 
-Status HostTensorToBorrowingLiteral(const Tensor& host_tensor,
-                                    xla::BorrowingLiteral* literal) {
+absl::Status HostTensorToBorrowingLiteral(const Tensor& host_tensor,
+                                          xla::BorrowingLiteral* literal) {
   xla::Shape xla_shape;
   TF_RETURN_IF_ERROR(TensorShapeToXLAShape(host_tensor.dtype(),
                                            host_tensor.shape(), &xla_shape));
   return HostTensorToBorrowingLiteral(xla_shape, host_tensor, literal);
 }
 
-Status HostTensorToBorrowingLiteral(const xla::Shape& xla_shape,
-                                    const Tensor& host_tensor,
-                                    xla::BorrowingLiteral* literal) {
+absl::Status HostTensorToBorrowingLiteral(const xla::Shape& xla_shape,
+                                          const Tensor& host_tensor,
+                                          xla::BorrowingLiteral* literal) {
   const auto& tshape = host_tensor.shape();
   TF_RET_CHECK(tshape.IsFullyDefined() &&
-               tshape.dims() == xla_shape.dimensions_size() &&
+               tshape.dims() == xla_shape.dimensions().size() &&
                tshape.dim_sizes() == xla_shape.dimensions())
       << "Provided xla::Shape must have the same dims as the Tensor shape.";
   *literal = xla::BorrowingLiteral(
       static_cast<const char*>(DMAHelper::base(&host_tensor)), xla_shape);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-StatusOr<xla::Literal> HostTensorToLiteral(const Tensor& host_tensor) {
+absl::StatusOr<xla::Literal> HostTensorToLiteral(const Tensor& host_tensor) {
   xla::BorrowingLiteral literal;
   TF_RETURN_IF_ERROR(HostTensorToBorrowingLiteral(host_tensor, &literal));
   return literal.Clone();
 }
 
-Status HostTensorToMutableBorrowingLiteral(
+absl::Status HostTensorToMutableBorrowingLiteral(
     Tensor* host_tensor, xla::MutableBorrowingLiteral* literal) {
   xla::Shape xla_shape;
   TF_RETURN_IF_ERROR(TensorShapeToXLAShape(host_tensor->dtype(),
@@ -57,17 +71,17 @@ Status HostTensorToMutableBorrowingLiteral(
   return HostTensorToMutableBorrowingLiteral(xla_shape, host_tensor, literal);
 }
 
-Status HostTensorToMutableBorrowingLiteral(
+absl::Status HostTensorToMutableBorrowingLiteral(
     const xla::Shape& xla_shape, Tensor* host_tensor,
     xla::MutableBorrowingLiteral* literal) {
   *literal = xla::MutableBorrowingLiteral(
       static_cast<const char*>(DMAHelper::base(host_tensor)), xla_shape);
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status HostTensorsToBorrowingLiteralTuple(absl::Span<const Tensor> host_tensors,
-                                          xla::BorrowingLiteral* literal) {
+absl::Status HostTensorsToBorrowingLiteralTuple(
+    absl::Span<const Tensor> host_tensors, xla::BorrowingLiteral* literal) {
   std::vector<const char*> buf_ptrs;
   buf_ptrs.reserve(host_tensors.size());
   std::vector<xla::Shape> tensor_shapes(host_tensors.size());
@@ -83,11 +97,11 @@ Status HostTensorsToBorrowingLiteralTuple(absl::Span<const Tensor> host_tensors,
   *literal = xla::BorrowingLiteral(
       buf_ptrs, xla::ShapeUtil::MakeTupleShape(tensor_shapes));
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status CopyLiteralToHostTensor(const xla::LiteralSlice& literal,
-                               Tensor* host_tensor) {
+absl::Status CopyLiteralToHostTensor(const xla::LiteralSlice& literal,
+                                     Tensor* host_tensor) {
   TF_RET_CHECK(literal.shape().IsArray() &&
                xla::ShapeUtil::ElementsIn(literal.shape()) ==
                    host_tensor->NumElements());
@@ -106,11 +120,11 @@ Status CopyLiteralToHostTensor(const xla::LiteralSlice& literal,
     void* dst_ptr = DMAHelper::base(host_tensor);
     memcpy(dst_ptr, src_ptr, total_bytes);
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status LiteralToHostTensor(const xla::LiteralSlice& literal,
-                           DataType target_type, Tensor* host_tensor) {
+absl::Status LiteralToHostTensor(const xla::LiteralSlice& literal,
+                                 DataType target_type, Tensor* host_tensor) {
   TensorShape shape;
   TF_RETURN_IF_ERROR(XLAShapeToTensorShape(literal.shape(), &shape));
   *host_tensor = Tensor(target_type, shape);
