@@ -25,7 +25,7 @@ operators. Please reach out to the JAX team if you want to make changes.
 """
 
 from tensorflow.compiler.tf2xla.ops import gen_xla_ops
-from tensorflow.compiler.xla import xla_data_pb2
+from local_xla.xla import xla_data_pb2
 from tensorflow.core.framework import attr_value_pb2
 from tensorflow.python.framework import constant_op
 from tensorflow.python.framework import dtypes
@@ -637,6 +637,7 @@ def call_module(
     function_list=(),
     has_token_input_output=False,
     disabled_checks=(),
+    use_shardy_partitioner=False,
 ):
   """See documentation for the XlaCallModule op.
 
@@ -653,6 +654,7 @@ def call_module(
       function_list=function_list,
       has_token_input_output=has_token_input_output,
       disabled_checks=disabled_checks,
+      use_shardy_partitioner=use_shardy_partitioner,
   )
   # Since XLACallModule op is stateful, zero return function will return the TF
   # op under tf.function. It creates trouble for downstream codes.
@@ -667,9 +669,9 @@ def call_module_maximum_supported_version():
   """Maximum version of XlaCallModule op supported.
 
   See versioning details documentation for the XlaCallModule op at:
-  https://github.com/search?q=repo%3Atensorflow%2Ftensorflow+path%3Axla_call_module+%22int+VERSION_MAXIMUM_SUPPORTED%22&type=code
+  https://github.com/search?q=repo%3Atensorflow%2Ftensorflow+path%3Axla_call_module+%22int+kVersionMaximumSupported%22&type=code
   """
-  return 8
+  return 10
 
 # pylint: enable=g-doc-args
 # pylint: enable=g-doc-return-or-yield
