@@ -40,8 +40,8 @@ limitations under the License.
 namespace tensorflow {
 namespace {
 
-void ExpectErrorContains(const Status& status, absl::string_view str) {
-  EXPECT_NE(OkStatus(), status);
+void ExpectErrorContains(const absl::Status& status, absl::string_view str) {
+  EXPECT_NE(absl::OkStatus(), status);
   EXPECT_TRUE(absl::StrContains(status.message(), str))
       << "expected error: " << status.message() << " to contain: " << str;
 }
@@ -337,7 +337,7 @@ TEST(CachedFunctionHandles, Basic) {
 }
 
 TEST(PropagateConstIntoFunctionalNodes, WhileLoopWithResourceInput) {
-  FunctionLibraryDefinition fld(OpRegistry::Global(), {});
+  FunctionLibraryDefinition fld(OpRegistry::Global(), FunctionDefLibrary());
   {
     // Cond graph & body graph.
     Scope scope = Scope::NewRootScope().ExitOnError();
@@ -369,7 +369,7 @@ TEST(PropagateConstIntoFunctionalNodes, WhileLoopWithResourceInput) {
 }
 
 TEST(PropagateConstIntoFunctionalNodes, CopiedConstNodeHasUniqueName) {
-  FunctionLibraryDefinition fld(OpRegistry::Global(), {});
+  FunctionLibraryDefinition fld(OpRegistry::Global(), FunctionDefLibrary());
   {
     // Cond graph & body graph.
     Scope scope = Scope::NewRootScope().ExitOnError();
@@ -422,7 +422,7 @@ TEST(PropagateConstIntoFunctionalNodes, CopiedConstNodeHasUniqueName) {
 }
 
 TEST(PropagateConstIntoFunctionalNodes, RewriteTensorListWithConstMember) {
-  FunctionLibraryDefinition fld(OpRegistry::Global(), {});
+  FunctionLibraryDefinition fld(OpRegistry::Global(), FunctionDefLibrary());
   {
     // Cond graph
     Scope scope = Scope::NewRootScope().ExitOnError();
