@@ -18,11 +18,22 @@ limitations under the License.
 #include <functional>
 #include <memory>
 
-#include "absl/memory/memory.h"
-#include "tensorflow/compiler/tf2xla/shape_util.h"
-#include "tensorflow/compiler/tf2xla/sharding_util.h"
+#include "absl/log/check.h"
+#include "absl/log/log.h"
+#include "absl/status/status.h"
+#include "absl/strings/str_cat.h"
+#include "absl/strings/string_view.h"
 #include "tensorflow/compiler/tf2xla/xla_helpers.h"
-#include "tensorflow/compiler/xla/client/xla_builder.h"
+#include "xla/hlo/builder/xla_builder.h"
+#include "xla/status_macros.h"
+#include "tensorflow/core/framework/tensor_shape.h"
+#include "tensorflow/core/framework/types.h"
+#include "tensorflow/core/framework/types.pb.h"
+#include "tensorflow/core/platform/errors.h"
+#include "tensorflow/core/platform/status.h"
+#include "tensorflow/core/platform/types.h"
+#include "tensorflow/core/util/managed_stack_trace.h"
+#include "tsl/platform/errors.h"
 
 namespace tensorflow {
 
@@ -87,7 +98,8 @@ XlaResource::XlaResource(
   }
 }
 
-Status XlaResource::SetTypeAndShape(DataType type, const TensorShape& shape) {
+absl::Status XlaResource::SetTypeAndShape(DataType type,
+                                          const TensorShape& shape) {
   if (type == DT_INVALID) {
     return errors::InvalidArgument(
         "Attempted to set type of resource '", name_, "'' to an invalid type",
@@ -109,10 +121,10 @@ Status XlaResource::SetTypeAndShape(DataType type, const TensorShape& shape) {
   }
   type_ = type;
   shape_ = shape;
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status XlaResource::SetValue(const xla::XlaOp& value) {
+absl::Status XlaResource::SetValue(const xla::XlaOp value) {
   if (type_ == DT_INVALID) {
     return errors::InvalidArgument(
         "Resource '", name_,
@@ -120,10 +132,10 @@ Status XlaResource::SetValue(const xla::XlaOp& value) {
   }
   value_ = value;
   is_overwritten_ = true;
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status XlaResource::SetZeroValue(xla::XlaBuilder* builder) {
+absl::Status XlaResource::SetZeroValue(xla::XlaBuilder* builder) {
   is_overwritten_ = true;
   if (type_ == DT_INVALID) {
     return errors::InvalidArgument(
@@ -159,12 +171,12 @@ Status XlaResource::SetZeroValue(xla::XlaBuilder* builder) {
     default:
       LOG(FATAL) << "Invalid resource type";
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status XlaResource::GetOrCreateTensorArrayGradient(const string& source,
-                                                   xla::XlaBuilder* builder,
-                                                   XlaResource** gradient_out) {
+absl::Status XlaResource::GetOrCreateTensorArrayGradient(
+    const string& source, xla::XlaBuilder* builder,
+    XlaResource** gradient_out) {
   VLOG(2) << "Gradient lookup for resource: " << name_
           << " gradient: " << source;
   TF_RET_CHECK(kind_ == kTensorArray);
@@ -183,10 +195,11 @@ Status XlaResource::GetOrCreateTensorArrayGradient(const string& source,
                         /*tensor_array_multiple_writes_aggregate=*/true));
   }
   *gradient_out = gradient.get();
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status XlaResource::Pack(xla::XlaOp* pack, xla::XlaBuilder* builder) const {
+absl::Status XlaResource::Pack(xla::XlaOp* pack,
+                               xla::XlaBuilder* builder) const {
   if (tensor_array_gradients_.empty()) {
     *pack = value_;
   } else {
@@ -198,12 +211,12 @@ Status XlaResource::Pack(xla::XlaOp* pack, xla::XlaBuilder* builder) const {
     }
     *pack = xla::Tuple(builder, elems);
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status XlaResource::SetFromPack(const std::set<string>& gradient_sources,
-                                const xla::XlaOp& pack,
-                                xla::XlaBuilder* builder) {
+absl::Status XlaResource::SetFromPack(const std::set<string>& gradient_sources,
+                                      const xla::XlaOp pack,
+                                      xla::XlaBuilder* builder) {
   if (gradient_sources.empty()) {
     if (!initialized()) {
       initial_value_ = pack;
@@ -229,7 +242,7 @@ Status XlaResource::SetFromPack(const std::set<string>& gradient_sources,
       gradient->value_ = v;
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace tensorflow
