@@ -35,7 +35,6 @@ limitations under the License.
 #include "tensorflow/core/lib/strings/strcat.h"
 #include "tensorflow/core/platform/macros.h"
 #include "tensorflow/core/platform/refcount.h"
-#include "tensorflow/core/platform/tracing.h"
 #include "tensorflow/core/platform/types.h"
 #include "tensorflow/core/profiler/lib/connected_traceme.h"
 #include "tensorflow/core/profiler/lib/scoped_memory_debug_annotation.h"
@@ -153,7 +152,7 @@ class CollectiveAdapterImpl : public CollectiveAdapter {
 
   Tensor TempChunk(int i) const override {
     AllocationAttributes empty;
-    profiler::ScopedMemoryDebugAnnotation op_annotation(
+    tsl::profiler::ScopedMemoryDebugAnnotation op_annotation(
         "CollectiveAdapterImpl::TempChunk");
     return Tensor(allocator_, dt_, {ChunkElts(i)}, empty);
   }
@@ -229,8 +228,8 @@ CollectiveAdapter* MakeCollectiveAdapter(Tensor* output, int num_chunks,
 
 BaseCollectiveExecutor::~BaseCollectiveExecutor() {}
 
-void BaseCollectiveExecutor::StartAbort(const Status& s) {
-  Status status;
+void BaseCollectiveExecutor::StartAbort(const absl::Status& s) {
+  absl::Status status;
   {
     mutex_lock l(status_mu_);
     if (!status_.ok()) {
@@ -238,7 +237,7 @@ void BaseCollectiveExecutor::StartAbort(const Status& s) {
               << s;
       return;
     }
-    status_ = StatusGroup::MakeDerived(Status(
+    status_ = StatusGroup::MakeDerived(absl::Status(
         s.code(),
         absl::StrCat(
             "Collective ops is aborted by: ", s.message(),
@@ -254,7 +253,7 @@ void BaseCollectiveExecutor::StartAbort(const Status& s) {
   }
 }
 
-Status BaseCollectiveExecutor::GetStatus(const Status& s) {
+absl::Status BaseCollectiveExecutor::GetStatus(const absl::Status& s) {
   if (s.ok()) return s;
   mutex_lock l(status_mu_);
   // If the collective executor is already aborted, use the aborted status
@@ -275,7 +274,8 @@ void BaseCollectiveExecutor::ExecuteAsync(OpKernelContext* ctx,
                                           StatusCallback done) {
   // See CompleteParamsAsync() how done() and the timeout callback interacts.
   const auto is_callback_called = std::make_shared<std::atomic<bool>>(false);
-  auto done_safe = [this, done, ctx, is_callback_called](const Status& s) {
+  auto done_safe = [this, done, ctx,
+                    is_callback_called](const absl::Status& s) {
     bool called = is_callback_called->exchange(true);
     if (!called) {
       if (!s.ok() && !IsCancelled(ctx->cancellation_manager())) {
@@ -294,8 +294,8 @@ void BaseCollectiveExecutor::ExecuteAsync(OpKernelContext* ctx,
         timeout_microseconds, [this, is_callback_called, done] {
           bool called = is_callback_called->exchange(true);
           if (!called) {
-            Status status(absl::StatusCode::kDeadlineExceeded,
-                          "Collective has timed out during execution.");
+            absl::Status status(absl::StatusCode::kDeadlineExceeded,
+                                "Collective has timed out during execution.");
             StartAbort(status);
             done(status);
           }
@@ -314,7 +314,7 @@ void BaseCollectiveExecutor::ExecuteAsync(OpKernelContext* ctx,
           ? &ctx->input(0)
           : nullptr;
   CollectiveImplementationInterface* col_impl = nullptr;
-  Status status = CreateCollective(*col_params, &col_impl);
+  absl::Status status = CreateCollective(*col_params, &col_impl);
   if (!status.ok()) {
     done_safe(status);
     DCHECK_EQ(nullptr, col_impl);
@@ -332,23 +332,28 @@ void BaseCollectiveExecutor::ExecuteAsync(OpKernelContext* ctx,
   // Run on an unbounded work queue that can handle blocking work so as to not
   // starve executor threads.
   col_impl->Ref();
-  profiler::TraceMeProducer producer("BaseCollectiveExecutor::ExecuteAsync");
+  tsl::profiler::TraceMeProducer producer(
+      "BaseCollectiveExecutor::ExecuteAsync");
   RunClosure([col_impl, col_ctx, done_safe, ctx,
               context_id = producer.GetContextId()]() {
     core::ScopedUnref unref(col_impl);
-    profiler::TraceMeConsumer consumer(
+    tsl::profiler::TraceMeConsumer consumer(
         [ctx, col_ctx] {
-          string op = profiler::TraceMeOp(ctx->op_kernel().name_view(),
-                                          ctx->op_kernel().type_string_view());
-          return profiler::TraceMeEncode(
+          string op =
+              tsl::profiler::TraceMeOp(ctx->op_kernel().name_view(),
+                                       ctx->op_kernel().type_string_view());
+          return tsl::profiler::TraceMeEncode(
               std::move(op),
-              {{"id", ctx->step_id()},
+              {{"step_id", ctx->step_id()},
+               {"iter_id", ctx->frame_iter().iter_id},
+               {"frame_id", ctx->frame_iter().frame_id},
                {"instance_key", col_ctx->col_params->instance.instance_key},
+               {"group_key", col_ctx->col_params->group.group_key},
                {"collective", col_ctx->col_params->instance.type}});
         },
         context_id);
     col_impl->Ref();
-    col_impl->Run([col_impl, col_ctx, done_safe](const Status& s) {
+    col_impl->Run([col_impl, col_ctx, done_safe](const absl::Status& s) {
       core::ScopedUnref unref(col_impl);
       done_safe(s);
     });
@@ -365,15 +370,15 @@ void BaseCollectiveExecutor::CompleteParamsAsync(
   // timeout callback executes, done_safe will become a no-op and the timeout
   // callback is responsible for invoking done() at the end.
   const auto is_callback_called = std::make_shared<std::atomic<bool>>(false);
-  int64_t trace_id = profiler::TraceMe::ActivityStart([cp]() {
-    return profiler::TraceMeEncode("CollectiveExecutor::CompleteParams",
-                                   {{"group_key", cp->group.group_key},
-                                    {"group_size", cp->group.group_size}});
+  int64_t trace_id = tsl::profiler::TraceMe::ActivityStart([cp]() {
+    return tsl::profiler::TraceMeEncode("CollectiveExecutor::CompleteParams",
+                                        {{"group_key", cp->group.group_key},
+                                         {"group_size", cp->group.group_size}});
   });
 
   auto done_safe = [this, is_callback_called, cancel_mgr, trace_id,
-                    done](const Status& s) {
-    profiler::TraceMe::ActivityEnd(trace_id);
+                    done](const absl::Status& s) {
+    tsl::profiler::TraceMe::ActivityEnd(trace_id);
     bool called = is_callback_called->exchange(true);
     if (!called) {
       if (!s.ok() && !IsCancelled(cancel_mgr)) {
@@ -392,7 +397,7 @@ void BaseCollectiveExecutor::CompleteParamsAsync(
         timeout_microseconds, [this, is_callback_called, done]() {
           bool called = is_callback_called->exchange(true);
           if (!called) {
-            Status status(
+            absl::Status status(
                 absl::StatusCode::kDeadlineExceeded,
                 "Collective has timed out waiting for other workers.");
             StartAbort(status);
@@ -404,7 +409,7 @@ void BaseCollectiveExecutor::CompleteParamsAsync(
                                                 done_safe);
 }
 
-Status BaseCollectiveExecutor::CreateCollective(
+absl::Status BaseCollectiveExecutor::CreateCollective(
     const CollectiveParams& col_params,
     CollectiveImplementationInterface** col_impl) {
   VLOG(2) << "CreateCollective type "
