@@ -43,7 +43,7 @@ BufRendezvous::~BufRendezvous() {
   }
 }
 
-void BufRendezvous::StartAbort(const Status& s) {
+void BufRendezvous::StartAbort(const absl::Status& s) {
   CHECK(!s.ok());
   HookTable dummy_table;
   {
@@ -58,7 +58,7 @@ void BufRendezvous::StartAbort(const Status& s) {
   PurgeTable(s, &dummy_table);
 }
 
-void BufRendezvous::PurgeTable(const Status& s, HookTable* table) {
+void BufRendezvous::PurgeTable(const absl::Status& s, HookTable* table) {
   for (auto& it : *table) {
     Hook* h = it.second;
     if (h->cancellation_manager != nullptr) {
@@ -96,7 +96,7 @@ void BufRendezvous::ProvideBuf(const string& key, Device* dev,
   }
 #endif
   Hook* h = nullptr;
-  Status providebuf_status;
+  absl::Status providebuf_status;
   do {
     mutex_lock l(mu_);
     if (!status_.ok()) {
@@ -148,7 +148,7 @@ void BufRendezvous::ProvideBuf(const string& key, Device* dev,
     DVLOG(4) << "ProvideBuf: key = " << key << ": calling cons_cb"
              << h->DebugString();
     DeregisterCancellation(h);
-    h->cons_cb(OkStatus(), h);
+    h->cons_cb(absl::OkStatus(), h);
   }
   if (!providebuf_status.ok()) {
     done(providebuf_status);
@@ -168,7 +168,7 @@ void BufRendezvous::ConsumeBuf(const string& key, const string& device_name,
   // Check the incarnation in the request matches the current device
   // incarnation of the producer.
   Device* device;
-  Status consumebuf_status = dev_mgr_->LookupDevice(device_name, &device);
+  absl::Status consumebuf_status = dev_mgr_->LookupDevice(device_name, &device);
   if (consumebuf_status.ok() &&
       device->attributes().incarnation() != device_incarnation) {
     consumebuf_status = errors::FailedPrecondition(
@@ -224,7 +224,7 @@ void BufRendezvous::ConsumeBuf(const string& key, const string& device_name,
     DVLOG(4) << "ConsumeBuf: key = " << key << ": calling cons_cb"
              << existing_hook->DebugString();
     DeregisterCancellation(existing_hook);
-    existing_hook->cons_cb(OkStatus(), existing_hook);
+    existing_hook->cons_cb(absl::OkStatus(), existing_hook);
     return;
   }
   if (!consumebuf_status.ok()) {
@@ -257,7 +257,7 @@ void BufRendezvous::CancelHook(const string& key) {
 
 /*static*/
 void BufRendezvous::DoneWithHook(Hook* h) {
-  h->prod_cb(OkStatus());
+  h->prod_cb(absl::OkStatus());
   delete h;
 }
 
