@@ -101,7 +101,8 @@ struct RankFormatter {
   }
 };
 
-Status CheckUserSpecifiedRanks(const std::vector<CollGroupMember> members) {
+absl::Status CheckUserSpecifiedRanks(
+    const std::vector<CollGroupMember> members) {
   absl::flat_hash_set<int> user_ranks = {};
   bool at_least_one_member_with_no_rank = false;
   bool at_least_one_member_with_user_rank = false;
@@ -127,7 +128,7 @@ Status CheckUserSpecifiedRanks(const std::vector<CollGroupMember> members) {
         "Duplicate ranks specified for group members. Received ranks: ",
         received_ranks);
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 }  // namespace
 
@@ -139,7 +140,7 @@ void CollectiveParamResolverLocal::CompleteGroupLocal(
   std::vector<StatusCallback> to_be_called;
 
   GroupRec* gr = nullptr;
-  Status status;
+  absl::Status status;
   {
     mutex_lock l(group_mu_);
     auto it = group_table_.find(group_params->group_key);
@@ -182,7 +183,7 @@ void CollectiveParamResolverLocal::CompleteGroupLocal(
       return;
     }
     done = [cancel_mgr, token,
-            original_done = std::move(done)](const Status& status) {
+            original_done = std::move(done)](const absl::Status& status) {
       cancel_mgr->TryDeregisterCallback(token);
       original_done(status);
     };
@@ -338,7 +339,7 @@ bool ParseRingOrder(const string& gpu_ring_order_str, TaskDeviceMap* tdm) {
   for (int32_t rank = 0;
        rank < static_cast<int32>(split_gpu_ring_order_str.size()); ++rank) {
     int32_t tmp;
-    if (strings::safe_strto32(split_gpu_ring_order_str[rank], &tmp)) {
+    if (absl::SimpleAtoi(split_gpu_ring_order_str[rank], &tmp)) {
       gpu_ranks[tmp] = rank;
     } else {
       return false;
@@ -612,7 +613,7 @@ CollectiveParamResolverLocal::GetOrCreateInstanceRec(CollectiveParams* cp,
       instance_table_[cp->group.group_key][key].reset(irec);
     }
   }
-  Status status;
+  absl::Status status;
   {
     mutex_lock l(status_mu_);
     status = status_;
@@ -624,8 +625,8 @@ CollectiveParamResolverLocal::GetOrCreateInstanceRec(CollectiveParams* cp,
   return irec;
 }
 
-Status CollectiveParamResolverLocal::LookupGroup(int32_t group_key,
-                                                 CollGroupParams* group) {
+absl::Status CollectiveParamResolverLocal::LookupGroup(int32_t group_key,
+                                                       CollGroupParams* group) {
   mutex_lock l(group_mu_);
   auto group_rec = group_table_.find(group_key);
   if (group_rec == group_table_.end()) {
@@ -644,7 +645,7 @@ Status CollectiveParamResolverLocal::LookupGroup(int32_t group_key,
         group_rec->second->status.ToString());
   }
   *group = group_rec->second->group;
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 void CollectiveParamResolverLocal::CompleteParamsAsync(
@@ -654,7 +655,7 @@ void CollectiveParamResolverLocal::CompleteParamsAsync(
           << cp->ToString();
   if (cp->run_group_initialization) {
     CompleteGroupLocal(device, &cp->group, cancel_mgr,
-                       [this, device, cp, done](const Status& s) {
+                       [this, device, cp, done](const absl::Status& s) {
                          if (s.ok()) {
                            CompleteInstanceLocal(device.name(), cp, done);
                          } else {
@@ -731,7 +732,7 @@ void CollectiveParamResolverLocal::CompleteInstanceFromInitializedIRec(
     const string& device, CollectiveParams* cp, InstanceRec* ir,
     const StatusCallback& done) {
   auto expected_shape = cp->instance.shape;
-  Status status;
+  absl::Status status;
   // Populate the fields common across instance.
   {
     mutex_lock l(ir->mu);
@@ -771,7 +772,7 @@ void CollectiveParamResolverLocal::CompleteInstanceFromInitializedIRec(
   //  discovery.
   if (cp->instance.type == BROADCAST_COLLECTIVE) {
     WaitForGroup(ir, cp, [col_impl, ir, device, cp, done](InstanceRec* irec) {
-      Status s;
+      absl::Status s;
       if (ir != irec) {
         s = errors::Internal("Expected ir ", ir, " and irec ", irec,
                              " to be equal");
@@ -841,7 +842,7 @@ void CollectiveParamResolverLocal::WaitForGroup(InstanceRec* ir,
   }
 }
 
-void CollectiveParamResolverLocal::StartAbort(const Status& s) {
+void CollectiveParamResolverLocal::StartAbort(const absl::Status& s) {
   {
     mutex_lock l(status_mu_);
     if (!status_.ok()) {
@@ -855,7 +856,7 @@ void CollectiveParamResolverLocal::StartAbort(const Status& s) {
   StartAbortLocal(s);
 }
 
-void CollectiveParamResolverLocal::StartAbortLocal(const Status& s) {
+void CollectiveParamResolverLocal::StartAbortLocal(const absl::Status& s) {
   std::vector<StatusCallback> pending_done;
   {
     mutex_lock l(group_mu_);
