@@ -99,7 +99,9 @@ class FakeDevice : public Device {
       : Device(nullptr, device_attributes) {}
 
  public:
-  Status Sync() override { return errors::Unimplemented("FakeDevice::Sync()"); }
+  absl::Status Sync() override {
+    return errors::Unimplemented("FakeDevice::Sync()");
+  }
 
   Allocator* GetAllocator(AllocatorAttributes attr) override { return nullptr; }
 
@@ -137,7 +139,8 @@ TEST_F(ConstantFoldingTest, Basic) {
 // Tests that different node creation ordering creates same graph after constant
 // folding.
 TEST_F(ConstantFoldingTest, DeterministicFolding) {
-  auto build_graph_and_constant_folding = [](Graph& g, bool swap) -> Status {
+  auto build_graph_and_constant_folding = [](Graph& g,
+                                             bool swap) -> absl::Status {
     Scope s = Scope::NewRootScope();
     auto a = ops::Const<float>(s, {1.0}, {});
     auto b = ops::Const<float>(s, {2.0}, {});
@@ -169,7 +172,7 @@ TEST_F(ConstantFoldingTest, DeterministicFolding) {
     opt.generate_new_name = generate_new_name;
     TF_CHECK_OK(
         ConstantFold(opt, nullptr, Env::Default(), nullptr, &g, &was_mutated));
-    return OkStatus();
+    return absl::OkStatus();
   };
 
   Graph g1(OpRegistry::Global());
@@ -351,7 +354,7 @@ TEST_F(ConstantFoldingTest, TestNoReplaceFunctionCall) {
         NodeDefBuilder("times_two", "XTimesTwo", s.graph()->op_registry())
             .Input(c.name(), 0, DT_INT32)
             .Finalize(&def));
-    Status status;
+    absl::Status status;
     Node* times_two = s.graph()->AddNode(def, &status);
     TF_ASSERT_OK(status);
     TF_ASSERT_OK(s.DoShapeInference(times_two));
@@ -385,7 +388,7 @@ TEST_F(ConstantFoldingTest, TestNoReplaceNonCPUOp) {
     TF_ASSERT_OK(NodeDefBuilder("testop", "ConstantFoldingTestOp")
                      .Input(aconst.name(), 0, DT_INT64)
                      .Finalize(&def));
-    Status status;
+    absl::Status status;
     Node* non_cpu = s.graph()->AddNode(def, &status);
     TF_ASSERT_OK(status);
     TF_ASSERT_OK(s.DoShapeInference(non_cpu));
@@ -681,17 +684,17 @@ class TestTFFileSystem : public ::tensorflow::NullFileSystem {
 
   using ::tensorflow::NullFileSystem::NewReadOnlyMemoryRegionFromFile;
 
-  ::tensorflow::Status NewReadOnlyMemoryRegionFromFile(
+  absl::Status NewReadOnlyMemoryRegionFromFile(
       const string& fname, ::tensorflow::TransactionToken* token,
       std::unique_ptr<::tensorflow::ReadOnlyMemoryRegion>* result) override {
     if (fname != kTestMemRegionName) {
       return ::tensorflow::errors::Unimplemented(
           "NewReadOnlyMemoryRegionFromFile unimplemented");
     }
-    const ::tensorflow::StringPiece sp = data_tensor_.tensor_data();
+    const absl::string_view sp = data_tensor_.tensor_data();
     *result = std::unique_ptr<::tensorflow::ReadOnlyMemoryRegion>(
         new TestReadOnlyMemoryRegion(sp.data(), sp.size()));
-    return OkStatus();
+    return absl::OkStatus();
   }
 
  protected:
@@ -703,12 +706,12 @@ class TestTFEnvironment : public ::tensorflow::EnvWrapper {
  public:
   using tf_base = ::tensorflow::EnvWrapper;
   TestTFEnvironment() : ::tensorflow::EnvWrapper(Default()) {}
-  ::tensorflow::Status GetFileSystemForFile(
+  absl::Status GetFileSystemForFile(
       const string& fname, ::tensorflow::FileSystem** result) override {
     was_used_ = true;
     if (fname == "test://test") {
       *result = &test_filesystem_;
-      return OkStatus();
+      return absl::OkStatus();
     }
     return tf_base::GetFileSystemForFile(fname, result);
   }
@@ -732,8 +735,8 @@ TEST_F(ConstantFoldingTest, TestImmutableConst) {
   TF_ASSERT_OK(root.ToGraph(&g));
   TestTFEnvironment test_env;
   bool was_mutated;
-  Status status = ConstantFold(ConstantFoldingOptions{}, nullptr,
-                               Env::Default(), nullptr, &g, &was_mutated);
+  absl::Status status = ConstantFold(ConstantFoldingOptions{}, nullptr,
+                                     Env::Default(), nullptr, &g, &was_mutated);
   EXPECT_FALSE(was_mutated);
   EXPECT_FALSE(status.ok());
   TF_EXPECT_OK(ConstantFold(ConstantFoldingOptions{}, nullptr, &test_env,
