@@ -19,14 +19,14 @@ limitations under the License.
 
 #include <atomic>
 
-#include "tensorflow/compiler/xla/stream_executor/gpu/gpu_init.h"
+#include "xla/stream_executor/gpu/gpu_init.h"
+#include "xla/tsl/framework/device_id.h"
 #include "tensorflow/core/common_runtime/dma_helper.h"
 #include "tensorflow/core/common_runtime/gpu/gpu_device.h"
 #include "tensorflow/core/common_runtime/gpu/gpu_process_state.h"
 #include "tensorflow/core/framework/fake_input.h"
 #include "tensorflow/core/framework/node_def.pb.h"
 #include "tensorflow/core/framework/node_def_builder.h"
-#include "tensorflow/core/graph/node_builder.h"
 #include "tensorflow/core/lib/core/notification.h"
 #include "tensorflow/core/lib/core/status_test_util.h"
 #include "tensorflow/core/platform/stream_executor.h"
@@ -34,7 +34,6 @@ limitations under the License.
 #include "tensorflow/core/platform/test_benchmark.h"
 #include "tensorflow/core/protobuf/config.pb.h"
 #include "tensorflow/core/public/version.h"
-#include "tensorflow/tsl/framework/device_id.h"
 
 namespace tensorflow {
 
@@ -71,8 +70,12 @@ class TEST_EventMgrHelper {
 
   void PollEvents() {
     while (queue_size() > 0) {
-      mutex_lock l(em_->mu_);
-      em_->PollEvents();
+      EventMgr::ToFreeVector to_free;
+      {
+        mutex_lock l(em_->mu_);
+        em_->PollEvents(nullptr, &to_free);
+      }
+      em_->FreeMemory(to_free);
     }
   }
 
@@ -120,9 +123,7 @@ TEST(EventMgr, WarnIfInCallback) {
   auto stream_exec = se::GPUMachineManager()->ExecutorForDevice(0).value();
   TEST_EventMgr em(stream_exec, GPUOptions());
   TEST_EventMgrHelper th(&em);
-  std::unique_ptr<se::Stream> stream(new se::Stream(stream_exec));
-  CHECK(stream);
-  stream->Init();
+  TF_ASSERT_OK_AND_ASSIGN(auto stream, stream_exec->CreateStream());
   bool hit = false;
   th.StartPollingLoop();
   device_event_mgr::WarnIfInCallback([&hit] { hit = true; });
@@ -379,18 +380,20 @@ class EMBenchmarkHelper {
         times->at(r).iter = r;
         times->at(r).start = Env::Default()->NowMicros();
       }
-      gpu_helper_->h2d_stream()->ThenWaitFor(gpu_helper_->compute_stream());
+      TF_ASSERT_OK(
+          gpu_helper_->h2d_stream()->WaitFor(gpu_helper_->compute_stream()));
       // Begin by copying the input values from CPU to GPU.
       const int64_t src_bytes = host_inputs_[0].TotalBytes();
       se::DeviceMemoryBase gpu_dst_ptr0(DMAHelper::base(&gpu_inputs_[0]),
                                         src_bytes);
-      gpu_helper_->h2d_stream()->ThenMemcpy(
-          &gpu_dst_ptr0, DMAHelper::base(&host_inputs_[0]), src_bytes);
+      TF_ASSERT_OK(gpu_helper_->h2d_stream()->Memcpy(
+          &gpu_dst_ptr0, DMAHelper::base(&host_inputs_[0]), src_bytes));
       se::DeviceMemoryBase gpu_dst_ptr1(DMAHelper::base(&gpu_inputs_[1]),
                                         src_bytes);
-      gpu_helper_->h2d_stream()->ThenMemcpy(
-          &gpu_dst_ptr1, DMAHelper::base(&host_inputs_[1]), src_bytes);
-      gpu_helper_->compute_stream()->ThenWaitFor(gpu_helper_->h2d_stream());
+      TF_ASSERT_OK(gpu_helper_->h2d_stream()->Memcpy(
+          &gpu_dst_ptr1, DMAHelper::base(&host_inputs_[1]), src_bytes));
+      TF_ASSERT_OK(
+          gpu_helper_->compute_stream()->WaitFor(gpu_helper_->h2d_stream()));
       if (times) {
         gpu_helper_->event_mgr()->ThenExecute(
             gpu_helper_->compute_stream(), [times, r]() {
@@ -414,12 +417,13 @@ class EMBenchmarkHelper {
               times->at(r).compute_done = Env::Default()->NowMicros();
             });
       }
-      gpu_helper_->d2h_stream()->ThenWaitFor(gpu_helper_->compute_stream());
+      TF_ASSERT_OK(
+          gpu_helper_->d2h_stream()->WaitFor(gpu_helper_->compute_stream()));
       const int64_t return_bytes = ctx->mutable_output(0)->TotalBytes();
       se::DeviceMemoryBase gpu_src_ptr(DMAHelper::base(ctx->mutable_output(0)),
                                        return_bytes);
-      gpu_helper_->d2h_stream()->ThenMemcpy(DMAHelper::base(&host_outputs_[0]),
-                                            gpu_src_ptr, return_bytes);
+      TF_ASSERT_OK(gpu_helper_->d2h_stream()->Memcpy(
+          DMAHelper::base(&host_outputs_[0]), gpu_src_ptr, return_bytes));
       gpu_helper_->event_mgr()->ThenExecute(gpu_helper_->d2h_stream(),
                                             callback);
       if (times) {
@@ -437,9 +441,7 @@ static void BM_no_ops(::testing::benchmark::State& state) {
   const int iters = state.max_iterations;
 
   auto stream_exec = se::GPUMachineManager()->ExecutorForDevice(0).value();
-  std::unique_ptr<se::Stream> stream(new se::Stream(stream_exec));
-  CHECK(stream);
-  stream->Init();
+  TF_ASSERT_OK_AND_ASSIGN(auto stream, stream_exec->CreateStream());
   TEST_EventMgr em(stream_exec, GPUOptions());
 
   auto benchmark_exec = [&]() {
