@@ -41,7 +41,7 @@ TEST(AttrTypeMap, Lookup) {
   // Unknown ops are assumed to be functions.
   // Their maps are filled with default attributes.
   bool is_function = false;
-  Status s = AttrTypeMapForOp("SomeFunctionName", &m, &is_function);
+  absl::Status s = AttrTypeMapForOp("SomeFunctionName", &m, &is_function);
   EXPECT_TRUE(s.ok());
   EXPECT_TRUE(is_function);
   ASSERT_NE(m->end(), m->find("executor_type"));
@@ -132,9 +132,9 @@ TEST(AttrBuilder, GetTypeAndNumber) {
 
 TEST(AttrBuilder, GetTypeList) {
   AttrBuilder a("IdentityN");
-  a.Set("T", gtl::ArraySlice<DataType>({DT_FLOAT, DT_INT64}));
+  a.Set("T", absl::Span<const DataType>({DT_FLOAT, DT_INT64}));
   absl::InlinedVector<DataType, 4> type_list;
-  Status s = a.GetTypeList("T", &type_list);
+  absl::Status s = a.GetTypeList("T", &type_list);
   ASSERT_TRUE(s.ok()) << s;
   ASSERT_EQ(2, type_list.size()) << type_list.size();
   ASSERT_EQ(DT_FLOAT, type_list[0]) << type_list[0];
@@ -162,10 +162,12 @@ TEST(AttrBuilder, BuildNodeDef_Modified) {
   AttrBuilder a("MatMul");
   a.Set("transpose_a", true);
   a.Set("transpose_b", false);
+  a.Set("grad_x", true);
+  a.Set("grad_y", false);
   a.NumInputs(2);
 
   const NodeDef& node_def = a.BuildNodeDef();
-  EXPECT_EQ(node_def.attr().size(), 2);
+  EXPECT_EQ(node_def.attr().size(), 6);
 
   a.Set("new_attr", 15);
   a.NumInputs(3);
@@ -173,11 +175,15 @@ TEST(AttrBuilder, BuildNodeDef_Modified) {
   const NodeDef& node_def2 = a.BuildNodeDef();
 
   auto attrs = node_def2.attr();
-  EXPECT_EQ(attrs.size(), 3);
+  EXPECT_EQ(attrs.size(), 7);
   ASSERT_NE(attrs.find("transpose_a"), attrs.end());
   EXPECT_EQ(attrs.find("transpose_a")->second.b(), true);
   ASSERT_NE(attrs.find("transpose_b"), attrs.end());
   EXPECT_EQ(attrs.find("transpose_b")->second.b(), false);
+  ASSERT_NE(attrs.find("grad_x"), attrs.end());
+  EXPECT_EQ(attrs.find("grad_x")->second.b(), true);
+  ASSERT_NE(attrs.find("grad_y"), attrs.end());
+  EXPECT_EQ(attrs.find("grad_y")->second.b(), false);
   ASSERT_NE(attrs.find("new_attr"), attrs.end());
   EXPECT_EQ(attrs.find("new_attr")->second.i(), 15);
 }
