@@ -26,6 +26,8 @@ limitations under the License.
 
 // clang-format off
 // Required for IS_MOBILE_PLATFORM
+#include "absl/status/status.h"
+#include "absl/strings/str_cat.h"
 #include "tensorflow/c/eager/immediate_execution_context.h"
 #include "tensorflow/core/common_runtime/function.h"
 #include "tensorflow/core/common_runtime/process_function_library_runtime.h"
@@ -33,6 +35,7 @@ limitations under the License.
 #include "tensorflow/core/common_runtime/stats_publisher_interface.h"
 #include "tensorflow/core/common_runtime/eager/rendezvous_cache.h"
 #include "tensorflow/core/framework/device_attributes.pb.h"
+#include "tensorflow/core/framework/function.pb.h"
 #include "tensorflow/core/lib/core/refcount.h"
 #include "tensorflow/core/lib/gtl/map_util.h"
 #include "tensorflow/core/nccl/collective_communicator.h"
@@ -43,6 +46,8 @@ limitations under the License.
 
 #include "tensorflow/c/tf_tensor.h"
 #include "tensorflow/c/tf_tensor_internal.h"
+#include "xla/tsl/platform/statusor.h"
+#include "xla/tsl/util/env_var.h"
 #include "tensorflow/core/common_runtime/collective_executor_mgr.h"
 #include "tensorflow/core/common_runtime/collective_param_resolver_local.h"
 #include "tensorflow/core/common_runtime/colocation_graph.h"
@@ -54,10 +59,11 @@ limitations under the License.
 #include "tensorflow/core/framework/function.h"
 #include "tensorflow/core/framework/graph_def_util.h"
 #include "tensorflow/core/lib/core/errors.h"
+#include "tensorflow/core/platform/refcount.h"
 #include "tensorflow/core/protobuf/config.pb.h"
 #include "tensorflow/core/public/version.h"
 #include "tensorflow/core/util/device_name_utils.h"
-#include "tensorflow/tsl/platform/refcount.h"
+#include "tsl/platform/refcount.h"
 #if !defined(IS_MOBILE_PLATFORM)
 #include "tensorflow/core/distributed_runtime/cluster_function_library_runtime.h"
 #include "tensorflow/core/distributed_runtime/collective_param_resolver_distributed.h"
@@ -87,7 +93,7 @@ EagerContext* GetCEagerContext() { return global_c_eager_context; }
 
 namespace {
 
-bool ReadBoolFromEnvVar(StringPiece env_var_name, bool default_val) {
+bool ReadBoolFromEnvVar(absl::string_view env_var_name, bool default_val) {
   bool val;
   if (tensorflow::ReadBoolFromEnvVar(env_var_name, default_val, &val).ok()) {
     return val;
@@ -103,6 +109,16 @@ auto* eager_context_created =
 
 const int64_t EagerContext::kGlobalRendezvousId = -1;
 
+bool SkipRemoteHandleWaitReady() {
+  static bool skip_remote_handle_wait_ready = []() {
+    bool result;
+    TF_CHECK_OK(tsl::ReadBoolFromEnvVar("TF_REMOTE_HANDLE_SKIP_WAIT_FOR_READY",
+                                        false, &result));
+    return result;
+  }();
+  return skip_remote_handle_wait_ready;
+}
+
 // Find the rendezvous instance corresponding to the step id, or create a
 // new instance if not existing.
 tsl::core::RefCountPtr<IntraProcessRendezvous>
@@ -135,7 +151,9 @@ EagerContext::EagerContext(
       num_active_steps_(0),
       step_container_(std::make_unique<ScopedStepContainer>(
           0, [this](const string& name) { ClearResourceContainer(name); })),
-      default_executor_(async, /*enable_streaming_enqueue=*/true),
+      default_executor_(async,
+                        /*enable_streaming_enqueue=*/!opts.config.experimental()
+                            .disable_eager_executor_streaming_enqueue()),
       log_memory_(LogMemory::IsEnabled()),
       env_(opts.env),
       collective_executor_mgr_(collective_executor_mgr, /*owned=*/false),
@@ -143,13 +161,15 @@ EagerContext::EagerContext(
       pin_small_ops_to_cpu_(ReadBoolFromEnvVar(
           "TF_EAGER_ENABLE_SMALL_TENSOR_CPU_PINNING", false)),
       run_eager_op_as_function_(run_eager_op_as_function),
-      jit_compile_rewrite_(jit_compile_rewrite) {
+      jit_compile_rewrite_(jit_compile_rewrite),
+      register_abstract_functions_local_only_(ReadBoolFromEnvVar(
+          "TF_EAGER_REGISTER_ABSTRACT_FUNCTIONS_LOCAL_ONLY", false)) {
   ResetPFLR(device_mgr, opts.env, &opts.config, TF_GRAPH_DEF_VERSION,
             &func_lib_def_, opts.config.graph_options().optimizer_options(),
             thread_pool_.get(), cluster_flr);
   // Starts exporting metrics through a platform-specific monitoring API (if
-  // provided). For builds using "tensorflow/tsl/platform/default", this is
-  // currently a no-op.
+  // provided). For builds using "tensorflow/compiler/xla/tsl/platform/default",
+  // this is currently a no-op.
   eager_context_created->GetCell()->Set(true);
   InitPrioritizedDeviceTypeList();
   runner_ = [this](std::function<void()> closure) {
@@ -312,8 +332,9 @@ Device* SelectBestMatchingDevice(const DeviceNameUtils::ParsedName& pattern,
 
 }  // namespace
 
-Status EagerContext::SelectDevice(DeviceNameUtils::ParsedName preferred,
-                                  const NodeDef& ndef, Device** out) const {
+absl::Status EagerContext::SelectDevice(DeviceNameUtils::ParsedName preferred,
+                                        const NodeDef& ndef,
+                                        Device** out) const {
   DCHECK(out != nullptr);
 
   PrioritizedDeviceTypeVector supported_devs;
@@ -336,7 +357,7 @@ Status EagerContext::SelectDevice(DeviceNameUtils::ParsedName preferred,
       pflr_device_set->prioritized_devices();
   *out = SelectBestMatchingDevice(preferred, existing, supported_devs);
   if (*out != nullptr) {
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   if (AllowSoftPlacement()) {
@@ -348,7 +369,7 @@ Status EagerContext::SelectDevice(DeviceNameUtils::ParsedName preferred,
     // requested does not exist.
     *out = SelectBestMatchingDevice(soft_device_name, existing, supported_devs);
     if (*out != nullptr) {
-      return OkStatus();
+      return absl::OkStatus();
     }
   }
 
@@ -534,11 +555,11 @@ void EagerContext::CloseRemoteContexts(
   int i = 0;
   for (const auto& worker : remote_contexts) {
     core::RefCountPtr<eager::EagerClient> client;
-    Status s = GetClient(worker, &client);
+    absl::Status s = GetClient(worker, &client);
 
     client->CloseContextAsync(
         &request, &responses[i],
-        [&worker, &counter, context_id](const Status& s) {
+        [&worker, &counter, context_id](const absl::Status& s) {
           if (!s.ok()) {
             LOG(ERROR) << "Unable to close remote context with ID "
                        << context_id << " for worker: " << worker << " due to "
@@ -629,7 +650,7 @@ EagerContext::~EagerContext() {
     // shutdown is supported.
     if (server_->worker_env()->session_mgr != nullptr) {
       // Tear down coordination service.
-      Status s = server_->StopCoordinationService();
+      absl::Status s = server_->StopCoordinationService();
       if (!s.ok()) {
         LOG(ERROR) << "Failed to stop coordination service: " << s;
       }
@@ -672,7 +693,7 @@ bool EagerContext::FindFunctionByName(const string& name) const {
   return func_lib_def_.Find(name) != nullptr;
 }
 
-Status EagerContext::FindFunctionOpData(
+absl::Status EagerContext::FindFunctionOpData(
     const string& name, const tensorflow::OpRegistrationData** op_data) {
   return func_lib_def_.LookUp(name, op_data);
 }
@@ -698,13 +719,15 @@ ImmediateExecutionTensorHandle* EagerContext::TFTensorHandleFromInterface(
   return handle;
 }
 
-Status EagerContext::RegisterFunction(AbstractFunction* f) {
-  FunctionDef* fdef;
-  TF_RETURN_IF_ERROR(f->GetFunctionDef(&fdef));
-  if (!fdef) {
-    return errors::InvalidArgument("GetFunctionDef returned nullptr.");
+absl::Status EagerContext::RegisterFunction(AbstractFunction* f) {
+  TF_ASSIGN_OR_RETURN(core::RefCountPtr<FunctionRecord> record,
+                      f->GetFunctionRecord());
+  if (!record) {
+    return absl::InvalidArgumentError("GetFunctionRecord returned nullptr.");
   }
-  return AddFunctionDef(*fdef);
+
+  return AddFunctionRecord(std::move(record), FunctionDefLibrary(),
+                           register_abstract_functions_local_only_);
 }
 
 bool EagerContext::UsesTFRT() { return false; }
@@ -755,7 +778,7 @@ std::vector<Device*> EagerContext::ListAllTfDevices() {
     for (const auto& dev : remote_device_mgr()->ListDevices()) {
       Device* device = nullptr;
       if (local_device_mgr()->LookupDevice(dev->name(), &device) !=
-          OkStatus()) {
+          absl::OkStatus()) {
         // Include this device from remote_device_mgr only if it does not exist
         // in local_device_mgr.
         devices.emplace_back(dev);
@@ -766,7 +789,8 @@ std::vector<Device*> EagerContext::ListAllTfDevices() {
   return devices;
 }
 
-Status EagerContext::AddDevices(std::vector<std::unique_ptr<Device>> devices) {
+absl::Status EagerContext::AddDevices(
+    std::vector<std::unique_ptr<Device>> devices) {
   std::vector<std::unique_ptr<Device>> local_devices, remote_devices;
   while (!devices.empty()) {
     if (devices.front()->IsLocal()) {
@@ -793,7 +817,7 @@ Status EagerContext::AddDevices(std::vector<std::unique_ptr<Device>> devices) {
   // Add the devices to pflr's device set.
   pflr_->InitializeDeviceAndFlr();
   InitPrioritizedDeviceTypeList();
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 void EagerContext::StartStep() {
@@ -817,9 +841,10 @@ ScopedStepContainer* EagerContext::StepContainer() {
   return step_container_.get();
 }
 
-Status EagerContext::MaybeRegisterFunctionRemotely(const FunctionDef& fdef) {
+absl::Status EagerContext::MaybeRegisterFunctionRemotely(
+    const FunctionDef& fdef) {
   // Only client context can register function on remote worker context.
-  if (!remote_device_manager_.Owned()) return OkStatus();
+  if (!remote_device_manager_.Owned()) return absl::OkStatus();
 #if !defined(IS_MOBILE_PLATFORM)
   std::shared_ptr<eager::EnqueueRequest> request(new eager::EnqueueRequest);
   request->set_context_id(GetContextId());
@@ -840,7 +865,7 @@ Status EagerContext::MaybeRegisterFunctionRemotely(const FunctionDef& fdef) {
     eager_client->StreamingEnqueueAsync(
         this->Executor().StreamingEnqueue(),
         /*call_opts=*/nullptr, request.get(), response,
-        [request, response](const Status& status) {
+        [request, response](const absl::Status& status) {
           if (!status.ok()) {
             LOG(ERROR) << "Failed to register function remotely due to "
                        << status.message()
@@ -851,13 +876,14 @@ Status EagerContext::MaybeRegisterFunctionRemotely(const FunctionDef& fdef) {
         });
   }
 #endif  // !IS_MOBILE_PLATFORM
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status EagerContext::MaybeRemoveFunctionRemotely(const string& function_name) {
+absl::Status EagerContext::MaybeRemoveFunctionRemotely(
+    const string& function_name) {
   // Only client context can remove function on remote worker context.
   if (!remote_device_manager_.Owned()) {
-    return OkStatus();
+    return absl::OkStatus();
   }
 
 #if !defined(IS_MOBILE_PLATFORM)
@@ -877,7 +903,7 @@ Status EagerContext::MaybeRemoveFunctionRemotely(const string& function_name) {
     eager_client->StreamingEnqueueAsync(
         this->Executor().StreamingEnqueue(),
         /*call_opts=*/nullptr, request.get(), response.get(),
-        [request, response](const Status& status) {
+        [request, response](const absl::Status& status) {
           if (!status.ok()) {
             LOG(ERROR) << "Failed to remove function remotely due to "
                        << status.message()
@@ -887,10 +913,10 @@ Status EagerContext::MaybeRemoveFunctionRemotely(const string& function_name) {
         });
   }
 #endif  // !IS_MOBILE_PLATFORM
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status EagerContext::RegisterExistingFunctionsOnRemoteWorkers(
+absl::Status EagerContext::RegisterExistingFunctionsOnRemoteWorkers(
     const std::vector<string>& remote_workers) {
 #if !defined(IS_MOBILE_PLATFORM)
   // Register multiple functions on selected remote workers.
@@ -912,7 +938,7 @@ Status EagerContext::RegisterExistingFunctionsOnRemoteWorkers(
 
   for (auto& remote_worker : remote_workers) {
     core::RefCountPtr<eager::EagerClient> eager_client;
-    Status s = GetClient(remote_worker, &eager_client);
+    absl::Status s = GetClient(remote_worker, &eager_client);
     if (!s.ok()) {
       continue;
     }
@@ -921,7 +947,7 @@ Status EagerContext::RegisterExistingFunctionsOnRemoteWorkers(
       eager_client->StreamingEnqueueAsync(
           this->Executor().StreamingEnqueue(),
           /*call_opts=*/nullptr, requests[i].get(), response.get(),
-          [request = requests[i], response](const Status& s) {
+          [request = requests[i], response](const absl::Status& s) {
             if (!s.ok()) {
               LOG(ERROR) << "Failed to register function remotely due to "
                          << s.message()
@@ -932,24 +958,34 @@ Status EagerContext::RegisterExistingFunctionsOnRemoteWorkers(
     }
   }
 #endif  // !IS_MOBILE_PLATFORM
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status EagerContext::AddFunctionDefWithStackTraces(
+absl::Status EagerContext::AddFunctionDefWithStackTraces(
     const FunctionDef& fdef, const StackTracesMap& stack_traces) {
   return AddFunctionDef(fdef, FunctionDefLibrary(),
                         /* add_to_local_only=*/false, stack_traces);
 }
 
-Status EagerContext::AddFunctionDef(const FunctionDef& fdef) {
+absl::Status EagerContext::AddFunctionDef(const FunctionDef& fdef) {
   return AddFunctionDef(fdef, FunctionDefLibrary(),
                         /* add_to_local_only=*/false);
 }
 
-Status EagerContext::AddFunctionDef(const FunctionDef& fdef,
-                                    const FunctionDefLibrary& library,
-                                    const bool add_to_local_only,
-                                    const StackTracesMap& stack_traces) {
+absl::Status EagerContext::AddFunctionDef(const FunctionDef& fdef,
+                                          const FunctionDefLibrary& library,
+                                          const bool add_to_local_only,
+                                          const StackTracesMap& stack_traces) {
+  core::RefCountPtr<FunctionRecord> func_record(
+      new FunctionRecord(fdef, stack_traces, true));
+  return AddFunctionRecord(std::move(func_record), library, add_to_local_only);
+}
+
+absl::Status EagerContext::AddFunctionRecord(
+    core::RefCountPtr<FunctionRecord> func_record,
+    const FunctionDefLibrary& library, bool add_to_local_only) {
+  const FunctionDef& fdef = func_record->fdef();
+  const StackTracesMap& stack_traces = func_record->stack_traces();
   auto fdefs_to_add =
       small_constants_optimizer::FoldInputTensors(fdef, func_lib_def_);
   for (const auto& fdef_to_add : fdefs_to_add) {
@@ -996,14 +1032,51 @@ Status EagerContext::AddFunctionDef(const FunctionDef& fdef,
     }
     is_first_ref = registered_function->RefCountIsOne();
     if (is_first_ref) {
-      TF_RETURN_IF_ERROR(func_lib_def_.AddFunctionDef(fdef, stack_traces));
+      TF_RETURN_IF_ERROR(
+          func_lib_def_.AddFunctionRecord(func_record.GetNewRef()));
       TF_RETURN_IF_ERROR(func_lib_def_.AddLibrary(library));
     }
   }
   if (is_first_ref && !add_to_local_only) {
     return MaybeRegisterFunctionRemotely(fdef);
   }
-  return OkStatus();
+  return absl::OkStatus();
+}
+
+absl::Status EagerContext::AddComponentFunction(
+    const FunctionDef& fdef, const FunctionDefLibrary& library) {
+  {
+    mutex_lock l(cache_mu_);
+    auto iter = component_function_libraries_.find(fdef.signature().name());
+    if (iter == component_function_libraries_.end()) {
+      // TODO(mrry): For any functions in the main function library, consider
+      //   deduplicating them here.
+      auto component_func_lib_def = std::make_unique<FunctionLibraryDefinition>(
+          OpRegistry::Global(), library);
+      TF_RETURN_IF_ERROR(component_func_lib_def->AddFunctionDef(fdef, {}));
+      component_function_libraries_.insert(
+          {fdef.signature().name(), std::move(component_func_lib_def)});
+    } else {
+      // The function has been registered before. If the function is different,
+      // we error out.
+      const FunctionDef* prev_fdef =
+          iter->second->Find(fdef.signature().name());
+      if (prev_fdef == nullptr) {
+        return absl::InternalError(
+            absl::StrCat("Component function: ", fdef.signature().name(),
+                         " is in the cache but not in the library"));
+      }
+      if (!FunctionDefsEqual(fdef, *prev_fdef)) {
+        return absl::InvalidArgumentError(absl::StrCat(
+            "Attempting to add a duplicate function with name: ",
+            fdef.signature().name(), " where the previous and current ",
+            "definitions differ. Previous definition: ",
+            prev_fdef->DebugString(),
+            " and current definition: ", fdef.DebugString()));
+      }
+    }
+  }
+  return absl::OkStatus();
 }
 
 const FunctionDef* EagerContext::GetFunctionDef(const string& function_name) {
@@ -1014,8 +1087,8 @@ std::vector<string> EagerContext::ListFunctionNames() {
   return func_lib_def_.ListFunctionNames();
 }
 
-Status EagerContext::AddRemoveFunctionNotifier(const string& func,
-                                               std::function<void()> notifier) {
+absl::Status EagerContext::AddRemoveFunctionNotifier(
+    const string& func, std::function<void()> notifier) {
   mutex_lock l(remove_function_notifiers_mu_);
   auto iter = remove_function_notifiers_.find(func);
   if (iter != remove_function_notifiers_.end()) {
@@ -1024,7 +1097,7 @@ Status EagerContext::AddRemoveFunctionNotifier(const string& func,
     std::vector<std::function<void()>> notifiers = {notifier};
     remove_function_notifiers_.insert({func, notifiers});
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 tensorflow::ImmediateExecutionContext::CacheStats
@@ -1049,7 +1122,7 @@ EagerContext::GetCacheStats() {
   return stats;
 }
 
-Status EagerContext::RemoveFunction(const string& func) {
+absl::Status EagerContext::RemoveFunction(const string& func) {
   // TODO(mdan): The context owns these functions. Why check refcount then?
   std::vector<std::function<void()>> notifiers;
   bool is_last_ref = false;
@@ -1087,10 +1160,10 @@ Status EagerContext::RemoveFunction(const string& func) {
     }
     return MaybeRemoveFunctionRemotely(func);
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status EagerContext::SyncExecutors() {
+absl::Status EagerContext::SyncExecutors() {
   VLOG(6) << "Calling SyncExecutors";
   StatusGroup sg;
   // Synchronize on context default executor
@@ -1115,7 +1188,7 @@ Status EagerContext::SyncExecutors() {
   request.set_context_id(GetContextId());
   request.add_queue()->mutable_sync_remote_executor_for_stream();
   BlockingCounter counter(static_cast<int>(remote_contexts.size()));
-  std::vector<Status> statuses(remote_contexts.size());
+  std::vector<absl::Status> statuses(remote_contexts.size());
 
   for (int i = 0; i < remote_contexts.size(); i++) {
     const auto& target = remote_contexts[i];
@@ -1126,14 +1199,15 @@ Status EagerContext::SyncExecutors() {
     eager_client->StreamingEnqueueAsync(
         this->Executor().StreamingEnqueue(),
         /*call_opts=*/nullptr, &request, response,
-        [response, target, &counter, &s = statuses[i]](const Status& status) {
+        [response, target, &counter,
+         &s = statuses[i]](const absl::Status& status) {
           s = status;
           delete response;
           counter.DecrementCount();
         });
   }
   counter.Wait();
-  for (const Status& s : statuses) {
+  for (const absl::Status& s : statuses) {
     sg.Update(s);
   }
 #endif  // !IS_MOBILE_PLATFORM
@@ -1163,10 +1237,16 @@ Device* EagerContext::GetCachedDevice(Fprint128 device_cache_key) {
   return iter->second;
 }
 
-void EagerContext::AddKernelToCache(Fprint128 cache_key,
-                                    KernelAndDevice* kernel) {
+core::RefCountPtr<KernelAndDevice> EagerContext::AddKernelToCache(
+    Fprint128 cache_key, core::RefCountPtr<KernelAndDevice> kernel) {
   mutex_lock ml(cache_mu_);
-  core::RefCountPtr<KernelAndDevice> new_ref(kernel);
+  auto iter = kernel_cache_.find(cache_key);
+  if (iter != kernel_cache_.end()) {
+    core::RefCountPtr<KernelAndDevice> new_ref(iter->second.get());
+    new_ref->Ref();
+    return new_ref;
+  }
+  core::RefCountPtr<KernelAndDevice> new_ref(kernel.get());
   new_ref->Ref();
   kernel_cache_[cache_key] = std::move(new_ref);
   auto* registered_function =
@@ -1178,6 +1258,7 @@ void EagerContext::AddKernelToCache(Fprint128 cache_key,
     VLOG(5) << "Cached key size of kernel " << kernel->name()
             << " is: " << registered_function->cached_kernel_keys->size();
   }
+  return kernel;
 }
 
 void EagerContext::AddDeviceToCache(Fprint128 device_cache_key,
@@ -1196,11 +1277,11 @@ void EagerContext::SetShouldStoreGraphs(bool value) {
   }
 }
 
-Status EagerContext::FindDeviceFromName(const char* device_name,
-                                        Device** device) const {
+absl::Status EagerContext::FindDeviceFromName(const char* device_name,
+                                              Device** device) const {
   *device = HostCPU();
   if (device_name == nullptr || strlen(device_name) == 0) {
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   auto status = local_device_mgr()->LookupDevice(device_name, device);
@@ -1215,13 +1296,13 @@ Status EagerContext::FindDeviceFromName(const char* device_name,
   return status;
 }
 
-Status EagerContext::FindCompositeDeviceFromName(
-    StringPiece device_name, CompositeDevice** device) const {
+absl::Status EagerContext::FindCompositeDeviceFromName(
+    absl::string_view device_name, CompositeDevice** device) const {
   tf_shared_lock l(composite_devices_mu_);
   for (const auto& d : composite_devices_) {
     if (d.second->name() == device_name) {
       *device = d.second.get();
-      return OkStatus();
+      return absl::OkStatus();
     }
   }
   return errors::NotFound("Unknown composite device: ", device_name);
@@ -1233,7 +1314,7 @@ bool EagerContext::IsCustomDevice(const string& device_name) {
                                                             &device);
 }
 
-Status EagerContext::RegisterCustomDevice(
+absl::Status EagerContext::RegisterCustomDevice(
     const string& device_name, std::unique_ptr<CustomDevice> device) {
   Device* existing_physical_device = nullptr;
   if (FindDeviceFromName(device_name.c_str(), &existing_physical_device).ok()) {
@@ -1244,12 +1325,12 @@ Status EagerContext::RegisterCustomDevice(
                                                         std::move(device));
 }
 
-Status EagerContext::FindOrCreateCompositeDevice(
+absl::Status EagerContext::FindOrCreateCompositeDevice(
     const std::vector<string>& underlying_devices, const string& device_name,
     CompositeDevice** composite_device) {
   if (!device_name.empty() &&
       FindCompositeDeviceFromName(device_name, composite_device).ok()) {
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   const uint64 hash_key = Fingerprint64(absl::StrJoin(underlying_devices, ","));
@@ -1258,10 +1339,10 @@ Status EagerContext::FindOrCreateCompositeDevice(
   auto iter = composite_devices_.find(hash_key);
   if (iter != composite_devices_.end()) {
     *composite_device = iter->second.get();
-    return OkStatus();
+    return absl::OkStatus();
   }
 
-  Status s;
+  absl::Status s;
   std::unique_ptr<CompositeDevice> device;
   if (device_name.empty()) {
     // Create a CompositeDevice on the same task as the host CPU, in order to
@@ -1276,7 +1357,7 @@ Status EagerContext::FindOrCreateCompositeDevice(
   *composite_device = device.get();
   pflr_->AddCompositeDevice(*composite_device);
   composite_devices_.emplace(hash_key, std::move(device));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 bool EagerContext::OnSameTask(const Device* first, const Device* second) const {
@@ -1288,8 +1369,8 @@ bool EagerContext::OnSameTask(const Device* first, const Device* second) const {
 }
 
 // Gets the CPU device on the task of device.
-Status EagerContext::CPUDeviceOnTask(const Device* device,
-                                     Device** cpu_device) const {
+absl::Status EagerContext::CPUDeviceOnTask(const Device* device,
+                                           Device** cpu_device) const {
   string cpu_device_name;
   TF_RETURN_IF_ERROR(DeviceNameUtils::DeviceNameToCpuDeviceName(
       device->name(), &cpu_device_name));
@@ -1306,11 +1387,12 @@ void EagerContext::ClearResourceContainer(const string& name) {
   }
 }
 
-Status EagerContext::GetGlobalRendezvousForFunctionLocalRendezvousStatus() {
+absl::Status
+EagerContext::GetGlobalRendezvousForFunctionLocalRendezvousStatus() {
   mutex_lock l(global_rendezvous_mu_);
   tsl::core::RefCountPtr<IntraProcessRendezvous> rendezvous =
       local_rendezvous_cache_.Find(kGlobalRendezvousId);
-  if (rendezvous == nullptr) return OkStatus();
+  if (rendezvous == nullptr) return absl::OkStatus();
   return rendezvous->GetLocalRendezvousStatus();
 }
 
@@ -1324,24 +1406,25 @@ void EagerContext::UpdateGlobalRendezvousDeviceManager(
 }
 
 namespace {
-Status GetTaskName(Device* d, string* task_name) {
+absl::Status GetTaskName(Device* d, string* task_name) {
   string ignored;
   if (!DeviceNameUtils::SplitDeviceName(d->name(), task_name, &ignored)) {
     return errors::InvalidArgument("Unable to parse device name: ", d->name());
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 }  // namespace
 
 #if !defined(IS_MOBILE_PLATFORM)
-Status EagerContext::GetClient(Device* device,
-                               core::RefCountPtr<eager::EagerClient>* client) {
+absl::Status EagerContext::GetClient(
+    Device* device, core::RefCountPtr<eager::EagerClient>* client) {
   return GetClient(device->parsed_name(), client);
 }
 
-Status EagerContext::GetClient(const DeviceNameUtils::ParsedName& device_name,
-                               core::RefCountPtr<eager::EagerClient>* client) {
+absl::Status EagerContext::GetClient(
+    const DeviceNameUtils::ParsedName& device_name,
+    core::RefCountPtr<eager::EagerClient>* client) {
   string device_task_name;
   if (!DeviceNameUtils::GetTaskName(device_name, &device_task_name)) {
     return errors::InvalidArgument(
@@ -1370,11 +1453,11 @@ Status EagerContext::GetClient(const DeviceNameUtils::ParsedName& device_name,
     }
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status EagerContext::GetClient(const string& remote_task,
-                               core::RefCountPtr<eager::EagerClient>* client) {
+absl::Status EagerContext::GetClient(
+    const string& remote_task, core::RefCountPtr<eager::EagerClient>* client) {
   {
     tf_shared_lock l(remote_state_mu_);
     if (remote_eager_workers_ == nullptr) {
@@ -1388,7 +1471,7 @@ Status EagerContext::GetClient(const string& remote_task,
     return errors::InvalidArgument(
         "Unable to find eager client corresponding to target ", remote_task);
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 uint64 EagerContext::GetContextId() const {
@@ -1406,13 +1489,13 @@ void EagerContext::IncrementContextViewId() {
   context_view_id_ += 1;
 }
 
-Status EagerContext::EnableCollectiveOps(const ServerDef& server_def) {
+absl::Status EagerContext::EnableCollectiveOps(const ServerDef& server_def) {
   return distributed_manager_->EnableCollectiveOps(server_def);
 }
 
 // Set collective ops related state in the context. Passing nullptr to
 // `new_server` will reuse the existing GRPC server in context.
-Status EagerContext::StoreCollectiveOpsServer(
+absl::Status EagerContext::StoreCollectiveOpsServer(
     std::unique_ptr<ServerInterface> new_server, DeviceMgr* device_mgr,
     CollectiveExecutorMgrInterface* rpc_collective_executor_mgr) {
   collective_executor_mgr_.Reset(rpc_collective_executor_mgr);
@@ -1457,10 +1540,10 @@ Status EagerContext::StoreCollectiveOpsServer(
   }
   DCHECK(server_ != nullptr);
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status EagerContext::SetRemoteDeviceFilters(
+absl::Status EagerContext::SetRemoteDeviceFilters(
     const string& remote_worker, const std::vector<string>& device_filters) {
   // Get fully specified task name for remote worker
   string remote_worker_task_name;
@@ -1496,7 +1579,7 @@ Status EagerContext::SetRemoteDeviceFilters(
   }
   mutex_lock l(remote_state_mu_);
   cluster_device_filters_.emplace(remote_worker_task_name, parsed_filters);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 void EagerContext::FilterDevicesForRemoteWorkers(
@@ -1546,7 +1629,7 @@ void EagerContext::SetWorkerEnv(WorkerEnv* worker_env,
   worker_session_ = worker_session;
 }
 
-Status EagerContext::InitializeRemoteMaster(
+absl::Status EagerContext::InitializeRemoteMaster(
     std::unique_ptr<ServerInterface> server, WorkerEnv* worker_env,
     std::shared_ptr<WorkerSession> worker_session,
     std::unique_ptr<eager::EagerClientCache> remote_eager_workers,
@@ -1577,7 +1660,7 @@ Status EagerContext::InitializeRemoteMaster(
       cluster_flr, std::move(remote_mgr));
 }
 
-Status EagerContext::UpdateRemoteMaster(
+absl::Status EagerContext::UpdateRemoteMaster(
     uint64 context_id,
     std::unique_ptr<eager::EagerClientCache> remote_eager_workers,
     const std::vector<string>& add_remote_contexts,
@@ -1640,11 +1723,11 @@ Status EagerContext::UpdateRemoteMaster(
   // ignored by the remote workers.
   TF_RETURN_IF_ERROR(
       RegisterExistingFunctionsOnRemoteWorkers(add_remote_contexts));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Set distributed execution related state in the master context.
-Status EagerContext::SetMasterContextState(
+absl::Status EagerContext::SetMasterContextState(
     std::unique_ptr<ServerInterface> server, WorkerEnv* worker_env,
     std::shared_ptr<WorkerSession> worker_session,
     std::unique_ptr<eager::EagerClientCache> remote_eager_workers,
@@ -1733,7 +1816,7 @@ Status EagerContext::SetMasterContextState(
                   {
                     for (const auto& worker : remote_contexts_) {
                       core::RefCountPtr<eager::EagerClient> client;
-                      Status s =
+                      absl::Status s =
                           remote_eager_workers_->GetClient(worker, &client);
 
                       if (!s.ok()) {
@@ -1751,7 +1834,7 @@ Status EagerContext::SetMasterContextState(
                       request->set_context_id(context_id_);
                       client->KeepAliveAsync(
                           request, response,
-                          [request, response](const Status& s) {
+                          [request, response](const absl::Status& s) {
                             delete request;
                             delete response;
                           });
@@ -1763,10 +1846,10 @@ Status EagerContext::SetMasterContextState(
           }
         }));
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status EagerContext::InitializeRemoteWorker(
+absl::Status EagerContext::InitializeRemoteWorker(
     std::unique_ptr<eager::EagerClientCache> remote_eager_workers,
     DynamicDeviceMgr* remote_device_mgr,
     const std::vector<string>& remote_contexts, uint64 context_id,
@@ -1820,10 +1903,10 @@ Status EagerContext::InitializeRemoteWorker(
 
   resource_deallocator_ = std::move(resource_deallocator);
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status EagerContext::UpdateRemoteWorker(
+absl::Status EagerContext::UpdateRemoteWorker(
     std::unique_ptr<eager::EagerClientCache> remote_eager_workers,
     const std::vector<string>& remote_contexts, uint64 context_id) {
   {
@@ -1858,7 +1941,7 @@ Status EagerContext::UpdateRemoteWorker(
       entry.second->ClearError();
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 #endif  // !IS_MOBILE_PLATFORM
 
