@@ -16,15 +16,23 @@ limitations under the License.
 #include "tensorflow/core/common_runtime/eager/context_distributed_manager.h"
 
 #include <algorithm>
+#include <cstdint>
 #include <iterator>
+#include <map>
 #include <memory>
 #include <numeric>
 #include <string>
+#include <string_view>
 #include <unordered_set>
 #include <utility>
 #include <vector>
 
 #include "google/protobuf/any.pb.h"
+#include "absl/time/time.h"
+#include "xla/tsl/platform/errors.h"
+#include "xla/tsl/platform/macros.h"
+#include "xla/tsl/platform/statusor.h"
+#include "xla/tsl/protobuf/coordination_config.pb.h"
 #include "tensorflow/core/common_runtime/copy_tensor.h"
 #include "tensorflow/core/common_runtime/device.h"
 #include "tensorflow/core/common_runtime/device_mgr.h"
@@ -47,9 +55,15 @@ limitations under the License.
 #include "tensorflow/core/protobuf/device_filters.pb.h"
 #include "tensorflow/core/protobuf/error_codes.pb.h"
 #include "tensorflow/core/util/device_name_utils.h"
-#include "tensorflow/tsl/protobuf/coordination_config.pb.h"
 
 #if !defined(IS_MOBILE_PLATFORM)
+#include "absl/base/thread_annotations.h"
+#include "absl/container/flat_hash_map.h"
+#include "absl/log/log.h"
+#include "absl/status/status.h"
+#include "absl/status/statusor.h"
+#include "absl/strings/str_cat.h"
+#include "absl/synchronization/mutex.h"
 #include "tensorflow/core/distributed_runtime/eager/cluster_function_library_runtime.h"
 #include "tensorflow/core/distributed_runtime/eager/eager_client.h"
 #include "tensorflow/core/distributed_runtime/eager/remote_mgr.h"
@@ -62,23 +76,281 @@ limitations under the License.
 #include "tensorflow/core/distributed_runtime/worker_interface.h"
 #endif  // !IS_MOBILE_PLATFORM
 
+#if (defined(PLATFORM_GOOGLE) && defined(TF_PLATFORM_LINUX_X86_64))
+#define TF_GPU_USE_PJRT
+#include "xla/pjrt/distributed/key_value_store_interface.h"
+#include "xla/pjrt/gpu/gpu_topology.h"
+#include "xla/pjrt/gpu/se_gpu_pjrt_client.h"
+#include "xla/pjrt/local_device_state.h"
+#include "xla/pjrt/pjrt_compiler.h"
+#include "xla/service/gpu/gpu_executable_run_options.h"
+#include "tensorflow/core/framework/resource_base.h"
+#include "tensorflow/core/framework/resource_mgr.h"
+#include "tensorflow/core/tfrt/common/global_state.h"
+#include "tensorflow/core/tfrt/common/pjrt_state.h"
+#include "tensorflow/core/tfrt/common/pjrt_util.h"
+#endif
+
 namespace tensorflow {
 
+#if !defined(IS_MOBILE_PLATFORM)
+namespace {
+
 // We don't use the TF_RETURN_IF_ERROR macro directly since that destroys the
 // server object (which currently CHECK-fails) and we miss the error, instead,
 // we log the error, and then return to allow the user to see the error
 // message.
-#define LOG_AND_RETURN_IF_ERROR(...)                  \
-  do {                                                \
-    const tensorflow::Status _status = (__VA_ARGS__); \
-    if (TF_PREDICT_FALSE(!_status.ok())) {            \
-      LOG(ERROR) << _status.message();                \
-      return _status;                                 \
-    }                                                 \
+#define LOG_AND_RETURN_IF_ERROR(...)            \
+  do {                                          \
+    const absl::Status _status = (__VA_ARGS__); \
+    if (TF_PREDICT_FALSE(!_status.ok())) {      \
+      LOG(ERROR) << _status.message();          \
+      return _status;                           \
+    }                                           \
   } while (0);
 
-#if !defined(IS_MOBILE_PLATFORM)
-namespace {
+#ifdef TF_GPU_USE_PJRT
+// Provide a KeyValue interface to the coordination service agent for use by
+// BuildDistributedDevices.
+class XlaKeyValueStore : public xla::KeyValueStoreInterface {
+ public:
+  explicit XlaKeyValueStore(
+      tsl::CoordinationServiceAgent* coordination_service_agent,
+      std::string key_prefix = "gpu:")
+      : coordination_service_agent_(coordination_service_agent),
+        key_prefix_(key_prefix) {}
+  absl::StatusOr<std::string> Get(std::string_view key,
+                                  absl::Duration timeout) override {
+    return coordination_service_agent_->GetKeyValue(
+        absl::StrCat(key_prefix_, key), timeout);
+  }
+
+  absl::StatusOr<std::string> TryGet(std::string_view key) override {
+    return coordination_service_agent_->TryGetKeyValue(
+        absl::StrCat(key_prefix_, key));
+  }
+
+  absl::Status Set(std::string_view key, std::string_view value) override {
+    return coordination_service_agent_->InsertKeyValue(
+        absl::StrCat(key_prefix_, key), value);
+  };
+
+ private:
+  tsl::CoordinationServiceAgent* coordination_service_agent_;
+  std::string key_prefix_;
+};
+
+// Coordinate creation of a PjRt GPU client with distributed devices when there
+// are multiple threads (which typically occurs in test environments that use
+// multiple threads to simulate multiple workers).
+class ClientCreationState : public ResourceBase {
+ public:
+  static ClientCreationState* Create() { return new ClientCreationState(); }
+
+  // Returns true for the thread that should create the PjRt GPU client,
+  // which is first thread to call this method.
+  // Each thread should only call this once (i.e. it will return
+  // false after the first call). This modifies internal state (i.e. the first
+  // call clears `first_task_`).
+  bool FirstThread() ABSL_LOCKS_EXCLUDED(mu_) {
+    absl::MutexLock lock(&mu_);
+    if (first_task_) {
+      first_task_ = false;
+      return true;
+    }
+    return false;
+  }
+
+  // The first task signals other tasks that it is ready to run
+  // BuildDistributedDevices.
+  void SetReady() { ready_notification_.Notify(); }
+
+  // The first task signals other tasks that either it has finished either after
+  // creating a client or an error occurred.
+  void SetDone() { done_notification_.Notify(); }
+
+  // The first task signals other tasks that an error occurred.
+  // done_notification_ is notified first and then ready_notification_ is
+  // notified second so any task waiting for ready_notification_ will have the
+  // proper value for done_notification_ before it unblocks to use for
+  // determining if an error occurred.
+  void SetErrorBeforeReady() {
+    done_notification_.Notify();
+    ready_notification_.Notify();
+  }
+
+  // Wait for the first task to notify that it is either ready to run
+  // BuildDistributedDevices and return true or there is an error and return
+  // false.
+  bool WaitForReadyOrError() {
+    ready_notification_.WaitForNotification();
+    return !done_notification_.HasBeenNotified();
+  }
+
+  // Wait for first task to notify that it created a client or there is an
+  // error.
+  void WaitForDone() { done_notification_.WaitForNotification(); };
+
+  std::string DebugString() const override { return "ClientCreationState"; }
+
+ private:
+  absl::Mutex mu_;
+
+  // Only the task with `first_task_ == true` (the first task) creates
+  // the PjRt GPU client.
+  bool first_task_ ABSL_GUARDED_BY(mu_) = true;
+
+  // The first task notifies any other tasks just before it is ready to run
+  // BuildDistributedDevices or if there is an error. It must notify in every
+  // codepath in the first thread, esp. every early return for errors, etc.,
+  // i.e. an error might need to notify both ready_notification_ and
+  // done_notification_.
+  Notification ready_notification_;
+
+  // The first task notifies after the PjRT GPU client is created or if
+  // there is an error. It must notify every codepath in the first
+  // thread, esp. every early return for errors, etc., i.e. an error might need
+  // to notify both ready_notification_ and done_notification_.
+  Notification done_notification_;
+};
+
+absl::StatusOr<ClientCreationState*> GetOrCreateClientCreationState() {
+  ResourceMgr* rmgr = tfrt_global::GetTFGlobalResourceMgr();
+  ClientCreationState* client_creation_state;
+  TF_RETURN_IF_ERROR(rmgr->LookupOrCreate<ClientCreationState>(
+      rmgr->default_container(), kPjRtStateResourceName, &client_creation_state,
+      [&](ClientCreationState** ret) {
+        *ret = ClientCreationState::Create();
+        return absl::OkStatus();
+      }));
+  core::ScopedUnref client_creation_state_ref(client_creation_state);
+  return client_creation_state;
+}
+
+// PjRt GPU specific code for creating a PjRt GPU client that knows about
+// remote devices. If run by multiple threads (which is done by tests using
+// threads to simulate multiple workers), all threads run
+// BuildDistributedDevices but only one thread creates the client.
+// Synchronization assures that if the task designated to create the client
+// exits early without running BuildDistributedDevices, all tasks exit early.
+// For the usual case where a client is created, synchonization assures that the
+// tasks that do not create the client wait for client creation before exiting.
+absl::Status CreateClientOnce(
+    int node_id, int num_nodes,
+    tsl::CoordinationServiceAgent* coordination_service_agent) {
+  TF_ASSIGN_OR_RETURN(auto creation_state, GetOrCreateClientCreationState());
+  bool use_creation_info = creation_state->FirstThread();
+
+  PjRtGpuClientCreationInfo* info = nullptr;
+  if (use_creation_info) {
+    auto obtained_info = GetPjRtGpuClientCreationInfo();
+    if (!obtained_info.ok()) {
+      creation_state->SetErrorBeforeReady();
+      return obtained_info.status();
+    }
+    info = obtained_info.value();
+    if (info == nullptr) {
+      VLOG(2) << "No PjRtGpuClientCreationInfo in CreateClientOnce";
+      creation_state->SetErrorBeforeReady();
+      return absl::OkStatus();
+    }
+    VLOG(2) << "Creating PjRtGpuClientCreationInfo in CreateClientOnce.";
+  } else {
+    LOG(INFO)
+        << "Skipping using GetPjRtGpuClientCreationInfo in CreateClientOnce "
+           "since it has already been used. This is expected in tests that use "
+           "multiple threads to simulate multiple workers. If this occurs in "
+           "production and op execution on GPU fails, this could be related.";
+    if (!creation_state->WaitForReadyOrError()) {
+      LOG(INFO) << "In CreateClientOnce, first thread exited early, causing "
+                   "this thread to exit.";
+      return absl::OkStatus();
+    }
+  }
+
+  std::vector<std::unique_ptr<xla::PjRtStreamExecutorDevice>> pjrt_devices;
+  auto gpu_run_options = std::make_unique<xla::gpu::GpuExecutableRunOptions>();
+#if TENSORFLOW_USE_ROCM
+  auto platform_name = xla::RocmName();
+#elif TENSORFLOW_USE_SYCL
+  auto pjrt_platform_name = xla::SyclName();
+#else   // TENSORFLOW_USE_ROCM
+  auto platform_name = xla::CudaName();
+#endif  // TENSORFLOW_USE_ROCM
+
+  auto kv_store =
+      std::make_shared<XlaKeyValueStore>(coordination_service_agent);
+  std::map<int, std::unique_ptr<xla::LocalDeviceState>> local_device_states;
+  if (use_creation_info) {
+    local_device_states = std::move(info->local_device_states);
+  }
+  if (use_creation_info) {
+    // Tell any other threads are waiting to call BuildDistributedDevices to
+    // proceed.
+    creation_state->SetReady();
+  }
+  auto device_topology_pair = BuildDistributedDevices(
+      platform_name, std::move(local_device_states), node_id, num_nodes,
+      gpu_run_options.get(), kv_store, /*enable_mock_nccl=*/false);
+  if (!device_topology_pair.ok()) {
+    if (use_creation_info) {
+      creation_state->SetDone();
+    }
+    return device_topology_pair.status();
+  }
+
+  pjrt_devices = std::move(device_topology_pair->first);
+  VLOG(2) << "Distributed devices built with size=" << pjrt_devices.size();
+  int i = 0;
+  for (const auto& pjrt_device : pjrt_devices) {
+    if (pjrt_device != nullptr) {
+      VLOG(2) << "  pjrt_device " << i++ << ":"
+              << pjrt_device->description().DebugString();
+    } else {
+      VLOG(2) << "  pjrt_device " << i++ << ":" << "nullptr";
+    }
+  }
+
+  if (use_creation_info) {
+    std::unique_ptr<xla::PjRtClient> pjrt_client =
+        std::make_unique<xla::StreamExecutorGpuClient>(
+            platform_name, info->local_client, std::move(pjrt_devices),
+            /*process_index=*/node_id,
+            /*allocator=*/std::move(info->allocator),
+            /*host_memory_allocator=*/std::move(info->host_memory_allocator),
+            /*should_stage_host_to_device_transfers=*/true,
+            /*gpu_run_options=*/std::move(gpu_run_options), kv_store,
+            /*distributed_client=*/nullptr,
+            /*abort_collectives_on_failure=*/false,
+            /*gpu_topology=*/
+            xla::GpuTopology::FromProto(device_topology_pair->second),
+            /*num_nodes=*/num_nodes);
+    VLOG(2) << "PJRT GPU client with remote devices created.";
+    auto status = SetPjRtClientInTFGlobalResourceManager(
+        DeviceType(DEVICE_GPU), std::move(pjrt_client));
+    creation_state->SetDone();
+    return status;
+  } else {
+    LOG(INFO) << "Skipping creating PJRT GPU client, another thread has "
+                 "already created the client.";
+    creation_state->WaitForDone();
+    return absl::OkStatus();
+  }
+}
+#endif  // TF_GPU_USE_PJRT
+
+absl::Status CreatePjRtGpuClientWithDistributedDevices(
+    int node_id, int num_nodes,
+    tsl::CoordinationServiceAgent* coordination_service_agent) {
+#ifdef TF_GPU_USE_PJRT
+  if (num_nodes <= 1) {
+    return absl::OkStatus();
+  }
+  return CreateClientOnce(node_id, num_nodes, coordination_service_agent);
+#else   // TF_GPU_USE_PJRT
+  return absl::OkStatus();
+#endif  // TF_GPU_USE_PJRT
+}
 
 bool AreLocalDevicesCompatible(const EagerContext* context,
                                const ServerDef& server_def) {
@@ -89,19 +361,19 @@ bool AreLocalDevicesCompatible(const EagerContext* context,
          context->session_options().config.SerializeAsString();
 }
 
-Status AddRemoteDevicesToMgr(const std::vector<string>& added_remote_workers,
-                             WorkerCacheInterface* worker_cache,
-                             DynamicDeviceMgr* remote_device_mgr) {
+absl::Status AddRemoteDevicesToMgr(
+    const std::vector<string>& added_remote_workers,
+    WorkerCacheInterface* worker_cache, DynamicDeviceMgr* remote_device_mgr) {
   std::vector<std::unique_ptr<Device>> remote_devices;
   mutex remote_devices_mu;
   int num_added_workers = added_remote_workers.size();
   BlockingCounter counter(num_added_workers);
-  std::vector<Status> statuses(num_added_workers);
+  std::vector<absl::Status> statuses(num_added_workers);
   for (int i = 0; i < num_added_workers; i++) {
     NewRemoteDevices(
         Env::Default(), worker_cache, added_remote_workers[i],
         [i, &statuses, &counter, &remote_devices, &remote_devices_mu](
-            const Status& s, std::vector<Device*>* devices) {
+            const absl::Status& s, std::vector<Device*>* devices) {
           statuses[i] = s;
           if (s.ok()) {
             mutex_lock l(remote_devices_mu);
@@ -118,20 +390,21 @@ Status AddRemoteDevicesToMgr(const std::vector<string>& added_remote_workers,
   }
 
   TF_RETURN_IF_ERROR(remote_device_mgr->AddDevices(std::move(remote_devices)));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status GetAllRemoteDevices(const std::vector<string>& remote_workers,
-                           WorkerCacheInterface* worker_cache,
-                           std::unique_ptr<DynamicDeviceMgr>* device_mgr) {
+absl::Status GetAllRemoteDevices(
+    const std::vector<string>& remote_workers,
+    WorkerCacheInterface* worker_cache,
+    std::unique_ptr<DynamicDeviceMgr>* device_mgr) {
   auto remote_device_mgr = std::make_unique<DynamicDeviceMgr>();
   TF_RETURN_IF_ERROR(AddRemoteDevicesToMgr(remote_workers, worker_cache,
                                            remote_device_mgr.get()));
   *device_mgr = std::move(remote_device_mgr);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status RemoveRemoteDevicesFromMgr(
+absl::Status RemoveRemoteDevicesFromMgr(
     const std::vector<string>& removed_remote_workers,
     DynamicDeviceMgr* remote_device_mgr) {
   const std::vector<Device*> remote_devices =
@@ -146,16 +419,17 @@ Status RemoveRemoteDevicesFromMgr(
     }
   }
   TF_RETURN_IF_ERROR(remote_device_mgr->RemoveDevices(devices_to_remove));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status ListRemoteWorkers(ServerInterface* server, const string& local_worker,
-                         std::vector<string>* remote_workers) {
+absl::Status ListRemoteWorkers(ServerInterface* server,
+                               const string& local_worker,
+                               std::vector<string>* remote_workers) {
   server->master_env()->worker_cache->ListWorkers(remote_workers);
   remote_workers->erase(
       std::remove(remote_workers->begin(), remote_workers->end(), local_worker),
       remote_workers->end());
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 void DifferentiateWorkerLists(const std::vector<string>* current_list,
@@ -191,13 +465,13 @@ void DifferentiateWorkerLists(const std::vector<string>* current_list,
   existing->resize(existing_it - existing->begin());
 }
 
-Status GetReplacedFromExistingWorkers(
+absl::Status GetReplacedFromExistingWorkers(
     const std::vector<string>* existing_workers, uint64 context_id,
     uint64 context_view_id, const ServerDef& server_def,
     eager::EagerClientCache* client_cache,
     std::vector<string>* replaced_workers) {
   BlockingCounter counter(existing_workers->size());
-  std::vector<Status> statuses(existing_workers->size());
+  std::vector<absl::Status> statuses(existing_workers->size());
   eager::KeepAliveRequest request;
   request.set_context_id(context_id);
   std::vector<eager::KeepAliveResponse> responses(existing_workers->size());
@@ -209,11 +483,12 @@ Status GetReplacedFromExistingWorkers(
       counter.DecrementCount();
       continue;
     }
-    eager_client->KeepAliveAsync(&request, &responses[i],
-                                 [i, &statuses, &counter](const Status& s) {
-                                   statuses[i] = s;
-                                   counter.DecrementCount();
-                                 });
+    eager_client->KeepAliveAsync(
+        &request, &responses[i],
+        [i, &statuses, &counter](const absl::Status& s) {
+          statuses[i] = s;
+          counter.DecrementCount();
+        });
   }
   counter.Wait();
   for (int i = 0; i < existing_workers->size(); i++) {
@@ -226,20 +501,18 @@ Status GetReplacedFromExistingWorkers(
       replaced_workers->emplace_back(existing_workers->at(i));
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status CreateRemoteContexts(EagerContext* context,
-                            const std::vector<string>& remote_workers,
-                            uint64 context_id, uint64 context_view_id,
-                            int keep_alive_secs, const ServerDef& server_def,
-                            eager::EagerClientCache* remote_eager_workers,
-                            bool async,
-                            const eager::CreateContextRequest& base_request,
-                            int64_t init_timeout_in_ms) {
+absl::Status CreateRemoteContexts(
+    EagerContext* context, const std::vector<string>& remote_workers,
+    uint64 context_id, uint64 context_view_id, int keep_alive_secs,
+    const ServerDef& server_def, eager::EagerClientCache* remote_eager_workers,
+    bool async, const eager::CreateContextRequest& base_request,
+    int64_t init_timeout_in_ms, int retries, bool clear_existing_contexts) {
   int num_remote_workers = remote_workers.size();
   BlockingCounter counter(num_remote_workers);
-  std::vector<Status> statuses(num_remote_workers);
+  std::vector<absl::Status> statuses(num_remote_workers);
   for (int i = 0; i < num_remote_workers; i++) {
     const string& remote_worker = remote_workers[i];
     DeviceNameUtils::ParsedName parsed_name;
@@ -270,6 +543,7 @@ Status CreateRemoteContexts(EagerContext* context,
     request.mutable_server_def()->set_task_index(parsed_name.task);
     request.mutable_server_def()->mutable_default_session_config()->MergeFrom(
         server_def.default_session_config());
+    request.set_clear_existing_contexts(clear_existing_contexts);
 
     std::vector<bool> filtered_device_mask;
     context->FilterDevicesForRemoteWorkers(
@@ -291,12 +565,12 @@ Status CreateRemoteContexts(EagerContext* context,
 
     eager_client->CreateContextAsync(
         &request, response,
-        [i, &statuses, &counter, response](const Status& s) {
+        [i, &statuses, &counter, response](const absl::Status& s) {
           statuses[i] = s;
           delete response;
           counter.DecrementCount();
         },
-        init_timeout_in_ms);
+        init_timeout_in_ms, retries);
   }
   counter.Wait();
   StatusGroup sg;
@@ -308,17 +582,16 @@ Status CreateRemoteContexts(EagerContext* context,
   return sg.as_summary_status();
 }
 
-Status UpdateRemoteContexts(EagerContext* context,
-                            const std::vector<string>& remote_workers,
-                            const std::vector<string>& added_workers,
-                            const std::vector<string>& removed_workers,
-                            uint64 context_id, uint64 context_view_id,
-                            const ServerDef& server_def,
-                            eager::EagerClientCache* remote_eager_workers,
-                            const eager::CreateContextRequest& base_request) {
+absl::Status UpdateRemoteContexts(
+    EagerContext* context, const std::vector<string>& remote_workers,
+    const std::vector<string>& added_workers,
+    const std::vector<string>& removed_workers, uint64 context_id,
+    uint64 context_view_id, const ServerDef& server_def,
+    eager::EagerClientCache* remote_eager_workers,
+    const eager::CreateContextRequest& base_request) {
   int num_remote_workers = remote_workers.size();
   BlockingCounter counter(num_remote_workers);
-  std::vector<Status> statuses(num_remote_workers);
+  std::vector<absl::Status> statuses(num_remote_workers);
 
   int cluster_device_count = base_request.cluster_device_attributes_size();
   std::unordered_set<string> added_or_removed(added_workers.begin(),
@@ -398,7 +671,7 @@ Status UpdateRemoteContexts(EagerContext* context,
 
     eager_client->UpdateContextAsync(
         &request, response,
-        [i, &statuses, &counter, response](const Status& s) {
+        [i, &statuses, &counter, response](const absl::Status& s) {
           statuses[i] = s;
           delete response;
           counter.DecrementCount();
@@ -408,26 +681,14 @@ Status UpdateRemoteContexts(EagerContext* context,
   for (int i = 0; i < num_remote_workers; i++) {
     TF_RETURN_IF_ERROR(statuses[i]);
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status UpdateContextWithServerDef(EagerContext* context,
-                                  const ServerDef& server_def,
-                                  bool reset_context, int keep_alive_secs,
-                                  int64_t init_timeout_in_ms) {
-  // We don't use the TF_RETURN_IF_ERROR macro directly since that destroys the
-  // server object (which currently CHECK-fails) and we miss the error, instead,
-  // we log the error, and then return to allow the user to see the error
-  // message.
-#define LOG_AND_RETURN_IF_ERROR(...)                  \
-  do {                                                \
-    const tensorflow::Status _status = (__VA_ARGS__); \
-    if (TF_PREDICT_FALSE(!_status.ok())) {            \
-      LOG(ERROR) << _status.message();                \
-      return _status;                                 \
-    }                                                 \
-  } while (0);
-
+absl::Status UpdateContextWithServerDef(EagerContext* context,
+                                        const ServerDef& server_def,
+                                        bool reset_context, int keep_alive_secs,
+                                        int64_t init_timeout_in_ms, int retries,
+                                        bool clear_existing_contexts = false) {
   string worker_name =
       strings::StrCat("/job:", server_def.job_name(),
                       "/replica:0/task:", server_def.task_index());
@@ -572,12 +833,12 @@ Status UpdateContextWithServerDef(EagerContext* context,
   }
 
   // Initialize remote eager workers.
-  Status reset_context_status = OkStatus();
+  absl::Status reset_context_status = absl::OkStatus();
   if (reset_context) {
     reset_context_status = CreateRemoteContexts(
         context, remote_workers, context_id, context_view_id, keep_alive_secs,
         server_def, remote_eager_workers.get(), context->Executor().Async(),
-        base_request, init_timeout_in_ms);
+        base_request, init_timeout_in_ms, retries, clear_existing_contexts);
     // NOTE: the remote tasks could fail after `GetAllRemoteDevices` and cause
     // the CreateRemoteContexts to fail. We currently only log instead of
     // directly returning the error, since returning here will cause the server
@@ -602,7 +863,8 @@ Status UpdateContextWithServerDef(EagerContext* context,
       sg.Update(CreateRemoteContexts(
           context, added_workers, context_id, context_view_id + 1,
           keep_alive_secs, server_def, remote_eager_workers.get(),
-          context->Executor().Async(), base_request, init_timeout_in_ms));
+          context->Executor().Async(), base_request, init_timeout_in_ms,
+          /*retries=*/0, /*clear_existing_contexts=*/false));
     }
     if (!existing_workers.empty()) {
       if (VLOG_IS_ON(1)) {
@@ -668,9 +930,9 @@ Status UpdateContextWithServerDef(EagerContext* context,
 }
 }  // namespace
 
-Status EagerContextDistributedManager::SetOrUpdateServerDef(
+absl::Status EagerContextDistributedManager::SetOrUpdateServerDef(
     const ServerDef& server_def, bool reset_context, int keep_alive_secs,
-    int64_t init_timeout_in_ms) {
+    int64_t init_timeout_in_ms, int retries, bool clear_existing_contexts) {
   if (server_def.has_cluster_device_filters()) {
     if (reset_context) {
       const auto& cdf = server_def.cluster_device_filters();
@@ -694,17 +956,22 @@ Status EagerContextDistributedManager::SetOrUpdateServerDef(
                       "when updating the server def.";
     }
   }
-  Status s = UpdateContextWithServerDef(context_, server_def, reset_context,
-                                        keep_alive_secs, init_timeout_in_ms);
+  absl::Status s = UpdateContextWithServerDef(
+      context_, server_def, reset_context, keep_alive_secs, init_timeout_in_ms,
+      retries, clear_existing_contexts);
+  if (!s.ok()) {
+    coordination_service_agent_ = nullptr;
+    return s;
+  }
   // If context is reset, make sure pointer is set to the new agent.
   coordination_service_agent_ =
       context_->GetServer()
           ->worker_env()
           ->session_mgr->GetCoordinationServiceAgent();
-  return s;
+  return absl::OkStatus();
 }
 
-Status EagerContextDistributedManager::InitializeLocalOnlyContext(
+absl::Status EagerContextDistributedManager::InitializeLocalOnlyContext(
     const ServerDef& server_def, int keep_alive_secs) {
   string worker_name =
       strings::StrCat("/job:", server_def.job_name(),
@@ -769,10 +1036,10 @@ Status EagerContextDistributedManager::InitializeLocalOnlyContext(
       context_->GetServer()
           ->worker_env()
           ->session_mgr->GetCoordinationServiceAgent();
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status EagerContextDistributedManager::EnableCollectiveOps(
+absl::Status EagerContextDistributedManager::EnableCollectiveOps(
     const ServerDef& server_def) {
   ServerInterface* server = context_->GetServer();
   if (server == nullptr) {
@@ -795,7 +1062,7 @@ Status EagerContextDistributedManager::EnableCollectiveOps(
       LOG_AND_RETURN_IF_ERROR(session_mgr->CreateSession(
           session_name, server_def,
           context_->session_options().config.isolate_session_state(),
-          [this](Status s) {
+          [this](absl::Status s) {
             context_->GetCollectiveExecutorHandle()->get()->StartAbort(s);
           }));
       LOG_AND_RETURN_IF_ERROR(
@@ -810,16 +1077,18 @@ Status EagerContextDistributedManager::EnableCollectiveOps(
           "sigterm", Env::Default());
       preemption_notifier_->WillBePreemptedAtAsync(
           [coord_agent = coordination_service_agent_](
-              StatusOr<absl::Time> time_or_status) {
+              absl::StatusOr<absl::Time> time_or_status) {
             if (time_or_status.ok()) {
               const auto coord_task = coord_agent->GetOwnTask().value();
-              Status s = coord_agent->InsertKeyValue(
+              absl::Status s = coord_agent->InsertKeyValue(
                   "TF_DEFAULT_PREEMPTION_NOTICE_KEY",
                   absl::StrCat("/job:", coord_task.job_name(),
                                "/task:", coord_task.task_id()));
               if (!s.ok()) {
-                LOG(INFO) << "Preemption not exported to coordination service: "
-                          << s;
+                // Dev note: `ALREADY_EXISTS` errors are expected if multiple
+                // workers receive a SIGTERM.
+                VLOG(3) << "Preemption not exported to coordination service: "
+                        << s;
               }
             }
           });
@@ -841,6 +1110,27 @@ Status EagerContextDistributedManager::EnableCollectiveOps(
       LOG_AND_RETURN_IF_ERROR(coordination_service_agent_->Connect());
       LOG_AND_RETURN_IF_ERROR(
           coordination_service_agent_->WaitForAllTasks(devices));
+      // Coordination service agent is now connected.
+
+      // Convert nested "job name" and "task index" into a flat "node_id" index
+      // in 0..num_nodes-1. num_nodes is the sum of the number of tasks in each
+      // job.
+      int num_nodes = 0;
+      int node_id = -1;
+      const std::string& job_name = server_def.job_name();
+      int task_index = server_def.task_index();
+      for (const auto& job : server_def.cluster().job()) {
+        if (job.name() == job_name) {
+          node_id = num_nodes + task_index;
+        }
+        num_nodes += job.tasks().size();
+      }
+      VLOG(2)
+          << "Creating PJRT client with distributed devices with num_nodes: "
+          << num_nodes << ", node_id: " << node_id;
+
+      LOG_AND_RETURN_IF_ERROR(CreatePjRtGpuClientWithDistributedDevices(
+          node_id, num_nodes, coordination_service_agent_));
 
       // Add remote devices to eager context.
       std::vector<std::unique_ptr<Device>> remote_devices;
@@ -866,11 +1156,10 @@ Status EagerContextDistributedManager::EnableCollectiveOps(
         /*new_server=*/nullptr, server->worker_env()->device_mgr,
         server->worker_env()->collective_executor_mgr.get()));
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-
-Status EagerContextDistributedManager::CheckRemoteAlive(
+absl::Status EagerContextDistributedManager::CheckRemoteAlive(
     const std::string& remote_task_name, bool* is_alive) {
   *is_alive = false;
   WorkerInterface* wi =
@@ -884,10 +1173,10 @@ Status EagerContextDistributedManager::CheckRemoteAlive(
 
   GetStatusRequest request;
   GetStatusResponse response;
-  Status remote_status;
+  absl::Status remote_status;
   Notification done;
   wi->GetStatusAsync(/*opts_=*/nullptr, &request, &response, /*fail_fast=*/true,
-                     [&remote_status, &done](const Status& s) {
+                     [&remote_status, &done](const absl::Status& s) {
                        remote_status = s;
                        done.Notify();
                      });
@@ -899,7 +1188,7 @@ Status EagerContextDistributedManager::CheckRemoteAlive(
     LOG(INFO) << "Remote worker " << remote_task_name
               << " is not alive: " << remote_status.message();
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 #endif  // !IS_MOBILE_PLATFORM
 }  // namespace tensorflow
