@@ -19,6 +19,8 @@ limitations under the License.
 #include <variant>
 #include <vector>
 
+#include "absl/status/status.h"
+#include "absl/strings/str_cat.h"
 #include "absl/types/span.h"
 #include "tensorflow/c/eager/abstract_operation.h"
 #include "tensorflow/c/eager/abstract_tensor_handle.h"
@@ -27,6 +29,7 @@ limitations under the License.
 #include "tensorflow/core/common_runtime/eager/attr_builder.h"
 #include "tensorflow/core/common_runtime/eager/custom_device.h"
 #include "tensorflow/core/common_runtime/input_colocation_exemption_registry.h"
+#include "tensorflow/core/framework/function.h"
 #include "tensorflow/core/platform/casts.h"
 #include "tensorflow/core/platform/errors.h"
 #include "tensorflow/core/platform/host_info.h"
@@ -45,40 +48,42 @@ void EagerOperation::Clear() {
   ClearInferenceState();
 }
 
-Status EagerOperation::SetAttrValue(const char* attr_name,
-                                    const AttrValue& value) {
+absl::Status EagerOperation::SetAttrValue(const char* attr_name,
+                                          const AttrValue& value) {
   MutableAttrs()->Set(attr_name, value);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status EagerOperation::SetAttrString(const char* attr_name, const char* data,
-                                     size_t length) {
-  MutableAttrs()->Set(attr_name, StringPiece(data, length));
-  return OkStatus();
+absl::Status EagerOperation::SetAttrString(const char* attr_name,
+                                           const char* data, size_t length) {
+  MutableAttrs()->Set(attr_name, absl::string_view(data, length));
+  return absl::OkStatus();
 }
 
-Status EagerOperation::SetAttrInt(const char* attr_name, int64_t value) {
+absl::Status EagerOperation::SetAttrInt(const char* attr_name, int64_t value) {
   MutableAttrs()->Set(attr_name, static_cast<int64_t>(value));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status EagerOperation::SetAttrFloat(const char* attr_name, float value) {
+absl::Status EagerOperation::SetAttrFloat(const char* attr_name, float value) {
   MutableAttrs()->Set(attr_name, value);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status EagerOperation::SetAttrBool(const char* attr_name, bool value) {
+absl::Status EagerOperation::SetAttrBool(const char* attr_name, bool value) {
   MutableAttrs()->Set(attr_name, value);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status EagerOperation::SetAttrType(const char* attr_name, DataType value) {
+absl::Status EagerOperation::SetAttrType(const char* attr_name,
+                                         DataType value) {
   MutableAttrs()->Set(attr_name, value);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status EagerOperation::SetAttrShape(const char* attr_name, const int64_t* dims,
-                                    const int num_dims) {
+absl::Status EagerOperation::SetAttrShape(const char* attr_name,
+                                          const int64_t* dims,
+                                          const int num_dims) {
   if (num_dims > TensorShape::MaxDimensions()) {
     return errors::InvalidArgument("Value specified for `", attr_name, "` has ",
                                    num_dims,
@@ -97,86 +102,91 @@ Status EagerOperation::SetAttrShape(const char* attr_name, const int64_t* dims,
 
   MutableAttrs()->Set(attr_name, proto);
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status EagerOperation::SetAttrFunction(const char* attr_name,
-                                       const AbstractOperation* value) {
+absl::Status EagerOperation::SetAttrFunction(const char* attr_name,
+                                             const AbstractOperation* value) {
   AttrValue attr_value;
   NameAttrList* func = attr_value.mutable_func();
   func->set_name(value->Name());
   auto* value_operation = down_cast<const EagerOperation*>(value);
   value_operation->Attrs().FillAttrValueMap(func->mutable_attr());
   MutableAttrs()->Set(attr_name, attr_value);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status EagerOperation::SetAttrFunctionName(const char* attr_name,
-                                           const char* data, size_t length) {
+absl::Status EagerOperation::SetAttrFunctionName(const char* attr_name,
+                                                 const char* data,
+                                                 size_t length) {
   AttrValue attr_value;
   NameAttrList* func = attr_value.mutable_func();
   func->set_name(data, length);
   MutableAttrs()->Set(attr_name, attr_value);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status EagerOperation::SetAttrTensor(const char* attr_name,
-                                     AbstractTensorInterface* tensor) {
+absl::Status EagerOperation::SetAttrTensor(const char* attr_name,
+                                           AbstractTensorInterface* tensor) {
   Tensor t = TensorFromInterface(tensor);
   MutableAttrs()->Set(attr_name, t);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status EagerOperation::SetAttrStringList(const char* attr_name,
-                                         const void* const* values,
-                                         const size_t* lengths,
-                                         int num_values) {
-  std::vector<StringPiece> v(num_values);
+absl::Status EagerOperation::SetAttrStringList(const char* attr_name,
+                                               const void* const* values,
+                                               const size_t* lengths,
+                                               int num_values) {
+  std::vector<absl::string_view> v(num_values);
   for (int i = 0; i < num_values; ++i) {
-    v[i] = StringPiece(static_cast<const char*>(values[i]), lengths[i]);
+    v[i] = absl::string_view(static_cast<const char*>(values[i]), lengths[i]);
   }
   MutableAttrs()->Set(attr_name, v);
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status EagerOperation::SetAttrFloatList(const char* attr_name,
-                                        const float* values, int num_values) {
+absl::Status EagerOperation::SetAttrFloatList(const char* attr_name,
+                                              const float* values,
+                                              int num_values) {
   MutableAttrs()->Set(attr_name,
                       gtl::ArraySlice<const float>(values, num_values));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status EagerOperation::SetAttrIntList(const char* attr_name,
-                                      const int64_t* values, int num_values) {
+absl::Status EagerOperation::SetAttrIntList(const char* attr_name,
+                                            const int64_t* values,
+                                            int num_values) {
   MutableAttrs()->Set(
       attr_name, gtl::ArraySlice<const int64_t>(
                      reinterpret_cast<const int64_t*>(values), num_values));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status EagerOperation::SetAttrTypeList(const char* attr_name,
-                                       const DataType* values, int num_values) {
+absl::Status EagerOperation::SetAttrTypeList(const char* attr_name,
+                                             const DataType* values,
+                                             int num_values) {
   MutableAttrs()->Set(attr_name,
                       gtl::ArraySlice<const DataType>(values, num_values));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status EagerOperation::SetAttrBoolList(const char* attr_name,
-                                       const unsigned char* values,
-                                       int num_values) {
+absl::Status EagerOperation::SetAttrBoolList(const char* attr_name,
+                                             const unsigned char* values,
+                                             int num_values) {
   std::unique_ptr<bool[]> b(new bool[num_values]);
   for (int i = 0; i < num_values; ++i) {
     b[i] = values[i];
   }
   MutableAttrs()->Set(attr_name,
                       gtl::ArraySlice<const bool>(b.get(), num_values));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status EagerOperation::SetAttrShapeList(const char* attr_name,
-                                        const int64_t** dims,
-                                        const int* num_dims, int num_values) {
+absl::Status EagerOperation::SetAttrShapeList(const char* attr_name,
+                                              const int64_t** dims,
+                                              const int* num_dims,
+                                              int num_values) {
   std::unique_ptr<TensorShapeProto[]> proto(new TensorShapeProto[num_values]);
   for (int i = 0; i < num_values; ++i) {
     const auto num_dims_i = num_dims[i];
@@ -198,11 +208,11 @@ Status EagerOperation::SetAttrShapeList(const char* attr_name,
     }
   }
   MutableAttrs()->Set(
-      attr_name, gtl::ArraySlice<TensorShapeProto>(proto.get(), num_values));
-  return OkStatus();
+      attr_name, absl::Span<const TensorShapeProto>(proto.get(), num_values));
+  return absl::OkStatus();
 }
 
-Status EagerOperation::SetAttrFunctionList(
+absl::Status EagerOperation::SetAttrFunctionList(
     const char* attr_name, absl::Span<const AbstractOperation*> values) {
   size_t num_values = values.size();
   std::unique_ptr<NameAttrList[]> funcs(new NameAttrList[num_values]);
@@ -213,18 +223,18 @@ Status EagerOperation::SetAttrFunctionList(
   }
   MutableAttrs()->Set(
       attr_name, gtl::ArraySlice<const NameAttrList>(funcs.get(), num_values));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-const OpDef* EagerOperation::GetOpDef(Status* status) {
+const OpDef* EagerOperation::GetOpDef(absl::Status* status) {
   const tensorflow::OpDef* op_def = OpDef();
   if (op_def) return op_def;
   *status = OpDefForOp(Name(), &op_def);
   return op_def;
 }
 
-Status EagerOperation::InputLength(const char* input_name, int* length) {
-  Status status;
+absl::Status EagerOperation::InputLength(const char* input_name, int* length) {
+  absl::Status status;
   const tensorflow::OpDef* op_def = GetOpDef(&status);
   if (!status.ok()) {
     return status;
@@ -239,7 +249,7 @@ Status EagerOperation::InputLength(const char* input_name, int* length) {
     return errors::InvalidArgument("Input '", input_name, "' not found");
   }
   *length = iter->second.second - iter->second.first;
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 absl::Span<ImmediateExecutionTensorHandle* const> EagerOperation::GetInputs()
@@ -250,8 +260,9 @@ absl::Span<ImmediateExecutionTensorHandle* const> EagerOperation::GetInputs()
       inputs_.size());
 }
 
-Status EagerOperation::OutputLength(const char* output_name, int* length) {
-  Status status;
+absl::Status EagerOperation::OutputLength(const char* output_name,
+                                          int* length) {
+  absl::Status status;
   const tensorflow::OpDef* op_def = GetOpDef(&status);
   if (!status.ok()) {
     return status;
@@ -266,10 +277,10 @@ Status EagerOperation::OutputLength(const char* output_name, int* length) {
     return errors::InvalidArgument("Output '", output_name, "' not found");
   }
   *length = iter->second.second - iter->second.first;
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status EagerOperation::AddInput(AbstractTensorHandle* input) {
+absl::Status EagerOperation::AddInput(AbstractTensorHandle* input) {
   ImmediateExecutionTensorHandle* h =
       down_cast<ImmediateExecutionTensorHandle*>(input);
   // TODO(b/175427838): It would be nice to be able to use tensorflow::isa here.
@@ -280,7 +291,7 @@ Status EagerOperation::AddInput(AbstractTensorHandle* input) {
   return MaybeInferSingleInputAttrs(h);
 }
 
-Status EagerOperation::AddInputList(
+absl::Status EagerOperation::AddInputList(
     absl::Span<AbstractTensorHandle* const> inputs) {
   for (auto& input : inputs) {
     // TODO(b/175427838): It would be nice to be able to use tensorflow::isa
@@ -295,8 +306,8 @@ Status EagerOperation::AddInputList(
   return InferInputListAttrs(inputs.size());
 }
 
-Status EagerOperation::SetInput(size_t index,
-                                ImmediateExecutionTensorHandle* input) {
+absl::Status EagerOperation::SetInput(size_t index,
+                                      ImmediateExecutionTensorHandle* input) {
   if (index >= inputs_.size()) {
     return errors::InvalidArgument("Index >= inputs.size: %d >= %d", index,
                                    inputs_.size());
@@ -311,10 +322,10 @@ Status EagerOperation::SetInput(size_t index,
   input->Ref();
   inputs_[index] = input;
   previous->Unref();
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status EagerOperation::Reset(
+absl::Status EagerOperation::Reset(
     const char* op, const char* device_name, bool remote,
     EagerExecutor* executor,
     const absl::optional<EagerFunctionParams> eager_func_params) {
@@ -333,16 +344,24 @@ Status EagerOperation::Reset(
   if (!is_function) {
     const auto& exempt_ops = InputColocationExemptionRegistry::Global()->Get();
     colocation_exempt_ = exempt_ops.find(op) != exempt_ops.end();
-
     TF_RETURN_IF_ERROR(OpDefForOp(op, &op_def_));
-  } else if (!remote && !ctx_.FindFunctionByName(op)) {
-    return errors::NotFound(
-        "'", op,
-        "' is neither a type of a primitive operation nor a name "
-        "of a function registered in binary running on ",
-        port::Hostname(),
-        ". Make sure the operation or function is "
-        "registered in the binary running in this process.");
+  } else if (!remote) {
+    const FunctionLibraryDefinition* func_lib_def;
+    if (eager_func_params.has_value() &&
+        eager_func_params.value().func_lib_def_override != nullptr) {
+      func_lib_def = eager_func_params.value().func_lib_def_override;
+    } else {
+      func_lib_def = ctx_.FuncLibDef();
+    }
+    if (func_lib_def->Find(op) == nullptr) {
+      return absl::NotFoundError(absl::StrCat(
+          "'", op,
+          "' is neither a type of a primitive operation nor a name "
+          "of a function registered in binary running on ",
+          port::Hostname(),
+          ". Make sure the operation or function is "
+          "registered in the binary running in this process."));
+    }
   }
   attrs_.Reset(op);
   stack_trace_.reset();
@@ -352,13 +371,13 @@ Status EagerOperation::Reset(
   if (eager_func_params.has_value()) {
     eager_func_params_ = eager_func_params;
   }
-  op_name_ = op;
+  op_name_ = std::string(op);
   return SetDeviceName(device_name);
 }
 
-Status EagerOperation::MaybeInferSingleInputAttrs(
+absl::Status EagerOperation::MaybeInferSingleInputAttrs(
     ImmediateExecutionTensorHandle* handle) {
-  if (!op_def_) return OkStatus();
+  if (!op_def_) return absl::OkStatus();
 
   const auto& input_def = op_def_->input_arg(inference_arg_idx_++);
   if (!input_def.number_attr().empty() || !input_def.type_list_attr().empty()) {
@@ -369,7 +388,7 @@ Status EagerOperation::MaybeInferSingleInputAttrs(
     // arguments in the op definition. To guarantee backward compatibility with
     // those clients, disable automatic inference in this case.
     ClearInferenceState();
-    return OkStatus();
+    return absl::OkStatus();
   }
   const std::string& type_attr = input_def.type_attr();
   if (!type_attr.empty() &&
@@ -377,7 +396,7 @@ Status EagerOperation::MaybeInferSingleInputAttrs(
     MutableAttrs()->Set(type_attr, handle->DataType());
     inference_attrs_.insert(type_attr);
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 void EagerOperation::InferSingleTypeInputListAttrs(
@@ -404,8 +423,8 @@ void EagerOperation::InferMixedTypeInputListAttrs(
   }
 }
 
-Status EagerOperation::InferInputListAttrs(int num_inputs) {
-  if (!op_def_) return OkStatus();
+absl::Status EagerOperation::InferInputListAttrs(int num_inputs) {
+  if (!op_def_) return absl::OkStatus();
 
   int start = inference_arg_idx_;
   const auto& input_def = op_def_->input_arg(inference_arg_idx_++);
@@ -428,32 +447,32 @@ Status EagerOperation::InferInputListAttrs(int num_inputs) {
   } else {
     return errors::InvalidArgument("Invalid input list definition");
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status EagerOperation::TensorHandleInputs(
+absl::Status EagerOperation::TensorHandleInputs(
     const absl::InlinedVector<TensorHandle*, 4>** inputs) const {
   if (TF_PREDICT_TRUE(!HasCustomDeviceInput())) {
     *inputs = reinterpret_cast<const absl::InlinedVector<TensorHandle*, 4>*>(
         &inputs_);
-    return OkStatus();
+    return absl::OkStatus();
   } else {
     return errors::Internal("The operation unexpectedly had custom devices.");
   }
 }
 
-Status EagerOperation::MutableTensorHandleInputs(
+absl::Status EagerOperation::MutableTensorHandleInputs(
     absl::InlinedVector<TensorHandle*, 4>** inputs) {
   if (TF_PREDICT_TRUE(!HasCustomDeviceInput())) {
     *inputs =
         reinterpret_cast<absl::InlinedVector<TensorHandle*, 4>*>(&inputs_);
-    return OkStatus();
+    return absl::OkStatus();
   } else {
     return errors::Internal("The operation unexpectedly had custom devices.");
   }
 }
 
-Status EagerOperation::SetDeviceName(const char* c_name) {
+absl::Status EagerOperation::SetDeviceName(const char* c_name) {
   string name(c_name != nullptr ? c_name : "");
   if (name != last_set_device_name_) {
     if (!DeviceNameUtils::ParseFullName(name, &device_parsed_name_)) {
@@ -464,7 +483,7 @@ Status EagerOperation::SetDeviceName(const char* c_name) {
     device_name_ = DeviceNameUtils::ParsedNameToString(device_parsed_name_);
     device_ = kVariantDeviceNull;
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 bool EagerOperation::IsLocal() const {
