@@ -23,6 +23,7 @@ limitations under the License.
 
 #include "absl/status/status.h"
 #include "absl/strings/match.h"
+#include "absl/types/optional.h"
 #include "tensorflow/core/common_runtime/device_factory.h"
 #include "tensorflow/core/common_runtime/eager/attr_builder.h"
 #include "tensorflow/core/common_runtime/process_function_library_runtime.h"
@@ -56,8 +57,8 @@ limitations under the License.
 
 namespace tensorflow {
 
-Status EagerKernelArgs::GetLocalArg(const FunctionArgIndex& index,
-                                    Tensor* val) const {
+absl::Status EagerKernelArgs::GetLocalArg(const FunctionArgIndex& index,
+                                          Tensor* val) const {
   if (index.sub_index >= 0) {
     return errors::InvalidArgument("Got unexpected sub_index ", index.sub_index,
                                    " for argument ", index.index);
@@ -65,7 +66,7 @@ Status EagerKernelArgs::GetLocalArg(const FunctionArgIndex& index,
   Tensor* arg = tensor_args_.at(index.index).tensor;
   if (arg) {
     *val = *arg;
-    return OkStatus();
+    return absl::OkStatus();
   } else {
     return errors::NotFound("Argument ", index.index, " has no local tensor.");
   }
@@ -94,7 +95,7 @@ std::function<void(std::function<void()>)>* KernelAndDevice::get_runner()
 
 KernelAndDeviceFunc::~KernelAndDeviceFunc() {
   if (handle_ != kInvalidHandle) {
-    Status status = pflr_->ReleaseHandle(handle_);
+    absl::Status status = pflr_->ReleaseHandle(handle_);
     if (!status.ok()) {
       LOG(INFO) << "Ignoring error status when releasing multi-device function "
                    "handle "
@@ -103,9 +104,14 @@ KernelAndDeviceFunc::~KernelAndDeviceFunc() {
   }
 }
 
-Status KernelAndDeviceOp::Init(const bool log_device_placement,
-                               const NodeDef& ndef,
-                               GraphCollector* graph_collector) {
+absl::Status KernelAndDeviceOp::Init(
+    const bool log_device_placement, const NodeDef& ndef,
+    GraphCollector* graph_collecto,
+    const absl::optional<EagerFunctionParams>& eager_func_params) {
+  if (eager_func_params.has_value()) {
+    return absl::InternalError(
+        "KernelAndDeviceOp does not support EagerFunctionParams.");
+  }
   OpKernel* k = nullptr;
   if (flr_ == nullptr) {
     return errors::Internal(
@@ -138,25 +144,34 @@ Status KernelAndDeviceOp::Init(const bool log_device_placement,
                                        tensorflow::HOST_MEMORY);
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status KernelAndDeviceFunc::InstantiateFunc(const bool log_device_placement,
-                                            const NodeDef& ndef,
-                                            GraphCollector* graph_collector) {
+absl::Status KernelAndDeviceFunc::InstantiateFunc(
+    const bool log_device_placement, const NodeDef& ndef,
+    GraphCollector* graph_collector,
+    const absl::optional<EagerFunctionParams>& eager_func_params) {
   const OpDef* op_def = nullptr;
-  const FunctionDef* function_def;
-  if (flr_ == nullptr) {
-    // If function is being executed without an explicit device request,
-    // lookup the FunctionDef in the CPU's FLR. All FLRs share the same
-    // library.
-    function_def = pflr_->GetFLR(host_cpu_device_->name())
-                       ->GetFunctionLibraryDefinition()
-                       ->Find(ndef.op());
+  const FunctionLibraryDefinition* func_lib_def;
+  FunctionLibraryRuntime::InstantiateOptions options;
+
+  if (eager_func_params.has_value() &&
+      eager_func_params.value().func_lib_def_override != nullptr) {
+    func_lib_def = eager_func_params.value().func_lib_def_override;
+    options.lib_def = func_lib_def;
   } else {
-    function_def = flr_->GetFunctionLibraryDefinition()->Find(ndef.op());
+    if (flr_ == nullptr) {
+      // If function is being executed without an explicit device request,
+      // lookup the FunctionDef in the CPU's FLR. All FLRs share the same
+      // library.
+      func_lib_def = pflr_->GetFLR(host_cpu_device_->name())
+                         ->GetFunctionLibraryDefinition();
+    } else {
+      func_lib_def = flr_->GetFunctionLibraryDefinition();
+    }
   }
 
+  const FunctionDef* function_def = func_lib_def->Find(ndef.op());
   if (function_def != nullptr) {
     op_def = &(function_def->signature());
   } else {
@@ -165,7 +180,6 @@ Status KernelAndDeviceFunc::InstantiateFunc(const bool log_device_placement,
   TF_RETURN_IF_ERROR(
       InOutTypesForNode(ndef, *op_def, &input_dtypes_, &output_dtypes_));
 
-  FunctionLibraryRuntime::InstantiateOptions options;
   options.target = device_ == nullptr ? "" : device_->name();
   options.is_multi_device_function = true;
   for (const Device* device : input_devices_) {
@@ -174,13 +188,10 @@ Status KernelAndDeviceFunc::InstantiateFunc(const bool log_device_placement,
   options.composite_devices = composite_devices_;
   options.input_resource_dtypes_and_shapes = input_resource_dtypes_and_shapes_;
   if (outputs_on_op_device_) {
-    const FunctionLibraryDefinition* lib_def =
-        pflr_->GetFunctionLibraryDefinition();
-    const FunctionDef* fdef = lib_def->Find(ndef.op());
-    if (fdef == nullptr) {
+    if (function_def == nullptr) {
       return errors::InvalidArgument("Failed to find function ", ndef.op());
     }
-    for (int i = 0; i < fdef->signature().output_arg_size(); ++i) {
+    for (int i = 0; i < function_def->signature().output_arg_size(); ++i) {
       options.output_devices.push_back(options.target);
     }
   }
@@ -236,6 +247,7 @@ Status KernelAndDeviceFunc::InstantiateFunc(const bool log_device_placement,
   options.config_proto.set_log_device_placement(log_device_placement);
 
   options.int_args_and_retvals_on_device = int_args_and_retvals_on_device_;
+  options.function_runs_at_most_once = function_runs_at_most_once_;
 
   if (xla_compile_device_type_.has_value()) {
     options.xla_compile_device_type = xla_compile_device_type_.value();
@@ -248,11 +260,12 @@ Status KernelAndDeviceFunc::InstantiateFunc(const bool log_device_placement,
   return pflr_->IsCrossProcess(handle_, &is_cross_process_);
 }
 
-Status KernelAndDeviceFunc::Init(const bool log_device_placement,
-                                 const NodeDef& ndef,
-                                 GraphCollector* graph_collector) {
-  TF_RETURN_IF_ERROR(
-      InstantiateFunc(log_device_placement, ndef, graph_collector));
+absl::Status KernelAndDeviceFunc::Init(
+    const bool log_device_placement, const NodeDef& ndef,
+    GraphCollector* graph_collector,
+    const absl::optional<EagerFunctionParams>& eager_func_params) {
+  TF_RETURN_IF_ERROR(InstantiateFunc(log_device_placement, ndef,
+                                     graph_collector, eager_func_params));
   return pflr_->GetOutputDevices(handle_, &output_devices_);
 }
 
@@ -267,7 +280,7 @@ struct OpExecutionState : public core::RefCounted {
 };
 }  // anonymous namespace
 
-Status KernelAndDeviceOp::Run(
+absl::Status KernelAndDeviceOp::Run(
     ScopedStepContainer* step_container, const EagerKernelArgs& inputs,
     std::vector<EagerKernelRet>* outputs,
     CancellationManager* cancellation_manager,
@@ -324,7 +337,7 @@ Status KernelAndDeviceOp::Run(
     // time on device of the OpKernel.
     profiler::AnnotatedTraceMe activity(
         [&] { return kernel_->TraceString(context, /*verbose=*/false); },
-        profiler::TraceMeLevel::kInfo);
+        tsl::profiler::TraceMeLevel::kInfo);
     device_->Compute(kernel_.get(), &context);
   }
 
@@ -333,7 +346,7 @@ Status KernelAndDeviceOp::Run(
     op_execution_state->Unref();
   }
 
-  Status s = context.status();
+  absl::Status s = context.status();
   if (TF_PREDICT_FALSE(!s.ok())) {
     if (absl::IsUnavailable(s) && !is_distributed_communication_op_) {
       s = errors::ReplaceErrorFromNonCommunicationOps(s, kernel_->name());
@@ -352,7 +365,7 @@ Status KernelAndDeviceOp::Run(
       }
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 std::shared_ptr<FunctionLibraryRuntime::Options>
@@ -417,22 +430,22 @@ KernelAndDeviceFunc::PrepareForRun(
   return opts;
 }
 
-Status KernelAndDeviceFunc::Run(
+absl::Status KernelAndDeviceFunc::Run(
     ScopedStepContainer* step_container, const EagerKernelArgs& inputs,
     std::vector<EagerKernelRet>* outputs,
     CancellationManager* cancellation_manager,
     const absl::optional<EagerFunctionParams>& eager_func_params,
     const absl::optional<ManagedStackTrace>& stack_trace,
     tsl::CoordinationServiceAgent* coordination_service_agent) {
-  profiler::TraceMe activity("KernelAndDeviceFunc::Run",
-                             profiler::TraceMeLevel::kInfo);
+  tsl::profiler::TraceMe activity("KernelAndDeviceFunc::Run",
+                                  tsl::profiler::TraceMeLevel::kInfo);
   // Don't try to handle packed or remote inputs synchronously.
   if (inputs.HasRemoteOrPackedInputs() || eager_func_params.has_value()) {
     Notification n;
-    Status status;
+    absl::Status status;
     RunAsync(step_container, inputs, outputs, cancellation_manager,
              eager_func_params, coordination_service_agent,
-             [&status, &n](Status s) {
+             [&status, &n](absl::Status s) {
                status = s;
                n.Notify();
              });
@@ -445,7 +458,7 @@ Status KernelAndDeviceFunc::Run(
       stack_trace, coordination_service_agent, &created_rendezvous);
 
   std::vector<Tensor> rets;
-  Status s;
+  absl::Status s;
   {
     port::ScopedFlushDenormal flush;
     port::ScopedSetRound round(FE_TONEAREST);
@@ -468,28 +481,28 @@ void KernelAndDeviceFunc::RunAsync(
     CancellationManager* cancellation_manager,
     const absl::optional<EagerFunctionParams>& eager_func_params,
     tsl::CoordinationServiceAgent* coordination_service_agent,
-    std::function<void(const Status&)> done) {
-  profiler::TraceMe activity(
+    std::function<void(const absl::Status&)> done) {
+  tsl::profiler::TraceMe activity(
       [] {
-        return profiler::TraceMeEncode("KernelAndDeviceFunc::RunAsync",
-                                       {{"_r", 1}});
+        return tsl::profiler::TraceMeEncode("KernelAndDeviceFunc::RunAsync",
+                                            {{"_r", 1}});
       },
-      profiler::TraceMeLevel::kInfo);
+      tsl::profiler::TraceMeLevel::kInfo);
   tsl::core::RefCountPtr<Rendezvous> created_rendezvous;
   std::shared_ptr<FunctionLibraryRuntime::Options> opts = PrepareForRun(
       step_container, outputs, cancellation_manager, eager_func_params,
       std::nullopt, coordination_service_agent, &created_rendezvous);
 
-  pflr_->Run(
-      *opts, handle_, inputs, outputs,
-      [opts, cancellation_manager, done = std::move(done),
-       created_rendezvous = created_rendezvous.release()](const Status& s) {
-        if (cancellation_manager == nullptr) {
-          delete opts->cancellation_manager;
-        }
-        created_rendezvous->Unref();
-        done(s);
-      });
+  pflr_->Run(*opts, handle_, inputs, outputs,
+             [opts, cancellation_manager, done = std::move(done),
+              created_rendezvous =
+                  created_rendezvous.release()](const absl::Status& s) {
+               if (cancellation_manager == nullptr) {
+                 delete opts->cancellation_manager;
+               }
+               created_rendezvous->Unref();
+               done(s);
+             });
 }
 
 tensorflow::Device* KernelAndDeviceOp::OutputDevice(int idx) const {
