@@ -25,6 +25,8 @@ limitations under the License.
 #include <variant>
 #include <vector>
 
+#include "absl/status/status.h"
+#include "absl/strings/str_cat.h"
 #include "absl/strings/substitute.h"
 #include "absl/types/variant.h"
 #include "tensorflow/c/tf_tensor_internal.h"
@@ -64,6 +66,7 @@ string SafeDeviceDebugString(Device* device) {
     return device->DebugString();
   }
 }
+
 }  // namespace
 
 TensorHandle::PackedTensorHandleData::PackedTensorHandleData(
@@ -80,34 +83,36 @@ TensorHandle::PackedTensorHandleData::~PackedTensorHandleData() {
   }
 }
 
-Status TensorHandle::PackedTensorHandleData::Shape(TensorShape* shape) const {
+absl::Status TensorHandle::PackedTensorHandleData::Shape(
+    TensorShape* shape) const {
   *shape = shape_;
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status TensorHandle::PackedTensorHandleData::NumDims(int* num_dims) const {
+absl::Status TensorHandle::PackedTensorHandleData::NumDims(
+    int* num_dims) const {
   *num_dims = shape_.dims();
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status TensorHandle::PackedTensorHandleData::Dim(int dim_index,
-                                                 int64_t* dim) const {
+absl::Status TensorHandle::PackedTensorHandleData::Dim(int dim_index,
+                                                       int64_t* dim) const {
   *dim = shape_.dim_size(dim_index);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status TensorHandle::PackedTensorHandleData::NumElements(
+absl::Status TensorHandle::PackedTensorHandleData::NumElements(
     int64_t* num_elements) const {
   *num_elements = shape_.num_elements();
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status TensorHandle::PackedTensorHandleData::Unprotect() {
+absl::Status TensorHandle::PackedTensorHandleData::Unprotect() {
   for (auto* handle : handles_) {
     TF_RETURN_IF_ERROR(
         std::visit([](auto& data) { return data.Unprotect(); }, handle->data_));
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 bool TensorHandle::PackedTensorHandleData::IsReady() const {
@@ -125,7 +130,7 @@ bool TensorHandle::PackedTensorHandleData::IsReady() const {
   return true;
 }
 
-Status TensorHandle::PackedTensorHandleData::WaitReady(
+absl::Status TensorHandle::PackedTensorHandleData::WaitReady(
     const char* caller) const {
   {
     tf_shared_lock l(mu_);
@@ -136,10 +141,10 @@ Status TensorHandle::PackedTensorHandleData::WaitReady(
   for (auto* handle : handles_) {
     TF_RETURN_IF_ERROR(handle->WaitReady(caller));
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-void TensorHandle::PackedTensorHandleData::Poison(Status status) {
+void TensorHandle::PackedTensorHandleData::Poison(absl::Status status) {
   mutex_lock l(mu_);
   is_poisoned_ = status;
 }
@@ -159,14 +164,14 @@ int TensorHandle::PackedTensorHandleData::NumPackedHandles() const {
   return handles_.size();
 }
 
-Status TensorHandle::PackedTensorHandleData::ExtractPackedHandle(
+absl::Status TensorHandle::PackedTensorHandleData::ExtractPackedHandle(
     const int index, TensorHandle** handle) const {
   if (index < 0 || index >= handles_.size()) {
     return errors::InvalidArgument("Expect an index within [0, ",
                                    handles_.size(), "), but got ", index);
   }
   *handle = handles_.at(index);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 void TensorHandle::SetResourceHandleDtypeAndShape(
@@ -174,7 +179,7 @@ void TensorHandle::SetResourceHandleDtypeAndShape(
   handle_dtypes_and_shapes_ = std::move(dtypes_and_shapes);
 }
 
-Status TensorHandle::GetResourceHandleDtypesAndShapes(
+absl::Status TensorHandle::GetResourceHandleDtypesAndShapes(
     std::vector<DtypeAndPartialTensorShape>* result) {
   if (dtype != DT_RESOURCE) {
     return errors::InvalidArgument(
@@ -185,17 +190,18 @@ Status TensorHandle::GetResourceHandleDtypesAndShapes(
 
   if (Type() != LOCAL) {
     *result = handle_dtypes_and_shapes_;
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   // Wait for this TensorHandle to be ready.
-  profiler::TraceMe activity("TensorHandle::GetResourceHandleInfo WaitReady",
-                             profiler::TraceMeLevel::kVerbose);
+  tsl::profiler::TraceMe activity(
+      "TensorHandle::GetResourceHandleInfo WaitReady",
+      tsl::profiler::TraceMeLevel::kVerbose);
   auto& data = std::get<LocalTensorHandleData>(data_);
   TF_RETURN_IF_ERROR(data.WaitReady("TensorHandle::GetResourceHandleInfo"));
 
   *result = handle_dtypes_and_shapes_;
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 int TensorHandle::NumPackedHandles() const {
@@ -205,8 +211,8 @@ int TensorHandle::NumPackedHandles() const {
   return std::get<PackedTensorHandleData>(data_).NumPackedHandles();
 }
 
-Status TensorHandle::ExtractPackedHandle(const int index,
-                                         TensorHandle** handle) const {
+absl::Status TensorHandle::ExtractPackedHandle(const int index,
+                                               TensorHandle** handle) const {
   if (Type() != PACKED) {
     return errors::Internal("Invalid ExtractPackedHandleOnDevice call on a",
                             TypeString(), " handle: ", this);
@@ -276,7 +282,6 @@ TensorHandle::TensorHandle(tensorflow::Tensor&& t, Device* d, Device* op_device,
            << " tensor: " << t.DeviceSafeDebugString();
 }
 
-
 TensorHandle* TensorHandle::CreateEmptyLocalHandle(Device* d, Device* op_device,
                                                    Device* resource_device,
                                                    tensorflow::DataType dtype,
@@ -300,12 +305,10 @@ TensorHandle::TensorHandle(Device* d, Device* op_device,
            << " device: " << SafeDeviceDebugString(device_);
 }
 
-Status TensorHandle::CreatePackedHandle(std::vector<TensorHandle*>&& handles,
-                                        const tensorflow::DataType dtype,
-                                        const tensorflow::TensorShape& shape,
-                                        const string& device_name,
-                                        EagerContext* ctx,
-                                        TensorHandle** packed_handle) {
+absl::Status TensorHandle::CreatePackedHandle(
+    std::vector<TensorHandle*>&& handles, const tensorflow::DataType dtype,
+    const tensorflow::TensorShape& shape, const string& device_name,
+    EagerContext* ctx, TensorHandle** packed_handle) {
   if (handles.empty()) {
     return errors::InvalidArgument("Handles should not be empty.");
   }
@@ -329,12 +332,12 @@ Status TensorHandle::CreatePackedHandle(std::vector<TensorHandle*>&& handles,
       new TensorHandle(std::move(handles), composite_device, dtype, shape, ctx);
   (*packed_handle)
       ->SetResourceHandleDtypeAndShape(std::move(dtypes_and_shapes));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status TensorHandle::CreatePackedHandle(std::vector<TensorHandle*>&& handles,
-                                        EagerContext* ctx,
-                                        TensorHandle** packed_handle) {
+absl::Status TensorHandle::CreatePackedHandle(
+    std::vector<TensorHandle*>&& handles, EagerContext* ctx,
+    TensorHandle** packed_handle) {
   if (handles.empty()) {
     return errors::InvalidArgument("Handles should not be empty.");
   }
@@ -431,7 +434,7 @@ bool TensorHandle::IsReady() const {
   return std::visit([](auto& data) { return data.IsReady(); }, data_);
 }
 
-Status TensorHandle::WaitReady(const char* caller) const {
+absl::Status TensorHandle::WaitReady(const char* caller) const {
   return std::visit([caller](auto& data) { return data.WaitReady(caller); },
                     data_);
 }
@@ -456,7 +459,7 @@ string TensorHandle::TypeString() const {
   }
 }
 
-Status TensorHandle::Tensor(const tensorflow::Tensor** t) const {
+absl::Status TensorHandle::Tensor(const tensorflow::Tensor** t) const {
   DVLOG(3) << "Tensor on TensorHandle: " << this;
 
   if (Type() != LOCAL) {
@@ -468,8 +471,8 @@ Status TensorHandle::Tensor(const tensorflow::Tensor** t) const {
   return data.Tensor(t);
 }
 
-Status TensorHandle::TensorFromDevice(const Device* d,
-                                      const tensorflow::Tensor** t) const {
+absl::Status TensorHandle::TensorFromDevice(
+    const Device* d, const tensorflow::Tensor** t) const {
   DVLOG(3) << "TensorFromDevice on TensorHandle: " << this << " device: " << d;
 
   if (d == device_) {
@@ -493,7 +496,8 @@ Status TensorHandle::TensorFromDevice(const Device* d,
   return mirror.Tensor(t);
 }
 
-Status TensorHandle::TensorValue(const Device* d, tensorflow::TensorValue* t) {
+absl::Status TensorHandle::TensorValue(const Device* d,
+                                       tensorflow::TensorValue* t) {
   DVLOG(3) << "TensorValue on TensorHandle: " << this << " device: " << d;
 
   if (d == device_) {
@@ -517,7 +521,7 @@ Status TensorHandle::TensorValue(const Device* d, tensorflow::TensorValue* t) {
   return mirror.TensorValue(t);
 }
 
-Status TensorHandle::WaitUnknownDevice() const {
+absl::Status TensorHandle::WaitUnknownDevice() const {
   if (unknown_device_) {
     TF_RETURN_IF_ERROR(std::visit(
         [](auto& data) {
@@ -525,28 +529,27 @@ Status TensorHandle::WaitUnknownDevice() const {
         },
         data_));
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 Device* TensorHandle::DeviceOrHostCPU(const EagerContext& ctx) const {
   return (device_ == nullptr) ? ctx.HostCPU() : device_;
 }
 
-Status TensorHandle::Shape(tensorflow::TensorShape* shape) {
+absl::Status TensorHandle::Shape(tensorflow::TensorShape* shape) {
   if (!IsReady() && inference_shape_.IsFullyDefined()) {
     bool fill = inference_shape_.AsTensorShape(shape);
     DCHECK(fill);
-    return OkStatus();
+    return absl::OkStatus();
   } else {
     return std::visit([shape](auto& data) { return data.Shape(shape); }, data_);
   }
 }
 
-Status TensorHandle::InferenceShape(
+absl::Status TensorHandle::InferenceShape(
     shape_inference::InferenceContext* const inference_context,
     shape_inference::ShapeHandle* shape_handle) {
   if (IsReady()) {
-    TF_RETURN_IF_ERROR(is_poisoned_);
     std::vector<shape_inference::DimensionHandle> dims_handle;
     int num_dims;
     TF_RETURN_IF_ERROR(NumDims(&num_dims));
@@ -556,11 +559,11 @@ Status TensorHandle::InferenceShape(
       dims_handle.push_back(inference_context->MakeDim(dims));
     }
     *shape_handle = inference_context->MakeShape(dims_handle);
-    return OkStatus();
+    return absl::OkStatus();
   } else {
     if (inference_shape_.unknown_rank()) {
       *shape_handle = inference_context->UnknownShape();
-      return OkStatus();
+      return absl::OkStatus();
     }
     std::vector<shape_inference::DimensionHandle> dims_handle(
         inference_shape_.dims());
@@ -568,7 +571,7 @@ Status TensorHandle::InferenceShape(
       dims_handle[i] = inference_context->MakeDim(inference_shape_.dim_size(i));
     }
     *shape_handle = inference_context->MakeShape(dims_handle);
-    return OkStatus();
+    return absl::OkStatus();
   }
 }
 
@@ -591,10 +594,9 @@ void TensorHandle::SetInferenceShape(
   TF_DCHECK_OK(s);
 }
 
-Status TensorHandle::CopyInferenceShape(TensorHandle* other) {
+absl::Status TensorHandle::CopyInferenceShape(TensorHandle* other) {
   if (IsReady()) {
-    TF_RETURN_IF_ERROR(is_poisoned_);
-    return OkStatus();
+    return absl::OkStatus();
   }
   if (other->IsReady()) {
     TensorShape other_shape;
@@ -603,45 +605,45 @@ Status TensorHandle::CopyInferenceShape(TensorHandle* other) {
   } else {
     inference_shape_ = other->inference_shape_;
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status TensorHandle::Shape(tensorflow::PartialTensorShape* shape) const {
+absl::Status TensorHandle::Shape(tensorflow::PartialTensorShape* shape) const {
   DCHECK(shape != nullptr);
   if (!IsReady() && !inference_shape_.unknown_rank()) {
     *shape = inference_shape_;
-    return OkStatus();
+    return absl::OkStatus();
   } else {
     auto result = std::visit(
         [](auto& data) {
           TensorShape shape;
-          Status s = data.Shape(&shape);
+          absl::Status s = data.Shape(&shape);
           return std::make_pair(shape, s);
         },
         data_);
     TF_RETURN_IF_ERROR(result.second);
     *shape = result.first;
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status TensorHandle::NumDims(int* num_dims) const {
+absl::Status TensorHandle::NumDims(int* num_dims) const {
   DCHECK(num_dims != nullptr);
   if (!IsReady() && !inference_shape_.unknown_rank()) {
     *num_dims = inference_shape_.dims();
-    return OkStatus();
+    return absl::OkStatus();
   } else {
     return std::visit([num_dims](auto& data) { return data.NumDims(num_dims); },
                       data_);
   }
 }
 
-Status TensorHandle::Dim(int dim_index, int64_t* dim) const {
+absl::Status TensorHandle::Dim(int dim_index, int64_t* dim) const {
   DCHECK(dim != nullptr);
   if (!IsReady() && !inference_shape_.unknown_rank() &&
       inference_shape_.dim_size(dim_index) != -1) {
     *dim = inference_shape_.dim_size(dim_index);
-    return OkStatus();
+    return absl::OkStatus();
   } else {
     return std::visit(
         [dim_index, dim](auto& data) { return data.Dim(dim_index, dim); },
@@ -649,11 +651,11 @@ Status TensorHandle::Dim(int dim_index, int64_t* dim) const {
   }
 }
 
-Status TensorHandle::NumElements(int64_t* num_elements) const {
+absl::Status TensorHandle::NumElements(int64_t* num_elements) const {
   DCHECK(num_elements != nullptr);
   if (!IsReady() && inference_shape_.IsFullyDefined()) {
     *num_elements = inference_shape_.num_elements();
-    return OkStatus();
+    return absl::OkStatus();
   } else {
     return std::visit(
         [num_elements](auto& data) { return data.NumElements(num_elements); },
@@ -661,7 +663,7 @@ Status TensorHandle::NumElements(int64_t* num_elements) const {
   }
 }
 
-Status TensorHandle::Unprotect(const Device* d) {
+absl::Status TensorHandle::Unprotect(const Device* d) {
   DVLOG(3) << "Unprotect on TensorHandle: " << this << " device: " << d;
 
   if (d == device_) {
@@ -687,7 +689,7 @@ bool TensorHandle::HasLocalMirror(const Device* d) const {
   return local_mirrors_.find(d) != local_mirrors_.end();
 }
 
-Status TensorHandle::AddEmptyLocalMirror(const Device* d) {
+absl::Status TensorHandle::AddEmptyLocalMirror(const Device* d) {
   DVLOG(3) << "AddEmptyLocalMirror on TensorHandle: " << this
            << " device: " << d;
 
@@ -703,25 +705,42 @@ Status TensorHandle::AddEmptyLocalMirror(const Device* d) {
   local_mirrors_.emplace(std::piecewise_construct, std::forward_as_tuple(d),
                          std::forward_as_tuple());
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 #if !defined(IS_MOBILE_PLATFORM)
-Status TensorHandle::RemoteAddress(const Device* d, const bool wait_until_ready,
-                                   int64_t* op_id, int32* output_num) const {
+absl::Status TensorHandle::RemoteAddress(const Device* d,
+                                         const bool wait_until_ready,
+                                         int64_t* op_id,
+                                         int32* output_num) const {
   DVLOG(3) << "RemoteAddress on TensorHandle: " << this << " device: " << d
            << " " << d->name();
 
+  const tensorflow::RemoteTensorHandleData* remote_data = nullptr;
+
+  // Waiting on a different device handle.
   if (d != device_) {
     tf_shared_lock l(mu_);
     auto mirror = remote_mirrors_.find(d->name());
     if (mirror != remote_mirrors_.end()) {
-      return mirror->second.OpIdAndOutputNum(wait_until_ready, op_id,
-                                             output_num);
+      remote_data = &mirror->second;
+    } else {
+      return errors::FailedPrecondition(
+          "Could not find remote mirror for specified device");
     }
+  }
 
-    return errors::FailedPrecondition(
-        "Could not find remote mirror for specified device");
+  if (remote_data != nullptr) {
+    auto status =
+        remote_data->OpIdAndOutputNum(wait_until_ready, op_id, output_num);
+    if (!status.ok()) {
+      return errors::Internal(
+          absl::StrCat("Remote address looked up from remote mirrors found to "
+                       "be poisoned with status ",
+                       status.ToString()));
+    } else {
+      return absl::OkStatus();
+    }
   }
 
   if (Type() != REMOTE) {
@@ -729,7 +748,13 @@ Status TensorHandle::RemoteAddress(const Device* d, const bool wait_until_ready,
   }
 
   auto& data = std::get<RemoteTensorHandleData>(data_);
-  return data.OpIdAndOutputNum(wait_until_ready, op_id, output_num);
+  auto status = data.OpIdAndOutputNum(wait_until_ready, op_id, output_num);
+  if (!status.ok()) {
+    return errors::Internal(
+        "Remote address looked up from remote data found to be poisoned");
+  } else {
+    return absl::OkStatus();
+  }
 }
 
 bool TensorHandle::HasRemoteMirror(const Device* d,
@@ -767,10 +792,11 @@ bool TensorHandle::HasResourceShapeMirror(const Device* d,
   return false;
 }
 
-Status TensorHandle::AddUnshapedRemoteMirror(const Device* d, int64_t op_id,
-                                             int output_num,
-                                             const string& remote_task,
-                                             EagerContext* ctx) {
+absl::Status TensorHandle::AddUnshapedRemoteMirror(const Device* d,
+                                                   int64_t op_id,
+                                                   int output_num,
+                                                   const string& remote_task,
+                                                   EagerContext* ctx) {
   DVLOG(3) << "AddUnshapedRemoteMirror on TensorHandle: " << this
            << " device: " << d << " " << d->name() << " op_id: " << op_id
            << " output_num: " << output_num;
@@ -778,8 +804,10 @@ Status TensorHandle::AddUnshapedRemoteMirror(const Device* d, int64_t op_id,
   mutex_lock l(mu_);
   auto remote_mirror = remote_mirrors_.find(d->name());
   if (remote_mirror != remote_mirrors_.end()) {
-    if (remote_mirror->second.context_view_id() >= ctx->GetContextId()) {
-      return errors::Internal("Attempted to duplicate a remote mirror.");
+    if (remote_mirror->second.context_view_id() > ctx->GetContextId()) {
+      return errors::Internal(
+          "Attempted to duplicate a remote mirror with inconsistent "
+          "arguments.");
     }
     // Remove stale mirror
     remote_mirrors_.erase(remote_mirror);
@@ -789,18 +817,28 @@ Status TensorHandle::AddUnshapedRemoteMirror(const Device* d, int64_t op_id,
       std::piecewise_construct, std::forward_as_tuple(d->name()),
       std::forward_as_tuple(op_id, output_num, remote_task, ctx));
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status TensorHandle::AddResourceShapeMirror(const Device* d, int64_t op_id,
-                                            int output_num, EagerContext* ctx) {
+absl::Status TensorHandle::AddResourceShapeMirror(const Device* d,
+                                                  int64_t op_id, int output_num,
+                                                  EagerContext* ctx) {
   DVLOG(3) << "AddResourceShapeMirror on TensorHandle: " << this;
 
   mutex_lock l(mu_);
   auto mirror = resource_shape_mirrors_.find(d->name());
   if (mirror != resource_shape_mirrors_.end()) {
     if (mirror->second.context_view_id() == ctx->GetContextViewId()) {
-      return errors::Internal(
+      int64_t existing_op_id;
+      int existing_output_num;
+      TF_RETURN_IF_ERROR(mirror->second.OpIdAndOutputNum(false, &existing_op_id,
+                                                         &existing_output_num));
+      if (op_id == existing_op_id && output_num == existing_output_num) {
+        // The mirror has already been added by a concurrent op/function
+        // invocation.
+        return absl::OkStatus();
+      }
+      return absl::InternalError(
           "Attempted to duplicate a resource shape mirror.");
     }
     // Remove stale mirror
@@ -812,18 +850,19 @@ Status TensorHandle::AddResourceShapeMirror(const Device* d, int64_t op_id,
       std::forward_as_tuple(op_id, output_num, ctx->GetContextViewId(),
                             /*is_ready=*/true));
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status TensorHandle::SetRemoteShape(const TensorShape& shape, const Device* d,
-                                    uint64 context_view_id) {
+absl::Status TensorHandle::SetRemoteShape(const TensorShape& shape,
+                                          const Device* d,
+                                          uint64 context_view_id) {
   return SetRemoteShapeAndDevice(shape, d, context_view_id, /*op_device=*/"");
 }
 
-Status TensorHandle::SetRemoteShapeAndDevice(const TensorShape& shape,
-                                             const Device* d,
-                                             uint64 context_view_id,
-                                             string op_device) {
+absl::Status TensorHandle::SetRemoteShapeAndDevice(const TensorShape& shape,
+                                                   const Device* d,
+                                                   uint64 context_view_id,
+                                                   string op_device) {
   DVLOG(3) << "SetRemoteShape on TensorHandle: " << this << " device: " << d
            << " " << d->name();
 
@@ -831,11 +870,18 @@ Status TensorHandle::SetRemoteShapeAndDevice(const TensorShape& shape,
     tf_shared_lock l(mu_);
     auto remote_mirror = remote_mirrors_.find(d->name());
     if (remote_mirror == remote_mirrors_.end()) {
-      return OkStatus();
+      return absl::OkStatus();
     }
     auto& mirror = remote_mirror->second;
     if (mirror.context_view_id() == context_view_id) {
-      return mirror.SetShape(shape);
+      auto status = mirror.SetShape(shape);
+      if (!status.ok()) {
+        // TODO(b/307761242) Return an error or assert that this condition will
+        // never occur.
+        LOG(ERROR) << "SetShape returned " << status.message()
+                   << ". This should never occur.";
+      }
+      return status;
     } else if (mirror.context_view_id() < context_view_id) {
       return errors::Internal(
           absl::Substitute("Unexpected context_view_id ($0) which should not "
@@ -846,7 +892,7 @@ Status TensorHandle::SetRemoteShapeAndDevice(const TensorShape& shape,
       LOG(WARNING) << "SetRemoteShape is ignored for a remote mirror that is "
                       "associated with a newer context_view_id.";
     }
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   if (Type() != REMOTE) {
@@ -864,7 +910,14 @@ Status TensorHandle::SetRemoteShapeAndDevice(const TensorShape& shape,
   // consuming op/function device, and we (for now) have to aggressively
   // invalidate those copies to avoid any false positives during cluster update.
   if (op_device.empty()) {
-    return data.SetShape(shape);
+    auto status = data.SetShape(shape);
+    if (!status.ok()) {
+      // TODO(anshumang) Return an error or assert that this condition will
+      // never occur.
+      LOG(ERROR) << "SetShape returned " << status.message()
+                 << ". This should never occur.";
+    }
+    return status;
   } else {
     if (!unknown_device_) {
       return errors::Internal("Cannot reset known devices.");
@@ -882,11 +935,18 @@ Status TensorHandle::SetRemoteShapeAndDevice(const TensorShape& shape,
           "Unable to find remote task corresponding to device ",
           device->name());
     }
-    return data.SetShapeAndRemoteTask(shape, remote_task);
+    auto status = data.SetShapeAndRemoteTask(shape, remote_task);
+    if (!status.ok()) {
+      // TODO(anshumang) Return an error or assert that this condition will
+      // never occur.
+      LOG(ERROR) << "SetShape returned " << status
+                 << ". This should never occur.";
+    }
+    return status;
   }
 }
 
-void TensorHandle::PoisonRemote(Status status, const Device* d,
+void TensorHandle::PoisonRemote(absl::Status status, const Device* d,
                                 uint64 context_view_id) {
   DVLOG(3) << "PoisonRemote on TensorHandle: " << this << " device: " << d
            << " " << d->name();
@@ -909,8 +969,8 @@ void TensorHandle::PoisonRemote(Status status, const Device* d,
 }
 #endif
 
-Status TensorHandle::AddLocalMirror(tensorflow::Tensor&& tensor,
-                                    const Device* d) {
+absl::Status TensorHandle::AddLocalMirror(tensorflow::Tensor&& tensor,
+                                          const Device* d) {
   if (d == device_) {
     return errors::Internal(
         "Local mirror assign conflicts with primary device.");
@@ -924,10 +984,10 @@ Status TensorHandle::AddLocalMirror(tensorflow::Tensor&& tensor,
     return errors::AlreadyExists("Attempted to add existing mirror.");
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status TensorHandle::SetTensor(tensorflow::Tensor&& t, const Device* d) {
+absl::Status TensorHandle::SetTensor(tensorflow::Tensor&& t, const Device* d) {
   DVLOG(3) << "SetTensor on TensorHandle: " << this << " device: " << d;
 
   if (d == device_) {
@@ -951,10 +1011,10 @@ Status TensorHandle::SetTensor(tensorflow::Tensor&& t, const Device* d) {
     return mirror.SetTensor(std::move(t));
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-void TensorHandle::Poison(Status status, const Device* d) {
+void TensorHandle::Poison(absl::Status status, const Device* d) {
   DVLOG(3) << "Poison on TensorHandle: " << this << " device: " << d;
 
   if (d == device_) {
@@ -972,9 +1032,9 @@ void TensorHandle::Poison(Status status, const Device* d) {
   }
 }
 
-Status TensorHandle::CopyToDevice(const EagerContext& ctx,
-                                  tensorflow::Device* d,
-                                  tensorflow::Tensor* output) const {
+absl::Status TensorHandle::CopyToDevice(const EagerContext& ctx,
+                                        tensorflow::Device* d,
+                                        tensorflow::Tensor* output) const {
   tensorflow::Device* dstd = (d == nullptr) ? ctx.HostCPU() : d;
   tensorflow::Device* srcd = DeviceOrHostCPU(ctx);
   const bool dst_cpu = dstd->tensorflow_accelerator_device_info() == nullptr;
@@ -986,7 +1046,7 @@ Status TensorHandle::CopyToDevice(const EagerContext& ctx,
   TF_RETURN_IF_ERROR(Tensor(&src));
   if (is_same_device) {
     *output = *src;
-    return OkStatus();
+    return absl::OkStatus();
   }
   if (!dst_cpu && (src->dtype() != tensorflow::DT_VARIANT &&
                    !tensorflow::DataTypeCanUseMemcpy(src->dtype()))) {
@@ -1003,7 +1063,7 @@ Status TensorHandle::CopyToDevice(const EagerContext& ctx,
   tensorflow::Tensor dst(dstd->GetAllocator(attr), src->dtype(), src->shape());
   if (src->shape().num_elements() == 0) {
     *output = dst;
-    return OkStatus();
+    return absl::OkStatus();
   }
   tensorflow::DeviceContext* src_device_context = nullptr;
   if (!src_cpu) {
@@ -1012,7 +1072,14 @@ Status TensorHandle::CopyToDevice(const EagerContext& ctx,
   }
   tensorflow::DeviceContext* dst_device_context = nullptr;
   if (!dst_cpu) {
-    if (dstd_info->use_pjrt_tensor_buffer) {
+    // PJRT will soon pack int4 tensors when transferring them to device (once
+    // XLA int4 support is implemented), but TF currently represents int4 as
+    // unpacked, so do not use PJRT for int4 tensors.
+    // TODO(b/226482736): Either pack int4, or support unpacked int4 tensors in
+    // PJRT. Also update beginning of comment from future to present tense once
+    // PJRT packs int4 tensors.
+    if (dstd_info->use_pjrt_tensor_buffer && DataType() != DT_INT4 &&
+        DataType() != DT_UINT4) {
       dst_device_context = dstd_info->pjrt_context;
     } else {
       dst_device_context = dstd_info->default_context;
@@ -1027,19 +1094,19 @@ Status TensorHandle::CopyToDevice(const EagerContext& ctx,
   // nothing to do with this tensor to complete).
   TF_RETURN_IF_ERROR(srcd->Sync());
   tensorflow::Notification n;
-  tensorflow::Status status;
+  absl::Status status;
   tensorflow::CopyTensor::ViaDMA("copy", src_device_context, dst_device_context,
                                  srcd, dstd, tensorflow::AllocatorAttributes(),
                                  tensorflow::AllocatorAttributes(), src, &dst,
                                  0 /*dev_to_dev_stream_index*/,
-                                 [&status, &n](const tensorflow::Status& s) {
+                                 [&status, &n](const absl::Status& s) {
                                    status = s;
                                    n.Notify();
                                  });
   n.WaitForNotification();
   if (status.ok()) {
     *output = dst;
-    return OkStatus();
+    return absl::OkStatus();
   }
   return status;
 }
@@ -1056,27 +1123,27 @@ Device* GetResourceDevice(const ResourceHandle& handle, EagerContext* ctx) {
   return device;
 }
 
-const char* TensorHandle::DeviceName(Status* status) const {
+const char* TensorHandle::DeviceName(absl::Status* status) const {
   status->Update(WaitUnknownDevice());
   tensorflow::Device* d = op_device();
   return (d == nullptr) ? "/job:localhost/replica:0/task:0/device:CPU:0"
                         : d->name().c_str();
 }
 
-const char* TensorHandle::BackingDeviceName(Status* status) const {
+const char* TensorHandle::BackingDeviceName(absl::Status* status) const {
   status->Update(WaitUnknownDevice());
   tensorflow::Device* d = device();
   return (d == nullptr) ? "/job:localhost/replica:0/task:0/device:CPU:0"
                         : d->name().c_str();
 }
 
-const char* TensorHandle::DeviceType(Status* status) const {
+const char* TensorHandle::DeviceType(absl::Status* status) const {
   status->Update(WaitUnknownDevice());
   tensorflow::Device* d = op_device();
   return (d == nullptr) ? "CPU" : d->parsed_name().type.c_str();
 }
 
-int TensorHandle::DeviceId(Status* status) const {
+int TensorHandle::DeviceId(absl::Status* status) const {
   status->Update(WaitUnknownDevice());
   tensorflow::Device* d = op_device();
   return (d == nullptr) ? 0 : d->parsed_name().id;
