@@ -17,20 +17,28 @@ limitations under the License.
 
 #include <iostream>
 #include <memory>
+#include <string>
 #include <utility>
 #include <vector>
 
+#include "absl/cleanup/cleanup.h"
 #include "tensorflow/core/common_runtime/composite_device.h"
 #include "tensorflow/core/common_runtime/device_mgr.h"
+#include "tensorflow/core/framework/allocator.h"
+#include "tensorflow/core/framework/device.h"
 #include "tensorflow/core/framework/tensor_shape.h"
 #include "tensorflow/core/lib/core/status_test_util.h"
 #include "tensorflow/core/platform/random.h"
+#include "tensorflow/core/platform/status.h"
 #include "tensorflow/core/platform/status_matchers.h"
 #include "tensorflow/core/platform/test.h"
 
 namespace tensorflow {
 namespace {
 
+using tensorflow::testing::StatusIs;
+using ::testing::HasSubstr;
+
 TEST(TensorHandle_ShapeTest, AsyncShape) {
   Tensor t(DT_UINT16, TensorShape({2, 2}));
   EXPECT_TRUE(t.shape().IsSameSize(TensorShape({2, 2})));
@@ -47,10 +55,13 @@ TEST(TensorHandle_ShapeTest, AsyncShape) {
       tensorflow::ContextDevicePlacementPolicy::DEVICE_PLACEMENT_SILENT, false,
       &device_mgr, false, nullptr, nullptr, nullptr,
       /*run_eager_op_as_function=*/true);
+  absl::Cleanup ctx_cleanup = [&]() { ctx->Unref(); };
   TensorHandle* sync_th =
       TensorHandle::CreateLocalHandle(std::move(t), nullptr, nullptr, ctx);
+  absl::Cleanup sync_th_cleanup = [&]() { sync_th->Unref(); };
   TensorHandle* async_th = TensorHandle::CreateEmptyLocalHandle(
       nullptr, nullptr, nullptr, DataType::DT_UINT16, ctx);
+  absl::Cleanup async_th_cleanup = [&]() { async_th->Unref(); };
 
   EXPECT_TRUE(async_th->CopyInferenceShape(sync_th).ok());
 
@@ -67,25 +78,23 @@ TEST(TensorHandle_ShapeTest, AsyncShape) {
   int64_t num_elements = -1;
   EXPECT_TRUE(async_th->NumElements(&num_elements).ok());
   EXPECT_EQ(num_elements, 4);
-
-  sync_th->Unref();
-  async_th->Unref();
-  ctx->Unref();
 }
 
-static Device* CreateDevice(const char* type, const char* name,
-                            bool is_local = true) {
-  class FakeDevice : public Device {
-   public:
-    explicit FakeDevice(const DeviceAttributes& attr, bool is_local)
-        : Device(nullptr, attr), is_local_(is_local) {}
-    Status Sync() override { return OkStatus(); }
-    Allocator* GetAllocator(AllocatorAttributes) override { return nullptr; }
-    bool IsLocal() const override { return is_local_; }
+class FakeDevice : public Device {
+ public:
+  explicit FakeDevice(const DeviceAttributes& attr, bool is_local)
+      : Device(nullptr, attr), is_local_(is_local) {}
+  absl::Status Sync() override { return absl::OkStatus(); }
+  Allocator* GetAllocator(AllocatorAttributes) override { return nullptr; }
+  bool IsLocal() const override { return is_local_; }
 
-   private:
-    const bool is_local_;
-  };
+ private:
+  const bool is_local_;
+};
+
+static std::unique_ptr<FakeDevice> CreateDevice(const char* type,
+                                                const char* name,
+                                                bool is_local = true) {
   DeviceAttributes attr;
   attr.set_name(name);
   attr.set_device_type(type);
@@ -94,7 +103,7 @@ static Device* CreateDevice(const char* type, const char* name,
     incarnation = random::New64();
   }
   attr.set_incarnation(incarnation);
-  return new FakeDevice(attr, is_local);
+  return std::make_unique<FakeDevice>(attr, is_local);
 }
 
 }  // namespace
@@ -103,9 +112,9 @@ class PackedTensorHandleTest : public ::testing::Test {
  public:
   PackedTensorHandleTest() {
     std::vector<std::unique_ptr<Device>> devices;
-    devices.emplace_back(CreateDevice("CPU", host_name_));
+    devices.push_back(CreateDevice("CPU", host_name_));
     for (const char* name : device_names_) {
-      devices.emplace_back(CreateDevice("GPU", name));
+      devices.push_back(CreateDevice("GPU", name));
     }
     device_mgr_ = new StaticDeviceMgr(std::move(devices));
 
@@ -132,7 +141,7 @@ class PackedTensorHandleTest : public ::testing::Test {
   }
 
   bool IsReady(TensorHandle* handle) const { return handle->IsReady(); }
-  Status WaitReady(TensorHandle* handle) const {
+  absl::Status WaitReady(TensorHandle* handle) const {
     return handle->WaitReady("Test");
   }
 
@@ -160,12 +169,14 @@ TEST_F(PackedTensorHandleTest, PackedHandle) {
   Device* d0 = ListGPUDevices().at(0);
   TensorHandle* h0 =
       TensorHandle::CreateLocalHandle(std::move(t0), d0, d0, d0, context());
+  absl::Cleanup h0_cleanup = [&]() { h0->Unref(); };
   h0->SetResourceHandleDtypeAndShape({dtype_and_shape});
   handles.push_back(h0);
   Tensor t1(dtype, shape);
   Device* d1 = ListGPUDevices().at(1);
   TensorHandle* h1 =
       TensorHandle::CreateLocalHandle(std::move(t1), d1, d1, d1, context());
+  absl::Cleanup h1_cleanup = [&]() { h1->Unref(); };
   h1->SetResourceHandleDtypeAndShape({dtype_and_shape});
   handles.push_back(h1);
 
@@ -174,20 +185,18 @@ TEST_F(PackedTensorHandleTest, PackedHandle) {
   Device* d2 = ListGPUDevices().at(2);
   TensorHandle* h2 = TensorHandle::CreateUnshapedRemoteHandle(
       /*op_id=*/0, /*output_num=*/0, remote_task, dtype, d2, context());
+  absl::Cleanup h2_cleanup = [&]() { h2->Unref(); };
   handles.push_back(h2);
   Device* d3 = ListGPUDevices().at(3);
   TensorHandle* h3 = TensorHandle::CreateUnshapedRemoteHandle(
       /*op_id=*/1, /*output_num=*/0, remote_task, dtype, d3, context());
+  absl::Cleanup h3_cleanup = [&]() { h3->Unref(); };
   handles.push_back(h3);
 
   TensorHandle* packed_handle = nullptr;
   TF_EXPECT_OK(TensorHandle::CreatePackedHandle(std::move(handles), context(),
                                                 &packed_handle));
-
-  h0->Unref();
-  h1->Unref();
-  h2->Unref();
-  h3->Unref();
+  absl::Cleanup packed_handle_cleanup = [&]() { packed_handle->Unref(); };
 
   EXPECT_EQ(packed_handle->NumPackedHandles(), 4);
   EXPECT_EQ(packed_handle->Type(), TensorHandle::PACKED);
@@ -225,8 +234,6 @@ TEST_F(PackedTensorHandleTest, PackedHandle) {
   TF_ASSERT_OK(h3->SetRemoteShape(shape, ListGPUDevices().at(3),
                                   context()->GetContextViewId()));
   EXPECT_TRUE(IsReady(packed_handle));
-
-  packed_handle->Unref();
 }
 
 TEST_F(PackedTensorHandleTest, PackedSingleHandle) {
@@ -237,12 +244,13 @@ TEST_F(PackedTensorHandleTest, PackedSingleHandle) {
   Device* d = ListGPUDevices().at(0);
   TensorHandle* h =
       TensorHandle::CreateLocalHandle(std::move(t), d, d, d, context());
+  absl::Cleanup h_cleanup = [&]() { h->Unref(); };
   std::vector<TensorHandle*> handles = {h};
 
   TensorHandle* packed_handle = nullptr;
   TF_EXPECT_OK(TensorHandle::CreatePackedHandle(std::move(handles), context(),
                                                 &packed_handle));
-  h->Unref();
+  absl::Cleanup packed_handle_cleanup = [&]() { packed_handle->Unref(); };
 
   EXPECT_EQ(packed_handle->Type(), TensorHandle::PACKED);
   EXPECT_EQ(packed_handle->dtype, dtype);
@@ -259,7 +267,6 @@ TEST_F(PackedTensorHandleTest, PackedSingleHandle) {
   TF_ASSERT_OK(packed_handle->ExtractPackedHandle(0, &h0));
   EXPECT_EQ(h0->device(), d);
   EXPECT_TRUE(IsReady(packed_handle));
-  packed_handle->Unref();
 }
 
 TEST_F(PackedTensorHandleTest, PoisonHandle) {
@@ -270,26 +277,23 @@ TEST_F(PackedTensorHandleTest, PoisonHandle) {
   Device* d = ListGPUDevices().at(0);
   TensorHandle* h =
       TensorHandle::CreateLocalHandle(std::move(t), d, d, d, context());
+  absl::Cleanup h_cleanup = [&]() { h->Unref(); };
   std::vector<TensorHandle*> handles = {h};
 
   TensorHandle* packed_handle = nullptr;
   TF_EXPECT_OK(TensorHandle::CreatePackedHandle(std::move(handles), context(),
                                                 &packed_handle));
-  h->Unref();
+  absl::Cleanup packed_handle_cleanup = [&]() { packed_handle->Unref(); };
 
   // Should be ready on creation.
   TF_EXPECT_OK(WaitReady(packed_handle));
 
   // Poisoning the handle will make WaitReady fail.
-  tensorflow::Status fake_failure_status(absl::StatusCode::kAborted,
-                                         "Fake failure.");
+  absl::Status fake_failure_status(absl::StatusCode::kAborted, "Fake failure.");
   packed_handle->Poison(fake_failure_status, packed_handle->device());
   EXPECT_THAT(WaitReady(packed_handle),
-              tensorflow::testing::StatusIs(
-                  fake_failure_status.code(),
-                  std::string(fake_failure_status.message())));
-
-  packed_handle->Unref();
+              StatusIs(fake_failure_status.code(),
+                       std::string(fake_failure_status.message())));
 }
 
 TEST(TensorHandle_ResourceDeviceTest, OnLocalDevice) {
@@ -301,6 +305,7 @@ TEST(TensorHandle_ResourceDeviceTest, OnLocalDevice) {
       tensorflow::ContextDevicePlacementPolicy::DEVICE_PLACEMENT_SILENT, false,
       &local_device_mgr, false, nullptr, nullptr, nullptr,
       /*run_eager_op_as_function=*/true);
+  absl::Cleanup ctx_cleanup = [&]() { ctx->Unref(); };
 
   tensorflow::DataType dtype = DT_RESOURCE;
   TensorShape shape = {2};
@@ -309,6 +314,7 @@ TEST(TensorHandle_ResourceDeviceTest, OnLocalDevice) {
   Device* d = local_device_mgr.ListDevices()[0];
   TensorHandle* th =
       TensorHandle::CreateLocalHandle(std::move(t), d, d, d, ctx);
+  absl::Cleanup th_cleanup = [&]() { th->Unref(); };
   // Remote device incarnation for local resource should be 0 (invalid)
   EXPECT_EQ(0, th->resource_remote_device_incarnation());
   // Local device manager must contain the resource device.
@@ -320,9 +326,6 @@ TEST(TensorHandle_ResourceDeviceTest, OnLocalDevice) {
   StaticDeviceMgr new_device_mgr(std::move(d1));
   EXPECT_FALSE(new_device_mgr.ContainsDevice(
       th->resource_device()->attributes().incarnation()));
-
-  th->Unref();
-  ctx->Unref();
 }
 
 TEST(TensorHandle_ResourceDeviceTest, OnRemoteDevice) {
@@ -334,6 +337,7 @@ TEST(TensorHandle_ResourceDeviceTest, OnRemoteDevice) {
       tensorflow::ContextDevicePlacementPolicy::DEVICE_PLACEMENT_SILENT, false,
       &local_device_mgr, false, nullptr, nullptr, nullptr,
       /*run_eager_op_as_function=*/true);
+  absl::Cleanup ctx_cleanup = [&]() { ctx->Unref(); };
 
   std::unique_ptr<Device> d0(
       CreateDevice("CPU", "/job:worker/task:0/device:CPU:0", false));
@@ -344,22 +348,24 @@ TEST(TensorHandle_ResourceDeviceTest, OnRemoteDevice) {
 
   DynamicDeviceMgr remote_device_mgr;
   std::vector<std::unique_ptr<Device>> vector_d0;
-  vector_d0.emplace_back(std::move(d0));
+  vector_d0.push_back(std::move(d0));
   TF_ASSERT_OK(remote_device_mgr.AddDevices(std::move(vector_d0)));
 
   TensorHandle* th0 = TensorHandle::CreateUnshapedRemoteHandle(
       0, 0, "", DT_RESOURCE, d0_ptr, ctx);
+  absl::Cleanup th0_cleanup = [&]() { th0->Unref(); };
   EXPECT_TRUE(remote_device_mgr.ContainsDevice(
       th0->resource_remote_device_incarnation()));
 
   std::vector<std::unique_ptr<Device>> vector_d1;
-  vector_d1.emplace_back(std::move(d1));
+  vector_d1.push_back(std::move(d1));
   TF_ASSERT_OK(remote_device_mgr.AddDevices(std::move(vector_d1)));
   EXPECT_TRUE(remote_device_mgr.ContainsDevice(
       th0->resource_remote_device_incarnation()));
 
   TensorHandle* th1 = TensorHandle::CreateUnshapedRemoteHandle(
       0, 0, "", DT_RESOURCE, d1_ptr, ctx);
+  absl::Cleanup th1_cleanup = [&]() { th1->Unref(); };
   EXPECT_TRUE(remote_device_mgr.ContainsDevice(
       th1->resource_remote_device_incarnation()));
 
@@ -369,10 +375,6 @@ TEST(TensorHandle_ResourceDeviceTest, OnRemoteDevice) {
       th1->resource_remote_device_incarnation()));
   EXPECT_TRUE(remote_device_mgr.ContainsDevice(
       th0->resource_remote_device_incarnation()));
-
-  th0->Unref();
-  th1->Unref();
-  ctx->Unref();
 }
 
 class RemoteTensorHandleTest : public ::testing::Test {
@@ -380,7 +382,7 @@ class RemoteTensorHandleTest : public ::testing::Test {
   RemoteTensorHandleTest() {
     std::vector<std::unique_ptr<Device>> devices;
     for (const char* name : device_names_) {
-      devices.emplace_back(CreateDevice("CPU", name));
+      devices.push_back(CreateDevice("CPU", name));
     }
     device_mgr_ = new StaticDeviceMgr(std::move(devices));
 
@@ -416,11 +418,11 @@ class RemoteTensorHandleTest : public ::testing::Test {
 
 TEST_F(RemoteTensorHandleTest, UnknownRemoteDevice) {
   std::vector<std::unique_ptr<Device>> devices;
-  devices.emplace_back(
+  devices.push_back(
       CreateDevice("CPU", "/job:worker/replica:0/task:0/device:CPU:0"));
-  devices.emplace_back(
+  devices.push_back(
       CreateDevice("CPU", "/job:worker/replica:0/task:1/device:CPU:0"));
-  devices.emplace_back(
+  devices.push_back(
       CreateDevice("CPU", "/job:worker/replica:0/task:2/device:CPU:0"));
   StaticDeviceMgr device_mgr(std::move(devices));
 
@@ -431,6 +433,7 @@ TEST_F(RemoteTensorHandleTest, UnknownRemoteDevice) {
       /* device_mgr_owned= */ false, /* rendezvous= */ nullptr,
       /* cluster_flr= */ nullptr, /*collective_executor_mgr=*/nullptr,
       /*run_eager_op_as_function=*/true);
+  absl::Cleanup context_cleanup = [&]() { context->Unref(); };
 
   tensorflow::DataType dtype = DT_FLOAT;
   TensorShape shape = {};
@@ -440,26 +443,25 @@ TEST_F(RemoteTensorHandleTest, UnknownRemoteDevice) {
   TensorHandle* h = TensorHandle::CreateUnshapedRemoteHandle(
       /*op_id=*/0, /*output_num=*/0, remote_task, dtype, d1, context,
       /*unknown_device=*/true);
+  absl::Cleanup h_cleanup = [&]() { h->Unref(); };
   EXPECT_EQ(h->device(), d1);
 
   Device* d2 = device_mgr.ListDevices().at(2);
   TF_ASSERT_OK(h->SetRemoteShapeAndDevice(
       shape, d1, context->GetContextViewId(), d2->name()));
-  Status s;
+  absl::Status s;
   EXPECT_EQ(h->BackingDeviceName(&s), d2->name());
   TF_EXPECT_OK(s);
   EXPECT_EQ(h->device(), d2);
-  h->Unref();
-  context->Unref();
 }
 
 TEST_F(RemoteTensorHandleTest, PoisonRemote) {
   std::vector<std::unique_ptr<Device>> devices;
-  devices.emplace_back(
+  devices.push_back(
       CreateDevice("CPU", "/job:worker/replica:0/task:0/device:CPU:0"));
-  devices.emplace_back(
+  devices.push_back(
       CreateDevice("CPU", "/job:worker/replica:0/task:1/device:CPU:0"));
-  devices.emplace_back(
+  devices.push_back(
       CreateDevice("CPU", "/job:worker/replica:0/task:2/device:CPU:0"));
   StaticDeviceMgr device_mgr(std::move(devices));
 
@@ -470,6 +472,7 @@ TEST_F(RemoteTensorHandleTest, PoisonRemote) {
       /* device_mgr_owned= */ false, /* rendezvous= */ nullptr,
       /* cluster_flr= */ nullptr, /*collective_executor_mgr=*/nullptr,
       /*run_eager_op_as_function=*/true);
+  absl::Cleanup context_cleanup = [&]() { context->Unref(); };
 
   tensorflow::DataType dtype = DT_FLOAT;
   TensorShape shape = {};
@@ -479,30 +482,26 @@ TEST_F(RemoteTensorHandleTest, PoisonRemote) {
   TensorHandle* h = TensorHandle::CreateUnshapedRemoteHandle(
       /*op_id=*/0, /*output_num=*/0, remote_task, dtype, d1, context,
       /*unknown_device=*/true);
+  absl::Cleanup h_cleanup = [&]() { h->Unref(); };
   EXPECT_EQ(h->device(), d1);
 
-  tensorflow::Status fake_failure_status(absl::StatusCode::kAborted,
-                                         "Fake failure.");
+  absl::Status fake_failure_status(absl::StatusCode::kAborted, "Fake failure.");
   h->PoisonRemote(fake_failure_status, d1, context->GetContextViewId());
 
   Device* d2 = device_mgr.ListDevices().at(2);
   EXPECT_THAT(h->SetRemoteShapeAndDevice(shape, d1, context->GetContextViewId(),
                                          d2->name()),
-              tensorflow::testing::StatusIs(
-                  fake_failure_status.code(),
-                  std::string(fake_failure_status.message())));
-
-  h->Unref();
-  context->Unref();
+              StatusIs(fake_failure_status.code(),
+                       std::string(fake_failure_status.message())));
 }
 
 TEST_F(RemoteTensorHandleTest, PoisonRemoteMirror) {
   std::vector<std::unique_ptr<Device>> devices;
-  devices.emplace_back(
+  devices.push_back(
       CreateDevice("CPU", "/job:worker/replica:0/task:0/device:CPU:0"));
-  devices.emplace_back(
+  devices.push_back(
       CreateDevice("CPU", "/job:worker/replica:0/task:1/device:CPU:0"));
-  devices.emplace_back(
+  devices.push_back(
       CreateDevice("CPU", "/job:worker/replica:0/task:2/device:CPU:0"));
   StaticDeviceMgr device_mgr(std::move(devices));
 
@@ -513,6 +512,7 @@ TEST_F(RemoteTensorHandleTest, PoisonRemoteMirror) {
       /* device_mgr_owned= */ false, /* rendezvous= */ nullptr,
       /* cluster_flr= */ nullptr, /*collective_executor_mgr=*/nullptr,
       /*run_eager_op_as_function=*/true);
+  absl::Cleanup context_cleanup = [&]() { context->Unref(); };
 
   tensorflow::DataType dtype = DT_FLOAT;
   TensorShape shape = {};
@@ -522,6 +522,7 @@ TEST_F(RemoteTensorHandleTest, PoisonRemoteMirror) {
   TensorHandle* h = TensorHandle::CreateUnshapedRemoteHandle(
       /*op_id=*/0, /*output_num=*/0, remote_task, dtype, d1, context,
       /*unknown_device=*/true);
+  absl::Cleanup h_cleanup = [&]() { h->Unref(); };
   EXPECT_EQ(h->device(), d1);
 
   Device* d2 = device_mgr.ListDevices().at(2);
@@ -530,58 +531,80 @@ TEST_F(RemoteTensorHandleTest, PoisonRemoteMirror) {
   TF_ASSERT_OK(
       h->AddUnshapedRemoteMirror(d2, op_id, output_num, remote_task, context));
 
-  tensorflow::Status fake_failure_status(absl::StatusCode::kAborted,
-                                         "Fake failure.");
+  absl::Status fake_failure_status(absl::StatusCode::kAborted, "Fake failure.");
   h->PoisonRemote(fake_failure_status, d2, context->GetContextViewId());
 
   EXPECT_THAT(h->SetRemoteShapeAndDevice(shape, d2, context->GetContextViewId(),
                                          d2->name()),
-              tensorflow::testing::StatusIs(
-                  fake_failure_status.code(),
-                  std::string(fake_failure_status.message())));
-
-  h->Unref();
-  context->Unref();
+              StatusIs(fake_failure_status.code(),
+                       std::string(fake_failure_status.message())));
 }
 
-TEST(TensorHandle_LocalTest, TensorFromDeviceSameDevice) {
+TEST_F(RemoteTensorHandleTest, SetRemoteTensorHandleShapeTwice) {
   std::vector<std::unique_ptr<Device>> devices;
-  devices.emplace_back(
-      CreateDevice("CPU", "/job:localhost/replica:0/task:0/device:CPU:0"));
-  devices.emplace_back(
-      CreateDevice("CPU", "/job:localhost/replica:0/task:0/device:CPU:1"));
+  devices.push_back(
+      CreateDevice("CPU", "/job:worker/replica:0/task:0/device:CPU:0"));
+  devices.push_back(
+      CreateDevice("CPU", "/job:worker/replica:0/task:1/device:CPU:0"));
+  devices.push_back(
+      CreateDevice("CPU", "/job:worker/replica:0/task:2/device:CPU:0"));
   StaticDeviceMgr device_mgr(std::move(devices));
 
   EagerContext* context = new EagerContext(
       SessionOptions(),
       tensorflow::ContextDevicePlacementPolicy::DEVICE_PLACEMENT_SILENT,
-      /* async= */ false, &device_mgr,
-      /* device_mgr_owned= */ false, /* rendezvous= */ nullptr,
-      /* cluster_flr= */ nullptr, /*collective_executor_mgr=*/nullptr,
+      /* async=*/false, &device_mgr,
+      /* device_mgr_owned=*/false, /* rendezvous=*/nullptr,
+      /* cluster_flr=*/nullptr, /*collective_executor_mgr=*/nullptr,
       /*run_eager_op_as_function=*/true);
+  absl::Cleanup context_cleanup = [&]() { context->Unref(); };
 
   tensorflow::DataType dtype = DT_FLOAT;
   TensorShape shape = {};
 
-  Tensor t0(dtype, shape);
-  Device* d0 = device_mgr.ListDevices().at(1);
-  TensorHandle* h =
-      TensorHandle::CreateLocalHandle(std::move(t0), d0, d0, d0, context);
+  const string remote_task = "/job:worker/replica:0/task:1";
+  Device* d1 = device_mgr.ListDevices().at(1);
+  TensorHandle* h = TensorHandle::CreateUnshapedRemoteHandle(
+      /*op_id=*/0, /*output_num=*/0, remote_task, dtype, d1, context,
+      /*unknown_device=*/true);
+  absl::Cleanup h_cleanup = [&]() { h->Unref(); };
+  EXPECT_EQ(h->device(), d1);
 
-  const Tensor* tensor_from_device;
-  TF_EXPECT_OK(h->TensorFromDevice(d0, &tensor_from_device));
+  Device* d2 = device_mgr.ListDevices().at(2);
+  int64_t op_id = 1;
+  int output_num = 2;
+  TF_ASSERT_OK(
+      h->AddUnshapedRemoteMirror(d2, op_id, output_num, remote_task, context));
 
-  h->Unref();
-  context->Unref();
+  // Finds device_ != d, sets shape of `data` the first time.
+  TF_ASSERT_OK(h->SetRemoteShapeAndDevice(
+      shape, d2, context->GetContextViewId(), d2->name()));
+
+  // Finds device_ == d, sets shape of `data` the first time.
+  TF_ASSERT_OK(h->SetRemoteShapeAndDevice(
+      shape, d1, context->GetContextViewId(), d1->name()));
+
+  // Finds device_ == d, attempts to set shape of `data` the second time with
+  // the same value. No error message emitted.
+  TF_ASSERT_OK(h->SetRemoteShapeAndDevice(
+      shape, d1, context->GetContextViewId(), d1->name()));
+
+  // Finds device_ == d, attempts to set shape of `data` the third time with a
+  // different value. Results in error.
+  TensorShape another_shape({1});
+  EXPECT_THAT(h->SetRemoteShapeAndDevice(
+                  another_shape, d1, context->GetContextViewId(), d1->name()),
+              StatusIs(tensorflow::error::INTERNAL,
+                       HasSubstr("Trying to change shape to")));
 }
 
-TEST(TensorHandle_LocalTest, TensorFromDeviceDifferentDevice) {
+TEST_F(RemoteTensorHandleTest, SetRemoteMirrorShapeTwice) {
   std::vector<std::unique_ptr<Device>> devices;
-  devices.emplace_back(
-      CreateDevice("CPU", "/job:localhost/replica:0/task:0/device:CPU:0"));
-  devices.emplace_back(
-      CreateDevice("CPU", "/job:localhost/replica:0/task:0/device:CPU:1"));
-  devices.emplace_back(
+  devices.push_back(
+      CreateDevice("CPU", "/job:worker/replica:0/task:0/device:CPU:0"));
+  devices.push_back(
+      CreateDevice("CPU", "/job:worker/replica:0/task:1/device:CPU:0"));
+  devices.push_back(
       CreateDevice("CPU", "/job:worker/replica:0/task:2/device:CPU:0"));
   StaticDeviceMgr device_mgr(std::move(devices));
 
@@ -592,6 +615,58 @@ TEST(TensorHandle_LocalTest, TensorFromDeviceDifferentDevice) {
       /* device_mgr_owned= */ false, /* rendezvous= */ nullptr,
       /* cluster_flr= */ nullptr, /*collective_executor_mgr=*/nullptr,
       /*run_eager_op_as_function=*/true);
+  absl::Cleanup context_cleanup = [&]() { context->Unref(); };
+
+  tensorflow::DataType dtype = DT_FLOAT;
+  TensorShape shape = {};
+
+  const string remote_task = "/job:worker/replica:0/task:1";
+  Device* d1 = device_mgr.ListDevices().at(1);
+  TensorHandle* h = TensorHandle::CreateUnshapedRemoteHandle(
+      /*op_id=*/0, /*output_num=*/0, remote_task, dtype, d1, context,
+      /*unknown_device=*/true);
+  absl::Cleanup h_cleanup = [&]() { h->Unref(); };
+  EXPECT_EQ(h->device(), d1);
+
+  Device* d2 = device_mgr.ListDevices().at(2);
+  // Finds device_ == d, sets shape of `data` the first time.
+  TF_ASSERT_OK(h->SetRemoteShapeAndDevice(
+      shape, d1, context->GetContextViewId(), d2->name()));
+
+  int64_t op_id = 1;
+  int output_num = 2;
+  TF_ASSERT_OK(
+      h->AddUnshapedRemoteMirror(d1, op_id, output_num, remote_task, context));
+
+  // Finds device_ != d, sets shape of `remote_mirror` the first time.
+  TF_ASSERT_OK(h->SetRemoteShapeAndDevice(
+      shape, d1, context->GetContextViewId(), d2->name()));
+
+  // Finds device_ != d, attempts to set shape of `remote_mirror` the second
+  // time with a different value. Results in error.
+  TensorShape another_shape({1});
+  EXPECT_THAT(h->SetRemoteShapeAndDevice(
+                  another_shape, d1, context->GetContextViewId(), d2->name()),
+              StatusIs(tensorflow::error::INTERNAL,
+                       HasSubstr("Trying to change shape to")));
+}
+
+TEST(TensorHandle_LocalTest, TensorFromDeviceSameDevice) {
+  std::vector<std::unique_ptr<Device>> devices;
+  devices.push_back(
+      CreateDevice("CPU", "/job:localhost/replica:0/task:0/device:CPU:0"));
+  devices.push_back(
+      CreateDevice("CPU", "/job:localhost/replica:0/task:0/device:CPU:1"));
+  StaticDeviceMgr device_mgr(std::move(devices));
+
+  EagerContext* context = new EagerContext(
+      SessionOptions(),
+      tensorflow::ContextDevicePlacementPolicy::DEVICE_PLACEMENT_SILENT,
+      /* async= */ false, &device_mgr,
+      /* device_mgr_owned= */ false, /* rendezvous= */ nullptr,
+      /* cluster_flr= */ nullptr, /*collective_executor_mgr=*/nullptr,
+      /*run_eager_op_as_function=*/true);
+  absl::Cleanup context_cleanup = [&]() { context->Unref(); };
 
   tensorflow::DataType dtype = DT_FLOAT;
   TensorShape shape = {};
@@ -600,6 +675,39 @@ TEST(TensorHandle_LocalTest, TensorFromDeviceDifferentDevice) {
   Device* d0 = device_mgr.ListDevices().at(1);
   TensorHandle* h =
       TensorHandle::CreateLocalHandle(std::move(t0), d0, d0, d0, context);
+  absl::Cleanup h_cleanup = [&]() { h->Unref(); };
+
+  const Tensor* tensor_from_device;
+  TF_EXPECT_OK(h->TensorFromDevice(d0, &tensor_from_device));
+}
+
+TEST(TensorHandle_LocalTest, TensorFromDeviceDifferentDevice) {
+  std::vector<std::unique_ptr<Device>> devices;
+  devices.push_back(
+      CreateDevice("CPU", "/job:localhost/replica:0/task:0/device:CPU:0"));
+  devices.push_back(
+      CreateDevice("CPU", "/job:localhost/replica:0/task:0/device:CPU:1"));
+  devices.push_back(
+      CreateDevice("CPU", "/job:worker/replica:0/task:2/device:CPU:0"));
+  StaticDeviceMgr device_mgr(std::move(devices));
+
+  EagerContext* context = new EagerContext(
+      SessionOptions(),
+      tensorflow::ContextDevicePlacementPolicy::DEVICE_PLACEMENT_SILENT,
+      /* async= */ false, &device_mgr,
+      /* device_mgr_owned= */ false, /* rendezvous= */ nullptr,
+      /* cluster_flr= */ nullptr, /*collective_executor_mgr=*/nullptr,
+      /*run_eager_op_as_function=*/true);
+  absl::Cleanup context_cleanup = [&]() { context->Unref(); };
+
+  tensorflow::DataType dtype = DT_FLOAT;
+  TensorShape shape = {};
+
+  Tensor t0(dtype, shape);
+  Device* d0 = device_mgr.ListDevices().at(1);
+  TensorHandle* h =
+      TensorHandle::CreateLocalHandle(std::move(t0), d0, d0, d0, context);
+  absl::Cleanup h_cleanup = [&]() { h->Unref(); };
 
   Device* d1 = device_mgr.ListDevices().at(2);
   tensorflow::Tensor tensor;
@@ -608,18 +716,15 @@ TEST(TensorHandle_LocalTest, TensorFromDeviceDifferentDevice) {
 
   const Tensor* tensor_from_device;
   TF_EXPECT_OK(h->TensorFromDevice(d1, &tensor_from_device));
-
-  h->Unref();
-  context->Unref();
 }
 
 TEST(TensorHandle_LocalTest, TensorFromDeviceInvalidDevice) {
   std::vector<std::unique_ptr<Device>> devices;
-  devices.emplace_back(
+  devices.push_back(
       CreateDevice("CPU", "/job:localhost/replica:0/task:0/device:CPU:0"));
-  devices.emplace_back(
+  devices.push_back(
       CreateDevice("CPU", "/job:localhost/replica:0/task:0/device:CPU:1"));
-  devices.emplace_back(
+  devices.push_back(
       CreateDevice("CPU", "/job:worker/replica:0/task:2/device:CPU:0"));
   StaticDeviceMgr device_mgr(std::move(devices));
 
@@ -630,6 +735,7 @@ TEST(TensorHandle_LocalTest, TensorFromDeviceInvalidDevice) {
       /* device_mgr_owned= */ false, /* rendezvous= */ nullptr,
       /* cluster_flr= */ nullptr, /*collective_executor_mgr=*/nullptr,
       /*run_eager_op_as_function=*/true);
+  absl::Cleanup context_cleanup = [&]() { context->Unref(); };
 
   tensorflow::DataType dtype = DT_FLOAT;
   TensorShape shape = {};
@@ -638,24 +744,22 @@ TEST(TensorHandle_LocalTest, TensorFromDeviceInvalidDevice) {
   Device* d0 = device_mgr.ListDevices().at(1);
   TensorHandle* h =
       TensorHandle::CreateLocalHandle(std::move(t0), d0, d0, d0, context);
+  absl::Cleanup h_cleanup = [&]() { h->Unref(); };
 
   Device* d1 = device_mgr.ListDevices().at(2);
 
   const Tensor* tensor_from_device;
   EXPECT_THAT(h->TensorFromDevice(d1, &tensor_from_device),
-              tensorflow::testing::StatusIs(tensorflow::error::INTERNAL));
-
-  h->Unref();
-  context->Unref();
+              StatusIs(tensorflow::error::INTERNAL));
 }
 
 TEST(TensorHandle_ResourceShapeMirror, CreateAndCheckMirror) {
   std::vector<std::unique_ptr<Device>> devices;
-  devices.emplace_back(
+  devices.push_back(
       CreateDevice("CPU", "/job:localhost/replica:0/task:0/device:CPU:0"));
-  devices.emplace_back(
+  devices.push_back(
       CreateDevice("CPU", "/job:localhost/replica:0/task:0/device:CPU:1"));
-  devices.emplace_back(
+  devices.push_back(
       CreateDevice("CPU", "/job:localhost/replica:0/task:0/device:CPU:2"));
   StaticDeviceMgr device_mgr(std::move(devices));
 
@@ -666,6 +770,7 @@ TEST(TensorHandle_ResourceShapeMirror, CreateAndCheckMirror) {
       /* device_mgr_owned= */ false, /* rendezvous= */ nullptr,
       /* cluster_flr= */ nullptr, /*collective_executor_mgr=*/nullptr,
       /*run_eager_op_as_function=*/true);
+  absl::Cleanup context_cleanup = [&]() { context->Unref(); };
 
   tensorflow::DataType dtype = DT_RESOURCE;
   TensorShape shape = {};
@@ -674,6 +779,7 @@ TEST(TensorHandle_ResourceShapeMirror, CreateAndCheckMirror) {
   Device* d0 = device_mgr.ListDevices().at(1);
   TensorHandle* h =
       TensorHandle::CreateLocalHandle(std::move(t0), d0, d0, d0, context);
+  absl::Cleanup h_cleanup = [&]() { h->Unref(); };
 
   Device* d1 = device_mgr.ListDevices().at(2);
   int64_t op_id = 1;
@@ -683,18 +789,19 @@ TEST(TensorHandle_ResourceShapeMirror, CreateAndCheckMirror) {
   TF_EXPECT_OK(h->AddResourceShapeMirror(d1, op_id, output_num, context));
   EXPECT_TRUE(h->HasResourceShapeMirror(d1, context->GetContextViewId()));
 
-  // Adding a duplicate leads to failure
-  EXPECT_THAT(h->AddResourceShapeMirror(d1, op_id, output_num, context),
-              tensorflow::testing::StatusIs(tensorflow::error::INTERNAL));
-  h->Unref();
-  context->Unref();
+  // Adding an identical mirror is idempotent.
+  TF_EXPECT_OK(h->AddResourceShapeMirror(d1, op_id, output_num, context));
+
+  // Adding a duplicate mirror with inconsistent arguments leads to failure.
+  EXPECT_THAT(h->AddResourceShapeMirror(d1, op_id + 1, output_num, context),
+              StatusIs(tensorflow::error::INTERNAL));
 }
 
 TEST(TensorHandle_DeviceNameTest, OnLocalDevice) {
   std::vector<std::unique_ptr<Device>> devices;
-  devices.emplace_back(
+  devices.push_back(
       CreateDevice("CPU", "/job:localhost/replica:0/task:0/device:CPU:0"));
-  devices.emplace_back(
+  devices.push_back(
       CreateDevice("GPU", "/job:localhost/replica:0/task:0/device:GPU:0"));
   StaticDeviceMgr local_device_mgr(std::move(devices));
   auto ctx = new EagerContext(
@@ -702,6 +809,7 @@ TEST(TensorHandle_DeviceNameTest, OnLocalDevice) {
       tensorflow::ContextDevicePlacementPolicy::DEVICE_PLACEMENT_SILENT, false,
       &local_device_mgr, false, nullptr, nullptr, nullptr,
       /*run_eager_op_as_function=*/true);
+  absl::Cleanup ctx_cleanup = [&]() { ctx->Unref(); };
 
   Device* dcpu = local_device_mgr.ListDevices()[0];
   Device* dgpu = local_device_mgr.ListDevices()[1];
@@ -709,11 +817,12 @@ TEST(TensorHandle_DeviceNameTest, OnLocalDevice) {
   TensorShape shape = {2};
   Tensor tcpu(dtype, shape);
   Tensor tgpu(dtype, shape);
-  Status s;
+  absl::Status s;
 
   TensorHandle* th_cpu =
       TensorHandle::CreateLocalHandle(std::move(tcpu), dcpu, dcpu, dcpu, ctx);
   const char* device_name = th_cpu->DeviceName(&s);
+  absl::Cleanup th_cpu_cleanup = [&]() { th_cpu->Unref(); };
   TF_EXPECT_OK(s);
   ASSERT_TRUE(absl::StrContains(device_name, "CPU")) << device_name;
   const char* backing_device_name = th_cpu->BackingDeviceName(&s);
@@ -729,6 +838,7 @@ TEST(TensorHandle_DeviceNameTest, OnLocalDevice) {
 
   TensorHandle* th_gpu =
       TensorHandle::CreateLocalHandle(std::move(tgpu), dgpu, dgpu, dgpu, ctx);
+  absl::Cleanup th_gpu_cleanup = [&]() { th_gpu->Unref(); };
   device_name = th_gpu->DeviceName(&s);
   TF_EXPECT_OK(s);
   ASSERT_TRUE(absl::StrContains(device_name, "GPU")) << device_name;
@@ -744,10 +854,6 @@ TEST(TensorHandle_DeviceNameTest, OnLocalDevice) {
   device_id = th_gpu->DeviceId(&s);
   TF_EXPECT_OK(s);
   ASSERT_EQ(0, device_id) << device_id;
-
-  th_cpu->Unref();
-  th_gpu->Unref();
-  ctx->Unref();
 }
 
 }  // namespace tensorflow
