@@ -20,6 +20,7 @@ limitations under the License.
 #include <memory>
 #include <utility>
 
+#include <gmock/gmock.h>
 #include <gtest/gtest.h>
 #include "absl/memory/memory.h"
 #include "absl/status/status.h"
@@ -30,6 +31,8 @@ limitations under the License.
 #include "tensorflow/cc/ops/functional_ops.h"
 #include "tensorflow/cc/ops/sendrecv_ops.h"
 #include "tensorflow/cc/ops/standard_ops.h"
+#include "xla/tsl/platform/status.h"
+#include "xla/tsl/platform/status_matchers.h"
 #include "tensorflow/core/common_runtime/constant_folding.h"
 #include "tensorflow/core/common_runtime/device.h"
 #include "tensorflow/core/common_runtime/device_factory.h"
@@ -65,15 +68,17 @@ limitations under the License.
 namespace tensorflow {
 namespace {
 
+using ::testing::HasSubstr;
+using ::tsl::testing::StatusIs;
 using FDH = ::tensorflow::FunctionDefHelper;
-
 using OutputControlSrc = InlineFunctionBodyOptions::OutputControlSource;
 
-Status GetOpSig(const string& op, const OpDef** sig) {
+absl::Status GetOpSig(const string& op, const OpDef** sig) {
   return OpRegistry::Global()->LookUpOpDef(op, sig);
 }
 
-void HasError(const Status& s, const error::Code code, StringPiece substr) {
+void HasError(const absl::Status& s, const error::Code code,
+              absl::string_view substr) {
   EXPECT_EQ(s.code(), code) << s;
   EXPECT_TRUE(absl::StrContains(s.message(), substr))
       << s << ", expected substring " << substr;
@@ -179,7 +184,7 @@ class FunctionLibraryRuntimeTest : public ::testing::Test {
                                tsl::core::RefCountPtr<Rendezvous>* r) {
           *r = tsl::core::RefCountPtr<Rendezvous>(
               new IntraProcessRendezvous(device_mgr));
-          return OkStatus();
+          return absl::OkStatus();
         }}));
     flr0_ = pflr_->GetFLR("/job:localhost/replica:0/task:0/cpu:0");
     flr1_ = pflr_->GetFLR("/job:localhost/replica:0/task:0/cpu:1");
@@ -187,9 +192,10 @@ class FunctionLibraryRuntimeTest : public ::testing::Test {
     fdef_lib_ = lib_def_->ToProto();
   }
 
-  Status Run(FunctionLibraryRuntime* flr, FunctionLibraryRuntime::Handle handle,
-             FunctionLibraryRuntime::Options opts,
-             const std::vector<Tensor>& args, std::vector<Tensor*> rets) {
+  absl::Status Run(FunctionLibraryRuntime* flr,
+                   FunctionLibraryRuntime::Handle handle,
+                   FunctionLibraryRuntime::Options opts,
+                   const std::vector<Tensor>& args, std::vector<Tensor*> rets) {
     std::function<void(std::function<void()>)> runner =
         [](std::function<void()> fn) {
           test::function::FunctionTestSchedClosure(fn);
@@ -197,8 +203,8 @@ class FunctionLibraryRuntimeTest : public ::testing::Test {
     opts.runner = &runner;
     Notification done;
     std::vector<Tensor> out;
-    Status status;
-    flr->Run(opts, handle, args, &out, [&status, &done](const Status& s) {
+    absl::Status status;
+    flr->Run(opts, handle, args, &out, [&status, &done](const absl::Status& s) {
       status = s;
       done.Notify();
     });
@@ -210,38 +216,40 @@ class FunctionLibraryRuntimeTest : public ::testing::Test {
     for (size_t i = 0; i < rets.size(); ++i) {
       *rets[i] = out[i];
     }
-    return OkStatus();
+    return absl::OkStatus();
   }
 
-  Status Instantiate(FunctionLibraryRuntime* flr, const string& name,
-                     test::function::Attrs attrs,
-                     FunctionLibraryRuntime::Handle* handle) {
+  absl::Status Instantiate(FunctionLibraryRuntime* flr, const string& name,
+                           test::function::Attrs attrs,
+                           FunctionLibraryRuntime::Handle* handle) {
     return flr->Instantiate(name, attrs, handle);
   }
 
-  Status Instantiate(FunctionLibraryRuntime* flr, const string& name,
-                     test::function::Attrs attrs,
-                     const FunctionLibraryRuntime::InstantiateOptions& options,
-                     FunctionLibraryRuntime::Handle* handle) {
+  absl::Status Instantiate(
+      FunctionLibraryRuntime* flr, const string& name,
+      test::function::Attrs attrs,
+      const FunctionLibraryRuntime::InstantiateOptions& options,
+      FunctionLibraryRuntime::Handle* handle) {
     return flr->Instantiate(name, attrs, options, handle);
   }
 
-  Status InstantiateAndRun(FunctionLibraryRuntime* flr, const string& name,
-                           test::function::Attrs attrs,
-                           const std::vector<Tensor>& args,
-                           std::vector<Tensor*> rets) {
+  absl::Status InstantiateAndRun(FunctionLibraryRuntime* flr,
+                                 const string& name,
+                                 test::function::Attrs attrs,
+                                 const std::vector<Tensor>& args,
+                                 std::vector<Tensor*> rets) {
     return InstantiateAndRun(flr, name, attrs,
                              FunctionLibraryRuntime::InstantiateOptions(), args,
                              std::move(rets));
   }
 
-  Status InstantiateAndRun(
+  absl::Status InstantiateAndRun(
       FunctionLibraryRuntime* flr, const string& name,
       test::function::Attrs attrs,
       const FunctionLibraryRuntime::InstantiateOptions& options,
       const std::vector<Tensor>& args, std::vector<Tensor*> rets) {
     FunctionLibraryRuntime::Handle handle;
-    Status status = flr->Instantiate(name, attrs, options, &handle);
+    absl::Status status = flr->Instantiate(name, attrs, options, &handle);
     if (!status.ok()) {
       return status;
     }
@@ -253,7 +261,7 @@ class FunctionLibraryRuntimeTest : public ::testing::Test {
     status = flr->ReleaseHandle(handle);
     if (!status.ok()) return status;
 
-    Status status2 = Run(flr, handle, opts, args, std::move(rets));
+    absl::Status status2 = Run(flr, handle, opts, args, std::move(rets));
     EXPECT_TRUE(absl::IsNotFound(status2))
         << "Actual status: " << status2.ToString();
     EXPECT_TRUE(absl::StrContains(status2.message(), "Handle"));
@@ -262,16 +270,18 @@ class FunctionLibraryRuntimeTest : public ::testing::Test {
     return status;
   }
 
-  Status Run(FunctionLibraryRuntime* flr, FunctionLibraryRuntime::Handle handle,
-             FunctionLibraryRuntime::Options opts, CallFrameInterface* frame) {
+  absl::Status Run(FunctionLibraryRuntime* flr,
+                   FunctionLibraryRuntime::Handle handle,
+                   FunctionLibraryRuntime::Options opts,
+                   CallFrameInterface* frame) {
     std::function<void(std::function<void()>)> runner =
         [](std::function<void()> fn) {
           test::function::FunctionTestSchedClosure(fn);
         };
     opts.runner = &runner;
     Notification done;
-    Status status;
-    flr->Run(opts, handle, frame, [&status, &done](const Status& s) {
+    absl::Status status;
+    flr->Run(opts, handle, frame, [&status, &done](const absl::Status& s) {
       status = s;
       done.Notify();
     });
@@ -280,16 +290,15 @@ class FunctionLibraryRuntimeTest : public ::testing::Test {
       return status;
     }
 
-    return OkStatus();
+    return absl::OkStatus();
   }
 
-  Status InstantiateAndRunViaCallFrameInterface(FunctionLibraryRuntime* flr,
-                                                const string& name,
-                                                test::function::Attrs attrs,
-                                                const std::vector<Tensor>& args,
-                                                std::vector<Tensor*> rets) {
+  absl::Status InstantiateAndRunViaCallFrameInterface(
+      FunctionLibraryRuntime* flr, const string& name,
+      test::function::Attrs attrs, const std::vector<Tensor>& args,
+      std::vector<Tensor*> rets) {
     FunctionLibraryRuntime::Handle handle;
-    Status status = flr->Instantiate(name, attrs, &handle);
+    absl::Status status = flr->Instantiate(name, attrs, &handle);
     if (!status.ok()) {
       return status;
     }
@@ -312,7 +321,7 @@ class FunctionLibraryRuntimeTest : public ::testing::Test {
     status = flr->ReleaseHandle(handle);
     if (!status.ok()) return status;
 
-    Status status2 = Run(flr, handle, opts, args, std::move(rets));
+    absl::Status status2 = Run(flr, handle, opts, args, std::move(rets));
     EXPECT_TRUE(absl::IsNotFound(status2));
     EXPECT_TRUE(absl::StrContains(status2.message(), "Handle"));
     EXPECT_TRUE(absl::StrContains(status2.message(), "not found"));
@@ -324,7 +333,7 @@ class FunctionLibraryRuntimeTest : public ::testing::Test {
                                      const string& name,
                                      test::function::Attrs attrs) {
     FunctionLibraryRuntime::Handle handle;
-    Status status = flr->Instantiate(name, attrs, &handle);
+    absl::Status status = flr->Instantiate(name, attrs, &handle);
     if (!status.ok()) {
       LOG(ERROR) << status;
       return nullptr;
@@ -340,7 +349,7 @@ class FunctionLibraryRuntimeTest : public ::testing::Test {
                                      const string& func,
                                      test::function::Attrs attrs) {
     FunctionLibraryRuntime::Handle handle;
-    Status status = flr->Instantiate(func, attrs, &handle);
+    absl::Status status = flr->Instantiate(func, attrs, &handle);
     if (!status.ok()) {
       LOG(ERROR) << status;
       return nullptr;
@@ -385,6 +394,7 @@ TEST_F(FunctionLibraryRuntimeTest, XTimesTwo) {
 TEST_F(FunctionLibraryRuntimeTest, InstantiationStackTraceCopying) {
   class DummyStackTrace : public AbstractStackTrace {
     absl::Span<StackFrame const> ToFrames() const override { return {}; }
+    std::vector<StackFrame> ToUncachedFrames() const override { return {}; }
 
     std::string ToString(const TracePrintingOptions& opts) const override {
       return "DummyStackTrace";
@@ -438,7 +448,7 @@ class ConsumeArgumentCallFrame : public CallFrameInterface {
   size_t num_args() const override { return 1; }
   size_t num_retvals() const override { return 1; }
 
-  Status GetArg(int index, const Tensor** val) override {
+  absl::Status GetArg(int index, const Tensor** val) override {
     LOG(FATAL) << "Should not be called.";
   }
 
@@ -446,10 +456,10 @@ class ConsumeArgumentCallFrame : public CallFrameInterface {
 
   void ConsumeArg(int index, Tensor* val) override { *val = std::move(*arg_); }
 
-  Status SetRetval(int index, const Tensor& val) override {
+  absl::Status SetRetval(int index, const Tensor& val) override {
     CHECK_EQ(index, 0);
     *retval_ = val;
-    return OkStatus();
+    return absl::OkStatus();
   }
 
  private:
@@ -635,7 +645,7 @@ TEST_F(FunctionLibraryRuntimeTest, StateHandle) {
       // Attrs
       {},
       // Nodes
-      {FDH::Const<int32>("shape", gtl::ArraySlice<int32>({1})),
+      {FDH::Const<int32>("shape", absl::Span<const int32>({1})),
        FDH::Const<int32>("minval", 0),
        FDH::Const<int32>("maxval", 10),
        // A stateful node.
@@ -721,6 +731,65 @@ TEST_F(FunctionLibraryRuntimeTest, StateHandle) {
   }
 }
 
+TEST_F(FunctionLibraryRuntimeTest, FunctionLibraryRuntimeRunAfterFinalize) {
+  // Instantiate the function before finalization.
+  Init({test::function::XTimesTwo()});
+  FunctionLibraryRuntime::Handle handle;
+  TF_CHECK_OK(Instantiate(flr0_, "XTimesTwo", {{"T", DT_FLOAT}}, &handle));
+
+  // Run the function before finalization.
+  FunctionLibraryRuntime::Options opts;
+  Tensor x = test::AsTensor<float>({1, 2, 3, 4});
+  Tensor y;
+  TF_CHECK_OK(Run(flr0_, handle, opts, {x}, {&y}));
+  Tensor expected_y = test::AsTensor<float>({2, 4, 6, 8});
+  test::ExpectTensorEqual<float>(y, expected_y);
+
+  // Finalize the function library runtime.
+  TF_CHECK_OK(flr0_->Finalize());
+
+  // Run the function again after finalization.
+  TF_CHECK_OK(Run(flr0_, handle, opts, {x}, {&y}));
+  test::ExpectTensorEqual<float>(y, expected_y);
+}
+
+TEST_F(FunctionLibraryRuntimeTest, FunctionLibraryRuntimeMultipleFinalizeOk) {
+  // Instantiate the function before finalization.
+  Init({test::function::XTimesTwo()});
+  FunctionLibraryRuntime::Handle handle;
+  TF_CHECK_OK(Instantiate(flr0_, "XTimesTwo", {{"T", DT_FLOAT}}, &handle));
+
+  // Run the function before finalization.
+  FunctionLibraryRuntime::Options opts;
+  Tensor x = test::AsTensor<float>({1, 2, 3, 4});
+  Tensor y;
+  TF_CHECK_OK(Run(flr0_, handle, opts, {x}, {&y}));
+  Tensor expected_y = test::AsTensor<float>({2, 4, 6, 8});
+  test::ExpectTensorEqual<float>(y, expected_y);
+
+  // Finalize the function library runtime twice.
+  TF_CHECK_OK(flr0_->Finalize());
+  TF_CHECK_OK(flr0_->Finalize());
+
+  // Run the function again after finalization.
+  TF_CHECK_OK(Run(flr0_, handle, opts, {x}, {&y}));
+  test::ExpectTensorEqual<float>(y, expected_y);
+}
+
+TEST_F(FunctionLibraryRuntimeTest,
+       FunctionLibraryRuntimeInstantiateFailAfterFinalize) {
+  // Instantiate the function before finalization.
+  Init({test::function::XTimesTwo()});
+
+  // Finalize the function library runtime.
+  TF_CHECK_OK(flr0_->Finalize());
+  FunctionLibraryRuntime::Handle handle;
+  EXPECT_THAT(Instantiate(flr0_, "XTimesTwo", {{"T", DT_FLOAT}}, &handle),
+              StatusIs(absl::StatusCode::kFailedPrecondition,
+                       HasSubstr("FunctionLibraryRuntimeImpl is finalized and "
+                                 "cannot instantiate a new function handle.")));
+}
+
 namespace {
 class DummyExecutorRegistrar {
  public:
@@ -730,8 +799,9 @@ class DummyExecutorRegistrar {
 
  private:
   class Factory : public ExecutorFactory {
-    Status NewExecutor(const LocalExecutorParams& params, const Graph& graph,
-                       std::unique_ptr<Executor>* out_executor) override {
+    absl::Status NewExecutor(const LocalExecutorParams& params,
+                             const Graph& graph,
+                             std::unique_ptr<Executor>* out_executor) override {
       return errors::Internal("This is a dummy.");
     }
   };
@@ -1122,13 +1192,13 @@ TEST_F(FunctionLibraryRuntimeTest, ExpandInlineFunctionsAndKeepCallerNode) {
   // Construct a graph:
   //   a = Arg[dtype=DT_FLOAT]
   //   b = FunctionWithControlOutputs(a)
-  auto construct_graph = [this](std::unique_ptr<Graph>* g) -> Status {
+  auto construct_graph = [this](std::unique_ptr<Graph>* g) -> absl::Status {
     Scope s = Scope::NewRootScope();
     TF_RETURN_IF_ERROR(s.graph()->AddFunctionLibrary(fdef_lib_));
     auto a = ops::_Arg(s.WithOpName("a"), DT_FLOAT, 0);
     auto b = test::function::Call(&s, "b", "AddAndMul", {a});
     TF_RETURN_IF_ERROR(s.ToGraph(g->get()));
-    return OkStatus();
+    return absl::OkStatus();
   };
 
   const string input_node = "Func/b/input/_0";
@@ -1207,7 +1277,7 @@ TEST_F(FunctionLibraryRuntimeTest, ExpandInlineFunctionsAndPlaceInlinedNodes) {
   // Construct a graph:
   //   a = Arg[dtype=DT_FLOAT, _device=arg_device]
   //   b = AddFunc[_device=call_device](a)
-  auto construct_graph = [&](std::unique_ptr<Graph>* g) -> Status {
+  auto construct_graph = [&](std::unique_ptr<Graph>* g) -> absl::Status {
     Scope s = Scope::NewRootScope();
     TF_RETURN_IF_ERROR(s.graph()->AddFunctionLibrary(fdef_lib_));
     auto a = ops::_Arg(s.WithOpName("a").WithDevice(arg_device), DT_FLOAT, 0);
@@ -1216,7 +1286,7 @@ TEST_F(FunctionLibraryRuntimeTest, ExpandInlineFunctionsAndPlaceInlinedNodes) {
     for (Node* node : (*g)->op_nodes()) {
       if (node->name() == "b") node->set_requested_device(call_device);
     }
-    return OkStatus();
+    return absl::OkStatus();
   };
 
   const string input_node = "Func/b/input/_0";
