@@ -22,28 +22,24 @@ limitations under the License.
 #include <optional>
 #include <vector>
 
-#include "tensorflow/compiler/xla/stream_executor/device_id_utils.h"
-#include "tensorflow/compiler/xla/stream_executor/gpu/gpu_driver.h"
-#include "tensorflow/compiler/xla/stream_executor/gpu/gpu_init.h"
-#include "tensorflow/compiler/xla/stream_executor/stream_executor.h"
+#include "xla/stream_executor/gpu/gpu_init.h"
+#include "xla/stream_executor/stream_executor.h"
+#include "xla/tsl/framework/device_id.h"
+#include "xla/tsl/lib/gtl/inlined_vector.h"
+#include "xla/tsl/lib/random/simple_philox.h"
+#include "xla/tsl/platform/logging.h"
+#include "xla/tsl/platform/test.h"
+#include "xla/tsl/platform/test_benchmark.h"
+#include "xla/tsl/platform/threadpool.h"
+#include "xla/tsl/platform/types.h"
 #include "tensorflow/core/common_runtime/device/device_mem_allocator.h"
-#include "tensorflow/core/common_runtime/gpu/gpu_virtual_mem_allocator.h"
 #include "tensorflow/core/framework/typed_allocator.h"
 #include "tensorflow/core/protobuf/bfc_memory_map.pb.h"
 #include "tensorflow/core/protobuf/config.pb.h"
-#include "tensorflow/tsl/framework/device_id.h"
-#include "tensorflow/tsl/lib/gtl/inlined_vector.h"
-#include "tensorflow/tsl/lib/random/simple_philox.h"
-#include "tensorflow/tsl/platform/logging.h"
-#include "tensorflow/tsl/platform/strcat.h"
-#include "tensorflow/tsl/platform/test.h"
-#include "tensorflow/tsl/platform/test_benchmark.h"
-#include "tensorflow/tsl/platform/threadpool.h"
-#include "tensorflow/tsl/platform/types.h"
+#include "tsl/platform/strcat.h"
 
 namespace tsl {
 namespace {
-using stream_executor::DeviceIdUtil;
 using stream_executor::GPUMachineManager;
 using tensorflow::BinSummary;
 using tensorflow::DeviceMemAllocator;
@@ -69,46 +65,19 @@ class GPUBFCAllocatorTest
     : public ::testing::TestWithParam<std::unique_ptr<SubAllocator> (*)(
           size_t)> {};
 
-#if CUDA_VERSION >= 10020
-std::unique_ptr<SubAllocator> CreateVirtualMemorySubAllocator(
-    size_t virtual_address_space_size = 1ull << 32) {
-  PlatformDeviceId gpu_id(0);
-  auto executor =
-      DeviceIdUtil::ExecutorForPlatformDeviceId(GPUMachineManager(), gpu_id)
-          .value();
-  auto* gpu_context = reinterpret_cast<stream_executor::gpu::GpuContext*>(
-      executor->implementation()->GpuContextHack());
-  return tensorflow::GpuVirtualMemAllocator::Create(
-             {}, {}, *gpu_context, gpu_id, virtual_address_space_size, {})
-      .value();
-}
-#endif
-
 std::unique_ptr<SubAllocator> CreateGPUMemAllocator(size_t) {
   PlatformDeviceId gpu_id(0);
   return absl::WrapUnique(new DeviceMemAllocator(
-      DeviceIdUtil::ExecutorForPlatformDeviceId(GPUMachineManager(), gpu_id)
-          .value(),
-      gpu_id, /*use_unified_memory=*/false, {}, {}));
+      GPUMachineManager()->ExecutorForDevice(gpu_id.value()).value(), gpu_id,
+      {}));
 }
 
 std::unique_ptr<SubAllocator> CreateSubAllocator(
     size_t virtual_address_space_size = 1ull << 32) {
-#if CUDA_VERSION >= 10020
-  return CreateVirtualMemorySubAllocator(virtual_address_space_size);
-#else
   return CreateGPUMemAllocator(virtual_address_space_size);
-#endif
 }
 
-auto TestSuiteValues() {
-#if CUDA_VERSION >= 10020
-  return ::testing::Values(&CreateGPUMemAllocator,
-                           &CreateVirtualMemorySubAllocator);
-#else
-  return ::testing::Values(&CreateGPUMemAllocator);
-#endif
-}
+auto TestSuiteValues() { return ::testing::Values(&CreateGPUMemAllocator); }
 
 TEST_P(GPUBFCAllocatorTest, NoDups) {
   GPUBFCAllocator a(GetParam()(1ull << 32), 1 << 30, "GPU_0_bfc", {});
@@ -465,7 +434,7 @@ class GPUBFCAllocatorPrivateMethodsTest
 
     std::array<BFCAllocator::BinDebugInfo, BFCAllocator::kNumBins> bin_infos;
     {
-      mutex_lock l(a.lock_);
+      absl::MutexLock l(&a.mutex_);
       bin_infos = a.get_bin_debug_info();
     }
 
@@ -517,7 +486,7 @@ class GPUBFCAllocatorPrivateMethodsTest
       initial_ptrs[i] = nullptr;
     }
     {
-      mutex_lock l(a.lock_);
+      absl::MutexLock l(&a.mutex_);
       bin_infos = a.get_bin_debug_info();
     }
     for (int i = 0; i < BFCAllocator::kNumBins; i++) {
@@ -542,18 +511,6 @@ class GPUBFCAllocatorPrivateMethodsTest
     }
   }
 
-  void TestLog2FloorNonZeroSlow() {
-    GPUBFCAllocator a(GetParam()(1ull << 32), 1 /* total_memory */, "GPU_0_bfc",
-                      {});
-    EXPECT_EQ(-1, a.Log2FloorNonZeroSlow(0));
-    EXPECT_EQ(0, a.Log2FloorNonZeroSlow(1));
-    EXPECT_EQ(1, a.Log2FloorNonZeroSlow(2));
-    EXPECT_EQ(1, a.Log2FloorNonZeroSlow(3));
-    EXPECT_EQ(9, a.Log2FloorNonZeroSlow(1023));
-    EXPECT_EQ(10, a.Log2FloorNonZeroSlow(1024));
-    EXPECT_EQ(10, a.Log2FloorNonZeroSlow(1025));
-  }
-
   void TestForceAllowGrowth() {
     // Unset flag value uses provided option.
     unsetenv("TF_FORCE_GPU_ALLOW_GROWTH");
@@ -593,10 +550,6 @@ class GPUBFCAllocatorPrivateMethodsTest
 
 TEST_P(GPUBFCAllocatorPrivateMethodsTest, BinDebugInfo) { TestBinDebugInfo(); }
 
-TEST_P(GPUBFCAllocatorPrivateMethodsTest, Log2FloorNonZeroSlow) {
-  TestLog2FloorNonZeroSlow();
-}
-
 TEST_P(GPUBFCAllocatorPrivateMethodsTest, ForceAllowGrowth) {
   TestForceAllowGrowth();
 }
@@ -607,34 +560,6 @@ INSTANTIATE_TEST_SUITE_P(GPUBFCAllocatorPrivateMethodTestSuite,
 // Tests that cannot be trivially parameterized for both suballocator types.
 class GPUBFCAllocatorTest_SubAllocatorSpecific : public ::testing::Test {};
 
-#if CUDA_VERSION >= 10020
-// Benchmark for measuring "high water mark" for BFCAllocator owned memory.
-TEST_F(GPUBFCAllocatorTest_SubAllocatorSpecific,
-       VirtualAllocatorPromotesReuse) {
-  GPUBFCAllocator::Options options;
-  options.allow_growth = true;
-
-  constexpr size_t k512MiB = 512ull << 20;
-
-  // 512 MiB allocator.
-  GPUBFCAllocator a(CreateVirtualMemorySubAllocator(1ull << 32), k512MiB,
-                    "GPU_0_bfc", options);
-  // Allocate 128 raw pointers of 4 megs.
-  const size_t size = 1LL << 22;
-  std::vector<void*> initial_ptrs;
-  for (size_t s = 0; s < 128; s++) {
-    void* raw = a.AllocateRaw(1, size);
-    initial_ptrs.push_back(raw);
-  }
-  // Deallocate all but the last one so the big chunk cannot be GC'd
-  for (int i = 0; i < 127; ++i) {
-    a.DeallocateRaw(initial_ptrs[i]);
-  }
-  void* big_alloc = a.AllocateRaw(1, k512MiB - size);
-  EXPECT_NE(big_alloc, nullptr);
-}
-#endif
-
 TEST_F(GPUBFCAllocatorTest_SubAllocatorSpecific,
        PhysicalAllocatorOomsFragmentation) {
   GPUBFCAllocator::Options options;
@@ -685,7 +610,7 @@ class GPUBFCAllocatorPrivateMethodsTest_SubAllocatorSpecific
     }
 
     {
-      mutex_lock l(a.lock_);
+      absl::MutexLock l(&a.mutex_);
       // Make sure there are more than 1 regions in preparation for the test.
       EXPECT_LT(1, a.region_manager_.regions().size());
     }
@@ -698,7 +623,7 @@ class GPUBFCAllocatorPrivateMethodsTest_SubAllocatorSpecific
     // Deallocate free regions and there shall be only one region left.
     EXPECT_EQ(true, a.DeallocateFreeRegions(/*rounded_bytes=*/0));
     {
-      mutex_lock l(a.lock_);
+      absl::MutexLock l(&a.mutex_);
       EXPECT_EQ(1, a.region_manager_.regions().size());
     }
 
@@ -710,45 +635,6 @@ class GPUBFCAllocatorPrivateMethodsTest_SubAllocatorSpecific
     }
     EXPECT_EQ(1, num_chunks_in_bins);
   }
-
-#if CUDA_VERSION >= 10020
-  // Counterpart to the GPUMemAllocator test suite TestRegionDeallocation tests.
-  // Here we expect no deallocations because all allocations are coalesced into
-  // a single region.
-  void TestNoRegionDeallocation() {
-    GPUBFCAllocator::Options options;
-    options.allow_growth = true;
-
-    // Max of 2GiB, but starts out small.
-    GPUBFCAllocator a(CreateVirtualMemorySubAllocator(1uLL << 32), 1LL << 31,
-                      "GPU_0_bfc", options);
-
-    // Allocate 128 raw pointers of 4 megs.
-    const size_t size = 1LL << 22;
-    std::vector<void*> initial_ptrs;
-    for (size_t s = 0; s < 128; s++) {
-      void* raw = a.AllocateRaw(1, size);
-      initial_ptrs.push_back(raw);
-    }
-
-    {
-      mutex_lock l(a.lock_);
-      EXPECT_EQ(1, a.region_manager_.regions().size());
-    }
-
-    // Deallocate all the memories except the last one.
-    for (size_t i = 0; i < initial_ptrs.size() - 1; i++) {
-      a.DeallocateRaw(initial_ptrs[i]);
-    }
-
-    // Deallocate free regions and there should still be only one.
-    EXPECT_EQ(false, a.DeallocateFreeRegions(/*rounded_bytes=*/0));
-    {
-      mutex_lock l(a.lock_);
-      EXPECT_EQ(1, a.region_manager_.regions().size());
-    }
-  }
-#endif
 };
 
 TEST_F(GPUBFCAllocatorPrivateMethodsTest_SubAllocatorSpecific,
@@ -756,13 +642,6 @@ TEST_F(GPUBFCAllocatorPrivateMethodsTest_SubAllocatorSpecific,
   TestRegionDeallocation();
 }
 
-#if CUDA_VERSION >= 10020
-TEST_F(GPUBFCAllocatorPrivateMethodsTest_SubAllocatorSpecific,
-       TestNoRegionDeallocation) {
-  TestNoRegionDeallocation();
-}
-#endif
-
 }  // namespace tsl
 
 #endif  // GOOGLE_CUDA || TENSORFLOW_USE_ROCM
