@@ -244,7 +244,7 @@ class HierarchicalTreeBroadcasterTest : public ::testing::Test {
     Tensor output_tensor_;
     Device* device_;
     core::RefCountPtr<CollectiveParams> col_params_;
-    Status status_;
+    absl::Status status_;
   };  // class DeviceInstance
 
   std::unique_ptr<CollectiveTestEnv> test_env_;
