@@ -42,18 +42,18 @@ class CaseBuilder {
               bool keep_node_fetchable, Graph* graph);
 
   // Constructs the basic conditional control flow using switch and merge nodes.
-  Status CreatePivotNodes();
+  absl::Status CreatePivotNodes();
 
   // Adds the inputs from the if node to the merge nodes of the lowered if.
-  Status AddInputs();
+  absl::Status AddInputs();
 
   // Adds the outputs from the if node to the merge nodes of the lowered if.
   // Note: no inputs can be added once outputs are added as the then and else
   // nodes are finalized while adding outputs.
-  Status AddOutputs();
+  absl::Status AddOutputs();
 
   // Builds an identity node with the same outputs as Case.
-  Status BuildLoweredCaseOutput();
+  absl::Status BuildLoweredCaseOutput();
 
  private:
   // Returns unique name containing the name of the Case op being rewritten
@@ -61,7 +61,7 @@ class CaseBuilder {
   string NewName(const string& infix);
 
   // Adds input to both the then and else nodes from src:src_output.
-  Status AddInput(Node* src, int src_output);
+  absl::Status AddInput(Node* src, int src_output);
 
   // The merged outputs of the then and else nodes.
   std::vector<NodeOut> outputs_;
@@ -115,7 +115,7 @@ CaseBuilder::CaseBuilder(Node* case_op,
   TF_CHECK_OK(case_op_->input_tensor(0, &branch_index_));
 }
 
-Status CaseBuilder::CreatePivotNodes() {
+absl::Status CaseBuilder::CreatePivotNodes() {
   // Construct the basic case body (consisting of feeding in the val to
   // create pivot nodes).
   Node* branch_index;
@@ -136,14 +136,14 @@ Status CaseBuilder::CreatePivotNodes() {
                            .Device(case_op_->requested_device())
                            .Finalize(graph_, &pivots_[b]));
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 string CaseBuilder::NewName(const string& infix) {
   return graph_->NewName(strings::StrCat(name_, "/", infix));
 }
 
-Status CaseBuilder::AddInput(Node* src, int src_output) {
+absl::Status CaseBuilder::AddInput(Node* src, int src_output) {
   Node* input;
   NodeDebugInfo debug_info(*src);
   // Colocate the Switch node with the `src` node.
@@ -166,10 +166,10 @@ Status CaseBuilder::AddInput(Node* src, int src_output) {
   for (int b = 0; b < num_branches_; b++) {
     branch_call_builders_[b].Input(input, b);
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status CaseBuilder::AddInputs() {
+absl::Status CaseBuilder::AddInputs() {
   // Add input data edges.
   std::vector<const Edge*> edges;
   TF_RETURN_IF_ERROR(case_op_->input_edges(&edges));
@@ -184,10 +184,10 @@ Status CaseBuilder::AddInputs() {
       graph_->AddControlEdge(e->src(), control_predecessor_);
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status CaseBuilder::AddOutputs() {
+absl::Status CaseBuilder::AddOutputs() {
   // Construct the call nodes for each branch.
   call_nodes_.resize(num_branches_, nullptr);
   for (int b = 0; b < num_branches_; b++) {
@@ -247,10 +247,10 @@ Status CaseBuilder::AddOutputs() {
       graph_->AddEdge(merges[e->src_output()], 0, e->dst(), e->dst_input());
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status CaseBuilder::BuildLoweredCaseOutput() {
+absl::Status CaseBuilder::BuildLoweredCaseOutput() {
   // If outputs are empty, it means that we might have only output control
   // edges (already connected to the `branch_executed_node`). Furthermore it's
   // illegal to have an IdentityN with empty inputs.
@@ -267,7 +267,7 @@ Status CaseBuilder::BuildLoweredCaseOutput() {
 
 }  // namespace
 
-Status RewriteCaseNode(Node* n, Graph* g, bool keep_node_fetchable) {
+absl::Status RewriteCaseNode(Node* n, Graph* g, bool keep_node_fetchable) {
   VLOG(2) << "Lower Case node (keep_node_fetchable=" << keep_node_fetchable
           << "): " << SummarizeNode(*n);
   const AttrValue* branches_attr = n->attrs().Find("branches");
@@ -287,7 +287,7 @@ Status RewriteCaseNode(Node* n, Graph* g, bool keep_node_fetchable) {
   TF_RETURN_IF_ERROR(cb.AddOutputs());
   g->RemoveNode(n);
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace tensorflow
