@@ -15,11 +15,16 @@ limitations under the License.
 
 #include "tensorflow/core/common_runtime/lower_while_op.h"
 
+#include "xla/tsl/platform/errors.h"
 #include "tensorflow/core/common_runtime/inline_function_utils.h"
+#include "tensorflow/core/config/flag_defs.h"
 #include "tensorflow/core/framework/node_def_builder.h"
+#include "tensorflow/core/framework/node_def_util.h"
+#include "tensorflow/core/framework/op.h"
 #include "tensorflow/core/framework/types.pb.h"
 #include "tensorflow/core/graph/graph.h"
 #include "tensorflow/core/graph/node_builder.h"
+#include "tensorflow/core/platform/status.h"
 
 namespace tensorflow {
 
@@ -62,10 +67,11 @@ constexpr const char* const kLowerAsMultiDeviceFunctionAttr =
 //   consumer
 class LowerWhileHelper {
  public:
-  static Status Run(Node* while_op, const NameAttrList& cond_fn,
-                    const NameAttrList& body_fn, int parallel_iterations,
-                    Graph* graph, const FunctionLibraryDefinition* flib_def,
-                    bool keep_node_fetchable) {
+  static absl::Status Run(Node* while_op, const NameAttrList& cond_fn,
+                          const NameAttrList& body_fn, int parallel_iterations,
+                          Graph* graph,
+                          const FunctionLibraryDefinition* flib_def,
+                          bool keep_node_fetchable) {
     LowerWhileHelper helper(while_op, cond_fn, body_fn, parallel_iterations,
                             graph, flib_def, keep_node_fetchable);
     return helper.RunInternal();
@@ -80,49 +86,49 @@ class LowerWhileHelper {
                    Graph* graph, const FunctionLibraryDefinition* flib_def,
                    bool keep_node_fetchable);
 
-  Status RunInternal();
+  absl::Status RunInternal();
 
   void InitializeInputOutputToLoweredNodeMap();
 
   // Creates an Enter node for each `while_op_` input and adds them to
   // `enter_nodes_`. If the `while_op_` has an incoming control edge from a
   // `src` node we add a control edge from `src` to each Enter node.
-  Status CreateEnterNodes();
+  absl::Status CreateEnterNodes();
 
   // Creates a Merge node for each Enter node and adds to `merge_nodes_`.
   // Initially now both inputs of a Merge node are the Enter node. Input at
   // index 1 is later updated to the output of NextIteration node in
   // `UpdateMergeNodes`.
-  Status CreateMergeNodes();
+  absl::Status CreateMergeNodes();
 
   // Creates the call node for cond func and stores in `cond_call_node_`.
-  Status CreateCondFuncCallNode();
+  absl::Status CreateCondFuncCallNode();
 
   // Creates a Switch node for each loop var and adds to `switch_nodes_`.
   // Output at index 1(true) of a Switch node is fed into the loop body.
   // Output at index 0(false) of a Switch node is fed into the Exit nodes.
-  Status CreateSwitchNodes();
+  absl::Status CreateSwitchNodes();
 
   // Creates the call node for body func and stores in `body_call_node_`.
-  Status CreateBodyFuncCallNode();
+  absl::Status CreateBodyFuncCallNode();
 
   // Creates an Exit node for each loop var and adds to `exit_nodes_`. These
   // are fed into the consumers of the `while_op_`.
-  Status CreateExitNodes();
+  absl::Status CreateExitNodes();
 
   // Creates an NextIteration node for each loop var and adds to
   // `next_iteration_nodes_`.
-  Status CreateNextIterationNodes();
+  absl::Status CreateNextIterationNodes();
 
   // Updates input at index 1 of each merge node created in `CreateMergeNodes`
   // to use the output of NextIteration node created in
   // `CreateNextIterationNodes` instead.
-  Status UpdateMergeNodes();
+  absl::Status UpdateMergeNodes();
 
   // Updates consumers of the original `while_op_` to instead use the outputs
   // from the exit nodes in `exit_nodes_`. Also updates any outgoing control
   // edges to depend on `lowered_while_executed_` instead.
-  Status UpdateConsumers();
+  absl::Status UpdateConsumers();
 
   // Returns unique name containing the name of the While op being rewritten
   // (name_), infix and a suffix to ensure it is unique within the graph.
@@ -175,6 +181,10 @@ class LowerWhileHelper {
   // in which case the mapping contains -1.
   std::vector<int> op_input_output_to_lowered_node_;
 
+  // Indicates whether to propagate colocation key attribute during the
+  // lowering.
+  bool propagate_colocation_key_;
+
   size_t num_loop_inputs_;
 };
 
@@ -211,9 +221,12 @@ LowerWhileHelper::LowerWhileHelper(Node* while_op, const NameAttrList& cond_fn,
   exit_nodes_.reserve(num_loop_inputs_);
   next_iterations_nodes_.reserve(num_loop_inputs_);
   op_input_output_to_lowered_node_.resize(num_loop_inputs_, -1);
+  propagate_colocation_key_ =
+      flags::Global()
+          .enable_colocation_key_propagation_in_while_op_lowering.value();
 }
 
-Status LowerWhileHelper::RunInternal() {
+absl::Status LowerWhileHelper::RunInternal() {
   InitializeInputOutputToLoweredNodeMap();
   TF_RETURN_IF_ERROR(CreateEnterNodes());
   TF_RETURN_IF_ERROR(CreateMergeNodes());
@@ -224,7 +237,7 @@ Status LowerWhileHelper::RunInternal() {
   TF_RETURN_IF_ERROR(CreateNextIterationNodes());
   TF_RETURN_IF_ERROR(UpdateMergeNodes());
   TF_RETURN_IF_ERROR(UpdateConsumers());
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 void LowerWhileHelper::InitializeInputOutputToLoweredNodeMap() {
@@ -236,7 +249,7 @@ void LowerWhileHelper::InitializeInputOutputToLoweredNodeMap() {
   }
 }
 
-Status LowerWhileHelper::CreateEnterNodes() {
+absl::Status LowerWhileHelper::CreateEnterNodes() {
   // Note: `Node::input_edge` runs in  O(num_inputs) so we use
   // `Node::input_edges` instead so that below loop runs in O(num_inputs) time
   // and not O(num_inputs^2).
@@ -251,6 +264,13 @@ Status LowerWhileHelper::CreateEnterNodes() {
             .Attr("parallel_iterations", parallel_iterations_)
             .Device(edge->src()->requested_device())
             .AssignedDevice(edge->src()->assigned_device_name());
+    if (propagate_colocation_key_) {
+      auto colocation_attr = edge->src()->attrs().Find(kColocationAttrName);
+      if (colocation_attr) {
+        builder.Attr(kColocationAttrName, *colocation_attr);
+      }
+    }
+
     if (IsLoopCarriedResource(edge->dst_input())) {
       builder.Attr("is_constant", true);
     }
@@ -267,37 +287,49 @@ Status LowerWhileHelper::CreateEnterNodes() {
   }
   if (!control_inputs.empty()) {
     Node* incoming_control_node;
-    TF_RETURN_IF_ERROR(NodeBuilder(NewName("LoopControlInputs"), "NoOp",
-                                   flib_def_, &debug_info_)
-                           .ControlInputs(control_inputs)
-                           .Device(while_op_->requested_device())
-                           .Finalize(graph_, &incoming_control_node));
+    NodeBuilder builder = NodeBuilder(NewName("LoopControlInputs"), "NoOp",
+                                      flib_def_, &debug_info_)
+                              .ControlInputs(control_inputs)
+                              .Device(while_op_->requested_device());
+    if (propagate_colocation_key_) {
+      auto colocation_attr = while_op_->attrs().Find(kColocationAttrName);
+      if (colocation_attr) {
+        builder.Attr(kColocationAttrName, *colocation_attr);
+      }
+    }
+    TF_RETURN_IF_ERROR(builder.Finalize(graph_, &incoming_control_node));
     for (Node* n : enter_nodes_) {
       graph_->AddControlEdge(incoming_control_node, n);
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status LowerWhileHelper::CreateMergeNodes() {
+absl::Status LowerWhileHelper::CreateMergeNodes() {
   for (Node* enter_node : enter_nodes_) {
     bool is_constant = enter_node->attrs().FindByString("is_constant")->b();
     if (is_constant && enter_node->output_type(0) == DT_RESOURCE) {
       continue;
     }
     Node* merge_node;
-    TF_RETURN_IF_ERROR(
+    NodeBuilder builder =
         NodeBuilder(NewName("merge"), "Merge", flib_def_, &debug_info_)
             .Input({NodeOut(enter_node, 0), NodeOut(enter_node, 0)})
             .Device(enter_node->requested_device())
-            .AssignedDevice(enter_node->assigned_device_name())
-            .Finalize(graph_, &merge_node));
+            .AssignedDevice(enter_node->assigned_device_name());
+    if (propagate_colocation_key_) {
+      auto colocation_attr = enter_node->attrs().Find(kColocationAttrName);
+      if (colocation_attr) {
+        builder.Attr(kColocationAttrName, *colocation_attr);
+      }
+    }
+    TF_RETURN_IF_ERROR(builder.Finalize(graph_, &merge_node));
     merge_nodes_.emplace_back(merge_node);
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status LowerWhileHelper::CreateCondFuncCallNode() {
+absl::Status LowerWhileHelper::CreateCondFuncCallNode() {
   for (int i = 0; i < num_loop_inputs_; i++) {
     if (IsLoopCarriedResource(i)) {
       cond_call_builder_.Input(NodeOut(enter_nodes_[i], 0));
@@ -312,15 +344,21 @@ Status LowerWhileHelper::CreateCondFuncCallNode() {
   // are in the same frame as the rest of the function, otherwise
   // `BuildControlFlowInfo` throws an error.
   graph_->AddControlEdge(merge_nodes_[0], cond_call_node_);
-  TF_RETURN_IF_ERROR(
+  NodeBuilder builder =
       NodeBuilder(NewName("LoopCond"), "LoopCond", flib_def_, &debug_info_)
           .Input(NodeOut(cond_call_node_, 0))
-          .Device(while_op_->requested_device())
-          .Finalize(graph_, &loop_cond_node_));
-  return OkStatus();
+          .Device(while_op_->requested_device());
+  if (propagate_colocation_key_) {
+    auto colocation_attr = while_op_->attrs().Find(kColocationAttrName);
+    if (colocation_attr) {
+      builder.Attr(kColocationAttrName, *colocation_attr);
+    }
+  }
+  TF_RETURN_IF_ERROR(builder.Finalize(graph_, &loop_cond_node_));
+  return absl::OkStatus();
 }
 
-Status LowerWhileHelper::CreateSwitchNodes() {
+absl::Status LowerWhileHelper::CreateSwitchNodes() {
   for (int i = 0; i < num_loop_inputs_; i++) {
     if (IsLoopCarriedResource(i)) {
       continue;
@@ -337,19 +375,26 @@ Status LowerWhileHelper::CreateSwitchNodes() {
     if (IsRefType(merge_node->output_type(0))) {
       op_type = "RefSwitch";
     }
-    TF_RETURN_IF_ERROR(
+    NodeBuilder builder =
         NodeBuilder(NewName(op_name), op_type, flib_def_, &debug_info_)
             .Input(NodeOut(merge_node, 0))
             .Input(NodeOut(loop_cond_node_, 0))
             .Device(merge_node->requested_device())
-            .AssignedDevice(merge_node->assigned_device_name())
-            .Finalize(graph_, &switch_node));
+            .AssignedDevice(merge_node->assigned_device_name());
+
+    if (propagate_colocation_key_) {
+      auto colocation_attr = merge_node->attrs().Find(kColocationAttrName);
+      if (colocation_attr) {
+        builder.Attr(kColocationAttrName, *colocation_attr);
+      }
+    }
+    TF_RETURN_IF_ERROR(builder.Finalize(graph_, &switch_node));
     switch_nodes_.emplace_back(switch_node);
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status LowerWhileHelper::CreateBodyFuncCallNode() {
+absl::Status LowerWhileHelper::CreateBodyFuncCallNode() {
   for (int i = 0; i < num_loop_inputs_; i++) {
     if (IsLoopCarriedResource(i)) {
       body_call_builder_.Input(NodeOut(enter_nodes_[i], 0));
@@ -372,16 +417,22 @@ Status LowerWhileHelper::CreateBodyFuncCallNode() {
   if (IsRefType(switch_nodes_[0]->output_type(1))) {
     op_type = "RefIdentity";
   }
-  TF_RETURN_IF_ERROR(NodeBuilder(NewName("loop_body_control"), op_type,
-                                 flib_def_, &debug_info_)
-                         .Input(NodeOut(switch_nodes_[0], 1))
-                         .Device(while_op_->requested_device())
-                         .Finalize(graph_, &body_control_node_));
+  NodeBuilder builder = NodeBuilder(NewName("loop_body_control"), op_type,
+                                    flib_def_, &debug_info_)
+                            .Input(NodeOut(switch_nodes_[0], 1))
+                            .Device(while_op_->requested_device());
+  if (propagate_colocation_key_) {
+    auto colocation_attr = while_op_->attrs().Find(kColocationAttrName);
+    if (colocation_attr) {
+      builder.Attr(kColocationAttrName, *colocation_attr);
+    }
+  }
+  TF_RETURN_IF_ERROR(builder.Finalize(graph_, &body_control_node_));
   graph_->AddControlEdge(body_control_node_, body_call_node_);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status LowerWhileHelper::CreateExitNodes() {
+absl::Status LowerWhileHelper::CreateExitNodes() {
   std::vector<NodeOut> outputs;
   outputs.reserve(num_loop_inputs_);
   for (int i = 0; i < num_loop_inputs_; i++) {
@@ -393,15 +444,24 @@ Status LowerWhileHelper::CreateExitNodes() {
       outputs.emplace_back(resource_tensor);
     } else {
       Node* exit_node;
-      TF_RETURN_IF_ERROR(
+      NodeBuilder builder =
           NodeBuilder(NewName("exit"), "Exit", flib_def_, &debug_info_)
               .Input(NodeOut(switch_nodes_[op_input_output_to_lowered_node_[i]],
                              0))
               .Device(switch_nodes_[op_input_output_to_lowered_node_[i]]
                           ->requested_device())
               .AssignedDevice(switch_nodes_[op_input_output_to_lowered_node_[i]]
-                                  ->assigned_device_name())
-              .Finalize(graph_, &exit_node));
+                                  ->assigned_device_name());
+
+      if (propagate_colocation_key_) {
+        auto colocation_attr =
+            switch_nodes_[op_input_output_to_lowered_node_[i]]->attrs().Find(
+                kColocationAttrName);
+        if (colocation_attr) {
+          builder.Attr(kColocationAttrName, *colocation_attr);
+        }
+      }
+      TF_RETURN_IF_ERROR(builder.Finalize(graph_, &exit_node));
       exit_nodes_.emplace_back(exit_node);
       outputs.emplace_back(NodeOut(exit_node, 0));
     }
@@ -413,11 +473,17 @@ Status LowerWhileHelper::CreateExitNodes() {
 
   // Add a NoOp node that has control edges from all Exit nodes. This node is
   // used for rewriting control edges with the original while op as src.
-  TF_RETURN_IF_ERROR(NodeBuilder(NewName("LoopExecuted"), "NoOp",
-                                 OpRegistry::Global(), &debug_info_)
-                         .ControlInputs(exit_nodes_)
-                         .Device(while_op_->requested_device())
-                         .Finalize(graph_, &lowered_while_executed_));
+  NodeBuilder builder = NodeBuilder(NewName("LoopExecuted"), "NoOp",
+                                    OpRegistry::Global(), &debug_info_)
+                            .ControlInputs(exit_nodes_)
+                            .Device(while_op_->requested_device());
+  if (propagate_colocation_key_) {
+    auto colocation_attr = while_op_->attrs().Find(kColocationAttrName);
+    if (colocation_attr) {
+      builder.Attr(kColocationAttrName, *colocation_attr);
+    }
+  }
+  TF_RETURN_IF_ERROR(builder.Finalize(graph_, &lowered_while_executed_));
 
   if (keep_node_fetchable_) {
     // Add an IdentityN node that has the same outputs and same name as the
@@ -439,37 +505,44 @@ Status LowerWhileHelper::CreateExitNodes() {
             .Finalize(graph_, &lowered_while_output_));
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status LowerWhileHelper::CreateNextIterationNodes() {
+absl::Status LowerWhileHelper::CreateNextIterationNodes() {
   for (int i = 0; i < num_loop_inputs_; i++) {
     Node* next_iteration;
     if (IsLoopCarriedResource(i)) {
       continue;
     }
     Node* merge_node = merge_nodes_[op_input_output_to_lowered_node_[i]];
-    TF_RETURN_IF_ERROR(NodeBuilder(NewName("next_iteration"), "NextIteration",
-                                   flib_def_, &debug_info_)
-                           .Input(NodeOut(body_call_node_, i))
-                           .ControlInput(body_call_node_)
-                           .Device(merge_node->requested_device())
-                           .AssignedDevice(merge_node->assigned_device_name())
-                           .Finalize(graph_, &next_iteration));
+    NodeBuilder builder =
+        NodeBuilder(NewName("next_iteration"), "NextIteration", flib_def_,
+                    &debug_info_)
+            .Input(NodeOut(body_call_node_, i))
+            .ControlInput(body_call_node_)
+            .Device(merge_node->requested_device())
+            .AssignedDevice(merge_node->assigned_device_name());
+    if (propagate_colocation_key_) {
+      auto colocation_attr = merge_node->attrs().Find(kColocationAttrName);
+      if (colocation_attr) {
+        builder.Attr(kColocationAttrName, *colocation_attr);
+      }
+    }
+    TF_RETURN_IF_ERROR(builder.Finalize(graph_, &next_iteration));
     next_iterations_nodes_.emplace_back(next_iteration);
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status LowerWhileHelper::UpdateMergeNodes() {
+absl::Status LowerWhileHelper::UpdateMergeNodes() {
   for (int i = 0; i < merge_nodes_.size(); i++) {
     TF_RETURN_IF_ERROR(
         graph_->UpdateEdge(next_iterations_nodes_[i], 0, merge_nodes_[i], 1));
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status LowerWhileHelper::UpdateConsumers() {
+absl::Status LowerWhileHelper::UpdateConsumers() {
   for (const Edge* e : while_op_->out_edges()) {
     if (e->IsControlEdge()) {
       graph_->AddControlEdge(lowered_while_executed_, e->dst());
@@ -493,7 +566,7 @@ Status LowerWhileHelper::UpdateConsumers() {
       }
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 string LowerWhileHelper::NewName(const string& infix) {
@@ -515,9 +588,9 @@ bool LowerWhileHelper::IsLoopCarriedResource(int index) {
 
 }  // namespace
 
-Status RewriteWhileNode(Node* n, Graph* g,
-                        const FunctionLibraryDefinition* flib_def,
-                        bool keep_node_fetchable) {
+absl::Status RewriteWhileNode(Node* n, Graph* g,
+                              const FunctionLibraryDefinition* flib_def,
+                              bool keep_node_fetchable) {
   VLOG(2) << "Lower While node (keep_node_fetchable=" << keep_node_fetchable
           << "): " << SummarizeNode(*n);
 
@@ -543,7 +616,7 @@ Status RewriteWhileNode(Node* n, Graph* g,
       flib_def, keep_node_fetchable));
   g->RemoveNode(n);
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace tensorflow
