@@ -25,8 +25,14 @@ limitations under the License.
 #include <vector>
 
 #include "absl/status/status.h"
+#include "absl/strings/str_cat.h"
 #include "absl/time/clock.h"
 #include "absl/time/time.h"
+#include "xla/tsl/platform/env.h"
+#include "xla/tsl/platform/errors.h"
+#include "xla/tsl/platform/logging.h"
+#include "xla/tsl/platform/status.h"
+#include "xla/tsl/platform/statusor.h"
 #include "tensorflow/core/common_runtime/composite_device.h"
 #include "tensorflow/core/common_runtime/device_set.h"
 #include "tensorflow/core/common_runtime/function_body.h"
@@ -40,23 +46,21 @@ limitations under the License.
 #include "tensorflow/core/common_runtime/placer.h"
 #include "tensorflow/core/common_runtime/replicate_per_replica_nodes.h"
 #include "tensorflow/core/framework/device.h"
+#include "tensorflow/core/framework/function.h"
 #include "tensorflow/core/framework/graph.pb.h"
 #include "tensorflow/core/framework/metrics.h"
 #include "tensorflow/core/framework/optimized_function_graph.pb.h"
+#include "tensorflow/core/framework/types.h"
 #include "tensorflow/core/graph/graph.h"
 #include "tensorflow/core/graph/graph_node_util.h"
 #include "tensorflow/core/platform/env.h"
+#include "tensorflow/core/platform/refcount.h"
 #include "tensorflow/core/util/debug_data_dumper.h"
-#include "tensorflow/core/util/dump_graph.h"
-#include "tensorflow/tsl/platform/env.h"
-#include "tensorflow/tsl/platform/errors.h"
-#include "tensorflow/tsl/platform/host_info.h"
-#include "tensorflow/tsl/platform/status.h"
-#include "tensorflow/tsl/platform/statusor.h"
+#include "tsl/platform/host_info.h"
 
 namespace tensorflow {
 namespace {
-Status ValidateNoListArguments(
+absl::Status ValidateNoListArguments(
     const protobuf::RepeatedPtrField<OpDef::ArgDef>& args, const char* arg_type,
     const string& function_name) {
   for (const OpDef::ArgDef& arg : args) {
@@ -69,10 +73,10 @@ Status ValidateNoListArguments(
           " and outputs");
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status ValidateMultiDeviceOptions(
+absl::Status ValidateMultiDeviceOptions(
     const FunctionDef& fdef,
     const FunctionLibraryRuntime::InstantiateOptions& options) {
   const OpDef& signature = fdef.signature();
@@ -104,12 +108,13 @@ Status ValidateMultiDeviceOptions(
         options.output_devices.size(),
         " number of arguments = ", signature.output_arg_size());
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status SetArgShape(const std::unordered_map<int, DtypeAndPartialTensorShape>&
-                       input_resource_dtypes_and_shapes,
-                   const std::vector<Node*>& arg_nodes) {
+absl::Status SetArgShape(
+    const std::unordered_map<int, DtypeAndPartialTensorShape>&
+        input_resource_dtypes_and_shapes,
+    const std::vector<Node*>& arg_nodes) {
   for (Node* n : arg_nodes) {
     int index;
     TF_RETURN_IF_ERROR(GetNodeAttr(n->def(), "index", &index));
@@ -130,7 +135,7 @@ Status SetArgShape(const std::unordered_map<int, DtypeAndPartialTensorShape>&
       }
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 const string* AssignedOrRequestedDeviceName(const Node& node) {
@@ -145,7 +150,8 @@ const string* AssignedOrRequestedDeviceName(const Node& node) {
 void GetColocationGroup(const Node* node, string* group) {
   // We hoist the conversion from C-style string literal to string here,
   // so that we can avoid the many repeated calls to strlen().
-  static const StringPiece kColocationAttrNameStringPiece(kColocationAttrName);
+  static const absl::string_view kColocationAttrNameStringPiece(
+      kColocationAttrName);
   const AttrValue* attr_value =
       node->attrs().Find(kColocationAttrNameStringPiece);
   if (attr_value != nullptr && attr_value->has_list() &&
@@ -156,9 +162,9 @@ void GetColocationGroup(const Node* node, string* group) {
 
 // Writes the OptimizedFunctionGraphInfo proto into a cache file.
 // Returns error if the cache file writing fails.
-Status WriteToCache(const string& dir_name, const string& file_name,
-                    OptimizedFunctionGraphInfo& optimized_function_graph_info,
-                    Env* env) {
+absl::Status WriteToCache(
+    const std::string& dir_name, const std::string& file_name,
+    OptimizedFunctionGraphInfo& optimized_function_graph_info, Env* env) {
   const absl::Time cache_writing_start_time = absl::Now();
 
   OptimizedFunctionGraph optimized_function_graph_proto;
@@ -172,8 +178,26 @@ Status WriteToCache(const string& dir_name, const string& file_name,
   if (!env->FileExists(dir_name).ok()) {
     TF_RETURN_IF_ERROR(env->RecursivelyCreateDir(dir_name));
   }
+  {
+    bool has_atomic_move = false;
+    TF_RETURN_IF_ERROR(env->HasAtomicMove(dir_name, &has_atomic_move));
+    if (!has_atomic_move) {
+      LOG_EVERY_POW_2(WARNING)
+          << "Filesystem for OptimizedFunctionGraphInfo persistent cache at "
+          << dir_name
+          << " does not support atomic moves. Therefore the "
+             "persistent cache is racy if you have multiple optimizations "
+             "occurring simultaneously!";
+    }
+  }
+  std::string temp_file_name = file_name;
+  if (!env->CreateUniqueFileName(&temp_file_name, ".pb.tmp")) {
+    return absl::UnavailableError(
+        absl::StrCat("Could not create a unique file inside ", dir_name));
+  }
   TF_RETURN_IF_ERROR(tsl::WriteStringToFile(
-      env, file_name, optimized_function_graph_proto_str));
+      env, temp_file_name, optimized_function_graph_proto_str));
+  TF_RETURN_IF_ERROR(env->RenameFile(temp_file_name, file_name));
 
   const absl::Duration cache_writing_duration =
       absl::Now() - cache_writing_start_time;
@@ -181,13 +205,13 @@ Status WriteToCache(const string& dir_name, const string& file_name,
           << absl::ToInt64Milliseconds(cache_writing_duration)
           << " msecs, file name: " << file_name;
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Retrieves the OptimizedFunctionGraphInfo from a cache file.
 // Returns error if cache file loading fails.
-StatusOr<OptimizedFunctionGraphInfo> ReadFromCache(const string& file_name,
-                                                   Env* env) {
+absl::StatusOr<OptimizedFunctionGraphInfo> ReadFromCache(
+    const string& file_name, Env* env) {
   absl::Time cache_reading_start_time = absl::Now();
 
   OptimizedFunctionGraph optimized_function_graph_proto;
@@ -197,10 +221,10 @@ StatusOr<OptimizedFunctionGraphInfo> ReadFromCache(const string& file_name,
 
   optimized_function_graph_proto.ParseFromString(
       optimized_function_graph_proto_str);
-  TF_ASSIGN_OR_RETURN(
-      StatusOr<OptimizedFunctionGraphInfo>
-          optimized_function_graph_info_restored,
-      OptimizedFunctionGraphInfo::FromProto(optimized_function_graph_proto));
+  TF_ASSIGN_OR_RETURN(absl::StatusOr<OptimizedFunctionGraphInfo>
+                          optimized_function_graph_info_restored,
+                      OptimizedFunctionGraphInfo::FromProto(
+                          std::move(optimized_function_graph_proto)));
 
   const absl::Duration cache_reading_duration =
       absl::Now() - cache_reading_start_time;
@@ -234,17 +258,21 @@ string GetFileCacheName(const string& dir_name, const string& function_name,
                       tsl::port::TaskId(), "_", plain_func_name, "_",
                       fdef->node_def_size());
 }
-}  // namespace
 
-Status GetGraphAndArgRets(
-    const string& function_name, AttrSlice attrs, const FunctionDef* fdef,
-    const FunctionLibraryDefinition* lib_def, std::unique_ptr<Graph>* graph,
-    std::vector<Node*>* arg_nodes, std::vector<Node*>* ret_nodes,
-    std::vector<string>* ret_node_names, DataTypeVector* ret_types,
-    std::vector<string>* control_ret_node_names) {
+// Generates graph and return information given the input function name,
+// attributes and function definition.
+absl::Status GetGraphAndArgRets(const string& function_name, AttrSlice attrs,
+                                core::RefCountPtr<FunctionRecord>&& fdef,
+                                const FunctionLibraryDefinition* lib_def,
+                                std::unique_ptr<Graph>* graph,
+                                std::vector<Node*>* arg_nodes,
+                                std::vector<Node*>* ret_nodes,
+                                std::vector<string>* ret_node_names,
+                                DataTypeVector* ret_types,
+                                std::vector<string>* control_ret_node_names) {
   std::unique_ptr<FunctionBody> fbody;
-  // TODO(iga): FunctionDefToBodyHelper copies fdef. Avoid this copy.
-  TF_RETURN_IF_ERROR(FunctionDefToBodyHelper(*fdef, attrs, lib_def, &fbody));
+  TF_RETURN_IF_ERROR(
+      FunctionDefToBodyHelper(std::move(fdef), attrs, lib_def, &fbody));
   if (!fbody) {
     LOG(ERROR) << "Failed to get FunctionBody for \"" << function_name << "\"";
     return errors::Internal("Failed to construct FunctionBody for ",
@@ -269,16 +297,17 @@ Status GetGraphAndArgRets(
   for (const Node* node : fbody->control_ret_nodes) {
     control_ret_node_names->push_back(node->name());
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
+}  // namespace
 
-Status PinArgsAndRets(const std::vector<string>& input_devices,
-                      const std::vector<string>& output_devices,
-                      const DeviceSet& device_set,
-                      const std::vector<Node*>& arg_nodes,
-                      const std::vector<Node*>& ret_nodes,
-                      const FunctionLibraryDefinition* lib_def,
-                      Device* default_device) {
+absl::Status PinArgsAndRets(const std::vector<string>& input_devices,
+                            const std::vector<string>& output_devices,
+                            const DeviceSet& device_set,
+                            const std::vector<Node*>& arg_nodes,
+                            const std::vector<Node*>& ret_nodes,
+                            const FunctionLibraryDefinition* lib_def,
+                            Device* default_device) {
   // If output_devices are not specified, we want to set the output device
   // based on the device of the output producing node. The output producing
   // node can be an arg node because functions can simply return their
@@ -289,6 +318,9 @@ Status PinArgsAndRets(const std::vector<string>& input_devices,
     TF_RETURN_IF_ERROR(node->attrs().Find("index", &attr_value));
     int64_t index = attr_value->i();
     node->set_assigned_device_name(input_devices[index]);
+
+    VLOG(3) << "Setting device to " << input_devices[index] << " for node "
+            << node->name();
   }
 
   for (Node* node : ret_nodes) {
@@ -434,10 +466,10 @@ Status PinArgsAndRets(const std::vector<string>& input_devices,
       node->set_assigned_device_name(output_devices[index]);
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-StatusOr<OptimizedFunctionGraphInfo> OptimizeFunctionGraph(
+absl::StatusOr<OptimizedFunctionGraphInfo> OptimizeFunctionGraph(
     const string& function_name, AttrSlice attrs,
     const FunctionLibraryRuntime::InstantiateOptions& options,
     const DeviceSet& dev_set, const FunctionLibraryDefinition* input_lib_def,
@@ -448,13 +480,13 @@ StatusOr<OptimizedFunctionGraphInfo> OptimizeFunctionGraph(
   const FunctionLibraryDefinition* lib_def =
       options.lib_def == nullptr ? input_lib_def : options.lib_def;
 
-  const FunctionDef* fdef = lib_def->Find(function_name);
+  core::RefCountPtr<FunctionRecord> fdef = lib_def->FindRecord(function_name);
   if (fdef == nullptr) {
     return errors::InvalidArgument("Failed to find function \"", function_name,
                                    "\" in function library: ", lib_def);
   }
 
-  TF_RETURN_IF_ERROR(ValidateMultiDeviceOptions(*fdef, options));
+  TF_RETURN_IF_ERROR(ValidateMultiDeviceOptions(fdef->fdef(), options));
 
   std::unique_ptr<Graph> graph;
   std::vector<Node*> arg_nodes, ret_nodes;
@@ -463,18 +495,19 @@ StatusOr<OptimizedFunctionGraphInfo> OptimizeFunctionGraph(
   std::vector<string> control_ret_node_names;
 
   TF_RETURN_IF_ERROR(GetGraphAndArgRets(
-      function_name, attrs, fdef, lib_def, &graph, &arg_nodes, &ret_nodes,
-      &ret_node_names, &ret_types, &control_ret_node_names));
+      function_name, attrs, fdef.GetNewRef(), lib_def, &graph, &arg_nodes,
+      &ret_nodes, &ret_node_names, &ret_types, &control_ret_node_names));
 
   DEBUG_DATA_DUMPER()->DumpOpCreationStackTraces(
       function_name, kDebugGroupOpStacktrace, "before_opt", graph.get());
 
-  GraphDef graph_def;
-  graph->ToGraphDef(&graph_def);
   FunctionLibraryDefinition reachable_lib_def =
-      lib_def->ReachableDefinitions(graph_def);
-  *graph_def.mutable_library() = reachable_lib_def.ToProto();
+      lib_def->ReachableDefinitions(*graph);
+
   if (options.graph_collector != nullptr) {
+    GraphDef graph_def;
+    graph->ToGraphDef(&graph_def);
+    *graph_def.mutable_library() = reachable_lib_def.ToProto();
     options.graph_collector->CollectRawGraph(graph_def);
   }
 
@@ -501,6 +534,10 @@ StatusOr<OptimizedFunctionGraphInfo> OptimizeFunctionGraph(
       ret_nodes, lib_def,
       options.config_proto.allow_soft_placement() ? default_device : nullptr));
 
+  DEBUG_DATA_DUMPER()->DumpGraph(function_name, kDebugGroupMain,
+                                 "before_bridge", graph.get(),
+                                 &reachable_lib_def, false);
+
   // The runtime shouldn't depend on duplication between the function library
   // owned by the graph and the one owned by the runtime. To ensure this, for
   // now we ensure that the graph function library is empty and the runtime
@@ -536,7 +573,7 @@ StatusOr<OptimizedFunctionGraphInfo> OptimizeFunctionGraph(
       node_name_to_control_ret.emplace(control_ret, control_ret);
     }
   } else {
-    for (const auto& control_ret : fdef->control_ret()) {
+    for (const auto& control_ret : fdef->fdef().control_ret()) {
       node_name_to_control_ret.emplace(control_ret.second, control_ret.first);
     }
   }
@@ -553,7 +590,7 @@ StatusOr<OptimizedFunctionGraphInfo> OptimizeFunctionGraph(
   optimization_options.is_function_graph = true;
   optimization_options.composite_devices = &composite_devices;
   optimization_options.default_function_device = default_device;
-  optimization_options.function_def = fdef;
+  optimization_options.function_def = &fdef->fdef();
   optimization_options.shape_inference_on_tfe_dialect_import =
       options.shape_inference_on_tfe_dialect_import;
   optimization_options.debug_filename_prefix = function_name;
@@ -589,7 +626,7 @@ StatusOr<OptimizedFunctionGraphInfo> OptimizeFunctionGraph(
     DEBUG_DATA_DUMPER()->DumpGraph(function_name, kDebugGroupMain,
                                    "before_graph_optimization", graph.get(),
                                    &reachable_lib_def, false);
-    Status status = options.optimize_graph_fn(
+    absl::Status status = options.optimize_graph_fn(
         std::move(ret_node_names), std::move(control_ret_node_names),
         &reachable_lib_def, dev_set, cpu_device, &graph);
     if (!status.ok()) {
@@ -614,14 +651,19 @@ StatusOr<OptimizedFunctionGraphInfo> OptimizeFunctionGraph(
 
   graph->mutable_flib_def()->set_default_registry(nullptr);
   graph->mutable_flib_def()->Clear();
+
+  FunctionLibraryDefinition pruned_lib_def =
+      reachable_lib_def.ReachableDefinitions(*graph);
+
   return OptimizedFunctionGraphInfo(
-      function_name, std::move(graph), std::move(reachable_lib_def),
+      function_name, std::move(graph), std::move(pruned_lib_def),
       node_name_to_control_ret, ret_types, ret_nodes.size(),
       env->NowMicros() - graph_optimization_start_time_usecs,
       optimization_source);
 }
 
-StatusOr<OptimizedFunctionGraphInfo> OptimizeFunctionGraphOrReadFromFileCache(
+absl::StatusOr<OptimizedFunctionGraphInfo>
+OptimizeFunctionGraphOrReadFromFileCache(
     const string& function_name, AttrSlice attrs,
     const FunctionLibraryRuntime::InstantiateOptions& options,
     const DeviceSet& dev_set, const FunctionLibraryDefinition* input_lib_def,
@@ -662,7 +704,7 @@ StatusOr<OptimizedFunctionGraphInfo> OptimizeFunctionGraphOrReadFromFileCache(
         << "TensorFlow graph cache existed; reading from cache; function name: "
         << function_name << ", full cache file path: " << file_name;
 
-    StatusOr<OptimizedFunctionGraphInfo> optimized_function_graph_info =
+    absl::StatusOr<OptimizedFunctionGraphInfo> optimized_function_graph_info =
         ReadFromCache(file_name, env);
     if (optimized_function_graph_info.ok()) {
       metrics::UpdateFunctionGraphOptimizationSavingTime(
@@ -703,7 +745,7 @@ StatusOr<OptimizedFunctionGraphInfo> OptimizeFunctionGraphOrReadFromFileCache(
   // Step 1: Run the graph optimization passes normally.
   absl::Time optimization_start_time = absl::Now();
   TF_ASSIGN_OR_RETURN(
-      StatusOr<OptimizedFunctionGraphInfo> optimized_function_graph_info,
+      absl::StatusOr<OptimizedFunctionGraphInfo> optimized_function_graph_info,
       OptimizeFunctionGraph(function_name, attrs, options, dev_set,
                             input_lib_def, composite_devices, cpu_device,
                             default_device, env, OptimizedFunctionGraph::JIT));
@@ -718,8 +760,8 @@ StatusOr<OptimizedFunctionGraphInfo> OptimizeFunctionGraphOrReadFromFileCache(
     LOG(INFO)
         << "Writing the optimized TensorFlow graph into cache: function name: "
         << function_name << ", full cache file path: " << file_name;
-    Status s = WriteToCache(dir_name, file_name,
-                            optimized_function_graph_info.value(), env);
+    absl::Status s = WriteToCache(dir_name, file_name,
+                                  optimized_function_graph_info.value(), env);
     // If writing to cache failed, log the error message and move on without
     // failing the program.
     if (!s.ok()) {
@@ -739,7 +781,8 @@ StatusOr<OptimizedFunctionGraphInfo> OptimizeFunctionGraphOrReadFromFileCache(
   return optimized_function_graph_info;
 }
 
-StatusOr<std::unique_ptr<std::unordered_map<string, std::unique_ptr<Graph>>>>
+absl::StatusOr<
+    std::unique_ptr<std::unordered_map<string, std::unique_ptr<Graph>>>>
 PreprocessAndPartitionGraph(
     const std::string& function_name,
     OptimizedFunctionGraphInfo& input_optimized_graph,
