@@ -16,9 +16,8 @@ limitations under the License.
 #include "tensorflow/core/common_runtime/pluggable_device/pluggable_device.h"
 
 #include <stdlib.h>
-#include <string.h>
 
-#include <algorithm>
+#include <cstdint>
 #include <list>
 #include <map>
 #include <memory>
@@ -27,38 +26,42 @@ limitations under the License.
 #include <utility>
 #include <vector>
 
-#include "tensorflow/compiler/xla/stream_executor/device_id_utils.h"
+#include "absl/log/check.h"
+#include "absl/log/log.h"
+#include "absl/status/status.h"
+#include "absl/strings/ascii.h"
+#include "xla/stream_executor/stream_executor.h"
+#include "xla/tsl/platform/errors.h"
 #include "tensorflow/core/common_runtime/device/device_event_mgr.h"
 #include "tensorflow/core/common_runtime/device/device_id.h"
 #include "tensorflow/core/common_runtime/device/device_id_manager.h"
-#include "tensorflow/core/common_runtime/device_factory.h"
+#include "tensorflow/core/common_runtime/device_id_utils.h"
 #include "tensorflow/core/common_runtime/local_device.h"
 #include "tensorflow/core/common_runtime/pluggable_device/pluggable_device_context.h"
-#include "tensorflow/core/common_runtime/pluggable_device/pluggable_device_factory.h"
 #include "tensorflow/core/common_runtime/pluggable_device/pluggable_device_init.h"
 #include "tensorflow/core/common_runtime/pluggable_device/pluggable_device_process_state.h"
 #include "tensorflow/core/common_runtime/pluggable_device/pluggable_device_util.h"
 #include "tensorflow/core/framework/allocator.h"
+#include "tensorflow/core/framework/device_attributes.pb.h"
 #include "tensorflow/core/framework/device_base.h"
 #include "tensorflow/core/framework/op_kernel.h"
 #include "tensorflow/core/framework/tensor.h"
 #include "tensorflow/core/framework/tensor.pb.h"
 #include "tensorflow/core/framework/types.h"
+#include "tensorflow/core/framework/types.pb.h"
 #include "tensorflow/core/framework/variant_op_registry.h"
 #include "tensorflow/core/graph/types.h"
 #include "tensorflow/core/lib/core/errors.h"
 #include "tensorflow/core/lib/core/status.h"
-#include "tensorflow/core/lib/strings/numbers.h"
-#include "tensorflow/core/lib/strings/str_util.h"
 #include "tensorflow/core/lib/strings/strcat.h"
+#include "tensorflow/core/platform/env.h"
 #include "tensorflow/core/platform/logging.h"
-#include "tensorflow/core/platform/macros.h"
-#include "tensorflow/core/platform/stream_executor.h"
+#include "tensorflow/core/platform/mutex.h"
+#include "tensorflow/core/platform/notification.h"
+#include "tensorflow/core/platform/threadpool.h"
 #include "tensorflow/core/platform/types.h"
 #include "tensorflow/core/public/session_options.h"
-#include "tensorflow/core/util/device_name_utils.h"
 #include "tensorflow/core/util/env_var.h"
-#include "tensorflow/core/util/stream_executor_util.h"
 
 namespace tensorflow {
 
@@ -85,18 +88,41 @@ class PluggableDevice::StreamGroupFactory {
     StreamGroup* group = &streams_[key_type(device_type, tf_device_id.value(),
                                             stream_group_within_device)];
     if (!group->compute) {
-      group->compute = new se::Stream(executor);
-      group->compute->Init();
+      auto stream_or_status = executor->CreateStream();
+      if (!stream_or_status.ok()) {
+        LOG(ERROR) << "Failed to create stream for device "
+                   << tf_device_id.value()
+                   << " with status: " << stream_or_status.status();
+        return group;
+      }
+      group->compute = stream_or_status->get();
+      allocated_streams_.emplace_back(std::move(stream_or_status.value()));
       VLOG(2) << "Created stream[" << stream_group_within_device
               << "] = " << group->compute;
 
-      group->host_to_device = new se::Stream(executor);
-      group->host_to_device->Init();
+      stream_or_status = executor->CreateStream();
+      if (!stream_or_status.ok()) {
+        LOG(ERROR) << "Failed to create stream for device "
+                   << tf_device_id.value()
+                   << " with status: " << stream_or_status.status();
+        return group;
+      }
+      group->compute = stream_or_status->get();
+      group->host_to_device = stream_or_status->get();
+      allocated_streams_.emplace_back(std::move(stream_or_status.value()));
       VLOG(2) << "Created host_to_device_stream[" << stream_group_within_device
               << "] = " << group->host_to_device;
 
-      group->device_to_host = new se::Stream(executor);
-      group->device_to_host->Init();
+      stream_or_status = executor->CreateStream();
+      if (!stream_or_status.ok()) {
+        LOG(ERROR) << "Failed to create stream for device "
+                   << tf_device_id.value()
+                   << " with status: " << stream_or_status.status();
+        return group;
+      }
+      group->compute = stream_or_status->get();
+      group->device_to_host = stream_or_status->get();
+      allocated_streams_.emplace_back(std::move(stream_or_status.value()));
       VLOG(2) << "Created device_to_host_stream[" << stream_group_within_device
               << "] = " << group->device_to_host;
 
@@ -110,9 +136,16 @@ class PluggableDevice::StreamGroupFactory {
         num_d2d_streams = 1;
       }
       for (int i = 0; i < num_d2d_streams; ++i) {
-        se::Stream* stream = new se::Stream(executor);
-        stream->Init();
-        group->device_to_device.push_back(stream);
+        stream_or_status = executor->CreateStream();
+        if (!stream_or_status.ok()) {
+          LOG(ERROR) << "Failed to create stream for device "
+                     << tf_device_id.value()
+                     << " with status: " << stream_or_status.status();
+          return group;
+        }
+        group->compute = stream_or_status->get();
+        group->device_to_device.push_back(stream_or_status->get());
+        allocated_streams_.emplace_back(std::move(stream_or_status.value()));
         VLOG(2) << "Created device_to_device_stream["
                 << stream_group_within_device
                 << "] = " << group->device_to_device.back();
@@ -132,11 +165,12 @@ class PluggableDevice::StreamGroupFactory {
   mutex lock_;
   using key_type = std::tuple<std::string, int, int>;
   std::map<key_type, StreamGroup> streams_;
-
+  std::vector<std::unique_ptr<se::Stream>> allocated_streams_;
   // StreamGroupFactory cannot be created directly; Call
   // StreamGroupFactory::Global to get the global instance.
   StreamGroupFactory() = default;
-  TF_DISALLOW_COPY_AND_ASSIGN(StreamGroupFactory);
+  StreamGroupFactory(const StreamGroupFactory&) = delete;
+  void operator=(const StreamGroupFactory&) = delete;
 };
 
 PluggableDevice::PluggableDevice(
@@ -153,8 +187,9 @@ PluggableDevice::PluggableDevice(
       tf_device_id_(tf_device_id),
       platform_name_(platform_name),
       sync_every_op_(sync_every_op) {
-  if (options.config.has_gpu_options()) {
-    force_gpu_compatible_ = options.config.gpu_options().force_gpu_compatible();
+  if (options.config.has_pluggable_device_options()) {
+    force_gpu_compatible_ =
+        options.config.pluggable_device_options().force_gpu_compatible();
   }
   PluggableDeviceProcessState::singleton(device_type, platform_name)
       ->EnablePluggableDevice();
@@ -165,9 +200,9 @@ PluggableDevice::~PluggableDevice() {
   device_context_->Unref();
 }
 
-Status PluggableDevice::Init(const SessionOptions& options) {
+absl::Status PluggableDevice::Init(const SessionOptions& options) {
   se::Platform* platform = PluggableDeviceMachineManager(platform_name_);
-  auto executor_status = se::DeviceIdUtil::ExecutorForTfDeviceId(
+  auto executor_status = DeviceIdUtil::ExecutorForTfDeviceId(
       DeviceType(device_type()), platform, tf_device_id_);
   if (!executor_status.status().ok()) {
     return errors::Internal("Failed to get StreamExecutor for device",
@@ -175,11 +210,12 @@ Status PluggableDevice::Init(const SessionOptions& options) {
   }
   executor_ = executor_status.value();
 
-  em_ = EventMgrFactory::Singleton()->GetEventMgr(executor_,
-                                                  options.config.gpu_options());
+  em_ = EventMgrFactory::Singleton()->GetEventMgr(
+      executor_, options.config.pluggable_device_options());
 
   stream_ = StreamGroupFactory::Global().GetOrCreate(
-      device_type(), tf_device_id_, 0, executor_, options.config.gpu_options());
+      device_type(), tf_device_id_, 0, executor_,
+      options.config.pluggable_device_options());
   device_context_ = new PluggableDeviceContext(
       0, stream_->compute, stream_->host_to_device, stream_->device_to_host,
       stream_->device_to_device);
@@ -240,7 +276,7 @@ Status PluggableDevice::Init(const SessionOptions& options) {
     }
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 Allocator* PluggableDevice::GetAllocator(AllocatorAttributes attr) {
@@ -302,7 +338,9 @@ void PluggableDevice::Compute(OpKernel* op_kernel, OpKernelContext* context) {
 
 // Based on the semantics of Device::Sync, this call should wait for
 // all streams not just the current one.
-Status PluggableDevice::Sync() { return PluggableDeviceUtil::SyncAll(this); }
+absl::Status PluggableDevice::Sync() {
+  return PluggableDeviceUtil::SyncAll(this);
+}
 
 void PluggableDevice::ComputeAsync(AsyncOpKernel* op_kernel,
                                    OpKernelContext* context,
@@ -320,17 +358,18 @@ void PluggableDevice::ComputeAsync(AsyncOpKernel* op_kernel,
   op_kernel->ComputeAsync(context, std::move(done));
 }
 
-Status PluggableDevice::MaybeCopyTensorToPluggableDevice(
+absl::Status PluggableDevice::MaybeCopyTensorToPluggableDevice(
     const AllocatorAttributes& alloc_attrs, const Tensor& from, Tensor* to,
     StatusCallback done) {
   if (alloc_attrs.on_host()) {
     *to = from;
-    done(OkStatus());
-    return OkStatus();
+    done(absl::OkStatus());
+    return absl::OkStatus();
   } else {
     if (!DMAHelper::CanUseDMA(&from)) {
-      Status err = errors::Internal("PluggableDevice copy from non-DMA ",
-                                    DataTypeString(from.dtype()), " tensor");
+      absl::Status err =
+          errors::Internal("PluggableDevice copy from non-DMA ",
+                           DataTypeString(from.dtype()), " tensor");
       done(err);
       return err;
     }
@@ -341,14 +380,15 @@ Status PluggableDevice::MaybeCopyTensorToPluggableDevice(
     // If the tensor is not initialized, we likely ran out of memory.
     if (!copy->IsInitialized()) {
       delete copy;
-      Status err = errors::ResourceExhausted(
+      absl::Status err = errors::ResourceExhausted(
           "OOM when allocating tensor of shape ", from.shape().DebugString(),
           " and type ", DataTypeString(from.dtype()));
       done(err);
       return err;
     }
 
-    auto wrapped_done = [to, copy, done = std::move(done)](const Status& s) {
+    auto wrapped_done = [to, copy,
+                         done = std::move(done)](const absl::Status& s) {
       if (s.ok()) {
         *to = std::move(*copy);
       }
@@ -358,11 +398,11 @@ Status PluggableDevice::MaybeCopyTensorToPluggableDevice(
 
     device_context_->CopyCPUTensorToDevice(
         &from, this, copy, std::move(wrapped_done), false /*sync_dst_compute*/);
-    return OkStatus();
+    return absl::OkStatus();
   }
 }
 
-Status PluggableDevice::MakeTensorFromProto(
+absl::Status PluggableDevice::MakeTensorFromProto(
     const TensorProto& tensor_proto, const AllocatorAttributes alloc_attrs,
     Tensor* tensor) {
   AllocatorAttributes attr;
@@ -382,7 +422,7 @@ Status PluggableDevice::MakeTensorFromProto(
     Variant* copy_variant = copy.flat<Variant>().data();
 
     std::list<Notification> notifications;
-    Status copy_status;
+    absl::Status copy_status;
     auto copier = [this, &alloc_attrs, &notifications, &copy_status](
                       const Tensor& from, Tensor* to) {
       // Copier isn't run in a multithreaded environment, so we don't
@@ -390,14 +430,14 @@ Status PluggableDevice::MakeTensorFromProto(
       notifications.emplace_back();
       Notification& n = *notifications.rbegin();
       return MaybeCopyTensorToPluggableDevice(
-          alloc_attrs, from, to, [&n, &copy_status](const Status& s) {
+          alloc_attrs, from, to, [&n, &copy_status](const absl::Status& s) {
             if (copy_status.ok()) {
               copy_status.Update(s);
             }
             n.Notify();
           });
     };
-    Status s;
+    absl::Status s;
     for (int64_t ix = 0; ix < parsed.NumElements(); ++ix) {
       s = VariantDeviceCopy(VariantDeviceCopyDirection::HOST_TO_DEVICE,
                             from[ix], &copy_variant[ix], copier);
@@ -415,9 +455,9 @@ Status PluggableDevice::MakeTensorFromProto(
     return copy_status;
   } else {
     Notification n;
-    Status status;
+    absl::Status status;
     TF_RETURN_IF_ERROR(MaybeCopyTensorToPluggableDevice(
-        alloc_attrs, parsed, tensor, [&n, &status](const Status& s) {
+        alloc_attrs, parsed, tensor, [&n, &status](const absl::Status& s) {
           status = s;
           n.Notify();
         }));
