@@ -42,6 +42,7 @@ limitations under the License.
 #include "tensorflow/core/protobuf/config.pb.h"
 #include "tensorflow/core/public/session_options.h"
 #include "tensorflow/core/public/version.h"
+#include "tsl/platform/protobuf.h"
 
 #if GOOGLE_CUDA
 #include "third_party/gpus/cuda/include/cuda.h"
@@ -67,17 +68,17 @@ class TestClusterFLR : public DistributedFunctionLibraryRuntime {
       *handle = next_handle_;
       next_handle_++;
     }
-    done(OkStatus());
+    done(absl::OkStatus());
   }
 
   void Run(const FunctionLibraryRuntime::Options& opts,
            FunctionLibraryRuntime::LocalHandle handle,
-           gtl::ArraySlice<Tensor> args, std::vector<Tensor>* rets,
+           absl::Span<const Tensor> args, std::vector<Tensor>* rets,
            FunctionLibraryRuntime::DoneCallback done) override {}
 
   void Run(const FunctionLibraryRuntime::Options& opts,
            FunctionLibraryRuntime::LocalHandle handle,
-           gtl::ArraySlice<FunctionArg> args, std::vector<FunctionRet>* rets,
+           absl::Span<const FunctionArg> args, std::vector<FunctionRet>* rets,
            FunctionLibraryRuntime::DoneCallback done) override {}
 
   void CleanUp(uint64 step_id, FunctionLibraryRuntime::LocalHandle handle,
@@ -128,7 +129,7 @@ class ProcessFunctionLibraryRuntimeTest : public ::testing::Test {
             ->LookupDevice("/job:a/replica:0/task:0/device:CPU:2", &device2_ptr)
             .code());
     // If no GPU is available, gpu_device_ will remain nullptr.
-    Status status = device_mgr_->LookupDevice(
+    absl::Status status = device_mgr_->LookupDevice(
         "/job:a/replica:0/task:0/device:GPU:0", &gpu_device_);
     if (!status.ok()) {
       CHECK_EQ(nullptr, gpu_device_);
@@ -158,7 +159,7 @@ class ProcessFunctionLibraryRuntimeTest : public ::testing::Test {
             return tsl::core::RefCountPtr<IntraProcessRendezvous>(
                 new IntraProcessRendezvous(device_mgr));
           });
-          return OkStatus();
+          return absl::OkStatus();
         }}));
   }
 
@@ -166,7 +167,7 @@ class ProcessFunctionLibraryRuntimeTest : public ::testing::Test {
     proc_flr_->AddCompositeDevice(d);
   }
 
-  Status Instantiate(
+  absl::Status Instantiate(
       const string& name, test::function::Attrs attrs,
       const FunctionLibraryRuntime::InstantiateOptions& instantiate_opts,
       FunctionLibraryRuntime::Handle* handle) {
@@ -211,14 +212,15 @@ class ProcessFunctionLibraryRuntimeTest : public ::testing::Test {
   }
 
   template <typename T, typename K>
-  Status RunWithRuntime(
+  absl::Status RunWithRuntime(
       const string& name, FunctionLibraryRuntime::Options opts,
       test::function::Attrs attrs,
       const FunctionLibraryRuntime::InstantiateOptions& instantiate_opts,
       const T& args, std::vector<K*> rets,
       ProcessFunctionLibraryRuntime* pflr) {
     FunctionLibraryRuntime::Handle handle;
-    Status status = pflr->Instantiate(name, attrs, instantiate_opts, &handle);
+    absl::Status status =
+        pflr->Instantiate(name, attrs, instantiate_opts, &handle);
     if (!status.ok()) {
       return status;
     }
@@ -233,10 +235,11 @@ class ProcessFunctionLibraryRuntimeTest : public ::testing::Test {
     Notification done;
     opts.runner = &runner;
     std::vector<K> out;
-    pflr->Run(opts, handle, args, &out, [&status, &done](const Status& s) {
-      status = s;
-      done.Notify();
-    });
+    pflr->Run(opts, handle, args, &out,
+              [&status, &done](const absl::Status& s) {
+                status = s;
+                done.Notify();
+              });
     done.WaitForNotification();
     if (!status.ok()) {
       return status;
@@ -253,27 +256,29 @@ class ProcessFunctionLibraryRuntimeTest : public ::testing::Test {
       return status;
     }
     Notification done2;
-    pflr->Run(opts, handle, args, &out, [&status, &done2](const Status& s) {
-      status = s;
-      done2.Notify();
-    });
+    pflr->Run(opts, handle, args, &out,
+              [&status, &done2](const absl::Status& s) {
+                status = s;
+                done2.Notify();
+              });
     done2.WaitForNotification();
-    EXPECT_TRUE(errors::IsNotFound(status)) << "Actual status: " << status;
+    EXPECT_TRUE(absl::IsNotFound(status)) << "Actual status: " << status;
     EXPECT_TRUE(absl::StrContains(status.message(), "not found."));
 
-    return OkStatus();
+    return absl::OkStatus();
   }
 
-  Status Run(const string& name, FunctionLibraryRuntime::Options opts,
-             test::function::Attrs attrs,
-             const FunctionLibraryRuntime::InstantiateOptions& instantiate_opts,
-             const std::vector<Tensor>& args, std::vector<Tensor*> rets,
-             ProcessFunctionLibraryRuntime* pflr = nullptr) {
+  absl::Status Run(
+      const string& name, FunctionLibraryRuntime::Options opts,
+      test::function::Attrs attrs,
+      const FunctionLibraryRuntime::InstantiateOptions& instantiate_opts,
+      const std::vector<Tensor>& args, std::vector<Tensor*> rets,
+      ProcessFunctionLibraryRuntime* pflr = nullptr) {
     return RunWithRuntime<std::vector<Tensor>, Tensor>(
         name, opts, attrs, instantiate_opts, args, rets, proc_flr_.get());
   }
 
-  Status RunWithPackedArgs(
+  absl::Status RunWithPackedArgs(
       const string& name, FunctionLibraryRuntime::Options opts,
       test::function::Attrs attrs,
       const FunctionLibraryRuntime::InstantiateOptions& instantiate_opts,
@@ -283,23 +288,24 @@ class ProcessFunctionLibraryRuntimeTest : public ::testing::Test {
         name, opts, attrs, instantiate_opts, args, rets, proc_flr_.get());
   }
 
-  Status RunInstantiated(FunctionLibraryRuntime::Handle handle,
-                         FunctionLibraryRuntime::Options opts,
-                         const std::vector<Tensor>& args,
-                         std::vector<Tensor*> rets) {
+  absl::Status RunInstantiated(FunctionLibraryRuntime::Handle handle,
+                               FunctionLibraryRuntime::Options opts,
+                               const std::vector<Tensor>& args,
+                               std::vector<Tensor*> rets) {
     std::function<void(std::function<void()>)> runner =
         [](std::function<void()> fn) {
           test::function::FunctionTestSchedClosure(fn);
         };
 
     opts.runner = &runner;
-    Status status;
+    absl::Status status;
     Notification done;
     std::vector<Tensor> out;
-    proc_flr_->Run(opts, handle, args, &out, [&status, &done](const Status& s) {
-      status = s;
-      done.Notify();
-    });
+    proc_flr_->Run(opts, handle, args, &out,
+                   [&status, &done](const absl::Status& s) {
+                     status = s;
+                     done.Notify();
+                   });
     done.WaitForNotification();
     if (!status.ok()) {
       return status;
@@ -308,7 +314,7 @@ class ProcessFunctionLibraryRuntimeTest : public ::testing::Test {
     for (size_t i = 0; i < rets.size(); ++i) {
       *rets[i] = out[i];
     }
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   std::unique_ptr<DynamicDeviceMgr> device_mgr_;
@@ -395,8 +401,8 @@ TEST_F(ProcessFunctionLibraryRuntimeTest, GetDeviceIncarnation) {
                                                &incarnation));
   // Incarnation is a random number other than 0.
   EXPECT_NE(incarnation, 0);
-  Status s = proc_flr_->GetDeviceIncarnation("/job:a/replica:0/task:0/cpu:2",
-                                             &incarnation);
+  absl::Status s = proc_flr_->GetDeviceIncarnation(
+      "/job:a/replica:0/task:0/cpu:2", &incarnation);
   EXPECT_EQ(s.code(), error::INVALID_ARGUMENT);
 }
 
@@ -429,6 +435,53 @@ TEST_F(ProcessFunctionLibraryRuntimeTest, SingleCallFindDevice) {
   EXPECT_EQ(0, rendezvous_cache_->Size());
 }
 
+TEST_F(ProcessFunctionLibraryRuntimeTest, SingleCallRunAfterInstalantiation) {
+  Init({test::function::XTimesTwo()});
+
+  // Instantiate the function.
+  FunctionLibraryRuntime::Handle handle;
+  FunctionLibraryRuntime::InstantiateOptions instantiate_opts;
+  instantiate_opts.target = "/job:a/replica:0/task:0/cpu:0";
+  TF_CHECK_OK(
+      Instantiate("XTimesTwo", {{"T", DT_FLOAT}}, instantiate_opts, &handle));
+  FunctionLibraryRuntime::Options opts;
+  opts.source_device = "/job:a/replica:0/task:0/cpu:0";
+  opts.remote_execution = true;
+
+  // Run the function.
+  auto x = test::AsTensor<float>({1, 2, 3, 4});
+  Tensor y;
+  TF_CHECK_OK(RunInstantiated(handle, opts, {x}, {&y}));
+  test::ExpectTensorEqual<float>(y, test::AsTensor<float>({2, 4, 6, 8}));
+
+  // Re-run the function.
+  TF_CHECK_OK(RunInstantiated(handle, opts, {x}, {&y}));
+  test::ExpectTensorEqual<float>(y, test::AsTensor<float>({2, 4, 6, 8}));
+}
+
+TEST_F(ProcessFunctionLibraryRuntimeTest, SingleCallRunAfterFinalizaztion) {
+  Init({test::function::XTimesTwo()});
+
+  // Instantiate the function.
+  FunctionLibraryRuntime::Handle handle;
+  FunctionLibraryRuntime::InstantiateOptions instantiate_opts;
+  instantiate_opts.target = "/job:a/replica:0/task:0/cpu:0";
+  TF_CHECK_OK(
+      Instantiate("XTimesTwo", {{"T", DT_FLOAT}}, instantiate_opts, &handle));
+  FunctionLibraryRuntime::Options opts;
+  opts.source_device = "/job:a/replica:0/task:0/cpu:0";
+  opts.remote_execution = true;
+  auto x = test::AsTensor<float>({1, 2, 3, 4});
+  Tensor y;
+  TF_CHECK_OK(RunInstantiated(handle, opts, {x}, {&y}));
+  test::ExpectTensorEqual<float>(y, test::AsTensor<float>({2, 4, 6, 8}));
+
+  TF_CHECK_OK(proc_flr_->Finalize());
+
+  TF_CHECK_OK(RunInstantiated(handle, opts, {x}, {&y}));
+  test::ExpectTensorEqual<float>(y, test::AsTensor<float>({2, 4, 6, 8}));
+}
+
 TEST_F(ProcessFunctionLibraryRuntimeTest, MultipleCallsSameDeviceXTimes) {
   Init({test::function::XTimesTwo(), test::function::XTimesFour()});
   auto x = test::AsTensor<float>({1, 2, 3, 4});
@@ -595,8 +648,7 @@ TEST_F(ProcessFunctionLibraryRuntimeTest, ClusterFLRParallelTest) {
 bool IsCUDATensor(const Tensor& t) {
 #if GOOGLE_CUDA
   cudaPointerAttributes attributes;
-  cudaError_t err =
-      cudaPointerGetAttributes(&attributes, t.tensor_data().data());
+  cudaError_t err = cudaPointerGetAttributes(&attributes, t.data());
   if (err == cudaErrorInvalidValue) return false;
   CHECK_EQ(cudaSuccess, err) << cudaGetErrorString(err);
   return (attributes.type == cudaMemoryTypeDevice);
@@ -621,11 +673,10 @@ void TestTwoDeviceMult(
   auto x = test::AsTensor<float>({1, 2, 3});
   Tensor y_cpu;
   Tensor y_gpu;
-  Status status = fixture->Run("TwoDeviceMult", opts, {{"T", DT_FLOAT}},
-                               inst_opts, {x}, {&y_cpu, &y_gpu});
+  absl::Status status = fixture->Run("TwoDeviceMult", opts, {{"T", DT_FLOAT}},
+                                     inst_opts, {x}, {&y_cpu, &y_gpu});
   if (!error.empty()) {
-    EXPECT_TRUE(errors::IsInvalidArgument(status))
-        << "Actual status: " << status;
+    EXPECT_TRUE(absl::IsInvalidArgument(status)) << "Actual status: " << status;
     EXPECT_TRUE(absl::StrContains(status.message(), error))
         << "Actual error message: " << status.message();
     return;
@@ -781,10 +832,10 @@ TEST_F(ProcessFunctionLibraryRuntimeTest, MultiDevice_ErrorWhenListInput) {
   const FunctionDef& def = test::function::FuncWithListInput();
   Init({def});
   FunctionLibraryRuntime::Handle handle;
-  Status status = proc_flr_->Instantiate(
+  absl::Status status = proc_flr_->Instantiate(
       "FuncWithListInput", test::function::Attrs({{"T", DT_FLOAT}, {"N", 1}}),
       MakeOptions("CPU:0", {"CPU:0"}, {}), &handle);
-  ASSERT_TRUE(errors::IsInvalidArgument(status)) << "Actual status: " << status;
+  ASSERT_TRUE(absl::IsInvalidArgument(status)) << "Actual status: " << status;
   ASSERT_TRUE(absl::StrContains(
       status.message(),
       "FuncWithListInput has an input named \"x1\" that is a list of tensors"))
@@ -802,10 +853,10 @@ TEST_F(ProcessFunctionLibraryRuntimeTest, FullTypeForInt32) {
       TFT_TENSOR);
   Init({def});
   FunctionLibraryRuntime::Handle handle;
-  Status status =
+  absl::Status status =
       proc_flr_->Instantiate("XTimesTwoInt32", test::function::Attrs({}),
                              MakeOptions("CPU:0", {"CPU:0"}, {}), &handle);
-  ASSERT_TRUE(errors::IsInvalidArgument(status)) << "Actual status: " << status;
+  ASSERT_TRUE(absl::IsInvalidArgument(status)) << "Actual status: " << status;
   // Check that the error is found by earlier in ProcessFunctionLibraryRuntime
   // and not later in FunctionLibraryRuntime.
   EXPECT_TRUE(absl::StrContains(
@@ -818,10 +869,10 @@ TEST_F(ProcessFunctionLibraryRuntimeTest, MultiDevice_ErrorWhenListOutput) {
   const FunctionDef& def = test::function::FuncWithListOutput();
   Init({def});
   FunctionLibraryRuntime::Handle handle;
-  Status status = proc_flr_->Instantiate(
+  absl::Status status = proc_flr_->Instantiate(
       "FuncWithListOutput", test::function::Attrs({{"T", DT_FLOAT}, {"N", 1}}),
       MakeOptions("CPU:0", {}, {"CPU:0"}), &handle);
-  ASSERT_TRUE(errors::IsInvalidArgument(status)) << "Actual status: " << status;
+  ASSERT_TRUE(absl::IsInvalidArgument(status)) << "Actual status: " << status;
   ASSERT_TRUE(absl::StrContains(
       status.message(),
       "FuncWithListOutput has an output named \"y\" that is a list of tensors"))
@@ -924,7 +975,7 @@ FunctionDef AddVarAcrossDevices() {
 class TestFunctionPackedArgs : public FunctionArgsInterface {
  public:
   TestFunctionPackedArgs(const int index,
-                         gtl::InlinedVector<TensorValue, 4>&& tensor_args) {
+                         absl::InlinedVector<TensorValue, 4UL>&& tensor_args) {
     packed_args_.emplace(index, std::move(tensor_args));
   }
 
@@ -932,16 +983,16 @@ class TestFunctionPackedArgs : public FunctionArgsInterface {
 
   bool HasRemoteOrPackedInputs() const override { return true; };
 
-  Status GetLocalArg(const FunctionArgIndex& index,
-                     Tensor* val) const override {
+  absl::Status GetLocalArg(const FunctionArgIndex& index,
+                           Tensor* val) const override {
     *val = *packed_args_.at(index.index).at(index.sub_index).tensor;
-    return OkStatus();
+    return absl::OkStatus();
   };
 
   std::vector<Tensor> GetLocalTensors() const override { return {}; }
 
  private:
-  absl::flat_hash_map<int, gtl::InlinedVector<TensorValue, 4>> packed_args_;
+  absl::flat_hash_map<int, absl::InlinedVector<TensorValue, 4UL>> packed_args_;
 };
 
 TEST_F(ProcessFunctionLibraryRuntimeTest, MultiDevice_CompositeDevice) {
@@ -966,7 +1017,7 @@ TEST_F(ProcessFunctionLibraryRuntimeTest, MultiDevice_CompositeDevice) {
       GetResourceHandle("var", mgr1->default_container(), device1_->name());
 
   // Create a CompositeDevice
-  Status s;
+  absl::Status s;
   std::unique_ptr<CompositeDevice> composite_device =
       CompositeDevice::MakeDevice({device0_->name(), device1_->name()},
                                   /*unique_device_id=*/0,
@@ -984,7 +1035,7 @@ TEST_F(ProcessFunctionLibraryRuntimeTest, MultiDevice_CompositeDevice) {
 
   // Packed TensorHandle
   {
-    gtl::InlinedVector<TensorValue, 4> handles;
+    absl::InlinedVector<TensorValue, 4UL> handles;
     handles.push_back(TensorValue(&resource_handle0));
     handles.push_back(TensorValue(&resource_handle1));
     TestFunctionPackedArgs args(0, std::move(handles));
@@ -1024,7 +1075,8 @@ TEST_F(ProcessFunctionLibraryRuntimeTest, MultiDevice_ResourceOutput_GPU) {
   *resource->tensor() = resource_value;
   resource->is_initialized = true;
   ResourceMgr* mgr = gpu_device_->resource_manager();
-  Status status = mgr->Create(mgr->default_container(), "my_gpu_var", resource);
+  absl::Status status =
+      mgr->Create(mgr->default_container(), "my_gpu_var", resource);
   ASSERT_TRUE(status.ok()) << status.message();
 
   // Run the function taking a resource and outputting it
@@ -1067,10 +1119,10 @@ TEST_F(ProcessFunctionLibraryRuntimeTest, MultiDevice_PlacerError) {
         test::function::ReadResourceVariable()});
 
   FunctionLibraryRuntime::Handle handle;
-  Status status = proc_flr_->Instantiate(
+  absl::Status status = proc_flr_->Instantiate(
       "ResourceOutput", test::function::Attrs({{"T", DT_FLOAT}}), inst_opts,
       &handle);
-  ASSERT_TRUE(errors::IsInvalidArgument(status)) << "Actual status: " << status;
+  ASSERT_TRUE(absl::IsInvalidArgument(status)) << "Actual status: " << status;
   ASSERT_TRUE(absl::StrContains(status.message(), "Cannot place"));
 }
 
@@ -1118,8 +1170,9 @@ TEST_F(ProcessFunctionLibraryRuntimeTest, MultiDevice_CreateKernelsEagerly) {
   // Instantiating the broken function while creating kernels eagerly should
   // fail.
   inst_opts.create_kernels_eagerly = true;
-  Status status = Instantiate("Broken", {{"T", DT_INT32}}, inst_opts, &handle);
-  EXPECT_TRUE(errors::IsInternal(status));
+  absl::Status status =
+      Instantiate("Broken", {{"T", DT_INT32}}, inst_opts, &handle);
+  EXPECT_TRUE(absl::IsInternal(status));
 }
 
 TEST_F(ProcessFunctionLibraryRuntimeTest, MultiDevice_StateHandle) {
@@ -1135,7 +1188,7 @@ TEST_F(ProcessFunctionLibraryRuntimeTest, MultiDevice_StateHandle) {
       // Attrs
       {},
       // Nodes
-      {FunctionDefHelper::Const<int32>("shape", gtl::ArraySlice<int32>({1})),
+      {FunctionDefHelper::Const<int32>("shape", absl::Span<const int32>({1})),
        FunctionDefHelper::Const<int32>("minval", 0),
        {{"maxval"}, "ReadVariableOp", {"x"}, {{"dtype", T}}, {}},
        // A stateful node.
@@ -1154,7 +1207,8 @@ TEST_F(ProcessFunctionLibraryRuntimeTest, MultiDevice_StateHandle) {
   Var* resource = new Var(T);
   *resource->tensor() = resource_value;
   resource->is_initialized = true;
-  Status status = mgr->Create(mgr->default_container(), "my_gpu_var", resource);
+  absl::Status status =
+      mgr->Create(mgr->default_container(), "my_gpu_var", resource);
   ASSERT_TRUE(status.ok()) << status.message();
 
   Tensor x = GetResourceHandle("my_gpu_var", mgr->default_container(),
@@ -1243,7 +1297,8 @@ class SessionMetadataReaderOp : public OpKernel {
     OP_REQUIRES_OK(ctx,
                    ctx->allocate_output("y", TensorShape({}), &out_tensor));
     if (ctx->session_metadata() != nullptr) {
-      out_tensor->scalar<tstring>()() = ctx->session_metadata()->DebugString();
+      out_tensor->scalar<tstring>()() =
+          tsl::LegacyUnredactedDebugString(*ctx->session_metadata());
     } else {
       out_tensor->scalar<tstring>()() = "";
     }
@@ -1302,7 +1357,7 @@ TEST_F(ProcessFunctionLibraryRuntimeTest, SessionMetadataPresent) {
 TEST_F(ProcessFunctionLibraryRuntimeTest, CompositeDevicesAfterCloning) {
   Init({AddVarAcrossDevices()});
 
-  Status s;
+  absl::Status s;
   std::unique_ptr<CompositeDevice> composite_device =
       CompositeDevice::MakeDevice({device0_->name(), device1_->name()},
                                   /*unique_device_id=*/0,
@@ -1337,7 +1392,7 @@ TEST_F(ProcessFunctionLibraryRuntimeTest, SessionMetadataPresentAfterCloning) {
   instantiate_opts.target = "/job:a/replica:0/task:0/cpu:0";
   const auto x = test::AsTensor<int64_t>({17});
   Tensor y;
-  Status s = RunWithRuntime<std::vector<Tensor>, Tensor>(
+  absl::Status s = RunWithRuntime<std::vector<Tensor>, Tensor>(
       "SessionMetadataReaderFn", opts, {}, instantiate_opts, {x}, {&y},
       cloned_proc_flr.get());
   TF_CHECK_OK(s);
