@@ -26,9 +26,9 @@ limitations under the License.
 #include "tensorflow/core/platform/byte_order.h"
 #include "tensorflow/core/platform/cpu_info.h"
 #include "tensorflow/core/platform/logging.h"
-#include "tensorflow/core/platform/tracing.h"
 #include "tensorflow/core/platform/types.h"
 #include "tensorflow/core/util/util.h"
+#include "tsl/platform/tracing.h"
 
 namespace tensorflow {
 
@@ -93,13 +93,13 @@ thread::ThreadPool* ComputePool(const SessionOptions& options) {
 int32 NumInterOpThreadsFromEnvironment() {
   int32_t num;
   const char* val = std::getenv("TF_NUM_INTEROP_THREADS");
-  return (val && strings::safe_strto32(val, &num)) ? num : 0;
+  return (val && absl::SimpleAtoi(val, &num)) ? num : 0;
 }
 
 int32 NumIntraOpThreadsFromEnvironment() {
   int32_t num;
   const char* val = std::getenv("TF_NUM_INTRAOP_THREADS");
-  return (val && strings::safe_strto32(val, &num)) ? num : 0;
+  return (val && absl::SimpleAtoi(val, &num)) ? num : 0;
 }
 #if defined(ENABLE_ONEDNN_OPENMP) && defined(ENABLE_MKL)
 int32 OMPThreadsFromEnvironment() {
@@ -166,14 +166,15 @@ thread::ThreadPool* NewThreadPoolFromSessionOptions(
 }
 
 void SchedClosure(absl::AnyInvocable<void()> closure) {
-  if (!tracing::EventCollector::IsEnabled()) {
+  if (!tsl::tracing::EventCollector::IsEnabled()) {
     return Env::Default()->SchedClosure(std::move(closure));
   }
-  uint64 id = tracing::GetUniqueArg();
-  tracing::RecordEvent(tracing::EventCategory::kScheduleClosure, id);
+  uint64 id = tsl::tracing::GetUniqueArg();
+  tsl::tracing::RecordEvent(tsl::tracing::EventCategory::kScheduleClosure, id);
 
   Env::Default()->SchedClosure([id, closure = std::move(closure)]() mutable {
-    tracing::ScopedRegion region(tracing::EventCategory::kRunClosure, id);
+    tsl::tracing::ScopedRegion region(tsl::tracing::EventCategory::kRunClosure,
+                                      id);
     closure();
   });
 }
