@@ -134,8 +134,8 @@ bool FindType(const Graph* graph, const Node* node, bool* signed_input,
 }
 
 // Find the Save op and inputs.
-Status FindSaveOp(const Graph* graph, Node** save_op,
-                  std::vector<const Edge*>* in_edges, bool* found) {
+absl::Status FindSaveOp(const Graph* graph, Node** save_op,
+                        std::vector<const Edge*>* in_edges, bool* found) {
   *found = false;
   for (Node* node : graph->op_nodes()) {
     if (node->type_string() == "SaveV2") {
@@ -148,10 +148,10 @@ Status FindSaveOp(const Graph* graph, Node** save_op,
       TF_RETURN_IF_ERROR(node->input_edges(in_edges));
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Node* FindRestoreAllOp(const Graph* graph, StringPiece save_prefix) {
+Node* FindRestoreAllOp(const Graph* graph, absl::string_view save_prefix) {
   for (Node* node : graph->op_nodes()) {
     // The restore_all op should have the same prefix of the save_op.
     if (node->name() == strings::StrCat(save_prefix, "/restore_all")) {
@@ -164,8 +164,8 @@ Node* FindRestoreAllOp(const Graph* graph, StringPiece save_prefix) {
 // Strips the last "/suffix" from a name.
 // We use this to construct the name of restore ops in the same way they are
 // constructed by the Saver.
-StringPiece GetNodeNamePrefix(const Node* node) {
-  StringPiece name = node->name();
+absl::string_view GetNodeNamePrefix(const Node* node) {
+  absl::string_view name = node->name();
   return name.substr(0, name.rfind('/'));
 }
 
@@ -180,9 +180,9 @@ void FillStringTensor(Tensor* dst, const Tensor& src) {
 // Add the added_variables as an inputs to the Save op.
 // We change the inputs of the SaveV2 op to include the names of the added
 // variables. We also add the variables as inputs to the save op.
-Status ConnectVariablesToSaveOp(Graph* graph, Node* save_op,
-                                const std::vector<const Edge*>& in_edges,
-                                const std::vector<Node*>& added_variables) {
+absl::Status ConnectVariablesToSaveOp(
+    Graph* graph, Node* save_op, const std::vector<const Edge*>& in_edges,
+    const std::vector<Node*>& added_variables) {
   Node* tensor_names_op = in_edges[1]->src();
   Node* shape_and_slices_op = in_edges[2]->src();
 
@@ -237,7 +237,7 @@ Status ConnectVariablesToSaveOp(Graph* graph, Node* save_op,
   }
   graph->RemoveNode(save_op);
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Add a restore subgraph for each variable and connect to the restore_all op.
@@ -245,11 +245,11 @@ Status ConnectVariablesToSaveOp(Graph* graph, Node* save_op,
 //           Assign----restore_all
 //          |      |
 //   RestoreV2    Variable
-Status AddRestoreVariableSubgraphs(Graph* graph, Node* save_op,
-                                   const std::vector<const Edge*>& in_edges,
-                                   const std::vector<Node*>& variables) {
+absl::Status AddRestoreVariableSubgraphs(
+    Graph* graph, Node* save_op, const std::vector<const Edge*>& in_edges,
+    const std::vector<Node*>& variables) {
   Node* prefix_op = in_edges[0]->src();
-  StringPiece name_prefix = GetNodeNamePrefix(save_op);
+  absl::string_view name_prefix = GetNodeNamePrefix(save_op);
   Node* restore_all = FindRestoreAllOp(graph, name_prefix);
   if (restore_all == nullptr) {
     return errors::InvalidArgument("graph has SaveOp, but no restore_all NoOp");
@@ -307,12 +307,13 @@ Status AddRestoreVariableSubgraphs(Graph* graph, Node* save_op,
     // Add a control edge from the assign op to restore_all op.
     graph->AddControlEdge(assign_op, restore_all);
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Adds new variables to save and restore ops matching the Save and Restore
 // graphs created in tensorflow/python/training/saver.py.
-Status AddSaveAndRestore(Graph* graph, const std::vector<Node*>& variables) {
+absl::Status AddSaveAndRestore(Graph* graph,
+                               const std::vector<Node*>& variables) {
   Node* save_op = nullptr;
   std::vector<const Edge*> in_edges;
   bool found = false;
@@ -323,13 +324,13 @@ Status AddSaveAndRestore(Graph* graph, const std::vector<Node*>& variables) {
     TF_RETURN_IF_ERROR(
         ConnectVariablesToSaveOp(graph, save_op, in_edges, variables));
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Sets output to the Node that computes reduction axes corresponding to all
 // dimensions of input and return.
-Status MakeReductionAxes(Graph* graph, string name_prefix, Node* input,
-                         Node** output) {
+absl::Status MakeReductionAxes(Graph* graph, string name_prefix, Node* input,
+                               Node** output) {
   name_prefix = strings::StrCat(name_prefix, "/ReductionAxes");
   Node* start;
   Tensor zero_tensor(DT_INT32, TensorShape());
@@ -358,14 +359,14 @@ Status MakeReductionAxes(Graph* graph, string name_prefix, Node* input,
           .Input(rank)
           .Input(delta)
           .Finalize(graph, output));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Computes the exponential moving average of input, updated in update_variable.
-Status MakeExponentialMovingAverage(Graph* graph, string name_prefix,
-                                    const NodeBuilder::NodeOut& input,
-                                    Node* decay, Node* update_variable,
-                                    Node** assign_value) {
+absl::Status MakeExponentialMovingAverage(Graph* graph, string name_prefix,
+                                          const NodeBuilder::NodeOut& input,
+                                          Node* decay, Node* update_variable,
+                                          Node** assign_value) {
   // variable_t+1 = variable_t - [(variable_t - value) * (1 - decay)]
   name_prefix = strings::StrCat(name_prefix, "/EMA");
   Node* one;
@@ -401,7 +402,7 @@ Status MakeExponentialMovingAverage(Graph* graph, string name_prefix,
           .Input(update_variable)
           .Input(update_value)
           .Finalize(graph, assign_value));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Creates an automatically initialized exponential moving average variable.
@@ -415,10 +416,10 @@ Status MakeExponentialMovingAverage(Graph* graph, string name_prefix,
 //       |         EMA    init_val
 //       |           \      /
 //       +----------- assign
-Status MakeInitializedEMAVariable(Graph* graph, const string& name, Node* decay,
-                                  Node* init_val,
-                                  std::vector<Node*>* added_variables,
-                                  Node** var) {
+absl::Status MakeInitializedEMAVariable(Graph* graph, const string& name,
+                                        Node* decay, Node* init_val,
+                                        std::vector<Node*>* added_variables,
+                                        Node** var) {
   // TODO(suharshs): Update this to use ResourceVariables when they are ready.
   TF_RETURN_IF_ERROR(
       NodeBuilder(strings::StrCat(name, "/Variable"), "VariableV2")
@@ -454,13 +455,13 @@ Status MakeInitializedEMAVariable(Graph* graph, const string& name, Node* decay,
           .Input(*var)
           .Input(assign_value)
           .Finalize(graph, var));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Computes the min and max EMA of input and stores them in min_var and max_var.
-Status MakeEMAMinMaxVars(Graph* graph, const string& name_prefix, Node* input,
-                         std::vector<Node*>* added_variables, Node** min_var,
-                         Node** max_var) {
+absl::Status MakeEMAMinMaxVars(Graph* graph, const string& name_prefix,
+                               Node* input, std::vector<Node*>* added_variables,
+                               Node** min_var, Node** max_var) {
   // TODO(suharshs): The decay will be constant, so we could make only one for
   // all quantize_and_dequantize ops to share, this would have to live outside
   // this function.
@@ -492,15 +493,15 @@ Status MakeEMAMinMaxVars(Graph* graph, const string& name_prefix, Node* input,
                                                 added_variables, min_var));
   TF_RETURN_IF_ERROR(MakeInitializedEMAVariable(graph, max_name, decay, max,
                                                 added_variables, max_var));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Makes an input min and max constant if the range is given. Otherwise, makes
 // min and max variables that are updated by an EMA.
-Status MakeInputMinMax(Graph* graph, const string& name_prefix,
-                       const EdgeToConvert& edge,
-                       std::vector<Node*>* added_variables, Node** input_min,
-                       Node** input_max) {
+absl::Status MakeInputMinMax(Graph* graph, const string& name_prefix,
+                             const EdgeToConvert& edge,
+                             std::vector<Node*>* added_variables,
+                             Node** input_min, Node** input_max) {
   if (edge.range_given) {
     // Make constant nodes for the input_min and input_max if the range is
     // provided.
@@ -525,16 +526,17 @@ Status MakeInputMinMax(Graph* graph, const string& name_prefix,
                                          input_max));
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Adds a QuantizeAndDequantizeV2 or FakeQuantizeWithMinMaxVars op
 // (and required input nodes) based on edge.
 // The result is stored in convert_node.
-Status MakeQuantizeOp(Graph* graph, const string& name_prefix,
-                      const string& quant_op_type, const EdgeToConvert& edge,
-                      std::vector<Node*>* added_variables,
-                      Node** convert_node) {
+absl::Status MakeQuantizeOp(Graph* graph, const string& name_prefix,
+                            const string& quant_op_type,
+                            const EdgeToConvert& edge,
+                            std::vector<Node*>* added_variables,
+                            Node** convert_node) {
   Node* input_min;
   Node* input_max;
   TF_RETURN_IF_ERROR(MakeInputMinMax(graph, name_prefix, edge, added_variables,
@@ -559,12 +561,13 @@ Status MakeQuantizeOp(Graph* graph, const string& name_prefix,
   } else {
     return errors::InvalidArgument("Unknown quant op type: ", quant_op_type);
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Insert conversion op, connect it to the graph and remove the old edge.
-Status ProcessTargetEdges(Graph* graph, const string& quant_op_type,
-                          const std::vector<EdgeToConvert>& target_edges) {
+absl::Status ProcessTargetEdges(
+    Graph* graph, const string& quant_op_type,
+    const std::vector<EdgeToConvert>& target_edges) {
   // Remember previously converted ops to avoid duplicated conversion on the
   // same input.
   std::unordered_map<string, Node*, StringPieceHasher> name_index;
@@ -588,13 +591,13 @@ Status ProcessTargetEdges(Graph* graph, const string& quant_op_type,
 
   TF_RETURN_IF_ERROR(AddSaveAndRestore(graph, added_variables));
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace
 
-Status DoQuantizeTraining(int32_t num_bits, const string& quant_op_type,
-                          Graph* graph) {
+absl::Status DoQuantizeTraining(int32_t num_bits, const string& quant_op_type,
+                                Graph* graph) {
   if (graph == nullptr) {
     return errors::InvalidArgument("Cannot accept empty graph pointer.");
   }
@@ -655,13 +658,13 @@ Status DoQuantizeTraining(int32_t num_bits, const string& quant_op_type,
 
   TF_RETURN_IF_ERROR(ProcessTargetEdges(graph, quant_op_type, target_edges));
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status DoQuantizeTrainingOnGraphDef(const GraphDef& input_graphdef,
-                                    int32_t num_bits,
-                                    const string& quant_op_type,
-                                    GraphDef* result_graphdef) {
+absl::Status DoQuantizeTrainingOnGraphDef(const GraphDef& input_graphdef,
+                                          int32_t num_bits,
+                                          const string& quant_op_type,
+                                          GraphDef* result_graphdef) {
   Graph graph(OpRegistry::Global());
   GraphConstructorOptions opts;
   TF_RETURN_IF_ERROR(ConvertGraphDefToGraph(opts, input_graphdef, &graph));
@@ -671,13 +674,12 @@ Status DoQuantizeTrainingOnGraphDef(const GraphDef& input_graphdef,
 
   // Convert the result graph back to a GraphDef.
   graph.ToGraphDef(result_graphdef);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status DoQuantizeTrainingOnSerializedGraphDef(const string& input_graph_string,
-                                              int32_t num_bits,
-                                              const string& quant_op_type,
-                                              string* result_graph_string) {
+absl::Status DoQuantizeTrainingOnSerializedGraphDef(
+    const string& input_graph_string, int32_t num_bits,
+    const string& quant_op_type, string* result_graph_string) {
   // First create the graph from the GraphDef.
   GraphDef input_graphdef;
   if (!ParseProtoUnlimited(&input_graphdef, input_graph_string)) {
@@ -692,7 +694,7 @@ Status DoQuantizeTrainingOnSerializedGraphDef(const string& input_graph_string,
     return errors::Internal(
         "quantize training transformation resulted in invalid GraphDef");
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace tensorflow
