@@ -36,38 +36,38 @@ Session::Session() {}
 
 Session::~Session() {}
 
-Status Session::Run(const RunOptions& run_options,
-                    const std::vector<std::pair<string, Tensor> >& inputs,
-                    const std::vector<string>& output_tensor_names,
-                    const std::vector<string>& target_tensor_names,
-                    std::vector<Tensor>* outputs, RunMetadata* run_metadata) {
+absl::Status Session::Run(const RunOptions& run_options,
+                          const std::vector<std::pair<string, Tensor> >& inputs,
+                          const std::vector<string>& output_tensor_names,
+                          const std::vector<string>& target_tensor_names,
+                          std::vector<Tensor>* outputs,
+                          RunMetadata* run_metadata) {
   return errors::Unimplemented(
       "Run with options is not supported for this session.");
 }
 
-Status Session::PRunSetup(const std::vector<string>& input_names,
-                          const std::vector<string>& output_names,
-                          const std::vector<string>& target_nodes,
-                          string* handle) {
+absl::Status Session::PRunSetup(const std::vector<string>& input_names,
+                                const std::vector<string>& output_names,
+                                const std::vector<string>& target_nodes,
+                                string* handle) {
   return errors::Unimplemented(
       "Partial run is not supported for this session.");
 }
 
-Status Session::PRun(const string& handle,
-                     const std::vector<std::pair<string, Tensor> >& inputs,
-                     const std::vector<string>& output_names,
-                     std::vector<Tensor>* outputs) {
+absl::Status Session::PRun(
+    const string& handle, const std::vector<std::pair<string, Tensor> >& inputs,
+    const std::vector<string>& output_names, std::vector<Tensor>* outputs) {
   return errors::Unimplemented(
       "Partial run is not supported for this session.");
 }
 
 Session* NewSession(const SessionOptions& options) {
   // Starts exporting metrics through a platform-specific monitoring API (if
-  // provided). For builds using "tensorflow/tsl/platform/default", this is
-  // currently a no-op.
+  // provided). For builds using "tensorflow/compiler/xla/tsl/platform/default",
+  // this is currently a no-op.
   SetSessionCreatedMetric();
   Session* out_session;
-  Status s = NewSession(options, &out_session);
+  absl::Status s = NewSession(options, &out_session);
   if (!s.ok()) {
     LOG(ERROR) << "Failed to create session: " << s;
     return nullptr;
@@ -75,17 +75,17 @@ Session* NewSession(const SessionOptions& options) {
   return out_session;
 }
 
-Status NewSession(const SessionOptions& options, Session** out_session) {
+absl::Status NewSession(const SessionOptions& options, Session** out_session) {
   SessionFactory* factory;
-  Status s = SessionFactory::GetFactory(options, &factory);
+  absl::Status s = SessionFactory::GetFactory(options, &factory);
   if (!s.ok()) {
     *out_session = nullptr;
     LOG(ERROR) << "Failed to get session factory: " << s;
     return s;
   }
   // Starts exporting metrics through a platform-specific monitoring API (if
-  // provided). For builds using "tensorflow/tsl/platform/default", this is
-  // currently a no-op.
+  // provided). For builds using "tensorflow/compiler/xla/tsl/platform/default",
+  // this is currently a no-op.
   SetSessionCreatedMetric();
   s = factory->NewSession(options, out_session);
   if (!s.ok()) {
@@ -95,8 +95,8 @@ Status NewSession(const SessionOptions& options, Session** out_session) {
   return s;
 }
 
-Status Reset(const SessionOptions& options,
-             const std::vector<string>& containers) {
+absl::Status Reset(const SessionOptions& options,
+                   const std::vector<string>& containers) {
   SessionFactory* factory;
   TF_RETURN_IF_ERROR(SessionFactory::GetFactory(options, &factory));
   return factory->Reset(options, containers);
