@@ -23,7 +23,7 @@ namespace tensorflow {
 // kTensorHandleResourceTypeName.
 const char* SessionState::kTensorHandleResourceTypeName = "TensorHandle";
 
-Status SessionState::GetTensor(const string& handle, Tensor* tensor) {
+absl::Status SessionState::GetTensor(const string& handle, Tensor* tensor) {
   mutex_lock l(state_lock_);
   auto it = tensors_.find(handle);
   if (it == tensors_.end()) {
@@ -31,25 +31,26 @@ Status SessionState::GetTensor(const string& handle, Tensor* tensor) {
                                    "' is not in the session store.");
   }
   *tensor = it->second;
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status SessionState::AddTensor(const string& handle, const Tensor& tensor) {
+absl::Status SessionState::AddTensor(const string& handle,
+                                     const Tensor& tensor) {
   mutex_lock l(state_lock_);
   if (!tensors_.insert({handle, tensor}).second) {
     return errors::InvalidArgument("Failed to add a tensor with handle '",
                                    handle, "' to the session store.");
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status SessionState::DeleteTensor(const string& handle) {
+absl::Status SessionState::DeleteTensor(const string& handle) {
   mutex_lock l(state_lock_);
   if (tensors_.erase(handle) == 0) {
     return errors::InvalidArgument("Failed to delete a tensor with handle '",
                                    handle, "' in the session store.");
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 int64_t SessionState::GetNewId() {
@@ -57,18 +58,19 @@ int64_t SessionState::GetNewId() {
   return tensor_id_++;
 }
 
-Status TensorStore::AddTensor(const string& name, const TensorAndKey& tk) {
+absl::Status TensorStore::AddTensor(const string& name,
+                                    const TensorAndKey& tk) {
   mutex_lock l(lock_);
   if (!tensors_.insert({name, tk}).second) {
     return errors::InvalidArgument("Failed to add a tensor with name '", name,
                                    "' to the tensor store.");
   }
   dirty_ = true;
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status TensorStore::SaveTensors(const std::vector<string>& output_names,
-                                SessionState* session_state) {
+absl::Status TensorStore::SaveTensors(const std::vector<string>& output_names,
+                                      SessionState* session_state) {
   mutex_lock l(lock_);
   if (!tensors_.empty()) {
     // Save only the tensors in output_names in the session.
@@ -83,7 +85,7 @@ Status TensorStore::SaveTensors(const std::vector<string>& output_names,
       }
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace tensorflow
