@@ -65,7 +65,7 @@ class ShapeRefinerTest : public ::testing::Test {
                         int end, int stride, const char* expected,
                         int begin_mask = 0, int end_mask = 0,
                         int ellipsis_mask = 0, int shrink_axis_mask = 0,
-                        StringPiece test_op = "TensorAsShapeInt32") {
+                        absl::string_view test_op = "TensorAsShapeInt32") {
     Scope root = Scope::DisabledShapeInferenceScope();
     auto placeholder =
         ops::Placeholder(root, DT_INT32, ops::Placeholder::Shape(input_shape));
@@ -162,7 +162,7 @@ TEST_F(ShapeRefinerTest, BadShapes) {
   TF_ASSERT_OK(m.AddNode(b.node()));
   // The shape of the inputs are not compatible, so we should expect
   // an error.
-  Status s = m.AddNode(mm.node());
+  absl::Status s = m.AddNode(mm.node());
   ASSERT_FALSE(s.ok());
   ASSERT_TRUE(absl::StrContains(s.message(),
                                 "Dimensions must be equal, but are 1 and 2"));
@@ -330,7 +330,7 @@ REGISTER_OP("TestOp")
       if (c->input_tensor(0)) {
         if (c->input_tensor(1)) {
           c->set_output(0, c->Matrix(10, 10));
-          return OkStatus();
+          return absl::OkStatus();
         }
         return shape_inference::ScalarShape(c);
       }
@@ -384,7 +384,7 @@ REGISTER_OP("ShapeData")
       }
 
       c->set_output(0, c->MakeShape(dims));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("ShapeDataInt64")
@@ -403,7 +403,7 @@ REGISTER_OP("ShapeDataInt64")
       }
 
       c->set_output(0, c->MakeShape(dims));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 // An op with a shape function that looks at its input tensor
@@ -422,7 +422,7 @@ REGISTER_OP("ShapeVectorForAllElements")
       }
 
       c->set_output(0, c->Vector(total));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("MultiIdentity")
@@ -433,7 +433,7 @@ REGISTER_OP("MultiIdentity")
       for (int i = 0; i < c->num_inputs(); ++i) {
         c->set_output(i, c->input(i));
       }
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 class MultiIdentity : public OpKernel {
@@ -830,24 +830,24 @@ TEST_F(ShapeRefinerTest, ConstantValueVisitNodeTwice) {
 
 namespace {
 
-Status TensorAsShapeShapeFn(shape_inference::InferenceContext* c) {
+absl::Status TensorAsShapeShapeFn(shape_inference::InferenceContext* c) {
   shape_inference::ShapeHandle out;
   TF_RETURN_IF_ERROR(c->MakeShapeFromShapeTensor(0 /* input_idx */, &out));
   c->set_output(0, out);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status PartialTensorAsShapeShapeFn(shape_inference::InferenceContext* c) {
+absl::Status PartialTensorAsShapeShapeFn(shape_inference::InferenceContext* c) {
   shape_inference::ShapeHandle out;
   const Tensor* t = c->input_tensor(0);
   if (t == nullptr || t->NumElements() != 1) {
     c->set_output(0, c->UnknownShape());
-    return OkStatus();
+    return absl::OkStatus();
   }
   TF_RETURN_IF_ERROR(
       c->MakeShapeFromTensorShape(TensorShape({t->flat<int32>()(0)}), &out));
   c->set_output(0, out);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Register ops used by the ConstantValueAsShape* tests.
@@ -881,7 +881,7 @@ REGISTER_OP("WithEmptyVectorShape")
     .SetDoNotOptimize()
     .SetShapeFn([](shape_inference::InferenceContext* c) {
       c->set_output(0, c->Vector(0));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("WithPartialShape")
@@ -891,7 +891,7 @@ REGISTER_OP("WithPartialShape")
       c->set_output(
           0, c->MakeShape({1, shape_inference::InferenceContext::kUnknownDim, 3,
                            shape_inference::InferenceContext::kUnknownDim, 5}));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("WithPartialShape2")
@@ -901,7 +901,7 @@ REGISTER_OP("WithPartialShape2")
       c->set_output(
           0,
           c->MakeShape({6, shape_inference::InferenceContext::kUnknownDim, 8}));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("WithUnknownShape")
@@ -909,7 +909,7 @@ REGISTER_OP("WithUnknownShape")
     .SetDoNotOptimize()
     .SetShapeFn([](shape_inference::InferenceContext* c) {
       c->set_output(0, c->UnknownShape());
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 }  // namespace
