@@ -30,7 +30,7 @@ limitations under the License.
 
 namespace tensorflow {
 
-Status ValidateOpIsSafeForSyncExecution(
+absl::Status ValidateOpIsSafeForSyncExecution(
     const Node& n, bool allow_control_flow_sync_execution) {
   for (DataType dt : n.output_types()) {
     if (IsRefType(dt)) {
@@ -57,13 +57,13 @@ Status ValidateOpIsSafeForSyncExecution(
         ".  Perhaps your graph contains old-style control flow primitives? "
         "Try using tf.compat.v1.enable_control_flow_v2().");
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 namespace {
 
-typedef gtl::InlinedVector<TensorValue, 4> TensorValueVec;
-typedef gtl::InlinedVector<AllocatorAttributes, 4> AllocatorAttributeVec;
+typedef absl::InlinedVector<TensorValue, 4UL> TensorValueVec;
+typedef absl::InlinedVector<AllocatorAttributes, 4UL> AllocatorAttributeVec;
 
 static const string& kSingleThreadedExecutor =
     *new string("SINGLE_THREADED_EXECUTOR");
@@ -82,7 +82,7 @@ class SingleThreadedExecutorImpl : public Executor {
     }
   }
 
-  Status Initialize(const Graph& graph) {
+  absl::Status Initialize(const Graph& graph) {
     // Topologicially sort `graph` to get a sequence of OpKernels.
     std::vector<Node*> ordered_nodes;
     ordered_nodes.reserve(graph.num_nodes());
@@ -251,10 +251,10 @@ class SingleThreadedExecutorImpl : public Executor {
     } else {
       total_num_inputs_ = 0;
     }
-    return OkStatus();
+    return absl::OkStatus();
   }
 
-  Status Run(const Args& args) override {
+  absl::Status Run(const Args& args) override {
     // The inputs to each kernel are stored contiguously in `inputs`.
     //
     // We use `kernels_[i].input_start_index` and `kernels_[i].num_inputs` to
@@ -315,6 +315,7 @@ class SingleThreadedExecutorImpl : public Executor {
     params.session_metadata = params_.session_metadata;
     params.tensor_store = args.tensor_store;
     params.cancellation_manager = args.cancellation_manager;
+    params.session_config = args.session_config;
     params.call_frame = args.call_frame;
     params.function_library = params_.function_library;
     params.resource_manager = device->resource_manager();
@@ -481,19 +482,19 @@ class SingleThreadedExecutorImpl : public Executor {
         delete val.tensor;
       }
     }
-    return OkStatus();
+    return absl::OkStatus();
   }
 
+ private:
   // Execute all operations in the calling thread when asynchronous execution
   // is requested. Callers may expect to perform expensive work in the calling
   // thread even when the execution itself is single-threaded.
   //
   // This also avoid stack-overflow issues with functional control flow.
-  void RunAsync(const Args& args, DoneCallback done) override {
+  void RunAsyncInternal(const Args& args, DoneCallback done) override {
     args.runner([this, args, done]() { done(Run(args)); });
   }
 
- private:
   const LocalExecutorParams params_;
 
   // All following members are read-only after Initialize().
@@ -577,12 +578,13 @@ class SingleThreadedExecutorRegistrar {
 
  private:
   class Factory : public ExecutorFactory {
-    Status NewExecutor(const LocalExecutorParams& params, const Graph& graph,
-                       std::unique_ptr<Executor>* out_executor) override {
+    absl::Status NewExecutor(const LocalExecutorParams& params,
+                             const Graph& graph,
+                             std::unique_ptr<Executor>* out_executor) override {
       Executor* ret;
       TF_RETURN_IF_ERROR(NewSingleThreadedExecutor(params, graph, &ret));
       out_executor->reset(ret);
-      return OkStatus();
+      return absl::OkStatus();
     }
   };
 };
@@ -590,12 +592,13 @@ static SingleThreadedExecutorRegistrar registrar;
 
 }  // namespace
 
-Status NewSingleThreadedExecutor(const LocalExecutorParams& params,
-                                 const Graph& graph, Executor** executor) {
+absl::Status NewSingleThreadedExecutor(const LocalExecutorParams& params,
+                                       const Graph& graph,
+                                       Executor** executor) {
   auto impl = std::make_unique<SingleThreadedExecutorImpl>(params);
   TF_RETURN_IF_ERROR(impl->Initialize(graph));
   *executor = impl.release();
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace tensorflow
