@@ -17,11 +17,14 @@ limitations under the License.
 
 #include <optional>
 
+#include <gmock/gmock.h>
+#include <gtest/gtest.h>
+#include "absl/status/status.h"
 #include "absl/time/time.h"
+#include "xla/tsl/lib/core/status_test_util.h"
+#include "xla/tsl/platform/status_matchers.h"
 #include "tensorflow/core/framework/metrics.h"
 #include "tensorflow/core/lib/monitoring/cell_reader.h"
-#include "tensorflow/tsl/lib/core/status_test_util.h"
-#include "tensorflow/tsl/platform/status_matchers.h"
 
 namespace tensorflow {
 namespace data {
@@ -202,14 +205,14 @@ TEST(AutoScalerTest, ReportProcessingTimeNewAndExisting) {
 
 TEST(AutoScalerTest, ReportProcessingTimeZeroDuration) {
   AutoScaler auto_scaler;
-  tsl::Status result = auto_scaler.ReportProcessingTime("/worker/task/0:20000",
-                                                        absl::ZeroDuration());
+  absl::Status result = auto_scaler.ReportProcessingTime("/worker/task/0:20000",
+                                                         absl::ZeroDuration());
   EXPECT_THAT(result, StatusIs(absl::StatusCode::kInvalidArgument));
 }
 
 TEST(AutoScalerTest, ReportProcessingTimeNegativeDuration) {
   AutoScaler auto_scaler;
-  tsl::Status result = auto_scaler.ReportProcessingTime(
+  absl::Status result = auto_scaler.ReportProcessingTime(
       "/worker/task/0:20000", absl::Microseconds(-10));
   EXPECT_THAT(result, StatusIs(absl::StatusCode::kInvalidArgument));
 }
@@ -246,14 +249,14 @@ TEST(AutoScalerTest, ReportTargetProcessingTimeNewAndExisting) {
 
 TEST(AutoScalerTest, ReportTargetProcessingTimeZeroDuration) {
   AutoScaler auto_scaler;
-  tsl::Status result =
+  absl::Status result =
       auto_scaler.ReportTargetProcessingTime(0, absl::ZeroDuration());
   EXPECT_THAT(result, StatusIs(absl::StatusCode::kInvalidArgument));
 }
 
 TEST(AutoScalerTest, ReportTargetProcessingTimeNegativeDuration) {
   AutoScaler auto_scaler;
-  tsl::Status result =
+  absl::Status result =
       auto_scaler.ReportTargetProcessingTime(0, absl::Microseconds(-10));
   EXPECT_THAT(result, StatusIs(absl::StatusCode::kInvalidArgument));
 }
@@ -324,7 +327,7 @@ TEST(MultipleIterationsAutoScalerTest, UnregisterNonexistentIteration) {
 TEST(MultipleIterationsAutoScalerTest,
      UpdateOptimalNumberOfWorkersMetricInvalidCurrentWorkers) {
   MultipleIterationsAutoScaler auto_scaler;
-  tsl::Status status = auto_scaler.UpdateOptimalNumberOfWorkersMetric(0);
+  absl::Status status = auto_scaler.UpdateOptimalNumberOfWorkersMetric(0);
   EXPECT_THAT(status, StatusIs(absl::StatusCode::kInvalidArgument));
   status = auto_scaler.UpdateOptimalNumberOfWorkersMetric(-1);
   EXPECT_THAT(status, StatusIs(absl::StatusCode::kInvalidArgument));
@@ -333,7 +336,7 @@ TEST(MultipleIterationsAutoScalerTest,
 TEST(MultipleIterationsAutoScalerTest,
      UpdateOptimalNumberOfWorkersMetricNoReportedTimes) {
   MultipleIterationsAutoScaler auto_scaler;
-  tsl::Status status = auto_scaler.UpdateOptimalNumberOfWorkersMetric(1);
+  absl::Status status = auto_scaler.UpdateOptimalNumberOfWorkersMetric(1);
   EXPECT_THAT(status, StatusIs(absl::StatusCode::kUnavailable));
 }
 
@@ -345,7 +348,7 @@ TEST(MultipleIterationsAutoScalerTest,
       auto_scaler.ReportTargetProcessingTime(0, 0, absl::Microseconds(5)));
   TF_ASSERT_OK(
       auto_scaler.ReportTargetProcessingTime(1, 0, absl::Microseconds(5)));
-  tsl::Status status = auto_scaler.UpdateOptimalNumberOfWorkersMetric(1);
+  absl::Status status = auto_scaler.UpdateOptimalNumberOfWorkersMetric(1);
   EXPECT_THAT(status, StatusIs(absl::StatusCode::kUnavailable));
 }
 
@@ -357,7 +360,7 @@ TEST(MultipleIterationsAutoScalerTest,
                                                 absl::Microseconds(10)));
   TF_ASSERT_OK(auto_scaler.ReportProcessingTime(1, "/worker/task/0:20000",
                                                 absl::Microseconds(10)));
-  tsl::Status status = auto_scaler.UpdateOptimalNumberOfWorkersMetric(1);
+  absl::Status status = auto_scaler.UpdateOptimalNumberOfWorkersMetric(1);
   EXPECT_THAT(status, StatusIs(absl::StatusCode::kUnavailable));
 }
 
@@ -587,7 +590,7 @@ TEST(MultipleIterationsAutoScalerTest, ReportProcessingTimeNewAndExisting) {
 TEST(MultipleIterationsAutoScalerTest, ReportProcessingTimeZeroDuration) {
   MultipleIterationsAutoScaler auto_scaler;
 
-  tsl::Status result = auto_scaler.ReportProcessingTime(
+  absl::Status result = auto_scaler.ReportProcessingTime(
       0, "/worker/task/0:20000", absl::ZeroDuration());
   EXPECT_THAT(result, StatusIs(absl::StatusCode::kInvalidArgument));
 }
@@ -595,7 +598,7 @@ TEST(MultipleIterationsAutoScalerTest, ReportProcessingTimeZeroDuration) {
 TEST(MultipleIterationsAutoScalerTest, ReportProcessingTimeNegativeDuration) {
   MultipleIterationsAutoScaler auto_scaler;
 
-  tsl::Status result = auto_scaler.ReportProcessingTime(
+  absl::Status result = auto_scaler.ReportProcessingTime(
       0, "/worker/task/0:20000", absl::Microseconds(-10));
   EXPECT_THAT(result, StatusIs(absl::StatusCode::kInvalidArgument));
 }
@@ -655,7 +658,7 @@ TEST(MultipleIterationsAutoScalerTest,
 TEST(MultipleIterationsAutoScalerTest, ReportTargetProcessingTimeZeroDuration) {
   MultipleIterationsAutoScaler auto_scaler;
 
-  tsl::Status result =
+  absl::Status result =
       auto_scaler.ReportTargetProcessingTime(0, 0, absl::ZeroDuration());
   EXPECT_THAT(result, StatusIs(absl::StatusCode::kInvalidArgument));
 }
@@ -664,7 +667,7 @@ TEST(MultipleIterationsAutoScalerTest,
      ReportTargetProcessingTimeNegativeDuration) {
   MultipleIterationsAutoScaler auto_scaler;
 
-  tsl::Status result =
+  absl::Status result =
       auto_scaler.ReportTargetProcessingTime(0, 0, absl::Microseconds(-10));
   EXPECT_THAT(result, StatusIs(absl::StatusCode::kInvalidArgument));
 }
