@@ -18,35 +18,37 @@ limitations under the License.
 #include <utility>
 #include <vector>
 
+#include "absl/status/status.h"
+#include "absl/status/statusor.h"
 #include "absl/time/time.h"
+#include "xla/tsl/lib/core/status_test_util.h"
+#include "xla/tsl/lib/io/compression.h"
+#include "xla/tsl/platform/env.h"
+#include "xla/tsl/platform/status_matchers.h"
+#include "xla/tsl/platform/test.h"
+#include "tensorflow/core/data/service/byte_size.h"
 #include "tensorflow/core/data/service/common.pb.h"
 #include "tensorflow/core/data/service/snapshot/path_utils.h"
 #include "tensorflow/core/data/service/snapshot/snapshot_stream_writer.h"
 #include "tensorflow/core/data/service/snapshot/test_utils.h"
 #include "tensorflow/core/data/service/task_runner.h"
 #include "tensorflow/core/data/service/test_util.h"
-#include "tensorflow/tsl/lib/core/status_test_util.h"
-#include "tensorflow/tsl/lib/io/compression.h"
-#include "tensorflow/tsl/platform/env.h"
-#include "tensorflow/tsl/platform/errors.h"
-#include "tensorflow/tsl/platform/random.h"
-#include "tensorflow/tsl/platform/status.h"
-#include "tensorflow/tsl/platform/status_matchers.h"
-#include "tensorflow/tsl/platform/statusor.h"
-#include "tensorflow/tsl/platform/test.h"
+#include "tsl/platform/random.h"
 
 namespace tensorflow {
 namespace data {
 namespace {
 
+using ::testing::HasSubstr;
 using ::testing::UnorderedElementsAre;
 using ::testing::ValuesIn;
 using ::tsl::testing::IsOkAndHolds;
+using ::tsl::testing::StatusIs;
 
-StatusOr<std::string> CreateSnapshotDirectory() {
+absl::StatusOr<std::string> CreateSnapshotDirectory() {
   std::string snapshot_path;
   if (!Env::Default()->LocalTempFilename(&snapshot_path)) {
-    return errors::FailedPrecondition(
+    return absl::FailedPreconditionError(
         "Failed to create local temp file for snapshot.");
   }
   TF_RETURN_IF_ERROR(Env::Default()->RecursivelyCreateDir(
@@ -54,8 +56,8 @@ StatusOr<std::string> CreateSnapshotDirectory() {
   return snapshot_path;
 }
 
-StatusOr<int64_t> NumCheckpoints(const std::string& snapshot_path,
-                                 int64_t stream_index) {
+absl::StatusOr<int64_t> NumCheckpoints(const std::string& snapshot_path,
+                                       int64_t stream_index) {
   std::string checkpoints_directory =
       CheckpointsDirectory(snapshot_path, stream_index);
   std::vector<std::string> checkpoint_filenames;
@@ -85,6 +87,7 @@ TEST_P(SnapshotStreamWriterParameterizedTest, SaveAndRestoreFromCheckpoint) {
   // Generates a snapshot directory with a single checkpoint. The stream writer
   // should resume from this checkpoint.
   TF_ASSERT_OK(partial_writer.WriteCheckpoints({5}));
+  TF_ASSERT_OK(partial_writer.WriteCommittedChunks({4}));
 
   SnapshotWriterParams writer_params{snapshot_path,
                                      /*stream_index=*/0, compression,
@@ -94,25 +97,26 @@ TEST_P(SnapshotStreamWriterParameterizedTest, SaveAndRestoreFromCheckpoint) {
   SnapshotStreamWriter snapshot_writer(writer_params, std::move(iterator));
   EXPECT_THAT(snapshot_writer.Wait(), IsOkAndHolds(true));
   EXPECT_THAT(testing::ReadSnapshot<int64_t>(snapshot_path, compression),
-              IsOkAndHolds(UnorderedElementsAre(6, 7, 8, 9)));
+              IsOkAndHolds(UnorderedElementsAre(4, 5, 6, 7, 8, 9)));
 }
 
 TEST_P(SnapshotStreamWriterParameterizedTest,
        SaveAndRestoreFromEndOfSequenceCheckpoint) {
-  const int64_t range = 5;
+  const int64_t range = 10;
   const std::string compression = GetParam();
   const DatasetDef dataset = testing::RangeDataset(range);
   const int64_t stream_index = 0;
   TF_ASSERT_OK_AND_ASSIGN(const std::string snapshot_path,
                           CreateSnapshotDirectory());
-  TF_ASSERT_OK_AND_ASSIGN(testing::PartialSnapshotWriter partial_writer,
-                          testing::PartialSnapshotWriter::Create(
-                              dataset, snapshot_path, stream_index, compression,
-                              /*max_chunk_size_bytes=*/16));
+  TF_ASSERT_OK_AND_ASSIGN(
+      testing::PartialSnapshotWriter partial_writer,
+      testing::PartialSnapshotWriter::Create(dataset, snapshot_path,
+                                             stream_index, compression));
 
-  // Each chunk contains 2 elements. There are 3 chunks. The third checkpoint is
+  // Each chunk contains 1 element. There are 10 chunks. The 10th checkpoint is
   // for the iterator that has reached the end of sequence.
-  TF_ASSERT_OK(partial_writer.WriteCheckpoints({2}));
+  TF_ASSERT_OK(partial_writer.WriteCheckpoints({10}));
+  TF_ASSERT_OK(partial_writer.WriteCommittedChunks({9}));
 
   SnapshotWriterParams writer_params{snapshot_path,
                                      /*stream_index=*/0, compression,
@@ -125,7 +129,7 @@ TEST_P(SnapshotStreamWriterParameterizedTest,
   // Since the end-of-sequence iterator is checkpointed, no more elements are
   // written here.
   EXPECT_THAT(testing::ReadSnapshot<int64_t>(snapshot_path, compression),
-              IsOkAndHolds(UnorderedElementsAre()));
+              IsOkAndHolds(UnorderedElementsAre(9)));
 }
 
 TEST_P(SnapshotStreamWriterParameterizedTest, VaryingCheckpointInterval) {
@@ -139,7 +143,7 @@ TEST_P(SnapshotStreamWriterParameterizedTest, VaryingCheckpointInterval) {
       testing::PartialSnapshotWriter partial_writer,
       testing::PartialSnapshotWriter::Create(
           dataset, snapshot_path, stream_index, compression,
-          /*max_chunk_size_bytes=*/1, /*checkpoint_interval=*/
+          /*max_chunk_size=*/ByteSize::Bytes(1), /*checkpoint_interval=*/
           absl::Milliseconds(tsl::random::New64() % 1000)));
   TF_ASSERT_OK(
       partial_writer.WriteCommittedChunks({0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10}));
@@ -151,7 +155,7 @@ TEST_P(SnapshotStreamWriterParameterizedTest, VaryingCheckpointInterval) {
       /*stream_index=*/0,
       compression,
       Env::Default(),
-      /*max_chunk_size_bytes=*/1,
+      /*max_chunk_size=*/ByteSize::Bytes(1),
       /*checkpoint_interval=*/absl::Milliseconds(tsl::random::New64() % 1000),
       /*test_only_keep_temp_files=*/true};
   TF_ASSERT_OK_AND_ASSIGN(std::unique_ptr<StandaloneTaskIterator> iterator,
@@ -175,14 +179,13 @@ TEST(SnapshotStreamWriterCheckpointTest, SingleCheckpoint) {
 
   const std::string compression = tsl::io::compression::kSnappy;
   TF_ASSERT_OK_AND_ASSIGN(std::string snapshot_path, CreateSnapshotDirectory());
-  SnapshotWriterParams writer_params{
-      snapshot_path,
-      /*stream_index=*/0,
-      compression,
-      Env::Default(),
-      /*max_chunk_size_bytes=*/kint64max,
-      /*checkpoint_interval=*/absl::Microseconds(1),
-      /*test_only_keep_temp_files=*/true};
+  SnapshotWriterParams writer_params{snapshot_path,
+                                     /*stream_index=*/0,
+                                     compression,
+                                     Env::Default(),
+                                     /*max_chunk_size=*/ByteSize::Bytes(1),
+                                     /*checkpoint_interval=*/absl::Hours(1),
+                                     /*test_only_keep_temp_files=*/true};
   SnapshotStreamWriter snapshot_writer(writer_params, std::move(iterator));
   EXPECT_THAT(snapshot_writer.Wait(), IsOkAndHolds(true));
   EXPECT_THAT(testing::ReadSnapshot<int64_t>(snapshot_path, compression),
@@ -191,7 +194,7 @@ TEST(SnapshotStreamWriterCheckpointTest, SingleCheckpoint) {
               IsOkAndHolds(1));
 }
 
-TEST(SnapshotStreamWriterCheckpointTest, WithCheckpoint) {
+TEST(SnapshotStreamWriterCheckpointTest, MultipleCheckpoints) {
   int64_t range = 5;
   TF_ASSERT_OK_AND_ASSIGN(std::unique_ptr<StandaloneTaskIterator> iterator,
                           testing::TestIterator(testing::RangeDataset(range)));
@@ -203,7 +206,7 @@ TEST(SnapshotStreamWriterCheckpointTest, WithCheckpoint) {
       /*stream_index=*/0,
       compression,
       Env::Default(),
-      /*max_chunk_size_bytes=*/20,
+      /*max_chunk_size=*/ByteSize::Bytes(1),
       /*checkpoint_interval=*/absl::Microseconds(1),
       /*test_only_keep_temp_files=*/true};
   SnapshotStreamWriter snapshot_writer(writer_params, std::move(iterator));
@@ -211,16 +214,9 @@ TEST(SnapshotStreamWriterCheckpointTest, WithCheckpoint) {
   EXPECT_THAT(testing::ReadSnapshot<int64_t>(snapshot_path, compression),
               IsOkAndHolds(UnorderedElementsAre(0, 1, 2, 3, 4)));
 
-  // Expected files:
-  // Checkpoint 0
-  // Chunk 0 (elements 0, 1)
-  // Checkpoint 1
-  // Chunk 1 (elements 2, 3)
-  // Checkpoint 2
-  // Chunk 2 (element 4)
-  // DONE
+  // Wrote one checkpoint for each element, and one for end_of_sequence.
   EXPECT_THAT(NumCheckpoints(snapshot_path, /*stream_index=*/0),
-              IsOkAndHolds(3));
+              IsOkAndHolds(range + 1));
 }
 
 TEST(SnapshotStreamWriterCheckpointTest, CleanupCheckpoint) {
@@ -235,7 +231,7 @@ TEST(SnapshotStreamWriterCheckpointTest, CleanupCheckpoint) {
       /*stream_index=*/0,
       compression,
       Env::Default(),
-      /*max_chunk_size_bytes=*/20,
+      /*max_chunk_size=*/ByteSize::Bytes(20),
       /*checkpoint_interval=*/absl::Microseconds(1),
       /*test_only_keep_temp_files=*/false};
   SnapshotStreamWriter snapshot_writer(writer_params, std::move(iterator));
@@ -261,11 +257,10 @@ TEST(SnapshotStreamWriterCheckpointTest, SyncCheckpointsWithChunksByRenaming) {
       testing::PartialSnapshotWriter::Create(dataset, snapshot_path,
                                              stream_index, compression));
 
-  // Unlike the previous test, this test has generated some uncommitted chunks.
-  // The stream writer will first move the uncommitted chunks to the committed
-  // chunks directory.
-  TF_ASSERT_OK(partial_writer.WriteUncommittedChunks({0, 1, 2}));
+  // This test has generated some uncommitted chunks. The stream writer will
+  // first move the uncommitted chunks to the committed chunks directory.
   TF_ASSERT_OK(partial_writer.WriteCheckpoints({5}));
+  TF_ASSERT_OK(partial_writer.WriteUncommittedChunks({0, 1, 2, 3, 4}));
 
   SnapshotWriterParams writer_params{snapshot_path, stream_index, compression,
                                      Env::Default()};
@@ -274,7 +269,7 @@ TEST(SnapshotStreamWriterCheckpointTest, SyncCheckpointsWithChunksByRenaming) {
   SnapshotStreamWriter writer(writer_params, std::move(iterator));
   EXPECT_THAT(writer.Wait(), IsOkAndHolds(true));
   EXPECT_THAT(testing::ReadSnapshot<int64_t>(snapshot_path, compression),
-              IsOkAndHolds(UnorderedElementsAre(0, 1, 2, 6, 7, 8, 9)));
+              IsOkAndHolds(UnorderedElementsAre(0, 1, 2, 3, 4, 5, 6, 7, 8, 9)));
 }
 
 TEST(SnapshotStreamWriterCheckpointTest,
@@ -318,9 +313,10 @@ TEST(SnapshotStreamWriterCheckpointTest, SyncCheckpointsWithChunksByDeleting) {
 
   // The writer will first delete uncommitted chunks after the checkpoint when
   // it is restored. In the end, only chunks 6--9 are written.
+  TF_ASSERT_OK(partial_writer.WriteCheckpoints({5}));
+  TF_ASSERT_OK(partial_writer.WriteCommittedChunks({4}));
   TF_ASSERT_OK(
       partial_writer.WriteUncommittedChunks({6, 7, 8, 9, 10, 11, 12, 13, 14}));
-  TF_ASSERT_OK(partial_writer.WriteCheckpoints({5}));
 
   TF_ASSERT_OK_AND_ASSIGN(std::unique_ptr<StandaloneTaskIterator> iterator,
                           testing::TestIterator(dataset));
@@ -329,7 +325,7 @@ TEST(SnapshotStreamWriterCheckpointTest, SyncCheckpointsWithChunksByDeleting) {
   SnapshotStreamWriter writer(writer_params, std::move(iterator));
   EXPECT_THAT(writer.Wait(), IsOkAndHolds(true));
   EXPECT_THAT(testing::ReadSnapshot<int64_t>(snapshot_path, compression),
-              IsOkAndHolds(UnorderedElementsAre(6, 7, 8, 9)));
+              IsOkAndHolds(UnorderedElementsAre(4, 5, 6, 7, 8, 9)));
 }
 
 TEST(SnapshotStreamWriterCheckpointTest, SyncCheckpointsWithChunks) {
@@ -348,9 +344,10 @@ TEST(SnapshotStreamWriterCheckpointTest, SyncCheckpointsWithChunks) {
   // This test combines the previous two test cases: Uncommitted chunks before
   // the checkpoint are committed; Uncommitted chunks after the checkpoint are
   // deleted.
-  TF_ASSERT_OK(partial_writer.WriteUncommittedChunks(
-      {1, 2, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14}));
   TF_ASSERT_OK(partial_writer.WriteCheckpoints({5}));
+  TF_ASSERT_OK(partial_writer.WriteCommittedChunks({0}));
+  TF_ASSERT_OK(partial_writer.WriteUncommittedChunks(
+      {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14}));
 
   TF_ASSERT_OK_AND_ASSIGN(std::unique_ptr<StandaloneTaskIterator> iterator,
                           testing::TestIterator(dataset));
@@ -359,7 +356,34 @@ TEST(SnapshotStreamWriterCheckpointTest, SyncCheckpointsWithChunks) {
   SnapshotStreamWriter writer(writer_params, std::move(iterator));
   EXPECT_THAT(writer.Wait(), IsOkAndHolds(true));
   EXPECT_THAT(testing::ReadSnapshot<int64_t>(snapshot_path, compression),
-              IsOkAndHolds(UnorderedElementsAre(1, 2, 5, 6, 7, 8, 9)));
+              IsOkAndHolds(UnorderedElementsAre(0, 1, 2, 3, 4, 5, 6, 7, 8, 9)));
+}
+
+TEST(SnapshotStreamWriterCheckpointTest, LostChunks) {
+  const int64_t range = 10;
+  const std::string compression = tsl::io::compression::kZlib;
+  const DatasetDef dataset = testing::RangeDataset(range);
+  const int64_t stream_index = 0;
+  TF_ASSERT_OK_AND_ASSIGN(const std::string snapshot_path,
+                          CreateSnapshotDirectory());
+  // Generates some additional chunks.
+  TF_ASSERT_OK_AND_ASSIGN(testing::PartialSnapshotWriter partial_writer,
+                          testing::PartialSnapshotWriter::Create(
+                              testing::RangeDataset(range + 5), snapshot_path,
+                              stream_index, compression));
+
+  // There is a checkpoint 5 but no chunks [2, 5). Should report lost chunks.
+  TF_ASSERT_OK(partial_writer.WriteCheckpoints({5}));
+  TF_ASSERT_OK(partial_writer.WriteCommittedChunks({0, 1}));
+
+  TF_ASSERT_OK_AND_ASSIGN(std::unique_ptr<StandaloneTaskIterator> iterator,
+                          testing::TestIterator(dataset));
+  SnapshotWriterParams writer_params{snapshot_path, stream_index, compression,
+                                     Env::Default()};
+  SnapshotStreamWriter writer(writer_params, std::move(iterator));
+  EXPECT_THAT(writer.Wait(),
+              StatusIs(absl::StatusCode::kInternal,
+                       HasSubstr("Unable to find chunks [2, 5).")));
 }
 
 }  // namespace
