@@ -15,17 +15,18 @@ limitations under the License.
 #ifndef TENSORFLOW_CORE_DATA_SERVICE_SNAPSHOT_UTILS_H_
 #define TENSORFLOW_CORE_DATA_SERVICE_SNAPSHOT_UTILS_H_
 
-#include <cstdint>
 #include <vector>
 
 #include "absl/strings/string_view.h"
+#include "xla/tsl/platform/status.h"
+#include "tensorflow/core/data/service/byte_size.h"
 #include "tensorflow/core/framework/tensor.h"
-#include "tensorflow/tsl/platform/status.h"
 
 namespace tensorflow {
 namespace data {
 
-int64_t EstimatedSizeBytes(const std::vector<Tensor>& tensors);
+// Estimates the size of the Tensors when serialized as TensorProtos.
+ByteSize EstimatedSize(const std::vector<Tensor>& tensors);
 
 }  // namespace data
 }  // namespace tensorflow
