@@ -13,6 +13,8 @@ See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
 
+#include <memory>
+
 #include "tensorflow/core/debug/debug_graph_utils.h"
 #include "tensorflow/core/debug/debug_grpc_testlib.h"
 #include "tensorflow/core/debug/debug_io_utils.h"
@@ -24,7 +26,6 @@ limitations under the License.
 #include "tensorflow/core/lib/strings/stringprintf.h"
 #include "tensorflow/core/platform/env.h"
 #include "tensorflow/core/platform/test.h"
-#include "tensorflow/core/platform/tracing.h"
 
 namespace tensorflow {
 
@@ -48,10 +49,10 @@ class GrpcDebugTest : public ::testing::Test {
                             int64_t server_start_delay_micros) {
     server_data->port = testing::PickUnusedPortOrDie();
     server_data->url = strings::StrCat("grpc://localhost:", server_data->port);
-    server_data->server.reset(new test::TestEventListenerImpl());
+    server_data->server = std::make_unique<test::TestEventListenerImpl>();
 
-    server_data->thread_pool.reset(
-        new thread::ThreadPool(Env::Default(), "test_server", 1));
+    server_data->thread_pool =
+        std::make_unique<thread::ThreadPool>(Env::Default(), "test_server", 1);
     server_data->thread_pool->Schedule(
         [server_data, server_start_delay_micros]() {
           Env::Default()->SleepForMicroseconds(server_start_delay_micros);
@@ -88,7 +89,7 @@ TEST_F(GrpcDebugTest, ConnectionTimeoutWorks) {
       strings::StrCat("grpc://localhost:", testing::PickUnusedPortOrDie());
   Tensor tensor(DT_FLOAT, TensorShape({1, 1}));
   tensor.flat<float>()(0) = 42.0;
-  Status publish_status = DebugIO::PublishDebugTensor(
+  absl::Status publish_status = DebugIO::PublishDebugTensor(
       DebugNodeKey("/job:localhost/replica:0/task:0/cpu:0", "foo_tensor", 0,
                    "DebugIdentity"),
       tensor, Env::Default()->NowMicros(), {kInvalidGrpcUrl});
@@ -111,7 +112,7 @@ TEST_F(GrpcDebugTest, ConnectionToDelayedStartingServerWorks) {
   tensor.flat<float>()(0) = 42.0;
   const DebugNodeKey kDebugNodeKey("/job:localhost/replica:0/task:0/cpu:0",
                                    "foo_tensor", 0, "DebugIdentity");
-  Status publish_status = DebugIO::PublishDebugTensor(
+  absl::Status publish_status = DebugIO::PublishDebugTensor(
       kDebugNodeKey, tensor, Env::Default()->NowMicros(), {server_data.url});
   ASSERT_TRUE(publish_status.ok());
   TF_ASSERT_OK(DebugIO::CloseDebugURL(server_data.url));
@@ -150,7 +151,7 @@ TEST_F(GrpcDebugTest, SendDebugTensorWithLargeStringAtIndex0ViaGrpcTest) {
   tensor.flat<tstring>()(0) = string(5000 * 1024, 'A');
   const DebugNodeKey kDebugNodeKey("/job:localhost/replica:0/task:0/cpu:0",
                                    "foo_tensor", 0, "DebugIdentity");
-  const Status status = DebugIO::PublishDebugTensor(
+  const absl::Status status = DebugIO::PublishDebugTensor(
       kDebugNodeKey, tensor, Env::Default()->NowMicros(), {server_data_.url});
   ASSERT_FALSE(status.ok());
   ASSERT_NE(status.message().find("string value at index 0 from debug "
@@ -166,7 +167,7 @@ TEST_F(GrpcDebugTest, SendDebugTensorWithLargeStringAtIndex1ViaGrpcTest) {
   tensor.flat<tstring>()(1) = string(5000 * 1024, 'A');
   const DebugNodeKey kDebugNodeKey("/job:localhost/replica:0/task:0/cpu:0",
                                    "foo_tensor", 0, "DebugIdentity");
-  const Status status = DebugIO::PublishDebugTensor(
+  const absl::Status status = DebugIO::PublishDebugTensor(
       kDebugNodeKey, tensor, Env::Default()->NowMicros(), {server_data_.url});
   ASSERT_FALSE(status.ok());
   ASSERT_NE(status.message().find("string value at index 1 from debug "
@@ -193,7 +194,7 @@ TEST_F(GrpcDebugTest, SendMultipleDebugTensorsSynchronizedViaGrpcTest) {
   mutex mu;
   Notification all_done;
   int tensor_count TF_GUARDED_BY(mu) = 0;
-  std::vector<Status> statuses TF_GUARDED_BY(mu);
+  std::vector<absl::Status> statuses TF_GUARDED_BY(mu);
 
   const std::vector<string> urls({server_data_.url});
 
@@ -209,7 +210,7 @@ TEST_F(GrpcDebugTest, SendMultipleDebugTensorsSynchronizedViaGrpcTest) {
 
     // Different concurrent tasks will send different tensors.
     const uint64 wall_time = Env::Default()->NowMicros();
-    Status publish_status = DebugIO::PublishDebugTensor(
+    absl::Status publish_status = DebugIO::PublishDebugTensor(
         DebugNodeKey("/job:localhost/replica:0/task:0/cpu:0",
                      strings::StrCat("synchronized_node_", this_count), 0,
                      "DebugIdentity"),
@@ -234,11 +235,11 @@ TEST_F(GrpcDebugTest, SendMultipleDebugTensorsSynchronizedViaGrpcTest) {
   delete tp;
 
   // Close the debug gRPC stream.
-  Status close_status = DebugIO::CloseDebugURL(server_data_.url);
+  absl::Status close_status = DebugIO::CloseDebugURL(server_data_.url);
   ASSERT_TRUE(close_status.ok());
 
   // Check all statuses from the PublishDebugTensor calls().
-  for (const Status& status : statuses) {
+  for (const absl::Status& status : statuses) {
     TF_ASSERT_OK(status);
   }
 
@@ -285,7 +286,7 @@ TEST_F(GrpcDebugTest, SendDebugTensorsThroughMultipleRoundsUsingGrpcGating) {
         kDebugNodeKey);
 
     // Close the debug gRPC stream.
-    Status close_status = DebugIO::CloseDebugURL(server_data_.url);
+    absl::Status close_status = DebugIO::CloseDebugURL(server_data_.url);
     ASSERT_TRUE(close_status.ok());
 
     // Check dumped files according to the expected gating results.
@@ -335,7 +336,7 @@ TEST_F(GrpcDebugTest, SendDebugTensorsThroughMultipleRoundsUnderReadWriteMode) {
         kDebugNodeKey);
 
     // Close the debug gRPC stream.
-    Status close_status = DebugIO::CloseDebugURL(server_data_.url);
+    absl::Status close_status = DebugIO::CloseDebugURL(server_data_.url);
     ASSERT_TRUE(close_status.ok());
 
     // Check dumped files according to the expected gating results.
