@@ -25,7 +25,7 @@ void CancellableCall::Start(const StatusCallback& done) {
   const bool not_yet_cancelled =
       cancel_mgr_->RegisterCallback(token, [this]() { Cancel(); });
   if (not_yet_cancelled) {
-    IssueCall([this, token, done](const Status& s) {
+    IssueCall([this, token, done](const absl::Status& s) {
       cancel_mgr_->DeregisterCallback(token);
       done(s);
     });
