@@ -27,8 +27,8 @@ namespace eager {
 void RemoteExecuteNode::RunAsync(StatusCallback done) {
   auto response = std::make_shared<EnqueueResponse>();
 
-  const gtl::InlinedVector<TensorHandle*, 4>& inputs = inputs_;
-  const gtl::InlinedVector<TensorHandle*, 2>& retvals = retvals_;
+  const absl::InlinedVector<TensorHandle*, 4UL>& inputs = inputs_;
+  const absl::InlinedVector<TensorHandle*, 2UL>& retvals = retvals_;
   Device* device = device_;
 
   // Filled and used only when VLOG(3) is on.
@@ -60,7 +60,7 @@ void RemoteExecuteNode::RunAsync(StatusCallback done) {
     const bool already_cancelled = !cm->RegisterCallback(
         token, [call_opts, response, done]() { call_opts->StartCancel(); });
     if (already_cancelled) {
-      Status s = errors::Cancelled("RemoteExecuteNode::RunAsync");
+      absl::Status s = errors::Cancelled("RemoteExecuteNode::RunAsync");
       for (size_t i = 0; i < retvals.size(); ++i) {
         retvals[i]->PoisonRemote(s, device, context_view_id_);
       }
@@ -81,7 +81,7 @@ void RemoteExecuteNode::RunAsync(StatusCallback done) {
       request_.get(), response.get(),
       [inputs, retvals, call_opts, response, device,
        context_view_id = context_view_id_, rpc_description, cm, token,
-       done](const Status& status) {
+       done](const absl::Status& status) {
         if (cm != nullptr) {
           cm->TryDeregisterCallback(token);
         }
@@ -100,7 +100,7 @@ void RemoteExecuteNode::RunAsync(StatusCallback done) {
                 response->queue_response(0).device().empty()
                     ? ""
                     : response->queue_response(0).device(i);
-            Status s = retvals[i]->SetRemoteShapeAndDevice(
+            absl::Status s = retvals[i]->SetRemoteShapeAndDevice(
                 response->queue_response(0).shape(i), device, context_view_id,
                 output_device);
 
