@@ -25,6 +25,7 @@ limitations under the License.
 #include <vector>
 
 #include "absl/status/status.h"
+#include "xla/tsl/protobuf/coordination_config.pb.h"
 #include "tensorflow/core/common_runtime/process_util.h"
 #include "tensorflow/core/common_runtime/profile_handler.h"
 #include "tensorflow/core/common_runtime/stats_publisher_interface.h"
@@ -60,11 +61,10 @@ limitations under the License.
 #include "tensorflow/core/platform/logging.h"
 #include "tensorflow/core/platform/macros.h"
 #include "tensorflow/core/platform/mutex.h"
-#include "tensorflow/core/platform/tracing.h"
 #include "tensorflow/core/protobuf/config.pb.h"
 #include "tensorflow/core/public/session_options.h"
 #include "tensorflow/core/util/device_name_utils.h"
-#include "tensorflow/tsl/protobuf/coordination_config.pb.h"
+#include "tsl/platform/tracing.h"
 
 namespace tensorflow {
 
@@ -148,15 +148,17 @@ class MasterSession::ReffedClientGraph : public core::RefCounted {
       }
       LoggingResponse* resp = new LoggingResponse;
       Ref();
-      p.worker->LoggingAsync(req, resp, [this, req, resp](const Status& s) {
-        delete req;
-        delete resp;
-        // ReffedClientGraph owns p.worker so we need to hold a ref to
-        // ensure that the method doesn't attempt to access p.worker after
-        // ReffedClient graph has deleted it.
-        // TODO(suharshs): Simplify this ownership model.
-        Unref();
-      });
+      p.worker->LoggingAsync(req, resp,
+                             [this, req, resp](const absl::Status& s) {
+                               delete req;
+                               delete resp;
+                               // ReffedClientGraph owns p.worker so we need to
+                               // hold a ref to ensure that the method doesn't
+                               // attempt to access p.worker after ReffedClient
+                               // graph has deleted it.
+                               // TODO(suharshs): Simplify this ownership model.
+                               Unref();
+                             });
     }
   }
 
@@ -178,7 +180,7 @@ class MasterSession::ReffedClientGraph : public core::RefCounted {
         LoggingResponse* resp = new LoggingResponse;
         p.worker->LoggingAsync(
             &req, resp,
-            [step_id, ss, resp, &scoped_mu, &all_done](const Status& s) {
+            [step_id, ss, resp, &scoped_mu, &all_done](const absl::Status& s) {
               {
                 mutex_lock l(scoped_mu);
                 if (s.ok()) {
@@ -205,18 +207,22 @@ class MasterSession::ReffedClientGraph : public core::RefCounted {
 
   // Partitions the graph into subgraphs and registers them on
   // workers.
-  Status RegisterPartitions(PartitionOptions popts);
+  absl::Status RegisterPartitions(PartitionOptions popts);
 
   // Runs one step of all partitions.
-  Status RunPartitions(const MasterEnv* env, int64_t step_id,
-                       int64_t execution_count, PerStepState* pss,
-                       CallOptions* opts, const RunStepRequestWrapper& req,
-                       MutableRunStepResponseWrapper* resp,
-                       CancellationManager* cm, const bool is_last_partial_run);
-  Status RunPartitions(const MasterEnv* env, int64_t step_id,
-                       int64_t execution_count, PerStepState* pss,
-                       CallOptions* call_opts, const RunCallableRequest& req,
-                       RunCallableResponse* resp, CancellationManager* cm);
+  absl::Status RunPartitions(const MasterEnv* env, int64_t step_id,
+                             int64_t execution_count, PerStepState* pss,
+                             CallOptions* opts,
+                             const RunStepRequestWrapper& req,
+                             MutableRunStepResponseWrapper* resp,
+                             CancellationManager* cm,
+                             const bool is_last_partial_run);
+  absl::Status RunPartitions(const MasterEnv* env, int64_t step_id,
+                             int64_t execution_count, PerStepState* pss,
+                             CallOptions* call_opts,
+                             const RunCallableRequest& req,
+                             RunCallableResponse* resp,
+                             CancellationManager* cm);
 
   // Calls workers to cleanup states for the step "step_id".  Calls
   // `done` when all cleanup RPCs have completed.
@@ -228,9 +234,9 @@ class MasterSession::ReffedClientGraph : public core::RefCounted {
   void ProcessDeviceStats(ProfileHandler* ph, const DeviceStepStats& ds,
                           bool is_rpc);
   // Checks that the requested fetches can be computed from the provided feeds.
-  Status CheckFetches(const RunStepRequestWrapper& req,
-                      const RunState* run_state,
-                      GraphExecutionState* execution_state);
+  absl::Status CheckFetches(const RunStepRequestWrapper& req,
+                            const RunState* run_state,
+                            GraphExecutionState* execution_state);
 
  private:
   const string session_handle_;
@@ -290,7 +296,7 @@ class MasterSession::ReffedClientGraph : public core::RefCounted {
   Notification init_done_;
 
   // init_result_ remembers the initialization error if any.
-  Status init_result_ TF_GUARDED_BY(mu_);
+  absl::Status init_result_ TF_GUARDED_BY(mu_);
 
   std::unique_ptr<StatsPublisherInterface> stats_publisher_;
 
@@ -314,10 +320,10 @@ class MasterSession::ReffedClientGraph : public core::RefCounted {
                                    const PartitionOptions& popts);
 
   // The actual graph partitioning and registration implementation.
-  Status DoBuildPartitions(
+  absl::Status DoBuildPartitions(
       PartitionOptions popts, ClientGraph* client_graph,
       std::unordered_map<string, GraphDef>* out_partitions);
-  Status DoRegisterPartitions(
+  absl::Status DoRegisterPartitions(
       const PartitionOptions& popts,
       std::unordered_map<string, GraphDef> graph_partitions);
 
@@ -325,8 +331,9 @@ class MasterSession::ReffedClientGraph : public core::RefCounted {
   // This is a generic method that handles Run, PartialRun, and RunCallable.
   template <class FetchListType, class ClientRequestType,
             class ClientResponseType>
-  Status RunPartitionsHelper(
-      const std::unordered_map<StringPiece, size_t, StringPieceHasher>& feeds,
+  absl::Status RunPartitionsHelper(
+      const std::unordered_map<absl::string_view, size_t, StringPieceHasher>&
+          feeds,
       const FetchListType& fetches, const MasterEnv* env, int64_t step_id,
       int64_t execution_count, PerStepState* pss, CallOptions* call_opts,
       const ClientRequestType& req, ClientResponseType* resp,
@@ -336,10 +343,11 @@ class MasterSession::ReffedClientGraph : public core::RefCounted {
   // destructor and does not wait for the rpc completion.
   void DeregisterPartitions();
 
-  TF_DISALLOW_COPY_AND_ASSIGN(ReffedClientGraph);
+  ReffedClientGraph(const ReffedClientGraph&) = delete;
+  void operator=(const ReffedClientGraph&) = delete;
 };
 
-Status MasterSession::ReffedClientGraph::RegisterPartitions(
+absl::Status MasterSession::ReffedClientGraph::RegisterPartitions(
     PartitionOptions popts) {
   {  // Ensure register once.
     mu_.lock();
@@ -353,7 +361,8 @@ Status MasterSession::ReffedClientGraph::RegisterPartitions(
       mu_.unlock();
       std::unordered_map<string, GraphDef> graph_defs;
       popts.flib_def = client_graph->flib_def.get();
-      Status s = DoBuildPartitions(popts, client_graph.get(), &graph_defs);
+      absl::Status s =
+          DoBuildPartitions(popts, client_graph.get(), &graph_defs);
       if (s.ok()) {
         // NOTE(mrry): The pointers in `graph_defs_for_publishing` do not remain
         // valid after the call to DoRegisterPartitions begins, so
@@ -375,7 +384,7 @@ Status MasterSession::ReffedClientGraph::RegisterPartitions(
       init_done_.WaitForNotification();
       mu_.lock();
     }
-    const Status result = init_result_;
+    const absl::Status result = init_result_;
     mu_.unlock();
     return result;
   }
@@ -428,7 +437,7 @@ void MasterSession::ReffedClientGraph::TrackFeedsAndFetches(
   }
 }
 
-Status MasterSession::ReffedClientGraph::DoBuildPartitions(
+absl::Status MasterSession::ReffedClientGraph::DoBuildPartitions(
     PartitionOptions popts, ClientGraph* client_graph,
     std::unordered_map<string, GraphDef>* out_partitions) {
   if (popts.need_to_record_start_times) {
@@ -444,11 +453,11 @@ Status MasterSession::ReffedClientGraph::DoBuildPartitions(
   return Partition(popts, &client_graph->graph, out_partitions);
 }
 
-Status MasterSession::ReffedClientGraph::DoRegisterPartitions(
+absl::Status MasterSession::ReffedClientGraph::DoRegisterPartitions(
     const PartitionOptions& popts,
     std::unordered_map<string, GraphDef> graph_partitions) {
   partitions_.reserve(graph_partitions.size());
-  Status s;
+  absl::Status s;
   for (auto& name_def : graph_partitions) {
     partitions_.emplace_back();
     Part* part = &partitions_.back();
@@ -470,10 +479,10 @@ Status MasterSession::ReffedClientGraph::DoRegisterPartitions(
   struct Call {
     RegisterGraphRequest req;
     RegisterGraphResponse resp;
-    Status status;
+    absl::Status status;
   };
   const int num = partitions_.size();
-  gtl::InlinedVector<Call, 4> calls(num);
+  absl::InlinedVector<Call, 4UL> calls(num);
   BlockingCounter done(num);
   for (int i = 0; i < num; ++i) {
     const Part& part = partitions_[i];
@@ -489,7 +498,7 @@ Status MasterSession::ReffedClientGraph::DoRegisterPartitions(
         callable_opts_.run_options().debug_options();
     c->req.set_collective_graph_key(collective_graph_key_);
     VLOG(2) << "Register " << c->req.graph_def().DebugString();
-    auto cb = [c, &done](const Status& s) {
+    auto cb = [c, &done](const absl::Status& s) {
       c->status = s;
       done.DecrementCount();
     };
@@ -523,15 +532,15 @@ class RunManyGraphs {
   Call* get(int index) { return &calls_[index]; }
 
   // When the index-th call is done, updates the overall status.
-  void WhenDone(int index, const Status& s) {
-    TRACEPRINTF("Partition %d %s", index, s.ToString().c_str());
+  void WhenDone(int index, const absl::Status& s) {
+    TRACEPRINTF("Partition %d %v", index, s);
     Call* call = get(index);
     call->done = true;
     auto resp = call->resp.get();
     if (resp->status_code() != absl::StatusCode::kOk) {
       // resp->status_code will only be non-OK if s.ok().
       mutex_lock l(mu_);
-      Status resp_status = call->resp->status();
+      absl::Status resp_status = call->resp->status();
       ReportBadStatus(errors::CreateWithUpdatedMessage(
           resp_status, strings::StrCat("From ", *call->worker_name, ":\n",
                                        resp_status.message())));
@@ -579,7 +588,7 @@ class RunManyGraphs {
     pending_.Wait();
   }
 
-  Status status() const {
+  absl::Status status() const {
     mutex_lock l(mu_);
     // Concat status objects in this StatusGroup to get the aggregated status,
     // as each status in status_group_ is already summarized status.
@@ -587,14 +596,14 @@ class RunManyGraphs {
   }
 
  private:
-  gtl::InlinedVector<Call, 4> calls_;
+  absl::InlinedVector<Call, 4UL> calls_;
 
   BlockingCounter pending_;
   mutable mutex mu_;
   StatusGroup status_group_ TF_GUARDED_BY(mu_);
   bool cancel_issued_ TF_GUARDED_BY(mu_) = false;
 
-  void ReportBadStatus(const Status& s) TF_EXCLUSIVE_LOCKS_REQUIRED(mu_) {
+  void ReportBadStatus(const absl::Status& s) TF_EXCLUSIVE_LOCKS_REQUIRED(mu_) {
     VLOG(1) << "Master received error status " << s;
     if (!cancel_issued_ && !StatusGroup::IsDerived(s)) {
       // Only start cancelling other workers upon receiving a non-derived
@@ -610,18 +619,19 @@ class RunManyGraphs {
     status_group_.Update(s);
   }
 
-  TF_DISALLOW_COPY_AND_ASSIGN(RunManyGraphs);
+  RunManyGraphs(const RunManyGraphs&) = delete;
+  void operator=(const RunManyGraphs&) = delete;
 };
 
-Status AddSendFromClientRequest(const RunStepRequestWrapper& client_req,
-                                MutableRunGraphRequestWrapper* worker_req,
-                                size_t index, const string& send_key) {
+absl::Status AddSendFromClientRequest(const RunStepRequestWrapper& client_req,
+                                      MutableRunGraphRequestWrapper* worker_req,
+                                      size_t index, const string& send_key) {
   return worker_req->AddSendFromRunStepRequest(client_req, index, send_key);
 }
 
-Status AddSendFromClientRequest(const RunCallableRequest& client_req,
-                                MutableRunGraphRequestWrapper* worker_req,
-                                size_t index, const string& send_key) {
+absl::Status AddSendFromClientRequest(const RunCallableRequest& client_req,
+                                      MutableRunGraphRequestWrapper* worker_req,
+                                      size_t index, const string& send_key) {
   return worker_req->AddSendFromRunCallableRequest(client_req, index, send_key);
 }
 
@@ -633,7 +643,7 @@ struct RunCallableResponseWrapper {
 
   RunMetadata* mutable_metadata() { return resp->mutable_metadata(); }
 
-  Status AddTensorFromRunGraphResponse(
+  absl::Status AddTensorFromRunGraphResponse(
       const string& tensor_name, MutableRunGraphResponseWrapper* worker_resp,
       size_t index) {
     return worker_resp->RecvValue(index, &fetch_key_to_protos[tensor_name]);
@@ -643,8 +653,9 @@ struct RunCallableResponseWrapper {
 
 template <class FetchListType, class ClientRequestType,
           class ClientResponseType>
-Status MasterSession::ReffedClientGraph::RunPartitionsHelper(
-    const std::unordered_map<StringPiece, size_t, StringPieceHasher>& feeds,
+absl::Status MasterSession::ReffedClientGraph::RunPartitionsHelper(
+    const std::unordered_map<absl::string_view, size_t, StringPieceHasher>&
+        feeds,
     const FetchListType& fetches, const MasterEnv* env, int64_t step_id,
     int64_t execution_count, PerStepState* pss, CallOptions* call_opts,
     const ClientRequestType& req, ClientResponseType* resp,
@@ -740,7 +751,7 @@ Status MasterSession::ReffedClientGraph::RunPartitionsHelper(
   for (int i = 0; i < num; ++i) {
     const Part& part = partitions_[i];
     RunManyGraphs::Call* call = calls.get(i);
-    TRACEPRINTF("Partition %d %s", i, part.name.c_str());
+    TRACEPRINTF("Partition %d %s", i, part.name);
     part.worker->RunGraphAsync(
         &call->opts, call->req.get(), call->resp.get(),
         std::bind(&RunManyGraphs::WhenDone, &calls, i, std::placeholders::_1));
@@ -768,7 +779,7 @@ Status MasterSession::ReffedClientGraph::RunPartitionsHelper(
   TF_RETURN_IF_ERROR(calls.status());
 
   // Collects fetches and metadata.
-  Status status;
+  absl::Status status;
   for (int i = 0; i < num; ++i) {
     const Part& part = partitions_[i];
     MutableRunGraphResponseWrapper* run_graph_resp = calls.get(i)->resp.get();
@@ -808,7 +819,7 @@ Status MasterSession::ReffedClientGraph::RunPartitionsHelper(
   return status;
 }
 
-Status MasterSession::ReffedClientGraph::RunPartitions(
+absl::Status MasterSession::ReffedClientGraph::RunPartitions(
     const MasterEnv* env, int64_t step_id, int64_t execution_count,
     PerStepState* pss, CallOptions* call_opts, const RunStepRequestWrapper& req,
     MutableRunStepResponseWrapper* resp, CancellationManager* cm,
@@ -816,7 +827,7 @@ Status MasterSession::ReffedClientGraph::RunPartitions(
   VLOG(2) << "RunPartitions step_id " << step_id << " execution_count "
           << execution_count;
   // Maps the names of fed tensors to their index in `req`.
-  std::unordered_map<StringPiece, size_t, StringPieceHasher> feeds(3);
+  std::unordered_map<absl::string_view, size_t, StringPieceHasher> feeds(3);
   for (size_t i = 0; i < req.num_feeds(); ++i) {
     if (!feeds.insert({req.feed_name(i), i}).second) {
       return errors::InvalidArgument("Duplicated feeds: ", req.feed_name(i));
@@ -833,14 +844,14 @@ Status MasterSession::ReffedClientGraph::RunPartitions(
                              call_opts, req, resp, cm, is_last_partial_run);
 }
 
-Status MasterSession::ReffedClientGraph::RunPartitions(
+absl::Status MasterSession::ReffedClientGraph::RunPartitions(
     const MasterEnv* env, int64_t step_id, int64_t execution_count,
     PerStepState* pss, CallOptions* call_opts, const RunCallableRequest& req,
     RunCallableResponse* resp, CancellationManager* cm) {
   VLOG(2) << "RunPartitions step_id " << step_id << " execution_count "
           << execution_count;
   // Maps the names of fed tensors to their index in `req`.
-  std::unordered_map<StringPiece, size_t, StringPieceHasher> feeds(3);
+  std::unordered_map<absl::string_view, size_t, StringPieceHasher> feeds(3);
   for (size_t i = 0, end = callable_opts_.feed_size(); i < end; ++i) {
     if (!feeds.insert({callable_opts_.feed(i), i}).second) {
       // MakeCallable will fail if there are two feeds with the same name.
@@ -868,7 +879,7 @@ Status MasterSession::ReffedClientGraph::RunPartitions(
     }
     fetch_proto->Swap(&iter->second);
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 namespace {
@@ -887,9 +898,9 @@ class CleanupBroadcastHelper {
   CleanupGraphResponse* response(int i) { return &resps_[i]; }
 
   // Called when the ith response is received.
-  void call_done(int i, const Status& s) {
+  void call_done(int i, const absl::Status& s) {
     bool run_callback = false;
-    Status status_copy;
+    absl::Status status_copy;
     {
       mutex_lock l(mu_);
       status_.Update(s);
@@ -909,17 +920,18 @@ class CleanupBroadcastHelper {
   // A single request shared between all workers.
   CleanupGraphRequest req_;
   // One response buffer for each worker.
-  gtl::InlinedVector<CleanupGraphResponse, 4> resps_;
+  absl::InlinedVector<CleanupGraphResponse, 4UL> resps_;
 
   mutex mu_;
   // Number of requests remaining to be collected.
   int num_pending_ TF_GUARDED_BY(mu_);
   // Aggregate status of the operation.
-  Status status_ TF_GUARDED_BY(mu_);
+  absl::Status status_ TF_GUARDED_BY(mu_);
   // Callback to be called when all operations complete.
   StatusCallback done_;
 
-  TF_DISALLOW_COPY_AND_ASSIGN(CleanupBroadcastHelper);
+  CleanupBroadcastHelper(const CleanupBroadcastHelper&) = delete;
+  void operator=(const CleanupBroadcastHelper&) = delete;
 };
 
 }  // namespace
@@ -934,7 +946,7 @@ void MasterSession::ReffedClientGraph::CleanupPartitionsAsync(
     const Part& part = partitions_[i];
     part.worker->CleanupGraphAsync(
         helper->request(), helper->response(i),
-        [helper, i](const Status& s) { helper->call_done(i, s); });
+        [helper, i](const absl::Status& s) { helper->call_done(i, s); });
   }
 }
 
@@ -964,7 +976,7 @@ void MasterSession::ReffedClientGraph::ProcessStats(int64_t step_id,
     }
     ph->StepDone(pss->start_micros, pss->end_micros,
                  Microseconds(0) /*cleanup_time*/, 0 /*total_runops*/,
-                 OkStatus());
+                 absl::OkStatus());
   }
   // Assemble all stats for this timeline into a merged StepStats.
   if (pss->collect_timeline) {
@@ -1034,7 +1046,7 @@ void MasterSession::ReffedClientGraph::ProcessDeviceStats(
 // TODO(suharsh,mrry): Build a map from fetch target to set of feeds it depends
 // on once at setup time to prevent us from computing the dependencies
 // everytime.
-Status MasterSession::ReffedClientGraph::CheckFetches(
+absl::Status MasterSession::ReffedClientGraph::CheckFetches(
     const RunStepRequestWrapper& req, const RunState* run_state,
     GraphExecutionState* execution_state) {
   // Build the set of pending feeds that we haven't seen.
@@ -1088,7 +1100,7 @@ Status MasterSession::ReffedClientGraph::CheckFetches(
       }
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Asynchronously deregisters subgraphs on the workers, without waiting for the
@@ -1111,7 +1123,7 @@ void MasterSession::ReffedClientGraph::DeregisterPartitions() {
       const string name = part.name;
       WorkerInterface* w = part.worker;
       CHECK_NOTNULL(w);
-      auto cb = [worker_cache, c, name, w](const Status& s) {
+      auto cb = [worker_cache, c, name, w](const absl::Status& s) {
         if (!s.ok()) {
           // This error is potentially benign, so we don't log at the
           // error level.
@@ -1261,8 +1273,8 @@ void MasterSession::UpdateLastAccessTime() {
   last_access_time_usec_.store(Env::Default()->NowMicros());
 }
 
-Status MasterSession::Create(GraphDef&& graph_def,
-                             const ClusterDef& cluster_def) {
+absl::Status MasterSession::Create(GraphDef&& graph_def,
+                                   const ClusterDef& cluster_def) {
   if (session_opts_.config.use_per_session_threads() ||
       session_opts_.config.session_inter_op_thread_pool_size() > 0) {
     return errors::InvalidArgument(
@@ -1287,7 +1299,8 @@ Status MasterSession::Create(GraphDef&& graph_def,
   return CreateWorkerSessions(cluster_def);
 }
 
-Status MasterSession::CreateWorkerSessions(const ClusterDef& cluster_def) {
+absl::Status MasterSession::CreateWorkerSessions(
+    const ClusterDef& cluster_def) {
   const std::vector<string> worker_names = filtered_worker_list_;
   WorkerCacheInterface* worker_cache = get_worker_cache();
 
@@ -1301,7 +1314,7 @@ Status MasterSession::CreateWorkerSessions(const ClusterDef& cluster_def) {
     // Request and responses used for a given worker.
     CreateWorkerSessionRequest request;
     CreateWorkerSessionResponse response;
-    Status status = OkStatus();
+    absl::Status status = absl::OkStatus();
   };
   BlockingCounter done(worker_names.size());
   std::vector<WorkerGroup> workers(worker_names.size());
@@ -1322,7 +1335,7 @@ Status MasterSession::CreateWorkerSessions(const ClusterDef& cluster_def) {
   const int64_t client_device_incarnation =
       devices_->client_device()->attributes().incarnation();
 
-  Status status = OkStatus();
+  absl::Status status = absl::OkStatus();
   // Create all the workers & kick off the computations.
   for (size_t i = 0; i < worker_names.size(); ++i) {
     workers[i].name = &worker_names[i];
@@ -1405,7 +1418,7 @@ Status MasterSession::CreateWorkerSessions(const ClusterDef& cluster_def) {
   }
 
   for (size_t i = 0; i < worker_names.size(); ++i) {
-    auto cb = [i, &workers, &done](const Status& s) {
+    auto cb = [i, &workers, &done](const absl::Status& s) {
       workers[i].status = s;
       done.DecrementCount();
     };
@@ -1420,7 +1433,7 @@ Status MasterSession::CreateWorkerSessions(const ClusterDef& cluster_def) {
   return status;
 }
 
-Status MasterSession::DeleteWorkerSessions() {
+absl::Status MasterSession::DeleteWorkerSessions() {
   WorkerCacheInterface* worker_cache = get_worker_cache();
   const std::vector<string>& worker_names = filtered_worker_list_;
 
@@ -1436,7 +1449,7 @@ Status MasterSession::DeleteWorkerSessions() {
     // Request and responses used for a given worker.
     DeleteWorkerSessionRequest request;
     DeleteWorkerSessionResponse response;
-    Status status = OkStatus();
+    absl::Status status = absl::OkStatus();
   };
   BlockingCounter done(worker_names.size());
   std::vector<WorkerGroup> workers(worker_names.size());
@@ -1450,7 +1463,7 @@ Status MasterSession::DeleteWorkerSessions() {
     }
   });
 
-  Status status = OkStatus();
+  absl::Status status = absl::OkStatus();
   // Create all the workers & kick off the computations.
   for (size_t i = 0; i < worker_names.size(); ++i) {
     workers[i].name = &worker_names[i];
@@ -1462,7 +1475,7 @@ Status MasterSession::DeleteWorkerSessions() {
   }
 
   for (size_t i = 0; i < worker_names.size(); ++i) {
-    auto cb = [i, &workers, &done](const Status& s) {
+    auto cb = [i, &workers, &done](const absl::Status& s) {
       workers[i].status = s;
       done.DecrementCount();
     };
@@ -1477,7 +1490,7 @@ Status MasterSession::DeleteWorkerSessions() {
   return status;
 }
 
-Status MasterSession::ListDevices(ListDevicesResponse* resp) const {
+absl::Status MasterSession::ListDevices(ListDevicesResponse* resp) const {
   if (worker_cache_) {
     // This is a ClusterSpec-propagated session, and thus env_->local_devices
     // are invalid.
@@ -1498,11 +1511,11 @@ Status MasterSession::ListDevices(ListDevicesResponse* resp) const {
       *(resp->add_local_device()) = dev->attributes();
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status MasterSession::Extend(const ExtendSessionRequest* req,
-                             ExtendSessionResponse* resp) {
+absl::Status MasterSession::Extend(const ExtendSessionRequest* req,
+                                   ExtendSessionResponse* resp) {
   UpdateLastAccessTime();
   std::unique_ptr<GraphExecutionState> extended_execution_state;
   {
@@ -1527,7 +1540,7 @@ Status MasterSession::Extend(const ExtendSessionRequest* req,
     ++graph_version_;
     resp->set_new_graph_version(graph_version_);
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 WorkerCacheInterface* MasterSession::get_worker_cache() const {
@@ -1537,9 +1550,10 @@ WorkerCacheInterface* MasterSession::get_worker_cache() const {
   return env_->worker_cache;
 }
 
-Status MasterSession::StartStep(const BuildGraphOptions& opts, bool is_partial,
-                                ReffedClientGraph** out_rcg,
-                                int64_t* out_count) {
+absl::Status MasterSession::StartStep(const BuildGraphOptions& opts,
+                                      bool is_partial,
+                                      ReffedClientGraph** out_rcg,
+                                      int64_t* out_count) {
   const uint64 hash = HashBuildGraphOptions(opts);
   {
     mutex_lock l(mu_);
@@ -1568,7 +1582,7 @@ Status MasterSession::StartStep(const BuildGraphOptions& opts, bool is_partial,
     (*out_rcg)->Ref();
     *out_count = (*out_rcg)->get_and_increment_execution_count();
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 void MasterSession::ClearRunsTable(std::vector<ReffedClientGraph*>* to_unref,
@@ -1594,9 +1608,9 @@ uint64 MasterSession::NewStepId(int64_t graph_key) {
     int32_t retry_count = 0;
     while (static_cast<int64_t>(step_id) == CollectiveExecutor::kInvalidId) {
       Notification note;
-      Status status;
+      absl::Status status;
       env_->collective_executor_mgr->RefreshStepIdSequenceAsync(
-          graph_key, [&status, &note](const Status& s) {
+          graph_key, [&status, &note](const absl::Status& s) {
             status = s;
             note.Notify();
           });
@@ -1615,8 +1629,8 @@ uint64 MasterSession::NewStepId(int64_t graph_key) {
   }
 }
 
-Status MasterSession::PartialRunSetup(const PartialRunSetupRequest* req,
-                                      PartialRunSetupResponse* resp) {
+absl::Status MasterSession::PartialRunSetup(const PartialRunSetupRequest* req,
+                                            PartialRunSetupResponse* resp) {
   std::vector<string> inputs, outputs, targets;
   for (const auto& feed : req->feed()) {
     inputs.push_back(feed);
@@ -1651,11 +1665,12 @@ Status MasterSession::PartialRunSetup(const PartialRunSetupRequest* req,
   TF_RETURN_IF_ERROR(BuildAndRegisterPartitions(rcg));
 
   resp->set_partial_run_handle(handle);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status MasterSession::Run(CallOptions* opts, const RunStepRequestWrapper& req,
-                          MutableRunStepResponseWrapper* resp) {
+absl::Status MasterSession::Run(CallOptions* opts,
+                                const RunStepRequestWrapper& req,
+                                MutableRunStepResponseWrapper* resp) {
   UpdateLastAccessTime();
   {
     mutex_lock l(mu_);
@@ -1666,7 +1681,7 @@ Status MasterSession::Run(CallOptions* opts, const RunStepRequestWrapper& req,
     // Note: all code paths must eventually call MarkRunCompletion()
     // in order to appropriate decrement the num_running_ counter.
   }
-  Status status;
+  absl::Status status;
   if (!req.partial_run_handle().empty()) {
     status = DoPartialRun(opts, req, resp);
   } else {
@@ -1684,7 +1699,7 @@ void MasterSession::MarkRunCompletion() {
   }
 }
 
-Status MasterSession::BuildAndRegisterPartitions(ReffedClientGraph* rcg) {
+absl::Status MasterSession::BuildAndRegisterPartitions(ReffedClientGraph* rcg) {
   // Registers subgraphs if haven't done so.
   PartitionOptions popts;
   popts.node_to_loc = SplitByWorker;
@@ -1724,12 +1739,12 @@ Status MasterSession::BuildAndRegisterPartitions(ReffedClientGraph* rcg) {
 
   TF_RETURN_IF_ERROR(rcg->RegisterPartitions(std::move(popts)));
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status MasterSession::DoPartialRun(CallOptions* opts,
-                                   const RunStepRequestWrapper& req,
-                                   MutableRunStepResponseWrapper* resp) {
+absl::Status MasterSession::DoPartialRun(CallOptions* opts,
+                                         const RunStepRequestWrapper& req,
+                                         MutableRunStepResponseWrapper* resp) {
   auto cleanup = gtl::MakeCleanup([this] { MarkRunCompletion(); });
   const string& prun_handle = req.partial_run_handle();
   RunState* run_state = nullptr;
@@ -1828,7 +1843,7 @@ Status MasterSession::DoPartialRun(CallOptions* opts,
   }
   bool is_last_partial_run = run_state->PendingDone();
 
-  Status s = run_state->rcg->RunPartitions(
+  absl::Status s = run_state->rcg->RunPartitions(
       env_, run_state->step_id, run_state->count, &run_state->pss, opts, req,
       resp, &cancellation_manager_, is_last_partial_run);
 
@@ -1843,7 +1858,7 @@ Status MasterSession::DoPartialRun(CallOptions* opts,
                       req.options(), resp->mutable_metadata());
     cleanup.release();  // MarkRunCompletion called in done closure.
     rcg->CleanupPartitionsAsync(
-        run_state->step_id, [this, rcg, prun_handle](const Status& s) {
+        run_state->step_id, [this, rcg, prun_handle](const absl::Status& s) {
           if (!s.ok()) {
             LOG(ERROR) << "Cleanup partition error: " << s;
           }
@@ -1857,7 +1872,7 @@ Status MasterSession::DoPartialRun(CallOptions* opts,
   return s;
 }
 
-Status MasterSession::CreateDebuggerState(
+absl::Status MasterSession::CreateDebuggerState(
     const DebugOptions& debug_options, const RunStepRequestWrapper& req,
     int64_t rcg_execution_count,
     std::unique_ptr<DebuggerStateInterface>* debugger_state) {
@@ -1865,14 +1880,17 @@ Status MasterSession::CreateDebuggerState(
       DebuggerStateRegistry::CreateState(debug_options, debugger_state));
 
   std::vector<string> input_names;
+  input_names.reserve(req.num_feeds());
   for (size_t i = 0; i < req.num_feeds(); ++i) {
     input_names.push_back(req.feed_name(i));
   }
   std::vector<string> output_names;
+  output_names.reserve(req.num_fetches());
   for (size_t i = 0; i < req.num_fetches(); ++i) {
     output_names.push_back(req.fetch_name(i));
   }
   std::vector<string> target_names;
+  target_names.reserve(req.num_targets());
   for (size_t i = 0; i < req.num_targets(); ++i) {
     target_names.push_back(req.target_name(i));
   }
@@ -1885,7 +1903,7 @@ Status MasterSession::CreateDebuggerState(
       debug_options.global_step(), rcg_execution_count, rcg_execution_count,
       input_names, output_names, target_names));
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 void MasterSession::FillPerStepState(MasterSession::ReffedClientGraph* rcg,
@@ -1916,14 +1934,12 @@ void MasterSession::FillPerStepState(MasterSession::ReffedClientGraph* rcg,
   }
 }
 
-Status MasterSession::PostRunCleanup(MasterSession::ReffedClientGraph* rcg,
-                                     uint64 step_id,
-                                     const RunOptions& run_options,
-                                     PerStepState* pss,
-                                     const std::unique_ptr<ProfileHandler>& ph,
-                                     const Status& run_status,
-                                     RunMetadata* out_run_metadata) {
-  Status s = run_status;
+absl::Status MasterSession::PostRunCleanup(
+    MasterSession::ReffedClientGraph* rcg, uint64 step_id,
+    const RunOptions& run_options, PerStepState* pss,
+    const std::unique_ptr<ProfileHandler>& ph, const absl::Status& run_status,
+    RunMetadata* out_run_metadata) {
+  absl::Status s = run_status;
   if (s.ok()) {
     pss->end_micros = Env::Default()->NowMicros();
     if (rcg->collective_graph_key() !=
@@ -1948,7 +1964,7 @@ Status MasterSession::PostRunCleanup(MasterSession::ReffedClientGraph* rcg,
   }
   Ref();
   rcg->Ref();
-  rcg->CleanupPartitionsAsync(step_id, [this, rcg](const Status& s) {
+  rcg->CleanupPartitionsAsync(step_id, [this, rcg](const absl::Status& s) {
     if (!s.ok()) {
       LOG(ERROR) << "Cleanup partition error: " << s;
     }
@@ -1959,7 +1975,7 @@ Status MasterSession::PostRunCleanup(MasterSession::ReffedClientGraph* rcg,
   return s;
 }
 
-Status MasterSession::DoRunWithLocalExecution(
+absl::Status MasterSession::DoRunWithLocalExecution(
     CallOptions* opts, const RunStepRequestWrapper& req,
     MutableRunStepResponseWrapper* resp) {
   VLOG(2) << "DoRunWithLocalExecution req: " << req.DebugString();
@@ -2001,16 +2017,16 @@ Status MasterSession::DoRunWithLocalExecution(
         "disable_output_partition_graphs is true.");
   }
 
-  Status s = rcg->RunPartitions(env_, step_id, count, &pss, opts, req, resp,
-                                &cancellation_manager_, false);
+  absl::Status s = rcg->RunPartitions(env_, step_id, count, &pss, opts, req,
+                                      resp, &cancellation_manager_, false);
 
   cleanup.release();  // MarkRunCompletion called in PostRunCleanup().
   return PostRunCleanup(rcg, step_id, req.options(), &pss, ph, s,
                         resp->mutable_metadata());
 }
 
-Status MasterSession::MakeCallable(const MakeCallableRequest& req,
-                                   MakeCallableResponse* resp) {
+absl::Status MasterSession::MakeCallable(const MakeCallableRequest& req,
+                                         MakeCallableResponse* resp) {
   UpdateLastAccessTime();
 
   BuildGraphOptions opts;
@@ -2032,7 +2048,7 @@ Status MasterSession::MakeCallable(const MakeCallableRequest& req,
                                      !should_delete_worker_sessions_);
   }
 
-  Status s = BuildAndRegisterPartitions(callable);
+  absl::Status s = BuildAndRegisterPartitions(callable);
   if (!s.ok()) {
     callable->Unref();
     return s;
@@ -2046,12 +2062,13 @@ Status MasterSession::MakeCallable(const MakeCallableRequest& req,
   }
 
   resp->set_handle(handle);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status MasterSession::DoRunCallable(CallOptions* opts, ReffedClientGraph* rcg,
-                                    const RunCallableRequest& req,
-                                    RunCallableResponse* resp) {
+absl::Status MasterSession::DoRunCallable(CallOptions* opts,
+                                          ReffedClientGraph* rcg,
+                                          const RunCallableRequest& req,
+                                          RunCallableResponse* resp) {
   VLOG(2) << "DoRunCallable req: " << req.DebugString();
   PerStepState pss;
   pss.start_micros = Env::Default()->NowMicros();
@@ -2071,16 +2088,16 @@ Status MasterSession::DoRunCallable(CallOptions* opts, ReffedClientGraph* rcg,
 
   std::unique_ptr<ProfileHandler> ph;
   FillPerStepState(rcg, run_options, step_id, count, &pss, &ph);
-  Status s = rcg->RunPartitions(env_, step_id, count, &pss, opts, req, resp,
-                                &cancellation_manager_);
+  absl::Status s = rcg->RunPartitions(env_, step_id, count, &pss, opts, req,
+                                      resp, &cancellation_manager_);
   cleanup.release();  // MarkRunCompletion called in PostRunCleanup().
   return PostRunCleanup(rcg, step_id, run_options, &pss, ph, s,
                         resp->mutable_metadata());
 }
 
-Status MasterSession::RunCallable(CallOptions* opts,
-                                  const RunCallableRequest& req,
-                                  RunCallableResponse* resp) {
+absl::Status MasterSession::RunCallable(CallOptions* opts,
+                                        const RunCallableRequest& req,
+                                        RunCallableResponse* resp) {
   UpdateLastAccessTime();
   ReffedClientGraph* callable;
   {
@@ -2105,8 +2122,8 @@ Status MasterSession::RunCallable(CallOptions* opts,
   return DoRunCallable(opts, callable, req, resp);
 }
 
-Status MasterSession::ReleaseCallable(const ReleaseCallableRequest& req,
-                                      ReleaseCallableResponse* resp) {
+absl::Status MasterSession::ReleaseCallable(const ReleaseCallableRequest& req,
+                                            ReleaseCallableResponse* resp) {
   UpdateLastAccessTime();
   ReffedClientGraph* to_unref = nullptr;
   {
@@ -2120,10 +2137,10 @@ Status MasterSession::ReleaseCallable(const ReleaseCallableRequest& req,
   if (to_unref != nullptr) {
     to_unref->Unref();
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status MasterSession::Close() {
+absl::Status MasterSession::Close() {
   {
     mutex_lock l(mu_);
     closed_ = true;  // All subsequent calls to Run() or Extend() will fail.
@@ -2141,12 +2158,12 @@ Status MasterSession::Close() {
   }
   for (ReffedClientGraph* rcg : to_unref) rcg->Unref();
   if (should_delete_worker_sessions_) {
-    Status s = DeleteWorkerSessions();
+    absl::Status s = DeleteWorkerSessions();
     if (!s.ok()) {
       LOG(WARNING) << s;
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 void MasterSession::GarbageCollect() {
