@@ -18,11 +18,17 @@ limitations under the License.
 #include <deque>
 #include <memory>
 #include <unordered_map>
+#include <utility>
 #include <vector>
 
 #include "grpcpp/alarm.h"
 #include "grpcpp/server_builder.h"
 #include "absl/container/flat_hash_map.h"
+#include "absl/status/status.h"
+#include "absl/strings/str_cat.h"
+#include "xla/tsl/distributed_runtime/rpc/async_service_interface.h"
+#include "xla/tsl/distributed_runtime/rpc/grpc_call.h"
+#include "xla/tsl/protobuf/rpc_options.pb.h"
 #include "tensorflow/core/common_runtime/buf_rendezvous.h"
 #include "tensorflow/core/common_runtime/copy_tensor.h"
 #include "tensorflow/core/common_runtime/device.h"
@@ -50,13 +56,10 @@ limitations under the License.
 #include "tensorflow/core/lib/strings/stringprintf.h"
 #include "tensorflow/core/platform/logging.h"
 #include "tensorflow/core/platform/mutex.h"
-#include "tensorflow/core/platform/tracing.h"
 #include "tensorflow/core/profiler/lib/scoped_memory_debug_annotation.h"
 #include "tensorflow/core/protobuf/transport_options.pb.h"
 #include "tensorflow/core/protobuf/worker.pb.h"
-#include "tensorflow/tsl/distributed_runtime/rpc/async_service_interface.h"
-#include "tensorflow/tsl/distributed_runtime/rpc/grpc_call.h"
-#include "tensorflow/tsl/protobuf/rpc_options.pb.h"
+#include "tsl/platform/tracing.h"
 
 namespace tensorflow {
 
@@ -223,7 +226,7 @@ class GrpcWorkerServiceThread {
       WorkerCall<GetStepSequenceRequest, GetStepSequenceResponse>* call) {
     Schedule([this, call]() {
       worker_->GetStepSequenceAsync(
-          &call->request, &call->response, [call](const Status& s) {
+          &call->request, &call->response, [call](const absl::Status& s) {
             VLOG(3) << "Bad response from GetStepSequence:" << s;
             call->SendResponse(ToGrpcStatus(s));
           });
@@ -249,7 +252,7 @@ class GrpcWorkerServiceThread {
       call->SetCancelCallback([call_opts]() { call_opts->StartCancel(); });
       worker_->RunGraphAsync(call_opts, wrapped_request, wrapped_response,
                              [call, call_opts, wrapped_request,
-                              wrapped_response](const Status& s) {
+                              wrapped_response](const absl::Status& s) {
                                VLOG(3) << "RunGraph::Done";
                                if (!s.ok()) {
                                  VLOG(3) << "Bad response from RunGraph:" << s;
@@ -272,7 +275,7 @@ class GrpcWorkerServiceThread {
 
       worker_->GrpcRecvTensorAsync(
           call_opts, &call->request, &call->response,
-          [call, call_opts](const Status& s) {
+          [call, call_opts](const absl::Status& s) {
             call->ClearCancelCallback();
             delete call_opts;
             if (!s.ok()) {
@@ -289,7 +292,7 @@ class GrpcWorkerServiceThread {
       CallOptions* call_opts = new CallOptions;
       call->SetCancelCallback([call_opts]() { call_opts->StartCancel(); });
       worker_->RecvBufAsync(call_opts, &call->request, &call->response,
-                            [call, call_opts](const Status& s) {
+                            [call, call_opts](const absl::Status& s) {
                               call->ClearCancelCallback();
                               delete call_opts;
                               if (!s.ok()) {
@@ -308,7 +311,7 @@ class GrpcWorkerServiceThread {
       call->SetCancelCallback([call_opts]() { call_opts->StartCancel(); });
       worker_->CompleteGroupAsync(
           call_opts, &call->request, &call->response,
-          [call, call_opts](const Status& s) {
+          [call, call_opts](const absl::Status& s) {
             call->ClearCancelCallback();
             delete call_opts;
             if (!s.ok()) {
@@ -327,7 +330,7 @@ class GrpcWorkerServiceThread {
       call->SetCancelCallback([call_opts]() { call_opts->StartCancel(); });
       worker_->CompleteInstanceAsync(
           call_opts, &call->request, &call->response,
-          [call, call_opts](const Status& s) {
+          [call, call_opts](const absl::Status& s) {
             call->ClearCancelCallback();
             delete call_opts;
             if (!s.ok()) {
@@ -361,7 +364,8 @@ class GrpcWorkerServiceThread {
 
   mutex shutdown_mu_;
   bool is_shutdown_ TF_GUARDED_BY(shutdown_mu_);
-  TF_DISALLOW_COPY_AND_ASSIGN(GrpcWorkerServiceThread);
+  GrpcWorkerServiceThread(const GrpcWorkerServiceThread&) = delete;
+  void operator=(const GrpcWorkerServiceThread&) = delete;
 };
 
 class GrpcWorkerService : public tsl::AsyncServiceInterface {
@@ -370,6 +374,14 @@ class GrpcWorkerService : public tsl::AsyncServiceInterface {
                     GrpcWorkerServiceOptions options)
       : is_shutdown_(false) {
     builder->RegisterService(&worker_service_);
+    // gRPC by default will cancel requests that sit in a completion queue for
+    // more than 30s. See
+    // https://github.com/grpc/grpc/blob/e52e48b7ef83feeff56ed0894ce39841ea8bd483/include/grpc/impl/channel_arg_names.h#L106-L111
+    // Extending this to 1 hour for Tensorflow since some graphs may have
+    // periods of heavy load which may cause the server to run into these
+    // cancellations.
+    builder->AddChannelArgument("grpc.server_max_unrequested_time_in_server",
+                                3600);
 
     for (int i = 0; i < options.num_serving_threads; i++) {
       threads_.emplace_back(new GrpcWorkerServiceThread(
@@ -411,7 +423,8 @@ class GrpcWorkerService : public tsl::AsyncServiceInterface {
   mutex service_shutdown_mu_;
   bool is_shutdown_ TF_GUARDED_BY(service_shutdown_mu_);
 
-  TF_DISALLOW_COPY_AND_ASSIGN(GrpcWorkerService);
+  GrpcWorkerService(const GrpcWorkerService&) = delete;
+  void operator=(const GrpcWorkerService&) = delete;
 };
 
 }  // namespace
@@ -445,13 +458,26 @@ void GrpcWorker::GrpcRecvTensorAsync(CallOptions* opts,
 
   bool cache_enabled = (response_cache_ != nullptr && request_id != 0);
 
-  auto do_response = [response, done, cache_enabled](const Tensor& tensor,
-                                                     bool is_dead,
-                                                     const Status& status) {
+  auto do_response = [request, response, done = std::move(done), cache_enabled](
+                         const Tensor& tensor, bool is_dead,
+                         const absl::Status& status) {
+    absl::Status updated_status;
     if (status.ok()) {
-      grpc::EncodeTensorToByteBuffer(is_dead, tensor, cache_enabled, response);
+      updated_status = grpc::EncodeTensorToByteBuffer(is_dead, tensor,
+                                                      cache_enabled, response);
+      if (!updated_status.ok()) {
+        updated_status = absl::InternalError(absl::StrCat(
+            "Failed to encode tensor to byte buffer: ",
+            updated_status.message(), " (request_id: ", request->request_id(),
+            " step_id: ", request->step_id(),
+            " rendezvous_key: ", request->rendezvous_key(), ")"));
+        LOG(ERROR) << "Failure to encode response during GrpcRecvTensorAsync: "
+                   << updated_status;
+      }
+    } else {
+      updated_status = status;
     }
-    done(status);
+    done(updated_status);
   };
 
   // If response cache is enabled and the response cache already contains the
@@ -465,7 +491,7 @@ void GrpcWorker::GrpcRecvTensorAsync(CallOptions* opts,
 
   auto rendezvous_done = [this, request_id, do_response, cache_enabled](
                              const Tensor& tensor, bool is_dead,
-                             const Status& status) {
+                             const absl::Status& status) {
     if (cache_enabled) {
       // Data is ready. Process all pending requests in the response cache.
       response_cache_->RequestFinished(request_id, tensor, is_dead, status);
@@ -474,11 +500,11 @@ void GrpcWorker::GrpcRecvTensorAsync(CallOptions* opts,
     }
   };
 
-  auto fail = [&rendezvous_done](const Status& status) {
+  auto fail = [&rendezvous_done](const absl::Status& status) {
     rendezvous_done(Tensor(), false, status);
   };
 
-  Status s = recent_request_ids_.TrackUnique(
+  absl::Status s = recent_request_ids_.TrackUnique(
       request_id, "RecvTensor (GrpcWorker)", *request);
   if (!s.ok()) {
     fail(s);
@@ -486,7 +512,7 @@ void GrpcWorker::GrpcRecvTensorAsync(CallOptions* opts,
   }
 
   const string& key = request->rendezvous_key();
-  TRACEPRINTF("RecvTensor: %lld %s", step_id, key.c_str());
+  TRACEPRINTF("RecvTensor: %lld %s", step_id, key);
   Rendezvous::ParsedKey parsed;
   s = Rendezvous::ParseKey(key, &parsed);
   Device* src_dev = nullptr;
@@ -513,7 +539,7 @@ void GrpcWorker::GrpcRecvTensorAsync(CallOptions* opts,
   env_->rendezvous_mgr->RecvLocalAsync(
       step_id, parsed,
       [opts, rendezvous_done, src_dev, request](
-          const Status& status, const Rendezvous::Args& send_args,
+          const absl::Status& status, const Rendezvous::Args& send_args,
           const Rendezvous::Args& recv_args, const Tensor& val,
           const bool is_dead) {
         opts->ClearCancelCallback();
@@ -530,7 +556,7 @@ void GrpcWorker::GrpcRecvTensorAsync(CallOptions* opts,
         AllocatorAttributes alloc_attrs;
         alloc_attrs.set_gpu_compatible(true);
         alloc_attrs.set_on_host(true);
-        profiler::ScopedMemoryDebugAnnotation op_annotation(
+        tsl::profiler::ScopedMemoryDebugAnnotation op_annotation(
             "GrpcWorker::RecvTensorAsync::consumer_callback",
             request->step_id(), "dynamic", val.dtype(),
             [shape = val.shape()]() { return shape.DebugString(); });
@@ -541,7 +567,7 @@ void GrpcWorker::GrpcRecvTensorAsync(CallOptions* opts,
             << " gpu_info: " << src_dev->tensorflow_accelerator_device_info();
 
         StatusCallback copy_ready = [rendezvous_done, copy,
-                                     is_dead](const Status& s) {
+                                     is_dead](const absl::Status& s) {
           // The value is now ready to be returned on the wire.
           rendezvous_done(*copy, is_dead, s);
           delete copy;
@@ -588,7 +614,7 @@ void GrpcWorker::RecvBufAsync(CallOptions* opts, const RecvBufRequest* request,
 
   auto do_response = [this, response, done, cache_enabled](
                          const Tensor& tensor, bool is_dead,
-                         const Status& status) {
+                         const absl::Status& status) {
     if (status.ok()) {
       SetTensorInRecvBufResp(recv_buf_max_chunk_, &tensor, response);
     }
@@ -607,7 +633,7 @@ void GrpcWorker::RecvBufAsync(CallOptions* opts, const RecvBufRequest* request,
   }
 
   auto rendezvous_done = [this, request_id, do_response, cache_enabled](
-                             const Tensor& tensor, const Status& status) {
+                             const Tensor& tensor, const absl::Status& status) {
     if (cache_enabled) {
       // Data is ready. Process all pending requests in the response cache.
       response_cache_->RequestFinished(request_id, tensor, false, status);
@@ -616,13 +642,13 @@ void GrpcWorker::RecvBufAsync(CallOptions* opts, const RecvBufRequest* request,
     }
   };
 
-  auto fail = [&rendezvous_done](const Status& status) {
+  auto fail = [&rendezvous_done](const absl::Status& status) {
     rendezvous_done(Tensor(), status);
   };
 
   // This is a generic, low performance implementation appropriate for grpc.
-  Status s = recent_request_ids_.TrackUnique(request_id, "RecvBuf (GrpcWorker)",
-                                             *request);
+  absl::Status s = recent_request_ids_.TrackUnique(
+      request_id, "RecvBuf (GrpcWorker)", *request);
   if (!s.ok()) {
     fail(s);
     return;
@@ -632,9 +658,9 @@ void GrpcWorker::RecvBufAsync(CallOptions* opts, const RecvBufRequest* request,
       env_->collective_executor_mgr->FindOrCreate(step_id), true);
   CollectiveRemoteAccess* rma = ce_handle.get()->remote_access();
   auto consumer_callback = [this, request, rendezvous_done](
-                               const Status& status,
+                               const absl::Status& status,
                                BufRendezvous::Hook* hook) {
-    Status s = status;
+    absl::Status s = status;
     if (s.ok()) {
       if (hook == nullptr) {
         s = errors::Internal("Invalid null hook for key ",
@@ -667,7 +693,7 @@ void GrpcWorker::RecvBufAsync(CallOptions* opts, const RecvBufRequest* request,
           AllocatorAttributes cpu_attr;
           cpu_attr.set_gpu_compatible(true);
           cpu_attr.set_nic_compatible(true);
-          profiler::ScopedMemoryDebugAnnotation op_annotation(
+          tsl::profiler::ScopedMemoryDebugAnnotation op_annotation(
               "GrpcWorker::RecvBufAsync::consumer_callback", request->step_id(),
               "dynamic", hook->prod_value->dtype(),
               [hook]() { return hook->prod_value->shape().DebugString(); });
@@ -676,7 +702,7 @@ void GrpcWorker::RecvBufAsync(CallOptions* opts, const RecvBufRequest* request,
                          hook->prod_value->dtype(), hook->prod_value->shape());
           hook->prod_ctx->CopyDeviceTensorToCPU(
               hook->prod_value, "empty_name", hook->prod_dev, cpu_tensor,
-              [hook, cpu_tensor, rendezvous_done](const Status& s) {
+              [hook, cpu_tensor, rendezvous_done](const absl::Status& s) {
                 rendezvous_done(*cpu_tensor, s);
                 BufRendezvous::DoneWithHook(hook);
                 delete cpu_tensor;
@@ -723,7 +749,7 @@ void GrpcWorker::LoggingAsync(const LoggingRequest* request,
       }
     }
   }
-  done(OkStatus());
+  done(absl::OkStatus());
 }
 
 void GrpcWorker::CleanupGraphAsync(const CleanupGraphRequest* request,
