@@ -15,21 +15,41 @@ limitations under the License.
 
 #include "tensorflow/core/framework/attr_value_util.h"
 
+#include <algorithm>
+#include <cstdint>
+#include <functional>
+#include <initializer_list>
+#include <map>
 #include <string>
 #include <unordered_map>
+#include <utility>
 #include <vector>
 
+#include "absl/log/log.h"
+#include "absl/status/status.h"
 #include "absl/strings/escaping.h"
+#include "absl/strings/match.h"
+#include "absl/strings/str_cat.h"
+#include "absl/strings/str_join.h"
+#include "absl/strings/strip.h"
+#include "absl/types/span.h"
 #include "tensorflow/core/framework/attr_value.pb_text.h"
-#include "tensorflow/core/framework/tensor.pb_text.h"
+#include "tensorflow/core/framework/tensor.h"
+#include "tensorflow/core/framework/tensor.pb.h"
+#include "tensorflow/core/framework/tensor_shape.h"
 #include "tensorflow/core/framework/tensor_shape.pb.h"
 #include "tensorflow/core/framework/types.h"
 #include "tensorflow/core/framework/types.pb_text.h"
 #include "tensorflow/core/lib/core/errors.h"
 #include "tensorflow/core/lib/core/stringpiece.h"
+#include "tensorflow/core/lib/gtl/array_slice.h"
 #include "tensorflow/core/lib/strings/proto_serialization.h"
 #include "tensorflow/core/lib/strings/str_util.h"
 #include "tensorflow/core/platform/fingerprint.h"
+#include "tensorflow/core/platform/hash.h"
+#include "tensorflow/core/platform/numbers.h"
+#include "tensorflow/core/platform/tstring.h"
+#include "tensorflow/core/platform/types.h"
 #include "tensorflow/core/util/overflow.h"
 
 namespace tensorflow {
@@ -186,13 +206,13 @@ string SummarizeString(const string& str) {
   // If the string is long, replace the middle with ellipses.
   constexpr int kMaxStringSummarySize = 80;
   if (escaped.size() >= kMaxStringSummarySize) {
-    StringPiece prefix(escaped);
-    StringPiece suffix = prefix;
+    absl::string_view prefix(escaped);
+    absl::string_view suffix = prefix;
     prefix.remove_suffix(escaped.size() - 10);
     suffix.remove_prefix(escaped.size() - 10);
-    return strings::StrCat("\"", prefix, "...", suffix, "\"");
+    return absl::StrCat("\"", prefix, "...", suffix, "\"");
   } else {
-    return strings::StrCat("\"", escaped, "\"");
+    return absl::StrCat("\"", escaped, "\"");
   }
 }
 
@@ -205,10 +225,9 @@ string SummarizeTensor(const TensorProto& tensor_proto) {
   ) {
     // Do not load large or unknown-shape Tensor to compute detailed
     // DebugString()
-    return strings::StrCat("<TensorProto: ", tensor_proto.ShortDebugString(),
-                           ">");
+    return absl::StrCat("<TensorProto: ", tensor_proto.ShortDebugString(), ">");
   } else if (!t.FromProto(tensor_proto)) {
-    return strings::StrCat(
+    return absl::StrCat(
         "<Invalid TensorProto: ", tensor_proto.ShortDebugString(), ">");
   }
   return t.DebugString();
@@ -217,11 +236,10 @@ string SummarizeTensor(const TensorProto& tensor_proto) {
 string SummarizeFunc(const NameAttrList& func) {
   std::vector<string> entries;
   for (const auto& p : func.attr()) {
-    entries.push_back(
-        strings::StrCat(p.first, "=", SummarizeAttrValue(p.second)));
+    entries.push_back(absl::StrCat(p.first, "=", SummarizeAttrValue(p.second)));
   }
   std::sort(entries.begin(), entries.end());
-  return strings::StrCat(func.name(), "[", absl::StrJoin(entries, ", "), "]");
+  return absl::StrCat(func.name(), "[", absl::StrJoin(entries, ", "), "]");
 }
 
 bool ParseAttrValueHelper_TensorNestsUnderLimit(int limit, string to_parse) {
@@ -279,9 +297,9 @@ string SummarizeAttrValue(const AttrValue& attr_value) {
     case AttrValue::kS:
       return SummarizeString(attr_value.s());
     case AttrValue::kI:
-      return strings::StrCat(attr_value.i());
+      return absl::StrCat(attr_value.i());
     case AttrValue::kF:
-      return strings::StrCat(attr_value.f());
+      return absl::StrCat(strings::LegacyPrecision(attr_value.f()));
     case AttrValue::kB:
       return attr_value.b() ? "true" : "false";
     case AttrValue::kType:
@@ -298,11 +316,12 @@ string SummarizeAttrValue(const AttrValue& attr_value) {
         }
       } else if (attr_value.list().i_size() > 0) {
         for (int i = 0; i < attr_value.list().i_size(); ++i) {
-          pieces.push_back(strings::StrCat(attr_value.list().i(i)));
+          pieces.push_back(absl::StrCat(attr_value.list().i(i)));
         }
       } else if (attr_value.list().f_size() > 0) {
         for (int i = 0; i < attr_value.list().f_size(); ++i) {
-          pieces.push_back(strings::StrCat(attr_value.list().f(i)));
+          pieces.push_back(
+              absl::StrCat(strings::LegacyPrecision(attr_value.list().f(i))));
         }
       } else if (attr_value.list().b_size() > 0) {
         for (int i = 0; i < attr_value.list().b_size(); ++i) {
@@ -334,24 +353,25 @@ string SummarizeAttrValue(const AttrValue& attr_value) {
             Fingerprint64(absl::StrJoin(pieces.begin(), pieces.end(), ","));
         pieces.erase(pieces.begin() + 5, pieces.end() - 6);
         pieces[5] = "...";
-        return strings::StrCat("[", absl::StrJoin(pieces, ", "),
-                               "]{attr_hash=", fingerprint, "}");
+        return absl::StrCat("[", absl::StrJoin(pieces, ", "),
+                            "]{attr_hash=", fingerprint, "}");
       } else {
-        return strings::StrCat("[", absl::StrJoin(pieces, ", "), "]");
+        return absl::StrCat("[", absl::StrJoin(pieces, ", "), "]");
       }
     }
     case AttrValue::kFunc: {
       return SummarizeFunc(attr_value.func());
     }
     case AttrValue::kPlaceholder:
-      return strings::StrCat("$", attr_value.placeholder());
+      return absl::StrCat("$", attr_value.placeholder());
     case AttrValue::VALUE_NOT_SET:
       return "<Unknown AttrValue type>";
   }
   return "<Unknown AttrValue type>";  // Prevent missing return warning
 }
 
-Status AttrValueHasType(const AttrValue& attr_value, StringPiece type) {
+absl::Status AttrValueHasType(const AttrValue& attr_value,
+                              absl::string_view type) {
   int num_set = 0;
 
 #define VALIDATE_FIELD(name, type_string, oneof_case)                         \
@@ -446,10 +466,11 @@ Status AttrValueHasType(const AttrValue& attr_value, StringPiece type) {
     }
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-bool ParseAttrValue(StringPiece type, StringPiece text, AttrValue* out) {
+bool ParseAttrValue(absl::string_view type, absl::string_view text,
+                    AttrValue* out) {
   // Parse type.
   string field_name;
   bool is_list = absl::ConsumePrefix(&type, "list(");
@@ -483,7 +504,7 @@ bool ParseAttrValue(StringPiece type, StringPiece text, AttrValue* out) {
   if (is_list) {
     // TextFormat parser considers "i: 7" to be the same as "i: [7]",
     // but we only want to allow list values with [].
-    StringPiece cleaned = text;
+    absl::string_view cleaned = text;
     str_util::RemoveLeadingWhitespace(&cleaned);
     str_util::RemoveTrailingWhitespace(&cleaned);
     if (cleaned.size() < 2 || cleaned[0] != '[' ||
@@ -499,9 +520,9 @@ bool ParseAttrValue(StringPiece type, StringPiece text, AttrValue* out) {
       out->mutable_list();
       return true;
     }
-    to_parse = strings::StrCat("list { ", field_name, ": ", text, " }");
+    to_parse = absl::StrCat("list { ", field_name, ": ", text, " }");
   } else {
-    to_parse = strings::StrCat(field_name, ": ", text);
+    to_parse = absl::StrCat(field_name, ": ", text);
   }
   if (field_name == "tensor") {
     if (!ParseAttrValueHelper_TensorNestsUnderLimit(kMaxTensorNestDepth,
@@ -530,7 +551,7 @@ void SetAttrValue(const AttrValue& value, AttrValue* out) { *out = value; }
   DEFINE_SET_ATTR_VALUE_LIST(gtl::ArraySlice<ARG_TYPE>, FIELD)
 
 DEFINE_SET_ATTR_VALUE_ONE(const string&, s)
-DEFINE_SET_ATTR_VALUE_LIST(gtl::ArraySlice<string>, s)
+DEFINE_SET_ATTR_VALUE_LIST(absl::Span<const string>, s)
 DEFINE_SET_ATTR_VALUE_BOTH(const char*, s)
 DEFINE_SET_ATTR_VALUE_BOTH(int64_t, i)
 DEFINE_SET_ATTR_VALUE_BOTH(int32_t, i)
@@ -545,18 +566,19 @@ void SetAttrValue(const tstring& value, AttrValue* out) {
   out->set_s(value.data(), value.size());
 }
 
-void SetAttrValue(gtl::ArraySlice<tstring> value, AttrValue* out) {
+void SetAttrValue(absl::Span<const tstring> value, AttrValue* out) {
   out->mutable_list()->Clear();
   for (const auto& v : value) {
     out->mutable_list()->add_s(v.data(), v.size());
   }
 }
 
-void SetAttrValue(StringPiece value, AttrValue* out) {
+void SetAttrValue(absl::string_view value, AttrValue* out) {
   out->set_s(value.data(), value.size());
 }
 
-void SetAttrValue(const gtl::ArraySlice<StringPiece> value, AttrValue* out) {
+void SetAttrValue(const absl::Span<const absl::string_view> value,
+                  AttrValue* out) {
   out->mutable_list()->Clear();  // Create list() even if value empty.
   for (const auto& v : value) {
     out->mutable_list()->add_s(v.data(), v.size());
@@ -582,21 +604,21 @@ void SetAttrValue(const PartialTensorShape& value, AttrValue* out) {
   value.AsProto(out->mutable_shape());
 }
 
-void SetAttrValue(const gtl::ArraySlice<TensorShape> value, AttrValue* out) {
+void SetAttrValue(const absl::Span<const TensorShape> value, AttrValue* out) {
   out->mutable_list()->Clear();  // Create list() even if value empty.
   for (const auto& v : value) {
     v.AsProto(out->mutable_list()->add_shape());
   }
 }
 
-void SetAttrValue(gtl::ArraySlice<TensorShapeProto> value, AttrValue* out) {
+void SetAttrValue(absl::Span<const TensorShapeProto> value, AttrValue* out) {
   out->mutable_list()->Clear();  // Create list() even if value empty.
   for (const auto& v : value) {
     *out->mutable_list()->add_shape() = v;
   }
 }
 
-void SetAttrValue(const gtl::ArraySlice<PartialTensorShape> value,
+void SetAttrValue(const absl::Span<const PartialTensorShape> value,
                   AttrValue* out) {
   out->mutable_list()->Clear();  // Create list() even if value empty.
   for (const auto& v : value) {
@@ -612,7 +634,7 @@ void SetAttrValue(const Tensor& value, AttrValue* out) {
   }
 }
 
-void SetAttrValue(const gtl::ArraySlice<Tensor> value, AttrValue* out) {
+void SetAttrValue(const absl::Span<const Tensor> value, AttrValue* out) {
   out->mutable_list()->Clear();  // Create list() even if value empty.
   for (const auto& v : value) {
     if (v.NumElements() > 1) {
@@ -627,7 +649,7 @@ void SetAttrValue(const TensorProto& value, AttrValue* out) {
   *out->mutable_tensor() = value;
 }
 
-void SetAttrValue(const gtl::ArraySlice<TensorProto> value, AttrValue* out) {
+void SetAttrValue(const absl::Span<const TensorProto> value, AttrValue* out) {
   out->mutable_list()->Clear();  // Create list() even if value empty.
   for (const auto& v : value) {
     *out->mutable_list()->add_tensor() = v;
@@ -638,7 +660,7 @@ void SetAttrValue(const NameAttrList& value, AttrValue* out) {
   *out->mutable_func() = value;
 }
 
-void SetAttrValue(gtl::ArraySlice<NameAttrList> value, AttrValue* out) {
+void SetAttrValue(absl::Span<const NameAttrList> value, AttrValue* out) {
   out->mutable_list()->Clear();  // Create list() even if value empty.
   for (const auto& v : value) {
     *out->mutable_list()->add_func() = v;
