@@ -43,7 +43,7 @@ class NodeDefBuilderTest : public ::testing::Test {
   // to Op() above.
   NodeDefBuilder& Builder() {
     EXPECT_FALSE(op_def_.name().empty()) << "Must call Op() before Builder()";
-    builder_.reset(new NodeDefBuilder("n", &op_def_));
+    builder_ = std::make_unique<NodeDefBuilder>("n", &op_def_);
     return *builder_;
   }
 
@@ -51,9 +51,10 @@ class NodeDefBuilderTest : public ::testing::Test {
   // expectations.
   void ExpectSuccess(NodeDefBuilder& builder,  // NOLINT
                      DataTypeSlice expected_in_types,
-                     DataTypeSlice expected_out_types, StringPiece proto) {
+                     DataTypeSlice expected_out_types,
+                     absl::string_view proto) {
     NodeDef node_def;
-    Status status = builder.Finalize(&node_def);
+    absl::Status status = builder.Finalize(&node_def);
     TF_EXPECT_OK(status);
     if (!status.ok()) return;
     NodeDef expected;
@@ -80,7 +81,7 @@ class NodeDefBuilderTest : public ::testing::Test {
   void ExpectFailures(NodeDefBuilder& builder,  // NOLINT
                       const std::vector<string>& messages) {
     NodeDef node_def;
-    Status status = builder.Finalize(&node_def);
+    absl::Status status = builder.Finalize(&node_def);
     EXPECT_FALSE(status.ok()) << SummarizeNodeDef(node_def);
     if (status.ok()) return;
     for (const string& message : messages) {
@@ -101,7 +102,7 @@ class NodeDefBuilderTest : public ::testing::Test {
   void ExpectInvalid(NodeDefBuilder& builder,  // NOLINT
                      const string& message) {
     NodeDef node_def;
-    Status status = builder.Finalize(&node_def);
+    absl::Status status = builder.Finalize(&node_def);
     if (status.ok()) {
       status = ValidateNodeDef(node_def, op_def_);
     }
