@@ -36,11 +36,11 @@ namespace tensorflow {
 
 namespace {
 
-string AttrError(StringPiece orig, const string& op_name) {
+string AttrError(absl::string_view orig, const string& op_name) {
   return strings::StrCat(" from Attr(\"", orig, "\") for Op ", op_name);
 }
 
-bool ConsumeAttrName(StringPiece* sp, StringPiece* out) {
+bool ConsumeAttrName(absl::string_view* sp, absl::string_view* out) {
   return Scanner(*sp)
       .One(Scanner::LETTER)
       .Any(Scanner::LETTER_DIGIT_UNDERSCORE)
@@ -51,7 +51,7 @@ bool ConsumeAttrName(StringPiece* sp, StringPiece* out) {
       .GetResult(sp, out);
 }
 
-bool ConsumeListPrefix(StringPiece* sp) {
+bool ConsumeListPrefix(absl::string_view* sp) {
   return Scanner(*sp)
       .OneLiteral("list")
       .AnySpace()
@@ -60,7 +60,8 @@ bool ConsumeListPrefix(StringPiece* sp) {
       .GetResult(sp);
 }
 
-bool ConsumeQuotedString(char quote_ch, StringPiece* sp, StringPiece* out) {
+bool ConsumeQuotedString(char quote_ch, absl::string_view* sp,
+                         absl::string_view* out) {
   const string quote_str(1, quote_ch);
   return Scanner(*sp)
       .OneLiteral(quote_str.c_str())
@@ -72,7 +73,7 @@ bool ConsumeQuotedString(char quote_ch, StringPiece* sp, StringPiece* out) {
       .GetResult(sp, out);
 }
 
-bool ConsumeAttrType(StringPiece* sp, StringPiece* out) {
+bool ConsumeAttrType(absl::string_view* sp, absl::string_view* out) {
   return Scanner(*sp)
       .Many(Scanner::LOWERLETTER_DIGIT)
       .StopCapture()
@@ -80,10 +81,10 @@ bool ConsumeAttrType(StringPiece* sp, StringPiece* out) {
       .GetResult(sp, out);
 }
 
-bool ConsumeAttrNumber(StringPiece* sp, int64_t* out) {
+bool ConsumeAttrNumber(absl::string_view* sp, int64_t* out) {
   Scanner scan(*sp);
-  StringPiece match;
-  StringPiece remaining;
+  absl::string_view match;
+  absl::string_view remaining;
 
   scan.AnySpace().RestartCapture();
   if (scan.Peek() == '-') {
@@ -96,7 +97,7 @@ bool ConsumeAttrNumber(StringPiece* sp, int64_t* out) {
     return false;
   }
   int64_t value = 0;
-  if (!strings::safe_strto64(match, &value)) {
+  if (!absl::SimpleAtoi(match, &value)) {
     return false;
   }
   *out = value;
@@ -113,7 +114,7 @@ bool ConsumeAttrNumber(StringPiece* sp, int64_t* out) {
     }                                                                     \
   } while (false)
 
-bool ConsumeCompoundAttrType(StringPiece* sp, StringPiece* out) {
+bool ConsumeCompoundAttrType(absl::string_view* sp, absl::string_view* out) {
   auto capture_data = sp->data();
   auto capture_begin = sp->begin();
   if (absl::ConsumePrefix(sp, "numbertype") ||
@@ -121,13 +122,14 @@ bool ConsumeCompoundAttrType(StringPiece* sp, StringPiece* out) {
       absl::ConsumePrefix(sp, "quantizedtype") ||
       absl::ConsumePrefix(sp, "realnumbertype") ||
       absl::ConsumePrefix(sp, "realnumberictype")) {
-    *out = StringPiece(capture_data, sp->begin() - capture_begin);
+    *out = absl::string_view(capture_data, sp->begin() - capture_begin);
     return true;
   }
   return false;
 }
 
-bool ProcessCompoundType(const StringPiece type_string, AttrValue* allowed) {
+bool ProcessCompoundType(const absl::string_view type_string,
+                         AttrValue* allowed) {
   if (type_string == "numbertype" || type_string == "numerictype") {
     for (DataType dt : NumberTypes()) {
       allowed->mutable_list()->add_type(dt);
@@ -147,20 +149,20 @@ bool ProcessCompoundType(const StringPiece type_string, AttrValue* allowed) {
   return true;
 }
 
-void FinalizeAttr(StringPiece spec, bool allow_attr_type_any, OpDef* op_def,
-                  std::vector<string>* errors) {
+void FinalizeAttr(absl::string_view spec, bool allow_attr_type_any,
+                  OpDef* op_def, std::vector<string>* errors) {
   OpDef::AttrDef* attr = op_def->add_attr();
-  StringPiece orig(spec);
+  absl::string_view orig(spec);
 
   // Parse "<name>:" at the beginning.
-  StringPiece tmp_name;
+  absl::string_view tmp_name;
   VERIFY(ConsumeAttrName(&spec, &tmp_name), "Trouble parsing '<name>:'");
   attr->set_name(tmp_name.data(), tmp_name.size());
 
   // Read "<type>" or "list(<type>)".
   bool is_list = ConsumeListPrefix(&spec);
   string type;
-  StringPiece type_string;  // Used if type == "type"
+  absl::string_view type_string;  // Used if type == "type"
   if (absl::ConsumePrefix(&spec, "string")) {
     type = "string";
   } else if (absl::ConsumePrefix(&spec, "int")) {
@@ -191,7 +193,7 @@ void FinalizeAttr(StringPiece spec, bool allow_attr_type_any, OpDef* op_def,
     if (absl::StartsWith(spec, "\"") || absl::StartsWith(spec, "'")) {
       type = "string";  // "{ \"foo\", \"bar\" }" or "{ 'foo', 'bar' }"
       while (true) {
-        StringPiece escaped_string;
+        absl::string_view escaped_string;
         VERIFY(ConsumeQuotedString('"', &spec, &escaped_string) ||
                    ConsumeQuotedString('\'', &spec, &escaped_string),
                "Trouble parsing allowed string at '", spec, "'");
@@ -272,12 +274,13 @@ void FinalizeAttr(StringPiece spec, bool allow_attr_type_any, OpDef* op_def,
 
 #undef VERIFY
 
-string InOutError(bool is_output, StringPiece orig, const string& op_name) {
+string InOutError(bool is_output, absl::string_view orig,
+                  const string& op_name) {
   return strings::StrCat(" from ", is_output ? "Output" : "Input", "(\"", orig,
                          "\") for Op ", op_name);
 }
 
-bool ConsumeInOutName(StringPiece* sp, StringPiece* out) {
+bool ConsumeInOutName(absl::string_view* sp, absl::string_view* out) {
   return Scanner(*sp)
       .One(Scanner::LOWERLETTER)
       .Any(Scanner::LOWERLETTER_DIGIT_UNDERSCORE)
@@ -288,7 +291,7 @@ bool ConsumeInOutName(StringPiece* sp, StringPiece* out) {
       .GetResult(sp, out);
 }
 
-bool ConsumeInOutRefOpen(StringPiece* sp) {
+bool ConsumeInOutRefOpen(absl::string_view* sp) {
   return Scanner(*sp)
       .OneLiteral("Ref")
       .AnySpace()
@@ -297,11 +300,11 @@ bool ConsumeInOutRefOpen(StringPiece* sp) {
       .GetResult(sp);
 }
 
-bool ConsumeInOutRefClose(StringPiece* sp) {
+bool ConsumeInOutRefClose(absl::string_view* sp) {
   return Scanner(*sp).OneLiteral(")").AnySpace().GetResult(sp);
 }
 
-bool ConsumeInOutNameOrType(StringPiece* sp, StringPiece* out) {
+bool ConsumeInOutNameOrType(absl::string_view* sp, absl::string_view* out) {
   return Scanner(*sp)
       .One(Scanner::LETTER)
       .Any(Scanner::LETTER_DIGIT_UNDERSCORE)
@@ -310,7 +313,7 @@ bool ConsumeInOutNameOrType(StringPiece* sp, StringPiece* out) {
       .GetResult(sp, out);
 }
 
-bool ConsumeInOutTimesType(StringPiece* sp, StringPiece* out) {
+bool ConsumeInOutTimesType(absl::string_view* sp, absl::string_view* out) {
   return Scanner(*sp)
       .OneLiteral("*")
       .AnySpace()
@@ -322,7 +325,7 @@ bool ConsumeInOutTimesType(StringPiece* sp, StringPiece* out) {
       .GetResult(sp, out);
 }
 
-bool ConsumeControlOutName(StringPiece* sp, StringPiece* out) {
+bool ConsumeControlOutName(absl::string_view* sp, absl::string_view* out) {
   return Scanner(*sp)
       .One(Scanner::LETTER)
       .Any(Scanner::LETTER_DIGIT_UNDERSCORE)
@@ -339,15 +342,15 @@ bool ConsumeControlOutName(StringPiece* sp, StringPiece* out) {
     }                                                                 \
   } while (false)
 
-void FinalizeInputOrOutput(StringPiece spec, bool is_output, OpDef* op_def,
-                           std::vector<string>* errors) {
+void FinalizeInputOrOutput(absl::string_view spec, bool is_output,
+                           OpDef* op_def, std::vector<string>* errors) {
   OpDef::ArgDef* arg =
       is_output ? op_def->add_output_arg() : op_def->add_input_arg();
 
-  StringPiece orig(spec);
+  absl::string_view orig(spec);
 
   // Parse "<name>:" at the beginning.
-  StringPiece tmp_name;
+  absl::string_view tmp_name;
   VERIFY(ConsumeInOutName(&spec, &tmp_name), "Trouble parsing 'name:'");
   arg->set_name(tmp_name.data(), tmp_name.size());
 
@@ -357,7 +360,7 @@ void FinalizeInputOrOutput(StringPiece spec, bool is_output, OpDef* op_def,
   }
 
   {  // Parse "<name|type>" or "<name>*<name|type>".
-    StringPiece first, second, type_or_attr;
+    absl::string_view first, second, type_or_attr;
     VERIFY(ConsumeInOutNameOrType(&spec, &first),
            "Trouble parsing either a type or an attr name at '", spec, "'");
     if (ConsumeInOutTimesType(&spec, &second)) {
@@ -423,17 +426,17 @@ void FinalizeInputOrOutput(StringPiece spec, bool is_output, OpDef* op_def,
 
 #undef VERIFY
 
-string ControlOutError(StringPiece orig, const string& op_name) {
+string ControlOutError(absl::string_view orig, const string& op_name) {
   return strings::StrCat(" from ControlOutput(\"", orig, "\") for Op ",
                          op_name);
 }
 
-void FinalizeControlOutput(StringPiece name, OpDef* op_def,
+void FinalizeControlOutput(absl::string_view name, OpDef* op_def,
                            std::vector<string>* errors) {
-  StringPiece orig(name);
+  absl::string_view orig(name);
 
   // Parse control output name.
-  StringPiece tmp_name;
+  absl::string_view tmp_name;
   if (!ConsumeControlOutName(&orig, &tmp_name)) {
     errors->push_back(strings::StrCat("Trouble parsing 'name:'",
                                       ControlOutError(orig, op_def->name())));
@@ -442,7 +445,7 @@ void FinalizeControlOutput(StringPiece name, OpDef* op_def,
   *op_def->add_control_output() = string(tmp_name.data(), tmp_name.size());
 }
 
-int num_leading_spaces(StringPiece s) {
+int num_leading_spaces(absl::string_view s) {
   size_t i = 0;
   while (i < s.size() && s[i] == ' ') {
     ++i;
@@ -450,7 +453,7 @@ int num_leading_spaces(StringPiece s) {
   return i;
 }
 
-bool ConsumeDocNameColon(StringPiece* sp, StringPiece* out) {
+bool ConsumeDocNameColon(absl::string_view* sp, absl::string_view* out) {
   return Scanner(*sp)
       .One(Scanner::LETTER)
       .Any(Scanner::LETTER_DIGIT_UNDERSCORE)
@@ -461,7 +464,7 @@ bool ConsumeDocNameColon(StringPiece* sp, StringPiece* out) {
       .GetResult(sp, out);
 }
 
-bool IsDocNameColon(StringPiece s) {
+bool IsDocNameColon(absl::string_view s) {
   return ConsumeDocNameColon(&s, nullptr /* out */);
 }
 
@@ -492,14 +495,14 @@ void FinalizeDoc(const string& text, OpDef* op_def,
   // Trim trailing blank lines from the description.
   while (start_l < end_l && lines[end_l - 1].empty()) --end_l;
   string desc = absl::StrJoin(
-      gtl::ArraySlice<string>(lines.data() + start_l, end_l - start_l), "\n");
+      absl::Span<const string>(lines.data() + start_l, end_l - start_l), "\n");
   if (!desc.empty()) op_def->set_description(desc);
 
   // name: description
   //   possibly continued on the next line
   //   if so, we remove the minimum indent
-  StringPiece name;
-  std::vector<StringPiece> description;
+  absl::string_view name;
+  std::vector<absl::string_view> description;
   while (static_cast<size_t>(l) < lines.size()) {
     description.clear();
     description.push_back(lines[l]);
@@ -664,21 +667,21 @@ OpDefBuilder& OpDefBuilder::AllowAttrTypeAny() {
   return *this;
 }
 
-Status OpDefBuilder::Finalize(OpRegistrationData* op_reg_data) const {
+absl::Status OpDefBuilder::Finalize(OpRegistrationData* op_reg_data) const {
   std::vector<string> errors = errors_;
   *op_reg_data = op_reg_data_;
 
   OpDef* op_def = &op_reg_data->op_def;
-  for (StringPiece attr : attrs_) {
+  for (absl::string_view attr : attrs_) {
     FinalizeAttr(attr, allow_attr_type_any_, op_def, &errors);
   }
-  for (StringPiece input : inputs_) {
+  for (absl::string_view input : inputs_) {
     FinalizeInputOrOutput(input, false, op_def, &errors);
   }
-  for (StringPiece output : outputs_) {
+  for (absl::string_view output : outputs_) {
     FinalizeInputOrOutput(output, true, op_def, &errors);
   }
-  for (StringPiece control_output : control_outputs_) {
+  for (absl::string_view control_output : control_outputs_) {
     FinalizeControlOutput(control_output, op_def, &errors);
   }
   FinalizeDoc(doc_, op_def, &errors);
@@ -687,7 +690,7 @@ Status OpDefBuilder::Finalize(OpRegistrationData* op_reg_data) const {
     TF_RETURN_IF_ERROR(op_reg_data->type_ctor(op_def));
   }
 
-  if (errors.empty()) return OkStatus();
+  if (errors.empty()) return absl::OkStatus();
   return errors::InvalidArgument(absl::StrJoin(errors, "\n"));
 }
 
