@@ -16,16 +16,25 @@ limitations under the License.
 #include "tensorflow/core/framework/resource_mgr.h"
 
 #include <atomic>
+#include <memory>
+#include <string>
+#include <utility>
+#include <variant>
+#include <vector>
 
+#include "absl/container/flat_hash_map.h"
 #include "tensorflow/core/framework/device_attributes.pb.h"
 #include "tensorflow/core/framework/node_def.pb.h"
 #include "tensorflow/core/framework/node_def_util.h"
+#include "tensorflow/core/framework/resource_base.h"
 #include "tensorflow/core/lib/core/errors.h"
 #include "tensorflow/core/lib/gtl/map_util.h"
 #include "tensorflow/core/lib/strings/scanner.h"
 #include "tensorflow/core/lib/strings/str_util.h"
 #include "tensorflow/core/lib/strings/stringprintf.h"
 #include "tensorflow/core/platform/demangle.h"
+#include "tensorflow/core/platform/mutex.h"
+#include "tensorflow/core/platform/refcount.h"
 #include "tensorflow/core/platform/stacktrace.h"
 
 namespace tensorflow {
@@ -51,38 +60,40 @@ ResourceHandle MakeResourceHandle(
   return result;
 }
 
-Status MakeResourceHandleToOutput(OpKernelContext* context, int output_index,
-                                  const string& container, const string& name,
-                                  const TypeIndex& type_index) {
+absl::Status MakeResourceHandleToOutput(OpKernelContext* context,
+                                        int output_index,
+                                        const string& container,
+                                        const string& name,
+                                        const TypeIndex& type_index) {
   Tensor* handle;
   TF_RETURN_IF_ERROR(
       context->allocate_output(output_index, TensorShape({}), &handle));
   handle->scalar<ResourceHandle>()() =
       MakeResourceHandle(container, name, *context->device(), type_index);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 namespace internal {
 
-Status ValidateDevice(OpKernelContext* ctx, const ResourceHandle& p) {
+absl::Status ValidateDevice(OpKernelContext* ctx, const ResourceHandle& p) {
   if (ctx->device()->attributes().name() != p.device()) {
     return errors::InvalidArgument(
         "Trying to access resource ", p.name(), " located in device ",
         p.device(), " from device ", ctx->device()->attributes().name());
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 }  // end namespace internal
 
-Status ResourceMgr::InsertDebugTypeName(uint64 hash_code,
-                                        const string& type_name) {
+absl::Status ResourceMgr::InsertDebugTypeName(uint64 hash_code,
+                                              const string& type_name) {
   auto iter = debug_type_names_.emplace(hash_code, type_name);
   if (iter.first->second != type_name) {
     return errors::AlreadyExists("Duplicate hash code found for type ",
                                  type_name);
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 const char* ResourceMgr::DebugTypeName(uint64 hash_code) const {
@@ -97,17 +108,17 @@ const char* ResourceMgr::DebugTypeName(uint64 hash_code) const {
 ResourceMgr::ResourceAndName::ResourceAndName() : name(nullptr) {}
 
 ResourceMgr::ResourceAndName::ResourceAndName(const string& name)
-    : name(absl::make_unique<string>(name)) {}
+    : name(std::make_unique<string>(name)) {}
 
 core::RefCountPtr<ResourceBase> ResourceMgr::ResourceAndName::GetResource()
     const {
-  if (absl::holds_alternative<core::RefCountPtr<ResourceBase>>(resource)) {
+  if (std::holds_alternative<core::RefCountPtr<ResourceBase>>(resource)) {
     ResourceBase* ptr =
-        absl::get<core::RefCountPtr<ResourceBase>>(resource).get();
+        std::get<core::RefCountPtr<ResourceBase>>(resource).get();
     ptr->Ref();
     return core::RefCountPtr<ResourceBase>(ptr);
-  } else if (absl::holds_alternative<core::WeakPtr<ResourceBase>>(resource)) {
-    return absl::get<core::WeakPtr<ResourceBase>>(resource).GetNewRef();
+  } else if (std::holds_alternative<core::WeakPtr<ResourceBase>>(resource)) {
+    return std::get<core::WeakPtr<ResourceBase>>(resource).GetNewRef();
   } else {
     return nullptr;
   }
@@ -142,11 +153,32 @@ void ResourceMgr::Clear() {
   {
     mutex_lock l(mu_);
     tmp_containers = std::move(containers_);
+    containers_.clear();  // reinitialize after move.
   }
   for (const auto& p : tmp_containers) {
     delete p.second;
   }
-  tmp_containers.clear();
+}
+
+void ResourceMgr::Finalize() {
+  const mutex_lock l(mu_);
+  if (finalized_) return;
+  for (const auto& [name, this_container] : containers_) {
+    absl::erase_if(*this_container,
+                   [&](std::pair<const Key, ResourceAndName>& entry) {
+                     ResourceAndName& resource_and_name = entry.second;
+                     const core::RefCountPtr<ResourceBase> resource =
+                         resource_and_name.GetResource();
+                     if (resource == nullptr) {
+                       return true;
+                     }
+
+                     resource->Finalize();
+
+                     return false;
+                   });
+  }
+  finalized_ = true;
 }
 
 string ResourceMgr::DebugString() const {
@@ -180,9 +212,14 @@ string ResourceMgr::DebugString() const {
   return absl::StrJoin(text, "\n");
 }
 
-Status ResourceMgr::DoCreate(const string& container_name, TypeIndex type,
-                             const string& name, ResourceBase* resource,
-                             bool owns_resource) {
+absl::Status ResourceMgr::DoCreate(const string& container_name, TypeIndex type,
+                                   const string& name, ResourceBase* resource,
+                                   bool owns_resource) {
+  if (finalized_) {
+    return absl::FailedPreconditionError(
+        "ResourceMgr is finalized. Cannot create a new resource");
+  }
+
   Container* container = [&]() TF_EXCLUSIVE_LOCKS_REQUIRED(mu_) {
     Container** ptr = &containers_[container_name];
     if (*ptr == nullptr) {
@@ -195,7 +232,7 @@ Status ResourceMgr::DoCreate(const string& container_name, TypeIndex type,
   // key can contain a StringPiece that borrows from the string in the value.
   ResourceAndName resource_and_name(name);
 
-  StringPiece borrowed_name(*resource_and_name.name);
+  absl::string_view borrowed_name(*resource_and_name.name);
 
   if (owns_resource) {
     resource_and_name.resource = core::RefCountPtr<ResourceBase>(resource);
@@ -217,29 +254,30 @@ Status ResourceMgr::DoCreate(const string& container_name, TypeIndex type,
   auto st = container->insert(std::move(key_and_value));
   if (st.second) {
     TF_RETURN_IF_ERROR(InsertDebugTypeName(type.hash_code(), type.name()));
-    return OkStatus();
+    return absl::OkStatus();
   }
   return errors::AlreadyExists("Resource ", container_name, "/", name, "/",
                                type.name());
 }
 
-Status ResourceMgr::Lookup(const ResourceHandle& handle,
-                           ResourceBase** resource) const {
+absl::Status ResourceMgr::Lookup(const ResourceHandle& handle,
+                                 ResourceBase** resource) const {
   tf_shared_lock l(mu_);
   return DoLookup(handle.container(), handle.hash_code(),
                   /*type_name=*/"ResourceBase", handle.name(), resource);
 }
 
-Status ResourceMgr::DoLookup(const string& container, TypeIndex type,
-                             const string& name,
-                             ResourceBase** resource) const {
+absl::Status ResourceMgr::DoLookup(const string& container, TypeIndex type,
+                                   const string& name,
+                                   ResourceBase** resource) const {
   return DoLookup(container, type.hash_code(), type.name(), name, resource);
 }
 
-Status ResourceMgr::DoLookup(const string& container, uint64 type_hash_code,
-                             const string& type_name,
-                             const string& resource_name,
-                             ResourceBase** resource) const {
+absl::Status ResourceMgr::DoLookup(const string& container,
+                                   uint64 type_hash_code,
+                                   const string& type_name,
+                                   const string& resource_name,
+                                   ResourceBase** resource) const {
   const Container* b = gtl::FindPtrOrNull(containers_, container);
   if (b == nullptr) {
     return errors::NotFound("Container ", container,
@@ -257,14 +295,12 @@ Status ResourceMgr::DoLookup(const string& container, uint64 type_hash_code,
                             type_name, " has been destroyed.");
   }
   *resource = ptr;
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status ResourceMgr::PopResourceAndName(const string& container,
-                                       uint64 type_hash_code,
-                                       const string& resource_name,
-                                       const string& type_name,
-                                       ResourceAndName& resource_and_name) {
+absl::Status ResourceMgr::PopResourceAndName(
+    const string& container, uint64 type_hash_code, const string& resource_name,
+    const string& type_name, ResourceAndName& resource_and_name) {
   mutex_lock l(mu_);
   Container* b = gtl::FindPtrOrNull(containers_, container);
   if (b == nullptr) {
@@ -277,17 +313,18 @@ Status ResourceMgr::PopResourceAndName(const string& container,
   }
   std::swap(resource_and_name, iter->second);
   b->erase(iter);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status ResourceMgr::DoDelete(const string& container, uint64 type_hash_code,
-                             const string& resource_name,
-                             const string& type_name) {
+absl::Status ResourceMgr::DoDelete(const string& container,
+                                   uint64 type_hash_code,
+                                   const string& resource_name,
+                                   const string& type_name) {
   ResourceAndName resource_and_name;
   TF_RETURN_IF_ERROR(PopResourceAndName(
       container, type_hash_code, resource_name, type_name, resource_and_name));
 
-  if (absl::holds_alternative<core::WeakPtr<ResourceBase>>(
+  if (std::holds_alternative<core::WeakPtr<ResourceBase>>(
           resource_and_name.resource)) {
     return errors::Internal(
         "Cannot delete an unowned Resource ", container, "/", resource_name,
@@ -295,25 +332,25 @@ Status ResourceMgr::DoDelete(const string& container, uint64 type_hash_code,
         "This indicates ref-counting ResourceHandle is exposed to weak "
         "ResourceHandle code paths.");
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status ResourceMgr::DoDelete(const string& container, TypeIndex type,
-                             const string& resource_name) {
+absl::Status ResourceMgr::DoDelete(const string& container, TypeIndex type,
+                                   const string& resource_name) {
   return DoDelete(container, type.hash_code(), resource_name, type.name());
 }
 
-Status ResourceMgr::Delete(const ResourceHandle& handle) {
+absl::Status ResourceMgr::Delete(const ResourceHandle& handle) {
   return DoDelete(handle.container(), handle.hash_code(), handle.name(),
                   "<unknown>");
 }
 
-Status ResourceMgr::Cleanup(const string& container) {
+absl::Status ResourceMgr::Cleanup(const string& container) {
   {
     tf_shared_lock l(mu_);
     if (!gtl::FindOrNull(containers_, container)) {
       // Nothing to cleanup.
-      return OkStatus();
+      return absl::OkStatus();
     }
   }
   Container* b = nullptr;
@@ -322,17 +359,17 @@ Status ResourceMgr::Cleanup(const string& container) {
     auto iter = containers_.find(container);
     if (iter == containers_.end()) {
       // Nothing to cleanup, it's OK (concurrent cleanup).
-      return OkStatus();
+      return absl::OkStatus();
     }
     b = iter->second;
     containers_.erase(iter);
   }
   CHECK(b != nullptr);
   delete b;
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-static bool IsValidContainerName(StringPiece s) {
+static bool IsValidContainerName(absl::string_view s) {
   using ::tensorflow::strings::Scanner;
   return Scanner(s)
       .One(Scanner::LETTER_DIGIT_DOT)
@@ -341,8 +378,8 @@ static bool IsValidContainerName(StringPiece s) {
       .GetResult();
 }
 
-Status ContainerInfo::Init(ResourceMgr* rmgr, const NodeDef& ndef,
-                           bool use_node_name_as_default) {
+absl::Status ContainerInfo::Init(ResourceMgr* rmgr, const NodeDef& ndef,
+                                 bool use_node_name_as_default) {
   CHECK(rmgr);
   rmgr_ = rmgr;
   string attr_container;
@@ -371,7 +408,7 @@ Status ContainerInfo::Init(ResourceMgr* rmgr, const NodeDef& ndef,
     static std::atomic<int64_t> counter(0);
     name_ = strings::StrCat("_", counter.fetch_add(1), "_", ndef.name());
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 string ContainerInfo::DebugString() const {
@@ -385,42 +422,42 @@ const ResourceHandle& HandleFromInput(OpKernelContext* ctx, int input) {
   return ctx->input(input).flat<ResourceHandle>()(0);
 }
 
-Status HandleFromInput(OpKernelContext* ctx, int input,
-                       ResourceHandle* handle) {
+absl::Status HandleFromInput(OpKernelContext* ctx, int input,
+                             ResourceHandle* handle) {
   TF_ASSIGN_OR_RETURN(const Tensor* tensor, ctx->get_input(input));
   if (tensor->NumElements() == 0) {
     return absl::InvalidArgumentError("Empty resource handle");
   }
   *handle = tensor->flat<ResourceHandle>()(0);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status HandleFromInput(OpKernelContext* ctx, StringPiece input,
-                       ResourceHandle* handle) {
+absl::Status HandleFromInput(OpKernelContext* ctx, absl::string_view input,
+                             ResourceHandle* handle) {
   const Tensor* tensor;
   TF_RETURN_IF_ERROR(ctx->input(input, &tensor));
   if (tensor->NumElements() == 0) {
     return absl::InvalidArgumentError("Empty resource handle");
   }
   *handle = tensor->flat<ResourceHandle>()(0);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status LookupResource(OpKernelContext* ctx, const ResourceHandle& p,
-                      ResourceBase** value) {
+absl::Status LookupResource(OpKernelContext* ctx, const ResourceHandle& p,
+                            ResourceBase** value) {
   TF_RETURN_IF_ERROR(internal::ValidateDevice(ctx, p));
   if (p.IsRefCounting()) {
     TF_ASSIGN_OR_RETURN(*value, p.GetResource<ResourceBase>());
     (*value)->Ref();
-    return OkStatus();
+    return absl::OkStatus();
   }
   return ctx->resource_manager()->Lookup(p, value);
 }
 
-Status DeleteResource(OpKernelContext* ctx, const ResourceHandle& p) {
+absl::Status DeleteResource(OpKernelContext* ctx, const ResourceHandle& p) {
   TF_RETURN_IF_ERROR(internal::ValidateDevice(ctx, p));
   if (p.IsRefCounting()) {
-    return OkStatus();
+    return absl::OkStatus();
   }
   return ctx->resource_manager()->Delete(p);
 }
