@@ -30,13 +30,18 @@ limitations under the License.
 #include "tensorflow/core/framework/tensor.h"
 
 #include <algorithm>
+#include <cstdint>
 #include <cstring>
 #include <memory>
 #include <ostream>
 #include <type_traits>
 #include <utility>
 
+#include "absl/log/check.h"
 #include "absl/strings/escaping.h"
+#include "absl/strings/str_cat.h"
+#include "absl/strings/string_view.h"
+#include "xla/tsl/util/byte_swap_array.h"
 #include "tensorflow/core/framework/allocation_description.pb.h"
 #include "tensorflow/core/framework/log_memory.h"
 #include "tensorflow/core/framework/resource_handle.h"
@@ -56,15 +61,14 @@ limitations under the License.
 #include "tensorflow/core/lib/core/errors.h"
 #include "tensorflow/core/lib/core/status.h"
 #include "tensorflow/core/lib/gtl/inlined_vector.h"
-#include "tensorflow/core/lib/strings/str_util.h"
-#include "tensorflow/core/lib/strings/strcat.h"
 #include "tensorflow/core/platform/errors.h"
 #include "tensorflow/core/platform/logging.h"
 #include "tensorflow/core/platform/macros.h"
+#include "tensorflow/core/platform/numbers.h"
 #include "tensorflow/core/platform/protobuf.h"
 #include "tensorflow/core/platform/tensor_coding.h"
 #include "tensorflow/core/platform/types.h"
-#include "tensorflow/tsl/util/byte_swap_array.h"
+#include "tsl/platform/ml_dtypes.h"
 
 namespace tensorflow {
 
@@ -154,7 +158,8 @@ class Buffer : public BufferBase {
 
   ~Buffer() override;
 
-  TF_DISALLOW_COPY_AND_ASSIGN(Buffer);
+  Buffer(const Buffer&) = delete;
+  void operator=(const Buffer&) = delete;
 };
 
 void LogUnexpectedSize(int64_t actual, int64_t expected) {
@@ -177,8 +182,8 @@ struct Helper {
   template <typename Destination>
   static void Encode(TensorBuffer* in, int64_t n, Destination* out) {
     DCHECK_EQ(in->size(), sizeof(T) * n);
-    port::AssignRefCounted(StringPiece(in->base<const char>(), in->size()), in,
-                           out);
+    port::AssignRefCounted(
+        absl::string_view(in->base<const char>(), in->size()), in, out);
   }
 
   // Decoder of simple type T. Copy the bytes from "in" into the
@@ -349,6 +354,7 @@ struct ProtoHelper {};
       proto->mutable_##N##_val()->Swap(&copy);                         \
     }                                                                  \
   };
+
 PROTO_TRAITS(float, float, float);
 PROTO_TRAITS(double, double, double);
 PROTO_TRAITS(int32, int32, int);
@@ -365,6 +371,35 @@ PROTO_TRAITS(qint16, int32, int);
 PROTO_TRAITS(quint16, int32, int);
 #undef PROTO_TRAITS
 
+template <typename T>
+struct LowBitIntProtoHelper {
+  typedef protobuf::RepeatedField<int> FieldType;
+  static FieldType::const_iterator Begin(const TensorProto& proto) {
+    return proto.int_val().begin();
+  }
+  static size_t NumElements(const TensorProto& proto) {
+    return proto.int_val().size();
+  }
+  static void Fill(const T* data, size_t n, TensorProto* proto) {
+    proto->mutable_int_val()->Reserve(n);
+    for (size_t i = 0; i < n; ++i) {
+      proto->mutable_int_val()->AddAlreadyReserved(static_cast<int>(data[i]));
+    }
+  }
+};
+
+template <>
+struct ProtoHelper<int2> : public LowBitIntProtoHelper<int2> {};
+
+template <>
+struct ProtoHelper<uint2> : public LowBitIntProtoHelper<uint2> {};
+
+template <>
+struct ProtoHelper<int4> : public LowBitIntProtoHelper<int4> {};
+
+template <>
+struct ProtoHelper<uint4> : public LowBitIntProtoHelper<uint4> {};
+
 template <>
 struct ProtoHelper<int64_t> {
   static protobuf::RepeatedField<int64_t>::const_iterator Begin(
@@ -527,6 +562,18 @@ struct ProtoHelper<float8_e5m2> : public Float8ProtoHelper<float8_e5m2> {};
 template <>
 struct ProtoHelper<float8_e4m3fn> : public Float8ProtoHelper<float8_e4m3fn> {};
 
+template <>
+struct ProtoHelper<float8_e4m3fnuz>
+    : public Float8ProtoHelper<float8_e4m3fnuz> {};
+
+template <>
+struct ProtoHelper<float8_e4m3b11fnuz>
+    : public Float8ProtoHelper<float8_e4m3b11fnuz> {};
+
+template <>
+struct ProtoHelper<float8_e5m2fnuz>
+    : public Float8ProtoHelper<float8_e5m2fnuz> {};
+
 template <typename T>
 Buffer<T>::Buffer(Allocator* a, int64_t n)
     : BufferBase(a, TypedAllocator::Allocate<T>(a, n, AllocationAttributes())),
@@ -590,6 +637,54 @@ TensorBuffer* FromProtoField(Allocator* a, const TensorProto& in, int64_t n) {
   return buf;
 }
 
+template <typename T>
+TensorBuffer* Int4OrInt2FromProtoField(Allocator* a, const TensorProto& in,
+                                       int64_t n) {
+  n = std::max<int64_t>(n, 0);
+  Buffer<T>* buf = new Buffer<T>(a, n);
+  int8_t* data = buf->template base<int8_t>();
+  if (data == nullptr) {
+    buf->Unref();
+    return nullptr;
+  }
+  const int64_t in_n = in.int_val().size();
+  auto begin = in.int_val().begin();
+  if (n <= in_n) {
+    std::copy_n(begin, n, data);
+  } else if (in_n > 0) {
+    std::copy_n(begin, in_n, data);
+    const uint16 last = *(data + in_n - 1);
+    std::fill_n(data + in_n, n - in_n, last);
+  } else {
+    std::fill_n(data, n, 0);
+  }
+  return buf;
+}
+
+template <>
+TensorBuffer* FromProtoField<int2>(Allocator* a, const TensorProto& in,
+                                   int64_t n) {
+  return Int4OrInt2FromProtoField<int2>(a, in, n);
+}
+
+template <>
+TensorBuffer* FromProtoField<uint2>(Allocator* a, const TensorProto& in,
+                                    int64_t n) {
+  return Int4OrInt2FromProtoField<uint2>(a, in, n);
+}
+
+template <>
+TensorBuffer* FromProtoField<int4>(Allocator* a, const TensorProto& in,
+                                   int64_t n) {
+  return Int4OrInt2FromProtoField<int4>(a, in, n);
+}
+
+template <>
+TensorBuffer* FromProtoField<uint4>(Allocator* a, const TensorProto& in,
+                                    int64_t n) {
+  return Int4OrInt2FromProtoField<uint4>(a, in, n);
+}
+
 // Separate implementation for `ResourceHandle` to handle the case when the
 // proto for the resource is invalid. See `resource_handle.h` constructor and
 // static factory builder.
@@ -613,8 +708,8 @@ TensorBuffer* FromProtoField<ResourceHandle>(Allocator* a,
     // the remaining elements up to n to be the default ResourceHandle() value.
     const int64_t real_n = n < in_n ? n : in_n;
     for (int64_t i = 0; i < real_n; ++i) {
-      Status s = ResourceHandle::BuildResourceHandle(in.resource_handle_val(i),
-                                                     &data[i]);
+      absl::Status s = ResourceHandle::BuildResourceHandle(
+          in.resource_handle_val(i), &data[i]);
       if (!s.ok()) {
         LOG(ERROR) << "Could not decode resource handle from proto \""
                    << in.resource_handle_val(i).ShortDebugString()
@@ -789,8 +884,8 @@ std::ostream& operator<<(std::ostream& out, const Tensor& tensor) {
   return out;
 }
 
-Status Tensor::BitcastFrom(const Tensor& other, DataType dtype,
-                           const TensorShape& shape) {
+absl::Status Tensor::BitcastFrom(const Tensor& other, DataType dtype,
+                                 const TensorShape& shape) {
   int in_size = DataTypeSize(other.dtype());
   int out_size = DataTypeSize(dtype);
   if (in_size == 0) {
@@ -823,7 +918,7 @@ Status Tensor::BitcastFrom(const Tensor& other, DataType dtype,
       RefIfNonNull(buf_);
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Notice that buf_ either points to a regular TensorBuffer or a SubBuffer.
@@ -878,6 +973,13 @@ int Tensor::RefCount() const {
     CASE(Variant, SINGLE_ARG(STMTS))                           \
     CASE(float8_e5m2, SINGLE_ARG(STMTS))                       \
     CASE(float8_e4m3fn, SINGLE_ARG(STMTS))                     \
+    CASE(float8_e4m3fnuz, SINGLE_ARG(STMTS))                   \
+    CASE(float8_e4m3b11fnuz, SINGLE_ARG(STMTS))                \
+    CASE(float8_e5m2fnuz, SINGLE_ARG(STMTS))                   \
+    CASE(int4, SINGLE_ARG(STMTS))                              \
+    CASE(uint4, SINGLE_ARG(STMTS))                             \
+    CASE(int2, SINGLE_ARG(STMTS))                              \
+    CASE(uint2, SINGLE_ARG(STMTS))                             \
     case DT_INVALID:                                           \
       INVALID;                                                 \
       break;                                                   \
@@ -918,14 +1020,14 @@ Tensor::Tensor(Allocator* a, DataType type, const TensorShape& shape,
   }
 }
 
-Status Tensor::BuildTensor(DataType type, const TensorShape& shape,
-                           Tensor* out_tensor) {
+absl::Status Tensor::BuildTensor(DataType type, const TensorShape& shape,
+                                 Tensor* out_tensor) {
   // Avoid crashes due to invalid or unsupported types.
   CASES_WITH_DEFAULT(
       type, {}, return errors::InvalidArgument("Type not set"),
       return errors::InvalidArgument("Unexpected type: ", DataType_Name(type)));
   *out_tensor = Tensor(type, shape);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // NOTE(mrry): The default allocator for a Tensor (when none is specified) is
@@ -992,7 +1094,8 @@ class SubBuffer : public TensorBuffer {
 
   ~SubBuffer() override { root_->Unref(); }
 
-  TF_DISALLOW_COPY_AND_ASSIGN(SubBuffer);
+  SubBuffer(const SubBuffer&) = delete;
+  void operator=(const SubBuffer&) = delete;
 };
 
 Tensor Tensor::Slice(int64_t start, int64_t limit) const {
@@ -1115,6 +1218,13 @@ size_t Tensor::TotalBytes() const {
   return 0;  // Makes compiler happy.
 }
 
+size_t Tensor::GetBufferSize() const {
+  if (buf_) {
+    return buf_->size();
+  }
+  return 0;
+}
+
 size_t Tensor::AllocatedBytes() const {
   if (buf_) {
     size_t ret;
@@ -1135,42 +1245,57 @@ bool Tensor::CanUseDMA() const {
 
 namespace {
 
-// StrCat and StrAppend don't support Eigen::half directly at the moment, and
-// we would like to keep them compatible with their absl counterparts, for ease
-// of migration. We could rely on errors::internal::PrepareForStrCat() but the
-// logic is so simple we can just replicate it here, where it is close to its
-// usage and easy to change later. And there's the extra benefit of not
-// accessing an 'internal' namespace.
-inline const strings::AlphaNum& PrintOneElement(const strings::AlphaNum& a,
-                                                bool print_v2) {
-  return a;
+// absl::StrCat and absl::StrAppend don't support Eigen::half, bfloat16...
+template <typename T>
+const T& PrintOneElement(const T& value, bool print_v2) {
+  return value;
 }
-inline string PrintOneElement(const tstring& a, bool print_v2) {
+string PrintOneElement(const tstring& a, bool print_v2) {
   if (print_v2) {
     return "\"" + absl::Utf8SafeCEscape(a) + "\"";
   } else {
     return absl::Utf8SafeCEscape(a);
   }
 }
-inline float PrintOneElement(const Eigen::half& h, bool print_v2) {
+float PrintOneElement(const Eigen::half& h, bool print_v2) {
   return static_cast<float>(h);
 }
 
-inline float PrintOneElement(bfloat16 f, bool print_v2) {
+float PrintOneElement(bfloat16 f, bool print_v2) {
   return static_cast<float>(f);
 }
 
-inline float PrintOneElement(float8_e5m2 f, bool print_v2) {
+float PrintOneElement(float8_e5m2 f, bool print_v2) {
   return static_cast<float>(f);
 }
 
-inline float PrintOneElement(float8_e4m3fn f, bool print_v2) {
+float PrintOneElement(float8_e4m3fn f, bool print_v2) {
   return static_cast<float>(f);
 }
 
+float PrintOneElement(float8_e4m3b11fnuz f, bool print_v2) {
+  return static_cast<float>(f);
+}
+
+int16_t PrintOneElement(int4 a, bool print_v2) {
+  return static_cast<int16_t>(a);
+}
+
+uint16_t PrintOneElement(uint4 a, bool print_v2) {
+  return static_cast<uint16_t>(a);
+}
+
+int16_t PrintOneElement(int2 a, bool print_v2) {
+  return static_cast<int16_t>(a);
+}
+
+uint16_t PrintOneElement(uint2 a, bool print_v2) {
+  return static_cast<uint16_t>(a);
+}
+
 // Print from left dim to right dim recursively.
 template <typename T>
-void PrintOneDim(int dim_index, const gtl::InlinedVector<int64, 4>& shape,
+void PrintOneDim(int dim_index, const absl::InlinedVector<int64, 4UL>& shape,
                  int64_t limit, int shape_size, const T* data,
                  int64_t* data_index, string* result) {
   if (*data_index >= limit) return;
@@ -1181,12 +1306,13 @@ void PrintOneDim(int dim_index, const gtl::InlinedVector<int64, 4>& shape,
       if (*data_index >= limit) {
         // If not enough elements has been printed, append "...".
         if (dim_index != 0) {
-          strings::StrAppend(result, "...");
+          absl::StrAppend(result, "...");
         }
         return;
       }
-      if (i > 0) strings::StrAppend(result, " ");
-      strings::StrAppend(result, PrintOneElement(data[(*data_index)++], false));
+      if (i > 0) absl::StrAppend(result, " ");
+      absl::StrAppend(result, strings::LegacyPrecision(PrintOneElement(
+                                  data[(*data_index)++], false)));
     }
     return;
   }
@@ -1194,14 +1320,14 @@ void PrintOneDim(int dim_index, const gtl::InlinedVector<int64, 4>& shape,
   for (int64_t i = 0; i < element_count; i++) {
     bool flag = false;
     if (*data_index < limit) {
-      strings::StrAppend(result, "[");
+      absl::StrAppend(result, "[");
       flag = true;
     }
     // As for each element, print the sub-dim.
     PrintOneDim(dim_index + 1, shape, limit, shape_size, data, data_index,
                 result);
     if (*data_index < limit || flag) {
-      strings::StrAppend(result, "]");
+      absl::StrAppend(result, "]");
       flag = false;
     }
   }
@@ -1210,30 +1336,31 @@ void PrintOneDim(int dim_index, const gtl::InlinedVector<int64, 4>& shape,
 // Appends the spacing between elements for a given dim onto a result string
 void PrintDimSpacing(int dim_index, int num_dims, string* result) {
   if (dim_index == num_dims - 1) {
-    strings::StrAppend(result, " ");
+    absl::StrAppend(result, " ");
     return;
   }
   for (int j = 0; j < num_dims - dim_index - 1; j++) {
-    strings::StrAppend(result, "\n");
+    absl::StrAppend(result, "\n");
   }
   for (int j = 0; j <= dim_index; j++) {
-    strings::StrAppend(result, " ");
+    absl::StrAppend(result, " ");
   }
 }
 
 // Print from left dim to right dim recursively.
 template <typename T>
-void PrintOneDimV2(int dim_index, const gtl::InlinedVector<int64, 4>& shape,
+void PrintOneDimV2(int dim_index, const absl::InlinedVector<int64, 4UL>& shape,
                    int64_t num_elts_at_ends, int num_dims, const T* data,
                    int64_t data_index, string* result) {
   // We have recursed beyond all the dimensions into a single element
   // of the tensor.
   if (dim_index == num_dims) {
-    strings::StrAppend(result, PrintOneElement(data[data_index], true));
+    absl::StrAppend(result, strings::LegacyPrecision(
+                                PrintOneElement(data[data_index], true)));
     return;
   }
 
-  strings::StrAppend(result, "[");
+  absl::StrAppend(result, "[");
   int64_t element_count = shape[dim_index];
   int64_t start_of_end =
       std::max(num_elts_at_ends, element_count - num_elts_at_ends);
@@ -1254,7 +1381,7 @@ void PrintOneDimV2(int dim_index, const gtl::InlinedVector<int64, 4>& shape,
   }
   if (element_count > 2 * num_elts_at_ends) {
     PrintDimSpacing(dim_index, num_dims, result);
-    strings::StrAppend(result, "...");
+    absl::StrAppend(result, "...");
   }
   for (int64_t i = start_of_end; i < element_count; i++) {
     // As for each element, print the sub-dim.
@@ -1263,7 +1390,7 @@ void PrintOneDimV2(int dim_index, const gtl::InlinedVector<int64, 4>& shape,
                   data_index + elements_per_iter * i, result);
   }
 
-  strings::StrAppend(result, "]");
+  absl::StrAppend(result, "]");
 }
 
 template <typename T>
@@ -1271,13 +1398,14 @@ string SummarizeArrayInternal(int64_t limit, int64_t num_elts,
                               const TensorShape& tensor_shape, const T* array,
                               const bool print_v2) {
   string ret;
-  const gtl::InlinedVector<int64_t, 4> shape = tensor_shape.dim_sizes();
+  const absl::InlinedVector<int64_t, 4UL> shape = tensor_shape.dim_sizes();
   if (shape.empty()) {
     for (int64_t i = 0; i < limit; ++i) {
-      if (i > 0) strings::StrAppend(&ret, " ");
-      strings::StrAppend(&ret, PrintOneElement(array[i], print_v2));
+      if (i > 0) absl::StrAppend(&ret, " ");
+      absl::StrAppend(
+          &ret, strings::LegacyPrecision(PrintOneElement(array[i], print_v2)));
     }
-    if (num_elts > limit) strings::StrAppend(&ret, "...");
+    if (num_elts > limit) absl::StrAppend(&ret, "...");
     return ret;
   }
   if (print_v2) {
@@ -1288,7 +1416,7 @@ string SummarizeArrayInternal(int64_t limit, int64_t num_elts,
     const int shape_size = tensor_shape.dims();
     PrintOneDim(0, shape, limit, shape_size, array, &data_index, &ret);
 
-    if (num_elts > limit) strings::StrAppend(&ret, "...");
+    if (num_elts > limit) absl::StrAppend(&ret, "...");
   }
 
   return ret;
@@ -1308,7 +1436,7 @@ string SummarizeArray<bool>(int64_t limit, int64_t num_elts,
                             const TensorShape& tensor_shape, const char* data,
                             const bool print_v2) {
   if (data == nullptr) {
-    return strings::StrCat("");  // we already print type and shape
+    return "";  // we already print type and shape
   }
   // We first convert all chars to be 0/1 to not get InvalidEnumValue sanitizer
   // error
@@ -1328,10 +1456,10 @@ string Tensor::SummarizeValue(int64_t max_entries, bool print_v2) const {
   }
   size_t limit = std::min(max_entries, num_elts);
   if ((limit > 0) && (buf_ == nullptr)) {
-    return strings::StrCat("uninitialized Tensor of ", num_elts,
-                           " elements of type ", dtype());
+    return absl::StrCat("uninitialized Tensor of ", num_elts,
+                        " elements of type ", dtype());
   }
-  const char* data = limit > 0 ? tensor_data().data() : nullptr;
+  const char* data = limit > 0 ? (const char*)this->data() : nullptr;
   switch (dtype()) {
     case DT_BFLOAT16:
       return SummarizeArray<bfloat16>(limit, num_elts, shape_, data, print_v2);
@@ -1346,6 +1474,9 @@ string Tensor::SummarizeValue(int64_t max_entries, bool print_v2) const {
     case DT_FLOAT8_E4M3FN:
       return SummarizeArray<float8_e4m3fn>(limit, num_elts, shape_, data,
                                            print_v2);
+    case DT_FLOAT8_E4M3B11FNUZ:
+      return SummarizeArray<float8_e4m3b11fnuz>(limit, num_elts, shape_, data,
+                                                print_v2);
     case DT_FLOAT:
       return SummarizeArray<float>(limit, num_elts, shape_, data, print_v2);
       break;
@@ -1388,43 +1519,56 @@ string Tensor::SummarizeValue(int64_t max_entries, bool print_v2) const {
     case DT_STRING:
       return SummarizeArray<tstring>(limit, num_elts, shape_, data, print_v2);
       break;
+    case DT_INT4:
+      return SummarizeArray<int4>(limit, num_elts, shape_, data, print_v2);
+    case DT_UINT4:
+      return SummarizeArray<uint4>(limit, num_elts, shape_, data, print_v2);
+    case DT_INT2:
+      return SummarizeArray<int2>(limit, num_elts, shape_, data, print_v2);
+    case DT_UINT2:
+      return SummarizeArray<uint2>(limit, num_elts, shape_, data, print_v2);
     default: {
       // All irregular cases
       string ret;
       if (print_v2 && (dims() > 0)) {
-        strings::StrAppend(&ret, "[");
+        absl::StrAppend(&ret, "[");
       }
       // TODO(irving): Don't call flat every time around this
       // loop.
       for (size_t i = 0; i < limit; ++i) {
-        if (i > 0) strings::StrAppend(&ret, " ");
+        if (i > 0) absl::StrAppend(&ret, " ");
         switch (dtype()) {
           case DT_VARIANT: {
             const Variant& v = flat<Variant>()(i);
-            strings::StrAppend(&ret, "<", v.SummarizeValue(), ">");
+            absl::StrAppend(&ret, "<", v.SummarizeValue(), ">");
           } break;
           case DT_RESOURCE: {
             const ResourceHandle& r = flat<ResourceHandle>()(i);
-            strings::StrAppend(&ret, "<", r.SummarizeValue(), ">");
+            absl::StrAppend(&ret, "<", r.SummarizeValue(), ">");
           } break;
           default:
             // TODO(zhifengc, josh11b): Pretty-print other types (bool,
             // complex64, quantized).
-            strings::StrAppend(&ret, "?");
+            absl::StrAppend(&ret, "?");
         }
       }
-      if (max_entries < num_elts) strings::StrAppend(&ret, "...");
+      if (max_entries < num_elts) absl::StrAppend(&ret, "...");
       if (print_v2 && (dims() > 0)) {
-        strings::StrAppend(&ret, "]");
+        absl::StrAppend(&ret, "]");
       }
       return ret;
     }
   }
 }
 
-StringPiece Tensor::tensor_data() const {
-  if (buf_ == nullptr) return StringPiece();  // Don't die for empty tensors
-  return StringPiece(static_cast<char*>(buf_->data()), TotalBytes());
+absl::string_view Tensor::tensor_data_internal() const {
+  return absl::string_view(static_cast<char*>(buf_->data()), GetBufferSize());
+}
+
+absl::string_view Tensor::tensor_data() const {
+  if (buf_ == nullptr) return absl::string_view();
+  CHECK(DataTypeCanUseMemcpy(dtype()));  // Crash OK
+  return tensor_data_internal();
 }
 
 void* Tensor::data() const {
@@ -1438,14 +1582,14 @@ bool Tensor::SharesBufferWith(const Tensor& b) const {
 }
 
 string Tensor::DebugString(int num_values) const {
-  return strings::StrCat("Tensor<type: ", DataTypeString(dtype()),
-                         " shape: ", shape().DebugString(),
-                         " values: ", SummarizeValue(num_values), ">");
+  return absl::StrCat("Tensor<type: ", DataTypeString(dtype()),
+                      " shape: ", shape().DebugString(),
+                      " values: ", SummarizeValue(num_values), ">");
 }
 
 string Tensor::DeviceSafeDebugString() const {
-  return strings::StrCat("Tensor<type: ", DataTypeString(dtype()),
-                         " shape: ", shape().DebugString(), ">");
+  return absl::StrCat("Tensor<type: ", DataTypeString(dtype()),
+                      " shape: ", shape().DebugString(), ">");
 }
 
 void Tensor::FillDescription(TensorDescription* description) const {
@@ -1457,9 +1601,9 @@ void Tensor::FillDescription(TensorDescription* description) const {
   }
 }
 
-gtl::InlinedVector<int64_t, 4> Tensor::ComputeFlatInnerDims(
-    gtl::ArraySlice<int64_t> orig, int64_t num_out_dims) {
-  gtl::InlinedVector<int64_t, 4> out_dims(num_out_dims, 0);
+absl::InlinedVector<int64_t, 4UL> Tensor::ComputeFlatInnerDims(
+    absl::Span<const int64_t> orig, int64_t num_out_dims) {
+  absl::InlinedVector<int64_t, 4UL> out_dims(num_out_dims, 0);
   int64_t offset = orig.size() - num_out_dims;
   for (int64_t out_dim = num_out_dims - 1; out_dim >= 0; --out_dim) {
     const int64_t in_dim = out_dim + offset;
@@ -1471,9 +1615,9 @@ gtl::InlinedVector<int64_t, 4> Tensor::ComputeFlatInnerDims(
   return out_dims;
 }
 
-gtl::InlinedVector<int64_t, 4> Tensor::ComputeFlatOuterDims(
-    gtl::ArraySlice<int64_t> orig, int64_t num_out_dims) {
-  gtl::InlinedVector<int64_t, 4> out_dims(num_out_dims, 0);
+absl::InlinedVector<int64_t, 4UL> Tensor::ComputeFlatOuterDims(
+    absl::Span<const int64_t> orig, int64_t num_out_dims) {
+  absl::InlinedVector<int64_t, 4UL> out_dims(num_out_dims, 0);
   for (int64_t out_dim = 0; out_dim <= num_out_dims - 1; ++out_dim) {
     out_dims[out_dim] = out_dim >= orig.size() ? 1 : orig[out_dim];
   }
