@@ -43,7 +43,8 @@ namespace subgraph {
 
 namespace {
 
-typedef std::unordered_map<StringPiece, Node*, StringPieceHasher> NameIndex;
+typedef std::unordered_map<absl::string_view, Node*, StringPieceHasher>
+    NameIndex;
 
 // Rewrite graph by replacing the output tensors specified in
 // "fed_outputs" with special feed nodes for each specified output
@@ -54,7 +55,7 @@ typedef std::unordered_map<StringPiece, Node*, StringPieceHasher> NameIndex;
 // Return true on success.  On error, return false and sets *error to
 // an appropriate error message (and *g is left in an indeterminate
 // state).
-Status FeedInputs(
+absl::Status FeedInputs(
     Graph* g, const std::vector<std::unique_ptr<PruneRewrite>>& feed_rewrites,
     NameIndex* name_index, DataTypeVector* out_feed_types) {
   out_feed_types->clear();
@@ -116,10 +117,10 @@ Status FeedInputs(
     }
     out_feed_types->push_back(BaseType(n->output_type(id.second)));
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status FetchOutputs(
+absl::Status FetchOutputs(
     Graph* g, const std::vector<std::unique_ptr<PruneRewrite>>& fetch_rewrites,
     NameIndex* name_index, std::vector<Node*>* out_fetch_nodes,
     DataTypeVector* out_fetch_types) {
@@ -170,7 +171,7 @@ Status FetchOutputs(
     out_fetch_types->push_back(BaseType(n->output_type(id.second)));
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 bool AddNodeToTargets(const string& node_or_tensor_name,
@@ -187,9 +188,9 @@ bool AddNodeToTargets(const string& node_or_tensor_name,
   return true;
 }
 
-Status PruneForTargets(Graph* g, const NameIndex& name_index,
-                       const std::vector<Node*>& fetch_nodes,
-                       const gtl::ArraySlice<string>& target_nodes) {
+absl::Status PruneForTargets(Graph* g, const NameIndex& name_index,
+                             const std::vector<Node*>& fetch_nodes,
+                             const absl::Span<const string>& target_nodes) {
   string not_found;
   std::unordered_set<const Node*> targets;
   for (Node* n : fetch_nodes) {
@@ -211,13 +212,13 @@ Status PruneForTargets(Graph* g, const NameIndex& name_index,
   // Reconnect nodes with no outgoing edges to the sink node
   FixupSourceAndSinkEdges(g);
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace
 
-Status ArgFeedRewrite::AddNode(Graph* g, NodeBuilder::NodeOut feed_tensor,
-                               Node** out_node) {
+absl::Status ArgFeedRewrite::AddNode(Graph* g, NodeBuilder::NodeOut feed_tensor,
+                                     Node** out_node) {
   // NOTE(mrry): We must include the index as part of the node
   // name, because _Arg is a "stateful" kernel and therefore
   // its name must uniquely identify a kernel instance across all
@@ -230,11 +231,12 @@ Status ArgFeedRewrite::AddNode(Graph* g, NodeBuilder::NodeOut feed_tensor,
           .Attr("index", arg_index_)
           .Finalize(g, out_node, /*consume=*/true));
   (*out_node)->set_assigned_device_name(device_info().name());
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status RecvFeedRewrite::AddNode(Graph* g, NodeBuilder::NodeOut feed_tensor,
-                                Node** out_node) {
+absl::Status RecvFeedRewrite::AddNode(Graph* g,
+                                      NodeBuilder::NodeOut feed_tensor,
+                                      Node** out_node) {
   TF_RETURN_IF_ERROR(
       NodeBuilder(strings::StrCat("_recv_", feed_tensor.node->name(), "_",
                                   feed_tensor.index),
@@ -250,11 +252,12 @@ Status RecvFeedRewrite::AddNode(Graph* g, NodeBuilder::NodeOut feed_tensor,
           .Finalize(g, out_node, /*consume=*/true));
 
   (*out_node)->set_assigned_device_name(device_info().name());
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status RetvalFetchRewrite::AddNode(Graph* g, NodeBuilder::NodeOut fetch_tensor,
-                                   Node** out_node) {
+absl::Status RetvalFetchRewrite::AddNode(Graph* g,
+                                         NodeBuilder::NodeOut fetch_tensor,
+                                         Node** out_node) {
   // NOTE(mrry): We must include the index as part of the node
   // name, because _Retval is a "stateful" kernel and therefore
   // its name must uniquely identify a kernel instance across all
@@ -269,11 +272,12 @@ Status RetvalFetchRewrite::AddNode(Graph* g, NodeBuilder::NodeOut fetch_tensor,
           .Attr("index", retval_index_)
           .Finalize(g, out_node, /*consume=*/true));
   (*out_node)->set_assigned_device_name(device_info().name());
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status SendFetchRewrite::AddNode(Graph* g, NodeBuilder::NodeOut fetch_tensor,
-                                 Node** out_node) {
+absl::Status SendFetchRewrite::AddNode(Graph* g,
+                                       NodeBuilder::NodeOut fetch_tensor,
+                                       Node** out_node) {
   TF_RETURN_IF_ERROR(
       NodeBuilder(strings::StrCat("_send_", fetch_tensor.node->name(), "_",
                                   fetch_tensor.index),
@@ -287,13 +291,13 @@ Status SendFetchRewrite::AddNode(Graph* g, NodeBuilder::NodeOut fetch_tensor,
           .Attr("client_terminated", true)
           .Finalize(g, out_node, /*consume=*/true));
   (*out_node)->set_assigned_device_name(device_info().name());
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status RewriteGraphForExecution(
-    Graph* g, const gtl::ArraySlice<string>& fed_outputs,
-    const gtl::ArraySlice<string>& fetch_outputs,
-    const gtl::ArraySlice<string>& target_node_names,
+absl::Status RewriteGraphForExecution(
+    Graph* g, const absl::Span<const string>& fed_outputs,
+    const absl::Span<const string>& fetch_outputs,
+    const absl::Span<const string>& target_node_names,
     const DeviceAttributes& device_info, bool use_function_convention,
     RewriteGraphMetadata* out_metadata) {
   std::vector<std::unique_ptr<PruneRewrite>> feed_rewrites;
@@ -335,10 +339,10 @@ std::vector<string> ConvertToVector(StringContainer field) {
 }
 }  // namespace
 
-Status RewriteGraphForExecution(
+absl::Status RewriteGraphForExecution(
     Graph* g, const std::vector<std::unique_ptr<PruneRewrite>>& feed_rewrites,
     const std::vector<std::unique_ptr<PruneRewrite>>& fetch_rewrites,
-    const gtl::ArraySlice<string>& target_node_names,
+    const absl::Span<const string>& target_node_names,
     RewriteGraphMetadata* out_metadata) {
   if (fetch_rewrites.empty() && target_node_names.empty()) {
     return errors::InvalidArgument(
@@ -392,7 +396,7 @@ Status RewriteGraphForExecution(
         PruneForTargets(g, name_index, fetch_nodes, target_node_names));
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace subgraph
