@@ -16,15 +16,28 @@ limitations under the License.
 #include "tensorflow/core/grappler/clusters/single_machine.h"
 
 #include <atomic>
+#include <cstddef>
+#include <cstdint>
 #include <memory>
+#include <unordered_map>
+#include <unordered_set>
+#include <utility>
+#include <vector>
 
+#include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/status/status.h"
 #include "absl/strings/str_cat.h"
+#include "absl/synchronization/notification.h"
+#include "absl/time/time.h"
+#include "absl/types/optional.h"
 #include "tensorflow/cc/training/queue_runner.h"
 #include "tensorflow/core/common_runtime/device.h"
 #include "tensorflow/core/common_runtime/device_mgr.h"
 #include "tensorflow/core/common_runtime/gpu/gpu_id.h"
 #include "tensorflow/core/common_runtime/gpu/gpu_id_manager.h"
+#include "tensorflow/core/framework/cost_graph.pb.h"
+#include "tensorflow/core/framework/graph.pb.h"
 #include "tensorflow/core/grappler/clusters/utils.h"
 #include "tensorflow/core/grappler/utils.h"
 #include "tensorflow/core/kernels/ops_util.h"
@@ -33,6 +46,8 @@ limitations under the License.
 #include "tensorflow/core/platform/mutex.h"
 #include "tensorflow/core/platform/notification.h"
 #include "tensorflow/core/platform/types.h"
+#include "tensorflow/core/protobuf/config.pb.h"
+#include "tensorflow/core/protobuf/device_properties.pb.h"
 #include "tensorflow/core/public/session.h"
 
 namespace tensorflow {
@@ -70,7 +85,7 @@ SingleMachine::~SingleMachine() {
   thread_pool_.reset();
 }
 
-Status SingleMachine::Provision() {
+absl::Status SingleMachine::Provision() {
   // This is really ugly: to avoid leaking variables, we need to reset the tf
   // session every time we're done processing a grappler item. However,
   // variables are global, and therefore we can't have more than 1 session alive
@@ -96,7 +111,7 @@ Status SingleMachine::Provision() {
       }
       TfDeviceId tf_device_id(parsed.id);
       PlatformDeviceId platform_device_id;
-      Status s =
+      absl::Status s =
           GpuIdManager::TfToPlatformDeviceId(tf_device_id, &platform_device_id);
       if (!s.ok()) {
         return absl::UnavailableError(
@@ -120,10 +135,10 @@ Status SingleMachine::Provision() {
   if (cpu_allocator_stats_enabled_) {
     TF_RETURN_IF_ERROR(ClearAllocatorStats());
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status SingleMachine::Initialize(const GrapplerItem& item) {
+absl::Status SingleMachine::Initialize(const GrapplerItem& item) {
   mutex_lock l(this->last_graph_mu_);
   if (last_graph_ != &item.graph || last_graph_id_ != item.id) {
     init_ops_ = item.init_ops;
@@ -132,23 +147,23 @@ Status SingleMachine::Initialize(const GrapplerItem& item) {
     queue_runner_defs_ = item.queue_runners;
     last_graph_id_ = item.id;
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status SingleMachine::Shutdown() {
+absl::Status SingleMachine::Shutdown() {
   TF_RETURN_IF_ERROR(ShutdownSession());
 
   mutex_lock l(this->last_graph_mu_);
   last_graph_ = nullptr;
   already_provisioned = false;
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status SingleMachine::Run(const GraphDef& graph_def,
-                          const std::vector<std::pair<string, Tensor>>& feed,
-                          const std::vector<string>& fetch,
-                          RunMetadata* metadata) {
+absl::Status SingleMachine::Run(
+    const GraphDef& graph_def,
+    const std::vector<std::pair<string, Tensor>>& feed,
+    const std::vector<string>& fetch, RunMetadata* metadata) {
   mutex_lock l(this->last_graph_mu_);
   if (last_graph_ != &graph_def) {
     TF_RETURN_IF_ERROR(ResetSession());
@@ -203,23 +218,23 @@ Status SingleMachine::Run(const GraphDef& graph_def,
 
   last_graph_ = &graph_def;
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status SingleMachine::EnablePeakMemoryStats() {
+absl::Status SingleMachine::EnablePeakMemoryStats() {
   EnableCPUAllocatorStats();
   cpu_allocator_stats_enabled_ = true;
   // No need to enable GPU allocator stats since its stats are always collected.
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status SingleMachine::GetPeakMemoryUsage(
+absl::Status SingleMachine::GetPeakMemoryUsage(
     std::unordered_map<string, uint64>* device_peak_memory) const {
   // Cpu_allocator->TracksAllocationSizes() returns true doesn't always mean the
   // the AllocatorStats would be collected.
   if (!cpu_allocator_stats_enabled_) {
-    return Status(absl::StatusCode::kInvalidArgument,
-                  "Tracking allocation for CPU is not enabled.");
+    return absl::Status(absl::StatusCode::kInvalidArgument,
+                        "Tracking allocation for CPU is not enabled.");
   }
 
   const DeviceMgr* device_mgr;
@@ -230,24 +245,24 @@ Status SingleMachine::GetPeakMemoryUsage(
   for (Device* device : devices) {
     auto* allocator = device->GetAllocator(AllocatorAttributes());
     if (!allocator->TracksAllocationSizes()) {
-      return Status(absl::StatusCode::kInvalidArgument,
-                    "Tracking allocation is not enabled.");
+      return absl::Status(absl::StatusCode::kInvalidArgument,
+                          "Tracking allocation is not enabled.");
     }
     absl::optional<AllocatorStats> stats = allocator->GetStats();
     (*device_peak_memory)[device->name()] =
         (stats ? stats->peak_bytes_in_use : 0);
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status SingleMachine::RunWithTimeout(
+absl::Status SingleMachine::RunWithTimeout(
     const std::vector<std::pair<string, Tensor>>& feed,
     const std::vector<string>& fetch, RunMetadata* run_metadata) {
   return RunWithTimeout(feed, fetch, run_metadata, timeout_s_);
 }
 
-Status SingleMachine::RunWithTimeout(
+absl::Status SingleMachine::RunWithTimeout(
     const std::vector<std::pair<string, Tensor>>& feed,
     const std::vector<string>& fetch, RunMetadata* run_metadata,
     int64_t timeout_s) {
@@ -257,7 +272,7 @@ Status SingleMachine::RunWithTimeout(
     CHECK(!closing_);
   }
 
-  auto status = std::make_shared<Status>();
+  auto status = std::make_shared<absl::Status>();
   auto local_metadata = std::make_shared<RunMetadata>();
   const bool executed_in_time = ExecuteWithTimeout(
       [this, status, local_metadata, feed, fetch]() {
@@ -274,9 +289,9 @@ Status SingleMachine::RunWithTimeout(
   return *status;
 }
 
-Status SingleMachine::CloseSession(bool use_timeout) {
+absl::Status SingleMachine::CloseSession(bool use_timeout) {
   if (!session_ || !thread_pool_) {
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   {
@@ -317,10 +332,10 @@ Status SingleMachine::CloseSession(bool use_timeout) {
                      " seconds, aborting"));
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status SingleMachine::ShutdownSession() {
+absl::Status SingleMachine::ShutdownSession() {
   TF_RETURN_IF_ERROR(CloseSession(true /*use_timeout*/));
 
   // Delete the threadpool: this ensures that all the pending closures complete
@@ -329,13 +344,13 @@ Status SingleMachine::ShutdownSession() {
   // therefore we need to delete the threadpool with the background thread.
   // That thread itself will also never complete, so the user should
   // abort the process to avoid leaking too many resources.
-  auto n = std::make_shared<Notification>();
+  auto n = std::make_shared<absl::Notification>();
   Env::Default()->SchedClosure([this, n]() {
     thread_pool_.reset();
     n->Notify();
   });
-  int64_t timeout_us = 1000000ll * timeout_s_;
-  const bool notified = WaitForNotificationWithTimeout(n.get(), timeout_us);
+  const bool notified =
+      n->WaitForNotificationWithTimeout(absl::Seconds(timeout_s_));
   if (!notified) {
     // Let the caller know that we can't shutdown the session properly since
     // there are calls to Session::Run() still running.
@@ -343,10 +358,10 @@ Status SingleMachine::ShutdownSession() {
         "The session is still running graphs after ", timeout_s_, " seconds"));
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status SingleMachine::ResetSession() {
+absl::Status SingleMachine::ResetSession() {
   if (session_) {
     LOG(INFO) << "Cleaning up previous session";
 
@@ -379,7 +394,7 @@ Status SingleMachine::ResetSession() {
     // We currently don't care about the client device.
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 void SingleMachine::MergeCosts(CostGraphDef* graph_costs,
@@ -444,12 +459,12 @@ void SingleMachine::MergeCosts(CostGraphDef* graph_costs,
   }
 }
 
-Status SingleMachine::ClearAllocatorStats() const {
+absl::Status SingleMachine::ClearAllocatorStats() const {
   // Cpu_allocator->TracksAllocationSizes() returns true doesn't always mean the
   // the AllocatorStats would be collected.
   if (!cpu_allocator_stats_enabled_) {
-    return Status(absl::StatusCode::kInvalidArgument,
-                  "Tracking allocation for CPU is not enabled.");
+    return absl::Status(absl::StatusCode::kInvalidArgument,
+                        "Tracking allocation for CPU is not enabled.");
   }
 
   const DeviceMgr* device_mgr;
@@ -459,17 +474,17 @@ Status SingleMachine::ClearAllocatorStats() const {
   for (Device* device : devices) {
     auto* allocator = device->GetAllocator(AllocatorAttributes());
     if (!allocator->TracksAllocationSizes()) {
-      return Status(absl::StatusCode::kInvalidArgument,
-                    "Tracking allocation is not enabled.");
+      return absl::Status(absl::StatusCode::kInvalidArgument,
+                          "Tracking allocation is not enabled.");
     }
     if (!allocator->ClearStats()) {
-      return Status(
+      return absl::Status(
           absl::StatusCode::kInvalidArgument,
           absl::StrCat("Clearing allocation stats is not supported for ",
                        device->name()));
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace grappler
