@@ -479,7 +479,8 @@ TEST_F(SigNodeTest, EqualsLinkSize) {
   (*graph1.add_node()) = MakeNodeMul("node2", "node1", "node1");
 
   GenNodeMap gen_map1;
-  ASSERT_THAT(GenNode::BuildGraphInMap(graph1, &gen_map1), Eq(OkStatus()));
+  ASSERT_THAT(GenNode::BuildGraphInMap(graph1, &gen_map1),
+              Eq(absl::OkStatus()));
 
   Subgraph::Identity id1;
   id1.insert(gen_map1["node1"].get());
@@ -497,7 +498,8 @@ TEST_F(SigNodeTest, EqualsLinkSize) {
   node22->add_input("node2");
 
   GenNodeMap gen_map2;
-  ASSERT_THAT(GenNode::BuildGraphInMap(graph2, &gen_map2), Eq(OkStatus()));
+  ASSERT_THAT(GenNode::BuildGraphInMap(graph2, &gen_map2),
+              Eq(absl::OkStatus()));
 
   Subgraph::Identity id2;
   id2.insert(gen_map2["node1"].get());
@@ -519,7 +521,8 @@ TEST_F(SigNodeTest, EqualsLinks) {
   (*graph1.add_node()) = MakeNodeMul("node2", "node1", "node1");
 
   GenNodeMap gen_map1;
-  ASSERT_THAT(GenNode::BuildGraphInMap(graph1, &gen_map1), Eq(OkStatus()));
+  ASSERT_THAT(GenNode::BuildGraphInMap(graph1, &gen_map1),
+              Eq(absl::OkStatus()));
 
   Subgraph::Identity id1;
   id1.insert(gen_map1["node1"].get());
@@ -530,7 +533,8 @@ TEST_F(SigNodeTest, EqualsLinks) {
   sg1.ExtractForSignature(&sig_map1);
 
   GenNodeMap gen_map2;
-  ASSERT_THAT(GenNode::BuildGraphInMap(graph1, &gen_map2), Eq(OkStatus()));
+  ASSERT_THAT(GenNode::BuildGraphInMap(graph1, &gen_map2),
+              Eq(absl::OkStatus()));
 
   Subgraph::Identity id2;
   id2.insert(gen_map2["node1"].get());
@@ -609,8 +613,8 @@ class SignatureTest : public SigBaseTest {
 
     gen_map_.clear();
     sig_.map.clear();
-    Status result = GenNode::BuildGraphInMap(graph, &gen_map_);
-    ASSERT_THAT(result, Eq(OkStatus()));
+    absl::Status result = GenNode::BuildGraphInMap(graph, &gen_map_);
+    ASSERT_THAT(result, Eq(absl::OkStatus()));
     Subgraph::Identity id;
     for (const auto& entry : gen_map_) {
       id.insert(entry.second.get());
@@ -668,7 +672,7 @@ class SignatureTest : public SigBaseTest {
       OrderLinks(&sig_);
 
       // The test as such.
-      ASSERT_THAT(sig_.Compute(), Eq(OkStatus()));
+      ASSERT_THAT(sig_.Compute(), Eq(absl::OkStatus()));
 
       signatures.insert(sig_.ToString());
 
@@ -1034,8 +1038,9 @@ TEST_F(SignatureTest, ComputeOneRoundSplitLinear) {
 TEST_F(SignatureTest, OrderLinks) {
   gen_map_.clear();
   sig_.map.clear();
-  Status result = GenNode::BuildGraphInMap(graph_for_link_order_, &gen_map_);
-  ASSERT_THAT(result, Eq(OkStatus()));
+  absl::Status result =
+      GenNode::BuildGraphInMap(graph_for_link_order_, &gen_map_);
+  ASSERT_THAT(result, Eq(absl::OkStatus()));
   Subgraph::Identity id;
   for (const auto& entry : gen_map_) {
     id.insert(entry.second.get());
@@ -1084,7 +1089,7 @@ TEST_F(SignatureTest, GraphTooBig) {
     (*graph.add_node()) = MakeNodeConst(absl::StrFormat("node%d", i));
   }
 
-  ASSERT_THAT(GenNode::BuildGraphInMap(graph, &gen_map_), Eq(OkStatus()));
+  ASSERT_THAT(GenNode::BuildGraphInMap(graph, &gen_map_), Eq(absl::OkStatus()));
 
   Subgraph::Identity id;
   for (const auto& entry : gen_map_) {
@@ -1093,11 +1098,12 @@ TEST_F(SignatureTest, GraphTooBig) {
   Subgraph sg(id);
   sg.ExtractForSignature(&sig_.map);
 
-  ASSERT_THAT(sig_.Compute(),
-              Eq(Status(absl::StatusCode::kInvalidArgument,
-                        "A graph of 65 nodes is too big for signature "
-                        "computation, the maximal supported node count is "
-                        "64.")));
+  ASSERT_THAT(
+      sig_.Compute(),
+      Eq(absl::Status(absl::StatusCode::kInvalidArgument,
+                      "A graph of 65 nodes is too big for signature "
+                      "computation, the maximal supported node count is "
+                      "64.")));
 }
 
 TEST_F(SignatureTest, ToString) {
@@ -1173,7 +1179,7 @@ TEST_F(SignatureTest, Equals) {
   // Start with 2 copies of the same graph.
   GenNodeMap gen_map1;
   ASSERT_THAT(GenNode::BuildGraphInMap(graph_circular_bidir_, &gen_map1),
-              Eq(OkStatus()));
+              Eq(absl::OkStatus()));
 
   Subgraph::Identity id1;
   id1.insert(gen_map1["node1"].get());
@@ -1182,11 +1188,11 @@ TEST_F(SignatureTest, Equals) {
 
   Signature sig1;
   sg1.ExtractForSignature(&sig1.map);
-  ASSERT_THAT(sig1.Compute(), Eq(OkStatus()));
+  ASSERT_THAT(sig1.Compute(), Eq(absl::OkStatus()));
 
   GenNodeMap gen_map2;
   ASSERT_THAT(GenNode::BuildGraphInMap(graph_circular_bidir_, &gen_map2),
-              Eq(OkStatus()));
+              Eq(absl::OkStatus()));
 
   Subgraph::Identity id2;
   id2.insert(gen_map2["node1"].get());
@@ -1195,7 +1201,7 @@ TEST_F(SignatureTest, Equals) {
 
   Signature sig2;
   sg2.ExtractForSignature(&sig2.map);
-  ASSERT_THAT(sig2.Compute(), Eq(OkStatus()));
+  ASSERT_THAT(sig2.Compute(), Eq(absl::OkStatus()));
 
   EXPECT_TRUE(sig1 == sig2);
 
