@@ -250,13 +250,13 @@ bool HasFanoutValue(const FanoutsMap& fanouts, const FanoutsMap::iterator& it) {
   return it != fanouts.end() && !it->second.empty();
 }
 
-Status MutationError(absl::string_view function_name, absl::string_view params,
-                     absl::string_view msg) {
+absl::Status MutationError(absl::string_view function_name,
+                           absl::string_view params, absl::string_view msg) {
   return errors::InvalidArgument(absl::Substitute(
       "MutableGraphView::$0($1) error: $2.", function_name, params, msg));
 }
 
-using ErrorHandler = std::function<Status(absl::string_view)>;
+using ErrorHandler = std::function<absl::Status(absl::string_view)>;
 
 ErrorHandler UpdateFanoutsError(absl::string_view from_node_name,
                                 absl::string_view to_node_name) {
@@ -267,53 +267,55 @@ ErrorHandler UpdateFanoutsError(absl::string_view from_node_name,
   };
 }
 
-Status CheckFaninIsRegular(const TensorId& fanin, ErrorHandler handler) {
+absl::Status CheckFaninIsRegular(const TensorId& fanin, ErrorHandler handler) {
   if (!IsTensorIdRegular(fanin)) {
     return handler(absl::Substitute("fanin '$0' must be a regular tensor id",
                                     fanin.ToString()));
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status CheckFaninIsValid(const TensorId& fanin, ErrorHandler handler) {
+absl::Status CheckFaninIsValid(const TensorId& fanin, ErrorHandler handler) {
   if (!IsTensorIdPortValid(fanin)) {
     return handler(absl::Substitute("fanin '$0' must be a valid tensor id",
                                     fanin.ToString()));
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status CheckAddingFaninToSelf(absl::string_view node_name,
-                              const TensorId& fanin, ErrorHandler handler) {
+absl::Status CheckAddingFaninToSelf(absl::string_view node_name,
+                                    const TensorId& fanin,
+                                    ErrorHandler handler) {
   if (node_name == fanin.node()) {
     return handler(
         absl::Substitute("can't add fanin '$0' to self", fanin.ToString()));
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status CheckRemovingFaninFromSelf(absl::string_view node_name,
-                                  const TensorId& fanin, ErrorHandler handler) {
+absl::Status CheckRemovingFaninFromSelf(absl::string_view node_name,
+                                        const TensorId& fanin,
+                                        ErrorHandler handler) {
   if (node_name == fanin.node()) {
     return handler(absl::Substitute("can't remove fanin '$0' from self",
                                     fanin.ToString()));
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 string NodeMissingErrorMsg(absl::string_view node_name) {
   return absl::Substitute("node '$0' was not found", node_name);
 }
 
-Status CheckNodeExists(absl::string_view node_name, NodeDef* node,
-                       ErrorHandler handler) {
+absl::Status CheckNodeExists(absl::string_view node_name, NodeDef* node,
+                             ErrorHandler handler) {
   if (node == nullptr) {
     return handler(NodeMissingErrorMsg(node_name));
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status CheckPortRange(int port, int min, int max, ErrorHandler handler) {
+absl::Status CheckPortRange(int port, int min, int max, ErrorHandler handler) {
   if (port < min || port > max) {
     if (max < min) {
       return handler("no available ports as node has no regular fanins");
@@ -321,7 +323,7 @@ Status CheckPortRange(int port, int min, int max, ErrorHandler handler) {
     return handler(
         absl::Substitute("port must be in range [$0, $1]", min, max));
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 string SwapNodeNamesSwitchControlErrorMsg(absl::string_view node_name) {
@@ -386,8 +388,8 @@ void MutableGraphView::AddAndDedupFanouts(NodeDef* node) {
         fanouts()[output].emplace(node, Graph::kControlSlot);
       } else {
         max_input_port = pos;
-        max_regular_output_port()[output.node] =
-            std::max(max_regular_output_port()[output.node], output.port_id);
+        int& max_port = max_regular_output_port()[output.node];
+        max_port = std::max(max_port, output.port_id);
         fanouts()[output].emplace(node, pos);
       }
       ++pos;
@@ -462,7 +464,7 @@ NodeDef* MutableGraphView::AddNode(NodeDef&& node) {
   return node_in_graph;
 }
 
-Status MutableGraphView::AddSubgraph(GraphDef&& subgraph) {
+absl::Status MutableGraphView::AddSubgraph(GraphDef&& subgraph) {
   // 1. Add all new functions and check that functions with the same name
   // have identical definition.
   const int function_size = subgraph.library().function_size();
@@ -508,10 +510,10 @@ Status MutableGraphView::AddSubgraph(GraphDef&& subgraph) {
     AddAndDedupFanouts(node);
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status MutableGraphView::UpdateNode(
+absl::Status MutableGraphView::UpdateNode(
     absl::string_view node_name, absl::string_view op, absl::string_view device,
     absl::Span<const std::pair<string, AttrValue>> attrs) {
   auto error_status = [node_name, op, device, attrs](absl::string_view msg) {
@@ -549,7 +551,7 @@ Status MutableGraphView::UpdateNode(
   }
 
   if (node->op() == op) {
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   node->set_op(string(op));
@@ -562,12 +564,12 @@ Status MutableGraphView::UpdateNode(
     }
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status MutableGraphView::UpdateNodeName(absl::string_view from_node_name,
-                                        absl::string_view to_node_name,
-                                        bool update_fanouts) {
+absl::Status MutableGraphView::UpdateNodeName(absl::string_view from_node_name,
+                                              absl::string_view to_node_name,
+                                              bool update_fanouts) {
   auto error_status = [from_node_name, to_node_name,
                        update_fanouts](absl::string_view msg) {
     string params = absl::Substitute(
@@ -580,7 +582,7 @@ Status MutableGraphView::UpdateNodeName(absl::string_view from_node_name,
   TF_RETURN_IF_ERROR(CheckNodeExists(from_node_name, node, error_status));
 
   if (node->name() == to_node_name) {
-    return OkStatus();
+    return absl::OkStatus();
   }
   if (HasNode(to_node_name)) {
     return error_status(
@@ -605,12 +607,12 @@ Status MutableGraphView::UpdateNodeName(absl::string_view from_node_name,
   nodes().erase(node->name());
   node->set_name(string(to_node_name));
   nodes().emplace(node->name(), node);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status MutableGraphView::SwapNodeNames(absl::string_view from_node_name,
-                                       absl::string_view to_node_name,
-                                       bool update_fanouts) {
+absl::Status MutableGraphView::SwapNodeNames(absl::string_view from_node_name,
+                                             absl::string_view to_node_name,
+                                             bool update_fanouts) {
   auto error_status = [from_node_name, to_node_name,
                        update_fanouts](absl::string_view msg) {
     string params = absl::Substitute(
@@ -622,7 +624,7 @@ Status MutableGraphView::SwapNodeNames(absl::string_view from_node_name,
   NodeDef* from_node = GetNode(from_node_name);
   TF_RETURN_IF_ERROR(CheckNodeExists(from_node_name, from_node, error_status));
   if (from_node_name == to_node_name) {
-    return OkStatus();
+    return absl::OkStatus();
   }
   NodeDef* to_node = GetNode(to_node_name);
   TF_RETURN_IF_ERROR(CheckNodeExists(to_node_name, to_node, error_status));
@@ -639,7 +641,7 @@ Status MutableGraphView::SwapNodeNames(absl::string_view from_node_name,
     SwapFanoutInputs(*this, &fanouts(), &max_regular_output_port(), from_node,
                      to_node);
     swap_names();
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   bool from_is_switch = IsSwitch(*from_node);
@@ -750,11 +752,11 @@ Status MutableGraphView::SwapNodeNames(absl::string_view from_node_name,
     }
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status MutableGraphView::UpdateFanouts(absl::string_view from_node_name,
-                                       absl::string_view to_node_name) {
+absl::Status MutableGraphView::UpdateFanouts(absl::string_view from_node_name,
+                                             absl::string_view to_node_name) {
   NodeDef* from_node = GetNode(from_node_name);
   TF_RETURN_IF_ERROR(
       CheckNodeExists(from_node_name, from_node,
@@ -766,12 +768,12 @@ Status MutableGraphView::UpdateFanouts(absl::string_view from_node_name,
   return UpdateFanoutsInternal(from_node, to_node);
 }
 
-Status MutableGraphView::UpdateFanoutsInternal(NodeDef* from_node,
-                                               NodeDef* to_node) {
+absl::Status MutableGraphView::UpdateFanoutsInternal(NodeDef* from_node,
+                                                     NodeDef* to_node) {
   VLOG(2) << absl::Substitute("Update fanouts from '$0' to '$1'.",
                               from_node->name(), to_node->name());
   if (from_node == to_node) {
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   // Update internal state with the new output_port->input_port edge.
@@ -860,7 +862,7 @@ Status MutableGraphView::UpdateFanoutsInternal(NodeDef* from_node,
     max_regular_output_port().erase(from_node);
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 bool MutableGraphView::AddFaninInternal(NodeDef* node,
@@ -911,8 +913,8 @@ bool MutableGraphView::AddFaninInternal(NodeDef* node,
   return true;
 }
 
-Status MutableGraphView::AddRegularFanin(absl::string_view node_name,
-                                         const TensorId& fanin) {
+absl::Status MutableGraphView::AddRegularFanin(absl::string_view node_name,
+                                               const TensorId& fanin) {
   auto error_status = [node_name, fanin](absl::string_view msg) {
     string params = absl::Substitute("node_name='$0', fanin='$1'", node_name,
                                      fanin.ToString());
@@ -927,12 +929,11 @@ Status MutableGraphView::AddRegularFanin(absl::string_view node_name,
   TF_RETURN_IF_ERROR(CheckNodeExists(fanin.node(), fanin_node, error_status));
 
   AddFaninInternal(node, {fanin_node, fanin.index()});
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status MutableGraphView::AddRegularFaninByPort(absl::string_view node_name,
-                                               int port,
-                                               const TensorId& fanin) {
+absl::Status MutableGraphView::AddRegularFaninByPort(
+    absl::string_view node_name, int port, const TensorId& fanin) {
   auto error_status = [node_name, port, fanin](absl::string_view msg) {
     string params = absl::Substitute("node_name='$0', port=$1, fanin='$2'",
                                      node_name, port, fanin.ToString());
@@ -971,7 +972,7 @@ Status MutableGraphView::AddRegularFaninByPort(absl::string_view node_name,
     RemoveControllingFaninInternal(node, fanin_node);
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 NodeDef* MutableGraphView::GetControllingFaninToAdd(absl::string_view node_name,
@@ -1033,8 +1034,8 @@ NodeDef* MutableGraphView::GetOrCreateIdentityConsumingSwitch(
   return identity_node;
 }
 
-Status MutableGraphView::AddControllingFanin(absl::string_view node_name,
-                                             const TensorId& fanin) {
+absl::Status MutableGraphView::AddControllingFanin(absl::string_view node_name,
+                                                   const TensorId& fanin) {
   auto error_status = [node_name, fanin](absl::string_view msg) {
     string params = absl::Substitute("node_name='$0', fanin='$1'", node_name,
                                      fanin.ToString());
@@ -1061,7 +1062,7 @@ Status MutableGraphView::AddControllingFanin(absl::string_view node_name,
   }
   AddFaninInternal(node, {control_node, Graph::kControlSlot});
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 bool MutableGraphView::RemoveRegularFaninInternal(NodeDef* node,
@@ -1120,8 +1121,8 @@ bool MutableGraphView::RemoveRegularFaninInternal(NodeDef* node,
   return modified;
 }
 
-Status MutableGraphView::RemoveRegularFanin(absl::string_view node_name,
-                                            const TensorId& fanin) {
+absl::Status MutableGraphView::RemoveRegularFanin(absl::string_view node_name,
+                                                  const TensorId& fanin) {
   auto error_status = [node_name, fanin](absl::string_view msg) {
     string params = absl::Substitute("node_name='$0', fanin='$1'", node_name,
                                      fanin.ToString());
@@ -1137,11 +1138,11 @@ Status MutableGraphView::RemoveRegularFanin(absl::string_view node_name,
   TF_RETURN_IF_ERROR(CheckNodeExists(fanin.node(), fanin_node, error_status));
 
   RemoveRegularFaninInternal(node, {fanin_node, fanin.index()});
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status MutableGraphView::RemoveRegularFaninByPort(absl::string_view node_name,
-                                                  int port) {
+absl::Status MutableGraphView::RemoveRegularFaninByPort(
+    absl::string_view node_name, int port) {
   auto error_status = [node_name, port](absl::string_view msg) {
     string params =
         absl::Substitute("node_name='$0', port=$1", node_name, port);
@@ -1180,7 +1181,7 @@ Status MutableGraphView::RemoveRegularFaninByPort(absl::string_view node_name,
     max_regular_input_port()[node] = updated_last_regular_input_port;
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 bool MutableGraphView::RemoveControllingFaninInternal(NodeDef* node,
@@ -1201,7 +1202,7 @@ bool MutableGraphView::RemoveControllingFaninInternal(NodeDef* node,
   return false;
 }
 
-Status MutableGraphView::RemoveControllingFanin(
+absl::Status MutableGraphView::RemoveControllingFanin(
     absl::string_view node_name, absl::string_view fanin_node_name) {
   auto error_status = [node_name, fanin_node_name](absl::string_view msg) {
     string params = absl::Substitute("node_name='$0', fanin_node_name='$1'",
@@ -1218,11 +1219,11 @@ Status MutableGraphView::RemoveControllingFanin(
       CheckNodeExists(fanin_node_name, fanin_node, error_status));
 
   RemoveControllingFaninInternal(node, fanin_node);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status MutableGraphView::RemoveAllFanins(absl::string_view node_name,
-                                         bool keep_controlling_fanins) {
+absl::Status MutableGraphView::RemoveAllFanins(absl::string_view node_name,
+                                               bool keep_controlling_fanins) {
   NodeDef* node = GetNode(node_name);
   if (node == nullptr) {
     string params =
@@ -1233,7 +1234,7 @@ Status MutableGraphView::RemoveAllFanins(absl::string_view node_name,
   }
 
   if (node->input().empty()) {
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   const int num_regular_fanins =
@@ -1241,7 +1242,7 @@ Status MutableGraphView::RemoveAllFanins(absl::string_view node_name,
   RemoveFaninsInternal(node, keep_controlling_fanins);
   if (keep_controlling_fanins) {
     if (num_regular_fanins == 0) {
-      return OkStatus();
+      return absl::OkStatus();
     } else if (num_regular_fanins < node->input_size()) {
       node->mutable_input()->DeleteSubrange(0, num_regular_fanins);
     } else {
@@ -1250,12 +1251,12 @@ Status MutableGraphView::RemoveAllFanins(absl::string_view node_name,
   } else {
     node->clear_input();
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status MutableGraphView::UpdateFanin(absl::string_view node_name,
-                                     const TensorId& from_fanin,
-                                     const TensorId& to_fanin) {
+absl::Status MutableGraphView::UpdateFanin(absl::string_view node_name,
+                                           const TensorId& from_fanin,
+                                           const TensorId& to_fanin) {
   auto error_status = [node_name, from_fanin, to_fanin](absl::string_view msg) {
     string params =
         absl::Substitute("node_name='$0', from_fanin='$1', to_fanin='$2'",
@@ -1289,7 +1290,7 @@ Status MutableGraphView::UpdateFanin(absl::string_view node_name,
   }
 
   if (from_fanin == to_fanin) {
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   bool from_fanin_is_control = IsTensorIdControlling(from_fanin);
@@ -1304,7 +1305,7 @@ Status MutableGraphView::UpdateFanin(absl::string_view node_name,
     if (modified) {
       AddFaninInternal(node, {to_fanin_node, to_fanin.index()});
     }
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   // In place mutation of regular fanins, requires no shifting of ports.
@@ -1340,12 +1341,11 @@ Status MutableGraphView::UpdateFanin(absl::string_view node_name,
     }
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status MutableGraphView::UpdateRegularFaninByPort(absl::string_view node_name,
-                                                  int port,
-                                                  const TensorId& fanin) {
+absl::Status MutableGraphView::UpdateRegularFaninByPort(
+    absl::string_view node_name, int port, const TensorId& fanin) {
   auto error_status = [node_name, port, fanin](absl::string_view msg) {
     string params = absl::Substitute("node_name='$0', port=$1, fanin='$2'",
                                      node_name, port, fanin.ToString());
@@ -1365,7 +1365,7 @@ Status MutableGraphView::UpdateRegularFaninByPort(absl::string_view node_name,
 
   TensorId tensor_id = ParseTensorName(node->input(port));
   if (tensor_id == fanin) {
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   InputPort input(node, port);
@@ -1384,11 +1384,11 @@ Status MutableGraphView::UpdateRegularFaninByPort(absl::string_view node_name,
     RemoveControllingFaninInternal(node, fanin_node);
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status MutableGraphView::SwapRegularFaninsByPorts(absl::string_view node_name,
-                                                  int from_port, int to_port) {
+absl::Status MutableGraphView::SwapRegularFaninsByPorts(
+    absl::string_view node_name, int from_port, int to_port) {
   auto error_status = [node_name, from_port, to_port](absl::string_view msg) {
     string params = absl::Substitute("node_name='$0', from_port=$1, to_port=$2",
                                      node_name, from_port, to_port);
@@ -1405,12 +1405,12 @@ Status MutableGraphView::SwapRegularFaninsByPorts(absl::string_view node_name,
                                     error_status));
 
   if (from_port == to_port) {
-    return OkStatus();
+    return absl::OkStatus();
   }
   TensorId from_fanin = ParseTensorName(node->input(from_port));
   TensorId to_fanin = ParseTensorName(node->input(to_port));
   if (from_fanin == to_fanin) {
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   InputPort from_input(node, from_port);
@@ -1428,10 +1428,10 @@ Status MutableGraphView::SwapRegularFaninsByPorts(absl::string_view node_name,
 
   node->mutable_input()->SwapElements(from_port, to_port);
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status MutableGraphView::UpdateAllRegularFaninsToControlling(
+absl::Status MutableGraphView::UpdateAllRegularFaninsToControlling(
     absl::string_view node_name) {
   auto error_status = [node_name](absl::string_view msg) {
     string params = absl::Substitute("node_name='$0'", node_name);
@@ -1499,10 +1499,10 @@ Status MutableGraphView::UpdateAllRegularFaninsToControlling(
   node->mutable_input()->DeleteSubrange(pos, node->input_size() - pos);
   max_regular_input_port().erase(node);
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status MutableGraphView::CheckNodesCanBeDeleted(
+absl::Status MutableGraphView::CheckNodesCanBeDeleted(
     const absl::flat_hash_set<string>& nodes_to_delete) {
   std::vector<string> missing_nodes;
   std::vector<string> nodes_with_fanouts;
@@ -1562,10 +1562,10 @@ Status MutableGraphView::CheckNodesCanBeDeleted(
     return MutationError("DeleteNodes", params, error_msg);
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status MutableGraphView::DeleteNodes(
+absl::Status MutableGraphView::DeleteNodes(
     const absl::flat_hash_set<string>& nodes_to_delete) {
   TF_RETURN_IF_ERROR(CheckNodesCanBeDeleted(nodes_to_delete));
 
@@ -1600,7 +1600,7 @@ Status MutableGraphView::DeleteNodes(
     graph()->mutable_node()->DeleteSubrange(last_pos + 1, last_idx - last_pos);
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 void MutableGraphView::RemoveFaninsInternal(NodeDef* deleted_node,
