@@ -47,6 +47,7 @@ limitations under the License.
 #include "tensorflow/core/platform/status.h"
 #include "tensorflow/core/platform/statusor.h"
 #include "tensorflow/core/util/env_var.h"
+#include "tensorflow/core/util/util.h"
 
 namespace tensorflow {
 namespace grappler {
@@ -288,7 +289,7 @@ class NodeTypeAttrMap {
 
   explicit NodeTypeAttrMap(const GraphDef& graph) { TF_CHECK_OK(Init(graph)); }
 
-  Status Init(const GraphDef& graph) {
+  absl::Status Init(const GraphDef& graph) {
     if (graph_ != nullptr) {
       return errors::InvalidArgument("NodeTypeAttrMap is already initialized.");
     }
@@ -298,7 +299,7 @@ class NodeTypeAttrMap {
     for (const NodeDef& node : graph.node()) {
       TF_RETURN_IF_ERROR(AddNode(node));
     }
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   bool is_initialized() const { return graph_ != nullptr; }
@@ -347,7 +348,7 @@ class NodeTypeAttrMap {
   }
 
  private:
-  Status AddNode(const NodeDef& node) {
+  absl::Status AddNode(const NodeDef& node) {
     const OpDef* op_def_ptr = nullptr;
     TF_RETURN_IF_ERROR(function_library_->LookUpOpDef(node.op(), &op_def_ptr));
     const OpDef& op_def = *op_def_ptr;
@@ -412,7 +413,7 @@ class NodeTypeAttrMap {
         }
       }
     }
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   // WARN: `graph_` must outlive this object (node pointers must remain valid).
@@ -473,10 +474,11 @@ class GraphTypeTopologyView {
   // computing graph topology. Example: Tensorflow runtime allows concurrent
   // execution of dequeue/enqueue ops from the same queue resource, but we might
   // want to enforce ordering between them for the purpose of graph analysis.
-  Status InitializeFromGraph(const GraphDef& graph,
-                             const NodeTypeAttrMap& node_type_map);
+  absl::Status InitializeFromGraph(const GraphDef& graph,
+                                   const NodeTypeAttrMap& node_type_map);
 
-  Status AddEphemeralEdges(absl::Span<const NodeTypeIdEdge> ephemeral_edges);
+  absl::Status AddEphemeralEdges(
+      absl::Span<const NodeTypeIdEdge> ephemeral_edges);
 
   bool is_initialized() const { return graph_ != nullptr; }
   int num_nodes() const { return num_nodes_; }
@@ -547,7 +549,7 @@ inline void SortAndRemoveDuplicates(T* v) {
   v->erase(std::unique(v->begin(), v->end()), v->end());
 }
 
-Status GraphTypeTopologyView::InitializeFromGraph(
+absl::Status GraphTypeTopologyView::InitializeFromGraph(
     const GraphDef& graph, const NodeTypeAttrMap& node_type_map) {
   if (graph_ != nullptr) {
     return errors::InvalidArgument(
@@ -628,10 +630,10 @@ Status GraphTypeTopologyView::InitializeFromGraph(
     SortAndRemoveDuplicates(&fanouts_[node_type_idx]);
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status GraphTypeTopologyView::AddEphemeralEdges(
+absl::Status GraphTypeTopologyView::AddEphemeralEdges(
     absl::Span<const NodeTypeIdEdge> ephemeral_edges) {
   // Add ephemeral edges to the adjacency lists.
   for (const NodeTypeIdEdge& edge : ephemeral_edges) {
@@ -678,7 +680,7 @@ Status GraphTypeTopologyView::AddEphemeralEdges(
     SortAndRemoveDuplicates(&fanouts_[node_type_idx]);
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 bool GraphTypeTopologyView::HasNode(absl::string_view node_name,
@@ -911,10 +913,10 @@ DataTypeSet AllowedDataTypes(const OpDef& op_def, const TypeAttrId& t_attr_id) {
   return AllowedDataTypes(*attr_def);
 }
 
-Status ValidateLists(const gtl::FlatSet<string>& allow_list,
-                     const gtl::FlatSet<string>& deny_list,
-                     const gtl::FlatSet<string>& infer_list,
-                     const gtl::FlatSet<string>& clear_list) {
+absl::Status ValidateLists(const gtl::FlatSet<string>& allow_list,
+                           const gtl::FlatSet<string>& deny_list,
+                           const gtl::FlatSet<string>& infer_list,
+                           const gtl::FlatSet<string>& clear_list) {
   std::vector<gtl::FlatSet<string>> lists{allow_list, deny_list, infer_list,
                                           clear_list};
   std::multiset<string> counts;
@@ -931,13 +933,13 @@ Status ValidateLists(const gtl::FlatSet<string>& allow_list,
   if (duplicates) {
     return errors::InvalidArgument("Op lists have conflicting entries");
   } else {
-    return OkStatus();
+    return absl::OkStatus();
   }
 }
 
 bool HasInputOrOutputRefs(const NodeDef& node) {
   const OpDef* op_def;
-  Status status = OpRegistry::Global()->LookUpOpDef(node.op(), &op_def);
+  absl::Status status = OpRegistry::Global()->LookUpOpDef(node.op(), &op_def);
   if (!status.ok()) {
     return true;
   }
@@ -1051,11 +1053,12 @@ class AutoMixedPrecisionImpl {
         num_nonvar_casts_to_f16_(0),
         mode_(mode),
         target_dtype_((mode_ == AutoMixedPrecisionMode::CUDA ||
-                       mode_ == AutoMixedPrecisionMode::CPU)
+                       mode_ == AutoMixedPrecisionMode::CPU ||
+                       mode_ == AutoMixedPrecisionMode::FP16_CPU)
                           ? DT_HALF
                           : DT_BFLOAT16) {}
 
-  Status Optimize();
+  absl::Status Optimize();
 
  private:
   typedef absl::flat_hash_set<NodeTypeId> NodeTypeIdSet;
@@ -1063,19 +1066,21 @@ class AutoMixedPrecisionImpl {
   std::unique_ptr<AutoMixedPrecisionLists> get_mixed_precision_lists() const {
     switch (mode_) {
       case AutoMixedPrecisionMode::CUDA:
-        return std::make_unique<AutoMixedPrecisionListsCuda>(cuda_version_,
-                                                             cudnn_version_);
+        return std::make_unique<AutoMixedPrecisionListsFp16>(
+            cuda_version_, cudnn_version_, AutoMixedPrecisionMode::CUDA);
       case AutoMixedPrecisionMode::BF16:
         return std::make_unique<AutoMixedPrecisionListsMkl>();
       case AutoMixedPrecisionMode::CPU:
-        // Note: this is not a typo here. AutoMixedPrecisionListsCuda is used
-        // intentionally to make CPU and GPU have the same fp16 ops.
-        return std::make_unique<AutoMixedPrecisionListsCuda>(
-            /*cuda_version=*/10000,   // Hardcode cuda and cudnn version so
-            /*cudnn_version=*/8000);  // CPU emulates the same ops on GPU.
+        return std::make_unique<AutoMixedPrecisionListsFp16>(
+            /*cuda_version=*/10000,  // Hardcode cuda and cudnn version so
+            /*cudnn_version=*/8000,  // CPU emulates the same ops on GPU.
+            AutoMixedPrecisionMode::CPU);
+      case AutoMixedPrecisionMode::FP16_CPU:
+        return std::make_unique<AutoMixedPrecisionListsFp16>(
+            0, 0, AutoMixedPrecisionMode::FP16_CPU);
     }
   }
-  Status PrintDebugLogs(bool preop, size_t timestamp);
+  absl::Status PrintDebugLogs(bool preop, size_t timestamp);
   void LogSkippedNode(const NodeDef& node, const string& device_type) const;
   bool MustPreserve(const NodeDef& node) const;
   bool IsOnDevice(const NodeDef& node, const string& device_type) const;
@@ -1110,21 +1115,22 @@ class AutoMixedPrecisionImpl {
                                     absl::flat_hash_set<int>* allow_set) const;
   void PropagateAllowThroughClear(const absl::flat_hash_set<int>& deny_set,
                                   absl::flat_hash_set<int>* allow_set) const;
-  Status ForceColorMatchOnRecurrentEdges(
+  absl::Status ForceColorMatchOnRecurrentEdges(
       absl::flat_hash_set<int>* allow_set) const;
   void MakeCastsAllowIfAllOutputsAllow(
       absl::flat_hash_set<int>* allow_set) const;
   NodeDef BuildCastNode(const MutableGraphView::OutputPort& src, bool to_f16,
                         const string& device) const;
-  StatusOr<NodeDef*> InsertCastNodeAtFanout(
+  absl::StatusOr<NodeDef*> InsertCastNodeAtFanout(
       const absl::flat_hash_set<int>& allow_set, const bool src_is_allow,
       const CastType& cast_type, MutableGraphView::OutputPort& src);
 
-  StatusOr<DataType> GetCastToType(const NodeDef* node) const;
+  absl::StatusOr<DataType> GetCastToType(const NodeDef* node) const;
   void CollectOutputPorts(
       const TypeAttrId& type_attr, NodeDef* node,
       std::vector<MutableGraphView::OutputPort>& output_ports) const;
-  Status ChangeTypeAttrsAndAddCasts(const absl::flat_hash_set<int>& allow_set);
+  absl::Status ChangeTypeAttrsAndAddCasts(
+      const absl::flat_hash_set<int>& allow_set);
 
   std::unordered_map<string, DeviceProperties> devices_;
   VirtualPlacer virtual_placer_;
@@ -1191,11 +1197,12 @@ bool AutoMixedPrecisionImpl::NodeHasF16KernelForTypeAttr(
   return IsKernelRegisteredForNode(node_copy).ok();
 }
 
-Status AutoMixedPrecisionImpl::PrintDebugLogs(bool preop, size_t timestamp) {
+absl::Status AutoMixedPrecisionImpl::PrintDebugLogs(bool preop,
+                                                    size_t timestamp) {
   string prepend_path;
   TF_RETURN_IF_ERROR(ReadStringFromEnvVar(
       "TF_AUTO_MIXED_PRECISION_GRAPH_REWRITE_LOG_PATH", "", &prepend_path));
-  if (prepend_path.empty()) return OkStatus();
+  if (prepend_path.empty()) return absl::OkStatus();
 
   string suffix =
       strings::StrCat("_", preop ? "preop" : kSuffix, "_", id_, "_", timestamp);
@@ -1242,7 +1249,7 @@ Status AutoMixedPrecisionImpl::PrintDebugLogs(bool preop, size_t timestamp) {
     f.close();
     LOG(INFO) << "Saved paint bucket info to " << fname;
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 void AutoMixedPrecisionImpl::LogSkippedNode(const NodeDef& node,
@@ -1313,7 +1320,7 @@ bool IsTensorListWriterOp(const string& op) {
 
 bool AutoMixedPrecisionImpl::SupportsF16(const NodeTypeId& node_type) const {
   const OpDef* op_def;
-  Status status =
+  absl::Status status =
       OpRegistry::Global()->LookUpOpDef(node_type.node->op(), &op_def);
   if (!status.ok()) return false;
   return AllowedDataTypes(*op_def, node_type.type_attr)
@@ -1324,7 +1331,7 @@ bool AutoMixedPrecisionImpl::SupportsF16(const NodeTypeId& node_type) const {
 bool AutoMixedPrecisionImpl::SupportsF16DataType(
     const NodeTypeId& node_type) const {
   const OpDef* op_def;
-  Status status =
+  absl::Status status =
       OpRegistry::Global()->LookUpOpDef(node_type.node->op(), &op_def);
   if (!status.ok()) return false;
   return AllowedDataTypes(*op_def, node_type.type_attr).Contains(target_dtype_);
@@ -1378,15 +1385,16 @@ bool ShouldIgnorePerformance() {
   return is_enabled;
 }
 
-Status AutoMixedPrecisionImpl::Optimize() {
+absl::Status AutoMixedPrecisionImpl::Optimize() {
   string optimization_level;
   TF_RETURN_IF_ERROR(ReadStringFromEnvVar(
       "TF_AUTO_MIXED_PRECISION_GRAPH_REWRITE_LEVEL", "", &optimization_level));
   optimization_level = absl::AsciiStrToUpper(optimization_level);
   force_all_fp16_ = optimization_level == "UNSAFE_FORCE_ALL";
-  if (force_all_fp16_ && mode_ == AutoMixedPrecisionMode::BF16) {
-    // Many ops do not support bfloat16 on the CPU so we disallowing forcing to
-    // bfloat16.
+  if (force_all_fp16_ && (mode_ == AutoMixedPrecisionMode::BF16 ||
+                          mode_ == AutoMixedPrecisionMode::FP16_CPU)) {
+    // Many ops do not support bfloat16/fp16 on the CPU. So, disallowing
+    // forcing to bfloat16/fp16.
     return errors::InvalidArgument(
         "TF_AUTO_MIXED_PRECISION_GRAPH_REWRITE_LEVEL cannot be set to "
         "UNSAFE_FORCE_ALL when oneDNN is used");
@@ -1428,6 +1436,7 @@ Status AutoMixedPrecisionImpl::Optimize() {
         break;
       case AutoMixedPrecisionMode::BF16:
       case AutoMixedPrecisionMode::CPU:
+      case AutoMixedPrecisionMode::FP16_CPU:
         device_type = DEVICE_CPU;
         should_process = !MustPreserve(node) && IsOnDevice(node, device_type);
         break;
@@ -1500,7 +1509,7 @@ Status AutoMixedPrecisionImpl::Optimize() {
 
   if (allow_set.empty()) {
     LOG(INFO) << "No allowlist ops found, nothing to do";
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   VLOG(2) << "Beginning pass 2 to propagate deny forwards from denylist ops "
@@ -1552,7 +1561,7 @@ Status AutoMixedPrecisionImpl::Optimize() {
 
   TF_RETURN_IF_ERROR(PrintDebugLogs(/* preop = */ false, timestamp));
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // If node is a Tensor List op with a float32 data type attribute then this
@@ -1856,10 +1865,6 @@ void AutoMixedPrecisionImpl::PropagateAllowThroughClear(
 void AutoMixedPrecisionImpl::AddInferToAllowIfFollowAllow(
     const absl::flat_hash_set<int>& deny_set,
     absl::flat_hash_set<int>* allow_set) const {
-  // Currently only target for oneDNN
-  if (mode_ != AutoMixedPrecisionMode::BF16) {
-    return;
-  }
   for (int item_idx = 0; item_idx < graph_type_view_.num_nodes(); ++item_idx) {
     const NodeTypeId& item = *graph_type_view_.GetNode(item_idx);
     if (!ShouldProcess(*item.node) || deny_set.count(item_idx) ||
@@ -1912,7 +1917,7 @@ void AutoMixedPrecisionImpl::RemoveAllowsetWithFp32(
 // color. Specifically, it removes them all from allow_set if any of the Merge
 // nodes is not in allow_set, otherwise it adds the NextIteration node to
 // allow_set.
-Status AutoMixedPrecisionImpl::ForceColorMatchOnRecurrentEdges(
+absl::Status AutoMixedPrecisionImpl::ForceColorMatchOnRecurrentEdges(
     absl::flat_hash_set<int>* allow_set) const {
   for (const NodeDef& node : graph_->node()) {
     if (node.op() == "NextIteration") {
@@ -1968,7 +1973,7 @@ Status AutoMixedPrecisionImpl::ForceColorMatchOnRecurrentEdges(
       }
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Forces all of the given Tensor List nodes into the same color set.
@@ -2078,7 +2083,7 @@ void AutoMixedPrecisionImpl::MakeCastsAllowIfAllOutputsAllow(
 //   FP16: cast to float16
 //   FP32: cast to float32
 //   AUTO: cast to a data type that matches the fanout data type
-StatusOr<NodeDef*> AutoMixedPrecisionImpl::InsertCastNodeAtFanout(
+absl::StatusOr<NodeDef*> AutoMixedPrecisionImpl::InsertCastNodeAtFanout(
     const absl::flat_hash_set<int>& allow_set, const bool src_is_allow,
     const CastType& cast_type, MutableGraphView::OutputPort& src) {
   NodeDef* added_cast_node = nullptr;
@@ -2139,7 +2144,7 @@ StatusOr<NodeDef*> AutoMixedPrecisionImpl::InsertCastNodeAtFanout(
 
 // Get the destination data type of a cast op. Return error if the node is not
 // a Cast op.
-StatusOr<DataType> AutoMixedPrecisionImpl::GetCastToType(
+absl::StatusOr<DataType> AutoMixedPrecisionImpl::GetCastToType(
     const NodeDef* node) const {
   CHECK_EQ(node->op(), "Cast")  // Crash OK
       << "Node " << node->name() << " is not a Cast op";
@@ -2162,7 +2167,7 @@ void AutoMixedPrecisionImpl::CollectOutputPorts(
 // Changes all allow-painted type attributes to DT_HALF or DT_BFLOAT16, and
 // inserts Cast nodes at node outputs for all edges that connect
 // allow-painted <-> non-allow-painted type attributes.
-Status AutoMixedPrecisionImpl::ChangeTypeAttrsAndAddCasts(
+absl::Status AutoMixedPrecisionImpl::ChangeTypeAttrsAndAddCasts(
     const absl::flat_hash_set<int>& allow_set) {
   int num_nodes_changed = 0;
   const int num_nodes_preop = graph_->node_size();
@@ -2255,7 +2260,7 @@ Status AutoMixedPrecisionImpl::ChangeTypeAttrsAndAddCasts(
             << " nodes to " << type_str << " precision using "
             << num_nonvar_casts_to_f16_ << " cast(s) to " << type_str
             << " (excluding Const and Variable casts)";
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 int GetNumGPUs(const Cluster& cluster) {
@@ -2276,8 +2281,9 @@ int GetNumGPUs(const Cluster& cluster) {
 
 }  // end namespace
 
-Status AutoMixedPrecision::Optimize(Cluster* cluster, const GrapplerItem& item,
-                                    GraphDef* output) {
+absl::Status AutoMixedPrecision::Optimize(Cluster* cluster,
+                                          const GrapplerItem& item,
+                                          GraphDef* output) {
   if (cluster == nullptr) {
     return errors::InvalidArgument("cluster == nullptr");
   }
@@ -2299,10 +2305,18 @@ Status AutoMixedPrecision::Optimize(Cluster* cluster, const GrapplerItem& item,
 
   int num_gpus = GetNumGPUs(*cluster);
   if (num_gpus < 1 && mode_ == AutoMixedPrecisionMode::CUDA) {
-    // AutoMixedPrecision is currently only tuned for GPU.
-    LOG(WARNING) << "No (suitable) GPUs detected, skipping " << name()
-                 << " graph optimizer";
-    return OkStatus();
+    // No GPUs to run AutoMixedPrecision in FP16.
+    VLOG(1) << "No (suitable) GPUs detected, skipping " << name()
+            << " graph optimizer";
+    return absl::OkStatus();
+  }
+  // Check if CPU supports FP16, oneDNN supports FP16 on
+  // some platforms by converting to and from FP32
+  if (mode_ == AutoMixedPrecisionMode::FP16_CPU &&
+      !IsAMXDataTypeSupportedByOneDNNOnThisCPU(DT_HALF) &&
+      !IsAVXConvertSupportedByOneDNNOnThisCPU()) {
+    VLOG(1) << "No support for " << name() << " graph optimizer on CPU";
+    return absl::OkStatus();
   }
 
   if (num_gpus >= 1 && mode_ == AutoMixedPrecisionMode::BF16) {
@@ -2318,7 +2332,7 @@ Status AutoMixedPrecision::Optimize(Cluster* cluster, const GrapplerItem& item,
   } else {
     VLOG(1) << "Running " << name() << " graph optimizer on " << item.id;
   }
-  Status status = optimizer.Optimize();
+  absl::Status status = optimizer.Optimize();
   if (!status.ok()) {
     // Restore the original graph.
     *output = item.graph;
