@@ -89,40 +89,57 @@ void VerifyGraphsEquivalent(const GraphDef& original_graph,
   }
 }
 
-// Currently, this test suite only passes when TensorFlow passes with CUDA/HIP,
-// because otherwise the optimizer will not turn clearlist nodes to float16.
-// When looking at clearlist nodes, this optimizer checks if the nodes have a
-// float16 GPU OpKernel, but without CUDA/HIP there are no GPU OpKernels at all.
-#if GOOGLE_CUDA || TENSORFLOW_USE_ROCM
-
+// Currently on GPU, this test suite only passes when TensorFlow passes with
+// CUDA/HIP, because otherwise the optimizer will not turn clearlist nodes to
+// float16. When looking at clearlist nodes, this optimizer checks if the nodes
+// have a float16 GPU OpKernel, but without CUDA/HIP there are no GPU OpKernels
+// at all. And on CPU, this test suite passes when AMX FP16 is supported.
 const std::pair<int, int> kMinGPUArch = {7, 0};
 
 class AutoMixedPrecisionTest : public GrapplerTest {
  protected:
+  void SetMode(AutoMixedPrecisionMode mode) { mode_ = mode; }
   void SetUp() override {
-    int num_gpus = GetNumAvailableGPUs();
-    // If GPUs are available, require that they all satisfy the min arch.
-    gpu_available_ = (num_gpus > 0);
+    if (mode_ == AutoMixedPrecisionMode::CUDA) {
+      int num_gpus = GetNumAvailableGPUs();
+      // If GPUs are available, require that they all satisfy the min arch.
+      gpu_available_ = (num_gpus > 0);
 #if GOOGLE_CUDA
-    gpu_available_ =
-        gpu_available_ && (num_gpus == GetNumAvailableGPUs(kMinGPUArch));
+      gpu_available_ =
+          gpu_available_ && (num_gpus == GetNumAvailableGPUs(kMinGPUArch));
 #else  // Here we force Tensorflow to use the virtual GFX906
-    gpu_available_ = false;
+      gpu_available_ = false;
 #endif
-    if (gpu_available_) {
-      virtual_cluster_.reset(new SingleMachine(/* timeout_s = */ 10, 1, 1));
-    } else {
-      DeviceProperties device_properties;
-      device_properties.set_type("GPU");
+      if (gpu_available_) {
+        virtual_cluster_.reset(new SingleMachine(/* timeout_s = */ 10, 1, 1));
+      } else {
+        DeviceProperties device_properties;
+        device_properties.set_type("GPU");
 #if GOOGLE_CUDA
-      device_properties.mutable_environment()->insert({"architecture", "7"});
-      device_properties.mutable_environment()->insert({"cuda", "9010"});
+        device_properties.mutable_environment()->insert({"architecture", "7"});
+        device_properties.mutable_environment()->insert({"cuda", "9010"});
 #else
-      device_properties.mutable_environment()->insert(
-          {"architecture", "gfx906"});
+        device_properties.mutable_environment()->insert(
+            {"architecture", "gfx906"});
 #endif
-      virtual_cluster_.reset(
-          new VirtualCluster({{"/GPU:1", device_properties}}));
+        virtual_cluster_.reset(
+            new VirtualCluster({{"/GPU:1", device_properties}}));
+      }
+    } else if (mode_ == AutoMixedPrecisionMode::FP16_CPU) {
+      DeviceProperties device_properties;
+      device_properties.set_type("CPU");
+      virtual_cluster_.reset(new SingleMachine(/* timeout_s = */ 10, 1, 0));
+
+      bool is_fp16_enabled_on_cpu = false;
+#if defined(INTEL_MKL) && defined(ENABLE_ONEDNN_V3)
+      // oneDNN supports FP16 on some platforms by converting to and from FP32
+      is_fp16_enabled_on_cpu =
+          IsAMXDataTypeSupportedByOneDNNOnThisCPU(DT_HALF) ||
+          IsAVXConvertSupportedByOneDNNOnThisCPU();
+#endif  // INTEL_MKL && ENABLE_ONEDNN_V3
+      if (!IsMKLEnabled() || !is_fp16_enabled_on_cpu) {
+        GTEST_SKIP() << "This device doesn't support FP16";
+      }
     }
     TF_CHECK_OK(virtual_cluster_->Provision());
   }
@@ -189,7 +206,7 @@ class AutoMixedPrecisionTest : public GrapplerTest {
     std::vector<std::pair<string, Tensor>> feed = {{"input", input_tensor}};
     auto tensors_expected = EvaluateNodes(item.graph, item.fetch, feed);
 
-    AutoMixedPrecision optimizer;
+    AutoMixedPrecision optimizer(mode_);
     GraphDef output;
     TF_ASSERT_OK(optimizer.Optimize(virtual_cluster_.get(), item, &output));
 
@@ -212,9 +229,22 @@ class AutoMixedPrecisionTest : public GrapplerTest {
 
   std::unique_ptr<Cluster> virtual_cluster_;
   bool gpu_available_;
+  AutoMixedPrecisionMode mode_;
 };
 
-TEST_F(AutoMixedPrecisionTest, NoOp) {
+class AutoMixedPrecisionParamTest
+    : public AutoMixedPrecisionTest,
+      public ::testing::WithParamInterface<AutoMixedPrecisionMode> {
+ protected:
+  void SetUp() override {
+    mode_ = GetParam();
+    AutoMixedPrecisionTest::SetMode(mode_);
+    AutoMixedPrecisionTest::SetUp();
+  }
+  AutoMixedPrecisionMode mode_;
+};
+
+TEST_P(AutoMixedPrecisionParamTest, NoOp) {
   tensorflow::Scope s = tensorflow::Scope::NewRootScope();
   Output input = ops::Const(s.WithOpName("input"), 1.234f, {32});
   Output deny1 = ops::Exp(s.WithOpName("deny1"), input);
@@ -228,7 +258,7 @@ TEST_F(AutoMixedPrecisionTest, NoOp) {
   TF_CHECK_OK(s.ToGraphDef(&item.graph));
   auto tensors_expected = EvaluateNodes(item.graph, item.fetch);
 
-  AutoMixedPrecision optimizer;
+  AutoMixedPrecision optimizer(mode_);
   GraphDef output;
   TF_ASSERT_OK(optimizer.Optimize(virtual_cluster_.get(), item, &output));
 
@@ -251,7 +281,7 @@ TEST_F(AutoMixedPrecisionTest, NoOp) {
   }
 }
 
-TEST_F(AutoMixedPrecisionTest, AlreadyFp16) {
+TEST_P(AutoMixedPrecisionParamTest, AlreadyFp16) {
   tensorflow::Scope s = tensorflow::Scope::NewRootScope();
   Output input = ops::Const(s.WithOpName("input"), 1.f, {32, 32});
   Output cst1 = ops::Cast(s.WithOpName("cst1"), input, DT_HALF);
@@ -266,7 +296,7 @@ TEST_F(AutoMixedPrecisionTest, AlreadyFp16) {
   TF_CHECK_OK(s.ToGraphDef(&item.graph));
   auto tensors_expected = EvaluateNodes(item.graph, item.fetch);
 
-  AutoMixedPrecision optimizer;
+  AutoMixedPrecision optimizer(mode_);
   GraphDef output;
   TF_ASSERT_OK(optimizer.Optimize(virtual_cluster_.get(), item, &output));
   VLOG(1) << output.DebugString();
@@ -289,7 +319,7 @@ TEST_F(AutoMixedPrecisionTest, AlreadyFp16) {
   }
 }
 
-TEST_F(AutoMixedPrecisionTest, Simple) {
+TEST_P(AutoMixedPrecisionParamTest, Simple) {
   tensorflow::Scope s = tensorflow::Scope::NewRootScope();
   Output input = ops::Const(s.WithOpName("input"), 1.f / 32, {32, 32});
   Output deny1 = ops::Exp(s.WithOpName("deny1"), input);
@@ -309,7 +339,7 @@ TEST_F(AutoMixedPrecisionTest, Simple) {
   TF_CHECK_OK(s.ToGraphDef(&item.graph));
   auto tensors_expected = EvaluateNodes(item.graph, item.fetch);
 
-  AutoMixedPrecision optimizer;
+  AutoMixedPrecision optimizer(mode_);
   GraphDef output;
   TF_ASSERT_OK(optimizer.Optimize(virtual_cluster_.get(), item, &output));
 
@@ -338,10 +368,9 @@ TEST_F(AutoMixedPrecisionTest, Simple) {
   }
 }
 
-TEST_F(AutoMixedPrecisionTest, NoInferOp) {
+TEST_P(AutoMixedPrecisionParamTest, NoInferOp) {
   setenv("TF_AUTO_MIXED_PRECISION_GRAPH_REWRITE_LEVEL", "TREAT_INFER_AS_DENY",
          1 /* replace */);
-
   tensorflow::Scope s = tensorflow::Scope::NewRootScope();
   Output input = ops::Const(s.WithOpName("input"), 1.f / 32, {32, 32});
   Output deny1 = ops::Exp(s.WithOpName("deny1"), input);
@@ -361,7 +390,7 @@ TEST_F(AutoMixedPrecisionTest, NoInferOp) {
   TF_CHECK_OK(s.ToGraphDef(&item.graph));
   auto tensors_expected = EvaluateNodes(item.graph, item.fetch);
 
-  AutoMixedPrecision optimizer;
+  AutoMixedPrecision optimizer(mode_);
   GraphDef output;
   TF_ASSERT_OK(optimizer.Optimize(virtual_cluster_.get(), item, &output));
 
@@ -390,7 +419,7 @@ TEST_F(AutoMixedPrecisionTest, NoInferOp) {
   unsetenv("TF_AUTO_MIXED_PRECISION_GRAPH_REWRITE_LEVEL");
 }
 
-TEST_F(AutoMixedPrecisionTest, BidirectionalClearChain) {
+TEST_P(AutoMixedPrecisionParamTest, BidirectionalClearChain) {
   tensorflow::Scope s = tensorflow::Scope::NewRootScope();
   Output input = ops::Const(s.WithOpName("input"), 1.f / 32, {32, 32});
   Output clr1 = ops::Relu(s.WithOpName("clr1"), input);
@@ -406,7 +435,7 @@ TEST_F(AutoMixedPrecisionTest, BidirectionalClearChain) {
   TF_CHECK_OK(s.ToGraphDef(&item.graph));
   auto tensors_expected = EvaluateNodes(item.graph, item.fetch);
 
-  AutoMixedPrecision optimizer;
+  AutoMixedPrecision optimizer(mode_);
   GraphDef output;
   TF_ASSERT_OK(optimizer.Optimize(virtual_cluster_.get(), item, &output));
 
@@ -429,7 +458,7 @@ TEST_F(AutoMixedPrecisionTest, BidirectionalClearChain) {
   }
 }
 
-TEST_F(AutoMixedPrecisionTest, PreserveFetches) {
+TEST_P(AutoMixedPrecisionParamTest, PreserveFetches) {
   tensorflow::Scope s = tensorflow::Scope::NewRootScope();
   Output input = ops::Const(s.WithOpName("input"), 1.f / 32, {32, 32});
   Output allow1 = ops::MatMul(s.WithOpName("allow1"), input, input);
@@ -447,7 +476,7 @@ TEST_F(AutoMixedPrecisionTest, PreserveFetches) {
   TF_CHECK_OK(s.ToGraphDef(&item.graph));
   auto tensors_expected = EvaluateNodes(item.graph, item.fetch);
 
-  AutoMixedPrecision optimizer;
+  AutoMixedPrecision optimizer(mode_);
   GraphDef output;
   TF_ASSERT_OK(optimizer.Optimize(virtual_cluster_.get(), item, &output));
 
@@ -474,7 +503,10 @@ TEST_F(AutoMixedPrecisionTest, PreserveFetches) {
   }
 }
 
-TEST_F(AutoMixedPrecisionTest, PreserveCPUNodes) {
+TEST_P(AutoMixedPrecisionParamTest, PreserveCPUNodes) {
+  if (mode_ == AutoMixedPrecisionMode::FP16_CPU) {
+    GTEST_SKIP() << "This test is not required on CPU";
+  }
   tensorflow::Scope s = tensorflow::Scope::NewRootScope();
   Output input = ops::Const(s.WithOpName("input"), 1.f / 32, {32, 32});
   Output clr1 = ops::Relu(s.WithOpName("clr1"), input);
@@ -492,7 +524,7 @@ TEST_F(AutoMixedPrecisionTest, PreserveCPUNodes) {
   TF_CHECK_OK(s.ToGraphDef(&item.graph));
   auto tensors_expected = EvaluateNodes(item.graph, item.fetch);
 
-  AutoMixedPrecision optimizer;
+  AutoMixedPrecision optimizer(mode_);
   GraphDef output;
   TF_ASSERT_OK(optimizer.Optimize(virtual_cluster_.get(), item, &output));
 
@@ -503,7 +535,7 @@ TEST_F(AutoMixedPrecisionTest, PreserveCPUNodes) {
   EXPECT_EQ(output_view.GetNode("input")->attr().at("dtype").type(), DT_FLOAT);
   EXPECT_EQ(output_view.GetNode("clr1")->attr().at("T").type(), DT_HALF);
   EXPECT_EQ(output_view.GetNode("allow1")->attr().at("T").type(), DT_HALF);
-  EXPECT_EQ(output_view.GetNode("infer1")->attr().at("T").type(), DT_FLOAT);
+  EXPECT_EQ(output_view.GetNode("infer1")->attr().at("T").type(), DT_HALF);
   EXPECT_EQ(output_view.GetNode("allow2")->attr().at("T").type(), DT_FLOAT);
   EXPECT_EQ(output_view.GetNode("clr2")->attr().at("T").type(), DT_FLOAT);
 
@@ -511,11 +543,11 @@ TEST_F(AutoMixedPrecisionTest, PreserveCPUNodes) {
   EXPECT_EQ(tensors.size(), tensors_expected.size());
   EXPECT_EQ(tensors.size(), item.fetch.size());
   for (int i = 0; i < item.fetch.size(); ++i) {
-    test::ExpectTensorNear<float>(tensors_expected[i], tensors[i], 1e-6);
+    test::ExpectTensorNear<float>(tensors_expected[i], tensors[i], 1e-4);
   }
 }
 
-TEST_F(AutoMixedPrecisionTest, PreserveIdentityAfterVariable) {
+TEST_P(AutoMixedPrecisionParamTest, PreserveIdentityAfterVariable) {
   tensorflow::Scope s = tensorflow::Scope::NewRootScope();
   Output input = ops::Const(s.WithOpName("input"), 1.f / 32, {32, 32});
   Output var1 = ops::Variable(s.WithOpName("var1"), {32, 32}, DT_FLOAT);
@@ -535,7 +567,7 @@ TEST_F(AutoMixedPrecisionTest, PreserveIdentityAfterVariable) {
   std::vector<std::pair<string, Tensor>> feed = {{"var1", var1_tensor}};
   auto tensors_expected = EvaluateNodes(item.graph, item.fetch, feed);
 
-  AutoMixedPrecision optimizer;
+  AutoMixedPrecision optimizer(mode_);
   GraphDef output;
   TF_ASSERT_OK(optimizer.Optimize(virtual_cluster_.get(), item, &output));
 
@@ -559,7 +591,7 @@ TEST_F(AutoMixedPrecisionTest, PreserveIdentityAfterVariable) {
   }
 }
 
-TEST_F(AutoMixedPrecisionTest, FusedBatchNorm) {
+TEST_P(AutoMixedPrecisionParamTest, FusedBatchNorm) {
   tensorflow::Scope s = tensorflow::Scope::NewRootScope();
   // Uses NHWC data format because non-GPU execution does not support NCHW.
   Output input = ops::Const(s.WithOpName("input"), 1.f / 32, {8, 56, 56, 16});
@@ -592,7 +624,7 @@ TEST_F(AutoMixedPrecisionTest, FusedBatchNorm) {
   TF_CHECK_OK(s.ToGraphDef(&item.graph));
   auto tensors_expected = EvaluateNodes(item.graph, item.fetch);
 
-  AutoMixedPrecision optimizer;
+  AutoMixedPrecision optimizer(mode_);
   GraphDef output;
   TF_ASSERT_OK(optimizer.Optimize(virtual_cluster_.get(), item, &output));
 
@@ -618,7 +650,7 @@ TEST_F(AutoMixedPrecisionTest, FusedBatchNorm) {
   }
 }
 
-TEST_F(AutoMixedPrecisionTest, RepeatedAndListTypeAttrs) {
+TEST_P(AutoMixedPrecisionParamTest, RepeatedAndListTypeAttrs) {
   tensorflow::Scope s = tensorflow::Scope::NewRootScope();
   Output input = ops::Const(s.WithOpName("input"), 1.f / 32, {32, 32});
   Output allow1 = ops::MatMul(s.WithOpName("allow1"), input, input);
@@ -634,7 +666,7 @@ TEST_F(AutoMixedPrecisionTest, RepeatedAndListTypeAttrs) {
   TF_CHECK_OK(s.ToGraphDef(&item.graph));
   auto tensors_expected = EvaluateNodes(item.graph, item.fetch);
 
-  AutoMixedPrecision optimizer;
+  AutoMixedPrecision optimizer(mode_);
   GraphDef output;
   TF_ASSERT_OK(optimizer.Optimize(virtual_cluster_.get(), item, &output));
 
@@ -658,7 +690,7 @@ TEST_F(AutoMixedPrecisionTest, RepeatedAndListTypeAttrs) {
   }
 }
 
-TEST_F(AutoMixedPrecisionTest, ExistingCast) {
+TEST_P(AutoMixedPrecisionParamTest, ExistingCast) {
   tensorflow::Scope s = tensorflow::Scope::NewRootScope();
   Output input = ops::Const(s.WithOpName("input"), true, {32, 32});
   Output cst1 = ops::Cast(s.WithOpName("cst1"), input, DT_FLOAT);
@@ -670,7 +702,7 @@ TEST_F(AutoMixedPrecisionTest, ExistingCast) {
   TF_CHECK_OK(s.ToGraphDef(&item.graph));
   auto tensors_expected = EvaluateNodes(item.graph, item.fetch);
 
-  AutoMixedPrecision optimizer;
+  AutoMixedPrecision optimizer(mode_);
   GraphDef output;
   TF_ASSERT_OK(optimizer.Optimize(virtual_cluster_.get(), item, &output));
 
@@ -690,7 +722,7 @@ TEST_F(AutoMixedPrecisionTest, ExistingCast) {
   }
 }
 
-TEST_F(AutoMixedPrecisionTest, RecurrentEdgeColorMismatch) {
+TEST_P(AutoMixedPrecisionParamTest, RecurrentEdgeColorMismatch) {
   tensorflow::Scope s = tensorflow::Scope::NewRootScope();
   Output input = ops::Const(s.WithOpName("input"), 1.f / 32, {32, 32});
   Output deny1 = ops::Exp(s.WithOpName("deny1"), input);
@@ -723,7 +755,7 @@ TEST_F(AutoMixedPrecisionTest, RecurrentEdgeColorMismatch) {
   const_node->add_input("^mrg1");
   auto tensors_expected = EvaluateNodes(item.graph, item.fetch);
 
-  AutoMixedPrecision optimizer;
+  AutoMixedPrecision optimizer(mode_);
   GraphDef output;
   TF_ASSERT_OK(optimizer.Optimize(virtual_cluster_.get(), item, &output));
 
@@ -752,7 +784,7 @@ TEST_F(AutoMixedPrecisionTest, RecurrentEdgeColorMismatch) {
   }
 }
 
-TEST_F(AutoMixedPrecisionTest, TensorListSetGet) {
+TEST_P(AutoMixedPrecisionParamTest, TensorListSetGet) {
   tensorflow::Scope s = tensorflow::Scope::NewRootScope();
   tensorflow::Input shape = {32, 32};
   auto tl1 = ops::TensorListReserve(s.WithOpName("tl1"), {32, 32}, 8, DT_FLOAT);
@@ -794,7 +826,7 @@ TEST_F(AutoMixedPrecisionTest, TensorListSetGet) {
   TF_CHECK_OK(s.ToGraphDef(&item.graph));
   auto tensors_expected = EvaluateNodes(item.graph, item.fetch);
 
-  AutoMixedPrecision optimizer;
+  AutoMixedPrecision optimizer(mode_);
   GraphDef output;
   TF_ASSERT_OK(optimizer.Optimize(virtual_cluster_.get(), item, &output));
 
@@ -823,7 +855,7 @@ TEST_F(AutoMixedPrecisionTest, TensorListSetGet) {
   }
 }
 
-TEST_F(AutoMixedPrecisionTest, TensorListPushPop) {
+TEST_P(AutoMixedPrecisionParamTest, TensorListPushPop) {
   tensorflow::Scope s = tensorflow::Scope::NewRootScope();
   tensorflow::Input shape = {32, 32};
   auto tl1 = ops::EmptyTensorList(s.WithOpName("tl1"), {32, 32}, 8, DT_FLOAT);
@@ -857,7 +889,7 @@ TEST_F(AutoMixedPrecisionTest, TensorListPushPop) {
   TF_CHECK_OK(s.ToGraphDef(&item.graph));
   auto tensors_expected = EvaluateNodes(item.graph, item.fetch);
 
-  AutoMixedPrecision optimizer;
+  AutoMixedPrecision optimizer(mode_);
   GraphDef output;
   TF_ASSERT_OK(optimizer.Optimize(virtual_cluster_.get(), item, &output));
 
@@ -886,7 +918,7 @@ TEST_F(AutoMixedPrecisionTest, TensorListPushPop) {
   }
 }
 
-TEST_F(AutoMixedPrecisionTest, TensorListFromTensor) {
+TEST_P(AutoMixedPrecisionParamTest, TensorListFromTensor) {
   tensorflow::Scope s = tensorflow::Scope::NewRootScope();
   tensorflow::Input shape = {32};
   Output input = ops::Const(s.WithOpName("input"), 1.f / 32, {32, 32});
@@ -911,7 +943,7 @@ TEST_F(AutoMixedPrecisionTest, TensorListFromTensor) {
   TF_CHECK_OK(s.ToGraphDef(&item.graph));
   auto tensors_expected = EvaluateNodes(item.graph, item.fetch);
 
-  AutoMixedPrecision optimizer;
+  AutoMixedPrecision optimizer(mode_);
   GraphDef output;
   TF_ASSERT_OK(optimizer.Optimize(virtual_cluster_.get(), item, &output));
 
@@ -932,11 +964,11 @@ TEST_F(AutoMixedPrecisionTest, TensorListFromTensor) {
   EXPECT_EQ(tensors.size(), tensors_expected.size());
   EXPECT_EQ(tensors.size(), item.fetch.size());
   for (int i = 0; i < item.fetch.size(); ++i) {
-    test::ExpectClose(tensors_expected[i], tensors[i], -1, 2e-4);
+    test::ExpectClose(tensors_expected[i], tensors[i], -1, 4e-4);
   }
 }
 
-TEST_F(AutoMixedPrecisionTest, TensorListPushBackBatchAndConcatLists) {
+TEST_P(AutoMixedPrecisionParamTest, TensorListPushBackBatchAndConcatLists) {
   tensorflow::Scope s = tensorflow::Scope::NewRootScope();
   tensorflow::Input shape = {32, 32};
   auto tl1 = ops::EmptyTensorList(s.WithOpName("tl1"), {32, 32}, 8, DT_FLOAT);
@@ -971,7 +1003,7 @@ TEST_F(AutoMixedPrecisionTest, TensorListPushBackBatchAndConcatLists) {
   TF_CHECK_OK(s.ToGraphDef(&item.graph));
   auto tensors_expected = EvaluateNodes(item.graph, item.fetch);
 
-  AutoMixedPrecision optimizer;
+  AutoMixedPrecision optimizer(mode_);
   GraphDef output;
   TF_ASSERT_OK(optimizer.Optimize(virtual_cluster_.get(), item, &output));
 
@@ -996,7 +1028,7 @@ TEST_F(AutoMixedPrecisionTest, TensorListPushBackBatchAndConcatLists) {
   }
 }
 
-TEST_F(AutoMixedPrecisionTest, TensorListThroughFunction) {
+TEST_P(AutoMixedPrecisionParamTest, TensorListThroughFunction) {
   // This test passes a tensor list handle through a function with its own
   // Tensor List ops inside to test that the types are not changed to a
   // conflicting state.
@@ -1057,7 +1089,7 @@ TEST_F(AutoMixedPrecisionTest, TensorListThroughFunction) {
   TF_CHECK_OK(s.ToGraphDef(&item.graph));
   auto tensors_expected = EvaluateNodes(item.graph, item.fetch);
 
-  AutoMixedPrecision optimizer;
+  AutoMixedPrecision optimizer(mode_);
   GraphDef output;
   TF_ASSERT_OK(optimizer.Optimize(virtual_cluster_.get(), item, &output));
 
@@ -1104,7 +1136,7 @@ bool IsSupportedGPU(const Cluster& cluster) {
 #endif
 }
 
-TEST_F(AutoMixedPrecisionTest, BatchMatMul) {
+TEST_P(AutoMixedPrecisionParamTest, BatchMatMul) {
   tensorflow::Scope s = tensorflow::Scope::NewRootScope();
   Output input = ops::Const(s.WithOpName("input"), 1.f / 33, {64, 32, 32});
   Output allow1 = ops::BatchMatMul(s.WithOpName("allow1"), input, input);
@@ -1115,7 +1147,7 @@ TEST_F(AutoMixedPrecisionTest, BatchMatMul) {
   TF_CHECK_OK(s.ToGraphDef(&item.graph));
   auto tensors_expected = EvaluateNodes(item.graph, item.fetch);
 
-  AutoMixedPrecision optimizer;
+  AutoMixedPrecision optimizer(mode_);
   GraphDef output;
   TF_ASSERT_OK(optimizer.Optimize(virtual_cluster_.get(), item, &output));
 
@@ -1139,7 +1171,7 @@ TEST_F(AutoMixedPrecisionTest, BatchMatMul) {
   }
 }
 
-TEST_F(AutoMixedPrecisionTest, EluOp) {
+TEST_P(AutoMixedPrecisionParamTest, EluOp) {
   TestSimpleUnaryInferOp(
       -5, 5, 1.0e-3, 1.0e-3,
       [](const tensorflow::Scope& scope, Output input) -> Output {
@@ -1147,7 +1179,7 @@ TEST_F(AutoMixedPrecisionTest, EluOp) {
       });
 }
 
-TEST_F(AutoMixedPrecisionTest, ErfOp) {
+TEST_P(AutoMixedPrecisionParamTest, ErfOp) {
   TestSimpleUnaryInferOp(
       -5, 5, 1.0e-3, -1,
       [](const tensorflow::Scope& scope, Output input) -> Output {
@@ -1155,7 +1187,7 @@ TEST_F(AutoMixedPrecisionTest, ErfOp) {
       });
 }
 
-TEST_F(AutoMixedPrecisionTest, ErfcOp) {
+TEST_P(AutoMixedPrecisionParamTest, ErfcOp) {
   TestSimpleUnaryInferOp(
       -5, 5, 1.0e-3, -1,
       [](const tensorflow::Scope& scope, Output input) -> Output {
@@ -1163,7 +1195,7 @@ TEST_F(AutoMixedPrecisionTest, ErfcOp) {
       });
 }
 
-TEST_F(AutoMixedPrecisionTest, InvOp) {
+TEST_P(AutoMixedPrecisionParamTest, InvOp) {
   TestSimpleUnaryInferOp(
       0.01, 10, -1, 1.0e-3,
       [](const tensorflow::Scope& scope, Output input) -> Output {
@@ -1171,7 +1203,7 @@ TEST_F(AutoMixedPrecisionTest, InvOp) {
       });
 }
 
-TEST_F(AutoMixedPrecisionTest, LogOp) {
+TEST_P(AutoMixedPrecisionParamTest, LogOp) {
   TestSimpleUnaryInferOp(
       0.01, 10, 1.0e-3, 2.0e-3,
       [](const tensorflow::Scope& scope, Output input) -> Output {
@@ -1179,7 +1211,7 @@ TEST_F(AutoMixedPrecisionTest, LogOp) {
       });
 }
 
-TEST_F(AutoMixedPrecisionTest, Log1pOp) {
+TEST_P(AutoMixedPrecisionParamTest, Log1pOp) {
   TestSimpleUnaryInferOp(
       -0.99, 9, 1.0e-3, 5.0e-3,
       [](const tensorflow::Scope& scope, Output input) -> Output {
@@ -1187,7 +1219,7 @@ TEST_F(AutoMixedPrecisionTest, Log1pOp) {
       });
 }
 
-TEST_F(AutoMixedPrecisionTest, LogSoftmaxOp) {
+TEST_P(AutoMixedPrecisionParamTest, LogSoftmaxOp) {
   TestSimpleUnaryInferOp(
       -8, 8, -1, 1.0e-2,
       [](const tensorflow::Scope& scope, Output input) -> Output {
@@ -1195,7 +1227,7 @@ TEST_F(AutoMixedPrecisionTest, LogSoftmaxOp) {
       });
 }
 
-TEST_F(AutoMixedPrecisionTest, ReciprocalOp) {
+TEST_P(AutoMixedPrecisionParamTest, ReciprocalOp) {
   TestSimpleUnaryInferOp(
       0.01, 10, -1, 1.0e-3,
       [](const tensorflow::Scope& scope, Output input) -> Output {
@@ -1203,7 +1235,7 @@ TEST_F(AutoMixedPrecisionTest, ReciprocalOp) {
       });
 }
 
-TEST_F(AutoMixedPrecisionTest, SigmoidOp) {
+TEST_P(AutoMixedPrecisionParamTest, SigmoidOp) {
   TestSimpleUnaryInferOp(
       -5, 5, 1.0e-3, -1,
       [](const tensorflow::Scope& scope, Output input) -> Output {
@@ -1211,7 +1243,7 @@ TEST_F(AutoMixedPrecisionTest, SigmoidOp) {
       });
 }
 
-TEST_F(AutoMixedPrecisionTest, SoftmaxOp) {
+TEST_P(AutoMixedPrecisionParamTest, SoftmaxOp) {
   TestSimpleUnaryInferOp(
       -8, 8, 2.0e-3, -1,
       [](const tensorflow::Scope& scope, Output input) -> Output {
@@ -1219,7 +1251,7 @@ TEST_F(AutoMixedPrecisionTest, SoftmaxOp) {
       });
 }
 
-TEST_F(AutoMixedPrecisionTest, SoftplusOp) {
+TEST_P(AutoMixedPrecisionParamTest, SoftplusOp) {
   TestSimpleUnaryInferOp(
       -5, 5, 2.0e-3, 2.0e-3,
       [](const tensorflow::Scope& scope, Output input) -> Output {
@@ -1227,7 +1259,7 @@ TEST_F(AutoMixedPrecisionTest, SoftplusOp) {
       });
 }
 
-TEST_F(AutoMixedPrecisionTest, SqrtOp) {
+TEST_P(AutoMixedPrecisionParamTest, SqrtOp) {
   TestSimpleUnaryInferOp(
       0, 10, 1.0e-3, 1.0e-3,
       [](const tensorflow::Scope& scope, Output input) -> Output {
@@ -1235,7 +1267,7 @@ TEST_F(AutoMixedPrecisionTest, SqrtOp) {
       });
 }
 
-TEST_F(AutoMixedPrecisionTest, TanhOp) {
+TEST_P(AutoMixedPrecisionParamTest, TanhOp) {
   TestSimpleUnaryInferOp(
       -5, 5, 1.0e-3, -1,
       [](const tensorflow::Scope& scope, Output input) -> Output {
@@ -1243,6 +1275,19 @@ TEST_F(AutoMixedPrecisionTest, TanhOp) {
       });
 }
 
+constexpr AutoMixedPrecisionMode kTestValues[] = {
+#if GOOGLE_CUDA || TENSORFLOW_USE_ROCM
+    AutoMixedPrecisionMode::CUDA,
+#endif
+#if INTEL_MKL
+    AutoMixedPrecisionMode::FP16_CPU,
+#endif
+};
+
+INSTANTIATE_TEST_SUITE_P(AutoMixedPrecisionTest, AutoMixedPrecisionParamTest,
+                         ::testing::ValuesIn(kTestValues));
+
+#if GOOGLE_CUDA || TENSORFLOW_USE_ROCM
 class AutoMixedPrecisionCpuTest : public GrapplerTest {
  protected:
   void SetUp() override {
@@ -1461,10 +1506,13 @@ TEST_F(AutoMixedPrecisionSimulateGpuTest, Simple_SimulatedGpu_CpuScope) {
 
 class AutoMixedPrecisionMklTest : public GrapplerTest {
  protected:
+  void SetMode(AutoMixedPrecisionMode mode) { mode_ = mode; }
   void SetUp() override {
     virtual_cluster_.reset(new SingleMachine(/* timeout_s = */ 10, 1, 0));
     TF_CHECK_OK(virtual_cluster_->Provision());
   }
+  AutoMixedPrecisionMode mode_;
+
   void TearDown() override { TF_CHECK_OK(virtual_cluster_->Shutdown()); }
 
   std::unique_ptr<Cluster> virtual_cluster_;
@@ -1636,9 +1684,34 @@ TEST_F(AutoMixedPrecisionMklTest, TensorListSetGet) {
   }
 }
 
-TEST_F(AutoMixedPrecisionMklTest, InferFollowUpStreamAllow) {
-  if (!IsMKLEnabled())
+class AutoMixedPrecisionMklParamTest
+    : public AutoMixedPrecisionMklTest,
+      public ::testing::WithParamInterface<AutoMixedPrecisionMode> {
+ protected:
+  void SetUp() override {
+    mode_ = GetParam();
+    if (mode_ == AutoMixedPrecisionMode::BF16) {
+      dtype_ = DT_BFLOAT16;
+    } else if (mode_ == AutoMixedPrecisionMode::FP16_CPU) {
+      dtype_ = DT_HALF;
+      // oneDNN supports FP16 on some platforms by converting to and from FP32
+      is_fp16_supported = IsAMXDataTypeSupportedByOneDNNOnThisCPU(DT_HALF) ||
+                          IsAVXConvertSupportedByOneDNNOnThisCPU();
+    } else {
+      GTEST_SKIP() << "Mode not supported on this device";
+    }
+
+    AutoMixedPrecisionMklTest::SetMode(mode_);
+    AutoMixedPrecisionMklTest::SetUp();
+  }
+  DataType dtype_;
+  bool is_fp16_supported;
+};
+
+TEST_P(AutoMixedPrecisionMklParamTest, InferFollowUpStreamAllow) {
+  if (!IsMKLEnabled() || (dtype_ == DT_HALF && !is_fp16_supported))
     GTEST_SKIP() << "Test only applicable to MKL auto-mixed precision.";
+
   tensorflow::Scope s = tensorflow::Scope::NewRootScope().WithDevice(
       "/job:localhost/replica:0/task:0/device:CPU:0");
   Output input1 = ops::Const(s.WithOpName("input1"), 1.f / 32, {8, 56, 56, 16});
@@ -1656,7 +1729,7 @@ TEST_F(AutoMixedPrecisionMklTest, InferFollowUpStreamAllow) {
   TF_CHECK_OK(s.ToGraphDef(&item.graph));
   auto tensors_expected = EvaluateNodes(item.graph, item.fetch);
 
-  AutoMixedPrecision optimizer{AutoMixedPrecisionMode::BF16};
+  AutoMixedPrecision optimizer{mode_};
   GraphDef output;
   TF_ASSERT_OK(optimizer.Optimize(virtual_cluster_.get(), item, &output));
 
@@ -1667,9 +1740,9 @@ TEST_F(AutoMixedPrecisionMklTest, InferFollowUpStreamAllow) {
   EXPECT_EQ(output_view.GetNode("input1")->attr().at("dtype").type(), DT_FLOAT);
   EXPECT_EQ(output_view.GetNode("weight")->attr().at("dtype").type(), DT_FLOAT);
   EXPECT_EQ(output_view.GetNode("input2")->attr().at("dtype").type(), DT_FLOAT);
-  EXPECT_EQ(output_view.GetNode("allow")->attr().at("T").type(), DT_BFLOAT16);
-  EXPECT_EQ(output_view.GetNode("infer")->attr().at("T").type(), DT_BFLOAT16);
-  EXPECT_EQ(output_view.GetNode("clr")->attr().at("T").type(), DT_BFLOAT16);
+  EXPECT_EQ(output_view.GetNode("allow")->attr().at("T").type(), dtype_);
+  EXPECT_EQ(output_view.GetNode("infer")->attr().at("T").type(), dtype_);
+  EXPECT_EQ(output_view.GetNode("clr")->attr().at("T").type(), dtype_);
 
   auto tensors = EvaluateNodes(output, item.fetch);
   EXPECT_EQ(tensors.size(), tensors_expected.size());
@@ -1679,9 +1752,10 @@ TEST_F(AutoMixedPrecisionMklTest, InferFollowUpStreamAllow) {
   }
 }
 
-TEST_F(AutoMixedPrecisionMklTest, InferFollowUpStreamDeny) {
-  if (!IsMKLEnabled())
+TEST_P(AutoMixedPrecisionMklParamTest, InferFollowUpStreamDeny) {
+  if (!IsMKLEnabled() || (dtype_ == DT_HALF && !is_fp16_supported))
     GTEST_SKIP() << "Test only applicable to MKL auto-mixed precision.";
+
   tensorflow::Scope s = tensorflow::Scope::NewRootScope().WithDevice(
       "/job:localhost/replica:0/task:0/device:CPU:0");
   Output input1 = ops::Const(s.WithOpName("input1"), 1.f / 32, {8, 56, 56, 16});
@@ -1697,7 +1771,7 @@ TEST_F(AutoMixedPrecisionMklTest, InferFollowUpStreamDeny) {
   TF_CHECK_OK(s.ToGraphDef(&item.graph));
   auto tensors_expected = EvaluateNodes(item.graph, item.fetch);
 
-  AutoMixedPrecision optimizer{AutoMixedPrecisionMode::BF16};
+  AutoMixedPrecision optimizer{mode_};
   GraphDef output;
   TF_ASSERT_OK(optimizer.Optimize(virtual_cluster_.get(), item, &output));
 
@@ -1719,6 +1793,15 @@ TEST_F(AutoMixedPrecisionMklTest, InferFollowUpStreamDeny) {
     test::ExpectClose(tensors_expected[i], tensors[i]);
   }
 }
+
+constexpr AutoMixedPrecisionMode kMklTestValues[] = {
+    AutoMixedPrecisionMode::BF16,
+    AutoMixedPrecisionMode::FP16_CPU,
+};
+INSTANTIATE_TEST_SUITE_P(AutoMixedPrecisionMklTest,
+                         AutoMixedPrecisionMklParamTest,
+                         ::testing::ValuesIn(kMklTestValues));
+
 #endif  // INTEL_MKL
 
 }  // namespace
