@@ -15,6 +15,11 @@ limitations under the License.
 
 #include "tensorflow/core/grappler/optimizers/data/make_deterministic.h"
 
+#include <cstdint>
+#include <string>
+#include <tuple>
+#include <vector>
+
 #include "tensorflow/core/framework/attr_value_util.h"
 #include "tensorflow/core/framework/common_shape_fns.h"
 #include "tensorflow/core/framework/function_testlib.h"
@@ -85,7 +90,7 @@ TEST_P(SplitMapTest, SplitMapFunction) {
   } else {
     orig_map_node_def = graph_tests_utils::MakeParallelMapV2Node(
         "map", "range", "num_parallel_calls", "MyFunction",
-        deterministic ? "true" : "false");
+        deterministic ? "true" : "false", /*use_unbounded_threadpool=*/false);
   }
   orig_map_node_def.add_input("^start");
   AttrValue* attr_val = &(*orig_map_node_def.mutable_attr())["Targuments"];
@@ -321,7 +326,8 @@ TEST_P(MakeDeterministicTest, NoRewriteMap) {
             {{"value", 1}, {"dtype", DT_INT32}}),
        graph_tests_utils::MakeParallelMapV2Node(
            "map", "range", "num_parallel_calls", func_name,
-           deterministic ? "true" : "false")},
+           deterministic ? "true" : "false",
+           /*use_unbounded_threadpool=*/false)},
       // FunctionLib
       {test::function::XTimesTwo(), OuterXTimesTwo()});
 
@@ -387,7 +393,8 @@ TEST_P(MakeDeterministicTest, NoRewritePrefetch) {
             {{"value", Tensor(int64_t{1})}, {"dtype", DT_INT64}}),
        graph_tests_utils::MakeParallelMapV2Node(
            "map", "range", "num_parallel_calls", func_name,
-           deterministic ? "true" : "false"),
+           deterministic ? "true" : "false",
+           /*use_unbounded_threadpool=*/false),
        graph_tests_utils::MakePrefetchNode("prefetch", "map", "buffer_size")},
       // FunctionLib
       {test::function::RandomUniform(), OuterRandomUniform()});
@@ -485,7 +492,7 @@ TEST_P(RewriteMapWithoutSplitTest, RewriteMapWithoutSplit) {
 
   NodeDef map_node_def = graph_tests_utils::MakeParallelMapV2Node(
       "map", "range", "num_parallel_calls", func_name,
-      deterministic ? "true" : "false");
+      deterministic ? "true" : "false", /*use_unbounded_threadpool=*/false);
   map_node_def.add_input("^start");
 
   // Rewrite occurs due to parallelism in map function
@@ -587,7 +594,8 @@ TEST_P(MakeDeterministicTest, RewritePrefetch) {
             {{"value", Tensor(int64_t{1})}, {"dtype", DT_INT64}}),
        graph_tests_utils::MakeParallelMapV2Node(
            "map", "range", "num_parallel_calls", func_name,
-           deterministic ? "true" : "false"),
+           deterministic ? "true" : "false",
+           /*use_unbounded_threadpool=*/false),
        graph_tests_utils::MakePrefetchNode("prefetch", "map", "buffer_size")},
       // FunctionLib
       {test::function::ReadResourceVariable(), OuterReadResourceVariable()});
@@ -672,7 +680,7 @@ TEST_P(RewriteMapAndBatchWithoutSplitTest, RewriteMapAndBatchWithoutSplit) {
       "map_and_batch", "range", "batch_size", "num_parallel_calls",
       "drop_remainder", func.signature().name());
   SetAttrValue(
-      gtl::ArraySlice<PartialTensorShape>{
+      absl::Span<const PartialTensorShape>{
           {2}, {-1, 3, -1}, PartialTensorShape()},
       &(*map_and_batch_node_def.mutable_attr())["output_shapes"]);
 
