@@ -20,10 +20,11 @@ limitations under the License.
 namespace tensorflow {
 namespace grappler {
 
-Status TFDataOptimizerBase::Optimize(Cluster* cluster, const GrapplerItem& item,
-                                     GraphDef* output) {
+absl::Status TFDataOptimizerBase::Optimize(Cluster* cluster,
+                                           const GrapplerItem& item,
+                                           GraphDef* output) {
   OptimizationStats stats;
-  Status s = OptimizeAndCollectStats(cluster, item, output, &stats);
+  absl::Status s = OptimizeAndCollectStats(cluster, item, output, &stats);
   if (s.ok() && stats.num_changes > 0) {
     metrics::RecordTFDataOptimization(name(), stats.num_changes);
   }
