@@ -55,23 +55,24 @@ namespace tensorflow {
 namespace grappler {
 namespace {
 
-using TensorVector = gtl::InlinedVector<TensorValue, 4>;
+using TensorVector = absl::InlinedVector<TensorValue, 4UL>;
 
 class LoopInvariantNodeMotionOptimizer {
  public:
   explicit LoopInvariantNodeMotionOptimizer(GraphDef* optimized_graph)
       : optimized_graph_(optimized_graph) {}
   virtual ~LoopInvariantNodeMotionOptimizer() = default;
-  Status Optimize();
+  absl::Status Optimize();
 
  private:
-  Status FindInvariantNodes(NodeDef* node);
-  Status RevertInvariantNodes();
-  Status MoveInvariantNodes(const int frame_id);
-  Status HandleInvariantNode(NodeDef* node, const int num_outputs,
-                             const int frame_id);
-  Status HandleConst(NodeDef* node, const int num_outputs, const int frame_id);
-  Status HandleInvariantEnter(NodeDef* node, const int num_outputs);
+  absl::Status FindInvariantNodes(NodeDef* node);
+  absl::Status RevertInvariantNodes();
+  absl::Status MoveInvariantNodes(const int frame_id);
+  absl::Status HandleInvariantNode(NodeDef* node, const int num_outputs,
+                                   const int frame_id);
+  absl::Status HandleConst(NodeDef* node, const int num_outputs,
+                           const int frame_id);
+  absl::Status HandleInvariantEnter(NodeDef* node, const int num_outputs);
 
   GraphDef* optimized_graph_;  // Not owned.
   std::unique_ptr<NodeMap> node_map_;
@@ -84,7 +85,7 @@ class LoopInvariantNodeMotionOptimizer {
   int new_enter_id_;
 };
 
-Status LoopInvariantNodeMotionOptimizer::HandleInvariantEnter(
+absl::Status LoopInvariantNodeMotionOptimizer::HandleInvariantEnter(
     NodeDef* node, const int num_outputs) {
   auto consumers = node_map_->GetOutputs(node->name());
   std::vector<string> enter_control_inputs;
@@ -111,12 +112,11 @@ Status LoopInvariantNodeMotionOptimizer::HandleInvariantEnter(
       }
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status LoopInvariantNodeMotionOptimizer::HandleConst(NodeDef* node,
-                                                     const int num_outputs,
-                                                     const int frame_id) {
+absl::Status LoopInvariantNodeMotionOptimizer::HandleConst(
+    NodeDef* node, const int num_outputs, const int frame_id) {
   NodeDef* const_node = nullptr;
   if (num_outputs == 0) {
     // all successor nodes are invariant
@@ -183,10 +183,10 @@ Status LoopInvariantNodeMotionOptimizer::HandleConst(NodeDef* node,
     const_node->add_input(ctrl_dep);
     node_map_->AddOutput(NodeName(ctrl_dep), const_node->name());
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status LoopInvariantNodeMotionOptimizer::HandleInvariantNode(
+absl::Status LoopInvariantNodeMotionOptimizer::HandleInvariantNode(
     NodeDef* node, const int num_outputs, const int frame_id) {
   // have to remove control inputs to the invariant node from the same frame
   // when moving this node out of this frame
@@ -197,7 +197,7 @@ Status LoopInvariantNodeMotionOptimizer::HandleInvariantNode(
     }
   }
   if (num_outputs == 0) {
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   DataTypeVector input_types;
@@ -252,10 +252,10 @@ Status LoopInvariantNodeMotionOptimizer::HandleInvariantNode(
       }
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status LoopInvariantNodeMotionOptimizer::MoveInvariantNodes(
+absl::Status LoopInvariantNodeMotionOptimizer::MoveInvariantNodes(
     const int frame_id) {
   for (auto iter = invariant_nodes_.begin(); iter != invariant_nodes_.end();
        ++iter) {
@@ -270,10 +270,10 @@ Status LoopInvariantNodeMotionOptimizer::MoveInvariantNodes(
           HandleInvariantNode(invariant_node, num_outputs, frame_id));
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status LoopInvariantNodeMotionOptimizer::RevertInvariantNodes() {
+absl::Status LoopInvariantNodeMotionOptimizer::RevertInvariantNodes() {
   std::deque<const NodeDef*> reverted_nodes;
   for (auto iter = invariant_nodes_.begin(); iter != invariant_nodes_.end();) {
     bool erased = false;
@@ -327,10 +327,10 @@ Status LoopInvariantNodeMotionOptimizer::RevertInvariantNodes() {
       }
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status LoopInvariantNodeMotionOptimizer::FindInvariantNodes(
+absl::Status LoopInvariantNodeMotionOptimizer::FindInvariantNodes(
     NodeDef* start_node) {
   std::vector<NodeDef*> stack;
   stack.reserve(32);
@@ -376,10 +376,10 @@ Status LoopInvariantNodeMotionOptimizer::FindInvariantNodes(
       }
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status LoopInvariantNodeMotionOptimizer::Optimize() {
+absl::Status LoopInvariantNodeMotionOptimizer::Optimize() {
   node_map_.reset(new NodeMap(optimized_graph_));
   FrameView frame_view;
   // TODO(ezhulenev): Use GraphView when migrated from NodeMap.
@@ -450,7 +450,7 @@ Status LoopInvariantNodeMotionOptimizer::Optimize() {
 
     TF_RETURN_IF_ERROR(MoveInvariantNodes(frame_id));
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 std::vector<int> GetStackPushNodesToConvert(
@@ -516,8 +516,8 @@ std::vector<int> GetStackPushNodesToConvert(
   return nodes_to_convert;
 }
 
-Status RemoveStackOps(const std::unordered_set<string>& nodes_to_preserve,
-                      GraphDef* optimized_graph) {
+absl::Status RemoveStackOps(const std::unordered_set<string>& nodes_to_preserve,
+                            GraphDef* optimized_graph) {
   NodeMap node_map(optimized_graph);
   GraphTopologyView graph_view;
   TF_RETURN_IF_ERROR(graph_view.InitializeFromGraph(*optimized_graph));
@@ -544,7 +544,7 @@ Status RemoveStackOps(const std::unordered_set<string>& nodes_to_preserve,
       }
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 bool IsSimpleBinaryOperator(const NodeDef& node) {
@@ -552,12 +552,10 @@ bool IsSimpleBinaryOperator(const NodeDef& node) {
           IsGreaterEqual(node) || IsEqual(node));
 }
 
-Status EvaluateBoolOpForConstantOperands(const NodeDef& op_node,
-                                         const NodeDef& constant_operand_0,
-                                         const NodeDef& constant_operand_1,
-                                         DeviceBase* cpu_device,
-                                         ResourceMgr* resource_mgr,
-                                         bool* value) {
+absl::Status EvaluateBoolOpForConstantOperands(
+    const NodeDef& op_node, const NodeDef& constant_operand_0,
+    const NodeDef& constant_operand_1, DeviceBase* cpu_device,
+    ResourceMgr* resource_mgr, bool* value) {
   VLOG(4) << "Evaluate bool op: op_node=" << op_node.name()
           << " input0=" << constant_operand_0.name()
           << " input1=" << constant_operand_1.name();
@@ -577,12 +575,13 @@ Status EvaluateBoolOpForConstantOperands(const NodeDef& op_node,
       EvaluateNode(op_node, inputs, cpu_device, resource_mgr, &outputs));
 
   if (outputs.size() != 1 || outputs[0].tensor == nullptr) {
-    return Status(absl::StatusCode::kInvalidArgument, "Expected one output.");
+    return absl::Status(absl::StatusCode::kInvalidArgument,
+                        "Expected one output.");
   }
   *value = outputs[0].tensor->scalar<bool>()();
   delete outputs[0].tensor;
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // TODO(lyandy): Consolidate with ConstantFolding implementation.
@@ -595,11 +594,13 @@ bool IsReallyConstant(const NodeDef& node,
   return feed_nodes.find(node.name()) == feed_nodes.end();
 }
 
-Status CheckForDeadFanout(const MutableGraphView& view,
-                          const NodeDef& switch_node, const NodeMap& node_map,
-                          const absl::flat_hash_set<string>& feed_nodes,
-                          DeviceBase* cpu_device, ResourceMgr* resource_mgr,
-                          bool* has_dead_fanout, int* dead_fanout) {
+absl::Status CheckForDeadFanout(const MutableGraphView& view,
+                                const NodeDef& switch_node,
+                                const NodeMap& node_map,
+                                const absl::flat_hash_set<string>& feed_nodes,
+                                DeviceBase* cpu_device,
+                                ResourceMgr* resource_mgr,
+                                bool* has_dead_fanout, int* dead_fanout) {
   *has_dead_fanout = false;
   GraphView::InputPort switch_loopcond_port(&switch_node, 1);
   const NodeDef* switch_predicate =
@@ -614,7 +615,7 @@ Status CheckForDeadFanout(const MutableGraphView& view,
     CHECK(selector.FromProto(switch_predicate->attr().at("value").tensor()));
     *has_dead_fanout = true;
     *dead_fanout = selector.scalar<bool>()() ? 0 : 1;
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   GraphView::InputPort switch_input_port(&switch_node, 0);
@@ -625,7 +626,7 @@ Status CheckForDeadFanout(const MutableGraphView& view,
   // operator which returns false for the initialization value.
   // TODO(srjoglekar): Improve to work with arbitrary predicate subgraphs.
   if (!IsMerge(*switch_input) || !IsLoopCond(*switch_predicate)) {
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   VLOG(4) << "Try to find a zero iteration while loop:"
@@ -634,7 +635,7 @@ Status CheckForDeadFanout(const MutableGraphView& view,
   // Find the boolean predicate from a LoopCond node (e.g. Greater).
   NodeDef* switch_ctrl_node = view.GetRegularFanin({switch_predicate, 0}).node;
   if (!switch_ctrl_node || !IsSimpleBinaryOperator(*switch_ctrl_node)) {
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   // Find the Merge node & the Constant Operand to the condition node, if
@@ -656,7 +657,7 @@ Status CheckForDeadFanout(const MutableGraphView& view,
     }
   }
   if (merge_node == nullptr || constant_ctrl_input == nullptr) {
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   // Find the initialization constant (via Enter, if one exists).
@@ -672,7 +673,7 @@ Status CheckForDeadFanout(const MutableGraphView& view,
     }
   }
   if (enter_node != nullptr) {
-    if (constant_init_node != nullptr) return OkStatus();
+    if (constant_init_node != nullptr) return absl::OkStatus();
     for (const auto& input : enter_node->input()) {
       NodeDef* node = node_map.GetNode(input);
       if (IsReallyConstant(*node, feed_nodes)) {
@@ -681,7 +682,7 @@ Status CheckForDeadFanout(const MutableGraphView& view,
     }
   }
   if (constant_init_node == nullptr) {
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   VLOG(4) << "Check if loop will be 0 iterations:"
@@ -712,7 +713,7 @@ Status CheckForDeadFanout(const MutableGraphView& view,
   } else {
     VLOG(4) << "Was not able to prove that loop has 0 iterations.";
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace
@@ -730,8 +731,8 @@ LoopOptimizer::LoopOptimizer(RewriterConfig::Toggle opt_level,
   resource_mgr_.reset(new ResourceMgr());
 }
 
-Status LoopOptimizer::Optimize(Cluster* cluster, const GrapplerItem& item,
-                               GraphDef* optimized_graph) {
+absl::Status LoopOptimizer::Optimize(Cluster* cluster, const GrapplerItem& item,
+                                     GraphDef* optimized_graph) {
   if (!options_.enable_loop_invariant_node_motion &&
       !options_.enable_stack_push_removal &&
       !options_.enable_dead_branch_removal) {
@@ -756,13 +757,13 @@ Status LoopOptimizer::Optimize(Cluster* cluster, const GrapplerItem& item,
                                           feed_nodes, optimized_graph));
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // An Identity node has only 1 output, but Switch and Merge nodes have 2.
 // Update full type information (which describes the output) if present, i.e.
 // do simple type inference.
-static Status update_identity_node_type(NodeDef* sw_node) {
+static absl::Status update_identity_node_type(NodeDef* sw_node) {
   if (sw_node->has_experimental_type() &&
       (sw_node->experimental_type().type_id() == TFT_PRODUCT)) {
     FullTypeDef old_t = sw_node->experimental_type();
@@ -781,10 +782,10 @@ static Status update_identity_node_type(NodeDef* sw_node) {
     *(new_t.add_args()) = old_t.args()[0];
     *(sw_node->mutable_experimental_type()) = new_t;
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status LoopOptimizer::RemoveDeadBranches(
+absl::Status LoopOptimizer::RemoveDeadBranches(
     const std::unordered_set<string>& nodes_to_preserve, NodeMap& node_map,
     const absl::flat_hash_set<string>& feed_nodes, GraphDef* optimized_graph) {
   std::unordered_set<const NodeDef*> dead_nodes;
@@ -933,14 +934,14 @@ Status LoopOptimizer::RemoveDeadBranches(
       LOG(WARNING)
           << "Skipping loop optimization for Merge node with control input: "
           << merge_node->name();
-      return OkStatus();
+      return absl::OkStatus();
     } else if (dead_inputs.size() != 1 || num_data_inputs != 2) {
       LOG(WARNING) << "Skipping loop optimization for Merge node ("
                    << merge_node->name()
                    << ") with unexpected dead_inputs.size() ("
                    << dead_inputs.size() << " or  num_data_inputs"
                    << num_data_inputs;
-      return OkStatus();
+      return absl::OkStatus();
     }
   }
 
@@ -1013,7 +1014,7 @@ Status LoopOptimizer::RemoveDeadBranches(
   }
   EraseNodesFromGraph(std::move(nodes_idx_to_delete), optimized_graph);
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 }  // end namespace grappler
