@@ -13,7 +13,7 @@ See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
 
-#if defined(INTEL_MKL) && defined(ENABLE_MKL)
+#if defined(INTEL_MKL)
 #include "tensorflow/cc/ops/nn_ops_internal.h"
 #include "tensorflow/cc/ops/standard_ops.h"
 #include "tensorflow/core/framework/tensor_testutil.h"
@@ -24,6 +24,7 @@ limitations under the License.
 #include "tensorflow/core/lib/core/status_test_util.h"
 #include "tensorflow/core/platform/test.h"
 #include "tensorflow/core/util/mkl_util.h"
+#include "tensorflow/core/util/util.h"
 
 namespace tensorflow {
 namespace grappler {
@@ -38,8 +39,11 @@ class MklRemapperTest : public GrapplerTest {
   void FuseConv2DWithBiasAndAddNOrAdd(const string& data_format,
                                       const string& activation, string add_op,
                                       bool add_with_bcast) {
+#ifdef DNNL_AARCH64_USE_ACL
+    GTEST_SKIP() << "Skipping test due to different behaviour on AARCH64";
+#endif
+    if (!IsMKLEnabled()) GTEST_SKIP() << "Test only applicable to oneDNN.";
     using ::tensorflow::ops::Placeholder;
-
     tensorflow::Scope s = tensorflow::Scope::NewRootScope();
 
     auto input_shape = (data_format == "NHWC")
@@ -215,8 +219,10 @@ CREATE_CONV2DFUSION_ADD_BCAST_TEST(AddV2);
 #undef CREATE_CONV2DFUSION_ADD_ACTIVATION_TEST
 #undef CREATE_CONV2DFUSION_TEST
 
+#ifndef DNNL_AARCH64_USE_ACL
 #define REGISTER_TEST(NAME, T, INPUT)                                         \
   TEST_F(MklRemapperTest, NAME##_##T) {                                       \
+    if (!IsMKLEnabled()) GTEST_SKIP() << "Test only applicable to oneDNN.";   \
     using ::tensorflow::ops::Placeholder;                                     \
                                                                               \
     for (const string& activation : {"Relu", "Relu6", "Elu", "None"}) {       \
@@ -308,8 +314,10 @@ CREATE_CONV2DFUSION_ADD_BCAST_TEST(AddV2);
   }
 REGISTER_TEST_ALL_TYPES(FuseDepthwiseConv2DWithBiasAndActivation);
 #undef REGISTER_TEST
+#endif
 
 TEST_F(MklRemapperTest, FuseBatchNormWithRelu) {
+  if (!IsMKLEnabled()) GTEST_SKIP() << "Test only applicable to oneDNN.";
   using ::tensorflow::ops::Placeholder;
 
   for (bool is_training : {true, false}) {
@@ -441,12 +449,16 @@ TEST_F(MklRemapperTest, FuseBatchNormWithRelu) {
       ASSERT_EQ(tensors_expected.size(), 1);
       auto tensors = EvaluateNodes(output, item.fetch, item.feed);
       ASSERT_EQ(tensors.size(), 1);
-      test::ExpectTensorNear<float>(tensors[0], tensors_expected[0], 1e-6);
+      test::ExpectTensorNear<float>(tensors[0], tensors_expected[0], 2e-5);
     }
   }
 }
 
 TEST_F(MklRemapperTest, FuseMatMulWithBiasAddAndAdd) {
+#ifdef DNNL_AARCH64_USE_ACL
+  GTEST_SKIP() << "Skipping test due to different behaviour on AARCH64";
+#endif
+  if (!IsMKLEnabled()) GTEST_SKIP() << "Test only applicable to oneDNN.";
   using ::tensorflow::ops::Placeholder;
 
   for (const string& add_op : {"BiasAdd", "AddV2", "Add"}) {
@@ -539,6 +551,10 @@ class RelpaceAddWithBiasAddTest : public GrapplerTest {
  protected:
   template <DataType DTYPE>
   void RelpaceAddWithBiasAddDepthwiseConv2D(const string& add_op) {
+#ifdef DNNL_AARCH64_USE_ACL
+    GTEST_SKIP() << "Skipping test due to different behaviour on AARCH64";
+#endif
+    if (!IsMKLEnabled()) GTEST_SKIP() << "Test only applicable to oneDNN.";
     using ::tensorflow::ops::Placeholder;
 
     for (const string& activation : {"None", "Relu", "Relu6", "Elu"}) {
@@ -652,6 +668,11 @@ class FusedMatMulBiasAddAndGeluTest : public GrapplerTest {
  public:
   template <DataType DTYPE, bool is_pattern2>
   void RunTest() {
+    if (!IsMKLEnabled()) GTEST_SKIP() << "Test only applicable to oneDNN.";
+    if (!IsDataTypeSupportedByOneDNNOnThisCPU(DTYPE))
+      GTEST_SKIP()
+          << "Intel oneDNN with " << DataType_Name(DTYPE)
+          << " is not supported, skipping FusedMatMulBiasAddAndGelu test.";
     using ::tensorflow::ops::Placeholder;
 
     tensorflow::Scope s = tensorflow::Scope::NewRootScope();
@@ -771,6 +792,11 @@ class MklFusedBatchMatMul : public MklRemapperTest {
  public:
   template <typename T>
   void VerifyFused(bool adjx, bool adjy) {
+    if (!IsMKLEnabled()) GTEST_SKIP() << "Test only applicable to oneDNN.";
+    const DataType& dt = DataTypeToEnum<T>::v();
+    if (!IsDataTypeSupportedByOneDNNOnThisCPU(dt))
+      GTEST_SKIP() << "Intel oneDNN with " << DataType_Name(dt)
+                   << " is not supported, skipping MklFusedBatchMatMul test.";
     using ::tensorflow::ops::Placeholder;
     using normal_generator = Eigen::internal::NormalRandomGenerator<T>;
 
@@ -862,6 +888,11 @@ class MklFusedBatchMatMul : public MklRemapperTest {
 
   template <typename T>
   void VerifyPreceedingScalarMul(bool adjx, bool adjy) {
+    if (!IsMKLEnabled()) GTEST_SKIP() << "Test only applicable to oneDNN.";
+    const DataType& dt = DataTypeToEnum<T>::v();
+    if (!IsDataTypeSupportedByOneDNNOnThisCPU(dt))
+      GTEST_SKIP() << "Intel oneDNN with " << DataType_Name(dt)
+                   << " is not supported, skipping MklFusedBatchMatMul test.";
     using ::tensorflow::ops::Placeholder;
     using normal_generator = Eigen::internal::NormalRandomGenerator<T>;
 
@@ -953,6 +984,7 @@ class MklFusedBatchMatMul : public MklRemapperTest {
 };
 
 TEST_F(MklFusedBatchMatMul, MulAndAdd) {
+  if (!IsMKLEnabled()) GTEST_SKIP() << "Test only applicable to oneDNN.";
   for (const auto adjx : {false, true})
     for (const auto adjy : {false, true}) {
       this->VerifyFused<float>(adjx, adjy);
@@ -961,6 +993,7 @@ TEST_F(MklFusedBatchMatMul, MulAndAdd) {
 }
 
 TEST_F(MklFusedBatchMatMul, MulAndAdd2) {
+  if (!IsMKLEnabled()) GTEST_SKIP() << "Test only applicable to oneDNN.";
   for (const auto adjx : {false, true})
     for (const auto adjy : {false, true}) {
       this->VerifyPreceedingScalarMul<float>(adjx, adjy);
@@ -972,6 +1005,10 @@ class MklRemapperSwishTest : public GrapplerTest {
  protected:
   template <DataType DTYPE>
   void RunTest() {
+    if (!IsMKLEnabled()) GTEST_SKIP() << "Test only applicable to oneDNN.";
+    if (!IsDataTypeSupportedByOneDNNOnThisCPU(DTYPE))
+      GTEST_SKIP() << "Intel oneDNN with " << DataType_Name(DTYPE)
+                   << " is not supported, skipping MklRemapperSwishTest test.";
     using ::tensorflow::ops::Placeholder;
 
     tensorflow::Scope s = tensorflow::Scope::NewRootScope();
@@ -1061,6 +1098,14 @@ class MklRemapperConv2dBiasAddSwishTest : public GrapplerTest {
  protected:
   template <DataType DTYPE>
   void RunTest() {
+#ifdef DNNL_AARCH64_USE_ACL
+    GTEST_SKIP() << "Skipping test due to different behaviour on AARCH64";
+#endif
+    if (!IsMKLEnabled()) GTEST_SKIP() << "Test only applicable to oneDNN.";
+    if (!IsDataTypeSupportedByOneDNNOnThisCPU(DTYPE))
+      GTEST_SKIP() << "Intel oneDNN with " << DataType_Name(DTYPE)
+                   << " is not supported, skipping "
+                      "MklRemapperConv2dBiasAddSwishTest test.";
     using ::tensorflow::ops::Placeholder;
 
     tensorflow::Scope s = tensorflow::Scope::NewRootScope();
@@ -1141,6 +1186,10 @@ class MklRemapperConv2dFusedBatchNormSwishTest : public GrapplerTest {
  protected:
   template <DataType DTYPE>
   void RunTest() {
+#ifdef DNNL_AARCH64_USE_ACL
+    GTEST_SKIP() << "Skipping test due to different behaviour on AARCH64";
+#endif
+    if (!IsMKLEnabled()) GTEST_SKIP() << "Test only applicable to oneDNN.";
     using ::tensorflow::ops::Placeholder;
 
     tensorflow::Scope s = tensorflow::Scope::NewRootScope();
@@ -1228,6 +1277,7 @@ class MklFuseInstanceNormTest : public GrapplerTest {
  protected:
   template <DataType DTYPE>
   void FuseMklInstanceNorm5D_Runner(string FORMAT, string activation) {
+    if (!IsMKLEnabled()) GTEST_SKIP() << "Test only applicable to oneDNN.";
     using ::tensorflow::ops::Placeholder;
     tensorflow::Scope s = tensorflow::Scope::NewRootScope();
 
@@ -1343,12 +1393,13 @@ class MklFuseInstanceNormTest : public GrapplerTest {
     if (DTYPE == DT_BFLOAT16) {
       test::ExpectClose(tensors[0], tensors_expected[0], 1e-2, 1e-2);
     } else {
-      test::ExpectClose(tensors[0], tensors_expected[0], 2e-6, 1e-6);
+      test::ExpectClose(tensors[0], tensors_expected[0], 1e-5, 1e-5);
     }
   }
 
   template <DataType DTYPE>
   void FuseMklInstanceNorm4D_Runner(string FORMAT, string activation) {
+    if (!IsMKLEnabled()) GTEST_SKIP() << "Test only applicable to oneDNN.";
     using ::tensorflow::ops::Placeholder;
     tensorflow::Scope s = tensorflow::Scope::NewRootScope();
 
@@ -1462,12 +1513,13 @@ class MklFuseInstanceNormTest : public GrapplerTest {
     if (DTYPE == DT_BFLOAT16) {
       test::ExpectClose(tensors[0], tensors_expected[0], 1e-2, 1e-2);
     } else {
-      test::ExpectClose(tensors[0], tensors_expected[0], 2e-6, 1e-6);
+      test::ExpectClose(tensors[0], tensors_expected[0], 1e-5, 1e-5);
     }
   }
 
   template <DataType DTYPE>
   void FuseMklInstanceNorm4D(string FORMAT, bool add_activation = false) {
+    if (!IsMKLEnabled()) GTEST_SKIP() << "Test only applicable to oneDNN.";
     if (!add_activation) {
       return FuseMklInstanceNorm4D_Runner<DTYPE>(FORMAT, "None");
     }
@@ -1478,6 +1530,7 @@ class MklFuseInstanceNormTest : public GrapplerTest {
 
   template <DataType DTYPE>
   void FuseMklInstanceNorm5D(string FORMAT, bool add_activation = false) {
+    if (!IsMKLEnabled()) GTEST_SKIP() << "Test only applicable to oneDNN.";
     if (!add_activation) {
       return FuseMklInstanceNorm5D_Runner<DTYPE>(FORMAT, "None");
     }
@@ -1514,6 +1567,244 @@ TEST_F(MklFuseInstanceNormTest, FuseMklInstanceNormWithActivation4D_FP32_NCHW) {
   FuseMklInstanceNorm4D<DT_FLOAT>("NCHW", true);
 }
 
+class FusedConvBiasAddAndHardSwishTest : public GrapplerTest {
+ public:
+  const string kAddOp = "Add";
+  const string kAddV2Op = "AddV2";
+
+  template <DataType DType, bool with_cast_op = false>
+  void RunTest(const string& add_op, const bool is_depthwise) {
+#ifdef DNNL_AARCH64_USE_ACL
+    GTEST_SKIP() << "Skipping test due to different behaviour on AARCH64";
+#endif
+    if (!IsMKLEnabled()) GTEST_SKIP() << "Test only applicable to oneDNN.";
+    if (!IsDataTypeSupportedByOneDNNOnThisCPU(DType))
+      GTEST_SKIP() << "Intel oneDNN with " << DataType_Name(DType)
+                   << " is not supported, skipping "
+                      "FusedConvBiasAddAndHardSwishTest test.";
+    using ::tensorflow::ops::Placeholder;
+
+    tensorflow::Scope s = tensorflow::Scope::NewRootScope();
+
+    auto input_shape = ops::Placeholder::Shape({8, 32, 32, 3});
+    auto filter_shape = ops::Placeholder::Shape({1, 1, 3, 128});
+    auto bias_shape = ops::Placeholder::Shape({is_depthwise ? 384 : 128});
+
+    auto input = Placeholder(s.WithOpName("input"), DType, input_shape);
+    auto filter = Placeholder(s.WithOpName("filter"), DType, filter_shape);
+    auto bias = Placeholder(s.WithOpName("bias"), DType, bias_shape);
+    const DataType const_dt = with_cast_op ? DT_FLOAT : DType;
+    typedef typename EnumToDataType<const_dt>::Type DT;
+    Tensor three(const_dt, TensorShape({}));
+    Tensor one_sixth(const_dt, TensorShape({}));
+    three.scalar<DT>()() = static_cast<DT>(3.0f);
+    one_sixth.scalar<DT>()() = static_cast<DT>(1.0f / 6.0f);
+    auto three_op =
+        with_cast_op
+            ? ops::Cast(s.WithOpName("three"), Input::Initializer(three),
+                        DT_BFLOAT16)
+            : ops::Const(s.WithOpName("three"), Input::Initializer(three));
+    auto one_sixth_op =
+        with_cast_op ? ops::Cast(s.WithOpName("one_sixth"),
+                                 Input::Initializer(one_sixth), DT_BFLOAT16)
+                     : ops::Const(s.WithOpName("one_sixth"),
+                                  Input::Initializer(one_sixth));
+
+    std::vector<int> strides = {1, 1, 1, 1};
+    Output conv;
+    if (is_depthwise) {
+      conv = ops::DepthwiseConv2dNative(
+          s.WithOpName("conv"), input, filter, strides, "SAME",
+          ops::DepthwiseConv2dNative::Attrs().DataFormat("NHWC"));
+    } else {
+      conv = ops::Conv2D(s.WithOpName("conv"), input, filter, strides, "SAME",
+                         ops::Conv2D::Attrs().DataFormat("NHWC"));
+    }
+    auto bias_add = ops::BiasAdd(s.WithOpName("bias_add"), conv, bias,
+                                 ops::BiasAdd::Attrs().DataFormat("NHWC"));
+
+    Output add;
+    if (add_op == kAddV2Op) {
+      add = ops::AddV2(s.WithOpName(add_op), three_op, bias_add);
+    } else {
+      add = ops::Add(s.WithOpName(add_op), three_op, bias_add);
+    }
+
+    auto relu6 = ops::Relu6(s.WithOpName("relu_6"), add);
+    auto mul_one_sixth =
+        ops::Mul(s.WithOpName("mul_one_sixth"), one_sixth_op, bias_add);
+    auto mul_output = ops::Mul(s.WithOpName("output"), mul_one_sixth, relu6);
+
+    auto fetch = ops::Identity(s.WithOpName("fetch"), mul_output);
+
+    auto input_tensor = GenerateTensorWithSetRandom<DType>(
+        TensorShape(input_shape.shape_.dim_sizes()));
+    auto filter_tensor = GenerateTensorWithSetRandom<DType>(
+        TensorShape(filter_shape.shape_.dim_sizes()));
+    auto bias_tensor = GenerateTensorWithSetRandom<DType>(
+        TensorShape(bias_shape.shape_.dim_sizes()));
+
+    GrapplerItem item;
+    item.fetch = {"fetch"};
+    item.feed = {{"input", input_tensor},
+                 {"filter", filter_tensor},
+                 {"bias", bias_tensor}};
+
+    TF_CHECK_OK(s.ToGraphDef(&item.graph));
+
+    // Place all nodes on CPU.
+    for (int i = 0; i < item.graph.node_size(); ++i) {
+      item.graph.mutable_node(i)->set_device("/device:CPU:0");
+    }
+
+    Remapper optimizer(RewriterConfig::ON);
+    GraphDef output;
+    TF_CHECK_OK(optimizer.Optimize(nullptr, item, &output));
+
+    int found = 0;
+    for (const NodeDef& node : output.node()) {
+      if (node.name() == "output") {
+        if (is_depthwise) {
+          EXPECT_EQ("_FusedDepthwiseConv2dNative", node.op());
+        } else {
+          EXPECT_EQ("_FusedConv2D", node.op());
+        }
+        EXPECT_EQ("input", node.input(0));
+        EXPECT_EQ("filter", node.input(1));
+        EXPECT_EQ("bias", node.input(2));
+        EXPECT_EQ(1, node.attr().at("num_args").i());
+
+        const auto fused_ops = node.attr().at("fused_ops").list().s();
+        EXPECT_EQ(2, fused_ops.size());
+        EXPECT_EQ("BiasAdd", fused_ops[0]);
+        EXPECT_EQ("_FusedHardSwish", fused_ops[1]);
+        found++;
+      }
+    }
+    EXPECT_EQ(1, found);
+
+    auto tensors_expected = EvaluateNodes(item.graph, item.fetch, item.feed);
+    auto tensors = EvaluateNodes(output, item.fetch, item.feed);
+    EXPECT_EQ(1, tensors_expected.size());
+    EXPECT_EQ(1, tensors.size());
+    test::ExpectClose(tensors_expected[0], tensors[0], 1e-6);
+  }
+};
+
+TEST_F(FusedConvBiasAddAndHardSwishTest, Float32Conv2DBiasHardSwish) {
+  RunTest<DT_FLOAT>("AddV2", false);
+}
+TEST_F(FusedConvBiasAddAndHardSwishTest, Float32DWConv2DBiasHardSwish) {
+  RunTest<DT_FLOAT>("AddV2", true);
+}
+TEST_F(FusedConvBiasAddAndHardSwishTest, Bfloat16Conv2DBiasHardSwish) {
+  RunTest<DT_BFLOAT16>("Add", false);
+}
+TEST_F(FusedConvBiasAddAndHardSwishTest, Bfloat16DWConv2DBiasHardSwish) {
+  RunTest<DT_BFLOAT16>("Add", true);
+}
+TEST_F(FusedConvBiasAddAndHardSwishTest, Bfloat16Conv2DBiasHardSwishWithCast) {
+  RunTest<DT_BFLOAT16, true>("Add", false);
+}
+TEST_F(FusedConvBiasAddAndHardSwishTest,
+       Bfloat16DWConv2DBiasHardSwishWithCast) {
+  RunTest<DT_BFLOAT16, true>("Add", true);
+}
+
+class FusedMatMulBiasAddMulAddEluTest : public GrapplerTest {
+ public:
+  template <DataType DTYPE>
+  void RunTest() {
+    if (!IsMKLEnabled()) GTEST_SKIP() << "Test only applicable to oneDNN.";
+    if (!IsDataTypeSupportedByOneDNNOnThisCPU(DTYPE))
+      GTEST_SKIP()
+          << "Intel oneDNN with " << DataType_Name(DTYPE)
+          << " is not supported, skipping FusedMatMulBiasAddAndGelu test.";
+    using ::tensorflow::ops::Placeholder;
+
+    tensorflow::Scope s = tensorflow::Scope::NewRootScope();
+
+    auto addv2_shape = ops::Placeholder::Shape({8, 32});
+    auto addv2_in2_shape = ops::Placeholder::Shape({32});
+    auto lhs_shape = ops::Placeholder::Shape({8, 32});
+    auto rhs_shape = ops::Placeholder::Shape({32, 64});
+    auto bias_shape = ops::Placeholder::Shape({64});
+    auto mul_shape = ops::Placeholder::Shape({64});
+    auto add_shape = ops::Placeholder::Shape({64});
+
+    auto lhs_t = GenerateTensorWithSetRandom<DTYPE>({8, 32});
+    auto addv2_in2_t = GenerateTensorWithSetRandom<DTYPE>({32});
+    auto rhs_const = GenerateTensorWithSetRandom<DTYPE>({32, 64});
+    auto bias_const = GenerateTensorWithSetRandom<DTYPE>({64});
+    auto mul_const = GenerateTensorWithSetRandom<DTYPE>({64});
+    auto add_const = GenerateTensorWithSetRandom<DTYPE>({64});
+
+    auto addv2_in2 =
+        Placeholder(s.WithOpName("addv2_in2"), DTYPE, addv2_in2_shape);
+    auto lhs = Placeholder(s.WithOpName("lhs"), DTYPE, lhs_shape);
+    auto rhs = Placeholder(s.WithOpName("rhs"), DTYPE, rhs_shape);
+    auto bias = Placeholder(s.WithOpName("bias"), DTYPE, bias_shape);
+    auto mul_in = Placeholder(s.WithOpName("mul"), DTYPE, mul_shape);
+    auto add_in = Placeholder(s.WithOpName("add"), DTYPE, add_shape);
+
+    auto addv2 = ops::AddV2(s.WithOpName("addv2"), lhs, addv2_in2);
+    auto matmul = ops::MatMul(s.WithOpName("matmul"), addv2, rhs);
+    auto bias_add = ops::BiasAdd(s.WithOpName("bias_add"), matmul, bias);
+    auto mul = ops::Mul(s.WithOpName("mul"), bias_add, mul_in);
+    auto add = ops::AddV2(s.WithOpName("add"), mul, add_in);
+    auto elu = ops::Elu(s.WithOpName("elu"), add);
+
+    auto fetch = ops::Identity(s.WithOpName("fetch"), elu);
+
+    GrapplerItem item;
+    item.fetch = {"fetch"};
+    item.feed = {{"addv2_in2", addv2_in2_t}, {"lhs", lhs_t},
+                 {"rhs", rhs_const},         {"bias", bias_const},
+                 {"mul", mul_const},         {"add", add_const}};
+    TF_ASSERT_OK(s.ToGraphDef(&item.graph));
+
+    // Place all nodes on CPU.
+    for (int i = 0; i < item.graph.node_size(); ++i) {
+      item.graph.mutable_node(i)->set_device("/device:CPU:0");
+    }
+
+    Remapper optimizer(RewriterConfig::ON);
+    GraphDef optimized_graph;
+    TF_ASSERT_OK(optimizer.Optimize(nullptr, item, &optimized_graph));
+
+    int found = 0;
+    for (const NodeDef& node : optimized_graph.node()) {
+      if (node.name() == "elu") {
+        EXPECT_EQ(node.op(), "_FusedMatMul");
+        ASSERT_GE(node.input_size(), 3);
+        EXPECT_EQ(node.input(0), "addv2");
+        EXPECT_EQ(node.input(1), "rhs_folded");
+        EXPECT_EQ(node.input(2), "add_add_folded");
+        EXPECT_EQ(node.attr().at("num_args").i(), 1);
+        const auto fused_ops = node.attr().at("fused_ops").list().s();
+        ASSERT_EQ(fused_ops.size(), 2);
+        EXPECT_EQ(fused_ops[0], "BiasAdd");
+        EXPECT_EQ(fused_ops[1], "Elu");
+        found++;
+      }
+    }
+    EXPECT_EQ(1, found);
+
+    // Evaluate result without remapper fusion
+    auto tensors_expected = EvaluateNodes(item.graph, item.fetch, item.feed);
+    ASSERT_EQ(tensors_expected.size(), 1);
+
+    auto tensors_evaluated =
+        EvaluateNodes(optimized_graph, item.fetch, item.feed);
+    ASSERT_EQ(tensors_evaluated.size(), 1);
+    test::ExpectClose(tensors_evaluated[0], tensors_expected[0], 1e-6);
+  }
+};
+
+TEST_F(FusedMatMulBiasAddMulAddEluTest, fp32) { RunTest<DT_FLOAT>(); }
+TEST_F(FusedMatMulBiasAddMulAddEluTest, bf16) { RunTest<DT_BFLOAT16>(); }
+TEST_F(FusedMatMulBiasAddMulAddEluTest, fp16) { RunTest<DT_HALF>(); }
+
 }  // namespace grappler
 }  // namespace tensorflow
-#endif  // INTEL_MKL && ENABLE_MKL
+#endif  // INTEL_MKL
