@@ -13,7 +13,14 @@ See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
 
+#include <cstdint>
+
+#define EIGEN_USE_THREADS
+
+#include "unsupported/Eigen/CXX11/Tensor"  // from @eigen_archive
+#include "tensorflow/core/kernels/cast_op.h"
 #include "tensorflow/core/kernels/cast_op_impl.h"
+#include "tensorflow/core/platform/types.h"
 
 namespace tensorflow {
 
@@ -21,7 +28,9 @@ typedef Eigen::ThreadPoolDevice CPUDevice;
 typedef Eigen::GpuDevice GPUDevice;
 
 CastFunctorType GetCpuCastFromInt64(DataType dst_dtype) {
-  CURRY_TYPES3(CAST_CASE, CPUDevice, int64);
+  CURRY_TYPES3(CAST_CASE, CPUDevice, int64_t);
+  CAST_CASE(CPUDevice, int64_t, int4);
+  CAST_CASE(CPUDevice, int64_t, uint4);
   return nullptr;
 }
 
@@ -33,6 +42,8 @@ CastFunctorType GetGpuCastFromInt64(DataType dst_dtype) {
 #else
   CURRY_TYPES3(CAST_CASE, GPUDevice, int64);
 #endif
+  CAST_CASE(GPUDevice, int64_t, int4);
+  CAST_CASE(GPUDevice, int64_t, uint4);
   return nullptr;
 }
 #endif  // GOOGLE_CUDA || TENSORFLOW_USE_ROCM
