@@ -203,15 +203,23 @@ void ConcatGPUImpl(const Eigen::GpuDevice& gpu_device,
 
 TF_CALL_INTEGRAL_TYPES(REGISTER_GPUCONCAT32);  // int32 Needed for TensorLists.
 TF_CALL_GPU_ALL_TYPES(REGISTER_GPUCONCAT32);
+TF_CALL_float8_e5m2(REGISTER_GPUCONCAT32);
+TF_CALL_float8_e4m3fn(REGISTER_GPUCONCAT32);
 
 TF_CALL_INTEGRAL_TYPES(REGISTER_GPUCONCAT64);  // int32 Needed for TensorLists.
 TF_CALL_GPU_ALL_TYPES(REGISTER_GPUCONCAT64);
+TF_CALL_float8_e5m2(REGISTER_GPUCONCAT64);
+TF_CALL_float8_e4m3fn(REGISTER_GPUCONCAT64);
 
 TF_CALL_INTEGRAL_TYPES(REGISTER_GPU32);  // int32 Needed for TensorLists.
 TF_CALL_GPU_ALL_TYPES(REGISTER_GPU32);
+TF_CALL_float8_e5m2(REGISTER_GPU32);
+TF_CALL_float8_e4m3fn(REGISTER_GPU32);
 
 TF_CALL_INTEGRAL_TYPES(REGISTER_GPU64);  // int32 Needed for TensorLists.
 TF_CALL_GPU_ALL_TYPES(REGISTER_GPU64);
+TF_CALL_float8_e5m2(REGISTER_GPU64);
+TF_CALL_float8_e4m3fn(REGISTER_GPU64);
 
 #undef REGISTER_GPUCONCAT32
 #undef REGISTER_GPUCONCAT64
