@@ -26,9 +26,10 @@ namespace tensorflow {
 void SwitchOp::Compute(OpKernelContext* context) {
   const Tensor& outputPorts = context->input(1);
   OP_REQUIRES(context, TensorShapeUtils::IsScalar(outputPorts.shape()),
-              errors::InvalidArgument("The second input must be a scalar, "
-                                      "but it has shape ",
-                                      outputPorts.shape().DebugString()));
+              absl::InvalidArgumentError(
+                  absl::StrCat("The second input must be a scalar, "
+                               "but it has shape ",
+                               outputPorts.shape().DebugString())));
 
   bool pred = outputPorts.scalar<bool>()();
   int port = (pred) ? 1 : 0;
@@ -42,9 +43,10 @@ void SwitchOp::Compute(OpKernelContext* context) {
 void SwitchNOp::Compute(OpKernelContext* context) {
   const Tensor& output_index_t = context->input(1);
   OP_REQUIRES(context, TensorShapeUtils::IsScalar(output_index_t.shape()),
-              errors::InvalidArgument("The second input must be a scalar, "
-                                      "but it has shape ",
-                                      output_index_t.shape().DebugString()));
+              absl::InvalidArgumentError(
+                  absl::StrCat("The second input must be a scalar, "
+                               "but it has shape ",
+                               output_index_t.shape().DebugString())));
   int output_index = output_index_t.scalar<int>()();
   if (output_index < 0 || output_index >= num_outputs()) {
     output_index = num_outputs() - 1;
@@ -231,15 +233,17 @@ class RefSelectOp : public OpKernel {
   void Compute(OpKernelContext* context) override {
     const Tensor& index_tensor = context->input(0);
     OP_REQUIRES(context, TensorShapeUtils::IsScalar(index_tensor.shape()),
-                errors::InvalidArgument("Index must be a scalar, "
-                                        "but it has shape ",
-                                        index_tensor.shape().DebugString()));
+                absl::InvalidArgumentError(
+                    absl::StrCat("Index must be a scalar, "
+                                 "but it has shape ",
+                                 index_tensor.shape().DebugString())));
 
     int32_t index = index_tensor.scalar<int32>()();
 
     OP_REQUIRES(context, index >= 0 && index < num_ref_inputs_,
-                errors::InvalidArgument("Index must be in the range [0, ",
-                                        num_ref_inputs_, ") but got ", index));
+                absl::InvalidArgumentError(
+                    absl::StrCat("Index must be in the range [0, ",
+                                 num_ref_inputs_, ") but got ", index)));
     context->forward_ref_input_to_ref_output(index + 1, 0);
   }
 
@@ -247,7 +251,8 @@ class RefSelectOp : public OpKernel {
 
   ~RefSelectOp() override {}
 
-  TF_DISALLOW_COPY_AND_ASSIGN(RefSelectOp);
+  RefSelectOp(const RefSelectOp&) = delete;
+  void operator=(const RefSelectOp&) = delete;
 
  private:
   int num_ref_inputs_;
@@ -412,6 +417,7 @@ void EnterOp::Compute(OpKernelContext* context) {
   }
 }
 
+REGISTER_KERNEL_BUILDER(Name("Enter").Device(DEVICE_CPU), EnterOp);
 REGISTER_KERNEL_BUILDER(Name("Enter").Device(DEVICE_TPU_SYSTEM), EnterOp);
 REGISTER_KERNEL_BUILDER(Name("Enter").Device(DEVICE_TPU), EnterOp);
 REGISTER_KERNEL_BUILDER(Name("RefEnter").Device(DEVICE_CPU), EnterOp);
@@ -470,6 +476,8 @@ REGISTER_GPU_HOST_KERNEL(ResourceHandle);
 
 TF_CALL_NUMBER_TYPES_NO_INT32(REGISTER_DEFAULT_KERNEL);
 TF_CALL_NUMBER_TYPES_NO_INT32(REGISTER_DEFAULT_REF_KERNEL);
+TF_CALL_QUANTIZED_TYPES(REGISTER_DEFAULT_KERNEL);
+TF_CALL_QUANTIZED_TYPES(REGISTER_DEFAULT_REF_KERNEL);
 REGISTER_DEFAULT_KERNEL(bool);
 REGISTER_DEFAULT_REF_KERNEL(bool);
 TF_CALL_variant(REGISTER_DEFAULT_KERNEL);
@@ -510,6 +518,7 @@ void ExitOp::Compute(OpKernelContext* context) {
   }
 }
 
+REGISTER_KERNEL_BUILDER(Name("Exit").Device(DEVICE_CPU), ExitOp);
 REGISTER_KERNEL_BUILDER(Name("Exit").Device(DEVICE_TPU_SYSTEM), ExitOp);
 REGISTER_KERNEL_BUILDER(Name("Exit").Device(DEVICE_TPU), ExitOp);
 REGISTER_KERNEL_BUILDER(Name("RefExit").Device(DEVICE_CPU), ExitOp);
@@ -563,6 +572,8 @@ REGISTER_GPU_HOST_KERNEL(ResourceHandle);
 
 TF_CALL_NUMBER_TYPES_NO_INT32(REGISTER_DEFAULT_KERNEL);
 TF_CALL_NUMBER_TYPES_NO_INT32(REGISTER_DEFAULT_REF_KERNEL);
+TF_CALL_QUANTIZED_TYPES(REGISTER_DEFAULT_KERNEL);
+TF_CALL_QUANTIZED_TYPES(REGISTER_DEFAULT_REF_KERNEL);
 REGISTER_DEFAULT_KERNEL(bool);
 REGISTER_DEFAULT_REF_KERNEL(bool);
 TF_CALL_variant(REGISTER_DEFAULT_KERNEL);
@@ -685,7 +696,7 @@ void LoopCondOp::Compute(OpKernelContext* context) {
   if (cm != nullptr) {
     bool already_cancelled = cm->IsCancelled();
     OP_REQUIRES(context, !already_cancelled,
-                errors::Cancelled("Loop execution was cancelled."));
+                absl::CancelledError("Loop execution was cancelled."));
   }
 
   context->set_output(0, context->input(0));
