@@ -24,9 +24,9 @@ limitations under the License.
 
 #if GOOGLE_CUDA
 #include "third_party/gpus/cudnn/cudnn.h"
-#include "tensorflow/compiler/xla/stream_executor/gpu/gpu_asm_opts.h"
-#include "tensorflow/compiler/xla/stream_executor/gpu/redzone_allocator.h"
-#include "tensorflow/compiler/xla/stream_executor/tf_allocator_adapter.h"
+#include "xla/stream_executor/gpu/gpu_asm_opts.h"
+#include "xla/stream_executor/gpu/redzone_allocator.h"
+#include "xla/stream_executor/integrations/tf_allocator_adapter.h"
 #include "tensorflow/core/kernels/autotune_conv_impl.h"
 #include "tensorflow/core/kernels/numeric_options_utils.h"
 #include "tensorflow/core/platform/tensor_float_32_utils.h"
@@ -42,14 +42,14 @@ bool ComputeInNhwcEnabled(DataType data_type, se::Stream* stream,
   // configurations it's more efficient to run computation in NCHW data format.
   bool use_nhwc_tf32 = data_type == DT_FLOAT &&
                        stream->GetCudaComputeCapability().IsAtLeast(
-                           se::CudaComputeCapability::AMPERE) &&
+                           se::CudaComputeCapability::kAmpere) &&
                        tensorflow::tensor_float_32_execution_enabled();
   bool use_nhwc_fp16 =
       data_type == DT_HALF && stream->GetCudaComputeCapability().IsAtLeast(
-                                  se::CudaComputeCapability::VOLTA);
+                                  se::CudaComputeCapability::kVolta);
   bool use_nhwc_bf16 =
       data_type == DT_BFLOAT16 && stream->GetCudaComputeCapability().IsAtLeast(
-                                      se::CudaComputeCapability::AMPERE);
+                                      se::CudaComputeCapability::kAmpere);
   if (use_4d_tensor) {
     return use_nhwc_fp16 || use_nhwc_tf32 || use_nhwc_bf16;
   }
@@ -89,19 +89,22 @@ StatusOr<AutotuneEntry<se::dnn::FusedConvOp>> AutotuneFusedConv(
 
     se::TfAllocatorAdapter tf_allocator_adapter(ctx->device()->GetAllocator({}),
                                                 stream);
-    se::RedzoneAllocator rz_allocator(stream, &tf_allocator_adapter,
-                                      se::GpuAsmOpts());
+    se::RedzoneAllocator rz_allocator(stream, &tf_allocator_adapter);
     se::DeviceMemory<T> output_ptr_rz(
         WrapRedzoneBestEffort(&rz_allocator, output_ptr));
 
     std::vector<std::unique_ptr<const se::dnn::FusedConvRunner>> runners;
     auto element_type = se::dnn::ToDataType<T>::value;
-    TF_RETURN_IF_ERROR(stream->parent()->GetFusedConvolveRunners(
-        CudnnUseFrontend(), se::dnn::ConvolutionKind::FORWARD, element_type,
-        element_type, element_type, conv_scale, side_input_scale,
-        leakyrelu_alpha, stream, input_desc, filter_desc, bias_desc,
-        output_desc, conv_desc, /*use_fallback=*/false, activation_mode,
-        GetNumericOptions(), &runners));
+    auto dnn = stream->parent()->AsDnn();
+    if (dnn == nullptr) {
+      return absl::InvalidArgumentError("No DNN in stream executor.");
+    }
+    TF_RETURN_IF_ERROR(dnn->GetFusedConvolveRunners(
+        se::dnn::ConvolutionKind::FORWARD, element_type, element_type,
+        element_type, conv_scale, side_input_scale, leakyrelu_alpha, stream,
+        input_desc, filter_desc, bias_desc, output_desc, conv_desc,
+        /*use_fallback=*/false, activation_mode, GetNumericOptionsForCuDnn(),
+        &runners));
 
     auto launch_func =
         [&](se::ScratchAllocator* allocator_used,
@@ -136,7 +139,7 @@ StatusOr<AutotuneEntry<se::dnn::FusedConvOp>> AutotuneFusedConv(
       }
     }
 
-    if (!CudnnUseFrontend() || found_working_engine) {
+    if (found_working_engine) {
       TF_ASSIGN_OR_RETURN(autotune_entry,
                           BestCudnnConvAlgorithm<se::dnn::FusedConvOp>(
                               results, std::move(runners)));
@@ -147,12 +150,12 @@ StatusOr<AutotuneEntry<se::dnn::FusedConvOp>> AutotuneFusedConv(
           << params.ToString();
       std::vector<std::unique_ptr<const se::dnn::FusedConvRunner>>
           fallback_runners;
-      TF_RETURN_IF_ERROR(stream->parent()->GetFusedConvolveRunners(
-          CudnnUseFrontend(), se::dnn::ConvolutionKind::FORWARD, element_type,
-          element_type, element_type, conv_scale, side_input_scale,
-          leakyrelu_alpha, stream, input_desc, filter_desc, bias_desc,
-          output_desc, conv_desc, /*use_fallback=*/true, activation_mode,
-          GetNumericOptions(), &fallback_runners));
+      TF_RETURN_IF_ERROR(dnn->GetFusedConvolveRunners(
+          se::dnn::ConvolutionKind::FORWARD, element_type, element_type,
+          element_type, conv_scale, side_input_scale, leakyrelu_alpha, stream,
+          input_desc, filter_desc, bias_desc, output_desc, conv_desc,
+          /*use_fallback=*/true, activation_mode, GetNumericOptionsForCuDnn(),
+          &fallback_runners));
 
       TF_ASSIGN_OR_RETURN(auto fallback_results,
                           internal::AutotuneConvImpl(
@@ -252,8 +255,7 @@ StatusOr<AutotuneEntry<se::dnn::ConvOp>> AutotuneUnfusedConv(
 #if GOOGLE_CUDA
     se::TfAllocatorAdapter tf_allocator_adapter(ctx->device()->GetAllocator({}),
                                                 stream);
-    se::RedzoneAllocator rz_allocator(stream, &tf_allocator_adapter,
-                                      se::GpuAsmOpts());
+    se::RedzoneAllocator rz_allocator(stream, &tf_allocator_adapter);
 
     // TODO(awpr): second-guess whether it's okay that this profiles
     // convolutions on uninitialized memory.
@@ -279,10 +281,14 @@ StatusOr<AutotuneEntry<se::dnn::ConvOp>> AutotuneUnfusedConv(
 
     const auto element_type = se::dnn::ToDataType<T>::value;
     std::vector<std::unique_ptr<const se::dnn::ConvRunner>> runners;
-    TF_RETURN_IF_ERROR(stream->parent()->GetConvolveRunners(
-        CudnnUseFrontend(), kind, element_type, element_type, stream,
-        input_desc, input_ptr, filter_desc, filter_ptr, output_desc, output_ptr,
-        conv_desc, /*use_fallback=*/false, &rz_allocator, GetNumericOptions(),
+    auto dnn = stream->parent()->AsDnn();
+    if (dnn == nullptr) {
+      return absl::InvalidArgumentError("No DNN in stream executor.");
+    }
+    TF_RETURN_IF_ERROR(dnn->GetConvolveRunners(
+        kind, element_type, element_type, stream, input_desc, input_ptr,
+        filter_desc, filter_ptr, output_desc, output_ptr, conv_desc,
+        /*use_fallback=*/false, &rz_allocator, GetNumericOptionsForCuDnn(),
         &runners));
     auto launch_func =
         [&](se::ScratchAllocator* allocator_used,
@@ -314,7 +320,7 @@ StatusOr<AutotuneEntry<se::dnn::ConvOp>> AutotuneUnfusedConv(
       }
     }
 
-    if (!CudnnUseFrontend() || found_working_engine) {
+    if (found_working_engine) {
       TF_ASSIGN_OR_RETURN(
           autotune_entry,
           BestCudnnConvAlgorithm<se::dnn::ConvOp>(results, std::move(runners)));
@@ -324,11 +330,11 @@ StatusOr<AutotuneEntry<se::dnn::ConvOp>> AutotuneUnfusedConv(
              "worked; trying fallback algorithms.  Conv: "
           << conv_parameters.ToString();
       std::vector<std::unique_ptr<const se::dnn::ConvRunner>> fallback_runners;
-      TF_RETURN_IF_ERROR(stream->parent()->GetConvolveRunners(
-          CudnnUseFrontend(), kind, element_type, element_type, stream,
-          input_desc, input_ptr, filter_desc, filter_ptr, output_desc,
-          output_ptr, conv_desc, /*use_fallback=*/true, &rz_allocator,
-          GetNumericOptions(), &fallback_runners));
+      TF_RETURN_IF_ERROR(dnn->GetConvolveRunners(
+          kind, element_type, element_type, stream, input_desc, input_ptr,
+          filter_desc, filter_ptr, output_desc, output_ptr, conv_desc,
+          /*use_fallback=*/true, &rz_allocator, GetNumericOptionsForCuDnn(),
+          &fallback_runners));
 
       TF_ASSIGN_OR_RETURN(auto fallback_results,
                           internal::AutotuneConvImpl(
@@ -349,7 +355,11 @@ StatusOr<AutotuneEntry<se::dnn::ConvOp>> AutotuneUnfusedConv(
     DnnScratchAllocator scratch_allocator(scratch_size_limit, ctx);
 
     std::vector<se::dnn::ProfileResult> algorithms;
-    if (!stream->parent()->GetMIOpenConvolveAlgorithms(
+    auto dnn = stream->parent()->AsDnn();
+    if (dnn == nullptr) {
+      return absl::InvalidArgumentError("No DNN in stream executor.");
+    }
+    if (!dnn->GetMIOpenConvolveAlgorithms(
             kind, se::dnn::ToDataType<T>::value, stream, input_desc, input_ptr,
             filter_desc, filter_ptr, output_desc, output_ptr, conv_desc,
             &scratch_allocator, &algorithms)) {
@@ -373,9 +383,9 @@ StatusOr<AutotuneEntry<se::dnn::ConvOp>> AutotuneUnfusedConv(
       for (auto miopen_algorithm : algorithms) {
         auto profile_algorithm = miopen_algorithm.algorithm();
         se::dnn::ProfileResult profile_result;
-        auto miopen_launch_status = stream->ConvolveWithAlgorithm(
-            kind, input_desc, input_ptr, filter_desc, filter_ptr, output_desc,
-            output_ptr, conv_desc, &scratch_allocator,
+        auto miopen_launch_status = dnn->ConvolveWithAlgorithm(
+            stream, kind, input_desc, input_ptr, filter_desc, filter_ptr,
+            output_desc, output_ptr, conv_desc, &scratch_allocator,
             se::dnn::AlgorithmConfig(profile_algorithm,
                                      miopen_algorithm.scratch_size()),
             &profile_result);
