@@ -153,7 +153,7 @@ class CTCLossOp : public OpKernel {
         ctx, sparse::SparseTensor::Create(*labels_indices, *labels_values,
                                           labels_shape, order, &labels_sp));
 
-    Status labels_sp_valid = labels_sp.IndicesValid();
+    absl::Status labels_sp_valid = labels_sp.IndicesValid();
     OP_REQUIRES(ctx, labels_sp_valid.ok(),
                 errors::InvalidArgument("label SparseTensor is not valid: ",
                                         labels_sp_valid.message()));
@@ -221,7 +221,8 @@ class CTCLossOp : public OpKernel {
   bool ctc_merge_repeated_;
   bool ignore_longer_outputs_than_inputs_;
 
-  TF_DISALLOW_COPY_AND_ASSIGN(CTCLossOp<T>);
+  CTCLossOp<T>(const CTCLossOp<T>&) = delete;
+  void operator=(const CTCLossOp<T>&) = delete;
 };
 
 #define REGISTER_CPU(T)                                          \
@@ -330,13 +331,13 @@ class CTCLossOpGPU : public OpKernel {
     StreamExecutor* executor = ctx->op_device_context()->stream()->parent();
     se::dnn::DataType data_type = ToDataType<float>::value;
 
-    auto probs_desc_s = executor->createRnnStateTensorDescriptor(
+    auto probs_desc_s = executor->AsDnn()->CreateRnnStateTensorDescriptor(
         max_time, batch_size, num_classes, data_type);
     OP_REQUIRES_OK(ctx, probs_desc_s.status());
     std::unique_ptr<RnnStateTensorDescriptor> probs_desc =
         std::move(probs_desc_s).value();
 
-    auto grads_desc_s = executor->createRnnStateTensorDescriptor(
+    auto grads_desc_s = executor->AsDnn()->CreateRnnStateTensorDescriptor(
         max_time, batch_size, num_classes, data_type);
     OP_REQUIRES_OK(ctx, grads_desc_s.status());
     std::unique_ptr<RnnStateTensorDescriptor> grads_desc =
@@ -357,21 +358,32 @@ class CTCLossOpGPU : public OpKernel {
     DnnScratchAllocator workspace_allocator(1LL << 32, ctx);
 
     Stream* stream = ctx->op_device_context()->stream();
+    auto dnn = stream->parent()->AsDnn();
+    OP_REQUIRES(ctx, dnn != nullptr,
+                absl::InternalError("stream->parent() has no DNN support"));
+    stream_executor::DeviceMemory<uint8_t> scratch_memory;
+    int ctc_loss_algo_id;
     bool cudnn_launch_status =
-        stream
-            ->ThenCtcLoss(*probs_desc, probs_data, labels_data,
-                          labels_lengths_data, input_lengths_data,
-                          GetNumericOptions(), &costs_data, *grads_desc,
-                          &grads_data, &workspace_allocator)
+        dnn->PrepareForCtcLoss(stream, *probs_desc, probs_data, *grads_desc,
+                               labels_data, labels_lengths_data,
+                               input_lengths_data, GetNumericOptionsForCuDnn(),
+                               &workspace_allocator, &scratch_memory,
+                               &ctc_loss_algo_id)
             .ok();
-
+    if (cudnn_launch_status) {
+      cudnn_launch_status = dnn->DoCtcLoss<float>(
+          stream, *probs_desc, probs_data, labels_data, labels_lengths_data,
+          input_lengths_data, &costs_data, *grads_desc, &grads_data,
+          &scratch_memory, ctc_loss_algo_id);
+    }
     if (!cudnn_launch_status) {
       ctx->SetStatus(errors::Internal("cuDNN CTCLoss launch failure"));
     }
   }
 
  private:
-  TF_DISALLOW_COPY_AND_ASSIGN(CTCLossOpGPU);
+  CTCLossOpGPU(const CTCLossOpGPU&) = delete;
+  void operator=(const CTCLossOpGPU&) = delete;
 };
 
 REGISTER_KERNEL_BUILDER(Name("CTCLossV2")
