@@ -14,11 +14,16 @@ limitations under the License.
 ==============================================================================*/
 #include "tensorflow/core/kernels/data/cache_dataset_ops.h"
 
+#include <atomic>
+#include <cstddef>
+#include <cstdint>
 #include <memory>
 #include <string>
 #include <utility>
 #include <vector>
 
+#include "absl/status/status.h"
+#include "tensorflow/core/data/global_shuffle_utils.h"
 #include "tensorflow/core/data/name_utils.h"
 #include "tensorflow/core/data/serialization_utils.h"
 #include "tensorflow/core/framework/dataset.h"
@@ -33,6 +38,7 @@ limitations under the License.
 #include "tensorflow/core/platform/env.h"
 #include "tensorflow/core/platform/errors.h"
 #include "tensorflow/core/platform/refcount.h"
+#include "tensorflow/core/util/tensor_bundle/naming.h"
 #include "tensorflow/core/util/tensor_bundle/tensor_bundle.h"
 
 namespace tensorflow {
@@ -72,14 +78,15 @@ constexpr char kIncompleteCacheErrorMessage[] =
     "should use `dataset.take(k).cache().repeat()` instead.";
 }  // namespace
 
-class PartialCache {
+class DatasetRandomAccessCache {
  public:
-  explicit PartialCache(const DatasetBase* dataset) : input_(dataset) {}
+  explicit DatasetRandomAccessCache(const DatasetBase* dataset)
+      : input_(dataset) {}
 
   // Extends the temporary cache up to a given index and then updates
   // out_tensors with the element at that index.
-  Status Get(OpKernelContext* ctx, int64 index,
-             std::vector<Tensor>* out_tensors) {
+  absl::Status Get(OpKernelContext* ctx, int64 index,
+                   std::vector<Tensor>* out_tensors) {
     if (!iter_resource_) {
       TF_ASSIGN_OR_RETURN(iter_resource_,
                           GetIteratorResourceFromDataset(ctx, input_));
@@ -89,14 +96,14 @@ class PartialCache {
       TF_RETURN_IF_ERROR(ExtendTempCacheToIndex(index, ctx));
     }
     *out_tensors = cache_.at(index);
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   // Returns the data which has been cached up to this point.
   std::vector<std::vector<Tensor>> GetCacheData() { return cache_; }
 
  private:
-  Status ExtendTempCacheToIndex(int64 index, OpKernelContext* ctx) {
+  absl::Status ExtendTempCacheToIndex(int64 index, OpKernelContext* ctx) {
     bool end_of_sequence;
     while (cache_.size() <= index) {
       std::vector<Tensor> out_tensors;
@@ -108,11 +115,12 @@ class PartialCache {
       }
       cache_.push_back(out_tensors);
     }
-    return OkStatus();
+    return absl::OkStatus();
   }
 
-  StatusOr<core::RefCountPtr<IteratorResource>> GetIteratorResourceFromDataset(
-      OpKernelContext* ctx, const DatasetBase* dataset) {
+  absl::StatusOr<core::RefCountPtr<IteratorResource>>
+  GetIteratorResourceFromDataset(OpKernelContext* ctx,
+                                 const DatasetBase* dataset) {
     FunctionLibraryRuntime* flr;
     std::unique_ptr<DeviceMgr> device_mgr(nullptr);
     std::unique_ptr<FunctionLibraryDefinition> flib_def(nullptr);
@@ -131,6 +139,32 @@ class PartialCache {
   std::vector<std::vector<Tensor>> cache_;
 };
 
+// Caches dataset elements when global shuffling is enabled.
+class IteratorRandomAccessCache {
+ public:
+  explicit IteratorRandomAccessCache(const DatasetBase* input)
+      : input_(input) {}
+
+  absl::Status Get(AnyContext ctx, size_t element_position,
+                   std::vector<Tensor>* out_tensors) {
+    if (element_position < cache_.size() && !cache_[element_position].empty()) {
+      *out_tensors = cache_[element_position];
+      return absl::OkStatus();
+    }
+
+    TF_RETURN_IF_ERROR(input_->Get(ctx, element_position, out_tensors));
+    if (element_position >= cache_.size()) {
+      cache_.resize(element_position + 1);
+    }
+    cache_[element_position] = *out_tensors;
+    return absl::OkStatus();
+  }
+
+ private:
+  const DatasetBase* input_ = nullptr;
+  std::vector<std::vector<Tensor>> cache_;
+};
+
 class CacheDatasetOp::FileDatasetBase : public DatasetBase {
  public:
   FileDatasetBase(OpKernelContext* ctx, const DatasetBase* input,
@@ -177,12 +211,13 @@ class CacheDatasetOp::FileDatasetBase : public DatasetBase {
     return input_->Cardinality(options);
   }
 
-  Status InputDatasets(std::vector<const DatasetBase*>* inputs) const override {
+  absl::Status InputDatasets(
+      std::vector<const DatasetBase*>* inputs) const override {
     inputs->push_back(input_);
-    return OkStatus();
+    return absl::OkStatus();
   }
 
-  Status CheckExternalState() const override {
+  absl::Status CheckExternalState() const override {
     return input_->CheckExternalState();
   }
 
@@ -213,14 +248,14 @@ class CacheDatasetOp::FileDatasetBase : public DatasetBase {
       }
     }
 
-    Status Initialize(IteratorContext* ctx) override {
+    absl::Status Initialize(IteratorContext* ctx) override {
       mutex_lock l(mu_);
       return InitializeIterator(ctx);
     }
 
-    Status GetNextInternal(IteratorContext* ctx,
-                           std::vector<Tensor>* out_tensors,
-                           bool* end_of_sequence) override {
+    absl::Status GetNextInternal(IteratorContext* ctx,
+                                 std::vector<Tensor>* out_tensors,
+                                 bool* end_of_sequence) override {
       mutex_lock l(mu_);
       return iterator_->GetNext(ctx, out_tensors, end_of_sequence);
     }
@@ -232,14 +267,14 @@ class CacheDatasetOp::FileDatasetBase : public DatasetBase {
                                        /*ratio=*/1);
     }
 
-    Status SaveInternal(SerializationContext* ctx,
-                        IteratorStateWriter* writer) override {
+    absl::Status SaveInternal(SerializationContext* ctx,
+                              IteratorStateWriter* writer) override {
       mutex_lock l(mu_);
       TF_RETURN_IF_ERROR(writer->WriteScalar(prefix(), kMode, mode_));
       return SaveInput(ctx, writer, iterator_);
     }
-    Status RestoreInternal(IteratorContext* ctx,
-                           IteratorStateReader* reader) override {
+    absl::Status RestoreInternal(IteratorContext* ctx,
+                                 IteratorStateReader* reader) override {
       mutex_lock l(mu_);
       {
         int64_t temp;
@@ -297,7 +332,7 @@ class CacheDatasetOp::FileDatasetBase : public DatasetBase {
         if (!dataset()->env_->FileExists(MetaFilename(filename_)).ok()) {
           LOG(WARNING) << kIncompleteCacheErrorMessage;
           std::vector<string> cache_files;
-          Status s = dataset()->env_->GetMatchingPaths(
+          absl::Status s = dataset()->env_->GetMatchingPaths(
               strings::StrCat(filename_, "*"), &cache_files);
           if (!s.ok()) {
             LOG(WARNING) << "Failed to get matching files on " << filename_
@@ -313,24 +348,24 @@ class CacheDatasetOp::FileDatasetBase : public DatasetBase {
         }
       }
 
-      Status Initialize(IteratorContext* ctx) override {
+      absl::Status Initialize(IteratorContext* ctx) override {
         return dataset()->input_->MakeIterator(ctx, this, prefix(),
                                                &input_impl_);
       }
 
-      Status GetNextInternal(IteratorContext* ctx,
-                             std::vector<Tensor>* out_tensors,
-                             bool* end_of_sequence) override {
+      absl::Status GetNextInternal(IteratorContext* ctx,
+                                   std::vector<Tensor>* out_tensors,
+                                   bool* end_of_sequence) override {
         mutex_lock l(mu_);
         *end_of_sequence = false;
         TF_RETURN_IF_ERROR(EnsureLockFileExists(end_of_sequence));
         if (*end_of_sequence) {
-          return OkStatus();
+          return absl::OkStatus();
         }
         TF_RETURN_IF_ERROR(writer_->status());
         if (cur_index_ >= kMaxItems) {
           // As a courtesy, close the [truncated] cache file.
-          Status s = Finish();
+          absl::Status s = Finish();
           if (!s.ok()) {
             LOG(ERROR) << s;
           }
@@ -344,7 +379,7 @@ class CacheDatasetOp::FileDatasetBase : public DatasetBase {
         if (*end_of_sequence && out_tensors->empty()) {
           TF_RETURN_IF_ERROR(Finish());
           cur_index_++;
-          return OkStatus();
+          return absl::OkStatus();
         }
         if (out_tensors->size() != dataset()->num_tensors_) {
           return errors::Internal(
@@ -362,7 +397,7 @@ class CacheDatasetOp::FileDatasetBase : public DatasetBase {
           TF_RETURN_IF_ERROR(Finish());
         }
         cur_index_++;
-        return OkStatus();
+        return absl::OkStatus();
       }
 
      protected:
@@ -372,8 +407,8 @@ class CacheDatasetOp::FileDatasetBase : public DatasetBase {
                                          /*ratio=*/1);
       }
 
-      Status SaveInternal(SerializationContext* ctx,
-                          IteratorStateWriter* writer) override {
+      absl::Status SaveInternal(SerializationContext* ctx,
+                                IteratorStateWriter* writer) override {
         mutex_lock l(mu_);
         TF_RETURN_IF_ERROR(
             writer->WriteScalar(prefix(), kCurIndex, cur_index_));
@@ -381,7 +416,7 @@ class CacheDatasetOp::FileDatasetBase : public DatasetBase {
         if (iteration_completed_) {
           TF_RETURN_IF_ERROR(
               writer->WriteScalar(prefix(), kIterationCompleted, ""));
-          return OkStatus();
+          return absl::OkStatus();
         }
 
         // lockfile is created on the first call to GetNextInternal. The
@@ -405,11 +440,11 @@ class CacheDatasetOp::FileDatasetBase : public DatasetBase {
         }
         TF_RETURN_IF_ERROR(SaveInput(ctx, writer, input_impl_));
         TF_RETURN_IF_ERROR(writer->WriteScalar(prefix(), kShardId, shard_id_));
-        return OkStatus();
+        return absl::OkStatus();
       }
 
-      Status RestoreInternal(IteratorContext* ctx,
-                             IteratorStateReader* reader) override {
+      absl::Status RestoreInternal(IteratorContext* ctx,
+                                   IteratorStateReader* reader) override {
         mutex_lock l(mu_);
         int64_t temp;
         // TODO(b/78048575): Update this when saving size_t tensors directly
@@ -424,7 +459,7 @@ class CacheDatasetOp::FileDatasetBase : public DatasetBase {
 
         if (reader->Contains(prefix(), kIterationCompleted)) {
           iteration_completed_ = true;
-          return OkStatus();
+          return absl::OkStatus();
         }
 
         TF_RETURN_IF_ERROR(RestoreInput(ctx, reader, input_impl_));
@@ -441,18 +476,18 @@ class CacheDatasetOp::FileDatasetBase : public DatasetBase {
         filename_ = strings::StrCat(dataset()->filename_, "_", shard_id_);
         lockfile_ = strings::StrCat(filename_, kLockFileSuffix);
         writer_ = std::make_unique<BundleWriter>(dataset()->env_, filename_);
-        return OkStatus();
+        return absl::OkStatus();
       }
 
      private:
-      Status EnsureLockFileExists(bool* end_of_sequence)
+      absl::Status EnsureLockFileExists(bool* end_of_sequence)
           TF_EXCLUSIVE_LOCKS_REQUIRED(mu_) {
         if (iteration_completed_) {
           *end_of_sequence = true;
-          return OkStatus();
+          return absl::OkStatus();
         }
         if (lockfile_created_) {
-          return OkStatus();
+          return absl::OkStatus();
         }
 
         // Perform rudimentary locking to help catch concurrent writes to the
@@ -472,10 +507,11 @@ class CacheDatasetOp::FileDatasetBase : public DatasetBase {
         if (dataset()->env_->FileExists(lockfile_).ok()) {
           // Attempt to read the contents of the lockfile.
           char contents_scratch[151] = {0};  // Initialize all to 0.
-          StringPiece contents;
+          absl::string_view contents;
           std::unique_ptr<RandomAccessFile> file;
           if (dataset()->env_->NewRandomAccessFile(lockfile_, &file).ok()) {
-            file->Read(0, 150, &contents, contents_scratch).IgnoreError();
+            file->Read(0, contents, absl::MakeSpan(contents_scratch, 150))
+                .IgnoreError();
           }
           return errors::AlreadyExists(
               "There appears to be a concurrent caching iterator running - "
@@ -504,10 +540,10 @@ class CacheDatasetOp::FileDatasetBase : public DatasetBase {
         // BundleWriter in another Session.
         writer_ = std::make_unique<BundleWriter>(dataset()->env_, filename_);
         lockfile_created_ = true;
-        return OkStatus();
+        return absl::OkStatus();
       }
 
-      Status Finish() TF_EXCLUSIVE_LOCKS_REQUIRED(mu_) {
+      absl::Status Finish() TF_EXCLUSIVE_LOCKS_REQUIRED(mu_) {
         iteration_completed_ = true;
         // Flush the current bundle.
         TF_RETURN_IF_ERROR(writer_->Finish());
@@ -533,7 +569,7 @@ class CacheDatasetOp::FileDatasetBase : public DatasetBase {
           TF_RETURN_IF_ERROR(dataset()->env_->DeleteFile(
               strings::StrCat(dataset()->filename_, "_", i, kLockFileSuffix)));
         }
-        return OkStatus();
+        return absl::OkStatus();
       }
 
       mutex mu_;
@@ -559,15 +595,15 @@ class CacheDatasetOp::FileDatasetBase : public DatasetBase {
             reader_(dataset()->env_, dataset()->filename_),
             iterator_restored_(false) {}
 
-      Status GetNextInternal(IteratorContext* ctx,
-                             std::vector<Tensor>* out_tensors,
-                             bool* end_of_sequence) override {
+      absl::Status GetNextInternal(IteratorContext* ctx,
+                                   std::vector<Tensor>* out_tensors,
+                                   bool* end_of_sequence) override {
         mutex_lock l(mu_);
         *end_of_sequence = false;
         TF_RETURN_IF_ERROR(reader_.status());
         if (!reader_.Valid()) {
           *end_of_sequence = true;
-          return OkStatus();
+          return absl::OkStatus();
         }
         out_tensors->clear();
         out_tensors->resize(dataset()->num_tensors_);
@@ -584,15 +620,15 @@ class CacheDatasetOp::FileDatasetBase : public DatasetBase {
           if (!reader_.Valid()) {
             out_tensors->clear();
             *end_of_sequence = true;
-            return OkStatus();
+            return absl::OkStatus();
           }
-          StringPiece key = reader_.key();
+          absl::string_view key = reader_.key();
           DCHECK_EQ(key, dataset()->FormatName(cur_index_, i));
           TF_RETURN_IF_ERROR(reader_.ReadCurrent(&(*out_tensors)[i]));
           TF_RETURN_IF_ERROR(reader_.status());
         }
         cur_index_++;
-        return OkStatus();
+        return absl::OkStatus();
       }
 
      protected:
@@ -602,15 +638,15 @@ class CacheDatasetOp::FileDatasetBase : public DatasetBase {
                                          /*ratio=*/1);
       }
 
-      Status SaveInternal(SerializationContext* ctx,
-                          IteratorStateWriter* writer) override {
+      absl::Status SaveInternal(SerializationContext* ctx,
+                                IteratorStateWriter* writer) override {
         mutex_lock l(mu_);
         TF_RETURN_IF_ERROR(
             writer->WriteScalar(prefix(), kCurIndex, cur_index_));
-        return OkStatus();
+        return absl::OkStatus();
       }
 
-      Status RestoreInternal(
+      absl::Status RestoreInternal(
           IteratorContext* ctx,
           IteratorStateReader* iterator_state_reader) override {
         mutex_lock l(mu_);
@@ -630,7 +666,7 @@ class CacheDatasetOp::FileDatasetBase : public DatasetBase {
         }
         reader_.Seek(dataset()->FormatName(cur_index_, 0));
         iterator_restored_ = true;
-        return OkStatus();
+        return absl::OkStatus();
       }
 
      private:
@@ -640,7 +676,7 @@ class CacheDatasetOp::FileDatasetBase : public DatasetBase {
       bool iterator_restored_ TF_GUARDED_BY(mu_);
     };  // FileReaderIterator
 
-    Status InitializeIterator(IteratorContext* ctx)
+    absl::Status InitializeIterator(IteratorContext* ctx)
         TF_EXCLUSIVE_LOCKS_REQUIRED(mu_) {
       // We intentionally use the same prefix for both `FileReaderIterator` and
       // `FileWriterIterator`. Since at any time there will be at most one of
@@ -684,15 +720,15 @@ class CacheDatasetOp::FileDataset : public CacheDatasetOp::FileDatasetBase {
   using FileDatasetBase::FileDatasetBase;
 
  protected:
-  Status AsGraphDefInternal(SerializationContext* ctx,
-                            DatasetGraphDefBuilder* b,
-                            Node** output) const override {
+  absl::Status AsGraphDefInternal(SerializationContext* ctx,
+                                  DatasetGraphDefBuilder* b,
+                                  Node** output) const override {
     Node* input_graph = nullptr;
     TF_RETURN_IF_ERROR(b->AddInputDataset(ctx, input_, &input_graph));
     Node* filename = nullptr;
     TF_RETURN_IF_ERROR(b->AddScalar(filename_, &filename));
     TF_RETURN_IF_ERROR(b->AddDataset(this, {input_graph, filename}, output));
-    return OkStatus();
+    return absl::OkStatus();
   }
 };
 
@@ -705,9 +741,9 @@ class CacheDatasetOp::FileDatasetV2 : public CacheDatasetOp::FileDatasetBase {
         resource_handle_(resource_handle) {}
 
  protected:
-  Status AsGraphDefInternal(SerializationContext* ctx,
-                            DatasetGraphDefBuilder* b,
-                            Node** output) const override {
+  absl::Status AsGraphDefInternal(SerializationContext* ctx,
+                                  DatasetGraphDefBuilder* b,
+                                  Node** output) const override {
     Node* input_node = nullptr;
     TF_RETURN_IF_ERROR(b->AddInputDataset(ctx, input_, &input_node));
     Node* filename_node = nullptr;
@@ -716,7 +752,7 @@ class CacheDatasetOp::FileDatasetV2 : public CacheDatasetOp::FileDatasetBase {
     TF_RETURN_IF_ERROR(b->AddTensor(resource_handle_, &resource_handle_node));
     TF_RETURN_IF_ERROR(b->AddDataset(
         this, {input_node, filename_node, resource_handle_node}, output));
-    return OkStatus();
+    return absl::OkStatus();
   }
 
  private:
@@ -731,6 +767,7 @@ class CacheDatasetOp::MemoryDatasetBase : public DatasetBase {
         input_(input),
         cache_(std::move(cache)) {
     input_->Ref();
+    random_indexing_compatible_ = input_->RandomIndexingCompatible();
   }
 
   ~MemoryDatasetBase() override { input_->Unref(); }
@@ -763,8 +800,8 @@ class CacheDatasetOp::MemoryDatasetBase : public DatasetBase {
     return input_->Cardinality(options);
   };
 
-  Status Get(OpKernelContext* ctx, int64 index,
-             std::vector<Tensor>* out_tensors) const override {
+  absl::Status Get(OpKernelContext* ctx, int64 index,
+                   std::vector<Tensor>* out_tensors) const override {
     mutex_lock l(mu_);
 
     CardinalityOptions options;
@@ -776,35 +813,57 @@ class CacheDatasetOp::MemoryDatasetBase : public DatasetBase {
       return errors::OutOfRange("Index out of range [0, ", cardinality,
                                 "):", index);
     }
-    if (!partial_cache_) {
-      partial_cache_ = std::make_unique<PartialCache>(input_);
+    if (!dataset_random_access_cache_) {
+      dataset_random_access_cache_ =
+          std::make_unique<DatasetRandomAccessCache>(input_);
     }
-    return partial_cache_->Get(ctx, index, out_tensors);
+    return dataset_random_access_cache_->Get(ctx, index, out_tensors);
   }
 
-  Status InputDatasets(std::vector<const DatasetBase*>* inputs) const override {
+  absl::Status Get(AnyContext ctx, int64 index,
+                   std::vector<Tensor>* out_tensors) const override {
+    mutex_lock l(mu_);
+    if (!iterator_random_access_cache_) {
+      iterator_random_access_cache_ =
+          std::make_unique<IteratorRandomAccessCache>(input_);
+    }
+    return iterator_random_access_cache_->Get(ctx, index, out_tensors);
+  }
+
+  absl::Status InputDatasets(
+      std::vector<const DatasetBase*>* inputs) const override {
     inputs->push_back(input_);
-    return OkStatus();
+    return absl::OkStatus();
   }
 
-  Status CheckExternalState() const override {
+  absl::Status CheckExternalState() const override {
     return input_->CheckExternalState();
   }
 
+  absl::Status RandomIndexingCompatible() const override {
+    return random_indexing_compatible_;
+  }
+
  protected:
   class MemoryIterator : public DatasetIterator<MemoryDatasetBase> {
    public:
     explicit MemoryIterator(const Params& params, MemoryCache* cache)
-        : DatasetIterator<MemoryDatasetBase>(params), cache_(cache) {}
+        : DatasetIterator<MemoryDatasetBase>(params),
+          cache_(cache),
+          global_shuffle_iterator_(dataset()) {}
 
-    Status Initialize(IteratorContext* ctx) override {
+    absl::Status Initialize(IteratorContext* ctx) override {
       mutex_lock l(mu_);
       return InitializeIterator(ctx);
     }
 
-    Status GetNextInternal(IteratorContext* ctx,
-                           std::vector<Tensor>* out_tensors,
-                           bool* end_of_sequence) override {
+    absl::Status GetNextInternal(IteratorContext* ctx,
+                                 std::vector<Tensor>* out_tensors,
+                                 bool* end_of_sequence) override {
+      if (ctx->index_mapper() != nullptr) {
+        return global_shuffle_iterator_.GetNext(ctx, out_tensors,
+                                                end_of_sequence);
+      }
       mutex_lock l(mu_);
       return iterator_->GetNext(ctx, out_tensors, end_of_sequence);
     }
@@ -816,19 +875,23 @@ class CacheDatasetOp::MemoryDatasetBase : public DatasetBase {
                                        /*ratio=*/1);
     }
 
-    Status SaveInternal(SerializationContext* ctx,
-                        IteratorStateWriter* writer) override {
+    absl::Status SaveInternal(SerializationContext* ctx,
+                              IteratorStateWriter* writer) override {
       mutex_lock l(mu_);
       if (cache_->IsCompleted()) {
         TF_RETURN_IF_ERROR(writer->WriteScalar(prefix(), kCacheCompleted, ""));
         TF_RETURN_IF_ERROR(
             WriteElementsToCheckpoint(writer, prefix(), cache_->data()));
       }
+      TF_RETURN_IF_ERROR(global_shuffle_iterator_.Save(prefix(), ctx, writer));
       return SaveInput(ctx, writer, iterator_);
     }
 
-    Status RestoreInternal(IteratorContext* ctx,
-                           IteratorStateReader* reader) override {
+    absl::Status RestoreInternal(IteratorContext* ctx,
+                                 IteratorStateReader* reader) override {
+      if (ctx->restored_element_count().has_value()) {
+        return global_shuffle_iterator_.Restore(prefix(), ctx, reader);
+      }
       mutex_lock l(mu_);
       iterator_.reset();
       cache_->Reset();
@@ -856,14 +919,14 @@ class CacheDatasetOp::MemoryDatasetBase : public DatasetBase {
         }
       }
 
-      Status Initialize(IteratorContext* ctx) override {
+      absl::Status Initialize(IteratorContext* ctx) override {
         return dataset()->input_->MakeIterator(ctx, this, prefix(),
                                                &input_impl_);
       }
 
-      Status GetNextInternal(IteratorContext* ctx,
-                             std::vector<Tensor>* out_tensors,
-                             bool* end_of_sequence) override {
+      absl::Status GetNextInternal(IteratorContext* ctx,
+                                   std::vector<Tensor>* out_tensors,
+                                   bool* end_of_sequence) override {
         mutex_lock l(mu_);
         TF_RETURN_IF_ERROR(
             input_impl_->GetNext(ctx, out_tensors, end_of_sequence));
@@ -872,7 +935,7 @@ class CacheDatasetOp::MemoryDatasetBase : public DatasetBase {
             VLOG(2) << "Finalizing the cache because EOF has been reached.";
             cache_->Complete(std::move(temp_cache_));
           }
-          return OkStatus();
+          return absl::OkStatus();
         }
         RecordBufferEnqueue(ctx, *out_tensors);
         temp_cache_.emplace_back(*out_tensors);
@@ -881,7 +944,7 @@ class CacheDatasetOp::MemoryDatasetBase : public DatasetBase {
                      "expected input cardinality.";
           cache_->Complete(std::move(temp_cache_));
         }
-        return OkStatus();
+        return absl::OkStatus();
       }
 
      protected:
@@ -891,8 +954,8 @@ class CacheDatasetOp::MemoryDatasetBase : public DatasetBase {
                                          /*ratio=*/1);
       }
 
-      Status SaveInternal(SerializationContext* ctx,
-                          IteratorStateWriter* writer) override {
+      absl::Status SaveInternal(SerializationContext* ctx,
+                                IteratorStateWriter* writer) override {
         mutex_lock l(mu_);
         if (!cache_->IsCompleted()) {
           TF_RETURN_IF_ERROR(
@@ -901,8 +964,8 @@ class CacheDatasetOp::MemoryDatasetBase : public DatasetBase {
         return SaveInput(ctx, writer, input_impl_);
       }
 
-      Status RestoreInternal(IteratorContext* ctx,
-                             IteratorStateReader* reader) override {
+      absl::Status RestoreInternal(IteratorContext* ctx,
+                                   IteratorStateReader* reader) override {
         mutex_lock l(mu_);
         if (!reader->Contains(prefix(), kCacheCompleted)) {
           TF_RETURN_IF_ERROR(
@@ -925,7 +988,7 @@ class CacheDatasetOp::MemoryDatasetBase : public DatasetBase {
             cache_(cache),
             index_(0) {}
 
-      Status Initialize(IteratorContext* ctx) override {
+      absl::Status Initialize(IteratorContext* ctx) override {
         // The memory allocated for the cache is owned by the parent
         // dataset but performance modeling uses the iterator abstraction and
         // thus we record the memory allocated for the cache here. The caveat
@@ -935,12 +998,12 @@ class CacheDatasetOp::MemoryDatasetBase : public DatasetBase {
         for (size_t i = 0; i < cache_->size(); ++i) {
           RecordBufferEnqueue(ctx, cache_->at(i));
         }
-        return OkStatus();
+        return absl::OkStatus();
       }
 
-      Status GetNextInternal(IteratorContext* ctx,
-                             std::vector<Tensor>* out_tensors,
-                             bool* end_of_sequence) override {
+      absl::Status GetNextInternal(IteratorContext* ctx,
+                                   std::vector<Tensor>* out_tensors,
+                                   bool* end_of_sequence) override {
         mutex_lock l(mu_);
         if (index_ < cache_->size()) {
           const std::vector<Tensor>& cache_tensors = cache_->at(index_);
@@ -948,10 +1011,10 @@ class CacheDatasetOp::MemoryDatasetBase : public DatasetBase {
                               cache_tensors.end());
           index_++;
           *end_of_sequence = false;
-          return OkStatus();
+          return absl::OkStatus();
         } else {
           *end_of_sequence = true;
-          return OkStatus();
+          return absl::OkStatus();
         }
       }
 
@@ -962,15 +1025,15 @@ class CacheDatasetOp::MemoryDatasetBase : public DatasetBase {
                                          /*ratio=*/1);
       }
 
-      Status SaveInternal(SerializationContext* ctx,
-                          IteratorStateWriter* writer) override {
+      absl::Status SaveInternal(SerializationContext* ctx,
+                                IteratorStateWriter* writer) override {
         mutex_lock l(mu_);
         TF_RETURN_IF_ERROR(writer->WriteScalar(prefix(), kIndex, index_));
-        return OkStatus();
+        return absl::OkStatus();
       }
 
-      Status RestoreInternal(IteratorContext* ctx,
-                             IteratorStateReader* reader) override {
+      absl::Status RestoreInternal(IteratorContext* ctx,
+                                   IteratorStateReader* reader) override {
         mutex_lock l(mu_);
         {
           // kIndex will not be set if we are restoring from a checkpoint
@@ -981,7 +1044,7 @@ class CacheDatasetOp::MemoryDatasetBase : public DatasetBase {
           }
           index_ = static_cast<size_t>(temp);
         }
-        return OkStatus();
+        return absl::OkStatus();
       }
 
      private:
@@ -990,7 +1053,7 @@ class CacheDatasetOp::MemoryDatasetBase : public DatasetBase {
       size_t index_ TF_GUARDED_BY(mu_);
     };  // MemoryReaderIterator
 
-    Status InitializeIterator(IteratorContext* ctx)
+    absl::Status InitializeIterator(IteratorContext* ctx)
         TF_EXCLUSIVE_LOCKS_REQUIRED(mu_) {
       if (cache_->IsCompleted()) {
         iterator_ = std::make_unique<MemoryReaderIterator>(
@@ -1010,12 +1073,17 @@ class CacheDatasetOp::MemoryDatasetBase : public DatasetBase {
     mutex mu_;
     MemoryCache* cache_ TF_GUARDED_BY(mu_);  // not owned.
     std::unique_ptr<IteratorBase> iterator_ TF_GUARDED_BY(mu_);
+    GlobalShuffleIterator global_shuffle_iterator_;
   };  // MemoryIterator
 
   mutable mutex mu_;
   const DatasetBase* const input_;
   const std::shared_ptr<MemoryCache> cache_;
-  mutable std::unique_ptr<PartialCache> partial_cache_ TF_GUARDED_BY(mu_);
+  mutable std::unique_ptr<DatasetRandomAccessCache> dataset_random_access_cache_
+      TF_GUARDED_BY(mu_);
+  mutable std::unique_ptr<IteratorRandomAccessCache>
+      iterator_random_access_cache_;
+  absl::Status random_indexing_compatible_ = absl::OkStatus();
 };  // MemoryDatasetBase
 
 // This version of memory dataset has an exclusive ownership of the memory cache
@@ -1032,7 +1100,7 @@ class CacheDatasetOp::MemoryDataset : public CacheDatasetOp::MemoryDatasetBase {
 
   ~MemoryDataset() override {
     manager_->Unref();
-    Status s = resource_mgr_->Delete<MemoryCacheManager>(
+    absl::Status s = resource_mgr_->Delete<MemoryCacheManager>(
         resource_handle_.container(), resource_handle_.name());
     if (!s.ok()) {
       LOG(WARNING) << "Failed to delete cache resource: " << s.ToString();
@@ -1040,16 +1108,16 @@ class CacheDatasetOp::MemoryDataset : public CacheDatasetOp::MemoryDatasetBase {
   }
 
  protected:
-  Status AsGraphDefInternal(SerializationContext* ctx,
-                            DatasetGraphDefBuilder* b,
-                            Node** output) const override {
+  absl::Status AsGraphDefInternal(SerializationContext* ctx,
+                                  DatasetGraphDefBuilder* b,
+                                  Node** output) const override {
     Node* input_node = nullptr;
     TF_RETURN_IF_ERROR(b->AddInputDataset(ctx, input_, &input_node));
     Node* filename_node = nullptr;
     TF_RETURN_IF_ERROR(b->AddScalar(tstring(""), &filename_node));
     TF_RETURN_IF_ERROR(
         b->AddDataset(this, {input_node, filename_node}, output));
-    return OkStatus();
+    return absl::OkStatus();
   }
 
  private:
@@ -1076,7 +1144,7 @@ class CacheDatasetOp::MemoryDatasetV2
   ~MemoryDatasetV2() override {
     manager_->Unref();
     if (owns_resource_) {
-      Status s = resource_mgr_->Delete<MemoryCacheManager>(
+      absl::Status s = resource_mgr_->Delete<MemoryCacheManager>(
           resource_handle_.container(), resource_handle_.name());
       if (!s.ok()) {
         LOG(WARNING) << "Failed to delete cache resource: " << s.ToString();
@@ -1085,9 +1153,9 @@ class CacheDatasetOp::MemoryDatasetV2
   }
 
  protected:
-  Status AsGraphDefInternal(SerializationContext* ctx,
-                            DatasetGraphDefBuilder* b,
-                            Node** output) const override {
+  absl::Status AsGraphDefInternal(SerializationContext* ctx,
+                                  DatasetGraphDefBuilder* b,
+                                  Node** output) const override {
     Node* input_node = nullptr;
     TF_RETURN_IF_ERROR(b->AddInputDataset(ctx, input_, &input_node));
     Node* filename_node = nullptr;
@@ -1098,7 +1166,7 @@ class CacheDatasetOp::MemoryDatasetV2
     TF_RETURN_IF_ERROR(b->AddTensor(handle, &resource_handle_node));
     TF_RETURN_IF_ERROR(b->AddDataset(
         this, {input_node, filename_node, resource_handle_node}, output));
-    return OkStatus();
+    return absl::OkStatus();
   }
 
  private:
@@ -1126,16 +1194,16 @@ void CacheDatasetOp::MakeDataset(OpKernelContext* ctx, DatasetBase* input,
       bool owns_resource = false;
       MemoryCacheManager* manager = nullptr;
       auto handle = HandleFromInput(ctx, 2);
-      Status s = ctx->resource_manager()->Lookup<MemoryCacheManager>(
+      absl::Status s = ctx->resource_manager()->Lookup<MemoryCacheManager>(
           handle.container(), handle.name(), &manager);
-      if (errors::IsNotFound(s)) {
+      if (absl::IsNotFound(s)) {
         owns_resource = true;
         OP_REQUIRES_OK(
             ctx,
             ctx->resource_manager()->LookupOrCreate<MemoryCacheManager>(
                 container, name, &manager, [](MemoryCacheManager** manager) {
                   *manager = new MemoryCacheManager();
-                  return OkStatus();
+                  return absl::OkStatus();
                 }));
         handle = MakeResourceHandle<MemoryCacheManager>(ctx, container, name);
       } else {
@@ -1150,7 +1218,7 @@ void CacheDatasetOp::MakeDataset(OpKernelContext* ctx, DatasetBase* input,
           ctx, ctx->resource_manager()->LookupOrCreate<MemoryCacheManager>(
                    container, name, &manager, [](MemoryCacheManager** manager) {
                      *manager = new MemoryCacheManager();
-                     return OkStatus();
+                     return absl::OkStatus();
                    }));
       auto handle =
           MakeResourceHandle<MemoryCacheManager>(ctx, container, name);
