@@ -82,21 +82,23 @@ class MatchingFilesDatasetOp : public DatasetOpKernel {
       return "MatchingFilesDatasetOp::Dataset";
     }
 
-    Status InputDatasets(
+    absl::Status InputDatasets(
         std::vector<const DatasetBase*>* inputs) const override {
-      return OkStatus();
+      return absl::OkStatus();
     }
 
-    Status CheckExternalState() const override { return OkStatus(); }
+    absl::Status CheckExternalState() const override {
+      return absl::OkStatus();
+    }
 
    protected:
-    Status AsGraphDefInternal(SerializationContext* ctx,
-                              DatasetGraphDefBuilder* b,
-                              Node** output) const override {
+    absl::Status AsGraphDefInternal(SerializationContext* ctx,
+                                    DatasetGraphDefBuilder* b,
+                                    Node** output) const override {
       Node* patterns_node = nullptr;
       TF_RETURN_IF_ERROR(b->AddVector(patterns_, &patterns_node));
       TF_RETURN_IF_ERROR(b->AddDataset(this, {patterns_node}, output));
-      return OkStatus();
+      return absl::OkStatus();
     }
 
    private:
@@ -105,9 +107,9 @@ class MatchingFilesDatasetOp : public DatasetOpKernel {
       explicit Iterator(const Params& params)
           : DatasetIterator<Dataset>(params) {}
 
-      Status GetNextInternal(IteratorContext* ctx,
-                             std::vector<Tensor>* out_tensors,
-                             bool* end_of_sequence) override {
+      absl::Status GetNextInternal(IteratorContext* ctx,
+                                   std::vector<Tensor>* out_tensors,
+                                   bool* end_of_sequence) override {
         mutex_lock l(mu_);
         FileSystem* fs;
 
@@ -139,7 +141,7 @@ class MatchingFilesDatasetOp : public DatasetOpKernel {
               out_tensors->emplace_back(std::move(filepath_tensor));
               *end_of_sequence = false;
               hasMatch_ = true;
-              return OkStatus();
+              return absl::OkStatus();
             }
 
             // In this case, current_path is a directory. Then continue the
@@ -149,7 +151,8 @@ class MatchingFilesDatasetOp : public DatasetOpKernel {
           } else {
             // search a new pattern
             current_pattern_ = dataset()->patterns_[current_pattern_index_];
-            StringPiece current_pattern_view = StringPiece(current_pattern_);
+            absl::string_view current_pattern_view =
+                absl::string_view(current_pattern_);
 
             // Windows paths contain backslashes and Windows APIs accept forward
             // and backslashes equivalently, so we convert the pattern to use
@@ -166,7 +169,7 @@ class MatchingFilesDatasetOp : public DatasetOpKernel {
               isWindows_ = false;
             }
 
-            StringPiece fixed_prefix = current_pattern_view.substr(
+            absl::string_view fixed_prefix = current_pattern_view.substr(
                 0, current_pattern_view.find_first_of("*?[\\"));
             string current_dir(io::Dirname(fixed_prefix));
 
@@ -185,7 +188,7 @@ class MatchingFilesDatasetOp : public DatasetOpKernel {
 
         *end_of_sequence = true;
         if (hasMatch_) {
-          return OkStatus();
+          return absl::OkStatus();
         } else {
           return errors::NotFound("Don't find any matched files");
         }
@@ -197,8 +200,8 @@ class MatchingFilesDatasetOp : public DatasetOpKernel {
         return model::MakeSourceNode(std::move(args));
       }
 
-      Status SaveInternal(SerializationContext* ctx,
-                          IteratorStateWriter* writer) override {
+      absl::Status SaveInternal(SerializationContext* ctx,
+                                IteratorStateWriter* writer) override {
         mutex_lock l(mu_);
         TF_RETURN_IF_ERROR(writer->WriteScalar(
             full_name("current_pattern_index"), current_pattern_index_));
@@ -226,11 +229,11 @@ class MatchingFilesDatasetOp : public DatasetOpKernel {
           }
         }
 
-        return OkStatus();
+        return absl::OkStatus();
       }
 
-      Status RestoreInternal(IteratorContext* ctx,
-                             IteratorStateReader* reader) override {
+      absl::Status RestoreInternal(IteratorContext* ctx,
+                                   IteratorStateReader* reader) override {
         mutex_lock l(mu_);
         int64_t current_pattern_index;
         TF_RETURN_IF_ERROR(reader->ReadScalar(
@@ -268,19 +271,19 @@ class MatchingFilesDatasetOp : public DatasetOpKernel {
           }
         }
 
-        return OkStatus();
+        return absl::OkStatus();
       }
 
      private:
-      Status UpdateIterator(IteratorContext* ctx, FileSystem* fs,
-                            const string& dir, const string& eval_pattern)
+      absl::Status UpdateIterator(IteratorContext* ctx, FileSystem* fs,
+                                  const string& dir, const string& eval_pattern)
           TF_EXCLUSIVE_LOCKS_REQUIRED(mu_) {
-        StringPiece fixed_prefix =
-            StringPiece(eval_pattern)
+        absl::string_view fixed_prefix =
+            absl::string_view(eval_pattern)
                 .substr(0, eval_pattern.find_first_of("*?[\\"));
 
         filepath_queue_.push(PathStatus(dir, true));
-        Status ret;  // Status to return
+        absl::Status ret;  // Status to return
 
         // DFS to find the first element in the iterator.
         while (!filepath_queue_.empty()) {
@@ -289,7 +292,7 @@ class MatchingFilesDatasetOp : public DatasetOpKernel {
           // All the files in the heap are matched with the pattern, so finish
           // the search if current_path is a file.
           if (!current_path.second) {
-            return OkStatus();
+            return absl::OkStatus();
           }
 
           filepath_queue_.pop();
@@ -312,7 +315,7 @@ class MatchingFilesDatasetOp : public DatasetOpKernel {
           // three possible values: OK for true; FAILED_PRECONDITION for false;
           // CANCELLED if we don't calculate IsDirectory (we might do that
           // because there isn't any point in exploring that child path).
-          std::vector<Status> children_dir_status;
+          std::vector<absl::Status> children_dir_status;
           children_dir_status.resize(children.size());
 
           // This IsDirectory call can be expensive for some FS. Parallelizing
@@ -342,7 +345,7 @@ class MatchingFilesDatasetOp : public DatasetOpKernel {
           for (int i = 0; i < children.size(); i++) {
             const string& child_dir_path =
                 io::JoinPath(current_dir, children[i]);
-            const Status& child_dir_status = children_dir_status[i];
+            const absl::Status& child_dir_status = children_dir_status[i];
 
             // If the IsDirectory call was cancelled we bail.
             if (child_dir_status.code() == tensorflow::error::CANCELLED) {
