@@ -12,7 +12,14 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
+#include <cstddef>
+#include <cstdint>
 #include <deque>
+#include <map>
+#include <memory>
+#include <string>
+#include <utility>
+#include <vector>
 
 #include "tensorflow/core/common_runtime/device.h"
 #include "tensorflow/core/data/dataset_utils.h"
@@ -273,20 +280,20 @@ class ParseExampleDatasetOp : public UnaryDatasetOpKernel {
       return input_->Cardinality(options);
     }
 
-    Status InputDatasets(
+    absl::Status InputDatasets(
         std::vector<const DatasetBase*>* inputs) const override {
       inputs->push_back(input_);
-      return OkStatus();
+      return absl::OkStatus();
     }
 
-    Status CheckExternalState() const override {
+    absl::Status CheckExternalState() const override {
       return input_->CheckExternalState();
     }
 
    protected:
-    Status AsGraphDefInternal(SerializationContext* ctx,
-                              DatasetGraphDefBuilder* b,
-                              Node** output) const override {
+    absl::Status AsGraphDefInternal(SerializationContext* ctx,
+                                    DatasetGraphDefBuilder* b,
+                                    Node** output) const override {
       Node* input_graph_node = nullptr;
       TF_RETURN_IF_ERROR(b->AddInputDataset(ctx, input_, &input_graph_node));
 
@@ -303,7 +310,7 @@ class ParseExampleDatasetOp : public UnaryDatasetOpKernel {
         dense_defaults_nodes.emplace_back(node);
       }
 
-      std::vector<std::pair<StringPiece, AttrValue>> attrs;
+      std::vector<std::pair<absl::string_view, AttrValue>> attrs;
 
       AttrValue sparse_keys_attr;
       b->BuildAttrValue(sparse_keys_, &sparse_keys_attr);
@@ -357,7 +364,7 @@ class ParseExampleDatasetOp : public UnaryDatasetOpKernel {
                                        },
                                        {{2, dense_defaults_nodes}}, attrs,
                                        output));
-      return OkStatus();
+      return absl::OkStatus();
     }
 
    private:
@@ -379,7 +386,7 @@ class ParseExampleDatasetOp : public UnaryDatasetOpKernel {
         if (deregister_fn_) deregister_fn_();
       }
 
-      Status Initialize(IteratorContext* ctx) override {
+      absl::Status Initialize(IteratorContext* ctx) override {
         mutex_lock l(*mu_);
         if (num_parallel_calls_->value == model::kAutotune) {
           num_parallel_calls_->value = GetAutotuneDefaultParallelism(ctx);
@@ -391,9 +398,9 @@ class ParseExampleDatasetOp : public UnaryDatasetOpKernel {
                                                &input_impl_);
       }
 
-      Status GetNextInternal(IteratorContext* ctx,
-                             std::vector<Tensor>* out_tensors,
-                             bool* end_of_sequence) override {
+      absl::Status GetNextInternal(IteratorContext* ctx,
+                                   std::vector<Tensor>* out_tensors,
+                                   bool* end_of_sequence) override {
         std::shared_ptr<InvocationResult> result;
         {
           mutex_lock l(*mu_);
@@ -410,9 +417,9 @@ class ParseExampleDatasetOp : public UnaryDatasetOpKernel {
         RecordStop(ctx);
         result->notification.WaitForNotification();
         RecordStart(ctx);
-        profiler::TraceMe traceme([&] {
-          return profiler::TraceMeEncode("ParseExampleConsume",
-                                         {{"element_id", result->id}});
+        tsl::profiler::TraceMe traceme([&] {
+          return tsl::profiler::TraceMeEncode("ParseExampleConsume",
+                                              {{"element_id", result->id}});
         });
         return ProcessResult(ctx, result, out_tensors, end_of_sequence);
       }
@@ -427,8 +434,8 @@ class ParseExampleDatasetOp : public UnaryDatasetOpKernel {
                                   /*max=*/ctx->runner_threadpool_size())});
       }
 
-      Status SaveInternal(SerializationContext* ctx,
-                          IteratorStateWriter* writer) override {
+      absl::Status SaveInternal(SerializationContext* ctx,
+                                IteratorStateWriter* writer) override {
         mutex_lock l(*mu_);
         // Wait for all in-flight calls to complete.
         while (num_calls_ > 0) {
@@ -462,11 +469,11 @@ class ParseExampleDatasetOp : public UnaryDatasetOpKernel {
                 ""));
           }
         }
-        return OkStatus();
+        return absl::OkStatus();
       }
 
-      Status RestoreInternal(IteratorContext* ctx,
-                             IteratorStateReader* reader) override {
+      absl::Status RestoreInternal(IteratorContext* ctx,
+                                   IteratorStateReader* reader) override {
         mutex_lock l(*mu_);
         TF_RETURN_IF_ERROR(RestoreInput(ctx, reader, input_impl_));
         int64_t invocation_results_size;
@@ -507,7 +514,7 @@ class ParseExampleDatasetOp : public UnaryDatasetOpKernel {
           RecordBufferEnqueue(ctx, result.return_values);
           result.notification.Notify();
         }
-        return OkStatus();
+        return absl::OkStatus();
       }
 
       TraceMeMetadata GetTraceMeMetadata() const override {
@@ -537,7 +544,7 @@ class ParseExampleDatasetOp : public UnaryDatasetOpKernel {
         explicit InvocationResult(int64_t id) : id(id) {}
 
         Notification notification;
-        Status status;
+        absl::Status status;
         std::vector<Tensor> return_values;
         bool end_of_input = false;
         int64_t id = -1;
@@ -581,9 +588,9 @@ class ParseExampleDatasetOp : public UnaryDatasetOpKernel {
       void CallFunction(const std::shared_ptr<IteratorContext>& ctx,
                         const std::shared_ptr<InvocationResult>& result)
           TF_LOCKS_EXCLUDED(*mu_) {
-        profiler::TraceMe traceme([&] {
-          return profiler::TraceMeEncode("ParseExampleProduce",
-                                         {{"element_id", result->id}});
+        tsl::profiler::TraceMe traceme([&] {
+          return tsl::profiler::TraceMeEncode("ParseExampleProduce",
+                                              {{"element_id", result->id}});
         });
         // Get the next input element.
         std::vector<Tensor> input_element;
@@ -594,7 +601,7 @@ class ParseExampleDatasetOp : public UnaryDatasetOpKernel {
           return;
         }
 
-        auto done = [this, ctx, result](Status status) {
+        auto done = [this, ctx, result](absl::Status status) {
           result->status.Update(status);
           CallCompleted(ctx, result);
         };
@@ -626,8 +633,8 @@ class ParseExampleDatasetOp : public UnaryDatasetOpKernel {
         RecordStart(ctx.get());
       }
 
-      Status CheckOutputTensor(const Tensor& tensor, size_t value_index,
-                               size_t output_index) const {
+      absl::Status CheckOutputTensor(const Tensor& tensor, size_t value_index,
+                                     size_t output_index) const {
         if (tensor.dtype() != dataset()->output_dtypes()[output_index]) {
           return errors::InvalidArgument(
               "Got wrong type for FastParseExample return value ", value_index,
@@ -643,18 +650,18 @@ class ParseExampleDatasetOp : public UnaryDatasetOpKernel {
               dataset()->output_shapes()[output_index].DebugString(), ", got ",
               tensor.shape().DebugString(), ").");
         }
-        return OkStatus();
+        return absl::OkStatus();
       }
 
-      Status ParseExample(IteratorContext* ctx, std::vector<Tensor> input,
-                          std::vector<Tensor>* output) {
+      absl::Status ParseExample(IteratorContext* ctx, std::vector<Tensor> input,
+                                std::vector<Tensor>* output) {
         thread::ThreadPool* device_threadpool =
             ctx->flr()->device()->tensorflow_cpu_worker_threads()->workers;
         std::vector<tstring> slice_vec;
         for (const Tensor& t : input) {
           auto serialized_t = t.flat<tstring>();
-          gtl::ArraySlice<tstring> slice(serialized_t.data(),
-                                         serialized_t.size());
+          absl::Span<const tstring> slice(serialized_t.data(),
+                                          serialized_t.size());
           for (auto it = slice.begin(); it != slice.end(); it++)
             slice_vec.push_back(*it);
         }
@@ -722,20 +729,20 @@ class ParseExampleDatasetOp : public UnaryDatasetOpKernel {
                 steps);
           }
         }
-        return OkStatus();
+        return absl::OkStatus();
       }
 
-      Status ProcessResult(IteratorContext* ctx,
-                           const std::shared_ptr<InvocationResult>& result,
-                           std::vector<Tensor>* out_tensors,
-                           bool* end_of_sequence) TF_LOCKS_EXCLUDED(*mu_) {
+      absl::Status ProcessResult(
+          IteratorContext* ctx, const std::shared_ptr<InvocationResult>& result,
+          std::vector<Tensor>* out_tensors, bool* end_of_sequence)
+          TF_LOCKS_EXCLUDED(*mu_) {
         if (!result->end_of_input && result->status.ok()) {
           *out_tensors = std::move(result->return_values);
           RecordBufferDequeue(ctx, *out_tensors);
           *end_of_sequence = false;
-          return OkStatus();
+          return absl::OkStatus();
         }
-        if (errors::IsOutOfRange(result->status)) {
+        if (absl::IsOutOfRange(result->status)) {
           // To guarantee that the transformation preserves the cardinality of
           // the dataset, we convert `OutOfRange` to `InvalidArgument` as the
           // former may be interpreted by a caller as the end of sequence.
@@ -849,8 +856,8 @@ class ParseExampleDatasetOp : public UnaryDatasetOpKernel {
         }
       }
 
-      Status WriteStatusLocked(IteratorStateWriter* writer, size_t index,
-                               const Status& status)
+      absl::Status WriteStatusLocked(IteratorStateWriter* writer, size_t index,
+                                     const absl::Status& status)
           TF_EXCLUSIVE_LOCKS_REQUIRED(*mu_) {
         TF_RETURN_IF_ERROR(writer->WriteScalar(
             CodeKey(index), static_cast<int64_t>(status.code())));
@@ -858,11 +865,11 @@ class ParseExampleDatasetOp : public UnaryDatasetOpKernel {
           TF_RETURN_IF_ERROR(writer->WriteScalar(
               ErrorMessageKey(index), std::string(status.message())));
         }
-        return OkStatus();
+        return absl::OkStatus();
       }
 
-      Status ReadStatusLocked(IteratorStateReader* reader, size_t index,
-                              Status* status)
+      absl::Status ReadStatusLocked(IteratorStateReader* reader, size_t index,
+                                    absl::Status* status)
           TF_EXCLUSIVE_LOCKS_REQUIRED(*mu_) {
         int64_t code_int;
         TF_RETURN_IF_ERROR(reader->ReadScalar(CodeKey(index), &code_int));
@@ -872,11 +879,11 @@ class ParseExampleDatasetOp : public UnaryDatasetOpKernel {
           tstring error_message;
           TF_RETURN_IF_ERROR(
               reader->ReadScalar(ErrorMessageKey(index), &error_message));
-          *status = Status(code, error_message);
+          *status = absl::Status(code, error_message);
         } else {
-          *status = OkStatus();
+          *status = absl::OkStatus();
         }
-        return OkStatus();
+        return absl::OkStatus();
       }
 
       string CodeKey(size_t index) {
