@@ -13,8 +13,12 @@ See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
 
+#include <optional>
+
+#include "absl/strings/str_join.h"
 #include "tensorflow/core/data/name_utils.h"
 #include "tensorflow/core/framework/dataset.h"
+#include "tensorflow/core/framework/tensor.h"
 #include "tensorflow/core/framework/tensor_util.h"
 #include "tensorflow/core/platform/stringprintf.h"
 
@@ -91,27 +95,27 @@ class RebatchDatasetOp : public UnaryDatasetOpKernel {
       return name_utils::DatasetDebugString(kDatasetTypeV1, params);
     }
 
-    Status InputDatasets(
+    absl::Status InputDatasets(
         std::vector<const DatasetBase*>* inputs) const override {
       inputs->push_back(input_);
-      return OkStatus();
+      return absl::OkStatus();
     }
 
-    Status CheckExternalState() const override {
+    absl::Status CheckExternalState() const override {
       return input_->CheckExternalState();
     }
 
    protected:
-    Status AsGraphDefInternal(SerializationContext* ctx,
-                              DatasetGraphDefBuilder* b,
-                              Node** output) const override {
+    absl::Status AsGraphDefInternal(SerializationContext* ctx,
+                                    DatasetGraphDefBuilder* b,
+                                    Node** output) const override {
       Node* input_graph_node = nullptr;
       TF_RETURN_IF_ERROR(b->AddInputDataset(ctx, input_, &input_graph_node));
       Node* num_replicas = nullptr;
       TF_RETURN_IF_ERROR(b->AddScalar(num_replicas_, &num_replicas));
       TF_RETURN_IF_ERROR(
           b->AddDataset(this, {input_graph_node, num_replicas}, output));
-      return OkStatus();
+      return absl::OkStatus();
     }
 
    private:
@@ -122,14 +126,14 @@ class RebatchDatasetOp : public UnaryDatasetOpKernel {
 
       ~Iterator() override {}
 
-      Status Initialize(IteratorContext* ctx) override {
+      absl::Status Initialize(IteratorContext* ctx) override {
         return dataset()->input_->MakeIterator(ctx, this, prefix(),
                                                &input_impl_);
       }
 
-      Status GetNextInternal(IteratorContext* ctx,
-                             std::vector<Tensor>* out_tensors,
-                             bool* end_of_sequence) override {
+      absl::Status GetNextInternal(IteratorContext* ctx,
+                                   std::vector<Tensor>* out_tensors,
+                                   bool* end_of_sequence) override {
         mutex_lock l(mu_);
         *end_of_sequence = false;
         if (slice_number_ % dataset()->num_replicas_ == 0) {
@@ -138,7 +142,7 @@ class RebatchDatasetOp : public UnaryDatasetOpKernel {
           TF_RETURN_IF_ERROR(
               input_impl_->GetNext(ctx, &input_tensors, end_of_sequence));
           if (*end_of_sequence) {
-            return OkStatus();
+            return absl::OkStatus();
           }
 
           input_descriptors_.reserve(input_tensors.size());
@@ -182,12 +186,12 @@ class RebatchDatasetOp : public UnaryDatasetOpKernel {
           }
         }
         slice_number_ = (slice_number_ + 1) % dataset()->num_replicas_;
-        return OkStatus();
+        return absl::OkStatus();
       }
 
      protected:
-      Status SaveInternal(SerializationContext* ctx,
-                          IteratorStateWriter* writer) override {
+      absl::Status SaveInternal(SerializationContext* ctx,
+                                IteratorStateWriter* writer) override {
         mutex_lock l(mu_);
         if (!input_impl_) {
           TF_RETURN_IF_ERROR(
@@ -206,11 +210,11 @@ class RebatchDatasetOp : public UnaryDatasetOpKernel {
                 input_descriptors_[i].whole_tensor));
           }
         }
-        return OkStatus();
+        return absl::OkStatus();
       }
 
-      Status RestoreInternal(IteratorContext* ctx,
-                             IteratorStateReader* reader) override {
+      absl::Status RestoreInternal(IteratorContext* ctx,
+                                   IteratorStateReader* reader) override {
         mutex_lock l(mu_);
         if (!reader->Contains(full_name("input_impl_empty"))) {
           TF_RETURN_IF_ERROR(RestoreInput(ctx, reader, input_impl_));
@@ -234,7 +238,7 @@ class RebatchDatasetOp : public UnaryDatasetOpKernel {
                         dataset()->num_replicas_);
           }
         }
-        return OkStatus();
+        return absl::OkStatus();
       }
 
       TraceMeMetadata GetTraceMeMetadata() const override {
@@ -327,7 +331,7 @@ class RebatchDatasetV2Op : public UnaryDatasetOpKernel {
           output_types_(output_types),
           output_shapes_(output_shapes),
           traceme_metadata_(
-              {{"batch_sizes", absl::StrJoin(batch_sizes, ",")}}) {
+              {{"batch_sizes", absl::StrJoin(batch_sizes_, ",")}}) {
       input_->Ref();
     }
 
@@ -352,20 +356,20 @@ class RebatchDatasetV2Op : public UnaryDatasetOpKernel {
       return name_utils::DatasetDebugString(kDatasetTypeV2);
     }
 
-    Status InputDatasets(
+    absl::Status InputDatasets(
         std::vector<const DatasetBase*>* inputs) const override {
       inputs->push_back(input_);
-      return OkStatus();
+      return absl::OkStatus();
     }
 
-    Status CheckExternalState() const override {
+    absl::Status CheckExternalState() const override {
       return input_->CheckExternalState();
     }
 
    protected:
-    Status AsGraphDefInternal(SerializationContext* ctx,
-                              DatasetGraphDefBuilder* b,
-                              Node** output) const override {
+    absl::Status AsGraphDefInternal(SerializationContext* ctx,
+                                    DatasetGraphDefBuilder* b,
+                                    Node** output) const override {
       Node* input_graph_node = nullptr;
       TF_RETURN_IF_ERROR(b->AddInputDataset(ctx, input_, &input_graph_node));
       Node* batch_sizes = nullptr;
@@ -374,7 +378,7 @@ class RebatchDatasetV2Op : public UnaryDatasetOpKernel {
       TF_RETURN_IF_ERROR(b->AddScalar(drop_remainder_, &drop_remainder));
       TF_RETURN_IF_ERROR(b->AddDataset(
           this, {input_graph_node, batch_sizes, drop_remainder}, output));
-      return OkStatus();
+      return absl::OkStatus();
     }
 
    private:
@@ -385,18 +389,18 @@ class RebatchDatasetV2Op : public UnaryDatasetOpKernel {
 
       ~Iterator() override {}
 
-      Status Initialize(IteratorContext* ctx) override {
+      absl::Status Initialize(IteratorContext* ctx) override {
         return dataset()->input_->MakeIterator(ctx, this, prefix(),
                                                &input_impl_);
       }
 
-      Status GetNextInternal(IteratorContext* ctx,
-                             std::vector<Tensor>* out_tensors,
-                             bool* end_of_sequence) override {
+      absl::Status GetNextInternal(IteratorContext* ctx,
+                                   std::vector<Tensor>* out_tensors,
+                                   bool* end_of_sequence) override {
         mutex_lock l(mu_);
         if (end_of_sequence_) {
           *end_of_sequence = true;
-          return OkStatus();
+          return absl::OkStatus();
         }
 
         *end_of_sequence = false;
@@ -406,7 +410,7 @@ class RebatchDatasetV2Op : public UnaryDatasetOpKernel {
         // different input tensors.
         int64_t batch_size = 0;
 
-        std::vector<std::vector<Tensor>> slices_to_concatenate;
+        std::vector<std::vector<TensorSlice>> slices_to_concatenate;
         // Get slices from input tensors until they make up the whole batch
         // size or we run out of input.
         while (batch_size < desired_batch_size) {
@@ -427,10 +431,10 @@ class RebatchDatasetV2Op : public UnaryDatasetOpKernel {
               std::min(offset_ + desired_batch_size - batch_size,
                        tensors_[0].dim_size(0));
 
-          std::vector<Tensor> slices;
+          std::vector<TensorSlice> slices;
           slices.reserve(tensors_.size());
           for (const auto& tensor : tensors_) {
-            slices.push_back(tensor.Slice(offset_, slice_end));
+            slices.push_back(TensorSlice(tensor, offset_, slice_end));
           }
           slices_to_concatenate.push_back(std::move(slices));
 
@@ -452,7 +456,7 @@ class RebatchDatasetV2Op : public UnaryDatasetOpKernel {
             (dataset()->drop_remainder_ && batch_size < desired_batch_size)) {
           DCHECK(end_of_sequence_);
           *end_of_sequence = true;
-          return OkStatus();
+          return absl::OkStatus();
         }
 
         const size_t num_components = dataset()->output_dtypes().size();
@@ -487,27 +491,31 @@ class RebatchDatasetV2Op : public UnaryDatasetOpKernel {
                   Tensor(dataset()->output_dtypes()[i], tensor_shape));
             }
           }
-          return OkStatus();
+          return absl::OkStatus();
         }
 
         // Special case: when there's only one slice, we return the slice
         // directly where possible instead of copying the tensor data.
         if (slices_to_concatenate.size() == 1) {
-          auto tensors = std::move(slices_to_concatenate[0]);
+          std::vector<Tensor> tensors;
+          tensors.reserve(num_components);
           for (size_t i = 0; i < num_components; ++i) {
+            Tensor& tensor = slices_to_concatenate[0][i].Slice();
             // If the slice is aligned, we return it directly.
-            if (!tensors[i].IsAligned()) {
-              tensors[i] = tensor::DeepCopy(std::move(tensors[i]));
+            if (!tensor.IsAligned()) {
+              tensor = tensor::DeepCopy(tensor);
             }
+            tensors.push_back(std::move(tensor));
           }
           *out_tensors = std::move(tensors);
-          return OkStatus();
+          return absl::OkStatus();
         }
 
         // For each component, concatenate slices into one tensor.
         for (size_t i = 0; i < num_components; ++i) {
           TensorShape component_shape({batch_size});
-          TensorShape remaining_shape = slices_to_concatenate[0][i].shape();
+          TensorShape remaining_shape =
+              slices_to_concatenate[0][i].Slice().shape();
           remaining_shape.RemoveDim(0);
           component_shape.AppendShape(remaining_shape);
           out_tensors->emplace_back(ctx->allocator({}),
@@ -519,20 +527,53 @@ class RebatchDatasetV2Op : public UnaryDatasetOpKernel {
           }
           int64_t dst_offset = 0;
           for (size_t j = 0; j < slices_to_concatenate.size(); ++j) {
-            auto num_slices = slices_to_concatenate[j][i].shape().dim_size(0);
-            TF_RETURN_IF_ERROR(batch_util::CopyContiguousSlices(
-                slices_to_concatenate[j][i], 0, dst_offset, num_slices,
-                &(*out_tensors)[i]));
+            auto num_slices =
+                slices_to_concatenate[j][i].Slice().shape().dim_size(0);
+            TensorSlice& slice = slices_to_concatenate[j][i];
+            if (slice.OwnsTensor()) {
+              slice.ClearSliceRef();
+              // Instead of using the slice,
+              // we directly use its parent tensor to make sure
+              // the reference count is 1 and can move the data potentially.
+              TF_RETURN_IF_ERROR(batch_util::MaybeMoveContiguousSlices(
+                  slice.Parent(), slice.Start(), dst_offset, num_slices,
+                  &(*out_tensors)[i]));
+            } else if (slice.ParentRefCount() == 3 &&
+                       j == slices_to_concatenate.size() - 1 &&
+                       !tensors_.empty()) {
+              // Special case:
+              // When `tensors_` still holds a reference to the tensor buffer,
+              // we could clear both parent and slice so that we can
+              // potentially move the underlying data by dirctly using
+              // `tensors_[i]`.
+              //
+              // For example:
+              // B = 3, B_new = 2
+              // tensors_:  [| e | e | e |, ...]
+              //               v   v
+              // new batch:  | e | e |
+              slice.ClearAllRefs();
+              Tensor& parent = tensors_[i];
+              TF_RETURN_IF_ERROR(batch_util::MaybeMoveContiguousSlices(
+                  parent, slice.Start(), dst_offset, num_slices,
+                  &(*out_tensors)[i]));
+            } else {
+              // Other iterator ops are holding references,
+              // we have to copy the underlying tensor buffer.
+              TF_RETURN_IF_ERROR(batch_util::CopyContiguousSlices(
+                  slice.Slice(), 0, dst_offset, num_slices,
+                  &(*out_tensors)[i]));
+            }
             dst_offset += num_slices;
           }
         }
 
-        return OkStatus();
+        return absl::OkStatus();
       }
 
      protected:
-      Status SaveInternal(SerializationContext* ctx,
-                          IteratorStateWriter* writer) override {
+      absl::Status SaveInternal(SerializationContext* ctx,
+                                IteratorStateWriter* writer) override {
         mutex_lock l(mu_);
         if (!input_impl_) {
           TF_RETURN_IF_ERROR(
@@ -549,11 +590,11 @@ class RebatchDatasetV2Op : public UnaryDatasetOpKernel {
                 full_name(strings::StrCat("tensors[", i, "]")), tensors_[i]));
           }
         }
-        return OkStatus();
+        return absl::OkStatus();
       }
 
-      Status RestoreInternal(IteratorContext* ctx,
-                             IteratorStateReader* reader) override {
+      absl::Status RestoreInternal(IteratorContext* ctx,
+                                   IteratorStateReader* reader) override {
         mutex_lock l(mu_);
         if (!reader->Contains(full_name("input_impl_empty"))) {
           TF_RETURN_IF_ERROR(RestoreInput(ctx, reader, input_impl_));
@@ -573,7 +614,7 @@ class RebatchDatasetV2Op : public UnaryDatasetOpKernel {
                 &tensors_[i]));
           }
         }
-        return OkStatus();
+        return absl::OkStatus();
       }
 
       TraceMeMetadata GetTraceMeMetadata() const override {
@@ -581,7 +622,7 @@ class RebatchDatasetV2Op : public UnaryDatasetOpKernel {
       }
 
      private:
-      Status ValidateInputTensors() TF_EXCLUSIVE_LOCKS_REQUIRED(mu_) {
+      absl::Status ValidateInputTensors() TF_EXCLUSIVE_LOCKS_REQUIRED(mu_) {
         for (size_t i = 0; i < tensors_.size(); ++i) {
           if (tensors_[i].dims() == 0) {
             return errors::InvalidArgument(
@@ -596,9 +637,53 @@ class RebatchDatasetV2Op : public UnaryDatasetOpKernel {
                 tensors_[i].dim_size(0), ".");
           }
         }
-        return OkStatus();
+        return absl::OkStatus();
       }
 
+      class TensorSlice {
+       public:
+        TensorSlice(const Tensor& t, int64_t start, int64_t end)
+            : start_(start),
+              end_(end),
+              parent_(std::make_unique<Tensor>(t)),
+              slice_(std::make_unique<Tensor>(t.Slice(start, end))) {}
+        bool OwnsTensor() {
+          // If this iterator op owns this tensor,
+          // there will be one reference from `parent_` and one from `slice_`.
+          // Otherwise, some other iterator op might own this tensor.
+          // For example, tensor_dataset_op.cc
+          auto ref_count = ParentRefCount();
+          if (ref_count) {
+            return *ref_count == 2;
+          } else {
+            return false;
+          }
+        }
+        std::optional<int> ParentRefCount() {
+          if (parent_->data() == nullptr) {
+            return std::nullopt;
+          }
+          return parent_->RefCount();
+        }
+
+        Tensor& Slice() { return *slice_; }
+        Tensor& Parent() { return *parent_; }
+        inline void ClearSliceRef() { slice_.reset(); }
+        inline void ClearAllRefs() {
+          parent_.reset();
+          slice_.reset();
+        }
+        int64_t Start() { return start_; }
+        int64_t End() { return end_; }
+
+       private:
+        const int64_t start_;
+        const int64_t end_;
+        std::unique_ptr<Tensor> parent_;
+        // A slice taken from the first dimension of `parent`.
+        std::unique_ptr<Tensor> slice_;
+      };
+
       mutex mu_;
       std::unique_ptr<IteratorBase> input_impl_;
       // Whether we have reached the end of the input.
