@@ -12,9 +12,15 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
+#include <cstdint>
 #include <memory>
+#include <unordered_map>
+#include <utility>
 
+#include "absl/status/status.h"
+#include "absl/types/span.h"
 #include "tensorflow/core/framework/op_kernel.h"
+#include "tensorflow/core/framework/op_requires.h"
 #include "tensorflow/core/framework/resource_op_kernel.h"
 #include "tensorflow/core/framework/stats_aggregator.h"
 #include "tensorflow/core/framework/summary.pb.h"
@@ -48,7 +54,7 @@ class StatsAggregatorImpl : public StatsAggregator {
  public:
   StatsAggregatorImpl() {}
 
-  void AddToHistogram(const string& name, gtl::ArraySlice<double> values,
+  void AddToHistogram(const string& name, absl::Span<const double> values,
                       const int64_t steps) override {
     mutex_lock l(mu_);
     histogram::Histogram& histogram = histograms_[name];
@@ -83,9 +89,9 @@ class StatsAggregatorImpl : public StatsAggregator {
 
   // StatsAggregator implementation for V2 is based on push-based summary, no-op
   // in V1.
-  Status SetSummaryWriter(
+  absl::Status SetSummaryWriter(
       SummaryWriterInterface* summary_writer_interface) override {
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   void IncrementCounter(const string& name, const string& label,
@@ -109,7 +115,8 @@ class StatsAggregatorImpl : public StatsAggregator {
   std::unordered_map<string, histogram::Histogram> histograms_
       TF_GUARDED_BY(mu_);
   std::unordered_map<string, float> scalars_ TF_GUARDED_BY(mu_);
-  TF_DISALLOW_COPY_AND_ASSIGN(StatsAggregatorImpl);
+  StatsAggregatorImpl(const StatsAggregatorImpl&) = delete;
+  void operator=(const StatsAggregatorImpl&) = delete;
 };
 
 class StatsAggregatorHandleOp
@@ -119,10 +126,10 @@ class StatsAggregatorHandleOp
       : ResourceOpKernel<StatsAggregatorResource>(ctx) {}
 
  private:
-  Status CreateResource(StatsAggregatorResource** ret) override
+  absl::Status CreateResource(StatsAggregatorResource** ret) override
       TF_EXCLUSIVE_LOCKS_REQUIRED(mu_) {
     *ret = new StatsAggregatorResource(std::make_unique<StatsAggregatorImpl>());
-    return OkStatus();
+    return absl::OkStatus();
   }
 };
 
@@ -136,7 +143,7 @@ class StatsAggregatorImplV2 : public StatsAggregator {
     }
   }
 
-  void AddToHistogram(const string& name, gtl::ArraySlice<double> values,
+  void AddToHistogram(const string& name, absl::Span<const double> values,
                       const int64_t steps) override {
     mutex_lock l(mu_);
     histogram::Histogram& histogram = histograms_[name];
@@ -153,11 +160,11 @@ class StatsAggregatorImplV2 : public StatsAggregator {
   }
 
   // TODO(b/116314787): expose this is public API to manually flush summary.
-  Status Flush() {
+  absl::Status Flush() {
     mutex_lock l(mu_);
     if (summary_writer_interface_)
       TF_RETURN_IF_ERROR(summary_writer_interface_->Flush());
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   void IncrementCounter(const string& name, const string& label,
@@ -179,7 +186,7 @@ class StatsAggregatorImplV2 : public StatsAggregator {
   // in V2.
   void EncodeToProto(Summary* out_summary) override {}
 
-  Status SetSummaryWriter(
+  absl::Status SetSummaryWriter(
       SummaryWriterInterface* summary_writer_interface) override {
     mutex_lock l(mu_);
     if (summary_writer_interface_) {
@@ -192,7 +199,7 @@ class StatsAggregatorImplV2 : public StatsAggregator {
     }
     summary_writer_interface_ = summary_writer_interface;
     summary_writer_interface_->Ref();
-    return OkStatus();
+    return absl::OkStatus();
   }
 
  private:
@@ -233,7 +240,8 @@ class StatsAggregatorImplV2 : public StatsAggregator {
   // context
   std::unordered_map<string, histogram::Histogram> histograms_
       TF_GUARDED_BY(mu_);
-  TF_DISALLOW_COPY_AND_ASSIGN(StatsAggregatorImplV2);
+  StatsAggregatorImplV2(const StatsAggregatorImplV2&) = delete;
+  void operator=(const StatsAggregatorImplV2&) = delete;
 };
 
 class StatsAggregatorHandleOpV2
@@ -243,11 +251,11 @@ class StatsAggregatorHandleOpV2
       : ResourceOpKernel<StatsAggregatorResource>(ctx) {}
 
  private:
-  Status CreateResource(StatsAggregatorResource** ret) override
+  absl::Status CreateResource(StatsAggregatorResource** ret) override
       TF_EXCLUSIVE_LOCKS_REQUIRED(mu_) {
     *ret =
         new StatsAggregatorResource(std::make_unique<StatsAggregatorImplV2>());
-    return OkStatus();
+    return absl::OkStatus();
   }
 };
 
@@ -262,8 +270,9 @@ class StatsAggregatorSummaryOp : public OpKernel {
                 errors::InvalidArgument("resource_handle must be a scalar"));
 
     core::RefCountPtr<StatsAggregatorResource> resource;
-    OP_REQUIRES_OK(ctx,
-                   LookupResource(ctx, HandleFromInput(ctx, 0), &resource));
+    ResourceHandle handle;
+    OP_REQUIRES_OK(ctx, HandleFromInput(ctx, 0, &handle));
+    OP_REQUIRES_OK(ctx, LookupResource(ctx, handle, &resource));
 
     Tensor* summary_t;
     OP_REQUIRES_OK(ctx, ctx->allocate_output(0, TensorShape({}), &summary_t));
@@ -284,16 +293,19 @@ class StatsAggregatorSetSummaryWriterOp : public OpKernel {
                 errors::InvalidArgument("resource_handle must be a scalar"));
 
     core::RefCountPtr<StatsAggregatorResource> resource;
-    OP_REQUIRES_OK(ctx,
-                   LookupResource(ctx, HandleFromInput(ctx, 0), &resource));
+    ResourceHandle resource_handle;
+    OP_REQUIRES_OK(ctx, HandleFromInput(ctx, 0, &resource_handle));
+    OP_REQUIRES_OK(ctx, LookupResource(ctx, resource_handle, &resource));
 
     const Tensor& summary_resource_handle_t = ctx->input(1);
     OP_REQUIRES(ctx,
                 TensorShapeUtils::IsScalar(summary_resource_handle_t.shape()),
                 errors::InvalidArgument("resource_handle must be a scalar"));
     core::RefCountPtr<SummaryWriterInterface> summary_resource;
-    OP_REQUIRES_OK(
-        ctx, LookupResource(ctx, HandleFromInput(ctx, 1), &summary_resource));
+    ResourceHandle summary_r_handle;
+    OP_REQUIRES_OK(ctx, HandleFromInput(ctx, 1, &summary_r_handle));
+    OP_REQUIRES_OK(ctx,
+                   LookupResource(ctx, summary_r_handle, &summary_resource));
     TF_CHECK_OK(
         resource->stats_aggregator()->SetSummaryWriter(summary_resource.get()));
   }
