@@ -14,23 +14,34 @@ limitations under the License.
 ==============================================================================*/
 #include "tensorflow/core/kernels/data/parallel_map_dataset_op.h"
 
+#include <algorithm>
+#include <cstddef>
 #include <deque>
 #include <functional>
+#include <limits>
 #include <memory>
+#include <optional>
 #include <string>
 #include <utility>
 #include <vector>
 
+#include "absl/base/call_once.h"
+#include "absl/status/status.h"
+#include "absl/strings/str_format.h"
+#include "xla/tsl/platform/logging.h"
 #include "tensorflow/core/common_runtime/function.h"
 #include "tensorflow/core/common_runtime/input_colocation_exemption_registry.h"
 #include "tensorflow/core/data/dataset_utils.h"
 #include "tensorflow/core/data/name_utils.h"
 #include "tensorflow/core/data/stats_utils.h"
+#include "tensorflow/core/data/unbounded_thread_pool.h"
+#include "tensorflow/core/framework/dataset.h"
 #include "tensorflow/core/framework/metrics.h"
 #include "tensorflow/core/framework/model.h"
 #include "tensorflow/core/framework/partial_tensor_shape.h"
 #include "tensorflow/core/framework/stats_aggregator.h"
 #include "tensorflow/core/framework/tensor.h"
+#include "tensorflow/core/framework/tensor_shape.h"
 #include "tensorflow/core/lib/core/errors.h"
 #include "tensorflow/core/lib/random/random.h"
 #include "tensorflow/core/platform/status.h"
@@ -75,6 +86,13 @@ constexpr char kErrorMessage[] = "error_message";
 // Period between reporting dataset statistics.
 constexpr int kStatsReportingPeriodMillis = 1000;
 
+// Factor used to determine the autotune parallelism limit when using an
+// unbounded threadpool. The limit is determined by multiplying this factor
+// by the default threadpool size, which is typically based on the number of
+// CPU cores. Without this limit, we see autotune sometimes choose unreasonably
+// large values for the parallelism, e.g. creating 300k threads.
+constexpr int kUnboundedThreadpoolAutotuningFactor = 10;
+
 }  // namespace
 
 class ParallelMapDatasetOp::Dataset : public DatasetBase {
@@ -84,17 +102,19 @@ class ParallelMapDatasetOp::Dataset : public DatasetBase {
           const std::vector<PartialTensorShape>& output_shapes,
           DeterminismPolicy deterministic,
           std::unique_ptr<CapturedFunction> captured_func,
-          bool preserve_cardinality, int op_version)
+          bool preserve_cardinality, bool use_unbounded_threadpool,
+          int op_version)
       : Dataset(DatasetContext(ctx), input, num_parallel_calls, output_types,
                 output_shapes, deterministic, std::move(captured_func),
-                preserve_cardinality, op_version) {}
+                preserve_cardinality, use_unbounded_threadpool, op_version) {}
 
   Dataset(DatasetContext dataset_context, const DatasetBase* input,
           int64_t num_parallel_calls, const DataTypeVector& output_types,
           const std::vector<PartialTensorShape>& output_shapes,
           DeterminismPolicy deterministic,
           std::unique_ptr<CapturedFunction> captured_func,
-          bool preserve_cardinality, int op_version)
+          bool preserve_cardinality, bool use_unbounded_threadpool,
+          int op_version)
       : DatasetBase(std::move(dataset_context)),
         input_(input),
         num_parallel_calls_(num_parallel_calls),
@@ -102,9 +122,14 @@ class ParallelMapDatasetOp::Dataset : public DatasetBase {
         output_shapes_(output_shapes),
         deterministic_(deterministic),
         preserve_cardinality_(preserve_cardinality),
+        use_unbounded_threadpool_(use_unbounded_threadpool),
         captured_func_(std::move(captured_func)),
         op_version_(op_version) {
     input_->Ref();
+    random_indexing_compatible_ = absl::OkStatus();
+    if (input_ != nullptr) {
+      random_indexing_compatible_ = input_->RandomIndexingCompatible();
+    }
   }
 
   ~Dataset() override { input_->Unref(); }
@@ -138,33 +163,38 @@ class ParallelMapDatasetOp::Dataset : public DatasetBase {
     }
   }
 
-  Status Get(OpKernelContext* ctx, int64 index,
-             std::vector<Tensor>* out_tensors) const override {
+  absl::Status Get(OpKernelContext* ctx, int64 index,
+                   std::vector<Tensor>* out_tensors) const override {
     TF_RETURN_IF_ERROR(CheckRandomAccessCompatible(index));
+    absl::call_once(instantiated_captured_func_once_, [this, ctx] {
+      instantiated_captured_func_status_ = captured_func_->Instantiate(
+          InstantiateCapturedFunctionParams(ctx), &instantiated_captured_func_);
+    });
+    TF_RETURN_IF_ERROR(instantiated_captured_func_status_);
     std::vector<Tensor> args;
     TF_RETURN_IF_ERROR(input_->Get(ctx, index, &args));
-    if (!instantiated_captured_func_) {
-      TF_RETURN_IF_ERROR(
-          captured_func_->Instantiate(InstantiateCapturedFunctionParams(ctx),
-                                      &instantiated_captured_func_));
-    }
     return instantiated_captured_func_->RunInstantiated(args, out_tensors);
   }
 
-  Status InputDatasets(std::vector<const DatasetBase*>* inputs) const override {
+  absl::Status InputDatasets(
+      std::vector<const DatasetBase*>* inputs) const override {
     inputs->push_back(input_);
-    return OkStatus();
+    return absl::OkStatus();
   }
 
-  Status CheckExternalState() const override {
+  absl::Status CheckExternalState() const override {
     TF_RETURN_IF_ERROR(captured_func_->CheckExternalState());
     return input_->CheckExternalState();
   }
 
+  absl::Status RandomIndexingCompatible() const override {
+    return random_indexing_compatible_;
+  }
+
  protected:
-  Status AsGraphDefInternal(SerializationContext* ctx,
-                            DatasetGraphDefBuilder* b,
-                            Node** output) const override {
+  absl::Status AsGraphDefInternal(SerializationContext* ctx,
+                                  DatasetGraphDefBuilder* b,
+                                  Node** output) const override {
     // Input: input_dataset
     Node* input_graph_node = nullptr;
     TF_RETURN_IF_ERROR(b->AddInputDataset(ctx, input_, &input_graph_node));
@@ -184,7 +214,7 @@ class ParallelMapDatasetOp::Dataset : public DatasetBase {
       TF_RETURN_IF_ERROR(
           b->AddScalar(num_parallel_calls_, &num_parallel_calls));
     }
-    std::vector<std::pair<StringPiece, AttrValue>> attrs;
+    std::vector<std::pair<absl::string_view, AttrValue>> attrs;
 
     // Attr: f
     AttrValue f_attr;
@@ -219,13 +249,19 @@ class ParallelMapDatasetOp::Dataset : public DatasetBase {
     b->BuildAttrValue(preserve_cardinality_, &preserve_cardinality_attr);
     attrs.emplace_back(kPreserveCardinality, preserve_cardinality_attr);
 
+    // Attr: use_unbounded_threadpool
+    AttrValue use_unbounded_threadpool_attr;
+    b->BuildAttrValue(use_unbounded_threadpool_,
+                      &use_unbounded_threadpool_attr);
+    attrs.emplace_back(kUseUnboundedThreadpool, use_unbounded_threadpool_attr);
+
     TF_RETURN_IF_ERROR(b->AddDataset(
         this,
         {std::make_pair(0, input_graph_node),
          std::make_pair(2, num_parallel_calls)},  // Single tensor inputs.
         {std::make_pair(1, other_arguments)},     // Tensor list inputs.
         attrs, output));
-    return OkStatus();
+    return absl::OkStatus();
   }
 
  private:
@@ -240,6 +276,7 @@ class ParallelMapDatasetOp::Dataset : public DatasetBase {
           deterministic_(params.dataset->deterministic_.IsDeterministic() ||
                          params.dataset->deterministic_.IsDefault()),
           preserve_cardinality_(params.dataset->preserve_cardinality_),
+          use_unbounded_threadpool_(params.dataset->use_unbounded_threadpool_),
           autotune_(params.dataset->num_parallel_calls_ == model::kAutotune) {}
 
     ~Iterator() override {
@@ -252,10 +289,13 @@ class ParallelMapDatasetOp::Dataset : public DatasetBase {
       return deterministic_;
     }
 
-    Status Initialize(IteratorContext* ctx) override {
+    absl::Status Initialize(IteratorContext* ctx) override {
       mutex_lock l(*mu_);
       interleave_depth_ = ctx->interleave_depth();
-
+      if (use_unbounded_threadpool_) {
+        unbounded_thread_pool_ = std::make_unique<UnboundedThreadPool>(
+            ctx->env(), "tf_data_map_unbounded_thread_pool");
+      }
       if (num_parallel_calls_->value == model::kAutotune) {
         num_parallel_calls_->value = GetAutotuneDefaultParallelism(ctx);
       }
@@ -263,23 +303,23 @@ class ParallelMapDatasetOp::Dataset : public DatasetBase {
       TF_RETURN_IF_ERROR(RegisterCancellationCallback(
           ctx->cancellation_manager(),
           [this]() { CancelThreads(/*wait=*/false); }, &deregister_fn_));
-      IteratorContext::Params params(ctx);
-      params.cancellation_manager = cancellation_manager_.get();
-      IteratorContext iter_ctx(std::move(params));
+      auto params = std::make_unique<IteratorContext::Params>(ctx);
+      params->cancellation_manager = cancellation_manager_.get();
+      auto iter_ctx = std::make_unique<IteratorContext>(*params);
       TF_RETURN_IF_ERROR(dataset()->input_->MakeIterator(
-          &iter_ctx, this, prefix(), &input_impl_));
-      ctx->MergeCheckpoint(iter_ctx.checkpoint());
+          iter_ctx.get(), this, prefix(), &input_impl_));
+      ctx->MergeCheckpoint(iter_ctx->checkpoint());
       TF_RETURN_IF_ERROR(dataset()->captured_func_->Instantiate(
           ctx, &instantiated_captured_func_));
       if (ctx->warm_start() && !ctx->is_restoring()) {
         EnsureThreadsStarted(ctx);
       }
-      return OkStatus();
+      return absl::OkStatus();
     }
 
-    Status GetNextInternal(IteratorContext* ctx,
-                           std::vector<Tensor>* out_tensors,
-                           bool* end_of_sequence) override {
+    absl::Status GetNextInternal(IteratorContext* ctx,
+                                 std::vector<Tensor>* out_tensors,
+                                 bool* end_of_sequence) override {
       std::shared_ptr<InvocationResult> result;
       {
         mutex_lock l(*mu_);
@@ -296,9 +336,9 @@ class ParallelMapDatasetOp::Dataset : public DatasetBase {
       RecordStop(ctx);
       result->notification.WaitForNotification();
       RecordStart(ctx);
-      profiler::TraceMe traceme([&] {
-        return profiler::TraceMeEncode("ParallelMapConsume",
-                                       {{"element_id", result->uid}});
+      tsl::profiler::TraceMe traceme([&] {
+        return tsl::profiler::TraceMeEncode("ParallelMapConsume",
+                                            {{"element_id", result->uid}});
       });
       return ProcessResult(ctx, result, out_tensors, end_of_sequence);
     }
@@ -306,19 +346,50 @@ class ParallelMapDatasetOp::Dataset : public DatasetBase {
    protected:
     std::shared_ptr<model::Node> CreateNode(
         IteratorContext* ctx, model::Node::Args args) const override {
+      std::shared_ptr<model::Parameter> parameter;
+      double max_parallelism_value = ctx->runner_threadpool_size();
+      if (use_unbounded_threadpool_) {
+        max_parallelism_value *= kUnboundedThreadpoolAutotuningFactor;
+      }
+
+      int64_t min_parallelism =
+          std::min(static_cast<double>(GetAutotuneMinParallelism(ctx)),
+                   max_parallelism_value);
+      if (num_parallel_calls_ &&
+          dataset()->num_parallel_calls_ == model::kAutotune) {
+        parameter = model::MakeParameter(
+            "parallelism", num_parallel_calls_, /*min=*/min_parallelism,
+            /*max=*/max_parallelism_value,
+            // This is to ensure before this op has seen its first element,
+            // `MaximumBufferedBytes()` can use the correct `parameter->value`
+            // to estimate the maximum buffer bytes.
+            GetAutotuneDefaultParallelism(ctx));
+      } else {
+        parameter = model::MakeParameter("parallelism", num_parallel_calls_,
+                                         /*min=*/min_parallelism,
+                                         /*max=*/max_parallelism_value);
+      }
+      std::optional<int64_t> estimated_element_size =
+          dataset()->GetEstimatedElementSize();
+      if (!estimated_element_size) {
+        VLOG(2) << absl::StrFormat(
+            "Cannot estimate the size of the output tensor because the "
+            "output shape of node %s(id:%d) is only partially known.",
+            args.name, args.id);
+      }
+
       return model::MakeAsyncKnownRatioNode(
           std::move(args),
-          /*ratio=*/1,
-          {model::MakeParameter("parallelism", num_parallel_calls_, /*min=*/1,
-                                /*max=*/ctx->runner_threadpool_size())});
+          /*ratio=*/1, {std::move(parameter)},
+          /*is_legacy_prefetch_autotuned=*/false, estimated_element_size);
     }
 
-    Status SaveInternal(SerializationContext* ctx,
-                        IteratorStateWriter* writer) override {
+    absl::Status SaveInternal(SerializationContext* ctx,
+                              IteratorStateWriter* writer) override {
       TF_RETURN_IF_ERROR(ctx->HandleCheckExternalStateStatus(
           dataset()->captured_func_->CheckExternalState()));
       if (ctx->symbolic_checkpoint()) {
-        return OkStatus();
+        return absl::OkStatus();
       }
       mutex_lock l(*mu_);
       // Wait for all in-flight calls to complete.
@@ -350,16 +421,16 @@ class ParallelMapDatasetOp::Dataset : public DatasetBase {
             writer->WriteScalar(element_prefix, kEndOfInput,
                                 static_cast<int64_t>(result.end_of_input)));
       }
-      return OkStatus();
+      return absl::OkStatus();
     }
 
-    Status RestoreInternal(IteratorContext* ctx,
-                           IteratorStateReader* reader) override {
+    absl::Status RestoreInternal(IteratorContext* ctx,
+                                 IteratorStateReader* reader) override {
       mutex_lock l(*mu_);
       TF_RETURN_IF_ERROR(RestoreInput(ctx, reader, input_impl_));
       DCHECK(invocation_results_.empty());
       if (ctx->symbolic_checkpoint()) {
-        return OkStatus();
+        return absl::OkStatus();
       }
       int64_t invocation_results_size;
       TF_RETURN_IF_ERROR(reader->ReadScalar(
@@ -397,7 +468,7 @@ class ParallelMapDatasetOp::Dataset : public DatasetBase {
         RecordBufferEnqueue(ctx, result.return_values);
         result.notification.Notify();
       }
-      return OkStatus();
+      return absl::OkStatus();
     }
 
     TraceMeMetadata GetTraceMeMetadata() const override {
@@ -413,6 +484,9 @@ class ParallelMapDatasetOp::Dataset : public DatasetBase {
           std::make_pair("autotune", autotune_ ? "true" : "false"));
       result.push_back(
           std::make_pair("deterministic", deterministic_ ? "true" : "false"));
+      result.push_back(
+          std::make_pair("use_unbounded_threadpool",
+                         use_unbounded_threadpool_ ? "true" : "false"));
       result.push_back(std::make_pair(
           "parallelism",
           parallelism == -1
@@ -431,7 +505,7 @@ class ParallelMapDatasetOp::Dataset : public DatasetBase {
             checkpoint(MemoryCheckpoint{ctx->id_registry()}) {}
 
       Notification notification;
-      Status status;
+      absl::Status status;
       std::vector<Tensor> return_values;
       bool end_of_input = false;
       const int64_t uid;
@@ -476,9 +550,9 @@ class ParallelMapDatasetOp::Dataset : public DatasetBase {
     void CallFunction(const std::shared_ptr<IteratorContext>& ctx,
                       const std::shared_ptr<InvocationResult>& result)
         TF_LOCKS_EXCLUDED(*mu_) {
-      profiler::TraceMe traceme([&] {
-        return profiler::TraceMeEncode("ParallelMapProduce",
-                                       {{"element_id", result->uid}});
+      tsl::profiler::TraceMe traceme([&] {
+        return tsl::profiler::TraceMeEncode("ParallelMapProduce",
+                                            {{"element_id", result->uid}});
       });
       // Get the next input element.
       std::vector<Tensor> input_element;
@@ -490,15 +564,25 @@ class ParallelMapDatasetOp::Dataset : public DatasetBase {
         return;
       }
 
-      auto done = [this, ctx, result](Status status) {
-        result->status.Update(status);
+      auto done = [this, ctx, result](absl::Status status) {
+        if (!status.ok()) {
+          result->status = AddErrorContext(status);
+        }
         RecordBufferEnqueue(ctx.get(), result->return_values);
         CallCompleted(ctx, result);
       };
 
       // Apply the map function on `input_element`, storing the result in
       // `result->return_values`, and invoking `done` when finished.
-      if (dataset()->captured_func_->use_inter_op_parallelism()) {
+      if (use_unbounded_threadpool_) {
+        auto runner_fn = [this](std::function<void()> fn) {
+          this->unbounded_thread_pool_->Schedule(fn);
+        };
+        instantiated_captured_func_->RunAsync(
+            runner_fn, ctx->cancellation_manager(), ctx->collective_executor(),
+            std::move(input_element), &result->return_values, done,
+            model_node());
+      } else if (dataset()->captured_func_->use_inter_op_parallelism()) {
         instantiated_captured_func_->RunAsync(
             ctx.get(), std::move(input_element), &result->return_values,
             std::move(done), model_node());
@@ -515,7 +599,7 @@ class ParallelMapDatasetOp::Dataset : public DatasetBase {
             std::move(input_element));
         (*ctx->runner())(
             [this, ctx, fn = std::move(fn), done = std::move(done)]() {
-              Status s;
+              absl::Status s;
               // Check whether we are already recording to prevent invalid
               // nesting of `RecordStart` calls.
               if (IsRecording(ctx.get())) {
@@ -530,18 +614,18 @@ class ParallelMapDatasetOp::Dataset : public DatasetBase {
       }
     }
 
-    Status ProcessResult(IteratorContext* ctx,
-                         const std::shared_ptr<InvocationResult>& result,
-                         std::vector<Tensor>* out_tensors,
-                         bool* end_of_sequence) TF_LOCKS_EXCLUDED(*mu_) {
+    absl::Status ProcessResult(IteratorContext* ctx,
+                               const std::shared_ptr<InvocationResult>& result,
+                               std::vector<Tensor>* out_tensors,
+                               bool* end_of_sequence) TF_LOCKS_EXCLUDED(*mu_) {
       ctx->MergeCheckpoint(&result->checkpoint);
       if (!result->end_of_input && result->status.ok()) {
         *out_tensors = std::move(result->return_values);
         RecordBufferDequeue(ctx, *out_tensors);
         *end_of_sequence = false;
-        return OkStatus();
+        return absl::OkStatus();
       }
-      if (errors::IsOutOfRange(result->status)) {
+      if (absl::IsOutOfRange(result->status)) {
         if (preserve_cardinality_) {
           // To guarantee that the transformation preserves the cardinality of
           // the dataset, we convert `OutOfRange` to `InvalidArgument` as the
@@ -553,7 +637,7 @@ class ParallelMapDatasetOp::Dataset : public DatasetBase {
           // `f` may deliberately raise `errors::OutOfRange` to indicate
           // that we should terminate the iteration early.
           *end_of_sequence = true;
-          return OkStatus();
+          return absl::OkStatus();
         }
       }
       *end_of_sequence = result->end_of_input;
@@ -661,8 +745,9 @@ class ParallelMapDatasetOp::Dataset : public DatasetBase {
       }
     }
 
-    Status WriteStatusLocked(IteratorStateWriter* writer,
-                             const std::string& prefix, const Status& status)
+    absl::Status WriteStatusLocked(IteratorStateWriter* writer,
+                                   const std::string& prefix,
+                                   const absl::Status& status)
         TF_EXCLUSIVE_LOCKS_REQUIRED(*mu_) {
       TF_RETURN_IF_ERROR(
           writer->WriteScalar(prefix, absl::StrCat("_", kErrorCode),
@@ -672,11 +757,12 @@ class ParallelMapDatasetOp::Dataset : public DatasetBase {
                                                absl::StrCat("_", kErrorMessage),
                                                std::string(status.message())));
       }
-      return OkStatus();
+      return absl::OkStatus();
     }
 
-    Status ReadStatusLocked(IteratorStateReader* reader,
-                            const std::string& prefix, Status* status)
+    absl::Status ReadStatusLocked(IteratorStateReader* reader,
+                                  const std::string& prefix,
+                                  absl::Status* status)
         TF_EXCLUSIVE_LOCKS_REQUIRED(*mu_) {
       int64_t code_int;
       TF_RETURN_IF_ERROR(
@@ -687,11 +773,11 @@ class ParallelMapDatasetOp::Dataset : public DatasetBase {
         tstring error_message;
         TF_RETURN_IF_ERROR(reader->ReadScalar(
             prefix, absl::StrCat("_", kErrorMessage), &error_message));
-        *status = Status(code, error_message);
+        *status = absl::Status(code, error_message);
       } else {
-        *status = OkStatus();
+        *status = absl::OkStatus();
       }
-      return OkStatus();
+      return absl::OkStatus();
     }
 
     // Used for coordination between the main thread and the runner thread.
@@ -706,6 +792,7 @@ class ParallelMapDatasetOp::Dataset : public DatasetBase {
     const std::shared_ptr<model::SharedState> num_parallel_calls_;
     const bool deterministic_;
     const bool preserve_cardinality_;
+    const bool use_unbounded_threadpool_;
     const bool autotune_;
     // Counts the number of outstanding calls.
     int64_t num_calls_ TF_GUARDED_BY(*mu_) = 0;
@@ -722,6 +809,7 @@ class ParallelMapDatasetOp::Dataset : public DatasetBase {
     bool cancelled_ TF_GUARDED_BY(*mu_) = false;
     std::unique_ptr<Thread> runner_thread_ TF_GUARDED_BY(*mu_);
     std::unique_ptr<Thread> stats_thread_ TF_GUARDED_BY(*mu_);
+    std::unique_ptr<UnboundedThreadPool> unbounded_thread_pool_;
 
     // Method for deregistering the cancellation callback.
     std::function<void()> deregister_fn_;
@@ -739,11 +827,15 @@ class ParallelMapDatasetOp::Dataset : public DatasetBase {
   const std::vector<PartialTensorShape> output_shapes_;
   const DeterminismPolicy deterministic_;
   const bool preserve_cardinality_;
+  const bool use_unbounded_threadpool_;
   const std::unique_ptr<CapturedFunction> captured_func_;
   const int op_version_;
   // This is used for random access provided by Get().
+  mutable absl::once_flag instantiated_captured_func_once_;
+  mutable absl::Status instantiated_captured_func_status_;
   mutable std::unique_ptr<InstantiatedCapturedFunction>
       instantiated_captured_func_;
+  absl::Status random_indexing_compatible_;
 };
 
 ParallelMapDatasetOp::ParallelMapDatasetOp(OpKernelConstruction* ctx)
@@ -764,12 +856,15 @@ ParallelMapDatasetOp::ParallelMapDatasetOp(OpKernelConstruction* ctx)
     } else {
       deterministic_ = DeterminismPolicy(DeterminismPolicy::Type::kDefault);
     }
+    use_unbounded_threadpool_ = false;
   }
   if (op_version_ == 2) {
     std::string deterministic;
     OP_REQUIRES_OK(ctx, ctx->GetAttr(kDeterministic, &deterministic));
     OP_REQUIRES_OK(
         ctx, DeterminismPolicy::FromString(deterministic, &deterministic_));
+    OP_REQUIRES_OK(
+        ctx, ctx->GetAttr(kUseUnboundedThreadpool, &use_unbounded_threadpool_));
   }
   OP_REQUIRES_OK(ctx,
                  ctx->GetAttr(kPreserveCardinality, &preserve_cardinality_));
@@ -801,10 +896,10 @@ void ParallelMapDatasetOp::MakeDataset(OpKernelContext* ctx, DatasetBase* input,
     metrics::RecordTFDataAutotune(kDatasetType);
   }
 
-  *output =
-      new Dataset(ctx, input, num_parallel_calls, output_types_, output_shapes_,
-                  deterministic_, std::move(captured_func),
-                  preserve_cardinality_, op_version_);
+  *output = new Dataset(ctx, input, num_parallel_calls, output_types_,
+                        output_shapes_, deterministic_,
+                        std::move(captured_func), preserve_cardinality_,
+                        use_unbounded_threadpool_, op_version_);
 }
 
 std::unique_ptr<DatasetBase> MakeDataServiceUncompressDataset(
@@ -819,7 +914,8 @@ std::unique_ptr<DatasetBase> MakeDataServiceUncompressDataset(
       /*num_parallel_calls=*/model::kAutotune, output_types, output_shapes,
       DeterminismPolicy(DeterminismPolicy::Type::kDefault),
       std::move(captured_function),
-      /*preserve_cardinality=*/true, /*op_version=*/2);
+      /*preserve_cardinality=*/true,
+      /*use_unbounded_threadpool=*/false, /*op_version=*/2);
 }
 
 namespace {
