@@ -14,6 +14,7 @@ limitations under the License.
 ==============================================================================*/
 #include "tensorflow/core/kernels/data/shuffle_dataset_op.h"
 
+#include <atomic>
 #include <cstdint>
 #include <deque>
 #include <memory>
@@ -23,6 +24,8 @@ limitations under the License.
 #include <utility>
 #include <vector>
 
+#include "absl/container/flat_hash_set.h"
+#include "absl/strings/str_cat.h"
 #include "tensorflow/core/data/dataset_utils.h"
 #include "tensorflow/core/data/name_utils.h"
 #include "tensorflow/core/data/serialization_utils.h"
@@ -122,17 +125,18 @@ class ShuffleDatasetOpBase::ShuffleDatasetBase : public DatasetBase {
     }
   }
 
-  Status InputDatasets(std::vector<const DatasetBase*>* inputs) const override {
+  absl::Status InputDatasets(
+      std::vector<const DatasetBase*>* inputs) const override {
     inputs->push_back(input_);
-    return OkStatus();
+    return absl::OkStatus();
   }
 
-  Status CheckExternalState() const override {
+  absl::Status CheckExternalState() const override {
     return input_->CheckExternalState();
   }
 
-  Status Get(OpKernelContext* ctx, int64 index,
-             std::vector<Tensor>* out_tensors) const override {
+  absl::Status Get(OpKernelContext* ctx, int64 index,
+                   std::vector<Tensor>* out_tensors) const override {
     TF_RETURN_IF_ERROR(CheckRandomAccessCompatible(index));
     {
       mutex_lock l(mu_);
@@ -146,7 +150,7 @@ class ShuffleDatasetOpBase::ShuffleDatasetBase : public DatasetBase {
       shuffled_index = shuffled_indices_[index];
     }
     TF_RETURN_IF_ERROR(input_->Get(ctx, shuffled_index, out_tensors));
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   string DebugString() const override {
@@ -199,22 +203,28 @@ class ShuffleDatasetOpBase::ShuffleDatasetBase : public DatasetBase {
 
     bool SymbolicCheckpointCompatible() const override { return true; }
 
-    Status Initialize(IteratorContext* ctx) override {
+    absl::Status Initialize(IteratorContext* ctx) override {
       mutex_lock l(mu_);
       seed_generator_->GenerateSeeds(&seed_, &seed2_);
       ResetRngs();
-      return OkStatus();
+      // Initialize checkpoint_indices_ to the entire buffer.
+      if (ctx->symbolic_checkpoint()) {
+        for (int64_t i = 0; i < buffer_->size(); ++i) {
+          checkpoint_indices_.insert(i);
+        }
+      }
+      return absl::OkStatus();
     }
 
-    Status GetNextInternal(IteratorContext* ctx,
-                           std::vector<Tensor>* out_tensors,
-                           bool* end_of_sequence) override {
+    absl::Status GetNextInternal(IteratorContext* ctx,
+                                 std::vector<Tensor>* out_tensors,
+                                 bool* end_of_sequence) override {
       mutex_lock l(mu_);
       TF_RETURN_IF_ERROR(FillBuffer(ctx));
       if (num_elements_ == 0) {
         DCHECK(input_impl_ == nullptr);
         *end_of_sequence = true;
-        return OkStatus();
+        return absl::OkStatus();
       }
 
       *end_of_sequence = false;
@@ -229,9 +239,11 @@ class ShuffleDatasetOpBase::ShuffleDatasetBase : public DatasetBase {
       this->RecordBufferDequeue(ctx, *out_tensors);
       std::swap(buffer_->at(index),
                 buffer_->at(slices_.front()->start % buffer_->size()));
+      checkpoint_indices_.insert(index);
+      checkpoint_indices_.insert(slices_.front()->start % buffer_->size());
       slices_.front()->start++;
       num_elements_--;
-      return OkStatus();
+      return absl::OkStatus();
     }
 
    protected:
@@ -249,8 +261,8 @@ class ShuffleDatasetOpBase::ShuffleDatasetBase : public DatasetBase {
       generator_.Skip(num_random_samples_);
     }
 
-    Status SaveInternal(SerializationContext* ctx,
-                        IteratorStateWriter* writer) override {
+    absl::Status SaveInternal(SerializationContext* ctx,
+                              IteratorStateWriter* writer) override {
       mutex_lock l(mu_);
       // Save state needed to restore the random number generators.
       TF_RETURN_IF_ERROR(
@@ -273,8 +285,20 @@ class ShuffleDatasetOpBase::ShuffleDatasetBase : public DatasetBase {
       TF_RETURN_IF_ERROR(writer->WriteScalar(prefix(), kEpoch, epoch_));
       TF_RETURN_IF_ERROR(
           writer->WriteScalar(prefix(), kNumElements, num_elements_));
-      TF_RETURN_IF_ERROR(WriteElementsToCheckpoint(
-          writer, absl::StrCat(prefix(), kColon, "buffer"), *buffer_));
+      const std::string key_prefix = absl::StrCat(prefix(), kColon, "buffer");
+      if (ctx->symbolic_checkpoint()) {
+        // When symbolic checkpointing is turned on, `writer`
+        // already contains checkpoint of the shuffle buffer created by the
+        // previous invocation of this instance and the indices that need to be
+        // updated are stored in `checkpoint_indices`.
+        TF_RETURN_IF_ERROR(UpdateCheckpointElements(
+            writer, key_prefix, *buffer_, checkpoint_indices_));
+        checkpoint_indices_.clear();
+      } else {
+        TF_RETURN_IF_ERROR(
+            WriteElementsToCheckpoint(writer, key_prefix, *buffer_));
+      }
+
       TF_RETURN_IF_ERROR(
           writer->WriteScalar(prefix(), kSlicesSize, slices_.size()));
       for (size_t i = 0; i < slices_.size(); ++i) {
@@ -294,11 +318,11 @@ class ShuffleDatasetOpBase::ShuffleDatasetBase : public DatasetBase {
             writer->WriteScalar(this->prefix(), kDataProduced, ""));
       }
 
-      return OkStatus();
+      return absl::OkStatus();
     }
 
-    Status RestoreInternal(IteratorContext* ctx,
-                           IteratorStateReader* reader) override {
+    absl::Status RestoreInternal(IteratorContext* ctx,
+                                 IteratorStateReader* reader) override {
       mutex_lock l(mu_);
       // Restore the random number generators.
       int64_t num_random_samples;
@@ -339,6 +363,12 @@ class ShuffleDatasetOpBase::ShuffleDatasetBase : public DatasetBase {
       TF_RETURN_IF_ERROR(ReadElementsFromCheckpoint(
           ctx, reader, absl::StrCat(prefix(), kColon, "buffer"),
           buffer_.get()));
+      if (ctx->symbolic_checkpoint()) {
+        DCHECK(checkpoint_indices_.empty());
+        for (size_t i = 0; i < buffer_->size(); ++i) {
+          checkpoint_indices_.insert(i);
+        }
+      }
       for (const auto& element : *buffer_) {
         RecordBufferEnqueue(ctx, element);
       }
@@ -365,7 +395,7 @@ class ShuffleDatasetOpBase::ShuffleDatasetBase : public DatasetBase {
       }
       data_produced_ = reader->Contains(this->prefix(), kDataProduced);
 
-      return OkStatus();
+      return absl::OkStatus();
     }
 
     TraceMeMetadata GetTraceMeMetadata() const override {
@@ -412,7 +442,8 @@ class ShuffleDatasetOpBase::ShuffleDatasetBase : public DatasetBase {
     }
 
     // Fills the shuffle buffer, preparing the buffer for sampling.
-    Status FillBuffer(IteratorContext* ctx) TF_EXCLUSIVE_LOCKS_REQUIRED(mu_) {
+    absl::Status FillBuffer(IteratorContext* ctx)
+        TF_EXCLUSIVE_LOCKS_REQUIRED(mu_) {
       int64_t start_micros = EnvTime::NowMicros();
       int64_t num_log_entries = 0;
       while (ShouldFillBuffer()) {
@@ -445,13 +476,13 @@ class ShuffleDatasetOpBase::ShuffleDatasetBase : public DatasetBase {
           // If we encounter the end of sequence without producing data, we
           // terminate the iteration immediately. (Otherwise, this iterator
           // would loop infinitely and never produce a value.)
-          return OkStatus();
+          return absl::OkStatus();
         }
       }
       if (num_log_entries > 0) {
         LOG(INFO) << "Shuffle buffer filled.";
       }
-      return OkStatus();
+      return absl::OkStatus();
     }
 
     bool ShouldFillBuffer() TF_EXCLUSIVE_LOCKS_REQUIRED(mu_) {
@@ -475,7 +506,7 @@ class ShuffleDatasetOpBase::ShuffleDatasetBase : public DatasetBase {
       return num_elements_ < buffer_->size();
     }
 
-    Status PrepareNextEpoch(IteratorContext* ctx)
+    absl::Status PrepareNextEpoch(IteratorContext* ctx)
         TF_EXCLUSIVE_LOCKS_REQUIRED(mu_) {
       if (epoch_ == 0) {
         slices_.push_back(std::make_unique<Slice>(0, 0, false));
@@ -489,7 +520,7 @@ class ShuffleDatasetOpBase::ShuffleDatasetBase : public DatasetBase {
       TF_RETURN_IF_ERROR(this->dataset()->input_->MakeIterator(
           ctx, this, this->prefix(), &input_impl_));
       epoch_++;
-      return OkStatus();
+      return absl::OkStatus();
     }
 
     void AddToShuffleBuffer(IteratorContext* ctx, std::vector<Tensor>&& element)
@@ -502,9 +533,11 @@ class ShuffleDatasetOpBase::ShuffleDatasetBase : public DatasetBase {
       this->RecordBufferEnqueue(ctx, element);
       if (num_elements_ == buffer_->size()) {
         DCHECK(IsShuffleAll());
+        checkpoint_indices_.insert(buffer_->size());
         buffer_->push_back(element);
       } else {
         size_t index = slices_.back()->end % buffer_->size();
+        checkpoint_indices_.insert(index);
         buffer_->at(index) = std::move(element);
       }
       num_elements_++;
@@ -530,6 +563,10 @@ class ShuffleDatasetOpBase::ShuffleDatasetBase : public DatasetBase {
     SeedGenerator* const seed_generator_ TF_GUARDED_BY(mu_);  // Not owned.
     std::unique_ptr<std::vector<std::vector<Tensor>>> buffer_
         TF_GUARDED_BY(mu_);
+    // Holds the indices of `buffer_` that have changed since the previous
+    // `SaveInternal()` and need to be updated in the MemoryCheckpoint
+    // (if symbolic checkpointing is used) in the next `SaveInternal()`.
+    absl::flat_hash_set<int64_t> checkpoint_indices_ TF_GUARDED_BY(mu_);
     std::unique_ptr<IteratorBase> input_impl_ TF_GUARDED_BY(mu_) = nullptr;
     int64_t epoch_ TF_GUARDED_BY(mu_) = 0;
     int64_t num_elements_ TF_GUARDED_BY(mu_) = 0;
@@ -576,7 +613,7 @@ class ShuffleDatasetOp::Dataset : public ShuffleDatasetBase {
 
   ~Dataset() override {
     manager_->Unref();
-    Status s = resource_mgr_->Delete<SeedGeneratorManager>(
+    absl::Status s = resource_mgr_->Delete<SeedGeneratorManager>(
         resource_handle_.container(), resource_handle_.name());
     if (!s.ok()) {
       LOG(WARNING) << "Failed to delete RNG resource: " << s.ToString();
@@ -586,9 +623,9 @@ class ShuffleDatasetOp::Dataset : public ShuffleDatasetBase {
   string op_type() const override { return kDatasetType; }
 
  protected:
-  Status AsGraphDefInternal(SerializationContext* ctx,
-                            DatasetGraphDefBuilder* b,
-                            Node** output) const override {
+  absl::Status AsGraphDefInternal(SerializationContext* ctx,
+                                  DatasetGraphDefBuilder* b,
+                                  Node** output) const override {
     Node* input_graph_node = nullptr;
     TF_RETURN_IF_ERROR(b->AddInputDataset(ctx, input_, &input_graph_node));
     Node* buffer_size_node = nullptr;
@@ -607,7 +644,7 @@ class ShuffleDatasetOp::Dataset : public ShuffleDatasetBase {
         {std::make_pair(kReshuffleEachIteration,
                         reshuffle_each_iteration)},  // Attrs
         output));
-    return OkStatus();
+    return absl::OkStatus();
   }
 
  private:
@@ -635,7 +672,7 @@ class ShuffleDatasetOp::DatasetV2 : public ShuffleDatasetBase {
   ~DatasetV2() override {
     manager_->Unref();
     if (owns_resource_) {
-      Status s = resource_mgr_->Delete<SeedGeneratorManager>(
+      absl::Status s = resource_mgr_->Delete<SeedGeneratorManager>(
           resource_handle_.container(), resource_handle_.name());
       if (!s.ok()) {
         LOG(WARNING) << "Failed to delete RNG resource: " << s.ToString();
@@ -646,9 +683,9 @@ class ShuffleDatasetOp::DatasetV2 : public ShuffleDatasetBase {
   string op_type() const override { return kDatasetType; }
 
  protected:
-  Status AsGraphDefInternal(SerializationContext* ctx,
-                            DatasetGraphDefBuilder* b,
-                            Node** output) const override {
+  absl::Status AsGraphDefInternal(SerializationContext* ctx,
+                                  DatasetGraphDefBuilder* b,
+                                  Node** output) const override {
     Node* input_graph_node = nullptr;
     TF_RETURN_IF_ERROR(b->AddInputDataset(ctx, input_, &input_graph_node));
     Node* buffer_size_node = nullptr;
@@ -662,7 +699,7 @@ class ShuffleDatasetOp::DatasetV2 : public ShuffleDatasetBase {
         {input_graph_node, buffer_size_node, resource_handle_node},  // Inputs
         {},                                                          // Attrs
         output));
-    return OkStatus();
+    return absl::OkStatus();
   }
 
  private:
@@ -690,7 +727,7 @@ class ShuffleDatasetOp::DatasetV3 : public ShuffleDatasetBase {
   ~DatasetV3() override {
     manager_->Unref();
     if (owns_resource_) {
-      Status s = resource_mgr_->Delete<SeedGeneratorManager>(
+      absl::Status s = resource_mgr_->Delete<SeedGeneratorManager>(
           resource_handle_.container(), resource_handle_.name());
       if (!s.ok()) {
         LOG(WARNING) << "Failed to delete RNG resource: " << s.ToString();
@@ -701,9 +738,9 @@ class ShuffleDatasetOp::DatasetV3 : public ShuffleDatasetBase {
   string op_type() const override { return kDatasetType; }
 
  protected:
-  Status AsGraphDefInternal(SerializationContext* ctx,
-                            DatasetGraphDefBuilder* b,
-                            Node** output) const override {
+  absl::Status AsGraphDefInternal(SerializationContext* ctx,
+                                  DatasetGraphDefBuilder* b,
+                                  Node** output) const override {
     Node* input_graph_node = nullptr;
     TF_RETURN_IF_ERROR(b->AddInputDataset(ctx, input_, &input_graph_node));
     Node* buffer_size_node = nullptr;
@@ -726,7 +763,7 @@ class ShuffleDatasetOp::DatasetV3 : public ShuffleDatasetBase {
                       {std::make_pair(kReshuffleEachIteration,
                                       reshuffle_each_iteration)},  // Attrs
                       output));
-    return OkStatus();
+    return absl::OkStatus();
   }
 
  private:
@@ -771,7 +808,7 @@ void ShuffleDatasetOp::MakeDataset(OpKernelContext* ctx, DatasetBase* input,
   if (op_version_ == 3) {
     auto handle = HandleFromInput(ctx, 4);
     SeedGeneratorManager* manager = nullptr;
-    Status s = ctx->resource_manager()->Lookup<SeedGeneratorManager>(
+    absl::Status s = ctx->resource_manager()->Lookup<SeedGeneratorManager>(
         handle.container(), handle.name(), &manager);
     int64_t seed;
     OP_REQUIRES_OK(ctx, ParseScalarArgument<int64_t>(ctx, kSeed, &seed));
@@ -779,7 +816,7 @@ void ShuffleDatasetOp::MakeDataset(OpKernelContext* ctx, DatasetBase* input,
     OP_REQUIRES_OK(ctx, ParseScalarArgument<int64_t>(ctx, kSeed2, &seed2));
     RandomSeeds seeds(seed, seed2);
     bool owns_resource = false;
-    if (errors::IsNotFound(s)) {
+    if (absl::IsNotFound(s)) {
       owns_resource = true;
       OP_REQUIRES_OK(
           ctx,
@@ -794,7 +831,7 @@ void ShuffleDatasetOp::MakeDataset(OpKernelContext* ctx, DatasetBase* input,
                   *manager =
                       new SeedGeneratorManager(new FixedSeedGenerator(seeds));
                 }
-                return OkStatus();
+                return absl::OkStatus();
               }));
       handle = MakeResourceHandle<SeedGenerator>(ctx, container, name);
     } else {
@@ -808,10 +845,10 @@ void ShuffleDatasetOp::MakeDataset(OpKernelContext* ctx, DatasetBase* input,
   } else if (op_version_ == 2) {
     auto handle = HandleFromInput(ctx, 2);
     SeedGeneratorManager* manager = nullptr;
-    Status s = ctx->resource_manager()->Lookup<SeedGeneratorManager>(
+    absl::Status s = ctx->resource_manager()->Lookup<SeedGeneratorManager>(
         handle.container(), handle.name(), &manager);
     bool owns_resource = false;
-    if (errors::IsNotFound(s)) {
+    if (absl::IsNotFound(s)) {
       owns_resource = true;
       LOG(WARNING) << "Failed to find seed generator resource. Falling back to "
                       "using a non-deterministically seeded generator and "
@@ -823,7 +860,7 @@ void ShuffleDatasetOp::MakeDataset(OpKernelContext* ctx, DatasetBase* input,
                    [&seeds](SeedGeneratorManager** manager) {
                      *manager = new SeedGeneratorManager(
                          new RandomSeedGenerator(seeds));
-                     return OkStatus();
+                     return absl::OkStatus();
                    }));
       handle = MakeResourceHandle<SeedGeneratorManager>(ctx, container, name);
     } else {
@@ -858,7 +895,7 @@ void ShuffleDatasetOp::MakeDataset(OpKernelContext* ctx, DatasetBase* input,
                 *manager =
                     new SeedGeneratorManager(new FixedSeedGenerator(seeds));
               }
-              return OkStatus();
+              return absl::OkStatus();
             }));
     auto handle =
         MakeResourceHandle<SeedGeneratorManager>(ctx, container, name);
@@ -883,7 +920,7 @@ class ShuffleAndRepeatDatasetOp::Dataset : public ShuffleDatasetBase {
 
   ~Dataset() override {
     manager_->Unref();
-    Status s = resource_mgr_->Delete<SeedGeneratorManager>(
+    absl::Status s = resource_mgr_->Delete<SeedGeneratorManager>(
         resource_handle_.container(), resource_handle_.name());
     if (!s.ok()) {
       LOG(WARNING) << "Failed to delete RNG resource: " << s.ToString();
@@ -893,9 +930,9 @@ class ShuffleAndRepeatDatasetOp::Dataset : public ShuffleDatasetBase {
   string op_type() const override { return kDatasetType; }
 
  protected:
-  Status AsGraphDefInternal(SerializationContext* ctx,
-                            DatasetGraphDefBuilder* b,
-                            Node** output) const override {
+  absl::Status AsGraphDefInternal(SerializationContext* ctx,
+                                  DatasetGraphDefBuilder* b,
+                                  Node** output) const override {
     Node* input_graph_node = nullptr;
     TF_RETURN_IF_ERROR(b->AddInputDataset(ctx, input_, &input_graph_node));
     Node* buffer_size = nullptr;
@@ -915,7 +952,7 @@ class ShuffleAndRepeatDatasetOp::Dataset : public ShuffleDatasetBase {
         {std::make_pair(kReshuffleEachIteration,
                         reshuffle_each_iteration)},  // Attrs
         output));
-    return OkStatus();
+    return absl::OkStatus();
   }
 
  private:
@@ -940,7 +977,7 @@ class ShuffleAndRepeatDatasetOp::DatasetV2 : public ShuffleDatasetBase {
   ~DatasetV2() override {
     manager_->Unref();
     if (owns_resource_) {
-      Status s = resource_mgr_->Delete<SeedGeneratorManager>(
+      absl::Status s = resource_mgr_->Delete<SeedGeneratorManager>(
           resource_handle_.container(), resource_handle_.name());
       if (!s.ok()) {
         LOG(WARNING) << "Failed to delete RNG resource: " << s.ToString();
@@ -951,9 +988,9 @@ class ShuffleAndRepeatDatasetOp::DatasetV2 : public ShuffleDatasetBase {
   string op_type() const override { return kDatasetType; }
 
  protected:
-  Status AsGraphDefInternal(SerializationContext* ctx,
-                            DatasetGraphDefBuilder* b,
-                            Node** output) const override {
+  absl::Status AsGraphDefInternal(SerializationContext* ctx,
+                                  DatasetGraphDefBuilder* b,
+                                  Node** output) const override {
     Node* input_graph_node = nullptr;
     TF_RETURN_IF_ERROR(b->AddInputDataset(ctx, input_, &input_graph_node));
     Node* buffer_size_node = nullptr;
@@ -978,7 +1015,7 @@ class ShuffleAndRepeatDatasetOp::DatasetV2 : public ShuffleDatasetBase {
                       {std::make_pair(kReshuffleEachIteration,
                                       reshuffle_each_iteration)},  // Attrs
                       output));
-    return OkStatus();
+    return absl::OkStatus();
   }
 
  private:
@@ -1036,10 +1073,10 @@ void ShuffleAndRepeatDatasetOp::MakeDataset(OpKernelContext* ctx,
   if (op_version_ == 2) {
     auto handle = HandleFromInput(ctx, 5);
     SeedGeneratorManager* manager = nullptr;
-    Status s = ctx->resource_manager()->Lookup<SeedGeneratorManager>(
+    absl::Status s = ctx->resource_manager()->Lookup<SeedGeneratorManager>(
         handle.container(), handle.name(), &manager);
     bool owns_resource = false;
-    if (errors::IsNotFound(s)) {
+    if (absl::IsNotFound(s)) {
       owns_resource = true;
       OP_REQUIRES_OK(
           ctx,
@@ -1054,7 +1091,7 @@ void ShuffleAndRepeatDatasetOp::MakeDataset(OpKernelContext* ctx,
                   *manager =
                       new SeedGeneratorManager(new FixedSeedGenerator(seeds));
                 }
-                return OkStatus();
+                return absl::OkStatus();
               }));
       handle = MakeResourceHandle<SeedGenerator>(ctx, container, name);
     } else {
@@ -1084,7 +1121,7 @@ void ShuffleAndRepeatDatasetOp::MakeDataset(OpKernelContext* ctx,
                 *manager =
                     new SeedGeneratorManager(new FixedSeedGenerator(seeds));
               }
-              return OkStatus();
+              return absl::OkStatus();
             }));
     auto handle =
         MakeResourceHandle<SeedGeneratorManager>(ctx, container, name);
