@@ -59,7 +59,7 @@ class ShuffleDatasetParams : public DatasetParams {
     return input_tensors;
   }
 
-  Status GetInputNames(std::vector<string>* input_names) const override {
+  absl::Status GetInputNames(std::vector<string>* input_names) const override {
     input_names->clear();
     input_names->emplace_back(ShuffleDatasetOpBase::kInputDataset);
     input_names->emplace_back(ShuffleDatasetOpBase::kBufferSize);
@@ -68,17 +68,17 @@ class ShuffleDatasetParams : public DatasetParams {
     if (count_ != 1) {
       input_names->emplace_back(ShuffleAndRepeatDatasetOp::kCount);
     }
-    return OkStatus();
+    return absl::OkStatus();
   }
 
-  Status GetAttributes(AttributeVector* attr_vector) const override {
+  absl::Status GetAttributes(AttributeVector* attr_vector) const override {
     attr_vector->clear();
     attr_vector->emplace_back("output_types", output_dtypes_);
     attr_vector->emplace_back("output_shapes", output_shapes_);
     attr_vector->emplace_back("reshuffle_each_iteration",
                               reshuffle_each_iteration_);
     attr_vector->emplace_back("metadata", "");
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   string dataset_type() const override {
