@@ -14,6 +14,11 @@ limitations under the License.
 ==============================================================================*/
 #include "tensorflow/core/kernels/data/take_dataset_op.h"
 
+#include <cstdint>
+#include <memory>
+#include <vector>
+
+#include "absl/status/status.h"
 #include "tensorflow/core/data/name_utils.h"
 #include "tensorflow/core/framework/op_kernel.h"
 #include "tensorflow/core/framework/partial_tensor_shape.h"
@@ -45,6 +50,10 @@ TakeDataset::TakeDataset(DatasetContext::Params params, int64_t count,
       count_(count),
       input_(input) {
   input_->Ref();
+  random_indexing_compatible_ = absl::OkStatus();
+  if (input_ != nullptr) {
+    random_indexing_compatible_ = input_->RandomIndexingCompatible();
+  }
 }
 
 TakeDataset::~TakeDataset() { input_->Unref(); }
@@ -74,22 +83,26 @@ int64_t TakeDataset::CardinalityInternal(CardinalityOptions options) const {
   return std::min(n, count_);
 }
 
-Status TakeDataset::InputDatasets(
+absl::Status TakeDataset::InputDatasets(
     std::vector<const DatasetBase*>* inputs) const {
   inputs->push_back(input_);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status TakeDataset::CheckExternalState() const {
+absl::Status TakeDataset::CheckExternalState() const {
   return input_->CheckExternalState();
 }
 
-Status TakeDataset::Get(OpKernelContext* ctx, int64 index,
-                        std::vector<Tensor>* out_tensors) const {
+absl::Status TakeDataset::Get(OpKernelContext* ctx, int64 index,
+                              std::vector<Tensor>* out_tensors) const {
   TF_RETURN_IF_ERROR(CheckRandomAccessCompatible(index));
   return input_->Get(ctx, index, out_tensors);
 }
 
+absl::Status TakeDataset::RandomIndexingCompatible() const {
+  return random_indexing_compatible_;
+}
+
 class TakeDataset::EmptyIterator : public DatasetIterator<TakeDataset> {
  public:
   explicit EmptyIterator(const Params& params)
@@ -97,10 +110,11 @@ class TakeDataset::EmptyIterator : public DatasetIterator<TakeDataset> {
 
   bool SymbolicCheckpointCompatible() const override { return true; }
 
-  Status GetNextInternal(IteratorContext* ctx, std::vector<Tensor>* out_tensors,
-                         bool* end_of_sequence) override {
+  absl::Status GetNextInternal(IteratorContext* ctx,
+                               std::vector<Tensor>* out_tensors,
+                               bool* end_of_sequence) override {
     *end_of_sequence = true;
-    return OkStatus();
+    return absl::OkStatus();
   }
 
  protected:
@@ -110,14 +124,14 @@ class TakeDataset::EmptyIterator : public DatasetIterator<TakeDataset> {
                                      /*ratio=*/1);
   }
 
-  Status SaveInternal(SerializationContext* ctx,
-                      IteratorStateWriter* writer) override {
-    return OkStatus();
+  absl::Status SaveInternal(SerializationContext* ctx,
+                            IteratorStateWriter* writer) override {
+    return absl::OkStatus();
   }
 
-  Status RestoreInternal(IteratorContext* ctx,
-                         IteratorStateReader* reader) override {
-    return OkStatus();
+  absl::Status RestoreInternal(IteratorContext* ctx,
+                               IteratorStateReader* reader) override {
+    return absl::OkStatus();
   }
 };
 
@@ -128,29 +142,30 @@ class TakeDataset::FiniteIterator : public DatasetIterator<TakeDataset> {
 
   bool SymbolicCheckpointCompatible() const override { return true; }
 
-  Status Initialize(IteratorContext* ctx) override {
+  absl::Status Initialize(IteratorContext* ctx) override {
     return dataset()->input_->MakeIterator(ctx, this, prefix(), &input_impl_);
   }
 
-  Status GetNextInternal(IteratorContext* ctx, std::vector<Tensor>* out_tensors,
-                         bool* end_of_sequence) override {
+  absl::Status GetNextInternal(IteratorContext* ctx,
+                               std::vector<Tensor>* out_tensors,
+                               bool* end_of_sequence) override {
     mutex_lock l(mu_);  // TODO(mrry): Make locking less conservative.
     if (!input_impl_) {
       *end_of_sequence = true;
-      return OkStatus();
+      return absl::OkStatus();
     }
     while (dataset()->count_ < 0 || i_ < dataset()->count_) {
       TF_RETURN_IF_ERROR(
           input_impl_->GetNext(ctx, out_tensors, end_of_sequence));
       if (!*end_of_sequence) {
         ++i_;
-        return OkStatus();
+        return absl::OkStatus();
       }
       break;
     }
     *end_of_sequence = true;
     input_impl_.reset();
-    return OkStatus();
+    return absl::OkStatus();
   }
 
  protected:
@@ -160,8 +175,8 @@ class TakeDataset::FiniteIterator : public DatasetIterator<TakeDataset> {
                                      /*ratio=*/1);
   }
 
-  Status SaveInternal(SerializationContext* ctx,
-                      IteratorStateWriter* writer) override {
+  absl::Status SaveInternal(SerializationContext* ctx,
+                            IteratorStateWriter* writer) override {
     mutex_lock l(mu_);
     TF_RETURN_IF_ERROR(writer->WriteScalar(prefix(), kCurIndex, i_));
     TF_RETURN_IF_ERROR(writer->WriteScalar(prefix(), kInputImplEmpty,
@@ -169,11 +184,11 @@ class TakeDataset::FiniteIterator : public DatasetIterator<TakeDataset> {
     if (input_impl_) {
       TF_RETURN_IF_ERROR(SaveInput(ctx, writer, input_impl_));
     }
-    return OkStatus();
+    return absl::OkStatus();
   }
 
-  Status RestoreInternal(IteratorContext* ctx,
-                         IteratorStateReader* reader) override {
+  absl::Status RestoreInternal(IteratorContext* ctx,
+                               IteratorStateReader* reader) override {
     mutex_lock l(mu_);
     TF_RETURN_IF_ERROR(reader->ReadScalar(prefix(), kCurIndex, &i_));
     int64_t input_empty;
@@ -184,7 +199,7 @@ class TakeDataset::FiniteIterator : public DatasetIterator<TakeDataset> {
     } else {
       input_impl_.reset();
     }
-    return OkStatus();
+    return absl::OkStatus();
   }
 
  private:
@@ -206,15 +221,15 @@ std::unique_ptr<IteratorBase> TakeDataset::MakeIteratorInternal(
   }
 }
 
-Status TakeDataset::AsGraphDefInternal(SerializationContext* ctx,
-                                       DatasetGraphDefBuilder* b,
-                                       Node** output) const {
+absl::Status TakeDataset::AsGraphDefInternal(SerializationContext* ctx,
+                                             DatasetGraphDefBuilder* b,
+                                             Node** output) const {
   Node* input_graph_node = nullptr;
   TF_RETURN_IF_ERROR(b->AddInputDataset(ctx, input_, &input_graph_node));
   Node* count = nullptr;
   TF_RETURN_IF_ERROR(b->AddScalar(count_, &count));
   TF_RETURN_IF_ERROR(b->AddDataset(this, {input_graph_node, count}, output));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 TakeDatasetOp::TakeDatasetOp(OpKernelConstruction* ctx)
