@@ -40,7 +40,8 @@ namespace tensorflow {
 
 class DebugIdentityOpTest : public OpsTestBase {
  protected:
-  Status Init(DataType input_type, const std::vector<string>& debug_urls) {
+  absl::Status Init(DataType input_type,
+                    const std::vector<string>& debug_urls) {
     env_ = Env::Default();
 
     TF_CHECK_OK(NodeDefBuilder("op", "DebugIdentity")
@@ -51,7 +52,7 @@ class DebugIdentityOpTest : public OpsTestBase {
     return InitOp();
   }
 
-  Status Init(DataType input_type) {
+  absl::Status Init(DataType input_type) {
     std::vector<string> empty_debug_urls;
     return Init(input_type, empty_debug_urls);
   }
@@ -178,7 +179,7 @@ TEST_F(DebugIdentityOpTest, StringSuccess) {
 // Tests for DebugNanCountOp
 class DebugNanCountOpTest : public OpsTestBase {
  protected:
-  Status Init(DataType input_type) {
+  absl::Status Init(DataType input_type) {
     TF_CHECK_OK(NodeDefBuilder("op", "DebugNanCount")
                     .Input(FakeInput(input_type))
                     .Attr("tensor_name", "FakeTensor:0")
@@ -241,7 +242,7 @@ TEST_F(DebugNanCountOpTest, Double_no_NaNs) {
 // Tests for DebugNumericSummaryOp
 class DebugNumericSummaryOpTest : public OpsTestBase {
  protected:
-  Status Init(DataType input_type) {
+  absl::Status Init(DataType input_type) {
     TF_CHECK_OK(NodeDefBuilder("op", "DebugNumericSummary")
                     .Input(FakeInput(input_type))
                     .Attr("tensor_name", "FakeTensor:0")
@@ -249,7 +250,8 @@ class DebugNumericSummaryOpTest : public OpsTestBase {
     return InitOp();
   }
 
-  Status InitGated(DataType input_type, const std::vector<string>& debug_urls) {
+  absl::Status InitGated(DataType input_type,
+                         const std::vector<string>& debug_urls) {
     TF_CHECK_OK(NodeDefBuilder("op", "DebugNumericSummary")
                     .Input(FakeInput(input_type))
                     .Attr("tensor_name", "FakeTensor:0")
@@ -632,7 +634,7 @@ TEST_F(DebugNumericSummaryOpTest, DisabledDueToNonMatchingWatchKey) {
 // Tests for DebugNumericSummaryOp
 class DebugNumericSummaryOpCustomLowerBoundTest : public OpsTestBase {
  protected:
-  Status Init(DataType input_type) {
+  absl::Status Init(DataType input_type) {
     TF_CHECK_OK(NodeDefBuilder("op", "DebugNumericSummary")
                     .Input(FakeInput(input_type))
                     .Attr("tensor_name", "FakeTensor:0")
@@ -684,7 +686,7 @@ TEST_F(DebugNumericSummaryOpCustomLowerBoundTest, Float_full_house) {
 // Tests for DebugNumericSummaryOp
 class DebugNumericSummaryOpCustomLowerUpperBoundsTest : public OpsTestBase {
  protected:
-  Status Init(DataType input_type) {
+  absl::Status Init(DataType input_type) {
     TF_CHECK_OK(NodeDefBuilder("op", "DebugNumericSummary")
                     .Input(FakeInput(input_type))
                     .Attr("tensor_name", "FakeTensor:0")
