@@ -56,9 +56,9 @@ class ParseExampleOp : public OpKernel {
   void Compute(OpKernelContext* ctx) override {
     const Tensor* names;
     const Tensor* serialized;
-    std::vector<StringPiece> dense_keys_t;
-    std::vector<StringPiece> sparse_keys_t;
-    std::vector<StringPiece> ragged_keys_t;
+    std::vector<absl::string_view> dense_keys_t;
+    std::vector<absl::string_view> sparse_keys_t;
+    std::vector<absl::string_view> ragged_keys_t;
     OpInputList dense_defaults;
 
     // Grab the inputs.
@@ -102,8 +102,8 @@ class ParseExampleOp : public OpKernel {
 
  protected:
   // Copies keys from tensor to std::vector<string>.
-  Status GetTensorKeys(OpKernelContext* ctx, StringPiece input_name,
-                       std::vector<StringPiece>* keys) const {
+  absl::Status GetTensorKeys(OpKernelContext* ctx, absl::string_view input_name,
+                             std::vector<absl::string_view>* keys) const {
     const Tensor* key_t;
     TF_RETURN_IF_ERROR(ctx->input(input_name, &key_t));
     keys->reserve(key_t->NumElements());
@@ -111,27 +111,29 @@ class ParseExampleOp : public OpKernel {
     for (int i = 0; i < keys_flat.size(); ++i) {
       keys->push_back(keys_flat(i));
     }
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   // Copies keys from OpInputList of scalar to std::vector<string>.
-  Status GetInputListKeys(OpKernelContext* ctx, StringPiece input_name,
-                          std::vector<StringPiece>* keys) const {
+  absl::Status GetInputListKeys(OpKernelContext* ctx,
+                                absl::string_view input_name,
+                                std::vector<absl::string_view>* keys) const {
     OpInputList key_list;
     TF_RETURN_IF_ERROR(ctx->input_list(input_name, &key_list));
     keys->reserve(key_list.size());
     for (const auto& key : key_list) {
       keys->push_back(key.scalar<tstring>()());
     }
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   // Validates the shapes of input tensors.
-  Status CheckInputShapes(const Tensor* serialized, const Tensor* names,
-                          const OpInputList& dense_defaults,
-                          const std::vector<StringPiece>& dense_keys_t,
-                          const std::vector<StringPiece>& sparse_keys_t,
-                          const std::vector<StringPiece>& ragged_keys_t) const {
+  absl::Status CheckInputShapes(
+      const Tensor* serialized, const Tensor* names,
+      const OpInputList& dense_defaults,
+      const std::vector<absl::string_view>& dense_keys_t,
+      const std::vector<absl::string_view>& sparse_keys_t,
+      const std::vector<absl::string_view>& ragged_keys_t) const {
     if (op_version_ == 2) {
       if (TensorShapeUtils::IsMatrixOrHigher(serialized->shape())) {
         return errors::InvalidArgument(
@@ -205,14 +207,14 @@ class ParseExampleOp : public OpKernel {
             "] == ", DataTypeString(attrs_.dense_types[d]));
       }
     }
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   // Populates the FastParseExampleConfig from keys & defaults.
   example::FastParseExampleConfig MakeConfig(
-      const std::vector<StringPiece>& dense_keys_t,
-      const std::vector<StringPiece>& sparse_keys_t,
-      const std::vector<StringPiece>& ragged_keys_t,
+      const std::vector<absl::string_view>& dense_keys_t,
+      const std::vector<absl::string_view>& sparse_keys_t,
+      const std::vector<absl::string_view>& ragged_keys_t,
       const OpInputList& dense_defaults) const {
     example::FastParseExampleConfig config;
     config.dense.reserve(attrs_.num_dense);
@@ -235,29 +237,30 @@ class ParseExampleOp : public OpKernel {
   }
 
   // Parses a single example.
-  Status ParseExampleScalar(const example::FastParseExampleConfig& config,
-                            const Tensor* serialized, OpKernelContext* ctx,
-                            example::Result* result) const {
+  absl::Status ParseExampleScalar(const example::FastParseExampleConfig& config,
+                                  const Tensor* serialized,
+                                  OpKernelContext* ctx,
+                                  example::Result* result) const {
     const tstring& serialized_proto = serialized->scalar<tstring>()();
     return FastParseSingleExample(config, serialized_proto, result);
   }
 
   // Parses a vector of examples.
-  Status ParseExampleVector(const example::FastParseExampleConfig& config,
-                            const Tensor* serialized, const Tensor* names,
-                            OpKernelContext* ctx,
-                            example::Result* result) const {
+  absl::Status ParseExampleVector(const example::FastParseExampleConfig& config,
+                                  const Tensor* serialized, const Tensor* names,
+                                  OpKernelContext* ctx,
+                                  example::Result* result) const {
     auto serialized_t = serialized->flat<tstring>();
     auto names_t = names->flat<tstring>();
-    gtl::ArraySlice<tstring> slice(serialized_t.data(), serialized_t.size());
-    gtl::ArraySlice<tstring> names_slice(names_t.data(), names_t.size());
+    absl::Span<const tstring> slice(serialized_t.data(), serialized_t.size());
+    absl::Span<const tstring> names_slice(names_t.data(), names_t.size());
     return FastParseExample(
         config, slice, names_slice,
         ctx->device()->tensorflow_cpu_worker_threads()->workers, result);
   }
 
-  Status WriteOutput(const example::Result& result,
-                     OpKernelContext* ctx) const {
+  absl::Status WriteOutput(const example::Result& result,
+                           OpKernelContext* ctx) const {
     OpOutputList dense_values;
     OpOutputList sparse_indices;
     OpOutputList sparse_values;
@@ -284,7 +287,7 @@ class ParseExampleOp : public OpKernel {
         ragged_splits.set(d, result.ragged_splits[d]);
       }
     }
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   ParseExampleAttrs attrs_;
@@ -470,9 +473,9 @@ class ParseSequenceExampleOp : public OpKernel {
     bool is_batch = TensorShapeUtils::IsVector(serialized->shape());
     auto serialized_t = serialized->flat<tstring>();
     auto debug_name_t = debug_name->flat<tstring>();
-    gtl::ArraySlice<tstring> slice(serialized_t.data(), serialized_t.size());
-    gtl::ArraySlice<tstring> names_slice(debug_name_t.data(),
-                                         debug_name_t.size());
+    absl::Span<const tstring> slice(serialized_t.data(), serialized_t.size());
+    absl::Span<const tstring> names_slice(debug_name_t.data(),
+                                          debug_name_t.size());
 
     example::Result context_result, feature_list_result;
     std::vector<Tensor> dense_feature_lengths;
@@ -488,7 +491,7 @@ class ParseSequenceExampleOp : public OpKernel {
   }
 
  protected:
-  Status CheckInputShapes(
+  absl::Status CheckInputShapes(
       const Tensor* serialized, const Tensor* names,
       const OpInputList& context_dense_defaults,
 
@@ -566,7 +569,7 @@ class ParseSequenceExampleOp : public OpKernel {
         }
       }
     }
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   example::FastParseExampleConfig MakeContextConfig(
@@ -575,21 +578,21 @@ class ParseSequenceExampleOp : public OpKernel {
       const OpInputList& context_dense_defaults) const {
     // Convert the tensors/attrs to ArraySlices once, instead of re-evaluating
     // them in each loop iteration.
-    gtl::ArraySlice<tstring> dense_keys_slice =
+    absl::Span<const tstring> dense_keys_slice =
         dense_keys
-            ? gtl::ArraySlice<tstring>(dense_keys->flat<tstring>().data(),
-                                       attrs_.num_context_dense)
+            ? absl::Span<const tstring>(dense_keys->flat<tstring>().data(),
+                                        attrs_.num_context_dense)
             : attrs_.context_dense_keys;
-    gtl::ArraySlice<tstring> sparse_keys_slice =
+    absl::Span<const tstring> sparse_keys_slice =
         sparse_keys
-            ? gtl::ArraySlice<tstring>(sparse_keys->flat<tstring>().data(),
-                                       attrs_.num_context_sparse)
+            ? absl::Span<const tstring>(sparse_keys->flat<tstring>().data(),
+                                        attrs_.num_context_sparse)
             : attrs_.context_sparse_keys;
-    gtl::ArraySlice<tstring> ragged_keys_slice =
+    absl::Span<const tstring> ragged_keys_slice =
         ragged_keys
-            ? gtl::ArraySlice<tstring>(ragged_keys->flat<tstring>().data(),
-                                       attrs_.num_context_ragged)
-            : gtl::ArraySlice<tstring>(nullptr, 0);
+            ? absl::Span<const tstring>(ragged_keys->flat<tstring>().data(),
+                                        attrs_.num_context_ragged)
+            : absl::Span<const tstring>(nullptr, 0);
 
     example::FastParseExampleConfig config;
     config.dense.reserve(attrs_.num_context_dense);
@@ -634,29 +637,29 @@ class ParseSequenceExampleOp : public OpKernel {
       const Tensor* feature_list_dense_missing_assumed_empty) const {
     // Convert the tensors/attrs to ArraySlices once, instead of re-evaluating
     // them in each loop iteration.
-    gtl::ArraySlice<tstring> dense_keys_slice =
+    absl::Span<const tstring> dense_keys_slice =
         dense_keys
-            ? gtl::ArraySlice<tstring>(dense_keys->flat<tstring>().data(),
-                                       attrs_.num_feature_list_dense)
+            ? absl::Span<const tstring>(dense_keys->flat<tstring>().data(),
+                                        attrs_.num_feature_list_dense)
             : attrs_.feature_list_dense_keys;
-    gtl::ArraySlice<tstring> sparse_keys_slice =
+    absl::Span<const tstring> sparse_keys_slice =
         sparse_keys
-            ? gtl::ArraySlice<tstring>(sparse_keys->flat<tstring>().data(),
-                                       attrs_.num_feature_list_sparse)
+            ? absl::Span<const tstring>(sparse_keys->flat<tstring>().data(),
+                                        attrs_.num_feature_list_sparse)
             : attrs_.feature_list_sparse_keys;
-    gtl::ArraySlice<tstring> ragged_keys_slice =
+    absl::Span<const tstring> ragged_keys_slice =
         ragged_keys
-            ? gtl::ArraySlice<tstring>(ragged_keys->flat<tstring>().data(),
-                                       attrs_.num_feature_list_ragged)
-            : gtl::ArraySlice<tstring>(nullptr, 0);
+            ? absl::Span<const tstring>(ragged_keys->flat<tstring>().data(),
+                                        attrs_.num_feature_list_ragged)
+            : absl::Span<const tstring>(nullptr, 0);
     // Use an empty slice to indicate that the map in attrs_ should be used
     // instead.
-    gtl::ArraySlice<bool> feature_list_dense_missing_assumed_empty_slice =
+    absl::Span<const bool> feature_list_dense_missing_assumed_empty_slice =
         feature_list_dense_missing_assumed_empty
-            ? gtl::ArraySlice<bool>(
+            ? absl::Span<const bool>(
                   feature_list_dense_missing_assumed_empty->flat<bool>().data(),
                   attrs_.num_feature_list_dense)
-            : gtl::ArraySlice<bool>(nullptr, 0);
+            : absl::Span<const bool>(nullptr, 0);
 
     example::FastParseExampleConfig config;
     config.dense.reserve(attrs_.num_feature_list_dense);
@@ -686,10 +689,10 @@ class ParseSequenceExampleOp : public OpKernel {
     return config;
   }
 
-  Status WriteOutput(const example::Result& context_result,
-                     const example::Result& feature_list_result,
-                     const std::vector<Tensor>& dense_feature_lengths,
-                     OpKernelContext* ctx) const {
+  absl::Status WriteOutput(const example::Result& context_result,
+                           const example::Result& feature_list_result,
+                           const std::vector<Tensor>& dense_feature_lengths,
+                           OpKernelContext* ctx) const {
     OpOutputList context_sparse_indices;
     OpOutputList context_sparse_values;
     OpOutputList context_sparse_shapes;
@@ -765,7 +768,7 @@ class ParseSequenceExampleOp : public OpKernel {
             d, feature_list_result.ragged_splits[d]);
       }
     }
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   ParseSequenceExampleAttrs attrs_;
@@ -965,7 +968,7 @@ class ParseSingleSequenceExampleOp : public OpKernel {
     options.start_block_size = std::max(options.start_block_size, block_size);
     options.max_block_size = std::max(options.max_block_size, block_size);
     protobuf::Arena arena(options);
-    auto& ex = *protobuf::Arena::CreateMessage<SequenceExample>(&arena);
+    auto& ex = *protobuf::Arena::Create<SequenceExample>(&arena);
 
     OP_REQUIRES(
         ctx, ParseProtoUnlimited(&ex, serialized_t()),
