@@ -34,6 +34,8 @@ typedef Eigen::GpuDevice GPUDevice;
 
 TF_CALL_INTEGRAL_TYPES(DEFINE_GPU_SPECS);
 TF_CALL_GPU_ALL_TYPES(DEFINE_GPU_SPECS);
+TF_CALL_float8_e5m2(DEFINE_GPU_SPECS);
+TF_CALL_float8_e4m3fn(DEFINE_GPU_SPECS);
 
 #undef DEFINE_GPU_SPECS
 #undef DEFINE_GPU_SPECS_INDEX
