@@ -77,6 +77,9 @@ class GatherOp : public OpKernel {
                     errors::InvalidArgument("axis must be int32 or int64."));
       }
     }
+    // special case to avoid checkfail when axis = kint64max.
+    OP_REQUIRES(c, axis < kint64max,
+                absl::InvalidArgumentError("axis must be less than kint64max"));
 
     int64_t min_params_dim = axis < 0 ? -axis : axis + 1;
     OP_REQUIRES(
@@ -203,7 +206,7 @@ class GatherOp : public OpKernel {
                               .HostMemory("axis"),                     \
                           GatherOp<dev##Device, type, index_type>)
 
-#define REGISTER_GATHER_CPU(type)           \
+#define REGISTER_GATHER_CPU(type)         \
   REGISTER_GATHER_FULL(CPU, type, int16); \
   REGISTER_GATHER_FULL(CPU, type, int32); \
   REGISTER_GATHER_FULL(CPU, type, int64_t)
@@ -213,19 +216,23 @@ TF_CALL_ALL_TYPES(REGISTER_GATHER_CPU);
 TF_CALL_QUANTIZED_TYPES(REGISTER_GATHER_CPU);
 TF_CALL_quint16(REGISTER_GATHER_CPU);
 TF_CALL_qint16(REGISTER_GATHER_CPU);
+TF_CALL_float8_e5m2(REGISTER_GATHER_CPU);
+TF_CALL_float8_e4m3fn(REGISTER_GATHER_CPU);
 
 #undef REGISTER_GATHER_CPU
 
 #if GOOGLE_CUDA || TENSORFLOW_USE_ROCM
 
 // Registration of the GPU implementations.
-#define REGISTER_GATHER_GPU(type)           \
+#define REGISTER_GATHER_GPU(type)         \
   REGISTER_GATHER_FULL(GPU, type, int32); \
   REGISTER_GATHER_FULL(GPU, type, int64_t)
 
 TF_CALL_int32(REGISTER_GATHER_GPU);
 TF_CALL_int64(REGISTER_GATHER_GPU);
 TF_CALL_GPU_ALL_TYPES(REGISTER_GATHER_GPU);
+TF_CALL_float8_e5m2(REGISTER_GATHER_GPU);
+TF_CALL_float8_e4m3fn(REGISTER_GATHER_GPU);
 
 #undef REGISTER_GATHER_GPU
 
