@@ -14,6 +14,11 @@ limitations under the License.
 ==============================================================================*/
 
 // See docs in ../ops/image_ops.cc
+#include <cmath>
+#include <cstdint>
+#include <cstdlib>
+#include <limits>
+#include <vector>
 #define EIGEN_USE_THREADS
 
 #include <math.h>
@@ -21,7 +26,7 @@ limitations under the License.
 #include <algorithm>
 #include <array>
 
-#include "third_party/eigen3/unsupported/Eigen/CXX11/Tensor"
+#include "unsupported/Eigen/CXX11/Tensor"  // from @eigen_archive
 #include "tensorflow/core/framework/op_kernel.h"
 #include "tensorflow/core/framework/register_types.h"
 #include "tensorflow/core/framework/tensor.h"
