@@ -15,8 +15,13 @@ limitations under the License.
 // See docs in ../ops/image_ops.cc.
 #include <math.h>
 
+#include <algorithm>
 #include <cmath>
+#include <cstdint>
+#include <limits>
+#include <vector>
 
+#include "absl/log/check.h"
 #include "tensorflow/core/framework/bounds_check.h"
 #include "tensorflow/core/framework/op_kernel.h"
 #include "tensorflow/core/framework/register_types.h"
