@@ -20,7 +20,7 @@ limitations under the License.
 
 #include <memory>
 
-#include "third_party/eigen3/unsupported/Eigen/CXX11/Tensor"
+#include "unsupported/Eigen/CXX11/Tensor"  // from @eigen_archive
 #include "tensorflow/core/framework/bounds_check.h"
 #include "tensorflow/core/framework/op_kernel.h"
 #include "tensorflow/core/framework/register_types.h"
@@ -32,6 +32,7 @@ limitations under the License.
 #include "tensorflow/core/lib/core/status.h"
 #include "tensorflow/core/lib/strings/stringprintf.h"
 #include "tensorflow/core/platform/logging.h"
+#include "tsl/platform/threadpool.h"
 
 namespace tensorflow {
 using strings::Printf;
@@ -48,10 +49,11 @@ inline const T& Clamp(const T& low, const T& high, const T& value) {
 }
 
 template <typename Kernel>
-Status ComputeSpansCore(OpKernelContext* context, const Kernel& kernel,
-                        const int64_t output_size, const int64_t input_size,
-                        const float scale, const float translate,
-                        const bool antialias, Spans* spans) {
+absl::Status ComputeSpansCore(OpKernelContext* context, const Kernel& kernel,
+                              const int64_t output_size,
+                              const int64_t input_size, const float scale,
+                              const float translate, const bool antialias,
+                              Spans* spans) {
   // When sampling, we need the inverse scale and translation, to map from an
   // output to an input pixel.
   const float inv_scale = 1.0 / scale;
@@ -120,13 +122,13 @@ Status ComputeSpansCore(OpKernelContext* context, const Kernel& kernel,
     }
     starts_vec(x) = span_start;
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status ComputeGradSpansCore(OpKernelContext* context, const Spans& spans,
-                            const int64_t forward_output_size,
-                            const int64_t forward_input_size,
-                            Spans* grad_spans) {
+absl::Status ComputeGradSpansCore(OpKernelContext* context, const Spans& spans,
+                                  const int64_t forward_output_size,
+                                  const int64_t forward_input_size,
+                                  Spans* grad_spans) {
   struct GradComponent {
     int index;
     float weight;
@@ -180,18 +182,18 @@ Status ComputeGradSpansCore(OpKernelContext* context, const Spans& spans,
       grad_starts_vec(input_index) = 0;
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Computes the spans for the passed kernel, for a input dimension of length
 // input_size transformed by scale and translate to an output dimension of
 // length output_size. Note that there's no requirement that;
 // output_size = input_size * scale.
-Status ComputeSpans(OpKernelContext* context,
-                    const functor::SamplingKernelType kernel_type,
-                    const int64_t output_size, const int64_t input_size,
-                    const float scale, const float translate,
-                    const bool antialias, Spans* spans) {
+absl::Status ComputeSpans(OpKernelContext* context,
+                          const functor::SamplingKernelType kernel_type,
+                          const int64_t output_size, const int64_t input_size,
+                          const float scale, const float translate,
+                          const bool antialias, Spans* spans) {
   switch (kernel_type) {
     case functor::Lanczos1Kernel: {
       return ComputeSpansCore(context, CreateLanczos1Kernel(), output_size,
@@ -229,18 +231,18 @@ Status ComputeSpans(OpKernelContext* context,
       return errors::InvalidArgument(Printf("Unrecognized kernel type: %d",
                                             static_cast<int>(kernel_type)));
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Computes the grad spans for the passed kernel.
 // forward_input_size and forward_output_size are the input and output size from
 // the forward operation.
-Status ComputeGradSpans(OpKernelContext* context,
-                        const functor::SamplingKernelType kernel_type,
-                        const int64_t forward_output_size,
-                        const int64_t forward_input_size, const float scale,
-                        const float translate, const bool antialias,
-                        Spans* grad_spans) {
+absl::Status ComputeGradSpans(OpKernelContext* context,
+                              const functor::SamplingKernelType kernel_type,
+                              const int64_t forward_output_size,
+                              const int64_t forward_input_size,
+                              const float scale, const float translate,
+                              const bool antialias, Spans* grad_spans) {
   Spans spans;
   TF_RETURN_IF_ERROR(ComputeSpans(context, kernel_type, forward_output_size,
                                   forward_input_size, scale, translate,
@@ -368,9 +370,9 @@ class ScaleAndTranslateOp : public OpKernel {
         const_col_spans.weights.tensor<float, 1>());
 
     functor::GatherSpans<Device, T>()(
-        context->eigen_device<Device>(), row_spans.span_size, row_starts,
-        row_weights, col_spans.span_size, col_starts, col_weights, image_data,
-        intermediate_data, output_data);
+        context, context->eigen_device<Device>(), row_spans.span_size,
+        row_starts, row_weights, col_spans.span_size, col_starts, col_weights,
+        image_data, intermediate_data, output_data);
   }
   functor::SamplingKernelType kernel_type_;
   bool antialias_;
@@ -473,9 +475,9 @@ class ScaleAndTranslateGradOp : public OpKernel {
         const_col_spans.weights.tensor<float, 1>());
 
     functor::GatherSpans<Device, T>()(
-        context->eigen_device<Device>(), row_spans.span_size, row_starts,
-        row_weights, col_spans.span_size, col_starts, col_weights, input_grad,
-        intermediate_data, output_grad);
+        context, context->eigen_device<Device>(), row_spans.span_size,
+        row_starts, row_weights, col_spans.span_size, col_starts, col_weights,
+        input_grad, intermediate_data, output_grad);
   }
 
   functor::SamplingKernelType kernel_type_;
@@ -483,37 +485,47 @@ class ScaleAndTranslateGradOp : public OpKernel {
 };
 
 template <typename T>
-void GatherColumns(int span_size, const int32* starts, const float* weights,
-                   const T* image, const int64_t input_height,
-                   const int64_t input_width, const int64_t output_height,
-                   const int64_t output_width, const int channels,
-                   float* output) {
+void GatherColumns(OpKernelContext* context, int span_size, const int32* starts,
+                   const float* weights, const T* image,
+                   const int64_t input_height, const int64_t input_width,
+                   const int64_t output_height, const int64_t output_width,
+                   const int channels, float* output) {
   const int64_t in_row_size = input_width * channels;
   const int64_t out_row_size = output_width * channels;
 
-  for (int y = 0; y < output_height; ++y) {
-    const T* input_row_start = image + in_row_size * y;
-    float* out_pix = output + out_row_size * y;
-    for (int x = 0; x < output_width; ++x, out_pix += channels) {
-      const T* in_pix = input_row_start + starts[x] * channels;
-      const float* weights_start = weights + x * span_size;
-      const int real_span_size =
-          std::min(starts[x] + span_size, static_cast<int>(input_width)) -
-          starts[x];
-      const float* weights_end = weights_start + real_span_size;
-      for (int c = 0; c < channels; ++c) {
-        out_pix[c] = 0.0f;
-      }
-      for (const float* weight_ptr = weights_start; weight_ptr != weights_end;
-           ++weight_ptr) {
-        float w = *weight_ptr;
+  auto ParallelGatherColumns = [&](int64_t start, int64_t end) {
+    for (int y = start; y < end; ++y) {
+      const T* input_row_start = image + in_row_size * y;
+      float* out_pix = output + out_row_size * y;
+      for (int x = 0; x < output_width; ++x, out_pix += channels) {
+        const T* in_pix = input_row_start + starts[x] * channels;
+        const float* weights_start = weights + x * span_size;
+        const int real_span_size =
+            std::min(starts[x] + span_size, static_cast<int>(input_width)) -
+            starts[x];
+        const float* weights_end = weights_start + real_span_size;
         for (int c = 0; c < channels; ++c) {
-          out_pix[c] += w * static_cast<float>(in_pix[c]);
+          out_pix[c] = 0.0f;
+        }
+        for (const float* weight_ptr = weights_start; weight_ptr != weights_end;
+             ++weight_ptr) {
+          float w = *weight_ptr;
+          for (int c = 0; c < channels; ++c) {
+            out_pix[c] += w * static_cast<float>(in_pix[c]);
+          }
+          in_pix += channels;
         }
-        in_pix += channels;
       }
     }
-  }
+  };
+  auto worker_threads = *(context->device()->tensorflow_cpu_worker_threads());
+  const int64_t block_size = 1;
+  worker_threads.workers->ParallelFor(
+      output_height,
+      tsl::thread::ThreadPool::SchedulingParams(
+          tsl::thread::ThreadPool::SchedulingStrategy::kFixedBlockSize,
+          std::nullopt, block_size),
+      ParallelGatherColumns);
 }
 
 template <typename T>
@@ -526,29 +538,40 @@ inline void AddScaledVector(const T* in_vec, int vec_len, float weight,
 }
 
 template <typename T>
-void GatherRows(int span_size, const int32* starts, const float* weights,
-                const T* image, const int64_t input_height,
-                const int64_t input_width, const int64_t output_height,
-                const int64_t output_width, const int channels, float* output) {
+void GatherRows(OpKernelContext* context, int span_size, const int32* starts,
+                const float* weights, const T* image,
+                const int64_t input_height, const int64_t input_width,
+                const int64_t output_height, const int64_t output_width,
+                const int channels, float* output) {
   const int64_t in_row_size = input_width * channels;
   const int64_t out_row_size = output_width * channels;
 
-  for (int y = 0; y < output_height; ++y) {
-    float* out_row_data = output + out_row_size * y;
-    std::fill(out_row_data, out_row_data + out_row_size, 0.0f);
-    int in_row = starts[y];
-    const T* in_row_data = image + in_row_size * in_row;
-    const float* weights_start = weights + y * span_size;
-    const int real_span_size =
-        std::min(starts[y] + span_size, static_cast<int>(input_height)) -
-        starts[y];
-    const float* const weights_end = weights_start + real_span_size;
-    for (const float* weight_it = weights_start; weight_it != weights_end;
-         ++weight_it) {
-      AddScaledVector(in_row_data, in_row_size, *weight_it, out_row_data);
-      in_row_data += in_row_size;
+  auto ParallelGatherRows = [&](int64_t start, int64_t end) {
+    for (int y = start; y < end; ++y) {
+      float* out_row_data = output + out_row_size * y;
+      std::fill(out_row_data, out_row_data + out_row_size, 0.0f);
+      int in_row = starts[y];
+      const T* in_row_data = image + in_row_size * in_row;
+      const float* weights_start = weights + y * span_size;
+      const int real_span_size =
+          std::min(starts[y] + span_size, static_cast<int>(input_height)) -
+          starts[y];
+      const float* const weights_end = weights_start + real_span_size;
+      for (const float* weight_it = weights_start; weight_it != weights_end;
+           ++weight_it) {
+        AddScaledVector(in_row_data, in_row_size, *weight_it, out_row_data);
+        in_row_data += in_row_size;
+      }
     }
-  }
+  };
+  auto worker_threads = *(context->device()->tensorflow_cpu_worker_threads());
+  const int64_t block_size = 1;
+  worker_threads.workers->ParallelFor(
+      output_height,
+      tsl::thread::ThreadPool::SchedulingParams(
+          tsl::thread::ThreadPool::SchedulingStrategy::kFixedBlockSize,
+          std::nullopt, block_size),
+      ParallelGatherRows);
 }
 
 }  // namespace
@@ -556,7 +579,8 @@ void GatherRows(int span_size, const int32* starts, const float* weights,
 // Partial specialization of GatherSpans functor for a CPUDevice.
 template <typename T>
 struct GatherSpans<CPUDevice, T> {
-  void operator()(const CPUDevice& d, int row_span_size,
+  void operator()(OpKernelContext* context, const CPUDevice& d,
+                  int row_span_size,
                   typename TTypes<int32, 1>::ConstTensor row_starts,
                   typename TTypes<float, 1>::ConstTensor row_weights,
                   int col_span_size,
@@ -585,12 +609,13 @@ struct GatherSpans<CPUDevice, T> {
     for (int b = 0; b < batch_size; ++b, image_ptr += input_pix_per_batch,
              intermediate_ptr += intermediate_pix_per_batch,
              out_ptr += output_pix_per_batch) {
-      GatherRows(row_span_size, row_starts.data(), row_weights.data(),
+      GatherRows(context, row_span_size, row_starts.data(), row_weights.data(),
                  image_ptr, input_height, input_width, output_height,
                  input_width, channels, intermediate_ptr);
-      GatherColumns(col_span_size, col_starts.data(), col_weights.data(),
-                    intermediate_ptr, output_height, input_width, output_height,
-                    output_width, channels, out_ptr);
+      GatherColumns(context, col_span_size, col_starts.data(),
+                    col_weights.data(), intermediate_ptr, output_height,
+                    input_width, output_height, output_width, channels,
+                    out_ptr);
     }
   }
 };
