@@ -178,7 +178,7 @@ class InitializeTableFromTextFileOp : public OpKernel {
                   .WithAttr("delimiter", delimiter_string));
           *out = ops::UnaryOp("Identity", table,
                               builder->opts().WithControlInput(import_table));
-          return OkStatus();
+          return absl::OkStatus();
         });
   }
 
@@ -189,7 +189,8 @@ class InitializeTableFromTextFileOp : public OpKernel {
   int64_t value_index_;
   int64_t offset_ = 0;
 
-  TF_DISALLOW_COPY_AND_ASSIGN(InitializeTableFromTextFileOp);
+  InitializeTableFromTextFileOp(const InitializeTableFromTextFileOp&) = delete;
+  void operator=(const InitializeTableFromTextFileOp&) = delete;
 };
 
 REGISTER_KERNEL_BUILDER(Name("InitializeTableFromTextFile").Device(DEVICE_CPU),
