@@ -32,15 +32,20 @@ limitations under the License.
 #define EIGEN_USE_GPU
 #endif  // GOOGLE_CUDA
 
+#include <functional>
 #include <string>
+#include <type_traits>
 #include <utility>
 #include <vector>
 
+#include "Eigen/Core"  // from @eigen_archive
+#include "unsupported/Eigen/CXX11/Tensor"  // from @eigen_archive
 #include "tensorflow/core/framework/bounds_check.h"
 #include "tensorflow/core/framework/op_kernel.h"
 #include "tensorflow/core/framework/register_types.h"
 #include "tensorflow/core/framework/tensor.h"
 #include "tensorflow/core/framework/tensor_shape.h"
+#include "tensorflow/core/framework/types.h"
 #include "tensorflow/core/kernels/fill_functor.h"
 #include "tensorflow/core/kernels/fused_eigen_output_kernels.h"
 #include "tensorflow/core/platform/errors.h"
@@ -48,13 +53,12 @@ limitations under the License.
 #include "tensorflow/core/util/tensor_format.h"
 
 #if defined(TENSORFLOW_USE_CUSTOM_CONTRACTION_KERNEL)
-#include "tensorflow/tsl/framework/contraction/eigen_contraction_kernel.h"
+#include "xla/tsl/framework/contraction/eigen_contraction_kernel.h"
 #endif
 
-#if GOOGLE_CUDA
-#include "tensorflow/compiler/xla/stream_executor/gpu/gpu_asm_opts.h"
-#include "tensorflow/compiler/xla/stream_executor/gpu/redzone_allocator.h"
-#include "tensorflow/compiler/xla/stream_executor/tf_allocator_adapter.h"
+#if GOOGLE_CUDA || TENSORFLOW_USE_ROCM
+#include "xla/stream_executor/gpu/redzone_allocator.h"
+#include "xla/stream_executor/integrations/tf_allocator_adapter.h"
 #include "tensorflow/core/kernels/conv_ops_gpu.h"
 #include "tensorflow/core/kernels/gpu_utils.h"
 #include "tensorflow/core/kernels/matmul_op_impl.h"
@@ -67,7 +71,7 @@ limitations under the License.
 #include "tensorflow/core/util/autotune_maps/conv_parameters.h"
 #include "tensorflow/core/util/proto/proto_utils.h"
 #include "tensorflow/core/util/use_cudnn.h"
-#endif  // GOOGLE_CUDA
+#endif  // GOOGLE_CUDA || TENSORFLOW_USE_ROCM
 
 namespace tensorflow {
 
@@ -85,14 +89,16 @@ struct LaunchFusedMatMulOp {
 
 template <typename T>
 struct LaunchFusedMatMulOp<CPUDevice, T> {
+  // Use F32 compute for F16 inputs on CPU to preserve precision and reduce
+  // excessive casting during intermediate computations.
+  using ComputeType =
+      std::conditional_t<DataTypeToEnum<T>::value == DT_HALF, float, T>;
+
   void operator()(
       OpKernelContext* context, const Tensor& a, const Tensor& b,
       const Eigen::array<Eigen::IndexPair<Eigen::DenseIndex>, 1>& dim_pair,
       FusedComputationType fusion, const FusedComputationArgs& fusion_args,
       Tensor* output, bool use_autotune) {
-    OP_REQUIRES(context, DataTypeToEnum<T>::value != DT_HALF,
-                errors::InvalidArgument("_FusedMatMul doesn't support DT_HALF "
-                                        "data type on CPU devices."));
     auto lhs = a.matrix<T>();
     auto rhs = b.matrix<T>();
     auto out = output->matrix<T>();
@@ -104,13 +110,21 @@ struct LaunchFusedMatMulOp<CPUDevice, T> {
     auto executeWithOutputKernel = [&](auto output_kernel) {
       OutputKernelWrapper output_kernel_wrapper(
           [&output_kernel](
-              const ContractionOutputMapper<T, Eigen::Index>& output_mapper,
+              const ContractionOutputMapper<ComputeType, Eigen::Index>&
+                  output_mapper,
               const Eigen::TensorContractionParams& params, Eigen::Index i,
               Eigen::Index j, Eigen::Index num_rows, Eigen::Index num_cols) {
             output_kernel(output_mapper, params, i, j, num_rows, num_cols);
           });
 
-      out.device(d) = lhs.contract(rhs, dim_pair, output_kernel_wrapper);
+      if constexpr (std::is_same_v<ComputeType, T>) {
+        out.device(d) = lhs.contract(rhs, dim_pair, output_kernel_wrapper);
+      } else {
+        out.device(d) = lhs.template cast<ComputeType>()
+                            .contract(rhs.template cast<ComputeType>(),
+                                      dim_pair, output_kernel_wrapper)
+                            .template cast<T>();
+      }
     };
 
     BiasAddArgs<T> bias_add_args;
@@ -133,6 +147,12 @@ struct LaunchFusedMatMulOp<CPUDevice, T> {
       case FusedComputationType::kBiasAddWithRelu6:
         executeWithOutputKernel(WithBiasAddAndRelu6<T>(bias_add_args));
         break;
+      case FusedComputationType::kBiasAddWithTanh:
+        executeWithOutputKernel(WithBiasAddAndTanh<T>(bias_add_args));
+        break;
+      case FusedComputationType::kBiasAddWithSigmoid:
+        executeWithOutputKernel(WithBiasAddAndSigmoid<T>(bias_add_args));
+        break;
       case FusedComputationType::kBiasAddWithElu:
         executeWithOutputKernel(WithBiasAddAndElu<T>(bias_add_args));
         break;
@@ -155,16 +175,16 @@ struct LaunchFusedMatMulOp<CPUDevice, T> {
   // We do not pass std::function directly as an output kernel because it blows
   // up the binary size in debug mode with super long symbol names.
   struct OutputKernelWrapper {
-    using OutputKernelFn =
-        std::function<void(const ContractionOutputMapper<T, Eigen::Index>&,
-                           const Eigen::TensorContractionParams&, Eigen::Index,
-                           Eigen::Index, Eigen::Index, Eigen::Index)>;
+    using OutputKernelFn = std::function<void(
+        const ContractionOutputMapper<ComputeType, Eigen::Index>&,
+        const Eigen::TensorContractionParams&, Eigen::Index, Eigen::Index,
+        Eigen::Index, Eigen::Index)>;
 
     explicit OutputKernelWrapper(OutputKernelFn fn)
         : output_kernel_fn(std::move(fn)) {}
 
     void operator()(
-        const ContractionOutputMapper<T, Eigen::Index>& output_mapper,
+        const ContractionOutputMapper<ComputeType, Eigen::Index>& output_mapper,
         const Eigen::TensorContractionParams& params, Eigen::Index i,
         Eigen::Index j, Eigen::Index num_rows, Eigen::Index num_cols) const {
       output_kernel_fn(output_mapper, params, i, j, num_rows, num_cols);
@@ -174,17 +194,19 @@ struct LaunchFusedMatMulOp<CPUDevice, T> {
   };
 };
 
-#if GOOGLE_CUDA
+#if GOOGLE_CUDA || TENSORFLOW_USE_ROCM
+
 namespace {
 
-StatusOr<se::cuda::BlasLt::Epilogue> GetBlasLtEpilogOp(
+#if GOOGLE_CUDA || TF_HIPBLASLT
+StatusOr<se::gpu::BlasLt::Epilogue> GetBlasLtEpilogOp(
     FusedComputationType fusion) {
   if (fusion == FusedComputationType::kBiasAdd) {
-    return se::cuda::BlasLt::Epilogue::kBias;
+    return se::gpu::BlasLt::Epilogue::kBias;
   } else if (fusion == FusedComputationType::kBiasAddWithRelu) {
-    return se::cuda::BlasLt::Epilogue::kBiasThenReLU;
+    return se::gpu::BlasLt::Epilogue::kBiasThenReLU;
   } else if (fusion == FusedComputationType::kBiasAddWithGeluApproximate) {
-    return se::cuda::BlasLt::Epilogue::kBiasThenGELU;
+    return se::gpu::BlasLt::Epilogue::kBiasThenGELU;
   } else {
     return errors::Internal("Unsupported fusion for BlasLt Matmul");
   }
@@ -192,7 +214,7 @@ StatusOr<se::cuda::BlasLt::Epilogue> GetBlasLtEpilogOp(
 
 template <typename LaunchFunc>
 se::blas::AlgorithmConfig AutotuneMatmul(
-    const std::vector<se::cuda::BlasLt::MatmulAlgorithm>& algorithms,
+    const std::vector<se::gpu::BlasLt::MatmulAlgorithm>& algorithms,
     BlasLtMatmulPlanParams& matmul_params, OpKernelContext* context,
     const LaunchFunc& launch_func) {
   // Note that algorithm_config.algorithm() here is used to refer
@@ -214,7 +236,7 @@ se::blas::AlgorithmConfig AutotuneMatmul(
       BlasScratchAllocator scratch_allocator(context);
 
       Status cublaslt_launch =
-          launch_func(scratch_allocator, profile_algorithm, &profile_result);
+          launch_func(scratch_allocator, i, &profile_result);
 
       VLOG(4) << "  Autotune algorithm " << i
               << " result: " << profile_result.elapsed_time_in_ms()
@@ -240,6 +262,7 @@ se::blas::AlgorithmConfig AutotuneMatmul(
   }
   return algorithm_config;
 }
+#endif
 
 template <typename LaunchFunc, typename Sig>
 StatusOr<std::vector<xla::AutotuneResult>> AutotuneMatMulImpl(
@@ -259,7 +282,7 @@ StatusOr<std::vector<xla::AutotuneResult>> AutotuneMatMulImpl(
     // TODO(zhengxq): profile each algorithm multiple times to better
     // accuracy.
     se::RedzoneAllocator rz_scratch_allocator(
-        stream, &tf_allocator_adapter, se::GpuAsmOpts(),
+        stream, &tf_allocator_adapter,
         /*memory_limit=*/scratch_size_limit);
     BlasScratchAllocator scratch_allocator(ctx, scratch_size_limit);
     se::ScratchAllocator* allocator_used =
@@ -326,22 +349,24 @@ StatusOr<AutotuneEntry<se::dnn::FusedMatmulOp>> AutotuneFusedMatmul(
     int64_t scratch_size_limit) {
   AutotuneEntry<se::dnn::FusedMatmulOp> autotune_entry;
   auto* stream = ctx->op_device_context()->stream();
-
   if (!autotune_map->Find(params, &autotune_entry)) {
     profiler::ScopedAnnotation trace("cudnn_autotuning");
 
     se::TfAllocatorAdapter tf_allocator_adapter(ctx->device()->GetAllocator({}),
                                                 stream);
-    se::RedzoneAllocator rz_allocator(stream, &tf_allocator_adapter,
-                                      se::GpuAsmOpts());
+    se::RedzoneAllocator rz_allocator(stream, &tf_allocator_adapter);
     se::DeviceMemory<T> c_ptr_rz(WrapRedzoneBestEffort(&rz_allocator, c_ptr));
 
     std::vector<std::unique_ptr<const se::dnn::FusedMatmulRunner>> runners;
     auto element_type = se::dnn::ToDataType<T>::value;
-    TF_RETURN_IF_ERROR(stream->parent()->GetFusedMatmulRunners(
-        CudnnUseFrontend(), element_type, element_type, element_type, stream,
-        trans_a, trans_b, m, n, k, lda, ldb, ldc, activation_mode,
-        /*use_fallback=*/false, GetNumericOptions(), &runners));
+    auto dnn = stream->parent()->AsDnn();
+    if (dnn == nullptr) {
+      return errors::Internal("No DNN in stream executor.");
+    }
+    TF_RETURN_IF_ERROR(dnn->GetFusedMatmulRunners(
+        element_type, element_type, element_type, stream, trans_a, trans_b, m,
+        n, k, lda, ldb, ldc, activation_mode,
+        /*use_fallback=*/false, GetNumericOptionsForCuDnn(), &runners));
 
     auto launch_func =
         [&](se::ScratchAllocator* allocator_used,
@@ -382,10 +407,11 @@ StatusOr<AutotuneEntry<se::dnn::FusedMatmulOp>> AutotuneFusedMatmul(
           << params.ToString();
       std::vector<std::unique_ptr<const se::dnn::FusedMatmulRunner>>
           fallback_runners;
-      TF_RETURN_IF_ERROR(stream->parent()->GetFusedMatmulRunners(
-          CudnnUseFrontend(), element_type, element_type, element_type, stream,
-          trans_a, trans_b, m, n, k, lda, ldb, ldc, activation_mode,
-          /*use_fallback=*/true, GetNumericOptions(), &fallback_runners));
+      TF_RETURN_IF_ERROR(dnn->GetFusedMatmulRunners(
+          element_type, element_type, element_type, stream, trans_a, trans_b, m,
+          n, k, lda, ldb, ldc, activation_mode,
+          /*use_fallback=*/true, GetNumericOptionsForCuDnn(),
+          &fallback_runners));
 
       TF_ASSIGN_OR_RETURN(
           auto fallback_results,
@@ -401,7 +427,6 @@ StatusOr<AutotuneEntry<se::dnn::FusedMatmulOp>> AutotuneFusedMatmul(
                           BestCudnnConvAlgorithm<se::dnn::FusedMatmulOp>(
                               fallback_results, std::move(fallback_runners)));
     }
-
     autotune_map->Insert(params, autotune_entry);
   }
   return autotune_entry;
@@ -449,8 +474,8 @@ struct LaunchFusedMatMulOp<GPUDevice, T> {
     const int64_t k = a.dim_size(trans_a ? 0 : 1);
     const int64_t n = b.dim_size(trans_b ? 0 : 1);
 
-    bool use_cudnn = false;
     se::dnn::ActivationMode matmul_activation_mode;
+    bool use_cudnn = false;
     switch (fusion) {
       case FusedComputationType::kBiasAddWithGeluExact:
         matmul_activation_mode = se::dnn::ActivationMode::kGeluExact;
@@ -464,28 +489,48 @@ struct LaunchFusedMatMulOp<GPUDevice, T> {
         matmul_activation_mode = se::dnn::ActivationMode::kSigmoid;
         use_cudnn = true;
         break;
+      case FusedComputationType::kBiasAdd:
+        matmul_activation_mode = se::dnn::ActivationMode::kNone;
+        break;
+      case FusedComputationType::kBiasAddWithRelu:
+        matmul_activation_mode = se::dnn::ActivationMode::kRelu;
+        break;
+      case FusedComputationType::kBiasAddWithRelu6:
+        matmul_activation_mode = se::dnn::ActivationMode::kRelu6;
+        break;
+      case FusedComputationType::kBiasAddWithElu:
+        matmul_activation_mode = se::dnn::ActivationMode::kElu;
+        break;
+      case FusedComputationType::kBiasAddWithLeakyRelu:
+        matmul_activation_mode = se::dnn::ActivationMode::kLeakyRelu;
+        break;
+      case FusedComputationType::kBiasAddWithGeluApproximate:
+        matmul_activation_mode = se::dnn::ActivationMode::kGeluExact;
+        break;
       default:
         use_cudnn = false;
     }
+#if !(GOOGLE_CUDA || TF_HIPBLASLT)
+    use_cudnn = true;
+#endif
 
+    const auto& cc =
+        stream->parent()->GetDeviceDescription().gpu_compute_capability();
+    if (auto* procm = std::get_if<se::RocmComputeCapability>(&cc)) {
+      use_cudnn = !procm->gfx9_mi200_or_later();
+    }
     BlasScratchAllocator scratch_allocator(context);
 
     // The Gelu exact fusion is supported by the cuDNN.
     if (use_cudnn) {
-      MatmulParameters cudnn_matmul_params = {
+      // printf("trans_a %d trans_b %d\n", (int)trans_a, (int)trans_b);
+      MatmulParameters cudnn_matmul_params(
           stream->parent(),
           /*ab_type=*/a.dtype(),
-          /*c_type=*/output->dtype(),
-          trans_a,
-          trans_b,
-          static_cast<uint64_t>(m),
-          static_cast<uint64_t>(n),
-          static_cast<uint64_t>(k),
-          a.dim_size(1),
-          b.dim_size(1),
-          output->dim_size(1),
-          matmul_activation_mode,
-      };
+          /*c_type=*/output->dtype(), trans_a, trans_b,
+          static_cast<uint64_t>(m), static_cast<uint64_t>(n),
+          static_cast<uint64_t>(k), a.dim_size(1), b.dim_size(1),
+          output->dim_size(1), matmul_activation_mode);
 
       auto entry_or = AutotuneFusedMatmul<T>(
           use_autotune, FusedMatmulAutotuneMap::GetInstance(),
@@ -523,10 +568,10 @@ struct LaunchFusedMatMulOp<GPUDevice, T> {
       OP_REQUIRES_OK(context, cudnn_launch_status);
       return;
     }
-
+#if GOOGLE_CUDA || TF_HIPBLASLT
     auto epilog_op_or = GetBlasLtEpilogOp(fusion);
     OP_REQUIRES_OK(context, epilog_op_or.status());
-    se::cuda::BlasLt::Epilogue epilog_op = epilog_op_or.value();
+    se::gpu::BlasLt::Epilogue epilog_op = epilog_op_or.value();
 
     se::blas::Transpose trans[] = {se::blas::Transpose::kNoTranspose,
                                    se::blas::Transpose::kTranspose};
@@ -541,36 +586,38 @@ struct LaunchFusedMatMulOp<GPUDevice, T> {
                                          /*broadcast_a=*/false,
                                          /*broadcast_b=*/false,
                                          epilog_op};
-
-    auto plan_and_algorithms_or = GetPlanAndAlgorithms(stream, matmul_params);
+    absl::Mutex* pmu;
+    auto plan_and_algorithms_or =
+        PlanAndAlgorithms::GetOrCreate(stream, matmul_params, &pmu);
     OP_REQUIRES_OK(context, plan_and_algorithms_or.status());
+    absl::MutexLock lock(pmu);
     const auto* plan_and_algorithms = std::move(plan_and_algorithms_or).value();
-    const auto& plan = plan_and_algorithms->plan;
     const auto& algorithms = plan_and_algorithms->algorithms;
     OP_REQUIRES(context, algorithms.size() > 0,
                 errors::InvalidArgument("No matmul algorithm returned!"));
 
     auto launch_func = [&](BlasScratchAllocator& scratch_allocator,
-                           const se::cuda::BlasLt::MatmulAlgorithm& algorithm,
+                           size_t alg_idx,
                            se::blas::ProfileResult* profile_result) {
-      return DoBlasLtMatmul(stream, plan, a_ptr, b_ptr, c_ptr, algorithm,
-                            scratch_allocator, bias_ptr, profile_result);
+      return plan_and_algorithms->ExecuteOnStream(stream, a_ptr, b_ptr, c_ptr,
+                                                  alg_idx, scratch_allocator,
+                                                  bias_ptr, profile_result);
     };
 
-    se::cuda::BlasLt::MatmulAlgorithm algorithm = algorithms[0];
+    size_t alg_idx = 0;
     if (use_autotune) {
-      se::blas::AlgorithmConfig algorithm_config =
+      auto algorithm_config =
           AutotuneMatmul(algorithms, matmul_params, context, launch_func);
 
-      se::blas::AlgorithmType algorithm_idx = algorithm_config.algorithm();
-      algorithm = algorithms[algorithm_idx];
+      alg_idx = algorithm_config.algorithm();
     }
 
-    OP_REQUIRES_OK(context, launch_func(scratch_allocator, algorithm, nullptr));
+    OP_REQUIRES_OK(context, launch_func(scratch_allocator, alg_idx, nullptr));
+#endif
   }
 };
 
-#endif  // GOOGLE_CUDA
+#endif  // GOOGLE_CUDA || TENSORLFOW_USE_ROCM
 
 template <typename Device, typename T>
 class FusedMatMulOp : public OpKernel {
@@ -587,6 +634,8 @@ class FusedMatMulOp : public OpKernel {
           {FCT::kBiasAdd, {"BiasAdd"}},
           {FCT::kBiasAddWithRelu, {"BiasAdd", "Relu"}},
           {FCT::kBiasAddWithRelu6, {"BiasAdd", "Relu6"}},
+          {FCT::kBiasAddWithTanh, {"BiasAdd", "Tanh"}},
+          {FCT::kBiasAddWithSigmoid, {"BiasAdd", "Sigmoid"}},
           {FCT::kBiasAddWithElu, {"BiasAdd", "Elu"}},
           {FCT::kBiasAddWithLeakyRelu, {"BiasAdd", "LeakyRelu"}},
       };
@@ -676,7 +725,8 @@ class FusedMatMulOp : public OpKernel {
   FusedComputationType fused_computation_ = FusedComputationType::kUndefined;
   FusedComputationArgs fused_computation_args_;
 
-  TF_DISALLOW_COPY_AND_ASSIGN(FusedMatMulOp);
+  FusedMatMulOp(const FusedMatMulOp&) = delete;
+  void operator=(const FusedMatMulOp&) = delete;
 };
 
 // Registration of the CPU implementations.
@@ -686,10 +736,12 @@ class FusedMatMulOp : public OpKernel {
       FusedMatMulOp<CPUDevice, T>);
 
 TF_CALL_float(REGISTER_FUSED_CPU_MATMUL);
+TF_CALL_half(REGISTER_FUSED_CPU_MATMUL);
+TF_CALL_bfloat16(REGISTER_FUSED_CPU_MATMUL);
 
 #undef REGISTER_FUSED_CPU_MATMUL
 
-#if GOOGLE_CUDA
+#if GOOGLE_CUDA || TENSORFLOW_USE_ROCM
 
 // Registration of the GPU implementations.
 #define REGISTER_FUSED_GPU_MATMUL(T)                                  \
@@ -699,6 +751,7 @@ TF_CALL_float(REGISTER_FUSED_CPU_MATMUL);
 
 TF_CALL_float(REGISTER_FUSED_GPU_MATMUL);
 TF_CALL_half(REGISTER_FUSED_GPU_MATMUL);
+TF_CALL_bfloat16(REGISTER_FUSED_GPU_MATMUL);
 
 #undef REGISTER_FUSED_GPU_MATMUL
 
