@@ -648,6 +648,19 @@ T baseline_floor_mod(T lhs, T rhs) {
 
 /// Test the JIT-compiled kernels.
 #if defined(MLIR_GENERATED_GPU_KERNELS_ENABLED)
+TEST_F(BinaryOpsTest, FloorModFloatSpecialCase) {
+  TestEqualShapes<float, double, float, double>(
+      "FloorMod", /*shape=*/{20}, test::InputAsVector<float>({1.34e8f}),
+      test::InputAsVector<float>({0.6f}), baseline_floor_mod,
+      test::OpsTestConfig().ATol(1e-11).RTol(1e-6));
+}
+TEST_F(BinaryOpsTest, FloorModDoubleSpecialCase) {
+  TestEqualShapes<double, double, double, double>(
+      "FloorMod", /*shape=*/{20}, test::InputAsVector<double>({1.34e8}),
+      test::InputAsVector<double>({0.6}), baseline_floor_mod,
+      test::OpsTestConfig().ATol(1e-11).RTol(1e-6));
+}
+
 GENERATE_DEFAULT_TESTS_WITH_SPECIFIC_INPUT_VALUES(
     FloorMod,
     /*test_name=*/Int8, int8_t, int8_t, test::DefaultInput<int8_t>(),
@@ -683,11 +696,13 @@ GENERATE_DEFAULT_TESTS_WITH_SPECIFIC_INPUT_VALUES(
     /*test_name=*/UInt64, uint64_t, uint64_t, test::DefaultInput<uint64_t>(),
     test::DefaultInputNonZero<uint64_t>(), baseline_floor_mod,
     test::OpsTestConfig().ExpectStrictlyEqual());
+#if !TENSORFLOW_USE_ROCM
 GENERATE_DEFAULT_TESTS_WITH_SPECIFIC_INPUT_VALUES(
     FloorMod,
     /*test_name=*/Half, Eigen::half, Eigen::half,
     test::DefaultInput<Eigen::half>(), test::DefaultInputNonZero<Eigen::half>(),
     baseline_floor_mod, test::OpsTestConfig());
+#endif
 GENERATE_DEFAULT_TESTS_WITH_SPECIFIC_INPUT_VALUES(
     FloorMod,
     /*test_name=*/Float, float, float, test::DefaultInput<float>(),
@@ -1000,6 +1015,12 @@ T baseline_mul(T lhs, T rhs) {
   return lhs * rhs;
 }
 
+template <typename T>
+std::complex<T> baseline_cmulf(std::complex<T> lhs, std::complex<T> rhs) {
+  return std::complex<T>(lhs.real() * rhs.real() - lhs.imag() * rhs.imag(),
+                         lhs.real() * rhs.imag() + lhs.imag() * rhs.real());
+}
+
 GENERATE_DEFAULT_TESTS(Mul, /*test_name=*/Half, Eigen::half, Eigen::half,
                        baseline_mul,
                        test::OpsTestConfig().ExpectStrictlyEqual())
@@ -1041,7 +1062,7 @@ TEST_F(BinaryOpsTest, MulComplex64SpecialCases) {
       test::NearZeroInfAndNanInput<std::complex<float>>(),
       test::RepeatElements(test::NearZeroInfAndNanInput<std::complex<float>>(),
                            64),
-      baseline_mul, test::OpsTestConfig());
+      baseline_cmulf, test::OpsTestConfig());
 }
 
 TEST_F(BinaryOpsTest, MulComplex128SpecialCases) {
@@ -1051,7 +1072,7 @@ TEST_F(BinaryOpsTest, MulComplex128SpecialCases) {
       test::NearZeroInfAndNanInput<std::complex<double>>(),
       test::RepeatElements(test::NearZeroInfAndNanInput<std::complex<double>>(),
                            64),
-      baseline_mul, test::OpsTestConfig());
+      baseline_cmulf, test::OpsTestConfig());
 }
 #endif
 
@@ -1488,7 +1509,7 @@ GENERATE_DEFAULT_TESTS(Xlogy, /*test_name=*/Complex64, std::complex<float>,
                        test::OpsTestConfig().ATol(2e-6).RTol(2e-6))
 GENERATE_DEFAULT_TESTS(Xlogy, /*test_name=*/Complex128, std::complex<double>,
                        std::complex<double>, baseline_xlogy,
-                       test::OpsTestConfig())
+                       test::OpsTestConfig().ATol(1e-12).RTol(1e-12))
 
 /// Test `tf.Xlog1py`.
 
