@@ -18,8 +18,8 @@ limitations under the License.
 
 #include <array>
 
-#include "third_party/eigen3/Eigen/Core"
-#include "third_party/eigen3/unsupported/Eigen/CXX11/Tensor"
+#include "Eigen/Core"  // from @eigen_archive
+#include "unsupported/Eigen/CXX11/Tensor"  // from @eigen_archive
 #include "tensorflow/core/framework/kernel_shape_util.h"
 #include "tensorflow/core/framework/numeric_op.h"
 #include "tensorflow/core/framework/op_kernel.h"
@@ -88,7 +88,7 @@ Pool3dParameters::Pool3dParameters(OpKernelContext* context,
                               col_stride, padding, &out_width, &pad_cols));
 }
 
-Status Pool3dParameters::forward_output_shape(TensorShape* shape) {
+absl::Status Pool3dParameters::forward_output_shape(TensorShape* shape) {
   return ShapeFromFormatWithStatus(data_format, tensor_in_batch,
                                    {{out_plane, out_height, out_width}}, depth,
                                    shape);
