@@ -51,8 +51,9 @@ typedef Eigen::GpuDevice GPUDevice;
 
 namespace {
 
-static Status AllocateOutputWithShape(OpKernelContext* ctx, const Tensor& shape,
-                                      int index, Tensor** output) {
+static absl::Status AllocateOutputWithShape(OpKernelContext* ctx,
+                                            const Tensor& shape, int index,
+                                            Tensor** output) {
   TensorShape tensor_shape;
   TF_RETURN_IF_ERROR(tensor::MakeShape(shape, &tensor_shape));
   return ctx->allocate_output(index, tensor_shape, output);
@@ -321,7 +322,8 @@ class RandomGammaOp : public OpKernel {
  private:
   GuardedPhiloxRandom generator_;
 
-  TF_DISALLOW_COPY_AND_ASSIGN(RandomGammaOp);
+  RandomGammaOp(const RandomGammaOp&) = delete;
+  void operator=(const RandomGammaOp&) = delete;
 };
 
 }  // namespace
