@@ -21,6 +21,7 @@ limitations under the License.
 #include <utility>
 #include <vector>
 
+#include "xla/tsl/platform/env.h"
 #include "tensorflow/core/framework/bounds_check.h"
 #include "tensorflow/core/framework/op_kernel.h"
 #include "tensorflow/core/framework/register_types.h"
@@ -87,7 +88,7 @@ void SaveTensors(
   checkpoint::TensorSliceWriter writer(filename_t.flat<tstring>()(0),
                                        std::move(builder_func));
 
-  Status s;
+  absl::Status s;
   auto tensor_names_flat = tensor_names_t.flat<tstring>();
 
   // Process tensors in sorted name order.  This allows us to avoid seeking
@@ -271,7 +272,7 @@ struct RestoreOp {
   RestoreOp(RestoreOp&&) = default;
   RestoreOp& operator=(RestoreOp&&) = default;
 
-  bool should_run_in_pool(BundleReader* reader) const {
+  bool is_large_shape(BundleReader* reader) const {
     TensorShape restored_full_shape;
 
     // Ignore status here; we'll catch the error later.
@@ -283,8 +284,8 @@ struct RestoreOp {
   }
 
   // Run this restore operation using a new BundleReader.
-  void run_with_new_reader() {
-    BundleReader reader(Env::Default(), reader_prefix);
+  void run_with_new_reader(BundleCache* cache) {
+    BundleReader reader(tsl::Env::Default(), reader_prefix, {cache, false});
     if (!reader.status().ok()) {
       status = reader.status();
       return;
@@ -293,7 +294,7 @@ struct RestoreOp {
     status = run(&reader);
   }
 
-  Status run(BundleReader* reader) {
+  absl::Status run(BundleReader* reader) {
     TensorShape restored_full_shape;
     TF_RETURN_IF_ERROR(
         reader->LookupTensorShape(tensor_name, &restored_full_shape));
@@ -346,7 +347,7 @@ struct RestoreOp {
     }
     VLOG(1) << "Done restoring tensor " << idx << " : " << tensor_name << " : "
             << restored_full_shape.num_elements();
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   OpKernelContext* context;
@@ -356,15 +357,15 @@ struct RestoreOp {
   string reader_prefix;
   DataType dtype;
 
-  ::tensorflow::Status status;
+  absl::Status status;
 };
 
 }  // namespace
 
-Status RestoreTensorsV2(OpKernelContext* context, const Tensor& prefix,
-                        const Tensor& tensor_names,
-                        const Tensor& shape_and_slices,
-                        gtl::ArraySlice<DataType> dtypes) {
+absl::Status RestoreTensorsV2(OpKernelContext* context, const Tensor& prefix,
+                              const Tensor& tensor_names,
+                              const Tensor& shape_and_slices,
+                              absl::Span<const DataType> dtypes) {
   const string& prefix_string = prefix.scalar<tstring>()();
 
   const auto& tensor_names_flat = tensor_names.flat<tstring>();
@@ -377,7 +378,9 @@ Status RestoreTensorsV2(OpKernelContext* context, const Tensor& prefix,
                            shape_and_slices_flat(i), prefix_string, dtypes[i]});
   }
 
-  BundleReader default_reader(Env::Default(), prefix_string);
+  tsl::Env* const env = tsl::Env::Default();
+  BundleCache cache(env);
+  BundleReader default_reader(env, prefix_string, {&cache, false});
   TF_RETURN_IF_ERROR(default_reader.status());
 
   TF_RETURN_IF_ERROR(default_reader.SortForSequentialAccess<RestoreOp>(
@@ -402,37 +405,68 @@ Status RestoreTensorsV2(OpKernelContext* context, const Tensor& prefix,
     return errors::InvalidArgument(error_msg);
   }
 
-  std::vector<RestoreOp*> pool_restore_ops;
-  std::vector<RestoreOp*> direct_restore_ops;
+  // Split restore ops into two groups: large and small. We schedule
+  // large ops first, to prevent them from waiting on the small op.
+  std::vector<RestoreOp*> large_restore_ops;
+  std::vector<RestoreOp*> small_restore_ops;
   for (RestoreOp& restore_op : restore_ops) {
-    if (restore_op.should_run_in_pool(&default_reader)) {
-      pool_restore_ops.push_back(&restore_op);
+    if (restore_op.is_large_shape(&default_reader)) {
+      large_restore_ops.push_back(&restore_op);
     } else {
-      direct_restore_ops.push_back(&restore_op);
+      small_restore_ops.push_back(&restore_op);
     }
   }
 
-  {
-    // Schedule any threaded operations first, skipping thread pool creation if
-    // we don't have any expensive operations.
+  if (context->session_config() != nullptr &&
+      context->session_config()->intra_op_parallelism_threads() > 0) {
+    // If an explicit restore parallelism is specified, we use it to run
+    // run both small and large restore ops in parallel.
+    auto reader_pool = std::make_unique<thread::ThreadPool>(
+        tsl::Env::Default(), "restore_tensors",
+        context->session_config()->intra_op_parallelism_threads());
+
+    // Schedule large ops first, followed by the small.
+    for (auto* op : large_restore_ops) {
+      reader_pool->Schedule(
+          [op, &cache]() { op->run_with_new_reader(&cache); });
+    }
+    for (auto* op : small_restore_ops) {
+      reader_pool->Schedule(
+          [op, &cache]() { op->run_with_new_reader(&cache); });
+    }
+
+    // Wait for all scheduled work to finish and check the status of all
+    // ops that ran in the pool.
+    reader_pool.reset();
+    for (auto& op : restore_ops) {
+      TF_RETURN_IF_ERROR(op.status);
+    }
+  } else {
+    // If no restore parallelism is specified, we run large restore ops with
+    // a modest parallelism, and small restore ops serially.
+
+    // Avoid creating a pool if there are no large restore ops.
     std::unique_ptr<thread::ThreadPool> reader_pool;
-    if (!pool_restore_ops.empty()) {
+    if (!large_restore_ops.empty()) {
       reader_pool.reset(
           new thread::ThreadPool(Env::Default(), "restore_tensors", 8));
-      for (auto* op : pool_restore_ops) {
-        reader_pool->Schedule([op]() { op->run_with_new_reader(); });
+      for (auto* op : large_restore_ops) {
+        reader_pool->Schedule(
+            [op, &cache]() { op->run_with_new_reader(&cache); });
       }
     }
 
-    // Read small tensors from the op thread
-    for (auto* op : direct_restore_ops) {
+    // Read small tensors from the op thread.
+    for (auto* op : small_restore_ops) {
       TF_RETURN_IF_ERROR(op->run(&default_reader));
     }
-  }
 
-  // Check status of pool ops; this must come after the pool shuts down.
-  for (auto* op : pool_restore_ops) {
-    TF_RETURN_IF_ERROR(op->status);
+    // Wait for all scheduled work to finish and check the status of all
+    // ops that ran in the pool.
+    reader_pool.reset();
+    for (auto* op : large_restore_ops) {
+      TF_RETURN_IF_ERROR(op->status);
+    }
   }
 
   for (const RestoreOp& restore_op : restore_ops) {
@@ -444,7 +478,7 @@ Status RestoreTensorsV2(OpKernelContext* context, const Tensor& prefix,
     }
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace tensorflow
