@@ -34,7 +34,6 @@ limitations under the License.
 #include "tensorflow/core/framework/variant_encode_decode.h"
 #include "tensorflow/core/kernels/reshape_util.h"
 #include "tensorflow/core/lib/gtl/inlined_vector.h"
-#include "tensorflow/core/lib/gtl/optional.h"
 #include "tensorflow/core/util/sparse/group_iterator.h"
 #include "tensorflow/core/util/sparse/sparse_tensor.h"
 
@@ -52,8 +51,8 @@ class SerializeSparseOp : public OpKernel {
 
   bool IsExpensive() override;
 
-  Status Initialize(Tensor* result);
-  Status Serialize(const Tensor& input, T* result);
+  absl::Status Initialize(Tensor* result);
+  absl::Status Serialize(const Tensor& input, T* result);
 
   void Compute(OpKernelContext* context) override {
     const Tensor* input_indices;
@@ -106,18 +105,18 @@ bool SerializeSparseOp<Variant>::IsExpensive() {
 }
 
 template <>
-Status SerializeSparseOp<tstring>::Initialize(Tensor* result) {
+absl::Status SerializeSparseOp<tstring>::Initialize(Tensor* result) {
   *result = Tensor(DT_STRING, TensorShape({3}));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 template <>
-Status SerializeSparseOp<tstring>::Serialize(const Tensor& input,
-                                             tstring* result) {
+absl::Status SerializeSparseOp<tstring>::Serialize(const Tensor& input,
+                                                   tstring* result) {
   TensorProto proto;
   input.AsProtoTensorContent(&proto);
   *result = proto.SerializeAsString();
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 REGISTER_KERNEL_BUILDER(Name("SerializeSparse")
@@ -126,16 +125,16 @@ REGISTER_KERNEL_BUILDER(Name("SerializeSparse")
                         SerializeSparseOp<tstring>);
 
 template <>
-Status SerializeSparseOp<Variant>::Initialize(Tensor* result) {
+absl::Status SerializeSparseOp<Variant>::Initialize(Tensor* result) {
   *result = Tensor(DT_VARIANT, TensorShape({3}));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 template <>
-Status SerializeSparseOp<Variant>::Serialize(const Tensor& input,
-                                             Variant* result) {
+absl::Status SerializeSparseOp<Variant>::Serialize(const Tensor& input,
+                                                   Variant* result) {
   *result = input;
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 REGISTER_KERNEL_BUILDER(Name("SerializeSparse")
@@ -148,9 +147,9 @@ struct SerializeGroups {};
 
 template <typename T>
 struct SerializeGroups<T, tstring> {
-  Status operator()(sparse::GroupIterable* minibatch,
-                    const Tensor& output_shape, int64_t N, int rank,
-                    Tensor* serialized_sparse) {
+  absl::Status operator()(sparse::GroupIterable* minibatch,
+                          const Tensor& output_shape, int64_t N, int rank,
+                          Tensor* serialized_sparse) {
     auto serialized_sparse_t = serialized_sparse->matrix<tstring>();
 
     int64_t last_nonempty_group = -1;
@@ -214,7 +213,7 @@ struct SerializeGroups<T, tstring> {
       serialize_empty_element(empty_b);
     }
 
-    return OkStatus();
+    return absl::OkStatus();
   }
 };
 
@@ -252,9 +251,9 @@ void CopyValues<Eigen::half>(const Eigen::half* src, Eigen::half* dest,
 
 template <typename T>
 struct SerializeGroups<T, Variant> {
-  Status operator()(sparse::GroupIterable* minibatch,
-                    const Tensor& output_shape, int64_t N, int rank,
-                    Tensor* serialized_sparse) {
+  absl::Status operator()(sparse::GroupIterable* minibatch,
+                          const Tensor& output_shape, int64_t N, int rank,
+                          Tensor* serialized_sparse) {
     auto serialized_sparse_t = serialized_sparse->template matrix<Variant>();
 
     int64_t last_nonempty_group = -1;
@@ -329,7 +328,7 @@ struct SerializeGroups<T, Variant> {
       serialize_empty_element(empty_b);
     }
 
-    return OkStatus();
+    return absl::OkStatus();
   }
 };
 
@@ -372,7 +371,7 @@ class SerializeManySparseOp : public OpKernel {
     OP_REQUIRES_OK(context,
                    TensorShape::BuildTensorShape(input_shape->vec<int64_t>(),
                                                  &tensor_input_shape));
-    gtl::InlinedVector<int64_t, 8> std_order(rank);
+    absl::InlinedVector<int64_t, 8> std_order(rank);
     std::iota(std_order.begin(), std_order.end(), 0);
     SparseTensor input_st;
     OP_REQUIRES_OK(context, SparseTensor::Create(*input_indices, *input_values,
@@ -410,7 +409,6 @@ class SerializeManySparseOp : public OpKernel {
 TF_CALL_ALL_TYPES(REGISTER_KERNELS);
 #undef REGISTER_KERNELS
 
-
 #define REGISTER_KERNELS(type)                                      \
   REGISTER_KERNEL_BUILDER(Name("SerializeManySparse")               \
                               .Device(DEVICE_CPU)                   \
