@@ -19,7 +19,7 @@ limitations under the License.
 #define EIGEN_USE_GPU
 #endif
 
-#include "third_party/eigen3/unsupported/Eigen/CXX11/Tensor"
+#include "unsupported/Eigen/CXX11/Tensor"  // from @eigen_archive
 #include "tensorflow/core/framework/op.h"
 #include "tensorflow/core/framework/op_kernel.h"
 #include "tensorflow/core/framework/tensor_shape.h"
@@ -49,8 +49,8 @@ class CSRSparseMatrixAddFunctor {
                                      const T beta)
       : ctx_(ctx), alpha_(alpha), beta_(beta) {}
 
-  Status operator()(const CSRSparseMatrix& a, const CSRSparseMatrix& b,
-                    CSRSparseMatrix* c) {
+  absl::Status operator()(const CSRSparseMatrix& a, const CSRSparseMatrix& b,
+                          CSRSparseMatrix* c) {
     TensorShape a_tensor_shape;
     TensorShape b_tensor_shape;
     TF_RETURN_IF_ERROR(TensorShapeUtils::MakeShape(
@@ -175,7 +175,7 @@ class CSRSparseMatrixAddFunctor {
       TF_RETURN_IF_ERROR(csr_geam.Compute(a_comp, b_comp, &c_comp, workspace));
     }
 
-    return OkStatus();
+    return absl::OkStatus();
   }
 
  private:
@@ -381,7 +381,8 @@ struct CSRSparseMatrixAdd<GPUDevice, T>
   const T beta_;
   bool initialized_;
 
-  TF_DISALLOW_COPY_AND_ASSIGN(CSRSparseMatrixAdd);
+  CSRSparseMatrixAdd(const CSRSparseMatrixAdd&) = delete;
+  void operator=(const CSRSparseMatrixAdd&) = delete;
 };
 
 }  // namespace functor
