@@ -148,8 +148,9 @@ namespace {
 
 // Ensures indices, values, shape are all of the proper ranks and are
 // compatible.
-Status ValidateSparseTensorShape(const Tensor& indices, const Tensor& values,
-                                 const Tensor& shape) {
+absl::Status ValidateSparseTensorShape(const Tensor& indices,
+                                       const Tensor& values,
+                                       const Tensor& shape) {
   // Indices must be a matrix, and values/shape must be a vector.
   if (!TensorShapeUtils::IsMatrix(indices.shape())) {
     return errors::InvalidArgument("Sparse indices must be rank 2 but is rank ",
@@ -176,7 +177,7 @@ Status ValidateSparseTensorShape(const Tensor& indices, const Tensor& values,
                                    shape.NumElements(), ") do not match");
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Creates a debug string for the index tuple in indices(row, :).
@@ -196,8 +197,8 @@ string CreateIndexString(const IndexTensor& indices, int64_t row) {
 
 // Ensures all sparse indices are within correct bounds.
 template <typename Tindices>
-Status ValidateSparseTensorIndicesUnordered(const Tensor& indices,
-                                            const Tensor& shape) {
+absl::Status ValidateSparseTensorIndicesUnordered(const Tensor& indices,
+                                                  const Tensor& shape) {
   // Ensure no index is out-of-bounds.
   const auto indices_mat = indices.flat_inner_dims<Tindices>();
   const auto shape_vec = shape.flat<Tindices>();
@@ -215,21 +216,21 @@ Status ValidateSparseTensorIndicesUnordered(const Tensor& indices,
     }
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Ensures all sparse indices are within correct bounds and are
 // lexicographically ordered.
 template <typename Tindices>
-Status ValidateSparseTensorIndicesOrdered(const Tensor& indices,
-                                          const Tensor& shape) {
+absl::Status ValidateSparseTensorIndicesOrdered(const Tensor& indices,
+                                                const Tensor& shape) {
   const auto indices_mat = indices.flat_inner_dims<Tindices>();
   const auto shape_vec = shape.flat<Tindices>();
   int64_t nnz = indices.dim_size(0);
   int64_t ndims = indices.dim_size(1);
 
   if (nnz == 0) {
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   // First set of indices must be within range.
@@ -282,15 +283,15 @@ Status ValidateSparseTensorIndicesOrdered(const Tensor& indices,
     }
   }  // for i in [1, nnz)
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace
 
 template <typename Tindices>
-Status ValidateSparseTensor(const Tensor& indices, const Tensor& values,
-                            const Tensor& shape,
-                            IndexValidation index_validation) {
+absl::Status ValidateSparseTensor(const Tensor& indices, const Tensor& values,
+                                  const Tensor& shape,
+                                  IndexValidation index_validation) {
   TF_RETURN_IF_ERROR(ValidateSparseTensorShape(indices, values, shape));
   switch (index_validation) {
     case IndexValidation::kOrdered:
@@ -300,7 +301,7 @@ Status ValidateSparseTensor(const Tensor& indices, const Tensor& values,
     case IndexValidation::kNone: {
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 #define REGISTER_SPARSE_UTIL_FUNCTIONS(TypeIndex)                           \
