@@ -17,7 +17,7 @@ limitations under the License.
 
 #define EIGEN_USE_THREADS
 
-#include "third_party/eigen3/unsupported/Eigen/CXX11/Tensor"
+#include "unsupported/Eigen/CXX11/Tensor"  // from @eigen_archive
 #include "tensorflow/core/framework/op_kernel.h"
 #include "tensorflow/core/kernels/cwise_ops.h"
 #include "tensorflow/core/kernels/cwise_ops_common.h"
@@ -56,8 +56,8 @@ struct UnaryOpsCompositionBase {
  private:
   friend class UnaryOpsComposition<T>;
 
-  Status ExportComputeFns(const std::vector<string>& op_names,
-                          std::vector<ComputeFn>* fns, int* cost) {
+  absl::Status ExportComputeFns(const std::vector<string>& op_names,
+                                std::vector<ComputeFn>* fns, int* cost) {
     for (const string& op_name : op_names) {
       auto it = compute_fns.find(op_name);
       if (it == compute_fns.end())
@@ -69,7 +69,7 @@ struct UnaryOpsCompositionBase {
       *cost += reg.cost;
     }
 
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   std::unordered_map<string, ComputeFnRegistration> compute_fns;
