@@ -18,6 +18,10 @@ limitations under the License.
 #include <assert.h>
 #include <stddef.h>
 
+#include <cstdint>
+
+#include "absl/log/check.h"
+#include "xla/tsl/lib/core/bits.h"
 #include "tensorflow/core/platform/logging.h"
 #include "tensorflow/core/platform/stringpiece.h"
 
@@ -160,7 +164,7 @@ const char* OrderedCode::TEST_SkipToNextSpecialByte(const char* start,
 
 // Helper routine to encode "s" and append to "*dest", escaping special
 // characters.
-inline static void EncodeStringFragment(string* dest, StringPiece s) {
+inline static void EncodeStringFragment(string* dest, absl::string_view s) {
   const char* p = s.data();
   const char* limit = p + s.size();
   const char* copy_start = p;
@@ -187,7 +191,7 @@ inline static void EncodeStringFragment(string* dest, StringPiece s) {
   }
 }
 
-void OrderedCode::WriteString(string* dest, StringPiece s) {
+void OrderedCode::WriteString(string* dest, absl::string_view s) {
   EncodeStringFragment(dest, s);
   AppendBytes(dest, kEscape1_Separator, 2);
 }
@@ -212,7 +216,7 @@ void OrderedCode::WriteNumIncreasing(string* dest, uint64 val) {
 // If parse succeeds, return true, consume encoding from
 // "*src", and if result != NULL append the decoded string to "*result".
 // Otherwise, return false and leave both undefined.
-inline static bool ReadStringInternal(StringPiece* src, string* result) {
+inline static bool ReadStringInternal(absl::string_view* src, string* result) {
   const char* start = src->data();
   const char* string_limit = src->data() + src->size();
 
@@ -267,11 +271,11 @@ inline static bool ReadStringInternal(StringPiece* src, string* result) {
   return false;
 }
 
-bool OrderedCode::ReadString(StringPiece* src, string* result) {
+bool OrderedCode::ReadString(absl::string_view* src, string* result) {
   return ReadStringInternal(src, result);
 }
 
-bool OrderedCode::ReadNumIncreasing(StringPiece* src, uint64* result) {
+bool OrderedCode::ReadNumIncreasing(absl::string_view* src, uint64* result) {
   if (src->empty()) {
     return false;  // Not enough bytes
   }
@@ -403,49 +407,15 @@ static const uint64 kLengthToMask[1 + kMaxSigned64Length] = {
 // For positive numbers, the number of bits is 1 plus the most significant
 // bit position (the highest bit position in a positive int64 is 63).
 // For a negative number n, we count the bits in ~n.
-// That is, length = kBitsToLength[Bits::Log2Floor64(n < 0 ? ~n : n) + 1].
+// That is, length = kBitsToLength[tsl::Log2Floor64(n < 0 ? ~n : n) + 1].
 static const int8 kBitsToLength[1 + 63] = {
     1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 4,
     4, 4, 4, 4, 4, 4, 5, 5, 5, 5, 5, 5, 5, 6, 6, 6, 6, 6, 6, 6, 7, 7,
     7, 7, 7, 7, 7, 8, 8, 8, 8, 8, 8, 8, 9, 9, 9, 9, 9, 9, 9, 10};
 
-#if defined(__GNUC__)
-// Returns floor(lg(n)).  Returns -1 if n == 0.
-static int Log2Floor64(uint64 n) {
-  return n == 0 ? -1 : 63 ^ __builtin_clzll(n);
-}
-#else
-// Portable slow version
-static int Log2Floor32_Portable(uint32 n) {
-  if (n == 0) return -1;
-  int log = 0;
-  uint32 value = n;
-  for (int i = 4; i >= 0; --i) {
-    int shift = (1 << i);
-    uint32 x = value >> shift;
-    if (x != 0) {
-      value = x;
-      log += shift;
-    }
-  }
-  assert(value == 1);
-  return log;
-}
-// Returns floor(lg(n)).  Returns -1 if n == 0.
-static int Log2Floor64(uint64 n) {
-  const uint32 topbits = static_cast<uint32>(n >> 32);
-  if (topbits == 0) {
-    // Top bits are zero, so scan in bottom bits
-    return Log2Floor32_Portable(static_cast<uint32>(n));
-  } else {
-    return 32 + Log2Floor32_Portable(topbits);
-  }
-}
-#endif
-
 // Calculates the encoding length in bytes of the signed number n.
 static inline int SignedEncodingLength(int64_t n) {
-  return kBitsToLength[Log2Floor64(n < 0 ? ~n : n) + 1];
+  return kBitsToLength[tsl::Log2Floor64(n < 0 ? ~n : n) + 1];
 }
 
 static void StoreBigEndian64(char* dst, uint64 v) {
@@ -485,7 +455,8 @@ void OrderedCode::WriteSignedNumIncreasing(string* dest, int64_t val) {
   dest->append(begin, len);
 }
 
-bool OrderedCode::ReadSignedNumIncreasing(StringPiece* src, int64_t* result) {
+bool OrderedCode::ReadSignedNumIncreasing(absl::string_view* src,
+                                          int64_t* result) {
   if (src->empty()) return false;
   const uint64 xor_mask = (!((*src)[0] & 0x80)) ? ~0ULL : 0ULL;
   const unsigned char first_byte = (*src)[0] ^ (xor_mask & 0xff);
@@ -494,7 +465,7 @@ bool OrderedCode::ReadSignedNumIncreasing(StringPiece* src, int64_t* result) {
   int len;
   uint64 x;
   if (first_byte != 0xff) {
-    len = 7 - Log2Floor64(first_byte ^ 0xff);
+    len = 7 - tsl::Log2Floor64(first_byte ^ 0xff);
     if (src->size() < static_cast<size_t>(len)) return false;
     x = xor_mask;  // sign extend using xor_mask
     for (int i = 0; i < len; ++i)
