@@ -29,8 +29,11 @@ REGISTER_OP_NO_GRADIENT("OnesLike");
 REGISTER_OP_NO_GRADIENT("Const");
 REGISTER_OP_NO_GRADIENT("EditDistance");
 REGISTER_OP_NO_GRADIENT("StopGradient");
+REGISTER_OP_NO_GRADIENT("FakeQuantWithMinMaxArgsGradient");
+REGISTER_OP_NO_GRADIENT("FakeQuantWithMinMaxVarsGradient");
+REGISTER_OP_NO_GRADIENT("FakeQuantWithMinMaxVarsPerChannelGradient");
 
-Status ReshapeGrad(const AttrSlice& attrs, FunctionDef* g) {
+absl::Status ReshapeGrad(const AttrSlice& attrs, FunctionDef* g) {
   // clang-format off
   *g = FDH::Define(
       // Arg defs
@@ -46,12 +49,12 @@ Status ReshapeGrad(const AttrSlice& attrs, FunctionDef* g) {
         {{"dshape"}, "ZerosLike", {"shape"}, {{"T", DT_INT32}}},
       });
   // clang-format on
-  return OkStatus();
+  return absl::OkStatus();
 }
 REGISTER_OP_GRADIENT("Reshape", ReshapeGrad);
 REGISTER_OP_GRADIENT("ExpandDims", ReshapeGrad);
 
-Status SqueezeGrad(const AttrSlice& attrs, FunctionDef* g) {
+absl::Status SqueezeGrad(const AttrSlice& attrs, FunctionDef* g) {
   // clang-format off
   *g = FDH::Define(
       // Arg defs
@@ -66,11 +69,11 @@ Status SqueezeGrad(const AttrSlice& attrs, FunctionDef* g) {
         {{"dx"}, "Reshape", {"dy", "x_shape"}, {{"T", "$T"}}},
       });
   // clang-format on
-  return OkStatus();
+  return absl::OkStatus();
 }
 REGISTER_OP_GRADIENT("Squeeze", SqueezeGrad);
 
-Status IdentityGrad(const AttrSlice& attrs, FunctionDef* g) {
+absl::Status IdentityGrad(const AttrSlice& attrs, FunctionDef* g) {
   // clang-format off
   *g = FDH::Define(
       // Arg defs
@@ -85,11 +88,11 @@ Status IdentityGrad(const AttrSlice& attrs, FunctionDef* g) {
       });
   // clang-format on
   VLOG(1) << "IdentityGrad " << DebugString(*g);
-  return OkStatus();
+  return absl::OkStatus();
 }
 REGISTER_OP_GRADIENT("Identity", IdentityGrad);
 
-Status PackGrad(const AttrSlice& attrs, FunctionDef* g) {
+absl::Status PackGrad(const AttrSlice& attrs, FunctionDef* g) {
   // clang-format off
   *g = FDH::Create(
       "_",
@@ -111,11 +114,11 @@ Status PackGrad(const AttrSlice& attrs, FunctionDef* g) {
       {{"dx", "dx:output"}});
   // clang-format on
   VLOG(1) << "PackGrad " << DebugString(*g);
-  return OkStatus();
+  return absl::OkStatus();
 }
 REGISTER_OP_GRADIENT("Pack", PackGrad);
 
-Status UnpackGrad(const AttrSlice& attrs, FunctionDef* g) {
+absl::Status UnpackGrad(const AttrSlice& attrs, FunctionDef* g) {
   // clang-format off
   *g = FDH::Define(
       // Arg defs
@@ -135,12 +138,12 @@ Status UnpackGrad(const AttrSlice& attrs, FunctionDef* g) {
       });
   // clang-format on
   VLOG(1) << "UnpackGrad " << DebugString(*g);
-  return OkStatus();
+  return absl::OkStatus();
 }
 REGISTER_OP_GRADIENT("Unpack", UnpackGrad);
 
-Status ConcatGradHelper(const AttrSlice& attrs, FunctionDef* g,
-                        bool dim_is_last_arg) {
+absl::Status ConcatGradHelper(const AttrSlice& attrs, FunctionDef* g,
+                              bool dim_is_last_arg) {
   int N;
   TF_RETURN_IF_ERROR(GetNodeAttr(attrs, "N", &N));
   DataType T;
@@ -154,7 +157,6 @@ Status ConcatGradHelper(const AttrSlice& attrs, FunctionDef* g,
     offset_i.push_back(strings::StrCat("offset:offset:", i));
     dx_i.push_back(strings::StrCat("dx_", i, ":output:0"));
   }
-  DataTypeVector dtype_list(N, T);
 
   // ConcatGrad(dim, x, dy):
   //   for i in range(N):
@@ -210,21 +212,21 @@ Status ConcatGradHelper(const AttrSlice& attrs, FunctionDef* g,
     // clang-format on
   }
   VLOG(1) << "ConcatGrad " << DebugString(*g);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status ConcatGrad(const AttrSlice& attrs, FunctionDef* g) {
+absl::Status ConcatGrad(const AttrSlice& attrs, FunctionDef* g) {
   return ConcatGradHelper(attrs, g, false);
 }
 
-Status ConcatGradV2(const AttrSlice& attrs, FunctionDef* g) {
+absl::Status ConcatGradV2(const AttrSlice& attrs, FunctionDef* g) {
   return ConcatGradHelper(attrs, g, true);
 }
 
 REGISTER_OP_GRADIENT("Concat", ConcatGrad);
 REGISTER_OP_GRADIENT("ConcatV2", ConcatGradV2);
 
-Status SplitGrad(const AttrSlice& attrs, FunctionDef* g) {
+absl::Status SplitGrad(const AttrSlice& attrs, FunctionDef* g) {
   // clang-format off
   *g = FDH::Define(
       // Arg defs
@@ -240,11 +242,11 @@ Status SplitGrad(const AttrSlice& attrs, FunctionDef* g) {
       });
   // clang-format on
   VLOG(1) << "SplitGrad " << DebugString(*g);
-  return OkStatus();
+  return absl::OkStatus();
 }
 REGISTER_OP_GRADIENT("Split", SplitGrad);
 
-Status SplitVGrad(const AttrSlice& attrs, FunctionDef* g) {
+absl::Status SplitVGrad(const AttrSlice& attrs, FunctionDef* g) {
   // clang-format off
   *g = FDH::Define(
       // Arg defs
@@ -261,11 +263,11 @@ Status SplitVGrad(const AttrSlice& attrs, FunctionDef* g) {
       });
   // clang-format on
   VLOG(1) << "SplitVGrad " << DebugString(*g);
-  return OkStatus();
+  return absl::OkStatus();
 }
 REGISTER_OP_GRADIENT("SplitV", SplitVGrad);
 
-Status ArrayToListGrad(const AttrSlice& attrs, FunctionDef* g) {
+absl::Status ArrayToListGrad(const AttrSlice& attrs, FunctionDef* g) {
   int N;
   TF_RETURN_IF_ERROR(GetNodeAttr(attrs, "N", &N));
   std::vector<string> dys;
@@ -288,11 +290,11 @@ Status ArrayToListGrad(const AttrSlice& attrs, FunctionDef* g) {
       });
   // clang-format on
   VLOG(1) << "ArrayToListGrad " << DebugString(*g);
-  return OkStatus();
+  return absl::OkStatus();
 }
 REGISTER_OP_GRADIENT("_ArrayToList", ArrayToListGrad);
 
-Status ListToArrayGrad(const AttrSlice& attrs, FunctionDef* g) {
+absl::Status ListToArrayGrad(const AttrSlice& attrs, FunctionDef* g) {
   // clang-format off
   *g = FDH::Define(
       // Arg defs
@@ -308,11 +310,11 @@ Status ListToArrayGrad(const AttrSlice& attrs, FunctionDef* g) {
       });
   // clang-format on
   VLOG(1) << "ListToArrayGrad " << DebugString(*g);
-  return OkStatus();
+  return absl::OkStatus();
 }
 REGISTER_OP_GRADIENT("_ListToArray", ListToArrayGrad);
 
-Status FillGrad(const AttrSlice& attrs, FunctionDef* g) {
+absl::Status FillGrad(const AttrSlice& attrs, FunctionDef* g) {
   *g = FDH::Define(
       // Arg defs
       {"dims: int32", "x: T", "dy: T"},
@@ -331,11 +333,11 @@ Status FillGrad(const AttrSlice& attrs, FunctionDef* g) {
           {{"dx"}, "Sum", {"dy", "r"}, {{"T", "$T"}}},
       });
   VLOG(1) << "FillGrad " << DebugString(*g);
-  return OkStatus();
+  return absl::OkStatus();
 }
 REGISTER_OP_GRADIENT("Fill", FillGrad);
 
-Status TransposeGrad(const AttrSlice& attrs, FunctionDef* g) {
+absl::Status TransposeGrad(const AttrSlice& attrs, FunctionDef* g) {
   *g = FDH::Define(
       // Arg defs
       {"x: T", "p: int32", "dy: T"},
@@ -350,11 +352,11 @@ Status TransposeGrad(const AttrSlice& attrs, FunctionDef* g) {
           {{"dp"}, "ZerosLike", {"p"}, {{"T", DT_INT32}}},
       });
   VLOG(1) << "TransposeGrad " << DebugString(*g);
-  return OkStatus();
+  return absl::OkStatus();
 }
 REGISTER_OP_GRADIENT("Transpose", TransposeGrad);
 
-Status GatherNdGrad(const AttrSlice& attrs, FunctionDef* g) {
+absl::Status GatherNdGrad(const AttrSlice& attrs, FunctionDef* g) {
   // clang-format off
   *g = FDH::Define(
       // Arg defs
@@ -371,11 +373,11 @@ Status GatherNdGrad(const AttrSlice& attrs, FunctionDef* g) {
         {{"dindices"}, "ZerosLike", {"indices"}, {{"T", "$Tindices"}}},
       });
   // clang-format on
-  return OkStatus();
+  return absl::OkStatus();
 }
 REGISTER_OP_GRADIENT("GatherNd", GatherNdGrad);
 
-Status ConjugateTransposeGrad(const AttrSlice& attrs, FunctionDef* g) {
+absl::Status ConjugateTransposeGrad(const AttrSlice& attrs, FunctionDef* g) {
   *g = FDH::Define(
       // Arg defs
       {"x: T", "p: int32", "dy: T"},
@@ -390,11 +392,11 @@ Status ConjugateTransposeGrad(const AttrSlice& attrs, FunctionDef* g) {
           {{"dp"}, "ZerosLike", {"p"}, {{"T", DT_INT32}}},
       });
   VLOG(1) << "ConjugateTransposeGrad " << DebugString(*g);
-  return OkStatus();
+  return absl::OkStatus();
 }
 REGISTER_OP_GRADIENT("ConjugateTranspose", ConjugateTransposeGrad);
 
-Status ReverseGrad(const AttrSlice& attrs, FunctionDef* g) {
+absl::Status ReverseGrad(const AttrSlice& attrs, FunctionDef* g) {
   *g = FDH::Define(
       // Arg defs
       {"x: T", "d: bool", "dy: T"},
@@ -408,11 +410,11 @@ Status ReverseGrad(const AttrSlice& attrs, FunctionDef* g) {
           {{"dd"}, "ZerosLike", {"d"}, {{"T", DT_BOOL}}},
       });
   VLOG(1) << "ReverseGrad " << DebugString(*g);
-  return OkStatus();
+  return absl::OkStatus();
 }
 REGISTER_OP_GRADIENT("Reverse", ReverseGrad);
 
-Status ReverseV2Grad(const AttrSlice& attrs, FunctionDef* g) {
+absl::Status ReverseV2Grad(const AttrSlice& attrs, FunctionDef* g) {
   DataType itype;
   TF_RETURN_IF_ERROR(GetNodeAttr(attrs, "Tidx", &itype));
   if (itype != DT_INT32) {
@@ -432,11 +434,11 @@ Status ReverseV2Grad(const AttrSlice& attrs, FunctionDef* g) {
           {{"dd"}, "ZerosLike", {"d"}, {{"T", "$Tidx"}}},
       });
   VLOG(1) << "ReverseGrad " << DebugString(*g);
-  return OkStatus();
+  return absl::OkStatus();
 }
 REGISTER_OP_GRADIENT("ReverseV2", ReverseV2Grad);
 
-Status SliceGrad(const AttrSlice& attrs, FunctionDef* g) {
+absl::Status SliceGrad(const AttrSlice& attrs, FunctionDef* g) {
   DataType itype;
   TF_RETURN_IF_ERROR(GetNodeAttr(attrs, "Index", &itype));
   if (itype != DT_INT32) {
@@ -467,11 +469,11 @@ Status SliceGrad(const AttrSlice& attrs, FunctionDef* g) {
        {{"begin_grad"}, "ZerosLike", {"begin"}, {{"T", DT_INT32}}},
        {{"size_grad"}, "ZerosLike", {"size"}, {{"T", DT_INT32}}}});
   VLOG(1) << "SliceGrad " << DebugString(*g);
-  return OkStatus();
+  return absl::OkStatus();
 }
 REGISTER_OP_GRADIENT("Slice", SliceGrad);
 
-Status StridedSliceGrad(const AttrSlice& attrs, FunctionDef* g) {
+absl::Status StridedSliceGrad(const AttrSlice& attrs, FunctionDef* g) {
   DataType itype;
   TF_RETURN_IF_ERROR(GetNodeAttr(attrs, "Index", &itype));
   if (itype != DT_INT32) {
@@ -504,11 +506,11 @@ Status StridedSliceGrad(const AttrSlice& attrs, FunctionDef* g) {
         {{"stride_grad"}, "ZerosLike", {"stride"}, {{"T", DT_INT32}}}}});
 
   VLOG(1) << "StridedSliceGrad " << DebugString(*g);
-  return OkStatus();
+  return absl::OkStatus();
 }
 REGISTER_OP_GRADIENT("StridedSlice", StridedSliceGrad);
 
-Status StridedSliceGradGrad(const AttrSlice& attrs, FunctionDef* g) {
+absl::Status StridedSliceGradGrad(const AttrSlice& attrs, FunctionDef* g) {
   DataType itype;
   TF_RETURN_IF_ERROR(GetNodeAttr(attrs, "Index", &itype));
   if (itype != DT_INT32) {
@@ -546,11 +548,11 @@ Status StridedSliceGradGrad(const AttrSlice& attrs, FunctionDef* g) {
           {"shrink_axis_mask", "$shrink_axis_mask"}}}}});
 
   VLOG(1) << "StridedSliceGrad " << DebugString(*g);
-  return OkStatus();
+  return absl::OkStatus();
 }
 REGISTER_OP_GRADIENT("StridedSliceGrad", StridedSliceGradGrad);
 
-Status BroadcastToGrad(const AttrSlice& attrs, FunctionDef* g) {
+absl::Status BroadcastToGrad(const AttrSlice& attrs, FunctionDef* g) {
   DataType itype;
   TF_RETURN_IF_ERROR(GetNodeAttr(attrs, "Tidx", &itype));
   if (itype != DT_INT32) {
@@ -572,7 +574,7 @@ Status BroadcastToGrad(const AttrSlice& attrs, FunctionDef* g) {
       {{"T: type"}, {"Tidx: {int32, int64}"}},
       // Nodes
       nodes);
-  return OkStatus();
+  return absl::OkStatus();
 }
 REGISTER_OP_GRADIENT("BroadcastTo", BroadcastToGrad);
 
