@@ -15,9 +15,11 @@ limitations under the License.
 
 #include <algorithm>
 
+#include "xla/tsl/platform/statusor.h"
 #include "tensorflow/core/framework/common_shape_fns.h"
 #include "tensorflow/core/framework/op.h"
 #include "tensorflow/core/framework/shape_inference.h"
+#include "tensorflow/core/platform/errors.h"
 
 namespace tensorflow {
 
@@ -29,8 +31,10 @@ namespace {
 
 // Sets output[0] to shape [batch_dim,height,width,channel_dim], where
 // height and width come from the size_tensor.
-Status SetOutputToSizedImage(InferenceContext* c, DimensionHandle batch_dim,
-                             int size_input_idx, DimensionHandle channel_dim) {
+absl::Status SetOutputToSizedImage(InferenceContext* c,
+                                   DimensionHandle batch_dim,
+                                   int size_input_idx,
+                                   DimensionHandle channel_dim) {
   // Verify shape of size input.
   ShapeHandle size;
   TF_RETURN_IF_ERROR(c->WithRank(c->input(size_input_idx), 1, &size));
@@ -58,89 +62,83 @@ Status SetOutputToSizedImage(InferenceContext* c, DimensionHandle batch_dim,
     width = c->MakeDim(vec(1));
   }
   c->set_output(0, c->MakeShape({batch_dim, height, width, channel_dim}));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status ResizeShapeFn(InferenceContext* c) {
+absl::Status ResizeShapeFn(InferenceContext* c) {
   ShapeHandle input;
   TF_RETURN_IF_ERROR(c->WithRank(c->input(0), 4, &input));
   return SetOutputToSizedImage(c, c->Dim(input, 0), 1 /* size_input_idx */,
                                c->Dim(input, 3));
 }
 
-Status DecodeImageShapeFn(InferenceContext* c) {
-  ShapeHandle unused;
-  TF_RETURN_IF_ERROR(c->WithRank(c->input(0), 0, &unused));
-  DimensionHandle channels_dim;
+absl::StatusOr<DimensionHandle> GetChannelsDim(InferenceContext* c) {
   int32_t channels;
   TF_RETURN_IF_ERROR(c->GetAttr("channels", &channels));
   if (channels == 0) {
-    channels_dim = c->UnknownDim();
-  } else {
-    if (channels < 0) {
-      return errors::InvalidArgument("channels must be non-negative, got ",
-                                     channels);
-    }
-    channels_dim = c->MakeDim(channels);
+    return c->UnknownDim();
   }
 
+  if (channels < 0) {
+    return errors::InvalidArgument("channels must be non-negative, got ",
+                                   channels);
+  }
+
+  return c->MakeDim(channels);
+}
+
+absl::Status DecodeImageShapeFn(InferenceContext* c) {
+  ShapeHandle unused;
+  TF_RETURN_IF_ERROR(c->WithRank(c->input(0), 0, &unused));
+  TF_ASSIGN_OR_RETURN(DimensionHandle channels_dim, GetChannelsDim(c));
+
   c->set_output(0, c->MakeShape({InferenceContext::kUnknownDim,
                                  InferenceContext::kUnknownDim, channels_dim}));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status DecodeImageV2ShapeFn(InferenceContext* c) {
+absl::Status DecodeImageV2ShapeFn(InferenceContext* c) {
   ShapeHandle unused;
-  int32_t channels;
   bool expand_animations;
-  DimensionHandle channels_dim;
 
   TF_RETURN_IF_ERROR(c->WithRank(c->input(0), 0, &unused));
-  TF_RETURN_IF_ERROR(c->GetAttr("channels", &channels));
+  TF_ASSIGN_OR_RETURN(DimensionHandle channels_dim, GetChannelsDim(c));
   TF_RETURN_IF_ERROR(c->GetAttr("expand_animations", &expand_animations));
 
-  if (channels == 0) {
-    channels_dim = c->UnknownDim();
-  } else {
-    if (channels < 0) {
-      return errors::InvalidArgument("channels must be non-negative, got ",
-                                     channels);
-    }
-    channels_dim = c->MakeDim(channels);
-  }
-
-  // `expand_animations` set to true will return 4-D shapes for GIF. 3-D shapes
-  // will be returned for jpg, png, and bmp. `expand_animations` set to false
-  // will always return 3-D shapes for all (jpg, png, bmp, gif).
+  // `expand_animations` set to true will return 4-D shapes for GIF and
+  // WebP. 3-D shapes will be returned for jpg, png, and
+  // bmp. `expand_animations` set to false will always return 3-D shapes for all
+  // (jpg, png, bmp, gif, webp). So we *may* have a mix of 3D and 4D
+  // shapes. Just return unknown.
   if (expand_animations) {
     c->set_output(0, c->UnknownShape());
-    return OkStatus();
-  } else {
-    c->set_output(0,
-                  c->MakeShape({InferenceContext::kUnknownDim,
-                                InferenceContext::kUnknownDim, channels_dim}));
-    return OkStatus();
+    return absl::OkStatus();
   }
+
+  // expand_animations is False. We'll have a 3D tensor.
+  c->set_output(0, c->MakeShape({InferenceContext::kUnknownDim,
+                                 InferenceContext::kUnknownDim, channels_dim}));
+  return absl::OkStatus();
 }
 
-Status EncodeImageShapeFn(InferenceContext* c) {
+absl::Status EncodeImageShapeFn(InferenceContext* c) {
   ShapeHandle unused;
   TF_RETURN_IF_ERROR(c->WithRank(c->input(0), 3, &unused));
   c->set_output(0, c->Scalar());
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Allow encoding batches of images.
-Status BatchedEncodeImageShapeFn(InferenceContext* c) {
+absl::Status BatchedEncodeImageShapeFn(InferenceContext* c) {
   ShapeHandle input;
   TF_RETURN_IF_ERROR(c->WithRankAtLeast(c->input(0), 3, &input));
   ShapeHandle s;
   TF_RETURN_IF_ERROR(c->Subshape(input, 0, -3, &s));
   c->set_output(0, s);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status ColorspaceShapeFn(InferenceContext* c) {
+absl::Status ColorspaceShapeFn(InferenceContext* c) {
   ShapeHandle input;
   TF_RETURN_IF_ERROR(c->WithRankAtLeast(c->input(0), 1, &input));
 
@@ -151,10 +149,10 @@ Status ColorspaceShapeFn(InferenceContext* c) {
   TF_RETURN_IF_ERROR(c->ReplaceDim(input, -1, last_dim, &out));
   c->set_output(0, out);
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status NMSShapeFn(InferenceContext* c) {
+absl::Status NMSShapeFn(InferenceContext* c) {
   // Get inputs and validate ranks.
   ShapeHandle boxes;
   TF_RETURN_IF_ERROR(c->WithRank(c->input(0), 2, &boxes));
@@ -174,10 +172,10 @@ Status NMSShapeFn(InferenceContext* c) {
   TF_RETURN_IF_ERROR(c->WithValue(c->Dim(boxes, 1), 4, &unused));
 
   c->set_output(0, c->Vector(c->UnknownDim()));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status SoftNMSShapeFn(InferenceContext* c) {
+absl::Status SoftNMSShapeFn(InferenceContext* c) {
   // Get inputs and validate ranks.
   ShapeHandle boxes;
   TF_RETURN_IF_ERROR(c->WithRank(c->input(0), 2, &boxes));
@@ -200,10 +198,10 @@ Status SoftNMSShapeFn(InferenceContext* c) {
 
   c->set_output(0, c->Vector(c->UnknownDim()));
   c->set_output(1, c->Vector(c->UnknownDim()));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status CombinedNMSShapeFn(InferenceContext* c) {
+absl::Status CombinedNMSShapeFn(InferenceContext* c) {
   // Get inputs and validate ranks
   ShapeHandle boxes;
   // boxes is a tensor of Dimensions [batch_size, num_anchors, q, 4]
@@ -267,7 +265,7 @@ Status CombinedNMSShapeFn(InferenceContext* c) {
   c->set_output(1, c->MakeShape({batch_dim, output_size}));
   c->set_output(2, c->MakeShape({batch_dim, output_size}));
   c->set_output(3, c->Vector(batch_dim));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace
@@ -305,7 +303,7 @@ REGISTER_OP("ResizeBicubicGrad")
     .Attr("half_pixel_centers: bool = false")
     .SetShapeFn([](InferenceContext* c) {
       c->set_output(0, c->input(1));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 // --------------------------------------------------------------------------
@@ -354,7 +352,7 @@ REGISTER_OP("QuantizedResizeBilinear")
       TF_RETURN_IF_ERROR(c->WithRank(c->input(3), 0, &max_shape));
       c->set_output(1, c->MakeShape({}));
       c->set_output(2, c->MakeShape({}));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 // --------------------------------------------------------------------------
@@ -367,7 +365,7 @@ REGISTER_OP("ResizeBilinearGrad")
     .Attr("half_pixel_centers: bool = false")
     .SetShapeFn([](InferenceContext* c) {
       c->set_output(0, c->input(1));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 // --------------------------------------------------------------------------
@@ -382,7 +380,7 @@ REGISTER_OP("ScaleAndTranslateGrad")
     .Attr("antialias: bool = true")
     .SetShapeFn([](InferenceContext* c) {
       c->set_output(0, c->input(1));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 // --------------------------------------------------------------------------
@@ -424,7 +422,7 @@ REGISTER_OP("ResizeNearestNeighborGrad")
             c->ReplaceDim(input, 2, c->MakeDim(size_vec(1)), &input));
       }
       c->set_output(0, input);
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 // --------------------------------------------------------------------------
@@ -457,7 +455,7 @@ REGISTER_OP("RandomCrop")
         w = c->MakeDim(size_vec(1));
       }
       c->set_output(0, c->MakeShape({h, w, channels}));
-      return OkStatus();
+      return absl::OkStatus();
     });
 // TODO(shlens): Support variable rank in RandomCrop.
 
@@ -523,7 +521,7 @@ REGISTER_OP("DecodeAndCropJpeg")
         w = c->MakeDim(crop_window_vec(3));
       }
       c->set_output(0, c->MakeShape({h, w, channels_dim}));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 // --------------------------------------------------------------------------
@@ -557,7 +555,7 @@ REGISTER_OP("ExtractJpegShape")
       ShapeHandle unused;
       TF_RETURN_IF_ERROR(c->WithRank(c->input(0), 0, &unused));
       c->set_output(0, c->Vector(3));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 // --------------------------------------------------------------------------
@@ -638,13 +636,34 @@ REGISTER_OP("DecodeBmp")
 REGISTER_OP("DecodeGif")
     .Input("contents: string")
     .Output("image: uint8")
+    // Always a 4D tensor, and no Alpha support, so channels=3.
     .SetShapeFn([](InferenceContext* c) {
       ShapeHandle unused;
       TF_RETURN_IF_ERROR(c->WithRank(c->input(0), 0, &unused));
       c->set_output(0, c->MakeShape({InferenceContext::kUnknownDim,
                                      InferenceContext::kUnknownDim,
                                      InferenceContext::kUnknownDim, 3}));
-      return OkStatus();
+      return absl::OkStatus();
+    });
+
+// --------------------------------------------------------------------------
+REGISTER_OP("DecodeWebP")
+    .Input("contents: string")
+    .Attr("channels: int = 0")
+    // Add this dtype arg for now, even if we don't yet support conversion.
+    .Attr("dtype: {uint8} = DT_UINT8")
+    .Output("image: dtype")
+    .SetShapeFn([](InferenceContext* c) {
+      ShapeHandle unused;
+      TF_RETURN_IF_ERROR(c->WithRank(c->input(0), 0, &unused));
+      TF_ASSIGN_OR_RETURN(DimensionHandle channels_dim, GetChannelsDim(c));
+
+      // Always a 4D tensor, but channels is dynamic.
+      c->set_output(
+          0, c->MakeShape({InferenceContext::kUnknownDim,
+                           InferenceContext::kUnknownDim,
+                           InferenceContext::kUnknownDim, channels_dim}));
+      return absl::OkStatus();
     });
 
 // --------------------------------------------------------------------------
@@ -735,7 +754,7 @@ REGISTER_OP("SampleDistortedBoundingBox")
       c->set_output(0, c->Vector(3));
       c->set_output(1, c->Vector(3));
       c->set_output(2, c->MakeShape({1, 1, 4}));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("SampleDistortedBoundingBoxV2")
@@ -770,7 +789,7 @@ REGISTER_OP("SampleDistortedBoundingBoxV2")
       c->set_output(0, c->Vector(3));
       c->set_output(1, c->Vector(3));
       c->set_output(2, c->MakeShape({1, 1, 4}));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("StatelessSampleDistortedBoundingBox")
@@ -808,7 +827,7 @@ REGISTER_OP("StatelessSampleDistortedBoundingBox")
       c->set_output(1, c->Vector(3));
       c->set_output(2, c->MakeShape({1, 1, 4}));
 
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 // --------------------------------------------------------------------------
@@ -935,7 +954,7 @@ REGISTER_OP("CropAndResizeGradImage")
       TF_RETURN_IF_ERROR(c->MakeShapeFromShapeTensor(3, &out));
       TF_RETURN_IF_ERROR(c->WithRank(out, 4, &out));
       c->set_output(0, out);
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("CropAndResizeGradBoxes")
@@ -948,7 +967,7 @@ REGISTER_OP("CropAndResizeGradBoxes")
     .Attr("method: {'bilinear'} = 'bilinear'")
     .SetShapeFn([](InferenceContext* c) {
       c->set_output(0, c->input(2));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 // --------------------------------------------------------------------------
@@ -976,7 +995,7 @@ REGISTER_OP("NonMaxSuppression")
       TF_RETURN_IF_ERROR(c->WithValue(c->Dim(boxes, 1), 4, &unused));
 
       c->set_output(0, c->Vector(c->UnknownDim()));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("NonMaxSuppressionV2")
@@ -1006,7 +1025,7 @@ REGISTER_OP("NonMaxSuppressionV2")
       TF_RETURN_IF_ERROR(c->WithValue(c->Dim(boxes, 1), 4, &unused));
 
       c->set_output(0, c->Vector(c->UnknownDim()));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("NonMaxSuppressionV3")
@@ -1043,7 +1062,7 @@ REGISTER_OP("NonMaxSuppressionV4")
         c->set_output(0, c->MakeShape({output_dim}));
       }
       c->set_output(1, c->MakeShape({}));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("NonMaxSuppressionV5")
@@ -1072,7 +1091,7 @@ REGISTER_OP("NonMaxSuppressionV5")
       }
 
       c->set_output(2, c->MakeShape({}));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("NonMaxSuppressionWithOverlaps")
@@ -1104,7 +1123,7 @@ REGISTER_OP("NonMaxSuppressionWithOverlaps")
           c->Merge(c->Dim(overlaps, 0), c->Dim(overlaps, 1), &unused));
 
       c->set_output(0, c->Vector(c->UnknownDim()));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("CombinedNonMaxSuppression")
@@ -1133,7 +1152,7 @@ REGISTER_OP("GenerateBoundingBoxProposals")
     .Output("rois: float")
     .Output("roi_probabilities: float")
     .Attr("post_nms_topn: int = 300")
-    .SetShapeFn([](InferenceContext* c) -> Status {
+    .SetShapeFn([](InferenceContext* c) -> absl::Status {
       // make sure input tensors have are correct rank
       ShapeHandle scores, images, bounding_boxes, anchors, nms_threshold,
           n_pre_nms, min_box_size;
@@ -1158,7 +1177,7 @@ REGISTER_OP("GenerateBoundingBoxProposals")
           {c->Dim(scores, 0), post_nms_top_n});  // (N,post_nms_top_n)
       c->set_output(0, roi_shape);
       c->set_output(1, prob_shape);
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 // V3 op supports fill_value.
@@ -1168,7 +1187,7 @@ REGISTER_OP("ImageProjectiveTransformV2")
     .Input("images: dtype")
     .Input("transforms: float32")
     .Input("output_shape: int32")
-    .Attr("dtype: {uint8, int32, int64, float16, float32, float64}")
+    .Attr("dtype: {uint8, int32, int64, float16, bfloat16, float32, float64}")
     .Attr("interpolation: string")
     .Attr("fill_mode: string = 'CONSTANT'")
     .Output("transformed_images: dtype")
@@ -1184,7 +1203,7 @@ REGISTER_OP("ImageProjectiveTransformV3")
     .Input("transforms: float32")
     .Input("output_shape: int32")
     .Input("fill_value: float32")
-    .Attr("dtype: {uint8, int32, int64, float16, float32, float64}")
+    .Attr("dtype: {uint8, int32, int64, float16, bfloat16, float32, float64}")
     .Attr("interpolation: string")
     .Attr("fill_mode: string = 'CONSTANT'")
     .Output("transformed_images: dtype")
