@@ -19,6 +19,10 @@ limitations under the License.
 #include "tensorflow/core/framework/numeric_op.h"
 #include "tensorflow/core/framework/op.h"
 #include "tensorflow/core/framework/shape_inference.h"
+#include "tensorflow/core/platform/types.h"
+
+// TODO(intel-tf): Move all MKL ops in this file to a separate file,
+// mkl_math_ops.cc.
 
 namespace tensorflow {
 
@@ -47,7 +51,7 @@ REGISTER_OP("AddN")
 
       if (dtype != DT_VARIANT) {
         // Exit early if not DT_VARIANT.
-        return OkStatus();
+        return absl::OkStatus();
       } else {
         // DT_VARIANT shape handle shape inference.  All sizes and dtypes must
         // be the same; all shapes must be compatible via Merge.
@@ -94,7 +98,7 @@ REGISTER_OP("AddN")
         if (shapes_and_types) {
           c->set_output_handle_shapes_and_types(0, cur_shapes_and_types);
         }
-        return OkStatus();
+        return absl::OkStatus();
       }
     });
 
@@ -126,6 +130,8 @@ REGISTER_OP("BatchMatMul")
         "complex128}")
     .Attr("adj_x: bool = false")
     .Attr("adj_y: bool = false")
+    .Attr("grad_x: bool = false")
+    .Attr("grad_y: bool = false")
     .SetShapeFn(shape_inference::BatchMatMulShape);
 
 REGISTER_OP("BatchMatMulV2")
@@ -137,6 +143,8 @@ REGISTER_OP("BatchMatMulV2")
         "uint16, uint32, uint64, complex64, complex128}")
     .Attr("adj_x: bool = false")
     .Attr("adj_y: bool = false")
+    .Attr("grad_x: bool = false")
+    .Attr("grad_y: bool = false")
     .SetShapeFn(shape_inference::BatchMatMulV2Shape);
 
 REGISTER_OP("BatchMatMulV3")
@@ -154,6 +162,8 @@ REGISTER_OP("BatchMatMulV3")
         "complex128}")
     .Attr("adj_x: bool = false")
     .Attr("adj_y: bool = false")
+    .Attr("grad_x: bool = false")
+    .Attr("grad_y: bool = false")
     .SetShapeFn(shape_inference::BatchMatMulV2Shape);
 
 #ifdef INTEL_MKL
@@ -161,18 +171,22 @@ REGISTER_OP("_MklBatchMatMul")
     .Input("x: T")
     .Input("y: T")
     .Output("output: T")
-    .Attr("T: {bfloat16, float}")
+    .Attr("T: {bfloat16, float, half}")
     .Attr("adj_x: bool = false")
     .Attr("adj_y: bool = false")
+    .Attr("grad_x: bool = false")
+    .Attr("grad_y: bool = false")
     .SetShapeFn(shape_inference::BatchMatMulShape);
 
 REGISTER_OP("_MklBatchMatMulV2")
     .Input("x: T")
     .Input("y: T")
     .Output("output: T")
-    .Attr("T: {bfloat16, float}")
+    .Attr("T: {bfloat16, float, half}")
     .Attr("adj_x: bool = false")
     .Attr("adj_y: bool = false")
+    .Attr("grad_x: bool = false")
+    .Attr("grad_y: bool = false")
     .SetShapeFn(shape_inference::BatchMatMulV2Shape);
 #endif  // INTEL_MKL
 
@@ -419,45 +433,6 @@ REGISTER_OP("AddV2")
     .SetIsAggregate()
     .SetIsCommutative();
 
-#ifdef INTEL_MKL
-REGISTER_OP("_MklAdd")
-    .Input("x: T")
-    .Input("y: T")
-    .Input("mkl_x: uint8")
-    .Input("mkl_y: uint8")
-    .Output("z: T")
-    .Output("mkl_z: uint8")
-    .Attr(
-        "T: {half, float, double, uint8, int8, int16, int32, int64, complex64, "
-        "complex128, string, bfloat16}")
-    .SetShapeFn(shape_inference::BroadcastBinaryOpShapeFn)
-    .Doc(R"doc(
-Returns `x` + `y` element-wise.
-
-*NOTE*: `tf.math.add` supports broadcasting. `tf.math.add_n` does not. More about broadcasting
-[here](http://docs.scipy.org/doc/numpy/user/basics.broadcasting.html).
-)doc");
-
-REGISTER_OP("_MklAddV2")
-    .Input("x: T")
-    .Input("y: T")
-    .Input("mkl_x: uint8")
-    .Input("mkl_y: uint8")
-    .Output("z: T")
-    .Output("mkl_z: uint8")
-    .Attr(
-        "T: {bfloat16, half, float, double, uint8, int8, int16, int32, int64, "
-        "complex64, complex128}")
-    .SetShapeFn(shape_inference::BroadcastBinaryOpShapeFn)
-    .SetIsAggregate()
-    .SetIsCommutative()
-    .Doc(R"doc(
-Returns `x` + `y` element-wise.
-*NOTE*: `tf.math.add` supports broadcasting. `tf.math.add_n` does not. More about broadcasting
-[here](http://docs.scipy.org/doc/numpy/user/basics.broadcasting.html).
-)doc");
-#endif  // INTEL_MKL
-
 REGISTER_OP("Sub")
     .Input("x: T")
     .Input("y: T")
@@ -467,19 +442,6 @@ REGISTER_OP("Sub")
         "int64, complex64, complex128, uint32, uint64}")
     .SetShapeFn(shape_inference::BroadcastBinaryOpShapeFn);
 
-REGISTER_OP("_MklSub")
-    .BINARY_FEWER()
-    .Input("mkl_x: uint8")
-    .Input("mkl_y: uint8")
-    .Output("mkl_z: uint8")
-    .SetShapeFn(shape_inference::BroadcastBinaryOpShapeFn)
-    .Doc(R"doc(
-Returns x - y element-wise.
-
-*NOTE*: `Sub` supports broadcasting. More about broadcasting
-[here](http://docs.scipy.org/doc/numpy/user/basics.broadcasting.html)
-)doc");
-
 REGISTER_OP("Mul").BINARY_MORE().SetIsCommutative().SetShapeFn(
     shape_inference::BroadcastBinaryOpShapeFn);
 
@@ -490,20 +452,6 @@ REGISTER_OP("MulNoNan")
     .Attr("T: {bfloat16, half, float, double, complex64, complex128}")
     .SetShapeFn(shape_inference::BroadcastBinaryOpShapeFn);
 
-// Note: This op is not commutative w.r.t. to all its inputs.
-REGISTER_OP("_MklMul")
-    .BINARY_MORE()
-    .Input("mkl_x: uint8")
-    .Input("mkl_y: uint8")
-    .Output("mkl_z: uint8")
-    .SetShapeFn(shape_inference::BroadcastBinaryOpShapeFn)
-    .Doc(R"doc(
-Returns x * y element-wise.
-
-*NOTE*: `Mul` supports broadcasting. More about broadcasting
-[here](http://docs.scipy.org/doc/numpy/user/basics.broadcasting.html)
-)doc");
-
 REGISTER_OP("Div").BINARY_MORE().SetShapeFn(
     shape_inference::BroadcastBinaryOpShapeFn);
 
@@ -531,20 +479,6 @@ REGISTER_OP("SquaredDifference")
     .SetIsCommutative()
     .SetShapeFn(shape_inference::BroadcastBinaryOpShapeFn);
 
-// Note: This op is not commutative w.r.t. to all its inputs.
-REGISTER_OP("_MklSquaredDifference")
-    .BINARY_FEWER()
-    .Input("mkl_x: uint8")
-    .Input("mkl_y: uint8")
-    .Output("mkl_z: uint8")
-    .SetShapeFn(shape_inference::BroadcastBinaryOpShapeFn)
-    .Doc(R"doc(
-Returns (x - y)(x - y) element-wise.
-
-*NOTE*: `SquaredDifference` supports broadcasting. More about broadcasting
-[here](http://docs.scipy.org/doc/numpy/user/basics.broadcasting.html)
-)doc");
-
 REGISTER_OP("Xlogy")
     .Input("x: T")
     .Input("y: T")
@@ -578,22 +512,6 @@ REGISTER_OP("Maximum")
         "int32, uint32, int64, uint64}")
     .SetShapeFn(shape_inference::BroadcastBinaryOpShapeFn);
 
-REGISTER_OP("_MklMaximum")
-    .Input("x: T")
-    .Input("y: T")
-    .Input("mkl_x: uint8")
-    .Input("mkl_y: uint8")
-    .Output("z: T")
-    .Output("mkl_z: uint8")
-    .Attr("T: {half, float, double, int32, int64, bfloat16}")
-    .SetShapeFn(shape_inference::BroadcastBinaryOpShapeFn)
-    .Doc(R"doc(
-Returns the max of x and y (i.e. x > y ? x : y) element-wise.
-
-*NOTE*: `Maximum` supports broadcasting. More about broadcasting
-[here](http://docs.scipy.org/doc/numpy/user/basics.broadcasting.html)
-)doc");
-
 REGISTER_OP("Minimum")
     .Input("x: T")
     .Input("y: T")
@@ -713,7 +631,7 @@ REGISTER_OP("Betainc")
       }
 
       c->set_output(0, output);
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 // --------------------------------------------------------------------------
@@ -848,7 +766,7 @@ REGISTER_OP("Select")
 
       if (!c->RankKnown(cond) || !c->RankKnown(data)) {
         c->set_output(0, data);
-        return OkStatus();
+        return absl::OkStatus();
       }
 
       // rank of shape and data is known.
@@ -860,7 +778,7 @@ REGISTER_OP("Select")
         // The rank of 'cond' is a scalar.
         // t and e can have any shape.
         c->set_output(0, data);
-        return OkStatus();
+        return absl::OkStatus();
       }
 
       if (cond_rank != 1) {
@@ -868,14 +786,14 @@ REGISTER_OP("Select")
         // then shape must match 'then' and 'else'
         TF_RETURN_IF_ERROR(c->Merge(data, cond, &data));
         c->set_output(0, data);
-        return OkStatus();
+        return absl::OkStatus();
       }
 
       if (data_rank == 0) {
         // if 'then' and 'else' are scalar also the cond must be
         TF_RETURN_IF_ERROR(c->Merge(data, cond, &data));
         c->set_output(0, data);
-        return OkStatus();
+        return absl::OkStatus();
       }
 
       if (cond_rank == 1) {
@@ -883,12 +801,12 @@ REGISTER_OP("Select")
         // the first dimension of 'then' and 'else'
         TF_RETURN_IF_ERROR(c->Merge(cond, c->Vector(c->Dim(data, 0)), &cond));
         c->set_output(0, data);
-        return OkStatus();
+        return absl::OkStatus();
       }
 
       c->set_output(0, data);
 
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("SelectV2")
@@ -939,7 +857,7 @@ REGISTER_OP("SelectV2")
       TF_RETURN_IF_ERROR(
           BroadcastBinaryOpOutputShapeFnHelper(c, cond, other, true, &output));
       c->set_output(0, output);
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 // --------------------------------------------------------------------------
@@ -953,6 +871,8 @@ REGISTER_OP("MatMul")
     .Attr(
         "T: {bfloat16, half, float, double, int32, int64, uint8, "
         "uint16, uint32, uint64, complex64, complex128}")
+    .Attr("grad_a: bool = false")
+    .Attr("grad_b: bool = false")
     .SetShapeFn(shape_inference::MatMulShape);
 
 #ifdef INTEL_MKL
@@ -962,7 +882,9 @@ REGISTER_OP("_MklMatMul")
     .Output("product: T")
     .Attr("transpose_a: bool = false")
     .Attr("transpose_b: bool = false")
-    .Attr("T: {bfloat16, float}")
+    .Attr("T: {bfloat16, float, half}")
+    .Attr("grad_a: bool = false")
+    .Attr("grad_b: bool = false")
     .SetShapeFn(shape_inference::MatMulShape);
 #endif  // INTEL_MKL
 
@@ -1076,7 +998,7 @@ REGISTER_OP("Max")
 
 namespace {
 
-Status ArgOpShape(shape_inference::InferenceContext* c) {
+absl::Status ArgOpShape(shape_inference::InferenceContext* c) {
   ShapeHandle dimension_shape;
   TF_RETURN_IF_ERROR(c->WithRank(c->input(1), 0, &dimension_shape));
 
@@ -1102,7 +1024,7 @@ Status ArgOpShape(shape_inference::InferenceContext* c) {
     }
 
     c->set_output(0, c->MakeShape(dims));
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   int64_t dimension_val;
@@ -1128,7 +1050,7 @@ Status ArgOpShape(shape_inference::InferenceContext* c) {
     }
   }
   c->set_output(0, c->MakeShape(dims));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace
@@ -1153,7 +1075,7 @@ REGISTER_OP("ArgMin")
 
 namespace {
 
-Status SegmentReductionShapeFn(InferenceContext* c) {
+absl::Status SegmentReductionShapeFn(InferenceContext* c) {
   ShapeHandle data_shape;
   ShapeHandle segment_ids_shape;
   TF_RETURN_IF_ERROR(c->WithRankAtLeast(c->input(0), 1, &data_shape));
@@ -1166,10 +1088,10 @@ Status SegmentReductionShapeFn(InferenceContext* c) {
   TF_RETURN_IF_ERROR(
       c->Concatenate(c->Vector(InferenceContext::kUnknownDim), subshape, &out));
   c->set_output(0, out);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status SparseSegmentReductionShapeFn(InferenceContext* c) {
+absl::Status SparseSegmentReductionShapeFn(InferenceContext* c) {
   ShapeHandle data_shape;
   TF_RETURN_IF_ERROR(c->WithRankAtLeast(c->input(0), 1, &data_shape));
 
@@ -1190,11 +1112,11 @@ Status SparseSegmentReductionShapeFn(InferenceContext* c) {
   TF_RETURN_IF_ERROR(
       c->Concatenate(c->Vector(InferenceContext::kUnknownDim), subshape, &out));
   c->set_output(0, out);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status SparseSegmentReductionGradShapeFnImpl(InferenceContext* c,
-                                             bool outputs_unique_indices) {
+absl::Status SparseSegmentReductionGradShapeFnImpl(
+    InferenceContext* c, bool outputs_unique_indices) {
   ShapeHandle data_shape;
   TF_RETURN_IF_ERROR(c->WithRankAtLeast(c->input(0), 1, &data_shape));
 
@@ -1234,21 +1156,21 @@ Status SparseSegmentReductionGradShapeFnImpl(InferenceContext* c,
   if (outputs_unique_indices) {
     c->set_output(1, c->Vector(InferenceContext::kUnknownDim));
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status SparseSegmentReductionGradShapeFn(InferenceContext* c) {
+absl::Status SparseSegmentReductionGradShapeFn(InferenceContext* c) {
   return SparseSegmentReductionGradShapeFnImpl(
       c,
       /*outputs_unique_indices=*/false);
 }
 
-Status SparseSegmentReductionGradV2ShapeFn(InferenceContext* c) {
+absl::Status SparseSegmentReductionGradV2ShapeFn(InferenceContext* c) {
   return SparseSegmentReductionGradShapeFnImpl(c,
                                                /*outputs_unique_indices=*/true);
 }
 
-Status SparseSegmentReductionWithNumSegmentsShapeFn(InferenceContext* c) {
+absl::Status SparseSegmentReductionWithNumSegmentsShapeFn(InferenceContext* c) {
   ShapeHandle data_shape;
   TF_RETURN_IF_ERROR(c->WithRankAtLeast(c->input(0), 1, &data_shape));
 
@@ -1284,7 +1206,7 @@ Status SparseSegmentReductionWithNumSegmentsShapeFn(InferenceContext* c) {
     TF_RETURN_IF_ERROR(c->Concatenate(c->Vector(dim0_value), subshape, &out));
   }
   c->set_output(0, out);
-  return OkStatus();
+  return absl::OkStatus();
 }
 }  // namespace
 
@@ -1417,6 +1339,7 @@ REGISTER_OP("SparseSegmentSum")
     .Attr("T: realnumbertype")
     .Attr("Tidx: {int32, int64} = DT_INT32")
     .Attr("Tsegmentids: {int32, int64} = DT_INT32")
+    .Attr("sparse_gradient: bool = false")
     .SetShapeFn(SparseSegmentReductionShapeFn);
 
 REGISTER_OP("SparseSegmentSumWithNumSegments")
@@ -1429,6 +1352,7 @@ REGISTER_OP("SparseSegmentSumWithNumSegments")
     .Attr("Tidx: {int32, int64} = DT_INT32")
     .Attr("Tnumsegments: {int32,int64} = DT_INT32")
     .Attr("Tsegmentids: {int32, int64} = DT_INT32")
+    .Attr("sparse_gradient: bool = false")
     .SetShapeFn(SparseSegmentReductionWithNumSegmentsShapeFn);
 
 REGISTER_OP("SparseSegmentSumGrad")
@@ -1464,6 +1388,7 @@ REGISTER_OP("SparseSegmentMean")
     .Attr("T: {bfloat16, half, float, double}")
     .Attr("Tidx: {int32, int64} = DT_INT32")
     .Attr("Tsegmentids: {int32, int64} = DT_INT32")
+    .Attr("sparse_gradient: bool = false")
     .SetShapeFn(SparseSegmentReductionShapeFn);
 
 REGISTER_OP("SparseSegmentMeanWithNumSegments")
@@ -1476,6 +1401,7 @@ REGISTER_OP("SparseSegmentMeanWithNumSegments")
     .Attr("Tidx: {int32, int64} = DT_INT32")
     .Attr("Tnumsegments: {int32,int64} = DT_INT32")
     .Attr("Tsegmentids: {int32, int64} = DT_INT32")
+    .Attr("sparse_gradient: bool = false")
     .SetShapeFn(SparseSegmentReductionWithNumSegmentsShapeFn);
 
 REGISTER_OP("SparseSegmentMeanGrad")
@@ -1509,6 +1435,7 @@ REGISTER_OP("SparseSegmentSqrtN")
     .Attr("T: {bfloat16, half, float, double}")
     .Attr("Tidx: {int32, int64} = DT_INT32")
     .Attr("Tsegmentids: {int32, int64} = DT_INT32")
+    .Attr("sparse_gradient: bool = false")
     .SetShapeFn(SparseSegmentReductionShapeFn);
 
 REGISTER_OP("SparseSegmentSqrtNWithNumSegments")
@@ -1521,6 +1448,7 @@ REGISTER_OP("SparseSegmentSqrtNWithNumSegments")
     .Attr("Tidx: {int32, int64} = DT_INT32")
     .Attr("Tnumsegments: {int32,int64} = DT_INT32")
     .Attr("Tsegmentids: {int32, int64} = DT_INT32")
+    .Attr("sparse_gradient: bool = false")
     .SetShapeFn(SparseSegmentReductionWithNumSegmentsShapeFn);
 
 REGISTER_OP("SparseSegmentSqrtNGrad")
@@ -1567,8 +1495,8 @@ REGISTER_OP("Any")
 namespace {
 
 template <typename T>
-Status RangeSize(const Tensor* start_t, const Tensor* limit_t,
-                 const Tensor* delta_t, InferenceContext* const c) {
+absl::Status RangeSize(const Tensor* start_t, const Tensor* limit_t,
+                       const Tensor* delta_t, InferenceContext* const c) {
   T start = start_t->scalar<T>()();
   T limit = limit_t->scalar<T>()();
   T delta = delta_t->scalar<T>()();
@@ -1586,8 +1514,21 @@ Status RangeSize(const Tensor* start_t, const Tensor* limit_t,
 
   int64_t size;
   if (std::is_integral<T>::value) {
-    size = Eigen::divup(static_cast<int64_t>(Eigen::numext::abs(limit - start)),
-                        static_cast<int64_t>(Eigen::numext::abs(delta)));
+    uint64_t range;
+    if ((limit > 0 && start < 0) || (limit < 0 && start > 0)) {
+      range = static_cast<uint64_t>(Eigen::numext::abs(limit)) +
+              static_cast<uint64_t>(Eigen::numext::abs(start));
+    } else {
+      range = static_cast<uint64_t>(Eigen::numext::abs(limit - start));
+    }
+
+    uint64_t size_unsigned =
+        Eigen::divup(range, static_cast<uint64_t>(Eigen::numext::abs(delta)));
+    if (size_unsigned > std::numeric_limits<int64_t>::max()) {
+      return errors::InvalidArgument("Requires ((limit - start) / delta) <= ",
+                                     std::numeric_limits<int64_t>::max());
+    }
+    size = static_cast<int64_t>(size_unsigned);
   } else {
     auto size_auto =
         Eigen::numext::ceil(Eigen::numext::abs((limit - start) / delta));
@@ -1599,7 +1540,7 @@ Status RangeSize(const Tensor* start_t, const Tensor* limit_t,
   }
 
   c->set_output(0, c->Vector(static_cast<int64_t>(size)));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace
@@ -1612,7 +1553,7 @@ REGISTER_OP("Range")
     .Attr(
         "Tidx: "
         "{bfloat16, half, float, double, int8, int16, int32, int64, uint16, "
-        "uint32} = "
+        "uint32, uint64} = "
         "DT_INT32")
     .SetShapeFn([](InferenceContext* c) {
       ShapeHandle unused;
@@ -1629,7 +1570,7 @@ REGISTER_OP("Range")
       TF_RETURN_IF_ERROR(c->GetAttr("Tidx", &dtype));
       if (start_t == nullptr || limit_t == nullptr || delta_t == nullptr) {
         c->set_output(0, c->Vector(InferenceContext::kUnknownDim));
-        return OkStatus();
+        return absl::OkStatus();
       }
       if (dtype == DT_INT32) {
         return RangeSize<int32>(start_t, limit_t, delta_t, c);
@@ -1643,6 +1584,8 @@ REGISTER_OP("Range")
         return RangeSize<uint16>(start_t, limit_t, delta_t, c);
       } else if (dtype == DT_UINT32) {
         return RangeSize<uint32>(start_t, limit_t, delta_t, c);
+      } else if (dtype == DT_UINT64) {
+        return RangeSize<uint64>(start_t, limit_t, delta_t, c);
       } else if (dtype == DT_FLOAT) {
         return RangeSize<float>(start_t, limit_t, delta_t, c);
       } else if (dtype == DT_DOUBLE) {
@@ -1652,7 +1595,7 @@ REGISTER_OP("Range")
       } else {
         return errors::InvalidArgument("Unsupported dtype", dtype);
       }
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("LinSpace")
@@ -1673,7 +1616,7 @@ REGISTER_OP("LinSpace")
       const Tensor* num_t = c->input_tensor(2);
       if (num_t == nullptr) {
         c->set_output(0, c->Vector(InferenceContext::kUnknownDim));
-        return OkStatus();
+        return absl::OkStatus();
       }
 
       int64_t num;
@@ -1684,7 +1627,7 @@ REGISTER_OP("LinSpace")
       }
       if (num <= 0) return errors::InvalidArgument("Requires num > 0: ", num);
       c->set_output(0, c->Vector(num));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("Complex")
@@ -1726,7 +1669,7 @@ REGISTER_OP("Conj")
       if (handle_data != nullptr) {
         c->set_output_handle_shapes_and_types(0, *handle_data);
       }
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 // --------------------------------------------------------------------------
@@ -1753,7 +1696,7 @@ REGISTER_OP("Cross")
         TF_RETURN_IF_ERROR(c->WithValue(dim, 3, &dim));
       }
       c->set_output(0, a_shape);
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 // --------------------------------------------------------------------------
@@ -1790,7 +1733,7 @@ REGISTER_OP("HistogramFixedWidth")
       } else {
         c->set_output(0, c->UnknownShapeOfRank(1));
       }
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("Bincount")
@@ -1808,7 +1751,7 @@ REGISTER_OP("Bincount")
       if (size_tensor == nullptr) {
         // Return unknown shape if size is not known.
         c->set_output(0, c->UnknownShapeOfRank(1));
-        return OkStatus();
+        return absl::OkStatus();
       }
 
       if (size_tensor->dims() != 0) {
@@ -1823,7 +1766,7 @@ REGISTER_OP("Bincount")
                                        ") must be non-negative");
       }
       c->set_output(0, c->MakeShape({size_val}));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("DenseBincount")
@@ -1843,9 +1786,11 @@ REGISTER_OP("DenseBincount")
 
       const Tensor* size_tensor = c->input_tensor(1);
       if (size_tensor == nullptr) {
-        // Return unknown shape if size is not known.
-        c->set_output(0, c->UnknownShape());
-        return OkStatus();
+        // Return "vector of unknown size", "matrix of unknown size" or
+        // "unknown shape" if size is unknown, based on whether the rank of the
+        // input is 1, 2 or unknown respectively.
+        c->set_output(0, c->UnknownShapeOfRank(c->Rank(c->input(0))));
+        return absl::OkStatus();
       }
       if (size_tensor->dims() != 0) {
         return errors::InvalidArgument("Shape must be rank 0 but is rank ",
@@ -1867,12 +1812,12 @@ REGISTER_OP("DenseBincount")
         return errors::InvalidArgument("size (", size_val,
                                        ") must be non-negative");
       }
-      if (c->Rank(c->input(0)) == 1) {
+      if (c->Rank(c->input(0)) == 1 || c->Rank(c->input(0)) == 0) {
         c->set_output(0, c->MakeShape({size_val}));
       } else if (c->Rank(c->input(0)) == 2) {
         c->set_output(0, c->MakeShape({c->Dim(c->input(0), 0), size_val}));
       }
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("SparseBincount")
@@ -1890,7 +1835,7 @@ REGISTER_OP("SparseBincount")
       if (size_tensor == nullptr) {
         // Return unknown shape if size is not known.
         c->set_output(0, c->UnknownShape());
-        return OkStatus();
+        return absl::OkStatus();
       }
       if (size_tensor->dims() != 0) {
         return errors::InvalidArgument("Shape must be rank 0 but is rank ",
@@ -1917,7 +1862,7 @@ REGISTER_OP("SparseBincount")
       if (shape_tensor == nullptr) {
         // Return unknown shape if size is not known.
         c->set_output(0, c->UnknownShape());
-        return OkStatus();
+        return absl::OkStatus();
       }
       if (shape_tensor->NumElements() == 1) {
         c->set_output(0, c->MakeShape({size_val}));
@@ -1927,7 +1872,7 @@ REGISTER_OP("SparseBincount")
       } else {
         return errors::InvalidArgument("Input must be less than rank 2");
       }
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("RaggedBincount")
@@ -1941,7 +1886,7 @@ REGISTER_OP("RaggedBincount")
     .Output("output: T")
     .SetShapeFn([](InferenceContext* c) {
       c->set_output(0, c->UnknownShape());
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("Cumsum")
@@ -2000,7 +1945,7 @@ REGISTER_OP("QuantizedMatMul")
 
       c->set_output(1, c->Scalar());
       c->set_output(2, c->Scalar());
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 // Note: This op is not commutative w.r.t. to all its inputs.
@@ -2021,7 +1966,7 @@ REGISTER_OP("QuantizedMul")
       TF_RETURN_IF_ERROR(shape_inference::BroadcastBinaryOpShapeFn(c));
       c->set_output(1, c->Scalar());
       c->set_output(2, c->Scalar());
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 // Note: This op is not commutative w.r.t. to all its inputs.
@@ -2049,7 +1994,7 @@ REGISTER_OP("QuantizedAdd")
 
       c->set_output(1, c->Scalar());
       c->set_output(2, c->Scalar());
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("QuantizeDownAndShrinkRange")
@@ -2068,7 +2013,7 @@ REGISTER_OP("QuantizeDownAndShrinkRange")
       TF_RETURN_IF_ERROR(c->WithRank(c->input(2), 0, &unused));
       c->set_output(1, c->Scalar());
       c->set_output(2, c->Scalar());
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("Requantize")
@@ -2091,7 +2036,7 @@ REGISTER_OP("Requantize")
       TF_RETURN_IF_ERROR(c->WithRank(c->input(4), 0, &unused));
       c->set_output(1, c->Scalar());
       c->set_output(2, c->Scalar());
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("RequantizationRange")
@@ -2107,7 +2052,7 @@ REGISTER_OP("RequantizationRange")
       TF_RETURN_IF_ERROR(c->WithRank(c->input(2), 0, &unused));
       c->set_output(0, c->Scalar());
       c->set_output(1, c->Scalar());
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 // --------------------------------------------------------------------------
@@ -2127,32 +2072,6 @@ REGISTER_OP("ClipByValue")
     .Attr("T: numbertype")
     .SetShapeFn(shape_inference::UnchangedShape);
 
-#ifdef INTEL_MKL
-// Note: This op is not commutative w.r.t. to all its inputs.
-REGISTER_OP("_MklAddN")
-    .Input("inputs: N * T")
-    .Input("mkl_input: N * uint8")
-    .Output("sum: T")
-    .Output("mkl_sum: uint8")
-    .Attr("N: int >= 1")
-    .Attr("T: numbertype")
-    .SetShapeFn([](InferenceContext* c) {
-      ShapeHandle cur = c->input(c->num_inputs() - 1);
-      for (int i = c->num_inputs() - 2; i >= 0; --i) {
-        TF_RETURN_WITH_CONTEXT_IF_ERROR(c->Merge(c->input(i), cur, &cur),
-                                        "From merging shape ", i,
-                                        " with other shapes.");
-      }
-      c->set_output(0, cur);
-      return OkStatus();
-    })
-    .Doc(R"doc(
-Add two input tensors element wise using mkl kernel sum.
-inputs: Must all be the same size and shape.
-)doc");
-
-#endif  // INTEL_MKL
-
 REGISTER_OP("RequantizePerChannel")
     .Input("input: T")
     .Input("input_min: float")
@@ -2173,7 +2092,7 @@ REGISTER_OP("RequantizePerChannel")
       TF_RETURN_IF_ERROR(c->WithRank(c->input(4), 0, &unused));
       c->set_output(1, c->Scalar());
       c->set_output(2, c->Scalar());
-      return OkStatus();
+      return absl::OkStatus();
     });
 REGISTER_OP("RequantizationRangePerChannel")
     .Input("input: T")
@@ -2189,7 +2108,7 @@ REGISTER_OP("RequantizationRangePerChannel")
       TF_RETURN_IF_ERROR(c->WithRank(c->input(2), 1, &unused));
       c->set_output(0, c->Scalar());
       c->set_output(1, c->Scalar());
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("NextAfter")
@@ -2224,7 +2143,7 @@ REGISTER_OP("SobolSample")
                                 : num_results_t->scalar<int32>()();
 
       c->set_output(0, c->Matrix(num_results, dim));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 }  // namespace tensorflow
