@@ -226,14 +226,14 @@ TEST(MathOpsTest, Select_ShapeFn) {
   typedef std::vector<std::pair<PartialTensorShape, DataType>> ShapeDtypeV;
   std::vector<std::unique_ptr<ShapeDtypeV>> handle_data;
   std::unique_ptr<shape_inference::InferenceContext> c;
-  auto run_inference_for_handles = [&]() -> Status {
+  auto run_inference_for_handles = [&]() -> absl::Status {
     CHECK(op_reg_data->shape_inference_fn != nullptr);
     c.reset(new shape_inference::InferenceContext(
         TF_GRAPH_DEF_VERSION, op.node_def, op_reg_data->op_def,
         {PartialTensorShape(), PartialTensorShape(), PartialTensorShape()}, {},
         {}, handle_data));
     TF_CHECK_OK(c->construction_status());
-    Status s = c->Run(op_reg_data->shape_inference_fn);
+    absl::Status s = c->Run(op_reg_data->shape_inference_fn);
     LOG(INFO) << "Inference got " << s;
     return s;
   };
