@@ -29,16 +29,16 @@ namespace {
 
 // Adds output shapes for dense tensors in Parse*Example ops.
 template <typename TensorShapeType>  // TensorShape or PartialTensorShape
-Status AddDenseOutputShapes(const std::vector<TensorShapeType>& dense_shapes,
-                            const ShapeHandle& prefix, InferenceContext* c,
-                            int* output_idx) {
+absl::Status AddDenseOutputShapes(
+    const std::vector<TensorShapeType>& dense_shapes, const ShapeHandle& prefix,
+    InferenceContext* c, int* output_idx) {
   for (const auto& dense_shape : dense_shapes) {
     ShapeHandle s;
     TF_RETURN_IF_ERROR(c->MakeShapeFromPartialTensorShape(dense_shape, &s));
     TF_RETURN_IF_ERROR(c->Concatenate(prefix, s, &s));
     c->set_output((*output_idx)++, s);
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Adds output shapes for sparse tensors in Parse*Example ops.
@@ -62,9 +62,9 @@ void AddSparseOutputShapes(int num_sparse, const ShapeHandle input_shape,
 }
 
 // Adds output shapes for ragged tensors in Parse*Example ops.
-Status AddRaggedOutputShapes(int num_ragged, bool ragged_rank_2,
-                             const DimensionHandle& num_examples,
-                             InferenceContext* c, int* output_idx) {
+absl::Status AddRaggedOutputShapes(int num_ragged, bool ragged_rank_2,
+                                   const DimensionHandle& num_examples,
+                                   InferenceContext* c, int* output_idx) {
   DimensionHandle num_splits;
   TF_RETURN_IF_ERROR(c->Add(num_examples, 1, &num_splits));
   // Values
@@ -81,7 +81,7 @@ Status AddRaggedOutputShapes(int num_ragged, bool ragged_rank_2,
       c->set_output((*output_idx)++, c->Vector(c->UnknownDim()));
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Adds output shapes for dense_lengths tensors in Parse*Example ops.
@@ -108,7 +108,7 @@ REGISTER_OP("DecodeRaw")
       TF_RETURN_IF_ERROR(c->Concatenate(
           c->input(0), c->Vector(InferenceContext::kUnknownDim), &out));
       c->set_output(0, out);
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("DecodePaddedRaw")
@@ -135,7 +135,7 @@ REGISTER_OP("DecodePaddedRaw")
       TF_RETURN_IF_ERROR(c->Concatenate(c->input(0), c->Vector(width), &out));
 
       c->set_output(0, out);
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("DecodeCompressed")
@@ -172,7 +172,7 @@ REGISTER_OP("ParseExample")
       AddSparseOutputShapes(attrs.num_sparse, input, 1, c, &output_idx);
       TF_RETURN_IF_ERROR(
           AddDenseOutputShapes(attrs.dense_shapes, input, c, &output_idx));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 // Differences between ParseExample and ParseExampleV2:
@@ -223,7 +223,7 @@ REGISTER_OP("ParseExampleV2")
       TF_RETURN_IF_ERROR(AddRaggedOutputShapes(attrs.num_ragged, false,
                                                num_examples, c, &output_idx));
 
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("ParseSingleExample")
@@ -250,7 +250,7 @@ REGISTER_OP("ParseSingleExample")
       AddSparseOutputShapes(attrs.sparse_keys.size(), input, 1, c, &output_idx);
       TF_RETURN_IF_ERROR(
           AddDenseOutputShapes(attrs.dense_shapes, input, c, &output_idx));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("ParseSequenceExample")
@@ -306,7 +306,7 @@ REGISTER_OP("ParseSequenceExample")
       AddDenseLengthsShapes(attrs.num_feature_list_dense, input, c,
                             &output_idx);
 
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 // Differences between ParseSequenceExample and ParseSequenceExampleV2:
@@ -395,7 +395,7 @@ REGISTER_OP("ParseSequenceExampleV2")
                             &output_idx);
       TF_RETURN_IF_ERROR(AddRaggedOutputShapes(
           attrs.num_feature_list_ragged, true, num_examples, c, &output_idx));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("ParseSingleSequenceExample")
@@ -449,7 +449,7 @@ REGISTER_OP("ParseSingleSequenceExample")
       TF_RETURN_IF_ERROR(AddDenseOutputShapes(attrs.feature_list_dense_shapes,
                                               c->UnknownShapeOfRank(1), c,
                                               &output_idx));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("ParseTensor")
@@ -492,13 +492,13 @@ REGISTER_OP("DecodeCSV")
 
       // Propagate shape of the records input.
       for (int i = 0; i < c->num_outputs(); ++i) c->set_output(i, c->input(0));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("StringToNumber")
     .Input("string_tensor: string")
     .Output("output: out_type")
-    .Attr("out_type: {float, double, int32, int64} = DT_FLOAT")
+    .Attr("out_type: {float, double, int32, int64, uint32, uint64} = DT_FLOAT")
     .SetShapeFn(shape_inference::UnchangedShape);
 
 }  // namespace tensorflow
