@@ -27,7 +27,7 @@ using shape_inference::ShapeHandle;
 
 namespace {
 
-Status SparseSparseMinOrMaxShapeFn(InferenceContext* c) {
+absl::Status SparseSparseMinOrMaxShapeFn(InferenceContext* c) {
   ShapeHandle unused;
   TF_RETURN_IF_ERROR(c->WithRank(c->input(0), 2, &unused));  // a_indices
   TF_RETURN_IF_ERROR(c->WithRank(c->input(1), 1, &unused));  // a_values
@@ -38,7 +38,7 @@ Status SparseSparseMinOrMaxShapeFn(InferenceContext* c) {
   c->set_output(0, c->Matrix(InferenceContext::kUnknownDim,
                              InferenceContext::kUnknownDim));
   c->set_output(1, c->Vector(InferenceContext::kUnknownDim));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace
@@ -58,7 +58,7 @@ REGISTER_OP("SparseAddGrad")
       TF_RETURN_IF_ERROR(c->WithRank(c->input(2), 2, &b_indices));
       c->set_output(0, c->Vector(c->Dim(a_indices, 0)));
       c->set_output(1, c->Vector(c->Dim(b_indices, 0)));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("SparseAdd")
@@ -81,7 +81,7 @@ REGISTER_OP("SparseAdd")
           0, c->Matrix(InferenceContext::kUnknownDim, c->Dim(a_shape, 0)));
       c->set_output(1, c->Vector(InferenceContext::kUnknownDim));
       c->set_output(2, a_shape);
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("SparseTensorDenseMatMul")
@@ -116,7 +116,7 @@ REGISTER_OP("SparseTensorDenseMatMul")
       DimensionHandle inner_right = c->Dim(b, adjoint_b ? 1 : 0);
       TF_RETURN_IF_ERROR(c->Merge(inner_left, inner_right, &unused_dim));
       c->set_output(0, c->Matrix(output_left, output_right));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("SerializeSparse")
@@ -132,7 +132,7 @@ REGISTER_OP("SerializeSparse")
       TF_RETURN_IF_ERROR(c->WithRank(c->input(1), 1, &unused));
       TF_RETURN_IF_ERROR(c->WithRank(c->input(2), 1, &unused));
       c->set_output(0, c->Vector(3));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("SerializeManySparse")
@@ -148,7 +148,7 @@ REGISTER_OP("SerializeManySparse")
       TF_RETURN_IF_ERROR(c->WithRank(c->input(1), 1, &unused));
       TF_RETURN_IF_ERROR(c->WithRank(c->input(2), 1, &unused));
       c->set_output(0, c->Matrix(InferenceContext::kUnknownDim, 3));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("DeserializeSparse")
@@ -168,7 +168,7 @@ REGISTER_OP("DeserializeSparse")
                                  InferenceContext::kUnknownDim));
       c->set_output(1, c->Vector(InferenceContext::kUnknownDim));
       c->set_output(2, c->Vector(InferenceContext::kUnknownDim));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("DeserializeManySparse")
@@ -189,7 +189,7 @@ REGISTER_OP("DeserializeManySparse")
                                  InferenceContext::kUnknownDim));
       c->set_output(1, c->Vector(InferenceContext::kUnknownDim));
       c->set_output(2, c->Vector(InferenceContext::kUnknownDim));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("SparseToDense")
@@ -205,7 +205,7 @@ REGISTER_OP("SparseToDense")
       ShapeHandle out;
       TF_RETURN_IF_ERROR(c->MakeShapeFromShapeTensor(1, &out));
       c->set_output(0, out);
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("SparseConcat")
@@ -250,7 +250,7 @@ REGISTER_OP("SparseConcat")
       c->set_output(0, c->Matrix(output_row_count, output_ind_cols));
       c->set_output(1, c->Vector(output_row_count));
       c->set_output(2, output_shape);
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("SparseCross")
@@ -273,7 +273,7 @@ REGISTER_OP("SparseCross")
       c->set_output(0, c->Matrix(c->UnknownDim(), 2));
       c->set_output(1, c->Vector(c->UnknownDim()));
       c->set_output(2, c->Vector(2));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("SparseCrossV2")
@@ -292,7 +292,7 @@ REGISTER_OP("SparseCrossV2")
       c->set_output(0, c->Matrix(c->UnknownDim(), 2));
       c->set_output(1, c->Vector(c->UnknownDim()));
       c->set_output(2, c->Vector(2));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("SparseCrossHashed")
@@ -313,7 +313,7 @@ REGISTER_OP("SparseCrossHashed")
       c->set_output(0, c->Matrix(c->UnknownDim(), 2));
       c->set_output(1, c->Vector(c->UnknownDim()));
       c->set_output(2, c->Vector(2));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("SparseSplit")
@@ -342,7 +342,7 @@ REGISTER_OP("SparseSplit")
         c->set_output(out_idx++, output_values);
       for (int i = 0; i < num_splits; ++i)
         c->set_output(out_idx++, output_shape);
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("SparseSliceGrad")
@@ -356,7 +356,7 @@ REGISTER_OP("SparseSliceGrad")
       ShapeHandle indices;
       TF_RETURN_IF_ERROR(c->WithRank(c->input(1), 2, &indices));
       c->set_output(0, c->Vector(c->Dim(indices, 0)));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("SparseSlice")
@@ -379,7 +379,7 @@ REGISTER_OP("SparseSlice")
       c->set_output(0, output_indices);
       c->set_output(1, output_values);
       c->set_output(2, output_shape);
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("SparseReorder")
@@ -400,7 +400,7 @@ REGISTER_OP("SparseReorder")
 
       c->set_output(0, indices);
       c->set_output(1, values);
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("SparseReshape")
@@ -420,7 +420,7 @@ REGISTER_OP("SparseReshape")
 
       c->set_output(0, c->Matrix(c->Dim(indices, 0), c->Dim(new_shape, 0)));
       c->set_output(1, new_shape);
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("SparseTensorDenseAdd")
@@ -433,7 +433,7 @@ REGISTER_OP("SparseTensorDenseAdd")
     .Attr("Tindices: {int32, int64}")
     .SetShapeFn([](InferenceContext* c) {
       c->set_output(0, c->input(3));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("SparseReduceMax")
@@ -515,7 +515,7 @@ REGISTER_OP("SparseSoftmax")
       TF_RETURN_IF_ERROR(c->WithRank(c->input(1), 1, &values));  // sp_values
       TF_RETURN_IF_ERROR(c->WithRank(c->input(2), 1, &unused));
       c->set_output(0, values);
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("SparseSparseMaximum")
@@ -557,7 +557,7 @@ REGISTER_OP("AddSparseToTensorsMap")
       TF_RETURN_IF_ERROR(c->WithRank(c->input(1), 1, &unused));
       TF_RETURN_IF_ERROR(c->WithRank(c->input(2), 1, &unused));
       c->set_output(0, c->Scalar());
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("AddManySparseToTensorsMap")
@@ -575,7 +575,7 @@ REGISTER_OP("AddManySparseToTensorsMap")
       TF_RETURN_IF_ERROR(c->WithRank(c->input(1), 1, &unused));
       TF_RETURN_IF_ERROR(c->WithRank(c->input(2), 1, &unused));
       c->set_output(0, c->Vector(InferenceContext::kUnknownDim));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("TakeManySparseFromTensorsMap")
@@ -596,7 +596,7 @@ REGISTER_OP("TakeManySparseFromTensorsMap")
                                  InferenceContext::kUnknownDim));
       c->set_output(1, c->Vector(InferenceContext::kUnknownDim));
       c->set_output(2, c->Vector(InferenceContext::kUnknownDim));
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("SparseFillEmptyRows")
@@ -637,7 +637,7 @@ REGISTER_OP("SparseFillEmptyRows")
       c->set_output(1, output_values);
       c->set_output(2, empty_row_indicator);
       c->set_output(3, reverse_index_map);
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 REGISTER_OP("SparseFillEmptyRowsGrad")
@@ -653,7 +653,7 @@ REGISTER_OP("SparseFillEmptyRowsGrad")
       TF_RETURN_IF_ERROR(c->WithRank(grad_values, 1, &grad_values));
       c->set_output(0, reverse_index_map);
       c->set_output(1, c->Scalar());
-      return OkStatus();
+      return absl::OkStatus();
     });
 
 }  // namespace tensorflow
