@@ -18,9 +18,9 @@ limitations under the License.
 
 #include <string>
 
+#include "xla/tsl/platform/cloud/auth_provider.h"
 #include "tensorflow/core/platform/errors.h"
 #include "tensorflow/core/platform/status.h"
-#include "tensorflow/tsl/platform/cloud/auth_provider.h"
 
 namespace tensorflow {
 // NOLINTBEGIN(misc-unused-using-decls)
