@@ -16,9 +16,9 @@ limitations under the License.
 #ifndef TENSORFLOW_CORE_PLATFORM_CLOUD_COMPUTE_ENGINE_ZONE_PROVIDER_H_
 #define TENSORFLOW_CORE_PLATFORM_CLOUD_COMPUTE_ENGINE_ZONE_PROVIDER_H_
 
+#include "xla/tsl/platform/cloud/compute_engine_zone_provider.h"
 #include "tensorflow/core/platform/cloud/compute_engine_metadata_client.h"
 #include "tensorflow/core/platform/cloud/zone_provider.h"
-#include "tensorflow/tsl/platform/cloud/compute_engine_zone_provider.h"
 
 namespace tensorflow {
 using tsl::ComputeEngineZoneProvider;  // NOLINT(misc-unused-using-decls)
