@@ -16,8 +16,8 @@ limitations under the License.
 #ifndef TENSORFLOW_CORE_PLATFORM_CLOUD_GCS_THROTTLE_H_
 #define TENSORFLOW_CORE_PLATFORM_CLOUD_GCS_THROTTLE_H_
 
+#include "xla/tsl/platform/cloud/gcs_throttle.h"
 #include "tensorflow/core/platform/env.h"
-#include "tensorflow/tsl/platform/cloud/gcs_throttle.h"
 
 namespace tensorflow {
 // NOLINTBEGIN(misc-unused-using-decls)
