@@ -22,6 +22,7 @@ limitations under the License.
 #include <vector>
 
 #include <curl/curl.h>
+#include "xla/tsl/platform/cloud/http_request_fake.h"
 #include "tensorflow/core/lib/core/status_test_util.h"
 #include "tensorflow/core/platform/cloud/curl_http_request.h"
 #include "tensorflow/core/platform/errors.h"
@@ -31,7 +32,6 @@ limitations under the License.
 #include "tensorflow/core/platform/stringpiece.h"
 #include "tensorflow/core/platform/test.h"
 #include "tensorflow/core/platform/types.h"
-#include "tensorflow/tsl/platform/cloud/http_request_fake.h"
 
 namespace tensorflow {
 // NOLINTBEGIN(misc-unused-using-decls)
