@@ -15,13 +15,16 @@ limitations under the License.
 
 #include "tensorflow/core/platform/error_payloads.h"
 
+#include "absl/status/status.h"
+#include "absl/strings/cord.h"
+#include "tensorflow/core/protobuf/core_platform_payloads.pb.h"
+
 namespace tsl {
 
 using ::tensorflow::core::platform::ErrorSourceProto;
 
 void OkOrSetErrorCounterPayload(
-    const ErrorSourceProto::ErrorSource& error_source,
-    tensorflow::Status& status) {
+    const ErrorSourceProto::ErrorSource& error_source, absl::Status& status) {
   if (!status.ok() &&
       !status.GetPayload(tensorflow::kErrorSource).has_value()) {
     ErrorSourceProto error_source_proto;
