@@ -15,17 +15,20 @@ limitations under the License.
 #ifndef TENSORFLOW_CORE_PROFILER_RPC_PROFILER_SERVER_H_
 #define TENSORFLOW_CORE_PROFILER_RPC_PROFILER_SERVER_H_
 
-#include <memory>
-
 #include "grpcpp/grpcpp.h"
-#include "tensorflow/core/platform/types.h"
-#include "tensorflow/tsl/profiler/protobuf/profiler_service.grpc.pb.h"
-#include "tensorflow/tsl/profiler/rpc/profiler_server.h"
+#include "absl/base/macros.h"
+#include "xla/tsl/profiler/rpc/profiler_server.h"
+
+// TODO: b/323943471 - This macro should eventually be provided by Abseil.
+#ifndef ABSL_DEPRECATE_AND_INLINE
+#define ABSL_DEPRECATE_AND_INLINE()
+#endif
 
 namespace tensorflow {
 namespace profiler {
 
-using tsl::profiler::ProfilerServer;  // NOLINT
+using ProfilerServer ABSL_DEPRECATE_AND_INLINE() =
+    tsl::profiler::ProfilerServer;  // NOLINT
 
 }  // namespace profiler
 }  // namespace tensorflow
