@@ -13,19 +13,26 @@ See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
 #include <algorithm>
-#include <cstdlib>
+#include <cassert>
+#include <cstdint>
 #include <functional>
 #include <memory>
 #include <string>
 #include <utility>
 #include <vector>
 
-#include "absl/strings/str_format.h"
+#include "absl/base/casts.h"
+#include "absl/log/check.h"
+#include "absl/status/status.h"
+#include "absl/status/statusor.h"
+#include "absl/strings/string_view.h"
+#include "absl/types/span.h"
 #include "tensorflow/core/framework/op_kernel.h"
 #include "tensorflow/core/framework/op_requires.h"
 #include "tensorflow/core/framework/resource_mgr.h"
 #include "tensorflow/core/kernels/batching_util/adaptive_shared_batch_scheduler.h"
 #include "tensorflow/core/kernels/batching_util/batch_resource_base.h"
+#include "tensorflow/core/kernels/batching_util/batch_scheduler.h"
 #include "tensorflow/core/platform/random.h"
 #include "tensorflow/core/platform/status.h"
 #include "tensorflow/core/platform/statusor.h"
@@ -56,8 +63,8 @@ using ::tfrt::AsyncValue;
 using ::tfrt::HostContext;
 using ::tfrt::RCReference;
 
-Status GetTfrtExecutionContext(OpKernelContext* c,
-                               const tfrt::ExecutionContext** exec_ctx) {
+absl::Status GetTfrtExecutionContext(OpKernelContext* c,
+                                     const tfrt::ExecutionContext** exec_ctx) {
   // ExecutionContext's address is passed in as an I64 input. exec_ctx is only
   // valid during the period of one bef execution. It should not be stored and
   // accessed after bef execution completes.
@@ -65,7 +72,7 @@ Status GetTfrtExecutionContext(OpKernelContext* c,
   TF_RETURN_IF_ERROR(c->input("tfrt_exec_ctx", &tensor));
   int64_t exec_ctx_intptr = *reinterpret_cast<const int64_t*>(tensor->data());
   *exec_ctx = absl::bit_cast<const tfrt::ExecutionContext*>(exec_ctx_intptr);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 class FallbackBatchResource : public tensorflow::serving::BatchResourceBase {
@@ -83,7 +90,7 @@ class FallbackBatchResource : public tensorflow::serving::BatchResourceBase {
     }
   };
 
-  static StatusOr<tfrt::ResourceContext*> GetClientGraphResourceContext(
+  static absl::StatusOr<tfrt::ResourceContext*> GetClientGraphResourceContext(
       OpKernelContext* context) {
     const tfrt::ExecutionContext* exec_ctx = nullptr;
     TF_RETURN_IF_ERROR(GetTfrtExecutionContext(context, &exec_ctx));
@@ -97,7 +104,7 @@ class FallbackBatchResource : public tensorflow::serving::BatchResourceBase {
                : exec_ctx->resource_context();
   }
 
-  static StatusOr<std::unique_ptr<BatchTask>> CreateBatchTask(
+  static absl::StatusOr<std::unique_ptr<BatchTask>> CreateBatchTask(
       OpKernelContext* context) {
     const tfrt::ExecutionContext* exec_ctx = nullptr;
     TF_RETURN_IF_ERROR(GetTfrtExecutionContext(context, &exec_ctx));
@@ -109,18 +116,22 @@ class FallbackBatchResource : public tensorflow::serving::BatchResourceBase {
     return batch_function->name();
   }
 
-  static Status Create(OpKernelContext* c, int32_t num_batch_threads,
-                       int32_t max_batch_size, int32_t batch_timeout_micros,
-                       int32_t max_enqueued_batches,
-                       ArrayRef<int32_t> allowed_batch_sizes,
-                       tsl::RCReference<const tfrt::Function> bef_func,
-                       bool enable_large_batch_splitting, bool disable_padding,
-                       std::unique_ptr<FallbackBatchResource>* resource) {
+  static absl::Status Create(OpKernelContext* c,
+                             const serving::BatchResourceOptions& options,
+                             tsl::RCReference<const tfrt::Function> bef_func,
+                             bool enable_large_batch_splitting,
+                             bool disable_padding,
+                             std::unique_ptr<FallbackBatchResource>* resource) {
     const tfrt::ExecutionContext* exec_ctx = nullptr;
     TF_RETURN_IF_ERROR(GetTfrtExecutionContext(c, &exec_ctx));
 
     BatcherT::Options batcher_options;
-    batcher_options.num_batch_threads = num_batch_threads;
+    batcher_options.num_batch_threads = options.num_batch_threads;
+    if (options.mixed_priority_batching_policy ==
+        serving::MixedPriorityBatchingPolicy::kPriorityMerge) {
+      batcher_options.use_global_scheduler = true;
+      batcher_options.rank_queues = true;
+    }
     std::shared_ptr<BatcherT> batcher;
     TF_RETURN_IF_ERROR(BatcherT::Create(batcher_options, &batcher));
 
@@ -135,15 +146,21 @@ class FallbackBatchResource : public tensorflow::serving::BatchResourceBase {
     resource->reset(new FallbackBatchResource(
         *exec_ctx, *fallback_request_state, std::move(bef_func),
         std::move(batcher),
-        GetBatcherQueueOptions(num_batch_threads, max_batch_size,
-                               batch_timeout_micros, max_enqueued_batches,
-                               allowed_batch_sizes,
-                               enable_large_batch_splitting, disable_padding),
-        allowed_batch_sizes));
-    return OkStatus();
+        GetBatcherQueueOptions(
+            options.num_batch_threads, options.max_batch_size,
+            options.batch_timeout_micros, options.max_enqueued_batches,
+            options.allowed_batch_sizes, enable_large_batch_splitting,
+            disable_padding, options.batch_padding_policy,
+            options.low_priority_max_batch_size,
+            options.low_priority_batch_timeout_micros,
+            options.low_priority_max_enqueued_batches,
+            options.low_priority_allowed_batch_sizes,
+            options.mixed_priority_batching_policy),
+        options.allowed_batch_sizes));
+    return absl::OkStatus();
   }
 
-  static Status Create(
+  static absl::Status Create(
       OpKernelContext* c,
       AdaptiveBatcherT::Options adaptive_shared_batch_scheduler_options,
       int32_t max_batch_size, int32_t batch_timeout_micros,
@@ -173,7 +190,7 @@ class FallbackBatchResource : public tensorflow::serving::BatchResourceBase {
                                        true /* enable large batch split */,
                                        allowed_batch_sizes, disable_padding),
         allowed_batch_sizes));
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   string DebugString() const final { return "FallbackBatchResource"; }
@@ -228,7 +245,7 @@ class FallbackBatchResource : public tensorflow::serving::BatchResourceBase {
   void ProcessFuncBatchImpl(
       const BatchTask& last_task, absl::Span<const Tensor> inputs,
       std::vector<Tensor>* combined_outputs,
-      std::function<void(const Status&)> done) const override;
+      std::function<void(const absl::Status&)> done) const override;
 
   HostContext* const host_ctx_;
   tfrt::ResourceContext* const resource_context_;
@@ -242,7 +259,7 @@ tfrt::AsyncValueRef<tfrt_stub::FallbackTensor> TFTensorToFallbackTensor(
   return tfrt::MakeAvailableAsyncValueRef<tfrt_stub::FallbackTensor>(tf_tensor);
 }
 
-Status SetUpKernelFallbackCompatRequestContextForBatch(
+absl::Status SetUpKernelFallbackCompatRequestContextForBatch(
     tfrt::RequestContextBuilder* builder,
     tfrt_stub::OpKernelRunnerTable* runner_table,
     tfd::FallbackResourceArray* resource_array,
@@ -275,15 +292,13 @@ Status SetUpKernelFallbackCompatRequestContextForBatch(
       src_fallback_request_state->runtime_config());
 }
 
-StatusOr<RCReference<tfrt::RequestContext>> SetUpRequestContext(
+absl::StatusOr<RCReference<tfrt::RequestContext>> SetUpRequestContext(
     HostContext* host_ctx, tfrt::ResourceContext* resource_context,
     tfrt_stub::OpKernelRunnerTable* runner_table,
     tfd::FallbackResourceArray* resource_array,
     tfrt::RequestContext* src_req_ctx) {
-  // Using the same logic as in the c'tor of FunctionLibraryRuntime::Options,
-  // to avoid clash with any Session-generated step ID. DirectSession and
-  // MasterSession generates non-negative step IDs.
-  int64_t step_id = -std::abs(static_cast<int64_t>(random::New64()));
+  // Connect to the batch step id propagated from batch task.
+  int64_t step_id = src_req_ctx->id();
 
   tfrt::RequestContextBuilder request_context_builder(
       host_ctx, resource_context, step_id);
@@ -303,7 +318,7 @@ StatusOr<RCReference<tfrt::RequestContext>> SetUpRequestContext(
 void FallbackBatchResource::ProcessFuncBatchImpl(
     const BatchTask& last_task, absl::Span<const Tensor> inputs,
     std::vector<Tensor>* combined_outputs,
-    std::function<void(const Status&)> done) const {
+    std::function<void(const absl::Status&)> done) const {
   std::vector<tsl::RCReference<AsyncValue>> arguments;
   arguments.reserve(inputs.size() + 1);
   // The first argument is a Chain.
@@ -327,14 +342,14 @@ void FallbackBatchResource::ProcessFuncBatchImpl(
   auto req_ctx = std::move(statusor).value();
 
   int64_t id = req_ctx->id();
-  tensorflow::profiler::TraceMeProducer activity(
+  tsl::profiler::TraceMeProducer activity(
       // To TraceMeConsumers in WorkQueue.
       [id] {
-        return tensorflow::profiler::TraceMeEncode("RunBefFunction",
-                                                   {{"id", id}, {"_r", 1}});
+        return tsl::profiler::TraceMeEncode("RunBefFunction",
+                                            {{"id", id}, {"_r", 1}});
       },
-      tensorflow::profiler::ContextType::kTfrtExecutor, id,
-      tensorflow::profiler::TraceMeLevel::kInfo);
+      tsl::profiler::ContextType::kTfrtExecutor, id,
+      tsl::profiler::TraceMeLevel::kInfo);
 
   tfrt::ExecutionContext batch_exec_ctx(std::move(req_ctx));
   batch_exec_ctx.set_work_queue(&exec_ctx.work_queue());
@@ -363,7 +378,7 @@ void FallbackBatchResource::ProcessFuncBatchImpl(
         result->get<tfrt_stub::FallbackTensor>().tensor();
   }
   // Aggregate errors.
-  Status final_status;
+  absl::Status final_status;
   if (!errors.empty()) {
     if (errors.size() > 1) {
       auto last = std::unique(errors.begin(), errors.end());
@@ -412,6 +427,37 @@ REGISTER_OP("_BatchFunctionFallback")
     .Attr("low_priority_batch_timeout_micros: int = 0")
     .Attr("low_priority_allowed_batch_sizes: list(int) = []")
     .Attr("low_priority_max_enqueued_batches: int = 0")
+    // Policy that determines the mixed priority batching behavior when low
+    // priority batch parameters are present.
+    //
+    // low_priority_padding_with_next_allowed_batch_size: If high priority
+    // batches time out without reaching the max batch size, low priority inputs
+    // pad the high priority batches up to the next allowed batch size. A low
+    // priority only batch gets schedule only when the low priority input times
+    // out or reaches the max batch size while there is no high priority input
+    // waiting to be processed.
+    // low_priority_padding_with_max_batch_size: Same as above but pad up to the
+    // max batch size.
+    // priority_isolation: High priority and low priority inputs never share the
+    // same batch, i.e., no low priority input padding high priority batches.
+    // Low priority inputs get scheduled only as part of low priority only
+    // batches as described above.
+    // priority_merge: High and low priority inputs are queued separately but
+    // when a batch needs to be scheduled, the two queues are treated as one
+    // merged flat list of inputs with high priority inputs at the front of the
+    // list of tasks to use for the next batch. If all inputs are of the same
+    // priority, the behavior is the same as disabling prioritization.
+    .Attr(
+        "mixed_priority_policy: "
+        "{'low_priority_padding_with_max_batch_size', "
+        "'low_priority_padding_with_next_allowed_batch_size', "
+        "'priority_isolation', "
+        "'priority_merge'} = 'low_priority_padding_with_max_batch_size'")
+    // See the description of the batch_padding_policy attribute of
+    // BatchFunction in core/ops/batch_ops.cc.
+    .Attr(
+        "batch_padding_policy: "
+        "{'PAD_UP', 'BATCH_DOWN', 'MINIMIZE_TPU_COST_PER_REQUEST'} = 'PAD_UP'")
     .Attr("Tin: list(type)")
     .Attr("Tcaptured: list(type) >= 0")
     .Attr("Tout: list(type)")
