@@ -16,18 +16,24 @@ limitations under the License.
 #include <memory>
 #include <utility>
 
-#include "tensorflow/compiler/xla/pjrt/tfrt_cpu_pjrt_client.h"
-#include "tensorflow/compiler/xla/statusor.h"
+#include "absl/status/statusor.h"
+#include "xla/pjrt/pjrt_client.h"
+#include "xla/pjrt/plugin/xla_cpu/cpu_client_options.h"
+#include "xla/pjrt/plugin/xla_cpu/xla_cpu_pjrt_client.h"
 #include "tensorflow/core/framework/types.h"
 #include "tensorflow/core/tfrt/common/pjrt_client_factory_options.h"
 #include "tensorflow/core/tfrt/common/pjrt_client_factory_registry.h"
+#include "tsl/platform/statusor.h"
 
 namespace xla {
 
-StatusOr<std::unique_ptr<xla::PjRtClient>> GetCpuClient(
+absl::StatusOr<std::unique_ptr<xla::PjRtClient>> GetCpuClient(
     const PjrtClientFactoryOptions& option) {
+  xla::CpuClientOptions cpu_options;
+  cpu_options.asynchronous = option.cpu_options.asynchronous;
+
   TF_ASSIGN_OR_RETURN(std::unique_ptr<PjRtClient> client,
-                      xla::GetTfrtCpuClient(option.cpu_options.asynchronous));
+                      xla::GetXlaPjrtCpuClient(cpu_options));
 
   return std::move(client);
 }
