@@ -13,13 +13,12 @@ See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
 
-#include <optional>
-
-#include <gmock/gmock.h>
 #include <gtest/gtest.h>
+#include "xla/tsl/framework/device_type.h"
 #include "tensorflow/core/framework/types.h"
 #include "tensorflow/core/tfrt/common/pjrt_client_factory_options.h"
 #include "tensorflow/core/tfrt/common/pjrt_client_factory_registry.h"
+#include "tsl/platform/statusor.h"
 
 namespace xla {
 namespace {
