@@ -14,8 +14,8 @@ limitations under the License.
 ==============================================================================*/
 #include "tensorflow/core/tfrt/graph_executor/graph_executor.h"
 
+#include <cstdint>
 #include <memory>
-#include <optional>
 #include <string>
 #include <utility>
 #include <vector>
@@ -24,23 +24,37 @@ limitations under the License.
 #include "learning/brain/experimental/tfrt/native_lowering/kernels/sync_fallback_kernels.h"
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
+#include "absl/status/status.h"
 #include "absl/time/time.h"
+#include "absl/types/span.h"
+#include "tensorflow/cc/framework/ops.h"
+#include "tensorflow/cc/framework/scope.h"
 #include "tensorflow/cc/ops/array_ops.h"
 #include "tensorflow/cc/ops/const_op.h"
+#include "xla/tsl/lib/core/status_test_util.h"
+#include "tensorflow/core/framework/common_shape_fns.h"
 #include "tensorflow/core/framework/graph.pb.h"
+#include "tensorflow/core/framework/op.h"
+#include "tensorflow/core/framework/op_kernel.h"
+#include "tensorflow/core/framework/tensor.h"
+#include "tensorflow/core/framework/tensor_shape.h"
+#include "tensorflow/core/framework/types.h"
 #include "tensorflow/core/framework/types.pb.h"
+#include "tensorflow/core/graph/graph.h"
 #include "tensorflow/core/graph/graph_def_builder.h"
-#include "tensorflow/core/grappler/utils/grappler_test.h"
-#include "tensorflow/core/platform/statusor.h"
+#include "tensorflow/core/platform/mutex.h"
+#include "tensorflow/core/platform/status.h"
 #include "tensorflow/core/protobuf/rewriter_config.pb.h"
+#include "tensorflow/core/tfrt/fallback/fallback_state.h"
+#include "tensorflow/core/tfrt/graph_executor/config.h"
+#include "tensorflow/core/tfrt/graph_executor/graph_execution_options.h"
 #include "tensorflow/core/tfrt/mlrt/interpreter/context.h"
 #include "tensorflow/core/tfrt/mlrt/interpreter/value.h"
 #include "tensorflow/core/tfrt/mlrt/kernel/kernel.h"
 #include "tensorflow/core/tfrt/saved_model/saved_model_testutil.h"
-#include "tensorflow/tsl/lib/core/status_test_util.h"
-#include "tensorflow/tsl/platform/status.h"
-#include "tensorflow/tsl/platform/statusor.h"
+#include "tsl/platform/statusor.h"
 #include "tfrt/cpp_tests/test_util.h"  // from @tf_runtime
+#include "tfrt/host_context/resource_context.h"  // from @tf_runtime
 #include "tfrt/tensor/dense_host_tensor.h"  // from @tf_runtime
 
 namespace tensorflow {
@@ -63,7 +77,7 @@ class GraphExecutorForTestingCostAnalysis : public GraphExecutor {
 
 class GraphExecutorTest : public ::testing::TestWithParam<bool> {};
 
-tensorflow::Status GetSimpleGraphDef(GraphDef& graph_def) {
+absl::Status GetSimpleGraphDef(GraphDef& graph_def) {
   auto scope = tensorflow::Scope::NewRootScope().WithDevice("/device:CPU:0");
 
   auto input = ops::Placeholder(scope.WithOpName("input"), DT_INT32);
@@ -96,7 +110,7 @@ TEST_P(GraphExecutorTest, Vanilla) {
   auto resource_context = std::make_unique<tfrt::ResourceContext>();
   TF_ASSERT_OK_AND_ASSIGN(
       auto graph_executor,
-      GraphExecutor::Create(std::move(options), *fallback_state,
+      GraphExecutor::Create(std::move(options), std::move(fallback_state),
                             std::move(resource_context), graph_def,
                             GetKernelRegistry()));
 
@@ -134,11 +148,15 @@ TEST_P(GraphExecutorTest, OnlineCostAnalysisOptionsOverrideToOnce) {
       tensorflow::tfrt_stub::FallbackState::Create(
           CreateDefaultSessionOptions(options), graph_def.library()));
   auto resource_context = std::make_unique<tfrt::ResourceContext>();
+  tensorflow::tfrt_stub::RuntimeConfig runtime_config;
+  tensorflow::tf2xla::v1::MlirBridgeConfig mlir_bridge_config;
+  mlir_bridge_config.set_enable_tf2xla_mlir_bridge(false);
+  TF_ASSERT_OK(runtime_config.Add(mlir_bridge_config));
   TF_ASSERT_OK_AND_ASSIGN(
       auto graph_executor_base,
-      GraphExecutor::Create(std::move(options), *fallback_state,
+      GraphExecutor::Create(std::move(options), std::move(fallback_state),
                             std::move(resource_context), graph_def,
-                            GetKernelRegistry()));
+                            GetKernelRegistry(), &runtime_config));
   auto graph_executor = std::unique_ptr<GraphExecutorForTestingCostAnalysis>(
       static_cast<GraphExecutorForTestingCostAnalysis*>(
           graph_executor_base.release()));
@@ -190,7 +208,7 @@ TEST_P(GraphExecutorTest, OnlineCostAnalysisEveryTime) {
   auto resource_context = std::make_unique<tfrt::ResourceContext>();
   TF_ASSERT_OK_AND_ASSIGN(
       auto graph_executor_base,
-      GraphExecutor::Create(std::move(options), *fallback_state,
+      GraphExecutor::Create(std::move(options), std::move(fallback_state),
                             std::move(resource_context), graph_def,
                             GetKernelRegistry()));
   auto graph_executor = std::unique_ptr<GraphExecutorForTestingCostAnalysis>(
@@ -234,7 +252,7 @@ TEST_P(GraphExecutorTest, OnlineCostAnalysisDisabled) {
   auto resource_context = std::make_unique<tfrt::ResourceContext>();
   TF_ASSERT_OK_AND_ASSIGN(
       auto graph_executor_base,
-      GraphExecutor::Create(std::move(options), *fallback_state,
+      GraphExecutor::Create(std::move(options), std::move(fallback_state),
                             std::move(resource_context), graph_def,
                             GetKernelRegistry()));
   auto graph_executor = std::unique_ptr<GraphExecutorForTestingCostAnalysis>(
@@ -273,7 +291,7 @@ TEST_P(GraphExecutorTest, OnlineCostAnalysisPeriodic) {
   auto resource_context = std::make_unique<tfrt::ResourceContext>();
   TF_ASSERT_OK_AND_ASSIGN(
       auto graph_executor_base,
-      GraphExecutor::Create(std::move(options), *fallback_state,
+      GraphExecutor::Create(std::move(options), std::move(fallback_state),
                             std::move(resource_context), graph_def,
                             GetKernelRegistry()));
   auto graph_executor = std::unique_ptr<GraphExecutorForTestingCostAnalysis>(
@@ -402,7 +420,7 @@ TEST_P(GraphExecutorTest, Cancellation) {
   auto resource_context = std::make_unique<tfrt::ResourceContext>();
   TF_ASSERT_OK_AND_ASSIGN(
       auto graph_executor,
-      GraphExecutor::Create(std::move(options), *fallback_state,
+      GraphExecutor::Create(std::move(options), std::move(fallback_state),
                             std::move(resource_context), graph_def,
                             GetKernelRegistry()));
   {
@@ -458,7 +476,7 @@ TEST_F(GraphExecutorTest, Extend) {
   auto resource_context = std::make_unique<tfrt::ResourceContext>();
   TF_ASSERT_OK_AND_ASSIGN(
       auto graph_executor,
-      GraphExecutor::Create(std::move(options), *fallback_state,
+      GraphExecutor::Create(std::move(options), std::move(fallback_state),
                             std::move(resource_context), graph_def,
                             GetKernelRegistry()));
 
@@ -503,7 +521,7 @@ TEST_F(GraphExecutorTest, DisableCompilation) {
   auto resource_context = std::make_unique<tfrt::ResourceContext>();
   TF_ASSERT_OK_AND_ASSIGN(
       auto graph_executor,
-      GraphExecutor::Create(std::move(options), *fallback_state,
+      GraphExecutor::Create(std::move(options), std::move(fallback_state),
                             std::move(resource_context), graph_def,
                             GetKernelRegistry()));
 
@@ -550,7 +568,7 @@ TEST_F(GraphExecutorTest, SyncExecute) {
   auto resource_context = std::make_unique<tfrt::ResourceContext>();
   TF_ASSERT_OK_AND_ASSIGN(
       auto graph_executor,
-      GraphExecutor::Create(std::move(options), *fallback_state,
+      GraphExecutor::Create(std::move(options), std::move(fallback_state),
                             std::move(resource_context), graph_def,
                             GetKernelRegistry()));
 
