@@ -17,6 +17,8 @@ limitations under the License.
 
 #include <algorithm>
 #include <atomic>
+#include <cstddef>
+#include <cstdint>
 #include <functional>
 #include <list>
 #include <memory>
@@ -26,10 +28,18 @@ limitations under the License.
 
 #include "absl/base/attributes.h"
 #include "absl/container/flat_hash_map.h"
+#include "absl/container/inlined_vector.h"
 #include "absl/functional/any_invocable.h"
+#include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/status/status.h"
+#include "absl/strings/string_view.h"
 #include "absl/types/span.h"
+#include "tensorflow/core/tfrt/mlrt/bytecode/bytecode.h"
 #include "tensorflow/core/tfrt/mlrt/bytecode/executable.h"
+#include "tensorflow/core/tfrt/mlrt/bytecode/function.h"
+#include "tensorflow/core/tfrt/mlrt/bytecode/kernel.h"
+#include "tensorflow/core/tfrt/mlrt/bytecode/span.h"
 #include "tensorflow/core/tfrt/mlrt/interpreter/attribute_span.h"
 #include "tensorflow/core/tfrt/mlrt/interpreter/register_span.h"
 #include "tensorflow/core/tfrt/mlrt/interpreter/value.h"
@@ -251,8 +261,10 @@ class ExecutionContext {
   ExecutionContext(
       const LoadedExecutable* loaded_executable,
       std::vector<std::unique_ptr<context_internal::UserContextBase>>
-          user_contexts)
+          user_contexts,
+      const std::vector<std::function<void(absl::Status)>>& user_error_loggers)
       : user_contexts_(std::move(user_contexts)),
+        user_error_loggers_(user_error_loggers),
         loaded_executable_(loaded_executable) {}
 
   void set_exit_handler(absl::AnyInvocable<void() &&> exit_handler) {
@@ -448,6 +460,21 @@ class ExecutionContext {
     return user_contexts;
   }
 
+  void AddUserErrorLogger(std::function<void(absl::Status)> error_logger) {
+    user_error_loggers_.push_back(error_logger);
+  }
+
+  const std::vector<std::function<void(absl::Status)>>& user_error_loggers()
+      const {
+    return user_error_loggers_;
+  }
+
+  void LogError(absl::Status status) {
+    for (auto& error_logger : user_error_loggers_) {
+      error_logger(status);
+    }
+  }
+
   enum class State {
     // The function is pushed to the stack, and ready for execution.
     kReady = 0,
@@ -486,6 +513,8 @@ class ExecutionContext {
   std::vector<std::unique_ptr<context_internal::UserContextBase>>
       user_contexts_;
 
+  std::vector<std::function<void(absl::Status)>> user_error_loggers_;
+
   const LoadedExecutable* loaded_executable_ = nullptr;
 
   friend class AsyncHandle;
