@@ -24,9 +24,9 @@ namespace tensorflow {
 //
 // TODO(jingdong): Merge this function with the InitializeTfrtSession() in
 // tfrt_session.h after we decouple TPU logic from TfrtSession.
-inline Status InitializeTfrtSession() {
+inline absl::Status InitializeTfrtSession() {
   SetDefaultLocalSessionImpl(LocalSessionImpl::kTfrtSession);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace tensorflow
