@@ -14,6 +14,7 @@ limitations under the License.
 ==============================================================================*/
 #include "tensorflow/core/tfrt/tfrt_session/tfrt_session.h"
 
+#include <cstdint>
 #include <memory>
 #include <string>
 #include <utility>
@@ -21,23 +22,31 @@ limitations under the License.
 
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
+#include "absl/memory/memory.h"
 #include "absl/time/time.h"
+#include "tensorflow/cc/framework/ops.h"
+#include "tensorflow/cc/framework/scope.h"
 #include "tensorflow/cc/ops/array_ops.h"
 #include "tensorflow/cc/ops/const_op.h"
 #include "tensorflow/cc/saved_model/reader.h"
+#include "xla/tsl/lib/core/status_test_util.h"
+#include "tensorflow/core/common_runtime/device_mgr.h"
 #include "tensorflow/core/framework/graph.pb.h"
 #include "tensorflow/core/framework/tensor_shape.h"
 #include "tensorflow/core/framework/tensor_testutil.h"
 #include "tensorflow/core/framework/types.pb.h"
-#include "tensorflow/core/lib/core/status_test_util.h"  // NOLINT(unused-includes): For TF_ASSERT_OK
 #include "tensorflow/core/platform/cpu_info.h"
 #include "tensorflow/core/platform/resource_loader.h"
+#include "tensorflow/core/platform/status.h"
 #include "tensorflow/core/platform/threadpool_options.h"
 #include "tensorflow/core/protobuf/config.pb.h"
 #include "tensorflow/core/protobuf/meta_graph.pb.h"
 #include "tensorflow/core/public/session.h"
+#include "tensorflow/core/public/session_options.h"
+#include "tensorflow/core/tfrt/runtime/runtime.h"
 #include "tensorflow/core/tfrt/saved_model/saved_model_testutil.h"
 #include "tensorflow/core/tfrt/utils/thread_pool.h"
+#include "tsl/platform/protobuf.h"
 
 namespace tensorflow {
 namespace {
@@ -72,7 +81,7 @@ class TfrtSessionTest : public ::testing::Test {
 
     // Initialize the session with a GraphDef.
     std::string saved_model_dir = GetDataDependencyFilepath(
-        "tensorflow/core/tfrt/saved_model/tests/toy_v1");
+        "tensorflow/core/tfrt/saved_model/tests/toy_v1/1");
 
     MetaGraphDef meta_graph_def;
     TF_ASSERT_OK(ReadMetaGraphDefFromSavedModel(saved_model_dir, {"serve"},
@@ -125,27 +134,83 @@ TEST_F(TfrtSessionTest, NoTargetNodes) {
 }
 
 TEST_F(TfrtSessionTest, RunOptions) {
+  SessionOptions options;
+  options.config.mutable_experimental()->set_use_tfrt(true);
+  auto* model_metadata =
+      options.config.mutable_experimental()->mutable_session_metadata();
+  model_metadata->set_name("toy_v1");
+  model_metadata->set_version(0);
+
+  auto session = absl::WrapUnique(NewSession(options));
+  ASSERT_TRUE(session != nullptr);
+
+  tensorflow::GraphDef graph_def;
+  ASSERT_TRUE(google::protobuf::TextFormat::ParseFromString(
+      R"pb(
+        node: {
+          name: "input"
+          op: "Placeholder"
+          attr: {
+            key: "dtype"
+            value: { type: DT_INT32 }
+          }
+        }
+        node: {
+          name: "sleep_seconds"
+          op: "Const"
+          attr: {
+            key: "dtype"
+            value: { type: DT_INT32 }
+          }
+          attr: {
+            key: "value"
+            value: {
+              tensor: {
+                tensor_shape: {}
+                dtype: DT_INT32
+                int_val: 2
+              }
+            }
+          }
+        }
+        node: {
+          name: "sleep"
+          op: "SleepIdentityOp"
+          input: "sleep_seconds:0"
+          input: "input:0"
+          attr: {
+            key: "T"
+            value: { type: DT_INT32 }
+          }
+        })pb"
+
+      ,
+      &graph_def));
+
+  TF_ASSERT_OK(session->Create(graph_def));
+
   std::vector<Tensor> outputs;
   // Test the Run() overload with RunOptions and RunMetadata
   RunMetadata run_metadata;
-  TF_ASSERT_OK(session_->Run(RunOptions{}, inputs_, output_tensor_names_,
-                             /*target_tensor_names=*/{}, &outputs,
-                             &run_metadata));
+  TF_ASSERT_OK(session->Run(
+      RunOptions{},
+      /*inputs=*/{{"input", test::AsTensor<int32_t>({1}, TensorShape{1})}},
+      /*output_tensor_names=*/{"sleep"},
+      /*target_tensor_names=*/{}, &outputs, &run_metadata));
 
-  ASSERT_EQ(outputs.size(), 3);
+  ASSERT_EQ(outputs.size(), 1);
 
   // Check output "r1".
-  test::ExpectEqual(outputs[0],
-                    test::AsTensor<int32_t>({6}, TensorShape{1, 1}));
+  test::ExpectEqual(outputs[0], test::AsTensor<int32_t>({1}, TensorShape{1}));
 
   // Test timeout.
   RunOptions run_options;
   run_options.set_timeout_in_ms(1);
-  // The "sleep" op will sleep for 1 second, so the Session::Run() call will
-  // time out.
-  auto status =
-      session_->Run(run_options, inputs_, output_tensor_names_,
-                    /*target_tensor_names=*/{"sleep"}, &outputs, &run_metadata);
+  auto status = session->Run(
+      run_options,
+      /*inputs=*/{{"input", test::AsTensor<int32_t>({1}, TensorShape{1})}},
+      /*output_tensor_names=*/{"sleep"},
+      /*target_tensor_names=*/{}, &outputs, &run_metadata);
 
   ASSERT_FALSE(status.ok());
   EXPECT_THAT(status.ToString(), ::testing::HasSubstr("Deadline exceeded"));
@@ -228,7 +293,7 @@ TEST_F(TfrtSessionTest, RunInCallerThreadSessionOptions) {
 
   // Initialize the session with a GraphDef.
   std::string saved_model_dir = GetDataDependencyFilepath(
-      "tensorflow/core/tfrt/saved_model/tests/toy_v1");
+      "tensorflow/core/tfrt/saved_model/tests/toy_v1/1");
 
   MetaGraphDef meta_graph_def;
   TF_ASSERT_OK(ReadMetaGraphDefFromSavedModel(saved_model_dir, {"serve"},
@@ -261,6 +326,38 @@ TEST_F(TfrtSessionTest, RunInCallerThreadRunOptions) {
                     test::AsTensor<int32_t>({6}, TensorShape{1, 1}));
 }
 
+TEST_F(TfrtSessionTest, DeviceManager) {
+  // Create a TfrtSession.
+  SessionOptions options;
+  options.config.mutable_experimental()->set_use_tfrt(true);
+  options.config.set_inter_op_parallelism_threads(-1);
+  session_.reset(NewSession(options));
+  ASSERT_TRUE(session_ != nullptr);
+
+  const DeviceMgr* device_manager;
+  TF_ASSERT_OK(session_->LocalDeviceManager(&device_manager));
+
+  // Initialize the session with a GraphDef.
+  std::string saved_model_dir = GetDataDependencyFilepath(
+      "tensorflow/core/tfrt/saved_model/tests/toy_v1/1");
+
+  MetaGraphDef meta_graph_def;
+  TF_ASSERT_OK(ReadMetaGraphDefFromSavedModel(saved_model_dir, {"serve"},
+                                              &meta_graph_def));
+
+  TF_ASSERT_OK(session_->Create(meta_graph_def.graph_def()));
+
+  // Run the init op. This also tests the case of no output tensors.
+  RunMetadata run_metadata;
+  TF_ASSERT_OK(session_->Run(
+      /*run_options=*/{}, /*inputs=*/{}, /*output_tensor_names=*/{},
+      /*target_tensor_names=*/{"init"}, nullptr, &run_metadata));
+
+  const DeviceMgr* device_manager_final;
+  TF_ASSERT_OK(session_->LocalDeviceManager(&device_manager_final));
+  ASSERT_EQ(device_manager, device_manager_final);
+}
+
 TEST_F(TfrtSessionTest, IntraOpThreadPoolOptionWarning) {
   // Create a TfrtSession.
   SessionOptions options;
@@ -302,6 +399,8 @@ TEST_F(TfrtSessionTest, Callable) {
   TF_ASSERT_OK(session_->ReleaseCallable(callable_handle));
 }
 
+TEST_F(TfrtSessionTest, Finalize) { TF_ASSERT_OK(session_->Finalize()); }
+
 TEST_F(TfrtSessionTest, WithTargetNodes) {
   std::vector<Tensor> outputs;
   TF_ASSERT_OK(session_->Run(inputs_, output_tensor_names_, target_node_names_,
@@ -334,7 +433,7 @@ TEST_F(TfrtSessionTest, CreateWithEmptyGraphIsNoop) {
 
   // Create agian with an unempty GraphDef.
   std::string saved_model_dir = GetDataDependencyFilepath(
-      "tensorflow/core/tfrt/saved_model/tests/toy_v1");
+      "tensorflow/core/tfrt/saved_model/tests/toy_v1/1");
 
   MetaGraphDef meta_graph_def;
   TF_ASSERT_OK(ReadMetaGraphDefFromSavedModel(saved_model_dir, {"serve"},
@@ -346,7 +445,7 @@ TEST_F(TfrtSessionTest, CreateWithEmptyGraphIsNoop) {
 TEST_F(TfrtSessionTest, CreateAgainError) {
   // On a created session, create agian with a GraphDef.
   std::string saved_model_dir = GetDataDependencyFilepath(
-      "tensorflow/core/tfrt/saved_model/tests/toy_v1");
+      "tensorflow/core/tfrt/saved_model/tests/toy_v1/1");
 
   MetaGraphDef meta_graph_def;
   TF_ASSERT_OK(ReadMetaGraphDefFromSavedModel(saved_model_dir, {"serve"},
@@ -372,7 +471,7 @@ TEST_F(TfrtSessionTest, CreateAfterCloseError) {
 
   // Create the session with a GraphDef.
   std::string saved_model_dir = GetDataDependencyFilepath(
-      "tensorflow/core/tfrt/saved_model/tests/toy_v1");
+      "tensorflow/core/tfrt/saved_model/tests/toy_v1/1");
 
   MetaGraphDef meta_graph_def;
   TF_ASSERT_OK(ReadMetaGraphDefFromSavedModel(saved_model_dir, {"serve"},
@@ -394,7 +493,7 @@ TEST_F(TfrtSessionTest, ExtendWhenNotCreated) {
 
   // Extend the session with a GraphDef.
   std::string saved_model_dir = GetDataDependencyFilepath(
-      "tensorflow/core/tfrt/saved_model/tests/toy_v1");
+      "tensorflow/core/tfrt/saved_model/tests/toy_v1/1");
 
   MetaGraphDef meta_graph_def;
   TF_ASSERT_OK(ReadMetaGraphDefFromSavedModel(saved_model_dir, {"serve"},
@@ -529,7 +628,7 @@ TEST_F(TfrtSessionTest, ExtendAfterCloseError) {
 
   // Extend the session with a GraphDef.
   std::string saved_model_dir = GetDataDependencyFilepath(
-      "tensorflow/core/tfrt/saved_model/tests/toy_v1");
+      "tensorflow/core/tfrt/saved_model/tests/toy_v1/1");
 
   MetaGraphDef meta_graph_def;
   TF_ASSERT_OK(ReadMetaGraphDefFromSavedModel(saved_model_dir, {"serve"},
@@ -555,6 +654,27 @@ TEST_F(TfrtSessionTest, RunAfterCloseError) {
               ::testing::HasSubstr("Session has been closed."));
 }
 
+TEST_F(TfrtSessionTest, InitializeTwiceCrashes) {
+  TfrtSessionOptions options;
+  auto second_initialize = [](TfrtSessionOptions options) {
+    auto status = InitializeTfrtSession(options);
+    TF_ASSERT_OK(status);  // Crashes before getting here.
+  };
+  ASSERT_DEBUG_DEATH(second_initialize(options), "");
+}
+
+TEST_F(TfrtSessionTest, GetRuntime) {
+  auto runtime = TfrtSessionFactory::GetRuntime();
+  EXPECT_NE(runtime, nullptr);
+}
+
+TEST_F(TfrtSessionTest, RegisterTwiceCrashes) {
+  TfrtSessionFactory::RegisterInitializer(
+      [](tfrt_stub::Runtime*) { return absl::OkStatus(); });
+  ASSERT_DEBUG_DEATH(TfrtSessionFactory::RegisterInitializer(
+                         [](tfrt_stub::Runtime*) { return absl::OkStatus(); }),
+                     "");
+}
 }  // namespace
 }  // namespace tensorflow
 
