@@ -16,26 +16,34 @@ limitations under the License.
 #include "tensorflow/core/tpu/graph_rewrite/combine_tpu_embedding_load_retrieve_pass.h"
 
 #include <algorithm>
-#include <iterator>
+#include <array>
 #include <string>
 #include <utility>
 #include <vector>
 
 #include "absl/container/flat_hash_map.h"
 #include "absl/container/flat_hash_set.h"
+#include "absl/log/check.h"
+#include "absl/log/log.h"
+#include "absl/status/status.h"
+#include "absl/strings/str_cat.h"
 #include "absl/strings/str_format.h"
-#include "tensorflow/compiler/xla/status_macros.h"
+#include "absl/strings/string_view.h"
+#include "xla/status_macros.h"
+#include "xla/tsl/platform/errors.h"
+#include "xla/tsl/platform/logging.h"  // IWYU pragma: keep
 #include "tensorflow/core/common_runtime/optimization_registry.h"
 #include "tensorflow/core/framework/node_def.pb.h"
 #include "tensorflow/core/framework/node_def_util.h"
 #include "tensorflow/core/framework/tensor.pb.h"
 #include "tensorflow/core/framework/tensor_util.h"
+#include "tensorflow/core/framework/types.pb.h"
 #include "tensorflow/core/graph/graph.h"
 #include "tensorflow/core/graph/node_builder.h"
-#include "tensorflow/core/lib/core/errors.h"
-#include "tensorflow/core/lib/core/status.h"
 #include "tensorflow/core/lib/gtl/map_util.h"
-#include "tensorflow/core/platform/stream_executor.h"
+#include "tensorflow/core/platform/errors.h"
+#include "tensorflow/core/platform/status.h"
+#include "tensorflow/core/protobuf/tpu/optimization_parameters.pb.h"
 #include "tensorflow/core/protobuf/tpu/tpu_embedding_configuration.pb.h"
 #include "tensorflow/core/tpu/graph_rewrite/tpu_embedding_rewrite_pass_utils.h"
 #include "tensorflow/core/tpu/ops/tpu_embedding_ops.h"
@@ -61,7 +69,7 @@ absl::flat_hash_set<std::string> GetLoadRetrieveNodeNames() {
 }
 
 // Gets TPUEmbeddingConfiguration proto from embedding ops.
-Status GetTPUEmbeddingConfiguration(
+absl::Status GetTPUEmbeddingConfiguration(
     Graph* graph,
     tensorflow::tpu::TPUEmbeddingConfiguration* tpu_embedding_config,
     std::string* tpu_embedding_config_str) {
@@ -99,11 +107,11 @@ Status GetTPUEmbeddingConfiguration(
   if (!have_config) {
     return errors::InvalidArgument("No TPU embedding config provided");
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Validates that all of the table names are distinct and non-empty.
-Status ValidateEmbeddingTableNames(
+absl::Status ValidateEmbeddingTableNames(
     const tensorflow::tpu::TPUEmbeddingConfiguration& tpu_embedding_config) {
   // Map from table names to first occurrences.
   TableNameToIntegerMap table_name_map;
@@ -122,7 +130,7 @@ Status ValidateEmbeddingTableNames(
                           table_name_map[name], table_id, name.c_str()));
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Gets single-table load-TPUEmbedding-parameter nodes in the graph.
@@ -182,7 +190,7 @@ absl::flat_hash_set<Node*> GetRetrieveNodes(Graph* graph) {
 //
 // Returns: Status (OK if successful; otherwise, an error status).
 //
-Status GetLoadOrRetrieveNodesByTable(
+absl::Status GetLoadOrRetrieveNodesByTable(
     const absl::flat_hash_set<Node*>& candidate_nodes,
     const tensorflow::tpu::TPUEmbeddingConfiguration& tpu_embedding_config,
     const TableNameToIntegerMap& table_name_to_id_map,
@@ -260,7 +268,7 @@ Status GetLoadOrRetrieveNodesByTable(
           tpu_embedding_config.table_descriptor(table_id).name(), table_id));
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Pair of a node and an input or output number used to record edge endpoints.
@@ -274,7 +282,7 @@ using LoadCombinedParametersType =
 
 // Computes an array of ports containing the source of each table/parameter
 // combination. Fills in any unused ports with {nullptr, 0}.
-Status CombinePerTableParametersForLoad(
+absl::Status CombinePerTableParametersForLoad(
     const absl::flat_hash_set<Node*>& load_nodes,
     std::vector<Node*>* nodes_per_table,
     std::vector<bool>* is_debug_load_retrieve_node,
@@ -297,7 +305,7 @@ Status CombinePerTableParametersForLoad(
                                  .optimization_parameters();
 
     std::vector<tpu::StateVariableSpecification> state_variable_specs;
-    Status status = tpu::GetOptimizationAlgorithmStateVariables(
+    absl::Status status = tpu::GetOptimizationAlgorithmStateVariables(
         opt_params, &state_variable_specs);
 
     if (!status.ok()) {
@@ -344,7 +352,7 @@ Status CombinePerTableParametersForLoad(
       }
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Removes edges between individual load/retrieve nodes that are added by
@@ -372,14 +380,14 @@ void RemoveEdgesBetweenIndividualNodes(
 
 }  // namespace
 
-Status CombineTPUEmbeddingLoadRetrievePass::Run(
+absl::Status CombineTPUEmbeddingLoadRetrievePass::Run(
     const GraphOptimizationPassOptions& options) {
   VLOG(2) << "Starting CombineTPUEmbeddingLoadRetrievePass";
   Graph* graph = options.graph->get();
 
   tensorflow::tpu::TPUEmbeddingConfiguration tpu_embedding_config;
   std::string tpu_embedding_config_str;
-  const Status tpu_embedding_config_error = GetTPUEmbeddingConfiguration(
+  const absl::Status tpu_embedding_config_error = GetTPUEmbeddingConfiguration(
       graph, &tpu_embedding_config, &tpu_embedding_config_str);
   TF_RETURN_IF_ERROR(ValidateEmbeddingTableNames(tpu_embedding_config));
 
@@ -738,7 +746,7 @@ Status CombineTPUEmbeddingLoadRetrievePass::Run(
   VLOG(2) << "Generated " << num_combined_nodes_added
           << " combined load or retrieve nodes.";
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace tensorflow
