@@ -108,19 +108,31 @@ limitations under the License.
 #ifndef TENSORFLOW_CORE_TPU_GRAPH_REWRITE_DISTRIBUTED_TPU_REWRITE_PASS_H_
 #define TENSORFLOW_CORE_TPU_GRAPH_REWRITE_DISTRIBUTED_TPU_REWRITE_PASS_H_
 
+#include <cstdint>
+#include <map>
+#include <memory>
+#include <set>
 #include <string>
+#include <unordered_map>
 #include <vector>
 
 #include "absl/container/flat_hash_map.h"
 #include "absl/container/node_hash_map.h"
+#include "absl/status/status.h"
 #include "absl/types/span.h"
 #include "tensorflow/compiler/jit/shape_inference.h"
-#include "tensorflow/compiler/xla/service/computation_placer.h"
-#include "tensorflow/compiler/xla/stream_executor/tpu/tpu_topology.h"
+#include "xla/service/computation_placer.h"
+#include "xla/stream_executor/tpu/tpu_topology.h"
+#include "xla/xla_data.pb.h"
 #include "tensorflow/core/common_runtime/optimization_registry.h"
+#include "tensorflow/core/framework/attr_value.pb.h"
 #include "tensorflow/core/framework/function.h"
+#include "tensorflow/core/framework/node_def_util.h"
+#include "tensorflow/core/framework/types.h"
+#include "tensorflow/core/framework/types.pb.h"
 #include "tensorflow/core/graph/graph.h"
-#include "tensorflow/core/platform/env.h"
+#include "tensorflow/core/platform/status.h"
+#include "tensorflow/core/util/device_name_utils.h"
 
 namespace tensorflow {
 
@@ -136,7 +148,7 @@ class DistributedTPURewritePass : public GraphOptimizationPass {
       bool enable_automatic_model_parallelism, bool enable_xla_param_broadcast,
       bool enable_multicore_locking, bool use_nd_sharding_ops);
 
-  Status Run(const GraphOptimizationPassOptions& options) override;
+  absl::Status Run(const GraphOptimizationPassOptions& options) override;
 
   // The following methods are public only for the use of unit tests.
 
@@ -145,7 +157,7 @@ class DistributedTPURewritePass : public GraphOptimizationPass {
   // information, and provide common APIs over them.
   class ParameterInfo {
    public:
-    ParameterInfo() {}
+    ParameterInfo() = default;
     ParameterInfo(int64_t num_replicas, int64_t num_per_replica_args,
                   int64_t num_distributed_args, int64_t num_broadcast_args,
                   int64_t num_variables, int64_t num_guaranteed_constants,
@@ -249,7 +261,8 @@ class DistributedTPURewritePass : public GraphOptimizationPass {
 
   // Mapping from TPUReplicate cluster name to tpu device names. Value is a
   // mapping from [replica][core] to a TF device name.
-  typedef absl::flat_hash_map<string, std::vector<std::vector<string>>>
+  typedef absl::flat_hash_map<std::string,
+                              std::vector<std::vector<std::string>>>
       TPUReplicateDeviceNamesMapping;
 
   // Determines which devices to use to run the computation.
@@ -271,12 +284,12 @@ class DistributedTPURewritePass : public GraphOptimizationPass {
   // * xla_device_assignment: a mapping from [replica][core] to a linearized TPU
   //   coordinate.
   // TODO(phawkins): change tf_device_assignment to an xla::Array2D.
-  static Status BuildDeviceAssignment(
+  static absl::Status BuildDeviceAssignment(
       const tpu::TpuTopologyExternal& topology, int num_tpus_per_task,
       const std::vector<std::vector<Device*>>& tpu_devices, int num_replicas,
-      int num_cores_per_replica, const string& topology_attr,
+      int num_cores_per_replica, const std::string& topology_attr,
       absl::Span<const int> device_assignment_attr,
-      std::vector<std::vector<string>>* tf_device_assignment,
+      std::vector<std::vector<std::string>>* tf_device_assignment,
       std::vector<int>* devices_to_lock,
       std::unique_ptr<xla::DeviceAssignment>* xla_device_assignment);
 
@@ -284,11 +297,10 @@ class DistributedTPURewritePass : public GraphOptimizationPass {
   // `node`. `flr` is a FunctionLibraryRuntime to use when
   // instantiating the function body. Sets `*arg_types` and
   // `*retval_types` to the argument/return types of the function.
-  static Status GetComputationForTPUReplicateOp(const NameAttrList& function,
-                                                FunctionLibraryRuntime* flr,
-                                                Graph* computation,
-                                                DataTypeVector* arg_types,
-                                                DataTypeVector* retval_types);
+  static absl::Status GetComputationForTPUReplicateOp(
+      const NameAttrList& function, FunctionLibraryRuntime* flr,
+      Graph* computation, DataTypeVector* arg_types,
+      DataTypeVector* retval_types);
 
   // Returns the shapes of the argument tensors and return values of the
   // TPUReplicate operator `node` using the _output_shapes,
@@ -300,7 +312,7 @@ class DistributedTPURewritePass : public GraphOptimizationPass {
   // Returns an error if the input shapes to `node` are not statically known.
   // Also verifies that all replicas have identical input shapes for their
   // per-replica inputs.
-  static Status GetArgAndRetvalShapes(
+  static absl::Status GetArgAndRetvalShapes(
       const GraphShapeInfo& shape_info, const Node& node,
       const ParameterInfo& params_info, std::vector<InferredShape>* arg_shapes,
       std::vector<InferredShape>* retval_shapes);
@@ -311,7 +323,7 @@ class DistributedTPURewritePass : public GraphOptimizationPass {
   // argument/retval number.
   // `arg_fast_mem` is vector of fast_mem indication which is indexed by
   // argument number.
-  static Status AssignArgsAndRetvalsToCores(
+  static absl::Status AssignArgsAndRetvalsToCores(
       int num_cores_per_replica, const ParameterInfo& params_info,
       const DataTypeVector& arg_types,
       const std::vector<InferredShape>& arg_shapes,
@@ -334,19 +346,19 @@ class DistributedTPURewritePass : public GraphOptimizationPass {
     // the output of 'variable', which is always DT_RESOURCE.
     DataType dtype;
   };
-  static Status FindVariableInputs(const Node& node,
-                                   const NameRangeMap& input_range_map,
-                                   std::vector<VariableInput>* variables);
+  static absl::Status FindVariableInputs(const Node& node,
+                                         const NameRangeMap& input_range_map,
+                                         std::vector<VariableInput>* variables);
 
   // Populates '*guaranteed_constants' with the "guaranteed_constants" inputs
   // to 'node'.
-  static Status FindGuaranteedConstantInputs(
+  static absl::Status FindGuaranteedConstantInputs(
       const Node& node, const NameRangeMap& input_range_map,
       std::vector<Node*>* guaranteed_constants);
 
   // Builds Shape nodes that compute the shapes of arguments whose shapes are
   // not statically known.
-  static Status BuildDynamicShapeNodes(
+  static absl::Status BuildDynamicShapeNodes(
       const Node& replicate_node, const std::vector<InferredShape>& arg_shapes,
       const ParameterInfo& params_info,
       const std::vector<Node*>& variable_reads, Graph* graph,
@@ -358,34 +370,34 @@ class DistributedTPURewritePass : public GraphOptimizationPass {
   // the `nodes` must correspond to the computations assigned to TPU:0,
   // TPU:1, ... in order since XLA hard-codes the chip IDs in the generated
   // executables.
-  static Status BuildCompileNode(
+  static absl::Status BuildCompileNode(
       const Node* replicate_node, const NameAttrList& function,
-      uint64 library_fingerprint, const ParameterInfo& params_info,
+      uint64_t library_fingerprint, const ParameterInfo& params_info,
       const std::vector<InferredShape>& arg_shapes,
       const DataTypeVector& arg_types,
       const std::vector<Node*>& guaranteed_constant_nodes,
-      const string& session_handle,
+      const std::string& session_handle,
       const std::vector<::xla::OpSharding>& arg_sharding,
       const std::vector<bool>& arg_fast_mem,
       const std::vector<std::string>& arg_names,
       const std::vector<::xla::OpSharding>& retval_sharding,
-      int num_cores_per_replica, const string& compile_device,
+      int num_cores_per_replica, const std::string& compile_device,
       const xla::DeviceAssignment* xla_device_assignment,
       const std::vector<Node*>& dynamic_shape_nodes, Graph* graph,
       Node** compile_node, int64_t autotuner_thresh);
 
   // Builds a TPUCompileSucceededAssert node that verifies that compilation
   // succeeded and replaces the TPUCompilationStatus node in the graph.
-  static Status BuildCompilationStatusReturnNodes(
+  static absl::Status BuildCompilationStatusReturnNodes(
       Node* replicate_node, Node* compile_node,
       absl::Span<const int> devices_to_lock, Node** control_after_compilation,
       Node** multilock_acquire, Graph* graph);
 
   // Builds ReadVariableOp nodes that read `variables`, with a control
   // edges that ensure they happen after `control_predecessor`.
-  static Status BuildVariableReads(absl::Span<const VariableInput> variables,
-                                   Node* control_predecessor, Graph* graph,
-                                   std::vector<Node*>* variable_reads);
+  static absl::Status BuildVariableReads(
+      absl::Span<const VariableInput> variables, Node* control_predecessor,
+      Graph* graph, std::vector<Node*>* variable_reads);
 
   // Returns true if graph or functions contain resource write op, otherwise
   // return false.
@@ -407,7 +419,7 @@ class DistributedTPURewritePass : public GraphOptimizationPass {
   // Builds AssignVariableOp nodes that write `variables` with the values from
   // `variable_writes`, with control edges that ensure the writes happen before
   // `control_successor`.
-  static Status BuildVariableWrites(
+  static absl::Status BuildVariableWrites(
       absl::Span<const VariableInput> variables, Node* control_successor,
       absl::Span<const VariableWrite> variable_writes, Graph* graph);
 
@@ -436,7 +448,7 @@ class DistributedTPURewritePass : public GraphOptimizationPass {
   //   `control_predecessor` and another control edge to `control_successor`.
   // Populates '*variable_writes' with information about variable values to
   // write back.
-  static Status BuildExecuteNodes(
+  static absl::Status BuildExecuteNodes(
       const ParameterInfo& params_info, int num_tasks,
       int num_cores_per_replica, const Node& replicate_node,
       const std::vector<std::string>& arg_names,
@@ -445,7 +457,7 @@ class DistributedTPURewritePass : public GraphOptimizationPass {
       const DataTypeVector& retval_types,
       const std::vector<::xla::OpSharding>& arg_shardings,
       const std::vector<::xla::OpSharding>& retval_shardings,
-      const std::vector<std::vector<string>>& tpu_device_names,
+      const std::vector<std::vector<std::string>>& tpu_device_names,
       Node* compile_node, const std::vector<Node*>& variable_reads,
       Node* control_predecessor, Node* control_successor,
       Node* multilock_acquire, std::vector<VariableWrite>* variable_writes,
@@ -459,9 +471,9 @@ class DistributedTPURewritePass : public GraphOptimizationPass {
   // host
   // * transfer nodes in the original graph.
   // * `graph` is the graph being rewritten.
-  static Status ConnectHostComputeNodes(Node* compile_node,
-                                        Node* key_placeholder_node,
-                                        Graph* graph);
+  static absl::Status ConnectHostComputeNodes(Node* compile_node,
+                                              Node* key_placeholder_node,
+                                              Graph* graph);
 
   // Map from a Node in an outside_compilation cluster in the original graph to
   // the list of Nodes, one for each replica, that it is expanded into during
@@ -470,15 +482,15 @@ class DistributedTPURewritePass : public GraphOptimizationPass {
 
   // Map from the name of an outside_compilation cluster to the model-parallel
   // core index that the HostCompute Op should be placed on in that cluster.
-  typedef std::map<string, int> HostComputeCoreMap;
+  typedef std::map<std::string, int> HostComputeCoreMap;
 
   // Map from the name of an outside_compilation cluster to the list of Nodes
   // that should run on the host for that cluster.
-  typedef std::map<string, std::vector<Node*>> OutsideCompilationNodeMap;
+  typedef std::map<std::string, std::vector<Node*>> OutsideCompilationNodeMap;
 
   // Copies the outside_compilation nodes in a cluster to create replica
   // replica_index.
-  static Status CopyOutsideCompilationNodes(
+  static absl::Status CopyOutsideCompilationNodes(
       int replica_index, const std::vector<Node*>& outside_compilation_nodes,
       const DeviceNameUtils::ParsedName& tpu_device,
       const DeviceNameUtils::ParsedName& partial_device,
@@ -486,66 +498,66 @@ class DistributedTPURewritePass : public GraphOptimizationPass {
 
   // Replicates all the nodes in outside_compilation clusters in a compiled
   // computation.
-  static Status ReplicateOutsideCompilationNodes(
-      const std::vector<std::vector<string>>& tf_device_assignment,
+  static absl::Status ReplicateOutsideCompilationNodes(
+      const std::vector<std::vector<std::string>>& tf_device_assignment,
       const HostComputeCoreMap& host_compute_core,
       const OutsideCompilationNodeMap& outside_compilation_nodes,
       NodeToNodeReplicasMap* node_images, Graph* graph);
 
   // Lifts the edges between original outside_compilation nodes in a cluster
   // onto their replicas.
-  static Status CopyOutsideCompilationEdges(
+  static absl::Status CopyOutsideCompilationEdges(
       const std::vector<Node*>& outside_compilation_nodes,
       const NodeToNodeReplicasMap& node_images,
-      const std::unordered_map<string, Node*> outside_compilation_inputs,
+      std::unordered_map<std::string, Node*> outside_compilation_inputs,
       Graph* graph);
 
   // Lifts all the edges in outside_compilation clusters in a compiled
   // computation to their replicas.
-  static Status ReplicateOutsideCompilationEdges(
+  static absl::Status ReplicateOutsideCompilationEdges(
       const OutsideCompilationNodeMap& outside_compilation_nodes,
       const NodeToNodeReplicasMap& node_images,
-      const std::unordered_map<string, Node*> outside_compilation_inputs,
+      std::unordered_map<std::string, Node*> outside_compilation_inputs,
       Graph* graph);
 
   // Removes all the original outside_compilation nodes from the graph,
   // following replication.
-  static Status RemoveOutsideCompilationNodes(
+  static absl::Status RemoveOutsideCompilationNodes(
       const NodeToNodeReplicasMap& node_images, Graph* graph);
 
   // Lowers outside compilation functional nodes (If/While/function call).
   // Otherwise, when we have multiple workers, device placer will not be able to
   // place nodes if outside compilation has DT_RESOURCE inputs (e.g. a
   // DT_RESOURCE input fed into multiple While nodes on different devices).
-  static Status LowerOutsideCompilationFunctionalNodes(
+  static absl::Status LowerOutsideCompilationFunctionalNodes(
       Graph* g, FunctionLibraryDefinition& flib_def,
       const TPUReplicateDeviceNamesMapping& tpu_replicate_device_names_mapping);
 
   // Parses the 'host_compute_core' attribute on replicate_node to get the
   // replicated core id of each outside_compilation cluster.
-  static Status ParseHostComputeCores(
+  static absl::Status ParseHostComputeCores(
       const Node& replicate_node,
       const OutsideCompilationNodeMap& outside_compilation_nodes,
       HostComputeCoreMap* host_compute_core);
 
   // Gets the physical topology information about the TPU system.
-  static Status GetDeviceTopology(
+  static absl::Status GetDeviceTopology(
       const DeviceSet& device_set, const Node& replicate_node,
       int* num_replicas, int* num_cores_per_replica, int* num_tasks,
-      std::vector<std::vector<string>>* tf_device_assignment,
+      std::vector<std::vector<std::string>>* tf_device_assignment,
       std::vector<int>* devices_to_lock,
       std::unique_ptr<xla::DeviceAssignment>* xla_device_assignment,
-      string* tpu_compilation_device);
+      std::string* tpu_compilation_device);
 
   // Gets the types of args, retvals, and parameters.
-  static Status GetIOTypes(
+  static absl::Status GetIOTypes(
       int num_replicas, const Node& replicate_node, FunctionLibraryRuntime* flr,
       Graph* graph, NameRangeMap* input_name_map, const NameAttrList** function,
       std::unique_ptr<Graph>* computation, DataTypeVector* arg_types,
       DataTypeVector* retval_types, ParameterInfo* params_info);
 
   // Find known constants and deals with variable reads.
-  static Status DealWithConstantsAndVariables(
+  static absl::Status DealWithConstantsAndVariables(
       const Node& replicate_node, const NameRangeMap& input_name_map,
       Graph* graph, Node* host_transfer_sequencer, Node* control_before,
       Node* control_after, absl::Span<const VariableInput> variable_nodes,
@@ -553,15 +565,14 @@ class DistributedTPURewritePass : public GraphOptimizationPass {
       std::vector<Node*>* variable_reads);
 
   // Adds NoOp nodes for sequencing computation and variable reads/writes.
-  static Status BuildSequencingNodes(const string& tpu_compilation_device,
-                                     const Node& replicate_node, Graph* graph,
-                                     Node** host_transfer_sequencer,
-                                     Node** control_before,
-                                     Node** control_after);
+  static absl::Status BuildSequencingNodes(
+      const std::string& tpu_compilation_device, const Node& replicate_node,
+      Graph* graph, Node** host_transfer_sequencer, Node** control_before,
+      Node** control_after);
 
   // Performs the pass's rewrite on a TPUReplicate node `node`.
-  static Status RewriteTPUReplicateNode(
-      const string& session_handle, const DeviceSet& device_set,
+  static absl::Status RewriteTPUReplicateNode(
+      const std::string& session_handle, const DeviceSet& device_set,
       Node* replicate_node, FunctionLibraryDefinition* flib_def,
       FunctionLibraryRuntime* flr, Node* host_compute_key_placeholder_node,
       const OutsideCompilationNodeMap& outside_compilation_nodes,
@@ -575,20 +586,20 @@ class DistributedTPURewritePass : public GraphOptimizationPass {
   // node is inside a while loop, then model weight variables can be sharded
   // in XLA preferred layout and then unsharded only at the very last iteration
   // to reduce the number of all_gather.
-  static Status PerformHostTrainingLoopOptimization(
+  static absl::Status PerformHostTrainingLoopOptimization(
       Graph* graph, FunctionLibraryDefinition* flib_def,
       FunctionLibraryRuntime* flr);
 
   // Heuristically place some nodes with unassigned devices on TPUs for
   // performance reasons.
-  static Status PlaceUnassignedDeviceNodesOnTPUIfPossible(Graph* graph);
+  static absl::Status PlaceUnassignedDeviceNodesOnTPUIfPossible(Graph* graph);
 
   // Updates the head and tail outside compiled nodes so that nodes have the
   // correct device and removes the replication and outside compilation
   // attributes so that these nodes do not trigger further graph optimization
   // passes.
-  static Status UpdateHeadTailOutsideCompilation(
-      const std::vector<std::vector<string>>& tf_device_assignment,
+  static absl::Status UpdateHeadTailOutsideCompilation(
+      const std::vector<std::vector<std::string>>& tf_device_assignment,
       const std::vector<Node*>& head_tail_outside_compilation_nodes);
 
  private:
@@ -600,7 +611,7 @@ class DistributedTPURewritePass : public GraphOptimizationPass {
   static bool enable_xla_param_broadcast_;
   static bool enable_multicore_locking_;
   static bool use_nd_sharding_ops_;
-  Status InternalRun(const GraphOptimizationPassOptions& options);
+  absl::Status InternalRun(const GraphOptimizationPassOptions& options);
 };
 
 }  // namespace tensorflow
