@@ -15,35 +15,44 @@ limitations under the License.
 
 #include "tensorflow/core/tpu/kernels/infeed_ops.h"
 
-#include <algorithm>
+#include <cstdint>
 #include <deque>
 #include <iterator>
 #include <memory>
 #include <utility>
 #include <vector>
 
-#include "tensorflow/compiler/jit/xla_device.h"
+#include "absl/status/status.h"
+#include "absl/strings/str_cat.h"
 #include "tensorflow/compiler/tf2xla/literal_util.h"
 #include "tensorflow/compiler/tf2xla/shape_util.h"
-#include "tensorflow/compiler/xla/stream_executor/multi_platform_manager.h"
-#include "tensorflow/compiler/xla/stream_executor/tpu/c_api_conversions.h"
-#include "tensorflow/compiler/xla/stream_executor/tpu/tpu_api.h"
-#include "tensorflow/compiler/xla/stream_executor/tpu/tpu_transfer_manager.h"
-#include "tensorflow/compiler/xla/stream_executor/tpu/tpu_transfer_manager_interface.h"
-#include "tensorflow/compiler/xla/util.h"
-#include "tensorflow/core/common_runtime/dma_helper.h"
+#include "xla/layout.h"
+#include "xla/layout_util.h"
+#include "xla/literal.h"
+#include "xla/shape.h"
+#include "xla/shape_util.h"
+#include "xla/stream_executor/tpu/c_api_conversions.h"
+#include "xla/stream_executor/tpu/c_api_decl.h"
+#include "xla/stream_executor/tpu/noncopyable_buffer.h"
+#include "xla/stream_executor/tpu/tpu_executor_api.h"
+#include "xla/stream_executor/tpu/tpu_transfer_manager_interface.h"
+#include "xla/tsl/platform/errors.h"
+#include "xla/tsl/platform/logging.h"  // IWYU pragma: keep
+#include "xla/tsl/platform/statusor.h"
 #include "tensorflow/core/framework/allocator.h"
-#include "tensorflow/core/framework/dataset.h"
 #include "tensorflow/core/framework/function.h"
-#include "tensorflow/core/framework/function_handle_cache.h"
-#include "tensorflow/core/framework/op.h"
 #include "tensorflow/core/framework/op_kernel.h"
-#include "tensorflow/core/framework/shape_inference.h"
+#include "tensorflow/core/framework/op_requires.h"
 #include "tensorflow/core/framework/tensor.h"
+#include "tensorflow/core/framework/tensor_shape.h"
+#include "tensorflow/core/framework/types.h"
+#include "tensorflow/core/framework/types.pb.h"
 #include "tensorflow/core/framework/variant.h"
-#include "tensorflow/core/framework/variant_encode_decode.h"
+#include "tensorflow/core/framework/variant_encode_decode.h"  // IWYU pragma: keep
 #include "tensorflow/core/framework/variant_tensor_data.h"
 #include "tensorflow/core/kernels/transpose_functor.h"
+#include "tensorflow/core/platform/status.h"
+#include "tensorflow/core/platform/types.h"
 #include "tensorflow/core/profiler/lib/traceme.h"
 #include "tensorflow/core/tpu/kernels/transfer_ops.h"
 #include "tensorflow/core/tpu/tpu_defs.h"
@@ -76,12 +85,12 @@ xla::Shape GetTPUInfeedLayout(const xla::Shape& shape) {
 // to obtain a XLA literal for the host tensor laid out as the given layout. The
 // returned tensor is normalized to the dim0major layout -- F32[10,20,30]{2,0,1}
 // is returned as F32[20,10,30]{2,1,0}.
-xla::StatusOr<Tensor> TransposeTensor(OpKernelContext* ctx,
-                                      const Tensor& input_tensor,
-                                      const xla::Shape& xla_shape) {
-  profiler::TraceMe trace_me("TransposeTensor", /*level=*/2);
-  const int64_t rank = xla_shape.rank();
-  std::vector<int32> permutation(rank);
+absl::StatusOr<Tensor> TransposeTensor(OpKernelContext* ctx,
+                                       const Tensor& input_tensor,
+                                       const xla::Shape& xla_shape) {
+  tsl::profiler::TraceMe trace_me("TransposeTensor", /*level=*/2);
+  const int64_t rank = xla_shape.dimensions().size();
+  std::vector<int32_t> permutation(rank);
   std::vector<int64_t> transposed_shapes(rank);
   for (int64_t i = 0; i < rank; ++i) {
     permutation[i] = xla_shape.layout().minor_to_major(rank - 1 - i);
@@ -115,9 +124,9 @@ xla::StatusOr<Tensor> TransposeTensor(OpKernelContext* ctx,
   return transposed_tensor;
 }
 
-xla::StatusOr<bool> GetLayoutOverride(OpKernelConstruction* ctx,
-                                      const char* attrn_name,
-                                      std::vector<int64_t>* minor_to_major) {
+absl::StatusOr<bool> GetLayoutOverride(OpKernelConstruction* ctx,
+                                       const char* attrn_name,
+                                       std::vector<int64_t>* minor_to_major) {
   if (!ctx->HasAttr(attrn_name)) {
     return false;
   }
@@ -125,10 +134,10 @@ xla::StatusOr<bool> GetLayoutOverride(OpKernelConstruction* ctx,
   return !minor_to_major->empty();
 }
 
-Status GetInfeedShapeWithLayout(OpKernelConstruction* ctx,
-                                const char* attrn_name,
-                                const xla::Shape& input_shape,
-                                xla::Shape* output_shape) {
+absl::Status GetInfeedShapeWithLayout(OpKernelConstruction* ctx,
+                                      const char* attrn_name,
+                                      const xla::Shape& input_shape,
+                                      xla::Shape* output_shape) {
   std::vector<int64_t> minor_to_major;
   TF_ASSIGN_OR_RETURN(bool has_override,
                       GetLayoutOverride(ctx, attrn_name, &minor_to_major));
@@ -145,7 +154,7 @@ Status GetInfeedShapeWithLayout(OpKernelConstruction* ctx,
       *output_shape->mutable_layout() =
           GetTPUInfeedLayout(*output_shape).layout();
     }
-    return OkStatus();
+    return absl::OkStatus();
   }
 
   auto layout_func = [](const xla::Shape& shape) -> xla::Layout {
@@ -196,13 +205,12 @@ struct LinearizedBuffersWrapper {
   std::vector<tensorflow::Tensor> tensors;
 };
 
-Status AutoTransposeAndLinearize(OpKernelContext* ctx,
-                                 const Tensor& input_tensor,
-                                 const xla::Shape& shape,
-                                 LinearizerBufferList* linearized_buffers,
-                                 std::vector<Tensor>* saved_input_tensors) {
+absl::Status AutoTransposeAndLinearize(
+    OpKernelContext* ctx, const Tensor& input_tensor, const xla::Shape& shape,
+    LinearizerBufferList* linearized_buffers,
+    std::vector<Tensor>* saved_input_tensors) {
   const Tensor* tensor = &input_tensor;
-  // If the given layout is not in dim0major layout, tranposes the tensor.
+  // If the given layout is not in dim0major layout, transposes the tensor.
   bool has_transposed = false;
   Tensor transposed_tensor;
   if (!xla::LayoutUtil::IsMonotonicWithDim0Major(shape.layout())) {
@@ -216,9 +224,11 @@ Status AutoTransposeAndLinearize(OpKernelContext* ctx,
   xla::BorrowingLiteral literal;
   TF_RETURN_IF_ERROR(HostTensorToBorrowingLiteral(*tensor, &literal));
 
-  TF_RETURN_IF_ERROR(
-      xla::TpuTransferManagerInterface::GetRegisteredTpuTransferManager()
-          ->LinearizeToBuffers(literal, linearized_buffers));
+  auto* transfer_manager =
+      xla::TpuTransferManagerInterface::GetRegisteredTpuTransferManager();
+  TF_RETURN_IF_ERROR(transfer_manager->LinearizeToBuffers(
+      literal, transfer_manager->HostShapeToDeviceShape(literal.shape()),
+      linearized_buffers));
 
   // The input tensor is ref-counted. Save a handle on the input tensor if
   // its underlying storage is shared with linearized buffers to prevent
@@ -236,148 +246,9 @@ Status AutoTransposeAndLinearize(OpKernelContext* ctx,
       break;
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-// PrelinearizeOp is used to linearize one tensor to the device format.
-class PrelinearizeOp : public OpKernel {
- public:
-  explicit PrelinearizeOp(OpKernelConstruction* ctx) : OpKernel(ctx) {
-    OP_REQUIRES_OK(ctx, ctx->GetAttr("shape", &shape_));
-    OP_REQUIRES_OK(ctx, ctx->GetAttr("dtype", &dtype_));
-    xla::Shape shape;
-    OP_REQUIRES_OK(ctx, TensorShapeToXLAShape(dtype_, shape_, &shape));
-    OP_REQUIRES_OK(ctx,
-                   GetInfeedShapeWithLayout(ctx, "layout", shape, &xla_shape_));
-  }
-
-  void Compute(OpKernelContext* ctx) override {
-    const Tensor& input_tensor = ctx->input(0);
-    // Validate input.
-    OP_REQUIRES(
-        ctx, input_tensor.dtype() == dtype_,
-        errors::InvalidArgument("Prelinearize dtype mismatch; expected ",
-                                DataType_Name(dtype_), ", got ",
-                                DataType_Name(input_tensor.dtype())));
-    OP_REQUIRES(
-        ctx, input_tensor.shape() == shape_,
-        errors::InvalidArgument("Prelinearize shape mismatch; expected ",
-                                shape_.DebugString(), ", got ",
-                                input_tensor.shape().DebugString()));
-
-    // Auto-transpose and prelinearize.
-    LinearizerBufferList linearized_buffers;
-    std::vector<Tensor> saved_input_tensors;
-    auto status =
-        AutoTransposeAndLinearize(ctx, input_tensor, xla_shape_,
-                                  &linearized_buffers, &saved_input_tensors);
-    OP_REQUIRES_OK(ctx, status);
-
-    // Write to output.
-    tensorflow::Tensor* output;
-    OP_REQUIRES_OK(ctx,
-                   ctx->allocate_output(0, tensorflow::TensorShape{}, &output));
-    output->scalar<tensorflow::Variant>()() = LinearizedBuffersWrapper{
-        std::move(linearized_buffers), std::move(saved_input_tensors)};
-  }
-
-  bool IsExpensive() override { return true; }
-
- private:
-  TensorShape shape_;
-  DataType dtype_;
-  xla::Shape xla_shape_;
-
-  // PrelinearizeOp is neither copyable nor movable.
-  PrelinearizeOp(const PrelinearizeOp&) = delete;
-  PrelinearizeOp& operator=(const PrelinearizeOp&) = delete;
-};
-
-// PrelinearizeTupleOp is used to linearize multiple tensors to the device
-// format.
-class PrelinearizeTupleOp : public OpKernel {
- public:
-  explicit PrelinearizeTupleOp(OpKernelConstruction* ctx) : OpKernel(ctx) {
-    OP_REQUIRES_OK(ctx, ctx->GetAttr("shapes", &shapes_));
-    OP_REQUIRES_OK(ctx, ctx->GetAttr("dtypes", &dtypes_));
-    OP_REQUIRES(
-        ctx, shapes_.size() == dtypes_.size(),
-        errors::InvalidArgument(
-            "shapes and dtypes must be the same length. shapes length = ",
-            shapes_.size(), ", dtypes length = ", dtypes_.size()));
-
-    std::vector<xla::Shape> xla_shapes;
-    for (int i = 0; i < shapes_.size(); i++) {
-      xla::Shape xla_shape;
-      OP_REQUIRES_OK(ctx,
-                     TensorShapeToXLAShape(dtypes_[i], shapes_[i], &xla_shape));
-      xla_shapes.push_back(xla_shape);
-    }
-    OP_REQUIRES_OK(
-        ctx, GetInfeedShapeWithLayout(
-                 ctx, "layouts", xla::ShapeUtil::MakeTupleShape(xla_shapes),
-                 &tuple_shape_));
-  }
-
-  void Compute(OpKernelContext* ctx) override {
-    OpInputList values;
-    OP_REQUIRES_OK(ctx, ctx->input_list("inputs", &values));
-    OP_REQUIRES(ctx, values.size() == shapes_.size(),
-                errors::InvalidArgument(
-                    "Wrong number of inputs to PrelinearizeTuple."));
-
-    LinearizerBufferList all_linearized_buffers;
-    std::vector<Tensor> all_saved_input_tensors;
-    for (int i = 0; i < values.size(); i++) {
-      // Validate input.
-      const Tensor& input_tensor = values[i];
-      OP_REQUIRES(ctx, input_tensor.dtype() == dtypes_[i],
-                  errors::InvalidArgument(
-                      "PrelinearizeTuple dtype mismatch at tuple element ", i,
-                      "; expected ", DataType_Name(dtypes_[i]), ", got ",
-                      DataType_Name(input_tensor.dtype())));
-      OP_REQUIRES(ctx, input_tensor.shape() == shapes_[i],
-                  errors::InvalidArgument(
-                      "PrelinearizeTuple shape mismatch at tuple element ", i,
-                      "; expected ", shapes_[i].DebugString(), ", got ",
-                      input_tensor.shape().DebugString()));
-
-      // Auto-transpose and prelinearize.
-      LinearizerBufferList linearized_buffers;
-      std::vector<Tensor> saved_input_tensors;
-      auto status = AutoTransposeAndLinearize(
-          ctx, input_tensor, tuple_shape_.tuple_shapes(i), &linearized_buffers,
-          &saved_input_tensors);
-      OP_REQUIRES_OK(ctx, status);
-      all_linearized_buffers.insert(
-          all_linearized_buffers.end(),
-          std::make_move_iterator(linearized_buffers.begin()),
-          std::make_move_iterator(linearized_buffers.end()));
-      all_saved_input_tensors.insert(
-          all_saved_input_tensors.end(),
-          std::make_move_iterator(saved_input_tensors.begin()),
-          std::make_move_iterator(saved_input_tensors.end()));
-    }
-
-    tensorflow::Tensor* output;
-    OP_REQUIRES_OK(ctx,
-                   ctx->allocate_output(0, tensorflow::TensorShape{}, &output));
-    output->scalar<tensorflow::Variant>()() = LinearizedBuffersWrapper{
-        std::move(all_linearized_buffers), std::move(all_saved_input_tensors)};
-  }
-
-  bool IsExpensive() override { return true; }
-
- private:
-  std::vector<TensorShape> shapes_;
-  DataTypeVector dtypes_;
-  xla::Shape tuple_shape_;
-
-  // PrelinearizeTupleOp is neither copyable nor movable.
-  PrelinearizeTupleOp(const PrelinearizeTupleOp&) = delete;
-  PrelinearizeTupleOp& operator=(const PrelinearizeTupleOp&) = delete;
-};
-
 class StreamExecutorInfeedEnqueueOp : public TpuInfeedEnqueueOp {
  public:
   explicit StreamExecutorInfeedEnqueueOp(OpKernelConstruction* ctx)
@@ -433,19 +304,20 @@ TpuInfeedEnqueueOp::TpuInfeedEnqueueOp(
                  GetInfeedShapeWithLayout(ctx, "layout", shape, &xla_shape_));
 }
 
-Status TpuInfeedEnqueueOp::DoWork(OpKernelContext* ctx, int device_ordinal) {
+absl::Status TpuInfeedEnqueueOp::DoWork(OpKernelContext* ctx,
+                                        int device_ordinal) {
   VLOG(1) << "TpuInfeedEnqueueOp::DoWork. iter_id=" << ctx->frame_iter().iter_id
           << " device_ordinal=" << device_ordinal;
   const Tensor& input_tensor = ctx->input(0);
 
   // Validate runtime shape and fail if it doesn't match the contract.
   if (input_tensor.dtype() != dtype_) {
-    return errors::InvalidArgument("Infeed dtype mismatch.");
+    return absl::InvalidArgumentError("Infeed dtype mismatch.");
   }
   if (input_tensor.shape() != shape_) {
-    return errors::InvalidArgument("Infeed shape mismatch; expected ",
-                                   shape_.DebugString(), ", got ",
-                                   input_tensor.shape().DebugString());
+    return absl::InvalidArgumentError(
+        absl::StrCat("Infeed shape mismatch; expected ", shape_.DebugString(),
+                     ", got ", input_tensor.shape().DebugString()));
   }
 
   const Tensor* tensor = &input_tensor;
@@ -465,7 +337,7 @@ Status TpuInfeedEnqueueOp::DoWork(OpKernelContext* ctx, int device_ordinal) {
       transfer_op_->TransferLiteralToInfeed(device_ordinal, literal));
   VLOG(1) << "TpuInfeedEnqueueOp completes. iter_id="
           << ctx->frame_iter().iter_id << " device_ordinal=" << device_ordinal;
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 TpuInfeedEnqueueTupleOp::TpuInfeedEnqueueTupleOp(
@@ -477,7 +349,7 @@ TpuInfeedEnqueueTupleOp::TpuInfeedEnqueueTupleOp(
   OP_REQUIRES_OK(ctx, ctx->GetAttr("dtypes", &dtypes_));
   OP_REQUIRES(
       ctx, shapes_.size() == dtypes_.size(),
-      errors::InvalidArgument("shapes and dtypes must be the same length."));
+      absl::InvalidArgumentError("shapes and dtypes must be the same length."));
 
   std::vector<xla::Shape> xla_shapes;
   for (int i = 0; i < shapes_.size(); i++) {
@@ -492,14 +364,14 @@ TpuInfeedEnqueueTupleOp::TpuInfeedEnqueueTupleOp(
                                     &tuple_shape_));
 }
 
-Status TpuInfeedEnqueueTupleOp::DoWork(OpKernelContext* ctx,
-                                       int device_ordinal) {
+absl::Status TpuInfeedEnqueueTupleOp::DoWork(OpKernelContext* ctx,
+                                             int device_ordinal) {
   VLOG(1) << "TpuInfeedEnqueueTupleOp::DoWork. iter_id="
           << ctx->frame_iter().iter_id << " device_ordinal=" << device_ordinal;
   OpInputList values;
   TF_RETURN_IF_ERROR(ctx->input_list("inputs", &values));
   if (values.size() != shapes_.size()) {
-    return errors::InvalidArgument(
+    return absl::InvalidArgumentError(
         "Wrong number of inputs to InfeedEnqueueTuple.");
   }
 
@@ -513,9 +385,9 @@ Status TpuInfeedEnqueueTupleOp::DoWork(OpKernelContext* ctx,
     // Validate runtime shapes and fail if it doesn't match the contract.
     const Tensor* tensor = &values[i];
     if (tensor->shape() != shapes_[i]) {
-      return errors::InvalidArgument("Infeed shape mismatch for tuple element ",
-                                     i, "; expected ", shapes_[i].DebugString(),
-                                     ", got ", tensor->shape().DebugString());
+      return absl::InvalidArgumentError(absl::StrCat(
+          "Infeed shape mismatch for tuple element ", i, "; expected ",
+          shapes_[i].DebugString(), ", got ", tensor->shape().DebugString()));
     }
     if (!xla::LayoutUtil::IsMonotonicWithDim0Major(
             tuple_shape_.tuple_shapes(i).layout())) {
@@ -541,7 +413,7 @@ Status TpuInfeedEnqueueTupleOp::DoWork(OpKernelContext* ctx,
   VLOG(1) << "TpuInfeedEnqueueTupleOp completes. iter_id="
           << ctx->frame_iter().iter_id << " device_ordinal=" << device_ordinal;
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 InfeedEnqueuePrelinearizedBufferOp::InfeedEnqueuePrelinearizedBufferOp(
@@ -549,8 +421,8 @@ InfeedEnqueuePrelinearizedBufferOp::InfeedEnqueuePrelinearizedBufferOp(
     std::unique_ptr<TpuTransferOpInterface> transfer_op)
     : TpuTransferAsyncOpKernel(ctx, "prelinearized_buffers_to_infeed", 8,
                                std::move(transfer_op)) {}
-Status InfeedEnqueuePrelinearizedBufferOp::DoWork(OpKernelContext* ctx,
-                                                  int device_ordinal) {
+absl::Status InfeedEnqueuePrelinearizedBufferOp::DoWork(OpKernelContext* ctx,
+                                                        int device_ordinal) {
   const Tensor& input_tensor = ctx->input(0);
   const LinearizedBuffersWrapper* wrapper =
       input_tensor.scalar<tensorflow::Variant>()()
@@ -558,7 +430,7 @@ Status InfeedEnqueuePrelinearizedBufferOp::DoWork(OpKernelContext* ctx,
   TF_RETURN_IF_ERROR(
       transfer_op_->TransferBuffersToInfeed(device_ordinal, wrapper->buffers));
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // These ops execute on either the TPU device or the CPU device. When running on
@@ -576,12 +448,6 @@ REGISTER_KERNEL_BUILDER(
 REGISTER_KERNEL_BUILDER(Name("InfeedEnqueueTuple").Device(DEVICE_CPU),
                         StreamExecutorInfeedEnqueueTupleOp);
 
-// Prelinearize ops run on CPU as part of tf.data input pipeline.
-REGISTER_KERNEL_BUILDER(Name("Prelinearize").Device(DEVICE_CPU),
-                        PrelinearizeOp);
-REGISTER_KERNEL_BUILDER(Name("PrelinearizeTuple").Device(DEVICE_CPU),
-                        PrelinearizeTupleOp);
-
 // InfeedEnqueuePrelinearizedBuffer op run on CPU and takes a device_ordinal to
 // select the right device to infeed.
 REGISTER_KERNEL_BUILDER(
