@@ -19,13 +19,17 @@ limitations under the License.
 #include <utility>
 #include <vector>
 
-#include "tensorflow/compiler/xla/hlo/ir/hlo_opcode.h"
-#include "tensorflow/compiler/xla/service/hlo.pb.h"
-#include "tensorflow/compiler/xla/xla_data.pb.h"
-#include "tensorflow/core/framework/attr_value.pb.h"
+#include "absl/container/flat_hash_set.h"
+#include "absl/status/status.h"
+#include "absl/strings/str_cat.h"
+#include "xla/service/hlo.pb.h"
+#include "xla/tsl/platform/errors.h"
+#include "xla/xla_data.pb.h"
 #include "tensorflow/core/framework/shape_inference.h"
-#include "tensorflow/core/lib/core/errors.h"
-#include "tensorflow/core/lib/strings/stringprintf.h"
+#include "tensorflow/core/platform/errors.h"
+#include "tensorflow/core/platform/stringprintf.h"
+#include "tensorflow/core/platform/types.h"
+#include "tensorflow/core/protobuf/tpu/optimization_parameters.pb.h"
 
 namespace tensorflow {
 namespace tpu {
@@ -38,6 +42,8 @@ std::string GetOptimizationAlgorithmName(OptimizationAlgorithm alg) {
       return "AdagradMomentum";
     case OptimizationAlgorithm::kBoundedAdagrad:
       return "BoundedAdagrad";
+    case OptimizationAlgorithm::kFrequencyAwareAdagrad:
+      return "FrequencyAwareAdagrad";
     case OptimizationAlgorithm::kStochasticGradientDescent:
       return "StochasticGradientDescent";
     case OptimizationAlgorithm::kFtrl:
@@ -82,6 +88,8 @@ std::string GetOptimizationAlgorithmFriendlyName(OptimizationAlgorithm alg) {
       return "Adagrad with Momentum";
     case OptimizationAlgorithm::kBoundedAdagrad:
       return "Bounded Adagrad";
+    case OptimizationAlgorithm::kFrequencyAwareAdagrad:
+      return "Frequency Aware Adagrad";
     case OptimizationAlgorithm::kStochasticGradientDescent:
       return "stochastic gradient descent";
     case OptimizationAlgorithm::kFtrl:
@@ -121,57 +129,60 @@ std::string GetOptimizationAlgorithmFriendlyName(OptimizationAlgorithm alg) {
 // Returns the number of optimization parameter vectors used by the optimization
 // algorithm, excluding the weights themselves and assuming no gradient
 // accumulation.
-Status GetBaseAuxiliaryParameterCount(const OptimizationParameters& params,
-                                      int* count) {
+absl::Status GetBaseAuxiliaryParameterCount(
+    const OptimizationParameters& params, int* count) {
   switch (params.parameters_case()) {
     case OptimizationAlgorithm::kAdagrad:
       *count = 1;
-      return OkStatus();
+      return absl::OkStatus();
     case OptimizationAlgorithm::kAdagradMomentum:
       *count = 2;
-      return OkStatus();
+      return absl::OkStatus();
     case OptimizationAlgorithm::kBoundedAdagrad:
       *count = 1;
-      return OkStatus();
+      return absl::OkStatus();
+    case OptimizationAlgorithm::kFrequencyAwareAdagrad:
+      *count = 2;
+      return absl::OkStatus();
     case OptimizationAlgorithm::kStochasticGradientDescent:
       *count = 0;
-      return OkStatus();
+      return absl::OkStatus();
     case OptimizationAlgorithm::kFtrl:
       *count = 2;
-      return OkStatus();
+      return absl::OkStatus();
     case OptimizationAlgorithm::kAdam:
       *count = 2;
-      return OkStatus();
+      return absl::OkStatus();
     case OptimizationAlgorithm::kMomentum:
       *count = 1;
-      return OkStatus();
+      return absl::OkStatus();
     case OptimizationAlgorithm::kLion:
       *count = 1;
-      return OkStatus();
+      return absl::OkStatus();
     case OptimizationAlgorithm::kRmsProp:
       *count = 2;
-      return OkStatus();
+      return absl::OkStatus();
     case OptimizationAlgorithm::kCenteredRmsProp:
       *count = 3;
-      return OkStatus();
+      return absl::OkStatus();
     case OptimizationAlgorithm::kMdlAdagradLight:
       *count = 3;
-      return OkStatus();
+      return absl::OkStatus();
     case OptimizationAlgorithm::kAdadelta:
       *count = 2;
-      return OkStatus();
+      return absl::OkStatus();
     case OptimizationAlgorithm::kProximalAdagrad:
       *count = 1;
-      return OkStatus();
+      return absl::OkStatus();
     case OptimizationAlgorithm::kOnlineYogi:
       *count = 2;
-      return OkStatus();
+      return absl::OkStatus();
     case OptimizationAlgorithm::kProximalYogi:
       *count = 2;
-      return OkStatus();
+      return absl::OkStatus();
     case OptimizationAlgorithm::kFrequencyEstimator:
       *count = 1;
-      return OkStatus();
+      return absl::OkStatus();
     case OptimizationAlgorithm::kUserDefinedProgram: {
       const xla::ProgramShapeProto& program_shape =
           params.user_defined_program().program().host_program_shape();
@@ -190,30 +201,31 @@ Status GetBaseAuxiliaryParameterCount(const OptimizationParameters& params,
 
       *count = num_outputs - 1;
 
-      return OkStatus();
+      return absl::OkStatus();
     }
     case OptimizationAlgorithm::kAssign:
       *count = 0;
-      return OkStatus();
+      return absl::OkStatus();
     case OptimizationAlgorithm::PARAMETERS_NOT_SET:
       return errors::InvalidArgument("No optimization algorithm specified");
   }
   return errors::InvalidArgument("No optimization algorithm specified");
 }
 
-Status GetGradientAccumulationSupport(const OptimizationParameters& params,
-                                      GradientAccumulationSupport* support) {
+absl::Status GetGradientAccumulationSupport(
+    const OptimizationParameters& params,
+    GradientAccumulationSupport* support) {
   int auxiliary_parameter_count;
   TF_RETURN_IF_ERROR(
       GetBaseAuxiliaryParameterCount(params, &auxiliary_parameter_count));
   *support = auxiliary_parameter_count + 1 <= kMaxAuxiliaryParameterCount
                  ? GradientAccumulationSupport::kSupported
                  : GradientAccumulationSupport::kNotSupported;
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status UseGradientAccumulation(const OptimizationParameters& params,
-                               bool* use_gradient_accumulation) {
+absl::Status UseGradientAccumulation(const OptimizationParameters& params,
+                                     bool* use_gradient_accumulation) {
   GradientAccumulationSupport support;
   TF_RETURN_IF_ERROR(GetGradientAccumulationSupport(params, &support));
   bool raw_gradient_accumulation_status = false;
@@ -253,10 +265,10 @@ Status UseGradientAccumulation(const OptimizationParameters& params,
       break;
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status GetOptimizationAlgorithmStateVariables(
+absl::Status GetOptimizationAlgorithmStateVariables(
     const OptimizationParameters& params,
     std::vector<StateVariableSpecification>* state_variables) {
   // The parameter set for the weights themselves is required to be named
@@ -291,6 +303,12 @@ Status GetOptimizationAlgorithmStateVariables(
       add_state_variable("accumulators");
       break;
     }
+    case OptimizationAlgorithm::kFrequencyAwareAdagrad: {
+      add_state_variable("parameters");
+      add_state_variable("accumulators");
+      add_state_variable("counters");
+      break;
+    }
     case OptimizationAlgorithm::kStochasticGradientDescent: {
       add_state_variable("parameters");
       break;
@@ -399,7 +417,34 @@ Status GetOptimizationAlgorithmStateVariables(
         "does not support gradient accumulation because it "
         "already has too many other accumulators");
   }
-  return OkStatus();
+  return absl::OkStatus();
+}
+
+absl::flat_hash_set<int> GetOptimizerDynamicInputTags(
+    const OptimizationParameters& params) {
+  absl::flat_hash_set<int> tags;
+  if (params.learning_rate().has_dynamic()) {
+    tags.insert(params.learning_rate().dynamic().tag());
+  }
+  tags.merge(GetOptimizerHyperParameterTags(params));
+  return tags;
+}
+
+absl::flat_hash_set<int> GetOptimizerHyperParameterTags(
+    const OptimizationParameters& params) {
+  absl::flat_hash_set<int> tags;
+  switch (params.parameters_case()) {
+    case OptimizationAlgorithm::kFrequencyAwareAdagrad:
+      tags.insert(params.frequency_aware_adagrad().step_counter().tag());
+      break;
+    default:
+      break;
+  }
+  return tags;
+}
+
+bool UsesDynamicInputsInOptimizer(const OptimizationParameters& params) {
+  return !GetOptimizerDynamicInputTags(params).empty();
 }
 
 std::vector<OptimizationAlgorithm> GetOptimizationAlgorithms() {
@@ -407,6 +452,7 @@ std::vector<OptimizationAlgorithm> GetOptimizationAlgorithms() {
       OptimizationAlgorithm::kAdagrad,
       OptimizationAlgorithm::kAdagradMomentum,
       OptimizationAlgorithm::kBoundedAdagrad,
+      OptimizationAlgorithm::kFrequencyAwareAdagrad,
       OptimizationAlgorithm::kStochasticGradientDescent,
       OptimizationAlgorithm::kFtrl,
       OptimizationAlgorithm::kAdam,
@@ -425,7 +471,7 @@ std::vector<OptimizationAlgorithm> GetOptimizationAlgorithms() {
   };
 }
 
-Status LoadOpShapeFunction::operator()(
+absl::Status LoadOpShapeFunction::operator()(
     shape_inference::InferenceContext* c) const {
   int table_id;
   TF_RETURN_IF_ERROR(c->GetAttr("table_id", &table_id));
@@ -452,10 +498,10 @@ Status LoadOpShapeFunction::operator()(
     TF_RETURN_IF_ERROR(c->Merge(parameter_shape, accumulator_j_shape, &merged));
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status RetrieveOpShapeFunction::operator()(
+absl::Status RetrieveOpShapeFunction::operator()(
     shape_inference::InferenceContext* c) const {
   int table_id;
   TF_RETURN_IF_ERROR(c->GetAttr("table_id", &table_id));
@@ -474,7 +520,7 @@ Status RetrieveOpShapeFunction::operator()(
     c->set_output(j, c->MakeShape(std::vector<shape_inference::DimensionHandle>(
                          2, c->UnknownDim())));
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace tpu
