@@ -21,7 +21,6 @@
 
 #include "third_party/gpus/cuda/include/cublas_v2.h"
 #include "third_party/gpus/cuda/include/cusolverDn.h"
-#include "tensorflow/compiler/xla/stream_executor/cuda/cuda_activation.h"
 #include "tensorflow/core/common_runtime/gpu/gpu_event_mgr.h"
 #include "tensorflow/core/framework/op_kernel.h"
 #include "tensorflow/core/framework/types.h"
@@ -98,13 +97,11 @@ using trsm_Z = cublasStatus_t(cublasContext*, cublasSideMode_t,
 namespace tensorflow {
 namespace {
 
-using se::cuda::ScopedActivateExecutorContext;
-
 inline bool CopyHostToDevice(OpKernelContext* context, void* dst,
                              const void* src, uint64 bytes) {
   auto stream = context->op_device_context()->stream();
   se::DeviceMemoryBase wrapped_dst(dst);
-  return stream->ThenMemcpy(&wrapped_dst, src, bytes).ok();
+  return stream->Memcpy(&wrapped_dst, src, bytes).ok();
 }
 
 // A set of initialized handles to the underlying Cuda libraries used by
@@ -167,12 +164,11 @@ HandleMap* GetHandleMapSingleton() {
 
 GpuSolver::GpuSolver(OpKernelContext* context) : context_(context) {
   mutex_lock lock(handle_map_mutex);
-  const cudaStream_t* cu_stream_ptr = CHECK_NOTNULL(
-      reinterpret_cast<const cudaStream_t*>(context->op_device_context()
-                                                ->stream()
-                                                ->implementation()
-                                                ->GpuStreamMemberHack()));
-  cuda_stream_ = *cu_stream_ptr;
+  cuda_stream_ = reinterpret_cast<cudaStream_t>(
+      CHECK_NOTNULL(context->op_device_context()
+                        ->stream()
+                        ->platform_specific_handle()
+                        .stream));
   HandleMap* handle_map = CHECK_NOTNULL(GetHandleMapSingleton());
   auto it = handle_map->find(cuda_stream_);
   if (it == handle_map->end()) {
@@ -229,7 +225,8 @@ void GpuSolver::CheckLapackInfoAndDeleteSolverAsync(
           std::function<void(const Status&, const std::vector<HostLapackInfo>&)>
               info_checker_callback,
           std::vector<HostLapackInfo> host_lapack_infos) {
-        ScopedActivateExecutorContext scoped_activation{stream->parent()};
+        std::unique_ptr<stream_executor::ActivateContext> scoped_activation =
+            stream->parent()->Activate();
         Status status;
         for (const auto& host_lapack_info : host_lapack_infos) {
           for (int i = 0; i < host_lapack_info.size() && status.ok(); ++i) {
@@ -654,7 +651,7 @@ static inline Status HeevdImpl(BufSizeFnT bufsize, SolverFnT solver,
   uint64_t work_size_in_bytes = static_cast<uint64_t>(lwork) * sizeof(Scalar);
   se::DeviceMemoryBase dev_workspace_ptr(dev_workspace.mutable_data(),
                                          work_size_in_bytes);
-  stream->ThenMemZero(&dev_workspace_ptr, work_size_in_bytes);
+  TF_RETURN_IF_ERROR(stream->MemZero(&dev_workspace_ptr, work_size_in_bytes));
 #endif
   /* Launch the solver kernel. */
   TF_RETURN_IF_CUSOLVER_ERROR(
