@@ -42,14 +42,14 @@ EventsWriter::~EventsWriter() {
   Close().IgnoreError();  // Autoclose in destructor.
 }
 
-Status EventsWriter::Init() { return InitWithSuffix(""); }
+absl::Status EventsWriter::Init() { return InitWithSuffix(""); }
 
-Status EventsWriter::InitWithSuffix(const string& suffix) {
+absl::Status EventsWriter::InitWithSuffix(const string& suffix) {
   file_suffix_ = suffix;
   return InitIfNeeded();
 }
 
-Status EventsWriter::InitIfNeeded() {
+absl::Status EventsWriter::InitIfNeeded() {
   if (recordio_writer_ != nullptr) {
     CHECK(!filename_.empty());
     if (!FileStillExists().ok()) {
@@ -60,7 +60,7 @@ Status EventsWriter::InitIfNeeded() {
       }
     } else {
       // No-op: File is present and writer is initialized.
-      return OkStatus();
+      return absl::OkStatus();
     }
   }
 
@@ -96,7 +96,7 @@ Status EventsWriter::InitIfNeeded() {
     WriteEvent(event);
     TF_RETURN_WITH_CONTEXT_IF_ERROR(Flush(), "Flushing first event.");
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 string EventsWriter::FileName() {
@@ -106,7 +106,7 @@ string EventsWriter::FileName() {
   return filename_;
 }
 
-void EventsWriter::WriteSerializedEvent(StringPiece event_str) {
+void EventsWriter::WriteSerializedEvent(absl::string_view event_str) {
   if (recordio_writer_ == nullptr) {
     if (!InitIfNeeded().ok()) {
       LOG(ERROR) << "Write failed because file could not be opened.";
@@ -125,8 +125,8 @@ void EventsWriter::WriteEvent(const Event& event) {
   WriteSerializedEvent(record);
 }
 
-Status EventsWriter::Flush() {
-  if (num_outstanding_events_ == 0) return OkStatus();
+absl::Status EventsWriter::Flush() {
+  if (num_outstanding_events_ == 0) return absl::OkStatus();
   CHECK(recordio_file_ != nullptr) << "Unexpected NULL file";
 
   TF_RETURN_WITH_CONTEXT_IF_ERROR(recordio_writer_->Flush(), "Failed to flush ",
@@ -137,13 +137,13 @@ Status EventsWriter::Flush() {
                                   filename_);
   VLOG(1) << "Wrote " << num_outstanding_events_ << " events to disk.";
   num_outstanding_events_ = 0;
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status EventsWriter::Close() {
-  Status status = Flush();
+absl::Status EventsWriter::Close() {
+  absl::Status status = Flush();
   if (recordio_file_ != nullptr) {
-    Status close_status = recordio_file_->Close();
+    absl::Status close_status = recordio_file_->Close();
     if (!close_status.ok()) {
       status = close_status;
     }
@@ -154,9 +154,9 @@ Status EventsWriter::Close() {
   return status;
 }
 
-Status EventsWriter::FileStillExists() {
+absl::Status EventsWriter::FileStillExists() {
   if (env_->FileExists(filename_).ok()) {
-    return OkStatus();
+    return absl::OkStatus();
   }
   // This can happen even with non-null recordio_writer_ if some other
   // process has removed the file.
