@@ -15,9 +15,12 @@ limitations under the License.
 #include "tensorflow/core/util/example_proto_helper.h"
 
 #include <algorithm>
+#include <cstddef>
 #include <limits>
 #include <vector>
 
+#include "absl/status/status.h"
+#include "absl/strings/str_format.h"
 #include "tensorflow/core/example/example.pb.h"
 #include "tensorflow/core/example/feature.pb.h"
 #include "tensorflow/core/framework/numeric_op.h"
@@ -29,20 +32,20 @@ limitations under the License.
 
 namespace tensorflow {
 
-Status CheckValidType(const DataType& dtype) {
+absl::Status CheckValidType(const DataType& dtype) {
   switch (dtype) {
     case DT_INT64:
     case DT_FLOAT:
     case DT_STRING:
-      return OkStatus();
+      return absl::OkStatus();
     default:
       return errors::InvalidArgument("Received input dtype: ",
                                      DataTypeString(dtype));
   }
 }
 
-Status CheckTypesMatch(const Feature& feature, const DataType& dtype,
-                       bool* match) {
+absl::Status CheckTypesMatch(const Feature& feature, const DataType& dtype,
+                             bool* match) {
   switch (dtype) {
     case DT_INT64:
       *match = (feature.kind_case() == Feature::kInt64List);
@@ -57,13 +60,13 @@ Status CheckTypesMatch(const Feature& feature, const DataType& dtype,
       return errors::InvalidArgument("Invalid input dtype: ",
                                      DataTypeString(dtype));
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status FeatureDenseCopy(const std::size_t out_index, const string& name,
-                        const string& key, const DataType& dtype,
-                        const TensorShape& shape, const Feature& feature,
-                        Tensor* out) {
+absl::Status FeatureDenseCopy(const std::size_t out_index, const string& name,
+                              const string& key, const DataType& dtype,
+                              const TensorShape& shape, const Feature& feature,
+                              Tensor* out) {
   const std::size_t num_elements = shape.num_elements();
   const std::size_t offset = out_index * num_elements;
 
@@ -79,7 +82,7 @@ Status FeatureDenseCopy(const std::size_t out_index, const string& name,
       }
       auto out_p = out->flat<int64_t>().data() + offset;
       std::copy_n(values.value().data(), num_elements, out_p);
-      return OkStatus();
+      return absl::OkStatus();
     }
     case DT_FLOAT: {
       const FloatList& values = feature.float_list();
@@ -92,7 +95,7 @@ Status FeatureDenseCopy(const std::size_t out_index, const string& name,
       }
       auto out_p = out->flat<float>().data() + offset;
       std::copy_n(values.value().data(), num_elements, out_p);
-      return OkStatus();
+      return absl::OkStatus();
     }
     case DT_STRING: {
       const BytesList& values = feature.bytes_list();
@@ -107,7 +110,7 @@ Status FeatureDenseCopy(const std::size_t out_index, const string& name,
       std::transform(values.value().data(),
                      values.value().data() + num_elements, out_p,
                      [](const string* s) { return *s; });
-      return OkStatus();
+      return absl::OkStatus();
     }
     default:
       return errors::InvalidArgument("Invalid input dtype: ",
@@ -217,7 +220,7 @@ void RowDenseCopy(const std::size_t& out_index, const DataType& dtype,
   }
 }
 
-Status SingleExampleProtoToTensors(
+absl::Status SingleExampleProtoToTensors(
     const Example& example, const string& example_name, const int batch_index,
     const std::vector<FixedLenFeature>& fixed_len_features,
     const std::vector<VarLenFeature>& var_len_features,
@@ -297,13 +300,13 @@ Status SingleExampleProtoToTensors(
           Tensor(dtype, TensorShape({0}));
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status GetSparseTensorShapes(const VarLenFeature& var_len_feature,
-                             const std::vector<Tensor>& sparse_values_tmp,
-                             const int batch_size,
-                             VarLenFeatureBatchShapes* output_shapes) {
+absl::Status GetSparseTensorShapes(const VarLenFeature& var_len_feature,
+                                   const std::vector<Tensor>& sparse_values_tmp,
+                                   const int batch_size,
+                                   VarLenFeatureBatchShapes* output_shapes) {
   int64_t total_num_features = 0;
   int64_t max_num_features = 0;
   for (int b = 0; b < batch_size; ++b) {
@@ -316,10 +319,10 @@ Status GetSparseTensorShapes(const VarLenFeature& var_len_feature,
   output_shapes->indices_shape.AddDim(2);
   output_shapes->values_shape.AddDim(total_num_features);
   output_shapes->max_num_features = max_num_features;
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status BatchExampleProtoToTensors(
+absl::Status BatchExampleProtoToTensors(
     const std::vector<const Example*>& examples,
     const std::vector<string>& names,
     const std::vector<FixedLenFeature>& fixed_len_features,
@@ -404,10 +407,26 @@ Status BatchExampleProtoToTensors(
       offset += num_elements;
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status ParseExampleAttrs::FinishInit(int op_version) {
+absl::Status ParseExampleAttrs::UpdateDenseShapes(
+    const std::vector<size_t>& got_dims) {
+  if (got_dims.size() != dense_shapes.size()) {
+    return absl::InvalidArgumentError(absl::StrFormat(
+        "got_dims.size() (%d) must match dense_shapes.size() (%d)",
+        got_dims.size(), dense_shapes.size()));
+  }
+  for (size_t d = 0; d < dense_shapes.size(); ++d) {
+    dense_shapes[d].set_dim(0, got_dims[d]);
+  }
+  // Recalculate relative fields.
+  variable_length.clear();
+  elements_per_stride.clear();
+  return GetDenseShapes(dense_shapes, &variable_length, &elements_per_stride);
+}
+
+absl::Status ParseExampleAttrs::FinishInit(int op_version) {
   switch (op_version) {
     case 1:
       num_ragged = 0;
@@ -454,10 +473,10 @@ Status ParseExampleAttrs::FinishInit(int op_version) {
                                      DataTypeString(type));
     }
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status ParseSingleExampleAttrs::FinishInit() {
+absl::Status ParseSingleExampleAttrs::FinishInit() {
   if (sparse_keys.size() != sparse_types.size()) {
     return errors::InvalidArgument("len(sparse_keys) != len(sparse_types)");
   }
@@ -473,10 +492,10 @@ Status ParseSingleExampleAttrs::FinishInit() {
   for (const DataType& type : sparse_types) {
     TF_RETURN_IF_ERROR(CheckValidType(type));
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status ParseSequenceExampleAttrs::FinishInit(int op_version) {
+absl::Status ParseSequenceExampleAttrs::FinishInit(int op_version) {
   switch (op_version) {
     case 1:
       num_context_ragged = 0;
@@ -590,10 +609,10 @@ Status ParseSequenceExampleAttrs::FinishInit(int op_version) {
     }
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status ParseSingleSequenceExampleAttrs::FinishInit() {
+absl::Status ParseSingleSequenceExampleAttrs::FinishInit() {
   if (static_cast<size_t>(num_context_sparse) != context_sparse_types.size()) {
     return errors::InvalidArgument(
         "len(context_sparse_keys) != len(context_sparse_types)");
@@ -629,12 +648,12 @@ Status ParseSingleSequenceExampleAttrs::FinishInit() {
   for (const DataType& type : feature_list_sparse_types) {
     TF_RETURN_IF_ERROR(CheckValidType(type));
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status GetDenseShapes(const std::vector<PartialTensorShape>& dense_shapes,
-                      std::vector<bool>* variable_length,
-                      std::vector<std::size_t>* elements_per_stride) {
+absl::Status GetDenseShapes(const std::vector<PartialTensorShape>& dense_shapes,
+                            std::vector<bool>* variable_length,
+                            std::vector<std::size_t>* elements_per_stride) {
   // Temporary check until we start allowing a variable length outer
   // dimension.
   for (int i = 0; i < dense_shapes.size(); ++i) {
@@ -666,7 +685,7 @@ Status GetDenseShapes(const std::vector<PartialTensorShape>& dense_shapes,
     }
     elements_per_stride->push_back(dense_shape.num_elements());
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace tensorflow
