@@ -15,12 +15,15 @@ limitations under the License.
 
 #include "tensorflow/core/util/tensor_bundle/tensor_bundle.h"
 
-#include <algorithm>
 #include <cstdlib>
 #include <cstring>
 #include <memory>
 #include <utility>
 
+#include "absl/base/call_once.h"
+#include "absl/synchronization/mutex.h"
+#include "xla/tsl/lib/io/buffered_file.h"
+#include "xla/tsl/util/byte_swap_array.h"
 #include "tensorflow/core/framework/register_types.h"
 #include "tensorflow/core/framework/tensor.pb.h"
 #include "tensorflow/core/framework/tensor_shape.pb.h"
@@ -36,21 +39,22 @@ limitations under the License.
 #include "tensorflow/core/lib/core/threadpool.h"
 #include "tensorflow/core/lib/gtl/map_util.h"
 #include "tensorflow/core/lib/hash/crc32c.h"
-#include "tensorflow/core/lib/io/path.h"
 #include "tensorflow/core/lib/io/table_builder.h"
-#include "tensorflow/core/lib/random/random.h"
 #include "tensorflow/core/lib/strings/str_util.h"
-#include "tensorflow/core/lib/strings/stringprintf.h"
 #include "tensorflow/core/platform/bfloat16.h"
 #include "tensorflow/core/platform/cord.h"
 #include "tensorflow/core/platform/errors.h"
 #include "tensorflow/core/platform/mem.h"
+#include "tensorflow/core/platform/path.h"
+#include "tensorflow/core/platform/random.h"
 #include "tensorflow/core/platform/status.h"
+#include "tensorflow/core/platform/stringprintf.h"
+#include "tensorflow/core/platform/tstring.h"
 #include "tensorflow/core/util/env_var.h"
 #include "tensorflow/core/util/saved_tensor_slice_util.h"
 #include "tensorflow/core/util/tensor_bundle/byte_swap_tensor.h"
+#include "tensorflow/core/util/tensor_bundle/naming.h"
 #include "tensorflow/core/util/tensor_slice_util.h"
-#include "tensorflow/tsl/lib/io/buffered_file.h"
 
 #ifdef PLATFORM_WINDOWS
 #undef DeleteFile
@@ -85,10 +89,11 @@ namespace {
 //
 // Checksums the string lengths (as restored uint32 or uint64, not varint64
 // bytes) and string bytes, and stores it into "actual_crc32c".
-Status ReadStringTensor(io::InputBuffer* buffered_file, size_t num_elements,
-                        size_t offset, size_t size, tstring* destination,
-                        uint32* actual_crc32c, bool need_to_swap_bytes) {
-  if (size == 0) return OkStatus();
+absl::Status ReadStringTensor(io::InputBuffer* buffered_file,
+                              size_t num_elements, size_t offset, size_t size,
+                              tstring* destination, uint32* actual_crc32c,
+                              bool need_to_swap_bytes) {
+  if (size == 0) return absl::OkStatus();
   CHECK_GT(size, 0);
 
   // Reads "num_elements" varint64's from "buffered_file".
@@ -153,18 +158,19 @@ Status ReadStringTensor(io::InputBuffer* buffered_file, size_t num_elements,
         buffered_file->ReadNBytes(string_length, &(*buffer)[0], &bytes_read));
     *actual_crc32c = crc32c::Extend(*actual_crc32c, buffer->data(), bytes_read);
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status ReadVariantTensor(io::InputBuffer* buffered_file, Tensor* ret,
-                         size_t offset, size_t size, uint32* actual_crc32c) {
+absl::Status ReadVariantTensor(io::InputBuffer* buffered_file, Tensor* ret,
+                               size_t offset, size_t size,
+                               uint32* actual_crc32c) {
   // On-disk format:
   //   [varint64 len1][bytes variant1][4 byte checksum]
   //   ..
   //   [varint64 lenN][bytes variantN][4 byte checksum]
   // Var "crc32c" checksums all the lens, variant bytes, individual variant
   // checksums (as uint32, not varint32 bytes).
-  if (size == 0) return OkStatus();
+  if (size == 0) return absl::OkStatus();
   size_t num_elements = ret->NumElements();
 
   // Reads the actual string bytes.
@@ -216,7 +222,7 @@ Status ReadVariantTensor(io::InputBuffer* buffered_file, Tensor* ret,
     ret->flat<Variant>()(i) = std::move(v);
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 char* GetBackingBuffer(const Tensor& val) {
@@ -229,26 +235,26 @@ tstring* GetStringBackingBuffer(const Tensor& val) {
   return const_cast<tstring*>(val.flat<tstring>().data());
 }
 
-Status ParseEntryProto(StringPiece key, StringPiece value,
-                       protobuf::MessageLite* out) {
+absl::Status ParseEntryProto(absl::string_view key, absl::string_view value,
+                             protobuf::MessageLite* out) {
   if (!out->ParseFromArray(value.data(), value.size())) {
     return errors::DataLoss("Entry for key ", key, " not parseable.");
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Serializes the data bytes of the non-string tensor "val".  Discards the
 // original content of "bytes_written", and on OK updates it with number of
 // bytes written.
 // REQUIRES: val.dtype() != DT_STRING
-Status WriteTensor(const Tensor& val, tsl::BufferedWritableFile* out,
-                   size_t* bytes_written) {
+absl::Status WriteTensor(const Tensor& val, tsl::BufferedWritableFile* out,
+                         size_t* bytes_written) {
   DCHECK_NE(val.dtype(), DT_STRING);
   DCHECK_NE(val.dtype(), DT_VARIANT);
   *bytes_written = val.TotalBytes();
   char* buf = GetBackingBuffer(val);
   VLOG(1) << "Appending " << *bytes_written << " bytes to file";
-  return out->Append(StringPiece(buf, *bytes_written));
+  return out->Append(absl::string_view(buf, *bytes_written));
 }
 
 // Serializes string tensor "val".  "bytes_written" is treated in the same
@@ -256,8 +262,9 @@ Status WriteTensor(const Tensor& val, tsl::BufferedWritableFile* out,
 //
 // Checksums all bytes written and stores it into "crc32c".
 // REQUIRES: val.dtype() == DT_STRING
-Status WriteStringTensor(const Tensor& val, tsl::BufferedWritableFile* out,
-                         size_t* bytes_written, uint32* crc32c) {
+absl::Status WriteStringTensor(const Tensor& val,
+                               tsl::BufferedWritableFile* out,
+                               size_t* bytes_written, uint32* crc32c) {
   // On-disk format:
   //   [varint64 len0]..[varint64 lenL][4 byte cksum on lengths][string bytes]
   // Var "crc32c" checksums the string lengths (as uint64, not varint64 bytes),
@@ -292,7 +299,7 @@ Status WriteStringTensor(const Tensor& val, tsl::BufferedWritableFile* out,
 
   // Writes the length checksum.
   const uint32 length_checksum = crc32c::Mask(*crc32c);
-  TF_RETURN_IF_ERROR(out->Append(StringPiece(
+  TF_RETURN_IF_ERROR(out->Append(absl::string_view(
       reinterpret_cast<const char*>(&length_checksum), sizeof(uint32))));
   *crc32c = crc32c::Extend(
       *crc32c, reinterpret_cast<const char*>(&length_checksum), sizeof(uint32));
@@ -305,11 +312,12 @@ Status WriteStringTensor(const Tensor& val, tsl::BufferedWritableFile* out,
     *bytes_written += string->size();
     *crc32c = crc32c::Extend(*crc32c, string->data(), string->size());
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status WriteVariantTensor(const Tensor& val, tsl::BufferedWritableFile* out,
-                          size_t* bytes_written, uint32* crc32c) {
+absl::Status WriteVariantTensor(const Tensor& val,
+                                tsl::BufferedWritableFile* out,
+                                size_t* bytes_written, uint32* crc32c) {
   // On-disk format:
   //   [varint64 len1][bytes variant1][4 byte checksum]
   //   ..
@@ -349,7 +357,7 @@ Status WriteVariantTensor(const Tensor& val, tsl::BufferedWritableFile* out,
 
     // Write the checksum.
     const uint32 length_checksum = crc32c::Mask(*crc32c);
-    TF_RETURN_IF_ERROR(out->Append(StringPiece(
+    TF_RETURN_IF_ERROR(out->Append(absl::string_view(
         reinterpret_cast<const char*>(&length_checksum), sizeof(uint32))));
     *crc32c =
         crc32c::Extend(*crc32c, reinterpret_cast<const char*>(&length_checksum),
@@ -357,7 +365,7 @@ Status WriteVariantTensor(const Tensor& val, tsl::BufferedWritableFile* out,
     *bytes_written += sizeof(uint32);
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Returns whether "slice_spec" is a full slice, with respect to the full shape.
@@ -376,8 +384,8 @@ bool IsFullSlice(const TensorSlice& slice_spec,
   }
 }
 
-Status CorruptFileError(const Status& in_status, const string& filename,
-                        const string& detail) {
+absl::Status CorruptFileError(const absl::Status& in_status,
+                              const string& filename, const string& detail) {
   if (in_status.ok()) {
     return errors::Internal("Unable to read file (", filename,
                             "). Perhaps the file is corrupt or was produced by "
@@ -385,7 +393,7 @@ Status CorruptFileError(const Status& in_status, const string& filename,
                             "(",
                             detail, ")");
   }
-  return Status(
+  return absl::Status(
       in_status.code(),
       strings::StrCat("Unable to read file (", filename,
                       "). Perhaps the file is corrupt or was produced by a "
@@ -406,14 +414,14 @@ table::Options TableBuilderOptions() {
 // Writes zeros to output buffer to align the next write to the requested
 // alignment. "size" is the current size of the buffer and is updated to the
 // new size.
-Status PadAlignment(tsl::BufferedWritableFile* out, int alignment,
-                    int64_t* size) {
+absl::Status PadAlignment(tsl::BufferedWritableFile* out, int alignment,
+                          int64_t* size) {
   int bytes_over = *size % alignment;
   if (bytes_over == 0) {
-    return OkStatus();
+    return absl::OkStatus();
   }
   int bytes_to_write = alignment - bytes_over;
-  Status status = out->Append(string(bytes_to_write, '\0'));
+  absl::Status status = out->Append(string(bytes_to_write, '\0'));
   if (status.ok()) {
     *size += bytes_to_write;
   }
@@ -422,7 +430,8 @@ Status PadAlignment(tsl::BufferedWritableFile* out, int alignment,
 
 }  // namespace
 
-BundleWriter::BundleWriter(Env* env, StringPiece prefix, const Options& options)
+BundleWriter::BundleWriter(Env* env, absl::string_view prefix,
+                           const Options& options)
     : env_(env), options_(options), prefix_(prefix), out_(nullptr), size_(0) {
   status_ = env_->HasAtomicMove(prefix_, &use_temp_file_);
   if (!status_.ok()) return;
@@ -436,7 +445,7 @@ BundleWriter::BundleWriter(Env* env, StringPiece prefix, const Options& options)
   }
 
   status_ = env_->CreateDir(string(io::Dirname(prefix_)));
-  if (!status_.ok() && !errors::IsAlreadyExists(status_)) {
+  if (!status_.ok() && !absl::IsAlreadyExists(status_)) {
     return;
   }
 
@@ -449,7 +458,7 @@ BundleWriter::BundleWriter(Env* env, StringPiece prefix, const Options& options)
   VLOG(1) << "Writing to file " << data_path_;
 }
 
-Status BundleWriter::Add(StringPiece key, const Tensor& val) {
+absl::Status BundleWriter::Add(absl::string_view key, const Tensor& val) {
   if (!status_.ok()) return status_;
   CHECK_NE(key, kHeaderEntryKey);
   const string key_string(key);
@@ -486,10 +495,10 @@ Status BundleWriter::Add(StringPiece key, const Tensor& val) {
   return status_;
 }
 
-Status BundleWriter::AddSlice(StringPiece full_tensor_key,
-                              const TensorShape& full_tensor_shape,
-                              const TensorSlice& slice_spec,
-                              const Tensor& slice_tensor) {
+absl::Status BundleWriter::AddSlice(absl::string_view full_tensor_key,
+                                    const TensorShape& full_tensor_shape,
+                                    const TensorSlice& slice_spec,
+                                    const Tensor& slice_tensor) {
   if (!status_.ok()) return status_;
   CHECK_NE(full_tensor_key, kHeaderEntryKey);
 
@@ -529,7 +538,7 @@ Status BundleWriter::AddSlice(StringPiece full_tensor_key,
 
 // TODO(zongheng): on metadata write failure or !status_.ok(), consider removing
 // the orphaned data file.
-Status BundleWriter::Finish() {
+absl::Status BundleWriter::Finish() {
   if (out_) {
     status_.Update(out_->Close());
     out_ = nullptr;
@@ -579,7 +588,7 @@ Status BundleWriter::Finish() {
     if (!status_.ok()) return status_;
   }
   status_ = errors::Internal("BundleWriter is closed");
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Merging tensor bundles.
@@ -604,8 +613,8 @@ struct MergeState {
 
 // Merges entries of "prefix" into the accumulator state "merge".
 // Returns OK iff the merge succeeds.
-static Status MergeOneBundle(Env* env, StringPiece prefix,
-                             MergeState* merge_state) {
+static absl::Status MergeOneBundle(Env* env, absl::string_view prefix,
+                                   MergeState* merge_state) {
   VLOG(1) << "Merging bundle:" << prefix;
   const string filename = MetaFilename(prefix);
   uint64 file_size;
@@ -628,7 +637,7 @@ static Status MergeOneBundle(Env* env, StringPiece prefix,
                               "failed to seek to header entry");
     }
     BundleHeaderProto header;
-    Status s = ParseEntryProto(iter->key(), iter->value(), &header);
+    absl::Status s = ParseEntryProto(iter->key(), iter->value(), &header);
     if (!s.ok()) return CorruptFileError(s, filename, "unable to parse header");
 
     merge_state->num_shards += header.num_shards();
@@ -700,16 +709,17 @@ static Status MergeOneBundle(Env* env, StringPiece prefix,
     to_merge_entry.set_shard_id(result.first->second);
     merge_state->entries[key] = to_merge_entry;
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status MergeBundles(Env* env, gtl::ArraySlice<tstring> prefixes,
-                    StringPiece merged_prefix, bool allow_missing_files) {
+absl::Status MergeBundles(Env* env, absl::Span<const tstring> prefixes,
+                          absl::string_view merged_prefix,
+                          bool allow_missing_files) {
   // Merges all metadata tables.
   // TODO(zhifengc): KeyValue sorter if it becomes too big.
   MergeState merge;
-  Status status = env->CreateDir(string(io::Dirname(merged_prefix)));
-  if (!status.ok() && !errors::IsAlreadyExists(status)) return status;
+  absl::Status status = env->CreateDir(string(io::Dirname(merged_prefix)));
+  if (!status.ok() && !absl::IsAlreadyExists(status)) return status;
   bool atleast_one_file_exists = false;
   for (auto& prefix : prefixes) {
     if (!env->FileExists(MetaFilename(prefix)).ok()) {
@@ -766,16 +776,28 @@ Status MergeBundles(Env* env, gtl::ArraySlice<tstring> prefixes,
 // Interface for reading a tensor bundle.
 
 BundleReader::BundleReader(
-    Env* env, StringPiece prefix,
+    Env* env, absl::string_view prefix,
     bool enable_multi_threading_for_testing /* = false */)
+    : BundleReader(env, prefix, {nullptr, enable_multi_threading_for_testing}) {
+}
+
+BundleReader::BundleReader(Env* env, absl::string_view prefix, Options options)
     : env_(env),
       prefix_(prefix),
+      cache_(options.cache),
       metadata_(nullptr),
       table_(nullptr),
       index_cache_(nullptr),
       iter_(nullptr),
       need_to_swap_bytes_(false),
-      enable_multi_threading_for_testing_(enable_multi_threading_for_testing) {
+      enable_multi_threading_for_testing_(
+          options.enable_multi_threading_for_testing) {
+  if (cache_ == nullptr) {
+    // Make a cache for use just by this BundleReader.
+    owned_cache_ = std::make_unique<BundleCache>(env);
+    cache_ = owned_cache_.get();
+  }
+
   const string filename = MetaFilename(prefix_);
   uint64 file_size;
   status_ = env_->GetFileSize(filename, &file_size);
@@ -789,7 +811,7 @@ BundleReader::BundleReader(
 
   table::Options o;
   int64_t cache_size;
-  Status s =
+  absl::Status s =
       ReadInt64FromEnvVar("TF_TABLE_INDEX_CACHE_SIZE_IN_MB", 0, &cache_size);
   if (s.ok() && cache_size > 0) {
     index_cache_ = table::NewLRUCache(cache_size << 20);
@@ -830,12 +852,6 @@ BundleReader::~BundleReader() {
   if (index_cache_) {
     delete index_cache_;
   }
-  // InputBuffer does not own the underlying RandomAccessFile.
-  for (auto pair : data_) {
-    if (pair.second != nullptr && pair.second->file() != nullptr) {
-      delete pair.second->file();
-    }
-  }
   for (auto& temp : data_) {
     delete temp.second;
   }
@@ -846,8 +862,8 @@ BundleReader::~BundleReader() {
   tensor_slices_.clear();
 }
 
-Status BundleReader::GetBundleEntryProto(StringPiece key,
-                                         BundleEntryProto* entry) {
+absl::Status BundleReader::GetBundleEntryProto(absl::string_view key,
+                                               BundleEntryProto* entry) {
   entry->Clear();
   TF_CHECK_OK(status_);
   Seek(key);
@@ -864,10 +880,11 @@ Status BundleReader::GetBundleEntryProto(StringPiece key,
   }
 
   entry->Swap(&entry_copy);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status BundleReader::GetValue(const BundleEntryProto& entry, Tensor* val) {
+absl::Status BundleReader::GetValue(const BundleEntryProto& entry,
+                                    Tensor* val) {
   Tensor* ret = val;
   const TensorShape stored_shape(TensorShape(entry.shape()));
   if (val->NumElements() == 0) {
@@ -899,11 +916,10 @@ Status BundleReader::GetValue(const BundleEntryProto& entry, Tensor* val) {
   // Open the data file if it has not been opened.
   io::InputBuffer* buffered_file = data_[entry.shard_id()];
   if (buffered_file == nullptr) {
-    std::unique_ptr<RandomAccessFile> file = nullptr;
-    TF_RETURN_IF_ERROR(env_->NewRandomAccessFile(
+    RandomAccessFile* file = nullptr;
+    TF_RETURN_IF_ERROR(cache_->GetFile(
         DataFilename(prefix_, entry.shard_id(), num_shards_), &file));
-    buffered_file = new io::InputBuffer(file.release(), kBufferSize);
-    // The InputBuffer and RandomAccessFile objects are both released in dtor.
+    buffered_file = new io::InputBuffer(file, kBufferSize);
     data_[entry.shard_id()] = buffered_file;
   }
   CHECK(buffered_file != nullptr);
@@ -914,12 +930,12 @@ Status BundleReader::GetValue(const BundleEntryProto& entry, Tensor* val) {
   if (DataTypeCanUseMemcpy(entry.dtype())) {
     char* backing_buffer = const_cast<char*>((ret->tensor_data().data()));
     size_t unused_bytes_read;
-    if (entry.size() > kBufferSize) {
-      StringPiece sp;
+    if (entry.size() > kBufferSize || enable_multi_threading_for_testing_) {
+      absl::string_view sp;
       if (!enable_multi_threading_for_testing_ &&
           entry.size() < kLargeTensorThreshold) {
         TF_RETURN_IF_ERROR(buffered_file->file()->Read(
-            entry.offset(), entry.size(), &sp, backing_buffer));
+            entry.offset(), sp, absl::MakeSpan(backing_buffer, entry.size())));
         if (sp.data() != backing_buffer) {
           memmove(backing_buffer, sp.data(), entry.size());
         }
@@ -934,7 +950,7 @@ Status BundleReader::GetValue(const BundleEntryProto& entry, Tensor* val) {
               (entry.size() + kMaxFileReadThreads - 1) / kMaxFileReadThreads;
         }
 
-        std::vector<Status> statuses(thread_pool_size);
+        std::vector<absl::Status> statuses(thread_pool_size);
         auto reader_pool = std::make_unique<thread::ThreadPool>(
             Env::Default(), "restore_large_tensor", thread_pool_size);
 
@@ -944,7 +960,7 @@ Status BundleReader::GetValue(const BundleEntryProto& entry, Tensor* val) {
             int64_t size = i == thread_pool_size - 1 ? entry.size() - offset
                                                      : section_size;
             std::unique_ptr<RandomAccessFile> section_reader = nullptr;
-            StringPiece sp;
+            absl::string_view sp;
             if (auto file_status = env_->NewRandomAccessFile(
                     DataFilename(prefix_, entry.shard_id(), num_shards_),
                     &section_reader);
@@ -954,14 +970,17 @@ Status BundleReader::GetValue(const BundleEntryProto& entry, Tensor* val) {
             }
 
             auto backing_buffer_current_pos = backing_buffer + offset;
-            auto status = section_reader->Read(entry.offset() + offset, size,
-                                               &sp, backing_buffer_current_pos);
+            auto status = section_reader->Read(
+                entry.offset() + offset, sp,
+                absl::MakeSpan(backing_buffer_current_pos, size));
             if (sp.data() != backing_buffer_current_pos) {
               memmove(backing_buffer_current_pos, sp.data(), size);
             }
             statuses[i] = std::move(status);
           });
         }
+        reader_pool = nullptr;  // Wait for reads to finish
+
         for (const auto& status : statuses) {
           TF_RETURN_IF_ERROR(status);
         }
@@ -1005,10 +1024,10 @@ Status BundleReader::GetValue(const BundleEntryProto& entry, Tensor* val) {
 
   *val = *ret;
   if (ret != val) delete ret;
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status BundleReader::Lookup(StringPiece key, Tensor* val) {
+absl::Status BundleReader::Lookup(absl::string_view key, Tensor* val) {
   CHECK(val != nullptr);
   BundleEntryProto entry;
   TF_RETURN_IF_ERROR(GetBundleEntryProto(key, &entry));
@@ -1022,7 +1041,7 @@ Status BundleReader::Lookup(StringPiece key, Tensor* val) {
   }
 }
 
-Status BundleReader::ReadCurrent(Tensor* val) {
+absl::Status BundleReader::ReadCurrent(Tensor* val) {
   CHECK(val != nullptr);
   BundleEntryProto entry;
   TF_RETURN_IF_ERROR(ParseEntryProto(iter_->key(), iter_->value(), &entry));
@@ -1040,8 +1059,8 @@ Status BundleReader::ReadCurrent(Tensor* val) {
   }
 }
 
-Status BundleReader::LookupTensorSlices(StringPiece key,
-                                        std::vector<TensorSlice>* slices) {
+absl::Status BundleReader::LookupTensorSlices(
+    absl::string_view key, std::vector<TensorSlice>* slices) {
   slices->clear();
   BundleEntryProto entry;
   TF_RETURN_IF_ERROR(GetBundleEntryProto(key, &entry));
@@ -1049,20 +1068,22 @@ Status BundleReader::LookupTensorSlices(StringPiece key,
   for (const auto& slice : entry.slices()) {
     slices->emplace_back(slice);
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status BundleReader::LookupSlice(StringPiece full_tensor_key,
-                                 const TensorSlice& slice_spec, Tensor* val) {
+absl::Status BundleReader::LookupSlice(absl::string_view full_tensor_key,
+                                       const TensorSlice& slice_spec,
+                                       Tensor* val) {
   CHECK(val != nullptr);
   BundleEntryProto entry;
   TF_RETURN_IF_ERROR(GetBundleEntryProto(full_tensor_key, &entry));
   return GetSliceValue(full_tensor_key, entry, slice_spec, val);
 }
 
-Status BundleReader::GetSliceValue(StringPiece full_tensor_key,
-                                   const BundleEntryProto& full_tensor_entry,
-                                   const TensorSlice& slice_spec, Tensor* val) {
+absl::Status BundleReader::GetSliceValue(
+    absl::string_view full_tensor_key,
+    const BundleEntryProto& full_tensor_entry, const TensorSlice& slice_spec,
+    Tensor* val) {
   using checkpoint::RegisterTensorSlice;
   using checkpoint::TensorSliceSet;
   DCHECK_GE(full_tensor_entry.slices_size(), 0);
@@ -1166,30 +1187,34 @@ Status BundleReader::GetSliceValue(StringPiece full_tensor_key,
       HANDLE_COPY(quint8)
       HANDLE_COPY(qint8)
       HANDLE_COPY(bfloat16)
+      HANDLE_COPY(int4)
+      HANDLE_COPY(uint4)
       default:
         return errors::InvalidArgument("Dtype ", DataTypeString(common_dtype),
                                        " not supported.");
     }
 #undef HANDLE_COPY
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-bool BundleReader::Contains(StringPiece key) {
+bool BundleReader::Contains(absl::string_view key) {
   Seek(key);
   return Valid() && (this->key() == key);
 }
 
-Status BundleReader::LookupDtypeAndShape(StringPiece key, DataType* dtype,
-                                         TensorShape* shape) {
+absl::Status BundleReader::LookupDtypeAndShape(absl::string_view key,
+                                               DataType* dtype,
+                                               TensorShape* shape) {
   BundleEntryProto entry;
   TF_RETURN_IF_ERROR(GetBundleEntryProto(key, &entry));
   *dtype = entry.dtype();
   *shape = TensorShape(entry.shape());
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status BundleReader::LookupTensorShape(StringPiece key, TensorShape* shape) {
+absl::Status BundleReader::LookupTensorShape(absl::string_view key,
+                                             TensorShape* shape) {
   DataType ignored;
   return LookupDtypeAndShape(key, &ignored, shape);
 }
@@ -1210,6 +1235,36 @@ string BundleReader::DebugString() {
   return shape_str;
 }
 
+BundleCache::BundleCache(Env* env) : env_(env) {}
+
+BundleCache::FileState* BundleCache::EnsureOpened(std::string name) {
+  // Get the file, opening it if necessary.
+  FileState* f;
+  {
+    absl::MutexLock l(&mu_);
+    auto& slot = opened_files_[name];
+    if (slot == nullptr) {
+      slot = std::make_unique<FileState>();
+    }
+    f = slot.get();
+  }
+
+  // Open the file or wait for a concurrent open to complete. We do not hold
+  // mu_ here to avoid blocking threads reading from other files.
+  absl::call_once(f->once, [this, name = std::move(name), f] {
+    f->open_status = env_->NewRandomAccessFile(name, &f->file);
+  });
+
+  return f;
+}
+
+absl::Status BundleCache::GetFile(const std::string& fname,
+                                  RandomAccessFile** file) {
+  FileState* f = EnsureOpened(fname);
+  *file = f->file.get();
+  return f->open_status;
+}
+
 namespace {
 inline char* AlignedMalloc(size_t size) {
   char* buffer = static_cast<char*>(port::AlignedMalloc(size, 64));
