@@ -15,23 +15,38 @@ limitations under the License.
 
 #include "tensorflow/dtensor/mlir/dtensor_send_recv.h"
 
+#include <cassert>
+#include <cstddef>
+#include <cstdint>
+#include <optional>
 #include <string>
+#include <vector>
 
+#include "absl/container/flat_hash_set.h"
+#include "absl/status/status.h"
+#include "absl/types/span.h"
 #include "llvm/ADT/SmallVector.h"
 #include "llvm/Support/FormatVariadic.h"
 #include "mlir/IR/Attributes.h"  // from @llvm-project
 #include "mlir/IR/BuiltinAttributes.h"  // from @llvm-project
+#include "mlir/IR/BuiltinTypes.h"  // from @llvm-project
 #include "mlir/Support/DebugStringHelper.h"  // from @llvm-project
+#include "mlir/Support/LLVM.h"  // from @llvm-project
+#include "tensorflow/compiler/mlir/tensorflow/ir/tf_attributes.h"
 #include "tensorflow/compiler/mlir/tensorflow/ir/tf_device.h"
 #include "tensorflow/compiler/mlir/tensorflow/ir/tf_ops.h"
 #include "tensorflow/compiler/mlir/tensorflow/transforms/collection_ops_util.h"
 #include "tensorflow/compiler/mlir/tensorflow/utils/convert_tensor.h"
 #include "tensorflow/core/platform/errors.h"
 #include "tensorflow/dtensor/cc/constants.h"
+#include "tensorflow/dtensor/cc/dstatus.h"
+#include "tensorflow/dtensor/cc/tensor_layout.h"
 #include "tensorflow/dtensor/mlir/collectives.h"
 #include "tensorflow/dtensor/mlir/device_utils.h"
+#include "tensorflow/dtensor/mlir/ir/tf_dtensor.h"
 #include "tensorflow/dtensor/mlir/layout_parsing.h"
 #include "tensorflow/dtensor/mlir/op_utils.h"
+#include "tensorflow/dtensor/mlir/shape_utils.h"
 #include "tensorflow/dtensor/mlir/spmd_expander_common.h"
 #include "tensorflow/dtensor/mlir/value_utils.h"
 
@@ -40,14 +55,14 @@ namespace dtensor {
 namespace {
 
 bool IsStringType(mlir::Type type) {
-  if (type.isa<mlir::TF::StringType>()) return true;
+  if (mlir::isa<mlir::TF::StringType>(type)) return true;
 
-  auto sub_type = type.dyn_cast<mlir::TF::TensorFlowTypeWithSubtype>();
+  auto sub_type = mlir::dyn_cast<mlir::TF::TensorFlowTypeWithSubtype>(type);
   if (!sub_type) return false;
 
   bool has_string =
       llvm::any_of(sub_type.GetSubtypes(), [](mlir::TensorType type) {
-        return type.getElementType().isa<mlir::TF::StringType>();
+        return mlir::isa<mlir::TF::StringType>(type.getElementType());
       });
   return has_string;
 }
@@ -181,6 +196,25 @@ StatusOr<mlir::Operation*> LowerDTensorSendToXlaOp(
   return lowered_send_op;
 }
 
+// Creates a shape attribute of the local shape version of RecvFromOp's result.
+StatusOr<mlir::TF::ShapeAttr> GetDTensorRecvLocalShapeAttr(
+    mlir::TF::DTensorRecv dtensor_recv) {
+  if (dtensor_recv->getNumResults() != 1) {
+    return absl::InvalidArgumentError(
+        "XlaRecvFromHostOp must have exactly one result.");
+  }
+  TF_ASSIGN_OR_RETURN(std::vector<Layout> layouts,
+                      ExtractRequiredLayoutFromOp(dtensor_recv));
+  if (layouts.empty() || layouts.size() > 1) {
+    return absl::InvalidArgumentError(
+        "invalid layout for XlaRecvFromHostOp specified");
+  }
+  auto result_type = dtensor_recv->getResult(0);
+  TF_ASSIGN_OR_RETURN(auto result_shape, GetShapeOfValue(result_type));
+  auto local_shape = layouts[0].LocalShapeFromGlobalShape(result_shape);
+  return mlir::TF::ShapeAttr::get(result_type.getContext(), local_shape);
+}
+
 // Lowers DTensorRecv op to either one of XlaRecvAtHost or XlaRecvFromHost,
 // depending on src mesh cluster configuration. `output_type` can be set to the
 // specific local tensor type needed, if different from the Recv op output type.
@@ -216,10 +250,12 @@ StatusOr<mlir::Operation*> LowerDTensorRecvToXlaOp(
         dtensor_recv.getLoc(), output_types,
         /*dynamic_key=*/program_key, device_ordinal, dtensor_recv.getKeyAttr());
   } else {
+    TF_ASSIGN_OR_RETURN(auto local_shape_attr,
+                        GetDTensorRecvLocalShapeAttr(dtensor_recv));
+
     // Create XlaRecvFromHost op.
     recv_xla_op = builder.create<mlir::TF::XlaRecvFromHostOp>(
-        dtensor_recv.getLoc(), output_type,
-        ConvertTypeToTensorShapeAttr(dtensor_recv.getType()),
+        dtensor_recv.getLoc(), output_type, local_shape_attr,
         dtensor_recv.getKeyAttr());
   }
 
@@ -392,7 +428,7 @@ StatusOr<mlir::Operation*> LowerOneToOneDTensorSendToTFHostSend(
                            op_builder.getStringAttr(send_layout.ToString()));
         mlir::Value val = arg;
         if (i32_copy) {
-          auto val_type = val.getType().cast<mlir::TensorType>();
+          auto val_type = mlir::cast<mlir::TensorType>(val.getType());
           val = op_builder
                     .create<mlir::TF::CastOp>(
                         loc,
@@ -644,7 +680,7 @@ StatusOr<mlir::Operation*> LowerDTensorSend(mlir::Operation* send_op,
                                                       dtensor_send));
     } else {
       mlir::TensorType send_type =
-          send_input.getType().cast<mlir::TensorType>();
+          mlir::cast<mlir::TensorType>(send_input.getType());
       if (!recv_mesh.is_cpu_mesh() &&
           send_type.getElementType().isInteger(32)) {
         builder.setInsertionPointAfter(send_input.getDefiningOp());
@@ -716,7 +752,7 @@ StatusOr<mlir::Operation*> LowerDTensorRecv(mlir::Operation* send_op,
     TF_ASSIGN_OR_RETURN(
         mlir::TensorType local_output_type,
         LocalTypeFromGlobalType(
-            recv_layout, dtensor_recv.getType().cast<mlir::TensorType>()));
+            recv_layout, mlir::cast<mlir::TensorType>(dtensor_recv.getType())));
     TF_ASSIGN_OR_RETURN(
         lowered_recv, LowerDTensorRecvToXlaOp(dtensor_recv, local_output_type));
     dtensor_recv->replaceAllUsesWith(lowered_recv);
