@@ -15,18 +15,33 @@ limitations under the License.
 
 #include "tensorflow/dtensor/mlir/expansions/conv_spmd_expander.h"
 
+#include <cassert>
+#include <cstddef>
+#include <cstdint>
 #include <string>
+#include <vector>
 
+#include "absl/status/status.h"
 #include "llvm/ADT/SmallVector.h"
 #include "llvm/Support/Casting.h"
 #include "llvm/Support/FormatVariadic.h"
+#include "mlir/IR/Attributes.h"  // from @llvm-project
+#include "mlir/IR/Builders.h"  // from @llvm-project
 #include "mlir/IR/BuiltinAttributes.h"  // from @llvm-project
 #include "mlir/IR/BuiltinTypes.h"  // from @llvm-project
+#include "mlir/IR/Value.h"  // from @llvm-project
+#include "mlir/IR/ValueRange.h"  // from @llvm-project
+#include "mlir/Support/LLVM.h"  // from @llvm-project
+#include "tensorflow/compiler/mlir/tensorflow/ir/tf_attributes.h"
 #include "tensorflow/compiler/mlir/tensorflow/ir/tf_device.h"
+#include "tensorflow/compiler/mlir/tensorflow/ir/tf_ops.h"
 #include "tensorflow/core/platform/errors.h"
+#include "tensorflow/core/platform/types.h"
 #include "tensorflow/dtensor/cc/dstatus.h"
 #include "tensorflow/dtensor/cc/tensor_layout.h"
 #include "tensorflow/dtensor/mlir/collectives.h"
+#include "tensorflow/dtensor/mlir/dtensor_dialect/ir/dtensor_attributes.h"
+#include "tensorflow/dtensor/mlir/ir/tf_dtensor.h"
 #include "tensorflow/dtensor/mlir/layout_parsing.h"
 #include "tensorflow/dtensor/mlir/op_utils.h"
 #include "tensorflow/dtensor/mlir/shape_utils.h"
@@ -39,8 +54,8 @@ namespace dtensor {
 namespace {
 
 template <typename ConvOp>
-Status VerifyConvLayout(const Layout& input_layout, const Layout& filter_layout,
-                        ConvOp conv_op) {
+absl::Status VerifyConvLayout(const Layout& input_layout,
+                              const Layout& filter_layout, ConvOp conv_op) {
   if (!filter_layout.IsFullyReplicated())
     return errors::InvalidArgument(
         "Filter for convolution must have fully replicated layout.");
@@ -58,7 +73,7 @@ Status VerifyConvLayout(const Layout& input_layout, const Layout& filter_layout,
 
   if (input_layout.IsBatchParallel() || input_layout.IsFullyReplicated())
     // No further checks needed for replicated case.
-    return OkStatus();
+    return absl::OkStatus();
 
   if (conv_op.getPadding() == "EXPLICIT")
     return errors::InvalidArgument(
@@ -67,7 +82,7 @@ Status VerifyConvLayout(const Layout& input_layout, const Layout& filter_layout,
 
   const int num_non_default_dilations =
       llvm::count_if(conv_op.getDilations(), [](mlir::Attribute dilation) {
-        return dilation.cast<mlir::IntegerAttr>().getInt() != 1;
+        return mlir::cast<mlir::IntegerAttr>(dilation).getInt() != 1;
       });
   if (num_non_default_dilations > 0)
     return errors::InvalidArgument(
@@ -77,21 +92,21 @@ Status VerifyConvLayout(const Layout& input_layout, const Layout& filter_layout,
   // TODO(b/208700444): support convolution with strides greater than 1.
   const int num_non_default_strides =
       llvm::count_if(conv_op.getStrides(), [](mlir::Attribute stride) {
-        return stride.cast<mlir::IntegerAttr>().getInt() != 1;
+        return mlir::cast<mlir::IntegerAttr>(stride).getInt() != 1;
       });
   if (num_non_default_strides > 0)
     return errors::InvalidArgument(
         "Only stride 1 is supported for convolution with spatial partitions.");
 
   mlir::Value input = conv_op.getInput();
-  auto input_type = input.getType().dyn_cast<mlir::RankedTensorType>();
+  auto input_type = mlir::dyn_cast<mlir::RankedTensorType>(input.getType());
   if (!input_type || !input_type.hasStaticShape())
     return errors::InvalidArgument(
         "Input must have static shapes for convolution with spatial "
         "partitions.");
 
   mlir::Value filter = conv_op.getFilter();
-  auto filter_type = filter.getType().dyn_cast<mlir::RankedTensorType>();
+  auto filter_type = mlir::dyn_cast<mlir::RankedTensorType>(filter.getType());
   if (!filter_type || !filter_type.hasStaticShape())
     return errors::InvalidArgument(
         "Filter must have static shapes for convolution with spatial "
@@ -105,7 +120,7 @@ Status VerifyConvLayout(const Layout& input_layout, const Layout& filter_layout,
           "spatial partitions.");
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 mlir::Value PadInputOnUnshardedDim(mlir::OpBuilder& builder,
@@ -113,7 +128,7 @@ mlir::Value PadInputOnUnshardedDim(mlir::OpBuilder& builder,
                                    mlir::Value input_tensor, int curr_input_dim,
                                    int64_t curr_filter_dim_size) {
   auto input_tensor_type =
-      input_tensor.getType().dyn_cast<mlir::RankedTensorType>();
+      mlir::dyn_cast<mlir::RankedTensorType>(input_tensor.getType());
   auto input_tensor_shape = input_tensor_type.getShape();
 
   const size_t paddings_flat_length = input_tensor_type.getRank() * 2;
@@ -170,7 +185,7 @@ StatusOr<mlir::Operation*> HandleConv(ConvOp conv_op) {
   const auto output_num_shards = output_layout.num_shards();
 
   auto filter_type =
-      conv_op.getFilter().getType().template dyn_cast<mlir::RankedTensorType>();
+      mlir::dyn_cast<mlir::RankedTensorType>(conv_op.getFilter().getType());
   auto filter_shape = filter_type.getShape();
 
   int begin_input_dim = -1, end_input_dim = -1;
@@ -193,9 +208,8 @@ StatusOr<mlir::Operation*> HandleConv(ConvOp conv_op) {
        ++curr_input_dim) {
     int curr_filter_dim = curr_input_dim - begin_input_dim;
 
-    auto input_type = conv_op.getInput()
-                          .getType()
-                          .template dyn_cast<mlir::RankedTensorType>();
+    auto input_type =
+        mlir::dyn_cast<mlir::RankedTensorType>(conv_op.getInput().getType());
     auto input_shape = input_type.getShape();
 
     if (input_sharding_spec[curr_input_dim] == Layout::kUnshardedDim) {
@@ -271,12 +285,18 @@ StatusOr<mlir::Operation*> HandleConv(ConvOp conv_op) {
                                                input_shape.end());
       slice_size[curr_input_dim] += halo_size;
       mlir::Value slice_size_const = Int64Const(builder, location, slice_size);
+      // slice_size_const and slize_begin_int64 has to be same type.
+      mlir::Value slice_begin_int64 = builder.create<mlir::TF::CastOp>(
+          location,
+          mlir::RankedTensorType::get({input_layout.rank()},
+                                      builder.getI64Type()),
+          slice_begin);
 
       mlir::RankedTensorType sliced_input_type =
           mlir::RankedTensorType::get(slice_size, input_type.getElementType());
       mlir::Value sliced_input = builder.create<mlir::TF::SliceOp>(
           location, sliced_input_type, /*input=*/halo_exchanged_input,
-          /*begin=*/slice_begin, /*size=*/slice_size_const);
+          /*begin=*/slice_begin_int64, /*size=*/slice_size_const);
       conv_op->setOperand(0, sliced_input);
     }
 
@@ -316,7 +336,7 @@ StatusOr<mlir::Operation*> HandleConvBackpropInput(
   }
 
   llvm::SmallVector<int64_t, 4> global_shape;
-  Status extract_status =
+  absl::Status extract_status =
       ExtractConstVectorFromValue(conv_op.getInputSizes(), &global_shape);
 
   // If the input is dynamic size, we expect the output is all so dynamic size
@@ -403,7 +423,7 @@ StatusOr<mlir::Operation*> HandleConvBackpropInputTensor(
   // HandleConvBackpropInput which expects there to be a layout here.
   mlir::TF::ShapeAttr global_input_shape_shape = mlir::TF::ShapeAttr::get(
       builder.getContext(),
-      global_input_shape.getType().cast<mlir::TensorType>());
+      mlir::cast<mlir::TensorType>(global_input_shape.getType()));
   mlir::TF::DTensorLayout global_input_shape_with_layout =
       builder.create<mlir::TF::DTensorLayout>(
           conv_op->getLoc(), global_input_shape,
@@ -529,7 +549,7 @@ StatusOr<mlir::Operation*> HandleConvBackpropFilterTensor(
   // HandleConvBackpropInput which expects there to be a layout here.
   mlir::TF::ShapeAttr global_filter_shape_shape = mlir::TF::ShapeAttr::get(
       builder.getContext(),
-      global_filter_shape_const.getType().cast<mlir::TensorType>());
+      mlir::cast<mlir::TensorType>(global_filter_shape_const.getType()));
   mlir::TF::DTensorLayout global_filter_shape_with_layout =
       builder.create<mlir::TF::DTensorLayout>(
           conv_op->getLoc(), global_filter_shape_const,
