@@ -15,12 +15,28 @@ limitations under the License.
 
 #include "tensorflow/dtensor/mlir/expansions/einsum_spmd_expander.h"
 
+#include <cassert>
+#include <cstddef>
 #include <string>
+#include <utility>
+#include <vector>
 
+#include "absl/container/flat_hash_map.h"
+#include "absl/container/flat_hash_set.h"
+#include "absl/status/status.h"
+#include "absl/strings/str_split.h"
 #include "absl/strings/string_view.h"
+#include "llvm/ADT/DenseMap.h"
+#include "llvm/ADT/STLExtras.h"
 #include "llvm/Support/FormatVariadic.h"
-#include "mlir/IR/BuiltinTypes.h"  // from @llvm-project
+#include "mlir/IR/Builders.h"  // from @llvm-project
 #include "mlir/IR/IRMapping.h"  // from @llvm-project
+#include "mlir/IR/Operation.h"  // from @llvm-project
+#include "mlir/IR/Value.h"  // from @llvm-project
+#include "mlir/Support/LLVM.h"  // from @llvm-project
+#include "tensorflow/compiler/mlir/tensorflow/ir/tf_ops.h"
+#include "tensorflow/core/platform/errors.h"
+#include "tensorflow/dtensor/cc/dstatus.h"
 #include "tensorflow/dtensor/cc/tensor_layout.h"
 #include "tensorflow/dtensor/mlir/collectives.h"
 #include "tensorflow/dtensor/mlir/layout_parsing.h"
@@ -85,7 +101,7 @@ StatusOr<mlir::Operation*> EinsumSPMDExpander::ExpandOp(mlir::Operation* op) {
 // input_mappings: for each equation input, the map from the equation labels
 //   to the tensor dimension of that label.
 // output_mapping: as above, but for the equation output.
-Status ExtractEquationRelations(
+absl::Status ExtractEquationRelations(
     absl::string_view equation, absl::flat_hash_set<char>& reduced_dims,
     std::vector<absl::flat_hash_map<char, std::vector<int>>>& input_mappings,
     absl::flat_hash_map<char, std::vector<int>>& output_mapping) {
@@ -134,7 +150,7 @@ Status ExtractEquationRelations(
     }
   }
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // For a set of layouts and mappings from labels to offsets in the layouts,
@@ -222,7 +238,6 @@ GetSpecsFromLabelsAndMap(
 
   std::vector<std::string> sharding_specs(layout_rank);
   absl::flat_hash_map<std::string, int> dimension_use_count;
-  absl::flat_hash_set<std::string> dimension_use_set;
   for (const auto& label_and_indices : label_to_index) {
     const auto& loc = label_to_sharding_spec.find(label_and_indices.first);
     if (loc != label_to_sharding_spec.end()) {
@@ -376,7 +391,7 @@ StatusOr<llvm::DenseMap<int, Layout>> EinsumSPMDExpander::ComputeLayoutBackward(
 //   for x is sharded. If both are sharded, we can compute the einsum on the
 //   diagonal machines in the mesh and 0s on the off diagonals and then all
 //   the much smaller matrix.
-Status EinsumSPMDExpander::MaybeRelayoutInputs(
+absl::Status EinsumSPMDExpander::MaybeRelayoutInputs(
     const std::vector<Layout>& input_layouts, mlir::Operation* op,
     const Layout& output_layout, absl::flat_hash_set<std::string>& reduce_dims,
     Layout& einsum_layout, std::vector<mlir::Value>& new_inputs) {
@@ -545,7 +560,7 @@ Status EinsumSPMDExpander::MaybeRelayoutInputs(
   for (const auto& contracting : contracting_labels)
     reduce_dims.emplace(input_label_to_sharding_spec[contracting]);
 
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace dtensor
