@@ -21,32 +21,24 @@ limitations under the License.
 #include <utility>
 #include <vector>
 
-#include "absl/container/flat_hash_set.h"
 #include "absl/status/status.h"
 #include "absl/strings/str_cat.h"
-#include "llvm/ADT/SmallSet.h"
 #include "llvm/ADT/SmallVector.h"
 #include "llvm/ADT/TypeSwitch.h"
 #include "mlir/IR/Builders.h"  // from @llvm-project
 #include "mlir/IR/BuiltinTypes.h"  // from @llvm-project
-#include "mlir/IR/IRMapping.h"  // from @llvm-project
+#include "mlir/IR/Location.h"  // from @llvm-project
 #include "mlir/IR/Operation.h"  // from @llvm-project
 #include "mlir/IR/Value.h"  // from @llvm-project
-#include "mlir/Support/DebugStringHelper.h"  // from @llvm-project
-#include "tensorflow/cc/framework/ops.h"
+#include "mlir/Support/LLVM.h"  // from @llvm-project
 #include "tensorflow/compiler/mlir/tensorflow/ir/tf_ops.h"
-#include "tensorflow/core/platform/errors.h"
-#include "tensorflow/core/platform/statusor.h"
+#include "tensorflow/core/platform/types.h"
 #include "tensorflow/dtensor/cc/dstatus.h"
 #include "tensorflow/dtensor/cc/tensor_layout.h"
 #include "tensorflow/dtensor/mlir/collectives.h"
-#include "tensorflow/dtensor/mlir/expansions/meta_spmd_expander.h"
-#include "tensorflow/dtensor/mlir/ir/tf_dtensor.h"
 #include "tensorflow/dtensor/mlir/layout_parsing.h"
 #include "tensorflow/dtensor/mlir/op_utils.h"
 #include "tensorflow/dtensor/mlir/shape_utils.h"
-#include "tensorflow/dtensor/mlir/spmd_expander.h"
-#include "tensorflow/dtensor/mlir/spmd_expander_common.h"
 #include "tensorflow/dtensor/mlir/value_utils.h"
 
 namespace tensorflow {
@@ -74,11 +66,11 @@ bool IsDistributedFFTN(int num_transform_axes, const Layout& layout) {
 bool IsComplexFFT(mlir::Value input) {
   auto data_type =
       mlir::dyn_cast<mlir::TensorType>(input.getType()).getElementType();
-  return data_type.isa<mlir::ComplexType>();
+  return mlir::isa<mlir::ComplexType>(data_type);
 }
 
-Status IsProperFFTLength(mlir::Operation* op,
-                         const llvm::SmallVector<int64_t, 4>& fft_length_vec) {
+absl::Status IsProperFFTLength(
+    mlir::Operation* op, const llvm::SmallVector<int64_t, 4>& fft_length_vec) {
   TF_ASSIGN_OR_RETURN(auto input_layout,
                       ExtractRequiredLayoutFromOperand(op->getOperand(0)));
   const Mesh& mesh = input_layout.mesh();
@@ -167,7 +159,7 @@ StatusOr<mlir::Value> EmitTransposeRelayout(mlir::OpBuilder& builder,
   return transposed_input;
 }
 
-Status NormalizeAxes(std::vector<int>& transform_axes, int input_rank) {
+absl::Status NormalizeAxes(std::vector<int>& transform_axes, int input_rank) {
   std::sort(transform_axes.begin(), transform_axes.end());
   for (int i = 0; i < transform_axes.size(); ++i) {
     if (transform_axes[i] >= input_rank) {
