@@ -15,17 +15,30 @@ limitations under the License.
 
 #include "tensorflow/dtensor/mlir/expansions/softmax_spmd_expander.h"
 
+#include <cassert>
+#include <cstdint>
 #include <optional>
 #include <string>
 #include <vector>
 
+#include "absl/container/flat_hash_set.h"
+#include "absl/status/status.h"
 #include "absl/strings/str_cat.h"
+#include "absl/strings/string_view.h"
 #include "llvm/ADT/SmallPtrSet.h"
-#include "llvm/Support/Alignment.h"
-#include "mlir/IR/IRMapping.h"  // from @llvm-project
+#include "mlir/IR/Builders.h"  // from @llvm-project
+#include "mlir/IR/BuiltinAttributes.h"  // from @llvm-project
+#include "mlir/IR/BuiltinTypes.h"  // from @llvm-project
+#include "mlir/IR/Location.h"  // from @llvm-project
 #include "mlir/IR/Operation.h"  // from @llvm-project
+#include "mlir/IR/Types.h"  // from @llvm-project
 #include "mlir/IR/Value.h"  // from @llvm-project
+#include "mlir/Support/LLVM.h"  // from @llvm-project
+#include "tensorflow/compiler/mlir/tensorflow/ir/tf_device.h"
 #include "tensorflow/compiler/mlir/tensorflow/ir/tf_ops.h"
+#include "tensorflow/core/platform/errors.h"
+#include "tensorflow/core/platform/types.h"
+#include "tensorflow/dtensor/cc/dstatus.h"
 #include "tensorflow/dtensor/cc/tensor_layout.h"
 #include "tensorflow/dtensor/mlir/collectives.h"
 #include "tensorflow/dtensor/mlir/layout_parsing.h"
@@ -79,10 +92,8 @@ StatusOr<mlir::Value> ComputeGlobalReduce(
                     local_reduce, reduce_op));
 
   if (!keep_dims) {
-    mlir::RankedTensorType output_type =
-        global_reduce->getResult(0)
-            .getType()
-            .dyn_cast<mlir::RankedTensorType>();
+    mlir::RankedTensorType output_type = mlir::dyn_cast<mlir::RankedTensorType>(
+        global_reduce->getResult(0).getType());
     if (!output_type)
       return errors::Internal(
           "output of EmitAllReduce is not a RankedTensorType");
@@ -105,11 +116,12 @@ StatusOr<mlir::Value> ComputeGlobalReduce(
 
 // Takes a sharded logits and compute both the shifted exponentiation of the
 // logits and its sum. Assumes that builder's insertion point is after logits.
-Status ComputeExpAndSum(mlir::OpBuilder& builder, const mlir::Value& logits,
-                        const Layout& logits_layout,
-                        mlir::Value& shifted_logits,
-                        mlir::Value& exp_of_shifted_logits,
-                        mlir::Value& sum_of_exp) {
+absl::Status ComputeExpAndSum(mlir::OpBuilder& builder,
+                              const mlir::Value& logits,
+                              const Layout& logits_layout,
+                              mlir::Value& shifted_logits,
+                              mlir::Value& exp_of_shifted_logits,
+                              mlir::Value& sum_of_exp) {
   auto loc = logits.getLoc();
 
   if (logits_layout.rank() == 0)
@@ -141,7 +153,7 @@ Status ComputeExpAndSum(mlir::OpBuilder& builder, const mlir::Value& logits,
       ComputeGlobalReduce(builder, exp_of_shifted_logits, logits_layout,
                           {class_dimension}, kReduceOpAdd,
                           /*keep_dims=*/true));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 // Computes softmax from its components. Assumes that builder's insertion point
@@ -209,7 +221,8 @@ StatusOr<Layout> GetBroadcastedLayout(llvm::ArrayRef<int64_t> global_shape,
 // value. Assumes builder's insertion point is after input.
 StatusOr<mlir::Value> GetFPConstOfType(mlir::OpBuilder& builder,
                                        const mlir::Value& input, float value) {
-  if (mlir::TensorType type = input.getType().dyn_cast<mlir::TensorType>()) {
+  if (mlir::TensorType type =
+          mlir::dyn_cast<mlir::TensorType>(input.getType())) {
     return builder
         .create<mlir::TF::ConstOp>(
             input.getLoc(),
@@ -239,7 +252,7 @@ StatusOr<mlir::Value> ComputeOneHot(mlir::OpBuilder& builder,
   // Get the number of classes for this onehot. The number of classes is the
   // global size of the last dimension of features.
   mlir::RankedTensorType features_type =
-      features.getType().dyn_cast<mlir::RankedTensorType>();
+      mlir::dyn_cast<mlir::RankedTensorType>(features.getType());
   if (!features_type)
     return errors::InvalidArgument(
         "feature input shape must be statically known");
@@ -297,7 +310,8 @@ StatusOr<mlir::Value> ComputeOneHot(mlir::OpBuilder& builder,
 
   // Note that the type of id_offset (int32) may not match the type of input.
   // So we insert a cast in this case.
-  mlir::TensorType input_type = input.getType().dyn_cast<mlir::TensorType>();
+  mlir::TensorType input_type =
+      mlir::dyn_cast<mlir::TensorType>(input.getType());
   if (!input_type) return errors::InvalidArgument("input is not a TensorType");
   if (!input_type.getElementType().isInteger(32))
     id_offset =
@@ -655,10 +669,10 @@ SoftmaxLossOpSPMDExpander::ComputeLayoutForward(
   // if one is replicated.
   // backprop is softmax(features) - labels
 
-  absl::optional<Layout> features_layout;
+  std::optional<Layout> features_layout;
   if (input_layouts.find(0) != input_layouts.end())
     features_layout.emplace(input_layouts.lookup(0));
-  absl::optional<Layout> labels_layout;
+  std::optional<Layout> labels_layout;
   if (input_layouts.find(1) != input_layouts.end())
     labels_layout.emplace(input_layouts.lookup(1));
 
@@ -704,10 +718,10 @@ SoftmaxLossOpSPMDExpander::ComputeLayoutBackward(
   const bool is_sparse =
       mlir::isa<mlir::TF::SparseSoftmaxCrossEntropyWithLogitsOp>(op);
 
-  absl::optional<Layout> loss_layout;
+  std::optional<Layout> loss_layout;
   if (output_layouts.find(0) != output_layouts.end())
     loss_layout.emplace(output_layouts.lookup(0));
-  absl::optional<Layout> backprop_layout;
+  std::optional<Layout> backprop_layout;
   if (output_layouts.find(1) != output_layouts.end())
     backprop_layout.emplace(output_layouts.lookup(1));
 
