@@ -15,20 +15,15 @@ limitations under the License.
 
 #include "tensorflow/dtensor/cc/slice_util.h"
 
-#include <map>
-#include <memory>
-#include <ostream>
+#include <cstdint>
 #include <string>
 #include <vector>
 
-#include "mlir/IR/BuiltinTypes.h"  // from @llvm-project
-#include "tensorflow/core/platform/errors.h"
-#include "tensorflow/core/platform/logging.h"
-#include "tensorflow/core/platform/statusor.h"
+#include <gmock/gmock.h>
+#include "xla/tsl/platform/status_matchers.h"
 #include "tensorflow/core/platform/test.h"
 #include "tensorflow/dtensor/cc/tensor_layout.h"
 #include "tensorflow/dtensor/proto/layout.pb.h"
-#include "tensorflow/tsl/platform/status_matchers.h"
 
 namespace tensorflow {
 namespace dtensor {
