@@ -16,15 +16,21 @@ limitations under the License.
 #include "tensorflow/dtensor/cc/tensor_layout.h"
 
 #include <map>
-#include <memory>
 #include <ostream>
 #include <string>
 #include <vector>
 
-#include "mlir/IR/BuiltinTypes.h"  // from @llvm-project
+#include <gmock/gmock.h>
+#include "absl/container/inlined_vector.h"
+#include "absl/strings/match.h"
+#include "mlir/IR/BuiltinTypeInterfaces.h"  // from @llvm-project
+#include "xla/tsl/platform/errors.h"
+#include "xla/tsl/platform/status_matchers.h"
+#include "xla/tsl/platform/statusor.h"
 #include "tensorflow/core/platform/test.h"
+#include "tensorflow/core/platform/types.h"
+#include "tensorflow/dtensor/cc/dstatus.h"
 #include "tensorflow/dtensor/proto/layout.pb.h"
-#include "tensorflow/tsl/platform/status_matchers.h"
 
 namespace tensorflow {
 namespace dtensor {
