@@ -13,35 +13,49 @@ See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
 
-#include <setjmp.h>
 #include <stdio.h>
 #include <string.h>
 
 #include <algorithm>
 #include <cmath>
+#include <cstdint>
 #include <fstream>
 #include <memory>
 #include <string>
 #include <vector>
 
+#include "absl/log/check.h"
+#include "absl/log/log.h"
+#include "absl/status/status.h"
+#include "absl/strings/match.h"
+#include "absl/strings/numbers.h"
+#include "tensorflow/cc/framework/ops.h"
+#include "tensorflow/cc/framework/scope.h"
+#include "tensorflow/cc/ops/array_ops.h"
 #include "tensorflow/cc/ops/const_op.h"
 #include "tensorflow/cc/ops/image_ops.h"
-#include "tensorflow/cc/ops/standard_ops.h"
+#include "tensorflow/cc/ops/io_ops.h"
+#include "tensorflow/cc/ops/math_ops.h"
+#include "tensorflow/cc/ops/nn_ops.h"
+#include "xla/tsl/platform/env.h"
+#include "xla/tsl/platform/errors.h"
+#include "xla/tsl/platform/types.h"
+#include "xla/tsl/util/command_line_flags.h"
 #include "tensorflow/core/framework/graph.pb.h"
 #include "tensorflow/core/framework/tensor.h"
-#include "tensorflow/core/graph/default_device.h"
-#include "tensorflow/core/graph/graph_def_builder.h"
+#include "tensorflow/core/framework/tensor_types.h"
+#include "tensorflow/core/framework/types.pb.h"
 #include "tensorflow/core/lib/core/errors.h"
 #include "tensorflow/core/lib/core/stringpiece.h"
-#include "tensorflow/core/lib/core/threadpool.h"
-#include "tensorflow/core/lib/io/path.h"
-#include "tensorflow/core/lib/strings/numbers.h"
 #include "tensorflow/core/lib/strings/str_util.h"
-#include "tensorflow/core/lib/strings/stringprintf.h"
+#include "tensorflow/core/platform/env.h"
 #include "tensorflow/core/platform/init_main.h"
 #include "tensorflow/core/platform/logging.h"
-#include "tensorflow/core/platform/types.h"
+#include "tensorflow/core/platform/numbers.h"
+#include "tensorflow/core/platform/path.h"
+#include "tensorflow/core/platform/status.h"
 #include "tensorflow/core/public/session.h"
+#include "tensorflow/core/public/session_options.h"
 #include "tensorflow/core/util/command_line_flags.h"
 
 // These are all common classes it's handy to reference with no namespace.
@@ -68,12 +82,12 @@ Status ReadLocationsFile(const string& file_name, std::vector<float>* result,
     result->reserve(string_tokens.size());
     for (const string& string_token : string_tokens) {
       float number;
-      CHECK(tensorflow::strings::safe_strtof(string_token, &number));
+      CHECK(absl::SimpleAtof(string_token, &number));
       result->push_back(number);
     }
   }
   *found_label_count = result->size();
-  return ::tensorflow::OkStatus();
+  return absl::OkStatus();
 }
 
 // Given an image file name, read in the data, try to decode it as an image,
@@ -93,10 +107,10 @@ Status ReadTensorFromImageFile(const string& file_name, const int input_height,
   // Now try to figure out what kind of file it is and decode it.
   const int wanted_channels = 3;
   tensorflow::Output image_reader;
-  if (tensorflow::str_util::EndsWith(file_name, ".png")) {
+  if (absl::EndsWith(file_name, ".png")) {
     image_reader = DecodePng(root.WithOpName("png_reader"), file_reader,
                              DecodePng::Channels(wanted_channels));
-  } else if (tensorflow::str_util::EndsWith(file_name, ".gif")) {
+  } else if (absl::EndsWith(file_name, ".gif")) {
     image_reader = DecodeGif(root.WithOpName("gif_reader"), file_reader);
   } else {
     // Assume if it's neither a PNG nor a GIF then it must be a JPEG.
@@ -135,12 +149,12 @@ Status ReadTensorFromImageFile(const string& file_name, const int input_height,
   TF_RETURN_IF_ERROR(session->Create(graph));
   TF_RETURN_IF_ERROR(
       session->Run({}, {output_name, original_name}, {}, out_tensors));
-  return ::tensorflow::OkStatus();
+  return absl::OkStatus();
 }
 
 Status SaveImage(const Tensor& tensor, const string& file_path) {
   LOG(INFO) << "Saving image to " << file_path;
-  CHECK(tensorflow::str_util::EndsWith(file_path, ".png"))
+  CHECK(absl::EndsWith(file_path, ".png"))
       << "Only saving of png files is supported.";
 
   auto root = tensorflow::Scope::NewRootScope();
@@ -163,7 +177,7 @@ Status SaveImage(const Tensor& tensor, const string& file_path) {
   std::vector<Tensor> outputs;
   TF_RETURN_IF_ERROR(session->Run({}, {}, {output_name}, &outputs));
 
-  return ::tensorflow::OkStatus();
+  return absl::OkStatus();
 }
 
 // Reads a model graph definition from disk, and creates a session object you
@@ -182,7 +196,7 @@ Status LoadGraph(const string& graph_file_name,
   if (!session_create_status.ok()) {
     return session_create_status;
   }
-  return ::tensorflow::OkStatus();
+  return absl::OkStatus();
 }
 
 // Analyzes the output of the MultiBox graph to retrieve the highest scores and
@@ -210,7 +224,7 @@ Status GetTopDetections(const std::vector<Tensor>& outputs, int how_many_labels,
                                   {}, &out_tensors));
   *scores = out_tensors[0];
   *indices = out_tensors[1];
-  return ::tensorflow::OkStatus();
+  return absl::OkStatus();
 }
 
 // Converts an encoded location to an actual box placement with the provided
@@ -329,7 +343,7 @@ Status PrintTopDetections(const std::vector<Tensor>& outputs,
   if (!image_file_name.empty()) {
     return SaveImage(*original_tensor, image_file_name);
   }
-  return ::tensorflow::OkStatus();
+  return absl::OkStatus();
 }
 
 int main(int argc, char* argv[]) {
