@@ -33,9 +33,8 @@ bazel run tensorflow/examples/speech_commands:train
 This will write out checkpoints to /tmp/speech_commands_train/, and will
 download over 1GB of open source training data, so you'll need enough free space
 and a good internet connection. The default data is a collection of thousands of
-one-second .wav files, each containing one spoken word. This data set is
-collected from https://aiyprojects.withgoogle.com/open_speech_recording, please
-consider contributing to help improve this and other models!
+one-second .wav files, each containing one spoken word. Learn more at 
+https://blog.research.google/2017/08/launching-speech-commands-dataset.html.
 
 As training progresses, it will print out its accuracy metrics, which should
 rise above 90% by the end. Once it's complete, you can run the freeze script to
