@@ -23,7 +23,6 @@ limitations under the License.
 #include <utility>
 #include <vector>
 
-#include "tensorflow/lite/builtin_ops.h"
 #include "tensorflow/lite/core/c/common.h"
 #include "tensorflow/lite/graph_info.h"
 #include "tensorflow/lite/simple_memory_arena.h"
@@ -42,6 +41,7 @@ ArenaPlanner::ArenaPlanner(TfLiteContext* context,
     : context_(context),
       graph_info_(std::move(graph_info)),
       arena_(kDefaultArenaAlignment, subgraph_index),
+      has_nonpersistent_memory_(false),
       persistent_arena_(kDefaultArenaAlignment, subgraph_index),
       preserve_all_tensors_(preserve_all_tensors),
       tensor_alignment_(tensor_alignment),
@@ -141,6 +141,10 @@ bool ArenaPlanner::InputTensorCanBeShared(const TfLiteTensor& input_tensor,
       input_allocation_type != kTfLiteArenaRw) {
     return false;
   }
+  if (preserve_all_tensors_) {
+    return false;
+  }
+
   return true;
 }
 
@@ -209,7 +213,7 @@ TfLiteStatus ArenaPlanner::PlanAllocations() {
       std::max(graph_info_->num_execution_nodes(), (size_t)1), {});
 
   // Keeps track of references to each tensor.
-  refcounts_.resize(num_tensors, 0);
+  refcounts_.assign(num_tensors, 0);
 
   auto allocate = [this](int node, int tensor) -> TfLiteStatus {
     if (alloc_node_[tensor] != kNodeNotAssigned) {
@@ -380,6 +384,7 @@ TfLiteStatus ArenaPlanner::ExecuteAllocations(int first_node, int last_node) {
 TfLiteStatus ArenaPlanner::ReleaseNonPersistentMemory() {
   // Clear non-persistent arena's buffer.
   TF_LITE_ENSURE_STATUS(arena_.ReleaseBuffer());
+  has_nonpersistent_memory_ = false;
   // Set data pointers for all non-persistent tensors to nullptr.
   TfLiteTensor* tensors = graph_info_->tensors();
   for (int i = 0; i < static_cast<int>(graph_info_->num_tensors()); ++i) {
@@ -394,7 +399,8 @@ TfLiteStatus ArenaPlanner::ReleaseNonPersistentMemory() {
 TfLiteStatus ArenaPlanner::AcquireNonPersistentMemory() {
   // First commit arena_ to allocate underlying buffer.
   bool reallocated;
-  TF_LITE_ENSURE_STATUS(arena_.Commit(context_, &reallocated));
+  TF_LITE_ENSURE_STATUS(arena_.Commit(&reallocated));
+  has_nonpersistent_memory_ = true;
   // Resolve allocations for all tensors not on the persistent arena.
   TfLiteTensor* tensors = graph_info_->tensors();
   for (int i = 0; i < static_cast<int>(graph_info_->num_tensors()); ++i) {
@@ -407,7 +413,7 @@ TfLiteStatus ArenaPlanner::AcquireNonPersistentMemory() {
 }
 
 bool ArenaPlanner::HasNonPersistentMemory() {
-  return arena_.GetBufferSize() != 0;
+  return has_nonpersistent_memory_;
 }
 
 void ArenaPlanner::DumpDebugInfo(const std::vector<int>& execution_plan) const {
@@ -424,9 +430,10 @@ void ArenaPlanner::GetAllocInfo(size_t* arena_size,
 
 TfLiteStatus ArenaPlanner::Commit(bool* reallocated) {
   bool arena_reallocated, persistent_arena_reallocated;
-  TF_LITE_ENSURE_STATUS(arena_.Commit(context_, &arena_reallocated));
+  TF_LITE_ENSURE_STATUS(arena_.Commit(&arena_reallocated));
+  has_nonpersistent_memory_ = true;
   TF_LITE_ENSURE_STATUS(
-      persistent_arena_.Commit(context_, &persistent_arena_reallocated));
+      persistent_arena_.Commit(&persistent_arena_reallocated));
   *reallocated = arena_reallocated;
   *reallocated |= persistent_arena_reallocated;
   return kTfLiteOk;
