@@ -15,7 +15,6 @@ limitations under the License.
 #include "tensorflow/lite/core/async/interop/variant.h"
 
 #include <cstring>
-#include <utility>
 
 namespace tflite {
 namespace interop {
@@ -37,6 +36,8 @@ bool Variant::operator==(const Variant& other) const {
       return val.s == other.val.s;
     case kString:
       return (val.c == other.val.c) || (strcmp(val.c, other.val.c) == 0);
+    case kBool:
+      return val.b == other.val.b;
   }
 }
 
