@@ -21,6 +21,8 @@ limitations under the License.
 
 #include <array>
 #include <cmath>
+#include <cstdlib>
+#include <cstring>
 #include <fstream>
 #include <ios>
 #include <string>
@@ -28,7 +30,7 @@ limitations under the License.
 
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
-#include "tensorflow/core/platform/resource_loader.h"
+#include "tensorflow/lite/builtin_ops.h"
 #include "tensorflow/lite/c/c_api_internal.h"
 #include "tensorflow/lite/core/c/builtin_op_data.h"
 #include "tensorflow/lite/core/c/c_api_opaque.h"
@@ -59,6 +61,21 @@ TEST(CApiSimple, Version) {
   EXPECT_STREQ(TfLiteVersion(), version);
 }
 
+TEST(CApiSimple, ExtensionApisVersion) {
+  const char* version = TfLiteExtensionApisVersion();
+  ASSERT_NE(version, nullptr);
+  EXPECT_STRNE(version, "");
+  int major = -1, minor = -1, patch = -1;
+  int ret = sscanf(version, "%d.%d.%d", &major, &minor, &patch);
+  // The version number should contain all three components.
+  EXPECT_GE(ret, 3);
+  EXPECT_GE(major, 0);
+  EXPECT_GE(minor, 0);
+  EXPECT_GE(patch, 0);
+  // Calling the function again should give the same result.
+  EXPECT_STREQ(TfLiteExtensionApisVersion(), version);
+}
+
 TEST(CApiSimple, SchemaVersion) {
   // The following checks will need updating if we change the schema version.
   EXPECT_EQ(TfLiteSchemaVersion(), 3);
@@ -67,9 +84,8 @@ TEST(CApiSimple, SchemaVersion) {
 }
 
 TEST(CApiSimple, Smoke) {
-  TfLiteModel* model = TfLiteModelCreateFromFile(
-      tensorflow::GetDataDependencyFilepath("tensorflow/lite/testdata/add.bin")
-          .c_str());
+  TfLiteModel* model =
+      TfLiteModelCreateFromFile("tensorflow/lite/testdata/add.bin");
   ASSERT_NE(model, nullptr);
 
   TfLiteInterpreterOptions* options = TfLiteInterpreterOptionsCreate();
@@ -145,9 +161,8 @@ TEST(CApiSimple, Smoke) {
 
 TEST(CApiSimple, QuantizationParams) {
   TfLiteModel* model = TfLiteModelCreateFromFile(
-      tensorflow::GetDataDependencyFilepath(
-          "tensorflow/lite/testdata/add_quantized.bin")
-          .c_str());
+
+      "tensorflow/lite/testdata/add_quantized.bin");
   ASSERT_NE(model, nullptr);
 
   TfLiteInterpreter* interpreter = TfLiteInterpreterCreate(model, nullptr);
@@ -206,9 +221,8 @@ TEST(CApiSimple, QuantizationParams) {
 }
 
 TEST(CApiSimple, TfLiteInterpreterGetTensor) {
-  TfLiteModel* model = TfLiteModelCreateFromFile(
-      tensorflow::GetDataDependencyFilepath("tensorflow/lite/testdata/add.bin")
-          .c_str());
+  TfLiteModel* model =
+      TfLiteModelCreateFromFile("tensorflow/lite/testdata/add.bin");
   ASSERT_NE(model, nullptr);
 
   TfLiteInterpreterOptions* options = TfLiteInterpreterOptionsCreate();
@@ -253,7 +267,7 @@ TEST(CApiSimple, TfLiteInterpreterGetTensor) {
 
   ASSERT_EQ(TfLiteInterpreterInvoke(interpreter), kTfLiteOk);
 
-  // The 'tensorflow/lite/testdata/add.bin' model uses model tensor
+  // The 'third_party/tensorflow/testdata/add.bin' model uses model tensor
   // at index 2 as the output tensor.
   const TfLiteTensor* output_tensor =
       TfLiteInterpreterGetTensor(interpreter, 2);
@@ -280,10 +294,10 @@ TEST(CApiSimple, TfLiteInterpreterGetTensor) {
   TfLiteInterpreterDelete(interpreter);
 }
 
+#if !TFLITE_USE_OPAQUE_DELEGATE
 TEST(CApiSimple, Delegate) {
-  TfLiteModel* model = TfLiteModelCreateFromFile(
-      tensorflow::GetDataDependencyFilepath("tensorflow/lite/testdata/add.bin")
-          .c_str());
+  TfLiteModel* model =
+      TfLiteModelCreateFromFile("tensorflow/lite/testdata/add.bin");
 
   // Create and install a delegate instance.
   bool delegate_prepared = false;
@@ -306,11 +320,11 @@ TEST(CApiSimple, Delegate) {
   EXPECT_EQ(TfLiteInterpreterInvoke(interpreter), kTfLiteOk);
   TfLiteInterpreterDelete(interpreter);
 }
+#endif
 
 TEST(CApiSimple, DelegateExternal_GetExecutionPlan) {
-  TfLiteModel* model = TfLiteModelCreateFromFile(
-      tensorflow::GetDataDependencyFilepath("tensorflow/lite/testdata/add.bin")
-          .c_str());
+  TfLiteModel* model =
+      TfLiteModelCreateFromFile("tensorflow/lite/testdata/add.bin");
 
   // Create and install a delegate instance.
   bool delegate_prepared = false;
@@ -365,10 +379,8 @@ bool SubgraphIsDelegationSkippable(TfLiteOpaqueContext* context,
 }
 
 TEST(CApiSimple, DelegateExternal_MarkSubgraphAsDelegationSkippable) {
-  TfLiteModel* model =
-      TfLiteModelCreateFromFile(tensorflow::GetDataDependencyFilepath(
-                                    "tensorflow/lite/testdata/2_subgraphs.bin")
-                                    .c_str());
+  TfLiteModel* model = TfLiteModelCreateFromFile(
+      "tensorflow/lite/testdata/2_subgraphs.bin");
 
   // Create and install a delegate instance.
   bool delegate_prepared = false;
@@ -402,10 +414,10 @@ TEST(CApiSimple, DelegateExternal_MarkSubgraphAsDelegationSkippable) {
   TfLiteOpaqueDelegateDelete(opaque_delegate);
 }
 
+#if !TFLITE_USE_OPAQUE_DELEGATE
 TEST(CApiSimple, DelegateFails) {
-  TfLiteModel* model = TfLiteModelCreateFromFile(
-      tensorflow::GetDataDependencyFilepath("tensorflow/lite/testdata/add.bin")
-          .c_str());
+  TfLiteModel* model =
+      TfLiteModelCreateFromFile("tensorflow/lite/testdata/add.bin");
 
   // Create and install a delegate instance.
   TfLiteDelegate delegate = TfLiteDelegateCreate();
@@ -422,6 +434,7 @@ TEST(CApiSimple, DelegateFails) {
   TfLiteInterpreterOptionsDelete(options);
   TfLiteModelDelete(model);
 }
+#endif
 
 struct DelegateState {
   bool delegate_prepared;
@@ -434,13 +447,13 @@ struct OpState {
 std::vector<int>* g_nodes_to_replace;
 TfLiteOpaqueDelegate* g_opaque_delegate_struct;
 
-TfLiteRegistrationExternal* CreateDelegateKernelExternalRegistration() {
-  TfLiteRegistrationExternal* delegate_kernel_registration_external =
-      TfLiteRegistrationExternalCreate(kTfLiteBuiltinDelegate,
-                                       "TEST DELEGATE KERNEL", /*version=*/1);
-  TfLiteRegistrationExternalSetInit(
+TfLiteOperator* CreateDelegateKernelExternalRegistration() {
+  TfLiteOperator* delegate_kernel_registration_external = TfLiteOperatorCreate(
+      kTfLiteBuiltinDelegate, "TEST DELEGATE KERNEL", /*version=*/1,
+      /*user_data=*/nullptr);
+  TfLiteOperatorSetInitWithData(
       delegate_kernel_registration_external,
-      [](TfLiteOpaqueContext* context, const char* buffer,
+      [](void* user_data, TfLiteOpaqueContext* context, const char* buffer,
          size_t length) -> void* {
         const TfLiteOpaqueDelegateParams* params =
             reinterpret_cast<const TfLiteOpaqueDelegateParams*>(buffer);
@@ -460,9 +473,9 @@ TfLiteRegistrationExternal* CreateDelegateKernelExternalRegistration() {
         }
         return new OpState{true};
       });
-  TfLiteRegistrationExternalSetFree(
+  TfLiteOperatorSetFreeWithData(
       delegate_kernel_registration_external,
-      [](TfLiteOpaqueContext* context, void* buffer) {
+      [](void* user_data, TfLiteOpaqueContext* context, void* buffer) {
         delete (reinterpret_cast<OpState*>(buffer));
       });
   return delegate_kernel_registration_external;
@@ -471,9 +484,8 @@ TfLiteRegistrationExternal* CreateDelegateKernelExternalRegistration() {
 TEST(CApiSimple, OpaqueDelegate_ReplaceNodeSubsetsWithDelegateKernels) {
   g_nodes_to_replace = new std::vector<int>();
 
-  TfLiteModel* model = TfLiteModelCreateFromFile(
-      tensorflow::GetDataDependencyFilepath("tensorflow/lite/testdata/add.bin")
-          .c_str());
+  TfLiteModel* model =
+      TfLiteModelCreateFromFile("tensorflow/lite/testdata/add.bin");
 
   // Create and install a delegate instance.
   DelegateState delegate_state{false};
@@ -490,7 +502,7 @@ TEST(CApiSimple, OpaqueDelegate_ReplaceNodeSubsetsWithDelegateKernels) {
     EXPECT_EQ(execution_plan->size, 2);
 
     TfLiteOpaqueNode* node = nullptr;
-    TfLiteRegistrationExternal* registration_external = nullptr;
+    TfLiteOperator* registration_external = nullptr;
     TfLiteOpaqueContextGetNodeAndRegistration(opaque_context, 0, &node,
                                               &registration_external);
     EXPECT_NE(node, nullptr);
@@ -529,13 +541,11 @@ TEST(CApiSimple, OpaqueDelegate_ReplaceNodeSubsetsWithDelegateKernels) {
   g_opaque_delegate_struct = nullptr;
 }
 
-TEST(CApiSimple,
-     OpaqueDelegate_TransferRegistrationExternalOwnershipWithoutNodeToReplace) {
+TEST(CApiSimple, OpaqueDelegate_TransferOperatorOwnershipWithoutNodeToReplace) {
   g_nodes_to_replace = new std::vector<int>();
 
-  TfLiteModel* model = TfLiteModelCreateFromFile(
-      tensorflow::GetDataDependencyFilepath("tensorflow/lite/testdata/add.bin")
-          .c_str());
+  TfLiteModel* model =
+      TfLiteModelCreateFromFile("tensorflow/lite/testdata/add.bin");
 
   // Create and install a delegate instance.
   DelegateState delegate_state{false};
@@ -548,7 +558,7 @@ TEST(CApiSimple,
     delegate_state->delegate_prepared = true;
 
     TfLiteOpaqueNode* node = nullptr;
-    TfLiteRegistrationExternal* registration_external = nullptr;
+    TfLiteOperator* registration_external = nullptr;
     TfLiteOpaqueContextGetNodeAndRegistration(opaque_context, 0, &node,
                                               &registration_external);
     EXPECT_NE(node, nullptr);
@@ -607,13 +617,13 @@ TEST_F(TestFP16Delegation,
     std::vector<int> nodes_to_replace;
     for (int i = 0; i < execution_plan->size; i++) {
       TfLiteOpaqueNode* node = nullptr;
-      TfLiteRegistrationExternal* registration_external = nullptr;
+      TfLiteOperator* registration_external = nullptr;
       TfLiteOpaqueContextGetNodeAndRegistration(opaque_context,
                                                 execution_plan->data[i], &node,
                                                 &registration_external);
       EXPECT_NE(node, nullptr);
       EXPECT_NE(registration_external, nullptr);
-      if (TfLiteRegistrationExternalGetBuiltInCode(registration_external) ==
+      if (TfLiteOperatorGetBuiltInCode(registration_external) ==
           kTfLiteBuiltinAdd) {
         nodes_to_replace.push_back(execution_plan->data[i]);
       }
@@ -668,9 +678,8 @@ TEST(CApiSimple, InterpreterOptionsCopy) {
 }
 
 TEST(CApiSimple, ErrorReporter) {
-  TfLiteModel* model = TfLiteModelCreateFromFile(
-      tensorflow::GetDataDependencyFilepath("tensorflow/lite/testdata/add.bin")
-          .c_str());
+  TfLiteModel* model =
+      TfLiteModelCreateFromFile("tensorflow/lite/testdata/add.bin");
   TfLiteInterpreterOptions* options = TfLiteInterpreterOptionsCreate();
 
   // Install a custom error reporter into the interpreter by way of options.
@@ -698,9 +707,7 @@ TEST(CApiSimple, ModelCreateWithErrorReporter) {
   tflite::TestErrorReporter reporter;
 
   // valid model with error reporter
-  std::ifstream model_file(
-      tensorflow::GetDataDependencyFilepath("tensorflow/lite/testdata/add.bin")
-          .c_str());
+  std::ifstream model_file("tensorflow/lite/testdata/add.bin");
   model_file.seekg(0, std::ios_base::end);
   std::vector<char> model_buffer(model_file.tellg());
   model_file.seekg(0, std::ios_base::beg);
@@ -727,9 +734,8 @@ TEST(CApiSimple, ModelCreateFromFileWithErrorReporter) {
 
   // valid model file with error reporter
   model = TfLiteModelCreateFromFileWithErrorReporter(
-      tensorflow::GetDataDependencyFilepath("tensorflow/lite/testdata/add.bin")
-          .c_str(),
-      error_reporter, &reporter);
+      "tensorflow/lite/testdata/add.bin", error_reporter,
+      &reporter);
   ASSERT_NE(model, nullptr);
   EXPECT_EQ(reporter.error_messages(), "");
   TfLiteModelDelete(model);
@@ -750,19 +756,20 @@ struct DelegateKernelState {
   TfLiteOpaqueTensor* output_tensor = nullptr;
 };
 
-TfLiteRegistrationExternal* CreateReg() {
-  auto reg_ex = TfLiteRegistrationExternalCreate(
-      kTfLiteBuiltinDelegate, "Test driver delegate", /*version=*/1);
-  TfLiteRegistrationExternalSetInit(
+TfLiteOperator* CreateReg() {
+  auto reg_ex = TfLiteOperatorCreate(kTfLiteBuiltinDelegate,
+                                     "Test driver delegate", /*version=*/1,
+                                     /*user_data=*/nullptr);
+  TfLiteOperatorSetInitWithData(
       reg_ex,
-      [](TfLiteOpaqueContext* context, const char* buffer,
+      [](void* user_data, TfLiteOpaqueContext* context, const char* buffer,
          size_t length) -> void* {
         const TfLiteOpaqueDelegateParams* params =
             reinterpret_cast<const TfLiteOpaqueDelegateParams*>(buffer);
 
         for (int i = 0; i < params->nodes_to_replace->size; ++i) {
           TfLiteOpaqueNode* node = nullptr;
-          TfLiteRegistrationExternal* registration_external = nullptr;
+          TfLiteOperator* registration_external = nullptr;
           TfLiteOpaqueContextGetNodeAndRegistration(
               context, params->nodes_to_replace->data[i], &node,
               &registration_external);
@@ -770,8 +777,8 @@ TfLiteRegistrationExternal* CreateReg() {
           EXPECT_NE(nullptr, registration_external);
           EXPECT_EQ(2, TfLiteOpaqueNodeNumberOfInputs(node));
           EXPECT_EQ(1, TfLiteOpaqueNodeNumberOfOutputs(node));
-          EXPECT_EQ(kTfLiteBuiltinAdd, TfLiteRegistrationExternalGetBuiltInCode(
-                                           registration_external));
+          EXPECT_EQ(kTfLiteBuiltinAdd,
+                    TfLiteOperatorGetBuiltInCode(registration_external));
         }
 
         TfLiteIntArray* input_tensors = params->input_tensors;
@@ -786,9 +793,9 @@ TfLiteRegistrationExternal* CreateReg() {
         return new DelegateKernelState{input_tensor, output_tensor};
       });
 
-  TfLiteRegistrationExternalSetInvoke(
+  TfLiteOperatorSetInvokeWithData(
       reg_ex,
-      [](TfLiteOpaqueContext* context,
+      [](void* user_data, TfLiteOpaqueContext* context,
          TfLiteOpaqueNode* opaque_node) -> TfLiteStatus {
         DelegateKernelState* delegate_kernel =
             reinterpret_cast<DelegateKernelState*>(
@@ -804,18 +811,18 @@ TfLiteRegistrationExternal* CreateReg() {
         return kTfLiteOk;
       });
 
-  TfLiteRegistrationExternalSetFree(reg_ex, [](TfLiteOpaqueContext* context,
-                                               void* data) {
-    DelegateKernelState* state = reinterpret_cast<DelegateKernelState*>(data);
-    delete state;
-  });
+  TfLiteOperatorSetFreeWithData(
+      reg_ex, [](void* user_data, TfLiteOpaqueContext* context, void* data) {
+        DelegateKernelState* state =
+            reinterpret_cast<DelegateKernelState*>(data);
+        delete state;
+      });
   return reg_ex;
 }
 
 TEST(CApiSimple, OpaqueDelegate_TfLiteOpaqueTensorGet) {
-  TfLiteModel* model = TfLiteModelCreateFromFile(
-      tensorflow::GetDataDependencyFilepath("tensorflow/lite/testdata/add.bin")
-          .c_str());
+  TfLiteModel* model =
+      TfLiteModelCreateFromFile("tensorflow/lite/testdata/add.bin");
 
   struct DelegateState {
     bool delegate_prepared = false;
@@ -838,15 +845,15 @@ TEST(CApiSimple, OpaqueDelegate_TfLiteOpaqueTensorGet) {
     std::vector<int> node_ids_to_replace;
     for (int i = 0; i < execution_plan->size; ++i) {
       TfLiteOpaqueNode* node = nullptr;
-      TfLiteRegistrationExternal* registration_external = nullptr;
+      TfLiteOperator* registration_external = nullptr;
       TfLiteOpaqueContextGetNodeAndRegistration(
           context, execution_plan->data[i], &node, &registration_external);
       EXPECT_NE(nullptr, node);
       EXPECT_NE(nullptr, registration_external);
       EXPECT_EQ(2, TfLiteOpaqueNodeNumberOfInputs(node));
       EXPECT_EQ(1, TfLiteOpaqueNodeNumberOfOutputs(node));
-      EXPECT_EQ(kTfLiteBuiltinAdd, TfLiteRegistrationExternalGetBuiltInCode(
-                                       registration_external));
+      EXPECT_EQ(kTfLiteBuiltinAdd,
+                TfLiteOperatorGetBuiltInCode(registration_external));
       node_ids_to_replace.push_back(execution_plan->data[i]);
     }
 
@@ -922,9 +929,8 @@ TEST(CApiSimple, OpaqueContextGetNodeAndRegistration) {
   };
   DelegatePrepareStatus delegate_state{false};
 
-  TfLiteModel* model = TfLiteModelCreateFromFile(
-      tensorflow::GetDataDependencyFilepath("tensorflow/lite/testdata/add.bin")
-          .c_str());
+  TfLiteModel* model =
+      TfLiteModelCreateFromFile("tensorflow/lite/testdata/add.bin");
 
   TfLiteOpaqueDelegateBuilder opaque_delegate_builder{};
   opaque_delegate_builder.data = &delegate_state;
@@ -941,17 +947,49 @@ TEST(CApiSimple, OpaqueContextGetNodeAndRegistration) {
 
     for (int i = 0; i < execution_plan->size; i++) {
       TfLiteOpaqueNode* node = nullptr;
-      TfLiteRegistrationExternal* registration_external = nullptr;
+      TfLiteOperator* registration_external = nullptr;
       TfLiteOpaqueContextGetNodeAndRegistration(opaque_context, 0, &node,
                                                 &registration_external);
       EXPECT_NE(node, nullptr);
       EXPECT_NE(registration_external, nullptr);
-      EXPECT_EQ(kTfLiteBuiltinAdd, TfLiteRegistrationExternalGetBuiltInCode(
-                                       registration_external));
-      EXPECT_EQ(1, TfLiteRegistrationExternalGetVersion(registration_external));
+      EXPECT_EQ(kTfLiteBuiltinAdd,
+                TfLiteOperatorGetBuiltInCode(registration_external));
+      EXPECT_EQ(1, TfLiteOperatorGetVersion(registration_external));
       EXPECT_EQ(2, TfLiteOpaqueNodeNumberOfInputs(node));
       EXPECT_EQ(1, TfLiteOpaqueNodeNumberOfOutputs(node));
     }
+
+    {
+      TfLiteOpaqueNode* node = nullptr;
+      TfLiteOperator* registration_external = nullptr;
+      TfLiteOpaqueContextGetNodeAndRegistration(opaque_context, 0, &node,
+                                                &registration_external);
+      EXPECT_EQ(1, TfLiteOpaqueNodeGetInputTensorIndex(node, 0));
+      EXPECT_EQ(1, TfLiteOpaqueNodeGetInputTensorIndex(node, 1));
+      EXPECT_EQ(-1, TfLiteOpaqueNodeGetInputTensorIndex(node, 2));
+      EXPECT_EQ(0, TfLiteOpaqueNodeGetOutputTensorIndex(node, 0));
+      EXPECT_EQ(-1, TfLiteOpaqueNodeGetOutputTensorIndex(node, 123));
+      EXPECT_EQ(-1, TfLiteOpaqueNodeGetOutputTensorIndex(node, -1));
+
+      const TfLiteOpaqueTensor* opaque_tensor =
+          TfLiteOpaqueContextGetOpaqueTensor(opaque_context, 0);
+      EXPECT_NE(opaque_tensor, nullptr);
+      EXPECT_EQ(kTfLiteFloat32, TfLiteOpaqueTensorType(opaque_tensor));
+      size_t bytes_float_32 = 0;
+      EXPECT_EQ(kTfLiteOk,
+                TfLiteOpaqueContextGetSizeOfType(opaque_context, kTfLiteFloat32,
+                                                 &bytes_float_32));
+      EXPECT_EQ(bytes_float_32, sizeof(float));
+    }
+    {
+      TfLiteOpaqueNode* node = nullptr;
+      TfLiteOperator* registration_external = nullptr;
+      TfLiteOpaqueContextGetNodeAndRegistration(opaque_context, 1, &node,
+                                                &registration_external);
+      EXPECT_EQ(0, TfLiteOpaqueNodeGetInputTensorIndex(node, 0));
+      EXPECT_EQ(1, TfLiteOpaqueNodeGetInputTensorIndex(node, 1));
+      EXPECT_EQ(2, TfLiteOpaqueNodeGetOutputTensorIndex(node, 0));
+    }
     return kTfLiteOk;
   };
 
@@ -970,8 +1008,8 @@ TEST(CApiSimple, OpaqueContextGetNodeAndRegistration) {
   TfLiteOpaqueDelegateDelete(opaque_delegate);
 }
 
-TEST(CApiSimple, TfLiteRegistrationExternalGetVersionNullptr) {
-  EXPECT_EQ(TfLiteRegistrationExternalGetVersion(nullptr), -1);
+TEST(CApiSimple, TfLiteOperatorGetVersionNullptr) {
+  EXPECT_EQ(TfLiteOperatorGetVersion(nullptr), -1);
 }
 
 TEST(CApiSimple, TfLiteOpaqueContextResizeTensor) {
@@ -980,9 +1018,8 @@ TEST(CApiSimple, TfLiteOpaqueContextResizeTensor) {
   };
   DelegatePrepareStatus delegate_state{false};
 
-  TfLiteModel* model = TfLiteModelCreateFromFile(
-      tensorflow::GetDataDependencyFilepath("tensorflow/lite/testdata/add.bin")
-          .c_str());
+  TfLiteModel* model =
+      TfLiteModelCreateFromFile("tensorflow/lite/testdata/add.bin");
 
   TfLiteOpaqueDelegateBuilder opaque_delegate_builder{};
   opaque_delegate_builder.data = &delegate_state;
@@ -1031,9 +1068,7 @@ TEST(CApiSimple, TfLiteOpaqueContextResizeTensor) {
 }
 
 TEST(CApiSimple, ValidModel) {
-  std::ifstream model_file(
-      tensorflow::GetDataDependencyFilepath("tensorflow/lite/testdata/add.bin")
-          .c_str());
+  std::ifstream model_file("tensorflow/lite/testdata/add.bin");
 
   model_file.seekg(0, std::ios_base::end);
   std::vector<char> model_buffer(model_file.tellg());
@@ -1048,9 +1083,8 @@ TEST(CApiSimple, ValidModel) {
 }
 
 TEST(CApiSimple, ValidModelFromFile) {
-  TfLiteModel* model = TfLiteModelCreateFromFile(
-      tensorflow::GetDataDependencyFilepath("tensorflow/lite/testdata/add.bin")
-          .c_str());
+  TfLiteModel* model =
+      TfLiteModelCreateFromFile("tensorflow/lite/testdata/add.bin");
   ASSERT_NE(model, nullptr);
   TfLiteModelDelete(model);
 }
@@ -1071,30 +1105,30 @@ struct SinhParams {
   bool use_cosh_instead = false;
 };
 
-void* FlexSinhInit(TfLiteOpaqueContext* context, const char* buffer,
-                   size_t length) {
+void* FlexSinhInit(void* user_data, TfLiteOpaqueContext* context,
+                   const char* buffer, size_t length) {
   auto sinh_params = new SinhParams;
   // The buffer that is passed into here is the custom_options
-  // field from the flatbuffer (tensorflow/lite/schema/schema.fbs)
-  // `Operator` for this node.
-  // Typically it should be stored as a FlexBuffer, but for this test
-  // we assume that it is just a string.
+  // field from the flatbuffer
+  // (third_party/tensorflow/compiler/mlir/lite/schema/schema.fbs) `Operator`
+  // for this node. Typically it should be stored as a FlexBuffer, but for this
+  // test we assume that it is just a string.
   if (std::string(buffer, length) == "use_cosh") {
     sinh_params->use_cosh_instead = true;
   }
   return sinh_params;
 }
 
-void FlexSinhFree(TfLiteOpaqueContext* context, void* data) {
+void FlexSinhFree(void* user_data, TfLiteOpaqueContext* context, void* data) {
   delete static_cast<SinhParams*>(data);
 }
 
-TfLiteStatus FlexSinhPrepare(TfLiteOpaqueContext* context,
+TfLiteStatus FlexSinhPrepare(void* user_data, TfLiteOpaqueContext* context,
                              TfLiteOpaqueNode* node) {
   return kTfLiteOk;
 }
 
-TfLiteStatus FlexSinhEval(TfLiteOpaqueContext* context,
+TfLiteStatus FlexSinhEval(void* user_data, TfLiteOpaqueContext* context,
                           TfLiteOpaqueNode* node) {
   auto sinh_params =
       static_cast<SinhParams*>(TfLiteOpaqueNodeGetUserData(node));
@@ -1114,24 +1148,23 @@ TfLiteStatus FlexSinhEval(TfLiteOpaqueContext* context,
 }
 
 TEST(CApiSimple, CustomOpSupport) {
-  TfLiteModel* model =
-      TfLiteModelCreateFromFile(tensorflow::GetDataDependencyFilepath(
-                                    "tensorflow/lite/testdata/custom_sinh.bin")
-                                    .c_str());
+  TfLiteModel* model = TfLiteModelCreateFromFile(
+      "tensorflow/lite/testdata/custom_sinh.bin");
   ASSERT_NE(model, nullptr);
 
-  TfLiteRegistrationExternal* reg =
-      TfLiteRegistrationExternalCreate(kTfLiteBuiltinCustom, "Sinh", 1);
-  TfLiteRegistrationExternalSetPrepare(reg, &FlexSinhPrepare);
-  TfLiteRegistrationExternalSetInit(reg, &FlexSinhInit);
-  TfLiteRegistrationExternalSetFree(reg, &FlexSinhFree);
-  TfLiteRegistrationExternalSetInvoke(reg, &FlexSinhEval);
+  TfLiteOperator* reg =
+      TfLiteOperatorCreate(kTfLiteBuiltinCustom, "Sinh", /*version=*/1,
+                           /*user_data=*/nullptr);
+  TfLiteOperatorSetPrepareWithData(reg, &FlexSinhPrepare);
+  TfLiteOperatorSetInitWithData(reg, &FlexSinhInit);
+  TfLiteOperatorSetFreeWithData(reg, &FlexSinhFree);
+  TfLiteOperatorSetInvokeWithData(reg, &FlexSinhEval);
 
-  const char* kCustomName = TfLiteRegistrationExternalGetCustomName(reg);
+  const char* kCustomName = TfLiteOperatorGetCustomName(reg);
   EXPECT_EQ("Sinh", kCustomName);
 
   TfLiteInterpreterOptions* options = TfLiteInterpreterOptionsCreate();
-  TfLiteInterpreterOptionsAddRegistrationExternal(options, reg);
+  TfLiteInterpreterOptionsAddOperator(options, reg);
 
   TfLiteInterpreter* interpreter = TfLiteInterpreterCreate(model, options);
 
@@ -1151,7 +1184,7 @@ TEST(CApiSimple, CustomOpSupport) {
 
   TfLiteInterpreterDelete(interpreter);
   TfLiteModelDelete(model);
-  TfLiteRegistrationExternalDelete(reg);
+  TfLiteOperatorDelete(reg);
 }
 
 const TfLiteRegistration* find_builtin_op_add(void* user_data,
@@ -1490,13 +1523,14 @@ TEST(CApiSimple, OpaqueApiAccessors) {
     // Define a delegate kernel that checks that the properties of the model
     // are accessible via the opaque API function.
     //
-    TfLiteRegistrationExternal* reg = TfLiteRegistrationExternalCreate(
-        kTfLiteBuiltinDelegate, "my delegate", 123);
-    EXPECT_EQ(123, TfLiteRegistrationExternalGetVersion(reg));
-    TfLiteRegistrationExternalSetInit(
+    TfLiteOperator* reg = TfLiteOperatorCreate(kTfLiteBuiltinDelegate,
+                                               "my delegate", /*version=*/123,
+                                               /*user_data=*/nullptr);
+    EXPECT_EQ(123, TfLiteOperatorGetVersion(reg));
+    TfLiteOperatorSetInitWithData(
         reg,
-        [](TfLiteOpaqueContext* opaque_context, const char* buffer,
-           size_t length) -> void* {
+        [](void* user_data, TfLiteOpaqueContext* opaque_context,
+           const char* buffer, size_t length) -> void* {
           const TfLiteOpaqueDelegateParams* params =
               reinterpret_cast<const TfLiteOpaqueDelegateParams*>(buffer);
           EXPECT_EQ(2, params->input_tensors->size);
@@ -1538,6 +1572,7 @@ TEST(CApiSimple, OpaqueApiAccessors) {
           EXPECT_STREQ(kSubgraphName,
                        TfLiteOpaqueContextGetName(opaque_context));
           EXPECT_EQ(4, TfLiteOpaqueContextGetNumTensors(opaque_context));
+
           EXPECT_EQ(-1,
                     TfLiteOpaqueTensorNumDims(
                         TfLiteOpaqueContextGetOpaqueTensor(opaque_context, 3)));
@@ -1559,14 +1594,144 @@ TEST(CApiSimple, OpaqueApiAccessors) {
           EXPECT_EQ(kTfLiteOk, TfLiteOpaqueContextReleaseSubgraphContext(
                                    opaque_context, 0));
 
+          //
+          // Create and configure a dynamic tensor.
+          //
+          {
+            TfLiteOpaqueTensorBuilder* builder =
+                TfLiteOpaqueTensorBuilderCreate();
+            TfLiteOpaqueTensorBuilderSetType(builder, kTfLiteUInt8);
+            void* dummy_data = malloc(sizeof(uint8_t));
+            TfLiteOpaqueTensorBuilderSetData(builder, dummy_data);
+            TfLiteQuantizationParams new_quantization_params{};
+            new_quantization_params.scale = 16.0 / 256;
+            new_quantization_params.zero_point = 255;
+
+            TfLiteQuantization new_quantization{};
+            new_quantization.type = kTfLiteAffineQuantization;
+            TfLiteAffineQuantization* affine_quant =
+                (TfLiteAffineQuantization*)malloc(
+                    sizeof(TfLiteAffineQuantization));
+            affine_quant->scale = TfLiteFloatArrayCreate(1);
+            affine_quant->zero_point = TfLiteIntArrayCreate(1);
+            new_quantization.params = affine_quant;
+            TfLiteOpaqueTensorBuilderSetQuantization(
+                TfLiteOpaqueTensorBuilderSetQuantizationParams(
+                    TfLiteOpaqueTensorBuilderSetAllocationType(builder,
+                                                               kTfLiteDynamic),
+                    new_quantization_params),
+                new_quantization);
+
+            int new_tensor_index = -1;
+            EXPECT_EQ(kTfLiteOk,
+                      TfLiteOpaqueContextAddTensor(acquired_opaque_context,
+                                                   builder, &new_tensor_index));
+            TfLiteOpaqueTensorBuilderDelete(builder);
+
+            TfLiteOpaqueTensor* new_tensor = TfLiteOpaqueContextGetOpaqueTensor(
+                opaque_context, new_tensor_index);
+            EXPECT_NE(new_tensor, nullptr);
+            EXPECT_EQ(dummy_data, TfLiteOpaqueTensorData(new_tensor));
+            EXPECT_EQ(kTfLiteUInt8, TfLiteOpaqueTensorType(new_tensor));
+            EXPECT_EQ(new_quantization.type,
+                      TfLiteOpaqueTensorGetQuantization(new_tensor).type);
+            EXPECT_NEAR(
+                new_quantization_params.scale,
+                TfLiteOpaqueTensorGetQuantizationParams(new_tensor).scale,
+                0.0001);
+            EXPECT_EQ(
+                new_quantization_params.zero_point,
+                TfLiteOpaqueTensorGetQuantizationParams(new_tensor).zero_point);
+          }
+          //
+          // Create and configure a 'kTfLiteArenaRw' tensor
+          //
+          {
+            TfLiteOpaqueTensorBuilder* builder =
+                TfLiteOpaqueTensorBuilderCreate();
+            TfLiteOpaqueTensorBuilderSetType(builder, kTfLiteUInt8);
+            TfLiteQuantizationParams new_quantization_params{};
+            new_quantization_params.scale = 16.0 / 256;
+            new_quantization_params.zero_point = 255;
+
+            TfLiteQuantization new_quantization{};
+            new_quantization.type = kTfLiteNoQuantization;
+            TfLiteOpaqueTensorBuilderSetQuantization(
+                TfLiteOpaqueTensorBuilderSetQuantizationParams(
+                    TfLiteOpaqueTensorBuilderSetAllocationType(builder,
+                                                               kTfLiteArenaRw),
+                    new_quantization_params),
+                new_quantization);
+
+            int new_tensor_index = -1;
+            EXPECT_EQ(kTfLiteOk,
+                      TfLiteOpaqueContextAddTensor(acquired_opaque_context,
+                                                   builder, &new_tensor_index));
+            TfLiteOpaqueTensorBuilderDelete(builder);
+
+            TfLiteOpaqueTensor* new_tensor = TfLiteOpaqueContextGetOpaqueTensor(
+                opaque_context, new_tensor_index);
+            EXPECT_NE(new_tensor, nullptr);
+            EXPECT_EQ(kTfLiteUInt8, TfLiteOpaqueTensorType(new_tensor));
+            EXPECT_EQ(new_quantization.type,
+                      TfLiteOpaqueTensorGetQuantization(new_tensor).type);
+            EXPECT_NEAR(
+                new_quantization_params.scale,
+                TfLiteOpaqueTensorGetQuantizationParams(new_tensor).scale,
+                0.0001);
+            EXPECT_EQ(
+                new_quantization_params.zero_point,
+                TfLiteOpaqueTensorGetQuantizationParams(new_tensor).zero_point);
+            EXPECT_EQ(kTfLiteArenaRw,
+                      TfLiteOpaqueTensorGetAllocationType(new_tensor));
+
+            // Now switch the tensor's allocation type to dynamic after it has
+            // been created.
+            TfLiteOpaqueTensorSetAllocationTypeToDynamic(new_tensor);
+            EXPECT_EQ(kTfLiteDynamic,
+                      TfLiteOpaqueTensorGetAllocationType(new_tensor));
+          }
+          //
+          // Create and configure a 'kTfLiteVariantObject' tensor, which will
+          // result in an error.
+          //
+          {
+            TfLiteOpaqueTensorBuilder* builder =
+                TfLiteOpaqueTensorBuilderCreate();
+            TfLiteOpaqueTensorBuilderSetAllocationType(builder,
+                                                       kTfLiteVariantObject);
+            int new_tensor_index = -1;
+            EXPECT_EQ(kTfLiteError,
+                      TfLiteOpaqueContextAddTensor(acquired_opaque_context,
+                                                   builder, &new_tensor_index));
+            TfLiteOpaqueTensorBuilderDelete(builder);
+          }
+          //
+          // Create and configure a 'kTfLiteArenaRw' tensor and attempt to
+          // provide 'data', which will result in an error because the TF Lite
+          // runtime will provide the memory in this case.
+          //
+          {
+            TfLiteOpaqueTensorBuilder* builder =
+                TfLiteOpaqueTensorBuilderCreate();
+            TfLiteOpaqueTensorBuilderSetType(builder, kTfLiteUInt8);
+            static int dummy_data = 0;
+            TfLiteOpaqueTensorBuilderSetData(builder, &dummy_data);
+            TfLiteOpaqueTensorBuilderSetAllocationType(builder, kTfLiteArenaRw);
+            int new_tensor_index = -1;
+            EXPECT_EQ(kTfLiteError,
+                      TfLiteOpaqueContextAddTensor(acquired_opaque_context,
+                                                   builder, &new_tensor_index));
+            TfLiteOpaqueTensorBuilderDelete(builder);
+          }
           TfLiteOpaqueNode* node = nullptr;
-          TfLiteRegistrationExternal* registration_external = nullptr;
+          TfLiteOperator* registration_external = nullptr;
           TfLiteOpaqueContextGetNodeAndRegistration(
               opaque_context, params->nodes_to_replace->data[0], &node,
               &registration_external);
           // ADD is a builtin OP, not a custom OP.
           const char* kCustomName =
-              TfLiteRegistrationExternalGetCustomName(registration_external);
+              TfLiteOperatorGetCustomName(registration_external);
           EXPECT_EQ(nullptr, kCustomName);
 
           const void* node_custom_init_data = nullptr;
@@ -1681,12 +1846,13 @@ TEST(CApiSimple, OpaqueApiAccessorsStrings) {
   opaque_delegate_builder.Prepare = [](TfLiteOpaqueContext* context,
                                        TfLiteOpaqueDelegate* delegate,
                                        void* data) -> TfLiteStatus {
-    TfLiteRegistrationExternal* registration = TfLiteRegistrationExternalCreate(
-        kTfLiteBuiltinDelegate, "my delegate", 123);
-    TfLiteRegistrationExternalSetInit(
+    TfLiteOperator* registration = TfLiteOperatorCreate(
+        kTfLiteBuiltinDelegate, "my delegate", /*version=*/123,
+        /*user_data=*/nullptr);
+    TfLiteOperatorSetInitWithData(
         registration,
-        [](TfLiteOpaqueContext* opaque_context, const char* buffer,
-           size_t length) -> void* {
+        [](void* user_data, TfLiteOpaqueContext* opaque_context,
+           const char* buffer, size_t length) -> void* {
           const TfLiteOpaqueDelegateParams* params =
               reinterpret_cast<const TfLiteOpaqueDelegateParams*>(buffer);
           EXPECT_EQ(2, params->input_tensors->size);
@@ -1704,14 +1870,14 @@ TEST(CApiSimple, OpaqueApiAccessorsStrings) {
           return nullptr;
         });
 
-    TfLiteRegistrationExternalSetPrepare(
+    TfLiteOperatorSetPrepareWithData(
         registration,
-        [](TfLiteOpaqueContext* context,
+        [](void* user_data, TfLiteOpaqueContext* context,
            TfLiteOpaqueNode* node) -> TfLiteStatus { return kTfLiteOk; });
 
-    TfLiteRegistrationExternalSetInvoke(
+    TfLiteOperatorSetInvokeWithData(
         registration,
-        [](TfLiteOpaqueContext* context,
+        [](void* user_data, TfLiteOpaqueContext* context,
            TfLiteOpaqueNode* node) -> TfLiteStatus {
           const TfLiteOpaqueTensor* input0 =
               TfLiteOpaqueNodeGetInput(context, node, 0);
@@ -1900,7 +2066,7 @@ TEST(CApiSimple, AddNodesAfterApplyingDelegate) {
     EXPECT_EQ(execution_plan->size, 1);
 
     TfLiteOpaqueNode* node = nullptr;
-    TfLiteRegistrationExternal* registration_external = nullptr;
+    TfLiteOperator* registration_external = nullptr;
     TfLiteOpaqueContextGetNodeAndRegistration(context, execution_plan->data[0],
                                               &node, &registration_external);
     EXPECT_NE(node, nullptr);
