@@ -26,7 +26,10 @@ limitations under the License.
 #include <utility>
 #include <vector>
 
-#include "flatbuffers/flatbuffers.h"  // from @flatbuffers
+#include "flatbuffers/buffer.h"  // from @flatbuffers
+#include "flatbuffers/vector.h"  // from @flatbuffers
+#include "tensorflow/compiler/mlir/lite/allocation.h"
+#include "tensorflow/compiler/mlir/lite/schema/schema_utils.h"
 #include "tensorflow/lite/core/api/error_reporter.h"
 #include "tensorflow/lite/core/api/flatbuffer_conversions.h"
 #include "tensorflow/lite/core/api/op_resolver.h"
@@ -36,15 +39,14 @@ limitations under the License.
 #include "tensorflow/lite/core/model_builder.h"
 #include "tensorflow/lite/core/subgraph.h"
 #include "tensorflow/lite/internal/signature_def.h"
-#include "tensorflow/lite/kernels/internal/compatibility.h"
+#include "tensorflow/lite/interpreter_options.h"
 #include "tensorflow/lite/profiling/platform_profiler.h"
+#include "tensorflow/lite/profiling/telemetry/c/telemetry_setting.h"
 #include "tensorflow/lite/profiling/telemetry/c/telemetry_setting_internal.h"
 #include "tensorflow/lite/schema/conversion_metadata_generated.h"
 #include "tensorflow/lite/schema/schema_generated.h"
-#include "tensorflow/lite/schema/schema_utils.h"
 #include "tensorflow/lite/shared_library.h"
 #include "tensorflow/lite/stderr_reporter.h"
-#include "tensorflow/lite/string_type.h"
 #include "tensorflow/lite/util.h"
 #include "tensorflow/lite/version.h"
 
@@ -365,14 +367,13 @@ TfLiteStatus InterpreterBuilder::ParseNodes(
                            EnumNameBuiltinOperator(op_type));
     }
 
+    void* builtin_data = nullptr;
+    const char* init_data = nullptr;
+    size_t init_data_size = 0;
     if (op_type == BuiltinOperator_CUSTOM) {
       if (op->custom_options()) {
-        subgraph->AddNodeWithParameters(
-            FlatBufferIntArrayToVector(op->inputs()),
-            FlatBufferIntArrayToVector(op->outputs()),
-            FlatBufferIntArrayToVector(op->intermediates()),
-            reinterpret_cast<const char*>(op->custom_options()->data()),
-            op->custom_options()->size(), nullptr, registration);
+        init_data = reinterpret_cast<const char*>(op->custom_options()->data());
+        init_data_size = op->custom_options()->size();
       } else if (op->large_custom_options_offset() > 1 && allocation_) {
         if (op->large_custom_options_offset() +
                 op->large_custom_options_size() >
@@ -384,31 +385,20 @@ TfLiteStatus InterpreterBuilder::ParseNodes(
           return kTfLiteError;
         }
         // If the custom op is storing payloads outside of flatbuffers
-        subgraph->AddNodeWithParameters(
-            FlatBufferIntArrayToVector(op->inputs()),
-            FlatBufferIntArrayToVector(op->outputs()),
-            FlatBufferIntArrayToVector(op->intermediates()),
-            reinterpret_cast<const char*>(allocation_->base()) +
-                op->large_custom_options_offset(),
-            op->large_custom_options_size(), nullptr, registration);
-      } else {
-        subgraph->AddNodeWithParameters(
-            FlatBufferIntArrayToVector(op->inputs()),
-            FlatBufferIntArrayToVector(op->outputs()),
-            FlatBufferIntArrayToVector(op->intermediates()), nullptr, 0,
-            nullptr, registration);
+        init_data = reinterpret_cast<const char*>(allocation_->base()) +
+                    op->large_custom_options_offset();
+        init_data_size = op->large_custom_options_size();
       }
     } else {
-      void* builtin_data = nullptr;
       MallocDataAllocator malloc_allocator;
       TF_LITE_ENSURE_STATUS(ParseOpData(op, op_type, error_reporter_,
                                         &malloc_allocator, &builtin_data));
-      subgraph->AddNodeWithParameters(
-          FlatBufferIntArrayToVector(op->inputs()),
-          FlatBufferIntArrayToVector(op->outputs()),
-          FlatBufferIntArrayToVector(op->intermediates()), nullptr, 0,
-          builtin_data, registration);
     }
+    subgraph->AddNodeWithParameters(
+        FlatBufferIntArrayToVector(op->inputs()),
+        FlatBufferIntArrayToVector(op->outputs()),
+        FlatBufferIntArrayToVector(op->intermediates()), init_data,
+        init_data_size, builtin_data, registration);
   }
 
   return status;
@@ -417,7 +407,22 @@ TfLiteStatus InterpreterBuilder::ParseNodes(
 TfLiteStatus InterpreterBuilder::ParseQuantization(
     const QuantizationParameters* src_quantization,
     TfLiteQuantization* quantization, const std::vector<int>& dims) {
+  // Blockwise quantization.
+  if (src_quantization && src_quantization->details_type() ==
+                              QuantizationDetails_BlockwiseQuantization) {
+    auto* src_quant = src_quantization->details_as_BlockwiseQuantization();
+    quantization->type = kTfLiteBlockwiseQuantization;
+    auto* blockwise_quantization =
+        reinterpret_cast<TfLiteBlockwiseQuantization*>(
+            malloc(sizeof(TfLiteBlockwiseQuantization)));
+    blockwise_quantization->scale = src_quant->scales();
+    blockwise_quantization->quantized_dimension = 0;
+    blockwise_quantization->blocksize = src_quant->block_size();
+    quantization->params = reinterpret_cast<void*>(blockwise_quantization);
+    return kTfLiteOk;
+  }
   quantization->type = kTfLiteNoQuantization;
+  quantization->params = nullptr;
   if (!src_quantization || !src_quantization->scale() ||
       src_quantization->scale()->size() == 0) {
     return kTfLiteOk;
@@ -443,6 +448,15 @@ TfLiteStatus InterpreterBuilder::ParseQuantization(
   }
 
   const size_t num_scales = src_quantization->scale()->size();
+  const size_t num_zero_points = src_quantization->zero_point()->size();
+  // If all of the zero points are the same, only store one to avoid large,
+  // redundant allocations.
+  bool all_zero_points_same = true;
+  int32_t zero_point = src_quantization->zero_point()->data()[0];
+  for (int i = 1; i < num_zero_points && all_zero_points_same; ++i) {
+    all_zero_points_same &=
+        (src_quantization->zero_point()->data()[i] == zero_point);
+  }
 
   // Ensure that the quantization dimension is valid.
   if (src_quantization->quantized_dimension() < 0 ||
@@ -473,11 +487,18 @@ TfLiteStatus InterpreterBuilder::ParseQuantization(
   auto* affine_quantization = reinterpret_cast<TfLiteAffineQuantization*>(
       malloc(sizeof(TfLiteAffineQuantization)));
   affine_quantization->scale = TfLiteFloatArrayCreate(num_scales);
-  affine_quantization->zero_point = TfLiteIntArrayCreate(num_scales);
   for (size_t i = 0; i < num_scales; ++i) {
     affine_quantization->scale->data[i] = src_quantization->scale()->Get(i);
-    affine_quantization->zero_point->data[i] =
-        src_quantization->zero_point()->Get(i);
+  }
+  if (all_zero_points_same) {
+    affine_quantization->zero_point = TfLiteIntArrayCreate(1);
+    affine_quantization->zero_point->data[0] = zero_point;
+  } else {
+    affine_quantization->zero_point = TfLiteIntArrayCreate(num_scales);
+    for (size_t i = 0; i < num_scales; ++i) {
+      affine_quantization->zero_point->data[i] =
+          src_quantization->zero_point()->Get(i);
+    }
   }
   affine_quantization->quantized_dimension =
       src_quantization->quantized_dimension();
@@ -667,7 +688,7 @@ TfLiteStatus InterpreterBuilder::ParseTensors(
     TF_LITE_ENSURE_STATUS(get_readonly_data(&buffer_ptr, &buffer_size));
 
     const auto* src_quantization = tensor->quantization();
-    TfLiteQuantization quantization;
+    TfLiteQuantization quantization{};
     if (ParseQuantization(src_quantization, &quantization, dims) != kTfLiteOk) {
       TF_LITE_REPORT_ERROR(error_reporter_,
                            "Tensor %d has invalid quantization parameters.", i);
@@ -701,7 +722,8 @@ TfLiteStatus InterpreterBuilder::ParseTensors(
 
       if (subgraph->SetTensorParametersReadOnly(
               i, type, get_name(tensor), dims, quantization, buffer_ptr,
-              buffer_size, allocation_, sparsity) != kTfLiteOk) {
+              buffer_size, allocation_, sparsity,
+              /*buffer_identifier=*/tensor->buffer()) != kTfLiteOk) {
         TF_LITE_REPORT_ERROR(error_reporter_,
                              "Tensor %d is invalidly specified in schema.\n",
                              i);
@@ -889,6 +911,20 @@ TfLiteStatus InterpreterBuilder::operator()(
     return cleanup_and_error();
   }
 
+  if (options_.GetUseSignatureTensorNames()) {
+    for (auto& signature_def : (*interpreter)->signature_defs_) {
+      auto* subgraph = (*interpreter)->subgraph(signature_def.subgraph_index);
+      for (auto& [name, tensor_index] : signature_def.inputs) {
+        auto tensor = subgraph->tensor(tensor_index);
+        tensor->name = name.c_str();
+      }
+      for (auto& [name, tensor_index] : signature_def.outputs) {
+        auto tensor = subgraph->tensor(tensor_index);
+        tensor->name = name.c_str();
+      }
+    }
+  }
+
   if ((*interpreter)->SetMetadata(metadata_) != kTfLiteOk) {
     return cleanup_and_error();
   }
