@@ -29,6 +29,7 @@ namespace gpu {
 namespace cl {
 
 absl::Status LoadOpenCL();
+void LoadOpenCLFunctionExtensions(cl_platform_id platform_id);
 
 typedef cl_int(CL_API_CALL *PFN_clGetPlatformIDs)(
     cl_uint /* num_entries */, cl_platform_id * /* platforms */,
@@ -543,6 +544,18 @@ typedef cl_int(CL_API_CALL *PFN_clEnqueueCommandBufferKHR)(
     cl_uint /*num_events_in_wait_list*/, const cl_event * /*event_wait_list*/,
     cl_event * /*event*/);
 
+#if CL_KHR_COMMAND_BUFFER_EXTENSION_VERSION >= CL_MAKE_VERSION(0, 9, 5)
+typedef cl_int(CL_API_CALL *PFN_clCommandNDRangeKernelKHR)(
+    cl_command_buffer_khr /*command_buffer*/,
+    cl_command_queue /*command_queue*/,
+    const cl_command_properties_khr * /*properties*/, cl_kernel /*kernel*/,
+    cl_uint /*work_dim*/, const size_t * /*global_work_offset*/,
+    const size_t * /*global_work_size*/, const size_t * /*local_work_size*/,
+    cl_uint /*num_sync_points_in_wait_list*/,
+    const cl_sync_point_khr * /*sync_point_wait_list*/,
+    cl_sync_point_khr * /*sync_point*/,
+    cl_mutable_command_khr * /*mutable_handle*/);
+#else
 typedef cl_int(CL_API_CALL *PFN_clCommandNDRangeKernelKHR)(
     cl_command_buffer_khr /*command_buffer*/,
     cl_command_queue /*command_queue*/,
@@ -554,12 +567,36 @@ typedef cl_int(CL_API_CALL *PFN_clCommandNDRangeKernelKHR)(
     const cl_sync_point_khr * /*sync_point_wait_list*/,
     cl_sync_point_khr * /*sync_point*/,
     cl_mutable_command_khr * /*mutable_handle*/);
+#endif
 
 typedef cl_int(CL_API_CALL *PFN_clGetCommandBufferInfoKHR)(
     cl_command_buffer_khr /*command_buffer*/,
     cl_command_buffer_info_khr /*param_name*/, size_t /*param_value_size*/,
     void * /*param_value*/, size_t * /*param_value_size_ret*/);
 
+// cl_arm_import_memory extension
+typedef cl_mem(CL_API_CALL *PFN_clImportMemoryARM)(
+    cl_context /*context*/, cl_mem_flags /*flags*/,
+    const cl_import_properties_arm * /*properties*/, void * /*memory*/,
+    size_t /*size*/, cl_int * /*errcode_ret*/);
+
+// cl_khr_semaphore extension
+typedef cl_semaphore_khr(CL_API_CALL *PFN_clCreateSemaphoreWithPropertiesKHR)(
+    cl_context /*context*/, const cl_semaphore_properties_khr * /*sema_props*/,
+    cl_int * /*errcode_ret*/);
+typedef cl_int(CL_API_CALL *PFN_clEnqueueWaitSemaphoresKHR)(
+    cl_command_queue /*command_queue*/, cl_uint /*num_sema_objects*/,
+    const cl_semaphore_khr * /*sema_objects*/,
+    const cl_semaphore_payload_khr * /*sema_payload_list*/,
+    cl_uint /*num_events_in_wait_list*/, const cl_event * /*event_wait_list*/,
+    cl_event * /*event*/);
+typedef cl_int(CL_API_CALL *PFN_clEnqueueSignalSemaphoresKHR)(
+    cl_command_queue /*command_queue*/, cl_uint /*num_sema_objects*/,
+    const cl_semaphore_khr * /*sema_objects*/,
+    const cl_semaphore_payload_khr * /*sema_payload_list*/,
+    cl_uint /*num_events_in_wait_list*/, const cl_event * /*event_wait_list*/,
+    cl_event * /*event*/);
+
 extern PFN_clGetPlatformIDs clGetPlatformIDs;
 extern PFN_clGetPlatformInfo clGetPlatformInfo;
 extern PFN_clGetDeviceIDs clGetDeviceIDs;
@@ -687,6 +724,15 @@ extern PFN_clEnqueueCommandBufferKHR clEnqueueCommandBufferKHR;
 extern PFN_clCommandNDRangeKernelKHR clCommandNDRangeKernelKHR;
 extern PFN_clGetCommandBufferInfoKHR clGetCommandBufferInfoKHR;
 
+// cl_arm_import_memory extension
+extern PFN_clImportMemoryARM clImportMemoryARM;
+
+// cl_khr_semaphore extension
+extern PFN_clCreateSemaphoreWithPropertiesKHR
+    clCreateSemaphoreWithPropertiesKHR;
+extern PFN_clEnqueueWaitSemaphoresKHR clEnqueueWaitSemaphoresKHR;
+extern PFN_clEnqueueSignalSemaphoresKHR clEnqueueSignalSemaphoresKHR;
+
 // For convenient image creation
 // It uses clCreateImage if it available (clCreateImage available since cl 1.2)
 // otherwise it will use legacy clCreateImage2D
