@@ -20,12 +20,10 @@ limitations under the License.
 
 #include "tensorflow/lite/delegates/gpu/delegate.h"
 
-#if defined(__ANDROID__)
-#include <android/hardware_buffer.h>
-#endif
-
 #include <algorithm>
+#include <atomic>
 #include <cstdint>
+#include <cstring>
 #include <memory>
 #include <string>
 #include <thread>  // NOLINT(build/c++11)
@@ -34,40 +32,47 @@ limitations under the License.
 
 #include "absl/container/flat_hash_map.h"
 #include "absl/container/flat_hash_set.h"
+#include "absl/strings/numbers.h"
 #include "absl/types/span.h"
 #include "tensorflow/lite/builtin_ops.h"
-
-#if defined(__ANDROID__)
-#include "tensorflow/lite/core/async/backend_async_kernel_interface.h"
-#include "tensorflow/lite/core/async/c/task.h"
-#include "tensorflow/lite/core/async/interop/c/attribute_map.h"
-#include "tensorflow/lite/core/async/interop/c/constants.h"
-#endif
-
 #include "tensorflow/lite/core/c/common.h"
 #include "tensorflow/lite/delegates/gpu/api.h"
 #include "tensorflow/lite/delegates/gpu/cl/api.h"
 #include "tensorflow/lite/delegates/gpu/cl/util.h"
+#include "tensorflow/lite/delegates/gpu/common/data_type.h"
+#include "tensorflow/lite/delegates/gpu/common/model.h"
 #include "tensorflow/lite/delegates/gpu/common/model_builder.h"
 #include "tensorflow/lite/delegates/gpu/common/model_builder_helper.h"
 #include "tensorflow/lite/delegates/gpu/common/quantization_util.h"
+#include "tensorflow/lite/delegates/gpu/common/status.h"
 #include "tensorflow/lite/delegates/gpu/delegate_options.h"
 #include "tensorflow/lite/delegates/gpu/tflite_profile.h"
 #include "tensorflow/lite/delegates/serialization.h"
+#include "tensorflow/lite/kernels/kernel_util.h"
+#include "tensorflow/lite/logger.h"
+#include "tensorflow/lite/minimal_logging.h"
+#include "tensorflow/lite/profiling/telemetry/c/telemetry_setting.h"
+#include "tensorflow/lite/profiling/telemetry/telemetry.h"
+#include "tensorflow/lite/profiling/telemetry/telemetry_status.h"
 
 #if defined(__ANDROID__)
+#include <android/hardware_buffer.h>
+
+#include "tensorflow/lite/async/backend_async_kernel_interface.h"
+#include "tensorflow/lite/core/async/c/task.h"
+#include "tensorflow/lite/core/async/interop/c/attribute_map.h"
+#include "tensorflow/lite/core/async/interop/c/constants.h"
+#include "tensorflow/lite/core/async/interop/c/types.h"
+#include "tensorflow/lite/delegates/gpu/android_hardware_buffer.h"
+#include "tensorflow/lite/delegates/gpu/async_buffers.h"
+#include "tensorflow/lite/delegates/gpu/gl/android_sync.h"
+#include "tensorflow/lite/delegates/gpu/gl/egl_environment.h"
 #include "tensorflow/lite/delegates/utils/async_type_helpers.h"
 #include "tensorflow/lite/delegates/utils/ret_macros.h"
 #include "tensorflow/lite/delegates/utils/sync_fence.h"
 #include "tensorflow/lite/delegates/utils/utils.h"
 #endif
 
-#include "tensorflow/lite/kernels/kernel_util.h"
-#include "tensorflow/lite/minimal_logging.h"
-#include "tensorflow/lite/profiling/telemetry/c/telemetry_setting_internal.h"
-#include "tensorflow/lite/profiling/telemetry/telemetry.h"
-#include "tensorflow/lite/profiling/telemetry/telemetry_status.h"
-
 #ifndef CL_DELEGATE_NO_GL
 #include "tensorflow/lite/delegates/gpu/gl/api2.h"
 #endif
@@ -101,24 +106,17 @@ using tflite::delegates::utils::WriteSyncAttrs;
   } while (false)
 
 // This idiom allows selecting alternate code paths depending on whether or not
-// AHWB is available.  However, it's still necessary to directly guard calls to
-// AHardwareBuffer_* functions with "if (__builtin_available(android 26, *))" to
-// avoid compiler errors.
-#define TFLITE_AHWB_AVAILABLE()               \
-  [] {                                        \
-    if (__builtin_available(android 26, *)) { \
-      return true;                            \
-    } else {                                  \
-      return false;                           \
-    }                                         \
-  }()
+// AHWB is available.
+#define TFLITE_AHWB_AVAILABLE() \
+  ::tflite::gpu::OptionalAndroidHardwareBuffer::Instance().Supported()
 
 namespace tflite {
 namespace gpu {
 namespace {
-
+// TODO(b/328628170): Add productive coverage to GPU delegate.
 using delegates::Serialization;
 using delegates::SerializationParams;
+using tflite::TFLITE_LOG_WARNING;
 
 constexpr char kSerializedDataPrefix[] = "gpuv2_data_";
 
@@ -154,6 +152,69 @@ InferenceUsage ToUsage(int32_t usage) {
   return InferenceUsage::UNKNOWN;
 }
 
+bool ParseOptions(const char* const* options_keys,
+                  const char* const* options_values, size_t num_options,
+                  TfLiteGpuDelegateOptionsV2* options) {
+  for (size_t i = 0; i < num_options; ++i) {
+    if (strcmp(options_keys[i], "is_precision_loss_allowed")) {
+      if (!absl::SimpleAtoi(options_values[i],
+                            &options->is_precision_loss_allowed)) {
+        TFLITE_LOG(TFLITE_LOG_WARNING, "ParseOptions: malformed option %s.",
+                   options_keys[i]);
+        return false;
+      }
+    } else if (strcmp(options_keys[i], "inference_preference")) {
+      if (!absl::SimpleAtoi(options_values[i],
+                            &options->inference_preference)) {
+        TFLITE_LOG(TFLITE_LOG_WARNING, "ParseOptions: malformed option %s.",
+                   options_keys[i]);
+        return false;
+      }
+    } else if (strcmp(options_keys[i], "inference_priority1")) {
+      if (!absl::SimpleAtoi(options_values[i], &options->inference_priority1)) {
+        TFLITE_LOG(TFLITE_LOG_WARNING, "ParseOptions: malformed option %s.",
+                   options_keys[i]);
+        return false;
+      }
+    } else if (strcmp(options_keys[i], "inference_priority2")) {
+      if (!absl::SimpleAtoi(options_values[i], &options->inference_priority2)) {
+        TFLITE_LOG(TFLITE_LOG_WARNING, "ParseOptions: malformed option %s.",
+                   options_keys[i]);
+        return false;
+      }
+    } else if (strcmp(options_keys[i], "inference_priority3")) {
+      if (!absl::SimpleAtoi(options_values[i], &options->inference_priority3)) {
+        TFLITE_LOG(TFLITE_LOG_WARNING, "ParseOptions: malformed option %s.",
+                   options_keys[i]);
+        return false;
+      }
+    } else if (strcmp(options_keys[i], "experimental_flags")) {
+      if (!absl::SimpleAtoi(options_values[i], &options->experimental_flags)) {
+        TFLITE_LOG(TFLITE_LOG_WARNING, "ParseOptions: malformed option %s.",
+                   options_keys[i]);
+        return false;
+      }
+    } else if (strcmp(options_keys[i], "max_delegated_partitions")) {
+      if (!absl::SimpleAtoi(options_values[i],
+                            &options->max_delegated_partitions)) {
+        TFLITE_LOG(TFLITE_LOG_WARNING, "ParseOptions: malformed option %s.",
+                   options_keys[i]);
+        return false;
+      }
+    } else if (strcmp(options_keys[i], "serialization_dir")) {
+      options->serialization_dir = options_values[i];
+    } else if (strcmp(options_keys[i], "model_token")) {
+      options->model_token = options_values[i];
+    } else {
+      TFLITE_LOG(TFLITE_LOG_WARNING, "ParseOptions: unknown option %s.",
+                 options_keys[i]);
+      return false;
+    }
+  }
+
+  return true;
+}
+
 // Forward declarations.
 TfLiteStatus DelegatePrepare(TfLiteContext* context, TfLiteDelegate* delegate);
 
@@ -313,7 +374,8 @@ ObjectDef DelegateKernelCore::GetObjectDef(int index,
   ObjectDef default_object_def;
   default_object_def.data_type = data_type;
   default_object_def.data_layout = DataLayout::BHWC;
-  default_object_def.object_type = ObjectType::CPU_MEMORY;
+  default_object_def.object_type =
+      delegate_->async() ? ObjectType::OPENGL_SSBO : ObjectType::CPU_MEMORY;
   default_object_def.user_provided = true;
   return default_object_def;
 }
@@ -396,11 +458,6 @@ absl::Status DelegateKernelCore::Setup(
                                         delegate_->serialization()));
     backend_opencl = true;
   } else if (experimental_flags & TFLITE_GPU_EXPERIMENTAL_FLAGS_GL_ONLY) {
-    if (delegate_->async()) {
-      // TODO(b/245966018): Add support for this. Probably straightforward, as
-      // Xeno API hides most of the CL/GL distinction.
-      return absl::UnimplementedError("async API does not support OpenGL");
-    }
     RETURN_IF_ERROR(InitializeOpenGlApi(&graph, &builder));
   } else {
     // By default, we try CL first & fall back to GL if that fails.
@@ -408,13 +465,8 @@ absl::Status DelegateKernelCore::Setup(
         InitializeOpenClApi(&graph, &builder, &graph_is_destroyed, context,
                             delegate_params, delegate_->serialization());
     if (!status.ok()) {
-      TF_LITE_KERNEL_LOG(context, std::string(status.message()).c_str());
+      TF_LITE_KERNEL_LOG(context, "%s", std::string(status.message()).c_str());
       TF_LITE_KERNEL_LOG(context, "Falling back to OpenGL");
-      if (delegate_->async()) {
-        // TODO(b/245966018): Add support for this. Probably straightforward, as
-        // Xeno API hides most of the CL/GL distinction.
-        return absl::UnimplementedError("async API does not support OpenGL");
-      }
 
       // Graph needs to be re-created because it is moved above.
       GraphFloat32 graph2;
@@ -488,6 +540,10 @@ absl::Status DelegateKernelCore::InitializeOpenClApi(
   }
   options.usage = ToUsage(delegate_options.inference_preference);
 
+#ifdef TFLITE_GPU_ENABLE_INVOKE_LOOP
+  options.gpu_invoke_loop_times = delegate_options.gpu_invoke_loop_times;
+#endif
+
   if (!serialization) {
     // This path is faster when there is no serialization involved.
     RETURN_IF_ERROR(cl::NewInferenceEnvironment(env_options, &cl_environment_,
@@ -526,12 +582,6 @@ absl::Status DelegateKernelCore::InitializeOpenClApi(
 absl::Status DelegateKernelCore::InitializeOpenGlApi(
     GraphFloat32* graph, std::unique_ptr<InferenceBuilder>* builder) {
 #ifndef CL_DELEGATE_NO_GL
-  if (delegate_->async()) {
-    // TODO(b/245966018): Add support for this. Probably straightforward, as
-    // Xeno API hides most of the CL/GL distinction.
-    return absl::UnimplementedError("async API does not support OpenGL");
-  }
-
   gl::InferenceEnvironmentOptions env_options;
   gl::InferenceEnvironmentProperties properties;
   RETURN_IF_ERROR(
@@ -539,9 +589,22 @@ absl::Status DelegateKernelCore::InitializeOpenGlApi(
   auto delegate_options = delegate_->options();
   gl::InferenceOptions options;
   options.usage = ToUsage(delegate_options.inference_preference);
-  options.priority1 = ToPriority(delegate_options.inference_priority1);
-  options.priority2 = ToPriority(delegate_options.inference_priority2);
-  options.priority3 = ToPriority(delegate_options.inference_priority3);
+  // If is_precision_loss_allowed == -1, then just use priorities instead
+  // of paying attention to is_precision_loss_allowed value.
+  if (delegate_options.is_precision_loss_allowed == -1) {
+    options.priority1 = ToPriority(delegate_options.inference_priority1);
+    options.priority2 = ToPriority(delegate_options.inference_priority2);
+    options.priority3 = ToPriority(delegate_options.inference_priority3);
+  } else {
+    if (delegate_options.is_precision_loss_allowed == 0) {
+      options.priority1 = InferencePriority::MAX_PRECISION;
+    } else {
+      options.priority1 = InferencePriority::MIN_LATENCY;
+    }
+  }
+#ifdef TFLITE_GPU_ENABLE_INVOKE_LOOP
+  options.gpu_invoke_loop_times = delegate_options.gpu_invoke_loop_times;
+#endif
   RETURN_IF_ERROR(gl_environment_->NewInferenceBuilder(std::move(*graph),
                                                        options, builder));
   enforce_same_thread_ = true;
@@ -742,6 +805,10 @@ class DelegateAsyncKernel : public BackendAsyncKernelInterface {
   TfLiteStatus SetAttributes(TfLiteOpaqueContext* context,
                              TfLiteOpaqueNode* node, int tensor_index,
                              const TfLiteAttributeMap* attrs) override;
+  TfLiteStatus SetBufferAttributes(const TfLiteBackendBuffer* buffer,
+                                   const TfLiteAttributeMap* attrs) override;
+  TfLiteStatus GetBufferAttributes(const TfLiteBackendBuffer* buffer,
+                                   TfLiteAttributeMap* attrs) override;
   TfLiteStatus Prepare(TfLiteOpaqueContext* context,
                        TfLiteOpaqueNode* node) override;
 
@@ -783,24 +850,24 @@ class DelegateAsyncKernel : public BackendAsyncKernelInterface {
   using UniquePtrAHardwareBuffer =
       std::unique_ptr<AHardwareBuffer, void (*)(AHardwareBuffer*)>;
   static UniquePtrAHardwareBuffer Acquire(AHardwareBuffer* ahwb) {
-    if (__builtin_available(android 26, *)) {
-      AHardwareBuffer_acquire(ahwb);
+    if (OptionalAndroidHardwareBuffer::Instance().Supported()) {
+      OptionalAndroidHardwareBuffer::Instance().Acquire(ahwb);
+      return UniquePtrAHardwareBuffer(ahwb, [](AHardwareBuffer* b) {
+        OptionalAndroidHardwareBuffer::Instance().Release(b);
+      });
     } else {
       TFLITE_LOG_PROD(TFLITE_LOG_ERROR,
                       "attempting AHardwareBuffer_acquire on a device without "
                       "AHardwareBuffer support");
+      return {nullptr, [](AHardwareBuffer*) {}};
     }
-    return UniquePtrAHardwareBuffer(ahwb, [](AHardwareBuffer* b) {
-      if (__builtin_available(android 26, *)) {
-        AHardwareBuffer_release(b);
-      }
-    });
   }
   static AHardwareBuffer_Desc Describe(
       const UniquePtrAHardwareBuffer& uptr_ahwb) {
     AHardwareBuffer_Desc desc_ahwb = {};
-    if (__builtin_available(android 26, *)) {
-      AHardwareBuffer_describe(uptr_ahwb.get(), &desc_ahwb);
+    if (OptionalAndroidHardwareBuffer::Instance().Supported()) {
+      OptionalAndroidHardwareBuffer::Instance().Describe(uptr_ahwb.get(),
+                                                         &desc_ahwb);
     } else {
       TFLITE_LOG_PROD(TFLITE_LOG_ERROR,
                       "attempting AHardwareBuffer_describe on a device without "
@@ -809,6 +876,37 @@ class DelegateAsyncKernel : public BackendAsyncKernelInterface {
     return desc_ahwb;
   }
 
+  // Validate the attributes passed in, return kTfLiteOk if the attributes
+  // meet the requirements. Return the registered buffer attributes in
+  // `buffer_attrs`.
+  static TfLiteStatus CheckAttributes(const TfLiteAttributeMap* attrs,
+                                      BufferAttributes& buffer_attrs) {
+    // Validate buffer attributes.
+    TFLITE_RET_CHECK_STATUS(
+        TfLiteAttributeMapIsBufferAttributeMap(attrs),
+        "calling RegisterBuffer with invalid attribute map type");
+    buffer_attrs = ReadBufferAttrs(attrs);
+    TFLITE_RET_CHECK_STATUS(
+        buffer_attrs.buffer_type.has_value(),
+        "calling RegisterBuffer with buffer resource type name unspecified");
+    TFLITE_RET_CHECK_STATUS(
+        buffer_attrs.buffer_type.value() != BufferType::kUnknown,
+        "calling RegisterBuffer with unknown buffer resource type");
+    size_t alignment = buffer_attrs.alignment.value_or(kRequiredByteAlignment);
+    TFLITE_RET_CHECK_STATUS(
+        alignment % kRequiredByteAlignment == 0,
+        "calling RegisterBuffer with non-zero buffer alignment");
+    size_t padding = buffer_attrs.padding.value_or(kRequiredBytePadding);
+    TFLITE_RET_CHECK_STATUS(
+        padding % kRequiredBytePadding == 0,
+        "calling RegisterBuffer with non-zero buffer padding");
+    size_t offset = buffer_attrs.offset.value_or(0);
+    TFLITE_RET_CHECK_STATUS(offset == 0,
+                            "calling RegisterBuffer with non-zero offset");
+
+    return kTfLiteOk;
+  }
+
   // For SupportedBufferTypes and SupportedSynchronizations
   const std::vector<const char*> supported_buffer_types_ = {
       ::tflite::delegates::utils::kBufferTypeAHardwareBufferBlob};
@@ -818,15 +916,22 @@ class DelegateAsyncKernel : public BackendAsyncKernelInterface {
 
   mutable absl::Mutex mutex_;
 
-  absl::flat_hash_map<TfLiteBufferHandle, UniquePtrAHardwareBuffer>
-      buffer_by_handle_ ABSL_GUARDED_BY(mutex_);
   absl::flat_hash_map<int, SyncType> sync_type_by_tensor_index_
       ABSL_GUARDED_BY(mutex_);
   std::vector<SyncType> input_sync_types_ ABSL_GUARDED_BY(mutex_);
-  std::vector<SyncType> output_sync_types_ ABSL_GUARDED_BY(mutex_);
 
   // Whether 'Prepare' is called or not.
   bool prepared_ ABSL_GUARDED_BY(mutex_) = false;
+
+  // Create mutex for thread-safe data transfer from GPU prepare -> GPU eval
+  mutable absl::Mutex eval_mutex_;
+
+  absl::flat_hash_map<TfLiteBufferHandle, UniquePtrAHardwareBuffer>
+      buffer_by_handle_ ABSL_GUARDED_BY(eval_mutex_);
+
+  absl::flat_hash_map<AHardwareBuffer*, BufferAttributes> attributes_by_buffer_
+      ABSL_GUARDED_BY(eval_mutex_);
+  std::vector<SyncType> output_sync_types_ ABSL_GUARDED_BY(eval_mutex_);
 };
 
 absl::Status DelegateAsyncKernel::Init(TfLiteContext* context,
@@ -997,6 +1102,82 @@ TfLiteStatus DelegateAsyncKernel::SetAttributesImpl(
   return kTfLiteOk;
 }
 
+TfLiteStatus DelegateAsyncKernel::SetBufferAttributes(
+    const TfLiteBackendBuffer* buffer, const TfLiteAttributeMap* attrs) {
+  TFLITE_ABORT_CHECK(buffer != nullptr, "Buffer is null");
+  TFLITE_ABORT_CHECK(attrs != nullptr, "Attribute is null");
+
+  // We depend on the availability of AHardwareBuffer.
+  TFLITE_RET_CHECK_STATUS(
+      TFLITE_AHWB_AVAILABLE(),
+      "calling tflite::gpu::DelegateAsyncKernel::SetBufferAttributes on device "
+      "without AHardwareBuffer support");
+  BufferAttributes buffer_attrs;
+  TFLITE_RET_CHECK_STATUS(CheckAttributes(attrs, buffer_attrs) == kTfLiteOk,
+                          "SetBufferAttributes(): Failed to check attributes");
+
+  // Validate ahardwarebuffer.
+  auto* ahwb =
+      reinterpret_cast<AHardwareBuffer*>(TfLiteBackendBufferGetPtr(buffer));
+  TFLITE_RET_CHECK_STATUS(ahwb != nullptr,
+                          "calling SetBufferAttributes with nullptr buffer");
+  UniquePtrAHardwareBuffer uptr_ahwb = Acquire(ahwb);
+  const AHardwareBuffer_Desc desc_ahwb = Describe(uptr_ahwb);
+  TFLITE_RET_CHECK_STATUS(desc_ahwb.format == AHARDWAREBUFFER_FORMAT_BLOB,
+                          "calling SetBufferAttributes with an AHardwareBuffer "
+                          "of format other than BLOB is not supported");
+  size_t size = buffer_attrs.size.value_or(desc_ahwb.width);
+  TFLITE_RET_CHECK_STATUS(
+      size <= desc_ahwb.width,
+      "calling SetBufferAttributes with buffer size larger than the actual "
+      "AHardwareBuffer size");
+
+  absl::MutexLock eval_lock(&eval_mutex_);
+  if (attributes_by_buffer_.find(uptr_ahwb.get()) !=
+      attributes_by_buffer_.end()) {
+    attributes_by_buffer_[uptr_ahwb.get()] = buffer_attrs;
+  } else {
+    TFLITE_LOG_PROD(
+        TFLITE_LOG_ERROR,
+        "SetBufferAttributes(): Unable to find the buffer in the map.");
+  }
+  return kTfLiteOk;
+}
+
+TfLiteStatus DelegateAsyncKernel::GetBufferAttributes(
+    const TfLiteBackendBuffer* buffer, TfLiteAttributeMap* attrs) {
+  TFLITE_ABORT_CHECK(buffer != nullptr, "Buffer is null");
+  TFLITE_ABORT_CHECK(attrs != nullptr, "Attribute map is null");
+
+  // We depend on the availability of AHardwareBuffer.
+  TFLITE_RET_CHECK_STATUS(
+      TFLITE_AHWB_AVAILABLE(),
+      "calling tflite::gpu::DelegateAsyncKernel::GetBufferAttributes on device "
+      "without AHardwareBuffer support");
+  TFLITE_RET_CHECK_STATUS(
+      TfLiteAttributeMapIsBufferAttributeMap(attrs),
+      "calling GetBufferAttributes with an invalid attribute map type");
+
+  // Validate ahardwarebuffer.
+  auto* ahwb =
+      reinterpret_cast<AHardwareBuffer*>(TfLiteBackendBufferGetPtr(buffer));
+  TFLITE_RET_CHECK_STATUS(ahwb != nullptr,
+                          "calling GetBufferAttributes with nullptr buffer");
+  UniquePtrAHardwareBuffer uptr_ahwb = Acquire(ahwb);
+  const AHardwareBuffer_Desc desc_ahwb = Describe(uptr_ahwb);
+  TFLITE_RET_CHECK_STATUS(desc_ahwb.format == AHARDWAREBUFFER_FORMAT_BLOB,
+                          "calling GetBufferAttributes with an AHardwareBuffer "
+                          "of format other than "
+                          "BLOB is not supported");
+
+  absl::MutexLock eval_lock(&eval_mutex_);
+  auto it = attributes_by_buffer_.find(uptr_ahwb.get());
+  TFLITE_RET_CHECK_STATUS(it != attributes_by_buffer_.end(),
+                          "Unable to find the buffer.");
+  WriteBufferAttrs(it->second, attrs);
+  return kTfLiteOk;
+}
+
 TfLiteStatus DelegateAsyncKernel::Prepare(TfLiteOpaqueContext* opaque_context,
                                           TfLiteOpaqueNode* opaque_node) {
   // The following cast is safe only because this code is part of the
@@ -1014,6 +1195,7 @@ TfLiteStatus DelegateAsyncKernel::PrepareImpl(TfLiteContext* context,
   TFLITE_RET_CHECK_STATUS(!prepared_, "Prepare must be called at most once");
 
   input_sync_types_.resize(node->inputs->size, SyncType::kNoSyncObj);
+  absl::MutexLock eval_lock(&eval_mutex_);
   output_sync_types_.resize(node->outputs->size, SyncType::kNoSyncObj);
   for (size_t i = 0; i < node->inputs->size; ++i) {
     auto it = sync_type_by_tensor_index_.find(node->inputs->data[i]);
@@ -1053,34 +1235,14 @@ TfLiteStatus DelegateAsyncKernel::RegisterBufferImpl(
   TFLITE_ABORT_CHECK(buffer != nullptr, "");                  // Crash OK
   TFLITE_ABORT_CHECK(attrs != nullptr, "");                   // Crash OK
   TFLITE_ABORT_CHECK(handle != kTfLiteNullBufferHandle, "");  // Crash OK
-
-  // Validate buffer attributes.
-  TFLITE_RET_CHECK_STATUS(
-      TfLiteAttributeMapIsBufferAttributeMap(attrs),
-      "calling RegisterBuffer with invalid attribute map type");
-  auto buffer_attrs = ReadBufferAttrs(attrs);
-  TFLITE_RET_CHECK_STATUS(
-      buffer_attrs.buffer_type.has_value(),
-      "calling RegisterBuffer with buffer resource type name unspecified");
-  TFLITE_RET_CHECK_STATUS(
-      buffer_attrs.buffer_type.value() != BufferType::kUnknown,
-      "calling RegisterBuffer with unknown buffer resource type");
-  size_t alignment = buffer_attrs.alignment.value_or(kRequiredByteAlignment);
-  TFLITE_RET_CHECK_STATUS(
-      alignment % kRequiredByteAlignment == 0,
-      "calling RegisterBuffer with invalid buffer alignment");
-  size_t padding = buffer_attrs.padding.value_or(kRequiredBytePadding);
-  TFLITE_RET_CHECK_STATUS(padding % kRequiredBytePadding == 0,
-                          "calling RegisterBuffer with invalid buffer padding");
-  size_t offset = buffer_attrs.offset.value_or(0);
-  TFLITE_RET_CHECK_STATUS(offset == 0,
-                          "calling RegisterBuffer with non-zero offset");
-
   // We depend on the availability of AHardwareBuffer.
   TFLITE_RET_CHECK_STATUS(
       TFLITE_AHWB_AVAILABLE(),
       "calling tflite::gpu::DelegateAsyncKernel::RegisterBuffer on device "
       "without AHardwareBuffer support");
+  BufferAttributes buffer_attrs;
+  TFLITE_RET_CHECK_STATUS(CheckAttributes(attrs, buffer_attrs) == kTfLiteOk,
+                          "RegisterBufferImpl(): Failed to check attributes");
 
   // Retrieve and validate the buffer.
   auto* ahwb =
@@ -1100,11 +1262,15 @@ TfLiteStatus DelegateAsyncKernel::RegisterBufferImpl(
       "AHardwareBuffer size");
 
   // Register the buffer.
-  absl::MutexLock lock(&mutex_);
+  absl::MutexLock eval_lock(&eval_mutex_);
   auto [it, did_something] =
       buffer_by_handle_.try_emplace(handle, std::move(uptr_ahwb));
   TFLITE_RET_CHECK_STATUS(did_something,
                           "RegisterBuffer called with duplicate handle");
+
+  auto [iterator, check] =
+      attributes_by_buffer_.try_emplace(it->second.get(), buffer_attrs);
+  TFLITE_RET_CHECK_STATUS(check, "RegisterBuffer called with same buffer");
   return kTfLiteOk;
 }
 
@@ -1120,7 +1286,7 @@ TfLiteStatus DelegateAsyncKernel::UnregisterBuffer(
 
 TfLiteStatus DelegateAsyncKernel::UnregisterBufferImpl(
     TfLiteContext* context, TfLiteBufferHandle handle) {
-  absl::MutexLock lock(&mutex_);
+  absl::MutexLock eval_lock(&eval_mutex_);
   auto it = buffer_by_handle_.find(handle);
   TFLITE_RET_CHECK_STATUS(it != buffer_by_handle_.end(),
                           "UnregisterBuffer called with unknown handle");
@@ -1154,181 +1320,76 @@ TfLiteStatus DelegateAsyncKernel::EvalImpl(TfLiteContext* context,
   TFLITE_RET_CHECK_STATUS(TFLITE_AHWB_AVAILABLE(),
                           "calling tflite::gpu::DelegateAsyncKernel::Eval on "
                           "device without AHardwareBuffer support");
-
-  // Utility class to assist in locking and unlocking buffers.  Allows us to
-  // automatically unlock buffers when we leave the scope of the class instance
-  // (e.g., early exit from EvalImpl()).
-  class LockedAHWBs {
-   public:
-    enum class Usage { READ, WRITE };
-
-    explicit LockedAHWBs(Usage usage)
-        : usage_(usage == Usage::READ
-                     ? AHARDWAREBUFFER_USAGE_CPU_READ_RARELY
-                     : AHARDWAREBUFFER_USAGE_CPU_WRITE_RARELY) {}
-    ~LockedAHWBs() { unlock(); }
-
-    // See AHardwareBuffer_lock() but also tracks the lock so that we can unlock
-    // later. If we successfully lock a buffer, and then try again without
-    // unlocking, we'll return the same outVirtualAddress and not try to re-lock
-    // the buffer.
-    int lock(AHardwareBuffer* _Nonnull buffer,
-             void* _Nullable* _Nonnull outVirtualAddress) {
-      if (auto it = buffers_to_addresses_.find(buffer);
-          it != buffers_to_addresses_.end()) {
-        *outVirtualAddress = it->second;
-        return 0;
-      }
-
-      int lock_error = [buffer, outVirtualAddress, this] {
-        if (__builtin_available(android 26, *)) {
-          return AHardwareBuffer_lock(buffer, this->usage_, -1 /* fence */,
-                                      nullptr /* rect */, outVirtualAddress);
-        } else {
-          return EINVAL;
-        }
-      }();
-      if (!lock_error) {
-        auto [it, inserted] =
-            buffers_to_addresses_.insert({buffer, *outVirtualAddress});
-        if (inserted == false) {
-          TFLITE_LOG_PROD(TFLITE_LOG_ERROR, "LockedAHWBs::lock inconsistency");
-          return -EINVAL;
-        }
-      }
-      return lock_error;
+  auto FenceFd = [](TfLiteSynchronization* sync) {
+    if (sync == nullptr) {
+      return -1;
     }
-
-    // Performs AHardwareBuffer_unlock() on all buffers we have locked.
-    // Attempts to unlock all buffers, even if unlock of one or more bufers
-    // fails. Returns the error from the first failure. After this call, we are
-    // no longer tracking any buffers for unlocking.
-    int unlock() {
-      int fn_error = 0;
-
-      for (auto [buffer, address] : buffers_to_addresses_) {
-        if (int unlock_error = [buffer = buffer] {
-              if (__builtin_available(android 26, *)) {
-                return AHardwareBuffer_unlock(buffer, nullptr /* fence */);
-              } else {
-                return EINVAL;
-              }
-            }()) {
-          if (!fn_error) {
-            fn_error = unlock_error;
-          }
-        }
-      }
-
-      buffers_to_addresses_.clear();
-      if (fn_error) {
-        TFLITE_LOG_PROD(TFLITE_LOG_ERROR, "LockedAHWBs::unlock error %d",
-                        fn_error);
-      }
-      return fn_error;
-    }
-
-   private:
-    const int
-        usage_;  // Marked const because this is frozen at construction time and
-                 // because we want the class instance to be non copyable.
-    absl::flat_hash_map<AHardwareBuffer*, void*> buffers_to_addresses_;
-  };
-
-  absl::MutexLock lock(&mutex_);
-  TFLITE_RET_CHECK_STATUS(prepared_, "Eval must be called after Prepare");
-
-  // TODO(b/245966018): Need to call GetRequiredTemporaries? Today the data
-  // lives on TfLiteNode -- is that what we want?
-  TFLITE_RET_CHECK_STATUS(core_.quant_conversion_map().empty(),
-                          "Quantized kernels are not supported");
-
-  // The code below needs to do work corresponding to DelegateKernel::Invoke:
-
-  // Wait on input sync objects.
-  //   Get unique input sync fences.
-  absl::flat_hash_set<int> unique_input_sync_fds_set;
-  for (size_t i = 0; i < node->inputs->size; ++i) {
-    if (input_sync_types_[i] == SyncType::kNoSyncObj) continue;
-    TfLiteSynchronization* sync =
-        TfLiteExecutionTaskGetSyncByIndex(task, node->inputs->data[i]);
-    if (sync == nullptr) continue;
     void* sync_obj = TfLiteSynchronizationGetPtr(sync);
-    if (sync_obj == nullptr) continue;
-    int sync_fence_fd = *(reinterpret_cast<int*>(sync_obj));
-    if (sync_fence_fd == -1) continue;
-    unique_input_sync_fds_set.insert(sync_fence_fd);
+    if (sync_obj == nullptr) {
+      return -1;
+    }
+    return *(reinterpret_cast<int*>(sync_obj));
+  };
+  absl::flat_hash_set<int> unique_input_sync_fds_set;
+  for (int i = 0; i < core_.runner()->inputs().size(); i++) {
+    int fd =
+        FenceFd(TfLiteExecutionTaskGetSyncByIndex(task, node->inputs->data[i]));
+    if (fd == -1) continue;
+    unique_input_sync_fds_set.insert(fd);
   }
+
   //  Wait for all input sync fences to be signalled.
-  std::vector<int> unique_input_sync_fds_vec;
-  unique_input_sync_fds_vec.reserve(unique_input_sync_fds_set.size());
-  for (auto fd : unique_input_sync_fds_set) {
-    unique_input_sync_fds_vec.push_back(fd);
+  std::vector<int> unique_input_cpu_sync_fds_vec;
+  unique_input_cpu_sync_fds_vec.reserve(unique_input_sync_fds_set.size());
+  for (int fd : unique_input_sync_fds_set) {
+    // Check if we can wait on GPU, else wait on CPU
+    if (gl::WaitFdGpu(fd)) continue;
+    unique_input_cpu_sync_fds_vec.push_back(fd);
   }
-  const auto waitfor = WaitForAllFds(unique_input_sync_fds_vec);
+  const auto waitfor = WaitForAllFds(unique_input_cpu_sync_fds_vec);
   TFLITE_RET_CHECK_STATUS(waitfor.has_value(), "wait for input fds");
 
-  // Iterate over inputs or outputs, and for each one do AHardwareBuffer_lock()
-  // and InferenceRunner::Set*Object(). Compare to
-  // DelegateKernel::SetInputsAndOutputs().
-  auto SetInputsOrOutputs =
-      [this, task](LockedAHWBs* locks,
-                   const std::vector<int64_t>& in_or_out_indices,
-                   absl::Status (InferenceRunner::*set_object)(
-                       int index, TensorObject object))
-          ABSL_SHARED_LOCKS_REQUIRED(mutex_) -> TfLiteStatus {
-    for (int i = 0; i < in_or_out_indices.size(); ++i) {
-      int context_tensor_index = in_or_out_indices[i];
-      TfLiteBufferHandle handle =
-          TfLiteExecutionTaskGetBufferByIndex(task, context_tensor_index);
-      TFLITE_RET_CHECK_STATUS(handle >= 0, "bad handle");
-      const UniquePtrAHardwareBuffer& uptr_ahwb = buffer_by_handle_.at(handle);
-      void* virtualAddress = nullptr;
-      TFLITE_RET_CHECK_STATUS(
-          0 == locks->lock(uptr_ahwb.get(), &virtualAddress), "lock() failed");
-      TFLITE_RETURN_IF_ABSL_ERROR((core_.runner().get()->*set_object)(
-          i,
-          MakeCpuMemory(absl::MakeSpan(reinterpret_cast<char*>(virtualAddress),
-                                       Describe(uptr_ahwb).width))));
-    }
-    return kTfLiteOk;
-  };
+  // Needed for cl inference. For gl it re-uses the existing context.
+  std::unique_ptr<gl::EglEnvironment> env;
+  TFLITE_RETURN_IF_ABSL_ERROR(gl::EglEnvironment::NewEglEnvironment(&env));
+  for (int i = 0; i < core_.runner()->inputs().size(); i++) {
+    TensorObjectDef tensor_def = core_.runner()->inputs()[i];
+    TfLiteBufferHandle handle =
+        TfLiteExecutionTaskGetBufferByIndex(task, core_.input_indices()[i]);
+    TFLITE_RET_CHECK_STATUS(handle >= 0, "bad handle");
 
-  LockedAHWBs read_locks(LockedAHWBs::Usage::READ);
-  TFLITE_RETURN_IF_ERROR(SetInputsOrOutputs(&read_locks, core_.input_indices(),
-                                            &InferenceRunner::SetInputObject));
-
-  // TODO(b/245966018): Dequantize inputs, if needed (see
-  // core_.quant_conversion_map())
-
-  LockedAHWBs write_locks(LockedAHWBs::Usage::WRITE);
-  TFLITE_RETURN_IF_ERROR(SetInputsOrOutputs(
-      &write_locks, core_.output_indices(), &InferenceRunner::SetOutputObject));
-
-  TFLITE_RETURN_IF_ABSL_ERROR(core_.runner()->Run());
-
-  int unlock_status = read_locks.unlock();
-
-  // TODO(b/245966018): Quantize outputs, if needed (see
-  // core_.quant_conversion_map())
-
-  if (int write_unlock_status = write_locks.unlock()) {
-    if (!unlock_status) {
-      unlock_status = write_unlock_status;
-    }
+    absl::MutexLock eval_lock(&eval_mutex_);
+    AHardwareBuffer* ahwb = buffer_by_handle_.at(handle).get();
+    AsyncBuffer async_buffer = AsyncBuffer(tensor_def, ahwb);
+    OpenGlBuffer buffer;
+    TFLITE_RETURN_IF_ABSL_ERROR(async_buffer.GetOpenGlBuffer(buffer.id));
+    TFLITE_RETURN_IF_ABSL_ERROR(
+        core_.runner()->SetInputObject(i, std::move(buffer)));
   }
-
-  // Signal sync objects by indicating that inference has completed and there
-  // are no output sync fences.
+  for (int i = 0; i < core_.runner()->outputs().size(); i++) {
+    TensorObjectDef tensor_def = core_.runner()->outputs()[i];
+    TfLiteBufferHandle handle =
+        TfLiteExecutionTaskGetBufferByIndex(task, core_.output_indices()[i]);
+    TFLITE_RET_CHECK_STATUS(handle >= 0, "bad handle");
+    absl::MutexLock eval_lock(&eval_mutex_);
+    AHardwareBuffer* ahwb = buffer_by_handle_.at(handle).get();
+    AsyncBuffer async_buffer = AsyncBuffer(tensor_def, ahwb);
+    OpenGlBuffer buffer;
+    TFLITE_RETURN_IF_ABSL_ERROR(async_buffer.GetOpenGlBuffer(buffer.id));
+    TFLITE_RETURN_IF_ABSL_ERROR(
+        core_.runner()->SetOutputObject(i, std::move(buffer)));
+  }
+  TFLITE_RETURN_IF_ABSL_ERROR(core_.runner()->Run());
+  // Add sync objects
   for (size_t i = 0; i < node->outputs->size; ++i) {
+    absl::MutexLock eval_lock(&eval_mutex_);
     if (output_sync_types_[i] == SyncType::kNoSyncObj) continue;
     TfLiteSynchronization* sync =
         TfLiteExecutionTaskGetSyncByIndex(task, node->outputs->data[i]);
     if (sync == nullptr) continue;
-    TfLiteSynchronizationSetPtr(sync, new int{-1});
+    TfLiteSynchronizationSetPtr(sync, new int{gl::CreateFdGpu()});
+    TfLiteExecutionTaskSetSyncByIndex(task, node->outputs->data[i], sync);
   }
-
-  TFLITE_RET_CHECK_STATUS(unlock_status == 0, "unlock() failed");
   return kTfLiteOk;
 }
 
@@ -1498,7 +1559,9 @@ TfLiteStatus DelegatePrepare(TfLiteContext* context, TfLiteDelegate* delegate) {
       context, "GpuDelegate::DelegatePrepare",
       telemetry::TelemetrySource::TFLITE_GPU, delegate_setting);
 
-  SetTfLiteProfiler(context->profiler);
+  if (delegate->flags & kTfLiteDelegateFlagsPerOperatorProfiling) {
+    SetTfLiteProfiler(context->profiler);
+  }
   return status;
 }
 
@@ -1535,3 +1598,19 @@ TfLiteDelegate* TfLiteGpuDelegateV2CreateAsync(
 void TfLiteGpuDelegateV2Delete(TfLiteDelegate* delegate) {
   delete tflite::gpu::GetDelegate(delegate);
 }
+
+TfLiteDelegate* tflite_plugin_create_delegate(
+    const char* const* options_keys, const char* const* options_values,
+    size_t num_options, void (*report_error)(const char*)) {
+  TfLiteGpuDelegateOptionsV2 options = TfLiteGpuDelegateOptionsV2Default();
+  if (!tflite::gpu::ParseOptions(options_keys, options_values, num_options,
+                                 &options)) {
+    return nullptr;
+  }
+
+  return TfLiteGpuDelegateV2Create(&options);
+}
+
+void tflite_plugin_destroy_delegate(TfLiteDelegate* delegate) {
+  TfLiteGpuDelegateV2Delete(delegate);
+}
