@@ -14,10 +14,12 @@ limitations under the License.
 ==============================================================================*/
 #include "tensorflow/lite/delegates/hexagon/builders/strided_slice_builder.h"
 
+#include <cstdint>
 #include <vector>
 
+#include "tensorflow/lite/core/c/builtin_op_data.h"
 #include "tensorflow/lite/core/c/common.h"
-#include "tensorflow/lite/kernels/internal/tensor.h"
+#include "tensorflow/lite/delegates/hexagon/builders/op_builder.h"
 
 namespace tflite {
 namespace delegates {
