@@ -14,17 +14,20 @@ limitations under the License.
 ==============================================================================*/
 #include "tensorflow/lite/delegates/hexagon/hexagon_delegate_kernel.h"
 
+#include <cstdint>
+#include <cstdio>
+#include <ctime>
 #include <memory>
 #include <string>
 #include <vector>
 
-#include "tensorflow/lite/builtin_ops.h"
+#include "hexagon/hexagon_nn.h"
 #include "tensorflow/lite/context_util.h"
-#include "tensorflow/lite/core/c/builtin_op_data.h"
+#include "tensorflow/lite/core/api/profiler.h"
 #include "tensorflow/lite/core/c/common.h"
+#include "tensorflow/lite/delegates/hexagon/builders/op_builder.h"
 #include "tensorflow/lite/delegates/hexagon/hexagon_implementation.h"
 #include "tensorflow/lite/delegates/hexagon/utils.h"
-#include "tensorflow/lite/kernels/kernel_util.h"
 
 namespace tflite {
 
