@@ -29,7 +29,10 @@ limitations under the License.
 #ifndef TENSORFLOW_LITE_DELEGATES_UTILS_SIMPLE_DELEGATE_H_
 #define TENSORFLOW_LITE_DELEGATES_UTILS_SIMPLE_DELEGATE_H_
 
+#include <stdint.h>
+
 #include <memory>
+#include <utility>
 
 #include "tensorflow/lite/core/c/common.h"
 
@@ -42,7 +45,7 @@ using TfLiteDelegateUniquePtr =
 // Each instance represents a single part of the graph (subgraph).
 class SimpleDelegateKernelInterface {
  public:
-  virtual ~SimpleDelegateKernelInterface() {}
+  virtual ~SimpleDelegateKernelInterface() = default;
 
   // Initializes a delegated subgraph.
   // The nodes in the subgraph are inside TfLiteDelegateParams->nodes_to_replace
@@ -85,7 +88,7 @@ class SimpleDelegateInterface {
     int min_nodes_per_partition = 0;
   };
 
-  virtual ~SimpleDelegateInterface() {}
+  virtual ~SimpleDelegateInterface() = default;
 
   // Returns true if 'node' is supported by the delegate. False otherwise.
   virtual bool IsNodeSupportedByDelegate(const TfLiteRegistration* registration,
@@ -112,6 +115,32 @@ class SimpleDelegateInterface {
   // Returns SimpleDelegateInterface::Options which has delegate properties
   // relevant for graph partitioning.
   virtual SimpleDelegateInterface::Options DelegateOptions() const = 0;
+
+  /// Optional method for supporting hardware buffers.
+  /// Copies the data from delegate buffer handle into raw memory of the given
+  /// `tensor`. Note that the delegate is allowed to allocate the raw bytes as
+  /// long as it follows the rules for kTfLiteDynamic tensors.
+  virtual TfLiteStatus CopyFromBufferHandle(TfLiteContext* context,
+                                            TfLiteBufferHandle buffer_handle,
+                                            TfLiteTensor* tensor) {
+    return kTfLiteError;
+  }
+
+  /// Optional method for supporting hardware buffers.
+  /// Copies the data from raw memory of the given `tensor` to delegate buffer
+  /// handle.
+  virtual TfLiteStatus CopyToBufferHandle(TfLiteContext* context,
+                                          TfLiteBufferHandle buffer_handle,
+                                          const TfLiteTensor* tensor) {
+    return kTfLiteError;
+  }
+
+  /// Optional method for supporting hardware buffers.
+  /// Frees the Delegate Buffer Handle. Note: This only frees the handle, but
+  /// this doesn't release the underlying resource (e.g. textures). The
+  /// resources are either owned by application layer or the delegate.
+  virtual void FreeBufferHandle(TfLiteContext* context,
+                                TfLiteBufferHandle* handle) {}
 };
 
 // Factory class that provides static methods to deal with SimpleDelegate
