@@ -17,8 +17,11 @@ limitations under the License.
 
 #include <algorithm>
 #include <array>
+#include <cstddef>
 #include <cstdint>
+#include <cstdlib>
 #include <functional>
+#include <limits>
 #include <memory>
 #include <numeric>
 #include <random>
@@ -26,12 +29,14 @@ limitations under the License.
 
 #include <gtest/gtest.h>
 #include "fp16.h"  // from @FP16
-#include "flatbuffers/flatbuffers.h"  // from @flatbuffers
+#include "flatbuffers/buffer.h"  // from @flatbuffers
+#include "flatbuffers/flatbuffer_builder.h"  // from @flatbuffers
+#include "flatbuffers/string.h"  // from @flatbuffers
+#include "tensorflow/compiler/mlir/lite/schema/schema_conversion_utils.h"
+#include "tensorflow/lite/core/interpreter_builder.h"
 #include "tensorflow/lite/core/kernels/register.h"
-#include "tensorflow/lite/core/model.h"
 #include "tensorflow/lite/delegates/xnnpack/test_util.h"
 #include "tensorflow/lite/interpreter.h"
-#include "tensorflow/lite/schema/schema_conversion_utils.h"
 #include "tensorflow/lite/schema/schema_generated.h"
 #include "tensorflow/lite/version.h"
 
@@ -87,6 +92,7 @@ void BinaryElementwiseTester::Test(tflite::BuiltinOperator binary_op,
       input2_distribution = std::uniform_real_distribution<float>(0.1f, 1.0f);
       break;
     case BuiltinOperator_MUL:
+    case BuiltinOperator_PRELU:
       input1_distribution = std::uniform_real_distribution<float>(-5.0f, 5.0f);
       input2_distribution = std::uniform_real_distribution<float>(-5.0f, 5.0f);
       break;
