@@ -19,6 +19,7 @@ limitations under the License.
 #include <random>
 
 #include <gtest/gtest.h>
+#include "tensorflow/lite/c/c_api_types.h"
 #include "tensorflow/lite/delegates/xnnpack/depthwise_conv_2d_tester.h"
 #include "tensorflow/lite/delegates/xnnpack/xnnpack_delegate.h"
 
@@ -130,8 +131,8 @@ TEST(DepthwiseConv2D, 5x5) {
       .InputHeight(input_rng())
       .InputWidth(input_rng())
       .InputChannels(channel_rng())
-      .KernelHeight(3)
-      .KernelWidth(3)
+      .KernelHeight(5)
+      .KernelWidth(5)
       .SamePadding()
       .Test(xnnpack_delegate.get());
 }
@@ -152,8 +153,8 @@ TEST(DepthwiseConv2D, 5x5Stride2) {
       .InputHeight(input_rng())
       .InputWidth(input_rng())
       .InputChannels(channel_rng())
-      .KernelHeight(3)
-      .KernelWidth(3)
+      .KernelHeight(5)
+      .KernelWidth(5)
       .StrideHeight(2)
       .StrideWidth(2)
       .SamePadding()
@@ -817,5 +818,37 @@ TEST(DepthwiseConv2D, WeightsCache) {
       .Test(xnnpack_delegate.get());
 }
 
+TEST(DepthwiseConv2D, TransientIndirectionBuffer) {
+  TfLiteXNNPackDelegateOptions xnnpack_options =
+      TfLiteXNNPackDelegateOptionsDefault();
+  xnnpack_options.num_threads = 2;
+  xnnpack_options.flags |=
+      TFLITE_XNNPACK_DELEGATE_FLAG_TRANSIENT_INDIRECTION_BUFFER;
+  std::unique_ptr<TfLiteDelegate, decltype(&TfLiteXNNPackDelegateDelete)>
+      xnnpack_delegate(TfLiteXNNPackDelegateCreate(&xnnpack_options),
+                       TfLiteXNNPackDelegateDelete);
+
+  std::random_device random_device;
+  auto rng = std::mt19937(random_device());
+  auto input_rng =
+      std::bind(std::uniform_int_distribution<int32_t>(5, 25), std::ref(rng));
+  auto kernel_rng =
+      std::bind(std::uniform_int_distribution<int32_t>(3, 5), std::ref(rng));
+  auto stride_rng =
+      std::bind(std::uniform_int_distribution<int32_t>(2, 3), std::ref(rng));
+  auto channel_rng =
+      std::bind(std::uniform_int_distribution<int32_t>(3, 32), std::ref(rng));
+
+  DepthwiseConv2DTester()
+      .InputHeight(input_rng())
+      .InputWidth(input_rng())
+      .InputChannels(channel_rng())
+      .KernelHeight(kernel_rng())
+      .KernelWidth(kernel_rng())
+      .StrideHeight(stride_rng())
+      .StrideWidth(stride_rng())
+      .Test(xnnpack_delegate.get());
+}
+
 }  // namespace xnnpack
 }  // namespace tflite
