@@ -17,10 +17,12 @@ limitations under the License.
 
 #include <algorithm>
 #include <array>
+#include <cstddef>
 #include <cstdint>
+#include <cstdlib>
 #include <functional>
+#include <limits>
 #include <memory>
-#include <numeric>
 #include <random>
 #include <vector>
 
