@@ -17,6 +17,8 @@ limitations under the License.
 
 #include <algorithm>
 #include <array>
+#include <cstddef>
+#include <cstdint>
 #include <functional>
 #include <memory>
 #include <random>
@@ -46,12 +48,9 @@ constexpr uint32_t CALL_ONCE = 3;
 std::unique_ptr<TfLiteDelegate, decltype(&TfLiteXNNPackDelegateDelete)>
 NewXnnPackDelegateSupportingVariableOps() {
   TfLiteXNNPackDelegateOptions options = TfLiteXNNPackDelegateOptionsDefault();
-  options.handle_variable_ops = true;
   std::unique_ptr<TfLiteDelegate, decltype(&TfLiteXNNPackDelegateDelete)>
       xnnpack_delegate(TfLiteXNNPackDelegateCreate(&options),
                        TfLiteXNNPackDelegateDelete);
-  TfLiteDelegate* delegate = xnnpack_delegate.get();
-  delegate->flags |= kTfLiteDelegateFlagsAllowDynamicTensors;
   return xnnpack_delegate;
 }
 
