@@ -16,8 +16,8 @@ limitations under the License.
 
 #include <string>
 
-#include <gmock/gmock.h>
 #include <gtest/gtest.h>
+#include "tensorflow/lite/experimental/acceleration/compatibility/variables.h"
 
 namespace tflite::acceleration {
 namespace {
@@ -41,5 +41,11 @@ TEST(CanonicalizeValue, OtherSpecialCharactersAreUnaffected) {
     EXPECT_EQ(CanonicalizeValue(s), s);
   }
 }
+
+TEST(CanonicalizeValue, SamsungXclipseGpuNormalized) {
+  EXPECT_EQ(CanonicalizeValueWithKey(
+                kGPUModel, "ANGLE (Samsung Xclipse 920) on Vulkan 1.1.179"),
+            "angle_(samsung_xclipse_920)_on_vulkan");
+}
 }  // namespace
 }  // namespace tflite::acceleration
