@@ -34,7 +34,7 @@ namespace {
 
 void CanonicalizeValues(std::map<std::string, std::string>* variable_values) {
   for (auto& i : *variable_values) {
-    i.second = CanonicalizeValue(i.second);
+    i.second = CanonicalizeValueWithKey(i.first, i.second);
   }
 }
 
@@ -142,10 +142,10 @@ std::map<std::string, std::string> GPUCompatibilityList::InfosToMap(
   variables[kManufacturer] = android_info.manufacturer;
   const auto& gl_info = gpu_info.opengl_info;
   variables[kGPUModel] = gl_info.renderer_name;
+
   char buffer[128];
-  int len = snprintf(buffer, 128 - 1, "%d.%d", gl_info.major_version,
-                     gl_info.minor_version);
-  buffer[len] = '\0';
+  snprintf(buffer, 128 - 1, "%d.%d", gl_info.major_version,
+           gl_info.minor_version);
   variables[kOpenGLESVersion] = std::string(buffer);
   return variables;
 }
