@@ -27,49 +27,49 @@ namespace acceleration {
 //
 // Android SDK version number. Android system property ro.build.version.sdk.
 // E.g., "28".
-constexpr char kAndroidSdkVersion[] = "tflite.android_sdk_version";
+inline constexpr char kAndroidSdkVersion[] = "tflite.android_sdk_version";
 // SoC model. Looked up from database or possibly returned from Android system
 // property ro.board.platform, normalized. E.g., "sdm450".
-constexpr char kSoCModel[] = "tflite.soc_model";
+inline constexpr char kSoCModel[] = "tflite.soc_model";
 // SoC vendor. Looked up from database. E.g., "qualcomm".
-constexpr char kSoCVendor[] = "tflite.soc_vendor";
+inline constexpr char kSoCVendor[] = "tflite.soc_vendor";
 // Device manufacturer. Android API android.os.Build.MANUFACTURER, normalized.
 // E.g., "google".
-constexpr char kManufacturer[] = "tflite.manufacturer";
+inline constexpr char kManufacturer[] = "tflite.manufacturer";
 // Device model. Android API android.os.Build.MODEL, normalized.
 // E.g., "pixel_2".
-constexpr char kDeviceModel[] = "tflite.device_model";
+inline constexpr char kDeviceModel[] = "tflite.device_model";
 // Device name. Android API android.os.Build.DEVICE, normalized.
 // E.g., "walleye".
-constexpr char kDeviceName[] = "tflite.device_name";
+inline constexpr char kDeviceName[] = "tflite.device_name";
 
 // GPU-related properties.
 //
 // OpenGL ES version. E.g., 3.2.
-constexpr char kOpenGLESVersion[] = "tflite.opengl_es_version";
+inline constexpr char kOpenGLESVersion[] = "tflite.opengl_es_version";
 // GPU model, result of querying GL_RENDERER, normalized. E.g.,
 // "adreno_(tm)_505".
-constexpr char kGPUModel[] = "tflite.gpu_model";
+inline constexpr char kGPUModel[] = "tflite.gpu_model";
 // GPU vendor, normalized. E.g., "adreno_(tm)_505".
-constexpr char kGPUVendor[] = "tflite.gpu_vendor";
+inline constexpr char kGPUVendor[] = "tflite.gpu_vendor";
 // OpenGL driver version, result of querying GL_VERSION. E.g.,
 // "opengl_es_3.2_v@328.0_(git@6fb5a5b,_ife855c4895)_(date:08/21/18)"
-constexpr char kOpenGLDriverVersion[] = "tflite.opengl_driver_version";
+inline constexpr char kOpenGLDriverVersion[] = "tflite.opengl_driver_version";
 
 // Allowlist use case. This property is used to allow joining multiple lists
 // into a single decision tree.
-constexpr char kUseCase[] = "tflite.use_case";
+inline constexpr char kUseCase[] = "tflite.use_case";
 
 // NNAPI-related properties.
 //
 // NNAPI accelerator name, returned by ANeuralNetworksDevice_getName. E.g.,
 // "qti-dsp".
-constexpr char kNNAPIAccelerator[] = "tflite.nnapi_accelerator";
+inline constexpr char kNNAPIAccelerator[] = "tflite.nnapi_accelerator";
 // NNAPI accelerator feature level, returned by
 // ANeuralNetworksDevice_getFeatureLevel. E.g., 29. Actual variables are named
 // "tflite.nnapi_feature_level.<accelerator name>", e.g.,
 // "tflite.nnapi_feature_level.qti-dsp".
-constexpr char kNNAPIFeatureLevelPrefix[] = "tflite.nnapi_feature_level";
+inline constexpr char kNNAPIFeatureLevelPrefix[] = "tflite.nnapi_feature_level";
 
 namespace gpu {
 // GPU-delegate derived properties.
@@ -77,11 +77,11 @@ namespace gpu {
 // Whether the GPU delegate works in general.
 // Possible values are ("", "SUPPORTED", "UNSUPPORTED"). An empty value for
 // this field means that the device is unsupported.
-constexpr char kStatus[] = "tflite.gpu.status";
+inline constexpr char kStatus[] = "tflite.gpu.status";
 
-constexpr char kStatusSupported[] = "SUPPORTED";
-constexpr char kStatusUnknown[] = "UNKNOWN";
-constexpr char kStatusUnsupported[] = "UNSUPPORTED";
+inline constexpr char kStatusSupported[] = "SUPPORTED";
+inline constexpr char kStatusUnknown[] = "UNKNOWN";
+inline constexpr char kStatusUnsupported[] = "UNSUPPORTED";
 
 enum class CompatibilityStatus {
   kUnknown = 0,
@@ -91,6 +91,15 @@ enum class CompatibilityStatus {
 
 }  // namespace gpu
 
+namespace metadata {
+// Latency for model to run.
+inline constexpr char kLatency[] = "tflite.latency";
+
+// Frame time for one frame through model.
+inline constexpr char kFrameTime[] = "tflite.frame_time";
+
+}  // namespace metadata
+
 }  // namespace acceleration
 }  // namespace tflite
 
