@@ -45,13 +45,14 @@ class PartitionGraphIntoIndependentNodeSubsetsImpl {
   PartitionGraphIntoIndependentNodeSubsetsImpl(
       const GraphInfo* info, const TfLiteIntArray* nodes_to_partition,
       std::vector<NodeSubset>* node_subsets, bool greedily,
-      const ControlEdges& control_edges)
+      const ControlEdges& control_edges, bool disable_node_fusion)
       : info_(info),
         node_subsets_(node_subsets),
         node_type_(info_->num_total_nodes(), NodeSubset::kTfNonPartition),
         greedily_(greedily),
         control_edges_(control_edges),
-        num_incoming_control_edges_(info_->num_execution_nodes(), 0) {
+        num_incoming_control_edges_(info_->num_execution_nodes(), 0),
+        disable_node_fusion_(disable_node_fusion) {
     // Populate the node_type_ map.
     for (auto node_index : TfLiteIntArrayView(nodes_to_partition)) {
       node_type_[node_index] = NodeSubset::kTfPartition;
@@ -134,6 +135,7 @@ class PartitionGraphIntoIndependentNodeSubsetsImpl {
   bool UpdateNode(int node_index) {
     const TfLiteNode& node = info_->node(node_index);
     NodeSubset& current_subset = node_subsets_->back();
+    if (disable_node_fusion_ && !current_subset.nodes.empty()) return false;
     int current_epoch = node_subsets_->size() - 1;
     // Check if node is already done.
     if (node_epochs_[node_index] != kEpochNotReady) {
@@ -257,6 +259,8 @@ class PartitionGraphIntoIndependentNodeSubsetsImpl {
   ControlEdges control_edges_;
   // Number of incoming control edges for each node.
   std::vector<int> num_incoming_control_edges_;
+  // Whether to disable node fusion.
+  const bool disable_node_fusion_;
 };
 // LINT.ThenChange(//tensorflow/lite/delegates/utils.h)
 
@@ -265,7 +269,7 @@ class PartitionGraphIntoIndependentNodeSubsetsImpl {
 TfLiteStatus PartitionGraphIntoIndependentNodeSubsets(
     const GraphInfo* info, const TfLiteIntArray* nodes_to_partition,
     std::vector<NodeSubset>* node_subsets, bool greedily,
-    const ControlEdges* control_edges) {
+    const ControlEdges* control_edges, bool disable_node_fusion) {
   ControlEdges my_control_edges;
   if (control_edges == nullptr) {
     control_edges = &my_control_edges;
@@ -284,7 +288,8 @@ TfLiteStatus PartitionGraphIntoIndependentNodeSubsets(
     }
   }
   PartitionGraphIntoIndependentNodeSubsetsImpl(
-      info, nodes_to_partition, node_subsets, greedily, *control_edges)
+      info, nodes_to_partition, node_subsets, greedily, *control_edges,
+      disable_node_fusion)
       .Partition();
   return kTfLiteOk;
 }
