@@ -18,14 +18,12 @@ limitations under the License.
 #include <stddef.h>
 
 #include <algorithm>
-#include <memory>
 #include <tuple>
 #include <vector>
 
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
 #include "tensorflow/lite/core/c/common.h"
-#include "tensorflow/lite/testing/util.h"
 
 namespace tflite {
 namespace {
