@@ -151,6 +151,12 @@ class NativeInterpreterWrapper implements AutoCloseable {
         outputTensors[i] = null;
       }
     }
+    // Close the delegates first as they may reference the model.
+    delegates.clear();
+    for (Delegate ownedDelegate : ownedDelegates) {
+      ownedDelegate.close();
+    }
+    ownedDelegates.clear();
     delete(errorHandle, modelHandle, interpreterHandle);
     deleteCancellationFlag(cancellationFlagHandle);
     errorHandle = 0;
@@ -161,11 +167,6 @@ class NativeInterpreterWrapper implements AutoCloseable {
     inputsIndexes = null;
     outputsIndexes = null;
     isMemoryAllocated = false;
-    delegates.clear();
-    for (Delegate ownedDelegate : ownedDelegates) {
-      ownedDelegate.close();
-    }
-    ownedDelegates.clear();
   }
 
   /** Runs model inference based on SignatureDef provided through {@code signatureKey}. */
@@ -194,12 +195,21 @@ class NativeInterpreterWrapper implements AutoCloseable {
       run(inputsList, outputsWithOutputIndex);
       return;
     }
-
     for (Map.Entry<String, Object> input : inputs.entrySet()) {
       TensorImpl tensor = getInputTensor(input.getKey(), signatureKey);
       int[] newShape = tensor.getInputShapeIfDifferent(input.getValue());
       if (newShape != null) {
-        signatureRunnerWrapper.resizeInput(input.getKey(), newShape);
+        try {
+          signatureRunnerWrapper.resizeInput(input.getKey(), newShape);
+        } catch (IllegalArgumentException e) {
+          throw (IllegalArgumentException)
+              new IllegalArgumentException(
+                      String.format(
+                          "Tensor passed for input '%s' of signature '%s' has different "
+                              + "shape than expected",
+                          input.getKey(), signatureKey))
+                  .initCause(e);
+        }
       }
     }
 
@@ -401,12 +411,11 @@ class NativeInterpreterWrapper implements AutoCloseable {
     }
     NativeSignatureRunnerWrapper signatureRunnerWrapper = getSignatureRunnerWrapper(signatureKey);
     int subgraphIndex = signatureRunnerWrapper.getSubgraphIndex();
-    if (subgraphIndex > 0) {
-      return signatureRunnerWrapper.getInputTensor(inputName);
+    if (subgraphIndex == 0) {
+      int inputIndex = signatureRunnerWrapper.getInputIndex(inputName);
+      return getInputTensor(inputIndex);
     }
-
-    int inputIndex = signatureRunnerWrapper.getInputIndex(inputName);
-    return getInputTensor(inputIndex);
+    return signatureRunnerWrapper.getInputTensor(inputName);
   }
 
   /** Gets the keys of SignatureDefs available in the model, if any. */
@@ -459,12 +468,11 @@ class NativeInterpreterWrapper implements AutoCloseable {
     }
     NativeSignatureRunnerWrapper signatureRunnerWrapper = getSignatureRunnerWrapper(signatureKey);
     int subgraphIndex = signatureRunnerWrapper.getSubgraphIndex();
-    if (subgraphIndex > 0) {
-      return signatureRunnerWrapper.getOutputTensor(outputName);
+    if (subgraphIndex == 0) {
+      int outputIndex = signatureRunnerWrapper.getOutputIndex(outputName);
+      return getOutputTensor(outputIndex);
     }
-
-    int outputIndex = signatureRunnerWrapper.getOutputIndex(outputName);
-    return getOutputTensor(outputIndex);
+    return signatureRunnerWrapper.getOutputTensor(outputName);
   }
 
   /** Gets the number of ops in the execution plan. */
