@@ -50,6 +50,10 @@ public final class InterpreterApiTest {
       "tensorflow/lite/testdata/dynamic_shapes.bin";
   private static final String BOOL_MODEL =
       "tensorflow/lite/java/src/testdata/tile_with_bool_input.bin";
+  private static final String MODEL_WITH_SIGNATURE_PATH =
+      "tensorflow/lite/java/src/testdata/mul_add_signature_def.bin";
+  private static final String MODEL_WITH_MULTI_SIGNATURE_PATH =
+      "tensorflow/lite/java/src/testdata/multi_signature_def.bin";
 
   private static final ByteBuffer MODEL_BUFFER = TestUtils.getTestFileAsBuffer(MODEL_PATH);
   private static final ByteBuffer MULTIPLE_INPUTS_MODEL_BUFFER =
@@ -61,6 +65,10 @@ public final class InterpreterApiTest {
   private static final ByteBuffer DYNAMIC_SHAPES_MODEL_BUFFER =
       TestUtils.getTestFileAsBuffer(DYNAMIC_SHAPES_MODEL_PATH);
   private static final ByteBuffer BOOL_MODEL_BUFFER = TestUtils.getTestFileAsBuffer(BOOL_MODEL);
+  private static final ByteBuffer MODEL_WITH_SIGNATURE_BUFFER =
+      TestUtils.getTestFileAsBuffer(MODEL_WITH_SIGNATURE_PATH);
+  private static final ByteBuffer MODEL_WITH_MULTI_SIGNATURE_BUFFER =
+      TestUtils.getTestFileAsBuffer(MODEL_WITH_MULTI_SIGNATURE_PATH);
 
   // We want to run these tests both with the TF Lite runtime library linked in,
   // and also using the system TF Lite runtime library if the client for that is linked in.
@@ -669,4 +677,250 @@ public final class InterpreterApiTest {
       assertThat(output.array()).usingTolerance(0.1f).containsExactly(expected.array()).inOrder();
     }
   }
+
+  @Test
+  public void testModelWithSignatureDef() {
+    try (InterpreterApi interpreter =
+        InterpreterApi.create(MODEL_WITH_SIGNATURE_BUFFER, TEST_OPTIONS)) {
+      String[] signatureKeys = interpreter.getSignatureKeys();
+      String[] expectedSignatureKeys = {"mul_add"};
+      assertThat(signatureKeys).isEqualTo(expectedSignatureKeys);
+
+      String[] signatureInputs = interpreter.getSignatureInputs(expectedSignatureKeys[0]);
+      String[] expectedSignatureInputs = {"x", "y"};
+      assertThat(signatureInputs).isEqualTo(expectedSignatureInputs);
+
+      String[] signatureOutputs = interpreter.getSignatureOutputs(expectedSignatureKeys[0]);
+      String[] expectedSignatureOutputs = {"output_0"};
+      assertThat(signatureOutputs).isEqualTo(expectedSignatureOutputs);
+
+      Tensor outputTensor = interpreter.getOutputTensorFromSignature("output_0", "mul_add");
+      Tensor inputTensor = interpreter.getInputTensorFromSignature("x", "mul_add");
+      assertThat(outputTensor.numElements()).isEqualTo(1);
+      assertThat(inputTensor.numElements()).isEqualTo(1);
+
+      // Test null input name.
+      try {
+        inputTensor = interpreter.getInputTensorFromSignature(null, "mul_add");
+        fail();
+      } catch (IllegalArgumentException e) {
+        assertThat(e).hasMessageThat().contains("Invalid input tensor name provided (null)");
+      }
+      // Test invalid input name.
+      try {
+        inputTensor = interpreter.getInputTensorFromSignature("xx", "mul_add");
+        fail();
+      } catch (IllegalArgumentException e) {
+        assertThat(e).hasMessageThat().contains("Input error: input xx not found.");
+      }
+
+      // Test null output name.
+      try {
+        outputTensor = interpreter.getOutputTensorFromSignature(null, "mul_add");
+        fail();
+      } catch (IllegalArgumentException e) {
+        assertThat(e).hasMessageThat().contains("Invalid output tensor name provided (null)");
+      }
+      // Test invalid output name.
+      try {
+        outputTensor = interpreter.getOutputTensorFromSignature("yy", "mul_add");
+        fail();
+      } catch (IllegalArgumentException e) {
+        assertThat(e).hasMessageThat().contains("Input error: output yy not found.");
+      }
+
+      FloatBuffer output = FloatBuffer.allocate(1);
+      float[] inputX = {2.0f};
+      float[] inputY = {4.0f};
+      Map<String, Object> inputs = new HashMap<>();
+      inputs.put("x", inputX);
+      inputs.put("y", inputY);
+      Map<String, Object> outputs = new HashMap<>();
+      outputs.put("output_0", output);
+      interpreter.runSignature(inputs, outputs, "mul_add");
+      // Result should be x * 3.0 + y
+      FloatBuffer expected = fill(FloatBuffer.allocate(1), 10.0f);
+      assertThat(output.array()).usingTolerance(0.1f).containsExactly(expected.array()).inOrder();
+    }
+  }
+
+  @Test
+  public void testModelWithSignatureDefNullMethodName() {
+    try (InterpreterApi interpreter =
+        InterpreterApi.create(MODEL_WITH_SIGNATURE_BUFFER, TEST_OPTIONS)) {
+      String[] signatureKeys = interpreter.getSignatureKeys();
+      String[] expectedSignatureKeys = {"mul_add"};
+      assertThat(signatureKeys).isEqualTo(expectedSignatureKeys);
+
+      String[] signatureInputs = interpreter.getSignatureInputs(expectedSignatureKeys[0]);
+      String[] expectedSignatureInputs = {"x", "y"};
+      assertThat(signatureInputs).isEqualTo(expectedSignatureInputs);
+
+      String[] signatureOutputs = interpreter.getSignatureOutputs(expectedSignatureKeys[0]);
+      String[] expectedSignatureOutputs = {"output_0"};
+      assertThat(signatureOutputs).isEqualTo(expectedSignatureOutputs);
+
+      FloatBuffer output = FloatBuffer.allocate(1);
+      float[] inputX = {2.0f};
+      float[] inputY = {4.0f};
+      Map<String, Object> inputs = new HashMap<>();
+      inputs.put("x", inputX);
+      inputs.put("y", inputY);
+      Map<String, Object> outputs = new HashMap<>();
+      outputs.put("output_0", output);
+      interpreter.runSignature(inputs, outputs, null);
+      // Result should be x * 3.0 + y
+      FloatBuffer expected = fill(FloatBuffer.allocate(1), 10.0f);
+      assertThat(output.array()).usingTolerance(0.1f).containsExactly(expected.array()).inOrder();
+      output = FloatBuffer.allocate(1);
+      outputs.put("output_0", output);
+      interpreter.runSignature(inputs, outputs);
+      assertThat(output.array()).usingTolerance(0.1f).containsExactly(expected.array()).inOrder();
+    }
+  }
+
+  @Test
+  public void testModelWithSignatureDefNoSignatures() {
+    try (InterpreterApi interpreter = InterpreterApi.create(MODEL_BUFFER, TEST_OPTIONS)) {
+      String[] signatureKeys = interpreter.getSignatureKeys();
+      String[] expectedSignatureKeys = {};
+      assertThat(signatureKeys).isEqualTo(expectedSignatureKeys);
+      Map<String, Object> inputs = new HashMap<>();
+      Map<String, Object> outputs = new HashMap<>();
+      try {
+        interpreter.runSignature(inputs, outputs);
+        fail();
+      } catch (IllegalArgumentException e) {
+        assertThat(e)
+            .hasMessageThat()
+            .contains(
+                "Input error: SignatureDef signatureKey should not be null. null is only allowed if"
+                    + " the model has a single Signature");
+      }
+    }
+  }
+
+  @Test
+  public void testModelWithMultiSignatureDef() {
+    try (InterpreterApi interpreter =
+        InterpreterApi.create(MODEL_WITH_MULTI_SIGNATURE_BUFFER, TEST_OPTIONS)) {
+      String[] signatureKeys = interpreter.getSignatureKeys();
+      String[] expectedSignatureKeys = {"add", "sub"};
+      assertThat(signatureKeys).isEqualTo(expectedSignatureKeys);
+
+      String[] addSignatureInputs = interpreter.getSignatureInputs(expectedSignatureKeys[0]);
+      String[] expectedAddSignatureInputs = {"a", "b"};
+      assertThat(addSignatureInputs).isEqualTo(expectedAddSignatureInputs);
+
+      String[] addSignatureOutputs = interpreter.getSignatureOutputs(expectedSignatureKeys[0]);
+      String[] expectedAddSignatureOutputs = {"add_result"};
+      assertThat(addSignatureOutputs).isEqualTo(expectedAddSignatureOutputs);
+
+      String[] subSignatureInputs = interpreter.getSignatureInputs(expectedSignatureKeys[1]);
+      String[] expectedSubSignatureInputs = {"x", "y"};
+      assertThat(subSignatureInputs).isEqualTo(expectedSubSignatureInputs);
+
+      String[] subSignatureOutputs = interpreter.getSignatureOutputs(expectedSignatureKeys[1]);
+      String[] expectedSubSignatureOutputs = {"sub_result"};
+      assertThat(subSignatureOutputs).isEqualTo(expectedSubSignatureOutputs);
+
+      // Test "add" signature.
+      FloatBuffer output = FloatBuffer.allocate(1);
+      float inputA = 2.0f;
+      float inputB = 4.0f;
+      Map<String, Object> inputs = new HashMap<>();
+      inputs.put("a", inputA);
+      inputs.put("b", inputB);
+      Map<String, Object> outputs = new HashMap<>();
+      outputs.put("add_result", output);
+      interpreter.runSignature(inputs, outputs, "add");
+      // Result should be a + b
+      FloatBuffer expected = fill(FloatBuffer.allocate(1), 6.0f);
+      assertThat(output.array()).usingTolerance(0.1f).containsExactly(expected.array()).inOrder();
+
+      // Test "sub" signature.
+      output = FloatBuffer.allocate(1);
+      float inputX = 4.0f;
+      float inputY = 2.0f;
+      inputs = new HashMap<>();
+      inputs.put("x", inputX);
+      inputs.put("y", inputY);
+      outputs = new HashMap<>();
+      outputs.put("sub_result", output);
+      interpreter.runSignature(inputs, outputs, "sub");
+      // Result should be x - y
+      expected = fill(FloatBuffer.allocate(1), 2.0f);
+      assertThat(output.array()).usingTolerance(0.1f).containsExactly(expected.array()).inOrder();
+    }
+  }
+
+  // This test is the same as testModelWithMultiSignatureDef above,
+  // except that it passes in vectors rather than scalars.
+  @Test
+  public void testImplicitNonstrictResize() {
+    try (InterpreterApi interpreter =
+        InterpreterApi.create(MODEL_WITH_MULTI_SIGNATURE_BUFFER, TEST_OPTIONS)) {
+      String[] signatureKeys = interpreter.getSignatureKeys();
+      String[] expectedSignatureKeys = {"add", "sub"};
+      assertThat(signatureKeys).isEqualTo(expectedSignatureKeys);
+
+      String[] addSignatureInputs = interpreter.getSignatureInputs(expectedSignatureKeys[0]);
+      String[] expectedAddSignatureInputs = {"a", "b"};
+      assertThat(addSignatureInputs).isEqualTo(expectedAddSignatureInputs);
+
+      String[] addSignatureOutputs = interpreter.getSignatureOutputs(expectedSignatureKeys[0]);
+      String[] expectedAddSignatureOutputs = {"add_result"};
+      assertThat(addSignatureOutputs).isEqualTo(expectedAddSignatureOutputs);
+
+      String[] subSignatureInputs = interpreter.getSignatureInputs(expectedSignatureKeys[1]);
+      String[] expectedSubSignatureInputs = {"x", "y"};
+      assertThat(subSignatureInputs).isEqualTo(expectedSubSignatureInputs);
+
+      String[] subSignatureOutputs = interpreter.getSignatureOutputs(expectedSignatureKeys[1]);
+      String[] expectedSubSignatureOutputs = {"sub_result"};
+      assertThat(subSignatureOutputs).isEqualTo(expectedSubSignatureOutputs);
+
+      // Test "add" signature.
+      FloatBuffer output = FloatBuffer.allocate(1);
+      float[] inputA = {2.0f};
+      float[] inputB = {4.0f};
+      Map<String, Object> inputs = new HashMap<>();
+      inputs.put("a", inputA);
+      inputs.put("b", inputB);
+      Map<String, Object> outputs = new HashMap<>();
+      outputs.put("add_result", output);
+      if (SupportedFeatures.supportsNonstrictResize()) {
+        interpreter.runSignature(inputs, outputs, "add");
+        // Result should be a + b
+        FloatBuffer expected = fill(FloatBuffer.allocate(1), 6.0f);
+        assertThat(output.array()).usingTolerance(0.1f).containsExactly(expected.array()).inOrder();
+
+        // Test "sub" signature.
+        output = FloatBuffer.allocate(1);
+        float[] inputX = {4.0f};
+        float[] inputY = {2.0f};
+        inputs = new HashMap<>();
+        inputs.put("x", inputX);
+        inputs.put("y", inputY);
+        outputs = new HashMap<>();
+        outputs.put("sub_result", output);
+        interpreter.runSignature(inputs, outputs, "sub");
+        // Result should be x - y
+        expected = fill(FloatBuffer.allocate(1), 2.0f);
+        assertThat(output.array()).usingTolerance(0.1f).containsExactly(expected.array()).inOrder();
+      } else {
+        try {
+          interpreter.runSignature(inputs, outputs, "add");
+          fail();
+        } catch (IllegalArgumentException e) {
+          // Could report error message for either input 'a' or input 'b'; both have wrong
+          // shape, and we don't want the test to depend on the order of error checking.
+          assertThat(e).hasMessageThat().contains("Tensor passed for input '");
+          assertThat(e)
+              .hasMessageThat()
+              .contains("' of signature 'add' has different shape than expected");
+        }
+      }
+    }
+  }
 }
