@@ -72,20 +72,31 @@ struct OpData {
   };
 };
 
+static constexpr int kInt16LUTArraySize = LUTSize<int16_t>();
+static constexpr int kTableSize = 256;
+
 struct SoftmaxOpData {
-  struct SoftmaxParams params = {};
-  float table[256];
+  ~SoftmaxOpData() {
+    delete[] table;
 #ifdef TFLITE_SOFTMAX_USE_UINT16_LUT
-  uint8_t uint8_table1[256];
-  uint8_t uint8_table2[256];
+    delete[] uint8_table1;
+    delete[] uint8_table2;
 #endif
-  static constexpr int kInt16LUTArraySize = LUTSize<int16_t>();
-  int16_t exp_lut[kInt16LUTArraySize];  // int16 LUT for exp(x), where x uniform
-                                        // distributed between [-10.0 , 0.0]
-  int16_t one_over_one_plus_x_lut[kInt16LUTArraySize];  // int16 LUT for 1 /
-                                                        // (1 + x), where x
-                                                        // uniform distributed
-                                                        // between [0.0 , 1.0]
+    delete[] exp_lut;
+    delete[] one_over_one_plus_x_lut;
+  }
+  struct SoftmaxParams params = {};
+  float* table = nullptr;
+#ifdef TFLITE_SOFTMAX_USE_UINT16_LUT
+  uint8_t* uint8_table1 = nullptr;
+  uint8_t* uint8_table2 = nullptr;
+#endif
+  int16_t* exp_lut = nullptr;  // int16 LUT for exp(x), where x uniform
+                               // distributed between [-10.0 , 0.0]
+  int16_t* one_over_one_plus_x_lut = nullptr;  // int16 LUT for 1 /
+                                               // (1 + x), where x is
+                                               // uniformly distributed
+                                               // between [0.0 , 1.0]
 };
 
 struct LogSoftmaxOpData : public OpData {
@@ -568,6 +579,12 @@ TfLiteStatus SoftmaxPrepare(TfLiteContext* context, TfLiteNode* node) {
           // Only apply when both input & output are uint8/int8 & build with
           // clang on aarch64.
           // TODO(b/143709993): Port to ARMv7 and other platforms.
+          if (!data->uint8_table1) {
+            data->uint8_table1 = new uint8_t[kTableSize];
+          }
+          if (!data->uint8_table2) {
+            data->uint8_table2 = new uint8_t[kTableSize];
+          }
           data->params.uint8_table1 = data->uint8_table1;
           data->params.uint8_table2 = data->uint8_table2;
           optimized_ops::PopulateSoftmaxUInt8LookupTable(
@@ -576,6 +593,9 @@ TfLiteStatus SoftmaxPrepare(TfLiteContext* context, TfLiteNode* node) {
 #endif
         case kTfLiteInt16:
         default:
+          if (!data->table) {
+            data->table = new float[kTableSize];
+          }
           data->params.table = data->table;
           optimized_ops::PopulateSoftmaxLookupTable(
               &data->params, input->params.scale, params->beta);
@@ -594,6 +614,9 @@ TfLiteStatus SoftmaxPrepare(TfLiteContext* context, TfLiteNode* node) {
     // range [-10.0; 0] as we consider resulting values smaller than exp(-10)
     // insignificant. Use a symmetric output range of [-1.0; 1.0] and double as
     // FloatT for backward compatibility.
+    if (!data->exp_lut) {
+      data->exp_lut = new int16_t[kInt16LUTArraySize];
+    }
     data->params.exp_lut = data->exp_lut;
     LUTPopulate<int16_t>(
         10.0 / range, std::numeric_limits<int16_t>::max(), 2.0 / range, 0,
@@ -601,6 +624,9 @@ TfLiteStatus SoftmaxPrepare(TfLiteContext* context, TfLiteNode* node) {
 
     // Input is in the [0; 1] range and use a symmetric output range of
     // [-1.0; 1.0] and double as FloatT for backward compatibility.
+    if (!data->one_over_one_plus_x_lut) {
+      data->one_over_one_plus_x_lut = new int16_t[kInt16LUTArraySize];
+    }
     data->params.one_over_one_plus_x_lut = data->one_over_one_plus_x_lut;
     LUTPopulate<int16_t>(
         1.0 / range, std::numeric_limits<int16_t>::min(), 2.0 / range, 0,
@@ -888,11 +914,9 @@ TfLiteStatus Relu6Eval(TfLiteContext* context, TfLiteNode* node) {
   ReluOpData* data = reinterpret_cast<ReluOpData*>(node->user_data);
   switch (input->type) {
     case kTfLiteFloat32: {
-      size_t elements = input->bytes / sizeof(float);
-      const float* in = GetTensorData<float>(input);
-      const float* in_end = in + elements;
-      float* out = GetTensorData<float>(output);
-      for (; in < in_end; in++, out++) *out = std::min(std::max(0.f, *in), 6.f);
+      reference_ops::Relu6(GetTensorShape(input), GetTensorData<float>(input),
+                           GetTensorShape(output),
+                           GetTensorData<float>(output));
       return kTfLiteOk;
     }
     case kTfLiteUInt8:
@@ -935,6 +959,30 @@ TfLiteStatus TanhEval(TfLiteContext* context, TfLiteNode* node) {
       }
       return kTfLiteOk;
     } break;
+    case kTfLiteFloat16: {
+      if (kernel_type == kReference) {
+        reference_ops::Tanh(
+            GetTensorShape(input), GetTensorData<Eigen::half>(input),
+            GetTensorShape(output), GetTensorData<Eigen::half>(output));
+      } else {
+        optimized_ops::Tanh(
+            GetTensorShape(input), GetTensorData<Eigen::half>(input),
+            GetTensorShape(output), GetTensorData<Eigen::half>(output));
+      }
+      return kTfLiteOk;
+    } break;
+    case kTfLiteBFloat16: {
+      if (kernel_type == kReference) {
+        reference_ops::Tanh(
+            GetTensorShape(input), GetTensorData<Eigen::bfloat16>(input),
+            GetTensorShape(output), GetTensorData<Eigen::bfloat16>(output));
+      } else {
+        optimized_ops::Tanh(
+            GetTensorShape(input), GetTensorData<Eigen::bfloat16>(input),
+            GetTensorShape(output), GetTensorData<Eigen::bfloat16>(output));
+      }
+      return kTfLiteOk;
+    } break;
     case kTfLiteInt16: {
       TanhParams params;
       params.input_left_shift = data->input_left_shift;
@@ -1007,7 +1055,7 @@ TfLiteStatus SigmoidEval(TfLiteContext* context, TfLiteNode* node) {
   switch (input->type) {
     case kTfLiteFloat32: {
       if (kernel_type == kReference) {
-        reference_ops::Logistic(
+        reference_ops::Logistic<float>(
             GetTensorShape(input), GetTensorData<float>(input),
             GetTensorShape(output), GetTensorData<float>(output));
       } else {
@@ -1017,6 +1065,30 @@ TfLiteStatus SigmoidEval(TfLiteContext* context, TfLiteNode* node) {
       }
       break;
     }
+    case kTfLiteFloat16: {
+      if (kernel_type == kReference) {
+        reference_ops::Logistic<Eigen::half>(
+            GetTensorShape(input), GetTensorData<Eigen::half>(input),
+            GetTensorShape(output), GetTensorData<Eigen::half>(output));
+      } else {
+        optimized_ops::Logistic<Eigen::half>(
+            GetTensorShape(input), GetTensorData<Eigen::half>(input),
+            GetTensorShape(output), GetTensorData<Eigen::half>(output));
+      }
+      break;
+    }
+    case kTfLiteBFloat16: {
+      if (kernel_type == kReference) {
+        reference_ops::Logistic<Eigen::bfloat16>(
+            GetTensorShape(input), GetTensorData<Eigen::bfloat16>(input),
+            GetTensorShape(output), GetTensorData<Eigen::bfloat16>(output));
+      } else {
+        optimized_ops::Logistic<Eigen::bfloat16>(
+            GetTensorShape(input), GetTensorData<Eigen::bfloat16>(input),
+            GetTensorShape(output), GetTensorData<Eigen::bfloat16>(output));
+      }
+      break;
+    }
     case kTfLiteInt16: {
       LogisticParams params;
       if (kernel_type == kReference || (data->input_multiplier > 0)) {
@@ -1408,7 +1480,7 @@ TfLiteStatus PreluEval(TfLiteContext* context, TfLiteNode* node) {
     default:
       TF_LITE_KERNEL_LOG(
           context,
-          "Only float32 and uint8 and int8 are supported currently, got %d.",
+          "Only float32 and uint8 and int8 are supported currently, got %s.",
           TfLiteTypeGetName(input->type));
       return kTfLiteError;
   }
@@ -1548,6 +1620,13 @@ TfLiteStatus GeluPrepare(TfLiteContext* context, TfLiteNode* node) {
                              ? reference_ops::GeluTransformApproximate
                              : reference_ops::GeluTransform,
                          data->lut_uint8);
+  } else if (input->type == kTfLiteInt16) {
+    LUTPopulate<int16_t>(input->params.scale, input->params.zero_point,
+                         output->params.scale, output->params.zero_point,
+                         params->approximate
+                             ? reference_ops::GeluTransformApproximate
+                             : reference_ops::GeluTransform,
+                         data->lut_int16);
   }
   return GenericPrepare(context, node);
 }
@@ -1578,13 +1657,18 @@ TfLiteStatus GeluEval(TfLiteContext* context, TfLiteNode* node) {
           MatchingFlatSize(GetTensorShape(input), GetTensorShape(output)),
           data->lut_int8, GetTensorData<int8_t>(output));
       return kTfLiteOk;
+    case kTfLiteInt16:
+      reference_ops::LookupTableInt16(
+          GetTensorData<int16_t>(input),
+          MatchingFlatSize(GetTensorShape(input), GetTensorShape(output)),
+          data->lut_int16, GetTensorData<int16_t>(output));
+      return kTfLiteOk;
     default:
       TF_LITE_KERNEL_LOG(
           context, "Only float32, int8 and uint8 supported currently, got %s.",
           TfLiteTypeGetName(input->type));
       return kTfLiteError;
   }
-  return kTfLiteOk;
 }
 
 }  // namespace activations
