@@ -16,6 +16,7 @@ limitations under the License.
 #include "tensorflow/lite/kernels/internal/reference/batch_matmul.h"
 
 #include <stddef.h>
+#include <string.h>
 
 #include <algorithm>
 #include <cstdint>
@@ -27,7 +28,6 @@ limitations under the License.
 #include "tensorflow/lite/kernels/internal/compatibility.h"
 #include "tensorflow/lite/kernels/internal/optimized/batch_matmul.h"
 #include "tensorflow/lite/kernels/internal/optimized/optimized_ops.h"
-#include "tensorflow/lite/kernels/internal/reference/reference_ops.h"
 #include "tensorflow/lite/kernels/internal/tensor.h"
 #include "tensorflow/lite/kernels/internal/tensor_ctypes.h"
 #include "tensorflow/lite/kernels/internal/tensor_utils.h"
@@ -86,9 +86,7 @@ void* Init(TfLiteContext* context, const char* buffer, size_t length) {
   op_data->rhs_transposed = false;
   // Creates the temp tensors to store the transposed LHS and/or RHS, and
   // extra buffers for the quantized case.
-  context->AddTensors(context,
-                      kNumTempTensorsForAdjoints + kNumTempTensorsForHybrid,
-                      &op_data->scratch_tensor_index);
+  op_data->scratch_tensor_index = -1;
   return op_data;
 }
 
@@ -292,12 +290,17 @@ TfLiteStatus InitializeTemporaries(TfLiteContext* context, TfLiteNode* node,
 }
 
 TfLiteStatus Prepare(TfLiteContext* context, TfLiteNode* node) {
+  OpData* op_data = reinterpret_cast<OpData*>(node->user_data);
+  if (op_data->scratch_tensor_index == -1) {
+    context->AddTensors(context,
+                        kNumTempTensorsForAdjoints + kNumTempTensorsForHybrid,
+                        &op_data->scratch_tensor_index);
+  }
   TF_LITE_ENSURE_EQ(context, NumInputs(node), 2);
   TF_LITE_ENSURE_EQ(context, NumOutputs(node), 1);
 
   OpContext op_context(context, node);
   TF_LITE_ENSURE_OK(context, InitializeTemporaries(context, node, &op_context));
-  OpData* op_data = reinterpret_cast<OpData*>(node->user_data);
 
   bool adj_x = op_context.params->adj_x;
   bool adj_y = op_context.params->adj_y;
@@ -441,7 +444,17 @@ RuntimeShape SwapRowColumnDims(const RuntimeShape& shape) {
   return swapped_shape;
 }
 
-template <KernelType kernel_type>
+TfLiteStatus VerifyPerChannelQuantization(TfLiteContext* context,
+                                          const TfLiteTensor* tensor) {
+  TF_LITE_ENSURE_EQ(context, tensor->quantization.type,
+                    kTfLiteAffineQuantization);
+  const auto* affine_quantization =
+      reinterpret_cast<TfLiteAffineQuantization*>(tensor->quantization.params);
+  TF_LITE_ENSURE(context, affine_quantization);
+  TF_LITE_ENSURE(context, affine_quantization->scale);
+  return affine_quantization->scale->size > 1 ? kTfLiteOk : kTfLiteError;
+}
+
 TfLiteStatus EvalHybrid(TfLiteContext* context, TfLiteNode* node, OpData* data,
                         const RuntimeShape& input_shape,
                         const TfLiteTensor* input,
@@ -483,9 +496,22 @@ TfLiteStatus EvalHybrid(TfLiteContext* context, TfLiteNode* node, OpData* data,
                                     input_size, quant_data, scaling_factors_ptr,
                                     input_offset_ptr,
                                     params->asymmetric_quantize_inputs);
-  for (int b = 0; b < num_batches_to_quantize; ++b) {
-    // Incorporate scaling of the filter.
-    scaling_factors_ptr[b] *= filter->params.scale;
+  float* per_channel_scale_ptr = nullptr;
+  if (VerifyPerChannelQuantization(context, filter) == kTfLiteOk) {
+    //  Per channel quantization.
+    const auto* affine_quantization =
+        reinterpret_cast<TfLiteAffineQuantization*>(
+            filter->quantization.params);
+    TF_LITE_ENSURE_EQ(
+        context, affine_quantization->scale->size,
+        filter->dims->data[affine_quantization->quantized_dimension]);
+    per_channel_scale_ptr = affine_quantization->scale->data;
+  } else {
+    // Per tensor quantization.
+    for (int b = 0; b < num_batches_to_quantize; ++b) {
+      // Incorporate scaling of the filter
+      scaling_factors_ptr[b] *= filter->params.scale;
+    }
   }
 
   RuntimeShape output_shape = GetTensorShape(output);
@@ -494,18 +520,11 @@ TfLiteStatus EvalHybrid(TfLiteContext* context, TfLiteNode* node, OpData* data,
     output_size *= output_shape.Dims(i);
   }
   std::fill_n(GetTensorData<float>(output), output_size, 0.0f);
-  if (kernel_type == kGenericOptimized) {
-    optimized_ops::BatchMatMul(
-        filter_shape, filter_data, input_shape, quant_data, scaling_factors_ptr,
-        input_offset_ptr, row_sums_ptr, GetTensorShape(output),
-        GetTensorData<int32_t>(accum_scratch), GetTensorData<float>(output),
-        &(data->compute_row_sums), CpuBackendContext::GetFromContext(context));
-  } else {
-    reference_ops::BatchMatMul(
-        filter_shape, filter_data, input_shape, quant_data, scaling_factors_ptr,
-        input_offset_ptr, row_sums_ptr, GetTensorShape(output),
-        GetTensorData<float>(output), &(data->compute_row_sums));
-  }
+  reference_ops::BatchMatMul(filter_shape, filter_data, input_shape, quant_data,
+                             scaling_factors_ptr, input_offset_ptr,
+                             row_sums_ptr, GetTensorShape(output),
+                             GetTensorData<float>(output),
+                             &(data->compute_row_sums), per_channel_scale_ptr);
 
   return kTfLiteOk;
 }
@@ -555,36 +574,12 @@ TfLiteStatus EvalInt8Int32(TfLiteContext* context, const OpData* data,
                            const RuntimeShape& rhs_shape,
                            const TfLiteTensor* rhs,
                            const RuntimeShape& output_shape,
-                           TfLiteTensor* output, bool transpose_lhs) {
-  // Reuse params struct from FullyConnected Op.
-  FullyConnectedParams op_params;
-  int32_t input_offset = -lhs->params.zero_point;
-  int32_t weights_offset = -rhs->params.zero_point;
-  int32_t output_offset = output->params.zero_point;
-  op_params.input_offset = input_offset;
-  op_params.weights_offset = weights_offset;
-  op_params.output_offset = output_offset;
-  op_params.output_multiplier = data->output_multiplier;
-  op_params.output_shift = data->output_shift;
-  op_params.quantized_activation_min = data->output_activation_min;
-  op_params.quantized_activation_max = data->output_activation_max;
-  op_params.lhs_cacheable = IsConstantTensor(lhs);
-  op_params.rhs_cacheable = IsConstantTensor(rhs);
-
+                           TfLiteTensor* output) {
   // Set BatchMatMul lhs param to rhs(filter) and rhs param to lhs(input). For
   // the reason, see comment of Eval() function.
-  if (kernel_type == kReference) {
-    reference_ops::BatchMatMul<int8, int8, int32>(
-        rhs_shape, GetTensorData<int8>(rhs), lhs_shape,
-        GetTensorData<int8>(lhs), GetTensorShape(output),
-        GetTensorData<int32>(output));
-  } else {
-    optimized_ops::BatchMatMul(
-        op_params, rhs_shape, GetTensorData<int8_t>(rhs), lhs_shape,
-        GetTensorData<int8_t>(lhs), GetTensorShape(output),
-        GetTensorData<int32_t>(output),
-        CpuBackendContext::GetFromContext(context), transpose_lhs);
-  }
+  reference_ops::BatchMatMul<int8, int8, int32>(
+      rhs_shape, GetTensorData<int8>(rhs), lhs_shape, GetTensorData<int8>(lhs),
+      GetTensorShape(output), GetTensorData<int32>(output));
   return kTfLiteOk;
 }
 
@@ -638,9 +633,9 @@ TfLiteStatus EvalQuantized(TfLiteContext* context, TfLiteNode* node,
     TfLiteTensor* row_sums;
     TF_LITE_ENSURE_OK(context,
                       GetTemporarySafe(context, node, /*index=*/6, &row_sums));
-    return EvalHybrid<kernel_type>(
-        context, node, data, lhs_shape, lhs, rhs_shape, rhs, input_quantized,
-        scaling_factors, accum_scratch, row_sums, input_offsets, output);
+    return EvalHybrid(context, node, data, lhs_shape, lhs, rhs_shape, rhs,
+                      input_quantized, scaling_factors, accum_scratch, row_sums,
+                      input_offsets, output);
   } else if (lhs->type == kTfLiteInt8 && rhs->type == kTfLiteInt8) {
     if (output->type == kTfLiteInt8) {
       return EvalInt8Int8<kernel_type>(context, data, lhs_shape, lhs, rhs_shape,
@@ -649,7 +644,7 @@ TfLiteStatus EvalQuantized(TfLiteContext* context, TfLiteNode* node,
     } else {
       return EvalInt8Int32<kernel_type>(context, data, lhs_shape, lhs,
                                         rhs_shape, rhs, GetTensorShape(output),
-                                        output, transpose_lhs);
+                                        output);
     }
   } else if (lhs->type == kTfLiteInt16 && rhs->type == kTfLiteInt16) {
     return EvalInt16<kernel_type>(context, data, lhs_shape, lhs, rhs_shape, rhs,
@@ -670,10 +665,44 @@ TfLiteTensor* GetTempRhs(TfLiteContext* context, TfLiteNode* node,
     return nullptr;
   }
 
+  TfLiteIntArrayFree(transposed_rhs->dims);
+  transposed_rhs->dims = TfLiteIntArrayCopy(rhs->dims);
+  std::swap(transposed_rhs->dims->data[transposed_rhs->dims->size - 1],
+            transposed_rhs->dims->data[transposed_rhs->dims->size - 2]);
   if (rhs->type == kTfLiteInt8 || rhs->type == kTfLiteInt16) {
     // Get the quantization params from the RHS tensor.
     transposed_rhs->params.scale = rhs->params.scale;
     transposed_rhs->params.zero_point = rhs->params.zero_point;
+    if (rhs->quantization.type == kTfLiteAffineQuantization) {
+      transposed_rhs->quantization.type = rhs->quantization.type;
+      if (transposed_rhs->quantization.params) {
+        auto* transposed_rhs_affine_quantization =
+            reinterpret_cast<TfLiteAffineQuantization*>(
+                transposed_rhs->quantization.params);
+        TfLiteIntArrayFree(transposed_rhs_affine_quantization->zero_point);
+        TfLiteFloatArrayFree(transposed_rhs_affine_quantization->scale);
+        free(transposed_rhs->quantization.params);
+      }
+      transposed_rhs->quantization.params =
+          malloc(sizeof(TfLiteAffineQuantization));
+      const auto* rhs_affine_quantization =
+          reinterpret_cast<TfLiteAffineQuantization*>(rhs->quantization.params);
+      auto* transposed_rhs_affine_quantization =
+          reinterpret_cast<TfLiteAffineQuantization*>(
+              transposed_rhs->quantization.params);
+      int quantized_dimension = rhs_affine_quantization->quantized_dimension;
+      if (quantized_dimension == rhs->dims->size - 1) {
+        quantized_dimension = rhs->dims->size - 2;
+      } else if (quantized_dimension == rhs->dims->size - 2) {
+        quantized_dimension = rhs->dims->size - 1;
+      }
+      transposed_rhs_affine_quantization->quantized_dimension =
+          quantized_dimension;
+      transposed_rhs_affine_quantization->zero_point =
+          TfLiteIntArrayCopy(rhs_affine_quantization->zero_point);
+      transposed_rhs_affine_quantization->scale =
+          TfLiteFloatArrayCopy(rhs_affine_quantization->scale);
+    }
   }
   return transposed_rhs;
 }
@@ -748,8 +777,9 @@ TfLiteStatus Eval(TfLiteContext* context, TfLiteNode* node) {
   lhs_dims_count = orig_lhs_shape.DimensionsCount();
   const TfLiteTensor* rhs_tensor = rhs;
   bool implicit_transpose_possible = true;
-  if ((lhs->type == kTfLiteFloat32 && rhs->type == kTfLiteInt8) ||
-      kernel_type == kReference || rhs->type == kTfLiteInt16) {
+  if (lhs->type == kTfLiteFloat32 || kernel_type == kReference ||
+      rhs->type == kTfLiteInt16 ||
+      (rhs->type == kTfLiteInt8 && output->type == kTfLiteInt32)) {
     implicit_transpose_possible = false;
   }
   bool do_implicit_transpose = !adj_y && implicit_transpose_possible;
@@ -777,18 +807,10 @@ TfLiteStatus Eval(TfLiteContext* context, TfLiteNode* node) {
   switch (rhs->type) {
     case kTfLiteFloat32:
       // Note we pass RHS args first, LHS args second. See note above.
-      if (kernel_type == kGenericOptimized) {
-        optimized_ops::BatchMatMul(
-            rhs_shape, GetTensorData<float>(rhs_tensor), lhs_shape,
-            GetTensorData<float>(lhs_tensor), GetTensorShape(output),
-            GetTensorData<float>(output),
-            CpuBackendContext::GetFromContext(context), do_implicit_transpose);
-      } else {
-        reference_ops::BatchMatMul(rhs_shape, GetTensorData<float>(rhs_tensor),
-                                   lhs_shape, GetTensorData<float>(lhs_tensor),
-                                   GetTensorShape(output),
-                                   GetTensorData<float>(output));
-      }
+      reference_ops::BatchMatMul(rhs_shape, GetTensorData<float>(rhs_tensor),
+                                 lhs_shape, GetTensorData<float>(lhs_tensor),
+                                 GetTensorShape(output),
+                                 GetTensorData<float>(output));
       break;
     case kTfLiteInt8:
     case kTfLiteInt16:
