@@ -108,6 +108,29 @@ TEST(ConcatenationOpTest, ThreeDimensionalOneInput) {
   EXPECT_THAT(m0.GetOutput(), ElementsAreArray({1, 3, 4, 7}));
 }
 
+TEST(ConcatenationOpTest, ThreeDimensionalOneInputBFloat16) {
+  ConcatenationOpModel<Eigen::bfloat16> m({TensorType_BFLOAT16, {2, 1, 2}},
+                                          /*axis=*/1,
+                                          /*num_inputs=*/1);
+  m.SetInput(
+      0,
+      {static_cast<Eigen::bfloat16>(1.0f), static_cast<Eigen::bfloat16>(3.0f),
+       static_cast<Eigen::bfloat16>(4.0f), static_cast<Eigen::bfloat16>(7.0f)});
+  ASSERT_EQ(m.Invoke(), kTfLiteOk);
+  EXPECT_THAT(m.GetOutput(), ElementsAreArray({1, 3, 4, 7}));
+}
+
+TEST(ConcatenationOpTest, ThreeDimensionalOneInputFloat16) {
+  ConcatenationOpModel<Eigen::half> m({TensorType_FLOAT16, {2, 1, 2}},
+                                      /*axis=*/1,
+                                      /*num_inputs=*/1);
+  m.SetInput(0,
+             {static_cast<Eigen::half>(1.0f), static_cast<Eigen::half>(3.0f),
+              static_cast<Eigen::half>(4.0f), static_cast<Eigen::half>(7.0f)});
+  ASSERT_EQ(m.Invoke(), kTfLiteOk);
+  EXPECT_THAT(m.GetOutput(), ElementsAreArray({1, 3, 4, 7}));
+}
+
 TEST(ConcatenationOpTest, ThreeDimensionalOneInputUInt32) {
   ConcatenationOpModel<uint32_t> m0({TensorType_UINT32, {2, 1, 2}}, /*axis=*/1,
                                     /*num_inputs=*/1);
@@ -152,6 +175,61 @@ TEST(ConcatenationOpTest, FiveDimensionalTwoInput) {
                         13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24}));
 }
 
+TEST(ConcatenationOpTest, FiveDimensionalTwoInputBFloat16) {
+  ConcatenationOpModel<Eigen::bfloat16> m(
+      {TensorType_BFLOAT16, {2, 1, 2, 1, 3}},
+      /*axis=*/0,
+      /*num_inputs=*/2);
+  m.SetInput(
+      0,
+      {static_cast<Eigen::bfloat16>(1.0f), static_cast<Eigen::bfloat16>(2.0f),
+       static_cast<Eigen::bfloat16>(3.0f), static_cast<Eigen::bfloat16>(4.0f),
+       static_cast<Eigen::bfloat16>(5.0f), static_cast<Eigen::bfloat16>(6.0f),
+       static_cast<Eigen::bfloat16>(7.0f), static_cast<Eigen::bfloat16>(8.0f),
+       static_cast<Eigen::bfloat16>(9.0f), static_cast<Eigen::bfloat16>(10.0f),
+       static_cast<Eigen::bfloat16>(11.0f),
+       static_cast<Eigen::bfloat16>(12.0f)});
+  m.SetInput(
+      1,
+      {static_cast<Eigen::bfloat16>(13.0f), static_cast<Eigen::bfloat16>(14.0f),
+       static_cast<Eigen::bfloat16>(15.0f), Eigen::bfloat16{16.0f},
+       static_cast<Eigen::bfloat16>(17.0f), static_cast<Eigen::bfloat16>(18.0f),
+       static_cast<Eigen::bfloat16>(19.0f), static_cast<Eigen::bfloat16>(20.0f),
+       static_cast<Eigen::bfloat16>(21.0f), static_cast<Eigen::bfloat16>(22.0f),
+       static_cast<Eigen::bfloat16>(23.0f),
+       static_cast<Eigen::bfloat16>(24.0f)});
+  ASSERT_EQ(m.Invoke(), kTfLiteOk);
+  EXPECT_THAT(
+      m.GetOutput(),
+      ElementsAreArray({1,  2,  3,  4,  5,  6,  7,  8,  9,  10, 11, 12,
+                        13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24}));
+}
+
+TEST(ConcatenationOpTest, FiveDimensionalTwoInputFloat16) {
+  ConcatenationOpModel<Eigen::half> m({TensorType_FLOAT16, {2, 1, 2, 1, 3}},
+                                      /*axis=*/0,
+                                      /*num_inputs=*/2);
+  m.SetInput(
+      0, {static_cast<Eigen::half>(1.0f), static_cast<Eigen::half>(2.0f),
+          static_cast<Eigen::half>(3.0f), static_cast<Eigen::half>(4.0f),
+          static_cast<Eigen::half>(5.0f), static_cast<Eigen::half>(6.0f),
+          static_cast<Eigen::half>(7.0f), Eigen::half{8.0f},
+          static_cast<Eigen::half>(9.0f), static_cast<Eigen::half>(10.0f),
+          static_cast<Eigen::half>(11.0f), static_cast<Eigen::half>(12.0f)});
+  m.SetInput(
+      1, {static_cast<Eigen::half>(13.0f), static_cast<Eigen::half>(14.0f),
+          Eigen::half{15.0f}, static_cast<Eigen::half>(16.0f),
+          Eigen::half{17.0f}, static_cast<Eigen::half>(18.0f),
+          static_cast<Eigen::half>(19.0f), static_cast<Eigen::half>(20.0f),
+          static_cast<Eigen::half>(21.0f), static_cast<Eigen::half>(22.0f),
+          static_cast<Eigen::half>(23.0f), static_cast<Eigen::half>(24.0f)});
+  ASSERT_EQ(m.Invoke(), kTfLiteOk);
+  EXPECT_THAT(
+      m.GetOutput(),
+      ElementsAreArray({1,  2,  3,  4,  5,  6,  7,  8,  9,  10, 11, 12,
+                        13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24}));
+}
+
 TEST(ConcatenationOpTest, FiveDimensionalTwoInputUInt32) {
   ConcatenationOpModel<uint32_t> m0({TensorType_UINT32, {2, 1, 2, 1, 3}},
                                     /*axis=*/0,
@@ -379,10 +457,11 @@ TEST(ConcatenationOpTest, FourInputs) {
   m0.SetInput(3, {1.3f, 3.3f, 4.3f, 7.3f});
   ASSERT_EQ(m0.Invoke(), kTfLiteOk);
   EXPECT_THAT(m0.GetOutput(),
-              ElementsAreArray({
-                  1.0f, 3.0f, 1.1f, 3.1f, 1.2f, 3.2f, 1.3f, 3.3f,  //
-                  4.0f, 7.0f, 4.1f, 7.1f, 4.2f, 7.2f, 4.3f, 7.3f,  //
-              }));
+              Pointwise(FloatingPointEq(),
+                        {
+                            1.0f, 3.0f, 1.1f, 3.1f, 1.2f, 3.2f, 1.3f, 3.3f,  //
+                            4.0f, 7.0f, 4.1f, 7.1f, 4.2f, 7.2f, 4.3f, 7.3f,  //
+                        }));
 }
 
 TEST(ConcatenationOpTest, FourInputsUInt32) {
@@ -762,6 +841,22 @@ TYPED_TEST(ConcatenationOpPersistentModelTest, PersistentTest) {
   }
 }
 
+TYPED_TEST(ConcatenationOpPersistentModelTest, PersistentScalarTest) {
+  PersistentTestCase test_case{TestInputType::kPersistentRo,
+                               GetTensorType<TypeParam>(), false};
+  std::vector<std::vector<TypeParam>> input_data_lists = {{1}, {7}};
+  std::vector<TensorData> input_template = {{GetTensorType<TypeParam>(), {}},
+                                            {GetTensorType<TypeParam>(), {}}};
+  TensorData output_template = {GetTensorType<TypeParam>(), {2}};
+  PersistentConcatenationOpModel<TypeParam> m0(
+      input_template, /*axis=*/0, output_template, test_case, input_data_lists);
+  m0.PopulateInputTensors();
+  ASSERT_EQ(m0.Invoke(), kTfLiteOk);
+  ASSERT_EQ(m0.IsPersistentOutput(),
+            test_case.test_type == TestInputType::kPersistentRo);
+  EXPECT_THAT(m0.GetOutput(), ElementsAreArray(ArrayFloatNear({1.0, 7.0})));
+}
+
 TYPED_TEST(ConcatenationOpPersistentModelTest, QuantizedPersistentTest) {
   const bool is_quantized = true;
   for (PersistentTestCase test_case :
