@@ -38,10 +38,9 @@ limitations under the License.
 #include "tensorflow/lite/kernels/internal/optimized/multithreaded_conv.h"
 #endif
 #include "tensorflow/lite/kernels/internal/optimized/optimized_ops.h"
-#include "tensorflow/lite/kernels/internal/quantization_util.h"
+#include "tensorflow/lite/kernels/internal/portable_tensor_utils.h"
 #include "tensorflow/lite/kernels/internal/reference/conv.h"
 #include "tensorflow/lite/kernels/internal/reference/integer_ops/conv.h"
-#include "tensorflow/lite/kernels/internal/tensor.h"
 #include "tensorflow/lite/kernels/internal/tensor_ctypes.h"
 #include "tensorflow/lite/kernels/internal/tensor_utils.h"
 #include "tensorflow/lite/kernels/kernel_util.h"
@@ -121,6 +120,8 @@ struct OpData {
 
   // Number of convolution groups.
   int32_t groups = 1;
+
+  TfLiteType quantized_bias_type = kTfLiteNoType;
 };
 
 inline PaddingType RuntimePaddingType(TfLitePadding padding) {
@@ -359,10 +360,6 @@ TfLiteStatus Prepare(KernelType kernel_type, TfLiteContext* context,
                      input_type == kTfLiteInt8 || input_type == kTfLiteInt16);
   TF_LITE_ENSURE_TYPES_EQ(context, output->type, input_type);
 
-  if (input_type == kTfLiteInt16) {
-    TF_LITE_ENSURE_EQ(context, input->params.zero_point, 0);
-    TF_LITE_ENSURE_EQ(context, output->params.zero_point, 0);
-  }
   // Filter must have zero zero-points in per-channel quantization.
   if (input_type == kTfLiteInt16 || input_type == kTfLiteInt8) {
     TF_LITE_ENSURE_EQ(context, filter->quantization.type,
@@ -370,11 +367,21 @@ TfLiteStatus Prepare(KernelType kernel_type, TfLiteContext* context,
     const auto* affine_quantization =
         reinterpret_cast<TfLiteAffineQuantization*>(
             filter->quantization.params);
-    for (int i = 0; i < affine_quantization->zero_point->size; ++i) {
-      TF_LITE_ENSURE_EQ(context, affine_quantization->zero_point->data[i], 0);
+    if (affine_quantization->zero_point) {
+      for (int i = 0; i < affine_quantization->zero_point->size; ++i) {
+        TF_LITE_ENSURE_EQ(context, affine_quantization->zero_point->data[i], 0);
+      }
     }
   }
 
+  // Validate stride values
+  TF_LITE_ENSURE(context, params->stride_height > 0);
+  TF_LITE_ENSURE(context, params->stride_width > 0);
+
+  // Validate dilation values
+  TF_LITE_ENSURE(context, params->dilation_height_factor > 0);
+  TF_LITE_ENSURE(context, params->dilation_width_factor > 0);
+
   const TfLiteTensor* bias = nullptr;
 
   // TODO(ahentz): At this point the optimized versions require 'bias'. We can
@@ -396,15 +403,36 @@ TfLiteStatus Prepare(KernelType kernel_type, TfLiteContext* context,
     TF_LITE_ENSURE_EQ(context, NumElements(bias), SizeOfDimension(filter, 0));
   }
 
+  if (input_type == kTfLiteInt16) {
+    // Quantization should be symmetric.
+    TF_LITE_ENSURE_EQ(context, input->params.zero_point, 0);
+    TF_LITE_ENSURE_EQ(context, output->params.zero_point, 0);
+
+    // Check quantized_bias_type is either kTfLiteInt64 or kTfLiteInt32.
+    if (params->quantized_bias_type != kTfLiteFloat32) {
+      TF_LITE_ENSURE(context, params->quantized_bias_type == kTfLiteInt32 ||
+                                  params->quantized_bias_type == kTfLiteInt64);
+      TF_LITE_ENSURE(context, (bias == nullptr) ||
+                                  bias->type == params->quantized_bias_type);
+      data->quantized_bias_type = params->quantized_bias_type;
+    }
+  }
+
   const bool is_hybrid =
       (input->type == kTfLiteFloat32 &&
-       (filter->type == kTfLiteUInt8 || filter->type == kTfLiteInt8));
+       (filter->type == kTfLiteUInt8 || filter->type == kTfLiteInt8 ||
+        filter->type == kTfLiteInt4));
 
-  if (is_hybrid && filter->type == kTfLiteInt8 &&
+  if (filter->quantization.type == kTfLiteAffineQuantization) {
+    TF_LITE_ENSURE(context, filter->quantization.params);
+    TF_LITE_ENSURE(context, reinterpret_cast<TfLiteAffineQuantization*>(
+                                filter->quantization.params)
+                                ->scale);
+  }
+
+  if (is_hybrid &&
+      (filter->type == kTfLiteInt8 || filter->type == kTfLiteInt4) &&
       filter->quantization.type == kTfLiteAffineQuantization &&
-      filter->quantization.params &&
-      reinterpret_cast<TfLiteAffineQuantization*>(filter->quantization.params)
-          ->scale &&
       reinterpret_cast<TfLiteAffineQuantization*>(filter->quantization.params)
               ->scale->size > 1) {
     const auto* affine_quantization =
@@ -505,7 +533,7 @@ TfLiteStatus Prepare(KernelType kernel_type, TfLiteContext* context,
         &context->tensors[node->temporaries->data[data->im2col_index]];
     im2col->type = input->type;
     if (is_hybrid) {
-      im2col->type = filter->type;
+      im2col->type = filter->type == kTfLiteInt4 ? kTfLiteInt8 : filter->type;
     }
     im2col->allocation_type = kTfLiteArenaRw;
     auto im2col_status = context->ResizeTensor(context, im2col, im2col_size);
@@ -681,6 +709,19 @@ void EvalQuantized(TfLiteContext* context, TfLiteNode* node,
     effective_kernel_type = kReference;
   }
 
+  const uint8_t* filter_data = nullptr;
+  std::unique_ptr<int8_t[]> unpacked_filter_data = nullptr;
+  if (filter->type == kTfLiteInt4) {
+    const size_t bytes_unpacked = filter->bytes * 2;
+    unpacked_filter_data = std::make_unique<int8_t[]>(bytes_unpacked);
+    tflite::tensor_utils::UnpackDenseInt4IntoInt8(
+        GetTensorData<int8_t>(filter), GetTensorShape(filter).FlatSize(),
+        unpacked_filter_data.get());
+    filter_data = reinterpret_cast<const uint8_t*>(unpacked_filter_data.get());
+  } else {
+    filter_data = GetTensorData<uint8_t>(filter);
+  }
+
   ConvParams op_params;
   op_params.padding_type = PaddingType::kSame;
   op_params.padding_values.width = data->padding.width;
@@ -700,10 +741,10 @@ void EvalQuantized(TfLiteContext* context, TfLiteNode* node,
     case kReference: {
       reference_ops::Conv(
           op_params, GetTensorShape(input), GetTensorData<uint8_t>(input),
-          GetTensorShape(filter), GetTensorData<uint8_t>(filter),
-          GetTensorShape(bias), GetTensorData<int32_t>(bias),
-          GetTensorShape(output), GetTensorData<uint8_t>(output),
-          GetTensorShape(im2col), GetTensorData<uint8_t>(im2col),
+          GetTensorShape(filter), filter_data, GetTensorShape(bias),
+          GetTensorData<int32_t>(bias), GetTensorShape(output),
+          GetTensorData<uint8_t>(output), GetTensorShape(im2col),
+          GetTensorData<uint8_t>(im2col),
           /* cpu_backend_context = */ nullptr);
       break;
     }
@@ -713,10 +754,10 @@ void EvalQuantized(TfLiteContext* context, TfLiteNode* node,
       // There is only one optimized implementation for Quantized Conv.
       optimized_ops::Conv(
           op_params, GetTensorShape(input), GetTensorData<uint8_t>(input),
-          GetTensorShape(filter), GetTensorData<uint8_t>(filter),
-          GetTensorShape(bias), GetTensorData<int32_t>(bias),
-          GetTensorShape(output), GetTensorData<uint8_t>(output),
-          GetTensorShape(im2col), GetTensorData<uint8_t>(im2col),
+          GetTensorShape(filter), filter_data, GetTensorShape(bias),
+          GetTensorData<int32_t>(bias), GetTensorShape(output),
+          GetTensorData<uint8_t>(output), GetTensorShape(im2col),
+          GetTensorData<uint8_t>(im2col),
           CpuBackendContext::GetFromContext(context));
       break;
     }
@@ -755,17 +796,17 @@ void EvalQuantizedPerChannel(TfLiteContext* context, TfLiteNode* node,
     effective_kernel_type = kReference;
   }
 
-  const int8_t* filter_data;
-  const size_t bytes_unpacked = filter->bytes * 2;
-  auto unpacked_filter_data = std::make_unique<int8_t[]>(bytes_unpacked);
-
+  const int8_t* filter_data = nullptr;
+  std::unique_ptr<int8_t[]> unpacked_filter_data = nullptr;
   if (filter->type == kTfLiteInt4) {
+    const size_t bytes_unpacked = filter->bytes * 2;
+    unpacked_filter_data = std::make_unique<int8_t[]>(bytes_unpacked);
     tflite::tensor_utils::UnpackDenseInt4IntoInt8(
         GetTensorData<int8_t>(filter), GetTensorShape(filter).FlatSize(),
         unpacked_filter_data.get());
     filter_data = unpacked_filter_data.get();
   } else {
-    filter_data = GetTensorData<int8>(filter);
+    filter_data = GetTensorData<int8_t>(filter);
   }
 
   switch (effective_kernel_type) {
@@ -855,34 +896,47 @@ void EvalQuantizedPerChannel16x8(TfLiteContext* context, TfLiteNode* node,
                             filter->params.zero_point ||
                             output->params.zero_point;
 
-  // Fallback to reference kernel when bias_type is int64 as
-  // there is no optimized kernel for int64 bias yet.
-  if (bias && bias->type == kTfLiteInt64) {
-    reference_integer_ops::ConvPerChannel(
-        op_params, data->per_channel_output_multiplier.data(),
-        data->per_channel_output_shift.data(), GetTensorShape(input),
-        GetTensorData<int16>(input), GetTensorShape(filter),
-        GetTensorData<int8>(filter), GetTensorShape(bias),
-        GetTensorData<std::int64_t>(bias), GetTensorShape(output),
-        GetTensorData<int16>(output));
-  } else if (effective_kernel_type == kReference || has_non_zero_point) {
-    reference_integer_ops::ConvPerChannel(
-        op_params, data->per_channel_output_multiplier.data(),
-        data->per_channel_output_shift.data(), GetTensorShape(input),
-        GetTensorData<int16>(input), GetTensorShape(filter),
-        GetTensorData<int8>(filter), GetTensorShape(bias),
-        GetTensorData<std::int32_t>(bias), GetTensorShape(output),
-        GetTensorData<int16>(output));
+  const int8_t* filter_data = nullptr;
+  std::unique_ptr<int8_t[]> unpacked_filter_data = nullptr;
+  if (filter->type == kTfLiteInt4) {
+    const size_t bytes_unpacked = filter->bytes * 2;
+    unpacked_filter_data = std::make_unique<int8_t[]>(bytes_unpacked);
+    tflite::tensor_utils::UnpackDenseInt4IntoInt8(
+        GetTensorData<int8_t>(filter), GetTensorShape(filter).FlatSize(),
+        unpacked_filter_data.get());
+    filter_data = unpacked_filter_data.get();
   } else {
-    optimized_integer_ops::ConvPerChannel(
+    filter_data = GetTensorData<int8_t>(filter);
+  }
+
+  if (data->quantized_bias_type == kTfLiteInt32) {
+    if (effective_kernel_type == kReference || has_non_zero_point) {
+      reference_integer_ops::ConvPerChannel(
+          op_params, data->per_channel_output_multiplier.data(),
+          data->per_channel_output_shift.data(), GetTensorShape(input),
+          GetTensorData<int16>(input), GetTensorShape(filter), filter_data,
+          GetTensorShape(bias), GetTensorData<int32_t>(bias),
+          GetTensorShape(output), GetTensorData<int16>(output));
+    } else {
+      optimized_integer_ops::ConvPerChannel(
+          op_params, data->per_channel_output_multiplier.data(),
+          data->per_channel_output_shift.data(), GetTensorShape(input),
+          GetTensorData<int16_t>(input), GetTensorShape(filter), filter_data,
+          GetTensorShape(bias), GetTensorData<int32_t>(bias),
+          GetTensorShape(output), GetTensorData<int16_t>(output),
+          GetTensorShape(im2col), GetTensorData<int16_t>(im2col),
+          CpuBackendContext::GetFromContext(context));
+    }
+  } else {
+    TFLITE_DCHECK(!has_non_zero_point);
+    // Fallback to reference kernel when bias_type is int64 as
+    // there is no optimized kernel for int64 bias yet.
+    reference_integer_ops::ConvPerChannel(
         op_params, data->per_channel_output_multiplier.data(),
         data->per_channel_output_shift.data(), GetTensorShape(input),
-        GetTensorData<int16_t>(input), GetTensorShape(filter),
-        GetTensorData<int8_t>(filter), GetTensorShape(bias),
-        GetTensorData<std::int32_t>(bias), GetTensorShape(output),
-        GetTensorData<int16_t>(output), GetTensorShape(im2col),
-        GetTensorData<int16_t>(im2col),
-        CpuBackendContext::GetFromContext(context));
+        GetTensorData<int16>(input), GetTensorShape(filter), filter_data,
+        GetTensorShape(bias), GetTensorData<int64_t>(bias),
+        GetTensorShape(output), GetTensorData<int16>(output));
   }
 }
 
@@ -1023,11 +1077,9 @@ TfLiteStatus EvalHybridPerChannel(TfLiteContext* context, TfLiteNode* node,
   }
 
   int8_t* im2col_ptr = nullptr;
-  int8_t* filter_ptr = nullptr;
   if (im2col != nullptr) {
     im2col_ptr = im2col->data.int8;
   }
-  filter_ptr = filter->data.int8;
   const auto* affine_quantization =
       reinterpret_cast<TfLiteAffineQuantization*>(filter->quantization.params);
 
@@ -1044,6 +1096,19 @@ TfLiteStatus EvalHybridPerChannel(TfLiteContext* context, TfLiteNode* node,
     effective_kernel_type = kReference;
   }
 
+  const int8_t* filter_data = nullptr;
+  std::unique_ptr<int8_t[]> unpacked_filter_data = nullptr;
+  if (filter->type == kTfLiteInt4) {
+    const size_t bytes_unpacked = filter->bytes * 2;
+    unpacked_filter_data = std::make_unique<int8_t[]>(bytes_unpacked);
+    tflite::tensor_utils::UnpackDenseInt4IntoInt8(
+        GetTensorData<int8_t>(filter), GetTensorShape(filter).FlatSize(),
+        unpacked_filter_data.get());
+    filter_data = unpacked_filter_data.get();
+  } else {
+    filter_data = GetTensorData<int8_t>(filter);
+  }
+
   ConvParams op_params;
   op_params.padding_type = PaddingType::kSame;
   op_params.padding_values.width = data->padding.width;
@@ -1058,7 +1123,7 @@ TfLiteStatus EvalHybridPerChannel(TfLiteContext* context, TfLiteNode* node,
     case kReference:
       reference_ops::HybridConvPerChannel(
           op_params, scaling_factors_ptr, GetTensorShape(input),
-          quantized_input_ptr_batch, GetTensorShape(filter), filter_ptr,
+          quantized_input_ptr_batch, GetTensorShape(filter), filter_data,
           GetTensorShape(bias), GetTensorData<float>(bias),
           GetTensorShape(output), GetTensorData<float>(output),
           GetTensorShape(im2col), im2col_ptr, affine_quantization->scale->data,
@@ -1077,7 +1142,7 @@ TfLiteStatus EvalHybridPerChannel(TfLiteContext* context, TfLiteNode* node,
           GetTemporarySafe(context, node, data->accum_scratch_index, &scratch));
       optimized_ops::HybridConvPerChannel(
           op_params, scaling_factors_ptr, GetTensorShape(input),
-          quantized_input_ptr_batch, GetTensorShape(filter), filter_ptr,
+          quantized_input_ptr_batch, GetTensorShape(filter), filter_data,
           GetTensorShape(bias), GetTensorData<float>(bias),
           GetTensorShape(output), GetTensorData<float>(output),
           GetTensorShape(im2col), im2col_ptr, affine_quantization->scale->data,
@@ -1119,6 +1184,15 @@ TfLiteStatus EvalHybrid(TfLiteContext* context, TfLiteNode* node,
                     GetTemporarySafe(context, node, data->scaling_factors_index,
                                      &scaling_factors_tensor));
   float* scaling_factors_ptr = GetTensorData<float>(scaling_factors_tensor);
+  float scale = filter->params.scale;
+  if (filter->quantization.type == kTfLiteAffineQuantization) {
+    const auto* affine_quantization =
+        reinterpret_cast<TfLiteAffineQuantization*>(
+            filter->quantization.params);
+    if (affine_quantization->scale->size > 1) {
+      scale = affine_quantization->scale->data[0];
+    }
+  }
 
   // Per-batch input quantization for higher accuracy.
   {
@@ -1129,10 +1203,23 @@ TfLiteStatus EvalHybrid(TfLiteContext* context, TfLiteNode* node,
       tensor_utils::SymmetricQuantizeFloats(
           input_ptr + offset, input_size, quantized_input_ptr_batch + offset,
           &unused_min, &unused_max, &scaling_factors_ptr[b]);
-      scaling_factors_ptr[b] *= filter->params.scale;
+      scaling_factors_ptr[b] *= scale;
     }
   }
 
+  const int8_t* filter_data = nullptr;
+  std::unique_ptr<int8_t[]> unpacked_filter_data = nullptr;
+  if (filter->type == kTfLiteInt4) {
+    const size_t bytes_unpacked = filter->bytes * 2;
+    unpacked_filter_data = std::make_unique<int8_t[]>(bytes_unpacked);
+    tflite::tensor_utils::UnpackDenseInt4IntoInt8(
+        GetTensorData<int8_t>(filter), GetTensorShape(filter).FlatSize(),
+        unpacked_filter_data.get());
+    filter_data = unpacked_filter_data.get();
+  } else {
+    filter_data = GetTensorData<int8_t>(filter);
+  }
+
   switch (kernel_type) {
     case kReference:
     case kGenericOptimized:
@@ -1152,9 +1239,9 @@ TfLiteStatus EvalHybrid(TfLiteContext* context, TfLiteNode* node,
       if (data->groups == 1) {
         optimized_ops::HybridConv(
             op_params, scaling_factors_ptr, GetTensorShape(input),
-            quantized_input_ptr_batch, GetTensorShape(filter),
-            GetTensorData<int8_t>(filter), GetTensorShape(bias),
-            GetTensorData<float>(bias), GetTensorShape(accum_scratch),
+            quantized_input_ptr_batch, GetTensorShape(filter), filter_data,
+            GetTensorShape(bias), GetTensorData<float>(bias),
+            GetTensorShape(accum_scratch),
             GetTensorData<int32_t>(accum_scratch), GetTensorShape(output),
             GetTensorData<float>(output), GetTensorShape(im2col),
             GetTensorData<int8_t>(im2col),
@@ -1204,7 +1291,8 @@ TfLiteStatus EvalImpl(TfLiteContext* context, TfLiteNode* node) {
   TFLITE_DCHECK_EQ(input_type, input->type);
   switch (input_type) {  // Already know in/outtypes are same.
     case kTfLiteFloat32:
-      if (filter->type == kTfLiteUInt8 || filter->type == kTfLiteInt8) {
+      if (filter->type == kTfLiteUInt8 || filter->type == kTfLiteInt8 ||
+          filter->type == kTfLiteInt4) {
         if (data->is_hybrid_per_channel ||
             // TODO(b/162870360): Fallback to PerChannel implementation
             // before we have grouped hybrid convolution.
