@@ -56,8 +56,8 @@ class BaseConvolutionOpModel : public SingleOpModel {
       int stride_height = 2, enum Padding padding = Padding_VALID,
       enum ActivationFunctionType activation = ActivationFunctionType_NONE,
       int dilation_width_factor = 1, int dilation_height_factor = 1,
-      int num_threads = -1,
-      std::initializer_list<FilterType> filter_data = {}) {
+      int num_threads = -1, std::initializer_list<FilterType> filter_data = {},
+      const TensorType bias_type = TensorType_INT32) {
     input_ = AddInput(input);
 
     if (filter_data.size()) {
@@ -85,11 +85,6 @@ class BaseConvolutionOpModel : public SingleOpModel {
               input.scale * filter.per_channel_quantization_scales[i];
           bias_zero_points[i] = 0;
         }
-        tflite::TensorType bias_type = TensorType_INT32;
-        if (input.type == TensorType_INT16) {
-          // In case of 16-bit, the bias type is set to be int 64.
-          bias_type = TensorType_INT64;
-        }
         TensorData bias{bias_type,
                         {bias_size},
                         /*min=*/0,
@@ -104,7 +99,7 @@ class BaseConvolutionOpModel : public SingleOpModel {
       } else {
         // per tensor quantization.
         auto bias_scale = GetScale(input_) * GetScale(filter_);
-        TensorData bias{TensorType_INT32, {bias_size}, 0, 0, bias_scale};
+        TensorData bias{bias_type, {bias_size}, 0, 0, bias_scale};
         bias_ = AddInput(bias);
       }
     }
@@ -114,7 +109,7 @@ class BaseConvolutionOpModel : public SingleOpModel {
     SetBuiltinOp(BuiltinOperator_CONV_2D, BuiltinOptions_Conv2DOptions,
                  CreateConv2DOptions(
                      builder_, padding, stride_width, stride_height, activation,
-                     dilation_width_factor, dilation_height_factor)
+                     dilation_width_factor, dilation_height_factor, bias_type)
                      .Union());
 
     resolver_ = std::make_unique<SingleOpResolver>(BuiltinOperator_CONV_2D,
@@ -468,14 +463,16 @@ TEST_P(ConvolutionOpTest, PointwiseFloat32) {
 
   ASSERT_EQ(m.Invoke(), kTfLiteOk);
 
-  EXPECT_THAT(m.GetOutput(), ElementsAreArray({
-                                 // First batch
-                                 1.5, 1.5, 1.5, 1.5,  // row = 1
-                                 3., 3., 3., 3.,      // row = 2
-                                 // Second batch
-                                 1.5, 3., 4.5, 6.,  // row = 1
-                                 1.5, 3., 4.5, 6.,  // row = 2
-                             }));
+  EXPECT_THAT(m.GetOutput(),
+              Pointwise(FloatingPointEq(), {
+                                               // First batch
+                                               1.5, 1.5, 1.5, 1.5,  // row = 1
+                                               3., 3., 3., 3.,      // row = 2
+
+                                               // Second batch
+                                               1.5, 3., 4.5, 6.,  // row = 1
+                                               1.5, 3., 4.5, 6.,  // row = 2
+                                           }));
 }
 
 // TODO(alanchiao): this passes locally, but fails on continuous build system.
@@ -502,12 +499,14 @@ TEST_P(ConvolutionOpTest, DISABLED_PointwiseMultifilterFloat32) {
 
   ASSERT_EQ(m.Invoke(), kTfLiteOk);
 
-  EXPECT_THAT(m.GetOutput(),
-              ElementsAreArray({
-                  1.5, 2.5, 1.5, 2.5, 1.5, 2.5, 1.5, 2.5, 3., 5.,  3.,
-                  5.,  3.,  5.,  3.,  5.,  1.5, 2.5, 3.,  5., 4.5, 7.5,
-                  6.,  10., 1.5, 2.5, 3.,  5.,  4.5, 7.5, 6., 10.,
-              }));
+  EXPECT_THAT(
+      m.GetOutput(),
+      Pointwise(FloatingPointEq(),
+                {
+                    1.5, 2.5, 1.5, 2.5, 1.5, 2.5, 1.5, 2.5, 3., 5.,  3.,
+                    5.,  3.,  5.,  3.,  5.,  1.5, 2.5, 3.,  5., 4.5, 7.5,
+                    6.,  10., 1.5, 2.5, 3.,  5.,  4.5, 7.5, 6., 10.,
+                }));
 }
 
 TEST_P(ConvolutionOpTest, SimpleTestFloat32WithAnisotropicStrides) {
@@ -1423,6 +1422,39 @@ TEST_P(ConvolutionOpTest, SimpleTestHybridInt8) {
                                  0.16)));
 }
 
+TEST_P(ConvolutionOpTest, SimpleTestHybridInt4) {
+  HybridConvolutionOpModel m(
+      GetRegistration(), {TensorType_FLOAT32, {2, 2, 4, 1}},
+      {TensorType_INT4, {3, 2, 2, 1}, 0, 0, 4.0 / 7.0, 0},
+      {TensorType_FLOAT32, {}});
+
+  m.SetInput({
+      // First batch
+      1, 1, 1, 1,  // row = 1
+      2, 2, 2, 2,  // row = 2
+      // Second batch
+      1, 2, 3, 4,  // row = 1
+      1, 2, 3, 4,  // row = 2
+  });
+  m.SetSignedFilter({
+      1, 2, 3, 4,    // first 2x2 filter
+      -1, 1, -1, 1,  // second 2x2 filter
+      -1, -1, 1, 1,  // third 2x2 filter
+  });
+  m.SetBias({1, 2, 3});
+
+  ASSERT_EQ(m.Invoke(), kTfLiteOk);
+
+  EXPECT_THAT(m.GetOutput(), ElementsAreArray(ArrayFloatNear(
+                                 {
+                                     18, 2, 5,  // first batch, left
+                                     18, 2, 5,  // first batch, right
+                                     17, 4, 3,  // second batch, left
+                                     36, 4, 3,  // second batch, right
+                                 },
+                                 0.45)));
+}
+
 TEST_P(ConvolutionOpTest, SimpleTestHybridInt8WithDilation) {
   const int stride_width = 1;
   const int stride_height = 1;
@@ -1657,8 +1689,9 @@ class PerChannelQuantizedConvolutionOpModel
  public:
   using BaseConvolutionOpModel::BaseConvolutionOpModel;
 
-  void SetInput(std::initializer_list<float> data) {
-    QuantizeAndPopulate<int8_t>(input_, data);
+  template <typename T>
+  void SetInput(const std::vector<float>& data) {
+    QuantizeAndPopulate<T>(input_, data);
   }
 
   void SetFilter(std::initializer_list<float> data) {
@@ -1669,10 +1702,15 @@ class PerChannelQuantizedConvolutionOpModel
     PerChannelQuantizeBias(bias_, data);
   }
 
-  std::vector<int8_t> GetOutput() { return ExtractVector<int8_t>(output_); }
+  template <typename T>
+  std::vector<T> GetOutput() {
+    return ExtractVector<T>(output_);
+  }
+
+  template <typename T>
   std::vector<float> GetDequantizedOutput() {
-    return Dequantize<int8_t>(ExtractVector<int8_t>(output_), GetScale(output_),
-                              GetZeroPoint(output_));
+    return Dequantize<T>(ExtractVector<T>(output_), GetScale(output_),
+                         GetZeroPoint(output_));
   }
 };
 
@@ -1714,7 +1752,7 @@ TEST_P(ConvolutionOpTest, SimplePerTensorTest) {
        /*channel_index=*/0},
       {TensorType_INT8, {}, -63.5, 64, 0.5, -1},
       /*stride_width=*/1, /*stride_height=*/1);
-  m.SetInput({
+  m.SetInput<int8_t>({
       // [1 * 2 * 3 * 2] as [batch, y, x, input_channel]
       3, 2,    // batch = 0, y = 0, x = 0
       1, -1,   // batch = 0, y = 0, x = 1
@@ -1740,9 +1778,56 @@ TEST_P(ConvolutionOpTest, SimplePerTensorTest) {
   // Invoke and verify output.
   // output has dimension [1 * 1 * 2 * 2] as [batch, y, x, output_channel]
   ASSERT_EQ(m.Invoke(), kTfLiteOk);
-  EXPECT_THAT(m.GetDequantizedOutput(),
+  EXPECT_THAT(m.GetDequantizedOutput<int8_t>(),
               ElementsAreArray(ArrayFloatNear({31, 56, -57, -44})));
-  EXPECT_THAT(m.GetOutput(), ElementsAreArray({61, 111, -115, -89}));
+  EXPECT_THAT(m.GetOutput<int8_t>(), ElementsAreArray({61, 111, -115, -89}));
+}
+
+TEST_P(ConvolutionOpTest, SimplePerTensorTest4bit) {
+  PerChannelQuantizedConvolutionOpModel m(
+      GetRegistration(), {TensorType_INT8, {1, 2, 3, 2}, -63.5, 64, 0.5, -1},
+      {TensorType_INT4,
+       // [2 * 2 * 2 * 2] as [output_channel, y, x, input_channel]
+       {2, 2, 2, 2},
+       0,
+       0,
+       0,
+       0,
+       /*per_channel_quantization=*/true,
+       /*per_channel_quantization_scales=*/{1},
+       /*per_channel_quantization_offsets=*/{0},
+       /*channel_index=*/0},
+      {TensorType_INT8, {}, -63.5, 64, 0.5, -1},
+      /*stride_width=*/1, /*stride_height=*/1);
+  m.SetInput<int8_t>({
+      // [1 * 2 * 3 * 2] as [batch, y, x, input_channel]
+      3, 2,    // batch = 0, y = 0, x = 0
+      1, -1,   // batch = 0, y = 0, x = 1
+      -2, -3,  // batch = 0, y = 0, x = 2
+      4, 3,    // batch = 0, y = 1, x = 0
+      2, -2,   // batch = 0, y = 1, x = 1
+      -3, -4,  // batch = 0, y = 1, x = 2
+  });
+  m.SetFilter(
+      // [2 * 2 * 2 * 2] as [output_channel, y, x, input_channel]
+      {
+          1, 2,  // out channel = 0, y = 0, x = 0
+          3, 4,  // out channel = 0, y = 0, x = 1
+          3, 4,  // out channel = 0, y = 1, x = 0
+          5, 6,  // out channel = 0, y = 1, x = 1
+          7, 7,  // out channel = 1, y = 0, x = 0
+          5, 6,  // out channel = 1, y = 0, x = 1
+          3, 4,  // out channel = 1, y = 1, x = 0
+          1, 2,  // out channel = 1, y = 1, x = 1
+      });
+  m.SetBias({3, -2});
+
+  // Invoke and verify output.
+  // output has dimension [1 * 1 * 2 * 2] as [batch, y, x, output_channel]
+  ASSERT_EQ(m.Invoke(), kTfLiteOk);
+  EXPECT_THAT(m.GetDequantizedOutput<int8_t>(),
+              ElementsAreArray(ArrayFloatNear({31, 54, -57, -43})));
+  EXPECT_THAT(m.GetOutput<int8_t>(), ElementsAreArray({61, 107, -115, -87}));
 }
 
 TEST_P(ConvolutionOpTest, SimplePerChannelTest) {
@@ -1761,7 +1846,7 @@ TEST_P(ConvolutionOpTest, SimplePerChannelTest) {
        /*channel_index=*/0},
       {TensorType_INT8, {}, -63.5, 64, 0.5, -1},
       /*stride_width=*/1, /*stride_height=*/1);
-  m.SetInput({
+  m.SetInput<int8_t>({
       // [1 * 2 * 3 * 2] as [batch, y, x, input_channel]
       3, 2,    // batch = 0, y = 0, x = 0
       1, -1,   // batch = 0, y = 0, x = 1
@@ -1787,9 +1872,180 @@ TEST_P(ConvolutionOpTest, SimplePerChannelTest) {
   // Invoke and verify output.
   // output has dimension [1 * 1 * 2 * 2] as [batch, y, x, output_channel]
   ASSERT_EQ(m.Invoke(), kTfLiteOk);
-  EXPECT_THAT(m.GetDequantizedOutput(),
+  EXPECT_THAT(m.GetDequantizedOutput<int8_t>(),
               ElementsAreArray(ArrayFloatNear({31, 64, -57, -46})));
-  EXPECT_THAT(m.GetOutput(), ElementsAreArray({61, 127, -115, -93}));
+  EXPECT_THAT(m.GetOutput<int8_t>(), ElementsAreArray({61, 127, -115, -93}));
+}
+
+TEST_P(ConvolutionOpTest, SimplePerChannel16x8Bias32) {
+  const float scale = 128.0 / 65536;
+  PerChannelQuantizedConvolutionOpModel m(
+      GetRegistration(), {TensorType_INT16, {1, 2, 3, 2}, 0, 0, scale, 0},
+      {TensorType_INT8,
+       // [2 * 2 * 2 * 2] as [output_channel, y, x, input_channel]
+       {2, 2, 2, 2},
+       0,
+       0,
+       0,
+       0,
+       /*per_channel_quantization=*/true,
+       /*per_channel_quantization_scales=*/{1, 2},
+       /*per_channel_quantization_offsets=*/{0, 0},
+       /*channel_index=*/0},
+      {TensorType_INT16, {}, 0, 0, scale, 0},
+      /*stride_width=*/1, /*stride_height=*/1,
+      /*padding=*/Padding_VALID,
+      /*activation=*/ActivationFunctionType_NONE,
+      /*dilation_width_factor=*/1,
+      /*dilation_height_factor=*/1,
+      /*num_threads=*/-1,
+      /*filter_data=*/{},
+      /*bias_type=*/TensorType_INT32);
+
+  m.SetInput<int16_t>({
+      // [1 * 2 * 3 * 2] as [batch, y, x, input_channel]
+      3, 2,    // batch = 0, y = 0, x = 0
+      1, -1,   // batch = 0, y = 0, x = 1
+      -2, -3,  // batch = 0, y = 0, x = 2
+      4, 3,    // batch = 0, y = 1, x = 0
+      2, -2,   // batch = 0, y = 1, x = 1
+      -3, -4,  // batch = 0, y = 1, x = 2
+  });
+  m.SetFilter(
+      // [2 * 2 * 2 * 2] as [output_channel, y, x, input_channel]
+      {
+          1, 2,  // out channel = 0, y = 0, x = 0
+          3, 4,  // out channel = 0, y = 0, x = 1
+          3, 4,  // out channel = 0, y = 1, x = 0
+          5, 6,  // out channel = 0, y = 1, x = 1
+          7, 8,  // out channel = 1, y = 0, x = 0
+          5, 6,  // out channel = 1, y = 0, x = 1
+          3, 4,  // out channel = 1, y = 1, x = 0
+          1, 2,  // out channel = 1, y = 1, x = 1
+      });
+  m.SetBias({3, -2});
+
+  // Invoke and verify output.
+  // output has dimension [1 * 1 * 2 * 2] as [batch, y, x, output_channel]
+  ASSERT_EQ(m.Invoke(), kTfLiteOk);
+  EXPECT_THAT(m.GetDequantizedOutput<int16_t>(),
+              ElementsAreArray(ArrayFloatNear({31, 63.99804688, -57, -46})));
+  EXPECT_THAT(m.GetOutput<int16_t>(),
+              ElementsAreArray({15872, 32767, -29184, -23552}));
+}
+
+TEST_P(ConvolutionOpTest, SimplePerChannel16x4Bias32) {
+  const float scale = 128.0 / 65536;
+  PerChannelQuantizedConvolutionOpModel m(
+      GetRegistration(), {TensorType_INT16, {1, 2, 3, 2}, 0, 0, scale, 0},
+      {TensorType_INT4,
+       // [2 * 2 * 2 * 2] as [output_channel, y, x, input_channel]
+       {2, 2, 2, 2},
+       0,
+       0,
+       0,
+       0,
+       /*per_channel_quantization=*/true,
+       /*per_channel_quantization_scales=*/{1, 1},
+       /*per_channel_quantization_offsets=*/{0, 0},
+       /*channel_index=*/0},
+      {TensorType_INT16, {}, 0, 0, scale, 0},
+      /*stride_width=*/1, /*stride_height=*/1,
+      /*padding=*/Padding_VALID,
+      /*activation=*/ActivationFunctionType_NONE,
+      /*dilation_width_factor=*/1,
+      /*dilation_height_factor=*/1,
+      /*num_threads=*/-1,
+      /*filter_data=*/{},
+      /*bias_type=*/TensorType_INT32);
+
+  m.SetInput<int16_t>({
+      // [1 * 2 * 3 * 2] as [batch, y, x, input_channel]
+      3, 2,    // batch = 0, y = 0, x = 0
+      1, -1,   // batch = 0, y = 0, x = 1
+      -2, -3,  // batch = 0, y = 0, x = 2
+      4, 3,    // batch = 0, y = 1, x = 0
+      2, -2,   // batch = 0, y = 1, x = 1
+      -3, -4,  // batch = 0, y = 1, x = 2
+  });
+  m.SetFilter(
+      // [2 * 2 * 2 * 2] as [output_channel, y, x, input_channel]
+      {
+          1, 2,  // out channel = 0, y = 0, x = 0
+          3, 4,  // out channel = 0, y = 0, x = 1
+          3, 4,  // out channel = 0, y = 1, x = 0
+          5, 6,  // out channel = 0, y = 1, x = 1
+          7, 7,  // out channel = 1, y = 0, x = 0
+          5, 6,  // out channel = 1, y = 0, x = 1
+          3, 4,  // out channel = 1, y = 1, x = 0
+          1, 2,  // out channel = 1, y = 1, x = 1
+      });
+  m.SetBias({3, -2});
+
+  // Invoke and verify output.
+  // output has dimension [1 * 1 * 2 * 2] as [batch, y, x, output_channel]
+  ASSERT_EQ(m.Invoke(), kTfLiteOk);
+  EXPECT_THAT(m.GetDequantizedOutput<int16_t>(),
+              ElementsAreArray(ArrayFloatNear({31, 54, -57, -43})));
+  EXPECT_THAT(m.GetOutput<int16_t>(),
+              ElementsAreArray({15872, 27648, -29184, -22016}));
+}
+
+TEST_P(ConvolutionOpTest, SimplePerChannel16x8Bias64) {
+  const float scale = 128.0 / 65536;
+  PerChannelQuantizedConvolutionOpModel m(
+      GetRegistration(), {TensorType_INT16, {1, 2, 3, 2}, 0, 0, scale, 0},
+      {TensorType_INT8,
+       // [2 * 2 * 2 * 2] as [output_channel, y, x, input_channel]
+       {2, 2, 2, 2},
+       0,
+       0,
+       0,
+       0,
+       /*per_channel_quantization=*/true,
+       /*per_channel_quantization_scales=*/{1, 2},
+       /*per_channel_quantization_offsets=*/{0, 0},
+       /*channel_index=*/0},
+      {TensorType_INT16, {}, 0, 0, scale, 0},
+      /*stride_width=*/1, /*stride_height=*/1,
+      /*padding=*/Padding_VALID,
+      /*activation=*/ActivationFunctionType_NONE,
+      /*dilation_width_factor=*/1,
+      /*dilation_height_factor=*/1,
+      /*num_threads=*/-1,
+      /*filter_data=*/{},
+      /*bias_type=*/TensorType_INT64);
+
+  m.SetInput<int16_t>({
+      // [1 * 2 * 3 * 2] as [batch, y, x, input_channel]
+      3, 2,    // batch = 0, y = 0, x = 0
+      1, -1,   // batch = 0, y = 0, x = 1
+      -2, -3,  // batch = 0, y = 0, x = 2
+      4, 3,    // batch = 0, y = 1, x = 0
+      2, -2,   // batch = 0, y = 1, x = 1
+      -3, -4,  // batch = 0, y = 1, x = 2
+  });
+  m.SetFilter(
+      // [2 * 2 * 2 * 2] as [output_channel, y, x, input_channel]
+      {
+          1, 2,  // out channel = 0, y = 0, x = 0
+          3, 4,  // out channel = 0, y = 0, x = 1
+          3, 4,  // out channel = 0, y = 1, x = 0
+          5, 6,  // out channel = 0, y = 1, x = 1
+          7, 8,  // out channel = 1, y = 0, x = 0
+          5, 6,  // out channel = 1, y = 0, x = 1
+          3, 4,  // out channel = 1, y = 1, x = 0
+          1, 2,  // out channel = 1, y = 1, x = 1
+      });
+  m.SetBias({3, -2});
+
+  // Invoke and verify output.
+  // output has dimension [1 * 1 * 2 * 2] as [batch, y, x, output_channel]
+  ASSERT_EQ(m.Invoke(), kTfLiteOk);
+  EXPECT_THAT(m.GetDequantizedOutput<int16_t>(),
+              ElementsAreArray(ArrayFloatNear({31, 63.99804688, -57, -46})));
+  EXPECT_THAT(m.GetOutput<int16_t>(),
+              ElementsAreArray({15872, 32767, -29184, -23552}));
 }
 
 TEST_P(ConvolutionOpTest, Simple4bitPerChannelTest) {
@@ -1808,7 +2064,7 @@ TEST_P(ConvolutionOpTest, Simple4bitPerChannelTest) {
        /*channel_index=*/0},
       {TensorType_INT8, {}, -63.5, 64, 0.5, -1},
       /*stride_width=*/1, /*stride_height=*/1);
-  m.SetInput({
+  m.SetInput<int8_t>({
       // [1 * 2 * 3 * 2] as [batch, y, x, input_channel]
       3, 2,    // batch = 0, y = 0, x = 0
       1, -1,   // batch = 0, y = 0, x = 1
@@ -1834,9 +2090,9 @@ TEST_P(ConvolutionOpTest, Simple4bitPerChannelTest) {
   // Invoke and verify output.
   // output has dimension [1 * 1 * 2 * 2] as [batch, y, x, output_channel]
   ASSERT_EQ(m.Invoke(), kTfLiteOk);
-  EXPECT_THAT(m.GetDequantizedOutput(),
+  EXPECT_THAT(m.GetDequantizedOutput<int8_t>(),
               ElementsAreArray(ArrayFloatNear({31, 64, -57, -46})));
-  EXPECT_THAT(m.GetOutput(), ElementsAreArray({61, 127, -115, -93}));
+  EXPECT_THAT(m.GetOutput<int8_t>(), ElementsAreArray({61, 127, -115, -93}));
 }
 
 class HybridPerChannelConvolutionOpModel
@@ -1908,6 +2164,50 @@ TEST_P(ConvolutionOpTest, SimpleTestHybridPerChannel) {
                                  0.16)));
 }
 
+TEST_P(ConvolutionOpTest, SimpleTestHybridPerChannelInt4) {
+  float scale = 4.0 / 7.0;
+  float scale2 = 1.0 / 7.0;
+  HybridPerChannelConvolutionOpModel m(
+      GetRegistration(), {TensorType_FLOAT32, {2, 2, 4, 2}},
+      {TensorType_INT4,
+       {3, 2, 2, 2},
+       0,
+       0,
+       0,
+       0,
+       /*per_channel_quantization=*/true,
+       /*per_channel_quantization_scales=*/{scale, scale2, scale2},
+       /*per_channel_quantization_offsets=*/{0, 0, 0},
+       /*channel_index=*/0},
+      {TensorType_FLOAT32, {}});
+
+  m.SetInput({
+      // First batch
+      0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5,  // row = 1
+      1, 1, 1, 1, 1, 1, 1, 1,                  // row = 2
+      // Second batch
+      0.5, 0.5, 1, 1, 1.5, 1.5, 2, 2,  // row = 1
+      0.5, 0.5, 1, 1, 1.5, 1.5, 2, 2   // row = 2
+  });
+  m.SetSignedFilter({
+      1,  1,  2,  2,  3,  3,  4, 4,  // first 2x2 filter
+      -1, -1, 1,  1,  -1, -1, 1, 1,  // second 2x2 filter
+      -1, -1, -1, -1, 1,  1,  1, 1   // third 2x2 filter
+  });
+  m.SetBias({1, 2, 3});
+
+  ASSERT_EQ(m.Invoke(), kTfLiteOk);
+
+  EXPECT_THAT(m.GetOutput(), ElementsAreArray(ArrayFloatNear(
+                                 {
+                                     18, 2, 5,  // first batch, left
+                                     18, 2, 5,  // first batch, right
+                                     16, 4, 3,  // second batch, left
+                                     36, 4, 3,  // second batch, right
+                                 },
+                                 0.45)));
+}
+
 TEST_P(ConvolutionOpTest, SimpleTestHybridPerChannelGrouped) {
   float scale = 4.0 / 127.0;
   float scale2 = 1.0 / 127.0;
