@@ -379,7 +379,8 @@ class DetectionPostprocessOpModelwithRegularNMS : public SingleOpModel {
       const TensorData& input1, const TensorData& input2,
       const TensorData& input3, const TensorData& output1,
       const TensorData& output2, const TensorData& output3,
-      const TensorData& output4, bool use_regular_nms, int num_threads = 1) {
+      const TensorData& output4, bool use_regular_nms, int num_threads = 1,
+      int max_detections = 3, int detection_per_class = 1) {
     input1_ = AddInput(input1);
     input2_ = AddInput(input2);
     input3_ = AddInput(input3);
@@ -390,9 +391,9 @@ class DetectionPostprocessOpModelwithRegularNMS : public SingleOpModel {
 
     flexbuffers::Builder fbb;
     fbb.Map([&]() {
-      fbb.Int("max_detections", 3);
+      fbb.Int("max_detections", max_detections);
       fbb.Int("max_classes_per_detection", 1);
-      fbb.Int("detections_per_class", 1);
+      fbb.Int("detections_per_class", detection_per_class);
       fbb.Bool("use_regular_nms", use_regular_nms);
       fbb.Float("nms_score_threshold", 0.0);
       fbb.Float("nms_iou_threshold", 0.5);
@@ -702,6 +703,234 @@ TEST_P(DetectionPostprocessOpRegularTest, RegularNMS) {
   }
 }
 
+TEST_P(DetectionPostprocessOpRegularTest, RegularNMSWithEqualScores) {
+  TensorData input1, input2, input3;
+  if (tensor_type_ == TensorType_UINT8) {
+    input1 = {tensor_type_, {1, 6, 4}, -1.0, 1.0};
+    input2 = {tensor_type_, {1, 6, 3}, 0.0, 1.0};
+    input3 = {tensor_type_, {6, 4}, 0.0, 100.5};
+  } else {
+    input1 = {tensor_type_, {1, 6, 4}};
+    input2 = {tensor_type_, {1, 6, 3}};
+    input3 = {tensor_type_, {6, 4}};
+  }
+  DetectionPostprocessOpModelwithRegularNMS m(
+      input1, input2, input3, {TensorType_FLOAT32, {}},
+      {TensorType_FLOAT32, {}}, {TensorType_FLOAT32, {}},
+      {TensorType_FLOAT32, {}}, true, num_threads_, /*max_detections=*/4,
+      /*detection_per_class=*/2);
+  auto inputs1 = {
+      0.0f, 0.0f, 0.0f, 0.0f,  // box #1 (0, 0, 1, 1)
+      0.0f, 0.0f, 0.0f, 0.0f,  // box #2 (0, 1, 1, 2)
+      0.0f, 0.0f, 0.0f, 0.0f,  // box #3 (0, 5, 1, 6)
+      0.0f, 0.0f, 0.0f, 0.0f,  // box #4 (0, 10, 1, 11)
+      0.0f, 0.0f, 0.0f, 0.0f,  // box #5 (0, 20, 1, 21)
+      0.0f, 0.0f, 0.0f, 0.0f   // box #6 (0, 100, 1, 101)
+  };
+
+  if (tensor_type_ == TensorType_UINT8) {
+    m.QuantizeAndPopulate<uint8_t>(m.input1(), std::vector<float>{inputs1});
+  } else {
+    m.SetInput1<float>(inputs1);
+  }
+  // class scores - two classes with background
+  auto inputs2 = {
+      0.f, .1f,  0.1f,   // box #1
+      0.f, .1f,  0.96f,  // box #2
+      0.f, .1f,  0.9f,   // box #3
+      0.f, .95f, 0.1f,   // box #4
+      0.f, .9f,  0.1f,   // box #5
+      0.f, .1f,  0.1f    // box #6
+  };
+  if (tensor_type_ == TensorType_UINT8) {
+    m.QuantizeAndPopulate<uint8_t>(m.input2(), std::vector<float>{inputs2});
+  } else {
+    m.SetInput2<float>(inputs2);
+  }
+  // six anchors in center-size encoding
+  auto inputs3 = {
+      0.5f, 0.5f,   1.0f, 1.0f,  // box #1
+      0.5f, 1.5f,   1.0f, 1.0f,  // box #2
+      0.5f, 5.5f,   1.0f, 1.0f,  // box #3
+      0.5f, 10.5f,  1.0f, 1.0f,  // box #4
+      0.5f, 20.5f,  1.0f, 1.0f,  // box #5
+      0.5f, 100.5f, 1.0f, 1.0f   // box #6
+  };
+  if (tensor_type_ == TensorType_UINT8) {
+    m.QuantizeAndPopulate<uint8_t>(m.input3(), std::vector<float>{inputs3});
+  } else {
+    m.SetInput3<float>(inputs3);
+  }
+  ASSERT_EQ(m.Invoke(), kTfLiteOk);
+  // detection_boxes
+  // in center-size
+  std::vector<int> output_shape1 = m.GetOutputShape1();
+  EXPECT_THAT(output_shape1, ElementsAre(1, 4, 4));
+  if (tensor_type_ == TensorType_UINT8) {
+    EXPECT_THAT(m.GetOutput1<float>(), ElementsAreArray(ArrayFloatNear(
+                                           {
+                                               0, 1, 1, 2,    // box #2
+                                               0, 10, 1, 11,  // box #4
+                                               0, 20, 1, 21,  // box #5
+                                               0, 5, 1, 6     // box #3
+                                           },
+                                           3e-1)));
+  } else {
+    EXPECT_THAT(m.GetOutput1<float>(), ElementsAreArray(ArrayFloatNear(
+                                           {
+                                               0, 1, 1, 2,    // box #2
+                                               0, 10, 1, 11,  // box #4
+                                               0, 20, 1, 21,  // box #5
+                                               0, 5, 1, 6     // box #3
+                                           },
+                                           3e-4)));
+  }
+  // detection_classes
+  std::vector<int> output_shape2 = m.GetOutputShape2();
+  EXPECT_THAT(output_shape2, ElementsAre(1, 4));
+  if (tensor_type_ == TensorType_UINT8) {
+    EXPECT_THAT(m.GetOutput2<float>(),
+                ElementsAreArray(ArrayFloatNear({1, 0, 0, 1}, 1e-1)));
+  } else {
+    EXPECT_THAT(m.GetOutput2<float>(),
+                ElementsAreArray(ArrayFloatNear({1, 0, 0, 1}, 1e-4)));
+  }
+  // detection_scores
+  std::vector<int> output_shape3 = m.GetOutputShape3();
+  EXPECT_THAT(output_shape3, ElementsAre(1, 4));
+  if (tensor_type_ == TensorType_UINT8) {
+    EXPECT_THAT(m.GetOutput3<float>(),
+                ElementsAreArray(ArrayFloatNear({0.96, 0.95, 0.9, 0.9}, 1e-1)));
+  } else {
+    EXPECT_THAT(m.GetOutput3<float>(),
+                ElementsAreArray(ArrayFloatNear({0.96, 0.95, 0.9, 0.9}, 1e-4)));
+  }
+  // num_detections
+  std::vector<int> output_shape4 = m.GetOutputShape4();
+  EXPECT_THAT(output_shape4, ElementsAre(1));
+  if (tensor_type_ == TensorType_UINT8) {
+    EXPECT_THAT(m.GetOutput4<float>(),
+                ElementsAreArray(ArrayFloatNear({4.0}, 1e-1)));
+  } else {
+    EXPECT_THAT(m.GetOutput4<float>(),
+                ElementsAreArray(ArrayFloatNear({4.0}, 1e-4)));
+  }
+}
+
+TEST_P(DetectionPostprocessOpRegularTest, FastNMSWithEqualScores) {
+  TensorData input1, input2, input3;
+  if (tensor_type_ == TensorType_UINT8) {
+    input1 = {tensor_type_, {1, 6, 4}, -1.0, 1.0};
+    input2 = {tensor_type_, {1, 6, 3}, 0.0, 1.0};
+    input3 = {tensor_type_, {6, 4}, 0.0, 100.5};
+  } else {
+    input1 = {tensor_type_, {1, 6, 4}};
+    input2 = {tensor_type_, {1, 6, 3}};
+    input3 = {tensor_type_, {6, 4}};
+  }
+  DetectionPostprocessOpModelwithRegularNMS m(
+      input1, input2, input3, {TensorType_FLOAT32, {}},
+      {TensorType_FLOAT32, {}}, {TensorType_FLOAT32, {}},
+      {TensorType_FLOAT32, {}}, false, num_threads_, /*max_detections=*/4,
+      /*detection_per_class=*/2);
+  auto inputs1 = {
+      0.0f, 0.0f, 0.0f, 0.0f,  // box #1 (0, 0, 1, 1)
+      0.0f, 0.0f, 0.0f, 0.0f,  // box #2 (0, 1, 1, 2)
+      0.0f, 0.0f, 0.0f, 0.0f,  // box #3 (0, 5, 1, 6)
+      0.0f, 0.0f, 0.0f, 0.0f,  // box #4 (0, 10, 1, 11)
+      0.0f, 0.0f, 0.0f, 0.0f,  // box #5 (0, 20, 1, 21)
+      0.0f, 0.0f, 0.0f, 0.0f   // box #6 (0, 100, 1, 101)
+  };
+
+  if (tensor_type_ == TensorType_UINT8) {
+    m.QuantizeAndPopulate<uint8_t>(m.input1(), std::vector<float>{inputs1});
+  } else {
+    m.SetInput1<float>(inputs1);
+  }
+  // class scores - two classes with background
+  auto inputs2 = {
+      0.f, .1f,  0.1f,   // box #1
+      0.f, .1f,  0.96f,  // box #2
+      0.f, .1f,  0.9f,   // box #3
+      0.f, .95f, 0.1f,   // box #4
+      0.f, .9f,  0.1f,   // box #5
+      0.f, .1f,  0.1f    // box #6
+  };
+  if (tensor_type_ == TensorType_UINT8) {
+    m.QuantizeAndPopulate<uint8_t>(m.input2(), std::vector<float>{inputs2});
+  } else {
+    m.SetInput2<float>(inputs2);
+  }
+  // six anchors in center-size encoding
+  auto inputs3 = {
+      0.5f, 0.5f,   1.0f, 1.0f,  // box #1
+      0.5f, 1.5f,   1.0f, 1.0f,  // box #2
+      0.5f, 5.5f,   1.0f, 1.0f,  // box #3
+      0.5f, 10.5f,  1.0f, 1.0f,  // box #4
+      0.5f, 20.5f,  1.0f, 1.0f,  // box #5
+      0.5f, 100.5f, 1.0f, 1.0f   // box #6
+  };
+  if (tensor_type_ == TensorType_UINT8) {
+    m.QuantizeAndPopulate<uint8_t>(m.input3(), std::vector<float>{inputs3});
+  } else {
+    m.SetInput3<float>(inputs3);
+  }
+  ASSERT_EQ(m.Invoke(), kTfLiteOk);
+  // detection_boxes
+  // in center-size
+  std::vector<int> output_shape1 = m.GetOutputShape1();
+  EXPECT_THAT(output_shape1, ElementsAre(1, 4, 4));
+  if (tensor_type_ == TensorType_UINT8) {
+    EXPECT_THAT(m.GetOutput1<float>(), ElementsAreArray(ArrayFloatNear(
+                                           {
+                                               0, 1, 1, 2,    // box #2
+                                               0, 10, 1, 11,  // box #4
+                                               0, 5, 1, 6,    // box #3
+                                               0, 20, 1, 21   // box #5
+                                           },
+                                           3e-1)));
+  } else {
+    EXPECT_THAT(m.GetOutput1<float>(), ElementsAreArray(ArrayFloatNear(
+                                           {
+                                               0, 1, 1, 2,    // box #2
+                                               0, 10, 1, 11,  // box #4
+                                               0, 5, 1, 6,    // box #3
+                                               0, 20, 1, 21   // box #5
+                                           },
+                                           3e-4)));
+  }
+  // detection_classes
+  std::vector<int> output_shape2 = m.GetOutputShape2();
+  EXPECT_THAT(output_shape2, ElementsAre(1, 4));
+  if (tensor_type_ == TensorType_UINT8) {
+    EXPECT_THAT(m.GetOutput2<float>(),
+                ElementsAreArray(ArrayFloatNear({1, 0, 1, 0}, 1e-1)));
+  } else {
+    EXPECT_THAT(m.GetOutput2<float>(),
+                ElementsAreArray(ArrayFloatNear({1, 0, 1, 0}, 1e-4)));
+  }
+  // detection_scores
+  std::vector<int> output_shape3 = m.GetOutputShape3();
+  EXPECT_THAT(output_shape3, ElementsAre(1, 4));
+  if (tensor_type_ == TensorType_UINT8) {
+    EXPECT_THAT(m.GetOutput3<float>(),
+                ElementsAreArray(ArrayFloatNear({0.96, 0.95, 0.9, 0.9}, 1e-1)));
+  } else {
+    EXPECT_THAT(m.GetOutput3<float>(),
+                ElementsAreArray(ArrayFloatNear({0.96, 0.95, 0.9, 0.9}, 1e-4)));
+  }
+  // num_detections
+  std::vector<int> output_shape4 = m.GetOutputShape4();
+  EXPECT_THAT(output_shape4, ElementsAre(1));
+  if (tensor_type_ == TensorType_UINT8) {
+    EXPECT_THAT(m.GetOutput4<float>(),
+                ElementsAreArray(ArrayFloatNear({4.0}, 1e-1)));
+  } else {
+    EXPECT_THAT(m.GetOutput4<float>(),
+                ElementsAreArray(ArrayFloatNear({4.0}, 1e-4)));
+  }
+}
+
 TEST(DetectionPostprocessOpTest, FloatTestwithNoBackgroundClassAndNoKeypoints) {
   DetectionPostprocessOpModelwithRegularNMS m(
       {TensorType_FLOAT32, {1, 6, 4}}, {TensorType_FLOAT32, {1, 6, 2}},
