@@ -260,6 +260,10 @@ TEST(QuantizedDivOpTest, QuantizedNoActivationUInt8) {
   QuantizedNoActivation<TensorType_UINT8, uint8_t>();
 }
 
+TEST(QuantizedDivOpTest, QuantizedNoActivationInt8) {
+  QuantizedNoActivation<TensorType_INT8, int8_t>();
+}
+
 template <TensorType tensor_type, typename integer_dtype>
 void QuantizedActivationRELU_N1_TO_1() {
   const float kQuantizedTolerance = GetTolerance(-1.0, 1.0);
@@ -288,6 +292,10 @@ TEST(QuantizedDivOpTest, QuantizedActivationRELU_N1_TO_1UInt8) {
   QuantizedActivationRELU_N1_TO_1<TensorType_UINT8, uint8_t>();
 }
 
+TEST(QuantizedDivOpTest, QuantizedActivationRELU_N1_TO_1Int8) {
+  QuantizedActivationRELU_N1_TO_1<TensorType_INT8, int8_t>();
+}
+
 template <TensorType tensor_type, typename integer_dtype>
 void QuantizedVariousInputShapes() {
   const float kQuantizedTolerance = GetTolerance(-3.0, 3.0);
@@ -315,6 +323,10 @@ TEST(QuantizedDivOpTest, QuantizedVariousInputShapesUInt8) {
   QuantizedVariousInputShapes<TensorType_UINT8, uint8_t>();
 }
 
+TEST(QuantizedDivOpTest, QuantizedVariousInputShapesInt8) {
+  QuantizedVariousInputShapes<TensorType_INT8, int8_t>();
+}
+
 template <TensorType tensor_type, typename integer_dtype>
 void QuantizedWithBroadcast() {
   const float kQuantizedTolerance = GetTolerance(-3.0, 3.0);
@@ -340,5 +352,9 @@ TEST(QuantizedDivOpTest, QuantizedWithBroadcastUInt8) {
   QuantizedWithBroadcast<TensorType_UINT8, uint8_t>();
 }
 
+TEST(QuantizedDivOpTest, QuantizedWithBroadcastInt8) {
+  QuantizedWithBroadcast<TensorType_INT8, int8_t>();
+}
+
 }  // namespace
 }  // namespace tflite
