@@ -70,7 +70,7 @@ TEST(GatherNdOpTest, ElementIndexingIntoMatrix) {
   m.SetPositions<int32_t>({0, 0, 1, 1});
   ASSERT_EQ(m.Invoke(), kTfLiteOk);
 
-  EXPECT_THAT(m.GetOutput<float>(), ElementsAreArray({1.1, 2.2}));
+  EXPECT_THAT(m.GetOutput<float>(), Pointwise(FloatingPointEq(), {1.1, 2.2}));
 }
 
 TEST(GatherNdOpTest, ErrorOnOutOfBoundsTooLarge) {
@@ -95,7 +95,8 @@ TEST(GatherNdOpTest, SliceIndexingIntoMatrix) {
   m.SetPositions<int32_t>({1, 0});
   ASSERT_EQ(m.Invoke(), kTfLiteOk);
 
-  EXPECT_THAT(m.GetOutput<float>(), ElementsAreArray({2.1, 2.2, 1.1, 1.2}));
+  EXPECT_THAT(m.GetOutput<float>(),
+              Pointwise(FloatingPointEq(), {2.1, 2.2, 1.1, 1.2}));
 }
 
 TEST(GatherNdOpTest, BatchedIndexingIntoMatrix1) {
@@ -105,7 +106,8 @@ TEST(GatherNdOpTest, BatchedIndexingIntoMatrix1) {
   m.SetPositions<int32_t>({1, 0});
   ASSERT_EQ(m.Invoke(), kTfLiteOk);
 
-  EXPECT_THAT(m.GetOutput<float>(), ElementsAreArray({2.1, 2.2, 1.1, 1.2}));
+  EXPECT_THAT(m.GetOutput<float>(),
+              Pointwise(FloatingPointEq(), {2.1, 2.2, 1.1, 1.2}));
 }
 
 TEST(GatherNdOpTest, BatchedIndexingIntoMatrix2) {
@@ -115,7 +117,7 @@ TEST(GatherNdOpTest, BatchedIndexingIntoMatrix2) {
   m.SetPositions<int32_t>({0, 0, 1, 1});
   ASSERT_EQ(m.Invoke(), kTfLiteOk);
 
-  EXPECT_THAT(m.GetOutput<float>(), ElementsAreArray({1.1, 2.2}));
+  EXPECT_THAT(m.GetOutput<float>(), Pointwise(FloatingPointEq(), {1.1, 2.2}));
 }
 
 TEST(GatherNdOpTest, DuplicateIndexingIntoMatrix) {
@@ -124,7 +126,7 @@ TEST(GatherNdOpTest, DuplicateIndexingIntoMatrix) {
   m.SetPositions<int32_t>({0, 0, 0, 0});
   ASSERT_EQ(m.Invoke(), kTfLiteOk);
 
-  EXPECT_THAT(m.GetOutput<float>(), ElementsAreArray({1.1, 1.1}));
+  EXPECT_THAT(m.GetOutput<float>(), Pointwise(FloatingPointEq(), {1.1, 1.1}));
 }
 
 TEST(GatherNdOpTest, ElementIndexingIntoRank3Tensor) {
@@ -136,7 +138,7 @@ TEST(GatherNdOpTest, ElementIndexingIntoRank3Tensor) {
   m.SetPositions<int32_t>({0, 0, 1, 1, 1, 0});
   ASSERT_EQ(m.Invoke(), kTfLiteOk);
 
-  EXPECT_THAT(m.GetOutput<float>(), ElementsAreArray({-1.2, -4.1}));
+  EXPECT_THAT(m.GetOutput<float>(), Pointwise(FloatingPointEq(), {-1.2, -4.1}));
 }
 
 TEST(GatherNdOpTest, SliceIndexingIntoRank3Tensor) {
@@ -149,8 +151,8 @@ TEST(GatherNdOpTest, SliceIndexingIntoRank3Tensor) {
   ASSERT_EQ(m.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(m.GetOutput<float>(),
-              ElementsAreArray({1.1, -1.2, 1.3, -2.1, 2.2, 2.3, 5.1, -5.2, 5.3,
-                                6.1, -6.2, 6.3}));
+              Pointwise(FloatingPointEq(), {1.1, -1.2, 1.3, -2.1, 2.2, 2.3, 5.1,
+                                            -5.2, 5.3, 6.1, -6.2, 6.3}));
 }
 
 TEST(GatherNdOpTest, BatchedIndexingIntoRank3Tensor1) {
@@ -162,7 +164,7 @@ TEST(GatherNdOpTest, BatchedIndexingIntoRank3Tensor1) {
   m.SetPositions<int32_t>({0, 0, 1, 1, 1, 0});
   ASSERT_EQ(m.Invoke(), kTfLiteOk);
 
-  EXPECT_THAT(m.GetOutput<float>(), ElementsAreArray({-1.2, -4.1}));
+  EXPECT_THAT(m.GetOutput<float>(), Pointwise(FloatingPointEq(), {-1.2, -4.1}));
 }
 
 TEST(GatherNdOpTest, BatchedIndexingIntoRank3Tensor2) {
@@ -175,8 +177,8 @@ TEST(GatherNdOpTest, BatchedIndexingIntoRank3Tensor2) {
   ASSERT_EQ(m.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(m.GetOutput<float>(),
-              ElementsAreArray({3.1, 3.2, -3.3, -4.1, -4.2, 4.3, 1.1, -1.2, 1.3,
-                                -2.1, 2.2, 2.3}));
+              Pointwise(FloatingPointEq(), {3.1, 3.2, -3.3, -4.1, -4.2, 4.3,
+                                            1.1, -1.2, 1.3, -2.1, 2.2, 2.3}));
 }
 
 TEST(GatherNdOpTest, BatchedIndexingIntoRank3Tensor3) {
@@ -189,8 +191,8 @@ TEST(GatherNdOpTest, BatchedIndexingIntoRank3Tensor3) {
   ASSERT_EQ(m.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(m.GetOutput<float>(),
-              ElementsAreArray({-2.1, 2.2, 2.3, 3.1, 3.2, -3.3, 1.1, -1.2, 1.3,
-                                6.1, -6.2, 6.3}));
+              Pointwise(FloatingPointEq(), {-2.1, 2.2, 2.3, 3.1, 3.2, -3.3, 1.1,
+                                            -1.2, 1.3, 6.1, -6.2, 6.3}));
 }
 
 TEST(GatherNdOpTest, BatchedIndexingIntoRank3Tensor4) {
@@ -202,7 +204,8 @@ TEST(GatherNdOpTest, BatchedIndexingIntoRank3Tensor4) {
   m.SetPositions<int32_t>({0, 0, 1, 1, 0, 1, 1, 1, 2, 2, 1, 2});
   ASSERT_EQ(m.Invoke(), kTfLiteOk);
 
-  EXPECT_THAT(m.GetOutput<float>(), ElementsAreArray({-1.2, 3.2, 4.3, 6.3}));
+  EXPECT_THAT(m.GetOutput<float>(),
+              Pointwise(FloatingPointEq(), {-1.2, 3.2, 4.3, 6.3}));
 }
 
 TEST(GatherNdOpTest, DuplicateIndexingIntoRank3Tensor) {
@@ -215,7 +218,47 @@ TEST(GatherNdOpTest, DuplicateIndexingIntoRank3Tensor) {
   ASSERT_EQ(m.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(m.GetOutput<float>(),
-              ElementsAreArray({-2.1, 2.2, 2.3, -2.1, 2.2, 2.3}));
+              Pointwise(FloatingPointEq(), {-2.1, 2.2, 2.3, -2.1, 2.2, 2.3}));
+}
+
+TEST(GatherNdOpTest, BFloat16Int32) {
+  GatherNdOpModel m({TensorType_BFLOAT16, {3, 2, 3}},
+                    {TensorType_INT32, {2, 2}});
+  m.SetInput<Eigen::bfloat16>(
+      {Eigen::bfloat16(1.1), Eigen::bfloat16(-1.2), Eigen::bfloat16(1.3),
+       Eigen::bfloat16(-2.1), Eigen::bfloat16(2.2), Eigen::bfloat16(2.3),  //
+       Eigen::bfloat16(3.1), Eigen::bfloat16(3.2), Eigen::bfloat16(-3.3),
+       Eigen::bfloat16(-4.1), Eigen::bfloat16(-4.2), Eigen::bfloat16(4.3),  //
+       Eigen::bfloat16(5.1), Eigen::bfloat16(-5.2), Eigen::bfloat16(5.3),
+       Eigen::bfloat16(6.1), Eigen::bfloat16(-6.2), Eigen::bfloat16(6.3)});
+  m.SetPositions<int32_t>({0, 1, 1, 0});
+  ASSERT_EQ(m.Invoke(), kTfLiteOk);
+
+  EXPECT_THAT(m.GetOutput<Eigen::bfloat16>(),
+              Pointwise(FloatingPointEq(),
+                        {Eigen::bfloat16(-2.1), Eigen::bfloat16(2.2),
+                         Eigen::bfloat16(2.3), Eigen::bfloat16(3.1),
+                         Eigen::bfloat16(3.2), Eigen::bfloat16(-3.3)}));
+}
+
+TEST(GatherNdOpTest, Float16Int32) {
+  GatherNdOpModel m({TensorType_FLOAT16, {3, 2, 3}},
+                    {TensorType_INT32, {2, 2}});
+  m.SetInput<Eigen::half>(
+      {Eigen::half(1.1), Eigen::half(-1.2), Eigen::half(1.3), Eigen::half(-2.1),
+       Eigen::half(2.2), Eigen::half(2.3),  //
+       Eigen::half(3.1), Eigen::half(3.2), Eigen::half(-3.3), Eigen::half(-4.1),
+       Eigen::half(-4.2), Eigen::half(4.3),  //
+       Eigen::half(5.1), Eigen::half(-5.2), Eigen::half(5.3), Eigen::half(6.1),
+       Eigen::half(-6.2), Eigen::half(6.3)});
+  m.SetPositions<int32_t>({0, 1, 1, 0});
+  ASSERT_EQ(m.Invoke(), kTfLiteOk);
+
+  EXPECT_THAT(
+      m.GetOutput<Eigen::half>(),
+      Pointwise(FloatingPointEq(),
+                {Eigen::half(-2.1), Eigen::half(2.2), Eigen::half(2.3),
+                 Eigen::half(3.1), Eigen::half(3.2), Eigen::half(-3.3)}));
 }
 
 TEST(GatherNdOpTest, Float32Int32) {
@@ -228,7 +271,47 @@ TEST(GatherNdOpTest, Float32Int32) {
   ASSERT_EQ(m.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(m.GetOutput<float>(),
-              ElementsAreArray({-2.1, 2.2, 2.3, 3.1, 3.2, -3.3}));
+              Pointwise(FloatingPointEq(), {-2.1, 2.2, 2.3, 3.1, 3.2, -3.3}));
+}
+
+TEST(GatherNdOpTest, BFloat16Int64) {
+  GatherNdOpModel m({TensorType_BFLOAT16, {3, 2, 3}},
+                    {TensorType_INT64, {2, 2}});
+  m.SetInput<Eigen::bfloat16>(
+      {Eigen::bfloat16(1.1), Eigen::bfloat16(-1.2), Eigen::bfloat16(1.3),
+       Eigen::bfloat16(-2.1), Eigen::bfloat16(2.2), Eigen::bfloat16(2.3),  //
+       Eigen::bfloat16(3.1), Eigen::bfloat16(3.2), Eigen::bfloat16(-3.3),
+       Eigen::bfloat16(-4.1), Eigen::bfloat16(-4.2), Eigen::bfloat16(4.3),  //
+       Eigen::bfloat16(5.1), Eigen::bfloat16(-5.2), Eigen::bfloat16(5.3),
+       Eigen::bfloat16(6.1), Eigen::bfloat16(-6.2), Eigen::bfloat16(6.3)});
+  m.SetPositions<int64_t>({0LL, 1LL, 1LL, 0LL});
+  ASSERT_EQ(m.Invoke(), kTfLiteOk);
+
+  EXPECT_THAT(m.GetOutput<Eigen::bfloat16>(),
+              Pointwise(FloatingPointEq(),
+                        {Eigen::bfloat16(-2.1), Eigen::bfloat16(2.2),
+                         Eigen::bfloat16(2.3), Eigen::bfloat16(3.1),
+                         Eigen::bfloat16(3.2), Eigen::bfloat16(-3.3)}));
+}
+
+TEST(GatherNdOpTest, Float16Int64) {
+  GatherNdOpModel m({TensorType_FLOAT16, {3, 2, 3}},
+                    {TensorType_INT64, {2, 2}});
+  m.SetInput<Eigen::half>(
+      {Eigen::half(1.1), Eigen::half(-1.2), Eigen::half(1.3), Eigen::half(-2.1),
+       Eigen::half(2.2), Eigen::half(2.3),  //
+       Eigen::half(3.1), Eigen::half(3.2), Eigen::half(-3.3), Eigen::half(-4.1),
+       Eigen::half(-4.2), Eigen::half(4.3),  //
+       Eigen::half(5.1), Eigen::half(-5.2), Eigen::half(5.3), Eigen::half(6.1),
+       Eigen::half(-6.2), Eigen::half(6.3)});
+  m.SetPositions<int64_t>({0LL, 1LL, 1LL, 0LL});
+  ASSERT_EQ(m.Invoke(), kTfLiteOk);
+
+  EXPECT_THAT(
+      m.GetOutput<Eigen::half>(),
+      Pointwise(FloatingPointEq(),
+                {Eigen::half(-2.1), Eigen::half(2.2), Eigen::half(2.3),
+                 Eigen::half(3.1), Eigen::half(3.2), Eigen::half(-3.3)}));
 }
 
 TEST(GatherNdOpTest, Float32Int64) {
@@ -241,7 +324,7 @@ TEST(GatherNdOpTest, Float32Int64) {
   ASSERT_EQ(m.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(m.GetOutput<float>(),
-              ElementsAreArray({-2.1, 2.2, 2.3, 3.1, 3.2, -3.3}));
+              Pointwise(FloatingPointEq(), {-2.1, 2.2, 2.3, 3.1, 3.2, -3.3}));
 }
 
 TEST(GatherNdOpTest, Int32Int32) {
@@ -356,6 +439,46 @@ TEST(GatherNdOpTest, Int64Int64) {
               ElementsAreArray({-2LL, 2LL, 2LL, 3LL, 3LL, -3LL}));
 }
 
+TEST(GatherNdOpTest, BFloat16Int16) {
+  GatherNdOpModel m({TensorType_BFLOAT16, {3, 2, 3}},
+                    {TensorType_INT16, {2, 2}});
+  m.SetInput<Eigen::bfloat16>(
+      {Eigen::bfloat16(1.1), Eigen::bfloat16(-1.2), Eigen::bfloat16(1.3),
+       Eigen::bfloat16(-2.1), Eigen::bfloat16(2.2), Eigen::bfloat16(2.3),  //
+       Eigen::bfloat16(3.1), Eigen::bfloat16(3.2), Eigen::bfloat16(-3.3),
+       Eigen::bfloat16(-4.1), Eigen::bfloat16(-4.2), Eigen::bfloat16(4.3),  //
+       Eigen::bfloat16(5.1), Eigen::bfloat16(-5.2), Eigen::bfloat16(5.3),
+       Eigen::bfloat16(6.1), Eigen::bfloat16(-6.2), Eigen::bfloat16(6.3)});
+  m.SetPositions<int16_t>({0, 1, 1, 0});
+  ASSERT_EQ(m.Invoke(), kTfLiteOk);
+
+  EXPECT_THAT(m.GetOutput<Eigen::bfloat16>(),
+              Pointwise(FloatingPointEq(),
+                        {Eigen::bfloat16(-2.1), Eigen::bfloat16(2.2),
+                         Eigen::bfloat16(2.3), Eigen::bfloat16(3.1),
+                         Eigen::bfloat16(3.2), Eigen::bfloat16(-3.3)}));
+}
+
+TEST(GatherNdOpTest, Float16Int16) {
+  GatherNdOpModel m({TensorType_FLOAT16, {3, 2, 3}},
+                    {TensorType_INT16, {2, 2}});
+  m.SetInput<Eigen::half>(
+      {Eigen::half(1.1), Eigen::half(-1.2), Eigen::half(1.3), Eigen::half(-2.1),
+       Eigen::half(2.2), Eigen::half(2.3),  //
+       Eigen::half(3.1), Eigen::half(3.2), Eigen::half(-3.3), Eigen::half(-4.1),
+       Eigen::half(-4.2), Eigen::half(4.3),  //
+       Eigen::half(5.1), Eigen::half(-5.2), Eigen::half(5.3), Eigen::half(6.1),
+       Eigen::half(-6.2), Eigen::half(6.3)});
+  m.SetPositions<int16_t>({0, 1, 1, 0});
+  ASSERT_EQ(m.Invoke(), kTfLiteOk);
+
+  EXPECT_THAT(
+      m.GetOutput<Eigen::half>(),
+      Pointwise(FloatingPointEq(),
+                {Eigen::half(-2.1), Eigen::half(2.2), Eigen::half(2.3),
+                 Eigen::half(3.1), Eigen::half(3.2), Eigen::half(-3.3)}));
+}
+
 TEST(GatherNdOpTest, Float32Int16) {
   GatherNdOpModel m({TensorType_FLOAT32, {3, 2, 3}},
                     {TensorType_INT16, {2, 2}});
@@ -366,7 +489,7 @@ TEST(GatherNdOpTest, Float32Int16) {
   ASSERT_EQ(m.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(m.GetOutput<float>(),
-              ElementsAreArray({-2.1, 2.2, 2.3, 3.1, 3.2, -3.3}));
+              Pointwise(FloatingPointEq(), {-2.1, 2.2, 2.3, 3.1, 3.2, -3.3}));
 }
 
 TEST(GatherNdOpTest, StringInt32) {
