@@ -89,14 +89,15 @@ void SparseMatrixBatchVectorMultiplyAccumulate1x16(
     const int32_t* __restrict__ indices, int m_rows, int m_cols,
     const int8_t* __restrict__ vector, const int32_t* __restrict__ bias_vector,
     int n_batch, const int32_t input_offset, const int32_t output_multiplier,
-    const int32_t output_shift, const int32_t output_offset,
+    const int32_t output_shift, const int32_t* per_channel_scale,
+    const int32_t* per_channel_shift, const int32_t output_offset,
     const int32_t output_activation_min, const int32_t output_activation_max,
     int8_t* __restrict__ result) {
   NEON_OR_PORTABLE(SparseMatrixBatchVectorMultiplyAccumulate1x16, matrix,
                    segments, indices, m_rows, m_cols, vector, bias_vector,
                    n_batch, input_offset, output_multiplier, output_shift,
-                   output_offset, output_activation_min, output_activation_max,
-                   result);
+                   per_channel_scale, per_channel_shift, output_offset,
+                   output_activation_min, output_activation_max, result);
 }
 
 void SparseMatrixBatchVectorMultiplyAccumulate(
@@ -111,9 +112,10 @@ void SparseMatrixBatchVectorMultiplyAccumulate(
     const int8_t* __restrict__ matrix, const uint8_t* __restrict__ ledger,
     const int m_rows, const int m_cols, const int8_t* __restrict__ vectors,
     const float* __restrict__ scaling_factors, int n_batch,
-    float* __restrict__ result) {
+    float* __restrict__ result, const float* per_channel_scale) {
   SSE_OR_PORTABLE(SparseMatrixBatchVectorMultiplyAccumulate, matrix, ledger,
-                  m_rows, m_cols, vectors, scaling_factors, n_batch, result);
+                  m_rows, m_cols, vectors, scaling_factors, n_batch, result,
+                  per_channel_scale);
 }
 
 void MatrixBatchVectorMultiplyAccumulate(
