@@ -14,9 +14,10 @@ limitations under the License.
 ==============================================================================*/
 #include "tensorflow/lite/kernels/internal/utils/sparsity_format_converter.h"
 
-#include <gmock/gmock.h>
+#include <vector>
+
 #include <gtest/gtest.h>
-#include "tensorflow/lite/core/model.h"
+#include "tensorflow/lite/c/common.h"
 
 namespace tflite {
 namespace internal {
