@@ -58,6 +58,15 @@ struct PadContext {
       case kTfLiteInt32:
         SetResizingCategory<int32_t>(context);
         break;
+      case kTfLiteInt8:
+        SetResizingCategory<int8_t>(context);
+        break;
+      case kTfLiteInt16:
+        SetResizingCategory<int16_t>(context);
+        break;
+      case kTfLiteBool:
+        SetResizingCategory<bool>(context);
+        break;
       default:
         TF_LITE_KERNEL_LOG(context,
                            "Padding type %s is currently not supported by Pad.",
@@ -96,7 +105,9 @@ bool CheckPaddingOverflow(PadContext* op_context) {
           static_cast<int64_t>(std::numeric_limits<int32_t>::min());
       int64_t int32_max =
           static_cast<int64_t>(std::numeric_limits<int32_t>::max());
-      for (int idx = 0; idx < op_context->dims; ++idx) {
+      const int paddings_total =
+          GetTensorShape(op_context->paddings).FlatSize();
+      for (int idx = 0; idx < paddings_total; ++idx) {
         int64_t padding = paddings_data[idx];
         if (padding < int32_min || padding > int32_max) {
           return true;
@@ -115,8 +126,12 @@ TfLiteStatus ResizeOutputTensor(TfLiteContext* context,
                                 PadContext* op_context) {
   if (op_context->paddings->type == kTfLiteInt64) {
     TF_LITE_ENSURE(context, (std::is_same_v<PaddingIntegerType, int64_t>));
-  } else {
+  } else if (op_context->paddings->type == kTfLiteInt32) {
     TF_LITE_ENSURE(context, (std::is_same_v<PaddingIntegerType, int32_t>));
+  } else if (op_context->paddings->type == kTfLiteInt8) {
+    TF_LITE_ENSURE(context, (std::is_same_v<PaddingIntegerType, int8_t>));
+  } else {
+    TF_LITE_ENSURE(context, (std::is_same_v<PaddingIntegerType, int16_t>));
   }
   // Ensures the paddings array is dims x 2.
   TF_LITE_ENSURE_EQ(context, SizeOfDimension(op_context->paddings, 0),
@@ -159,6 +174,15 @@ TfLiteStatus ResizeOutputTensor(TfLiteContext* context,
     case kTfLiteInt32: {
       return ResizeOutputTensor<int32_t>(context, op_context);
     }
+    case kTfLiteInt8: {
+      return ResizeOutputTensor<int8_t>(context, op_context);
+    }
+    case kTfLiteInt16: {
+      return ResizeOutputTensor<int16_t>(context, op_context);
+    }
+    case kTfLiteBool: {
+      return ResizeOutputTensor<bool>(context, op_context);
+    }
     default:
       TF_LITE_KERNEL_LOG(context,
                          "Padding type %s is currently not supported by Pad.",
@@ -174,10 +198,15 @@ tflite::PadParams GetPadParams(TfLiteContext* context,
                                const PadContext& op_context) {
   tflite::PadParams op_params;
   if (!(op_context.paddings->type == kTfLiteInt64 &&
-        std::is_same_v<PaddingIntegerType,
-                       int64_t>)&&!(op_context.paddings->type == kTfLiteInt32 &&
-                                    std::is_same_v<PaddingIntegerType,
-                                                   int32_t>)) {
+        std::is_same_v<PaddingIntegerType, int64_t>) &&
+      !(op_context.paddings->type == kTfLiteInt32 &&
+        std::is_same_v<PaddingIntegerType, int32_t>) &&
+      !(op_context.paddings->type == kTfLiteInt8 &&
+        std::is_same_v<PaddingIntegerType, int8_t>) &&
+      !(op_context.paddings->type == kTfLiteInt16 &&
+        std::is_same_v<PaddingIntegerType, int16_t>) &&
+      !(op_context.paddings->type == kTfLiteBool &&
+        std::is_same_v<PaddingIntegerType, bool>)) {
     TF_LITE_KERNEL_LOG(context, "Padding type %s doesn't match typename.",
                        TfLiteTypeGetName(op_context.paddings->type));
     return op_params;
@@ -202,7 +231,15 @@ tflite::PadParams GetPadParams(TfLiteContext* context,
     }
     case kTfLiteInt32: {
       return GetPadParams<int32_t>(context, op_context);
-      break;
+    }
+    case kTfLiteInt8: {
+      return GetPadParams<int8_t>(context, op_context);
+    }
+    case kTfLiteInt16: {
+      return GetPadParams<int16_t>(context, op_context);
+    }
+    case kTfLiteBool: {
+      return GetPadParams<bool>(context, op_context);
     }
     default:
       TF_LITE_KERNEL_LOG(context,
@@ -217,6 +254,7 @@ TfLiteStatus Prepare(TfLiteContext* context, TfLiteNode* node) {
   TF_LITE_ENSURE_EQ(context, NumOutputs(node), 1);
 
   PadContext op_context(context, node);
+  TF_LITE_ENSURE(context, op_context.output != nullptr);
   if (IsConstantTensor(op_context.paddings)) {
     TF_LITE_ENSURE_MSG(context, !CheckPaddingOverflow(&op_context),
                        "INT64 padding overflow. Only support value between "
@@ -331,14 +369,76 @@ TfLiteStatus Eval(TfLiteContext* context, TfLiteNode* node) {
         }
       }
     } break;
+    case kTfLiteFloat16: {
+      Eigen::half pad_value =
+          op_context.constant_values == nullptr
+              ? static_cast<Eigen::half>(0.f)
+              : *GetTensorData<Eigen::half>(op_context.constant_values);
+      if (kernel_type == kReference) {
+        if (op_context.resizing_category == ResizingCategory::kImageStyle) {
+          TF_LITE_PAD(reference_ops, PadImageStyle, Eigen::half, pad_value);
+        } else {
+          TF_LITE_PAD(reference_ops, Pad, Eigen::half, pad_value);
+        }
+      } else if (kernel_type == kGenericOptimized) {
+        if (op_context.resizing_category == ResizingCategory::kImageStyle) {
+          TF_LITE_PAD(optimized_ops, PadImageStyle, Eigen::half, pad_value);
+        } else {
+          TF_LITE_PAD(optimized_ops, Pad, Eigen::half, pad_value);
+        }
+      }
+    } break;
+    case kTfLiteBFloat16: {
+      Eigen::bfloat16 pad_value =
+          op_context.constant_values == nullptr
+              ? static_cast<Eigen::bfloat16>(0.f)
+              : *GetTensorData<Eigen::bfloat16>(op_context.constant_values);
+      if (kernel_type == kReference) {
+        if (op_context.resizing_category == ResizingCategory::kImageStyle) {
+          TF_LITE_PAD(reference_ops, PadImageStyle, Eigen::bfloat16, pad_value);
+        } else {
+          TF_LITE_PAD(reference_ops, Pad, Eigen::bfloat16, pad_value);
+        }
+      } else if (kernel_type == kGenericOptimized) {
+        if (op_context.resizing_category == ResizingCategory::kImageStyle) {
+          TF_LITE_PAD(optimized_ops, PadImageStyle, Eigen::bfloat16, pad_value);
+        } else {
+          TF_LITE_PAD(optimized_ops, Pad, Eigen::bfloat16, pad_value);
+        }
+      }
+    } break;
     case kTfLiteUInt8: {
       EvalInt<uint8_t>(context, op_context, op_params);
     } break;
     case kTfLiteInt8: {
-      EvalInt<int8_t>(context, op_context, op_params);
+      if (op_context.input->quantization.type != kTfLiteNoQuantization) {
+        EvalInt<int8_t>(context, op_context, op_params);
+      } else {
+        int8_t pad_value =
+            op_context.constant_values == nullptr
+                ? 0
+                : *GetTensorData<int8_t>(op_context.constant_values);
+        if (kernel_type == kReference) {
+          TF_LITE_PAD(reference_ops, Pad, int8_t, pad_value);
+        } else if (kernel_type == kGenericOptimized) {
+          TF_LITE_PAD(optimized_ops, Pad, int8_t, pad_value);
+        }
+      }
     } break;
     case kTfLiteInt16: {
-      EvalInt<int16_t>(context, op_context, op_params);
+      if (op_context.input->quantization.type != kTfLiteNoQuantization) {
+        EvalInt<int16_t>(context, op_context, op_params);
+      } else {
+        int16_t pad_value =
+            op_context.constant_values == nullptr
+                ? 0
+                : *GetTensorData<int16_t>(op_context.constant_values);
+        if (kernel_type == kReference) {
+          TF_LITE_PAD(reference_ops, Pad, int16_t, pad_value);
+        } else if (kernel_type == kGenericOptimized) {
+          TF_LITE_PAD(optimized_ops, Pad, int16_t, pad_value);
+        }
+      }
     } break;
     case kTfLiteInt32: {
       int32_t pad_value =
@@ -362,6 +462,16 @@ TfLiteStatus Eval(TfLiteContext* context, TfLiteNode* node) {
         TF_LITE_PAD(optimized_ops, Pad, int64_t, pad_value);
       }
     } break;
+    case kTfLiteBool: {
+      bool pad_value = op_context.constant_values == nullptr
+                           ? false
+                           : *GetTensorData<bool>(op_context.constant_values);
+      if (kernel_type == kReference) {
+        TF_LITE_PAD(reference_ops, Pad, bool, pad_value);
+      } else if (kernel_type == kGenericOptimized) {
+        TF_LITE_PAD(optimized_ops, Pad, bool, pad_value);
+      }
+    } break;
     default:
       TF_LITE_KERNEL_LOG(context, "Type %s is currently not supported by Pad.",
                          TfLiteTypeGetName(op_context.input->type));
