@@ -207,6 +207,10 @@ TEST_F(PadOpTest, Int32PaddingTooFewDimensions) { TooFewDimensions<int32_t>(); }
 
 TEST_F(PadOpTest, Int64PaddingTooFewDimensions) { TooFewDimensions<int64_t>(); }
 
+TEST_F(PadOpTest, Int8PaddingTooFewDimensions) { TooFewDimensions<int8_t>(); }
+
+TEST_F(PadOpTest, Int16PaddingTooFewDimensions) { TooFewDimensions<int16_t>(); }
+
 template <typename padding_integer_type>
 void UnequalDimensions() {
   EXPECT_DEATH(PadOpConstModel<padding_integer_type>(
@@ -223,6 +227,12 @@ TEST_F(PadOpTest, Int64PaddingUnequalDimensions) {
   UnequalDimensions<int64_t>();
 }
 
+TEST_F(PadOpTest, Int8PaddingUnequalDimensions) { UnequalDimensions<int8_t>(); }
+
+TEST_F(PadOpTest, Int16PaddingUnequalDimensions) {
+  UnequalDimensions<int16_t>();
+}
+
 template <typename padding_integer_type>
 void InvalidPadValue() {
   EXPECT_DEATH(PadOpConstModel<int32_t>({TensorType_FLOAT32, {1, 1, 2, 1}},
@@ -235,6 +245,10 @@ TEST_F(PadOpTest, Int32PaddingInvalidPadValue) { InvalidPadValue<int32_t>(); }
 
 TEST_F(PadOpTest, Int64PaddingInvalidPadValue) { InvalidPadValue<int64_t>(); }
 
+TEST_F(PadOpTest, Int8PaddingInvalidPadValue) { InvalidPadValue<int8_t>(); }
+
+TEST_F(PadOpTest, Int16PaddingInvalidPadValue) { InvalidPadValue<int16_t>(); }
+
 TEST_F(PadOpTest, Int64PaddingOverflow) {
   EXPECT_DEATH(PadOpConstModel<int64_t>(
                    {TensorType_FLOAT32, {1, 1, 2, 1}}, {4, 2},
@@ -242,6 +256,12 @@ TEST_F(PadOpTest, Int64PaddingOverflow) {
                    {TensorType_FLOAT32}),
                "INT64 padding overflow. Only support value between INT32_MIN "
                "and INT32_MAX.");
+  EXPECT_DEATH(PadOpConstModel<int64_t>(
+                   {TensorType_FLOAT32, {1, 1, 2, 1}}, {4, 2},
+                   {0, 0, 1, -1, 2, -1, std::numeric_limits<int64_t>::max(), 0},
+                   {TensorType_FLOAT32}),
+               "INT64 padding overflow. Only support value between INT32_MIN "
+               "and INT32_MAX.");
 }
 #endif
 
@@ -264,6 +284,10 @@ TEST_F(PadOpTest, Int32PaddingSimpleConstTest) { SimpleConstTest<int32_t>(); }
 
 TEST_F(PadOpTest, Int64PaddingSimpleConstTest) { SimpleConstTest<int64_t>(); }
 
+TEST_F(PadOpTest, Int8PaddingSimpleConstTest) { SimpleConstTest<int8_t>(); }
+
+TEST_F(PadOpTest, Int16PaddingSimpleConstTest) { SimpleConstTest<int16_t>(); }
+
 template <typename padding_integer_type>
 void SimpleConstImageStyleTest() {
   // Padding is represented as four 2-D lists representing above padding and
@@ -286,6 +310,14 @@ TEST_F(PadOpTest, Int64PaddingSimpleConstImageStyleTest) {
   SimpleConstImageStyleTest<int64_t>();
 }
 
+TEST_F(PadOpTest, Int8PaddingSimpleConstImageStyleTest) {
+  SimpleConstImageStyleTest<int8_t>();
+}
+
+TEST_F(PadOpTest, Int16PaddingSimpleConstImageStyleTest) {
+  SimpleConstImageStyleTest<int16_t>();
+}
+
 // Optimized versions may choose to handle zero-sized images differently.
 template <typename padding_integer_type>
 void ZeroHeightConstImageStyleTest() {
@@ -306,6 +338,14 @@ TEST_F(PadOpTest, Int64PaddingZeroHeightConstImageStyleTest) {
   ZeroHeightConstImageStyleTest<int64_t>();
 }
 
+TEST_F(PadOpTest, Int8PaddingZeroHeightConstImageStyleTest) {
+  ZeroHeightConstImageStyleTest<int8_t>();
+}
+
+TEST_F(PadOpTest, Int16PaddingZeroHeightConstImageStyleTest) {
+  ZeroHeightConstImageStyleTest<int16_t>();
+}
+
 // Optimized versions may choose to handle zero-sized images differently.
 template <typename padding_integer_type>
 void ZeroWidthConstImageStyleTest() {
@@ -326,6 +366,14 @@ TEST_F(PadOpTest, Int64PaddingZeroWidthConstImageStyleTest) {
   ZeroWidthConstImageStyleTest<int64_t>();
 }
 
+TEST_F(PadOpTest, Int8PaddingZeroWidthConstImageStyleTest) {
+  ZeroWidthConstImageStyleTest<int8_t>();
+}
+
+TEST_F(PadOpTest, Int16PaddingZeroWidthConstImageStyleTest) {
+  ZeroWidthConstImageStyleTest<int16_t>();
+}
+
 template <typename padding_integer_type>
 void SimpleConst1DTest() {
   PadOpConstModel<padding_integer_type> m({TensorType_FLOAT32, {2}}, {1, 2},
@@ -344,6 +392,12 @@ TEST_F(PadOpTest, Int64PaddingSimpleConst1DTest) {
   SimpleConst1DTest<int64_t>();
 }
 
+TEST_F(PadOpTest, Int8PaddingSimpleConst1DTest) { SimpleConst1DTest<int8_t>(); }
+
+TEST_F(PadOpTest, Int16PaddingSimpleConst1DTest) {
+  SimpleConst1DTest<int16_t>();
+}
+
 template <typename padding_integer_type>
 void SimpleConst1DDim0Test() {
   if (SingleOpModel::GetForceUseNnapi()) {
@@ -365,6 +419,14 @@ TEST_F(PadOpTest, Int64PaddingSimpleConst1DDim0Test) {
   SimpleConst1DDim0Test<int64_t>();
 }
 
+TEST_F(PadOpTest, Int8PaddingSimpleConst1DDim0Test) {
+  SimpleConst1DDim0Test<int8_t>();
+}
+
+TEST_F(PadOpTest, Int16PaddingSimpleConst1DDim0Test) {
+  SimpleConst1DDim0Test<int16_t>();
+}
+
 template <typename padding_integer_type>
 void SimpleDynamicTest() {
   PadOpDynamicModel<padding_integer_type> m({TensorType_FLOAT32, {1, 2, 2, 1}},
@@ -385,6 +447,12 @@ TEST_F(PadOpTest, Int64PaddingSimpleDynamicTest) {
   SimpleDynamicTest<int64_t>();
 }
 
+TEST_F(PadOpTest, Int8PaddingSimpleDynamicTest) { SimpleDynamicTest<int8_t>(); }
+
+TEST_F(PadOpTest, Int16PaddingSimpleDynamicTest) {
+  SimpleDynamicTest<int16_t>();
+}
+
 template <typename padding_integer_type>
 void DynamicUnequalDimensions() {
   if (SingleOpModel::GetForceUseNnapi()) {
@@ -406,6 +474,14 @@ TEST_F(PadOpTest, Int64PaddingDynamicUnequalDimensions) {
   DynamicUnequalDimensions<int64_t>();
 }
 
+TEST_F(PadOpTest, Int8PaddingDynamicUnequalDimensions) {
+  DynamicUnequalDimensions<int8_t>();
+}
+
+TEST_F(PadOpTest, Int16PaddingDynamicUnequalDimensions) {
+  DynamicUnequalDimensions<int16_t>();
+}
+
 template <typename padding_integer_type>
 void AdvancedConstTestV2() {
   PadOpConstModel<padding_integer_type> m({TensorType_FLOAT32, {1, 2, 3, 1}},
@@ -429,6 +505,14 @@ TEST_F(PadOpTest, Int64PaddingAdvancedConstTest) {
   AdvancedConstTestV2<int64_t>();
 }
 
+TEST_F(PadOpTest, Int8PaddingAdvancedConstTest) {
+  AdvancedConstTestV2<int8_t>();
+}
+
+TEST_F(PadOpTest, Int16PaddingAdvancedConstTest) {
+  AdvancedConstTestV2<int16_t>();
+}
+
 template <typename padding_integer_type>
 void AdvancedConstImageStyleTest() {
   PadOpConstModel<int32_t> m({TensorType_FLOAT32, {1, 2, 3, 1}}, {4, 2},
@@ -449,6 +533,14 @@ TEST_F(PadOpTest, Int64PaddingAdvancedConstImageStyleTest) {
   AdvancedConstImageStyleTest<int64_t>();
 }
 
+TEST_F(PadOpTest, Int8PaddingAdvancedConstImageStyleTest) {
+  AdvancedConstImageStyleTest<int8_t>();
+}
+
+TEST_F(PadOpTest, Int16PaddingAdvancedConstImageStyleTest) {
+  AdvancedConstImageStyleTest<int16_t>();
+}
+
 template <typename padding_integer_type>
 void AdvancedDynamicTest() {
   PadOpDynamicModel<padding_integer_type> m({TensorType_FLOAT32, {1, 2, 3, 1}},
@@ -470,6 +562,14 @@ TEST_F(PadOpTest, Int64PaddingAdvancedDynamicTest) {
   AdvancedDynamicTest<int64_t>();
 }
 
+TEST_F(PadOpTest, Int8PaddingAdvancedDynamicTest) {
+  AdvancedDynamicTest<int8_t>();
+}
+
+TEST_F(PadOpTest, Int16PaddingAdvancedDynamicTest) {
+  AdvancedDynamicTest<int16_t>();
+}
+
 std::vector<Matcher<float>> DequantizedArrayNear(
     const std::vector<float>& values, const float min, const float max) {
   const float quantization_tolerance = (max - min) / 255.0;
@@ -633,6 +733,14 @@ TEST_F(PadV2OpTest, Int64PaddingTooManyDimensions) {
   TooManyDimensions<int64_t>();
 }
 
+TEST_F(PadV2OpTest, Int8PaddingTooManyDimensions) {
+  TooManyDimensions<int8_t>();
+}
+
+TEST_F(PadV2OpTest, Int16PaddingTooManyDimensions) {
+  TooManyDimensions<int16_t>();
+}
+
 template <typename padding_integer_type>
 void UnequalDimensionsV2() {
   typedef PadV2OpConstModel<float, padding_integer_type> f;
@@ -649,6 +757,14 @@ TEST_F(PadV2OpTest, Int64PaddingUnequalDimensions) {
   UnequalDimensionsV2<int64_t>();
 }
 
+TEST_F(PadV2OpTest, Int8PaddingUnequalDimensions) {
+  UnequalDimensionsV2<int8_t>();
+}
+
+TEST_F(PadV2OpTest, Int16PaddingUnequalDimensions) {
+  UnequalDimensionsV2<int16_t>();
+}
+
 template <typename padding_integer_type>
 void InvalidPadValueV2() {
   typedef PadV2OpConstModel<float, padding_integer_type> f;
@@ -665,6 +781,12 @@ TEST_F(PadV2OpTest, Int64PaddingInvalidPadValue) {
   InvalidPadValueV2<int64_t>();
 }
 
+TEST_F(PadV2OpTest, Int8PaddingInvalidPadValue) { InvalidPadValueV2<int8_t>(); }
+
+TEST_F(PadV2OpTest, Int16PaddingInvalidPadValue) {
+  InvalidPadValueV2<int16_t>();
+}
+
 TEST_F(PadV2OpTest, Int64PaddingOverflow) {
   EXPECT_DEATH(PadOpConstModel<int64_t>(
                    {TensorType_FLOAT32, {1, 1, 2, 1}}, {4, 2},
@@ -705,6 +827,14 @@ TEST_F(PadV2OpTest, Int64PaddingSimpleConstTestUint8) {
   SimpleConstTestUint8<int64_t>();
 }
 
+TEST_F(PadV2OpTest, Int8PaddingSimpleConstTestUint8) {
+  SimpleConstTestUint8<int8_t>();
+}
+
+TEST_F(PadV2OpTest, Int16PaddingSimpleConstTestUint8) {
+  SimpleConstTestUint8<int16_t>();
+}
+
 template <typename padding_integer_type>
 void SimpleConstTestInt8() {
   // Padding is represented as four 2-D lists representing above padding and
@@ -727,6 +857,14 @@ TEST_F(PadV2OpTest, Int64PaddingSimpleConstTestInt8) {
   SimpleConstTestInt8<int64_t>();
 }
 
+TEST_F(PadV2OpTest, Int8PaddingSimpleConstTestInt8) {
+  SimpleConstTestInt8<int8_t>();
+}
+
+TEST_F(PadV2OpTest, Int16PaddingSimpleConstTestInt8) {
+  SimpleConstTestInt8<int16_t>();
+}
+
 template <typename padding_integer_type>
 void SimpleConstFloat32ValuedTestUint8() {
   // Padding is represented as four 2-D lists representing above padding and
@@ -749,6 +887,14 @@ TEST_F(PadV2OpTest, Int64PaddingSimpleConstFloat32ValuedTestUint8) {
   SimpleConstFloat32ValuedTestUint8<int64_t>();
 }
 
+TEST_F(PadV2OpTest, Int8PaddingSimpleConstFloat32ValuedTestUint8) {
+  SimpleConstFloat32ValuedTestUint8<int8_t>();
+}
+
+TEST_F(PadV2OpTest, Int16PaddingSimpleConstFloat32ValuedTestUint8) {
+  SimpleConstFloat32ValuedTestUint8<int16_t>();
+}
+
 template <typename padding_integer_type>
 void SimpleConstFloat32ValuedTestInt8() {
   // Padding is represented as four 2-D lists representing above padding and
@@ -771,6 +917,108 @@ TEST_F(PadV2OpTest, Int64PaddingSimpleConstFloat32ValuedTestInt8) {
   SimpleConstFloat32ValuedTestInt8<int64_t>();
 }
 
+TEST_F(PadV2OpTest, Int8PaddingSimpleConstFloat32ValuedTestInt8) {
+  SimpleConstFloat32ValuedTestInt8<int8_t>();
+}
+
+TEST_F(PadV2OpTest, Int16PaddingSimpleConstFloat32ValuedTestInt8) {
+  SimpleConstFloat32ValuedTestInt8<int16_t>();
+}
+
+template <typename padding_integer_type>
+void SimpleConstFloat16ValuedTest() {
+  PadV2OpConstModel<Eigen::half, padding_integer_type> m(
+      {TensorType_FLOAT16, {1, 2, 2, 1}}, {4, 2}, {0, 0, 1, 1, 1, 1, 0, 0},
+      Eigen::half{4.0f}, {TensorType_FLOAT16});
+  m.SetInput({Eigen::half{1.5f}, Eigen::half{2.5f}, Eigen::half{3.5f},
+              Eigen::half{4.5}});
+  ASSERT_EQ(m.Invoke(), kTfLiteOk);
+  EXPECT_THAT(
+      m.GetOutput(),
+      ElementsAreArray(ArrayFloatNear(
+          {Eigen::half{4}, Eigen::half{4}, Eigen::half{4}, Eigen::half{4},
+           Eigen::half{4}, Eigen::half{1.5}, Eigen::half{2.5}, Eigen::half{4},
+           Eigen::half{4}, Eigen::half{3.5}, Eigen::half{4.5}, Eigen::half{4},
+           Eigen::half{4}, Eigen::half{4}, Eigen::half{4}, Eigen::half{4}})));
+  EXPECT_THAT(m.GetOutputShape(), ElementsAreArray({1, 4, 4, 1}));
+}
+
+TEST_F(PadV2OpTest, Int32PaddingSimpleConstFloat16) {
+  SimpleConstFloat16ValuedTest<int32_t>();
+}
+
+TEST_F(PadV2OpTest, Int64PaddingSimpleConstFloat16) {
+  SimpleConstFloat16ValuedTest<int64_t>();
+}
+
+TEST_F(PadV2OpTest, Int8PaddingSimpleConstFloat16) {
+  SimpleConstFloat16ValuedTest<int8_t>();
+}
+
+TEST_F(PadV2OpTest, Int16PaddingSimpleConstFloat16) {
+  SimpleConstFloat16ValuedTest<int16_t>();
+}
+
+template <typename padding_integer_type>
+void SimpleConstBFloat16ValuedTest() {
+  PadV2OpConstModel<Eigen::bfloat16, padding_integer_type> m(
+      {TensorType_BFLOAT16, {1, 2, 2, 1}}, {4, 2}, {0, 0, 1, 1, 1, 1, 0, 0},
+      Eigen::bfloat16{6.0f}, {TensorType_BFLOAT16});
+  m.SetInput({Eigen::bfloat16{1.0f}, Eigen::bfloat16{2.0f},
+              Eigen::bfloat16{3.0f}, Eigen::bfloat16{4.0}});
+  ASSERT_EQ(m.Invoke(), kTfLiteOk);
+  EXPECT_THAT(m.GetOutput(), ElementsAreArray({6, 6, 6, 6, 6, 1, 2, 6, 6, 3, 4,
+                                               6, 6, 6, 6, 6}));
+  EXPECT_THAT(m.GetOutputShape(), ElementsAreArray({1, 4, 4, 1}));
+}
+
+TEST_F(PadV2OpTest, Int32PaddingSimpleConstBFloat16) {
+  SimpleConstBFloat16ValuedTest<int32_t>();
+}
+
+TEST_F(PadV2OpTest, Int64PaddingSimpleConstBFloat16) {
+  SimpleConstBFloat16ValuedTest<int64_t>();
+}
+
+TEST_F(PadV2OpTest, Int8PaddingSimpleConstBFloat16) {
+  SimpleConstBFloat16ValuedTest<int8_t>();
+}
+
+TEST_F(PadV2OpTest, Int16PaddingSimpleConstBFloat16) {
+  SimpleConstBFloat16ValuedTest<int16_t>();
+}
+
+template <typename padding_integer_type>
+void SimpleConstBoolValuedTest() {
+  PadV2OpConstModel<bool, padding_integer_type> m(
+      {TensorType_BOOL, {1, 2, 2, 1}}, {4, 2},
+      {false, false, true, true, true, true, false, false}, true,
+      {TensorType_BOOL});
+  m.SetInput({true, true, false, false});
+  ASSERT_EQ(m.Invoke(), kTfLiteOk);
+  EXPECT_THAT(
+      m.GetOutput(),
+      ElementsAreArray({true, true, true, true, true, true, true, true, true,
+                        false, false, true, true, true, true, true}));
+  EXPECT_THAT(m.GetOutputShape(), ElementsAreArray({1, 4, 4, 1}));
+}
+
+TEST_F(PadV2OpTest, Int32PaddingSimpleConstBool) {
+  SimpleConstBoolValuedTest<int32_t>();
+}
+
+TEST_F(PadV2OpTest, Int64PaddingSimpleConstBool) {
+  SimpleConstBoolValuedTest<int64_t>();
+}
+
+TEST_F(PadV2OpTest, Int8PaddingSimpleConstBool) {
+  SimpleConstBoolValuedTest<int8_t>();
+}
+
+TEST_F(PadV2OpTest, Int16PaddingSimpleConstBool) {
+  SimpleConstBoolValuedTest<int16_t>();
+}
+
 template <typename padding_integer_type>
 void Simple4DConstFloat32ValuedTest() {
   // Padding is represented as four 2-D lists representing above padding and
@@ -792,6 +1040,73 @@ TEST_F(PadV2OpTest, Int64PaddingSimple4DConstFloat32ValuedTest) {
   Simple4DConstFloat32ValuedTest<int64_t>();
 }
 
+TEST_F(PadV2OpTest, Int8PaddingSimple4DConstFloat32ValuedTest) {
+  Simple4DConstFloat32ValuedTest<int8_t>();
+}
+
+TEST_F(PadV2OpTest, Int16PaddingSimple4DConstFloat32ValuedTest) {
+  Simple4DConstFloat32ValuedTest<int16_t>();
+}
+
+template <typename padding_integer_type>
+void Simple4DConstFloat16ValuedTest() {
+  PadV2OpConstModel<Eigen::half, padding_integer_type> m(
+      {TensorType_FLOAT16, {1, 1, 2, 1}}, {4, 2}, {0, 1, 0, 0, 0, 0, 0, 1},
+      Eigen::half{7.0}, {TensorType_FLOAT16});
+  m.SetInput({Eigen::half{3.0f}, Eigen::half{6.0f}});
+  ASSERT_EQ(m.Invoke(), kTfLiteOk);
+  EXPECT_THAT(m.GetOutput(), ElementsAreArray({3, 7, 6, 7, 7, 7, 7, 7}));
+  EXPECT_THAT(m.GetOutputShape(), ElementsAreArray({2, 1, 2, 2}));
+}
+
+TEST_F(PadV2OpTest, Int32PaddingSimple4DConstFloat16ValuedTest) {
+  Simple4DConstFloat16ValuedTest<int32_t>();
+}
+
+TEST_F(PadV2OpTest, Int64PaddingSimple4DConstFloat16ValuedTest) {
+  Simple4DConstFloat16ValuedTest<int64_t>();
+}
+
+TEST_F(PadV2OpTest, Int8PaddingSimple4DConstFloat16ValuedTest) {
+  Simple4DConstFloat16ValuedTest<int8_t>();
+}
+
+TEST_F(PadV2OpTest, Int16PaddingSimple4DConstFloat16ValuedTest) {
+  Simple4DConstFloat16ValuedTest<int16_t>();
+}
+
+template <typename padding_integer_type>
+void Simple4DConstBFloat16ValuedTest() {
+  PadV2OpConstModel<Eigen::bfloat16, padding_integer_type> m(
+      {TensorType_BFLOAT16, {1, 1, 2, 1}}, {4, 2}, {0, 1, 0, 0, 0, 0, 0, 1},
+      Eigen::bfloat16{5.0}, {TensorType_BFLOAT16});
+  m.SetInput({Eigen::bfloat16{3.2f}, Eigen::bfloat16{6.4f}});
+  ASSERT_EQ(m.Invoke(), kTfLiteOk);
+  EXPECT_THAT(
+      m.GetOutput(),
+      ElementsAreArray(ArrayFloatNear(
+          {Eigen::bfloat16{3.2f}, Eigen::bfloat16{5.0f}, Eigen::bfloat16{6.4f},
+           Eigen::bfloat16{5.0f}, Eigen::bfloat16{5.0f}, Eigen::bfloat16{5.0f},
+           Eigen::bfloat16{5.0f}, Eigen::bfloat16{5.0f}})));
+  EXPECT_THAT(m.GetOutputShape(), ElementsAreArray({2, 1, 2, 2}));
+}
+
+TEST_F(PadV2OpTest, Int32PaddingSimple4DConstBFloat16ValuedTest) {
+  Simple4DConstBFloat16ValuedTest<int32_t>();
+}
+
+TEST_F(PadV2OpTest, Int64PaddingSimple4DConstBFloat16ValuedTest) {
+  Simple4DConstBFloat16ValuedTest<int64_t>();
+}
+
+TEST_F(PadV2OpTest, Int8PaddingSimple4DConstBFloat16ValuedTest) {
+  Simple4DConstBFloat16ValuedTest<int8_t>();
+}
+
+TEST_F(PadV2OpTest, Int16PaddingSimple4DConstBFloat16ValuedTest) {
+  Simple4DConstBFloat16ValuedTest<int16_t>();
+}
+
 template <typename padding_integer_type>
 void SimpleConstInt32ValuedTest() {
   // Padding is represented as four 2-D lists representing above padding and
@@ -814,6 +1129,14 @@ TEST_F(PadV2OpTest, Int64PaddingSimpleConstInt32ValuedTest) {
   SimpleConstInt32ValuedTest<int64_t>();
 }
 
+TEST_F(PadV2OpTest, Int8PaddingSimpleConstInt32ValuedTest) {
+  SimpleConstInt32ValuedTest<int8_t>();
+}
+
+TEST_F(PadV2OpTest, Int16PaddingSimpleConstInt32ValuedTest) {
+  SimpleConstInt32ValuedTest<int16_t>();
+}
+
 template <typename padding_integer_type>
 void SimpleDynamicTestV2() {
   PadV2OpDynamicModel<float, padding_integer_type> m(
@@ -834,6 +1157,104 @@ TEST_F(PadV2OpTest, Int64PaddingSimpleDynamicTest) {
   SimpleDynamicTestV2<int64_t>();
 }
 
+TEST_F(PadV2OpTest, Int8PaddingSimpleDynamicTest) {
+  SimpleDynamicTestV2<int8_t>();
+}
+
+TEST_F(PadV2OpTest, Int16PaddingSimpleDynamicTest) {
+  SimpleDynamicTestV2<int16_t>();
+}
+
+template <typename padding_integer_type>
+void SimpleDynamicTestV2Float16() {
+  PadV2OpDynamicModel<Eigen::half, padding_integer_type> m(
+      {TensorType_FLOAT16, {1, 2, 2, 1}}, {4, 2}, Eigen::half{0.0},
+      {TensorType_FLOAT16});
+  m.SetInput({Eigen::half{1.0f}, Eigen::half{2.0f}, Eigen::half{3.0f},
+              Eigen::half{4.0f}});
+  m.SetPaddings({0, 0, 1, 1, 1, 1, 0, 0});
+  ASSERT_EQ(m.Invoke(), kTfLiteOk);
+  EXPECT_THAT(m.GetOutput(), ElementsAreArray({0, 0, 0, 0, 0, 1, 2, 0, 0, 3, 4,
+                                               0, 0, 0, 0, 0}));
+  EXPECT_THAT(m.GetOutputShape(), ElementsAreArray({1, 4, 4, 1}));
+}
+
+TEST_F(PadV2OpTest, Int32PaddingSimpleDynamicTestFloat16) {
+  SimpleDynamicTestV2Float16<int32_t>();
+}
+
+TEST_F(PadV2OpTest, Int64PaddingSimpleDynamicTestFloat16) {
+  SimpleDynamicTestV2Float16<int64_t>();
+}
+
+TEST_F(PadV2OpTest, Int8PaddingSimpleDynamicTestFloat16) {
+  SimpleDynamicTestV2Float16<int8_t>();
+}
+
+TEST_F(PadV2OpTest, Int16PaddingSimpleDynamicTestFloat16) {
+  SimpleDynamicTestV2Float16<int16_t>();
+}
+
+template <typename padding_integer_type>
+void SimpleDynamicTestV2BFloat16() {
+  PadV2OpDynamicModel<Eigen::bfloat16, padding_integer_type> m(
+      {TensorType_BFLOAT16, {1, 2, 2, 1}}, {4, 2}, Eigen::bfloat16{2.0},
+      {TensorType_BFLOAT16});
+  m.SetInput({Eigen::bfloat16{5.0f}, Eigen::bfloat16{6.0f},
+              Eigen::bfloat16{7.0f}, Eigen::bfloat16{8.0f}});
+  m.SetPaddings({0, 0, 1, 1, 1, 1, 0, 0});
+  ASSERT_EQ(m.Invoke(), kTfLiteOk);
+  EXPECT_THAT(m.GetOutput(), ElementsAreArray({2, 2, 2, 2, 2, 5, 6, 2, 2, 7, 8,
+                                               2, 2, 2, 2, 2}));
+  EXPECT_THAT(m.GetOutputShape(), ElementsAreArray({1, 4, 4, 1}));
+}
+
+TEST_F(PadV2OpTest, Int32PaddingSimpleDynamicTestBFloat16) {
+  SimpleDynamicTestV2BFloat16<int32_t>();
+}
+
+TEST_F(PadV2OpTest, Int64PaddingSimpleDynamicTestBFloat16) {
+  SimpleDynamicTestV2BFloat16<int64_t>();
+}
+
+TEST_F(PadV2OpTest, Int8PaddingSimpleDynamicTestBFloat16) {
+  SimpleDynamicTestV2BFloat16<int8_t>();
+}
+
+TEST_F(PadV2OpTest, Int16PaddingSimpleDynamicTestBFloat16) {
+  SimpleDynamicTestV2BFloat16<int16_t>();
+}
+
+template <typename padding_integer_type>
+void SimpleDynamicTestBoolV2() {
+  PadV2OpDynamicModel<bool, padding_integer_type> m(
+      {TensorType_BOOL, {1, 2, 2, 1}}, {4, 2}, false, {TensorType_BOOL});
+  m.SetInput({true, false, true, false});
+  m.SetPaddings({0, 0, 1, 1, 1, 1, 0, 0});
+  ASSERT_EQ(m.Invoke(), kTfLiteOk);
+  EXPECT_THAT(m.GetOutput(),
+              ElementsAreArray({false, false, false, false, false, true, false,
+                                false, false, true, false, false, false, false,
+                                false, false}));
+  EXPECT_THAT(m.GetOutputShape(), ElementsAreArray({1, 4, 4, 1}));
+}
+
+TEST_F(PadV2OpTest, Int32PaddingSimpleDynamicTestBoolV2) {
+  SimpleDynamicTestBoolV2<int32_t>();
+}
+
+TEST_F(PadV2OpTest, Int64PaddingSimpleDynamicTestBoolV2) {
+  SimpleDynamicTestBoolV2<int64_t>();
+}
+
+TEST_F(PadV2OpTest, Int8PaddingSimpleDynamicTestBoolV2) {
+  SimpleDynamicTestBoolV2<int8_t>();
+}
+
+TEST_F(PadV2OpTest, Int16PaddingSimpleDynamicTestBoolV2) {
+  SimpleDynamicTestBoolV2<int16_t>();
+}
+
 template <typename padding_integer_type>
 void PadV2OpDynamicUnequalDimensions() {
   if (SingleOpModel::GetForceUseNnapi()) {
@@ -855,6 +1276,14 @@ TEST_F(PadV2OpTest, Int64PaddingDynamicUnequalDimensions) {
   PadV2OpDynamicUnequalDimensions<int64_t>();
 }
 
+TEST_F(PadV2OpTest, Int8PaddingDynamicUnequalDimensions) {
+  PadV2OpDynamicUnequalDimensions<int8_t>();
+}
+
+TEST_F(PadV2OpTest, Int16PaddingDynamicUnequalDimensions) {
+  PadV2OpDynamicUnequalDimensions<int16_t>();
+}
+
 template <typename padding_integer_type>
 void SimpleDynamicValuedTest() {
   PadV2OpDynamicModel<float, padding_integer_type> m(
@@ -875,6 +1304,14 @@ TEST_F(PadV2OpTest, Int64PaddingSimpleDynamicValuedTest) {
   SimpleDynamicValuedTest<int64_t>();
 }
 
+TEST_F(PadV2OpTest, Int8PaddingSimpleDynamicValuedTest) {
+  SimpleDynamicValuedTest<int8_t>();
+}
+
+TEST_F(PadV2OpTest, Int16PaddingSimpleDynamicValuedTest) {
+  SimpleDynamicValuedTest<int16_t>();
+}
+
 template <typename padding_integer_type>
 void SimpleTensorWithDim0Test() {
   PadV2OpDynamicModel<float, padding_integer_type> m(
@@ -900,6 +1337,14 @@ TEST_F(PadV2OpTest, Int64PaddingSimpleTensorWithDim0Test) {
   SimpleTensorWithDim0Test<int64_t>();
 }
 
+TEST_F(PadV2OpTest, Int8PaddingSimpleTensorWithDim0Test) {
+  SimpleTensorWithDim0Test<int8_t>();
+}
+
+TEST_F(PadV2OpTest, Int16PaddingSimpleTensorWithDim0Test) {
+  SimpleTensorWithDim0Test<int16_t>();
+}
+
 template <typename padding_integer_type>
 void Simple5DConstFloat32ValuedTest() {
   PadV2OpConstModel<float, padding_integer_type> m(
@@ -920,6 +1365,14 @@ TEST_F(PadV2OpTest, Int64PaddingSimple5DConstFloat32ValuedTest) {
   Simple5DConstFloat32ValuedTest<int64_t>();
 }
 
+TEST_F(PadV2OpTest, Int8PaddingSimple5DConstFloat32ValuedTest) {
+  Simple5DConstFloat32ValuedTest<int8_t>();
+}
+
+TEST_F(PadV2OpTest, Int16PaddingSimple5DConstFloat32ValuedTest) {
+  Simple5DConstFloat32ValuedTest<int16_t>();
+}
+
 template <typename padding_integer_type>
 void Simple5DConstInt32ValuedTest() {
   PadV2OpConstModel<int32_t, padding_integer_type> m(
@@ -943,6 +1396,14 @@ TEST_F(PadV2OpTest, Int64PaddingSimple5DConstInt32ValuedTest) {
   Simple5DConstInt32ValuedTest<int64_t>();
 }
 
+TEST_F(PadV2OpTest, Int8PaddingSimple5DConstInt32ValuedTest) {
+  Simple5DConstInt32ValuedTest<int8_t>();
+}
+
+TEST_F(PadV2OpTest, Int16PaddingSimple5DConstInt32ValuedTest) {
+  Simple5DConstInt32ValuedTest<int16_t>();
+}
+
 template <typename padding_integer_type>
 void Simple5DDynamicValuedTest() {
   PadV2OpDynamicModel<float, padding_integer_type> m(
@@ -966,6 +1427,14 @@ TEST_F(PadV2OpTest, Int64PaddingSimple5DDynamicValuedTest) {
   Simple5DDynamicValuedTest<int64_t>();
 }
 
+TEST_F(PadV2OpTest, Int8PaddingSimple5DDynamicValuedTest) {
+  Simple5DDynamicValuedTest<int8_t>();
+}
+
+TEST_F(PadV2OpTest, Int16PaddingSimple5DDynamicValuedTest) {
+  Simple5DDynamicValuedTest<int16_t>();
+}
+
 template <typename padding_integer_type>
 void AdvancedConstTest() {
   PadV2OpConstModel<float, padding_integer_type> m(
@@ -987,6 +1456,14 @@ TEST_F(PadV2OpTest, Int64PaddingAdvancedConstTest) {
   AdvancedConstTest<int64_t>();
 }
 
+TEST_F(PadV2OpTest, Int8PaddingAdvancedConstTest) {
+  AdvancedConstTest<int8_t>();
+}
+
+TEST_F(PadV2OpTest, Int16PaddingAdvancedConstTest) {
+  AdvancedConstTest<int16_t>();
+}
+
 template <typename padding_integer_type>
 void AdvancedDynamicTestV2() {
   PadV2OpDynamicModel<float, padding_integer_type> m(
@@ -1008,6 +1485,14 @@ TEST_F(PadV2OpTest, Int64PaddingAdvancedDynamicTest) {
   AdvancedDynamicTestV2<int64_t>();
 }
 
+TEST_F(PadV2OpTest, Int8PaddingAdvancedDynamicTest) {
+  AdvancedDynamicTestV2<int8_t>();
+}
+
+TEST_F(PadV2OpTest, Int16PaddingAdvancedDynamicTest) {
+  AdvancedDynamicTestV2<int16_t>();
+}
+
 class QuantizedPadV2OpTest : public ::testing::Test {
  protected:
   std::vector<Matcher<float>> DequantizedArrayNear(
