@@ -9,9 +9,10 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
+#include <cstdint>
+
 #include "pybind11/pybind11.h"  // from @pybind11
 #include "pybind11/pytypes.h"  // from @pybind11
-#include "tensorflow/lite/kernels/variants/list_ops_lib.h"
 #include "tensorflow/lite/kernels/variants/register_list_ops.h"
 #include "tensorflow/lite/mutable_op_resolver.h"
 
