@@ -27,8 +27,18 @@ void RegisterListOps(MutableOpResolver* resolver) {
   resolver->AddCustom("TensorListSetItem", Register_LIST_SET_ITEM());
   resolver->AddCustom("TensorListFromTensor", Register_LIST_FROM_TENSOR());
   resolver->AddCustom("TensorListGetItem", Register_LIST_GET_ITEM());
+  resolver->AddCustom("TensorListLength", Register_LIST_LENGTH());
+  resolver->AddCustom("TensorListElementShape", Register_LIST_ELEMENT_SHAPE());
+  resolver->AddCustom("TensorListPopBack", Register_LIST_POP_BACK());
+  resolver->AddCustom("TensorListPushBack", Register_LIST_PUSH_BACK());
+  resolver->AddCustom("VariantAddN", Register_VARIANT_ADD_N());
+  resolver->AddCustom("VariantZerosLike", Register_VARIANT_ZEROS_LIKE());
 }
 
 }  // namespace ops
 }  // namespace variants
 }  // namespace tflite
+
+void RegisterSelectedOps(tflite::MutableOpResolver* resolver) {
+  tflite::variants::ops::RegisterListOps(resolver);
+}
