@@ -51,7 +51,7 @@ TEST(ZerosLikeOpModel, ZerosLikeFloat) {
   m.PopulateTensor<float>(m.input(), {-2.0, -1.0, 0.0, 1.0, 2.0, 3.0});
   ASSERT_EQ(m.Invoke(), kTfLiteOk);
   EXPECT_THAT(m.ExtractVector<float>(m.output()),
-              ElementsAreArray({0.0, 0.0, 0.0, 0.0, 0.0, 0.0}));
+              Pointwise(FloatingPointEq(), {0.0, 0.0, 0.0, 0.0, 0.0, 0.0}));
   EXPECT_THAT(m.GetTensorShape(m.output()), ElementsAreArray({2, 3}));
 }
 
