@@ -12,10 +12,14 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
+#include <cstddef>
 #include <string>
 #include <vector>
 
+#include "absl/log/check.h"
+#include "absl/status/status.h"
 #include "tensorflow/core/platform/logging.h"
+#include "tensorflow/core/platform/status.h"
 #include "tensorflow/lite/toco/graph_transformations/graph_transformations.h"
 #include "tensorflow/lite/toco/model.h"
 #include "tensorflow/lite/toco/tooling_util.h"
@@ -49,13 +53,13 @@ bool TransposeAffectsMemoryOrder(std::vector<int> perm,
 
 }  // namespace
 
-::tensorflow::Status ConvertTrivialTransposeToReshape::Run(Model* model,
-                                                           std::size_t op_index,
-                                                           bool* modified) {
+absl::Status ConvertTrivialTransposeToReshape::Run(Model* model,
+                                                   std::size_t op_index,
+                                                   bool* modified) {
   *modified = false;
   auto transpose_it = model->operators.begin() + op_index;
   if (transpose_it->get()->type != OperatorType::kTranspose) {
-    return ::tensorflow::OkStatus();
+    return absl::OkStatus();
   }
   TransposeOperator* transpose_op =
       static_cast<TransposeOperator*>(transpose_it->get());
@@ -64,14 +68,14 @@ bool TransposeAffectsMemoryOrder(std::vector<int> perm,
   const auto& output_array = model->GetArray(transpose_op->outputs[0]);
   if (!input_array.has_shape() || !output_array.has_shape()) {
     // Yield until PropagateFixedSizes has been run on this op.
-    return ::tensorflow::OkStatus();
+    return absl::OkStatus();
   }
   // Note: We can assume we have error checked inputs in PropagateFixedSizes.
 
   // Check that the permutation has propagated.
   std::vector<int> const& perm = transpose_op->perm;
   if (perm.empty()) {
-    return ::tensorflow::OkStatus();
+    return absl::OkStatus();
   }
 
   // This transpose is trivial if non-unitary dimensions remain in the same
@@ -80,7 +84,7 @@ bool TransposeAffectsMemoryOrder(std::vector<int> perm,
   std::vector<int> const& output_dims = output_array.shape().dims();
 
   if (TransposeAffectsMemoryOrder(perm, input_dims)) {
-    return ::tensorflow::OkStatus();
+    return absl::OkStatus();
   }
 
   // This transpose is trivial. Replace it with a Reshape op.
@@ -113,7 +117,7 @@ bool TransposeAffectsMemoryOrder(std::vector<int> perm,
   DeleteOpAndArrays(model, transpose_op);
 
   *modified = true;
-  return ::tensorflow::OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace toco
