@@ -13,11 +13,18 @@ See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
 #include <algorithm>
+#include <cstddef>
+#include <cstdlib>
+#include <limits>
 #include <memory>
 #include <string>
 #include <vector>
 
+#include "absl/log/check.h"
+#include "absl/log/log.h"
+#include "absl/status/status.h"
 #include "tensorflow/core/platform/logging.h"
+#include "tensorflow/core/platform/status.h"
 #include "tensorflow/lite/toco/graph_transformations/graph_transformations.h"
 #include "tensorflow/lite/toco/model.h"
 #include "tensorflow/lite/toco/tooling_util.h"
@@ -423,8 +430,8 @@ bool HardcodeMinMaxForPack(Model* model, Operator* op) {
 
 }  // namespace
 
-::tensorflow::Status HardcodeMinMax::Run(Model* model, std::size_t op_index,
-                                         bool* modified) {
+absl::Status HardcodeMinMax::Run(Model* model, std::size_t op_index,
+                                 bool* modified) {
   *modified = false;
   auto it = model->operators.begin() + op_index;
   auto* op = it->get();
@@ -527,7 +534,7 @@ bool HardcodeMinMaxForPack(Model* model, Operator* op) {
     AddMessageF("Hardcoded min-max through %s", LogName(*op));
   }
   *modified = changed;
-  return ::tensorflow::OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace toco
