@@ -13,8 +13,12 @@
  limitations under the License.
  ==============================================================================*/
 #include <algorithm>
+#include <cstddef>
 #include <string>
 
+#include "absl/log/check.h"
+#include "absl/status/status.h"
+#include "tensorflow/core/platform/status.h"
 #include "tensorflow/lite/toco/graph_transformations/graph_transformations.h"
 #include "tensorflow/lite/toco/model.h"
 #include "tensorflow/lite/toco/tooling_util.h"
@@ -55,9 +59,9 @@ bool IsTailOfShape(const Shape& tail, const Shape& shape) {
 //
 // Note we are testing for one particular case of a broader set of possible
 // binary-reshape op transformations. This transformation could be generalized.
-::tensorflow::Status MoveBinaryOperatorBeforeReshape::Run(Model* model,
-                                                          std::size_t op_index,
-                                                          bool* modified) {
+absl::Status MoveBinaryOperatorBeforeReshape::Run(Model* model,
+                                                  std::size_t op_index,
+                                                  bool* modified) {
   *modified = false;
   const auto binary_it = model->operators.begin() + op_index;
   Operator* binary_op = binary_it->get();
@@ -73,7 +77,7 @@ bool IsTailOfShape(const Shape& tail, const Shape& shape) {
       binary_op->type != OperatorType::kLessEqual &&
       binary_op->type != OperatorType::kGreater &&
       binary_op->type != OperatorType::kGreaterEqual) {
-    return ::tensorflow::OkStatus();
+    return absl::OkStatus();
   }
 
   // BINARY OP INPUT CHECKS
@@ -85,11 +89,11 @@ bool IsTailOfShape(const Shape& tail, const Shape& shape) {
   if (!input_is_const[0] && !input_is_const[1]) {
     // To limit our scope, we require one constant input. Though there's no
     // reason this transformation wouldn't work with all variable inputs.
-    return ::tensorflow::OkStatus();
+    return absl::OkStatus();
   }
   if (input_is_const[0] && input_is_const[1]) {
     // Both inputs are constants. Leave this for constants propagation.
-    return ::tensorflow::OkStatus();
+    return absl::OkStatus();
   }
   const int constant_input_idx = input_is_const[0] ? 0 : 1;
   const int variable_input_idx = input_is_const[0] ? 1 : 0;
@@ -102,13 +106,13 @@ bool IsTailOfShape(const Shape& tail, const Shape& shape) {
     AddMessageF(
         "Not moving %s because it's non-constant input shape is not resolved.",
         LogName(*binary_op));
-    return ::tensorflow::OkStatus();
+    return absl::OkStatus();
   }
   if (!IsTailOfShape(
           model->GetArray(binary_op->inputs[constant_input_idx]).shape(),
           model->GetArray(binary_op->inputs[variable_input_idx]).shape())) {
     // Constant array shape must be the latter part of the variable shape.
-    return ::tensorflow::OkStatus();
+    return absl::OkStatus();
   }
 
   // RESHAPE OP CHECKS
@@ -117,13 +121,13 @@ bool IsTailOfShape(const Shape& tail, const Shape& shape) {
   if (reshape_it == model->operators.end()) {
     AddMessageF("Not moving %s because it's variable input is not connected.",
                 LogName(*binary_op));
-    return ::tensorflow::OkStatus();
+    return absl::OkStatus();
   }
   Operator* reshape_op = reshape_it->get();
   if (reshape_op->type != OperatorType::kReshape) {
     AddMessageF("Not moving %s because the preceding %s is not a reshape op",
                 LogName(*binary_op), LogName(*reshape_op));
-    return ::tensorflow::OkStatus();
+    return absl::OkStatus();
   }
   const auto& reshape_input_array = model->GetArray(reshape_op->inputs[0]);
   if (!reshape_input_array.has_shape()) {
@@ -131,14 +135,14 @@ bool IsTailOfShape(const Shape& tail, const Shape& shape) {
         "Not moving %s because it's non-constant input shape is not resolved "
         "yet",
         LogName(*binary_op));
-    return ::tensorflow::OkStatus();
+    return absl::OkStatus();
   }
   if (!IsTailOfShape(
           model->GetArray(binary_op->inputs[constant_input_idx]).shape(),
           model->GetArray(reshape_op->outputs[0]).shape())) {
     // Constant array shape must be the latter part of the binary op output
     // shape.
-    return ::tensorflow::OkStatus();
+    return absl::OkStatus();
   }
 
   // EXTRA CHECKS ON CONNECTING ARRAY
@@ -147,7 +151,7 @@ bool IsTailOfShape(const Shape& tail, const Shape& shape) {
       AddMessageF(
           "Not moving %s because the output of reshape op %s is an output op.",
           LogName(*binary_op), LogName(*reshape_op));
-      return ::tensorflow::OkStatus();
+      return absl::OkStatus();
     }
   }
   int count_ops_consuming_output =
@@ -158,7 +162,7 @@ bool IsTailOfShape(const Shape& tail, const Shape& shape) {
         "Not moving %s because the output of reshape op %s is consumed by "
         "another op",
         LogName(*binary_op), LogName(*reshape_op));
-    return ::tensorflow::OkStatus();
+    return absl::OkStatus();
   }
 
   // SWAP ORDER OF BINARY AND RESHAPE OPS
@@ -177,7 +181,7 @@ bool IsTailOfShape(const Shape& tail, const Shape& shape) {
   model->GetArray(binary_op->outputs[0]).clear_shape();
 
   *modified = true;
-  return ::tensorflow::OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace toco
