@@ -14,18 +14,25 @@ limitations under the License.
 ==============================================================================*/
 #include <algorithm>
 #include <cmath>
+#include <cstddef>
+#include <cstdint>
 #include <iterator>
 #include <memory>
-#include <numeric>
+#include <set>
 #include <string>
 #include <unordered_map>
 #include <vector>
 
+#include "absl/log/check.h"
+#include "absl/log/log.h"
+#include "absl/status/status.h"
 #include "absl/strings/str_join.h"
 #include "tensorflow/core/platform/logging.h"
+#include "tensorflow/core/platform/status.h"
 #include "tensorflow/lite/kernels/internal/strided_slice_logic.h"
 #include "tensorflow/lite/toco/graph_transformations/graph_transformations.h"
 #include "tensorflow/lite/toco/model.h"
+#include "tensorflow/lite/toco/toco_types.h"
 #include "tensorflow/lite/toco/tooling_util.h"
 
 namespace toco {
@@ -2144,9 +2151,8 @@ void ProcessScatterNdOperator(Model* model, ScatterNdOperator* op) {
 
 }  // namespace
 
-::tensorflow::Status PropagateFixedSizes::Run(Model* model,
-                                              std::size_t op_index,
-                                              bool* modified) {
+absl::Status PropagateFixedSizes::Run(Model* model, std::size_t op_index,
+                                      bool* modified) {
   *modified = false;
   auto it = model->operators.begin() + op_index;
   auto* op = it->get();
@@ -2398,7 +2404,7 @@ void ProcessScatterNdOperator(Model* model, ScatterNdOperator* op) {
           static_cast<TensorFlowUnsupportedOperator*>(op);
       // Attribute can be not specified, ignore it.
       if (unsupported_op->output_shapes.size() < op->outputs.size()) {
-        return ::tensorflow::OkStatus();
+        return absl::OkStatus();
       }
       for (size_t i = 0; i < op->outputs.size(); ++i) {
         const std::string& output = op->outputs[i];
@@ -2492,10 +2498,10 @@ void ProcessScatterNdOperator(Model* model, ScatterNdOperator* op) {
       AddMessageF("Set shape of %s to [%s]", output,
                   absl::StrJoin(model->GetArray(output).shape().dims(), ","));
       *modified = true;
-      return ::tensorflow::OkStatus();
+      return absl::OkStatus();
     }
   }
-  return ::tensorflow::OkStatus();
+  return absl::OkStatus();
 }
 
 }  // namespace toco
