@@ -12,11 +12,13 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
-#include <cstdio>
 #include <memory>
 #include <string>
 
-#include "absl/strings/string_view.h"
+#include "absl/log/check.h"
+#include "absl/status/status.h"
+#include "tensorflow/core/lib/core/errors.h"
+#include "tensorflow/core/platform/logging.h"
 #include "tensorflow/lite/toco/model.h"
 #include "tensorflow/lite/toco/model_cmdline_flags.h"
 #include "tensorflow/lite/toco/model_flags.pb.h"
@@ -25,8 +27,6 @@ limitations under the License.
 #include "tensorflow/lite/toco/toco_port.h"
 #include "tensorflow/lite/toco/toco_tooling.h"
 #include "tensorflow/lite/toco/toco_types.h"
-#include "tensorflow/core/lib/core/errors.h"
-#include "tensorflow/core/platform/logging.h"
 
 namespace toco {
 namespace {
@@ -64,17 +64,15 @@ void ReadInputData(const ParsedTocoFlags& parsed_toco_flags,
 
   // Checks the input file permissions and reads the contents.
   CheckFrozenModelPermissions(parsed_toco_flags.input_file);
-  CHECK(port::file::GetContents(parsed_toco_flags.input_file.value(),
-                                graph_def_contents, port::file::Defaults())
-            .ok());
+  CHECK_OK(port::file::GetContents(parsed_toco_flags.input_file.value(),
+                                   graph_def_contents, port::file::Defaults()));
 }
 }  // namespace
 
-tensorflow::Status Convert(const std::string& graph_def_contents,
-                           const TocoFlags& toco_flags,
-                           const ModelFlags& model_flags,
-                           std::string* output_file_contents,
-                           int64_t* arithmetic_ops_count = nullptr) {
+absl::Status Convert(const std::string& graph_def_contents,
+                     const TocoFlags& toco_flags, const ModelFlags& model_flags,
+                     std::string* output_file_contents,
+                     int64_t* arithmetic_ops_count = nullptr) {
   std::unique_ptr<Model> model =
       Import(toco_flags, model_flags, graph_def_contents);
   TF_RETURN_IF_ERROR(TransformWithStatus(toco_flags, model.get()));
@@ -83,11 +81,11 @@ tensorflow::Status Convert(const std::string& graph_def_contents,
   if (arithmetic_ops_count != nullptr) {
     *arithmetic_ops_count = model->ArithmeticOpsCount();
   }
-  return ::tensorflow::OkStatus();
+  return absl::OkStatus();
 }
 
-tensorflow::Status Convert(const ParsedTocoFlags& parsed_toco_flags,
-                           const ParsedModelFlags& parsed_model_flags) {
+absl::Status Convert(const ParsedTocoFlags& parsed_toco_flags,
+                     const ParsedModelFlags& parsed_model_flags) {
   ModelFlags model_flags;
   ReadModelFlagsFromCommandLineFlags(parsed_model_flags, &model_flags);
 
@@ -105,7 +103,7 @@ tensorflow::Status Convert(const ParsedTocoFlags& parsed_toco_flags,
   TF_RETURN_IF_ERROR(
       port::file::SetContents(parsed_toco_flags.output_file.value(),
                               output_file_contents, port::file::Defaults()));
-  return tensorflow::Status();
+  return absl::Status();
 }
 
 }  // namespace toco
