@@ -24,14 +24,13 @@ limitations under the License.
 
 namespace toco {
 
-tensorflow::Status Convert(const std::string& graph_def_contents,
-                           const TocoFlags& toco_flags,
-                           const ModelFlags& model_flags,
-                           std::string* output_file_contents,
-                           int64_t* arithmetic_ops_count = nullptr);
+absl::Status Convert(const std::string& graph_def_contents,
+                     const TocoFlags& toco_flags, const ModelFlags& model_flags,
+                     std::string* output_file_contents,
+                     int64_t* arithmetic_ops_count = nullptr);
 
-tensorflow::Status Convert(const ParsedTocoFlags& parsed_toco_flags,
-                           const ParsedModelFlags& parsed_model_flags);
+absl::Status Convert(const ParsedTocoFlags& parsed_toco_flags,
+                     const ParsedModelFlags& parsed_model_flags);
 }  // namespace toco
 
 #endif  // TENSORFLOW_LITE_TOCO_TOCO_CONVERT_H_
