@@ -15,9 +15,11 @@ limitations under the License.
 
 #include "tensorflow/lite/tools/benchmark/benchmark_tflite_model.h"
 
+#include <algorithm>
 #include <cstdarg>
 #include <cstdint>
 #include <cstdlib>
+#include <cstring>
 #include <fstream>
 #include <functional>
 #include <iostream>
@@ -25,6 +27,7 @@ limitations under the License.
 #include <random>
 #include <sstream>
 #include <string>
+#include <string_view>
 #include <unordered_set>
 #include <utility>
 #include <vector>
@@ -34,19 +37,25 @@ limitations under the License.
 #include "absl/strings/str_replace.h"
 #include "absl/strings/str_split.h"
 #include "absl/strings/string_view.h"
+#include "absl/types/span.h"
 #include "ruy/profiler/profiler.h"  // from @ruy
+#include "tensorflow/core/example/example.pb.h"
+#include "tensorflow/core/example/feature.pb.h"
 #include "tensorflow/lite/core/c/c_api_types.h"
 #include "tensorflow/lite/core/c/common.h"
 #include "tensorflow/lite/core/kernels/register.h"
 #include "tensorflow/lite/core/model.h"
 #include "tensorflow/lite/core/model_builder.h"
+#include "tensorflow/lite/core/signature_runner.h"
 #include "tensorflow/lite/core/subgraph.h"
 #include "tensorflow/lite/interpreter.h"
 #include "tensorflow/lite/kernels/cpu_backend_context.h"
 #include "tensorflow/lite/op_resolver.h"
 #include "tensorflow/lite/optional_debug_tools.h"
+#include "tensorflow/lite/profiling/model_runtime_info.h"
 #include "tensorflow/lite/profiling/profile_summary_formatter.h"
 #include "tensorflow/lite/string_util.h"
+#include "tensorflow/lite/tools/benchmark/benchmark_params.h"
 #include "tensorflow/lite/tools/benchmark/benchmark_utils.h"
 #include "tensorflow/lite/tools/benchmark/profiling_listener.h"
 #include "tensorflow/lite/tools/delegates/delegate_provider.h"
@@ -75,6 +84,58 @@ constexpr bool kOpProfilingEnabledDefault = true;
 constexpr bool kOpProfilingEnabledDefault = false;
 #endif
 
+// Op profiling output modes.
+constexpr char kOpProfilingOutputModeStdout[] = "stdout";
+constexpr char kOpProfilingOutputModeCsv[] = "csv";
+constexpr char kOpProfilingOutputModeProto[] = "proto";
+
+const char* kOpProfilingOutputModes[] = {kOpProfilingOutputModeStdout,
+                                         kOpProfilingOutputModeCsv,
+                                         kOpProfilingOutputModeProto};
+
+// Sets feature values in the tensorflow::Example proto from the tflite tensor.
+// Returns an error if the tensor type is not supported or the tensor dime is a
+// nullptr.
+TfLiteStatus MaybeSetFeatureValuesFromTensor(const TfLiteTensor& tensor,
+                                             tensorflow::Example& example) {
+  if (tensor.dims == nullptr) {
+    return kTfLiteError;
+  }
+
+  int total_elements = 1;
+  for (int i = 0; i < tensor.dims->size; i++) {
+    total_elements *= tensor.dims->data[i];
+  }
+  tensorflow::Feature& feature =
+      (*example.mutable_features()->mutable_feature())[tensor.name];
+  switch (tensor.type) {
+    case kTfLiteFloat32:
+    case kTfLiteFloat64:
+      feature.mutable_float_list()->mutable_value()->Resize(total_elements, 0);
+      return utils::TfLiteTensorToFloat32Array(
+          tensor,
+          absl::MakeSpan(
+              feature.mutable_float_list()->mutable_value()->mutable_data(),
+              feature.float_list().value_size()));
+    case kTfLiteUInt8:
+    case kTfLiteInt8:
+    case kTfLiteUInt16:
+    case kTfLiteInt16:
+    case kTfLiteInt32:
+    case kTfLiteUInt32:
+    case kTfLiteUInt64:
+    case kTfLiteInt64:
+      feature.mutable_int64_list()->mutable_value()->Resize(total_elements, 0);
+      return utils::TfLiteTensorToInt64Array(
+          tensor,
+          absl::MakeSpan(
+              feature.mutable_int64_list()->mutable_value()->mutable_data(),
+              feature.int64_list().value_size()));
+    default:
+      return kTfLiteError;
+  }
+}
+
 // Dumps ruy profiling events if the ruy profiler is enabled.
 class RuyProfileListener : public BenchmarkListener {
  public:
@@ -133,31 +194,76 @@ class InterpreterStatePrinter : public BenchmarkListener {
 
 class OutputSaver : public BenchmarkListener {
  public:
-  explicit OutputSaver(Interpreter* interpreter) : interpreter_(interpreter) {}
+  explicit OutputSaver(BenchmarkInterpreterRunner* runner)
+      : interpreter_runner_(runner) {}
 
   void OnBenchmarkStart(const BenchmarkParams& params) override {
     params_ = &params;
   }
 
   void OnBenchmarkEnd(const BenchmarkResults& results) override {
-    std::string path = params_->Get<std::string>("output_filepath");
-    if (path.empty()) return;
+    // If the output_filepath is specified, save the output tensors to the file.
+    const std::string path = params_->Get<std::string>("output_filepath");
+    if (!path.empty()) {
+      std::ofstream ofs(path, std::ofstream::out);
+      if (ofs.good()) {
+        for (int i = 0; i < interpreter_runner_->outputs().size(); i++) {
+          int tensor_index = interpreter_runner_->outputs()[i];
+          ofs.write(interpreter_runner_->tensor(tensor_index)->data.raw,
+                    interpreter_runner_->tensor(tensor_index)->bytes);
+        }
+        ofs.close();
+      }
+    }
 
-    std::ofstream ofs(path, std::ofstream::out);
-    if (ofs.good()) {
-      for (int i = 0; i < interpreter_->outputs().size(); i++) {
-        ofs.write(interpreter_->output_tensor(i)->data.raw,
-                  interpreter_->output_tensor(i)->bytes);
+    // If the output_proto_filepath is specified, save the output tensors as
+    // tensorflow::Example proto and serialize it to the file.
+    const std::string output_proto_path =
+        params_->Get<std::string>("output_proto_filepath");
+    if (!output_proto_path.empty()) {
+      tensorflow::Example example;
+      for (int i = 0; i < interpreter_runner_->outputs().size(); i++) {
+        const int tensor_index = interpreter_runner_->outputs()[i];
+        const TfLiteTensor& tensor =
+            *(interpreter_runner_->tensor(tensor_index));
+        MaybeSetFeatureValuesFromTensor(tensor, example);
+      }
+      std::ofstream ofs(output_proto_path, std::ios::out | std::ios::binary);
+      if (ofs.good()) {
+        example.SerializeToOstream(&ofs);
+        ofs.close();
       }
-      ofs.close();
     }
   }
 
  private:
-  Interpreter* const interpreter_ = nullptr;
+  BenchmarkInterpreterRunner* const interpreter_runner_;
   const BenchmarkParams* params_ = nullptr;
 };
 
+// Dumps the Model Runtime Info if enabled when export_model_runtime_info is
+// set to true.
+class ModelRuntimeInfoListener : public BenchmarkListener {
+ public:
+  explicit ModelRuntimeInfoListener(Interpreter* interpreter)
+      : interpreter_(interpreter) {}
+
+  // At this stage, the graph is fully modified with delegates.
+  // So the interpreter can be used to capture the ModelRuntimeDetails.
+  void OnBenchmarkStart(const BenchmarkParams& params) override {
+    const std::string output_file_path =
+        params.Get<std::string>("model_runtime_info_output_file");
+    const auto status =
+        profiling::GenerateModelRuntimeInfo(*interpreter_, output_file_path);
+    if (status != kTfLiteOk) {
+      TFLITE_LOG(ERROR) << "Failed to generate model runtime info: " << status;
+    }
+  }
+
+ private:
+  Interpreter* const interpreter_ = nullptr;  // not own the memory.
+};
+
 std::vector<std::string> Split(const std::string& str, const char delim) {
   if (str.empty()) {
     return {};
@@ -265,14 +371,13 @@ TfLiteStatus PopulateInputLayerInfo(
   std::vector<std::string> shapes = Split(shapes_string, ':');
 
   if (names.size() != shapes.size()) {
-    TFLITE_LOG(ERROR) << "The number of items in"
-                      << " --input_layer_shape (" << shapes_string << ", with "
-                      << shapes.size() << " items)"
-                      << " must match the number of items in"
-                      << " --input_layer (" << names_string << ", with "
-                      << names.size() << " items)."
-                      << " For example --input_layer=input1,input2"
-                      << " --input_layer_shape=1,224,224,4:1,20";
+    TFLITE_LOG(ERROR)
+        << "The number of items in --input_layer_shape (" << shapes_string
+        << ", with " << shapes.size()
+        << " items) must match the number of items in --input_layer ("
+        << names_string << ", with " << names.size()
+        << " items). For example --input_layer=input1,input2 "
+           "--input_layer_shape=1,224,224,4:1,20";
     return kTfLiteError;
   }
 
@@ -306,10 +411,14 @@ TfLiteStatus PopulateInputLayerInfo(
 }
 
 std::shared_ptr<profiling::ProfileSummaryFormatter>
-CreateProfileSummaryFormatter(bool format_as_csv) {
-  return format_as_csv
-             ? std::make_shared<profiling::ProfileSummaryCSVFormatter>()
-             : std::make_shared<profiling::ProfileSummaryDefaultFormatter>();
+CreateProfileSummaryFormatter(const std::string& output_mode) {
+  if (output_mode == kOpProfilingOutputModeCsv) {
+    return std::make_shared<profiling::ProfileSummaryCSVFormatter>();
+  } else if (output_mode == kOpProfilingOutputModeProto) {
+    return std::make_shared<profiling::ProfileSummaryProtoFormatter>();
+  } else {
+    return std::make_shared<profiling::ProfileSummaryDefaultFormatter>();
+  }
 }
 
 }  // namespace
@@ -347,9 +456,120 @@ TfLiteStatus SplitInputLayerNameAndValueFile(
   return kTfLiteOk;
 }
 
+std::pair<TfLiteStatus, std::unique_ptr<BenchmarkInterpreterRunner>>
+BenchmarkInterpreterRunner::Create(tflite::Interpreter* const interpreter,
+                                   std::string signature_key) {
+  if (!signature_key.empty()) {
+    const std::vector<const std::string*>& keys = interpreter->signature_keys();
+    bool found = std::any_of(
+        keys.begin(), keys.end(),
+        [&signature_key](const auto& k) { return *k == signature_key; });
+
+    if (keys.size() > 1 && (signature_key.empty() || !found)) {
+      TFLITE_LOG(ERROR)
+          << "Signature not specified or incorrect for graph with multiple "
+             "signatures. Pass one of the following to the flag "
+             "\"--signature_to_run_for\"";
+      for (const std::string* k : keys) {
+        TFLITE_LOG(ERROR) << " #> Signature key: " << *k;
+      }
+      return {kTfLiteError, nullptr};
+    } else if (keys.size() == 1 && signature_key.empty()) {
+      signature_key = *keys[0];
+    }
+
+    if (!signature_key.empty() && !keys.empty()) {
+      TFLITE_LOG(INFO) << "Using signature: " << signature_key;
+      auto signature_runner =
+          interpreter->GetSignatureRunner(signature_key.c_str());
+      if (signature_runner == nullptr) {
+        return {kTfLiteError, nullptr};
+      } else {
+        int subgraph_index =
+            interpreter->GetSubgraphIndexFromSignature(signature_key.c_str());
+
+        return {kTfLiteOk, std::make_unique<BenchmarkInterpreterRunner>(
+                               interpreter, signature_runner,
+                               interpreter->subgraph(subgraph_index))};
+      }
+    }
+  }
+  return {kTfLiteOk, std::make_unique<BenchmarkInterpreterRunner>(
+                         interpreter, nullptr, nullptr)};
+}
+
+TfLiteStatus BenchmarkInterpreterRunner::AllocateTensors() {
+  if (signature_runner_ != nullptr) {
+    return signature_runner_->AllocateTensors();
+  } else {
+    return interpreter_->AllocateTensors();
+  }
+}
+
+TfLiteStatus BenchmarkInterpreterRunner::Invoke() {
+  if (signature_runner_ != nullptr) {
+    return signature_runner_->Invoke();
+  } else {
+    return interpreter_->Invoke();
+  }
+}
+
+const std::vector<int>& BenchmarkInterpreterRunner::execution_plan() const {
+  if (signature_runner_ != nullptr) {
+    return subgraph_->execution_plan();
+  } else {
+    return interpreter_->execution_plan();
+  }
+}
+
+const std::vector<int>& BenchmarkInterpreterRunner::inputs() const {
+  if (signature_runner_ != nullptr) {
+    return subgraph_->inputs();
+  } else {
+    return interpreter_->inputs();
+  }
+}
+
+const std::vector<int>& BenchmarkInterpreterRunner::outputs() const {
+  if (signature_runner_ != nullptr) {
+    return subgraph_->outputs();
+  } else {
+    return interpreter_->outputs();
+  }
+}
+TfLiteTensor* BenchmarkInterpreterRunner::tensor(int tensor_index) {
+  if (signature_runner_ != nullptr) {
+    return subgraph_->tensor(tensor_index);
+  } else {
+    return interpreter_->tensor(tensor_index);
+  }
+}
+
+const std::pair<TfLiteNode, TfLiteRegistration>*
+BenchmarkInterpreterRunner::node_and_registration(int node_index) const {
+  if (signature_runner_ != nullptr) {
+    return subgraph_->node_and_registration(node_index);
+  } else {
+    return interpreter_->node_and_registration(node_index);
+  }
+}
+
+TfLiteStatus BenchmarkInterpreterRunner::ResizeInputTensor(
+    int tensor_index, const std::vector<int>& new_size) {
+  if (signature_runner_ != nullptr) {
+    return subgraph_->ResizeInputTensor(tensor_index, new_size);
+  } else {
+    return interpreter_->ResizeInputTensor(tensor_index, new_size);
+  }
+}
+
 BenchmarkParams BenchmarkTfLiteModel::DefaultParams() {
   BenchmarkParams default_params = BenchmarkModel::DefaultParams();
   default_params.AddParam("graph", BenchmarkParam::Create<std::string>(""));
+  default_params.AddParam("signature_to_run_for",
+                          BenchmarkParam::Create<std::string>(""));
+  default_params.AddParam("list_signatures",
+                          BenchmarkParam::Create<bool>(false));
   default_params.AddParam("input_layer",
                           BenchmarkParam::Create<std::string>(""));
   default_params.AddParam("input_layer_shape",
@@ -364,13 +584,21 @@ BenchmarkParams BenchmarkTfLiteModel::DefaultParams() {
   default_params.AddParam(
       "enable_op_profiling",
       BenchmarkParam::Create<bool>(kOpProfilingEnabledDefault));
+  default_params.AddParam(
+      "op_profiling_output_mode",
+      BenchmarkParam::Create<std::string>(kOpProfilingOutputModeStdout));
+  default_params.AddParam("op_profiling_output_file",
+                          BenchmarkParam::Create<std::string>(""));
   default_params.AddParam("max_profiling_buffer_entries",
                           BenchmarkParam::Create<int32_t>(1024));
   default_params.AddParam("allow_dynamic_profiling_buffer_increase",
                           BenchmarkParam::Create<bool>(false));
   default_params.AddParam("profiling_output_csv_file",
                           BenchmarkParam::Create<std::string>(""));
-
+  default_params.AddParam("export_model_runtime_info",
+                          BenchmarkParam::Create<bool>(false));
+  default_params.AddParam("model_runtime_info_output_file",
+                          BenchmarkParam::Create<std::string>(""));
   default_params.AddParam("print_preinvoke_state",
                           BenchmarkParam::Create<bool>(false));
   default_params.AddParam("print_postinvoke_state",
@@ -381,8 +609,12 @@ BenchmarkParams BenchmarkTfLiteModel::DefaultParams() {
                           BenchmarkParam::Create<int32_t>(0));
   default_params.AddParam("disable_delegate_clustering",
                           BenchmarkParam::Create<bool>(false));
+  default_params.AddParam("enable_builtin_cast_constant_cache",
+                          BenchmarkParam::Create<bool>(false));
   default_params.AddParam("output_filepath",
                           BenchmarkParam::Create<std::string>(""));
+  default_params.AddParam("output_proto_filepath",
+                          BenchmarkParam::Create<std::string>(""));
 
   default_params.AddParam("tensor_name_display_length",
                           BenchmarkParam::Create<int32_t>(25));
@@ -411,6 +643,10 @@ void BenchmarkTfLiteModel::CleanUp() {
 BenchmarkTfLiteModel::~BenchmarkTfLiteModel() {
   CleanUp();
 
+  // Release the pointer to the interpreter_runner_ before the interpreter is
+  // destroyed.
+  interpreter_runner_.reset();
+
   // Destory the owned interpreter earlier than other objects (specially
   // 'owned_delegates_').
   interpreter_.reset();
@@ -444,14 +680,25 @@ std::vector<Flag> BenchmarkTfLiteModel::GetFlags() {
       CreateFlag<bool>("require_full_delegation", &params_,
                        "require delegate to run the entire graph"),
       CreateFlag<bool>("enable_op_profiling", &params_, "enable op profiling"),
+      CreateFlag<std::string>(
+          "op_profiling_output_mode", &params_,
+          "Output mode for op profiling results. Supported values are: "
+          "'stdout', 'csv' and 'proto'."),
+      CreateFlag<std::string>("op_profiling_output_file", &params_,
+                              "Output file for op profiling results."),
       CreateFlag<int32_t>("max_profiling_buffer_entries", &params_,
                           "max initial profiling buffer entries"),
       CreateFlag<bool>("allow_dynamic_profiling_buffer_increase", &params_,
                        "allow dynamic increase on profiling buffer entries"),
-      CreateFlag<std::string>(
-          "profiling_output_csv_file", &params_,
-          "File path to export profile data as CSV, if not set "
-          "prints to stdout."),
+      CreateFlag<std::string>("profiling_output_csv_file", &params_,
+                              "[DEPRECATED: Use op_profiling_output_file and "
+                              "op_profiling_output_mode instead] File path to "
+                              "export profile data as CSV, if not set "
+                              "prints to stdout."),
+      CreateFlag<bool>("export_model_runtime_info", &params_,
+                       "Enable Model Runtime Info Export"),
+      CreateFlag<std::string>("model_runtime_info_output_file", &params_,
+                              "Proto File to export model runtime info to"),
       CreateFlag<bool>(
           "print_preinvoke_state", &params_,
           "print out the interpreter internals just before calling Invoke. The "
@@ -469,9 +716,16 @@ std::vector<Flag> BenchmarkTfLiteModel::GetFlags() {
           "Optimize memory usage for large tensors with sacrificing latency."),
       CreateFlag<bool>("disable_delegate_clustering", &params_,
                        "Disable delegate clustering."),
+      CreateFlag<bool>(
+          "enable_builtin_cast_constant_cache", &params_,
+          "Cache the output of the builtin cast operation when its input "
+          "is a constant tensor."),
       CreateFlag<std::string>(
           "output_filepath", &params_,
           "File path to export outputs layer as binary data."),
+      CreateFlag<std::string>(
+          "output_proto_filepath", &params_,
+          "File path to export outputs layer as tf example proto."),
       CreateFlag<int32_t>(
           "tensor_name_display_length", &params_,
           "The number of characters to show for the tensor's name when "
@@ -483,7 +737,17 @@ std::vector<Flag> BenchmarkTfLiteModel::GetFlags() {
       CreateFlag<int32_t>(
           "alloc_type_display_length", &params_,
           "The number of characters to show for the tensor's allocation type "
-          "when printing the interpeter's state, defaults to 18.")};
+          "when printing the interpeter's state, defaults to 18."),
+      CreateFlag<std::string>(
+          "signature_to_run_for", &params_,
+          "If the model contains multiple signatures, use this flag to specify "
+          "the signature to benchmark. If multiple signatures are present and "
+          "this flag is not specified, the benchmark will throw an error. If "
+          "only one signature is present and this flag is not specified, the "
+          "default signature will be used."),
+      CreateFlag<bool>("list_signatures", &params_,
+                       "Displays all signatures present in the model and then "
+                       "terminates the program.")};
 
   flags.insert(flags.end(), specific_flags.begin(), specific_flags.end());
 
@@ -498,6 +762,10 @@ void BenchmarkTfLiteModel::LogParams() {
   const bool verbose = params_.Get<bool>("verbose");
   // Always log the value of --graph.
   LOG_BENCHMARK_PARAM(std::string, "graph", "Graph", /*verbose*/ true);
+  LOG_BENCHMARK_PARAM(std::string, "signature_to_run_for", "Signature to run",
+                      /*verbose*/ true);
+  LOG_BENCHMARK_PARAM(bool, "list_signatures",
+                      "List signatures from the provided model", false);
   LOG_BENCHMARK_PARAM(std::string, "input_layer", "Input layers", verbose);
   LOG_BENCHMARK_PARAM(std::string, "input_layer_shape", "Input shapes",
                       verbose);
@@ -511,6 +779,10 @@ void BenchmarkTfLiteModel::LogParams() {
                       "Require full delegation", verbose);
   LOG_BENCHMARK_PARAM(bool, "enable_op_profiling", "Enable op profiling",
                       verbose);
+  LOG_BENCHMARK_PARAM(std::string, "op_profiling_output_mode",
+                      "Op profiling output mode.", verbose);
+  LOG_BENCHMARK_PARAM(std::string, "op_profiling_output_file",
+                      "Op profiling output file.", verbose);
   LOG_BENCHMARK_PARAM(int32_t, "max_profiling_buffer_entries",
                       "Max initial profiling buffer entries", verbose);
   LOG_BENCHMARK_PARAM(bool, "allow_dynamic_profiling_buffer_increase",
@@ -518,6 +790,10 @@ void BenchmarkTfLiteModel::LogParams() {
                       verbose);
   LOG_BENCHMARK_PARAM(std::string, "profiling_output_csv_file",
                       "CSV File to export profiling data to", verbose);
+  LOG_BENCHMARK_PARAM(bool, "export_model_runtime_info",
+                      "Enable Model Runtime Info Export", verbose);
+  LOG_BENCHMARK_PARAM(std::string, "model_runtime_info_output_file",
+                      "Proto File to export model runtime info to", verbose);
   LOG_BENCHMARK_PARAM(bool, "print_preinvoke_state",
                       "Print pre-invoke interpreter state", verbose);
   LOG_BENCHMARK_PARAM(bool, "print_postinvoke_state",
@@ -528,8 +804,13 @@ void BenchmarkTfLiteModel::LogParams() {
                       "Optimize memory usage for large tensors", verbose);
   LOG_BENCHMARK_PARAM(bool, "disable_delegate_clustering",
                       "Disable delegate clustering", verbose);
+  LOG_BENCHMARK_PARAM(bool, "enable_builtin_cast_constant_cache",
+                      "Constant CAST output cache", verbose);
   LOG_BENCHMARK_PARAM(std::string, "output_filepath",
                       "File path to export outputs layer to", verbose);
+  LOG_BENCHMARK_PARAM(std::string, "output_proto_filepath",
+                      "File path to export outputs layer as tf example to",
+                      verbose);
   LOG_BENCHMARK_PARAM(int32_t, "tensor_name_display_length",
                       "Tensor name display length", verbose);
   LOG_BENCHMARK_PARAM(int32_t, "tensor_type_display_length",
@@ -552,6 +833,31 @@ TfLiteStatus BenchmarkTfLiteModel::ValidateParams() {
     return kTfLiteError;
   }
 
+  if (params_.Get<bool>("enable_op_profiling")) {
+    bool found =
+        std::find(std::begin(kOpProfilingOutputModes),
+                  std::end(kOpProfilingOutputModes),
+                  params_.Get<std::string>("op_profiling_output_mode")) !=
+        std::end(kOpProfilingOutputModes);
+
+    if (!found) {
+      TFLITE_LOG(ERROR) << "Output mode"
+                        << params_.Get<std::string>("op_profiling_output_mode")
+                        << " is not supported. Supported values are: 'stdout', "
+                           "'csv' and 'proto'.";
+      return kTfLiteError;
+    }
+
+    if (!params_.Get<std::string>("profiling_output_csv_file").empty()) {
+      // Backward compatibility for profiling_output_csv_file.
+      params_.Set<std::string>("op_profiling_output_mode",
+                               kOpProfilingOutputModeCsv);
+      params_.Set<std::string>(
+          "op_profiling_output_file",
+          params_.Get<std::string>("profiling_output_csv_file"));
+    }
+  }
+
   return PopulateInputLayerInfo(
       params_.Get<std::string>("input_layer"),
       params_.Get<std::string>("input_layer_shape"),
@@ -560,10 +866,10 @@ TfLiteStatus BenchmarkTfLiteModel::ValidateParams() {
 }
 
 uint64_t BenchmarkTfLiteModel::ComputeInputBytes() {
-  TFLITE_TOOLS_CHECK(interpreter_);
+  TFLITE_TOOLS_CHECK(interpreter_runner_);
   uint64_t total_input_bytes = 0;
-  for (int input : interpreter_->inputs()) {
-    auto* t = interpreter_->tensor(input);
+  for (int input : interpreter_runner_->inputs()) {
+    auto* t = interpreter_runner_->tensor(input);
     total_input_bytes += t->bytes;
   }
   return total_input_bytes;
@@ -662,15 +968,15 @@ InputTensorData BenchmarkTfLiteModel::CreateRandomTensorData(
 TfLiteStatus BenchmarkTfLiteModel::PrepareInputData() {
   CleanUp();
 
-  // Note the corresponding relation between 'interpreter_inputs' and 'inputs_'
-  // (i.e. the specified input layer info) has been checked in
-  // BenchmarkTfLiteModel::Init() before calling this function. So, we simply
-  // use the corresponding input layer info to initialize the input data value
-  // properly.
-  auto interpreter_inputs = interpreter_->inputs();
-  for (int i = 0; i < interpreter_inputs.size(); ++i) {
-    int tensor_index = interpreter_inputs[i];
-    const TfLiteTensor& t = *(interpreter_->tensor(tensor_index));
+  // Note the corresponding relation between 'runner_inputs' and
+  // 'inputs_' (i.e. the specified input layer info) has been checked in
+  // BenchmarkTfLiteModel::Init() before calling this function. So, we
+  // simply use the corresponding input layer info to initialize the input
+  // data value properly.
+  const std::vector<int>& runner_inputs = interpreter_runner_->inputs();
+  for (int i = 0; i < runner_inputs.size(); ++i) {
+    int tensor_index = runner_inputs[i];
+    const TfLiteTensor& t = *(interpreter_runner_->tensor(tensor_index));
     const InputLayerInfo* input_layer_info = nullptr;
     // Note that when input layer parameters (i.e. --input_layer,
     // --input_layer_shape) are not specified, inputs_ is empty.
@@ -688,11 +994,11 @@ TfLiteStatus BenchmarkTfLiteModel::PrepareInputData() {
 }
 
 TfLiteStatus BenchmarkTfLiteModel::ResetInputsAndOutputs() {
-  auto interpreter_inputs = interpreter_->inputs();
+  const std::vector<int>& runner_inputs = interpreter_runner_->inputs();
   // Set the values of the input tensors from inputs_data_.
-  for (int j = 0; j < interpreter_inputs.size(); ++j) {
-    int i = interpreter_inputs[j];
-    TfLiteTensor* t = interpreter_->tensor(i);
+  for (int j = 0; j < runner_inputs.size(); ++j) {
+    int i = runner_inputs[j];
+    TfLiteTensor* t = interpreter_runner_->tensor(i);
     if (t->type == kTfLiteString) {
       if (inputs_data_[j].data) {
         static_cast<DynamicBuffer*>(inputs_data_[j].data.get())
@@ -700,7 +1006,8 @@ TfLiteStatus BenchmarkTfLiteModel::ResetInputsAndOutputs() {
       } else {
         tflite::DynamicBuffer buffer;
         FillRandomString(&buffer, t->dims, []() {
-          return "we're have some friends over saturday to hang out in the "
+          return "we're have some friends over saturday to hang out in "
+                 "the "
                  "yard";
         });
         buffer.WriteToTensor(t, /*new_shape=*/nullptr);
@@ -726,6 +1033,8 @@ TfLiteStatus BenchmarkTfLiteModel::InitInterpreter() {
       params_.Get<int32_t>("optimize_memory_for_large_tensors"));
   options.SetDisableDelegateClustering(
       params_.Get<bool>("disable_delegate_clustering"));
+  options.SetCacheConstantCastOp(
+      params_.Get<bool>("enable_builtin_cast_constant_cache"));
 
   tflite::InterpreterBuilder builder(*model_, *resolver, &options);
   if (builder.SetNumThreads(num_threads) != kTfLiteOk) {
@@ -758,6 +1067,20 @@ TfLiteStatus BenchmarkTfLiteModel::Init() {
   TF_LITE_ENSURE_STATUS(LoadModel());
   TF_LITE_ENSURE_STATUS(InitInterpreter());
 
+  if (params_.Get<bool>("list_signatures")) {
+    const std::vector<const std::string*>& keys =
+        interpreter_->signature_keys();
+    TFLITE_LOG(INFO) << "The Model contains " << keys.size()
+                     << " signature key(s).";
+    if (!keys.empty()) {
+      TFLITE_LOG(INFO) << "They are listed below: ";
+    }
+    for (const std::string* key : keys) {
+      TFLITE_LOG(INFO) << "-> Signature Key: " << *key;
+    }
+    return kTfLiteError;
+  }
+
   // Install profilers if necessary right after interpreter is created so that
   // any memory allocations inside the TFLite runtime could be recorded if the
   // installed profiler profile memory usage information.
@@ -778,12 +1101,59 @@ TfLiteStatus BenchmarkTfLiteModel::Init() {
   AddOwnedListener(std::unique_ptr<BenchmarkListener>(
       new InterpreterStatePrinter(interpreter_.get())));
 
+  if (params_.Get<bool>("export_model_runtime_info")) {
+    AddOwnedListener(std::unique_ptr<BenchmarkListener>(
+        new ModelRuntimeInfoListener(interpreter_.get())));
+  }
+
   interpreter_->SetAllowFp16PrecisionForFp32(params_.Get<bool>("allow_fp16"));
 
+  std::pair<TfLiteStatus, std::unique_ptr<BenchmarkInterpreterRunner>>
+      status_and_runner = BenchmarkInterpreterRunner::Create(
+          interpreter_.get(), params_.Get<std::string>("signature_to_run_for"));
+
+  TF_LITE_ENSURE_STATUS(status_and_runner.first);
+  interpreter_runner_ = std::move(status_and_runner.second);
+
+  const std::vector<int>& runner_inputs = interpreter_runner_->inputs();
+
+  if (!inputs_.empty()) {
+    TFLITE_TOOLS_CHECK_EQ(inputs_.size(), runner_inputs.size())
+        << "Inputs mismatch: Model inputs #:" << inputs_.size()
+        << " expected: " << runner_inputs.size();
+  }
+
+  // Check if the tensor names match, and log a warning if it doesn't.
+  for (int j = 0; j < inputs_.size(); ++j) {
+    const InputLayerInfo& input = inputs_[j];
+    int i = runner_inputs[j];
+    TfLiteTensor* t = interpreter_runner_->tensor(i);
+    if (input.name != t->name) {
+      TFLITE_LOG(WARN) << "Tensor # " << i << " is named " << t->name
+                       << " but flags call it " << input.name;
+    }
+
+    if (t->type != kTfLiteString && input.shape.size() != t->dims->size) {
+      TFLITE_LOG(ERROR) << "Input tensor #" << i << " should have "
+                        << t->dims->size << " dimensions!";
+      return kTfLiteError;
+    }
+  }
+
+  // Resize all non-string tensors.
+  for (int j = 0; j < inputs_.size(); ++j) {
+    const InputLayerInfo& input = inputs_[j];
+    int i = runner_inputs[j];
+    TfLiteTensor* t = interpreter_runner_->tensor(i);
+    if (t->type != kTfLiteString) {
+      interpreter_runner_->ResizeInputTensor(i, input.shape);
+    }
+  }
+
   owned_delegates_.clear();
 
-  // Contains all ids of TfLiteNodes that have been checked to see whether it's
-  // delegated or not.
+  // Contains all ids of TfLiteNodes that have been checked to see whether
+  // it's delegated or not.
   std::unordered_set<int> checked_node_ids;
   tools::ProvidedDelegateList delegate_providers(&params_);
   auto created_delegates = delegate_providers.CreateAllRankedDelegates();
@@ -791,9 +1161,9 @@ TfLiteStatus BenchmarkTfLiteModel::Init() {
       << "Going to apply " << created_delegates.size()
       << " delegates one after another.";
 
-  // If created_delegates is empty, 'require_full_delegation' flag will not be
-  // checked, thus CPU fallback will happen. Adding check here to avoid
-  // fallback in this situation.
+  // If created_delegates is empty, 'require_full_delegation' flag will
+  // not be checked, thus CPU fallback will happen. Adding check here to
+  // avoid fallback in this situation.
   if (created_delegates.empty() &&
       params_.Get<bool>("require_full_delegation")) {
     TFLITE_LOG(ERROR) << "Disallowed CPU fallback detected.";
@@ -805,10 +1175,10 @@ TfLiteStatus BenchmarkTfLiteModel::Init() {
     TFLITE_TOOLS_CHECK(delegate != nullptr)
         << "The created delegate by the delegate provider should not be "
            "nullptr!";
-    // The interpreter becomes dependent on the delegate once the delegate is
-    // used, so the order of destruction must be interpreter first, delegate
-    // later.
-    // Moving the delegate to a list of owned delegates to guarantee that.
+    // The interpreter becomes dependent on the delegate once the delegate
+    // is used, so the order of destruction must be interpreter first,
+    // delegate later. Moving the delegate to a list of owned delegates to
+    // guarantee that.
     owned_delegates_.emplace_back(std::move(created_delegate.delegate));
     if (interpreter_->ModifyGraphWithDelegate(delegate) != kTfLiteOk) {
       TFLITE_LOG(ERROR) << "Failed to apply " << delegate_provider->GetName()
@@ -818,25 +1188,27 @@ TfLiteStatus BenchmarkTfLiteModel::Init() {
       // Ideally, such delegate info should already be computed when the
       // delegate is being applied to the model graph.
       int num_delegated_kernels = 0;
-      for (int i = 0; i < interpreter_->execution_plan().size(); ++i) {
-        int node_id = interpreter_->execution_plan()[i];
+      for (int i = 0; i < interpreter_runner_->execution_plan().size(); ++i) {
+        int node_id = interpreter_runner_->execution_plan()[i];
         if (checked_node_ids.find(node_id) != checked_node_ids.end()) {
           continue;
         }
         const TfLiteNode& node =
-            interpreter_->node_and_registration(node_id)->first;
+            interpreter_runner_->node_and_registration(node_id)->first;
 
-        // Note that the 'delegate' here could be an ExternalDelegateWrapper
-        // object that wraps an actual external delegate, in which case,
-        // 'node.delegate' will be different from 'delegate' because
-        // 'node.delegate' refers to the actual external delegate.
+        // Note that the 'delegate' here could be an
+        // ExternalDelegateWrapper object that wraps an actual external
+        // delegate, in which case, 'node.delegate' will be different from
+        // 'delegate' because 'node.delegate' refers to the actual
+        // external delegate.
         if (node.delegate != nullptr) {
           num_delegated_kernels++;
           checked_node_ids.insert(node_id);
         }
       }
-      bool fully_delegated = (num_delegated_kernels == 1 &&
-                              interpreter_->execution_plan().size() == 1);
+      bool fully_delegated =
+          (num_delegated_kernels == 1 &&
+           interpreter_runner_->execution_plan().size() == 1);
 
       if (params_.Get<bool>("require_full_delegation") && !fully_delegated) {
         TFLITE_LOG(ERROR) << "Disallowed CPU fallback detected.";
@@ -854,58 +1226,24 @@ TfLiteStatus BenchmarkTfLiteModel::Init() {
                          << " executed by the delegate w/ "
                          << num_delegated_kernels << " delegate kernels.";
       } else {
-        TFLITE_LOG(INFO)
-            << "Though " << delegate_provider->GetName()
-            << " delegate is explicitly applied, the model graph will not be"
-            << " executed by the delegate.";
+        TFLITE_LOG(INFO) << "Though " << delegate_provider->GetName()
+                         << " delegate is explicitly applied, the model "
+                            "graph will not be"
+                         << " executed by the delegate.";
       }
     }
   }
 
-  auto interpreter_inputs = interpreter_->inputs();
-
-  if (!inputs_.empty()) {
-    TFLITE_TOOLS_CHECK_EQ(inputs_.size(), interpreter_inputs.size())
-        << "Inputs mismatch: Model inputs #:" << inputs_.size()
-        << " expected: " << interpreter_inputs.size();
-  }
-
-  // Check if the tensor names match, and log a warning if it doesn't.
-  for (int j = 0; j < inputs_.size(); ++j) {
-    const InputLayerInfo& input = inputs_[j];
-    int i = interpreter_inputs[j];
-    TfLiteTensor* t = interpreter_->tensor(i);
-    if (input.name != t->name) {
-      TFLITE_LOG(WARN) << "Tensor # " << i << " is named " << t->name
-                       << " but flags call it " << input.name;
-    }
-
-    if (t->type != kTfLiteString && input.shape.size() != t->dims->size) {
-      TFLITE_LOG(ERROR) << "Input tensor #" << i << " should have "
-                        << t->dims->size << " dimensions!";
-      return kTfLiteError;
-    }
-  }
-
-  // Resize all non-string tensors.
-  for (int j = 0; j < inputs_.size(); ++j) {
-    const InputLayerInfo& input = inputs_[j];
-    int i = interpreter_inputs[j];
-    TfLiteTensor* t = interpreter_->tensor(i);
-    if (t->type != kTfLiteString) {
-      interpreter_->ResizeInputTensor(i, input.shape);
-    }
-  }
-
-  if (interpreter_->AllocateTensors() != kTfLiteOk) {
+  if (interpreter_runner_->AllocateTensors() != kTfLiteOk) {
     TFLITE_LOG(ERROR) << "Failed to allocate tensors!";
     return kTfLiteError;
   }
 
   AddOwnedListener(
       std::unique_ptr<BenchmarkListener>(new RuyProfileListener()));
-  AddOwnedListener(
-      std::unique_ptr<BenchmarkListener>(new OutputSaver(interpreter_.get())));
+
+  AddOwnedListener(std::unique_ptr<BenchmarkListener>(
+      new OutputSaver(interpreter_runner_.get())));
 
   return kTfLiteOk;
 }
@@ -935,7 +1273,7 @@ std::unique_ptr<tflite::OpResolver> BenchmarkTfLiteModel::GetOpResolver()
   tflite::ops::builtin::BuiltinOpResolver* resolver = nullptr;
   // When --use_xnnpack is explicitly set to false, skip applying the default
   // XNNPACK delegate in TfLite runtime so that the original execution path
-  // based on the unmodified model graph is still excercised.
+  // based on the unmodified model graph is still exercised.
   if (params_.HasParam("use_xnnpack") &&
       params_.HasValueSet<bool>("use_xnnpack") &&
       !params_.Get<bool>("use_xnnpack")) {
@@ -955,12 +1293,14 @@ BenchmarkTfLiteModel::MayCreateProfilingListener() const {
   return std::unique_ptr<BenchmarkListener>(new ProfilingListener(
       interpreter_.get(), params_.Get<int32_t>("max_profiling_buffer_entries"),
       params_.Get<bool>("allow_dynamic_profiling_buffer_increase"),
-      params_.Get<std::string>("profiling_output_csv_file"),
+      params_.Get<std::string>("op_profiling_output_file"),
       CreateProfileSummaryFormatter(
-          !params_.Get<std::string>("profiling_output_csv_file").empty())));
+          params_.Get<std::string>("op_profiling_output_mode"))));
 }
 
-TfLiteStatus BenchmarkTfLiteModel::RunImpl() { return interpreter_->Invoke(); }
+TfLiteStatus BenchmarkTfLiteModel::RunImpl() {
+  return interpreter_runner_->Invoke();
+}
 
 }  // namespace benchmark
 }  // namespace tflite
