@@ -23,8 +23,6 @@ public interface BenchmarkAccuracy {
    *
    * @param context the context to use for finding the test models and exporting reports
    * @param tfliteSettingsJsonFiles the list of paths to delegate JSON configurations
-   * @return {@code true} if the benchmark was successfully initialized and executed. Otherwise,
-   *     returns {@code false}.
    */
-  boolean benchmark(Context context, String[] tfliteSettingsJsonFiles);
+  void benchmark(Context context, String[] tfliteSettingsJsonFiles);
 }
