@@ -12,7 +12,11 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
+#include <wrapper_internal_exception_macros.h>
+
+#include <cstdio>
 #include <string>
+#include <unordered_map>
 
 #include "tensorflow/lite/tools/optimize/modify_model_interface.h"
 //
