@@ -15,28 +15,35 @@ limitations under the License.
 
 #include "tensorflow/lite/tools/serialization/writer_lib.h"
 
+#include <cstdint>
 #include <cstdlib>
+#include <cstring>
 #include <fstream>
+#include <ios>
 #include <memory>
 #include <numeric>
 #include <sstream>
 #include <string>
-#include <tuple>
 #include <vector>
 
-#include <gmock/gmock.h>
 #include <gtest/gtest.h>
+#include "absl/log/check.h"
+#include "tensorflow/compiler/mlir/lite/schema/mutable/schema_generated.h"
 #include "tensorflow/lite/builtin_ops.h"
+#include "tensorflow/lite/context_util.h"
 #include "tensorflow/lite/core/c/builtin_op_data.h"
 #include "tensorflow/lite/core/c/c_api_types.h"
 #include "tensorflow/lite/core/c/common.h"
 #include "tensorflow/lite/core/interpreter.h"
+#include "tensorflow/lite/core/interpreter_builder.h"
 #include "tensorflow/lite/core/kernels/builtin_op_kernels.h"
 #include "tensorflow/lite/core/kernels/register.h"
-#include "tensorflow/lite/core/model.h"
+#include "tensorflow/lite/core/subgraph.h"
 #include "tensorflow/lite/kernels/subgraph_test_util.h"
+#include "tensorflow/lite/model_builder.h"
 #include "tensorflow/lite/schema/schema_generated.h"
-#include "tensorflow/lite/testing/util.h"
+#include "tensorflow/lite/util.h"
+#include "tsl/platform/logging.h"
 
 namespace tflite {
 
