@@ -14,19 +14,18 @@ limitations under the License.
 ==============================================================================*/
 #include "tensorflow/lite/tools/versioning/op_version.h"
 
-#include <algorithm>
-#include <string>
-#include <utility>
+#include <cstdint>
+#include <cstdlib>
 #include <vector>
 
-#include "absl/memory/memory.h"
-#include "absl/strings/numbers.h"
-#include "absl/strings/str_split.h"
+#include "absl/log/log.h"
+#include "tensorflow/compiler/mlir/lite/schema/mutable/schema_generated.h"
+#include "tensorflow/compiler/mlir/lite/schema/schema_generated.h"
 #include "tensorflow/core/platform/logging.h"
 #include "tensorflow/lite/builtin_op_data.h"
+#include "tensorflow/lite/core/c/builtin_op_data.h"
 #include "tensorflow/lite/core/c/c_api_types.h"
 #include "tensorflow/lite/kernels/internal/compatibility.h"
-#include "tensorflow/lite/schema/schema_generated.h"
 #include "tensorflow/lite/schema/schema_utils.h"
 
 namespace tflite {
@@ -53,7 +52,19 @@ int GetInputMaxDims(const OpSignature& op_sig) {
 
 int GetBuiltinOperatorVersion(const OpSignature& op_sig) {
   switch (op_sig.op) {
-    case BuiltinOperator_CONV_2D:
+    case BuiltinOperator_CONV_2D: {
+      if (op_sig.inputs.at(0).type == kTfLiteInt16 &&
+          op_sig.inputs.at(1).type == kTfLiteInt8 &&
+          op_sig.outputs.at(0).type == kTfLiteInt16) {
+        // `quantized_bias_type` is supported at version 8.
+        auto conv_params =
+            reinterpret_cast<TfLiteConvParams*>(op_sig.builtin_data);
+        TFLITE_DCHECK(conv_params != nullptr);
+        if (conv_params->quantized_bias_type) {
+          return 8;
+        }
+      }
+
       if (op_sig.ext_options.conv_2d.is_grouped_convolution) {
         return 6;
       }
@@ -90,7 +101,7 @@ int GetBuiltinOperatorVersion(const OpSignature& op_sig) {
         return 2;
       }
       return 1;
-
+    }
     case BuiltinOperator_DEPTHWISE_CONV_2D: {
       // If the op accepts int16, we return version 5.
       if (op_sig.inputs.at(0).type == kTfLiteInt16 &&
@@ -135,6 +146,14 @@ int GetBuiltinOperatorVersion(const OpSignature& op_sig) {
       return 1;
     }
 
+    case BuiltinOperator_EMBEDDING_LOOKUP: {
+      if (op_sig.inputs.at(1).type == kTfLiteInt4 ||
+          op_sig.ext_options.embedding_lookup.is_per_channel_quantized) {
+        return 4;
+      }
+      return 1;
+    }
+
     case BuiltinOperator_FAKE_QUANT: {
       auto fake_quant_params =
           reinterpret_cast<TfLiteFakeQuantParams*>(op_sig.builtin_data);
@@ -155,6 +174,32 @@ int GetBuiltinOperatorVersion(const OpSignature& op_sig) {
       // | Quantized Int8  |                  4 |                        4 |
       // +-----------------+--------------------+--------------------------+
 
+      auto fully_connected_params =
+          reinterpret_cast<TfLiteFullyConnectedParams*>(op_sig.builtin_data);
+      TFLITE_DCHECK(fully_connected_params != nullptr);
+
+      if (op_sig.inputs.at(0).type == kTfLiteInt16 &&
+          op_sig.inputs.at(1).type == kTfLiteInt4 &&
+          op_sig.outputs.at(0).type == kTfLiteInt16) {
+        return 13;
+      }
+
+      if (op_sig.inputs.at(0).type == kTfLiteFloat32 &&
+          op_sig.inputs.at(1).type == kTfLiteInt8 &&
+          op_sig.outputs.at(0).type == kTfLiteFloat32 &&
+          op_sig.ext_options.fully_connected.is_per_channel_quantized) {
+        return 12;
+      }
+
+      if (op_sig.inputs.at(0).type == kTfLiteInt16 &&
+          op_sig.inputs.at(1).type == kTfLiteInt8 &&
+          op_sig.outputs.at(0).type == kTfLiteInt16) {
+        // `quantized_bias_type` is supported at version 11.
+        if (fully_connected_params->quantized_bias_type) {
+          return 11;
+        }
+      }
+
       // FullyConnected with sparse weight is supported at version 8.
       if (op_sig.ext_options.fully_connected.sparse_weight) {
         return 8;
@@ -172,9 +217,6 @@ int GetBuiltinOperatorVersion(const OpSignature& op_sig) {
       if (op_sig.inputs.size() == 2) {
         return 6;
       }
-      auto fully_connected_params =
-          reinterpret_cast<TfLiteFullyConnectedParams*>(op_sig.builtin_data);
-      TFLITE_DCHECK(fully_connected_params != nullptr);
       // `keep_num_dims` is supported at version 5.
       if (fully_connected_params->keep_num_dims) {
         return 5;
@@ -216,6 +258,9 @@ int GetBuiltinOperatorVersion(const OpSignature& op_sig) {
     }
 
     case BuiltinOperator_GATHER: {
+      if (op_sig.inputs.at(0).type == kTfLiteInt4) {
+        return 7;
+      }
       if (op_sig.inputs.at(1).type == kTfLiteInt16) {
         return 6;
       }
@@ -224,7 +269,6 @@ int GetBuiltinOperatorVersion(const OpSignature& op_sig) {
       if (gather_params && gather_params->batch_dims != 0) {
         return 5;
       }
-
       if (op_sig.inputs.at(0).type == kTfLiteInt16) {
         return 4;
       }
@@ -312,6 +356,9 @@ int GetBuiltinOperatorVersion(const OpSignature& op_sig) {
       return 1;
 
     case BuiltinOperator_TRANSPOSE:
+      if (op_sig.inputs.at(0).type == kTfLiteInt4) {
+        return 7;
+      }
       if (op_sig.inputs.at(0).dims.size() > 5) {
         return 6;
       }
@@ -333,6 +380,15 @@ int GetBuiltinOperatorVersion(const OpSignature& op_sig) {
     case BuiltinOperator_TRANSPOSE_CONV: {
       auto transpose_conv_params =
           reinterpret_cast<TfLiteTransposeConvParams*>(op_sig.builtin_data);
+      if (op_sig.inputs.at(0).type == kTfLiteInt16 &&
+          op_sig.inputs.at(1).type == kTfLiteInt8 &&
+          op_sig.outputs.at(0).type == kTfLiteInt16) {
+        // `quantized_bias_type` is supported at version 5.
+        TFLITE_DCHECK(transpose_conv_params != nullptr);
+        if (transpose_conv_params->quantized_bias_type) {
+          return 5;
+        }
+      }
 
       // TransposeConvOp has fused activation function from version 4.
       if (transpose_conv_params != nullptr &&
@@ -686,6 +742,9 @@ int GetBuiltinOperatorVersion(const OpSignature& op_sig) {
     }
 
     case BuiltinOperator_GATHER_ND:
+      if (op_sig.inputs.at(0).type == kTfLiteBool) {
+        return 5;
+      }
       if (op_sig.inputs.at(1).type == kTfLiteInt16) {
         return 4;
       }
@@ -788,6 +847,9 @@ int GetBuiltinOperatorVersion(const OpSignature& op_sig) {
 
     case BuiltinOperator_PAD:
     case BuiltinOperator_PADV2:
+      if (op_sig.inputs.at(0).type == kTfLiteBool) {
+        return 5;
+      }
       if (op_sig.inputs.at(0).dims.size() > 4) {
         return 4;
       }
@@ -818,6 +880,7 @@ int GetBuiltinOperatorVersion(const OpSignature& op_sig) {
     case BuiltinOperator_REDUCE_MAX:
     case BuiltinOperator_REDUCE_MIN:
     case BuiltinOperator_RELU6:
+    case BuiltinOperator_RSQRT:
       // In case of int16 inputs, the version is 3.
       if (op_sig.inputs.at(0).type == kTfLiteInt16) {
         return 3;
@@ -962,7 +1025,6 @@ int GetBuiltinOperatorVersion(const OpSignature& op_sig) {
     case BuiltinOperator_LOG_SOFTMAX:
     case BuiltinOperator_GREATER:
     case BuiltinOperator_LESS_EQUAL:
-    case BuiltinOperator_RSQRT:
     case BuiltinOperator_SQUARED_DIFFERENCE:
     case BuiltinOperator_DEPTH_TO_SPACE:
       if (op_sig.inputs.at(0).type == kTfLiteInt8) {
@@ -981,13 +1043,22 @@ int GetBuiltinOperatorVersion(const OpSignature& op_sig) {
       return 1;
 
     case BuiltinOperator_EXP:
+    case BuiltinOperator_LOG:
     case BuiltinOperator_REDUCE_PROD:
       if (op_sig.inputs.at(0).type == kTfLiteInt8 ||
           op_sig.inputs.at(0).type == kTfLiteInt16) {
         return 2;
       }
       return 1;
-
+    case BuiltinOperator_DYNAMIC_UPDATE_SLICE:
+      if (op_sig.inputs.at(0).type == kTfLiteInt16) {
+        return 4;
+      } else if (op_sig.inputs.at(0).type == kTfLiteFloat16) {
+        return 3;
+      } else if (op_sig.inputs.at(2).type == kTfLiteInt64) {
+        return 2;
+      }
+      return 1;
     // The version one of broadcast to op won't be not supported since the
     // version one was rollbacked and the builtin op code number has been
     // changed because of builtin op code shortage problem.
@@ -999,10 +1070,16 @@ int GetBuiltinOperatorVersion(const OpSignature& op_sig) {
       }
       return 2;
     case BuiltinOperator_CAST:
-      if (op_sig.inputs.at(0).type == kTfLiteFloat64 ||
-          op_sig.outputs.at(0).type == kTfLiteFloat64 ||
-          op_sig.inputs.at(0).type == kTfLiteFloat16 ||
-          op_sig.outputs.at(0).type == kTfLiteFloat16) {
+      if (op_sig.inputs.at(0).type == kTfLiteBFloat16 ||
+          op_sig.outputs.at(0).type == kTfLiteBFloat16) {
+        return 7;
+      } else if (op_sig.inputs.at(0).type == kTfLiteInt4 &&
+                 op_sig.outputs.at(0).type == kTfLiteFloat32) {
+        return 6;
+      } else if (op_sig.inputs.at(0).type == kTfLiteFloat64 ||
+                 op_sig.outputs.at(0).type == kTfLiteFloat64 ||
+                 op_sig.inputs.at(0).type == kTfLiteFloat16 ||
+                 op_sig.outputs.at(0).type == kTfLiteFloat16) {
         return 5;
       } else if (op_sig.inputs.at(0).type == kTfLiteUInt16 ||
                  op_sig.outputs.at(0).type == kTfLiteUInt16) {
