@@ -15,9 +15,10 @@ limitations under the License.
 #ifndef TENSORFLOW_LITE_TOOLS_VERSIONING_OP_VERSION_H_
 #define TENSORFLOW_LITE_TOOLS_VERSIONING_OP_VERSION_H_
 
+#include <cstdint>
 #include <vector>
 
-#include "tensorflow/lite/schema/mutable/schema_generated.h"
+#include "tensorflow/compiler/mlir/lite/schema/mutable/schema_generated.h"
 #include "tensorflow/lite/tools/versioning/op_signature.h"
 
 namespace tflite {
