@@ -1330,7 +1330,7 @@ class SessionTest(test_util.TensorFlowTestCase):
       sess.close()
 
   @test_util.run_v1_only('b/120545219')
-  def testMultipleInteractiveSessionsWarning(self):
+  def testMultipleInteractiveSessionsError(self):
     # Reinitialize the global state to ensure that the expected warnings will
     # be emitted.
     session.InteractiveSession._active_session_count = 0  # pylint: disable=protected-access
@@ -1339,21 +1339,27 @@ class SessionTest(test_util.TensorFlowTestCase):
     sess.run(constant_op.constant(4.0))  # Run so that the session is "opened".
     sess.close()
     # Opening and closing interactive sessions serially should not warn.
-    with warnings.catch_warnings(record=True) as w:
+    with self.assertNoLogs(level='ERROR'):
       sess = session.InteractiveSession()
       sess.close()
-    self.assertEqual(0, len(w))
 
-    with warnings.catch_warnings(record=True) as w:
+    with self.assertNoLogs(level='ERROR'):
       sess = session.InteractiveSession()
-    self.assertEqual(0, len(w))
-    with warnings.catch_warnings(record=True) as w:
+
+    with self.assertLogs(level='ERROR') as log_output:
       sess2 = session.InteractiveSession()
-    self.assertEqual(1, len(w))
-    self.assertIn('An interactive session is already active. This can cause '
-                  'out-of-memory errors in some cases. You must explicitly '
-                  'call `InteractiveSession.close()` to release resources '
-                  'held by the other session(s).', str(w[0].message))
+
+    self.assertLen(log_output.output, 1)
+
+    self.assertIn(
+        'An interactive session is already active. This can cause'
+        ' out-of-memory errors or some other unexpected errors (due to'
+        ' the unpredictable timing of garbage collection) in some cases.'
+        ' You must explicitly call `InteractiveSession.close()` to release'
+        ' resources held by the other session(s). Please use `tf.Session()`'
+        ' if you intend to productionize.',
+        log_output.output[0],
+    )
     sess2.close()
     sess.close()
 
