@@ -13,10 +13,10 @@
 # limitations under the License.
 # ==============================================================================
 """Fault tolerance tests for tf.data service snapshots."""
+
 import collections
 import os
 import pathlib
-import tempfile
 import time
 
 from absl.testing import parameterized
@@ -26,15 +26,10 @@ from tensorflow.python.data.experimental.ops import distributed_save_op
 from tensorflow.python.data.experimental.service import _pywrap_snapshot_utils
 from tensorflow.python.data.kernel_tests import test_base
 from tensorflow.python.data.ops import dataset_ops
-from tensorflow.python.data.ops import test_mode
 from tensorflow.python.framework import combinations
 from tensorflow.python.framework import errors
 from tensorflow.python.platform import test
 
-# Enum value for `SnapshotStreamInfo` states.
-_ORPHAN = 2
-_DONE = 4
-
 
 def write_file(path):
   os.makedirs(os.path.dirname(path), exist_ok=True)
@@ -88,7 +83,9 @@ def snapshots_are_done(paths):
   return all([snapshot_is_done(path) for path in paths])
 
 
-def wait_for_snapshots(paths, f=lambda: None):
+def wait_for_snapshot(paths, f=lambda: None):
+  if isinstance(paths, str):
+    paths = [paths]
   while not all([snapshot_is_done(path) or snapshot_has_error(path)
                  for path in paths]):
     f()
@@ -99,102 +96,77 @@ class SnapshotFtTest(data_service_test_base.TestBase, parameterized.TestCase):
 
   maxDiff = None
 
-  def setUp(self):
-    super().setUp()
-    self._path = os.path.join(
-        tempfile.mkdtemp(dir=self.get_temp_dir()),
-        "snapshot_ft_test",
-    )
-    # TODO(b/268586560): Enable `warm_start` for `snapshot_ft_test`.
-    test_mode.toggle_test_mode(False)
-
-  # This "manual" setup function is needed due to some bad interaction between
-  # `setUp` and `combinations` that causes the dataset to be out-of-scope.
-  # It additionally can't take in a `Dataset` as input.
-  def setup(self, num_workers=1, ds_size=10, num_sources=1):
-    ds = dataset_ops.Dataset.range(ds_size)
-    if num_sources > 1:
-      ds = dataset_ops.Dataset.zip((ds,) * num_sources)
-    cluster = data_service_test_base.TestCluster(num_workers=num_workers)
-    self.evaluate(distributed_save_op.distributed_save(
-        ds, self._path, cluster.dispatcher_address()
-    ))
-    return cluster, ds
-
-  def splits_dir(self, stream_idx=0, worker=0):
-    stream_name = f"stream_{stream_idx}"
-    self._make_stream_dir(stream_name, worker=worker)
-    return os.path.join(
-        self._path,
-        "streams",
-        stream_name,
-        "splits",
-    )
-
-  def source_dir(self, stream_idx=0, source_idx=0, worker=0):
-    return os.path.join(
-        self.splits_dir(stream_idx, worker=worker),
-        f"source_{source_idx}",
-        "repetition_0",
-    )
-
-  def _make_stream_dir(self, stream_name, worker=0):
-    stream_dir = os.path.join(self._path, "streams", stream_name)
-    os.makedirs(stream_dir)
-    pathlib.Path(os.path.join(stream_dir, "owner_worker")).write_text(
-        f"{worker}"
-    )
-
   @combinations.generate(test_base.default_test_combinations())
   def testSnapshotRecoverySucceeds(self):
-    cluster, _ = self.setup()
+    cluster = data_service_test_base.TestCluster(num_workers=1)
+    snapshot_dir = data_service_test_base.TempDir()
+    dataset = self._get_dataset()
+    self.evaluate(distributed_save_op.distributed_save(
+        dataset, snapshot_dir.full_path, cluster.dispatcher_address()))
     cluster.restart_dispatcher()
 
   @combinations.generate(test_base.default_test_combinations())
   def testSnapshotRecoveryBlocksOverwrite(self):
-    cluster, ds = self.setup()
+    cluster = data_service_test_base.TestCluster(num_workers=1)
+    snapshot_dir = data_service_test_base.TempDir()
+    dataset = self._get_dataset()
+    self.evaluate(distributed_save_op.distributed_save(
+        dataset, snapshot_dir.full_path, cluster.dispatcher_address()))
+
     cluster.restart_dispatcher()
     with self.assertRaisesRegex(
-        errors.AlreadyExistsError, "is already started or completed"
-    ):
+        errors.AlreadyExistsError, "is already started or completed"):
       self.evaluate(distributed_save_op.distributed_save(
-          ds, self._path, cluster.dispatcher_address()
-      ))
+          dataset, snapshot_dir.full_path, cluster.dispatcher_address()))
 
   # TODO(b/250921378): Figure out why tsan times out when there is a worker.
   @combinations.generate(
       combinations.times(
           test_base.default_test_combinations(),
           combinations.combine(
-              bad_stream_dir_name=["stream_", "stream_x", "stream_-1"]
-          ),
-      )
-  )
+              bad_stream_dir_name=["stream_", "stream_x", "stream_-1"])))
   def testSnapshotRecoveryFailsWithBadStreamName(self, bad_stream_dir_name):
-    cluster, _ = self.setup(num_workers=0)
-    self._make_stream_dir(bad_stream_dir_name)
-    with self.assertRaisesRegex(ValueError, "Can't parse"):
+    cluster = data_service_test_base.TestCluster(num_workers=0)
+    snapshot_dir = data_service_test_base.TempDir()
+    self.evaluate(distributed_save_op.distributed_save(
+        self._get_dataset(),
+        snapshot_dir.full_path,
+        cluster.dispatcher_address()))
+
+    self._make_stream_dir(snapshot_dir.full_path, bad_stream_dir_name)
+    with self.assertRaisesRegex(RuntimeError, "Can't parse"):
       cluster.restart_dispatcher()
 
   @combinations.generate(
       combinations.times(
           test_base.default_test_combinations(),
           combinations.combine(
-              bad_source_dir_name=["source_", "source_x", "source_-1"]
-          ),
-      )
-  )
+              bad_source_dir_name=["source_", "source_x", "source_-1"])))
   def testSnapshotRecoveryFailsWithBadSourceName(self, bad_source_dir_name):
-    cluster, _ = self.setup(num_workers=0)
-    os.makedirs(os.path.join(self.splits_dir(), bad_source_dir_name))
-    with self.assertRaisesRegex(ValueError, "Can't parse"):
+    cluster = data_service_test_base.TestCluster(num_workers=0)
+    snapshot_dir = data_service_test_base.TempDir()
+    self.evaluate(distributed_save_op.distributed_save(
+        self._get_dataset(),
+        snapshot_dir.full_path,
+        cluster.dispatcher_address()))
+
+    os.makedirs(os.path.join(self._splits_dir(snapshot_dir.full_path),
+                             bad_source_dir_name))
+    with self.assertRaisesRegex(RuntimeError, "Can't parse"):
       cluster.restart_dispatcher()
 
   @combinations.generate(test_base.default_test_combinations())
   def testSnapshotRecoveryFailsWithOutOfBoundsSourceName(self):
-    cluster, _ = self.setup(num_workers=0)
-    os.makedirs(os.path.join(self.splits_dir(), "source_1"))
-    with self.assertRaisesRegex(ValueError, "Found conflict"):
+    cluster = data_service_test_base.TestCluster(num_workers=0)
+    snapshot_dir = data_service_test_base.TempDir()
+    self.evaluate(distributed_save_op.distributed_save(
+        self._get_dataset(),
+        snapshot_dir.full_path,
+        cluster.dispatcher_address()))
+
+    os.makedirs(os.path.join(self._splits_dir(snapshot_dir.full_path),
+                             "source_1"))
+    with self.assertRaisesRegex(RuntimeError, "Found conflict"):
       cluster.restart_dispatcher()
 
   @combinations.generate(
@@ -206,56 +178,94 @@ class SnapshotFtTest(data_service_test_base.TestBase, parameterized.TestCase):
                   "split_x_0",
                   "split_-1_0",
                   "split_0_x",
-                  "split_0_-1",
-              ]
-          ),
-      )
-  )
+                  "split_0_-1"])))
   def testSnapshotRecoveryFailsWithBadSplitNames(self, bad_split_filename):
-    cluster, _ = self.setup(num_workers=0)
-    write_file(os.path.join(self.source_dir(), bad_split_filename))
+    cluster = data_service_test_base.TestCluster(num_workers=0)
+    snapshot_dir = data_service_test_base.TempDir()
+    self.evaluate(distributed_save_op.distributed_save(
+        self._get_dataset(),
+        snapshot_dir.full_path,
+        cluster.dispatcher_address()))
+
+    write_file(os.path.join(self._source_dir(snapshot_dir.full_path),
+                            bad_split_filename))
     with self.assertRaisesRegex(
-        ValueError, "Expected split_<local_split_index>_<global_split_index>"):
+        ValueError,
+        "Expected split_<local_split_index>_<global_split_index>"):
       cluster.restart_dispatcher()
 
   @combinations.generate(test_base.default_test_combinations())
   def testSnapshotRecoveryFailsWithOutOfOrderSplitName(self):
-    cluster, _ = self.setup(num_workers=0)
-    write_file(os.path.join(self.source_dir(), "split_1_0"))
+    cluster = data_service_test_base.TestCluster(num_workers=0)
+    snapshot_dir = data_service_test_base.TempDir()
+    self.evaluate(distributed_save_op.distributed_save(
+        self._get_dataset(),
+        snapshot_dir.full_path,
+        cluster.dispatcher_address()))
+
+    write_file(os.path.join(self._source_dir(snapshot_dir.full_path),
+                            "split_1_0"))
     with self.assertRaisesRegex(
-        ValueError, "The local split index 1 exceeds the global split index 0"):
+        ValueError,
+        "The local split index 1 exceeds the global split index 0"):
       cluster.restart_dispatcher()
 
   @combinations.generate(test_base.default_test_combinations())
   def testSnapshotRecoveryFailsWithMissingGlobalIndexInSplitNames(self):
-    cluster, _ = self.setup(num_workers=0)
-    write_file(os.path.join(self.source_dir(), "split_0_1"))
-    with self.assertRaisesRegex(ValueError, "Found missing global"):
+    cluster = data_service_test_base.TestCluster(num_workers=0)
+    snapshot_dir = data_service_test_base.TempDir()
+    self.evaluate(distributed_save_op.distributed_save(
+        self._get_dataset(),
+        snapshot_dir.full_path,
+        cluster.dispatcher_address()))
+
+    write_file(os.path.join(self._source_dir(snapshot_dir.full_path),
+                            "split_0_1"))
+    with self.assertRaisesRegex(RuntimeError, "Found missing global"):
       cluster.restart_dispatcher()
 
   @combinations.generate(test_base.default_test_combinations())
   def testSnapshotRecoveryFailsWithDuplicateGlobalIndexInSplitName(self):
-    cluster, _ = self.setup(num_workers=0)
-    write_file(os.path.join(self.source_dir(stream_idx=0), "split_0_1"))
-    write_file(
-        os.path.join(self.source_dir(stream_idx=1, worker=1), "split_0_1")
-    )
-    with self.assertRaisesRegex(ValueError, "Found duplicate global"):
+    cluster = data_service_test_base.TestCluster(num_workers=0)
+    snapshot_dir = data_service_test_base.TempDir()
+    self.evaluate(distributed_save_op.distributed_save(
+        self._get_dataset(),
+        snapshot_dir.full_path,
+        cluster.dispatcher_address()))
+
+    write_file(os.path.join(self._source_dir(
+        snapshot_dir.full_path, stream_idx=0), "split_0_1"))
+    write_file(os.path.join(self._source_dir(
+        snapshot_dir.full_path, stream_idx=1, worker=1), "split_0_1"))
+    with self.assertRaisesRegex(RuntimeError, "Found duplicate global"):
       cluster.restart_dispatcher()
 
   @combinations.generate(test_base.default_test_combinations())
   def testSnapshotRecoveryFailsWithDuplicateWorkerAssignment(self):
-    cluster, _ = self.setup(num_workers=0)
-    write_file(os.path.join(self.source_dir(stream_idx=0), "split_0_1"))
-    write_file(os.path.join(self.source_dir(stream_idx=1), "split_0_1"))
-    with self.assertRaisesRegex(ValueError, "worker is already assigned"):
+    cluster = data_service_test_base.TestCluster(num_workers=0)
+    snapshot_dir = data_service_test_base.TempDir()
+    self.evaluate(distributed_save_op.distributed_save(
+        self._get_dataset(),
+        snapshot_dir.full_path,
+        cluster.dispatcher_address()))
+
+    write_file(os.path.join(
+        self._source_dir(snapshot_dir.full_path, stream_idx=0), "split_0_1"))
+    write_file(os.path.join(
+        self._source_dir(snapshot_dir.full_path, stream_idx=1), "split_0_1"))
+    with self.assertRaisesRegex(RuntimeError, "worker is already assigned"):
       cluster.restart_dispatcher()
 
   @combinations.generate(test_base.default_test_combinations())
   def testStreamsReassignedAfterDispatcherRestart(self):
     n = 5
-    cluster, _ = self.setup(num_workers=n, ds_size=10000)
-    get_streams = lambda: cluster.snapshot_streams(self._path)
+    cluster = data_service_test_base.TestCluster(num_workers=n)
+    snapshot_dir = data_service_test_base.TempDir()
+    dataset = self._get_dataset(dataset_range=10000)
+    self.evaluate(distributed_save_op.distributed_save(
+        dataset, snapshot_dir.full_path, cluster.dispatcher_address()))
+
+    get_streams = lambda: cluster.snapshot_streams(snapshot_dir.full_path)
     while len(get_streams()) != n:
       time.sleep(0.1)
     cluster.restart_dispatcher()
@@ -277,10 +287,10 @@ class SnapshotFtTest(data_service_test_base.TestBase, parameterized.TestCase):
     cluster = data_service_test_base.TestCluster(
         num_workers=num_workers,
         worker_max_concurrent_snapshots=worker_max_concurrent_snapshots)
-
+    snapshot_dir = data_service_test_base.TempDir()
     paths = []
     for i in range(num_snapshots):
-      paths.append(f"{self._path}_{i}")
+      paths.append(f"{snapshot_dir.full_path}_{i}")
       self.evaluate(
           distributed_save_op.distributed_save(
               dataset_ops.Dataset.range(5000),
@@ -310,136 +320,173 @@ class SnapshotFtTest(data_service_test_base.TestBase, parameterized.TestCase):
       time.sleep(0.1)
 
     cluster.restart_dispatcher()
-    wait_for_snapshots(paths, get_assignments_and_update_max_assignments)
+    wait_for_snapshot(paths, get_assignments_and_update_max_assignments)
     self.assertValuesEqual(list(max_assignments.values()),
                            [worker_max_concurrent_snapshots] * num_workers)
 
-  @combinations.generate(test_base.default_test_combinations())
-  def testDatasetRecoversAndCompletes(self):
-    cluster = data_service_test_base.TestCluster(num_workers=3)
-    ds = dataset_ops.Dataset.range(1000)
-    self.evaluate(distributed_save_op.distributed_save(
-        ds, self._path, cluster.dispatcher_address(), compression=None))
+  @combinations.generate(
+      combinations.times(
+          test_base.default_test_combinations(),
+          combinations.combine(num_workers=[1, 3])))
+  def testDatasetRecoversAndCompletes(self, num_workers):
+    cluster = data_service_test_base.TestCluster(num_workers=num_workers)
+    snapshot_dir = data_service_test_base.TempDir()
+    dataset = dataset_ops.Dataset.range(1000)
+    self.evaluate(
+        distributed_save_op.distributed_save(
+            dataset,
+            snapshot_dir.full_path,
+            cluster.dispatcher_address(),
+            compression=None))
 
-    # Blocks until all workers have streams.
-    get_stream_assignments(cluster, 3, [self._path])
-    cluster.stop_worker(0)
-    cluster.restart_dispatcher()
-    cluster.restart_worker(0)
-    self._wait_for_snapshot()
-    self.assertTrue(self._snapshot_is_done())
+    for i in range(num_workers):
+      cluster.stop_worker(i)
+      cluster.restart_dispatcher()
+      cluster.restart_worker(i)
+    wait_for_snapshot(snapshot_dir.full_path)
+    self.assertTrue(snapshot_is_done(snapshot_dir.full_path))
 
-    dataset = dataset_ops.Dataset.load(self._path)
+    dataset = dataset_ops.Dataset.load(snapshot_dir.full_path)
     self.assertDatasetProduces(dataset, range(1000), assert_items_equal=True)
 
-  @combinations.generate(test_base.default_test_combinations())
-  def testLargeMultiSourceSnapshotRecoversAndCompletes(self):
-    n = 5
-    cluster, _ = self.setup(num_workers=n, ds_size=1000, num_sources=3)
-    # Blocks until all workers have streams.
-    get_stream_assignments(cluster, n, [self._path])
-    cluster.stop_worker(0)
-    self.assertTrue(
-        os.path.exists(
-            os.path.join(self._path, "streams", "stream_0", "checkpoints")))
-
-    cluster.restart_dispatcher()
-    cluster.restart_worker(0)
-    self._wait_for_snapshot()
-    self.assertTrue(self._snapshot_is_done())
-    # TODO(b/250921378): Verify the number of elements.
-
-  @combinations.generate(test_base.default_test_combinations())
-  def testRepeatedDatasetRecoversAndCompletes(self):
-    cluster = data_service_test_base.TestCluster(num_workers=3)
-    ds = dataset_ops.Dataset.range(100)
-    ds = ds.repeat(10)
+  @combinations.generate(
+      combinations.times(
+          test_base.default_test_combinations(),
+          combinations.combine(num_workers=[1, 3], num_sources=[1, 3])))
+  def testLargeMultiSourceSnapshotRecoversAndCompletes(
+      self, num_workers, num_sources):
+    cluster = data_service_test_base.TestCluster(num_workers=num_workers)
+    snapshot_dir = data_service_test_base.TempDir()
+    dataset = self._get_dataset(dataset_range=1000, num_sources=num_sources)
     self.evaluate(distributed_save_op.distributed_save(
-        ds, self._path, cluster.dispatcher_address()))
+        dataset, snapshot_dir.full_path, cluster.dispatcher_address()))
 
-    # Blocks until all workers have streams.
-    get_stream_assignments(cluster, 3, [self._path])
-    cluster.stop_worker(0)
-    cluster.restart_dispatcher()
-    cluster.restart_worker(0)
-    self._wait_for_snapshot()
-    self.assertTrue(self._snapshot_is_done())
+    for i in range(num_workers):
+      cluster.stop_worker(i)
+      cluster.restart_dispatcher()
+      cluster.restart_worker(i)
+    wait_for_snapshot(snapshot_dir.full_path)
+    self.assertTrue(snapshot_is_done(snapshot_dir.full_path))
 
-    dataset = dataset_ops.Dataset.load(self._path)
+    if num_workers > 1:
+      # Can't verify the elements with more than 1 worker since the splits
+      # receive by each worker for each source is non-deterministic.
+      # For example, with dynamic sharding, worker 1 may receive 999 splits for
+      # source 0 and 0 splis for source 1; worker 2 may receive 0 splits for
+      # source 0 and 999 splits for source 1. In this case, the snapshot will
+      # contain no elements since `zip` drops the elements when the zipped
+      # datasets have different cardinalities.
+      return
+
+    dataset = dataset_ops.Dataset.load(snapshot_dir.full_path)
+    expected = (
+        list(range(1000))
+        if num_sources == 1
+        else list(zip(*([range(1000)] * num_sources))))
+    self.assertDatasetProduces(dataset, expected, assert_items_equal=True)
+
+  @combinations.generate(
+      combinations.times(
+          test_base.default_test_combinations(),
+          combinations.combine(
+              num_elements=[1, 2, 1000],
+              num_workers=[1, 3],
+              num_repetitions=[1, 10])))
+  def testRepeatedDatasetRecoversAndCompletes(
+      self, num_elements, num_workers, num_repetitions):
+    cluster = data_service_test_base.TestCluster(num_workers=num_workers)
+    snapshot_dir = data_service_test_base.TempDir()
+    ds = dataset_ops.Dataset.range(num_elements)
+    ds = ds.repeat(num_repetitions)
+    self.evaluate(distributed_save_op.distributed_save(
+        ds, snapshot_dir.full_path, cluster.dispatcher_address()))
+
+    for i in range(num_workers):
+      cluster.stop_worker(i)
+      cluster.restart_dispatcher()
+      cluster.restart_worker(i)
+    wait_for_snapshot(snapshot_dir.full_path)
+    self.assertTrue(snapshot_is_done(snapshot_dir.full_path))
+
+    dataset = dataset_ops.Dataset.load(snapshot_dir.full_path)
     self.assertDatasetProduces(
-        dataset, list(range(100)) * 10, assert_items_equal=True)
+        dataset,
+        list(range(num_elements)) * num_repetitions,
+        assert_items_equal=True)
 
-  @combinations.generate(test_base.default_test_combinations())
-  def testNonrepeatedDatasetDoesntProduceSecondRepetitionDir(self):
-    num_workers = 5
-    num_sources = 3
-    cluster, _ = self.setup(
-        num_workers=num_workers,
-        ds_size=1000,
-        num_sources=num_sources,
-    )
-    # Blocks until all workers have streams.
-    get_stream_assignments(cluster, num_workers, [self._path])
-    cluster.stop_worker(0)
-    cluster.restart_worker(0)
-    self._wait_for_snapshot()
-    self.assertTrue(self._snapshot_is_done())
+  @combinations.generate(
+      combinations.times(
+          test_base.default_test_combinations(),
+          combinations.combine(num_workers=[1, 5], num_sources=[1, 3])))
+  def testNonrepeatedDatasetDoesntProduceSecondRepetitionDir(
+      self, num_workers, num_sources):
+    cluster = data_service_test_base.TestCluster(num_workers=num_workers)
+    snapshot_dir = data_service_test_base.TempDir()
+    dataset = self._get_dataset(dataset_range=1000, num_sources=num_sources)
+    self.evaluate(distributed_save_op.distributed_save(
+        dataset, snapshot_dir.full_path, cluster.dispatcher_address()))
+
+    for i in range(num_workers):
+      cluster.stop_worker(i)
+      cluster.restart_dispatcher()
+      cluster.restart_worker(i)
+    wait_for_snapshot(snapshot_dir.full_path)
+    self.assertTrue(snapshot_is_done(snapshot_dir.full_path))
     for stream_idx in range(num_workers):
       for source_idx in range(num_sources):
         self.assertFalse(
             os.path.exists(
                 os.path.join(
-                    self._path,
+                    snapshot_dir.full_path,
                     "streams",
                     f"stream_{stream_idx}",
                     "splits",
                     f"source_{source_idx}",
-                    "repetition_1",
-                )
-            )
-        )
+                    "repetition_1")))
 
-  @combinations.generate(test_base.default_test_combinations())
-  def testMultipleDatasetRecoversAndCompletes(self):
-    cluster = data_service_test_base.TestCluster(num_workers=3)
+  @combinations.generate(
+      combinations.times(
+          test_base.default_test_combinations(),
+          combinations.combine(num_workers=[1, 3])))
+  def testMultipleDatasetRecoversAndCompletes(self, num_workers):
+    cluster = data_service_test_base.TestCluster(num_workers=num_workers)
+    snapshot_dir = data_service_test_base.TempDir()
     dataset1 = dataset_ops.Dataset.range(1000)
     datasets = [
         dataset_ops.Dataset.from_tensors("a").repeat(50),
         dataset_ops.Dataset.from_tensors("b").repeat(50),
-        dataset_ops.Dataset.from_tensors("c").repeat(50),
-    ]
+        dataset_ops.Dataset.from_tensors("c").repeat(50)]
     choice_dataset = dataset_ops.Dataset.range(3).repeat()
     dataset2 = dataset_ops.Dataset.choose_from_datasets(
-        datasets, choice_dataset
-    )
+        datasets, choice_dataset)
 
-    snapshot_path1 = os.path.join(self._path, "snapshot1")
-    snapshot_path2 = os.path.join(self._path, "snapshot2")
+    snapshot_path1 = os.path.join(snapshot_dir.full_path, "snapshot1")
+    snapshot_path2 = os.path.join(snapshot_dir.full_path, "snapshot2")
     self.evaluate(
         distributed_save_op.distributed_save(
-            dataset1, snapshot_path1, cluster.dispatcher_address()
-        )
-    )
+            dataset1, snapshot_path1, cluster.dispatcher_address()))
     self.evaluate(
         distributed_save_op.distributed_save(
-            dataset2, snapshot_path2, cluster.dispatcher_address()
-        )
-    )
+            dataset2, snapshot_path2, cluster.dispatcher_address()))
 
-    # Blocks until all workers have streams.
-    get_stream_assignments(cluster, 3, [snapshot_path1, snapshot_path2])
-    cluster.stop_worker(0)
-    cluster.restart_dispatcher()
-    cluster.restart_worker(0)
-    while not os.path.exists(os.path.join(snapshot_path1, "DONE")):
-      time.sleep(0.1)
-    while not os.path.exists(os.path.join(snapshot_path2, "DONE")):
-      time.sleep(0.1)
-    # TODO(b/250921378): Verify the number of elements.
+    for i in range(num_workers):
+      cluster.stop_worker(i)
+      cluster.restart_dispatcher()
+      cluster.restart_worker(i)
+    wait_for_snapshot(snapshot_path1)
+    wait_for_snapshot(snapshot_path2)
 
-  @combinations.generate(test_base.default_test_combinations())
-  def testNestedDataset(self):
-    cluster = data_service_test_base.TestCluster(num_workers=1)
+    dataset = dataset_ops.Dataset.load(snapshot_path1)
+    self.assertDatasetProduces(
+        dataset, list(range(1000)), assert_items_equal=True)
+
+  @combinations.generate(
+      combinations.times(
+          test_base.default_test_combinations(),
+          combinations.combine(num_workers=[1, 3])))
+  def testNestedDataset(self, num_workers):
+    cluster = data_service_test_base.TestCluster(num_workers=num_workers)
+    snapshot_dir = data_service_test_base.TempDir()
     dataset = dataset_ops.Dataset.from_tensor_slices(range(100))
     def interleave_fn(x):
       ds = dataset_ops.Dataset.from_tensor_slices(range(x))
@@ -448,27 +495,39 @@ class SnapshotFtTest(data_service_test_base.TestBase, parameterized.TestCase):
       return ds.flat_map(flat_map_fn)
     dataset = dataset.interleave(
         interleave_fn, cycle_length=2, num_parallel_calls=2)
-
     self.evaluate(
         distributed_save_op.distributed_save(
-            dataset, self._path, cluster.dispatcher_address()))
-    # Blocks until all workers have streams.
-    get_stream_assignments(cluster, 1, [self._path])
-    time.sleep(1)
-    cluster.stop_worker(0)
-    cluster.restart_dispatcher()
-    cluster.restart_worker(0)
-    self._wait_for_snapshot()
-    self.assertTrue(self._snapshot_is_done())
+            dataset, snapshot_dir.full_path, cluster.dispatcher_address()))
 
-  def _snapshot_is_done(self):
-    return snapshot_is_done(self._path)
+    for i in range(num_workers):
+      cluster.stop_worker(i)
+      cluster.restart_dispatcher()
+      cluster.restart_worker(i)
+    wait_for_snapshot(snapshot_dir.full_path)
+    self.assertTrue(snapshot_is_done(snapshot_dir.full_path))
 
-  def _snapshot_has_error(self):
-    return snapshot_has_error(self._path)
+  def _get_dataset(self, dataset_range=10, num_sources=1):
+    dataset = dataset_ops.Dataset.range(dataset_range)
+    if num_sources > 1:
+      dataset = dataset_ops.Dataset.zip((dataset,) * num_sources)
+    return dataset
 
-  def _wait_for_snapshot(self):
-    return wait_for_snapshots([self._path])
+  def _splits_dir(self, snapshot_path, stream_idx=0, worker=0):
+    stream_name = f"stream_{stream_idx}"
+    self._make_stream_dir(snapshot_path, stream_name, worker=worker)
+    return os.path.join(snapshot_path, "streams", stream_name, "splits")
+
+  def _source_dir(self, snapshot_path, stream_idx=0, source_idx=0, worker=0):
+    return os.path.join(
+        self._splits_dir(snapshot_path, stream_idx, worker=worker),
+        f"source_{source_idx}",
+        "repetition_0")
+
+  def _make_stream_dir(self, snapshot_path, stream_name, worker=0):
+    stream_dir = os.path.join(snapshot_path, "streams", stream_name)
+    os.makedirs(stream_dir)
+    pathlib.Path(os.path.join(stream_dir, "owner_worker")).write_text(
+        f"{worker}")
 
 
 if __name__ == "__main__":
