@@ -35,7 +35,6 @@ from tensorflow.python.distribute.cluster_resolver import cluster_resolver as cl
 from tensorflow.python.distribute.v1 import input_lib as input_lib_v1
 from tensorflow.python.eager import backprop
 from tensorflow.python.eager import context
-from tensorflow.python.estimator import run_config
 from tensorflow.python.framework import constant_op
 from tensorflow.python.framework import device as tf_device
 from tensorflow.python.framework import dtypes
@@ -54,9 +53,9 @@ from tensorflow.python.ops import variables
 from tensorflow.python.platform import test
 from tensorflow.python.training import training_util
 
-CHIEF = run_config.TaskType.CHIEF
-WORKER = run_config.TaskType.WORKER
-PS = run_config.TaskType.PS
+CHIEF = 'chief'
+WORKER = 'worker'
+PS = 'ps'
 
 
 def _get_replica_id_integer():
