@@ -14,7 +14,6 @@
 # ==============================================================================
 """Tests for TPUStrategy."""
 
-from absl import logging
 from absl.testing import parameterized
 
 from tensorflow.core.protobuf import config_pb2
@@ -26,7 +25,6 @@ from tensorflow.python.distribute import strategy_test_lib
 from tensorflow.python.distribute import tpu_strategy as tpu_lib
 from tensorflow.python.distribute import tpu_values
 from tensorflow.python.distribute.cluster_resolver import tpu_cluster_resolver
-from tensorflow.python.eager import context
 from tensorflow.python.eager import def_function
 from tensorflow.python.eager import remote
 from tensorflow.python.eager import test
@@ -45,20 +43,21 @@ from tensorflow.python.ops import array_ops
 from tensorflow.python.ops import control_flow_switch_case
 from tensorflow.python.ops import embedding_ops
 from tensorflow.python.ops import gen_dataset_ops
-from tensorflow.python.ops import logging_ops
 from tensorflow.python.ops import lookup_ops
 from tensorflow.python.ops import math_ops
 from tensorflow.python.ops import random_ops
-from tensorflow.python.ops import string_ops
 from tensorflow.python.ops import variable_scope
 from tensorflow.python.ops import variables
 from tensorflow.python.ops.ragged import ragged_tensor
 from tensorflow.python.platform import flags
 from tensorflow.python.platform import tf_logging as logging
+from tensorflow.python.saved_model import load
+from tensorflow.python.saved_model import save
 from tensorflow.python.tpu import device_assignment as device_assignment_lib
 from tensorflow.python.tpu import tpu_hardware_feature
 from tensorflow.python.tpu import tpu_replication
 from tensorflow.python.tpu import tpu_strategy_util
+from tensorflow.python.trackable import autotrackable
 from tensorflow.python.training import server_lib
 from tensorflow.python.util import nest
 
@@ -68,6 +67,27 @@ flags.DEFINE_string("project", None, "Name of GCP project with TPU.")
 flags.DEFINE_string("zone", None, "Name of GCP zone with TPU.")
 
 
+class TestExportArchive(autotrackable.AutoTrackable):
+
+  def __init__(self, init_value):
+    self._var = variables.Variable(init_value)
+    self._fn = def_function.function(self.update_var)
+    self._packed_var = self._var._packed_var
+
+  @def_function.function
+  def update_var(self):
+    # Use packed variable in function to simulate the error
+    # in b/323080532
+    self._packed_var.assign_add(3.0).assign_sub(1.0)
+
+  def save_function(self, directory):
+    save.save(self, directory)
+
+  @property
+  def packed_var(self):
+    return self._packed_var
+
+
 def get_tpu_cluster_resolver():
   resolver = tpu_cluster_resolver.TPUClusterResolver(
       tpu=FLAGS.tpu,
@@ -110,33 +130,6 @@ class TPUTest(test.TestCase):
       result = foo(a)
     self.assertAllEqual(6, result)
 
-  # In this case, the entire computation in foo is compiled using JIT
-  # compilation and contains unsupported ops that should be outside compiled.
-  def test_single_tpu_jit_compile_with_outside_compilation(self):
-    context.enable_jit_compile_rewrite()
-    get_tpu_strategy(True)
-    config.set_soft_device_placement(True)
-    with ops.device("/device:TPU:1"):
-      a = variables.Variable(1)
-
-    def get_a_plus_one():
-      return a + 1
-
-    @def_function.function(
-        input_signature=[tensor_spec.TensorSpec([], dtypes.int32)])
-    def foo(x):
-      b = x + get_a_plus_one()
-      my_str = string_ops.as_string(b)
-      new_str = my_str + "0"
-      c = string_ops.string_to_number(new_str, out_type=dtypes.int32)
-      logging_ops.print_v2(c)
-      b = c + get_a_plus_one()
-      return b + 1
-
-    with ops.device("/device:TPU:1"):
-      result = foo(a)
-    self.assertAllEqual(33, result)
-
   # In this case, each of the ops in the TPU device scope are compiled and run
   # individually.
   def test_single_tpu_on_demand(self):
@@ -286,6 +279,27 @@ class TPUStrategyTest(test.TestCase, parameterized.TestCase):
     ret = func()
     self.assertAllEqual(ret, 2.0)
 
+  def test_save(self, enable_packed_var):
+    strategy = get_tpu_strategy(enable_packed_var)
+    with strategy.scope():
+      v = variables.Variable(1.0)
+    export_dir = self.create_tempdir()
+    save.save(v, export_dir)
+    reloaded_var = load.load(export_dir)
+    self.assertAllEqual(reloaded_var, 1.0)
+
+  def test_packed_variable_export(self, enable_packed_var):
+    if not enable_packed_var:
+      self.skipTest("Test for Packed Variables only.")
+    strategy = get_tpu_strategy(enable_packed_var)
+    with strategy.scope():
+      export_dir = self.get_temp_dir()
+      export_archive = TestExportArchive(1.0)
+    export_archive.save_function(export_dir)
+    restored_object = load.load(export_dir)
+    with ops.device("/tpu:0"):
+      self.assertAllEqual(restored_object._packed_var, 1.0)
+
   def testStaticHashTableDatasetFnHostTrainingLoop(self, enable_packed_var):
     self._dataset_fn_tracing_count = 0
     strategy = get_tpu_strategy(enable_packed_var)
@@ -921,7 +935,7 @@ class TPUStrategyTest(test.TestCase, parameterized.TestCase):
 
     sparse, result = sparse_lookup(dataset)
 
-    # All replicas return identical reults.
+    # All replicas return identical results.
     for replica in range(strategy.num_replicas_in_sync):
       self.assertIsInstance(sparse[replica], sparse_tensor.SparseTensor)
       self.assertAllEqual(sparse[replica].indices, [[0, 0], [1, 0], [1, 1]])
@@ -971,7 +985,7 @@ class TPUStrategyTest(test.TestCase, parameterized.TestCase):
 
     output = sparse_lookup(dataset)
 
-    # All replicas return identical reults.
+    # All replicas return identical results.
     for replica in range(strategy.num_replicas_in_sync):
       self.assertIsInstance(output["sparse"][replica],
                             sparse_tensor.SparseTensor)
@@ -1086,7 +1100,7 @@ class TPUStrategyTest(test.TestCase, parameterized.TestCase):
 
     composite, result = test_fn(test_composite)
 
-    # All replicas return identical reults.
+    # All replicas return identical results.
     for replica in range(strategy.num_replicas_in_sync):
       self.assertIsInstance(composite[replica], TestComposite)
       self.assertAllEqual(composite[replica].values[0], a)
@@ -1208,6 +1222,7 @@ class TPUStrategyTest(test.TestCase, parameterized.TestCase):
     strategy = tpu_lib.TPUStrategyV2(
         resolver, experimental_device_assignment=device_assignment
     )
+    strategy.extended._enable_data_reorder = True
 
     dist_dataset = create_dist_dataset_fn(strategy)
     iterator = iter(dist_dataset)
@@ -1569,4 +1584,5 @@ class VariableCreationTest(test.TestCase):
 
 
 if __name__ == "__main__":
+  ops.enable_eager_execution()
   test.main()
