@@ -96,6 +96,20 @@ class TypesTest(test_util.TensorFlowTestCase, parameterized.TestCase):
     self.assertIs(dtypes.float8_e5m2, dtypes.as_dtype(dtypes._np_float8_e5m2))
     self.assertIs(dtypes.float8_e4m3fn,
                   dtypes.as_dtype(dtypes._np_float8_e4m3fn))
+    self.assertIs(
+        dtypes.float8_e4m3fnuz, dtypes.as_dtype(dtypes._np_float8_e4m3fnuz)
+    )
+    self.assertIs(
+        dtypes.float8_e4m3b11fnuz,
+        dtypes.as_dtype(dtypes._np_float8_e4m3b11fnuz),
+    )
+    self.assertIs(
+        dtypes.float8_e5m2fnuz, dtypes.as_dtype(dtypes._np_float8_e5m2fnuz)
+    )
+    self.assertIs(dtypes.int4, dtypes.as_dtype(dtypes._np_int4))
+    self.assertIs(dtypes.uint4, dtypes.as_dtype(dtypes._np_uint4))
+    self.assertIs(dtypes.int2, dtypes.as_dtype(dtypes._np_int2))
+    self.assertIs(dtypes.uint2, dtypes.as_dtype(dtypes._np_uint2))
     with self.assertRaises(TypeError):
       dtypes.as_dtype(np.dtype([("f1", np.uint), ("f2", np.int32)]))
 
@@ -111,9 +125,23 @@ class TypesTest(test_util.TensorFlowTestCase, parameterized.TestCase):
 
   def testRealDtype(self):
     for dtype in [
-        dtypes.float32, dtypes.float64, dtypes.bool, dtypes.uint8, dtypes.int8,
-        dtypes.int16, dtypes.int32, dtypes.int64, dtypes.float8_e5m2,
-        dtypes.float8_e4m3fn
+        dtypes.float32,
+        dtypes.float64,
+        dtypes.bool,
+        dtypes.uint8,
+        dtypes.int8,
+        dtypes.int16,
+        dtypes.int32,
+        dtypes.int64,
+        dtypes.float8_e5m2,
+        dtypes.float8_e4m3fn,
+        dtypes.float8_e4m3fnuz,
+        dtypes.float8_e4m3b11fnuz,
+        dtypes.float8_e5m2fnuz,
+        dtypes.int4,
+        dtypes.uint4,
+        dtypes.int2,
+        dtypes.uint2,
     ]:
       self.assertIs(dtype.real_dtype, dtype)
     self.assertIs(dtypes.complex64.real_dtype, dtypes.float32)
@@ -138,6 +166,15 @@ class TypesTest(test_util.TensorFlowTestCase, parameterized.TestCase):
     self.assertIs(dtypes.bfloat16, dtypes.as_dtype("bfloat16"))
     self.assertIs(dtypes.float8_e5m2, dtypes.as_dtype("float8_e5m2"))
     self.assertIs(dtypes.float8_e4m3fn, dtypes.as_dtype("float8_e4m3fn"))
+    self.assertIs(dtypes.float8_e4m3fnuz, dtypes.as_dtype("float8_e4m3fnuz"))
+    self.assertIs(
+        dtypes.float8_e4m3b11fnuz, dtypes.as_dtype("float8_e4m3b11fnuz")
+    )
+    self.assertIs(dtypes.float8_e5m2fnuz, dtypes.as_dtype("float8_e5m2fnuz"))
+    self.assertIs(dtypes.int4, dtypes.as_dtype("int4"))
+    self.assertIs(dtypes.uint4, dtypes.as_dtype("uint4"))
+    self.assertIs(dtypes.int2, dtypes.as_dtype("int2"))
+    self.assertIs(dtypes.uint2, dtypes.as_dtype("uint2"))
     self.assertIs(dtypes.float32_ref, dtypes.as_dtype("float32_ref"))
     self.assertIs(dtypes.float64_ref, dtypes.as_dtype("float64_ref"))
     self.assertIs(dtypes.int32_ref, dtypes.as_dtype("int32_ref"))
@@ -156,6 +193,10 @@ class TypesTest(test_util.TensorFlowTestCase, parameterized.TestCase):
     self.assertIs(dtypes.float8_e5m2_ref, dtypes.as_dtype("float8_e5m2_ref"))
     self.assertIs(dtypes.float8_e4m3fn_ref,
                   dtypes.as_dtype("float8_e4m3fn_ref"))
+    self.assertIs(dtypes.int4_ref, dtypes.as_dtype("int4_ref"))
+    self.assertIs(dtypes.uint4_ref, dtypes.as_dtype("uint4_ref"))
+    self.assertIs(dtypes.int2_ref, dtypes.as_dtype("int2_ref"))
+    self.assertIs(dtypes.uint2_ref, dtypes.as_dtype("uint2_ref"))
     with self.assertRaises(TypeError):
       dtypes.as_dtype("not_a_type")
 
@@ -186,6 +227,13 @@ class TypesTest(test_util.TensorFlowTestCase, parameterized.TestCase):
     self.assertEqual(dtypes.as_dtype("bfloat16").is_integer, False)
     self.assertEqual(dtypes.as_dtype("float8_e5m2").is_integer, False)
     self.assertEqual(dtypes.as_dtype("float8_e4m3fn").is_integer, False)
+    self.assertEqual(dtypes.as_dtype("float8_e4m3fnuz").is_integer, False)
+    self.assertEqual(dtypes.as_dtype("float8_e4m3b11fnuz").is_integer, False)
+    self.assertEqual(dtypes.as_dtype("float8_e5m2fnuz").is_integer, False)
+    self.assertEqual(dtypes.as_dtype("int4").is_integer, True)
+    self.assertEqual(dtypes.as_dtype("uint4").is_integer, True)
+    self.assertEqual(dtypes.as_dtype("int2").is_integer, True)
+    self.assertEqual(dtypes.as_dtype("uint2").is_integer, True)
     self.assertEqual(dtypes.as_dtype("qint8").is_integer, False)
     self.assertEqual(dtypes.as_dtype("qint16").is_integer, False)
     self.assertEqual(dtypes.as_dtype("qint32").is_integer, False)
@@ -208,6 +256,13 @@ class TypesTest(test_util.TensorFlowTestCase, parameterized.TestCase):
     self.assertEqual(dtypes.as_dtype("bfloat16").is_floating, True)
     self.assertEqual(dtypes.as_dtype("float8_e5m2").is_floating, True)
     self.assertEqual(dtypes.as_dtype("float8_e4m3fn").is_floating, True)
+    self.assertEqual(dtypes.as_dtype("float8_e4m3fnuz").is_floating, True)
+    self.assertEqual(dtypes.as_dtype("float8_e4m3b11fnuz").is_floating, True)
+    self.assertEqual(dtypes.as_dtype("float8_e5m2fnuz").is_floating, True)
+    self.assertEqual(dtypes.as_dtype("int4").is_floating, False)
+    self.assertEqual(dtypes.as_dtype("uint4").is_floating, False)
+    self.assertEqual(dtypes.as_dtype("int2").is_floating, False)
+    self.assertEqual(dtypes.as_dtype("uint2").is_floating, False)
     self.assertEqual(dtypes.as_dtype("qint8").is_floating, False)
     self.assertEqual(dtypes.as_dtype("qint16").is_floating, False)
     self.assertEqual(dtypes.as_dtype("qint32").is_floating, False)
@@ -230,6 +285,13 @@ class TypesTest(test_util.TensorFlowTestCase, parameterized.TestCase):
     self.assertEqual(dtypes.as_dtype("bfloat16").is_complex, False)
     self.assertEqual(dtypes.as_dtype("float8_e5m2").is_complex, False)
     self.assertEqual(dtypes.as_dtype("float8_e4m3fn").is_complex, False)
+    self.assertEqual(dtypes.as_dtype("float8_e4m3fnuz").is_complex, False)
+    self.assertEqual(dtypes.as_dtype("float8_e4m3b11fnuz").is_complex, False)
+    self.assertEqual(dtypes.as_dtype("float8_e5m2fnuz").is_complex, False)
+    self.assertEqual(dtypes.as_dtype("int4").is_complex, False)
+    self.assertEqual(dtypes.as_dtype("uint4").is_complex, False)
+    self.assertEqual(dtypes.as_dtype("int2").is_complex, False)
+    self.assertEqual(dtypes.as_dtype("uint2").is_complex, False)
     self.assertEqual(dtypes.as_dtype("qint8").is_complex, False)
     self.assertEqual(dtypes.as_dtype("qint16").is_complex, False)
     self.assertEqual(dtypes.as_dtype("qint32").is_complex, False)
@@ -252,6 +314,13 @@ class TypesTest(test_util.TensorFlowTestCase, parameterized.TestCase):
     self.assertEqual(dtypes.as_dtype("bfloat16").is_unsigned, False)
     self.assertEqual(dtypes.as_dtype("float8_e5m2").is_unsigned, False)
     self.assertEqual(dtypes.as_dtype("float8_e4m3fn").is_unsigned, False)
+    self.assertEqual(dtypes.as_dtype("float8_e4m3fnuz").is_unsigned, False)
+    self.assertEqual(dtypes.as_dtype("float8_e4m3b11fnuz").is_unsigned, False)
+    self.assertEqual(dtypes.as_dtype("float8_e5m2fnuz").is_unsigned, False)
+    self.assertEqual(dtypes.as_dtype("int4").is_unsigned, False)
+    self.assertEqual(dtypes.as_dtype("uint4").is_unsigned, True)
+    self.assertEqual(dtypes.as_dtype("int2").is_unsigned, False)
+    self.assertEqual(dtypes.as_dtype("uint2").is_unsigned, True)
     self.assertEqual(dtypes.as_dtype("qint8").is_unsigned, False)
     self.assertEqual(dtypes.as_dtype("qint16").is_unsigned, False)
     self.assertEqual(dtypes.as_dtype("qint32").is_unsigned, False)
@@ -320,6 +389,27 @@ class TypesTest(test_util.TensorFlowTestCase, parameterized.TestCase):
       if numpy_dtype == dtypes.float8_e4m3fn.as_numpy_dtype:
         self.assertEqual(dtype.min, -448.0)
         self.assertEqual(dtype.max, 448.0)
+      if numpy_dtype == dtypes.float8_e4m3fnuz.as_numpy_dtype:
+        self.assertEqual(dtype.min, -240.0)
+        self.assertEqual(dtype.max, 240.0)
+      if numpy_dtype == dtypes.float8_e4m3b11fnuz.as_numpy_dtype:
+        self.assertEqual(dtype.min, -30.0)
+        self.assertEqual(dtype.max, 30.0)
+      if numpy_dtype == dtypes.float8_e5m2fnuz.as_numpy_dtype:
+        self.assertEqual(dtype.min, -57344.0)
+        self.assertEqual(dtype.max, 57344.0)
+      if numpy_dtype == dtypes.int4.as_numpy_dtype:
+        self.assertEqual(dtype.min, -8)
+        self.assertEqual(dtype.max, 7)
+      if numpy_dtype == dtypes.uint4.as_numpy_dtype:
+        self.assertEqual(dtype.min, 0)
+        self.assertEqual(dtype.max, 15)
+      if numpy_dtype == dtypes.int2.as_numpy_dtype:
+        self.assertEqual(dtype.min, -2)
+        self.assertEqual(dtype.max, 1)
+      if numpy_dtype == dtypes.uint2.as_numpy_dtype:
+        self.assertEqual(dtype.min, 0)
+        self.assertEqual(dtype.max, 3)
 
   def testLimitsUndefinedError(self):
     with self.assertRaises(ValueError):
