@@ -13,7 +13,6 @@
 # limitations under the License.
 # =============================================================================
 
-from tensorflow.python.framework import _memory_checker_test_helper
 from tensorflow.python.framework import constant_op
 from tensorflow.python.framework import ops
 from tensorflow.python.framework import test_util
@@ -45,18 +44,6 @@ class MemoryCheckerTest(test.TestCase):
     memory_checker.report()
     memory_checker.assert_no_leak_if_all_possibly_except_one()
 
-  def testNoLeak2(self):
-    helper = _memory_checker_test_helper.MemoryCheckerTestHelper()
-    with MemoryChecker() as memory_checker:
-      memory_checker.record_snapshot()
-      helper.list_push_back(10)
-      memory_checker.record_snapshot()
-      memory_checker.record_snapshot()
-      memory_checker.record_snapshot()
-
-    memory_checker.report()
-    memory_checker.assert_no_leak_if_all_possibly_except_one()
-
   def testNoLeak3(self):
     with MemoryChecker() as memory_checker:
       tensors = []
@@ -81,20 +68,6 @@ class MemoryCheckerTest(test.TestCase):
     with self.assertRaises(AssertionError):
       memory_checker.assert_no_leak_if_all_possibly_except_one()
 
-  def testLeak2(self):
-    helper = _memory_checker_test_helper.MemoryCheckerTestHelper()
-    with MemoryChecker() as memory_checker:
-      memory_checker.record_snapshot()
-      helper.list_push_back(10)
-      memory_checker.record_snapshot()
-      helper.list_push_back(11)
-      memory_checker.record_snapshot()
-      memory_checker.record_snapshot()
-
-    memory_checker.report()
-    with self.assertRaises(AssertionError):
-      memory_checker.assert_no_leak_if_all_possibly_except_one()
-
   def testLeak3(self):
     with MemoryChecker() as memory_checker:
       tensors = []
@@ -106,18 +79,6 @@ class MemoryCheckerTest(test.TestCase):
     with self.assertRaises(AssertionError):
       memory_checker.assert_no_leak_if_all_possibly_except_one()
 
-  def testLeak4(self):
-    helper = _memory_checker_test_helper.MemoryCheckerTestHelper()
-
-    with MemoryChecker() as memory_checker:
-      for i in range(10):
-        helper.list_push_back(i)
-        memory_checker.record_snapshot()
-
-    memory_checker.report()
-    with self.assertRaises(AssertionError):
-      memory_checker.assert_no_leak_if_all_possibly_except_one()
-
   def testNoNewPythonObjectsEmpty(self):
     with MemoryChecker() as memory_checker:
       memory_checker.record_snapshot()
