@@ -254,18 +254,32 @@ class TensorUtilTest(test.TestCase, parameterized.TestCase):
   def testBfloat16(self):
     test_type = dtypes.bfloat16.as_numpy_dtype
     t = tensor_util.make_tensor_proto(np.array([10.0, 20.0], dtype=test_type))
-    # 10.0: 16672 = 010000010(130) 0100000: (1+0/2+1/4) * 2^(130-127)
-    # 20.0: 16800 = 010000011(131) 0100000: (1+0/2+1/4) * 2^(131-127)
-    self.assertProtoEquals("""
-      dtype: DT_BFLOAT16
-      tensor_shape {
-        dim {
-          size: 2
-        }
+    if sys.byteorder == "big":
+      self.assertProtoEquals(
+          """
+        dtype: DT_BFLOAT16
+        tensor_shape {
+          dim {
+            size: 2
+          }
       }
-      half_val: 16672
-      half_val: 16800
-      """, t)
+      tensor_content: "\x41\x20\x41\x5C\x32\x34\x30"
+      """,
+          t,
+      )
+    else:
+      self.assertProtoEquals(
+          """
+        dtype: DT_BFLOAT16
+        tensor_shape {
+          dim {
+            size: 2
+          }
+        }
+      tensor_content: "\x20\x41\x5C\x32\x34\x30\x41"
+      """,
+          t,
+      )
 
     a = tensor_util.MakeNdarray(t)
     self.assertEqual(test_type, a.dtype)
@@ -307,6 +321,60 @@ class TensorUtilTest(test.TestCase, parameterized.TestCase):
       tensor_content: "RZ"
       """, t)
 
+  def testFloat8e4m3fnuz(self):
+    test_type = dtypes.float8_e4m3fnuz.as_numpy_dtype
+    t = tensor_util.make_tensor_proto(np.array([10.0, 20.0], dtype=test_type))
+    # 10.0: "Z" = 90 = 1010 010: 2^(10 - 7) * (1 + 1/4) + 8
+    # 20.0: "b" = 98 = 1011 010: 2^(11 - 7) * (1 + 1/4) + 8
+    self.assertProtoEquals(
+        """
+      dtype: DT_FLOAT8_E4M3FNUZ
+      tensor_shape {
+        dim {
+          size: 2
+        }
+      }
+      tensor_content: "Zb"
+      """,
+        t,
+    )
+
+  def testFloat8e4m3b11fnuz(self):
+    test_type = dtypes.float8_e4m3b11fnuz.as_numpy_dtype
+    t = tensor_util.make_tensor_proto(np.array([10.0, 20.0], dtype=test_type))
+    # 10.0: "r" = 114 = 1010 010: 2^(10 - 7) * (1 + 1/4) + 36
+    # 20.0: "z" = 126 = 1011 010: 2^(11 - 7) * (1 + 1/4) + 36
+    self.assertProtoEquals(
+        """
+      dtype: DT_FLOAT8_E4M3B11FNUZ
+      tensor_shape {
+        dim {
+          size: 2
+        }
+      }
+      tensor_content: "rz"
+      """,
+        t,
+    )
+
+  def testFloat8e5m2fnuz(self):
+    test_type = dtypes.float8_e5m2fnuz.as_numpy_dtype
+    t = tensor_util.make_tensor_proto(np.array([10.0, 20.0], dtype=test_type))
+    # 10.0: "M" = 77 = 1010 010: 2^(10 - 7) * (1 + 1/4) - 3
+    # 20.0: "Q" = 87 = 1011 010: 2^(11 - 7) * (1 + 1/4) - 3
+    self.assertProtoEquals(
+        """
+      dtype: DT_FLOAT8_E5M2FNUZ
+      tensor_shape {
+        dim {
+          size: 2
+        }
+      }
+      tensor_content: "MQ"
+      """,
+        t,
+    )
+
   def testInt(self):
     t = tensor_util.make_tensor_proto(10)
     self.assertProtoEquals("""
@@ -318,6 +386,105 @@ class TensorUtilTest(test.TestCase, parameterized.TestCase):
     self.assertEqual(np.int32, a.dtype)
     self.assertAllClose(np.array(10, dtype=np.int32), a)
 
+  def testInt4(self):
+    test_type = dtypes.int4.as_numpy_dtype
+    t = tensor_util.make_tensor_proto(
+        np.array(
+            [-8, -1, 0, 1, 7],
+            dtype=test_type,
+        )
+    )
+    #
+    self.assertProtoEquals(
+        """
+      dtype: DT_INT4
+      tensor_shape {
+        dim {
+          size: 5
+        }
+      }
+      int_val: -8
+      int_val: -1
+      int_val: 0
+      int_val: 1
+      int_val: 7
+      """,
+        t,
+    )
+
+  def testUInt4(self):
+    test_type = dtypes.uint4.as_numpy_dtype
+    t = tensor_util.make_tensor_proto(
+        np.array(
+            [0, 1, 7, 8, 15],
+            dtype=test_type,
+        )
+    )
+    self.assertProtoEquals(
+        """
+      dtype: DT_UINT4
+      tensor_shape {
+        dim {
+          size: 5
+        }
+      }
+      int_val: 0
+      int_val: 1
+      int_val: 7
+      int_val: 8
+      int_val: 15
+      """,
+        t,
+    )
+
+  def testInt2(self):
+    test_type = dtypes.int2.as_numpy_dtype
+    t = tensor_util.make_tensor_proto(
+        np.array(
+            [-2, -1, 0, 1],
+            dtype=test_type,
+        )
+    )
+    self.assertProtoEquals(
+        """
+      dtype: DT_INT2
+      tensor_shape {
+        dim {
+          size: 4
+        }
+      }
+      int_val: -2
+      int_val: -1
+      int_val: 0
+      int_val: 1
+      """,
+        t,
+    )
+
+  def testUInt2(self):
+    test_type = dtypes.uint2.as_numpy_dtype
+    t = tensor_util.make_tensor_proto(
+        np.array(
+            [0, 1, 2, 3],
+            dtype=test_type,
+        )
+    )
+    self.assertProtoEquals(
+        """
+      dtype: DT_UINT2
+      tensor_shape {
+        dim {
+          size: 4
+        }
+      }
+      int_val: 0
+      int_val: 1
+      int_val: 2
+      int_val: 3
+      """,
+        t,
+    )
+
   def testLargeInt(self):
     value = np.iinfo(np.int64).max
     t = tensor_util.make_tensor_proto(value)
