@@ -378,8 +378,10 @@ class AutoMixedPrecisionTest(test.TestCase, parameterized.TestCase):
     if mode == 'mkl' and not test_util.IsMklEnabled():
       self.skipTest('MKL is not enabled')
     # Test will fail on machines without AVX512f, e.g., Broadwell
-    isAVX512f = _pywrap_utils.IsBF16SupportedByOneDNNOnThisCPU()
-    if mode == 'mkl' and not isAVX512f:
+    is_avx512f = _pywrap_utils.IsDataTypeSupportedByOneDNNOnThisCPU(
+        dtypes.bfloat16
+    )
+    if mode == 'mkl' and not is_avx512f:
       self.skipTest('Skipping test due to non-AVX512f machine')
 
   def _run_simple_loop_test(self, mode, inp, body, out):
@@ -581,7 +583,7 @@ class AutoMixedPrecisionTest(test.TestCase, parameterized.TestCase):
     self._assert_output_f16(mode, node_map, 'Relu')
     self._assert_output_f16(mode, node_map, 'MaxPool')
     self._assert_output_f16(mode, node_map, 'Conv2D_1')
-    self.assertEqual(num_to_f16, 4)
+    self.assertEqual(num_to_f16, 5)
     self.assertEqual(num_to_fp32, 1)
     tol = 5e-3 if mode == 'mkl' else 1e-3
     self.assertAllClose(output_val_ref, output_val, atol=tol, rtol=tol)
