@@ -111,6 +111,21 @@ class ConstantTest(test.TestCase):
         np.int64))
     self._testAll(np.empty((2, 0, 5)).astype(np.int64))
 
+  @test_util.run_deprecated_v1
+  @test_util.disable_xla("b/183567451: XLA doesn't yet support int4")
+  def testInt4(self):
+    for dtype in [dtypes_lib.int4, dtypes_lib.uint4]:
+      np_dtype = dtype.as_numpy_dtype
+      self._testAll(
+          np.arange(dtype.min, dtype.max + 1)
+          .reshape([2, 4, 2])
+          .astype(np_dtype)
+      )
+      self._testAll(
+          (7 * np.random.normal(size=30)).reshape([2, 3, 5]).astype(np_dtype)
+      )
+      self._testAll(np.empty((2, 0, 5)).astype(np_dtype))
+
   @test_util.run_deprecated_v1
   def testComplex64(self):
     self._testAll(
@@ -193,7 +208,7 @@ class ConstantTest(test.TestCase):
           shape=[2, 3, 5])
     self.assertEqual(c.get_shape(), [2, 3, 5])
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testEagerMemory(self):
     """Tests PyObject refs are managed correctly when executing eagerly."""
     constant_op.constant([[1.]])
@@ -497,7 +512,7 @@ class ZerosLikeTest(test.TestCase):
       # causes a TypeError in constant_op.constant below. Here we catch the
       # special case of tf.string and set the numpy dtype appropriately.
       if dtype == dtypes_lib.string:
-        numpy_dtype = np.string_
+        numpy_dtype = np.bytes_
       else:
         numpy_dtype = dtype.as_numpy_dtype
       if fully_defined_shape:
@@ -710,7 +725,7 @@ class FillTest(test.TestCase):
     with self.cached_session(use_gpu=use_gpu):
       tf_ans = array_ops.fill(dims, val, name="fill")
       out = self.evaluate(tf_ans)
-    self.assertAllClose(np_ans, out)
+    self.assertAllEqual(np_ans, out)
     # Fill does not set the shape.
     # self.assertShapeEqual(np_ans, tf_ans)
 
@@ -734,6 +749,13 @@ class FillTest(test.TestCase):
     np_ans = np.array([[-42] * 3] * 2).astype(np.int64)
     self._compareAll([2, 3], np_ans[0][0], np_ans)
 
+  @test_util.disable_xla("b/183567451: XLA doesn't yet support int4")
+  def testFillInt4(self):
+    np_ans = np.array([[-6] * 3] * 2).astype(dtypes_lib.int4.as_numpy_dtype)
+    self._compareAll([2, 3], np_ans[0][0], np_ans)
+    np_ans = np.array([[11] * 3] * 2).astype(dtypes_lib.uint4.as_numpy_dtype)
+    self._compareAll([2, 3], np_ans[0][0], np_ans)
+
   def testFillComplex64(self):
     np_ans = np.array([[0.15 + 0.3j] * 3] * 2).astype(np.complex64)
     self._compareAll([2, 3], np_ans[0][0], np_ans)
