@@ -115,27 +115,29 @@ class ShapeOpsTest(test.TestCase):
     self.assertAllEqual(np_ans, result)
     self.assertShapeEqual(np_ans, tf_ans)
 
-  def _testCpu(self, x):
+  def _testCpu(self, x, compare_sparse):
     self._compareShape(x, use_gpu=False)
     self._compareShapeN(x, use_gpu=False)
     self._compareRank(x, use_gpu=False)
     self._compareSize(x, use_gpu=False)
-    self._compareShapeSparse(x, use_gpu=False)
-    self._compareRankSparse(x, use_gpu=False)
-    self._compareSizeSparse(x, use_gpu=False)
+    if compare_sparse:
+      self._compareShapeSparse(x, use_gpu=False)
+      self._compareRankSparse(x, use_gpu=False)
+      self._compareSizeSparse(x, use_gpu=False)
 
-  def _testGpu(self, x):
+  def _testGpu(self, x, compare_sparse):
     self._compareShape(x, use_gpu=True)
     self._compareShapeN(x, use_gpu=True)
     self._compareRank(x, use_gpu=True)
     self._compareSize(x, use_gpu=True)
-    self._compareShapeSparse(x, use_gpu=True)
-    self._compareRankSparse(x, use_gpu=True)
-    self._compareSizeSparse(x, use_gpu=True)
+    if compare_sparse:
+      self._compareShapeSparse(x, use_gpu=True)
+      self._compareRankSparse(x, use_gpu=True)
+      self._compareSizeSparse(x, use_gpu=True)
 
-  def _testAll(self, x):
-    self._testCpu(x)
-    self._testGpu(x)
+  def _testAll(self, x, compare_sparse=True):
+    self._testCpu(x, compare_sparse)
+    self._testGpu(x, compare_sparse)
 
   def testBasic(self):
     self._testAll(np.random.randn(2))
@@ -153,6 +155,29 @@ class ShapeOpsTest(test.TestCase):
     self._testAll(np.random.choice((False, True), size=(2, 3, 5, 7, 11)))
     self._testAll(np.random.choice((False, True), size=(2, 3, 5, 7, 11, 13)))
 
+  def testString(self):
+    self._testAll(
+        np.random.choice(["abcd", "efgh"], size=(2,)), compare_sparse=False
+    )
+    self._testAll(
+        np.random.choice(["abcd", "efgh"], size=(2, 3)), compare_sparse=False
+    )
+    self._testAll(
+        np.random.choice(["abcd", "efgh"], size=(2, 3, 5)), compare_sparse=False
+    )
+    self._testAll(
+        np.random.choice(["abcd", "efgh"], size=(2, 3, 5, 7)),
+        compare_sparse=False,
+    )
+    self._testAll(
+        np.random.choice(["abcd", "efgh"], size=(2, 3, 5, 7, 11)),
+        compare_sparse=False,
+    )
+    self._testAll(
+        np.random.choice(["abcd", "efgh"], size=(2, 3, 5, 7, 11, 13)),
+        compare_sparse=False,
+    )
+
   # Disabled because it takes too long to run, but manually verified
   # as passing at time of writing.
   def _test64BitOutput(self):
