@@ -541,6 +541,17 @@ class TransposeTest(test.TestCase):
     self._testError(
         np.arange(0., 30).reshape([2, 3, 5]), [0, 1, 1], "2 is missing")
 
+  def testNegativePerm(self):
+    self.assertEqual(
+        [15, 100, 37],
+        array_ops.transpose(
+            constant_op.constant(1, dtype=dtypes.int32, shape=[100, 37, 15]),
+            [-1, -3, -2],
+        )
+        .get_shape()
+        .dims,
+    )
+
 
 if __name__ == "__main__":
   test.main()
