@@ -63,19 +63,28 @@ AFTER_EXPIRE = (2020, 10, 26)
 
 def invert_philox(key, value):
   """Invert the Philox bijection."""
-  key = np.array(key, dtype=np.uint32)
-  value = np.array(value, dtype=np.uint32)
-  step = np.array([0x9E3779B9, 0xBB67AE85], dtype=np.uint32)
+  key = np.array(key, dtype=np.uint64)
+  value = np.array(value, dtype=np.uint64)
+  step = np.array([0x9E3779B9, 0xBB67AE85], dtype=np.uint64)
   for n in range(10)[::-1]:
     key0, key1 = key + n * step
-    v0 = value[3] * 0x991a7cdb & 0xffffffff
-    v2 = value[1] * 0x6d7cae67 & 0xffffffff
-    hi0 = v0 * 0xD2511F53 >> 32
-    hi1 = v2 * 0xCD9E8D57 >> 32
+    mask = np.uint64(0xffffffff)
+
+    v0_mul = np.uint64(0x991a7cdb)
+    v2_mul = np.uint64(0x6d7cae67)
+    v0 = value[3] * v0_mul & mask
+    v2 = value[1] * v2_mul & mask
+
+    hi0_mul = np.uint64(0xD2511F53)
+    hi1_mul = np.uint64(0xCD9E8D57)
+
+    shift_amount = np.uint64(32)
+    hi0 = v0 * hi0_mul >> shift_amount
+    hi1 = v2 * hi1_mul >> shift_amount
     v1 = hi1 ^ value[0] ^ key0
     v3 = hi0 ^ value[2] ^ key1
     value = v0, v1, v2, v3
-  return np.array(value)
+  return np.array(value, dtype=np.uint64)
 
 
 SEEDS = ((7, 17), (11, 5), (2, 3))
@@ -102,7 +111,7 @@ def float_cases(shape_dtypes=(None,)):
   )
   # Explicitly passing in params because capturing cell variable from loop is
   # problematic in Python
-  def wrap(op, dtype, shape, shape_dtype, seed, **kwargs):
+  def wrap(op: ops.Operation, dtype, shape, shape_dtype, seed, **kwargs):
     device_type = get_device().device_type
     # Some dtypes are not supported on some devices
     if (dtype == dtypes.bfloat16 and device_type == 'GPU' and
@@ -134,7 +143,8 @@ def float_cases(shape_dtypes=(None,)):
 
 def int_cases(shape_dtypes=(None,), minval_maxval=None):
 
-  def wrap(op, minval, maxval, shape, shape_dtype, dtype, seed, **kwargs):
+  def wrap(op: ops.Operation, minval, maxval, shape, shape_dtype, dtype,
+           seed, **kwargs):
     shape_ = (constant_op.constant(shape, dtype=shape_dtype)
               if shape_dtype is not None else shape)
     return op(
@@ -156,11 +166,22 @@ def int_cases(shape_dtypes=(None,), minval_maxval=None):
 
 def multinomial_cases():
   num_samples = 10
-  def wrap(op, logits, logits_dtype, output_dtype, seed):
+  def wrap(op: ops.Operation, logits, logits_dtype, output_dtype, seed):
+    device_type = get_device().device_type
+    # Some dtypes are not supported on some devices
+    if (logits_dtype == dtypes.bfloat16 and device_type == 'GPU' and
+        not test_util.is_gpu_available(
+            cuda_only=True, min_cuda_compute_capability=(8, 0))):
+      logits_dtype = dtypes.float32
     return op(seed=seed,
               logits=constant_op.constant(logits, dtype=logits_dtype),
               num_samples=num_samples, output_dtype=output_dtype)
-  for logits_dtype in np.float16, np.float32, np.float64:
+  for logits_dtype in (
+      np.float16,
+      dtypes.bfloat16.as_numpy_dtype,
+      np.float32,
+      np.float64,
+  ):
     for output_dtype in dtypes.int32, dtypes.int64:
       for logits in ([[0.1, 0.25, 0.5, 0.15]], [[0.5, 0.5], [0.8, 0.2],
                                                 [0.25, 0.75]]):
@@ -172,7 +193,7 @@ def multinomial_cases():
 
 
 def gamma_cases():
-  def wrap(op, alpha, dtype, shape, seed):
+  def wrap(op: ops.Operation, alpha, dtype, shape, seed):
     return op(seed=seed, shape=shape,
               alpha=constant_op.constant(alpha, dtype=dtype), dtype=dtype)
   for dtype in np.float16, np.float32, np.float64:
@@ -185,7 +206,7 @@ def gamma_cases():
 
 
 def poisson_cases():
-  def wrap(op, lam, lam_dtype, out_dtype, shape, seed):
+  def wrap(op: ops.Operation, lam, lam_dtype, out_dtype, shape, seed):
     return op(seed=seed, shape=shape,
               lam=constant_op.constant(lam_dtype(lam), dtype=lam_dtype),
               dtype=out_dtype)
