@@ -27,12 +27,12 @@ from tensorflow.python.framework import dtypes
 from tensorflow.python.framework import errors
 from tensorflow.python.framework import ops
 from tensorflow.python.framework import test_util
-from tensorflow.python.layers import core as core_layers
 from tensorflow.python.ops import array_ops
 from tensorflow.python.ops import cond
 from tensorflow.python.ops import init_ops
 from tensorflow.python.ops import math_ops
 from tensorflow.python.ops import resource_variable_ops
+from tensorflow.python.ops import resource_variables_toggle
 from tensorflow.python.ops import state_ops
 from tensorflow.python.ops import variable_scope
 from tensorflow.python.ops import variable_v1
@@ -252,21 +252,6 @@ class VariableScopeTest(test.TestCase):
       self.assertFalse(ops.get_collection(ops.GraphKeys.GLOBAL_VARIABLES))
       self.assertFalse(ops.get_collection(ops.GraphKeys.TRAINABLE_VARIABLES))
 
-  def testEagerVariableStoreWithFunctionalLayer(self):
-    with context.eager_mode():
-      container = variable_scope.EagerVariableStore()
-      x = constant_op.constant([[2.0]])
-      with container.as_default():
-        y = core_layers.dense(x, 1, name="my_dense",
-                              kernel_initializer=init_ops.ones_initializer())
-      self.assertAllEqual(y, [[2.0]])
-      self.assertEqual(len(container.variables()), 2)
-      # Recreate the layer to test reuse.
-      with container.as_default():
-        core_layers.dense(x, 1, name="my_dense",
-                          kernel_initializer=init_ops.ones_initializer())
-      self.assertEqual(len(container.variables()), 2)
-
   # Not converted to use wrap_function because of
   # TypeError: Expected tf.group() expected Tensor arguments not 'None' with
   # type '<type 'NoneType'>'.
@@ -456,18 +441,18 @@ class VariableScopeTest(test.TestCase):
   # AssertionError: True is not false (last assertFalse)
   @test_util.run_deprecated_v1
   def testEnableResourceVariables(self):
-    old = variable_scope._DEFAULT_USE_RESOURCE
+    old = resource_variables_toggle._DEFAULT_USE_RESOURCE
     try:
-      variable_scope.enable_resource_variables()
+      resource_variables_toggle.enable_resource_variables()
       self.assertIsInstance(
           variable_v1.VariableV1(1.0),
           resource_variable_ops.ResourceVariable)
-      variable_scope.disable_resource_variables()
+      resource_variables_toggle.disable_resource_variables()
       self.assertNotIsInstance(
           variable_v1.VariableV1(1.0),
           resource_variable_ops.ResourceVariable)
     finally:
-      variable_scope._DEFAULT_USE_RESOURCE = old
+      resource_variables_toggle._DEFAULT_USE_RESOURCE = old
 
   # Not converted to use wrap_function because of
   # TypeError: Fetch argument None has invalid type <type 'NoneType'>
