@@ -14,7 +14,7 @@ limitations under the License.
 ==============================================================================*/
 // Must be included first
 // clang-format off
-#include "tensorflow/tsl/python/lib/core/numpy.h" //NOLINT
+#include "xla/tsl/python/lib/core/numpy.h" //NOLINT
 // clang-format on
 
 #include "tensorflow/python/lib/core/py_seq_tensor.h"
@@ -76,7 +76,7 @@ bool IsPyFloat(PyObject* obj) {
 
 struct ConverterState {
   // The inferred tensor shape.
-  gtl::InlinedVector<int64_t, 4> inferred_shape;
+  absl::InlinedVector<int64_t, 4UL> inferred_shape;
 
   // The inferred tensor data type.
   DataType inferred_dtype;
@@ -115,9 +115,9 @@ PyObject* ZeroDimArrayToScalar(PyObject* obj, ConverterState* state) {
 
 // Sets *elem to a NEW reference to an element in seq on success.
 // REQUIRES: PySequence_Check(seq) && PySequence_Length(seq) > 0.
-Status SampleElementFromSequence(PyObject* seq, PyObject** elem) {
+absl::Status SampleElementFromSequence(PyObject* seq, PyObject** elem) {
   *elem = PySequence_GetItem(seq, 0);
-  if (*elem != nullptr) return OkStatus();
+  if (*elem != nullptr) return absl::OkStatus();
   // seq may implement the sequence protocol (i.e., implement __getitem__)
   // but may legitimately not have a 0-th element (__getitem__(self, 0)
   // raises a KeyError). For example:
@@ -145,13 +145,13 @@ Status SampleElementFromSequence(PyObject* seq, PyObject** elem) {
                                    Py_TYPE(seq)->tp_name,
                                    " object since it is an empty sequence");
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 tstring PyRepr(PyObject* obj);
 bool IsPyDimension(PyObject* obj);
 
-Status InferShapeAndType(PyObject* obj, ConverterState* state) {
+absl::Status InferShapeAndType(PyObject* obj, ConverterState* state) {
   std::vector<Safe_PyObjectPtr> refs_to_clean;
   while (true) {
     // Convert any zero dimensional numpy arrays to scalars first of all.
@@ -208,7 +208,7 @@ Status InferShapeAndType(PyObject* obj, ConverterState* state) {
                                      ") with an unsupported type (",
                                      PyRepr(PyType(obj)), ") to a Tensor.");
     }
-    return OkStatus();
+    return absl::OkStatus();
   }
 }
 
@@ -281,8 +281,9 @@ struct Converter {
     return nullptr;
   }
 
-  static Status Convert(TFE_Context* ctx, PyObject* obj, ConverterState* state,
-                        TFE_TensorHandle** h, const char** error) {
+  static absl::Status Convert(TFE_Context* ctx, PyObject* obj,
+                              ConverterState* state, TFE_TensorHandle** h,
+                              const char** error) {
     // TODO(josh11b): Allocator & attributes
     AbstractTensorInterface* t;
     if (state->inferred_shape.empty()) { /* Scalar case */
@@ -311,7 +312,7 @@ struct Converter {
     }
     *h = tensorflow::wrap(tensorflow::unwrap(ctx)->CreateLocalHandle(t));
     t->Release();
-    return OkStatus();
+    return absl::OkStatus();
   }
 };
 
@@ -617,9 +618,9 @@ tstring PyRepr(PyObject* obj) {
 bool IsPyDimension(PyObject* obj) {
   const char* tp_name = obj->ob_type->tp_name;
   if (strcmp(tp_name, "Dimension") != 0) return false;
-  bool ret = str_util::EndsWith(
-      PyRepr(PyType(obj)),
-      "tensorflow.python.framework.tensor_shape.Dimension'>");
+  bool ret =
+      absl::EndsWith(PyRepr(PyType(obj)),
+                     "tensorflow.python.framework.tensor_shape.Dimension'>");
   return ret;
 }
 
@@ -690,7 +691,7 @@ typedef Converter<bool> BoolConverter;
 // other.
 TFE_TensorHandle* NumpyToTFE_TensorHandle(TFE_Context* ctx, PyObject* obj) {
   Safe_TF_TensorPtr tf_tensor = make_safe(static_cast<TF_Tensor*>(nullptr));
-  Status status = tensorflow::NdarrayToTensor(ctx, obj, &tf_tensor);
+  absl::Status status = tensorflow::NdarrayToTensor(ctx, obj, &tf_tensor);
 
   if (TF_PREDICT_FALSE(!status.ok())) {
     PyErr_SetString(PyExc_ValueError,
@@ -716,16 +717,22 @@ TFE_TensorHandle* PySeqToTFE_TensorHandle(TFE_Context* ctx, PyObject* obj,
   // These objects are efficiently handled by Numpy. We transform them into
   // Numpy arrays and handle them in the Numpy case below. Note that Tensors
   // implement the __array__ function, and will be handled in this shortcut.
-  Safe_PyObjectPtr array =
-      make_safe(PyArray_FromArrayAttr(obj, nullptr, nullptr));
-  if (array == nullptr) {
-    return nullptr;
+  // We used to call PyArray_FromArrayAttr here, but NumPy 2.0 changed its
+  // semantics such that it errors if a copy of the array is required.
+  // (Ideally no copy would be needed here, but that would be a larger change.)
+  Safe_PyObjectPtr array;
+  if (PyObject_HasAttrString(obj, "__array__")) {
+    array = make_safe(PyObject_CallMethod(obj, "__array__", nullptr));
+    if (array == nullptr) {
+      return nullptr;
+    }
+    if (!PyArray_Check(array.get())) {
+      PyErr_SetString(PyExc_ValueError,
+                      "Value returned by __array__ is not a NumPy array");
+      return nullptr;
+    }
   }
-  if (array.get() == Py_NotImplemented) {
-    // The Py_NotImplemented returned from PyArray_FromArrayAttr is not
-    // Py_INCREF'ed, so we don't want the Safe_PyObjectPtr to Py_DECREF it.
-    array.release();
-
+  if (!array) {
     // Try __array_interface__ objects (such as PIL Image).
     array = make_safe(PyArray_FromInterface(obj));
     if (array == nullptr) {
@@ -778,9 +785,9 @@ TFE_TensorHandle* PySeqToTFE_TensorHandle(TFE_Context* ctx, PyObject* obj,
   }
 
   ConverterState state;
-  Status status = InferShapeAndType(obj, &state);
+  absl::Status status = InferShapeAndType(obj, &state);
   if (!status.ok()) {
-    PyErr_SetString(PyExc_ValueError, tsl::NullTerminatedMessage(status));
+    PyErr_SetString(PyExc_ValueError, absl::StatusMessageAsCStr(status));
     return nullptr;
   }
   DataType requested_dtype = DT_INVALID;
@@ -909,7 +916,7 @@ TFE_TensorHandle* PySeqToTFE_TensorHandle(TFE_Context* ctx, PyObject* obj,
   }
 
   if (!status.ok()) {
-    PyErr_SetString(PyExc_ValueError, tsl::NullTerminatedMessage(status));
+    PyErr_SetString(PyExc_ValueError, absl::StatusMessageAsCStr(status));
     return nullptr;
   }
 
