@@ -17,6 +17,8 @@
 import abc
 import collections
 import itertools
+import sys
+import unittest
 
 from absl.testing import parameterized
 
@@ -514,6 +516,8 @@ class FlattenTest(parameterized.TestCase, test_util.TensorFlowTestCase):
     self.assertLen(mod.variables, 1)
     self.assertEqual(mod.variables[0], mod.normal_variable)
 
+  @unittest.skipIf(sys.version_info.major == 3 and sys.version_info.minor == 12,
+                   reason="b/313658911: _TupleWrapper __dict__ attribute error")
   def test_with_path(self):
     mod = module.Module()
     mod.w = variables.Variable(1.)
