@@ -45,6 +45,7 @@ from tensorflow.python.ops import functional_ops  # pylint: disable=unused-impor
 from tensorflow.python.ops import gradient_checker_v2
 from tensorflow.python.ops import gradients
 from tensorflow.python.ops import gradients_impl
+from tensorflow.python.ops import gradients_util
 from tensorflow.python.ops import init_ops
 from tensorflow.python.ops import list_ops
 from tensorflow.python.ops import math_grad  # pylint: disable=unused-import
@@ -462,6 +463,34 @@ class GradientsTest(test_util.TensorFlowTestCase, parameterized.TestCase):
       else:
         self.assertIsNone(output[0])
 
+  def testOptimizeIdentityN(self):
+    with ops.Graph().as_default():
+      #  a    b
+      #  |    |
+      # IdentityN
+      #  |    |
+      #  c    d
+      a = constant_op.constant(1.0)
+      b = constant_op.constant(1.0)
+      c, d = array_ops.identity_n([a, b])
+
+      def OptimizedGradients(y, x):
+        return gradients_util._GradientsHelper(
+            ys=y,
+            xs=x,
+            optimize_identity_n=True,
+        )
+
+      self.assertIsNone(OptimizedGradients(d, a)[0])
+      self.assertIsNone(OptimizedGradients(c, b)[0])
+
+      self.assertIsNotNone(gradients.gradients(d, a)[0])
+      self.assertIsNotNone(gradients.gradients(c, b)[0])
+
+      for grad_impl in [gradients.gradients, OptimizedGradients]:
+        self.assertIsNotNone(grad_impl(c, a)[0])
+        self.assertIsNotNone(grad_impl(d, b)[0])
+
   @parameterized.parameters(unconnected_gradients.UnconnectedGradients.ZERO,
                             unconnected_gradients.UnconnectedGradients.NONE)
   def testUnconnectedOpWithMultipleOutputsStopGradient(
@@ -1254,6 +1283,9 @@ class CustomGradientTest(test_util.TensorFlowTestCase, parameterized.TestCase):
 
   @test_util.enable_quantized_dtypes_training
   def testCustomGradientQuantizedDtypeTraining(self):
+    # TODO(b/309175067): Remove below skipTest() when fixed.
+    if sys.platform == "darwin":
+      self.skipTest("This test fails in TF MacOS nightly and continuous builds")
     with context.eager_mode():
       @custom_gradient.custom_gradient
       def F(x):
@@ -1315,7 +1347,7 @@ class CustomGradientTest(test_util.TensorFlowTestCase, parameterized.TestCase):
       with variable_scope.variable_scope("f", use_resource=True) as vs:
         a = array_ops.ones((2, 4))
 
-        # Variabes in these layers shouldn't be picked up by the decorator.
+        # Variables in these layers shouldn't be picked up by the decorator.
         b = core_layers.dense(a, 3, use_bias=False)
         c = core_layers.dense(b, 3, use_bias=False)
         x = core_layers.dense(b, 3, use_bias=False) + c
