@@ -17,6 +17,7 @@
 from absl.testing import parameterized
 import numpy as np
 
+from tensorflow.python.eager import def_function
 from tensorflow.python.framework import constant_op
 from tensorflow.python.framework import dtypes
 from tensorflow.python.framework import extension_type
@@ -241,6 +242,15 @@ class WeakTensorUnaryOpsTest(
     res = unary_api_specific_dtype(tensor_input)
     self.assertIsInstance(res, tensor.Tensor)
 
+  @test_util.run_in_graph_and_eager_modes
+  def test_weak_tensor_from_scalar_in_tf_func(self):
+    @def_function.function()
+    def f():
+      return 1
+
+    res = f()
+    self.assertIsInstance(res, WeakTensor)
+
   # Test unary ops with optional dtype arg.
   @parameterized.parameters(
       ("WeakTensor", dtypes.float32, WeakTensor),
@@ -510,8 +520,7 @@ class WeakTensorBinaryOpsTest(
           self.match_expected(y**x, reverse_expected_val, expected_dtype)
 
     run_test_pow(a=4, b=2)
-    run_test_pow(a=41, b=10)
-    run_test_pow(a=2, b=6)
+    run_test_pow(a=10, b=5)
 
   def test_weak_tensor_mod(self, a_dtype, b_dtype, expected_dtype):
     def run_test_mod(a, b):
@@ -806,11 +815,11 @@ class WeakTensorBinaryOpsTest(
     run_test_div_no_nan(a=2, b=6)
 
     # Test div_no_nan(x, 0) = 0 even if x is NaN or Inf.
-    x = np.NaN
+    x = np.nan
     y = 0
     self.match_expected(math_ops.div_no_nan(x, y), 0, (dtypes.float32, True))
 
-    x = np.Inf
+    x = np.inf
     self.match_expected(math_ops.div_no_nan(x, y), 0, (dtypes.float32, True))
 
   def test_weak_tensor_multiply_no_nan(self, a_dtype, b_dtype, expected_dtype):
@@ -838,13 +847,13 @@ class WeakTensorBinaryOpsTest(
     run_test_multiply_no_nan(a=2, b=6)
 
     # Test multiply_no_nan(x, 0) = 0 even if x is NaN or Inf.
-    x = np.NaN
+    x = np.nan
     y = 0
     self.match_expected(
         math_ops.multiply_no_nan(x, y), 0, (dtypes.float32, True)
     )
 
-    x = np.Inf
+    x = np.inf
     self.match_expected(
         math_ops.multiply_no_nan(x, y), 0, (dtypes.float32, True)
     )
