@@ -241,26 +241,26 @@ class PrintModelAnalysisTest(test.TestCase):
           self.assertLess(101600, tfprof_node.total_float_ops)
         else:
           self.assertLess(145660, tfprof_node.total_float_ops)
-        self.assertEqual(8, len(tfprof_node.children))
+        self.assertEqual(10, len(tfprof_node.children))
         self.assertEqual('_TFProfRoot', tfprof_node.name)
-        self.assertEqual('model_analyzer_testlib.py:63:BuildFullModel',
+        self.assertEqual('model_analyzer_testlib.py:59:BuildFullModel',
                          tfprof_node.children[0].name)
         self.assertEqual(
-            'model_analyzer_testlib.py:63:BuildFullModel (gradient)',
+            'model_analyzer_testlib.py:59:BuildFullModel (gradient)',
             tfprof_node.children[1].name)
-        self.assertEqual('model_analyzer_testlib.py:67:BuildFullModel',
+        self.assertEqual('model_analyzer_testlib.py:62:BuildFullModel',
                          tfprof_node.children[2].name)
         self.assertEqual(
-            'model_analyzer_testlib.py:67:BuildFullModel (gradient)',
+            'model_analyzer_testlib.py:62:BuildFullModel (gradient)',
             tfprof_node.children[3].name)
-        self.assertEqual('model_analyzer_testlib.py:69:BuildFullModel',
+        self.assertEqual('model_analyzer_testlib.py:63:BuildFullModel',
                          tfprof_node.children[4].name)
-        self.assertEqual('model_analyzer_testlib.py:70:BuildFullModel',
+        self.assertEqual('model_analyzer_testlib.py:63:BuildFullModel (gradient)',
                          tfprof_node.children[5].name)
         self.assertEqual(
-            'model_analyzer_testlib.py:70:BuildFullModel (gradient)',
+            'model_analyzer_testlib.py:65:BuildFullModel',
             tfprof_node.children[6].name)
-        self.assertEqual('model_analyzer_testlib.py:72:BuildFullModel',
+        self.assertEqual('model_analyzer_testlib.py:66:BuildFullModel',
                          tfprof_node.children[7].name)
         # pylint: enable=line-too-long
 
