@@ -882,7 +882,7 @@ class DispatchV2Test(test_util.TensorFlowTestCase):
 
     dispatch.unregister_dispatch_for(masked_tensor_equals)  # clean up.
 
-    with self.assertRaisesRegexp(
+    with self.assertRaisesRegex(
         ValueError, r"Dispatch function's signature \(self, other, name=None\) "
         r"does not match API's signature \(self, other\)\."):
 
@@ -957,7 +957,7 @@ class DispatchV2Test(test_util.TensorFlowTestCase):
       def silly_abs(x: SillyTensor):
         del x
 
-      # Note: `expeced` does not contain keys or values from SillyTensor.
+      # Note: `expected` does not contain keys or values from SillyTensor.
       targets = dispatch.type_based_dispatch_signatures_for(MaskedTensor)
       expected = {math_ops.add: [{"x": MaskedTensor, "y": MaskedTensor}],
                   array_ops.concat: [{"values": MaskedTensorList}]}
@@ -1143,21 +1143,22 @@ class DispatchV2Test(test_util.TensorFlowTestCase):
     dispatch.update_docstrings_with_api_lists()
     self.assertRegex(
         dispatch.dispatch_for_api.__doc__,
-        r"(?s)  The TensorFlow APIs that may be overridden "
-        r"by `@dispatch_for_api` are:\n\n.*"
-        r"  \* `tf\.concat\(values, axis, name\)`\n.*"
-        r"  \* `tf\.math\.add\(x, y, name\)`\n.*")
+        r"(?s)The TensorFlow APIs that may be overridden "
+        r"by `@dispatch_for_api` are:.*"
+        r"\* `tf\.concat\(values, axis, name\)`.*"
+        r"\* `tf\.math\.add\(x, y, name\)`.*")
     self.assertRegex(
         dispatch.dispatch_for_unary_elementwise_apis.__doc__,
-        r"(?s)  The unary elementwise APIs are:\n\n.*"
-        r"  \* `tf\.math\.abs\(x, name\)`\n.*"
-        r"  \* `tf\.math\.cos\(x, name\)`\n.*")
+        r"(?s)The unary elementwise APIs are:.*"
+        r"\* `tf\.math\.abs\(x, name\)`.*"
+        r"\* `tf\.math\.cos\(x, name\)`.*")
     self.assertRegex(
         dispatch.dispatch_for_binary_elementwise_apis.__doc__,
-        r"(?s)  The binary elementwise APIs are:\n\n.*"
-        r"  \* `tf\.math\.add\(x, y, name\)`\n.*"
-        r"  \* `tf\.math\.multiply\(x, y, name\)`\n.*")
+        r"(?s)The binary elementwise APIs are:.*"
+        r"\* `tf\.math\.add\(x, y, name\)`.*"
+        r"\* `tf\.math\.multiply\(x, y, name\)`.*")
 
 
 if __name__ == "__main__":
   googletest.main()
+
