@@ -26,6 +26,7 @@ import numpy as np
 from tensorflow.python.framework import constant_op
 from tensorflow.python.framework import dtypes
 from tensorflow.python.framework import ops
+from tensorflow.python.framework import tensor
 from tensorflow.python.framework import test_util
 from tensorflow.python.ops import array_ops
 from tensorflow.python.ops import math_ops
@@ -72,7 +73,7 @@ class _CustomSequenceThatRaisesException(collections.abc.Sequence):
 @dataclasses.dataclass
 class MaskedTensor:
   mask: bool
-  value: ops.Tensor
+  value: tensor.Tensor
 
   def __tf_flatten__(self):
     metadata = (self.mask,)
@@ -91,7 +92,7 @@ class MaskedTensor:
     )
 
   def __len__(self):
-    # Used by `nest.map_structure_up_to` and releatd functions to verify the
+    # Used by `nest.map_structure_up_to` and related functions to verify the
     # arity compatibility.
     return 1
 
@@ -153,24 +154,24 @@ class NestTest(parameterized.TestCase, test.TestCase):
       field1 = attr.ib()
       field2 = attr.ib()
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testDataclassCustomProtocol(self):
     mt = MaskedTensor(mask=True, value=constant_op.constant([1]))
     self.assertIsInstance(mt, CustomNestProtocol)
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testDataclassIsNested(self):
     mt = MaskedTensor(mask=True, value=constant_op.constant([1]))
     self.assertTrue(nest.is_nested(mt))
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testDataclassFlatten(self):
     mt = MaskedTensor(mask=True, value=constant_op.constant([1]))
     leaves = nest.flatten(mt)
     self.assertLen(leaves, 1)
     self.assertAllEqual(leaves[0], [1])
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testDataclassFlattenUpToCompatible(self):
     simple_list = [2]
     mt = MaskedTensor(mask=True, value=constant_op.constant([1]))
@@ -199,7 +200,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
     )
     self.assertAllEqual(flat_path_nested_list, [2])
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testDataclassFlattenUpToIncompatible(self):
     simple_list = [2]
     mt = MaskedTensor(mask=True, value=constant_op.constant([1]))
@@ -238,7 +239,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
           shallow_tree=nested_list, input_tree=mt, check_types=False
       )
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testDataclassFlattenWithTuplePathsUpToCompatible(self):
     simple_list = [2]
     mt = MaskedTensor(mask=True, value=constant_op.constant([1]))
@@ -270,7 +271,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
     )
     self.assertAllEqual(flat_path_nested_list, [[(0, 0), 2]])
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testDataclassFlattenWithTuplePathsUpToIncompatible(self):
     simple_list = [2]
     mt = MaskedTensor(mask=True, value=constant_op.constant([1]))
@@ -310,7 +311,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
           shallow_tree=nested_list2, input_tree=nmt, check_types=False
       )
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testDataclassFlattenAndPack(self):
     mt = MaskedTensor(mask=True, value=constant_op.constant([1]))
     leaves = nest.flatten(mt)
@@ -318,7 +319,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
     self.assertIsInstance(reconstructed_mt, MaskedTensor)
     self.assertEqual(reconstructed_mt, mt)
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testDataclassMapStructure(self):
     mt = MaskedTensor(mask=True, value=constant_op.constant([1]))
     mt_doubled = nest.map_structure(lambda x: x * 2, mt)
@@ -326,7 +327,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
     self.assertEqual(mt_doubled.mask, True)
     self.assertAllEqual(mt_doubled.value, [2])
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testDataclassMapStructureWithPaths(self):
     mt = MaskedTensor(mask=False, value=constant_op.constant([1]))
     mt2 = MaskedTensor(mask=True, value=constant_op.constant([2]))
@@ -359,7 +360,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
     self.assertAllEqual(nmt_combined_with_path.value.value[0], "0/0")
     self.assertAllEqual(nmt_combined_with_path.value.value[1], [9])
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testDataclassMapStructureWithTuplePaths(self):
     mt = MaskedTensor(mask=False, value=constant_op.constant([1]))
     mt2 = MaskedTensor(mask=True, value=constant_op.constant([2]))
@@ -394,7 +395,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
     self.assertAllEqual(nmt_combined_with_path.value.value[0], (0, 0))
     self.assertAllEqual(nmt_combined_with_path.value.value[1], [9])
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testDataclassMapStructureUpTo(self):
     mt = MaskedTensor(mask=True, value=constant_op.constant([1]))
     mt2 = MaskedTensor(mask=True, value=constant_op.constant([2]))
@@ -430,7 +431,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
     self.assertEqual(nmt_combined_with_path.value.mask, True)
     self.assertAllEqual(nmt_combined_with_path.value.value, [9])
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testDataclassMapStructureWithTuplePathsUoTo(self):
     mt = MaskedTensor(mask=True, value=constant_op.constant([1]))
     mt2 = MaskedTensor(mask=True, value=constant_op.constant([2]))
@@ -469,7 +470,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
     self.assertAllEqual(nmt_combined_with_path.value.value[0], (0, 0))
     self.assertAllEqual(nmt_combined_with_path.value.value[1], [9])
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testNestedDataclassIsNested(self):
     mt = MaskedTensor(mask=True, value=constant_op.constant([1]))
     self.assertTrue(nest.is_nested(mt))
@@ -479,7 +480,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
     )
     self.assertTrue(nest.is_nested(nmt))
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testDataclassAssertShallowStructure(self):
     # These assertions are expected to pass: two dataclasses with the same
     # component size are considered to have the same shallow structure.
@@ -534,7 +535,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
           shallow_tree=nmt, input_tree=mt, check_types=False
       )
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testDataclassGetTraverseShallowStructure(self):
     nmt = NestedMaskedTensor.nested_masked_tensor_with_opposite_masks(
         mask=True, inner_value=constant_op.constant([1])
@@ -560,14 +561,14 @@ class NestTest(parameterized.TestCase, test.TestCase):
     nest.assert_shallow_structure(traverse_result2, nmt)
 
     traverse_result3 = nest.get_traverse_shallow_structure(
-        lambda s: isinstance(s, ops.Tensor), nmt
+        lambda s: isinstance(s, tensor.Tensor), nmt
     )
     # Expected `traverse_result3 = False` because `nmt` doesn't pass the
     # traverse function.
     self.assertEqual(traverse_result3, False)
     nest.assert_shallow_structure(traverse_result3, nmt)
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testNestedDataclassFlatten(self):
     nmt = NestedMaskedTensor.nested_masked_tensor_with_opposite_masks(
         mask=True, inner_value=constant_op.constant([1])
@@ -576,7 +577,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
     self.assertLen(leaves, 1)
     self.assertAllEqual(leaves[0], [1])
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testNestedDataclassFlattenAndPack(self):
     nmt = NestedMaskedTensor.nested_masked_tensor_with_opposite_masks(
         mask=True, inner_value=constant_op.constant([1])
@@ -586,7 +587,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
     self.assertIsInstance(reconstructed_mt, NestedMaskedTensor)
     self.assertEqual(reconstructed_mt, nmt)
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testNestedDataclassMapStructure(self):
     nmt = NestedMaskedTensor.nested_masked_tensor_with_opposite_masks(
         mask=True, inner_value=constant_op.constant([1])
@@ -601,7 +602,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
     self.assertEqual(mt_doubled.value.mask, expected.value.mask)
     self.assertAllEqual(mt_doubled.value.value, expected.value.value)
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testDataclassYieldFlatPaths(self):
     mt = MaskedTensor(mask=True, value=constant_op.constant([1]))
     mt_flat_paths = list(nest.yield_flat_paths(mt))
@@ -625,7 +626,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
         ],
     )
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testDataclassFlattenWithStringPaths(self):
     sep = "/"
     mt = MaskedTensor(mask=True, value=constant_op.constant([1]))
@@ -649,7 +650,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
     self.assertEqual(dict_mt_nmt_flat_paths[1][0], "nmt/0/0")
     self.assertAllEqual(dict_mt_nmt_flat_paths[1][1], [2])
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testDataclassFlattenWithTuplePaths(self):
     mt = MaskedTensor(mask=True, value=constant_op.constant([1]))
     mt_flat_paths = nest.flatten_with_tuple_paths(mt)
@@ -670,7 +671,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
     self.assertEqual(dict_mt_nmt_flat_paths[1][0], ("nmt", 0, 0))
     self.assertAllEqual(dict_mt_nmt_flat_paths[1][1], [2])
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testDataclassListToTuple(self):
     mt = MaskedTensor(mask=True, value=constant_op.constant([1]))
     nmt = NestedMaskedTensor.nested_masked_tensor_with_opposite_masks(
@@ -689,7 +690,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
     )
     nest.assert_same_structure(results, expected)
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testAttrsFlattenAndPack(self):
     if attr is None:
       self.skipTest("attr module is unavailable.")
@@ -714,7 +715,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
       {"values": [(1, 2), [3, 4], 5]},
       {"values": [PointXY(1, 2), 3, 4]},
   )
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testAttrsMapStructure(self, values):
     if attr is None:
       self.skipTest("attr module is unavailable.")
@@ -723,7 +724,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
     new_structure = nest.map_structure(lambda x: x, structure)
     self.assertEqual(structure, new_structure)
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testFlattenAndPack(self):
     structure = ((3, 4), 5, (6, 7, (9, 10), 8))
     flat = ["a", "b", "c", "d", "e", "f", "g", "h"]
@@ -760,7 +761,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
 
   @parameterized.parameters({"mapping_type": collections.OrderedDict},
                             {"mapping_type": _CustomMapping})
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testFlattenDictOrder(self, mapping_type):
     """`flatten` orders dicts by key, including OrderedDicts."""
     ordered = mapping_type([("d", 3), ("b", 1), ("a", 0), ("c", 2)])
@@ -786,7 +787,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
         custom_reconstruction)
     self.assertEqual({"d": 3, "b": 1, "a": 0, "c": 2}, plain_reconstruction)
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testFlattenAndPackMappingViews(self):
     """`flatten` orders dicts by key, including OrderedDicts."""
     ordered = collections.OrderedDict([("d", 3), ("b", 1), ("a", 0), ("c", 2)])
@@ -805,7 +806,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
 
   Abc = collections.namedtuple("A", ("b", "c"))  # pylint: disable=invalid-name
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testFlattenAndPack_withDicts(self):
     # A nice messy mix of tuples, lists, dicts, and `OrderedDict`s.
     mess = [
@@ -888,7 +889,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
         ValueError, "Structure had 2 atoms, but flat_sequence had 1 items."):
       nest.pack_sequence_as(val, [val], expand_composites=True)
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testIsNested(self):
     self.assertFalse(nest.is_nested("1234"))
     self.assertTrue(nest.is_nested([1, 3, [4, 5]]))
@@ -941,7 +942,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
   class SameNamedType1(SameNameab):
     pass
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testAssertSameStructure(self):
     structure1 = (((1, 2), 3), 4, (5, 6))
     structure2 = ((("foo1", "foo2"), "foo3"), "foo4", ("foo5", "foo6"))
@@ -1052,7 +1053,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
     nest.assert_same_structure({"a": 4}, _CustomMapping(a=3))
     nest.assert_same_structure(_CustomMapping(b=3), {"b": 4})
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testMapStructure(self):
     structure1 = (((1, 2), 3), 4, (5, 6))
     structure2 = (((7, 8), 9), 10, (11, 12))
@@ -1128,7 +1129,7 @@ class NestTest(parameterized.TestCase, test.TestCase):
 
   ABTuple = collections.namedtuple("ab_tuple", "a, b")  # pylint: disable=invalid-name
 
-  @test_util.assert_no_new_pyobjects_executing_eagerly
+  @test_util.assert_no_new_pyobjects_executing_eagerly()
   def testMapStructureWithStrings(self):
     inp_a = NestTest.ABTuple(a="foo", b=("bar", "baz"))
     inp_b = NestTest.ABTuple(a=2, b=(1, 3))
