@@ -22,16 +22,38 @@ limitations under the License.
 #include <vector>
 
 #include "absl/memory/memory.h"
-#include "tensorflow/core/lib/gtl/map_util.h"
+#include "xla/tsl/platform/macros.h"
 #include "tensorflow/core/lib/strings/strcat.h"
 #include "tensorflow/core/platform/logging.h"
 #include "tensorflow/core/platform/mutex.h"
+#include "tensorflow/core/platform/stringpiece.h"
 #include "tensorflow/python/lib/core/safe_pyobject_ptr.h"
+#include "tsl/platform/thread_annotations.h"
 
 namespace tensorflow {
 namespace swig {
 
 namespace {
+constexpr const char ITERATOR_OPS_MODULE[] =
+    "tensorflow.python.data.ops.iterator_ops";
+constexpr const char COMPOSITE_TENSOR_MODULE[] =
+    "tensorflow.python.framework.composite_tensor";
+constexpr const char INDEXED_SLICES_MODULE[] =
+    "tensorflow.python.framework.indexed_slices";
+constexpr const char OPS_MODULE[] =
+    "tensorflow.python.framework.ops";
+constexpr const char SPARSE_TENSOR_MODULE[] =
+    "tensorflow.python.framework.sparse_tensor";
+constexpr const char TENSOR_MODULE[] =
+    "tensorflow.python.framework.tensor";
+constexpr const char TYPE_SPEC_MODULE[] =
+    "tensorflow.python.framework.type_spec";
+constexpr const char RESOURCE_VAR_MODULE[] =
+    "tensorflow.python.ops.resource_variable_ops";
+constexpr const char VARIABLES_MODULE[] =
+    "tensorflow.python.ops.variables";
+constexpr const char CORE_TYPES_MODULE[] =
+    "tensorflow.python.types.core";
 string PyObjectToString(PyObject* o);
 }  // namespace
 
@@ -53,17 +75,6 @@ PyObject* GetRegisteredPyObject(const string& name) {
   return it->second;
 }
 
-PyObject* RegisterType(PyObject* type_name, PyObject* type) {
-  if (!PyType_Check(type)) {
-    PyErr_SetString(PyExc_TypeError,
-                    tensorflow::strings::StrCat("Expecting a type, got ",
-                                                Py_TYPE(type)->tp_name)
-                        .c_str());
-    return nullptr;
-  }
-  return RegisterPyObject(type_name, type);
-}
-
 PyObject* RegisterPyObject(PyObject* name, PyObject* value) {
   string key;
   if (PyBytes_Check(name)) {
@@ -109,7 +120,7 @@ bool IsString(PyObject* o) {
 // Note that '__class__' attribute is set only in new-style classes.
 // A lot of tensorflow code uses __class__ without checks, so it seems like
 // we only support new-style classes.
-StringPiece GetClassName(PyObject* o) {
+absl::string_view GetClassName(PyObject* o) {
   // __class__ is equivalent to type() for new style classes.
   // type() is equivalent to PyObject_Type()
   // (https://docs.python.org/3.5/c-api/object.html#c.PyObject_Type)
@@ -119,9 +130,9 @@ StringPiece GetClassName(PyObject* o) {
 
   // __name__ is the value of `tp_name` after the last '.'
   // (https://docs.python.org/2/c-api/typeobj.html#c.PyTypeObject.tp_name)
-  StringPiece name(type->tp_name);
+  absl::string_view name(type->tp_name);
   size_t pos = name.rfind('.');
-  if (pos != StringPiece::npos) {
+  if (pos != absl::string_view::npos) {
     name.remove_prefix(pos + 1);
   }
   return name;
@@ -212,22 +223,31 @@ class CachedTypeCheck {
       TF_GUARDED_BY(type_to_sequence_map_mu_);
 };
 
-// Returns 1 if 'obj' is an instance of 'type_name'
-// Returns 0 otherwise.
-// Returns -1 if an error occurred (e.g., if 'type_name' is not registered.)
-int IsInstanceOfRegisteredType(PyObject* obj, const char* type_name) {
-  PyObject* type_obj = GetRegisteredPyObject(type_name);
-  if (TF_PREDICT_FALSE(type_obj == nullptr)) {
-    PyErr_SetString(PyExc_RuntimeError,
-                    tensorflow::strings::StrCat(
-                        type_name,
-                        " type has not been set. "
-                        "Please register the type with the identifier \"",
-                        type_name, "\" using RegisterType.")
-                        .c_str());
-    return -1;
+PyObject* ImportTypeFromModule(const char* module_name, const char* type_name) {
+  static PyObject* given_type;
+  given_type = [module_name, type_name]() {
+    PyObject* module = PyImport_ImportModule(module_name);
+    PyObject* attr =
+        module ? PyObject_GetAttrString(module, type_name) : nullptr;
+    if (attr == nullptr) {
+      PyErr_WriteUnraisable(nullptr);
+      PyErr_Clear();
+    }
+    if (module) Py_DECREF(module);
+    return attr;
+  }();
+  return given_type;
+}
+
+// Returns true if 'obj' is an instance of 'type_name'
+// Returns false otherwise.
+int IsInstanceOfGivenType(PyObject* obj, const char* module_name,
+                          const char* type_name) {
+  PyObject* given_type = ImportTypeFromModule(module_name, type_name);
+  if (TF_PREDICT_FALSE(given_type == nullptr)) {
+    return false;
   }
-  return PyObject_IsInstance(obj, type_obj);
+  return PyObject_IsInstance(obj, given_type);
 }
 
 // Returns 1 if `o` is considered a mapping for the purposes of Flatten().
@@ -235,7 +255,7 @@ int IsInstanceOfRegisteredType(PyObject* obj, const char* type_name) {
 // Returns -1 if an error occurred.
 int IsMappingHelper(PyObject* o) {
   static auto* const check_cache = new CachedTypeCheck([](PyObject* to_check) {
-    return IsInstanceOfRegisteredType(to_check, "Mapping");
+    return IsInstanceOfGivenType(to_check, "collections.abc", "Mapping");
   });
   if (PyDict_Check(o)) return true;
   return check_cache->CachedLookup(o);
@@ -245,7 +265,7 @@ int IsMappingHelper(PyObject* o) {
 // Flatten(). Returns 0 otherwise. Returns -1 if an error occurred.
 int IsMutableMappingHelper(PyObject* o) {
   static auto* const check_cache = new CachedTypeCheck([](PyObject* to_check) {
-    return IsInstanceOfRegisteredType(to_check, "MutableMapping");
+    return IsInstanceOfGivenType(to_check, "collections.abc", "MutableMapping");
   });
   if (PyDict_Check(o)) return true;
   return check_cache->CachedLookup(o);
@@ -256,7 +276,7 @@ int IsMutableMappingHelper(PyObject* o) {
 // Returns -1 if an error occurred.
 int IsMappingViewHelper(PyObject* o) {
   static auto* const check_cache = new CachedTypeCheck([](PyObject* to_check) {
-    return IsInstanceOfRegisteredType(to_check, "MappingView");
+    return IsInstanceOfGivenType(to_check, "collections.abc", "MappingView");
   });
   return check_cache->CachedLookup(o);
 }
@@ -266,7 +286,7 @@ int IsMappingViewHelper(PyObject* o) {
 // Returns -1 if an error occurred.
 int IsObjectProxy(PyObject* o) {
   static auto* const check_cache = new CachedTypeCheck([](PyObject* to_check) {
-    return IsInstanceOfRegisteredType(to_check, "ObjectProxy");
+    return IsInstanceOfGivenType(to_check, "wrapt", "ObjectProxy");
   });
   return check_cache->CachedLookup(o);
 }
@@ -309,7 +329,8 @@ int IsCustomNestProtocolDefined(PyObject* o) {
 // Returns -1 if an error occurred.
 int IsIndexedSlicesHelper(PyObject* o) {
   static auto* const check_cache = new CachedTypeCheck([](PyObject* to_check) {
-    return IsInstanceOfRegisteredType(to_check, "IndexedSlices");
+    return IsInstanceOfGivenType(to_check, INDEXED_SLICES_MODULE,
+                                 "IndexedSlices");
   });
   return check_cache->CachedLookup(o);
 }
@@ -319,7 +340,7 @@ int IsIndexedSlicesHelper(PyObject* o) {
 // Returns -1 if an error occurred.
 int IsTensorHelper(PyObject* o) {
   static auto* const check_cache = new CachedTypeCheck([](PyObject* to_check) {
-    return IsInstanceOfRegisteredType(to_check, "Tensor");
+    return IsInstanceOfGivenType(to_check, TENSOR_MODULE, "Tensor");
   });
   return check_cache->CachedLookup(o);
 }
@@ -329,7 +350,7 @@ int IsTensorHelper(PyObject* o) {
 // Returns -1 if an error occurred.
 int IsTensorSpecHelper(PyObject* o) {
   static auto* const check_cache = new CachedTypeCheck([](PyObject* to_check) {
-    return IsInstanceOfRegisteredType(to_check, "TensorSpec");
+    return IsInstanceOfGivenType(to_check, TENSOR_MODULE, "TensorSpec");
   });
   return check_cache->CachedLookup(o);
 }
@@ -339,7 +360,21 @@ int IsTensorSpecHelper(PyObject* o) {
 // Returns -1 if an error occurred.
 int IsEagerTensorHelper(PyObject* o) {
   static auto* const check_cache = new CachedTypeCheck([](PyObject* to_check) {
-    return IsInstanceOfRegisteredType(to_check, "EagerTensor");
+    return IsInstanceOfGivenType(to_check, OPS_MODULE, "EagerTensor");
+  });
+  return check_cache->CachedLookup(o);
+}
+
+int IsTensorProtocolHelper(PyObject* o) {
+  static auto* const check_cache = new CachedTypeCheck([](PyObject* to_check) {
+    return IsInstanceOfGivenType(to_check, CORE_TYPES_MODULE, "TensorProtocol");
+  });
+  return check_cache->CachedLookup(o);
+}
+
+int IsCoreTypeValueHelper(PyObject* o) {
+  static auto* const check_cache = new CachedTypeCheck([](PyObject* to_check) {
+    return IsInstanceOfGivenType(to_check, CORE_TYPES_MODULE, "Value");
   });
   return check_cache->CachedLookup(o);
 }
@@ -349,7 +384,8 @@ int IsEagerTensorHelper(PyObject* o) {
 // Returns -1 if an error occurred.
 int IsResourceVariableHelper(PyObject* o) {
   static auto* const check_cache = new CachedTypeCheck([](PyObject* to_check) {
-    return IsInstanceOfRegisteredType(to_check, "ResourceVariable");
+    return IsInstanceOfGivenType(to_check, RESOURCE_VAR_MODULE,
+                                 "ResourceVariable");
   });
   return check_cache->CachedLookup(o);
 }
@@ -359,7 +395,8 @@ int IsResourceVariableHelper(PyObject* o) {
 // Returns -1 if an error occurred.
 int IsOwnedIteratorHelper(PyObject* o) {
   static auto* const check_cache = new CachedTypeCheck([](PyObject* to_check) {
-    return IsInstanceOfRegisteredType(to_check, "OwnedIterator");
+    return IsInstanceOfGivenType(to_check, ITERATOR_OPS_MODULE,
+                                 "OwnedIterator");
   });
   return check_cache->CachedLookup(o);
 }
@@ -369,7 +406,7 @@ int IsOwnedIteratorHelper(PyObject* o) {
 // Returns -1 if an error occurred.
 int IsVariableHelper(PyObject* o) {
   static auto* const check_cache = new CachedTypeCheck([](PyObject* to_check) {
-    return IsInstanceOfRegisteredType(to_check, "Variable");
+    return IsInstanceOfGivenType(to_check, VARIABLES_MODULE, "Variable");
   });
   return check_cache->CachedLookup(o);
 }
@@ -385,7 +422,8 @@ int IsNestedHelper(PyObject* o) {
   if (IsCustomNestProtocolDefined(o)) return true;
 
   static auto* const check_cache = new CachedTypeCheck([](PyObject* to_check) {
-    int is_instance = IsInstanceOfRegisteredType(to_check, "Sequence");
+    int is_instance =
+        IsInstanceOfGivenType(to_check, "collections.abc", "Sequence");
 
     // Don't cache a failed is_instance check.
     if (is_instance == -1) return -1;
@@ -603,11 +641,10 @@ class CustomNestedIterator : public ValueIterator {
 
 bool IsSparseTensorValueType(PyObject* o) {
   PyObject* sparse_tensor_value_type =
-      GetRegisteredPyObject("SparseTensorValue");
+      ImportTypeFromModule(SPARSE_TENSOR_MODULE, "SparseTensorValue");
   if (TF_PREDICT_FALSE(sparse_tensor_value_type == nullptr)) {
     return false;
   }
-
   return PyObject_TypeCheck(
              o, reinterpret_cast<PyTypeObject*>(sparse_tensor_value_type)) == 1;
 }
@@ -618,7 +655,8 @@ bool IsSparseTensorValueType(PyObject* o) {
 bool IsCompositeTensorHelper(PyObject* o) {
   static auto* const check_cache = new CachedTypeCheck([](PyObject* to_check) {
     // TODO(b/246438937): Remove the ResourceVariable test.
-    return IsInstanceOfRegisteredType(to_check, "CompositeTensor") &&
+    return IsInstanceOfGivenType(to_check, COMPOSITE_TENSOR_MODULE,
+                                 "CompositeTensor") &&
            !IsResourceVariable(to_check);
   });
   return check_cache->CachedLookup(o);
@@ -630,10 +668,12 @@ bool IsCompositeTensorHelper(PyObject* o) {
 // Returns -1 if an error occurred.
 bool IsTypeSpecHelper(PyObject* o) {
   static auto* const check_cache = new CachedTypeCheck([](PyObject* to_check) {
-    int is_type_spec = IsInstanceOfRegisteredType(to_check, "TypeSpec");
+    int is_type_spec =
+        IsInstanceOfGivenType(to_check, TYPE_SPEC_MODULE, "TypeSpec");
     // TODO(b/246438937): Remove the VariableSpec special case.
-    int is_dense_spec = (IsInstanceOfRegisteredType(to_check, "TensorSpec") ||
-                         IsInstanceOfRegisteredType(to_check, "VariableSpec"));
+    int is_dense_spec =
+        (IsInstanceOfGivenType(to_check, TENSOR_MODULE, "TensorSpec") ||
+         IsInstanceOfGivenType(to_check, RESOURCE_VAR_MODULE, "VariableSpec"));
     if ((is_type_spec == -1) || (is_dense_spec == -1)) return -1;
     return static_cast<int>(is_type_spec && !is_dense_spec);
   });
@@ -776,7 +816,7 @@ void SetDifferentKeysError(PyObject* dict1, PyObject* dict2, string* error_msg,
 // Returns true iff there were no "internal" errors. In other words,
 // errors that has nothing to do with structure checking.
 // If an "internal" error occurred, the appropriate Python error will be
-// set and the caller can propage it directly to the user.
+// set and the caller can propagate it directly to the user.
 //
 // Both `error_msg` and `is_type_error` must be non-null. `error_msg` must
 // be empty.
@@ -1005,6 +1045,8 @@ bool IsOwnedIterator(PyObject* o) { return IsOwnedIteratorHelper(o) == 1; }
 bool IsVariable(PyObject* o) { return IsVariableHelper(o) == 1; }
 bool IsIndexedSlices(PyObject* o) { return IsIndexedSlicesHelper(o) == 1; }
 bool IsDispatchable(PyObject* o) { return IsDispatchableHelper(o) == 1; }
+bool IsTensorProtocol(PyObject* o) { return IsTensorProtocolHelper(o) == 1; }
+bool IsCoreTypeValue(PyObject* o) { return IsCoreTypeValueHelper(o) == 1; }
 
 bool IsTuple(PyObject* o) {
   tensorflow::Safe_PyObjectPtr wrapped;
@@ -1112,7 +1154,8 @@ PyObject* IsNamedtuple(PyObject* o, bool strict) {
   }
 
   Safe_PyObjectPtr fields = make_safe(PyObject_GetAttrString(o, "_fields"));
-  int is_instance = IsInstanceOfRegisteredType(fields.get(), "Sequence");
+  int is_instance =
+      IsInstanceOfGivenType(fields.get(), "collections.abc", "Sequence");
   if (is_instance == 0) {
     Py_RETURN_FALSE;
   } else if (is_instance == -1) {
