@@ -13,12 +13,12 @@ See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
 
-#include <cstdint>
-#include <cstdlib>
+#include <cassert>
 #include <string>
 #include <string_view>
 
 #include "fuzztest/fuzztest.h"
+#include "absl/status/status.h"
 #include "tensorflow/core/platform/base64.h"
 #include "tensorflow/core/platform/status.h"
 #include "tensorflow/core/platform/stringpiece.h"
@@ -30,7 +30,7 @@ namespace {
 void FuzzTest(std::string_view input) {
   std::string encoded_string;
   std::string decoded_string;
-  tensorflow::Status s;
+  absl::Status s;
   s = tensorflow::Base64Encode(input, &encoded_string);
   assert(s.ok());
   s = tensorflow::Base64Decode(encoded_string, &decoded_string);
