@@ -12,21 +12,20 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
-#include <cstdint>
-#include <cstdlib>
+#include <cassert>
 #include <string>
-#include <string_view>
 
 #include "fuzztest/fuzztest.h"
 #include "tensorflow/core/platform/str_util.h"
 #include "tensorflow/core/platform/stringpiece.h"
+#include "tensorflow/core/platform/types.h"
 
 // This is a fuzzer for tensorflow::str_util::ConsumeLeadingDigits
 
 namespace {
 
 void FuzzTest(std::string data) {
-  tensorflow::StringPiece sp(data);
+  absl::string_view sp(data);
   tensorflow::uint64 val;
 
   const bool leading_digits =
