@@ -12,8 +12,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include <gtest/gtest.h>
 #include "fuzztest/fuzztest.h"
+#include "tensorflow/cc/ops/math_ops.h"
 #include "tensorflow/cc/ops/standard_ops.h"
 #include "tensorflow/core/framework/types.pb.h"
 #include "tensorflow/security/fuzzing/cc/core/framework/tensor_domains.h"
@@ -23,11 +23,11 @@ namespace tensorflow {
 namespace fuzzing {
 
 // Creates FuzzIdentity class that wraps a single operation node session.
-BINARY_INPUT_OP_FUZZER(DT_INT32, MatMul);
+BINARY_INPUT_OP_FUZZER(DT_INT32, DT_INT32, MatMul);
 // Setup up fuzzing test.
 FUZZ_TEST_F(FuzzMatMul, Fuzz)
-    .WithDomains(fuzzing::AnySmallValidTensor(),
-                 fuzzing::AnySmallValidTensor());
+    .WithDomains(fuzzing::AnySmallValidNumericTensor(),
+                 fuzzing::AnySmallValidNumericTensor());
 
 }  // end namespace fuzzing
 }  // end namespace tensorflow
