@@ -15,33 +15,28 @@ limitations under the License.
 
 #include <fuzzer/FuzzedDataProvider.h>
 
-#include <iostream>
-#include <memory>
 #include <string>
-#include <vector>
 
 #include "fuzztest/fuzztest.h"
-#include "tensorflow/compiler/xla/literal.h"
-#include "tensorflow/compiler/xla/shape_util.h"
-#include "tensorflow/compiler/xla/text_literal_reader.h"
-#include "tensorflow/compiler/xla/types.h"
-#include "tensorflow/compiler/xla/xla_data.pb.h"
-#include "tensorflow/tsl/platform/env.h"
+#include "xla/hlo/parser/hlo_parser.h"
+#include "xla/text_literal_reader.h"
+#include "xla/tsl/platform/env.h"
 
 namespace xla {
 namespace {
 
 void FuzzFileRead(std::string data) {
   std::string fname = "/tmp/ReadsR3File.txt";
-  if (!tsl::WriteStringToFile(tsl::Env::Default(), fname, data.c_str()).ok()) {
-    return;
-  }
-
-  if (TextLiteralReader::ReadPath(fname).ok()) {
+  if (!tsl::WriteStringToFile(tsl::Env::Default(), fname, data).ok()) {
     return;
   }
+  TextLiteralReader::ReadPath(fname).IgnoreError();
 }
 FUZZ_TEST(TextReaderFuzzer, FuzzFileRead);
 
+void FuzzHLOParseUnverified(std::string data) {
+  xla::ParseAndReturnUnverifiedModule(data).IgnoreError();
+}
+FUZZ_TEST(TextReaderFuzzer, FuzzHLOParseUnverified);
 }  // namespace
 }  // namespace xla
