@@ -26,20 +26,20 @@ namespace tensorflow {
 namespace graph_transforms {
 namespace {
 // Ensures the tensor is the expected shape.
-Status ErrorIfNotVector(const Tensor& input, const string& input_name,
-                        int expected_width) {
+absl::Status ErrorIfNotVector(const Tensor& input, const string& input_name,
+                              int expected_width) {
   if ((input.shape().dims() != 1) ||
       (input.shape().dim_size(0) != expected_width)) {
     return errors::InvalidArgument(
         input_name,
         " input to batch norm has bad shape: ", input.shape().DebugString());
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status GetScaleAndOffsetValues(const NodeMatch& match,
-                               std::vector<float>* scale_values,
-                               std::vector<float>* offset_values) {
+absl::Status GetScaleAndOffsetValues(const NodeMatch& match,
+                                     std::vector<float>* scale_values,
+                                     std::vector<float>* offset_values) {
   // Find all the nodes we expect in the subgraph.
   const NodeDef& batch_norm_node = match.node;
   // BatchNormWithGlobalNormalization and FusedBatchNorm ops only differ
@@ -99,14 +99,13 @@ Status GetScaleAndOffsetValues(const NodeMatch& match,
     (*offset_values)[i] =
         (-mean.flat<float>()(i) * (*scale_values)[i]) + beta.flat<float>()(i);
   }
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status FuseScaleOffsetToConvWeights(const std::vector<float>& scale_values,
-                                    const std::vector<float>& offset_values,
-                                    const NodeMatch& conv_node_match,
-                                    const string& conv_output_name,
-                                    std::vector<NodeDef>* new_nodes) {
+absl::Status FuseScaleOffsetToConvWeights(
+    const std::vector<float>& scale_values,
+    const std::vector<float>& offset_values, const NodeMatch& conv_node_match,
+    const string& conv_output_name, std::vector<NodeDef>* new_nodes) {
   const NodeDef& conv_node = conv_node_match.node;
   // CHECK_EQ("Conv2D", conv_node.op());
   const NodeDef& input_node = conv_node_match.inputs[0].node;
@@ -169,11 +168,11 @@ Status FuseScaleOffsetToConvWeights(const std::vector<float>& scale_values,
   AddNodeInput(conv_node.name(), &bias_add_node);
   AddNodeInput(bias_offset_node.name(), &bias_add_node);
   new_nodes->push_back(bias_add_node);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status FuseBatchNormWithConv(const NodeMatch& match,
-                             std::vector<NodeDef>* new_nodes) {
+absl::Status FuseBatchNormWithConv(const NodeMatch& match,
+                                   std::vector<NodeDef>* new_nodes) {
   // Calculate the scale and offset values to apply.
   std::vector<float> scale_values;
   std::vector<float> offset_values;
@@ -185,11 +184,11 @@ Status FuseBatchNormWithConv(const NodeMatch& match,
   TF_RETURN_IF_ERROR(
       FuseScaleOffsetToConvWeights(scale_values, offset_values, match.inputs[0],
                                    batch_norm_node.name(), new_nodes));
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status FuseBatchNormWithBatchToSpace(const NodeMatch& match,
-                                     std::vector<NodeDef>* new_nodes) {
+absl::Status FuseBatchNormWithBatchToSpace(const NodeMatch& match,
+                                           std::vector<NodeDef>* new_nodes) {
   // Calculate the scale and offset values to apply.
   std::vector<float> scale_values;
   std::vector<float> offset_values;
@@ -214,11 +213,11 @@ Status FuseBatchNormWithBatchToSpace(const NodeMatch& match,
   new_nodes->push_back(batch_to_space_node_match.inputs[1].node);
   new_nodes->push_back(batch_to_space_node_match.inputs[2].node);
   new_nodes->push_back(new_batch_to_space_node);
-  return OkStatus();
+  return absl::OkStatus();
 }
 
-Status FuseBatchNormWithConvConcat(const NodeMatch& match,
-                                   std::vector<NodeDef>* new_nodes) {
+absl::Status FuseBatchNormWithConvConcat(const NodeMatch& match,
+                                         std::vector<NodeDef>* new_nodes) {
   // Calculate the scale and offset values to apply.
   std::vector<float> scale_values;
   std::vector<float> offset_values;
@@ -275,15 +274,15 @@ Status FuseBatchNormWithConvConcat(const NodeMatch& match,
   concat_node.set_input(0, concat0_output_name);
   concat_node.set_input(1, concat1_output_name);
   new_nodes->push_back(concat_node);
-  return OkStatus();
+  return absl::OkStatus();
 }
 }  // namespace
 
 // Finds monolithic batch norm ops (as used in early versions of TensorFlow) and
 // converts them into premultiplied weight inputs to convolutions.
-Status FoldOldBatchNorms(const GraphDef& input_graph_def,
-                         const TransformFuncContext& context,
-                         GraphDef* output_graph_def) {
+absl::Status FoldOldBatchNorms(const GraphDef& input_graph_def,
+                               const TransformFuncContext& context,
+                               GraphDef* output_graph_def) {
   GraphDef current_graph_def = input_graph_def;
   // We have to do several passes to catch all the old BN nodes, since many of
   // them may share inputs and so be excluded from replacement in one pass.
@@ -313,7 +312,7 @@ Status FoldOldBatchNorms(const GraphDef& input_graph_def,
                             std::vector<NodeDef>* new_nodes) {
           TF_RETURN_IF_ERROR(FuseBatchNormWithConv(match, new_nodes));
           did_graph_change = true;
-          return OkStatus();
+          return absl::OkStatus();
         },
         {}, &replaced_graph_def));
     current_graph_def = replaced_graph_def;
@@ -350,7 +349,7 @@ Status FoldOldBatchNorms(const GraphDef& input_graph_def,
                             std::vector<NodeDef>* new_nodes) {
           TF_RETURN_IF_ERROR(FuseBatchNormWithBatchToSpace(match, new_nodes));
           did_graph_change = true;
-          return OkStatus();
+          return absl::OkStatus();
         },
         {}, &replaced_graph_def));
     current_graph_def = replaced_graph_def;
@@ -393,14 +392,14 @@ Status FoldOldBatchNorms(const GraphDef& input_graph_def,
                             std::vector<NodeDef>* new_nodes) {
           TF_RETURN_IF_ERROR(FuseBatchNormWithConvConcat(match, new_nodes));
           did_graph_change = true;
-          return OkStatus();
+          return absl::OkStatus();
         },
         {}, &replaced_graph_def));
     current_graph_def = replaced_graph_def;
   } while (did_graph_change);
 
   *output_graph_def = current_graph_def;
-  return OkStatus();
+  return absl::OkStatus();
 }
 
 REGISTER_GRAPH_TRANSFORM("fold_old_batch_norms", FoldOldBatchNorms);
