@@ -17,8 +17,10 @@ limitations under the License.
 
 #include <algorithm>
 #include <set>
+#include <string>
 #include <unordered_set>
 
+#include "absl/strings/string_view.h"
 #include "tensorflow/core/platform/logging.h"
 #include "tensorflow/core/platform/macros.h"
 #include "tensorflow/core/platform/types.h"
@@ -130,7 +132,7 @@ class Generator {
   void AppendMessageFunctions(const Descriptor& md);
 
   // Appends lines to open or close namespace declarations.
-  void AddNamespaceToCurrentSection(const string& package, bool open);
+  void AddNamespaceToCurrentSection(absl::string_view package, bool open);
 
   // Appends the given headers as sorted #include lines.
   void AddHeadersToCurrentSection(const std::vector<string>& headers);
@@ -145,14 +147,15 @@ class Generator {
 
   std::unordered_set<string> map_append_signatures_included_;
 
-  TF_DISALLOW_COPY_AND_ASSIGN(Generator);
+  Generator(const Generator&) = delete;
+  void operator=(const Generator&) = delete;
 };
 
 // Returns the prefix needed to reference objects defined in <fd>. E.g.
 // "::tensorflow::test".
 string GetPackageReferencePrefix(const FileDescriptor* fd) {
   string result = "::";
-  const string& package = fd->package();
+  absl::string_view package = fd->package();
   for (size_t i = 0; i < package.size(); ++i) {
     if (package[i] == '.') {
       result += "::";
@@ -166,13 +169,13 @@ string GetPackageReferencePrefix(const FileDescriptor* fd) {
 
 // Returns the name of the class generated by proto to represent <d>.
 string GetClassName(const Descriptor& d) {
-  if (d.containing_type() == nullptr) return d.name();
+  if (d.containing_type() == nullptr) return std::string(d.name());
   return StrCat(GetClassName(*d.containing_type()), "_", d.name());
 }
 
 // Returns the name of the class generated by proto to represent <ed>.
 string GetClassName(const EnumDescriptor& ed) {
-  if (ed.containing_type() == nullptr) return ed.name();
+  if (ed.containing_type() == nullptr) return std::string(ed.name());
   return StrCat(GetClassName(*ed.containing_type()), "_", ed.name());
 }
 
@@ -211,21 +214,21 @@ string GetQualifiedEnumNameFn(const EnumDescriptor& enum_d) {
 // is false) or the internal implementation header (if impl is true).
 string GetProtoTextHeaderName(const FileDescriptor& fd, bool impl) {
   const int dot_index = fd.name().find_last_of('.');
-  return fd.name().substr(0, dot_index) +
-         (impl ? ".pb_text-impl.h" : ".pb_text.h");
+  return StrCat(fd.name().substr(0, dot_index),
+                (impl ? ".pb_text-impl.h" : ".pb_text.h"));
 }
 
 // Returns the name of the header generated by the proto library for <fd>.
 string GetProtoHeaderName(const FileDescriptor& fd) {
   const int dot_index = fd.name().find_last_of('.');
-  return fd.name().substr(0, dot_index) + ".pb.h";
+  return StrCat(fd.name().substr(0, dot_index), ".pb.h");
 }
 
 // Returns the C++ class name for the given proto field.
 string GetCppClass(const FieldDescriptor& d) {
   string cpp_class = d.cpp_type() == FieldDescriptor::CPPTYPE_MESSAGE
                          ? GetQualifiedName(*d.message_type())
-                         : d.cpp_type_name();
+                         : std::string(d.cpp_type_name());
 
   // In open-source TensorFlow, the definition of int64 varies across
   // platforms. The following line, which is manipulated during internal-
@@ -241,7 +244,7 @@ string GetCppClass(const FieldDescriptor& d) {
 // headers for <fd>, either for the public api (if impl is false) or the
 // internal implementation header (if impl is true).
 string GetHeaderGuard(const FileDescriptor& fd, bool impl) {
-  string s = fd.name();
+  string s(fd.name());
   std::replace(s.begin(), s.end(), '/', '_');
   std::replace(s.begin(), s.end(), '.', '_');
   return s + (impl ? "_IMPL_H_" : "_H_");
@@ -273,10 +276,16 @@ void Generator::AppendFieldValueAppend(const FieldDescriptor& field,
             field.name(), "\", ", field_expr, ");");
       break;
     case FieldDescriptor::CPPTYPE_STRING: {
+#if (TSL_IS_IN_OSS && GOOGLE_PROTOBUF_VERSION < 6030000)
       const auto ctype = field.options().ctype();
       CHECK(ctype == FieldOptions::CORD || ctype == FieldOptions::STRING)
           << "Unsupported ctype " << ctype;
-
+#else
+      const auto ctype = field.cpp_string_type();
+      CHECK(ctype == FieldDescriptor::CppStringType::kCord ||
+            ctype == FieldDescriptor::CppStringType::kString)
+          << "Unsupported string type " << static_cast<int>(ctype);
+#endif
       Print("o->", omit_default ? "AppendStringIfNotEmpty" : "AppendString",
             "(\"", field.name(), "\", ProtobufStringToString(", field_expr,
             "));");
@@ -314,7 +323,7 @@ void Generator::AppendFieldValueAppend(const FieldDescriptor& field,
 }
 
 void Generator::AppendFieldAppend(const FieldDescriptor& field) {
-  const string& name = field.name();
+  absl::string_view name = field.name();
 
   if (field.is_map()) {
     Print("{").Nest();
@@ -337,24 +346,24 @@ void Generator::AppendFieldAppend(const FieldDescriptor& field) {
     Print("for (int i = 0; i < msg.", name, "_size(); ++i) {");
     Nest();
     AppendFieldValueAppend(field, false /* omit_default */,
-                           "msg." + name + "(i)");
+                           StrCat("msg.", name, "(i)"));
     Unnest().Print("}");
   } else {
     const auto* oneof = field.containing_oneof();
     if (oneof != nullptr) {
-      string camel_name = field.camelcase_name();
+      string camel_name(field.camelcase_name());
       camel_name[0] = toupper(camel_name[0]);
       Print("if (msg.", oneof->name(), "_case() == ",
             GetQualifiedName(*oneof->containing_type()), "::k", camel_name,
             ") {");
       Nest();
       AppendFieldValueAppend(field, false /* omit_default */,
-                             "msg." + name + "()");
+                             StrCat("msg.", name, "()"));
       Unnest();
       Print("}");
     } else {
       AppendFieldValueAppend(field, true /* omit_default */,
-                             "msg." + name + "()");
+                             StrCat("msg.", name, "()"));
     }
   }
 }
@@ -466,7 +475,7 @@ void Generator::AppendParseMessageFunction(const Descriptor& md) {
   Unnest().Print("}");
   for (int i = 0; i < md.field_count(); ++i) {
     const FieldDescriptor* field = md.field(i);
-    const string& field_name = field->name();
+    absl::string_view field_name = field->name();
     string mutable_value_expr;
     string set_value_prefix;
     if (map_append) {
@@ -504,7 +513,7 @@ void Generator::AppendParseMessageFunction(const Descriptor& md) {
       CHECK(!map_append);
 
       // Detect duplicate oneof value.
-      const string oneof_name = field->containing_oneof()->name();
+      const absl::string_view oneof_name = field->containing_oneof()->name();
       Print("if (msg->", oneof_name, "_case() != 0) return false;");
     }
 
@@ -551,7 +560,7 @@ void Generator::AppendParseMessageFunction(const Descriptor& md) {
 
       for (int enum_i = 0; enum_i < enum_d->value_count(); ++enum_i) {
         const auto* value_d = enum_d->value(enum_i);
-        const string& value_name = value_d->name();
+        absl::string_view value_name = value_d->name();
         string condition = StrCat("value == \"", value_name, "\"");
 
         Print(enum_i == 0 ? "" : "} else ", "if (", condition, ") {");
@@ -691,7 +700,8 @@ void Generator::AppendMessageFunctions(const Descriptor& md) {
   AppendParseMessageFunction(md);
 }
 
-void Generator::AddNamespaceToCurrentSection(const string& package, bool open) {
+void Generator::AddNamespaceToCurrentSection(absl::string_view package,
+                                             bool open) {
   Print();
   std::vector<string> parts = {""};
   for (size_t i = 0; i < package.size(); ++i) {
@@ -768,7 +778,7 @@ void GetAllFileDescriptorsFromFile(const FileDescriptor* fd,
 }
 
 void Generator::Generate(const FileDescriptor& fd) {
-  const string package = fd.package();
+  const absl::string_view package = fd.package();
   std::set<const FileDescriptor*> all_fd;
   std::set<const Descriptor*> all_d;
   GetAllFileDescriptorsFromFile(&fd, &all_fd, &all_d);
@@ -825,7 +835,6 @@ void Generator::Generate(const FileDescriptor& fd) {
   Print();
   Print("using ::tensorflow::strings::ProtoSpaceAndComments;");
   Print("using ::tensorflow::strings::Scanner;");
-  Print("using ::tensorflow::strings::StrCat;");
   AddNamespaceToCurrentSection(package, true /* is_open */);
 
   // Add declarations and definitions.
